import androguard.core.api_specific_resources.api_permission_mappings.api_permission_mappings_api9 as api9
import androguard.core.api_specific_resources.api_permission_mappings.api_permission_mappings_api10 as api10
import androguard.core.api_specific_resources.api_permission_mappings.api_permission_mappings_api14 as api14
import androguard.core.api_specific_resources.api_permission_mappings.api_permission_mappings_api15 as api15
import androguard.core.api_specific_resources.api_permission_mappings.api_permission_mappings_api16 as api16
import androguard.core.api_specific_resources.api_permission_mappings.api_permission_mappings_api17 as api17
import androguard.core.api_specific_resources.api_permission_mappings.api_permission_mappings_api18 as api18
import androguard.core.api_specific_resources.api_permission_mappings.api_permission_mappings_api19 as api19
import androguard.core.api_specific_resources.api_permission_mappings.api_permission_mappings_api21 as api21
import androguard.core.api_specific_resources.api_permission_mappings.api_permission_mappings_api22 as api22


AOSP_PERMISSIONS_MAPPINGS = {
    "9" : {"AOSP_PERMISSIONS_BY_METHODS" : api9.AOSP_PERMISSIONS_BY_METHODS, "AOSP_PERMISSIONS_BY_FIELDS" : api9.AOSP_PERMISSIONS_BY_FIELDS},
    "10" : {"AOSP_PERMISSIONS_BY_METHODS" : api10.AOSP_PERMISSIONS_BY_METHODS, "AOSP_PERMISSIONS_BY_FIELDS" : api10.AOSP_PERMISSIONS_BY_FIELDS},
    "14" : {"AOSP_PERMISSIONS_BY_METHODS" : api14.AOSP_PERMISSIONS_BY_METHODS, "AOSP_PERMISSIONS_BY_FIELDS" : api14.AOSP_PERMISSIONS_BY_FIELDS},
    "15" : {"AOSP_PERMISSIONS_BY_METHODS" : api15.AOSP_PERMISSIONS_BY_METHODS, "AOSP_PERMISSIONS_BY_FIELDS" : api15.AOSP_PERMISSIONS_BY_FIELDS},
    "16" : {"AOSP_PERMISSIONS_BY_METHODS" : api16.AOSP_PERMISSIONS_BY_METHODS, "AOSP_PERMISSIONS_BY_FIELDS" : api16.AOSP_PERMISSIONS_BY_FIELDS},
    "17" : {"AOSP_PERMISSIONS_BY_METHODS" : api17.AOSP_PERMISSIONS_BY_METHODS, "AOSP_PERMISSIONS_BY_FIELDS" : api17.AOSP_PERMISSIONS_BY_FIELDS},
    "18" : {"AOSP_PERMISSIONS_BY_METHODS" : api18.AOSP_PERMISSIONS_BY_METHODS, "AOSP_PERMISSIONS_BY_FIELDS" : api18.AOSP_PERMISSIONS_BY_FIELDS},
    "19" : {"AOSP_PERMISSIONS_BY_METHODS" : api19.AOSP_PERMISSIONS_BY_METHODS, "AOSP_PERMISSIONS_BY_FIELDS" : api19.AOSP_PERMISSIONS_BY_FIELDS},
    "21" : {"AOSP_PERMISSIONS_BY_METHODS" : api21.AOSP_PERMISSIONS_BY_METHODS, "AOSP_PERMISSIONS_BY_FIELDS" : api21.AOSP_PERMISSIONS_BY_FIELDS},
    "22" : {"AOSP_PERMISSIONS_BY_METHODS" : api22.AOSP_PERMISSIONS_BY_METHODS, "AOSP_PERMISSIONS_BY_FIELDS" : api22.AOSP_PERMISSIONS_BY_FIELDS},
}
