#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is a part of Androguard.
#
# This file is generated automatically from the data
# provided by PScout tool [http://pscout.csl.toronto.edu/]
# using script: extract_api_permission_mappings.py
#
# Author: Yury Zhauniarovich
#
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS-IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License"

#################################################
### API version: 17 
#################################################


AOSP_PERMISSIONS_BY_METHODS = {
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handlePinMmi-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardEntry$BirthdayData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/group/GroupEditorFragment;-bindGroupMetaData-(Landroid/database/Cursor;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/model/dataitem/PhotoDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/HTML5VideoInline;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/ProfileProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/PhotoView;-checkHideUndoBar-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiStateMachine;-access$2700-(Landroid/net/wifi/WifiStateMachine; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/webkit/WebViewProvider;-destroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/LockPatternView;-addCellToPattern-(Lcom/android/internal/widget/LockPatternView$Cell;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/IccProvider;-updateIccRecordInEf-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/DataUsageSummary;-updateTabs-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$AudioOn;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/BrowserWebView;-zoomOut-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/adapter/CalendarSyncAdapter$EasCalendarSyncParser;-parse-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onEndBatchInput-(Lcom/android/inputmethod/latin/InputPointers;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/SnapshotTab$LoadData;-onPostExecute-(Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/soundrecorder/Recorder;-delete-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$WifiStateTracker;-setImageViewResources-(Landroid/content/Context; Landroid/widget/RemoteViews;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter$SlideshowSource;-findItemIndex-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-vibrate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/webkit/WebView;-performAccessibilityAction-(I Landroid/os/Bundle;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/videoeditor/VideoEditorActivity$23;-onStop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-handleKeyguardDone-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lorg/apache/http/impl/client/DefaultHttpClient;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/phone/InCallScreen;-resetInCallScreenMode-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/settings/wifi/WifiSettingsForSetupWizardXL;-updateConnectionState-(Landroid/net/NetworkInfo$DetailedState;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/WifiService$NotificationEnabledSettingObserver;-register-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/SuggestionsAdapter$SlowFilterTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/googlecode/mp4parser/authoring/builder/Mp4Builder;-build-(Lcom/googlecode/mp4parser/authoring/Movie;)Lcom/coremedia/iso/IsoFile;' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/CropFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/phone/sip/SipEditor;-saveAndRegisterProfile-(Landroid/net/sip/SipProfile;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManagerService;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-mayEndBatchInput-(J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/launcher2/PagedView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/content/ContentService;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-drawFrameLocked-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-doKeyguardLaterLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/content/ISyncAdapter;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/EventInfoFragment$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/base/FrameSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/CdmaVoicePrivacyCheckBoxPreference$MyHandler;-handleSetVPResponse-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/picasasource/PicasaSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-acknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5VideoViewProxy;-setBaseLayer-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag;-ndefWrite-(I Landroid/nfc/NdefMessage;)I' : ['android.permission.NFC'],
	'Lcom/android/internal/statusbar/IStatusBarService;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processNoiceReductionEvent-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/BackupManagerService;-opComplete-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/accounts/IAccountManager$Stub;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/settings/Settings$LocalePickerActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/CallLogAsync$AddCallTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/speech/SpeechRecognizer;-cancel-()V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/contacts/model/dataitem/OrganizationDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/MediaRecorder;-setVideoSource-(I)V' : ['android.permission.CAMERA'],
	'Landroid/nfc/tech/NdefFormatable;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-syncReloadDictionaryIfRequired-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/content/ContentService;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/browser/DownloadHandler;-onDownloadStart-(Landroid/app/Activity; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Z)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/contacts/model/dataitem/NoteDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/activities/ContactDetailActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-commitText-(Ljava/lang/CharSequence; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub;-getTetherableUsbRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/launcher2/AppsCustomizePagedView;-beginDragging-(Landroid/view/View;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-handleBackKey-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/telephony/TelephonyManager;-getDeviceSoftwareVersion-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/Settings$AndroidBeamSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/data/WorkingMessage;-access$1400-(Landroid/net/Uri; Lcom/google/android/mms/pdu/PduPersister; Lcom/android/mms/model/SlideshowModel; Lcom/google/android/mms/pdu/SendReq; Ljava/util/HashMap;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/server/wm/Session;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/phone/InCallScreen;-onStop-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-hostChanged-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-doKeyguardTimeout-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/PhotoPage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onDisconnectDone-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/InCallScreen;-isBluetoothAudioConnected-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/IConnectivityManager$Stub;-requestNetworkTransitionWakelock-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$10300-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/util/ReverseGeocoder;-computeAddress-(Lcom/android/gallery3d/util/ReverseGeocoder$SetLatLong;)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/downloads/StorageManager;-verifySpaceBeforeWritingToFile-(I Ljava/lang/String; J)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/server/LocationManagerService;-init-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-onDestroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/VoicemailStatusTable;-insert-(Lcom/android/providers/contacts/VoicemailContentProvider$UriData; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ThrottleService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-doNotifyDataConnection-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/group/GroupDetailFragment$3;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/CursorLoader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/DockService;-access$100-(Lcom/android/settings/bluetooth/DockService; Landroid/os/Message;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$DreamSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/UiModeManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-onStop-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/IActivityManager;-unbroadcastIntent-(Landroid/app/IApplicationThread; Landroid/content/Intent; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/systemui/recent/RecentsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Lcom/android/contacts/CallContactActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-updatePhones-(J Landroid/content/ContentValues;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/CropView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor;-access$300-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor$SimArgs;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/app/TrimTimeBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/VoicemailContract$Status;-buildSourceUri-(Ljava/lang/String;)Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/bluetooth/IBluetoothHeadset;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/AndroidClockTextView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-stopLoading-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-access$200-(Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-drawEvent-(Landroid/graphics/Canvas; Lcom/android/calendar/Event; I I I Z Z Z)I' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/Settings$DataUsageSummaryActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/Phone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/PhotoDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-addNotificationViews-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)Lcom/android/systemui/statusbar/StatusBarIconView;' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wm/WindowManagerService;-addWindow-(Lcom/android/server/wm/Session; Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/inputmethod/latin/Suggest;-getSuggestedWordsForTypingInput-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; Z)Lcom/android/inputmethod/latin/SuggestedWords;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/DataUsageListener;-updatePolicy-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView$AccountAnalyzer;-<init>-(Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView; Landroid/accounts/AccountManager; Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView$1;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/util/AccountSelectionUtil;-getSelectAccountDialog-(Landroid/content/Context; I Landroid/content/DialogInterface$OnClickListener; Landroid/content/DialogInterface$OnCancelListener;)Landroid/app/Dialog;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-updateState-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-gotoIdleAndNotifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/DhcpStateMachine$RunningState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-<init>-(Landroid/content/Context; Landroid/os/INetworkManagementService; Landroid/net/INetworkStatsService; Landroid/net/INetworkPolicyManager;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/SlideshowActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/Settings$LocalePickerActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/launcher2/WallpaperChooserDialogFragment;-access$000-(Lcom/android/launcher2/WallpaperChooserDialogFragment; I)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowManagerService;-moveAppWindowsLocked-(Ljava/util/List; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/FrameSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-setSystemUiVisibility-(I I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/exchange/EasSyncService;-canProvision-(Lcom/android/exchange/EasSyncService;)Lcom/android/exchange/adapter/ProvisionParser;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/test/FragmentTestActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Landroid/provider/Telephony$Sms$Sent;-addMessage-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/power/PowerManagerService$1;-onProximityPositive-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-addNotification-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapUtils;-isProfileSet-(Landroid/content/Context;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewBase;-verifyUnlock-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothInputDevice;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/content/IContentService;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mms/util/DraftCache;-access$000-(Lcom/android/mms/util/DraftCache;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/phone/InCallScreen;-updateManageConferencePanelIfNecessary-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/widget/ChartDataUsageView$DataUsageChartListener;-onWarningChanged-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/AccountFilterActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$7300-(Lcom/android/mms/ui/ComposeMessageActivity; I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/view/VolumePanel;-onRemoteVolumeChanged-(I I)V' : ['android.permission.VIBRATE'],
	'Landroid/media/IAudioService;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/NetworkTimeUpdateService;-systemReady-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-setNfcBeamPushUris-([Landroid/net/Uri;)V' : ['android.permission.NFC'],
	'Lcom/android/server/VibratorService;-startVibrationLocked-(Lcom/android/server/VibratorService$Vibration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/settings/Settings$PowerUsageSummaryActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/WifiService;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/ConnectivityService;-setLockdownTracker-(Lcom/android/server/net/LockdownVpnTracker;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/app/LocalePicker;-updateLocale-(Ljava/util/Locale;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService$AudioHandler;-access$4800-(Landroid/media/AudioService$AudioHandler; Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/LegacyPhoneNumberListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$DisplaySettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/emailcommon/service/IEmailService;-hostChanged-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/Search$SearchParser;-parseResponse-()Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/commands/bmgr/Bmgr;-doListRestoreSets-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothDevicePreference;-onBindView-(Landroid/view/View;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/model/ContactLoader;-loadGroupMetaData-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/IWifiManager;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/ConnectivityManager;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/nfc/NfcService$TagService;-isNdef-(I)Z' : ['android.permission.NFC'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-createBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/UrlInputView;-onSelect-(Ljava/lang/String; I Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/VibratorService;-onInputDeviceRemoved-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-showBugreportDialog-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/gallery3d/data/DataManager;-delete-(Lcom/android/gallery3d/data/Path;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/ActivityManagerNative;-handleApplicationWtf-(Landroid/os/IBinder; Ljava/lang/String; Landroid/app/ApplicationErrorReport$CrashInfo;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/providers/contacts/ProfileProvider;-enforceWritePermission-()V' : ['android.permission.WRITE_PROFILE'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-setUpAdapter-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/launcher2/PagedViewWithDraggableItems;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/phone/MobileNetworkSettings$MyHandler;-resetNetworkModeToDefault-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-acceptCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardStatusView$1;-onRefreshCarrierInfo-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/NicknameDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$DockEventHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/InCallScreen;-access$600-(Lcom/android/phone/InCallScreen;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/media/AudioService;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$3000-(Lcom/android/mms/ui/ComposeMessageActivity; J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/IccRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/BluetoothPhoneService$3;-processChld-(I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/settings/Settings$InputMethodAndLanguageSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/SearchActivity;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine;-access$17400-(Landroid/net/wifi/WifiStateMachine; Ljava/util/ArrayList;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ConnectivityService;-requestNetworkTransitionWakelock-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/policy/DateView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/GoogleAccountLogin;-invalidateTokens-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/DataUsageListener$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ConnectivityService;-isNetworkSupported-(I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateStart;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter;-handleCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/DownloadThread;-notifyThroughDatabase-(I Z I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/net/INetworkPolicyManager$Stub;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/EasOutboxService;-acknowledgeProvisionImpl-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String; Z)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebView;-clearHistory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/data/Contact$ContactsCache;-getContactInfo-(Lcom/android/mms/data/Contact;)Lcom/android/mms/data/Contact;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/ui/ComposeMessageActivity$36;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/AudioPreview;-playPauseClicked-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-forceStopUserLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/util/Log$1;-onTerribleFailure-(Ljava/lang/String; Landroid/util/Log$TerribleFailure;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/location/fused/FusedLocationProvider;-<init>-(Landroid/content/Context;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/SnapshotTab;-<init>-(Lcom/android/browser/WebViewController; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/IRingtonePlayer$Stub;-stopAsync-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/PushReceiver;-access$000-(Landroid/content/Context; Lcom/google/android/mms/pdu/GenericPdu; I)J' : ['android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/StatusBarManager;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/browser/Tab$2;-onPageFinished-(Landroid/webkit/WebView; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/TabControl;-setCurrentTab-(Lcom/android/browser/Tab; Z)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/text/StringSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternView;-handleActionUp-(Landroid/view/MotionEvent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/EditableInputConnection;-sendKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText$1;-onRefreshCarrierInfo-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/DockEventReceiver;-finishStartingService-(Landroid/app/Service; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/NfcAdapter;-enableForegroundDispatch-(Landroid/app/Activity; Landroid/app/PendingIntent; [Landroid/content/IntentFilter; [[Ljava/lang/String;)V' : ['android.permission.NFC'],
	'Lcom/android/contacts/list/EmailAddressListAdapter;-getContactUri-(I Landroid/database/Cursor; I I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardStatusViewManager$1;-onUserSwitched-(I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/MediaSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarEventModel;-<init>-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-wakingUp-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity;-onBackPressed-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetoothHeadset;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/commands/bmgr/Bmgr;-doWipe-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/group/GroupBrowseListFragment;-access$400-(Lcom/android/contacts/group/GroupBrowseListFragment;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/CallbackProxy;-createWindow-(Z Z)Landroid/webkit/BrowserFrame;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$WirelessSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/HeadsetProfile;-isPreferred-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/ContactsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/gallery3d/data/SnailSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-access$900-(Lcom/android/systemui/statusbar/phone/QuickSettings; Landroid/content/Intent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/app/LocalePicker;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/IThrottleManager$Stub;-getCliffThreshold-(Ljava/lang/String; I)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/sip/SipEditor;-access$400-(Lcom/android/phone/sip/SipEditor;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub;-getAllMessagesFromIccEf-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/MessagingPreferenceActivity;-setMessagePreferences-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/BluetoothSocket;-<init>-(I I Z Z Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/input/InputManagerService;-start-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-access$3000-(Lcom/android/systemui/statusbar/phone/PhoneStatusBar;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/VibratorService;-access$200-(Lcom/android/server/VibratorService;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/activities/ActionBarAdapter;-setCurrentTab-(I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/emailcommon/service/ServiceProxy$ProxyRunnable;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/phone/SipCallOptionHandler;-createSipPhoneIfNeeded-(Landroid/net/sip/SipProfile;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.USE_SIP'],
	'Lcom/android/contacts/editor/ContactEditorFragment$JoinSuggestedContactDialogFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/Phone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-dispatchUnhandledKey-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.CALL_PHONE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/sip/SipService$IntervalMeasurementProcess;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/CameraModule;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/model/dataitem/PhotoDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/CallbackProxy;-onJsPrompt-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/handover/HandoverTransfer;-updateFileProgress-(F)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/DayView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/data/MtpDevice;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/webkit/AccessibilityInjectorFallback;-performAccessibilityAction-(I Landroid/os/Bundle;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-scrollLayerTo-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/CallNotifier$EmergencyTonePlayerVibrator;-stop-()V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5VideoFullScreen;-access$500-(Landroid/webkit/HTML5VideoFullScreen;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/EmailAddressListAdapter;-bindQuickContact-(Lcom/android/contacts/list/ContactListItemView; I Landroid/database/Cursor; I I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-snapToPage-(I I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-setPasskey-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/videosrc/MediaSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/WifiService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_MULTICAST_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/detail/ContactDetailLayoutController;-<init>-(Landroid/app/Activity; Landroid/os/Bundle; Landroid/app/FragmentManager; Lcom/android/contacts/widget/TransitionAnimationView; Landroid/view/View; Lcom/android/contacts/detail/ContactDetailFragment$Listener;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/server/sip/SipService;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/preference/VolumePreference;-cleanup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/wifi/WifiSettings;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/CropImage;-getMediaItemFromIntentData-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/IAccountManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/nfc/tech/MifareUltralight;-setTimeout-(I)V' : ['android.permission.NFC'],
	'Lcom/android/settings/accounts/SyncSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/AccountsChangedReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/Settings$WirelessSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/hdp/HealthService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebViewCore;-contentScrollTo-(I I Z Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/VideoModule;-initializeRecorder-()V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/content/SyncManager;-sendSyncFinishedOrCanceledMessage-(Landroid/content/SyncManager$ActiveSyncContext; Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-access$400-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/net/INetworkPolicyManager;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-access$200-(Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hid/HidService;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-snapToDestination-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiManager$MulticastLock;-release-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/webkit/WebViewClassic;-reload-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/OrganizationDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-getContactCount-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/emailcommon/Device;-getDeviceId-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/DeviceAccountLogin;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$FullImageJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/camera/SecureCameraActivity;-access$100-(Lcom/android/camera/CameraActivity; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-getContactMethodType-(Landroid/net/Uri;)Ljava/lang/String;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/group/GroupBrowseListFragment;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-syncReloadDictionaryIfRequired-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/commands/svc/PowerCommand;-run-([Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/BluetoothManagerService;-disable-(Z)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/transaction/SmsReceiverService;-access$000-(Lcom/android/mms/transaction/SmsReceiverService; Landroid/content/Intent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Landroid/net/wifi/IWifiManager;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/webkit/ZoomControlExternal$3;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ShortcutIntentBuilder$ContactLoadingAsyncTask;-loadPhoto-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ThrottleService;-shutdown-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService$Stub;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/LocationManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-determineScrollingStart-(Landroid/view/MotionEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService$AppDeathRecipient;-binderDied-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/ConnectivityService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/LocalBluetoothManager;-getInstance-(Landroid/content/Context;)Lcom/android/settings/bluetooth/LocalBluetoothManager;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/transaction/SmsReceiverService;-insertMessage-(Landroid/content/Context; [Landroid/telephony/SmsMessage; I Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView;-onResume-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothSocket;-<init>-(I I Z Z Ljava/lang/String; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/media/effect/FilterGraphEffect;-apply-(I I I I)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/location/ILocationManager$Stub$Proxy;-addGpsStatusListener-(Landroid/location/IGpsStatusListener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/INfcTag$Stub;-close-(I)I' : ['android.permission.NFC'],
	'Lcom/android/emailcommon/service/IEmailService;-stopSync-(J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-killApplicationWithAppId-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-reset-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ViewInterface;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/Utils;-showError-(Landroid/content/Context; Ljava/lang/String; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/Phone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/SmsReceiverService;-access$200-(Lcom/android/mms/transaction/SmsReceiverService;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/Settings$PrivacySettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewBase;-wakeWhenReadyTq-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/GLTextureSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/VoicemailPermissions;-packageHasFullAccess-(Ljava/lang/String;)Z' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/calendar/EventLoader$LoadEventsRequest;-processRequest-(Lcom/android/calendar/EventLoader;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/editor/PhotoEditorView$1;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/downloads/DownloadThread;-updateDatabaseFromHeaders-(Lcom/android/providers/downloads/DownloadThread$State; Lcom/android/providers/downloads/DownloadThread$InnerState;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/music/MediaPlaybackService$5;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BackupManagerService$PerformRestoreTask;-restoreNextAgent-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/IWifiManager;-getScanResults-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/CallDetailActivityQueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PickerActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/ui/AudioAttachmentView;-onPlaybackError-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/EmailDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity$PersistTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/ConnectivityManager;-getMobileDataEnabled-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSimPukView;-onResume-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/PhoneGlobals;-handleSetTTYModeResponse-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/WifiService;-enforceMulticastChangePermission-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-getRecentTasks-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/gallery3d/ui/PhotoView;-pause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-invokeCodeInput-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPuk2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-switchWaitingOrHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewProvider;-zoomOut-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/EventInfoFragment;-showContactInfo-(Lcom/android/calendar/CalendarEventModel$Attendee; Landroid/graphics/Rect;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/PhotoView$1;-onClick-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/PowerManagerService;-access$800-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/event/AttendeesView;-addAttendees-(Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/IConnectivityManager$Stub;-getNetworkInfo-(I)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/IConnectivityManager;-tether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/inputmethod/InputMethodManager;-finishedEvent-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/calendar/CalendarProvider2;-getOriginalSyncId-(J)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/usb/UsbService;-systemReady-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/PhoneUtils;-handleHeadsetHook-(Lcom/android/internal/telephony/Phone; Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$CaptivePortalCheckState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/Settings$DreamSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/LocalSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/Settings$BluetoothSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/XLargeUi;-<init>-(Landroid/app/Activity; Lcom/android/browser/UiController;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-startScanning-(Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/ContactListFilterController;-setContactListFilter-(Lcom/android/contacts/list/ContactListFilter; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-confirmCredentialsAsUser-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler; Landroid/os/UserHandle;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/Tab;-createSubWindow-()Z' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/providers/contacts/ContactsProvider2;-insertSettings-(Landroid/net/Uri; Landroid/content/ContentValues;)J' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton$1;-onKeyguardVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/ExchangeService$AccountObserver;-access$700-(Lcom/android/exchange/ExchangeService$AccountObserver;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/PieRenderer;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/display/DisplayManagerService$DisplayAdapterListener;-onDisplayDeviceEvent-(Lcom/android/server/display/DisplayDevice; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/CaptivePortalTracker;-access$800-(Landroid/net/CaptivePortalTracker; Landroid/net/NetworkInfo;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/vpndialogs/ManageDialog;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/launcher2/SmoothPagedView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$ClassChangedHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-executeNextState-(Lcom/android/server/BackupManagerService$BackupState;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetoothManager;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-doFinish-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/power/PowerManagerService$WakeLock;-binderDied-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/preferences/WebsiteSettingsFragment$SiteAdapter$UpdateFromBookmarksDbTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-queueQuery-(Lcom/android/calendar/agenda/AgendaWindowAdapter$QuerySpec;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/net/wifi/WifiStateMachine;-stopTethering-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/nfc/TechListChooserActivity;-onStop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/ui/ActionModeHandler$2;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/nfc/INfcAdapterExtras;-getCardEmulationRoute-(Ljava/lang/String;)I' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/TintFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendEnvelopeWithStatus-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/ActivityState;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/CdmaSubscriptionListPreference;-onDialogClosed-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/MessageItem;-access$200-(Lcom/android/mms/ui/MessageItem; Lcom/google/android/mms/pdu/EncodedStringValue; Landroid/net/Uri;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/mms/data/Conversation;-dumpSmsTable-(Landroid/content/Context;)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/IccProvider;-loadFromEf-(I)Landroid/database/MatrixCursor;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/TetherSettings$TetherChangeReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewClassic;-sendBatchableInputMessage-(I I I Ljava/lang/Object;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/ImDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/location/LocationManager;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/accounts/AccountManager;-addOnAccountsUpdatedListener-(Landroid/accounts/OnAccountsUpdateListener; Landroid/os/Handler; Z)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/SlideshowPage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/accounts/AccountManager$2;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-adjustUserActivityLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/net/sip/SipAudioCall;-isWifiOn-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/ClusterSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/transaction/SendTransaction;-sendPdu-([B Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/effect/effects/TintEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage$7;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/systemui/SwipeHelper$2;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Lorg/apache/http/impl/client/DefaultHttpClient;-execute-(Lorg/apache/http/client/methods/HttpUriRequest;)Lorg/apache/http/HttpResponse;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView$2;-onClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-deleteSmsOnSim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/Suggest;-getSuggestedWords-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; Z I)Lcom/android/inputmethod/latin/SuggestedWords;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$2800-(Lcom/android/mms/ui/ComposeMessageActivity; J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-initialize-(Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/BluetoothDevice;-createRfcommSocketToServiceRecord-(Ljava/util/UUID;)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/CropView$HighlightRectangle;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/exchange/EasSyncService;-validateAccount-(Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/btservice/ProfileService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/test/FragmentTestActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$AccountSyncSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/ApplicationSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-onScreenTurnedOff-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterfw/core/Filter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/vcard/NotificationImportExportListener;-onImportParsed-(Lcom/android/contacts/vcard/ImportRequest; I Lcom/android/vcard/VCardEntry; I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/UI;-onBackKey-()Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/InCallScreen$12;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onSeListenActivated-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$5000-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Landroid/net/wifi/p2p/WifiP2pConfig; Z)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-hideSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Landroid/net/sip/SipAudioCall;-transferToNewSession-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothA2dp;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/NullFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/nfc/P2pEventManager;-onP2pReceiveComplete-(Z)V' : ['android.permission.VIBRATE'],
	'Lcom/coremedia/iso/IsoFile;-<init>-(Ljava/io/File;)V' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/list/ContactListFilterControllerImpl;-checkFilterValidity-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/IThrottleManager$Stub;-getCliffLevel-(Ljava/lang/String; I)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-setShuffleMode-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/UriSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifyTargetDeselected-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/phone/InCallScreen;-access$800-(Lcom/android/phone/InCallScreen;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/vcard/VCardEntry$PhotoData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/webkit/WebViewInputDispatcher;-skipWebKitEventLocked-(Landroid/webkit/WebViewInputDispatcher$DispatchEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccRecords;-onRefresh-(Z [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/PrivacySettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/ConnectivityManager;-setMobileDataEnabled-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/webkit/WebView;-loadUrl-(Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/location/ILocationManager$Stub;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/browser/SnapshotTab;-destroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/EventInfoFragment;-updateAttendees-(Landroid/view/View;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider;-updateWidget-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/provider/Contacts$Settings;-getSetting-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/WifiManager;-reconnect-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/webkit/WebViewClassic;-goBack-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/RuimSmsInterfaceManager;-copyMessageToIccEf-(I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/CommandsInterface;-iccIOForApp-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-getLastTetherError-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/WirelessSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/ui/MessagingPreferenceActivity;-getIsGroupMmsEnabled-(Landroid/content/Context;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-getContactUriForPhoneNumber-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-snapToPage-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/SettingsSafetyLegalActivity;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-onDestroy-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl;-getUsableInvitableAccountTypes-()Ljava/util/Map;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/Settings$ManageAccountsSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiStateMachine$SoftApStartedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/voicedialer/RecognizerClient;-onMicrophoneStart-(Ljava/io/InputStream;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-access$100-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/ConnectivityService;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/pm/PackageManagerService;-access$3700-(Lcom/android/server/pm/PackageManagerService; Ljava/lang/String; I I)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/WiredAccessoryManager;-notifyWiredAccessoryChanged-(J I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-sendUSSD-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IBackupAgent$Stub;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/content/CursorEntityIterator;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processAnswerCall-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler$PhotoListener;-onPhotoSelected-(Landroid/graphics/Bitmap;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/sip/SipService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/bluetooth/IBluetoothPbap;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$PlaybackPositionListener;-onStartTrackingTouch-(Landroid/widget/SeekBar;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcAdapterExtras$Stub;-getDriverName-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/RIL;-getIMSI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue;-releaseAllPointersOlderThan-(Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue$Element; J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-startDelayedRetry-(Lcom/android/internal/telephony/DataConnection$FailCause; Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService$SipSessionGroupExt;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/RuimRecords;-fetchRuimRecords-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/PhoneNumberListAdapter;-getContactUri-(I Landroid/database/Cursor; I I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/BluetoothDevicePreference;-onClicked-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/BluetoothDiscoverableEnabler;-setNumberOfPairedDevices-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/NfcService$TagService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-cancelPostDial-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/InCallScreen;-handleCallKey-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-open-([J I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/AccessibilityInjector;-removeCallbackApis-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/videosrc/MediaSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/Contact;-getPhoneUri-()Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/Gallery;-startDefaultPage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/accounts/AccountManagerService;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/settings/accounts/SyncSettings;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS'],
	'Lcom/android/systemui/power/PowerUI;-playLowBatterySound-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothSettings$1;-updateDeviceName-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PasswordUnlockScreen;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/webkit/AccessibilityInjector;-handleKeyEventIfNecessary-(Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/HTML5Audio;-setDataSource-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-computeScreenConfigurationLocked-(Landroid/content/res/Configuration;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/Context;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-access$2300-(Lcom/android/gallery3d/app/PhotoDataAdapter;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/pm/PackageManagerService$11;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/server/wm/WindowManagerService;-setForcedDisplayDensity-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/InputMethodManagerService;-buildInputMethodListLocked-(Ljava/util/ArrayList; Ljava/util/HashMap;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/HTML5VideoViewProxy;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-bluetoothStateChangeHandler-(I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-haveSomethingToCopyToSDCard-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-restartRadio-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/webkit/PluginFullScreenHolder$CustomFrameLayout;-onTrackballEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/ReadRecTransaction;-getPdu-(Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiStateMachine$ConnectModeState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$ManageAccountsSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/MediaPlaybackActivity;-startPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/calllog/CallLogFragment;-updateOnEntry-()V' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText;-onAttachedToWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/SyncManager;-onUserRemoved-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-doRecovery-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$AccountSyncSettingsInAddAccountActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager$Stub;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardEntry;-iterateAllData-(Lcom/android/vcard/VCardEntry$EntryElementIterator;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/DataUsageSummary$ConfirmLimitFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/google/android/mms/util/AbstractCache;-put-(Ljava/lang/Object; Ljava/lang/Object;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/PhotoPage;-onCommitDeleteImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/ui/InLineSettingMenu;-changeIndex-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-handleHide-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/mms/transaction/SmsReceiverService;-handleSmsReceived-(Landroid/content/Intent; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-processNextPostDialChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/recent/RecentsPanelView;-handleSwipe-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Lcom/android/settings/TetherSettings;-onStart-()V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/inputmethod/SpellCheckersSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/BackupManagerService$PerformFullRestoreTask;-restoreOneFile-(Ljava/io/InputStream; [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimPukUnlockScreen;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView$AccountAnalyzer;-<init>-(Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView; Landroid/accounts/AccountManager;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionRequest;-checkUserChoice-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/deskclock/TimerRingService;-play-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothInputDevice;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$ManageApplicationsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/IWindowManager$Stub$Proxy;-clearForcedDisplaySize-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-setBluetoothStateInt-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$GpsStateTracker;-onActualStateChange-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/webkit/WebViewCore$EventHub;-sendMessageDelayed-(Landroid/os/Message; J)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-getPackageGids-(Ljava/lang/String;)[I' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PickerActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame$1;-onClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$ManageAccountsSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/commands/monkey/Monkey;-run-([Ljava/lang/String;)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/webkit/WebViewCore$3;-onReceiveValue-(Ljava/lang/Object;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$DeviceInfoSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/PhoneGlobals;-dismissCallScreen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/telephony/SmsProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame;-onCameraLaunched-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/voicedialer/VoiceDialerActivity;-acquireWakeLock-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/MediaSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/inputmethod/InputMethodAndLanguageSettings$SettingsObserver;-resume-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/ChallengeLayout;-showChallenge-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub;-untether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/nfc/P2pEventListener;-onP2pSendComplete-()V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/WebViewClassic;-onTrackballEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/AllInOneActivity$2;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/EasSyncService;-sendHttpClientPost-(Ljava/lang/String; [B)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-loadData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/Ringer;-stopRing-()V' : ['android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$AudioOn;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/ITelephony$Stub;-getCellLocation-()Landroid/os/Bundle;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiManager$WifiLock;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/media/NotificationPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService;-dispatchUnhandledKey-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.CALL_PHONE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/HTML5VideoFullScreen;-pauseAndDispatch-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternView;-handleActionDown-(Landroid/view/MotionEvent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/wifi/WifiSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothPan;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$StateTracker;-setImageViewResources-(Landroid/content/Context; Landroid/widget/RemoteViews;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/PreloadedTabControl;-loadUrl-(Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/AlarmReceiver;-access$000-(Lcom/android/deskclock/AlarmReceiver; Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DataConnection;-access$2000-(Lcom/android/internal/telephony/DataConnection; Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$PrivacySettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/power/PowerManagerService;-wakeUpFromNative-(J)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/pan/PanService;-enableTethering-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/NsdService$NsdStateMachine;-<init>-(Lcom/android/server/NsdService; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/LocalePicker;-onLocaleSelected-(Ljava/util/Locale;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-getAwakeTimePlugged-()J' : ['android.permission.BATTERY_STATS'],
	'Lcom/android/internal/telephony/CommandsInterface;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/MessageUtils;-handleReadReport-(Landroid/content/Context; Ljava/util/Collection; I Ljava/lang/Runnable;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/providers/settings/SettingsBackupAgent;-onCreate-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-agentErrorCleanup-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/commands/am/Am;-runKillAll-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Landroid/net/ConnectivityManager;-tether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/PhoneBase;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/PieController$1;-onClick-(Lcom/android/camera/ui/PieItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-onDeviceAttributesChanged-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/editor/StructuredNameEditorView;-onOptionalFieldVisibilityChange-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/PhoneUtils;-switchHoldingAndActive-(Lcom/android/internal/telephony/Call;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-onSharedPreferenceChanged-(Landroid/content/SharedPreferences; Ljava/lang/String;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewStateManager;-onPageBeginMoving-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ContactListAdapter;-bindQuickContact-(Lcom/android/contacts/list/ContactListItemView; I Landroid/database/Cursor; I I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/camera/ui/TimeIntervalPopup;-updateInputState-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/location/fused/FusionEngine;-disableProvider-(Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/PowerManagerService$SettingsObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/recent/RecentsVerticalScrollView$3;-onClick-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/internal/telephony/cdma/RuimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothNameDialogFragment$3;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getContactsSize-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/bluetooth/IBluetooth$Stub;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/PhoneCallDetailsHelper;-setCallDetailsHeader-(Landroid/widget/TextView; Lcom/android/contacts/PhoneCallDetails;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/RIL;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/ConnectivityService;-getActiveNetworkQuotaInfo-()Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/ex/variablespeed/SingleThreadedMediaPlayerProxy;-prepare-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/GLView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/net/DataUsageMeteredSettings;-updateNetworks-(Landroid/content/Context;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/preferences/FontSizePreview;-onSharedPreferenceChanged-(Landroid/content/SharedPreferences; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/UiModeManagerService;-access$100-(Lcom/android/server/UiModeManagerService; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-handleAccessPermissionResult-(Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/os/IVibratorService;-vibratePattern-([J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/speech/SpeechRecognizer;-access$100-(Landroid/speech/SpeechRecognizer;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/systemui/statusbar/phone/NavigationBarView;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/view/IWindowManager;-isKeyguardSecure-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/UrlInputView;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-makeStatusBarView-()Landroid/view/View;' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onReplay-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangup-(Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag;-resetTimeouts-()V' : ['android.permission.NFC'],
	'Lcom/android/settings/Settings$WifiP2pSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/Gallery;-startGetContent-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/DataUsageSummary$UidDetailTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/BluetoothCallback;-onDeviceAdded-(Lcom/android/settings/bluetooth/CachedBluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/CryptKeeper$ValidationTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/MessagingNotification$3;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Integer;' : ['android.permission.READ_SMS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-start-()V' : ['android.permission.VIBRATE', 'android.permission.DISABLE_KEYGUARD', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH'],
	'Landroid/net/wifi/IWifiManager;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothA2dp;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Landroid/nfc/tech/MifareClassic;-decrement-(I I)V' : ['android.permission.NFC'],
	'Landroid/bluetooth/IBluetooth;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-dialRecipient-()V' : ['android.permission.CALL_PHONE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-onSystemReady-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/mms/ui/SlideViewInterface;-setAudio-(Landroid/net/Uri; Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/activities/PeopleActivity;-onProviderStatusChange-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/AccountManager;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/mms/transaction/SmsSingleRecipientSender;-sendMessage-(J)Z' : ['android.permission.SEND_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-onResume-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/EasOutboxService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-ensurePaired-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-onDestroyActionMode-(Landroid/view/ActionMode;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/Controller$5;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/model/dataitem/DataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebViewClassic;-access$7000-(Landroid/webkit/WebViewClassic;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-access$200-(Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;)V' : ['android.permission.CAMERA'],
	'Lcom/android/bluetooth/btservice/AdapterService;-processConnectOtherProfiles-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothHeadset;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncManager$SyncHandler;-closeActiveSyncContext-(Landroid/content/SyncManager$ActiveSyncContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/NoteDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/ManageCachePage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/calendar/event/EditEventView$TimeListener;-onTimeSet-(Landroid/widget/TimePicker; I I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/Controller;-openTabToHomePage-()Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/widget/BookmarkThumbnailWidgetService$BookmarkFactory;-loadBookmarks-()V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendShowPinReqToFrontApp-(Ljava/lang/String;)Z' : ['android.permission.GET_TASKS'],
	'Lcom/android/contacts/test/FragmentTestActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/view/IWindowManager$Stub;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.DISABLE_KEYGUARD', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-enableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAbsKeyInputView;-resetState-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/tech/NfcB;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/TelephonyRegistry;-broadcastSignalStrengthChanged-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/VideoView;-setVideoURI-(Landroid/net/Uri; Ljava/util/Map;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-access$2100-(Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/PhotoView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/model/dataitem/PhoneDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/DockService;-getCachedBluetoothDevice-(Landroid/bluetooth/BluetoothDevice;)Lcom/android/settings/bluetooth/CachedBluetoothDevice;' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/TransactionSafeActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/ComposeMessageActivity$30;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/fuelgauge/BatteryHistoryDetail;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-reevaluateStatusBarVisibility-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasAccountService;-runPingLoop-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/EasOutboxService;-sendPing-([B I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/ContactsUtils;-areContactWritableAccountsAvailable-(Landroid/content/Context;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/SlideshowView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/bluetooth/IBluetooth;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/util/Recycler;-checkForThreadsOverLimit-(Landroid/content/Context;)Z' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/CallManager;-acceptCall-(Lcom/android/internal/telephony/Call;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pEnabledState;-exit-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$5800-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Landroid/net/wifi/p2p/WifiP2pDevice; Landroid/net/wifi/p2p/WifiP2pConfig;)Z' : ['android.permission.GET_TASKS'],
	'Landroid/view/ViewRootImpl;-dispatchKey-(Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothHealth;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/InCallScreen;-bailOutAfterErrorDialog-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/provider/Browser;-truncateHistory-(Landroid/content/ContentResolver;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/ConnectivityService;-access$1100-(Lcom/android/server/ConnectivityService; Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-createBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/DataUsageSummary;-setMobileDataEnabled-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/DockObserver;-access$000-(Lcom/android/server/DockObserver;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/media/AudioManager;-stopBluetoothSco-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/MediaSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-access$400-(Lcom/android/bluetooth/btservice/BondStateMachine; Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/MenuExecutor$MediaOperation;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Landroid/bluetooth/BluetoothHealth;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/UiccCard;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/IccCardStatus;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/WallpaperManagerService;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/server/pm/PackageManagerService$7;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/NsdService$NsdStateMachine$DisabledState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewBase;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/IWallpaperManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_WALLPAPER_HINTS', 'android.permission.SET_WALLPAPER'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$Handler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-sendMeetingResponse-(J I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$BluetoothStateTracker;-setImageViewResources-(Landroid/content/Context; Landroid/widget/RemoteViews;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getNeighboringCids-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame$7;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/inputmethodservice/KeyboardView;-onLongPress-(Landroid/inputmethodservice/Keyboard$Key;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/view/IWindowSession;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/browser/Tab$SubWindowChromeClient;-onCloseWindow-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/inputmethod/UserDictionaryAddWordContents;-getLocalesList-(Landroid/app/Activity;)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/calendar/agenda/AgendaListView;-initView-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/agenda/AgendaListView$3;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/google/common/io/Files;-copy-(Ljava/io/File; Ljava/nio/charset/Charset; Lcom/google/common/io/OutputSupplier;)V' : ['android.permission.INTERNET'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-startSync-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag;-transceive-(I [B Z)Landroid/nfc/TransceiveResult;' : ['android.permission.NFC'],
	'Landroid/telephony/TelephonyManager;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-finish-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$ErrorState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/calendar/CalendarProvider2;-deleteMatchingColors-(Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.READ_CALENDAR'],
	'Landroid/content/ISyncAdapter$Stub;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/nfc/INfcTag;-rediscover-(I)Landroid/nfc/Tag;' : ['android.permission.NFC'],
	'Landroid/bluetooth/BluetoothAdapter;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/util/ReverseGeocoder;-lookupAddress-(D D Z)Landroid/location/Address;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetooth;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/media/IRingtonePlayer$Stub;-stop-(Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/DeleteEventHelper$5;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/vcard/ImportVCardActivity$VCardScanThread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/list/PhoneFavoriteFragment;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/HTML5VideoView;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UI;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/ConnectivityService;-getActiveNetworkInfoUnfiltered-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetooth;-sendConnectionStateChange-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$SetIpForwardingDisabledErrorState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/systemui/statusbar/tablet/NotificationPanel;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/webkit/WebViewCore;-resumeWebKitDraw-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasAccountService;-parsePingResult-(Ljava/io/InputStream; Landroid/content/ContentResolver; Ljava/util/HashMap;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$5800-(Lcom/android/mms/ui/ComposeMessageActivity; Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/view/ViewRootImpl;-enqueueInputEvent-(Landroid/view/InputEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onApnChanged-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Controller;-onDownloadStart-(Lcom/android/browser/Tab; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS', 'android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/AccessibilityInjector;-injectJavaScript-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothNameDialogFragment;-setDeviceName-(Ljava/lang/String;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/launcher2/Workspace$2;-run-()V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/internal/policy/impl/PhoneFallbackEventHandler;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManagerProxy;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/accounts/AccountManager$3;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/CallNotifier$EmergencyTonePlayerVibrator;-access$300-(Lcom/android/phone/CallNotifier$EmergencyTonePlayerVibrator;)V' : ['android.permission.VIBRATE'],
	'Landroid/net/sip/SipManager;-makeAudioCall-(Ljava/lang/String; Ljava/lang/String; Landroid/net/sip/SipAudioCall$Listener; I)Landroid/net/sip/SipAudioCall;' : ['android.permission.USE_SIP'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-changeVolumeBy-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarUtils$TimeZoneUtils;-getTimeZone-(Landroid/content/Context; Ljava/lang/Runnable;)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/Tab;-<init>-(Lcom/android/browser/WebViewController; Landroid/webkit/WebView; Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/base/ObjectSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/providers/contacts/DbModifierWithNotification;-insert-(Landroid/content/ContentValues;)J' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IIntentReceiver$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/phone/PhoneGlobals;-updatePhoneState-(Lcom/android/internal/telephony/PhoneConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/ProviderStatusWatcher$2;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/transaction/RetrieveTransaction;-sendPdu-(J [B Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage;-initializeData-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-access$1700-(Lcom/android/bluetooth/opp/BluetoothOppService; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/DayView;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/media/AsyncPlayer$Thread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/InputStreamSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStack;-activityPaused-(Landroid/os/IBinder; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mms/ui/QuickContactDivot;-assignContactFromEmail-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/InputMethodManagerService$3;-onUserSwitching-(I Landroid/os/IRemoteCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processUnknownAt-(Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-handleGroupCreationFailure-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetheredIfacePairs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/inputmethod/InputMethodManager;-timeoutEvent-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$ScanningStateChangedHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallbackProxy$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/StateManager;-finishState-(Lcom/android/gallery3d/app/ActivityState; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/Settings$TextToSpeechSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityManagerService;-startProcessLocked-(Ljava/lang/String; Landroid/content/pm/ApplicationInfo; Z I Ljava/lang/String; Landroid/content/ComponentName; Z Z)Lcom/android/server/am/ProcessRecord;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-updateMessageOnIccEf-(I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/webkit/WebViewClassic;-commitTextBatch-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/quicksearchbox/google/AbstractGoogleSource;-getSuggestions-(Ljava/lang/String; I Z)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/EasSyncService;-acknowledgeProvision-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothInputDevice;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/text/ToUpperCase;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/GroupEditorActivity;-onBackPressed-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/power/PowerManagerService;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DataUsageSummary;-onResume-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewInputDispatcher;-dispatchWebKitEvents-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/RequestPermissionHelperActivity;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/CaptivePortalTracker;-isActiveNetwork-(Landroid/net/NetworkInfo;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$AccountSyncSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IThrottleManager$Stub$Proxy;-getCliffLevel-(Ljava/lang/String; I)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/model/dataitem/EventDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/provider/ContactsContract$RawContacts$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/net/NetworkStatsService;-bootstrapStatsLocked-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/adapter/AbstractSyncParser;-commit-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/InCallScreen;-updateIncomingCallWidgetHint-(I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/settings/bluetooth/BluetoothPairingDialog;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/IThrottleManager$Stub;-getHelpUri-()Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/activities/ContactSelectionActivity;-onQueryTextChange-(Ljava/lang/String;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/MediaSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-explicitCallTransfer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfileManager;-<init>-(Landroid/content/Context; Lcom/android/settings/bluetooth/LocalBluetoothAdapter; Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager; Lcom/android/settings/bluetooth/BluetoothEventManager;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebIconDatabaseClassic;-retainIconForPageUrl-(Ljava/lang/String;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/RingtoneManager;-getRingtone-(Landroid/content/Context; Landroid/net/Uri;)Landroid/media/Ringtone;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/ActivityState;-onStateResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/sip/SipEditor;-unregisterProfile-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-sendAccessibilityEventForUnicodeCharacter-(I I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/settings/Settings$AccountSyncSettingsInAddAccountActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserWebView;-findAll-(Ljava/lang/String;)I' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CheckedTextView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-attachApplicationLocked-(Landroid/app/IApplicationThread; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetooth$Stub;-setPin-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/ChooseLockPassword$ChooseLockPasswordFragment;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-onRemoveViewAnimationCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/ActivityManagerNative;-registerReceiver-(Landroid/app/IApplicationThread; Ljava/lang/String; Landroid/content/IIntentReceiver; Landroid/content/IntentFilter; Ljava/lang/String; I)Landroid/content/Intent;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/nfc/INfcTag;-ndefMakeReadOnly-(I)I' : ['android.permission.NFC'],
	'Lcom/android/browser/BrowserWebViewFactory;-createWebView-(Z)Landroid/webkit/WebView;' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Landroid/view/IWindowSession$Stub$Proxy;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManagerService;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/data/WorkingMessage;-asyncDeleteDraftSmsMessage-(Lcom/android/mms/data/Conversation;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/app/backup/IBackupManager;-opComplete-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-getSubscriptionInfoAndStartPollingThreads-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/IConnectivityManager;-isNetworkSupported-(I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaSetsFromString-(Ljava/lang/String;)[Lcom/android/gallery3d/data/MediaSet;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/accounts/AccountPreferenceBase;-onResume-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/ComboAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-startDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfileManager$PanStateChangedHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/net/NetworkStatsService;-access$1000-(Lcom/android/server/net/NetworkStatsService; [I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService$19;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$LinkMonitoringState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-access$900-(Lcom/android/gallery3d/app/MoviePlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewCore;-access$2400-(Landroid/webkit/WebViewCore; Landroid/webkit/WebViewClassic$ViewSizeData;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/SharpenEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-sendConnectionStateChange-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mms/ui/ComposeMessageActivity$BackgroundQueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-selectDefaultContact-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/InputMethodManagerService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/RedEyeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/list/ContactEntryListAdapter;-getContactUri-(I Landroid/database/Cursor; I I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-delete-(Landroid/net/Uri; I J)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/widget/DialogTitle;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/support/v4/net/ConnectivityManagerCompat$ConnectivityManagerCompatImpl;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewProvider;-pauseTimers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$TextToSpeechSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/CalendarController$EventHandler;-eventsChanged-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/view/inputmethod/InputMethodManager$1;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/contacts/list/PhoneNumberListAdapter;-getDataUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/tv/TvStatusBar;-toggleRecentsActivity-()V' : ['android.permission.GET_TASKS'],
	'Lcom/android/contacts/group/GroupEditorFragment;-access$600-(Lcom/android/contacts/group/GroupEditorFragment; Landroid/database/Cursor;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiManager;-setCountryCode-(Ljava/lang/String; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/settings/Settings$AccessibilitySettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-setDiscoverableTimeout-(I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/nfc/tech/NfcA;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/settings/tts/TtsEngineSettingsFragment;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-setQueuePosition-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$KeyboardLayoutPickerActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/list/ContactPickerFragment;-finish-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/camera/ui/MoreSettingPopup;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/DevelopmentSettings;-access$100-(Lcom/android/settings/DevelopmentSettings;)V' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/server/ConnectivityService;-access$2200-(Lcom/android/server/ConnectivityService; I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/location/ILocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/model/ContactLoader;-loadDirectoryMetaData-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-addStatusBarWindow-()V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothDevice;-setPin-([B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/PbapServerProfile;-getConnectionStatus-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/power/PowerManagerService$1;-onProximityNegative-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/test/mock/MockContentResolver;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/content/SyncManager;-sendCheckAlarmsMessage-()V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/power/DisplayBlanker;-unblankAllDisplays-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PatternUnlockScreen;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/editor/TextFieldsEditorView;-onFieldChanged-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-pullVcardEntry-([B Lcom/android/bluetooth/pbap/BluetoothPbapObexServer$AppParamValue; Ljavax/obex/Operation; Ljava/lang/String; Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/gallery3d/app/TrimVideo;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-show-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/SlideshowPresenter;-presentSlide-(Lcom/android/mms/ui/SlideViewInterface; Lcom/android/mms/model/SlideModel;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$RunningServicesActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/IccCardProxy;-broadcastIccStateChangedIntent-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Controller;-closeCurrentTab-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/os/PowerManager$WakeLock;-release-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab;-setController-(Lcom/android/browser/WebViewController;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/IAudioService$Stub;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/contacts/calllog/ClearCallLogDialog$1$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/VideoView$6;-surfaceCreated-(Landroid/view/SurfaceHolder;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/view/IWindowManager$Stub$Proxy;-setForcedDisplaySize-(I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/RetargetFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/PhoneGlobals;-initForNewRadioTechnology-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/RuimRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService$SettingsObserver;-onChange-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/debug/ContactsDumpActivity$DumpDbTask;-doInBackground-([Ljava/lang/Void;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/launcher2/WallpaperChooserDialogFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ActivityBase$MyAppBridge;-access$200-(Lcom/android/camera/ActivityBase$MyAppBridge; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/InCallScreen;-delayedCleanupAfterDisconnect-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView$2;-onDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/event/EditEventFragment;-access$600-(Lcom/android/calendar/event/EditEventFragment; I)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/sip/SipAudioCall;-access$700-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/BrowserWebView;-<init>-(Landroid/content/Context;)V' : ['android.permission.INTERNET'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/alerts/AlertService;-rescheduleMissedAlarms-(Landroid/content/ContentResolver; Landroid/content/Context; Lcom/android/calendar/alerts/AlarmManagerInterface;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/http/RequestQueue;-enablePlatformNotifications-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/INetworkStatsService$Stub$Proxy;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/EmergencyDialer;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/WifiService;-startScan-(Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/settings/Settings$DataUsageSummaryActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-access$600-(Lcom/android/gallery3d/ui/MenuExecutor; Lcom/android/gallery3d/data/DataManager; Lcom/android/gallery3d/util/ThreadPool$JobContext; I Lcom/android/gallery3d/data/Path;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/statusbar/tv/TvStatusBar;-userSwitched-(I)V' : ['android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-exitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-addAccount-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/display/DisplayManagerService;-handleDisplayDeviceAdded-(Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/GroupMembershipDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/BluetoothPhoneService;-access$300-(Lcom/android/phone/BluetoothPhoneService;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/NetworkPolicyManagerService$8;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/ZoomManager$PostScale;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/SlideViewInterface;-stopVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserWebView;-saveWebArchive-(Ljava/lang/String; Z Landroid/webkit/ValueCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/speech/IRecognitionService$Stub$Proxy;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/gallery3d/app/PhotoPage;-requestDeferredUpdate-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/EmptyAlbumImage;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserActivity;-onLowMemory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-processCpbrCommand-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/gallery3d/onetimeinitializer/GalleryWidgetMigrator;-migrateGalleryWidgets-(Landroid/content/Context;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManager;-updateCredentials-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-setRepeatMode-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserSettings;-resetDefaultPreferences-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/EventDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/WiredAccessoryManager;-setDeviceStateLocked-(I I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-loadUrlFromContext-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-startDelayedRetry-(Lcom/android/internal/telephony/DataConnection$FailCause; Lcom/android/internal/telephony/ApnContext; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$NfcAdapterExtrasService;-transceive-(Ljava/lang/String; [B)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Landroid/filterpacks/base/GLTextureTarget;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/NfcAdapter;-disableForegroundNdefPush-(Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Lcom/android/browser/AddBookmarkPage;-completeOrCancelFolderNaming-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/IccCardProxy;-supplyPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/Transaction;-notifyObservers-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/PhotoPage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/am/ActivityManagerService;-handleAppCrashLocked-(Lcom/android/server/am/ProcessRecord;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/ISms$Stub;-enableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/base/InputStreamSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/AccessibilityInjector;-onPageFinished-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$AccountSyncSettingsInAddAccountActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardMessageArea$2;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothInputDevice;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/PeopleActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-getSuggestions-(Ljava/lang/String; I Z)Lcom/android/quicksearchbox/SuggestionCursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/SettingsLicenseActivity$3;-onUnhandledKeyEvent-(Landroid/webkit/WebView; Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService$DisplayBlankerImpl;-blankAllDisplays-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-userSwitched-(I)V' : ['android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-notifyNoData-(Lcom/android/internal/telephony/DataConnection$FailCause;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/voicedialer/VoiceContact;-getVoiceContacts-(Landroid/app/Activity;)Ljava/util/List;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/ui/PhotoView;-snapToNeighborImage-()Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CallerInfoAsyncQuery;-startQuery-(I Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/CallerInfoAsyncQuery$OnQueryCompleteListener; Ljava/lang/Object;)Lcom/android/internal/telephony/CallerInfoAsyncQuery;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/providers/downloads/StorageManager;-findSpace-(Ljava/io/File; J I)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue;-releaseAllPointers-(J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterfw/core/Filter;-process-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/list/PhoneNumberListAdapter;-bindQuickContact-(Lcom/android/contacts/list/ContactListItemView; I Landroid/database/Cursor; I I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/soundrecorder/SoundRecorder;-onStop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage$SequentialSource;-getMediaItem-(I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebViewProvider$ViewDelegate;-performLongClick-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/location/GeofenceManager;-scheduleUpdateFencesLocked-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-shutdown-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/webkit/WebViewClassic;-pauseTimers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupWaitingOrBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-openTab-(Ljava/lang/String; Lcom/android/browser/Tab; Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/webkit/WebViewClassic;-pasteFromClipboard-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/users/ProfileUpdateReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-startRequestProcessing-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/BluetoothPairingDialog;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/development/Connectivity;-onStartDelayedCycle-()V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimUnlockScreen;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/data/DataManager;-getContentUri-(Lcom/android/gallery3d/data/Path;)Landroid/net/Uri;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub$Proxy;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-requestRemoveProviderInfoWithDelay-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/PhoneUtils;-answerAndEndActive-(Lcom/android/internal/telephony/CallManager; Lcom/android/internal/telephony/Call;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/systemui/statusbar/policy/NetworkController;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewClassic;-access$3200-(Landroid/webkit/WebViewClassic; I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-interceptFallback-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.CALL_PHONE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-dispatch-(Landroid/content/Intent; Ljava/lang/String; Landroid/content/BroadcastReceiver;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-requestChildRectangleOnScreen-(Landroid/view/View; Landroid/graphics/Rect; Z)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth;-getRemoteName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/model/dataitem/RelationDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/content/IContentService;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleCancel-(Landroid/view/MotionEvent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processKeyPressed-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-cancelTransitionsInProgress-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumPage;-onUpPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/webkit/WebIconDatabaseClassic$EventHandler;-access$500-(Landroid/webkit/WebIconDatabaseClassic$EventHandler; Landroid/os/Message;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-handleRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipAudioCall$1;-onCallBusy-(Landroid/net/sip/SipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab;-putInForeground-()V' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/calendar/AllInOneActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/CalendarContract$EventsEntity$EntityIteratorImpl;-getEntityAndIncrementCursor-(Landroid/database/Cursor;)Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/EasSyncService;-tryAutodiscover-(Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/OtaUtils;-otaPerformActivation-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/downloads/DownloadService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendP2pPersistentGroupsChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/DayView$CalendarGestureListener;-onScroll-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/AdnRecordLoader;-loadAllFromEF-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView$FullPicture;-draw-(Lcom/android/gallery3d/ui/GLCanvas; Landroid/graphics/Rect;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/vcard/VCardParserImpl_V30;-parseOneVCard-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/calendar/CalendarAlarmManager;-rescheduleMissedAlarms-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getMobileDataEnabled-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/vcard/VCardParserImpl_V21;-parseItem-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-albumShouldOpenInFilmstrip-(Lcom/android/gallery3d/data/MediaSet;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/vcard/ImportVCardActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/browser/Tab$3;-onRequestFocus-(Landroid/webkit/WebView;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/accounts/IAccountManager;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/accounts/AccountManager$5;-doWork-()V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-<init>-(Landroid/os/Handler; Landroid/content/Context;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/settings/SecuritySettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/RingtoneManager;-getRingtone-(Landroid/content/Context; Landroid/net/Uri; I)Landroid/media/Ringtone;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/CallerInfoCache;-refreshCacheEntry-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/WebViewCore;-sendMessageAtFrontOfQueue-(I Ljava/lang/Object;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/TrimVideo$2;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$TextToSpeechSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/provider/ContactsContract$SyncState;-newSetOperation-(Landroid/accounts/Account; [B)Landroid/content/ContentProviderOperation;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-sendMessage-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/nfc/handover/HandoverTransfer;-finishTransfer-(Z Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/AllowBindAppWidgetActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/internal/view/IInputMethodClient$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/PhotoPage;-onActionBarWanted-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/Settings$WifiSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/util/ThumbnailManager;-getThumbnail-(Landroid/net/Uri; Lcom/android/mms/util/ItemLoadedCallback;)Lcom/android/mms/util/ItemLoadedFuture;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/IContentService;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/contacts/group/GroupEditorFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebViewProvider;-reload-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView$2;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/RadioInfo$12;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/filterpacks/base/ObjectSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-disable-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter;-isSyncable-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/EditableInputConnection;-setComposingRegion-(I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/sip/SipSettings$1;-run-()V' : ['android.permission.USE_SIP'],
	'Lcom/android/phone/InCallControlState;-update-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hdp/HealthService;-getHealthDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/power/PowerManagerService;-handleBatteryStateChangedLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/UiccCardApplication;-<init>-(Lcom/android/internal/telephony/UiccCard; Lcom/android/internal/telephony/IccCardApplicationStatus; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/InCallScreen;-dontAddVoiceMailNumber-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/contacts/calllog/DefaultVoicemailNotifier$NameLookupQuery;-query-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/editor/ContactEditorUtils;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Ljava/lang/String; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/ChangeIccPinScreen$2;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback;-pokeWakelock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/app/ActivityManagerNative;-getTasks-(I I Landroid/app/IThumbnailReceiver;)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Landroid/filterpacks/base/RetargetFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/DockService$1;-run-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/DreamSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/OutgoingCallBroadcaster;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.PROCESS_OUTGOING_CALLS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$DeviceInfoSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/TwilightService;-systemReady-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-updateParams-(Ljava/util/HashMap;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/wm/WindowManagerService;-clearForcedDisplayDensity-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/NfcService$TagService;-ndefMakeReadOnly-(I)I' : ['android.permission.NFC'],
	'Lcom/android/browser/Controller$SelectText;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/research/ResearchLogger;-scrubWord-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/WebViewCore;-sendMessage-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/TransactionSafeActivity;-onAttachFragment-(Landroid/app/Fragment;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/SlideshowPresenter;-presentRegionMedia-(Lcom/android/mms/ui/SlideViewInterface; Lcom/android/mms/model/RegionMediaModel; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/BluetoothPhoneService$3;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/WifiService;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/backup/IBackupManager$Stub;-opComplete-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/os/PowerManager$WakeLock;-acquire-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity;-access$800-(Lcom/android/contacts/activities/ConfirmAddDetailActivity; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/nfc/INfcAdapterExtras;-getDriverName-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.NFC'],
	'Lcom/android/emailcommon/Device;-getConsistentDeviceId-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-processPostDialChar-(C)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/CropView$FaceHighlightView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/location/GpsLocationProvider;-disable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/IWindowManager;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.DISABLE_KEYGUARD', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardMessageArea$2;-onKeyguardVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$SecuritySettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/media/effect/EffectContext;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-getAccounts-()[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebViewProvider;-removeJavascriptInterface-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$ApplicationSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/content/IContentService$Stub;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManagerProxy;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/content/ISyncAdapter$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager;-readPairedDevices-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/deskclock/AlarmClock$AlarmItemAdapter;-expandAlarm-(Lcom/android/deskclock/AlarmClock$AlarmItemAdapter$ItemHolder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Preloader;-cancelSearchBoxPreload-(Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/webkit/WebView;-destroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-updatePersistentNetworks-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/DuotoneEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/provider/Contacts$People;-markAsContacted-(Landroid/content/ContentResolver; J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/JoinContactListFragment;-setSearchMode-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/commands/am/Am;-runDisplayDensity-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/sip/SipSettings;-access$800-(Lcom/android/phone/sip/SipSettings;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/browser/Controller;-closeCurrentTab-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/content/SyncManager$SyncHandler;-runSyncFinishedOrCanceledLocked-(Landroid/content/SyncResult; Landroid/content/SyncManager$ActiveSyncContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-isSyncActive-()Z' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/group/GroupEditorFragment$2;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/CursorLoader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setNetworkSelectionModeManual-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/pm/IPackageManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CLEAR_APP_CACHE', 'android.permission.READ_EXTERNAL_STORAGE', 'android.permission.GET_PACKAGE_SIZE'],
	'Landroid/nfc/tech/MifareUltralight;-getTimeout-()I' : ['android.permission.NFC'],
	'Lcom/android/contacts/detail/ContactDetailUpdatesFragment;-access$000-(Lcom/android/contacts/detail/ContactDetailUpdatesFragment; Lcom/android/contacts/util/StreamItemEntry;)Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/BluetoothManagerService;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/UrlInputView$UrlInputListener;-onAction-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/view/IApplicationToken;-windowsVisible-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/BrowserActivity;-onPause-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-<init>-(Landroid/content/Context; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewCallback; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor; Lcom/android/internal/widget/LockPatternUtils; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardWindowController;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView$1;-onCodeInput-(I I I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/sip/SipService$MyExecutor;-executeInternal-(Ljava/lang/Runnable;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-systemReady-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter$EasContactsSyncParser;-commandsParser-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/IConnectivityManager;-getActiveNetworkQuotaInfo-()Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipCall;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/ims/IsimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifyServiceState-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/Ringtone;-play-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IThrottleManager;-getResetTime-(Ljava/lang/String;)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$WifiP2pSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-setDataSourceImpl-(Landroid/media/MediaPlayer; Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/RestoreSession;-getAvailableRestoreSets-(Landroid/app/backup/RestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Contacts$People;-addToGroup-(Landroid/content/ContentResolver; J J)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/os/IVibratorService;-vibrate-(J Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/IAccountManager;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/settings/Settings$ManageApplicationsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothDevice;-cancelBondProcess-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-resetMessage-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/bluetooth/HidProfile;-isPreferred-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/LocalBluetoothPreferences;-shouldShowDialogInForeground-(Landroid/content/Context; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/FindActionModeCallback;-findAll-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity;-startExtraInfoQuery-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/calendar/alerts/AlertAdapter;-bindView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onRadioOffOrNotAvailable-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcTag$Stub;-ndefMakeReadOnly-(I)I' : ['android.permission.NFC'],
	'Lcom/android/gallery3d/data/Clustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/statusbar/policy/AutoRotateController;-<init>-(Landroid/content/Context; Landroid/widget/CompoundButton; Lcom/android/systemui/statusbar/policy/AutoRotateController$RotationLockCallbacks;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewProvider;-findAllAsync-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$5300-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; Ljava/lang/String;)V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/server/wm/WindowManagerService;-displayReady-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/wifi/WifiSettingsForSetupWizardXL;-refreshAccessPoints-(Z)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/PreloadRequestReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-showNextSecurityScreenOrFinish-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-composeAndSendPhonebookOneVcard-(Ljavax/obex/Operation; I Z Ljava/lang/String; I)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-onDreamingStarted-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/app/IBatteryStats;-getStatistics-()[B' : ['android.permission.BATTERY_STATS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton;-onFinishInflate-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/AudioService$AudioHandler$1;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-onBluetoothStateChanged-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/ims/IsimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/dreams/phototable/LocalSource;-next-(Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/SuggestionsAdapter$SuggestCursor;-runQuery-(Ljava/lang/CharSequence;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/EventLoader;-startBackgroundThread-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/DayView;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/wm/WindowAnimator;-updateAnimToLayoutLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView$2;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/GlobalSearchSupport;-handleSearchSuggestionsQuery-(Landroid/database/sqlite/SQLiteDatabase; Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/UiController;-createNewSnapshotTab-(J Z)Lcom/android/browser/SnapshotTab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity$AccountSet;-buildDiff-()Ljava/util/ArrayList;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/location/GpsLocationProvider$3;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothHeadset;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/CallCard;-updatePhotoForCallState-(Lcom/android/internal/telephony/Call;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-cancelBondProcess-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Lcom/android/settings/Utils;-getProfileDisplayName-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/ui/MessageListAdapter;-bindView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/BluetoothAdapter;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/UiModeManagerService;-access$500-(Lcom/android/server/UiModeManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-scrollRight-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-confirmCredentials-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/downloads/DownloadService;-scanFile-(Lcom/android/providers/downloads/DownloadInfo; Z Z)Z' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/server/ThrottleService$MyHandler;-clearThrottleAndNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/systemui/statusbar/phone/SettingsPanelView;-updateResources-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-access$400-(Lcom/android/gallery3d/data/LocalAlbumSet; Lcom/android/gallery3d/data/DataManager; I Lcom/android/gallery3d/data/Path; I Ljava/lang/String;)Lcom/android/gallery3d/data/MediaSet;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/GpsLocationProvider;-updateNetworkState-(I Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/ContactsContract$SyncState;-get-(Landroid/content/ContentProviderClient; Landroid/accounts/Account;)[B' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-disableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/group/GroupEditorFragment$3;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/SMSDispatcher;-processMessagePart-([B Ljava/lang/String; I I I J I Z)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-loadMessagesAndDraft-(I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$3000-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setFacilityLock-(Ljava/lang/String; Z Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserSettings;-onSharedPreferenceChanged-(Landroid/content/SharedPreferences; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/pm/PackageManagerService$5;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.REORDER_TASKS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.WAKE_LOCK', 'android.permission.GET_TASKS', 'android.permission.WRITE_SETTINGS', 'android.permission.RESTART_PACKAGES'],
	'Landroid/media/effect/effects/DuotoneEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-dispatchMediaKeyWithWakeLock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-handleCpbsCommand-(Ljava/lang/String; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/browser/BrowserWebView;-saveWebArchive-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText$1;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity;-startDisambiguationQuery-(Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/app/IActivityManager;-killApplicationWithAppId-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-notifyNoData-(Lcom/android/internal/telephony/DataConnection$FailCause; Lcom/android/internal/telephony/ApnContext;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/Settings$DevelopmentSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/HTML5VideoView;-prepareDataAndDisplayMode-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewInputDispatcher;-dispatchUiEvent-(Landroid/view/MotionEvent; I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/TintFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SMSDispatcher;-dispatch-(Landroid/content/Intent; Ljava/lang/String; Landroid/content/BroadcastReceiver;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/ContactsListActivity;-getContactUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$EventDays;-query-(Landroid/content/ContentResolver; I I [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$DeviceFoundHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/AudioService;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-doVibratorOn-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/systemui/recent/RecentTasksLoader$3;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnection;-access$2000-(Lcom/android/internal/telephony/DataConnection; Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/voicedialer/RecognizerEngine;-recognize-(Lcom/android/voicedialer/RecognizerClient; Landroid/app/Activity; Ljava/io/File; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/DataController;-getInstance-(Landroid/content/Context;)Lcom/android/browser/DataController;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/phone/CallerInfoCache;-init-(Landroid/content/Context;)Lcom/android/phone/CallerInfoCache;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DeleteEventHelper;-access$900-(Lcom/android/calendar/DeleteEventHelper;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar$23$1$3;-run-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/model/dataitem/WebsiteDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/PhoneUtils;-activateSpeakerIfDocked-(Lcom/android/internal/telephony/Phone;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/GroupEditorActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallEnded-(Lcom/android/internal/telephony/Connection$DisconnectCause;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService$Stub;-startBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/calendar/CalendarAlarmManager;-rescheduleMissedAlarms-(Landroid/content/ContentResolver;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity$BluetoothTransferContentObserver;-onChange-(Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-setPostDialState-(Lcom/android/internal/telephony/Connection$PostDialState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$AccountSyncSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/EmailAddressAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/AddBookmarkPage$UpdateBookmarkTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/webkit/WebViewClassic;-clearCache-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/DownloadCache$DownloadTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/model/dataitem/OrganizationDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupCreatedState;-exit-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/calllog/CallLogFragment;-setMenuVisibility-(Z)V' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/nfc/tech/MifareClassic;-authenticateSectorWithKeyA-(I [B)Z' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifySignalStrength-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/systemui/statusbar/policy/BluetoothController;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onVoiceCallStarted-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/soundrecorder/Recorder$OnStateChangedListener;-onStateChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/LocationSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/CsimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-copyMedia-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/RIL;-getGsmBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onSingleTapUp-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/BrowserWebView;-destroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/PhoneDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/TabControl;-createNewWebView-()Landroid/webkit/WebView;' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Landroid/webkit/AccessibilityInjectorFallback;-onKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/Browser;-getVisitedHistory-(Landroid/content/ContentResolver;)[Ljava/lang/String;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/exchange/EmailSyncAdapterService;-performSync-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle; Ljava/lang/String; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/TetherSettings;-access$300-(Lcom/android/settings/TetherSettings;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar$H$1;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton;-onAttachedToWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/tech/NfcB;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/contacts/model/dataitem/RelationDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-operationComplete-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/NetInitiatedActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/browser/BaseUi;-<init>-(Landroid/app/Activity; Lcom/android/browser/UiController;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/vcard/VCardParser_V30;-parse-(Ljava/io/InputStream; Lcom/android/vcard/VCardInterpreter;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/MessageUtils$3$1;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/PhotoPage;-showBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/BlendFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothPan$Stub$Proxy;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/latin/LatinIME;-handleBackspace-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/WebViewProvider;-findAll-(Ljava/lang/String;)I' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/data/WorkingMessage;-access$1500-(Lcom/android/mms/data/WorkingMessage; Lcom/android/mms/data/Conversation; Z)V' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-onDetachedFromWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/TagClustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-fireDataChange-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/inputmethod/SpellCheckersSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/ITelephony;-getCellLocation-()Landroid/os/Bundle;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/tech/MifareClassic;-getTimeout-()I' : ['android.permission.NFC'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/telephony/gsm/SmsManager;-sendMultipartTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$5600-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/phone/NotificationMgr;-updateNotificationsAtStartup-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Landroid/webkit/WebViewClassic;-resumeTimers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-saveRingtone-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/Gallery;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/accounts/AccountPreferenceBase;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/VideoView;-setVideoURI-(Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/ZoomControlEmbedded$ZoomListener;-onVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/videosrc/MediaSource;-process-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-loadDataWithBaseURL-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$UserDictionarySettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/InCallScreen;-showPausePromptDialog-(Lcom/android/internal/telephony/Connection; Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-deleteInTransaction-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/AccessibilitySettings$ToggleScreenMagnificationPreferenceFragment;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterfw/core/SyncRunner;-performStep-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/PhoneInterfaceManager;-silenceRingerInternal-()V' : ['android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/power/Notifier;-onGoToSleepFinished-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-getTimeout-(I)I' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/group/GroupEditorFragment;-setupEditorForAccount-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-wakeWhenReadyTq-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcAdapterExtras$Stub;-transceive-(Ljava/lang/String; [B)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/contacts/editor/EventFieldEditorView;-notifyEditorListener-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-access$300-(Lcom/android/contacts/editor/ContactEditorFragment; Landroid/content/Context; Lcom/android/contacts/editor/RawContactEditorView;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/PhoneApp;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityStack;-moveHomeToFrontFromLaunchLocked-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/tv/TvStatusBar;-start-()V' : ['android.permission.VIBRATE', 'android.permission.DISABLE_KEYGUARD', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH'],
	'Landroid/filterfw/core/FieldPort;-transfer-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/DeskClock$TabsAdapter;-notifyPageChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/RadioInfo;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserWebView;-setBackgroundColor-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterfw/core/FieldPort;-setFieldFrame-(Landroid/filterfw/core/Frame; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothHeadset;-isDisabled-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/view/IWindowManager$Stub$Proxy;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-setup-(Lcom/android/systemui/statusbar/policy/NetworkController; Lcom/android/systemui/statusbar/policy/BluetoothController; Lcom/android/systemui/statusbar/policy/BatteryController; Lcom/android/systemui/statusbar/policy/LocationController;)V' : ['android.permission.BLUETOOTH', 'android.permission.DISABLE_KEYGUARD', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView$KeyTimerHandler;-startLongPressTimer-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/IIntentReceiver;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/Settings$DisplaySettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/net/NetworkStatsService;-performPoll-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$TetherSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-untether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/UrlInputView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/NullFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/CameraSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/IBluetooth$Stub;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiManager;-isWifiApEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/webkit/WebViewInputDispatcher;-enqueueUiEventLocked-(Landroid/webkit/WebViewInputDispatcher$DispatchEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$BluetoothSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/transaction/SmsReceiverService;-messageFailedToSend-(Landroid/net/Uri; I)V' : ['android.permission.READ_SMS'],
	'Lcom/android/mms/ui/ManageSimMessages$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/ExchangeService;-releaseMailbox-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-installSystemProviders-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub;-createSocketChannel-(I Ljava/lang/String; Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/FindActionModeCallback;-onDestroyActionMode-(Landroid/view/ActionMode;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$9;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-access$7200-(Landroid/webkit/WebViewClassic; Landroid/view/MotionEvent; I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/IWindowManager;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/gallery3d/app/PhotoPage;-toggleBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$3200-(Lcom/android/gallery3d/ui/PhotoView; I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-setCurrentPhoto-(Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onLongPressTimeout-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/WifiService;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/DockService;-handleDocked-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter$EasContactsSyncParser;-changeParser-(Lcom/android/exchange/adapter/ContactsSyncAdapter$ContactOperations;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/accounts/AccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextClock;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$5900-(Lcom/android/mms/ui/ComposeMessageActivity; Ljava/lang/String; Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/util/Recycler$MmsRecycler;-deleteOldMessages-(Landroid/content/Context;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/IccRecords;-setVoiceCallForwardingFlag-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventFragment;-onDestroy-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-getMediaItemFromPath-(Ljava/util/ArrayList; I I Lcom/android/gallery3d/data/DataManager;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/timer/TimerFragment;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/gsm/SIMRecords;-setVoiceCallForwardingFlag-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setTimeFromNITZString-(Ljava/lang/String; J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/contacts/editor/StructuredNameEditorView;-notifyEditorListener-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/OtaUtils;-setSpeaker-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/power/Notifier;-sendWakeUpBroadcast-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/speech/SpeechRecognizer;-handleStartListening-(Landroid/content/Intent;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/net/INetworkStatsService$Stub;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/VideoModule;-onResumeAfterSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/StateManager;-restoreFromState-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-onGenericMotionEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/Contact;-getFirstWritableRawContactId-(Landroid/content/Context;)J' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/CallerInfoCache$CacheAsyncTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthListView;-init-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/server/UiModeManagerService;-updateLocked-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getLastTetherError-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/activities/GroupDetailActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/calendar/alerts/AlertService;-generateAlerts-(Landroid/content/Context; Lcom/android/calendar/alerts/NotificationMgr; Lcom/android/calendar/alerts/AlarmManagerInterface; Landroid/content/SharedPreferences; Landroid/database/Cursor; J I)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/accounts/AccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/ActivityManager;-getRecentTasksForUser-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/IccCardProxy;-supplyPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/StateManager;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/wifi/WifiManager;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/power/Notifier$1;-onScreenOn-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/contacts/socialwidget/SocialWidgetProvider;-access$000-(Landroid/content/Context; I Landroid/appwidget/AppWidgetManager; Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/agenda/AgendaAdapter;-bindView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/debug/ContactsDumpActivity$DumpDbTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/quicklaunch/QuickLaunchSettings;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine;-registerForWatchdogToggle-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory$2;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/app/ActivityManagerNative;-setProcessLimit-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/Settings$ApplicationSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$WifiSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/detail/ContactDetailUpdatesFragment$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothAdapter;-closeProfileProxy-(I Landroid/bluetooth/BluetoothProfile;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-performEditorAction-(I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$CryptKeeperSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendThisDeviceChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/DeskClock;-registerPageChangedListener-(Lcom/android/deskclock/DeskClockFragment;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioManager;-startBluetoothSco-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/provider/CalendarContract$CalendarEntity$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/selectcalendars/SelectCalendarsSyncFragment;-onPause-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/policy/impl/GlobalActions$4;-onItemLongClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)Z' : ['android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/view/IWindowSession$Stub$Proxy;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/util/Recycler;-getAllThreads-(Landroid/content/Context;)Landroid/database/Cursor;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Landroid/net/http/RequestQueue$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/CallNotifier;-onCdmaCallWaitingReject-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/wifi/WifiSettingsForSetupWizardXL;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/KeyguardManager;-exitKeyguardSecurely-(Landroid/app/KeyguardManager$OnKeyguardExitResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.DISABLE_KEYGUARD', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE', 'android.permission.WRITE_SETTINGS', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-clearAll-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/transaction/TransactionService;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/model/SlideshowModel;-getPduBody-(Landroid/content/Context; Landroid/net/Uri;)Lcom/google/android/mms/pdu/PduBody;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/test/FragmentTestActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/wm/WindowManagerService;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/server/am/ActivityManagerService;-handleAppDiedLocked-(Lcom/android/server/am/ProcessRecord; Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setPairingConfirmation-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/server/net/NetworkStatsService;-getNetworkStatsUidDetail-()Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/DayView;-switchViews-(Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/ITelephony;-isSimPinEnabled-()Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/IAlarmManager$Stub;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/camera/ui/ListPrefSettingPopup;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/list/ContactPickerFragment;-onItemClick-(I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$StartTetheringErrorState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/TetherSettings;-startTethering-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/PhotoPage;-switchToGrid-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/content/ContextWrapper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioManager;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/browser/BrowserWebView;-reload-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-drawContent-(Landroid/graphics/Canvas;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/DataUsageSummary;-hasEthernet-(Landroid/content/Context;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/inputmethod/UserDictionaryList;-onResume-()V' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-startDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/PeopleActivity$ContactDetailLoaderFragmentListener$1;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/location/ILocationManager$Stub;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView;-onResume-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/IMediaPlaybackService;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage$ShuffleSource;-getMediaItem-(I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-connect-(Landroid/net/wifi/p2p/WifiP2pConfig; Z)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS'],
	'Lcom/google/common/io/ByteStreams$5;-getInput-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-keyPressed-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity;-access$1200-(Lcom/android/contacts/activities/ConfirmAddDetailActivity;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/accounts/AccountManagerService;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-loadDictionaryAsync-()V' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/contacts/list/ShortcutIntentBuilder$LoadingAsyncTask;-loadPhoto-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/accounts/AccountManager$Future2Task;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ConversationList;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/base/FrameSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5VideoView;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavScreen;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/IBluetooth$Stub;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/launcher2/PagedViewWithDraggableItems;-beginDragging-(Landroid/view/View;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/wallpaper/RenderScriptScene;-start-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPINView;-verifyPasswordAndUnlock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$BluetoothSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/SlideshowEditActivity$SlideListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/SipCallOptionHandler;-access$300-(Lcom/android/phone/SipCallOptionHandler; Landroid/net/sip/SipProfile;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.USE_SIP'],
	'Lcom/android/mms/data/WorkingMessage;-prepareForSave-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-onUuidChanged-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/calllog/CallLogQueryHandler;-fetchCalls-(I I Z I)V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/location/fused/FusionEngine;-setRequest-(Lcom/android/location/provider/ProviderRequestUnbundled; Landroid/os/WorkSource;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/list/ContactPickerFragment;-setQueryString-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/alerts/AlertActivity;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/input/InputManagerService;-registerPointerSpeedSettingObserver-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/AllInOneActivity;-onUserLeaveHint-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/CommandsInterface;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-updateParams-(Ljava/util/HashMap;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$2;-run-()V' : ['android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/webkit/WebView;-saveWebArchive-(Ljava/lang/String; Z Landroid/webkit/ValueCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/month/MonthListView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/AlbumPage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pEnabledState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/downloads/StorageManager;-verifySpace-(I Ljava/lang/String; J)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-isSyncActive-()Z' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-stopSample-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$3;-dismiss-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/group/GroupBrowseListFragment;-requestSelectionToScreen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/DayView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/settings/DateTimeSettingsSetupWizard;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/wallpaper/nexus/NexusRS;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/downloads/DownloadThread;-processResponseHeaders-(Lcom/android/providers/downloads/DownloadThread$State; Lcom/android/providers/downloads/DownloadThread$InnerState; Lorg/apache/http/HttpResponse;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/agenda/AgendaFragment;-onResume-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/VibratorService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/inputmethod/research/UploaderService;-doUpload-(Z)V' : ['android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-reloadButtons-()V' : ['android.permission.GET_TASKS'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneFallbackEventHandler;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSimPukView;-onWindowFocusChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/support/v4/net/ConnectivityManagerCompatHoneycombMR2;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/nfc/tech/BasicTagTechnology;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/providers/contacts/ProfileProvider;-insertInTransaction-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_PROFILE'],
	'Landroid/net/INetworkPolicyManager$Stub;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/launcher2/Launcher;-showFirstRunWorkspaceCling-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/PhoneUtils;-hangup-(Lcom/android/internal/telephony/CallManager;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Landroid/content/ISyncAdapter$Stub;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/wm/Session;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ConversationList;-confirmDeleteThread-(J Landroid/content/AsyncQueryHandler;)V' : ['android.permission.READ_SMS'],
	'Landroid/speech/SpeechRecognizer;-stopListening-()V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/providers/downloads/DownloadReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/StatusBarManagerService;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/Settings$KeyboardLayoutPickerActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/SecureSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/INfcAdapterExtras$Stub$Proxy;-open-(Ljava/lang/String; Landroid/os/IBinder;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Landroid/telephony/TelephonyManager;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/preference/YesNoPreference;-onDialogClosed-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/systemui/statusbar/BaseStatusBar$2;-onClickHandler-(Landroid/view/View; Landroid/app/PendingIntent; Landroid/content/Intent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/ex/variablespeed/MediaPlayerProxy;-prepare-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onResetDone-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/BlendFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/display/WifiDisplayController;-<init>-(Landroid/content/Context; Landroid/os/Handler; Lcom/android/server/display/WifiDisplayController$Listener;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-sendKey-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerSession;-onGetSuggestions-(Landroid/view/textservice/TextInfo; I)Landroid/view/textservice/SuggestionsInfo;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-onPause-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/calllog/CallLogFragment;-onResume-()V' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-wakeWhenReadyTq-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue$Element;-onPhantomUpEvent-(J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/provider/ContactsContract$SyncState;-set-(Landroid/content/ContentProviderClient; Landroid/accounts/Account; [B)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/WifiService;-getWifiStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/settings/Settings$WifiDisplaySettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/editor/PhoneticNameEditorView;-onTypeSelectionChange-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-<init>-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/launcher2/Workspace;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/Controller;-stopLoading-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/detail/ContactDetailUpdatesFragment$2;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/PhoneBase;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/SmsReceiverService;-access$400-(Lcom/android/mms/transaction/SmsReceiverService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/camera/VideoModule;-init-(Lcom/android/camera/CameraActivity; Landroid/view/View; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/googlecode/mp4parser/authoring/adaptivestreaming/FlatPackageWriterImpl;-write-(Lcom/googlecode/mp4parser/authoring/Movie;)V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/ui/DetailsHelper$DetailsViewContainer;-show-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiStateMachine$InitialState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.INTERNET', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$SoundSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ConversationListItem;-updateFromView-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/InCallScreen;-toggleSpeaker-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/net/wifi/SupplicantStateTracker;-access$200-(Landroid/net/wifi/SupplicantStateTracker; Landroid/net/wifi/SupplicantState; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/PhotoView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/power/PowerManagerService;-init-(Landroid/content/Context; Lcom/android/server/LightsService; Lcom/android/server/am/ActivityManagerService; Lcom/android/server/BatteryService; Lcom/android/internal/app/IBatteryStats; Lcom/android/server/display/DisplayManagerService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/inputmethod/BaseInputConnection;-performEditorAction-(I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/adapter/AccountSyncAdapter;-getSyncKey-()Ljava/lang/String;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/phone/StatusBarWindowView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/phone/Use2GOnlyCheckBoxPreference;-onClick-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewProvider;-loadUrl-(Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-hasContentChanged-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/NotificationMgr;-init-(Lcom/android/phone/PhoneGlobals;)Lcom/android/phone/NotificationMgr;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/settings/SubSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockScreen;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/ContactsActivity;-onBackPressed-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/SecureAlbum;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$1900-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator; Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager$ShowListener;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/handover/HandoverManager;-createBluetoothOobDataRecord-()Landroid/nfc/NdefRecord;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/base/RetargetFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/nsd/INsdManager$Stub;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Lcom/android/exchange/EasAccountService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-access$300-(Lcom/android/internal/widget/multiwaveview/GlowPadView; I F F)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/BluetoothVisibilityTimeoutFragment;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/display/DisplayManagerService;-setDisplayHasContent-(I Z Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/sip/SipEditor;-access$600-(Lcom/android/phone/sip/SipEditor; Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/providers/calendar/CalendarProvider2;-deleteReminders-(Landroid/net/Uri; Z Ljava/lang/String; [Ljava/lang/String; Z)I' : ['android.permission.READ_CALENDAR'],
	'Landroid/os/IHardwareService$Stub$Proxy;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Landroid/media/AudioService$SettingsObserver;-<init>-(Landroid/media/AudioService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-validate-(Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$5100-(Lcom/android/bluetooth/hfp/HeadsetStateMachine;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getRemoteAlias-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/IConnectivityManager;-isTetheringSupported-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterfw/core/Filter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-onStart-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-handleTimeout-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetooth;-setRemoteAlias-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/IAccountManager;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-composeAndSendVCards-(Ljavax/obex/Operation; Ljava/lang/String; Z Ljava/lang/String; Z)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-reportSimUnlocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-isConnected-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/SuggestionsAdapter$SuggestFilter;-performFiltering-(Ljava/lang/CharSequence;)Landroid/widget/Filter$FilterResults;' : ['android.permission.ACCESS_NETWORK_STATE', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView$9$1$1;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/quicksearchbox/google/SearchBaseUrlHelper$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/os/PowerManager$WakeLock;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-switchToState-(I F F)V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/group/GroupBrowseListAdapter;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/wfd/WifiDisplaySettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/DTMFTwelveKeyDialer;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/BluetoothHeadset;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-flashAndSetGenericTrue-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/WebViewTimersControl;-onPrerenderStart-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-onProgressComplete-(I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/google/common/io/Resources;-toString-(Ljava/net/URL; Ljava/nio/charset/Charset;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/providers/calendar/CalendarProvider2;-regenerateInstancesTable-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onEnableNewApn-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/detail/ContactLoaderFragment$ContactLoaderFragmentListener;-onDetailsLoaded-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/ISms;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-hasPendingChanges-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-<init>-(Lcom/android/music/MediaPlaybackService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitorCallback;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/GLView$OnClickListener;-onClick-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/IRecognitionService$Stub$Proxy;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String; I Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/WebViewController;-onProgressChanged-(Lcom/android/browser/Tab;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/event/EditEventView;-setViewStates-(I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/BrowserActivity;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-setRadios-(Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/ViewRootImpl;-handleImeFinishedEvent-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/PhoneDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/VibratorService$Vibration;-binderDied-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/bluetooth/hid/HidService;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetooth$Stub;-getRemoteClass-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/media/RingtonePickerActivity;-stopAnyPlayingRingtone-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5VideoFullScreen;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover$Callback;-onBluetoothHeadsetHandoverComplete-(Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/agenda/AgendaByDayAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-getLastLocation-()Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-updateActivityLockScreenState-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/WifiService;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText;-updateCarrierText-(Lcom/android/internal/telephony/IccCardConstants$State; Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/input/InputManagerService;-interceptMotionBeforeQueueingWhenScreenOff-(I)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothDiscoverableEnabler;-setEnabled-(Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-processResponse-(Landroid/os/Parcel;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPicker;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/Conversation;-startDeleteAll-(Lcom/android/mms/data/Conversation$ConversationQueryHandler; I Z)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/view/ViewRootImpl;-deliverKeyEventPostIme-(Landroid/view/ViewRootImpl$QueuedInputEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IActivityManager;-inputDispatchingTimedOut-(I Z)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/LegacyPhoneNumberListAdapter;-getContactUri-(I Landroid/database/Cursor; I I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/ContactPickerFragment;-setSearchMode-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-handleNotifyScreenOn-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewManager$ShowListener;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/BrowserWebView;-postUrl-(Ljava/lang/String; [B)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onConnectedInOrOut-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavigationBarBase;-onAction-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/activities/JoinContactActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/settings/Settings$CryptKeeperSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/search/SearchEngine;-getSuggestions-(Landroid/content/Context; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/BrowserHistoryPage$ClearHistoryTask;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/IBluetoothA2dp;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler;-<init>-(Lcom/android/contacts/activities/PhotoSelectionActivity; Landroid/content/Context; Landroid/view/View; I Lcom/android/contacts/model/RawContactDeltaList; Lcom/android/contacts/activities/PhotoSelectionActivity$1;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/Controller$6;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-queryExternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/InCallScreen;-onHoldClick-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/HTML5VideoViewProxy;-enterFullScreenVideo-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/P2pLinkManager$3;-doGet-(I Landroid/nfc/NdefMessage;)Lcom/android/nfc/snep/SnepMessage;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/LocationManagerService$Receiver;-access$1100-(Lcom/android/server/LocationManagerService$Receiver;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/BluetoothPhoneService;-handlePreciseCallStateChange-(Lcom/android/internal/telephony/Connection;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/ContactsListActivity;-getUriToQuery-()Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/WebsiteDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/BluetoothPhoneService;-access$400-(Lcom/android/phone/BluetoothPhoneService;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/data/Conversation;-access$700-(Landroid/content/Context;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-topAppWindowChanged-(Z)V' : ['android.permission.VIBRATE'],
	'Lcom/google/common/io/Files;-toString-(Ljava/io/File; Ljava/nio/charset/Charset;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/bluetooth/BluetoothA2dp;-shouldSendVolumeKeys-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView;-onResume-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/SearchActivity;-onCreateOptionsMenu-(Landroid/view/Menu;)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/IConnectivityManager$Stub;-isNetworkSupported-(I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-createIntent-(Landroid/content/Context; J)Landroid/content/Intent;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PasswordUnlockScreen$4;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/BluetoothPairingDialog;-createConsentDialog-(Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter$ContactOperations;-delete-(J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/BackupManagerService$PerformRestoreTask;-restorePmMetadata-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/am/BroadcastQueue;-broadcastTimeoutLocked-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$8200-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/IRecognitionService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-fillDigitsIfNecessary-(Landroid/content/Intent;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/google/AbstractGoogleSource;-queryInternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity$3;-run-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/QuickContactBadge;-assignContactFromEmail-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/WebViewClassic;-onFixedLengthZoomAnimationEnd-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$DateTimeSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/launcher2/AppsCustomizePagedView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/model/dataitem/GroupMembershipDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/list/PhoneNumberListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-determineScrollingStart-(Landroid/view/MotionEvent; F)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothDevice;-setPairingConfirmation-(Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityManagerService;-killAppAtUsersRequest-(Lcom/android/server/am/ProcessRecord; Landroid/app/Dialog;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/RIL;-queryCallForwardStatus-(I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-canUnbindBluetoothService-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/RemoteDevices;-pinRequestCallback-([B [B I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/month/MonthListView;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/BluetoothHeadset;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/content/SyncManager$ActiveSyncContext;-<init>-(Landroid/content/SyncManager; Landroid/content/SyncOperation; J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/GLRoot;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onTrySetupData-(Lcom/android/internal/telephony/ApnContext;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine;-handleScreenStateChanged-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/TabControl;-createNewTab-(Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/ContactSaveService;-renameGroup-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/ProviderStatusWatcher$LoaderTask;-onPostExecute-(Ljava/lang/Boolean;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/FaceUnlock;-handleCancel-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-access$500-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView$2;-onStartBatchInput-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/WebIconDatabaseClassic;-bulkRequestIconForPageUrl-(Landroid/content/ContentResolver; Ljava/lang/String; Landroid/webkit/WebIconDatabase$IconListener;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-addImage-(Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/provider/BrowserProvider2;-updateBookmarksInTransaction-(Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String; Z)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/list/ContactListAdapter;-getContactUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/HidProfile;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/NetworkPolicyManagerService$12;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText$1;-onClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$WirelessSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallback;-onError-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/RetryScheduler;-setRetryAlarm-(Landroid/content/Context;)V' : ['android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/sip/SipSettings$3;-run-()V' : ['android.permission.USE_SIP'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-checkAccessPermission-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/IConnectivityManager$Stub;-getAllNetworkState-()[Landroid/net/NetworkState;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-onPageSwitching-(Landroid/view/View; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-requestRouteToHostAddress-(I [B)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/bluetooth/hid/HidService;-broadcastConnectionState-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ThrottleService;-getCliffLevel-(Ljava/lang/String; I)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/model/VideoModel;-loadThumbnailBitmap-(Lcom/android/mms/util/ItemLoadedCallback;)Lcom/android/mms/util/ItemLoadedFuture;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/util/StateMachine$SmHandler$HaltingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onEnableApn-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-updateSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/exchange/EasSyncService;-hasConnectivity-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothHeadset;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiManager;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/activities/ContactSelectionActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/agenda/AgendaFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-getNfcAdapterExtrasInterface-(Ljava/lang/String;)Landroid/nfc/INfcAdapterExtras;' : ['android.permission.NFC'],
	'Lcom/android/gallery3d/app/TrimVideo;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/telephony/SmsProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/WifiStateTracker;-setRadio-(Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/IConnectivityManager$Stub;-startLegacyVpn-(Lcom/android/internal/net/VpnProfile;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/display/DisplayManagerService;-handleDisplayDeviceRemoved-(Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/DisplayPowerController$Callbacks;-onProximityPositive-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-queueNextSignalStrengthPoll-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Landroid/content/SyncManager$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/emailcommon/service/EmailServiceProxy;-<init>-(Landroid/content/Context; Landroid/content/Intent; Lcom/android/emailcommon/service/IEmailServiceCallback;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/util/NameConverter;-structuredNameToDisplayName-(Landroid/content/Context; Ljava/util/Map;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/nfc/INfcTag$Stub;-rediscover-(I)Landroid/nfc/Tag;' : ['android.permission.NFC'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/server/sip/SipSessionGroup;-processTransactionTerminated-(Ljavax/sip/TransactionTerminatedEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5VideoView;-reprepareData-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/LogTag$1;-run-()V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/BrowserWebView;-setLayerType-(I Landroid/graphics/Paint;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setOutgoingCallerIdDisplay-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothHealth;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-handleRadioNotAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/PhoneUtils;-hangup-(Lcom/android/internal/telephony/Call;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/NetworkStateTracker;-setRadio-(Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/browser/BrowserWebView;-addJavascriptInterface-(Ljava/lang/Object; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/GLTextureSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/Files;-copy-(Ljava/io/File; Lcom/google/common/io/OutputSupplier;)V' : ['android.permission.INTERNET'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$5700-(Lcom/android/mms/ui/ComposeMessageActivity; Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/SmsReceiver;-finishStartingService-(Landroid/app/Service; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout$4;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/preferences/WebsiteSettingsFragment$SiteAdapter$UpdateFromBookmarksDbTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/sip/SipService;-restartPortMappingLifetimeMeasurement-(Landroid/net/sip/SipProfile; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-updateClustersContents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/bluetooth/DockService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetooth$Stub;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getCallHistorySize-(I)I' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/server/am/ActivityManagerService;-finishInstrumentationLocked-(Lcom/android/server/am/ProcessRecord; I Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/NfcService$TagService;-ndefRead-(I)Landroid/nfc/NdefMessage;' : ['android.permission.NFC'],
	'Lcom/android/browser/Controller;-dismissSubWindow-(Lcom/android/browser/Tab;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-setHoldScreenLocked-(Lcom/android/server/wm/Session;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-buildAppWidgetModel-(Landroid/content/Context; Landroid/database/Cursor; Ljava/lang/String;)Lcom/android/calendar/widget/CalendarAppWidgetModel;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/MtpDeviceSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowAnimator;-access$100-(Lcom/android/server/wm/WindowAnimator;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfileManager;-updateLocalProfiles-([Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewInputDispatcher;-drainStaleWebKitEventsLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/ContactEditorAccountsChangedActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebViewClassic;-setBaseLayer-(I Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewProvider;-findNext-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/SmsStorageMonitor;-handleIccFull-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/PhoneInterfaceManager;-answerRingingCallInternal-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-openAssetFile-(Landroid/net/Uri; Ljava/lang/String;)Landroid/content/res/AssetFileDescriptor;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/MccTable;-setWifiCountryCodeFromMcc-(Landroid/content/Context; I)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/IWindowSession$Stub;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityStack;-moveTaskToBackLocked-(I Lcom/android/server/am/ActivityRecord;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/SnapshotTab;-setWebView-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/DataUsageSummary;-setMobilePolicySplit-(Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-getDrmMimeSavedStringRsrc-(J Z)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-gotoIdleAndNotifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/music/MusicPicker;-setSelected-(Landroid/database/Cursor;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager;-onScreenTurnedOff-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/PeopleActivity;-onPrepareOptionsMenu-(Landroid/view/Menu;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/NotificationManagerService$SettingsObserver;-observe-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/InputMethodManagerService;-showInputMethodSubtypeMenu-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView$2;-onDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/event/AttendeesView;-addAttendees-(Ljava/util/ArrayList;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$3800-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton$1;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ConversationList$2;-run-()V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/bluetooth/DockService;-connectIfEnabled-(Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/StitchingChangeListener;-onStitchingResult-(Landroid/net/Uri;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-access$1700-(Lcom/android/calendar/agenda/AgendaWindowAdapter; Lcom/android/calendar/agenda/AgendaWindowAdapter$QuerySpec;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/data/DownloadUtils;-requestDownload-(Lcom/android/gallery3d/util/ThreadPool$JobContext; Ljava/net/URL; Ljava/io/File;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-onSystemReady-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText$1;-onKeyguardVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/util/Recycler;-deleteMessagesForThread-(Landroid/content/Context; J I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-saveRingtone-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/am/BroadcastQueue;-skipPendingBroadcastLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/accounts/AccountSyncSettings;-onResume-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$3;-reportSuccessfulUnlockAttempt-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/NavScreen;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-onCreate-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/net/NetworkPolicyEditor;-read-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/widget/SlidingTab;-onVisibilityChanged-(Landroid/view/View; I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Ljava/net/URLConnection;-connect-()V' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/activities/TransactionSafeActivity;-onBackPressed-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-recreateUnlockScreen-(Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$UnlockMode;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment$2;-onChanged-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/alerts/AlertActivity;-access$400-(Lcom/android/calendar/alerts/AlertActivity; J)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$BluetoothStateTracker$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/location/ILocationManager$Stub$Proxy;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService;-prev-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton;-onDetachedFromWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/DayView$CalendarGestureListener;-onDown-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView$2;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/inputmethod/research/ResearchLogger;-resumeLogging-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/sip/SipService$AutoRegistrationProcess;-onRegistrationDone-(Landroid/net/sip/ISipSession; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadsetPhone;-processChld-(I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/SMSDispatcher;-dispatchPdus-([[B)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/Search;-searchMessages-(Landroid/content/Context; J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/settings/bluetooth/HeadsetProfile;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionActivity;-savePhonebookPermissionChoice-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/DockService;-applyBtSettings-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/transaction/MessageStatusService;-updateMessageStatus-(Landroid/content/Context; Landroid/net/Uri; [B Ljava/lang/String;)Landroid/telephony/SmsMessage;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/net/wifi/IWifiManager$Stub;-setCountryCode-(Ljava/lang/String; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$AccessibilityContentObserver;-<init>-(Lcom/android/server/accessibility/AccessibilityManagerService; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/SnapshotTab;-saveViewState-(Landroid/content/ContentValues;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/ConnectivityService;-access$1600-(Lcom/android/server/ConnectivityService; I Z)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/SlotView$Listener;-onSingleTapUp-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPasswordView;-onResume-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/launcher2/AppsCustomizePagedView;-beginDraggingWidget-(Landroid/view/View;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/Settings$PowerUsageSummaryActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/IThrottleManager$Stub;-getByteCount-(Ljava/lang/String; I I I)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/ui/ManageSimMessages;-access$000-(Lcom/android/mms/ui/ManageSimMessages;)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-addVideo-(Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getAdapterConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewClassic;-displaySoftKeyboard-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/webkit/WebViewInputDispatcher;-dispatchUiEvents-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/wifi/WifiApEnabler;-updateTetherState-([Ljava/lang/Object; [Ljava/lang/Object; [Ljava/lang/Object;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/ConnectivityManager;-getNetworkPreference-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/Tab$SubWindowClient;-shouldOverrideUrlLoading-(Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/detail/ContactDetailFragment;-buildEntries-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/DataManager;-getSupportedOperations-(Lcom/android/gallery3d/data/Path;)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-broadcastStickyIntent-(Landroid/content/Intent; Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-isDrmRingtoneWithRights-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/latin/LatinIME$BatchInputUpdater;-getSuggestedWordsGestureLocked-(Lcom/android/inputmethod/latin/InputPointers; Lcom/android/inputmethod/latin/LatinIME;)Lcom/android/inputmethod/latin/SuggestedWords;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/RIL;-setLocationUpdates-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameFetch;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewBase;-interceptMediaKey-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/EmailDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/inputmethodservice/KeyboardView;-openPopupIfRequired-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onCleanUpConnection-(Z I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/model/dataitem/SipAddressDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-access$000-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityRecord$Token;-keyDispatchingTimedOut-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$StartingState;-enter-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/MediaPlaybackService;-access$1200-(Lcom/android/music/MediaPlaybackService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onPressShift-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAccountView;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-process-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/tech/MifareClassic;-setTimeout-(I)V' : ['android.permission.NFC'],
	'Lcom/android/deskclock/AlarmKlaxon;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-createSocketChannel-(I Ljava/lang/String; Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/google/android/mms/pdu/PduPersister;-updateHeaders-(Landroid/net/Uri; Lcom/google/android/mms/pdu/SendReq;)V' : ['android.permission.READ_SMS'],
	'Landroid/net/ThrottleManager;-getResetTime-(Ljava/lang/String;)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/calllog/CallLogQueryHandler;-fetchVoicemailStatus-()V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/contacts/editor/StructuredNameEditorView;-onTypeSelectionChange-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$StorageUseActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/VideoModule;-onCancelBgTraining-(Landroid/view/View;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowManagerService;-setUniverseTransformLocked-(Lcom/android/server/wm/WindowState; F F F F F F F)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/location/GpsLocationProvider;-requestSetID-(I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/data/UriSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/SpeechRecognizer;-startListening-(Landroid/content/Intent;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/contacts/calllog/CallLogAdapter$CallFetcher;-fetchCalls-()V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/location/ILocationProvider$Stub;-disable-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/bluetooth/BluetoothDiscoverableEnabler$2;-run-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-finishComposingText-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/BluetoothManagerService;-access$1900-(Lcom/android/server/BluetoothManagerService; Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-getSuggestionsWithSessionId-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; I)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/AudioService$ScoClient;-requestScoState-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/contacts/ContactsProvider2;-getType-(Landroid/net/Uri;)Ljava/lang/String;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/exchange/utility/CalendarUtilities;-createMessageForEventId-(Landroid/content/Context; J I Ljava/lang/String; Lcom/android/emailcommon/provider/Account; Ljava/lang/String;)Lcom/android/emailcommon/provider/EmailContent$Message;' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterfw/core/Filter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub$Proxy;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/Files;-copy-(Ljava/io/File; Ljava/io/File;)V' : ['android.permission.INTERNET'],
	'Lcom/android/settings/bluetooth/A2dpProfile;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/TintFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-userSwitched-(I)V' : ['android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/BatteryService;-access$200-(Lcom/android/server/BatteryService;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/util/Recycler$MmsRecycler;-deleteMessagesForThread-(Landroid/content/Context; J I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/AddBookmarkPage;-getUriForFolder-(J)Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/os/IPowerManager;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-done-(Lcom/android/exchange/AbstractSyncService;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContentResolver;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/am/CoreSettingsObserver;-beginObserveCoreSettings-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-handleSetHidden-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/StateManager;-notifyActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/RIL;-setMute-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/KeyguardManager;-isKeyguardSecure-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Pending;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getCallWaiting-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-switchWaitingOrHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/Recorder;-setState-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-startRunning-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/emailcommon/service/IEmailService;-validate-(Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthDeathRecipient;-binderDied-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/p2p/WifiP2pManager;-initialize-(Landroid/content/Context; Landroid/os/Looper; Landroid/net/wifi/p2p/WifiP2pManager$ChannelListener;)Landroid/net/wifi/p2p/WifiP2pManager$Channel;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/widget/TransportControlView;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/webkit/WebViewClassic;-setUseMockDeviceOrientation-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterfw/core/FilterFunction;-executeWithArgList-([Ljava/lang/Object;)Landroid/filterfw/core/Frame;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/Settings$StorageSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/transaction/SendTransaction;-sendPdu-([B)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-setFilter-(Lcom/android/contacts/list/ContactListFilter;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/DataUsageSummary$AppDetailsFragment;-onStop-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onRecordsLoaded-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$NfcAdapterExtrasService;-close-(Ljava/lang/String; Landroid/os/IBinder;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Landroid/app/IActivityManager;-registerReceiver-(Landroid/app/IApplicationThread; Ljava/lang/String; Landroid/content/IIntentReceiver; Landroid/content/IntentFilter; Ljava/lang/String; I)Landroid/content/Intent;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/detail/ContactDetailFragment$AddToMyContactsQuickFix;-execute-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiStateMachine$DriverLoadingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/AudioService;-setBluetoothA2dpDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice; I)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleMultipartyIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/Session;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-notifyScreenOnLocked-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager$ShowListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-startCamera-()V' : ['android.permission.CAMERA'],
	'Lcom/android/server/ConnectivityService;-enforceChangePermission-()V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handlePollCalls-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/SnapshotTab$LoadData;-onPostExecute-(Landroid/database/Cursor;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-handleCdmaSubscriptionSource-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/calendar/AllInOneActivity$4;-onChange-(Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-<init>-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/CryptKeeper;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-addVideo-(Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityViewFlipper;-reset-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/internal/view/IInputMethodClient$Stub$Proxy;-onUnbindMethod-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/SaturateEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-untether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/inputmethod/BaseInputConnection;-sendKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/MessageItem$PduLoadedCallback;-onPduLoaded-(Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/camera/ui/RenderOverlay$RenderView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-getIntentBySingleSelectedPath-(Ljava/lang/String;)Landroid/content/Intent;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifyDataConnectionFailed-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/RetryScheduler;-getResponseStatus-(J)I' : ['android.permission.READ_SMS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-isConnectedProfile-(Lcom/android/settings/bluetooth/LocalBluetoothProfile;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/AccountsChangedReceiver$DeleteRemovedAccounts;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/browser/NavScreen;-access$000-(Lcom/android/browser/NavScreen; Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/AccountUnlockScreen;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/settings/Settings;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame;-onDetachedFromWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/Invert;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/RIL;-sendTerminalResponse-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$DisconnectingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/transaction/ReadRecTransaction;-ensureRouteToHost-(Ljava/lang/String; Lcom/android/mms/transaction/TransactionSettings;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/util/State;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothDevice;-createScoSocket-()Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wm/WindowManagerService;-startAppFreezingScreenLocked-(Lcom/android/server/wm/AppWindowToken; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-disable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-startLongPressTimer-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/SmsReceiverService;-storeMessage-(Landroid/content/Context; [Landroid/telephony/SmsMessage; I)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/AsyncQueryService;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/activities/PeopleActivity;-access$300-(Lcom/android/contacts/activities/PeopleActivity;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-conference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothHealth;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-setSelectedContactUri-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/AccessibilityInjectorFallback;-traverseCurrentAxis-(I Z Ljava/lang/String;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/OrganizationDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/MtpDeviceSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/SIMFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$DataUsageSummaryActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/os/IHardwareService$Stub;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/Filter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/LightsService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/mms/ui/MmsThumbnailPresenter;-presentFirstSlide-(Lcom/android/mms/ui/SlideViewInterface; Lcom/android/mms/model/SlideModel;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/media/AudioService;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/browser/BrowserWebView;-performAccessibilityAction-(I Landroid/os/Bundle;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardMessageArea$2;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-syncReloadDictionaryInternal-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getAllNetworkInfo-()[Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$DeviceInfoSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/PointerSpeedPreference;-showDialog-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/PeopleActivity;-showEmptyStateForTab-(I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/soundrecorder/Recorder;-clear-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/PhoneUi;-<init>-(Landroid/app/Activity; Lcom/android/browser/UiController;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/display/DisplayAdapter$2;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/transaction/PushReceiver$ReceivePushTask;-doInBackground-([Landroid/content/Intent;)Ljava/lang/Void;' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/phone/BluetoothPhoneService;-onCreate-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothAdapter;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/IWallpaperManager$Stub$Proxy;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Landroid/webkit/HTML5VideoView;-prepareDataCommon-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-saveDraft-(Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView$PageSwitchListener;-onPageSwitching-(Landroid/view/View; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiManager;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/gallery3d/util/JobLimiter$JobWrapper;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Landroid/bluetooth/BluetoothSocket;-bindListen-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/alerts/QuickResponseActivity;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/TelephonyRegistry;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/NfcService;-onSeEmvCardRemoval-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/phone/InCallScreen;-onDialerClose-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/WifiStateMachine$TetheredState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/nfc/NfcService;-onSeListenDeactivated-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/accounts/IAccountManager;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/provider/Browser;-addSearchUrl-(Landroid/content/ContentResolver; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-snapToPage-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/StatusBarManagerService;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-refreshProfiles-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/BatteryService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wm/InputMonitor;-notifyLidSwitchChanged-(J Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/SlidingTab$OnTriggerListener;-onTrigger-(Landroid/view/View; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame$1;-onDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame$Callbacks;-onCameraLaunchedSuccessfully-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setPin-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/AccessibilityInjector;-performAccessibilityAction-(I Landroid/os/Bundle;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneBase;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/ContactListFilterControllerImpl;-selectCustomFilter-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/StateManager;-startStateForResult-(Ljava/lang/Class; I Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/bluetooth/IBluetoothPbap$Stub$Proxy;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Landroid/nfc/tech/Ndef;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/phone/MobileNetworkSettings;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/nfc/tech/Ndef;-makeReadOnly-()Z' : ['android.permission.NFC'],
	'Lcom/android/server/UiModeManagerService;-access$000-(Lcom/android/server/UiModeManagerService; Ljava/lang/String; I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebView;-performLongClick-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity$12;-run-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-postEventFromNative-(Ljava/lang/Object; I I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FlipFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-onDevicePreferenceClick-(Lcom/android/settings/bluetooth/BluetoothDevicePreference;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-onPopupItemClick-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/detail/ContactDetailFragment;-showEmptyState-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/ManageCachePage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/webkit/AccessibilityInjector;-removeAccessibilityApisIfNecessary-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/IWindowManager$Stub;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-initDevicePreference-(Lcom/android/settings/bluetooth/BluetoothDevicePreference;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/phone/PhoneUtils;-mergeCalls-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/browser/TabControl;-createNewTab-()Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/VideoView;-release-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/exchange/EasAccountService;-tryAutodiscover-(Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/TabControl;-removeParentChildRelationShips-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/sip/SipService;-notifyProfileRemoved-(Landroid/net/sip/SipProfile;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/calllog/CallLogListItemHelper;-setPhoneCallDetails-(Lcom/android/contacts/calllog/CallLogListItemViews; Lcom/android/contacts/PhoneCallDetails; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$3;-reportFailedUnlockAttempt-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendEnvelope-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/RequestPermissionHelperActivity;-parseIntent-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/gsm/SmsManager;-getAllMessagesFromSim-()Ljava/util/ArrayList;' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-handleNotificationError-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification; Ljava/lang/String;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/gallery3d/ui/EdgeView;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-isSecure-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityRecord$Token;-windowsVisible-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/PhoneProxy;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/EventDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothAdapter;-disable-(Z)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$WifiSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-access$100-(Lcom/android/contacts/calllog/CallLogAdapter;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-handleCharacter-(I [I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$SecuritySettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-setKeyguardEnabled-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone; Landroid/telephony/CellInfo;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/google/common/io/ByteStreams;-getChecksum-(Lcom/google/common/io/InputSupplier; Ljava/util/zip/Checksum;)J' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/list/AccountFilterActivity$FilterListAdapter;-<init>-(Landroid/content/Context; Ljava/util/List; Lcom/android/contacts/list/ContactListFilter;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/adapter/AttachmentLoader;-loadAttachment-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onCleanUpAllConnections-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hid/HidService;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/AttachPhotoActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/accounts/IAccountManager;-addAccount-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/calendar/DayFragment;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/settings/DataUsageSummary;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/webkit/HTML5VideoViewProxy;-enterFullscreenForVideoLayer-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onCardEmulationAidSelected-([B)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/bluetooth/IBluetooth$Stub;-cancelBondProcess-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$MyGestureListener;-onFling-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-rescheduleMissedAlarms-(Landroid/content/ContentResolver; Landroid/content/Context; Landroid/app/AlarmManager;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/BluetoothInputDevice;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebView;-loadUrl-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/ActionModeHandler$2;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Void;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/bluetooth/IBluetoothHeadset;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-fadeInChallenge-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/util/AccountSelectionUtil;-getSelectAccountDialog-(Landroid/content/Context; I)Landroid/app/Dialog;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/AutoCompleteTextView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/calendar/CalendarDebug$FetchInfoTask;-doInBackground-([Ljava/lang/Void;)Ljava/util/List;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/SmsReceiverService;-replaceMessage-(Landroid/content/Context; [Landroid/telephony/SmsMessage; I)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAbsKeyInputView;-verifyPasswordAndUnlock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/TransactionService$ConnectivityBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/accounts/IAccountManager$Stub;-addAccount-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-checkAppWidgetConsistency-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-snapToPage-(I I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/SnailAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/Notifier;-playWirelessChargingStartedSound-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$26;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/WifiService;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/providers/downloads/DownloadReceiver;-hideNotification-(Landroid/content/Context; J)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/settings/Settings$InputMethodAndLanguageSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-onPressKey-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-setBatteryState-(I I I I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler$PhotoEditorListener;-onPhotoSelected-(Landroid/graphics/Bitmap;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/launcher2/DragController;-startDrag-(Landroid/view/View; Landroid/graphics/Bitmap; Lcom/android/launcher2/DragSource; Ljava/lang/Object; I Landroid/graphics/Rect; F)V' : ['android.permission.VIBRATE'],
	'Lcom/android/providers/contacts/debug/DataExporter;-exportData-(Landroid/content/Context;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/widget/MultiAutoCompleteTextView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$InitialState;-enter-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/ChooseLockGeneric$ChooseLockGenericFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManagerProxy;-disableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/bluetooth/opp/BluetoothOppService$UpdateThread;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/exchange/EasSyncService;-sendSettings-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PatternUnlockScreen$UnlockPatternListener;-onPatternDetected-(Ljava/util/List;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-<init>-(Landroid/content/Context; Lcom/android/internal/policy/impl/PhoneWindowManager;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-updateScreen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/UiccCard;-update-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/IccCardStatus;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$2;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$PrivacySettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ClassZeroActivity$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/CommandsInterface;-dial-(Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/voicedialer/VoiceContact;-redialNumber-(Landroid/app/Activity;)Ljava/lang/String;' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/server/am/ActivityManagerService;-moveTaskBackwards-(I)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-wakeWhenReadyLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneBase;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-getTetherableIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/vcard/VCardParserImpl_V30;-handleNest-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewCore;-sendMessage-(I I I Ljava/lang/Object;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/NicknameDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimPukUnlockScreen$TouchInput;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/data/MediaSet$ItemConsumer;-consume-(I Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-explicitCallTransfer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/calendar/CalendarProvider2;-modifyCalendarSubscription-(J Z)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/BluetoothHeadset;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/ContactListAdapter;-getContactUri-(I Landroid/database/Cursor;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewInputDispatcher$UiCallbacks;-dispatchUiEvent-(Landroid/view/MotionEvent; I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/camera/PhotoModule$4;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/UiController;-openIncognitoTab-()Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/BluetoothPan;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/Utils;-getDayOfWeekString-(I I J Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/sip/ISipService;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/phone/Use2GOnlyCheckBoxPreference;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothPairingDialog;-createUserEntryDialog-(Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onRemoteEndpointDiscovered-(Lcom/android/nfc/DeviceHost$TagEndpoint;)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/contacts/group/GroupEditorFragment;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/vpn2/VpnSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/IThrottleManager$Stub$Proxy;-getHelpUri-()Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/CallerInfoCache$CacheAsyncTask;-acquireWakeLockAndExecute-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-saveViewState-(Ljava/io/OutputStream; Landroid/webkit/ValueCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-addCachedDevices-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ConnectivityService;-enforceAccessPermission-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/pm/PackageManagerService;-main-(Landroid/content/Context; Lcom/android/server/pm/Installer; Z Z)Landroid/content/pm/IPackageManager;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Landroid/location/Criteria; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/AlarmKlaxon;-startAlarm-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity$RecipientsMenuClickListener;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/PhoneBase;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$10$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener$Adapter;-onCustomRequest-(I)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/btservice/AdapterState$PendingCommandState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/wm/WindowManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/WifiService;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/browser/Controller;-onUserCanceledSsl-(Lcom/android/browser/Tab;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothDeviceFilter$TransferFilter;-matches-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$AdvancedWifiSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/sip/SipManager;-makeAudioCall-(Landroid/net/sip/SipProfile; Landroid/net/sip/SipProfile; Landroid/net/sip/SipAudioCall$Listener; I)Landroid/net/sip/SipAudioCall;' : ['android.permission.USE_SIP'],
	'Lcom/android/gallery3d/app/SlideshowPage;-showPendingBitmap-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker$1;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage;-updateUIForCurrentPhoto-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothPan$Stub$Proxy;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/browser/preferences/InvertedContrastPreview;-setupWebView-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/NfcService$EnableDisableTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/AdapterState$OnState;-enter-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/detail/PhotoSelectionHandler;-getDeltaForAttachingPhotoToContact-()Lcom/android/contacts/model/RawContactDeltaList;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-setSuppServiceNotifications-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-access$1800-(Lcom/android/bluetooth/opp/BluetoothOppService; Landroid/database/Cursor; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/contacts/model/dataitem/StructuredPostalDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getActiveLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/editor/SplitContactConfirmationDialogFragment$Listener;-onSplitContactConfirmed-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/InCallScreen;-internalSilenceRinger-()V' : ['android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/settings/Settings$DevelopmentSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/SupplicantStateTracker$CompletedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/MtpDevice;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/transaction/RetrieveTransaction;-isDuplicateMessage-(Landroid/content/Context; Lcom/google/android/mms/pdu/RetrieveConf;)Z' : ['android.permission.READ_SMS'],
	'Lcom/android/mms/transaction/PushReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/ContactSelectionActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/Transaction;-ensureRouteToHost-(Ljava/lang/String; Lcom/android/mms/transaction/TransactionSettings;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/nfc/NfcService$ApplyRoutingTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothDiscoverableEnabler;-setDiscoverableTimeout-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-rightButtonAction-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/GrayscaleEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/IccLockSettings;-tryChangePin-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/ConnectivityManager;-getActiveLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-startRfcommSocketListener-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewManager;-onScreenTurnedOn-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewManager$ShowListener;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/DataController$DataControllerHandler;-doLoadThumbnail-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/FlipFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$2;-onAddView-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-closeDialpadInternal-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/IMediaPlaybackService;-openFile-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-setDiscoverableTimeout-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/WaveView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.VIBRATE', 'android.permission.USE_CREDENTIALS', 'android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/AlbumSetPage$4;-onSingleTapUp-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/ActivityController;-onPause-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.NFC'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-onTouch-(Landroid/view/View; Landroid/view/MotionEvent;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/coremedia/iso/IsoFile;-<init>-(Ljava/nio/channels/ReadableByteChannel;)V' : ['android.permission.INTERNET'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-startScanning-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$MyExecutor;-execute-(Ljava/lang/Runnable;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/SoundRecorder$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasAccountService;-sendHttpClientPost-(Ljava/lang/String; [B)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/TextFieldsEditorView;-notifyEditorListener-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewBase;-onScreenTurnedOn-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-access$400-(Lcom/android/server/am/ActivityManagerService; Ljava/lang/String; I Z Z Z Z I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/RIL;-dial-(Ljava/lang/String; I Lcom/android/internal/telephony/UUSInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-findOnPage-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-updateLockScreenTimeout-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/phone/QuickSettingsModel$NextAlarmObserver;-startObserving-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;-onUuidChanged-(Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/providers/downloads/DownloadThread;-runInternal-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/activities/GroupEditorActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/UsimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-requestRouteToHost-(I I)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/mms/ui/NoConfirmationSendService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountAdapter;-getChildrenCursor-(Landroid/database/Cursor;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/location/fused/FusedLocationService;-onUnbind-(Landroid/content/Intent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/calendar/SearchActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/settings/bluetooth/BluetoothCallback;-onBluetoothStateChanged-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/NavScreen$2;-onLayout-(I I I I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/location/ILocationManager$Stub;-isProviderEnabled-(Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiManager;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/LockPatternView;-detectAndAddHit-(F F)Lcom/android/internal/widget/LockPatternView$Cell;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/ISms;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/browser/AddBookmarkPage$UpdateBookmarkTask;-doInBackground-([Landroid/content/ContentValues;)Ljava/lang/Void;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$ProvisionDiscoveryState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/util/IState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.INTERNET', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/downloads/DownloadInfo;-isReadyToStart-(J)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-processStartTimedOutLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/cat/CatService;-sendTerminalResponse-(Lcom/android/internal/telephony/cat/CommandDetails; Lcom/android/internal/telephony/cat/ResultCode; Z I Lcom/android/internal/telephony/cat/ResponseData;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-deactivateDataCall-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/PieItem;-performClick-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/AccountsTester$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/CropFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/contacts/vcard/SelectAccountActivity;-onCreateDialog-(I Landroid/os/Bundle;)Landroid/app/Dialog;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-startSettingsActivity-(Landroid/content/Intent; Z)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$3100-(Lcom/android/mms/ui/ComposeMessageActivity; J)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetooth;-connectSocket-(Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Landroid/content/ISyncAdapter;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService$3;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/NfcService$TagService;-rediscover-(I)Landroid/nfc/Tag;' : ['android.permission.NFC'],
	'Lcom/android/gallery3d/app/AlbumDataLoader;-access$1800-(Lcom/android/gallery3d/app/AlbumDataLoader; Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/editor/AggregationSuggestionView;-handleItemClickEvent-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/GLTextureTarget;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-getSuggestions-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo;)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setLine1Number-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/GlobalActions$2;-onLongPress-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/latin/SubtypeSwitcher;-init-(Landroid/content/Context;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$DriverLoadedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-access$1700-(Lcom/android/gallery3d/app/AlbumSetPage; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/ui/SlideViewInterface;-startAudio-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-stopUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/Phone;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/webkit/WebViewClassic;-contentInvalidateAll-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/sip/ISipService;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-setCurrentPage-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/DataUsageSummary$LimitEditorFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-createDevicePreference-(Lcom/android/settings/bluetooth/CachedBluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-startLocationBasedDetector-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/HTML5VideoInline;-prepareDataAndDisplayMode-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/CallbackFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/MessagingNotification;-notifySendFailed-(Landroid/content/Context; Z)V' : ['android.permission.READ_SMS'],
	'Lcom/android/server/net/LockdownVpnTracker;-onVpnStateChanged-(Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-setWakeMode-(Landroid/content/Context; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/GLView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMALTEPhone;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/LocationClustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebView;-showFindDialog-(Ljava/lang/String; Z)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub;-clearForcedDisplaySize-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ContactListFilterControllerImpl;-setContactListFilter-(Lcom/android/contacts/list/ContactListFilter; Z Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetooth$Stub;-setRemoteAlias-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/SMSDispatcher$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$4300-(Lcom/android/mms/ui/ComposeMessageActivity; I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-<init>-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Settings$Bookmarks;-add-(Landroid/content/ContentResolver; Landroid/content/Intent; Ljava/lang/String; Ljava/lang/String; C I)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/HTML5VideoFullScreen;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-getSuggestionsWithSessionId-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; I)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-revertToNitzTimeZone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/app/PhotoPage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/TetherSettings;-updateBluetoothState-([Ljava/lang/String; [Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/DhcpStateMachine$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendRawPdu-([B [B Landroid/app/PendingIntent; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/OtaUtils;-onClickOtaEndButton-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/os/IVibratorService$Stub$Proxy;-vibratePattern-([J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifyServiceState-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/AlarmManagerService;-setWakelockWorkSource-(Landroid/app/PendingIntent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-dismiss-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/TetherSettings;-updateState-([Ljava/lang/String; [Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/view/RotationPolicy;-registerRotationPolicyListener-(Landroid/content/Context; Lcom/android/internal/view/RotationPolicy$RotationPolicyListener; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/bluetooth/DockService;-handleBluetoothStateOn-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/ExchangeService;-serviceRequest-(J J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-acknowledgeIncomingGsmSmsWithPdu-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/BummerView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/nsd/NsdManager;-init-()V' : ['android.permission.INTERNET'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/ContactSaveService;-removeMembersFromGroup-(Landroid/content/ContentResolver; [J J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/ContactSaveService$Listener;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/ApplicationPackageManager;-getPackageGids-(Ljava/lang/String;)[I' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Landroid/location/ILocationManager$Stub;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/net/sip/SipAudioCall;-setSpeakerMode-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPINView;-reset-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/IAudioService$Stub$Proxy;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsFragment;-onResume-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/accounts/AccountManagerService;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/phone/PhoneUtils;-startGetCallerInfo-(Landroid/content/Context; Lcom/android/internal/telephony/Connection; Lcom/android/internal/telephony/CallerInfoAsyncQuery$OnQueryCompleteListener; Ljava/lang/Object;)Lcom/android/phone/PhoneUtils$CallerInfoToken;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiStateMachine$DriverLoadingState$1;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/PlaybackQueueItem;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-sendEnvelopeWithStatus-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/OtaUtils;-otaShowProgramFailureDialog-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-startLongPressTimer-(Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-startAction-(I I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/display/DisplayManagerService;-scheduleTraversalLocked-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/ContactsProvider2$PipeMonitor;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetherableIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/CropView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/Tab;-removeFromTree-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/tech/NfcA;-getTimeout-()I' : ['android.permission.NFC'],
	'Lcom/android/contacts/list/AccountFilterActivity$MyLoaderCallbacks;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-setUpAdapter-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mms/transaction/MessagingNotification;-notifyDownloadFailed-(Landroid/content/Context; J)V' : ['android.permission.READ_SMS'],
	'Landroid/content/IContentService;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PatternUnlockScreen$UnlockPatternListener;-onPatternCellAdded-(Ljava/util/List;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/CropView;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/TabControl;-removeTab-(Lcom/android/browser/Tab;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/EventLoader$LoadEventDaysRequest;-processRequest-(Lcom/android/calendar/EventLoader;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-sendCharacter-(C)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cat/IconLoader;-loadIcon-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/CropEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/WallpaperManager;-setStream-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/contacts/list/JoinContactListAdapter;-getFirstContactUri-()Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/wifi/WifiSettingsForSetupWizardXL;-showConfigUi-(Lcom/android/settings/wifi/AccessPoint; Z)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/webkit/WebViewClassic;-loadUrl-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/PhoneGlobals$NotificationBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/detail/ContactDetailLayoutController;-setContactData-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/SettingsPreferenceFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothInputDevice;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimPukUnlockScreen;-<init>-(Landroid/content/Context; Landroid/content/res/Configuration; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback; Lcom/android/internal/widget/LockPatternUtils;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/net/IConnectivityManager$Stub;-getMobileDataEnabled-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onDownEventInternal-(I I J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/Settings$DisplaySettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/systemui/SearchPanelView$1;-run-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-onPause-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/deskclock/AlarmKlaxon$3;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/research/ResearchLogger;-latinIME_onWindowHidden-(I I Landroid/view/inputmethod/InputConnection;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/accounts/IAccountManager$Stub;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/power/PowerManagerService;-userActivityInternal-(J I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/GlobalActions$2;-onPress-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SettingsObserver;-startObserving-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-isForwardable-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/CdmaSystemSelectListPreference;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/app/MoviePlayer;-startVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewProvider$ViewDelegate;-performAccessibilityAction-(I Landroid/os/Bundle;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/group/GroupBrowseListAdapter;-getSelectedGroupPosition-()I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/IConnectivityManager;-getNetworkInfo-(I)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/voicedialer/VoiceDialerActivity;-access$600-(Lcom/android/voicedialer/VoiceDialerActivity; I)I' : ['android.permission.VIBRATE'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/CryptKeeper;-access$600-(Lcom/android/settings/CryptKeeper;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BaseUi;-showWeb-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/WebViewController;-onDownloadStart-(Lcom/android/browser/Tab; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS', 'android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityStack;-updateTransitLocked-(I Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wm/Session;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/settings/RadioInfo$20;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView;-onPause-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppManager;-enableBluetooth-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/event/EditEventFragment;-onActionBarItemSelected-(I)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onDataStateChanged-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView$1;-onCustomRequest-(I)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider;-onUpdate-(Landroid/content/Context; Landroid/appwidget/AppWidgetManager; [I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView$9;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-getDrmMimeMenuStringRsrc-(J)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/provider/BrowserProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/RadioInfo;-access$4300-(Lcom/android/settings/RadioInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-documentAsText-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-addAudio-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/BrowserWebView;-loadData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-iccIO-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-addNotificationViews-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)Lcom/android/systemui/statusbar/StatusBarIconView;' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-updateFromProvider-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame;-performAppWidgetSizeCallbacksIfNecessary-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/ConnectivityManager;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/BrowserBackupAgent;-addBookmark-(Lcom/android/browser/BrowserBackupAgent$Bookmark;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/Tab;-<init>-(Lcom/android/browser/WebViewController; Landroid/webkit/WebView;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/ui/PhotoView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/CallerInfoCache$CacheAsyncTask;-onCancelled-(Ljava/lang/Void;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaByDayAdapter;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/RuimRecords;-setVoiceCallForwardingFlag-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/wifi/WifiEnabler;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/bluetooth/IBluetooth;-getRemoteAlias-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/app/IActivityManager;-wakingUp-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BreadCrumbView;-pop-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/FilterSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/transaction/RetrieveTransaction;-sendPdu-([B Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-play-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/IsoDep;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/soundrecorder/SoundRecorder;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IBackupManager$Stub;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/app/IAlarmManager$Stub$Proxy;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback;-goToUnlockScreen-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/MtpDevice;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-updateUri-()Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-animateCollapsePanels-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/data/Conversation;-startQueryHaveLockedMessages-(Landroid/content/AsyncQueryHandler; Ljava/util/Collection; I)V' : ['android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/DownloadThread;-openResponseEntity-(Lcom/android/providers/downloads/DownloadThread$State; Lorg/apache/http/HttpResponse;)Ljava/io/InputStream;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewClassic;-cutSelection-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/LocationManagerService$Receiver;-<init>-(Lcom/android/server/LocationManagerService; Landroid/location/ILocationListener; Landroid/app/PendingIntent; I I Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-dispatchHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/InputMethodManagerService;-systemReady-(Lcom/android/server/StatusBarManagerService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/quickcontact/QuickContactActivity;-bindData-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/inputmethod/UserDictionaryAddWordFragment;-updateSpinner-()V' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/settings/Settings$PrivacySettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/google/common/io/CharStreams$4;-getInput-()Ljava/io/Reader;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-snapToPageImmediately-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-access$200-(Lcom/android/bluetooth/pbap/BluetoothPbapService;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-start-()V' : ['android.permission.VIBRATE', 'android.permission.DISABLE_KEYGUARD', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/AlbumSetSlidingWindow;-getLoadingTexture-()Lcom/android/gallery3d/ui/BitmapTexture;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/contacts/list/ContactBrowseListFragment$ContactLookupTask;-onPostExecute-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/camera/CameraActivity;-openModule-(Lcom/android/camera/CameraModule; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/app/GalleryActionBar$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/list/LegacyPostalAddressListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame;-<init>-(Landroid/content/Context; Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame$Callbacks; Lcom/android/internal/policy/impl/keyguard/KeyguardActivityLauncher; Lcom/android/internal/policy/impl/keyguard/KeyguardActivityLauncher$CameraWidgetInfo; Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/Tab$3;-onCloseWindow-(Landroid/webkit/WebView;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/nfc/tech/BasicTagTechnology;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/server/WifiService;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothA2dp;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/ContactsListActivity;-getSelectedUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/webkit/WebView;-findNext-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-addAppLocked-(Landroid/content/pm/ApplicationInfo; Z)Lcom/android/server/am/ProcessRecord;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothTetheringDataTracker;-teardown-()Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/DisplaySettings;-onPreferenceChange-(Landroid/preference/Preference; Ljava/lang/Object;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService$8;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexClientSession;-start-(Landroid/os/Handler;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/IBatteryStats$Stub$Proxy;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/editor/SelectAccountDialogFragment;-onCreateDialog-(Landroid/os/Bundle;)Landroid/app/Dialog;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/vcard/VCardEntry$EntryElement;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/BluetoothAdapter;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/content/AbstractThreadedSyncAdapter$ISyncAdapterImpl;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub;-setPairingConfirmation-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiManager;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;-getName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-reloadDataAndSetSelectedUri-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPINView;-onWindowFocusChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/transaction/NotificationPlayer$CreationAndCompletionThread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-onItemClick-(I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/SupplicantStateTracker$ScanState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.INTERNET', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/MusicPicker;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mms/ui/ClassZeroActivity;-access$200-(Lcom/android/mms/ui/ClassZeroActivity;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/wallpaper/galaxy/GalaxyRS;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$AccountAnalyzer;-next-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/RequestPermissionActivity;-proceedAndFinish-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/DataUsageSummary;-access$100-(Lcom/android/settings/DataUsageSummary;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/list/ContactPickerFragment;-onStart-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-doJoinSuggestedContact-([J)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/FaceUnlock;-handleUnlock-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/LegacyPostalAddressListAdapter;-getContactMethodUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/SlideshowPage;-findMediaItem-(Lcom/android/gallery3d/data/MediaSet; I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/ContentResolver;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/EdgeView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/nfc/INfcAdapterExtras;-close-(Ljava/lang/String; Landroid/os/IBinder;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Landroid/media/effect/effects/SepiaEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-determineScrollingStart-(Landroid/view/MotionEvent; F)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/handover/HandoverService;-disableBluetoothIfNeeded-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/IWifiManager$Stub;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/mms/data/WorkingMessage;-loadFromUri-(Landroid/net/Uri;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-setHdmiPlugged-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/list/ShortcutIntentBuilder$PhoneNumberLoadingAsyncTask;-loadPhoto-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/FilterFunction;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingInsecureRfcommWithServiceRecord-(Ljava/lang/String; Ljava/util/UUID;)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$WatchdogEnabledState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/BackupManagerService$PerformRestoreTask;-execute-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/hdp/HealthService;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/vcard/NfcImportVCardActivity$ImportTask;-doInBackground-([Lcom/android/contacts/vcard/VCardService;)Lcom/android/contacts/vcard/ImportRequest;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/fuelgauge/PowerUsageSummary;-refreshStats-()V' : ['android.permission.BATTERY_STATS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/bluetooth/BluetoothSocket;-initSocketNative-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserSettings;-initialize-(Landroid/content/Context;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/Controller;-switchToTab-(Lcom/android/browser/Tab;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/sip/SipService$AutoRegistrationProcess;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/LatinIME$BatchInputUpdater;-updateBatchInput-(Lcom/android/inputmethod/latin/InputPointers; Lcom/android/inputmethod/latin/LatinIME;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-setWakeMode-(Landroid/content/Context; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$StorageSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover;-onServiceConnected-(I Landroid/bluetooth/BluetoothProfile;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/IActivityManager;-killAllBackgroundProcesses-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/calendar/alerts/AlertReceiver;-finishStartingService-(Landroid/app/Service; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-requestTraversalLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/ConnectivityManager;-setRadios-(Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-getHealthDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/ConnectivityService;-handleCaptivePortalTrackerCheck-(Landroid/net/NetworkInfo;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/contacts/ContactsProvider2;-insertStatusUpdate-(Landroid/content/ContentValues;)J' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/TransactionService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/net/CaptivePortalTracker;-access$1800-(Landroid/net/CaptivePortalTracker; Ljava/net/InetAddress;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SyncStateTracker;-toggleState-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/GroupEditorActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler$PhotoEditorListener;-onRemovePictureChosen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/gadget/WidgetService$PhotoRVFactory;-getViewAt-(I)Landroid/widget/RemoteViews;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Landroid/net/wifi/SupplicantStateTracker$UninitializedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/GroupMemberLoader;-constructLoaderForGroupEditorQuery-(Landroid/content/Context; J)Lcom/android/contacts/GroupMemberLoader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/EventInfoFragment;-updateResponse-(J J I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-sendDownUpKeyEvents-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/calendar/CalendarProvider2;-deleteMatchingCalendars-(Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/PhotoPage;-transitionFromAlbumPageIfNeeded-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$10;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-disconnect-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebView;-findAllAsync-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onSetPolicyDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-finishBooting-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-access$000-(Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityStack;-moveHomeToFrontLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/widget/ChartDataUsageView$DataUsageChartListener;-onLimitChanged-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHeadset;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/EasSyncService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/EmailAddressListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/editor/RawContactReadOnlyEditorView;-setPhotoBitmap-(Landroid/graphics/Bitmap;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/location/ILocationProvider;-disable-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowManagerService;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-keyguardDone-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/LocationManagerService;-switchUser-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/Session;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/LatinIME;-onCreate-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView$2;-onPressKey-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/vcard/ProcessorBase;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/sip/SipAudioCall;-endCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$NfcAdapterExtrasService;-setCardEmulationRoute-(Ljava/lang/String; I)V' : ['android.permission.NFC'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-onKey-(I [I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-getHealthDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/EasOutboxService;-hasConnectivity-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/Bookmarks$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/TintFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-setAndPersistDeviceName-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothPan$Stub$Proxy;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/editor/BaseRawContactEditorView;-setPhotoBitmap-(Landroid/graphics/Bitmap;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/location/LocationManager;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserYesNoPreference;-onDialogClosed-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/app/AlbumPage;-access$1200-(Lcom/android/gallery3d/app/AlbumPage; I)V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/GeolocationService;-start-()Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService;-dispatchMediaKeyEventForCalls-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/WorkingMessage;-send-(Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getDeviceIdentity-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/preferences/WebViewPreview;-onBindView-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClientClassicExt;-doUpdateVisitedHistory-(Landroid/webkit/WebView; Ljava/lang/String; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/PhoneUi;-showNavScreen-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/SystemAllowGeolocationOrigins;-start-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-processMessagePart-([B Ljava/lang/String; I I I J I Z)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$StartStopButtonListener;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-determineScrollingStart-(Landroid/view/MotionEvent; F)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/power/PowerManagerService;-handleSandman-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-deleteAccountPIMData-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/ex/variablespeed/MediaPlayerProxy;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/accounts/AddAccountSettings;-addAccount-(Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/gadget/WidgetService$PhotoRVFactory;-onDataSetChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-loadDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-getSuggestionsExternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-interceptMotionBeforeQueueingWhenScreenOff-(I)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Utils;-getTetheringLabel-(Landroid/net/ConnectivityManager;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/IccCard;-supplyPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-stopUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/MtpImage;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onPictureCenter-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-setQueryString-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/WifiService;-access$1100-(Lcom/android/server/WifiService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller$Download;-saveDataUri-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/contacts/group/GroupEditorFragment$2;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-setSafeMode-(Z)V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/FindActionModeCallback;-findNext-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/CalendarController;-sendEvent-(Ljava/lang/Object; J Landroid/text/format/Time; Landroid/text/format/Time; Landroid/text/format/Time; J I J Ljava/lang/String; Landroid/content/ComponentName;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/Tab$2;-shouldOverrideUrlLoading-(Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/SIMRecords;-onRefresh-(Z [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/InCallScreen;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$DriverUnloadedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/SelectionManager;-getSelected-(Z)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/tech/MifareUltralight;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/AccountFilterActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/accounts/AccountManagerService;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/providers/contacts/VoicemailCleanupService;-onHandleIntent-(Landroid/content/Intent;)V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/server/am/ActivityManagerService;-finishUserStop-(Lcom/android/server/am/UserStartedState;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/nfc/tech/Ndef;-getNdefMessage-()Landroid/nfc/NdefMessage;' : ['android.permission.NFC'],
	'Landroid/inputmethodservice/KeyboardView;-sendAccessibilityEventForUnicodeCharacter-(I I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/webkit/WebView;-findAll-(Ljava/lang/String;)I' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-onPageBeginMoving-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/DevelopmentSettings$ConfirmEnforceFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ConnectivityService;-<init>-(Landroid/content/Context; Landroid/os/INetworkManagementService; Landroid/net/INetworkStatsService; Landroid/net/INetworkPolicyManager; Lcom/android/server/ConnectivityService$NetworkFactory;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-access$1500-(Lcom/android/bluetooth/btservice/BondStateMachine; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/EasOutboxService;-tryProvision-(Lcom/android/exchange/EasSyncService;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/JoinContactListAdapter;-getContactUri-(I Landroid/database/Cursor;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText$1;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/TetherSettings;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/PhotoPage;-onAlbumModeSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-writeSmsToSim-(I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/CameraSwitcher;-onCameraSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/AsyncRunner;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IEmailService;-deleteAccountPIMData-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setAndBroadcastNetworkSetTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/ContentService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/settings/DevelopmentSettings;-resetDangerousOptions-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_EXTERNAL_STORAGE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/list/AccountFilterActivity;-loadAccountFilters-(Landroid/content/Context;)Ljava/util/List;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Landroid/provider/Telephony$Threads;-getOrCreateThreadId-(Landroid/content/Context; Ljava/util/Set;)J' : ['android.permission.READ_SMS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$2900-(Lcom/android/mms/ui/ComposeMessageActivity; J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/PhoneFactory;-makeDefaultPhones-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/IConnectivityManager$Stub;-tether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ThrottleService;-systemReady-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/FrameFetch;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IThrottleManager$Stub;-getResetTime-(Ljava/lang/String;)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-snapToPage-(I I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/DisplaySettings;-writeFontSizePreference-(Ljava/lang/Object;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-onDeviceAdded-(Lcom/android/settings/bluetooth/CachedBluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/AccountFilterActivity$FilterLoader;-loadInBackground-()Ljava/util/List;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Disconnected;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/IAccountManager$Stub;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/settings/Settings$AdvancedWifiSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/AdnRecordCache;-updateAdnByIndex-(I Lcom/android/internal/telephony/AdnRecord; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-createContact-(Lcom/android/contacts/model/account/AccountWithDataSet;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/editor/StructuredNameEditorView;-rebuildStructuredName-(Lcom/android/contacts/model/RawContactDelta$ValuesDelta;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/ISms$Stub;-copyMessageToIccEf-(I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-startActivityDismissingKeyguard-(Landroid/content/Intent; Z)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$1;-onKeyguardVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-bindBackupAgent-(Landroid/content/pm/ApplicationInfo; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-setQueryString-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/DefaultContactListAdapter;-getContactUri-(I Landroid/database/Cursor;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/p2p/WifiP2pService;-enforceAccessPermission-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity$QueryEntitiesTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/media/MediaProvider;-getThumb-(Lcom/android/providers/media/MediaProvider$DatabaseHelper; Landroid/database/sqlite/SQLiteDatabase; Ljava/lang/String; J Landroid/net/Uri;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.READ_EXTERNAL_STORAGE', 'android.permission.WRITE_EXTERNAL_STORAGE'],
	'Lcom/android/internal/telephony/CallerInfo;-markAsVoiceMail-()Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-animateCollapsePanels-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/event/EditEventFragment;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/TabControl;-createNewWebView-(Z)Landroid/webkit/WebView;' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Landroid/telephony/TelephonyManager;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/webkit/WebViewClassic;-access$7100-(Landroid/webkit/WebViewClassic;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothServerSocket;-<init>-(I Z Z I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/Invert;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CallerInfoAsyncQuery$CallerInfoAsyncQueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/BluetoothEnabler;-<init>-(Landroid/content/Context; Landroid/widget/Switch;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/SlidingTab;-dispatchTriggerEvent-(I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.VIBRATE', 'android.permission.USE_CREDENTIALS', 'android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/base/NullFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/location/GpsLocationProvider;-access$1100-(Lcom/android/server/location/GpsLocationProvider; I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/PieController;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-onItemClick-(I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/view/VolumePanel;-onShowVolumeChanged-(I I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/sip/SipCommandInterface;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiWatchdogStateMachine;-makeWifiWatchdogStateMachine-(Landroid/content/Context;)Landroid/net/wifi/WifiWatchdogStateMachine;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/power/Notifier;-onWakeUpStarted-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-setKeyboardMode-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/SIMRecords;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$16;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/google/common/io/Files;-copy-(Lcom/google/common/io/InputSupplier; Ljava/io/File; Ljava/nio/charset/Charset;)V' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-generateBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Disconnected;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/launcher2/DragController;-startDrag-(Landroid/graphics/Bitmap; I I Lcom/android/launcher2/DragSource; Ljava/lang/Object; I Landroid/graphics/Point; Landroid/graphics/Rect; F)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/settings/Settings$AccessibilitySettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/SearchActivity;-handleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/Settings$ManageApplicationsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/app/IBackupAgent$Stub;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/vcard/ImportVCardActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/launcher2/InfoDropTarget;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-doQuery-(Lcom/android/calendar/agenda/AgendaWindowAdapter$QuerySpec;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/providers/contacts/ContactsProvider2;-queryLocal-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; J Landroid/os/CancellationSignal;)Landroid/database/Cursor;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/widget/RotarySelector;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncManager$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator$1;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/MessagingNotification;-getDownloadFailedMessageCount-(Landroid/content/Context;)I' : ['android.permission.READ_SMS'],
	'Lcom/android/server/LocationManagerService$3;-onSomePackagesChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-updateCurrentCarrierInProvider-(Ljava/lang/String;)Z' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/LocationManagerService;-getReceiver-(Landroid/location/ILocationListener; I I Ljava/lang/String;)Lcom/android/server/LocationManagerService$Receiver;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/CallNotifier;-showIncomingCall-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/dreams/phototable/LocalSource;-fillQueue-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothInputDevice;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$400-(Lcom/android/mms/ui/ComposeMessageActivity;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/systemui/recent/RecentsPanelView;-dismissAndGoBack-()V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-setTimeout-(I I)I' : ['android.permission.NFC'],
	'Landroid/widget/ZoomButtonsController$Container;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/util/AccountPromptUtils;-launchAccountPrompt-(Landroid/app/Activity;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothHeadsetPhone$Stub;-processChld-(I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/location/provider/LocationProviderBase;-onDisable-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/widget/PinnedHeaderListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$DisplaySettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-getTetheredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/PhoneUtils;-separateCall-(Lcom/android/internal/telephony/Connection;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/ThrottleService$DataRecorder;-addData-(J J)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/ConnectivityManager;-getTetheredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/google/common/io/Files;-copy-(Ljava/io/File; Ljava/nio/charset/Charset; Ljava/lang/Appendable;)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-pageBeginMoving-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-goBackOrForwardImpl-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewManager;-show-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/ConnectivityService;-setMobileDataEnabled-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-onCreate-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/WebView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I Ljava/util/Map; Z)V' : ['android.permission.INTERNET'],
	'Lcom/android/settings/MasterClear;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/ConnectivityService;-getTetheredIfacePairs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-initDevicePreference-(Lcom/android/settings/bluetooth/BluetoothDevicePreference;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/SelectActionModeCallback;-onActionItemClicked-(Landroid/view/ActionMode; Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$800-(Lcom/android/mms/ui/ComposeMessageActivity; Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/development/Connectivity;-access$2100-(Lcom/android/development/Connectivity;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/dreams/phototable/PhotoSource;-findImages-(I)Ljava/util/Collection;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/data/Conversation$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.READ_SMS'],
	'Lcom/android/settings/Settings$InputMethodAndLanguageSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-openMoreKeysKeyboardIfRequired-(Lcom/android/inputmethod/keyboard/Key; Lcom/android/inputmethod/keyboard/PointerTracker;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/alerts/AlertService;-postNotification-(Lcom/android/calendar/alerts/AlertService$NotificationInfo; Ljava/lang/String; Landroid/content/Context; Z Lcom/android/calendar/alerts/AlertService$NotificationPrefs; Lcom/android/calendar/alerts/NotificationMgr; I)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/app/DownloadManager;-addCompletedDownload-(Ljava/lang/String; Ljava/lang/String; Z Ljava/lang/String; Ljava/lang/String; J Z)J' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/mms/ui/QuickContactDivot;-assignContactFromEmail-(Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/RIL;-processUnsolicited-(Landroid/os/Parcel;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-enable-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-performHapticFeedbackLw-(Landroid/view/WindowManagerPolicy$WindowState; I Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-isValidWord-(Ljava/lang/CharSequence;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/wm/WindowManagerService;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.DISABLE_KEYGUARD', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/CameraActivity;-reuseCameraScreenNail-(Z)Lcom/android/gallery3d/ui/ScreenNail;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView$2;-onKeyguardVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/ContactSaveService$2;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/activities/PeopleActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/phone/QuickSettingsModel$BrightnessObserver;-startObserving-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-onPause-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-onScreenTurnedOff-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/OnContactBrowserActionListener;-onFinishAction-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/exchange/adapter/Parser;-parse-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/ZoomManager;-onFirstLayout-(Landroid/webkit/WebViewCore$DrawData;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/usb/UsbDebuggingActivity;-onStop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-beginScrollEdit-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/providers/contacts/VoicemailContentTable;-delete-(Lcom/android/providers/contacts/VoicemailContentProvider$UriData; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/settings/Settings$WifiP2pSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/settings/SettingsBackupAgent;-access$000-(Lcom/android/providers/settings/SettingsBackupAgent; Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/google/common/io/MultiReader;-advance-()V' : ['android.permission.INTERNET'],
	'Landroid/view/IWindowSession;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/webkit/WebView;-clearMatches-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/contacts/list/ContactPickerFragment;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-startRepeatKey-(Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/activities/NonPhoneActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout$6;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/CalendarController$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/wifi/WifiManager;-startScan-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-isBusy-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-performAuditoryFeedbackForAccessibilityIfNeed-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/data/Conversation$2;-run-()V' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/app/ResolverActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Landroid/webkit/ZoomManager;-zoomOut-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/util/PduLoaderManager;-getPdu-(Landroid/net/Uri; Z Lcom/android/mms/util/ItemLoadedCallback;)Lcom/android/mms/util/ItemLoadedFuture;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-handleStatusReport-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/app/AlbumDataLoader;-executeAndWait-(Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-drawTopPanel-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/Controller;-resumeWebViewTimers-(Lcom/android/browser/Tab;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame$7;-onClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/EventInfoFragment;-access$1900-(Lcom/android/calendar/EventInfoFragment; Landroid/view/View;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/view/IInputMethodCallback$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasAccountService;-validateAccount-(Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/latin/LatinIME;-onEndBatchInput-(Lcom/android/inputmethod/latin/InputPointers;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/app/ActivityManagerNative;-handleApplicationCrash-(Landroid/os/IBinder; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/phone/PhoneUtils;-restoreMuteState-()Ljava/lang/Boolean;' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/GroupMembershipDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/location/LocationBlacklist;-init-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-setName-(Ljava/lang/String;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/preference/VolumePreference;-onSampleStarting-(Landroid/preference/VolumePreference$SeekBarVolumizer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/calendar/CalendarProvider2;-doUpdateTimezoneDependentFields-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/phone/InCallScreen;-onMuteClick-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/settings/wifi/WifiApEnabler;-<init>-(Landroid/content/Context; Landroid/preference/CheckBoxPreference;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler;-getPhotoPickSize-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/AccessibilitySettings;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-clearSslPreferences-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionActivity;-showPhonebookDialog-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AlbumPage$4;-onLongTap-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-onPageSwitching-(Landroid/view/View; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/VibratorService;-onInputDeviceChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-animateChallengeTo-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onPollPdp-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-getConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/PhoneGlobals;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/SecureAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-onGenericMotionEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/ClusterSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/Browser;-getVisitedLike-(Landroid/content/ContentResolver; Ljava/lang/String;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/base/RetargetFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/SubSettings;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/sip/ISipService$Stub;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/systemui/statusbar/tv/TvStatusBar;-updateSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/LocationManagerService;-requestLocationUpdatesLocked-(Landroid/location/LocationRequest; Lcom/android/server/LocationManagerService$Receiver; I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothHeadset;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/SelectionManager$SelectionListener;-onSelectionModeChange-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/musicfx/ControlPanelEffect;-initEffectsPreferences-(Landroid/content/Context; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/PanoramaModule$8;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/group/SuggestedMemberListAdapter$SuggestedMemberFilter;-performFiltering-(Ljava/lang/CharSequence;)Landroid/widget/Filter$FilterResults;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/PeopleActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/SelectionManager;-enterSelectionMode-()V' : ['android.permission.VIBRATE'],
	'Landroid/test/mock/MockContentResolver;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/SmsUsageMonitor;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-requestRouteToHostAddress-(I [B)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/providers/downloads/DownloadReceiver$1;-run-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$3;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Exception;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/GroupDetailActivity;-onPrepareOptionsMenu-(Landroid/view/Menu;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-setQueryString-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/AlbumSetDataLoader$ReloadTask;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/power/PowerManagerService;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/WifiService;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/list/ContactBrowseListFragment$ContactLookupTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-onClose-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/FdnSetting;-updatePINChangeState-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/BondStateMachine$StableState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-setQueuePosition-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService;-connectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/model/dataitem/DataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/UsageStatsService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onDownEvent-(I I J Lcom/android/inputmethod/keyboard/PointerTracker$KeyEventHandler;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/group/GroupEditorFragment$3;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/month/MonthByWeekAdapter$2;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/SIMRecords;-handleFileUpdate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ConnectivityService;-setUsbTethering-(Z)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/IThrottleManager$Stub$Proxy;-getResetTime-(Ljava/lang/String;)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/transaction/TransactionService;-update-(Lcom/android/mms/transaction/Observable;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/webkit/WebView;-removeJavascriptInterface-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/ConnectivityManager;-setGlobalProxy-(Landroid/net/ProxyProperties;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/settings/wifi/AdvancedWifiSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebViewClassic;-saveWebArchiveImpl-(Ljava/lang/String; Z Landroid/webkit/ValueCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/location/ComprehensiveCountryDetector$2;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewProvider;-freeMemory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/PhoneUtils;-answerAndEndHolding-(Lcom/android/internal/telephony/CallManager; Lcom/android/internal/telephony/Call;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/telephony/SmsManager;-disableCellBroadcastRange-(I I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-playVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothPairingDialog;-createView-(Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;)Landroid/view/View;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/RotarySelector;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wm/DragState;-endDragLw-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/phone/PhoneGlobals;-setBeginningCall-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/Connection;-cancelPostDial-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/StringLogger;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/sip/SipAudioCall;-onError-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/MediaSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/common/contacts/DataUsageStatUpdater;-updateWithRfc822Address-(Ljava/util/Collection;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-handleTimeout-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardStatusViewManager;-<init>-(Landroid/view/View; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor; Lcom/android/internal/widget/LockPatternUtils; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/location/ILocationManager;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendP2pDiscoveryChangedBroadcast-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/NavigationBarPhone;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/vcard/VCardParserImpl_V21;-handleNest-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$4300-(Lcom/android/mms/ui/ComposeMessageActivity; I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetoothManager;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/DeliveryReportActivity;-getReportItems-()Ljava/util/List;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/CdmaSystemSelectListPreference$MyHandler;-handleQueryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ims/IsimUiccRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getBondState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/WebViewController;-doUpdateVisitedHistory-(Lcom/android/browser/Tab; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/ITelephony$Stub$Proxy;-getCellLocation-()Landroid/os/Bundle;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Connected;-processSlcConnected-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/Contacts$People;-tryGetMyContactsGroupId-(Landroid/content/ContentResolver;)J' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onShowMoreKeysPanel-(I I Lcom/android/inputmethod/keyboard/PointerTracker$KeyEventHandler;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/PhoneBase;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewInputDispatcher;-enqueueEventLocked-(Landroid/webkit/WebViewInputDispatcher$DispatchEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/WindowManagerPolicy;-finishPostLayoutPolicyLw-()I' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/DhcpStateMachine$RunningState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupConnectionByIndex-(Lcom/android/internal/telephony/cdma/CdmaCall; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/TrimVideo;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/SupplicantStateTracker$InactiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.INTERNET', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/tech/MifareClassic;-authenticate-(I [B Z)Z' : ['android.permission.NFC'],
	'Lcom/android/settings/WallpaperTypeSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-initLoader-(Ljava/lang/String;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/opp/TestTcpSessionNotifier;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/model/dataitem/EmailDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/MenuExecutor$ConfirmDialogListener;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserSettings;-setAutoFillProfile-(Landroid/webkit/WebSettingsClassic$AutoFillProfile; Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/AudioService;-access$6300-(Landroid/media/AudioService;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/ContactsContract$RawContacts$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/SmsStorageMonitor$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider;-buildUpdate-(Landroid/content/Context;)Landroid/widget/RemoteViews;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/Context;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/Settings$InputMethodAndSubtypeEnablerActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/InCallScreen;-onPause-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/contacts/detail/ContactDetailFragment;-configureQuickFix-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityManagerService;-setProcessLimit-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/MediaPlayer;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/DownloadTouchIcon;-doInBackground-([Ljava/lang/String;)Ljava/lang/Void;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/inputmethodservice/IInputMethodSessionWrapper$InputMethodEventCallbackWrapper;-finishedEvent-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$2;-userActivity-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/phone/SimContacts;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/accounts/SyncSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-refreshSelectedContactUri-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$AdvancedWifiSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/WiredAccessoryManager;-<init>-(Landroid/content/Context; Lcom/android/server/input/InputManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManagerProxy;-enableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/settings/Settings$CryptKeeperSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/vpn2/VpnSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/FilterSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/ExchangeService$ConnectivityReceiver$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$StorageSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-writeSmsToRuim-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-enqueue-([J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService$WindowManagerFuncs;-requestTraversal-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/StructuredPostalDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/DayView$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/net/NetworkStatsService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewClassic;-showFindDialog-(Ljava/lang/String; Z)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/WallpaperManager;-clear-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/PhotoView$FullPicture;-drawTileView-(Lcom/android/gallery3d/ui/GLCanvas; Landroid/graphics/Rect;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccCardProxy;-updateIccAvailability-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-addCallAndContactMenuItems-(Landroid/view/ContextMenu; Lcom/android/mms/ui/ComposeMessageActivity$MsgListMenuClickListener; Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/am/ActivityManagerService;-finishUserSwitch-(Lcom/android/server/am/UserStartedState;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/model/dataitem/IdentityDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/downloads/DownloadReceiver;-access$000-(Lcom/android/providers/downloads/DownloadReceiver; Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/nfc/NfcService;-doOpenSecureElementConnection-()I' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/ZoomManager;-zoomToReadingLevel-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/input/InputManagerService;-registerShowTouchesSettingObserver-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/LocationSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-<init>-(Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/gsm/GsmCallTracker; Lcom/android/internal/telephony/gsm/GsmCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavigationBarTablet;-stopOrRefresh-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/ConnectivityManager;-getTetheringErroredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/WifiStateMachine;-access$2600-(Landroid/net/wifi/WifiStateMachine; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/RingerVolumePreference;-onBindDialogView-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/preferences/WebViewPreview;-onCreateView-(Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/AddBookmarkPage$SaveBookmarkRunnable;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/transaction/NotificationTransaction;-<init>-(Landroid/content/Context; I Lcom/android/mms/transaction/TransactionSettings; Lcom/google/android/mms/pdu/NotificationInd;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Landroid/webkit/WebViewProvider;-resumeTimers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/BatteryService;-<init>-(Landroid/content/Context; Lcom/android/server/LightsService;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Tab;-postCapture-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-endReordering-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothDevicePreference;-<init>-(Landroid/content/Context; Lcom/android/settings/bluetooth/CachedBluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/TrafficStats;-stopDataProfiling-(Landroid/content/Context;)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SyncStateTracker$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/systemui/statusbar/phone/QuickSettingsModel$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/tts/TtsEngineSettingsFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/transaction/MessagingNotification;-notifyFailed-(Landroid/content/Context; Z J Z)V' : ['android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/SlideshowPage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/PhoneProxy;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/PhoneInterfaceManager;-isSimPinEnabled-()Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/activities/AttachPhotoActivity;-access$000-(Lcom/android/contacts/activities/AttachPhotoActivity; Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/IAudioService;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/contacts/model/ContactLoader;-loadInvitableAccountTypes-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-switchToState-(I F F)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/CropView$FaceHighlightView;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onVoiceCallStarted-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phone/InCallScreen;-updateInCallTouchUi-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/pm/PackageManagerService;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView;-maybeStartBiometricUnlock-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/tablet/SettingsView;-onClick-(Landroid/view/View;)V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/webkit/WebViewProvider;-goForward-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/vcard/VCardParser_V30;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/view/IWindowSession$Stub;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/webkit/WebViewClassic;-setFocusControllerActive-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/ContentResolver;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/telephony/SmsManager;-enableCellBroadcastRange-(I I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/settings/Settings$CryptKeeperSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CsimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-enableKeyguard-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-updateUserActivitySummaryLocked-(J I)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/NotificationPlayer;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/research/ResearchLogger;-sendFeedback-(Ljava/lang/String; Z)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-<init>-(J)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handleCallWaitingInfo-(Lcom/android/internal/telephony/cdma/CdmaCallWaitingNotification;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcAdapterExtras$Stub;-authenticate-(Ljava/lang/String; [B)V' : ['android.permission.NFC'],
	'Lcom/android/gallery3d/app/AlbumDataLoader$ReloadTask;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/DeviceStorageMonitorService;-access$000-(Lcom/android/server/DeviceStorageMonitorService; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.CLEAR_APP_CACHE'],
	'Landroid/app/ContextImpl;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/StateManager;-itemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/commands/bmgr/Bmgr;-doList-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/vpn2/VpnSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-createContact-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter$SlideshowSource;-getMediaItem-(I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardMessageArea;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/DisplayPowerState$PhotonicModulator$1;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/location/LocationManager;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/content/ContentService;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Ljava/net/HttpURLConnection;-connect-()V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifyServiceState-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/data/WorkingMessage$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/wifi/p2p/WifiP2pSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/EasOutboxService;-acknowledgeRemoteWipe-(Lcom/android/exchange/EasSyncService; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/text/ToUpperCase;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/list/ContactListFilterController;-selectCustomFilter-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/ApplicationSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$SpellCheckersSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getBondedDevices-()[Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/BackupManagerService;-clearBackupData-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-cancelBondProcess-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/wifi/WifiAPITest;-onPreferenceClick-(Landroid/preference/Preference;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/contacts/calllog/CallLogFragment;-updateOnTransition-(Z)V' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/net/IConnectivityManager$Stub;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/ActivityManagerNative;-killAllBackgroundProcesses-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Landroid/webkit/GeolocationService;-unregisterFromLocationUpdates-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/sip/ISipService$Stub;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/server/sip/SipService$IntervalMeasurementProcess;-onResponse-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-onSurfaceDestroyed-(Landroid/view/SurfaceHolder;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/users/UserSettings;-access$700-(Lcom/android/settings/users/UserSettings; Landroid/content/pm/UserInfo;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller$10;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/launcher2/WallpaperChooserDialogFragment$1;-onClick-(Landroid/view/View;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-isKeyguardSecure-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/deviceinfo/Status;-setWifiStatus-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/settings/wifi/AdvancedWifiSettings;-initPreferences-()V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-setSearchMode-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/PhoneGlobals;-updateInCallScreen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/latin/LatinIME;-resetContactsDictionary-(Lcom/android/inputmethod/latin/ContactsBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/view/ViewRootImpl;-updateJoystickDirection-(Landroid/view/MotionEvent; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/detail/ContactDetailFragment;-setData-(Landroid/net/Uri; Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothAdapter;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-handleModeChange-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-checkMailboxes-()J' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR'],
	'Lcom/android/browser/UrlHandler$RLZTask;-onPostExecute-(Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-access$200-(Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/SipAddressDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-getEnabledProviders-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/ILocationManager;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-startAction-(I I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener; Z Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-sendUssd-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/IBatteryStats$Stub$Proxy;-getStatistics-()[B' : ['android.permission.BATTERY_STATS', 'android.permission.BLUETOOTH'],
	'Landroid/webkit/GeolocationService;-registerForLocationUpdates-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-checkAndAddSourcePackageIntoValues-(Lcom/android/providers/contacts/VoicemailContentProvider$UriData; Landroid/content/ContentValues;)V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/AccountUnlockScreen$1;-run-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/phone/InCallScreen;-access$1100-(Lcom/android/phone/InCallScreen;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/accounts/AccountManager$2;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/EasAccountService;-postAutodiscover-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-checkSelection-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$4600-(Lcom/android/mms/ui/ComposeMessageActivity; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/sip/SipSessionGroup;-processIOException-(Ljavax/sip/IOExceptionEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/recent/RecentsPanelView;-handleOnClick-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub$Proxy;-startFreezingScreen-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub$Proxy;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/settings/Settings$SecuritySettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-turnOffUpstreamMobileConnection-()Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-isSyncActive-()Z' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/IConnectivityManager$Stub;-setGlobalProxy-(Landroid/net/ProxyProperties;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setPropertiesByCarrier-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ContactEntryListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$5500-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/contacts/activities/NonPhoneActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/SyncManager;-cancelActiveSync-(Landroid/accounts/Account; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-enableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/common/contacts/BaseEmailAddressAdapter$DirectoryPartitionFilter;-performFiltering-(Ljava/lang/CharSequence;)Landroid/widget/Filter$FilterResults;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/net/UidDetailProvider;-buildUidDetail-(I)Lcom/android/settings/net/UidDetail;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-retrieveSettings-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-setState-(I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-setQueryString-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/TileImageView;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestionProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-checkSelection-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-trySetupData-(Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/ActivityState;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackFragment;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/settings/accounts/ManageAccountsSettings;-onResume-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer$1;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ITelephony$Stub$Proxy;-isSimPinEnabled-()Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$ScreenNailJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-onResume-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetoothManager;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/ActivityController;-onLowMemory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/text/StringLogger;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/base/InputStreamSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/phone/OutgoingCallBroadcaster$OutgoingCallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-waitForKeyguard-(Landroid/view/WindowManagerPolicy$ScreenOnListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth;-disable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/InCallScreen;-onBackPressed-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-createSocketChannel-(I Ljava/lang/String; Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/FaceClustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/PhoneUtils;-turnOnSpeaker-(Landroid/content/Context; Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-updateUsbState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-disable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebSettingsClassic;-setDefaultZoom-(Landroid/webkit/WebSettings$ZoomDensity;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/OtaUtils;-onClickOtaActivateNextButton-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/mms/ui/DeliveryReportActivity;-getMmsReportRequests-()Ljava/util/List;' : ['android.permission.READ_SMS'],
	'Landroid/content/IContentService$Stub$Proxy;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/accounts/AccountManager$10;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/exchange/ExchangeService$2;-stopSync-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-isMailboxSyncable-(Lcom/android/emailcommon/provider/Account; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/calllog/CallLogListItemHelper;-getCallActionDescription-(Lcom/android/contacts/PhoneCallDetails;)Ljava/lang/CharSequence;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/webkit/WebViewClassic;-recordNewContentSize-(I I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter$DayAdapterInfo;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/providers/contacts/GlobalSearchSupport;-addSearchSuggestionsBasedOnFilter-(Landroid/database/MatrixCursor; Landroid/database/sqlite/SQLiteDatabase; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/OtaUtils;-otaShowActivateScreen-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/phone/PhoneGlobals;-updateBluetoothIndication-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/MmsMessageSender;-sendReadRec-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/picasasource/PicasaSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/data/WorkingMessage;-access$300-(Lcom/android/mms/data/WorkingMessage; Landroid/net/Uri;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView$2;-onBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/group/GroupBrowseListAdapter;-getGroupUriFromId-(J)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-dispatchNormalMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/model/RawContactModifier;-migrateStructuredName-(Landroid/content/Context; Lcom/android/contacts/model/RawContactDelta; Lcom/android/contacts/model/RawContactDelta; Lcom/android/contacts/model/dataitem/DataKind;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/ConnectivityService;-getTetheringErroredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/sip/SipEditor;-access$500-(Lcom/android/phone/sip/SipEditor; Landroid/net/sip/SipProfile;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/Settings$DataUsageSummaryActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/util/NameConverter;-displayNameToStructuredName-(Landroid/content/Context; Ljava/lang/String;)Ljava/util/Map;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/CallerInfoCache$CacheAsyncTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitorCallback;-onBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendP2pStateChangedBroadcast-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phone/EmergencyCallHelper;-onServiceStateChanged-(Landroid/os/Message;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/net/NetworkStatsService;-access$700-(Lcom/android/server/net/NetworkStatsService; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/BrowserWebView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.INTERNET'],
	'Lcom/android/phone/InCallScreen;-endInCallScreenSession-(Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout$OnChallengeScrolledListener;-onScrollStateChanged-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/datepicker/DatePickerDialog$OnDateSetListener;-onDateSet-(Lcom/android/contacts/datepicker/DatePicker; I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-getLastPdpFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/FlipEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/calllog/DefaultVoicemailNotifier;-updateNotification-(Landroid/net/Uri;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/power/DisplayPowerController$4;-run-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/Tab$2;-onUnhandledKeyEvent-(Landroid/webkit/WebView; Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/Phone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-onSaveInstanceState-(Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/content/SyncManager;-scheduleLocalSync-(Landroid/accounts/Account; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/sip/SipSettings$2$1;-run-()V' : ['android.permission.USE_SIP'],
	'Lcom/android/gallery3d/ui/PhotoView;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/Settings$SecuritySettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$PowerUsageSummaryActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/CountryDetectorService$2;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/transaction/MessagingNotification;-access$300-(Landroid/content/Context; [J)I' : ['android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-getAllMessagesFromIccEf-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/quicksearchbox/util/HttpHelper;-post-(Ljava/lang/String; Ljava/util/Map; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/calendar/DayView;-access$5100-(Lcom/android/calendar/DayView; Landroid/view/MotionEvent;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-invokeExitMethods-(Lcom/android/internal/util/StateMachine$SmHandler$StateInfo;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/provider/Contacts$People;-addToGroup-(Landroid/content/ContentResolver; J Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/text/ToUpperCase;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/SipAddressDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-requestChildFocus-(Landroid/view/View; Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/Utils$UsabilityStudyLogUtils$2;-run-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE'],
	'Lcom/android/server/CertBlacklister;-registerObservers-(Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/UserDictionary$Words;-addWord-(Landroid/content/Context; Ljava/lang/String; I Ljava/lang/String; Ljava/util/Locale;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Landroid/webkit/WebViewProvider;-addJavascriptInterface-(Ljava/lang/Object; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-removeAccount-(Landroid/accounts/Account; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/transaction/TransactionService;-renewMmsConnectivity-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/mms/ui/SlideshowPresenter$5;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-loadDictionaryAsync-()V' : ['android.permission.READ_USER_DICTIONARY'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStartingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager;-getTetherableWifiRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/telephony/TelephonyManager;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$BluetoothStateTracker;-setCurrentState-(Landroid/content/Context; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-selectDefaultContact-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState$SwitchActions;-startLongPressTimer-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewProvider;-documentHasImages-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/ShutdownThread;-access$000-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mms/ui/RecipientsAdapter;-runQueryOnBackgroundThread-(Ljava/lang/CharSequence;)Landroid/database/Cursor;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-setGrabbedState-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$12;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/mms/ui/ComposeMessageActivity$24;-run-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/Controller;-onProgressChanged-(Lcom/android/browser/Tab;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/DataUsageSummary$10;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/PanProfile;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/ISms;-updateMessageOnIccEf-(I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/browser/Tab$2;-onReceivedLoginRequest-(Landroid/webkit/WebView; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/effect/effects/BlackWhiteEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Connected;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/ITelephony$Stub$Proxy;-disableDataConnectivity-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$SoftApStartingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/ZoomManager;-setZoomScale-(F Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothHeadset;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$1100-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/phone/sip/SipEditor$2;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/TimerRingService;-onCreate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/DownloadThread;-executeDownload-(Lcom/android/providers/downloads/DownloadThread$State; Landroid/net/http/AndroidHttpClient; Lorg/apache/http/client/methods/HttpGet;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothPan;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/IMediaPlaybackService;-prev-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$OngoingGroupRemovalState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/am/ActivityStack;-scheduleIdleLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/webkit/WebViewCore$EventHub;-sendMessage-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/PreparePageFadeoutTexture;-onGLIdle-(Lcom/android/gallery3d/ui/GLCanvas; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-pickPhoto-(I Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/util/DownloadManager$4;-run-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/systemui/recent/RecentsVerticalScrollView;-onChildDismissed-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/CommandsInterface;-hangupConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vpndialogs/ManageDialog;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$InitialState;-chooseUpstreamType-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$HeaderAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-composeAndSendCallLogVcards-(I Ljavax/obex/Operation; I I Z)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-onScrubbingEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/SupplicantStateTracker$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-startScanActive-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/pm/IPackageManager$Stub;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/camera/PhotoModule;-onSharedPreferenceChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/deviceinfo/Status;-onResume-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAccountsByType-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-setFormattedDigits-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/os/IPowerManager$Stub;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/BrowserContract$Bookmarks;-buildFolderUri-(J)Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/Settings$AccessibilitySettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/BroadcastQueue;-deliverToRegisteredReceiverLocked-(Lcom/android/server/am/BroadcastRecord; Lcom/android/server/am/BroadcastFilter; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/ui/PhotoView$Picture;-draw-(Lcom/android/gallery3d/ui/GLCanvas; Landroid/graphics/Rect;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/accounts/ManageAccountsSettings;-onStart-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/AudioPreview;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-sendMessageRequest-(Lcom/android/exchange/Request;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/StructuredNameDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/ActivityManager;-getRecentTasks-(I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Landroid/content/ISyncContext$Stub$Proxy;-onFinished-(Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS'],
	'Landroid/net/sip/ISipService$Stub;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/wm/WindowManagerService;-handleAppTransitionReadyLocked-(Lcom/android/server/wm/WindowList;)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewInputDispatcher$WebKitCallbacks;-dispatchWebKitEvent-(Landroid/webkit/WebViewInputDispatcher; Landroid/view/MotionEvent; I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$WifiDisplaySettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-initApnContextsAndDataConnection-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/EmergencyCallHelper;-scheduleRetryOrBailOut-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/model/dataitem/DataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/PhotoPage;-access$1500-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/ContentService;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransfer;-startObexSession-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/NumPadKey;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/list/DefaultContactListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onActionBarAllowed-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/downloads/DownloadThread;-handleEndOfStream-(Lcom/android/providers/downloads/DownloadThread$State; Lcom/android/providers/downloads/DownloadThread$InnerState;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewClassic;-centerFitRect-(Landroid/graphics/Rect;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$2;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Exception;' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManagerService;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/googlecode/mp4parser/authoring/container/mp4/MovieCreator;-build-(Ljava/nio/channels/ReadableByteChannel;)Lcom/googlecode/mp4parser/authoring/Movie;' : ['android.permission.INTERNET'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter;-getSyncKey-()Ljava/lang/String;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/internal/SuddenJumpingTouchEventHandler;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/exchange/ExchangeService;-access$900-(Lcom/android/exchange/ExchangeService; J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/widget/PinnedHeaderListDemoActivity$TestPinnedHeaderListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame;-access$100-(Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-requestSync-(Lcom/android/emailcommon/provider/Mailbox; I Lcom/android/exchange/Request;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/BasePieView;-layout-(I I Z F I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-drawTopPanel-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-onFadeStart-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/LocationManagerService;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/IccRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IActivityManager;-goingToSleep-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/camera/PhotoController$2;-onClick-(Lcom/android/camera/ui/PieItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/dreams/phototable/PhotoCarousel$PhotoLoadTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter;-<init>-(Lcom/android/exchange/EasSyncService;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerSession;-onGetSuggestionsMultiple-([Landroid/view/textservice/TextInfo; I Z)[Landroid/view/textservice/SuggestionsInfo;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/ui/MessageItem;-interpretFrom-(Lcom/google/android/mms/pdu/EncodedStringValue; Landroid/net/Uri;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/IccProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/latin/Suggest;-getSuggestedWordsForBatchInput-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; I)Lcom/android/inputmethod/latin/SuggestedWords;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/preferences/FontSizePreview;-setupWebView-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/A2dpProfile;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/btservice/JniCallbacks;-pinRequestCallback-([B [B I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/IActivityManager;-killBackgroundProcesses-(Ljava/lang/String; I)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/browser/TabControl;-restoreState-(Landroid/os/Bundle; J Z Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-loadDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/model/dataitem/WebsiteDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransfer;-access$400-(Lcom/android/bluetooth/opp/BluetoothOppTransfer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/inputmethod/KeyboardLayoutPickerFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/power/PowerManagerService;-userActivityFromNative-(J I I)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-buildQueryUri-(I I Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Ljava/net/URLConnection;-getInputStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/CallDetailActivity;-getPhoneCallDetailsForUri-(Landroid/net/Uri;)Lcom/android/contacts/PhoneCallDetails;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/LocationManagerService$Receiver;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/StateManager;-switchState-(Lcom/android/gallery3d/app/ActivityState; Ljava/lang/Class; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/activities/AttachPhotoActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/bluetooth/btservice/ProfileService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/quicklaunch/QuickLaunchSettings;-clearShortcut-(C)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$InactiveState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/agenda/AgendaListView;-goTo-(Landroid/text/format/Time; J Ljava/lang/String; Z Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/settings/Settings$AccountSyncSettingsInAddAccountActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/location/ILocationManager;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/contacts/model/dataitem/DataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/content/ContextWrapper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/telephony/TelephonyManager;-listen-(Landroid/telephony/PhoneStateListener; I)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/CalendarViewAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_CALENDAR'],
	'Landroid/media/AsyncPlayer;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/IConnectivityManager$Stub;-getActiveLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/MediaSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar$7;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/ContactSaveService;-updateGroup-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/tech/NdefFormatable;-format-(Landroid/nfc/NdefMessage;)V' : ['android.permission.NFC'],
	'Lcom/android/mms/ui/ManageSimMessages;-init-()V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Landroid/view/WindowManagerPolicy$WindowManagerFuncs;-rebootSafeMode-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$1;-onDeleteGroup-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/gallery3d/data/LocationClustering;-generateName-(Ljava/util/ArrayList; Lcom/android/gallery3d/util/ReverseGeocoder;)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/TetherSettings;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/AccessibilityInjector;-toggleAndroidVox-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/os/IVibratorService$Stub;-vibratePattern-([J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker$ApnChangeObserver;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/FrameSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/test/IsolatedContext$MockAccountManager;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/net/ConnectivityManager;-getTetherableIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionActivity;-showConnectionDialog-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/quicksearchbox/AppsCorpus;-getSuggestions-(Ljava/lang/String; I Z)Lcom/android/quicksearchbox/CorpusResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/nfc/INfcAdapter$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/DataConnection$DcInactiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/NfcV;-reconnect-()V' : ['android.permission.NFC'],
	'Landroid/bluetooth/IBluetooth$Stub;-fetchRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/data/WorkingMessage;-discard-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onDataSetupComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/model/dataitem/WebsiteDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/util/DraftCache;-rebuildCache-()V' : ['android.permission.READ_SMS'],
	'Lcom/android/camera/SecureCameraActivity;-openModule-(Lcom/android/camera/CameraModule; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/app/PhotoPage;-setCurrentPhotoByIntent-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/Settings$PowerUsageSummaryActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-isNdef-(I)Z' : ['android.permission.NFC'],
	'Lcom/android/phone/PhoneUtils;-setMuteInternal-(Lcom/android/internal/telephony/Phone; Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onRadioAvailable-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/am/Am$IntentReceiver;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfile;-getConnectionStatus-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/SpecialCharSequenceMgr;-showMEIDPanel-(Landroid/content/Context; Z Landroid/telephony/TelephonyManager;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/location/ILocationManager$Stub;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/P2pLinkManager;-prepareMessageToSend-()V' : ['android.permission.GET_TASKS'],
	'Lcom/android/server/am/BatteryStatsService;-getStatistics-()[B' : ['android.permission.BATTERY_STATS', 'android.permission.BLUETOOTH'],
	'Landroid/net/IConnectivityManager;-getActiveLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-setQueryString-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/location/GpsLocationProvider$4;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/TelephonyManager;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ITelephony$Stub$Proxy;-enableDataConnectivity-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/IThrottleManager;-getCliffThreshold-(Ljava/lang/String; I)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/provider/Contacts$People;-addToMyContactsGroup-(Landroid/content/ContentResolver; J)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/DockObserver;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$1;-onDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/launcher2/Workspace;-setup-(Lcom/android/launcher2/DragController;)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/server/InputMethodManagerService;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/location/ILocationProvider$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityCallback;-showBackupSecurity-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/ZoomManager;-onNewPicture-(Landroid/webkit/WebViewCore$DrawData;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-access$300-(Lcom/android/server/am/ActivityManagerService; Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCallForward-(I I I Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-killApplication-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/keyguard/ObscureSpeechDelegate;-onInitializeAccessibilityNodeInfo-(Landroid/view/View; Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/CaptivePortalTracker$DelayedCaptiveCheckState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/systemui/statusbar/phone/QuickSettingsModel;-onUserSwitched-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/IWindowSession$Stub;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-onSurfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PickerActivity;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/BrowserActivity;-onSaveInstanceState-(Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/wm/WindowManagerService;-setForcedDisplaySizeLocked-(Lcom/android/server/wm/DisplayContent; I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/InCallScreen$14;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-access$000-(Lcom/android/bluetooth/opp/BluetoothOppService;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSimPinView;-onWindowFocusChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$AndroidBeamSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/EmergencyCallHelper;-onDisconnect-(Landroid/os/Message;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$14;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/AddBookmarkPage;-setupTopCrumb-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthTokenLabel-(Ljava/lang/String; Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/ConnectivityService;-enforcePreference-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/wifi/AdvancedWifiSettings;-refreshWifiInfo-()V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/DockService;-access$200-(Lcom/android/settings/bluetooth/DockService; Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfileManager;-setBluetoothStateOn-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/SupplicantStateTracker$HandshakeState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/Dictionary;-getSuggestions-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo;)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-onWindowFocusChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/util/IState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/nfc/tech/MifareUltralight;-writePage-(I [B)V' : ['android.permission.NFC'],
	'Lcom/android/server/ConnectivityService;-handleConnect-(Landroid/net/NetworkInfo;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter$EasEmailSyncParser;-parse-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/LatinIME;-restartSuggestionsOnWordBeforeCursor-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/app/ActivityManagerNative;-getRecentTasks-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-access$100-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-resetMessage-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout$9;-onAnimationStart-(Landroid/animation/Animator;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/HTML5VideoViewProxy$VideoPlayer;-pauseAndDispatch-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$SoftApStartingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/GrainEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/IConnectivityManager;-setMobileDataEnabled-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/webkit/WebView;-loadData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine;-registerForSettingsChanges-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/ContactsContract$ProfileSyncState;-get-(Landroid/content/ContentProviderClient; Landroid/accounts/Account;)[B' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView$2;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/DeliveryReportActivity;-getMmsReportItems-()Ljava/util/List;' : ['android.permission.READ_SMS'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-asyncReloadDictionaryIfRequired-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/CertBlacklister;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-onSurfaceRedrawNeeded-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/MediaPlayer;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/launcher2/PagedViewIcon;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub;-getTetherableWifiRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/location/fused/FusionEngine;-<init>-(Landroid/content/Context; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/PhoneUi;-showWeb-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-notifyApnIdUpToCurrent-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$3;-goToUnlockScreen-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/TabControl;-setCurrentTab-(Lcom/android/browser/Tab;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/WifiManager;-getWifiState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAbsKeyInputView;-onWindowFocusChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$ApplicationSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor;-handleUserSwitched-(I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebIconDatabaseClassic;-removeAllIcons-()V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/PhoneUi;-setUseQuickControls-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView$2;-onClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/ContentResolver;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/phone/PhoneUtils;-restoreNoiseSuppression-(Landroid/content/Context;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-setSelectedContactUri-(Landroid/net/Uri; Z Z Z Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/exchange/ContactsSyncAdapterService;-access$000-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle; Ljava/lang/String; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/wm/WindowManagerService;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/gallery3d/data/MtpDeviceSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/WiredAccessoryManager;-bootCompleted-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewCore$EventHub;-dispatchWebKitEvent-(Landroid/webkit/WebViewInputDispatcher; Landroid/view/MotionEvent; I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity;-eventsChanged-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/nfc/NfcAdapter;-disableForegroundDispatch-(Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Landroid/location/ILocationManager$Stub;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/INfcAdapterExtras;-authenticate-(Ljava/lang/String; [B)V' : ['android.permission.NFC'],
	'Lcom/android/voicedialer/VoiceDialerActivity$1;-onServiceConnected-(I Landroid/bluetooth/BluetoothProfile;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/users/UserSettings;-assignProfilePhoto-(Landroid/content/pm/UserInfo;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/WifiStateMachine$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/SMSDispatcher;-dispatchPortAddressedPdus-([[B I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/AudioPreview;-stopPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/am/ActivityManagerService$19;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/common/widget/GroupingListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/downloads/DownloadProvider;-checkInsertPermissions-(Landroid/content/ContentValues;)V' : ['android.permission.INTERNET'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-showBrightnessDialog-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-updateMonthHighlight-(Landroid/widget/AbsListView;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/content/pm/IPackageManager;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/browser/preferences/InvertedContrastPreview;-updatePreview-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/location/ILocationManager;-addGpsStatusListener-(Landroid/location/IGpsStatusListener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnablingActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/internal/telephony/CommandsInterface;-cancelPendingUssd-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-getSuggestionsWithSessionId-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; I)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/exchange/ExchangeService;-acquireWakeLock-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-isValidWord-(Ljava/lang/CharSequence;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity$QueryEntitiesTask;-doInBackground-([Landroid/content/Intent;)Lcom/android/contacts/model/RawContactDeltaList;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView$KeyTimerHandler;-startKeyRepeatTimer-(Lcom/android/inputmethod/keyboard/PointerTracker;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/nfc/NfcService$TagService;-getTimeout-(I)I' : ['android.permission.NFC'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/SpecialCharSequenceMgr;-showIMEIPanel-(Landroid/content/Context; Z Landroid/telephony/TelephonyManager;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/util/DownloadManager;-getMessage-(Landroid/net/Uri;)Ljava/lang/String;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/google/common/io/CharStreams$2;-getInput-()Ljava/io/InputStreamReader;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-onAttachedToWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneBase;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton$1;-onDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/agenda/AgendaFragment;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar$1;-run-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/exchange/adapter/CalendarSyncAdapter$EasCalendarSyncParser;-commandsParser-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/model/dataitem/WebsiteDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-startDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/PostalAddressListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-updateFolderList-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/DataManager;-onStitchingResult-(Landroid/net/Uri;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/AlarmInitReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FlipFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/LocalPhotoSource;-getImage-(I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/contacts/SplitAggregateView;-<init>-(Landroid/content/Context; Landroid/net/Uri;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-updateProgressbar-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView$2;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/InputMethodManagerService;-updateFromSettingsLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimUnlockScreen$1$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/calendar/SearchActivity$2;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/model/dataitem/EmailDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onProgressComplete-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/inputmethod/research/ResearchLogger;-onUserLoggingConsent-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/nfc/P2pEventManager;-onP2pHandoverNotSupported-()V' : ['android.permission.VIBRATE'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/SupplicantStateTracker$DisconnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.INTERNET', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/wm/WindowManagerService;-removeWindowInnerLocked-(Lcom/android/server/wm/Session; Lcom/android/server/wm/WindowState;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/VibratorService;-systemReady-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityStack;-activityIdleInternal-(Landroid/os/IBinder; Z Landroid/content/res/Configuration;)Lcom/android/server/am/ActivityRecord;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/Phone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-onGenericMotionEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/RequestPermissionHelperActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/settings/bluetooth/A2dpProfile;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/DTMFTwelveKeyDialer;-onDialerClose-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/LocationManagerService;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/Chronometer;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/server/BluetoothA2dpService;-onBluetoothEnable-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/ContactEntryListAdapter;-bindQuickContact-(Lcom/android/contacts/list/ContactListItemView; I Landroid/database/Cursor; I I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/model/RawContactModifier;-migrateStateForNewContact-(Landroid/content/Context; Lcom/android/contacts/model/RawContactDelta; Lcom/android/contacts/model/RawContactDelta; Lcom/android/contacts/model/account/AccountType; Lcom/android/contacts/model/account/AccountType;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-acquireAggregationSuggestions-(Landroid/content/Context; Lcom/android/contacts/editor/RawContactEditorView;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/PhoneProxy;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/EventInfoFragment$11;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/wm/KeyguardDisableHandler$KeyguardTokenWatcher;-released-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccCardProxy;-setExternalState-(Lcom/android/internal/telephony/IccCardConstants$State; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-getProfileProxy-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManagerService;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-setAddWidgetEnabled-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-cancelDiscovery-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/LocationManagerService;-addGpsStatusListener-(Landroid/location/IGpsStatusListener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-handleHide-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/speech/RecognitionService;-checkPermissions-(Landroid/speech/IRecognitionListener;)Z' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/server/input/InputManagerService;-notifyANR-(Lcom/android/server/input/InputApplicationHandle; Lcom/android/server/input/InputWindowHandle;)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onRemoteFieldActivated-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/accounts/AccountManager;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/systemui/media/NotificationPlayer$CmdThread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/calllog/CallLogFragment;-startVoicemailStatusQuery-()V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/contacts/list/AccountFilterActivity$MyLoaderCallbacks;-onLoadFinished-(Landroid/content/Loader; Ljava/util/List;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-scrollRight-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-trySwitchToFirstTouchState-(F F)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onSetUserDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardMessageArea$2;-onDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/monkey/MonkeyNetworkMonitor;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mms/data/Conversation;-startQueryHaveLockedMessages-(Landroid/content/AsyncQueryHandler; J I)V' : ['android.permission.READ_SMS'],
	'Landroid/bluetooth/BluetoothInputDevice;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/JoinContactListFragment;-onStart-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/AddBookmarkPage;-onRootFolderFound-(J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Ljava/net/URL;-openStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothPan;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/sip/ISipService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/provider/ContactsContract$Groups$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/IBluetoothPan;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewClassic;-updateTextSelectionFromMessage-(I I Landroid/webkit/WebViewCore$TextSelectionData;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/GoogleAccountLogin;-<init>-(Landroid/app/Activity; Landroid/accounts/Account; Ljava/lang/Runnable;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/nfc/NfcService$TagService;-setTimeout-(I I)I' : ['android.permission.NFC'],
	'Landroid/bluetooth/BluetoothAdapter;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/WifiService;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/settings/AccessibilitySettings$2;-register-(Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/contacts/editor/PhoneticNameEditorView;-onFieldChanged-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/event/EditEventHelper;-saveRemindersWithBackRef-(Ljava/util/ArrayList; I Ljava/util/ArrayList; Ljava/util/ArrayList; Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/AlbumPage;-doCluster-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-beginBackup-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/effect/effects/StraightenEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/browser/AddBookmarkPage;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/providers/calendar/CalendarReceiver;-removeScheduledAlarms-(Landroid/content/ContentResolver;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/EasAccountService;-sendMeetingResponse-(Lcom/android/exchange/MeetingResponseRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStoppingState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/SingleSourceCorpus;-getSuggestions-(Ljava/lang/String; I Z)Lcom/android/quicksearchbox/CorpusResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-onMenuClicked-(Landroid/view/MenuItem; Ljava/lang/String; Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/effect/effects/NegativeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/DataUsageSummary;-isMobilePolicySplit-()Z' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/tts/TextToSpeechSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/calllog/CallLogFragment;-callSelectedEntry-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/SubSettings;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager;-dispatchDeviceAdded-(Lcom/android/settings/bluetooth/CachedBluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-onScreenTurnedOn-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/location/LocationManager;-addNmeaListener-(Landroid/location/GpsStatus$NmeaListener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getScanResults-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-onLoadComplete-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsFragment;-eventsChanged-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/nfc/P2pEventManager;-onP2pInRange-()V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/AccessibilityInjector;-addTtsApis-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/RetrieveTransaction;-sendPdu-([B)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/group/GroupEditorFragment$3;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/BluetoothHeadset;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/AlarmManager;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/gallery3d/app/CropImage$LoadBitmapDataTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Landroid/net/wifi/WifiManager;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getOperator-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-replaceSelection-(Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/SmsUsageMonitor;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/display/DisplayManagerService;-setWindowManager-(Lcom/android/server/display/DisplayManagerService$WindowManagerFuncs;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/RecipientListActivity$RecipientListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/alerts/QuickResponseActivity$QueryThread;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-computeSharingIntent-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Landroid/content/Intent;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendRawPdu-([B [B Landroid/app/PendingIntent; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/UndoBarView;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccCardProxy;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$WatchdogDisabledState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-updateClusters-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/ConnectivityManager;-startUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/view/inputmethod/InputMethodManager;-dispatchGenericMotionEvent-(Landroid/content/Context; I Landroid/view/MotionEvent; Landroid/view/inputmethod/InputMethodManager$FinishedEventCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcTag$Stub;-formatNdef-(I [B)I' : ['android.permission.NFC'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-onSurfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/location/GeofenceManager;-updateFences-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/GLRootView;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-handleShift-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/view/WindowManagerPolicy;-isKeyguardSecure-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/vcard/VCardEntry$PostalData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/view/ViewRootImpl;-finishInputEvent-(Landroid/view/ViewRootImpl$QueuedInputEvent; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/data/WorkingMessage;-sendMmsWorker-(Lcom/android/mms/data/Conversation; Landroid/net/Uri; Lcom/google/android/mms/pdu/PduPersister; Lcom/android/mms/model/SlideshowModel; Lcom/google/android/mms/pdu/SendReq; Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/AlbumPicker;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onChanged-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-handleCarrierInfoUpdate-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/CallCard;-displaySecondaryCallStatus-(Lcom/android/internal/telephony/CallManager; Lcom/android/internal/telephony/Call;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/PhoneFallbackEventHandler;-handleMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-access$1800-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/IccCardProxy;-changeIccFdnPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaFragment;-goTo-(Lcom/android/calendar/CalendarController$EventInfo; Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/filterfw/core/SyncRunner;-step-()I' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/sip/SipManager;-createSipSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/SipSession$Listener;)Landroid/net/sip/SipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/PhoneGlobals;-access$900-(Lcom/android/phone/PhoneGlobals; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/TransactionService;-endMmsConnectivity-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/BackupManagerService$RunBackupReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BackupManagerService;-handleTimeout-(I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-setRadio-(I Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/IConnectivityManager;-getTetherableBluetoothRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/vcard/VCardParserImpl_V40;-parseItem-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/InCallTouchUi;-updateState-(Lcom/android/internal/telephony/CallManager;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/TrafficStats;-getDataLayerSnapshotForUid-(Landroid/content/Context;)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-syncReloadDictionaryInternal-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/data/MtpDeviceSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/wifi/WifiSettings;-forget-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/AudioService;-queueMsgUnderWakeLock-(Landroid/os/Handler; I I I Ljava/lang/Object; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DeviceInfoSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/PhoneUtils;-cancelMmiCode-(Lcom/android/internal/telephony/Phone;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/Tab$3;-access$1700-(Lcom/android/browser/Tab$3; Z Landroid/os/Message;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-showChallenge-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/VolumePanel;-onVolumeChanged-(I I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/phone/sip/SipSettings;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/providers/calendar/CalendarProvider2;-verifyAccounts-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/EasOutboxService;-tryAutodiscover-(Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHealth;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-sendEmergencyCallbackModeChange-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$800-(Lcom/android/server/net/NetworkPolicyManagerService;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/Utils;-getWifiIpAddresses-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-dispatch-(Landroid/content/Intent; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService;-refreshImeWindowVisibilityLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText$1;-onDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/Phone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-systemReady-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-pageBeginMoving-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/LockPatternView;-handleActionMove-(Landroid/view/MotionEvent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/location/ILocationProvider$Stub$Proxy;-disable-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/server/AlarmManagerService$ResultReceiver;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityRecord;-keyDispatchingTimedOut-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/inputmethodservice/KeyboardView;-access$400-(Landroid/inputmethodservice/KeyboardView; Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-removeAllDevices-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$WifiP2pSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/users/UserSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/InputStreamSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/app/MovieActivity;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/voicedialer/VoiceDialerActivity$4;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$2;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-startRunning-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumPage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/ui/AudioAttachmentView;-access$000-(Lcom/android/mms/ui/AudioAttachmentView;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/ComboAlbum;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/DevelopmentSettings;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/common/widget/CompositeCursorAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/net/NetworkPolicyEditor$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-stopDtmf-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-onScrubbingStart-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncManager$ActiveSyncContext;-onFinished-(Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/widget/CalendarAppWidgetModel;-populateDayInfo-(I Landroid/text/format/Time;)Lcom/android/calendar/widget/CalendarAppWidgetModel$DayInfo;' : ['android.permission.READ_CALENDAR'],
	'Landroid/location/ILocationManager;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/nfc/INfcTag$Stub;-isNdef-(I)Z' : ['android.permission.NFC'],
	'Landroid/location/ILocationManager;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/view/IWindowSession$Stub$Proxy;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserWebView;-requestFocus-(I Landroid/graphics/Rect;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/TechListChooserActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/exchange/adapter/AbstractSyncParser;-commandsParser-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/InCallScreen;-access$1000-(Lcom/android/phone/InCallScreen; Landroid/os/AsyncResult; C)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/widget/WaveView;-waveUpdateFrame-(F F Z)V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/WebViewInputDispatcher;-dispatchUiEvents-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcTag$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.NFC'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/CommandsInterface;-explicitCallTransfer-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/WorkingMessage;-ensureThreadIdIfNeeded-(Lcom/android/mms/data/Conversation; Z)V' : ['android.permission.READ_SMS'],
	'Lcom/android/contacts/model/dataitem/GroupMembershipDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-updateMonthHighlight-(Landroid/widget/AbsListView;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/HTML5VideoFullScreen$1;-surfaceDestroyed-(Landroid/view/SurfaceHolder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/EmergencyCallHelper;-startEmergencyCallFromAirplaneModeSequence-(Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/media/AudioService$AudioHandler;-cleanupPlayer-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-handleUp-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/VibratorService;-doVibratorOff-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferAdapter;-bindView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-getProfileConnectionState-(Lcom/android/settings/bluetooth/LocalBluetoothProfile;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar$24;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Landroid/media/IAudioService;-setBluetoothA2dpDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice; I)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler$PhotoEditorListener;-onRequest-(I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchToPrevImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ComposeMessageActivity$SendIgnoreInvalidRecipientListener;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-handleExitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IActivityManager;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readIapFileAndWait-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-registerForSuppServiceNotification-(Landroid/os/Handler; I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/SIMFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/InputStreamSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewProvider;-goBack-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SyncStateTracker;-setCurrentState-(Landroid/content/Context; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/data/WorkingMessage$1;-doInBackground-([Ljava/lang/Void;)Landroid/util/Pair;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onSingleTapUp-(I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/DeviceStorageMonitorService;-sendNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setDiscoverableTimeout-(I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView$1;-onTrigger-(Landroid/view/View; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/location/LocationManager;-isProviderEnabled-(Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-getBetterNumberFromContacts-(Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-updateSettings-(Landroid/content/ContentValues;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/P2pEventListener;-onP2pInRange-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/providers/contacts/VoicemailContentTable;-update-(Lcom/android/providers/contacts/VoicemailContentProvider$UriData; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/google/common/io/Files;-getDigest-(Ljava/io/File; Ljava/security/MessageDigest;)[B' : ['android.permission.INTERNET'],
	'Lcom/android/phone/InCallScreen;-requestUpdateBluetoothIndication-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/model/dataitem/PhoneDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ManageSimMessages;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$19;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Landroid/widget/VideoView$6;-surfaceDestroyed-(Landroid/view/SurfaceHolder;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/MessageUtils;-getNotificationIndDetails-(Landroid/content/Context; Landroid/database/Cursor;)Ljava/lang/String;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/sip/SipService$AutoRegistrationProcess;-onResponse-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onVoiceCallEnded-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/data/Conversation;-loadFromThreadId-(J Z)Z' : ['android.permission.READ_SMS'],
	'Landroid/net/wifi/IWifiManager;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-stayAwake-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService$Receiver;-callStatusChangedLocked-(Ljava/lang/String; I Landroid/os/Bundle;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCallForwardStatus-(I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-removeFence-(Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/widget/ChartSweepView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/provider/Browser;-clearSearches-(Landroid/content/ContentResolver;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/bluetooth/DockService;-msgTypeDocked-(Landroid/bluetooth/BluetoothDevice; I I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewInputDispatcher;-enqueueWebKitEventLocked-(Landroid/webkit/WebViewInputDispatcher$DispatchEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasAccountService;-acknowledgeProvisionImpl-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String; Z)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Contacts$ContactMethods;-addPostalLocation-(Landroid/content/Context; J D D)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-getFreshLocationLocked-()Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiStateMachine$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-trySwitchToFirstTouchState-(F F)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/provider/CallLog$Calls;-getLastOutgoingCall-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/EdgeView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/CallDetailActivity$1UpdateContactDetailsTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/vcard/SelectAccountActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/SIMFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IBackupAgent;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/p2p/WifiP2pService;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/editor/ContactEditorFragment$EntityDeltaComparator;-compare-(Ljava/lang/Object; Ljava/lang/Object;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/app/IBatteryStats;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/DataUsageSummary;-getActiveSubscriberId-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler;-getDeltaForAttachingPhotoToContact-()Lcom/android/contacts/model/RawContactDeltaList;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/InCallScreen;-internalAnswerCall-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-updateNetworkRulesLocked-()V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/inputmethod/latin/UserHistoryDictionary;-getSuggestionsWithSessionId-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; I)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarController;-sendEventRelatedEvent-(Ljava/lang/Object; J J J J I I J)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/power/PowerManagerService;-setTemporaryScreenBrightnessSettingOverrideInternal-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onCodeInput-(I I I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$DefaultState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/power/PowerManagerService;-wakeUpInternal-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/SnailAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/statusbar/BaseStatusBar$NotificationClicker;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.DISABLE_KEYGUARD', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewClassic;-finalize-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-updatePeople-(J Landroid/content/ContentValues;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-syncReloadDictionaryIfRequired-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/sip/ISipService;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Landroid/webkit/ZoomManager;-setZoomScale-(F Z Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-onClickRecentButton-()V' : ['android.permission.GET_TASKS'],
	'Landroid/app/NativeActivity;-preDispatchKeyEvent-(Landroid/view/KeyEvent; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebView;-goBack-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-pollState-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-addCallAndContactMenuItems-(Landroid/view/ContextMenu; Lcom/android/mms/ui/ComposeMessageActivity$MsgListMenuClickListener; Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/NfcService;-doDisconnect-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DataUsageSummary;-buildLimitedNetworksList-()Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-disableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/mms/transaction/RetrieveTransaction;-run-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/server/input/InputManagerService;-interceptKeyBeforeDispatching-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)J' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/content/IContentService$Stub$Proxy;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/phone/InCallTouchUi;-onGrabbedStateChange-(Landroid/view/View; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-outputRawContactsAsVCard-(Landroid/net/Uri; Ljava/io/OutputStream; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/content/SyncManager$SyncHandler;-maybeStartNextSyncLocked-()J' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/nfc/NfcAdapter;-setOnNdefPushCompleteCallback-(Landroid/nfc/NfcAdapter$OnNdefPushCompleteCallback; Landroid/app/Activity; [Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-selectAll-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/GlobalActions$GlobalActionsDialog;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/accounts/AccountManager;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/editor/SelectAccountDialogFragment;-onAccountSelected-(Lcom/android/contacts/model/account/AccountWithDataSet;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/AlbumPicker;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame;-onDetachedFromWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/IWindowManager$Stub;-setForcedDisplayDensity-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hdp/HealthService;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView$KeyTimerHandler;-startKeyRepeatTimer-(Lcom/android/inputmethod/keyboard/PointerTracker; J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/activities/GroupDetailActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SyncStateTracker;-getActualState-(Landroid/content/Context;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-updateWallpaperLocked-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/contacts/editor/LabeledEditorView;-notifyEditorListener-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-sendUpdates-(Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitorCallback;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/sip/SipService$ConnectivityReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerPolicy;-exitKeyguardSecurely-(Landroid/view/WindowManagerPolicy$OnKeyguardExitResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/SecureAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(J F Landroid/location/Criteria; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/research/ResearchLogger;-onWordComplete-(Ljava/lang/String;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-onContentChanged-()V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/telephony/ims/IsimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView$AccountAnalyzer;-next-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/wifi/WifiAPITest$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/contacts/CallLogProvider;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/BrowserBackupAgent;-onRestore-(Landroid/app/backup/BackupDataInput; I Landroid/os/ParcelFileDescriptor;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator$ViewMediatorCallback;-keyguardDone-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-setPhoto-(J Landroid/graphics/Bitmap; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-createServerSession-(Ljavax/obex/ObexTransport;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadsetPhone$Stub;-answerCall-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/music/IMediaPlaybackService;-setShuffleMode-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-setShuffleMode-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$StateTracker;-setCurrentState-(Landroid/content/Context; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/SlotView$Listener;-onLongTap-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/transaction/RetrieveTransaction;-getPdu-(Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/bluetooth/PbapServerProfile;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/wifi/WifiSettings;-onResume-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/camera/ActivityBase;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/BackupManagerService$PerformFullBackupTask;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/BackupManagerService$PerformFullRestoreTask$RestoreFileRunnable;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/model/dataitem/StructuredPostalDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothInputDevice;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/widget/MmsWidgetService$MmsFactory;-onDataSetChanged-()V' : ['android.permission.READ_SMS'],
	'Lcom/android/phone/sip/SipSettings;-access$500-(Lcom/android/phone/sip/SipSettings; Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/bluetooth/btservice/AdapterService;-fetchRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-updateShare-(Landroid/database/Cursor; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/phone/InCallScreen;-addVoiceMailNumberPanel-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/music/MusicUtils;-playAll-(Landroid/content/Context; [J I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-getCurrentDeviceName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/accounts/AddAccountSettings;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/editor/RawContactEditorView;-setPhotoBitmap-(Landroid/graphics/Bitmap;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/bluetooth/IBluetoothA2dp;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/EmailAddressListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/WebViewClassic;-goForward-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-updatePhotoByDataId-(J Landroid/content/ContentValues;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/camera/ui/InLineSettingItem;-changeIndex-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth;-createBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/vcard/VCardEntry;-toString-()Ljava/lang/String;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/IccCardProxy;-setIccFdnEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/SnapshotTab;-<init>-(Lcom/android/browser/WebViewController; Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-cancelBondProcess-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/common/widget/GroupingListAdapter;-bindStandAloneView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetooth;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity;-access$000-(Lcom/android/contacts/activities/PhotoSelectionActivity;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-dialRecipient-()V' : ['android.permission.CALL_PHONE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier; Landroid/net/sip/SipProfile;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-setCurrentPreferredNetworkType-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AutofillHandler;-asyncLoadFromDb-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-refresh-(Landroid/text/format/Time; J Ljava/lang/String; Z Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/phone/PhoneApp;-onCreate-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_CALL_LOG', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity;-setEntityDeltaList-(Lcom/android/contacts/model/RawContactDeltaList;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/detail/PhotoSelectionHandler;-getWritableEntityIndex-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/SupplicantStateTracker$DefaultState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSimPukView$4$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity;-access$900-(Lcom/android/contacts/activities/ConfirmAddDetailActivity; J Landroid/net/Uri;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/PhoneBase;-setPropertiesByCarrier-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$2700-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BluetoothManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/DhcpStateMachine$WaitBeforeRenewalState;-exit-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/util/NameConverter;-structuredNameToDisplayName-(Landroid/content/Context; Landroid/content/ContentValues;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/AdnRecordCache;-updateAdnBySearch-(I Lcom/android/internal/telephony/AdnRecord; Lcom/android/internal/telephony/AdnRecord; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/PhoneUi;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/net/NetworkStatsService;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/SlideshowView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/util/Recycler$MmsRecycler;-deleteOldMessagesInSameThreadAsMessage-(Landroid/content/Context; Landroid/net/Uri;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/Settings$VpnSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$2;-onRemoveViewAnimationCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/NavScreen$1;-onRemovePosition-(I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/data/Contact$ContactsCache;-getContactInfoForSelf-()Lcom/android/mms/data/Contact;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/CallbackProxy;-onSavePassword-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/PhoneInterfaceManager;-call-(Ljava/lang/String;)V' : ['android.permission.CALL_PHONE'],
	'Lcom/android/server/wm/WindowAnimator$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/LocationManagerService;-access$300-(Lcom/android/server/LocationManagerService; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ConversationListItem;-access$000-(Lcom/android/mms/ui/ConversationListItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-isTetheringSupported-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/ManageConferenceUtils$2;-onClick-(Landroid/view/View;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView$1;-onTextInput-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/BluetoothManagerService;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/server/pm/PackageManagerService;-installNewPackageLI-(Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/phone/CallerInfoCache$CacheAsyncTask;-onCancelled-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onActionIntentReconnectAlarm-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-onScrubbingEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/TechListChooserActivity;-onCreate-(Landroid/os/Bundle; Landroid/content/Intent; Ljava/lang/CharSequence; [Landroid/content/Intent; Ljava/util/List; Z)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-onMeasure-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasAccountService;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity;)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-trySetupData-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phone/InCallScreen;-requestCloseOtaFailureNotice-(J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-onPause-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/IccProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$BondStateChangedHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-snapToPageWithVelocity-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-onContactUriQueryFinished-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/Tab;-loadUrl-(Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/MediaSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionActivity;-onNegative-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/os/RuntimeInit$UncaughtHandler;-uncaughtException-(Ljava/lang/Thread; Ljava/lang/Throwable;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/content/SyncManager;-readDataConnectionState-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-applyWakeLockFlagsOnAcquireLocked-(Lcom/android/server/power/PowerManagerService$WakeLock;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/net/DataUsageMeteredSettings;-buildMobilePref-(Landroid/content/Context;)Landroid/preference/Preference;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-setOnChallengeScrolledListener-(Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout$OnChallengeScrolledListener;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/ContactEditorActivity;-onBackPressed-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/content/IContentService$Stub;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/deskclock/AlarmKlaxon;-onCreate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MediaPlaybackService;-removeTracksInternal-(I I)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/PhotoModule;-init-(Lcom/android/camera/CameraActivity; Landroid/view/View; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/model/dataitem/NicknameDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/effect/effects/AutoFixEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/BluetoothHeadset;-startScoUsingVirtualVoiceCall-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/model/dataitem/DataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/ZoomButtonsController;-onContainerKey-(Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/server/search/SearchManagerService$GlobalSearchProviderObserver;-<init>-(Landroid/server/search/SearchManagerService; Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/CommandQueue$Callbacks;-showSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Landroid/net/IConnectivityManager$Stub;-getAllNetworkInfo-()[Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/EmergencyDialer;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/vcard/VCardParserImpl_V21;-parseItems-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/IContentService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/browser/DeviceAccountLogin;-handleLogin-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-onProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/event/AttendeesView;-access$100-(Lcom/android/calendar/event/AttendeesView; Lcom/android/calendar/event/EditEventHelper$AttendeeItem;)Landroid/view/View;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/PhoneGlobals;-updateProximitySensorMode-(Lcom/android/internal/telephony/PhoneConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/location/GpsLocationProvider;-access$2100-(Lcom/android/server/location/GpsLocationProvider;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker$2;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/gallery3d/data/ClusterAlbum$2;-consume-(I Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/BatteryStatsService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BATTERY_STATS', 'android.permission.BLUETOOTH'],
	'Lcom/android/browser/AddBookmarkPage$EditBookmarkInfoLoader;-loadInBackground-()Ljava/lang/Object;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/content/SyncManager;-sendCancelSyncsMessage-(Landroid/accounts/Account; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/display/DisplayManagerService;-setInputManager-(Lcom/android/server/display/DisplayManagerService$InputManagerFuncs;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/btservice/AdapterService;-access$100-(Lcom/android/bluetooth/btservice/AdapterService; Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$GpsStateTracker;-toggleState-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/SnailSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-pollState-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-verifyUnlock-(Landroid/view/WindowManagerPolicy$OnKeyguardExitResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/provider/ContactsContract$ProfileSyncState;-getWithUri-(Landroid/content/ContentProviderClient; Landroid/accounts/Account;)Landroid/util/Pair;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/WifiStateMachine$DriverStartingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-access$2000-(Lcom/android/server/BluetoothManagerService;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/PhoneProxy;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/OnContactBrowserActionListener;-onInvalidSelection-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/NavScreen;-openNewTab-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/nfc/tech/NdefFormatable;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/gallery3d/app/PhotoPage$6;-onLoadingFinished-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/BrowserWebView;-zoomIn-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/group/GroupBrowseListFragment$3;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewInputDispatcher;-dispatchWebKitEvents-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleEctIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText;-getCarrierHelpTextForSimState-(Lcom/android/internal/telephony/IccCardConstants$State; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/CharSequence;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/vcard/VCardParser_V21;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/camera/ui/OverlayRenderer;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/util/ThumbnailManager$ThumbnailTask$1;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/SlotView$MyGestureListener;-onSingleTapUp-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter$EasContactsSyncParser;-parse-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-<init>-(J Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/activities/ActionBarAdapter;-initialize-(Landroid/os/Bundle; Lcom/android/contacts/list/ContactsRequest;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$AccountSyncSettingsInAddAccountActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler$PhotoListener;-onUseAsPrimaryChosen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/activities/ActionBarAdapter$MyTabListener;-onTabSelected-(Landroid/app/ActionBar$Tab; Landroid/app/FragmentTransaction;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/MtpDeviceSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/calendar/CalendarProvider2;-insertInTransaction-(Landroid/net/Uri; Landroid/content/ContentValues; Z)Landroid/net/Uri;' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/nfc/NfcActivityManager;-setNdefPushContentUri-(Landroid/app/Activity; [Landroid/net/Uri;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryFacilityLockForApp-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$AmsTask;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewClassic;-notifyFindDialogDismissed-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-serviceRequest-(J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-updateResources-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-access$300-(Lcom/android/internal/widget/multiwaveview/MultiWaveView; I F F)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/display/DisplayManagerService;-access$1400-(Lcom/android/server/display/DisplayManagerService; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/NfcAdapter;-enableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-updateSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/widget/LayoutSuppressingQuickContactBadge;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-setState-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/os/Vibrator;-vibrate-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/systemui/recent/RecentsActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Lcom/android/server/sip/SipService$SipSessionGroupExt;-onConnectivityChanged-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$24;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-syncReloadDictionaryInternal-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/gadget/WidgetSource;-reload-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/data/Conversation;-ensureThreadId-()J' : ['android.permission.READ_SMS'],
	'Lcom/google/android/mms/pdu/PduPersister;-loadRecipients-(I Ljava/util/HashSet; Ljava/util/HashMap; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$2;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/base/NullFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView$1;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-verifyUnlock-(Landroid/view/WindowManagerPolicy$OnKeyguardExitResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchToHitPicture-(I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-crashApplication-(Lcom/android/server/am/ProcessRecord; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/WifiService;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity$1;-onClick-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/settings/DateTimeSettingsSetupWizard;-onClick-(Landroid/view/View;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/detail/PhotoSelectionHandler$PhotoActionListener;-onRemovePictureChosen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/IConnectivityManager;-stopUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/GLView;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-hangupForegroundResumeBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-iccIOForApp-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-<init>-(Landroid/content/Context; Landroid/content/SharedPreferences;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/DataUsageSummary;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$HeaderAdapter;-resume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/settings/SettingsBackupAgent;-onRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J)V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/sip/SipManager;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/contacts/model/ContactLoader;-postViewNotificationToSyncAdapter-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-finalizeBackup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LightsService$1;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/contacts/model/RawContactDeltaList;-getFirstWritableRawContact-(Landroid/content/Context;)Lcom/android/contacts/model/RawContactDelta;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity$GroupDelta;-fromSettings-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Z)Lcom/android/contacts/list/CustomContactListFilterActivity$GroupDelta;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/accounts/AuthenticatorHelper;-onAccountsUpdated-(Landroid/content/Context; [Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setTimeFromNITZString-(Ljava/lang/String; J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/contacts/list/ContactListFilterController;-checkFilterValidity-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/CallFeaturesSetting;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/ShutdownThread;-beginShutdownSequence-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/accounts/AccountManager;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneProxy;-deleteAndCreatePhone-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE', 'android.permission.WRITE_SETTINGS', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/webkit/WebViewClassic;-onFocusChanged-(Z I Landroid/graphics/Rect;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/HTML5VideoViewProxy;-pauseAndDispatch-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserSettings;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onPressKey-(I Z I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterfw/core/FilterGraph;-tearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimUnlockScreen;-<init>-(Landroid/content/Context; Landroid/content/res/Configuration; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback; Lcom/android/internal/widget/LockPatternUtils;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/model/dataitem/RelationDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$AdapterStateChangedHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/ViewRootImpl;-dispatchUnhandledKey-(Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-connectSocket-(Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/group/GroupBrowseListAdapter;-setCursor-(Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/HeadsetProfile;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/group/GroupDetailFragment$2;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/text/StringLogger;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NsdService;-create-(Landroid/content/Context;)Lcom/android/server/NsdService;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onVoiceCallEnded-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager$3;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/OtaStartupReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/IActivityManager;-handleApplicationCrash-(Landroid/os/IBinder; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/location/LocationManager;-getProviders-(Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView$1;-onEndBatchInput-(Lcom/android/inputmethod/latin/InputPointers;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/InCallScreen;-access$2300-(Lcom/android/phone/InCallScreen;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/IccRecords;-onAllRecordsLoaded-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/tech/MifareClassic;-transfer-(I)V' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-cancel-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getCallWaiting-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/DownloadService;-access$500-(Lcom/android/providers/downloads/DownloadService; Lcom/android/providers/downloads/DownloadInfo$Reader; Lcom/android/providers/downloads/DownloadInfo; J)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-initSstIcc-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-dispatchNormalMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/webkit/HTML5VideoView;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-updateAudioSourceFunction-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-isNetworkSupported-(I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/media/IRingtonePlayer$Stub;-play-(Landroid/os/IBinder; Landroid/net/Uri; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/settings/Settings$DeviceInfoSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-dispatchPortAddressedPdus-([[B I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-createDictAndProximity-(Ljava/util/Locale;)Lcom/android/inputmethod/latin/spellcheck/DictAndProximity;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/IConnectivityManager$Stub;-getTetherableBluetoothRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView$3;-onFinish-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/FrameBranch;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/InputMethodManagerService;-showInputMethodMenu-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/DownloadManager;-getUriForDownloadedFile-(J)Landroid/net/Uri;' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/server/am/BroadcastQueue;-setBroadcastTimeoutLocked-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-dispatchUnhandledKey-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.CALL_PHONE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/BatteryStatsService;-setBatteryState-(I I I I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserHistoryPage;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/BackupManagerService;-bindToAgentSynchronous-(Landroid/content/pm/ApplicationInfo; I)Landroid/app/IBackupAgent;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewCallback;-pokeWakelock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/mms/transaction/SendTransaction;-getPdu-(Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/InputMethodManagerService;-setInputMethodLocked-(Ljava/lang/String; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/settings/Settings$ManageApplicationsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/accounts/IAccountManager;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/app/IWallpaperManager$Stub$Proxy;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Landroid/test/mock/MockContentResolver;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/gallery3d/ui/PhotoView$MyGestureListener;-onSingleTapUp-(F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-verifyUnlock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-startPairing-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/editor/LabeledEditorView;-onLabelRebuilt-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/SearchActivity;-onQueryTextSubmit-(Ljava/lang/String;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/WebViewClassic;-sendViewSizeZoom-(Z)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/event/EditEventFragment;-access$000-(Lcom/android/calendar/event/EditEventFragment; I)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/adapter/AbstractSyncAdapter;-<init>-(Lcom/android/exchange/EasSyncService;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/content/IIntentReceiver$Stub;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/IConnectivityManager;-updateLockdownVpn-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/AlarmManagerService;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/gallery3d/app/Gallery;-initializeByIntent-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/webkit/WebViewClassic;-dumpDomTree-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$VerifyingLinkState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/MediaPlaybackService;-reloadQueue-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/LockdownVpnTracker;-reset-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/FindActionModeCallback;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/NfcService;-onSeMifareAccess-([B)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/contacts/SpecialCharSequenceMgr;-handleIMEIDisplay-(Landroid/content/Context; Ljava/lang/String; Z)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/webkit/WebViewCore;-sendMessageAtFrontOfQueue-(I I I Ljava/lang/Object;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-access$2100-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-goingToSleep-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/DataManager;-rotate-(Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth;-getBondState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/LocationManagerService;-removeUpdatesLocked-(Lcom/android/server/LocationManagerService$Receiver;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/SearchActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController;-<init>-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewBase;-verifyUnlock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-snapToPage-(I I I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/media/RingtonePlayer;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAbsKeyInputView$3;-onFinish-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/BrowserWebView;-findAllAsync-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ims/IsimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NativeDaemonConnector;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/nfc/NfcService;-onRemoteFieldActivated-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onUpEvent-(I I J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/NavTabScroller;-onLayout-(Z I I I I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/systemui/recent/RecentTasksLoader;-getFirstTask-()Lcom/android/systemui/recent/TaskDescription;' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-recreateLockScreen-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-setFilter-(Lcom/android/contacts/list/ContactListFilter;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/INfcTag;-close-(I)I' : ['android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockScreen$SlidingTabMethods;-onTrigger-(Landroid/view/View; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IActivityManager;-bindBackupAgent-(Landroid/content/pm/ApplicationInfo; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/vcard/NfcImportVCardActivity;-createImportRequest-()Lcom/android/contacts/vcard/ImportRequest;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/BluetoothPbap;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/JoinContactListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/SearchPanelView$GlowPadTriggerListener;-onGrabbedStateChange-(Landroid/view/View; I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/ui/MessageListAdapter;-onContentChanged-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-generateBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/base/OutputStreamTarget;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-syncReloadDictionaryIfRequired-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/view/WindowManagerPolicy$WindowManagerFuncs;-reevaluateStatusBarVisibility-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/data/WorkingMessage;-deleteDraftSmsMessage-(J)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/os/IPowerManager$Stub$Proxy;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-dispatchAttributesChanged-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/wm/WindowManagerService;-moveAppWindowsLocked-(Lcom/android/server/wm/AppWindowToken; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/webkit/ZoomManager;-zoom-(F)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/pan/PanService;-handlePanDeviceStateChange-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String; I I I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/ui/ConversationList;-createNewMessage-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/net/wifi/WifiStateMachine$DriverStoppedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-getSuggestionsWithSessionId-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; I)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/model/RawContact;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-setRadioPower-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/vcard/NfcImportVCardActivity;-onImportFinished-(Lcom/android/contacts/vcard/ImportRequest; I Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-moveTo-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/Settings$InputMethodAndSubtypeEnablerActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/PhoneUi;-hideNavScreen-(I Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/wifi/WifiSettings;-constructAccessPoints-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/BaseUi;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-asyncReloadDictionaryIfRequired-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/bluetooth/IBluetoothHealth;-getConnectedHealthDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/statusbar/tablet/NotificationPanel;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/detail/ContactDetailFragment;-bindData-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/model/ImageModel;-loadThumbnailBitmap-(Lcom/android/mms/util/ItemLoadedCallback;)Lcom/android/mms/util/ItemLoadedFuture;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/ThrottleService;-getCliffThreshold-(Ljava/lang/String; I)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/InCallScreenShowActivation;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/calendar/CalendarController;-sendEvent-(Ljava/lang/Object; J Landroid/text/format/Time; Landroid/text/format/Time; J I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerSession;-onGetSuggestions-(Landroid/view/textservice/TextInfo; Ljava/lang/String; I)Landroid/view/textservice/SuggestionsInfo;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/widget/VideoView;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/wifi/WifiApEnabler;-setSoftapEnabled-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView$1;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/model/dataitem/ImDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/power/PowerManagerService;-access$1600-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiApConfigStore$InactiveState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$WirelessSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-onVisibilityChanged-(Z)V' : ['android.permission.CAMERA'],
	'Lcom/android/settings/SettingsLicenseActivity;-access$000-(Lcom/android/settings/SettingsLicenseActivity; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PasswordUnlockScreen;-<init>-(Landroid/content/Context; Landroid/content/res/Configuration; Lcom/android/internal/widget/LockPatternUtils; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/nfc/handover/HandoverService;-doIncomingTransfer-(Landroid/os/Message;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/alerts/SnoozeAlarmsService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$TetheredState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetheringErroredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/SearchActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/settings/Settings$SpellCheckersSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/os/IPowerManager$Stub;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-removeJavascriptInterface-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$MyGestureListener;-onSingleTapUp-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityCallback;-reportSuccessfulUnlockAttempt-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/test/mock/MockContentResolver;-getCurrentSync-()Landroid/content/SyncInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-invokeCustomRequest-(I)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SyncStateTracker;-requestStateChange-(Landroid/content/Context; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/settings/wfd/WifiDisplaySettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/NetworkTimeUpdateService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/group/GroupEditorFragment;-selectAccountAndCreateGroup-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/nfc/INfcAdapterExtras;-transceive-(Ljava/lang/String; [B)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/calendar/event/EditEventHelper$EditDoneRunnable;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-dispatchMediaKeyRepeatWithWakeLock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/fuelgauge/PowerUsageSummary;-processMiscUsage-()V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangup-(Lcom/android/internal/telephony/gsm/GsmCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackActivity;-cycleRepeat-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-performTransitions-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/text/StringSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-snapToPageImmediately-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcAdapterExtras$Stub$Proxy;-transceive-(Ljava/lang/String; [B)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/bluetooth/btservice/AdapterService;-createBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/OtaUtils;-onClickOtaActivateButton-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSimPukView;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setPropertiesByCarrier-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/media/IAudioService$Stub;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/INetworkPolicyManager$Stub;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IccCard;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DataConnection$DcActivatingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$DeviceAdminSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/data/WorkingMessage$4;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-alarmExists-(Landroid/content/ContentResolver; J J J)Z' : ['android.permission.READ_CALENDAR'],
	'Landroid/webkit/WebViewClassic;-passToJavaScript-(Ljava/lang/String; Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame$7;-onDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/DataUsageSummary$12;-onLimitChanged-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-editProperties-(Ljava/lang/String; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/ErrorConsoleView$1;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/ViewRootImpl;-deliverGenericMotionEvent-(Landroid/view/ViewRootImpl$QueuedInputEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-queryCallWaiting-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/location/LocationManager;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/AccountUnlockScreen;-asyncCheckPassword-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/DataConnectionTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView;-onFinishInflate-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitorCallback;-onRefreshCarrierInfo-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManagerProxy;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/settings/Settings$SpellCheckersSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onRegisterKey-(Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/model/dataitem/GroupMembershipDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothA2dp;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/IAccountManager$Stub;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/mms/data/WorkingMessage;-preSendSmsWorker-(Lcom/android/mms/data/Conversation; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-access$500-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/accounts/AccountPreferenceBase;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS'],
	'Lcom/android/gallery3d/ui/TileImageView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/media/IRingtonePlayer$Stub$Proxy;-stopAsync-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/WorkingMessage;-saveDraft-(Z)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/mms/data/WorkingMessage;-saveAsMms-(Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/ExchangeService$2;-hostChanged-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/usb/UsbAccessoryUriActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/IThrottleManager$Stub;-getPeriodStartTime-(Ljava/lang/String;)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/launcher2/AppsCustomizePagedView;-beginDraggingApplication-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/ZoomManager;-updateDoubleTapZoom-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/TimerRingService;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/group/GroupDetailFragment;-bindGroupMetaData-(Landroid/database/Cursor;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView;-onWindowFocusChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-setForegroundDispatch-(Landroid/app/PendingIntent; [Landroid/content/IntentFilter; Landroid/nfc/TechListParcel;)V' : ['android.permission.NFC'],
	'Lcom/android/contacts/quickcontact/DataAction;-<init>-(Landroid/content/Context; Lcom/android/contacts/model/dataitem/DataItem;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothPbap$Stub$Proxy;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame;-onMeasure-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/power/PowerUI;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.VIBRATE'],
	'Landroid/content/ContextWrapper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/NfcService$TagService;-reconnect-(I)I' : ['android.permission.NFC'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/ChallengeLayout;-hideBouncer-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/dialog/ClearFrequentsDialog$1$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-query-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/data/Conversation;-dumpThreadsTable-(Landroid/content/Context;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/mms/ui/MessageListItem;-updateAvatarView-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$LocationSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/launcher2/AppsCustomizePagedView;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/Phone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/NfcHandler;-register-(Landroid/app/Activity; Lcom/android/contacts/detail/ContactDetailFragment;)V' : ['android.permission.NFC'],
	'Landroid/net/IThrottleManager;-getThrottle-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter;-setSyncKey-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/MediaPlaybackService;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-copyMessageToIccEf-(I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/sip/SipManager;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-updateFolderList-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccCardProxy;-supplyPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebIconDatabaseClassic;-releaseIconForPageUrl-(Ljava/lang/String;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/activities/DialtactsActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/google/common/io/ByteStreams;-length-(Lcom/google/common/io/InputSupplier;)J' : ['android.permission.INTERNET'],
	'Lcom/android/launcher2/Workspace;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/MccTable;-setSystemLocale-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothDiscoverableEnabler$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/Connection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity$BackgroundQueryHandler;-onDeleteComplete-(I Ljava/lang/Object; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-removeBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/am/ActivityManagerService;-main-(I)Landroid/content/Context;' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/content/Context;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-scrollLeft-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiManager;-getWifiApState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-updateLockdownVpn-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-onVisibilityChanged-(Z)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothHealth;-registerAppConfiguration-(Ljava/lang/String; I I I Landroid/bluetooth/BluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ActivityBase;-createCameraScreenNail-(Z)Lcom/android/gallery3d/ui/ScreenNail;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/homepages/RequestHandler;-doHandleRequest-()V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-init-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/app/ActivityManagerNative;-updateConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-goToSleepInternal-(J I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-handleVolumeKey-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-copyMedia-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/ui/TileImageView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/DataConnectionTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-hangupWaitingOrBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfileManager$StateChangedHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/IConnectivityManager;-getLinkProperties-(I)Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimPukUnlockScreen;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/net/ThrottleManager;-getByteCount-(Ljava/lang/String; I I I)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/activities/PeopleActivity;-areGroupWritableAccountsAvailable-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/AudioService;-checkScoAudioState-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ITelephony$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_FINE_LOCATION', 'android.permission.CALL_PHONE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/media/RingtonePlayer$Client;-<init>-(Lcom/android/systemui/media/RingtonePlayer; Landroid/os/IBinder; Landroid/net/Uri; Landroid/os/UserHandle; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardParserImpl_V40;-parseItems-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/telephony/PhoneNumberUtils;-isVoiceMailNumber-(Ljava/lang/String;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/CallDetailActivity$1UpdateContactDetailsTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/group/GroupBrowseListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateStart;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-access$1900-(Lcom/android/bluetooth/pbap/BluetoothPbapService; I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity$CustomFilterConfigurationLoader;-loadInBackground-()Lcom/android/contacts/list/CustomContactListFilterActivity$AccountSet;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setPropertiesByCarrier-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-notifyScreenOffLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/CallCard;-updateForegroundCall-(Lcom/android/internal/telephony/CallManager;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/NfcActivityManager;-setOnNdefPushCompleteCallback-(Landroid/app/Activity; Landroid/nfc/NfcAdapter$OnNdefPushCompleteCallback;)V' : ['android.permission.NFC'],
	'Landroid/media/AsyncPlayer;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/phone/PhoneUtils;-placeCall-(Landroid/content/Context; Lcom/android/internal/telephony/Phone; Ljava/lang/String; Landroid/net/Uri; Z Landroid/net/Uri;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView$Listener;-pickSuggestionManually-(I Ljava/lang/CharSequence;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/Settings$DreamSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/users/UserSettings;-access$800-(Lcom/android/settings/users/UserSettings;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/list/JoinContactListFragment;-onItemClick-(I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/PhoneBase;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/CalendarSyncAdapter$EasCalendarSyncParser;-addEvent-(Lcom/android/exchange/adapter/CalendarSyncAdapter$CalendarOperations; Ljava/lang/String; Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/systemui/statusbar/phone/QuickSettingsModel$BugreportObserver;-startObserving-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/am/BroadcastQueue$AppNotResponding;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-keyguardDone-(Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-handleUp-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/transaction/ReadRecTransaction;-sendPdu-(J [B)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/InCallScreen;-onResume-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/InputMethodManagerService;-access$000-(Lcom/android/server/InputMethodManagerService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/emailcommon/provider/Account;-isAutomaticSyncDisabledByRoaming-(Landroid/content/Context; J)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-access$400-(Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager; [Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/settings/SettingsBackupAgent$WifiRestoreRunnable;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setPasskey-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/util/AddressUtils;-getFrom-(Landroid/content/Context; Landroid/net/Uri;)Ljava/lang/String;' : ['android.permission.READ_SMS'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Landroid/media/AudioService$SetModeDeathHandler;-binderDied-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/RingerVolumePreference;-cleanup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/ContactListFilterControllerImpl;-notifyContactListFilterChanged-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/LocationManagerService$3;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer$2;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/BluetoothDevice;-removeBond-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/nfc/NfcService$NfcAdapterExtrasService;-_transceive-([B)[B' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-handleSettingsChangedLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/provider/Browser;-deleteHistoryTimeFrame-(Landroid/content/ContentResolver; J J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewStateManager;-onScrollStateChanged-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/NfcAdapter$1;-onPaused-(Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/phone/SipCallOptionHandler;-access$100-(Lcom/android/phone/SipCallOptionHandler;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/systemui/recent/RecentsActivity;-onBackPressed-()V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Landroid/speech/IRecognitionService;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/IAudioService$Stub$Proxy;-setWiredDeviceConnectionState-(I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$AutoRegistrationProcess;-onRegistrationFailed-(Landroid/net/sip/ISipSession; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService$FeatureUser;-expire-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/SecureAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/Settings$TetherSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/inputmethod/UserDictionaryList;-getUserDictionaryLocalesSet-(Landroid/app/Activity;)Ljava/util/TreeSet;' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/internal/telephony/RIL;-acceptCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-sendIntentEnter-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-setQueuePosition-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/IconLoader;-readId-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$TagService;-formatNdef-(I [B)I' : ['android.permission.NFC'],
	'Landroid/filterfw/core/GraphRunner;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PasswordUnlockScreen$1;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/StatusBarManager;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPukForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/accounts/SyncSettings;-onResume-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$NotConnectedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/vcard/VCardEntry$AnniversaryData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/MmsThumbnailPresenter$1;-onItemLoaded-(Lcom/android/mms/util/ThumbnailManager$ImageLoaded; Ljava/lang/Throwable;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/gadget/MediaSetSource;-reload-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/Files;-move-(Ljava/io/File; Ljava/io/File;)V' : ['android.permission.INTERNET'],
	'Landroid/os/IVibratorService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/mms/ui/ClassZeroActivity;-saveMessage-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/MasterClear;-establishInitialState-()V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-getDeviceIdentity-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/datepicker/DatePickerDialog;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/view/inputmethod/BaseInputConnection;-commitText-(Ljava/lang/CharSequence; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-onSurfaceCreated-(Landroid/view/SurfaceHolder;)V' : ['android.permission.CAMERA'],
	'Lcom/android/server/power/Notifier;-updatePendingBroadcastLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCdmaSubscriptionSource-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/calllog/DefaultVoicemailNotifier$NewCallsQuery;-query-()[Lcom/android/contacts/calllog/DefaultVoicemailNotifier$NewCall;' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/webkit/WebViewProvider;-clearSslPreferences-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/LocationManagerService;-updateProvidersLocked-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/event/AttendeesView;-updateAttendeeView-(Lcom/android/calendar/event/EditEventHelper$AttendeeItem;)Landroid/view/View;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/pm/PackageManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CLEAR_APP_CACHE', 'android.permission.READ_EXTERNAL_STORAGE', 'android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/browser/DataController$DataControllerHandler;-doSaveThumbnail-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/RIL;-getIccCardStatus-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager;-setForcedDisplaySize-(I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/editor/EventFieldEditorView;-onTypeSelectionChange-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/Controller;-openIncognitoTab-()Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/AlarmAlertWakeLock;-releaseCpuLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$StorageSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ConversationList$7;-onKey-(Landroid/view/View; I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-getAllMessagesFromIccEf-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/gadget/PhotoAppWidgetProvider;-onUpdate-(Landroid/content/Context; Landroid/appwidget/AppWidgetManager; [I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/wifi/WifiSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/location/ILocationManager;-isProviderEnabled-(Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/list/ContactPickerFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/sip/SipService$AutoRegistrationProcess;-onRegistrationTimeout-(Landroid/net/sip/ISipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/TransactionSafeActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager$Stub;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-execute-(Lcom/android/gallery3d/data/DataManager; Lcom/android/gallery3d/util/ThreadPool$JobContext; I Lcom/android/gallery3d/data/Path;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$3;-forgotPattern-(Z)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterfw/core/Filter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/GlobalActions$SinglePressAction;-onLongPress-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mms/transaction/RetrieveTransaction;-sendAcknowledgeInd-(Lcom/google/android/mms/pdu/RetrieveConf;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/providers/downloads/SystemFacade;-isNetworkRoaming-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewProvider;-clearMatches-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar$8;-onClick-(Landroid/view/View;)V' : ['android.permission.GET_TASKS'],
	'Landroid/net/wifi/IWifiManager;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager;-wakeWhenReadyTq-(I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/RadioInfo$22;-onItemSelected-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/PushReceiver;-isDuplicateNotification-(Landroid/content/Context; Lcom/google/android/mms/pdu/NotificationInd;)Z' : ['android.permission.READ_SMS'],
	'Lcom/android/contacts/model/dataitem/ImDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/power/DisplayPowerController$3;-run-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/downloads/DownloadInfo;-getMyDownloadsUri-()Landroid/net/Uri;' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/server/BluetoothManagerService;-access$2700-(Lcom/android/server/BluetoothManagerService; I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/AttachmentEditor;-update-(Lcom/android/mms/data/WorkingMessage;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/agenda/AgendaListView;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/settings/DataUsageSummary;-access$800-(Lcom/android/settings/DataUsageSummary; J)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService$ScoClient;-incCount-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ViewRootImpl;-deliverTrackballEventPostIme-(Landroid/view/ViewRootImpl$QueuedInputEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-bindBackupAgent-(Landroid/content/pm/ApplicationInfo; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-insert-(Landroid/content/ContentResolver; J J J J I)Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/telephony/gsm/SmsManager;-copyMessageToSim-([B [B I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/widget/LockScreenWidgetCallback;-userActivity-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/view/IWindowManager;-startFreezingScreen-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText$1;-onBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/CommandQueue$Callbacks;-setSystemUiVisibility-(I I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SlotView$MyGestureListener;-onLongPress-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/vcard/ImportVCardActivity$VCardCacheThread;-constructImportRequest-([B Landroid/net/Uri; Ljava/lang/String;)Lcom/android/contacts/vcard/ImportRequest;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/CaptivePortalTracker$DefaultState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/EmailSyncAlarmReceiver;-handleReceive-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewFragment;-onDestroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/PreloadRequestReceiver;-handlePreload-(Landroid/content/Context; Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-endReordering-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/ui/RenderOverlay$Renderer;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/net/NetworkStatsService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/providers/downloads/DownloadInfo;-checkCanUseNetwork-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/BluetoothPhoneService;-sendClccResponseCdma-(I Lcom/android/internal/telephony/Connection;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-addAccount-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/settings/RadioInfo;-onPause-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/ContextImpl;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ITelephony$Stub;-enableDataConnectivity-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/contacts/list/PostalAddressListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/accounts/AccountManager$3;-doWork-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/BrowserActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/Controller$PruneThumbnails;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/SnapshotTab;-putInForeground-()V' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/phone/CdmaSystemSelectListPreference;-onDialogClosed-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-checkPermissionsAndCreateUriData-(Landroid/net/Uri;)Lcom/android/providers/contacts/VoicemailContentProvider$UriData;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendP2pConnectionChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/model/dataitem/IdentityDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.GET_TASKS'],
	'Lcom/android/camera/ShutterButton$OnShutterButtonListener;-onShutterButtonClick-()V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/sip/ISipService;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/BackupManagerService$RunInitializeReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-loadDictionaryAsync-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/NetworkStateTracker;-teardown-()Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/commands/pm/Pm;-main-([Ljava/lang/String;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/mms/transaction/Transaction;-getPdu-(Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover;-start-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/AudioService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/WaveView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.VIBRATE', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState$SwitchActions;-hapticAndAudioFeedback-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/RIL;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-slideToNextPicture-()Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$MyDownUpListener;-onUp-(Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/LocationBasedCountryDetector$1;-onLocationChanged-(Landroid/location/Location;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/AlarmPreference$1;-doInBackground-([Landroid/net/Uri;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/CallerInfoCache;-startAsyncCache-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/support/v4/net/ConnectivityManagerCompat;-getNetworkInfoFromBroadcast-(Landroid/net/ConnectivityManager; Landroid/content/Intent;)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/base/FrameBranch;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-requestRouteToHost-(I I)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$Listener;-onDown-(F F)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/InputMethodManagerService$MethodCallback;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/tech/MifareUltralight;-connect-()V' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/vcard/NfcImportVCardActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/ActivityManagerNative;-wakingUp-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiStateMachine;-setWifiApState-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/RuimSmsInterfaceManager;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/contacts/group/GroupEditorFragment$Member;-<init>-(J Ljava/lang/String; J Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView$2;-onBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/AllInOneActivity;-onSharedPreferenceChanged-(Landroid/content/SharedPreferences; Ljava/lang/String;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/phone/InCallTouchUi;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Pending;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/voicedialer/VoiceDialerActivity$3;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Landroid/nfc/NfcActivityManager;-requestNfcServiceCallback-(Z)V' : ['android.permission.NFC'],
	'Lcom/android/server/am/ActivityManagerService;-forceStopPackageLocked-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/Tab$2$4;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/test/mock/MockContentResolver;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/DialogPicker;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/ITelephony;-endCall-()Z' : ['android.permission.CALL_PHONE'],
	'Landroid/server/BluetoothA2dpService;-checkSinkSuspendState-(I)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/systemui/statusbar/CommandQueue$Callbacks;-hideSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/Controller;-setActiveTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifyDataConnectionFailed-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/systemui/media/NotificationPlayer;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/ObscureSpeechDelegate;-shouldObscureSpeech-()Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/RuimRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-dispatchUnhandledMove-(Landroid/view/View; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/IContentService$Stub$Proxy;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/webkit/WebViewCore;-jsConfirm-(Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionActivity;-createConnectionDialogView-()Landroid/view/View;' : ['android.permission.BLUETOOTH'],
	'Landroid/content/ContextWrapper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/webkit/WebViewClassic;-externalRepresentation-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WifiService;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-screenTurningOn-(Landroid/view/WindowManagerPolicy$ScreenOnListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/SmsSingleRecipientSender;-<init>-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; J Z Landroid/net/Uri;)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/phone/PhoneUtils;-answerCall-(Lcom/android/internal/telephony/Call;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/inputmethod/research/ResearchLogger;-restart-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/camera/ui/InLineSettingSwitch$1;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/TabBar$TabView;-closeTab-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/Preloader$PreloaderSession;-<init>-(Lcom/android/browser/Preloader; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/content/pm/IPackageManager$Stub;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/browser/search/OpenSearchSearchEngine;-getSuggestions-(Landroid/content/Context; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setWifiEnabled-(Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$UserAuthorizingInvitationState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS'],
	'Lcom/android/server/location/LocationProviderProxy;-disable-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/AlarmKlaxon;-play-(Lcom/android/deskclock/Alarm;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/server/BackupManagerService;-prepareOperationTimeout-(I J Lcom/android/server/BackupManagerService$BackupRestoreTask;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/model/RawContactDeltaList;-buildSplitContactDiff-(Ljava/util/ArrayList; [I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/view/inputmethod/BaseInputConnection;-setComposingRegion-(I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$VpnSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/display/DisplayManagerService;-setDisplayInfoOverrideFromWindowManager-(I Landroid/view/DisplayInfo;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/wifi/p2p/WifiP2pSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/event/EditEventView$DateListener;-onDateSet-(Landroid/widget/DatePicker; I I I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallEstablished-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/settings/SubSettings;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/view/WindowManagerPolicy;-screenTurningOn-(Landroid/view/WindowManagerPolicy$ScreenOnListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ims/IsimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getIMEISV-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/CropImage$SaveOutput;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/DataConnection;-tearDownData-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getVoiceRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/ContactListAdapter;-getFirstContactUri-()Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtErrorActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPINView;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/DetailsAddressResolver$AddressLookupJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Landroid/location/Address;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/transaction/PushReceiver$ReceivePushTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/editor/ContactEditorFragment$4;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/TileImageView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/BluetoothAdapter;-createNewRfcommSocketAndRecord-(Ljava/lang/String; Ljava/util/UUID; Z Z)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifySeFieldDeactivated-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/contacts/activities/GroupBrowserActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/ThrottleService;-getByteCount-(Ljava/lang/String; I I I)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/UsimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-addJavascriptInterface-(Ljava/lang/Object; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/commands/monkey/MonkeyNetworkMonitor;-register-(Landroid/app/IActivityManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/accounts/ManageAccountsSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/DataUsageSummary;-isMobileDataEnabled-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-hangupWaitingOrBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/sip/SipSettings;-access$700-(Lcom/android/phone/sip/SipSettings; Z)V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$SettingsObserver;-observe-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/inputmethod/InputMethodManager;-dispatchKeyEvent-(Landroid/content/Context; I Landroid/view/KeyEvent; Landroid/view/inputmethod/InputMethodManager$FinishedEventCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/CalendarContract$CalendarEntity$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/power/PowerManagerService;-handleDreamFinishedLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/EditableInputConnection;-performEditorAction-(I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiManager;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-showGlobalActionsDialog-()V' : ['android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/ILocationManager;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/usb/UsbDebuggingActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Landroid/app/KeyguardManager$KeyguardLock;-disableKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-toggleRecentsActivity-()V' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnection;-access$2000-(Lcom/android/internal/telephony/DataConnection; Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$3;-reportFailedUnlockAttempt-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/Phone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/accessibility/AccessibilityEntityProvider;-simulateKeyPress-(Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/dreams/phototable/StockSource;-<init>-(Landroid/content/Context; Landroid/content/SharedPreferences;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/EdgeView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/data/Contact$ContactsCache;-updateContact-(Lcom/android/mms/data/Contact;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/bluetooth/BluetoothAdapter;-setScanMode-(I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserWebView;-restoreState-(Landroid/os/Bundle;)Landroid/webkit/WebBackForwardList;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/CallLogProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/net/ThrottleManager;-getCliffLevel-(Ljava/lang/String; I)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/Preloader;-getSession-(Ljava/lang/String;)Lcom/android/browser/Preloader$PreloaderSession;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/ObscureSpeechDelegate;-onPopulateAccessibilityEvent-(Landroid/view/View; Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/os/IPowerManager$Stub;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-setDiscoverableTimeout-(I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewClassic;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Controller;-onPreloginFinished-(Landroid/os/Bundle; Landroid/content/Intent; J Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-createSocketChannel-(I Ljava/lang/String; Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-haveSomethingToCopyToSDCard-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-goToUserSwitcher-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-access$000-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/UiController;-switchToTab-(Lcom/android/browser/Tab;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-notifyApnIdDisconnected-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/Settings$AndroidBeamSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/IccCardProxy;-setIccLockEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternView;-notifyCellAdded-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/vcard/ExportProcessor;-runInternal-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-onCreate-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/PackageSummary$1;-onClick-(Landroid/view/View;)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onCardEmulationDeselected-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/telephony/SmsManager;-sendMultipartTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/calendar/EventInfoFragment;-initAttendeesCursor-(Landroid/view/View;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-tryVibrate-()V' : ['android.permission.VIBRATE'],
	'Landroid/app/backup/IBackupManager$Stub$Proxy;-clearBackupData-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/ClearMissedCallsService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/browser/BrowserWebView;-stopLoading-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-showSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/WebViewInputDispatcher;-scheduleLongPressLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothInputDevice;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/InputMethodManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/DayView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/ISms;-disableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/server/net/LockdownVpnTracker$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-notifyANR-(Lcom/android/server/input/InputApplicationHandle; Lcom/android/server/input/InputWindowHandle;)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/webkit/WebViewCore;-sendMessages-(Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-deleteSelection-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onSeListenDeactivated-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub$Proxy;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/browser/UiController;-closeCurrentTab-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/browser/DataController$DataControllerHandler;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockScreen;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/pm/PackageManagerService;-scanAvailableAsecs-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/inputmethodservice/ExtractButton;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/RequestPermissionActivity;-createDialog-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter$SlideshowSource;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaFragment$2;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mms/data/WorkingMessage;-access$1300-(Lcom/google/android/mms/pdu/PduPersister; Lcom/google/android/mms/pdu/SendReq; Lcom/android/mms/model/SlideshowModel; Landroid/net/Uri; Landroid/content/Context; Ljava/util/HashMap;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/detail/PhotoSelectionHandler$PhotoActionListener;-onPhotoSelected-(Landroid/graphics/Bitmap;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/wm/WindowManagerService;-makeWindowFreezingScreenIfNeededLocked-(Lcom/android/server/wm/WindowState;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-dismiss-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/FrameStore;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView;-access$400-(Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/wallpaper/grass/GrassRS;-start-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/sip/SipWakeLock;-acquire-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/inputmethod/UserDictionaryList;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-refreshName-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/CallerInfoCache$CacheAsyncTask;-onPostExecute-(Ljava/lang/Void;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ActivityBase;-reuseCameraScreenNail-(Z)Lcom/android/gallery3d/ui/ScreenNail;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/systemui/recent/RecentsHorizontalScrollView;-onChildDismissed-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/PhoneBase;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IThrottleManager$Stub;-getThrottle-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/webkit/WebViewCore;-sendMessageDelayed-(I Ljava/lang/Object; J)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$InputMethodAndSubtypeEnablerActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/RuimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewProvider;-restoreState-(Landroid/os/Bundle;)Landroid/webkit/WebBackForwardList;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/AutoFixEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/VolumePanel;-onVibrate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/browser/NetworkStateHandler;-access$000-(Lcom/android/browser/NetworkStateHandler; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/NotificationMgr;-updateNotificationAndLaunchIncomingCallUi-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/DataUsageSummary;-hasWifiRadio-(Landroid/content/Context;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/CallNotifier;-onCdmaCallWaiting-(Landroid/os/AsyncResult;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-handleNotifyScreenOn-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager$ShowListener;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/FaceUnlock;-handlePokeWakelock-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setPin-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-updateResizingWindows-(Lcom/android/server/wm/WindowState;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitorCallback;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/UsimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardParser_V40;-parse-(Ljava/io/InputStream; Lcom/android/vcard/VCardInterpreter;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/UsimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-loadUrlDataIn-(Lcom/android/browser/Tab; Lcom/android/browser/IntentHandler$UrlData;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-appDiedLocked-(Lcom/android/server/am/ProcessRecord; I Landroid/app/IApplicationThread;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/sip/SipService;-onConnectivityChanged-(Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/wm/DragState;-unregister-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/CallerInfoCache$CacheAsyncTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardStatusView;-onDetachedFromWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView$1;-onPressKey-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/editor/LabeledEditorView;-onTypeSelectionChange-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/IContentService;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/ConnectivityManager;-requestRouteToHostAddress-(I Ljava/net/InetAddress;)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/contacts/ProfileProvider;-deleteInTransaction-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_PROFILE'],
	'Lcom/android/browser/BrowserActivity;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/accounts/AccountPreferenceBase;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiManager;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-disconnect-(Lcom/android/settings/bluetooth/LocalBluetoothProfile;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/ApplicationThreadNative;-scheduleRegisteredReceiver-(Landroid/content/IIntentReceiver; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/power/Notifier$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth;-cancelBondProcess-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/Controller;-access$100-(Lcom/android/browser/Controller; Landroid/os/Bundle; Landroid/content/Intent; J Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/bluetooth/DockEventReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-updateNotification-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$HideNavInputEventReceiver$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManager$7;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/mms/ui/SlideView;-startVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/LoadingListener;-onLoadingFinished-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-enforceReceiveAndSend-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/vcard/VCardComposer;-init-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/providers/contacts/GlobalSearchSupport$SearchSuggestion;-asList-([Ljava/lang/String;)Ljava/util/ArrayList;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetooth;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/util/Recycler$SmsRecycler;-anyThreadOverLimit-(Landroid/content/Context;)Z' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Landroid/webkit/WebViewClassic;-zoomIn-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/RetryScheduler;-update-(Lcom/android/mms/transaction/Observable;)V' : ['android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/server/BluetoothManagerService;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Controller;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$1700-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/ui/ZoomRenderer;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothInputDevice;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/DhcpStateMachine$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/location/ILocationManager$Stub$Proxy;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/PhoneProxy;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/SearchActivity;-initFragments-(J Ljava/lang/String;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/HTML5VideoView;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/UiModeManagerService$4;-onTwilightStateChanged-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CsimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/android/mms/pdu/PduPersister;-load-(Landroid/net/Uri;)Lcom/google/android/mms/pdu/GenericPdu;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/nfc/INfcAdapterExtras$Stub$Proxy;-authenticate-(Ljava/lang/String; [B)V' : ['android.permission.NFC'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-getSuggestions-(Ljava/lang/String; I Z)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-setSearchMode-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/InputMethodManagerService;-switchUserLocked-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/WindowManagerPolicy$WindowManagerFuncs;-shutdown-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/ui/GLView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/WifiService;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/calendar/alerts/AlertReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-handleBroadcastSms-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/MultiInputStream;-read-()I' : ['android.permission.INTERNET'],
	'Landroid/net/IThrottleManager;-getPeriodStartTime-(Ljava/lang/String;)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/BrowserWebViewFactory;-initWebViewSettings-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/GLRootView;-onDrawFrameLocked-(Ljavax/microedition/khronos/opengles/GL10;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/SlideEditorActivity;-showCurrentSlide-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncManager$SyncHandler;-onBootCompleted-()V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiStateMachine;-setWifiState-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-showPrimarySecurityScreen-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/event/EditEventHelper;-saveEvent-(Lcom/android/calendar/CalendarEventModel; Lcom/android/calendar/CalendarEventModel; I)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-init-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE', 'android.permission.WRITE_SETTINGS', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/settings/inputmethod/UserDictionaryAddWordContents;-delete-(Landroid/content/Context;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/webkit/HTML5Audio;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/CameraModule;-init-(Lcom/android/camera/CameraActivity; Landroid/view/View; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockScreen$1;-onUserSwitched-(I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-notifyDefaultData-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/SnapshotTab;-dismissSubWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-access$500-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/SipCallOptionHandler;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/providers/contacts/DbModifierWithNotification;-notifyVoicemailChange-(Landroid/net/Uri; Ljava/util/Set; [Ljava/lang/String;)V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/display/DisplayAdapter$1;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumSetDataLoader;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/PhoneBase;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/util/DownloadManager;-markState-(Landroid/net/Uri; I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/data/LocalSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hdp/HealthService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/Controller;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-checkPackagePermission-(Lcom/android/providers/contacts/VoicemailContentProvider$UriData;)V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/widget/ZoomButtonsController;-access$800-(Landroid/widget/ZoomButtonsController; Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/ContentResolver;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-openFile-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$FullImageJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Landroid/graphics/BitmapRegionDecoder;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/server/sip/SipService$SipSessionGroupExt;-openToReceiveCalls-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcAdapter;-disableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-handleCancel-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/editor/SelectAccountDialogFragment$Listener;-onAccountChosen-(Lcom/android/contacts/model/account/AccountWithDataSet; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/widget/ChartDataUsageView$3;-onSweep-(Lcom/android/settings/widget/ChartSweepView; Z)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/group/GroupEditorFragment$3;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/CursorLoader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewClassic;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewInputDispatcher;-skipWebkitForRemainingTouchStream-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/HTML5VideoFullScreen;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/InCallScreen;-onMMIComplete-(Lcom/android/internal/telephony/MmiCode;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/wifi/WifiStatusTest$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter;-stopPlaybackAtPosition-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onDisconnectDone-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ConversationListItem;-bind-(Landroid/content/Context; Lcom/android/mms/data/Conversation;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/RIL;-changeIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/DeskClock;-createTabs-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/RelationDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/PhoneInterfaceManager;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/model/ContactLoader;-loadStreamItems-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/bluetooth/BluetoothInputDevice;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity$1;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-access$1900-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/browser/WebViewFactory;-createWebView-(Z)Landroid/webkit/WebView;' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/development/EnterURL$DisplayEditText;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-updateServiceLocation-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/location/LocationManager;-addTestProvider-(Ljava/lang/String; Z Z Z Z Z Z Z I I)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-getDataRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/GroupMemberLoader;-constructLoaderForGroupDetailQuery-(Landroid/content/Context; J)Lcom/android/contacts/GroupMemberLoader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/development/Connectivity;-access$2000-(Lcom/android/development/Connectivity;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/browser/UrlHandler;-startActivityForUrl-(Lcom/android/browser/Tab; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardMessageArea$2;-onClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/Controller;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/BluetoothPbap;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/BluetoothNameDialogFragment;-updateDeviceName-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/Gallery;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/InCallScreen$7;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/PeopleActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/data/WorkingMessage;-setHasMultipleRecipients-(Z Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/ui/ConversationListAdapter;-bindView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/ActivityManager;-getRunningTasks-(I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/camera/VideoModule;-onPauseBeforeSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getBondedDevices-()[Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$3;-onPostExecute-(Ljava/lang/Exception;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkPolicyManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/app/ChooserActivity;-onStop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/common/contacts/DataUsageStatUpdater;-updateWithAddress-(Ljava/util/Collection;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/ProfileAndContactsLoader;-loadInBackground-()Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/location/LocationManager;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/browser/SuggestionsAdapter$CompletionListener;-onSelect-(Ljava/lang/String; I Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/ui/SlideshowAttachmentView;-setVideo-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/media/NotificationPlayer;-play-(Landroid/content/Context; Landroid/net/Uri; Z I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NsdService$NsdStateMachine$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/ContactsActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/test/mock/MockContentResolver;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/media/IRingtonePlayer;-playAsync-(Landroid/net/Uri; Landroid/os/UserHandle; Z I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebIconDatabaseClassic;-open-(Ljava/lang/String;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/wm/WindowManagerService;-performLayoutAndPlaceSurfacesLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterfw/core/Filter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-freeMemory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter;-getAutomaticLookback-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/PhoneBase;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/OutgoingCallBroadcaster$OutgoingCallReceiver;-doReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/google/android/mms/util/PduCache;-put-(Landroid/net/Uri; Lcom/google/android/mms/util/PduCacheEntry;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/test/mock/MockContentResolver;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$ObtainingIpState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-switchHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-cancelPostDial-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/Gallery;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/Settings$PowerUsageSummaryActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewCore;-openFileChooser-(Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/month/MonthListView$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/music/MediaPlaybackService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothInputDevice;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/IConnectivityManager$Stub;-getTetheredIfacePairs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/ui/ClassZeroActivity;-replaceMessage-(Landroid/telephony/SmsMessage;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/media/effect/effects/CrossProcessEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-getVoiceRadioTechnology-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-loadMessagesAndDraft-(I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Landroid/widget/VideoView;-resume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl$1;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Browser;-getAllVisitedUrls-(Landroid/content/ContentResolver;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-updateOrganizations-(J Landroid/content/ContentValues;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/detail/ContactDetailFragment$AddToMyContactsQuickFix;-isApplicable-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$TextToSpeechSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.INTERNET'],
	'Landroid/content/ISyncStatusObserver$Stub$Proxy;-onStatusChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSimPinView;-onResume-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getLastCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-getNetworkQuotaInfoUnchecked-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/systemui/statusbar/policy/CurrentUserTracker;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-access$300-(Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor; Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor$SimArgs;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/pan/PanService;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/Call;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller$2;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/wifi/IWifiManager$Stub;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/NfcAdapter;-setNdefPushMessage-(Landroid/nfc/NdefMessage; Landroid/app/Activity; [Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Lcom/android/providers/downloads/DownloadProvider;-logVerboseOpenFileInfo-(Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.INTERNET'],
	'Lcom/android/settings/AccessibilitySettings$ToggleAccessibilityServicePreferenceFragment;-onResume-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/BroadcastQueue;-skipCurrentReceiverLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/content/SyncManager;-<init>-(Landroid/content/Context; Z)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/MediaPlayer;-access$300-(Landroid/media/MediaPlayer; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/SecureCameraActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/app/backup/IBackupManager$Stub$Proxy;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/tv/TvStatusBar;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-setCurrentPage-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ProviderStatusWatcher$LoaderTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Boolean;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/ui/MessageUtils;-getMultimediaMessageDetails-(Landroid/content/Context; Landroid/database/Cursor; I)Ljava/lang/String;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/am/ActivityManagerService;-makeAppNotRespondingLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setPasskey-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/util/ThreadPool$Job;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/gallery3d/data/SnailSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/AccessibilityInjector$1;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$UpdateContent;-call-()Ljava/lang/Void;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/pm/PackageManagerService;-access$1700-(Lcom/android/server/pm/PackageManagerService; Ljava/io/File; I I J Landroid/os/UserHandle;)Landroid/content/pm/PackageParser$Package;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/bluetooth/BluetoothDeviceFilter$ClassUuidFilter;-matches-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/activities/ContactEditorActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/power/ShutdownThread;-shutdown-(Landroid/content/Context; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/phone/PhoneGlobals;-clearInCallScreenMode-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/app/ContextImpl;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/GLRootView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/NotificationMgr;-updateInCallNotification-(Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/WallpaperHandler;-openStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;-onProgressStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/launcher2/DeleteDropTarget;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/detail/PhotoSelectionHandler;-<init>-(Landroid/content/Context; Landroid/view/View; I Z Lcom/android/contacts/model/RawContactDeltaList;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/mms/transaction/SmsSingleRecipientSender;-getOutgoingServiceCenter-(J)Ljava/lang/String;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Landroid/app/IWallpaperManager;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/contacts/editor/ContactEditorFragment$5;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/CursorLoader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/ContentService;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/ContextImpl;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/contacts/list/ProviderStatusWatcher;-notifyListeners-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-openFile-(Landroid/net/Uri; Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewClassic$8;-onPostExecute-(Landroid/webkit/WebViewCore$DrawData;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ConversationList;-onStart-()V' : ['android.permission.READ_SMS'],
	'Landroid/media/IRingtonePlayer$Stub$Proxy;-play-(Landroid/os/IBinder; Landroid/net/Uri; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager$Stub;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/SyncManager$ActiveSyncContext;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewProvider;-zoomIn-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-loadDictionaryAsync-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/data/MtpSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActiveServices;-startServiceLocked-(Landroid/app/IApplicationThread; Landroid/content/Intent; Ljava/lang/String; I I I)Landroid/content/ComponentName;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/soundrecorder/SoundRecorder;-access$000-(Lcom/android/soundrecorder/SoundRecorder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-queueNextSignalStrengthPoll-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/settings/users/ProfileUpdateReceiver;-copyProfileName-(Landroid/content/Context;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/launcher2/FolderEditText;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/EmergencyDialer;-keyPressed-(I)V' : ['android.permission.VIBRATE'],
	'Landroid/net/DnsPinger;-getCurrentLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/PhoneProxy;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-getAccountsByTypeAsUser-(Ljava/lang/String; Landroid/os/UserHandle;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/accounts/AccountManager$1;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/wm/WindowStateAnimator;-createSurfaceLocked-()Landroid/view/Surface;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/detail/ContactDetailLayoutController;-showEmptyState-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-onProgressStart-(Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/AccessibilitySettings$ToggleFeaturePreferenceFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/development/AccountsTester$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/ActivityController;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/view/WindowManagerPolicy;-systemReady-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/recent/RecentsPanelView;-simulateClick-(I)Z' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/phone/InCallScreen;-connectBluetoothAudio-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-stopSync-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/MccTable;-setLocaleFromMccIfNeeded-(Landroid/content/Context; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/DefaultContactListAdapter;-getContactUri-(I Landroid/database/Cursor; I I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/InputMethodManagerService;-access$1100-(Lcom/android/server/InputMethodManagerService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/tech/NfcBarcode;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/RIL;-switchWaitingOrHoldingAndActive-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-launchHomeFromHotKey-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WiredAccessoryManager;-access$100-(Lcom/android/server/WiredAccessoryManager; I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/LegacyPostalAddressListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/InputMethodManagerService$ImmsBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/ObjectSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService$UpdateRecord;-disposeLocked-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/StructuredNameDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/UiModeManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/TwilightService;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/MobileNetworkSettings;-onPreferenceChange-(Landroid/preference/Preference; Ljava/lang/Object;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$1;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/SIMFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SelectionManager;-toggle-(Lcom/android/gallery3d/data/Path;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/gallery3d/app/ActivityState;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/ex/variablespeed/MediaPlayerProxy;-seekTo-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/Connectivity;-onRoutedHttpRequest-()V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/net/NetworkStatsService;-updateIfacesLocked-()V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getOwnerPhoneNumberVcard-(Z [B)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/preferences/InvertedContrastPreview;-onBindView-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-showPreview-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/view/WindowManagerPolicy;-interceptMotionBeforeQueueingWhenScreenOff-(I)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onDeleteImage-(Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService;-access$900-(Lcom/android/server/LocationManagerService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-handleBackspace-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/inputmethod/InputMethodAndSubtypeEnabler;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$LocalePickerActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/view/IWindowSession;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/deviceinfo/Status;-setWimaxStatus-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/Connectivity;-onStopDelayedCycle-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-getProfileProxy-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener; I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/MessageUtils;-getMessageDetails-(Landroid/content/Context; Landroid/database/Cursor; I)Ljava/lang/String;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManager$6;-doWork-()V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$PlaybackPositionListener;-onStopTrackingTouch-(Landroid/widget/SeekBar;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-performLayoutAndPlaceSurfacesLockedInner-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateCmdParamsReady;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/CaptivePortalTracker$ActiveNetworkState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/BluetoothEnabler;-setSwitch-(Landroid/widget/Switch;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/BootReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-updateServiceLocation-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$DisplaySettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/downloads/DownloadInfo$Reader;-readRequestHeaders-(Lcom/android/providers/downloads/DownloadInfo;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/mms/model/SlideshowModel;-createFromMessageUri-(Landroid/content/Context; Landroid/net/Uri;)Lcom/android/mms/model/SlideshowModel;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/webkit/WebViewClassic;-setLayerType-(I Landroid/graphics/Paint;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/nsd/INsdManager$Stub$Proxy;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Lcom/android/settings/Settings$DreamSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/app/ApplicationPackageManager;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/server/BluetoothManagerService;-handleDisable-()V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$InitialState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setAndBroadcastNetworkSetTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/inputmethod/latin/LatinIME;-onPressKey-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/vcard/ImportProcessor;-runInternal-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$VpnSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/content/SyncManager$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/AlbumPage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-launchMultiplePhonePicker-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/tech/NdefFormatable;-formatReadOnly-(Landroid/nfc/NdefMessage;)V' : ['android.permission.NFC'],
	'Lcom/android/settings/Settings$AdvancedWifiSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/Bookmarks$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-access$800-(Lcom/android/gallery3d/ui/MenuExecutor; I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/server/LocationManagerService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexServerSession;-onPut-(Ljavax/obex/Operation;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/EmergencyCallHelper;-cleanup-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/widget/MmsWidgetService$MmsFactory;-queryUnreadCount-()I' : ['android.permission.READ_SMS'],
	'Lcom/android/phone/PhoneUtils;-getCallerInfo-(Landroid/content/Context; Lcom/android/internal/telephony/Connection;)Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/transaction/NotificationPlayer;-enqueueLocked-(Lcom/android/mms/transaction/NotificationPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/UsimDataDownloadHandler;-sendSmsAckForEnvelopeResponse-(Lcom/android/internal/telephony/IccIoResult; I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DayView$CalendarGestureListener;-onLongPress-(Landroid/view/MotionEvent;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onCallEnded-(Lcom/android/internal/telephony/Connection$DisconnectCause;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab$2;-onPageStarted-(Landroid/webkit/WebView; Ljava/lang/String; Landroid/graphics/Bitmap;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPin2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/StructuredNameDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/GlobalActions;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/phone/CallNotifier;-sendCdmaCallWaitingReject-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-comeOutOfSleepIfNeededLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DataUsageSummary;-buildLimitedNetworksString-()Ljava/lang/CharSequence;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/content/SyncManager;-updateRunningAccounts-()V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/LegacyPostalAddressListAdapter;-bindQuickContact-(Lcom/android/contacts/list/ContactListItemView; I Landroid/database/Cursor; I I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/MediaPlaybackService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/AttendeesView;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/calendar/alerts/AlertReceiver;-makeBasicNotification-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; J J J I Z I)Lcom/android/calendar/alerts/AlertService$NotificationWrapper;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/list/JoinContactListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/FrameSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/EmergencyCallHelper;-startRetryTimer-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/SyncManager$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onScroll-(Landroid/widget/AbsListView; I I I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/camera/SecureCameraActivity;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$2900-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/net/ConnectivityManagerCompat$BaseConnectivityManagerCompatImpl;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/InCallScreen;-isBluetoothAvailable-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityManagerService;-updateConfigurationLocked-(Landroid/content/res/Configuration; Lcom/android/server/am/ActivityRecord; Z Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/provider/BrowserProvider2;-updateHistoryInTransaction-(Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onRestartRadio-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AgendaWindowAdapter;-buildQueryUri-(I I)Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/pm/PackageManagerService;-replaceNonSystemPackageLI-(Landroid/content/pm/PackageParser$Package; Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$6;-onCameraLaunchedSuccessfully-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/NfcService$TagService;-transceive-(I [B Z)Landroid/nfc/TransceiveResult;' : ['android.permission.NFC'],
	'Lcom/android/phone/InCallScreen;-internalHangup-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Landroid/widget/VideoView;-access$2000-(Landroid/widget/VideoView;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-onAddView-(Landroid/view/View; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/IBackupAgent$Stub;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/DateTimeSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/pm/PackageManagerService;-getPackageGids-(Ljava/lang/String;)[I' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/vcard/VCardParserImpl_V30;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/ContactLoader;-loadInBackground-()Lcom/android/contacts/model/Contact;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/NetworkTimeUpdateService$SettingsObserver;-observe-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-onStart-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/CallNotifier;-silenceRinger-()V' : ['android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/browser/WebViewController;-dismissSubWindow-(Lcom/android/browser/Tab;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/googlecode/mp4parser/authoring/builder/FragmentedMp4Builder;-build-(Lcom/googlecode/mp4parser/authoring/Movie;)Lcom/coremedia/iso/IsoFile;' : ['android.permission.INTERNET'],
	'Lcom/android/settings/Settings$SoundSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/provider/Contacts$People;-queryGroups-(Landroid/content/ContentResolver; J)Landroid/database/Cursor;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/DataUsageSummary;-updateBody-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/FaceUnlock;-handleUnlock-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsFragment;-toggleVisibility-(I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/ContactsApplication$DelayedInitializer;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/IAudioService;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/commands/monkey/Monkey;-main-([Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/nfc/tech/NfcBarcode;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/settings/inputmethod/UserDictionaryLocalePicker;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/calllog/DefaultVoicemailNotifier$DefaultNameLookupQuery;-query-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/inputmethod/latin/LatinIME;-loadSettings-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/base/GLTextureTarget;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/SingleSourceCorpus;-getSuggestions-(Ljava/lang/String; I Z)Lcom/android/quicksearchbox/SuggestionCursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-startDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DataUsageSummary;-hasLimitedNetworks-()Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/Controller;-shouldOverrideUrlLoading-(Lcom/android/browser/Tab; Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/nfc/handover/HandoverService;-onTransferComplete-(Lcom/android/nfc/handover/HandoverTransfer; Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/AccountManager;-getAccountsByType-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/TwilightService;-registerListener-(Lcom/android/server/TwilightService$TwilightListener; Landroid/os/Handler;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler;-handlePhotoActivityResult-(I I Landroid/content/Intent;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$10;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/dreams/phototable/PhotoTable$PhotoLaunchTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewClassic;-restoreState-(Landroid/os/Bundle;)Landroid/webkit/WebBackForwardList;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WifiService;-access$2000-(Lcom/android/server/WifiService; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/IRecognitionService$Stub;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/nfc/INfcAdapterExtras$Stub$Proxy;-setCardEmulationRoute-(Ljava/lang/String; I)V' : ['android.permission.NFC'],
	'Lcom/android/settings/wifi/WifiSettings;-pauseWifiScan-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/NetworkPolicyManager;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/GLView;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/EventInfoFragment;-saveResponse-()Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/NetworkStatsService;-removeUserLocked-(I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/google/android/mms/pdu/PduPersister;-getPendingMessages-(J)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/P2pLinkManager$SendTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothA2dp;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/inputmethodservice/KeyboardView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendConnectNoticeToApp-(Landroid/net/wifi/p2p/WifiP2pDevice; Landroid/net/wifi/p2p/WifiP2pConfig;)Z' : ['android.permission.GET_TASKS'],
	'Lcom/android/settings/Settings$AccountSyncSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/MediaPlaybackService;-open-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccRecords;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/PushReceiver;-findThreadId-(Landroid/content/Context; Lcom/google/android/mms/pdu/GenericPdu; I)J' : ['android.permission.READ_SMS'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter;-dirtyContactsWithinDirtyGroups-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/CallNotifier;-init-(Lcom/android/phone/PhoneGlobals; Lcom/android/internal/telephony/Phone; Lcom/android/phone/Ringer; Lcom/android/phone/CallLogAsync;)Lcom/android/phone/CallNotifier;' : ['android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/NotificationMgr$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/nfc/NfcService;-onRemoteEndpointDiscovered-(Lcom/android/nfc/DeviceHost$TagEndpoint;)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/BaseStatusBar$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-access$500-(Lcom/android/contacts/editor/ContactEditorFragment; Lcom/android/contacts/model/RawContactDelta; Lcom/android/contacts/model/account/AccountWithDataSet; Lcom/android/contacts/model/account/AccountWithDataSet;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/backup/IBackupManager;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/SnapshotTab;-restoreState-(Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/phone/NotificationMgr;-updateInCallNotification-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onResetDone-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-sendDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-closeTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/bluetooth/HidProfile;-setPreferred-(Landroid/bluetooth/BluetoothDevice; Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/calllog/CallLogFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_CALL_LOG', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.READ_PHONE_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/mms/ui/MessageListItem;-bindNotifInd-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/WifiService;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/content/PackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$DreamSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/nfc/NfcAdapter;-getNfcAdapterExtrasInterface-()Landroid/nfc/INfcAdapterExtras;' : ['android.permission.NFC'],
	'Landroid/filterpacks/text/StringLogger;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-access$400-(Lcom/android/gallery3d/ui/MenuExecutor; I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/WallpaperManager;-suggestDesiredDimensions-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Landroid/webkit/WebViewClassic;-goBackOrForward-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/browser/GoogleAccountLogin$3;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onDataConnectionAttached-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/NfcService;-doTransceive-(I [B)[B' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MoviePlayer;-playVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/SuggestionsAdapter$SlowFilterTask;-doInBackground-([Ljava/lang/CharSequence;)Ljava/util/List;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardStatusViewManager;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/inputmethod/research/UploaderService;-onCreate-()V' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/model/dataitem/DataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthTokenByFeatures-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/app/Activity; Landroid/os/Bundle; Landroid/os/Bundle; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/location/fused/FusionEngine;-enableProvider-(Ljava/lang/String; J)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/contacts/ContactsProvider2;-enforceSocialStreamReadPermission-(Landroid/net/Uri;)V' : ['android.permission.READ_SOCIAL_STREAM'],
	'Landroid/os/IPowerManager$Stub$Proxy;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/contacts/activities/ContactDetailActivity;-onAttachFragment-(Landroid/app/Fragment;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/editor/PhotoActionPopup$Listener;-onRemovePictureChosen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/SlideView;-pauseAudio-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPasswordView;-reset-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-requestCloseSpcErrorNotice-(J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/musicfx/ActivityMusic;-onResume-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-endBatchEdit-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/ActivityState;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/app/StatusBarManager;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/browser/SnapshotTab;-goForward-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity$4;-run-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/server/WifiService;-setCountryCode-(Ljava/lang/String; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-setNetworkTemplateEnabled-(Landroid/net/NetworkTemplate; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/ui/CropView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/filterpacks/text/ToUpperCase;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-updateDreamLocked-(I)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity$33;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/vcard/VCardEntry$NameData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl;-processBroadcastIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/TabControl;-addPreloadedTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/Settings$AndroidBeamSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/media/AudioService;-filterMediaKeyEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/media/IAudioService$Stub;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothPbap;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$BluetoothStateTracker;-toggleState-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/contacts/CallLogProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setNetworkSelectionModeManual-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-removeProximityAlert-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-autoDiscover-(Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/GroupMemberLoader;-createUri-()Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/Context;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getOutgoingCallerIdDisplay-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-sendVcardListingXml-(I Ljavax/obex/Operation; I I Ljava/lang/String; Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-setRepeatMode-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmAlertWakeLock;-acquireScreenCpuWakeLock-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/widget/LayoutSuppressingQuickContactBadge;-assignContactFromPhone-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifySignalStrength-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-onStop-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/activities/DialtactsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/settings/AccessibilitySettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler;-getPhotoPickSize-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/test/SyncBaseInstrumentation;-cancelSyncsandDisableAutoSync-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-disable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/DeleteEventHelper$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/TabControl;-destroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebIconDatabaseClassic;-close-()V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/Utils;-getWeekNumberFromTime-(J Landroid/content/Context;)I' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$AutoRegistrationProcess;-start-(Lcom/android/server/sip/SipSessionGroup;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserBookmarksPage$OpenAllInTabsTask;-doInBackground-([Ljava/lang/Void;)Landroid/database/Cursor;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/RIL;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/BatteryStatsService;-getAwakeTimePlugged-()J' : ['android.permission.BATTERY_STATS'],
	'Landroid/webkit/WebViewClassic;-saveWebArchive-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardStatusView$1;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/GroupBrowserActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/downloads/DownloadInfo;-sendIntentIfRequested-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/server/LocationManagerService$Receiver;-incrementPendingBroadcastsLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/DevelopmentSettings;-onResume-()V' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-addNotification-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$3700-(Lcom/android/mms/ui/ComposeMessageActivity; J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/GroupMembershipDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/RotationToggle;-onAttachedToWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/wifi/IWifiManager$Stub;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/camera/VideoModule;-startVideoRecording-()V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-requestChildFocus-(Landroid/view/View; Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$4;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-bindEditorsForExistingContact-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardStatusView$1;-onClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setSuppServiceNotifications-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/model/dataitem/SipAddressDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/MtpDeviceSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/CallbackProxy;-onPageFinished-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-updateSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/exchange/EasSyncService;-sendMeetingResponse-(Lcom/android/exchange/MeetingResponseRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-snapToPage-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/NotificationTransaction;-sendPdu-(J [B Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/MediaSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$7700-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Ljava/lang/String;)Z' : ['android.permission.GET_TASKS'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-loadDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewCore;-sendMessage-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/Utils;-formatDateRange-(Landroid/content/Context; J J I)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mms/ui/SlideViewInterface;-pauseAudio-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$HeaderAdapter;-<init>-(Landroid/content/Context; Ljava/util/List; Lcom/android/settings/accounts/AuthenticatorHelper;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/BluetoothDevicePreference;-onDeviceAttributesChanged-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/BrowserBookmarksPage;-handleContextItem-(I I I)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/bluetooth/A2dpProfile;-<init>-(Landroid/content/Context; Lcom/android/settings/bluetooth/LocalBluetoothProfileManager;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothPbap;-disconnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/model/dataitem/EventDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/AllInOneActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$1400-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasOutboxService;-searchGal-(Landroid/content/Context; J Ljava/lang/String; I)Lcom/android/exchange/provider/GalResult;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/MediaSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/sip/SipAudioCall$1;-onError-(Landroid/net/sip/SipSession; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onStartBatchInput-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-updateServiceLocation-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/RotateFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/webkit/WebViewClassic$OnTrimMemoryListener;-onTrimMemory-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/calllog/ContactInfoHelper;-queryContactInfoForPhoneNumber-(Ljava/lang/String; Ljava/lang/String;)Lcom/android/contacts/calllog/ContactInfo;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/CallbackFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IBackupAgent;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/AudioService;-getBluetoothHeadset-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/ControllerOverlay$Listener;-onPlayPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/StringSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/location/LocationFudger;-<init>-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-onItemClick-(I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/MediaPlaybackService;-access$2300-(Lcom/android/music/MediaPlaybackService;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-doFinish-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/gallery3d/data/LocalSource;-processMapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewCore;-sendPluginDrawMsg-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothDeviceFilter$UnbondedDeviceFilter;-matches-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/CropImage;-access$500-(Lcom/android/gallery3d/app/CropImage; Lcom/android/gallery3d/util/ThreadPool$JobContext; Landroid/graphics/Bitmap;)Z' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/provider/BrowserProvider2;-insertInTransaction-(Landroid/net/Uri; Landroid/content/ContentValues; Z)Landroid/net/Uri;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/bluetooth/BluetoothSettings$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/backup/IBackupManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/model/dataitem/PhoneDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-turnOnUpstreamMobileConnection-(I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-access$000-(Lcom/android/internal/telephony/cdma/CdmaConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/EventDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/model/dataitem/NicknameDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/model/dataitem/StructuredPostalDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/Phone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/StructuredNameEditorView;-switchFromFullNameToStructuredName-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/common/widget/GroupingListAdapter$1;-onChange-(Z)V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/server/am/BroadcastQueue;-scheduleBroadcastsLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/IMediaPlaybackService;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-moveTaskToBack-(I)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/provider/Telephony$Mms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Lcom/android/settings/Settings$UserDictionarySettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-scrollRight-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/SuggestionsAdapter$CursorSource;-runQuery-(Ljava/lang/CharSequence;)V' : ['android.permission.ACCESS_NETWORK_STATE', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/LocationManagerService;-updateProviderListenersLocked-(Ljava/lang/String; Z I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/ActivityState;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$TextToSpeechSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-removeNotification-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/HTML5Audio;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService$ScoClient;-clearCount-(Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/CallCard;-updateNoCall-(Lcom/android/internal/telephony/CallManager;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/PhoneGlobals;-access$800-(Lcom/android/phone/PhoneGlobals; Landroid/os/Message;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-clearProfilePriorty-(Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler;-getWritableEntityIndex-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/EasOutboxService;-canProvision-(Lcom/android/exchange/EasSyncService;)Lcom/android/exchange/adapter/ProvisionParser;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/wifi/WifiStatusTest$3;-onClick-(Landroid/view/View;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/sip/SipService;-stopPortMappingMeasurement-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/media/RingtonePickerActivity;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothDevice;-getBluetoothClass-()Landroid/bluetooth/BluetoothClass;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/wifi/WifiSettings;-updateAccessPoints-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SyncStateTracker$1;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/os/Vibrator;-vibrate-([J I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView$2;-onKeyguardVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/AccessibilitySettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewCallback;-pokeWakelock-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/ConnectivityService;-access$400-(Lcom/android/server/ConnectivityService; Lcom/android/server/ConnectivityService$FeatureUser; Z)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/SmsUsageMonitor$SettingsObserverHandler;-<init>-(Landroid/content/Context; Ljava/util/concurrent/atomic/AtomicBoolean;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/RequestPermissionActivity;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$WifiStateTracker;-onActualStateChange-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-reloadFolderList-(Landroid/content/Context; J Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-blockingGetAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Z)Ljava/lang/String;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/speech/RecognitionService;-access$700-(Landroid/speech/RecognitionService; Landroid/speech/IRecognitionListener;)Z' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-handleNotificationError-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification; Ljava/lang/String;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/ThrottleService$DataRecorder;-<init>-(Landroid/content/Context; Lcom/android/server/ThrottleService;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/nfc/NfcService;-onLlcpLinkActivated-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/server/VibratorService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/settings/TetherSettings;-startProvisioningIfNecessary-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/SyncManager$9;-onServiceChanged-(Landroid/content/SyncAdapterType; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/PbapServerProfile;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager$ViewManagerHost;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$6500-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phone/InCallScreen;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-handlePhoneStateChanged-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter;-sendLocalChanges-(Lcom/android/exchange/adapter/Serializer;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/ZoomManager$ScaleDetectorListener;-onScale-(Landroid/view/ScaleGestureDetector;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiManager;-clearBlacklist-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/accounts/AccountPreferenceBase;-onSyncStateUpdated-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS'],
	'Lcom/android/contacts/editor/TextFieldsEditorView;-onOptionalFieldVisibilityChange-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/alerts/AlertReceiver;-createBroadcastMailIntent-(Landroid/content/Context; J Ljava/lang/String;)Landroid/app/PendingIntent;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/WifiService;-reportStartWorkSource-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SMSDispatcher;-dispatchMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/launcher2/WallpaperChooserDialogFragment;-selectWallpaper-(I)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManagerProxy;-getAllMessagesFromIccEf-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/wm/WindowManagerService;-access$300-(Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ContactListAdapter;-getContactUri-(I Landroid/database/Cursor; I I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onSeApduReceived-([B)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PasswordUnlockScreen;-verifyPasswordAndUnlock-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-notifyLidSwitchChanged-(J Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/google/common/io/MultiReader;-skip-(J)J' : ['android.permission.INTERNET'],
	'Lcom/android/camera/PhotoModule;-onResumeAfterSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar$27;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.VIBRATE', 'android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/calendar/EmailAddressAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/wallpaper/fall/FallView;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/sip/SipSessionGroup;-processDialogTerminated-(Ljavax/sip/DialogTerminatedEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-requestChildRectangleOnScreen-(Landroid/view/View; Landroid/graphics/Rect; Z)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/sip/SipAudioCall;-grabWifiHighPerfLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupCreatedState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/IContentService;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/Settings$AdvancedWifiSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/nfc/NfcAdapter;-setBeamPushUris-([Landroid/net/Uri; Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifyTransactionListeners-([B)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSimPinView$4$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/IWifiP2pManager$Stub;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/mms/data/Contact;-getUri-()Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/accounts/AccountSyncSettings;-setFeedsState-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS'],
	'Landroid/filterfw/core/GraphRunner;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ManageSimMessages;-access$500-(Lcom/android/mms/ui/ManageSimMessages; Landroid/database/Cursor;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/mms/util/DraftCache$1;-run-()V' : ['android.permission.READ_SMS'],
	'Lcom/android/browser/SnapshotTab;-loadUrl-(Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/wifi/WifiStatusTest;-access$100-(Lcom/android/settings/wifi/WifiStatusTest; Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/view/WindowManagerPolicy;-notifyLidSwitchChanged-(J Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/wifi/WifiSettingsForSetupWizardXL;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/ActivityManagerNative;-unbroadcastIntent-(Landroid/app/IApplicationThread; Landroid/content/Intent; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/power/PowerManagerService$UserSwitchedReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CsimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pan/PanService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/SlideView$1;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcAdapterExtras$Stub$Proxy;-getDriverName-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.NFC'],
	'Lcom/android/settings/Settings$WirelessSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/display/DisplayManagerService;-unblankAllDisplaysFromPowerManager-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-exitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-addPanelWindows-()V' : ['android.permission.VIBRATE'],
	'Landroid/content/IContentService$Stub$Proxy;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/contacts/list/ContactListFilterControllerImpl;-createContactListFilterController-(Landroid/content/Context;)Lcom/android/contacts/list/ContactListFilterController;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/NetworkPolicyManager;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/systemui/Gefingerpoken;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/accounts/AccountManagerService;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/contacts/editor/AggregationSuggestionView;-canEditSuggestedContact-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-startDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/UnlockImage;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onError-(Lcom/android/internal/telephony/Connection$DisconnectCause;)V' : ['android.permission.WAKE_LOCK'],
	'Ljava/net/ServerSocket;-bind-(Ljava/net/SocketAddress;)V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/AlbumPage$5;-onActionItemClicked-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getRemoteName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback;-reportFailedUnlockAttempt-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-next-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-openFile-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ITelephony$Stub$Proxy;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaSet-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaSet;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter;-sendLocalChanges-(Lcom/android/exchange/adapter/Serializer;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/PhoneGlobals$PhoneAppBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-addStatusBarWindow-()V' : ['android.permission.DISABLE_KEYGUARD', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/ConnectivityService$FeatureUser;-binderDied-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/ui/MessageUtils$ResizeImageResultCallback;-onResizeResult-(Lcom/google/android/mms/pdu/PduPart; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/widget/WaveView;-vibrate-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/media/IMediaScannerListener$Stub;-scanCompleted-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/mms/transaction/SendTransaction;-ensureRouteToHost-(Ljava/lang/String; Lcom/android/mms/transaction/TransactionSettings;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/CameraShortcutImage;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/deviceinfo/Memory;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Connected;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/EasOutboxService;-acknowledgeProvision-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardParser_V30;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/policy/BluetoothController;-<init>-(Landroid/content/Context;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/media/IAudioService$Stub$Proxy;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/browser/AddBookmarkPage;-descendInto-(Ljava/lang/String; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/camera/ui/PieItem$OnClickListener;-onClick-(Lcom/android/camera/ui/PieItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/Settings$SoundSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/wm/WindowAnimator;-animateLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-access$2100-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/view/IWindowManager;-clearForcedDisplayDensity-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/IdentityDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/IActivityManager;-moveTaskToBack-(I)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/browser/view/PieMenu$PieView$OnLayoutListener;-onLayout-(I I Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-handleMove-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/ISms;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage;-initializeViews-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/DTMFTwelveKeyDialer;-openDialer-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/providers/downloads/StorageManager;-trimDatabase-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/list/ContactListFilterController;-createContactListFilterController-(Landroid/content/Context;)Lcom/android/contacts/list/ContactListFilterController;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterfw/core/FilterContext;-tearDown-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-parseIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/nfc/handover/HandoverServer$ConnectionThread;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/transaction/SmsMessageSender;-sendMessage-(J)Z' : ['android.permission.SEND_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/PickerActivity;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getRemoteClass-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Contacts$Settings;-setSetting-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/sip/SipService$AutoRegistrationProcess;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5VideoInline;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Button;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/speech/IRecognitionService$Stub;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/calendar/event/EditEventView;-setModel-(Lcom/android/calendar/CalendarEventModel;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame;-onMeasure-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/EventInfoFragment;-access$1000-(Lcom/android/calendar/EventInfoFragment; Landroid/view/View;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/ConnectivityService;-access$1200-(Lcom/android/server/ConnectivityService; Landroid/net/NetworkInfo;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-setTokenVisibilityLocked-(Lcom/android/server/wm/AppWindowToken; Landroid/view/WindowManager$LayoutParams; Z I Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wm/InputMonitor;-interceptMotionBeforeQueueingWhenScreenOff-(I)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManager;-removeAccount-(Landroid/accounts/Account; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/app/AlertActivity;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processAtChld-(I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/NicknameDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiStateMachine$L2ConnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/AsyncPlayer;-access$500-(Landroid/media/AsyncPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-setForcedDisplayDensityLocked-(Lcom/android/server/wm/DisplayContent; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexServerSession;-preStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView$MyGestureListener;-onFling-(F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/transaction/RetrieveTransaction;-sendPdu-(J [B)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/wallpaper/fall/FallRS;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-open-([J I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/TintFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/activities/TransactionSafeActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/nfc/NfcService;-sendMockNdefTag-(Landroid/nfc/NdefMessage;)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/camera/ActivityBase;-gotoGallery-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleDown-(Landroid/view/MotionEvent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-revert-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity$QueryEntitiesTask;-onPostExecute-(Lcom/android/contacts/model/RawContactDeltaList;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$StorageUseActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/vcard/ImportVCardActivity$VCardCacheThread;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-dispatchPortAddressedPdus-([[B I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/CrashRecoveryHandler$2;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/am/ActivityStack;-activitySleptLocked-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/pm/PackageManagerService;-scanPackageLI-(Landroid/content/pm/PackageParser$Package; I I J Landroid/os/UserHandle;)Landroid/content/pm/PackageParser$Package;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/pm/PackageManagerService;-loadMediaPackages-(Ljava/util/HashMap; [I Ljava/util/HashSet;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/accounts/AccountSyncSettings$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/AlbumPage;-switchToFilmstrip-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/pm/PackageManagerService;-replacePackageLI-(Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/webkit/WebViewClassic;-sendKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/list/OnContactBrowserActionListener;-onViewContactAction-(Landroid/net/Uri;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/detail/ContactLoaderFragment;-getPreAuthorizedUri-(Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-handleMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothPan$Stub$Proxy;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/InCallScreen;-endInCallScreenSession-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/sip/SipManager;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-getSelectedMediaObjects-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupConnectionByIndex-(Lcom/android/internal/telephony/gsm/GsmCall; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/ShortcutIntentBuilder$LoadingAsyncTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-enqueue-([J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/WiredAccessoryManager;-access$000-(Lcom/android/server/WiredAccessoryManager;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/RuimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/SmsReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-setRadios-(Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-scrollLeft-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/Gallery;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover;-getProfileProxys-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/IAccountManager$Stub;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-updateCredentials-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/UndoBarView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/exchange/ExchangeService$5;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$DriverStoppingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-editMessageItem-(Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/phone/SimContacts;-actuallyImportOneSimContact-(Landroid/database/Cursor; Landroid/content/ContentResolver; Landroid/accounts/Account;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/FrameFetch;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/ContentService;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/mms/transaction/PushReceiver;-access$100-(Landroid/content/Context; Lcom/google/android/mms/pdu/NotificationInd;)Z' : ['android.permission.READ_SMS'],
	'Lcom/android/settings/DataUsageSummary$ConfirmRestrictFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-isNetworkConnected-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/SlideView;-stopVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-handleShow-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/BluetoothDiscoverableTimeoutReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/mms/transaction/Observer;-update-(Lcom/android/mms/transaction/Observable;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/settings/Utils;-isWifiOnly-(Landroid/content/Context;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-verifyUnlockLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-connectSocket-(Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-chooseUpstreamType-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/HidProfile;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/soundrecorder/SoundRecorder;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-addPreferencesForActivity-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onRoamingOff-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-snapToDestination-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$30;-onClick-(Landroid/view/View;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/DisplayPowerController;-<init>-(Landroid/os/Looper; Landroid/content/Context; Lcom/android/server/power/Notifier; Lcom/android/server/LightsService; Lcom/android/server/TwilightService; Landroid/hardware/SensorManager; Lcom/android/server/display/DisplayManagerService; Lcom/android/server/power/DisplayBlanker; Lcom/android/server/power/DisplayPowerController$Callbacks; Landroid/os/Handler;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/SearchActivity$TextViewSnippet;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/usb/UsbDeviceManager$1;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/location/ILocationManager$Stub$Proxy;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/contacts/detail/ContactDetailFragment$QuickFix;-execute-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/NfcService$NfcAdapterExtrasService;-authenticate-(Ljava/lang/String; [B)V' : ['android.permission.NFC'],
	'Lcom/android/exchange/AbstractSyncService;-hasConnectivity-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$InputMethodAndSubtypeEnablerActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-setSearchMode-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/activities/PeopleActivity;-configureContactListFragment-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener$Adapter;-onStartBatchInput-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-stopScanning-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView;-onPause-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/coremedia/iso/IsoFile;-createBoxParser-()Lcom/coremedia/iso/BoxParser;' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/model/dataitem/PhoneDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/CaptivePortalTracker$ActiveNetworkState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/BandMode;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmPreference$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/AccelerometerListener$OrientationListener;-orientationChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-interceptMediaKey-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-iccIO-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/IdentityDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/nfc/INfcTag;-ndefRead-(I)Landroid/nfc/NdefMessage;' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/CommandsInterface;-setLocationUpdates-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IIntentReceiver$Stub$Proxy;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/LocalePicker$1;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManagerService;-addAcount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/CallDetailActivity;-access$300-(Lcom/android/contacts/CallDetailActivity; Landroid/net/Uri;)Lcom/android/contacts/PhoneCallDetails;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-setupData-(Lcom/android/internal/telephony/ApnContext;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/model/AccountTypeManager;-getInstance-(Landroid/content/Context;)Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/TimerRingService$2;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-setUsbTethering-(Z)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/accounts/AccountSyncSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ComposeMessageActivity$11;-run-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetoothManager;-disable-(Z)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/AllInOneActivity;-onQueryTextSubmit-(Ljava/lang/String;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/group/GroupEditorFragment$4;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/InCallScreen$6;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/SlidingTab;-setGrabbedState-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/vcard/SelectAccountActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/util/ThreadPool$Worker;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/SecureCameraActivity;-onCameraSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/ContentResolver;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/content/IContentService$Stub;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-toggleBluetooth-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/phone/AccelerometerListener;-onSensorEvent-(D D D)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-cleanUpConnection-(Z Lcom/android/internal/telephony/ApnContext;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ActivityManagerNative;-updatePersistentConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSimPinView;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/coremedia/iso/IsoFile;-<init>-()V' : ['android.permission.INTERNET'],
	'Lcom/android/settings/accounts/ManageAccountsSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/research/UploaderService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/content/AbstractThreadedSyncAdapter$ISyncAdapterImpl;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/ConnectivityService;-systemReady-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/google/common/io/Files;-readBytes-(Ljava/io/File; Lcom/google/common/io/ByteProcessor;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/CalendarSyncAdapterService;-access$000-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle; Ljava/lang/String; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/bluetooth/A2dpProfile;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-setNdefPushCallback-(Landroid/nfc/INdefPushCallback;)V' : ['android.permission.NFC'],
	'Lcom/android/systemui/SearchPanelView;-access$300-(Lcom/android/systemui/SearchPanelView;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/CropImage$SaveOutput;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Landroid/content/Intent;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mms/data/Conversation;-get-(Landroid/content/Context; J Z)Lcom/android/mms/data/Conversation;' : ['android.permission.READ_SMS'],
	'Lcom/android/settings/DevelopmentSettings;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_EXTERNAL_STORAGE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/activities/ActionBarAdapter$Listener;-onSelectedTabChanged-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnection;-onConnect-(Lcom/android/internal/telephony/DataConnection$ConnectionParams;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService$Stub;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-getBondState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-refresh-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/InCallScreen;-syncWithPhoneState-()Lcom/android/phone/InCallScreen$SyncWithPhoneStateStatus;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/PhoneProxy;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/ContactEditorFragment$EntityDeltaComparator;-compare-(Lcom/android/contacts/model/RawContactDelta; Lcom/android/contacts/model/RawContactDelta;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-access$300-(Landroid/media/MediaPlayer; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-access$2100-(Lcom/android/contacts/editor/ContactEditorFragment;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewClassic;-setNewPicture-(Landroid/webkit/WebViewCore$DrawData; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ProviderStatusWatcher;-access$600-(Lcom/android/contacts/list/ProviderStatusWatcher;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/DownUpDetector;-onTouchEvent-(Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-setRadio-(I Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/contacts/CallDetailActivity$6;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/accounts/AccountManager;-confirmCredentialsAsUser-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler; Landroid/os/UserHandle;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityStack;-stopIfSleepingLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/systemui/media/RingtonePlayer$1;-playAsync-(Landroid/net/Uri; Landroid/os/UserHandle; Z I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/NdefFormatable;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/common/contacts/BaseEmailAddressAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/vcard/ExportProcessor;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/content/SyncManager$5;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/contacts/group/GroupDetailFragment;-updateAccountType-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/SecureAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/model/RawContactDelta;-getRawContactAccountType-(Landroid/content/Context;)Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/BackupManagerService$PerformClearTask;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-setUsbTethering-(Z)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-okToConnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebViewClassic;-onZoomAnimationEnd-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/CalendarController$EventHandler;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/CsimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebSettingsClassic;-<init>-(Landroid/content/Context; Landroid/webkit/WebViewClassic;)V' : ['android.permission.INTERNET'],
	'Lcom/android/browser/TabControl;-createSnapshotTab-(J)Lcom/android/browser/SnapshotTab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-setCellBroadcastActivation-(Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-revertToNitzTime-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionCallReceiverImpl;-processCommand-(Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/util/Recycler;-deleteOldMessages-(Landroid/content/Context;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/BluetoothAdapter;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$WifiDisplaySettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/DockService$5;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/providers/calendar/CalendarProvider2;-updateEventAttendeeStatus-(Landroid/database/sqlite/SQLiteDatabase; Landroid/content/ContentValues;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingScoOn-()Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$StopTetheringErrorState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/CropView$FaceHighlightView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-handleDown-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$ScanModeState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebViewCore;-sendMessage-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/SnapshotTab;-setWebView-(Landroid/webkit/WebView; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/PhoneGlobals;-onCreate-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_CALL_LOG', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/data/WorkingMessage;-setConversation-(Lcom/android/mms/data/Conversation;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/phone/OtaUtils;-otaShowProgramSuccessDialog-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/DhcpStateMachine$StoppedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-refreshSelectedContactUri-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DisplaySettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/am/ActivityManagerService;-trimApplications-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/AppWidgetService;-systemReady-(Z)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/DayView;-access$4900-(Lcom/android/calendar/DayView; Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/webkit/WebViewClassic;-contentSizeChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/IWindowManager;-prepareAppTransition-(I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/NfcAdapter;-setBeamPushUrisCallback-(Landroid/nfc/NfcAdapter$CreateBeamUrisCallback; Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Lcom/android/calendar/AllInOneActivity;-onNavigationItemSelected-(I J)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$ProfileSyncState;-newSetOperation-(Landroid/accounts/Account; [B)Landroid/content/ContentProviderOperation;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/SmsReceiverService;-moveOutboxMessagesToFailedBox-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/soundrecorder/SoundRecorder;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManager;-getRunningTasks-(I I Landroid/app/IThumbnailReceiver;)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/voicedialer/VoiceDialerActivity;-playSound-(I)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice$Callback;-onDeviceAttributesChanged-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/AccountManager;-getAccountsByTypeAndFeatures-(Ljava/lang/String; [Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/AccessibilityInjector$CallbackHandler;-requestCallback-(Landroid/webkit/WebView; Ljava/lang/Runnable;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-setCdmaSubscriptionSource-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ViewRootImpl$InputMethodCallback;-finishedEvent-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/PhoneUtils;-sendEmptyFlash-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-handleNotifyScreenOff-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-getContactUriForEmail-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSimPinView;-reset-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/location/ILocationManager;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/providers/contacts/ContactsProvider2;-isContactsAccount-(Landroid/accounts/Account;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/GalleryActionBar;-onNavigationItemSelected-(I J)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/IContentService$Stub$Proxy;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/inputmethod/research/ResearchLogger;-stop-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/transaction/MessagingNotification;-updateSendFailedNotificationForThread-(Landroid/content/Context; J)V' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-doRecovery-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/settings/wfd/WifiDisplaySettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/HapticFeedback;-vibrate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/webkit/HTML5VideoInline;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/util/Recycler$MmsRecycler;-getAllThreads-(Landroid/content/Context;)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Landroid/media/RingtoneManager;-stopPreviousRingtone-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/RuimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-updateDefaultZoomDensity-(F)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothInputDevice;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewCore;-jsUnload-(Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/AppWidgetPickActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/NfcActivityManager;-setNdefPushContentUriCallback-(Landroid/app/Activity; Landroid/nfc/NfcAdapter$CreateBeamUrisCallback;)V' : ['android.permission.NFC'],
	'Lcom/android/server/InputMethodManagerService;-checkCurrentLocaleChangedLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-getTetherableWifiRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/Tab;-persistThumbnail-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/editor/ContactEditorUtils;-getInstance-(Landroid/content/Context;)Lcom/android/contacts/editor/ContactEditorUtils;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothA2dp;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/accounts/ManageAccountsSettings;-onPrepareOptionsMenu-(Landroid/view/Menu;)V' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$4;-run-()V' : ['android.permission.REORDER_TASKS'],
	'Landroid/widget/Switch;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/downloads/DownloadProvider;-openFile-(Landroid/net/Uri; Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar$25;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/providers/contacts/ProfileProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; Landroid/os/CancellationSignal;)Landroid/database/Cursor;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$20;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/InCallScreen;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/power/Notifier;-access$500-(Lcom/android/server/power/Notifier;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/content/IContentService$Stub$Proxy;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/BackupManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/usb/UsbDeviceManager;-systemReady-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/SmsManager;-enableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/browser/DataController$DataControllerHandler;-doUpdateHistoryTitle-(Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/settings/Settings$TextToSpeechSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-onStart-()V' : ['android.permission.GET_TASKS'],
	'Ljava/net/URL;-getContent-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/settings/Settings$SoundSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/inputmethod/InputMethodAndLanguageSettings;-onResume-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hdp/HealthService;-registerAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/IBluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-addDefaultWidgets-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/FrameBranch;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$1000-(Lcom/android/server/net/NetworkPolicyManagerService; Landroid/net/NetworkTemplate; I)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/ui/Divot;-assignContactFromEmail-(Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothInputDevice;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ComposeMessageActivity$23;-run-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/ConnectivityService;-requestRouteToHost-(I I)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/settings/Settings$DeviceInfoSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/ConnectivityService;-enforceTetherChangePermission-()V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/contacts/ContactsActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/editor/StructuredNameEditorView;-onLabelRebuilt-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccCardApplication;-supplyPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-isDrmRingtoneWithRights-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/mms/ui/SlideView;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService$Receiver;-decrementPendingBroadcastsLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/BookmarkExpandableView;-onTop-(Lcom/android/browser/BreadCrumbView; I Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/SecureAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/contacts/ContactsProvider2;-enforceSocialStreamWritePermission-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-proceedAfterWildChar-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-requestRouteToHostAddress-(I [B)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/ConnectivityManager;-stopUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchWithCaptureAnimation-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/SecuritySettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/media/AudioService;-disconnectBluetoothSco-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider;-checkObserver-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/performance/ThroughputFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$HideNavInputEventReceiver;-onInputEvent-(Landroid/view/InputEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/settings/wifi/WifiApEnabler;-access$100-(Lcom/android/settings/wifi/WifiApEnabler; [Ljava/lang/Object; [Ljava/lang/Object; [Ljava/lang/Object;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/list/DefaultContactListAdapter;-bindQuickContact-(Lcom/android/contacts/list/ContactListItemView; I Landroid/database/Cursor; I I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/MovieActivity;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/location/fused/FusionEngine;-disable-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/IAudioService$Stub$Proxy;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/sip/SipAudioCall;-access$900-(Landroid/net/sip/SipAudioCall; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-stopDtmf-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/TelephonyManager;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/ConnectivityManager;-getTetherableBluetoothRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/WirelessSettings;-onResume-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager$Callbacks;-onAddView-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcAdapter$Stub;-getNfcAdapterExtrasInterface-(Ljava/lang/String;)Landroid/nfc/INfcAdapterExtras;' : ['android.permission.NFC'],
	'Lcom/android/server/wm/WindowManagerService;-detectSafeMode-()Z' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/provider/Browser;-updateVisitedHistory-(Landroid/content/ContentResolver; Ljava/lang/String; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/vpn2/VpnSettings;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ThrottleService$DataRecorder;-getDataFile-()Ljava/io/File;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/exchange/CalendarSyncAdapterService$SyncAdapterImpl;-onPerformSync-(Landroid/accounts/Account; Landroid/os/Bundle; Ljava/lang/String; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/Settings$AdvancedWifiSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/contacts/VoicemailPermissions;-callerHasOwnVoicemailAccess-()Z' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/location/fused/FusedLocationProvider;-onDisable-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/ActivityController;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame$7;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/pinyin/SkbContainer;-onTouch-(Landroid/view/View; Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardStatusView$1;-onDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-setCellBroadcastConfig-([Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/SipAddressDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$5700-(Lcom/android/mms/ui/ComposeMessageActivity; Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/ThrottleManager;-getCliffThreshold-(Ljava/lang/String; I)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$MediaPlayerCompletionListener$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/browser/Controller;-onDestroy-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hfp/HeadsetPhoneState;-cleanup-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/MmsThumbnailPresenter;-presentVideoThumbnail-(Lcom/android/mms/ui/SlideViewInterface; Lcom/android/mms/model/VideoModel;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-getHealthDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onCallEnded-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IThrottleManager$Stub$Proxy;-getPeriodStartTime-(Ljava/lang/String;)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/BluetoothPhoneService$3;-answerCall-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/music/AudioPreview;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/SharpenEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/browser/UrlHandler;-handleMenuClick-(Lcom/android/browser/Tab; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager$Stub;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ContextImpl;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/power/PowerManagerService;-handleUserActivityTimeout-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/ProfileProvider;-enforceReadPermission-(Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PatternUnlockScreen;-<init>-(Landroid/content/Context; Landroid/content/res/Configuration; Lcom/android/internal/widget/LockPatternUtils; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-access$1200-(Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/backup/IRestoreSession$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/Recorder;-startRecording-(I Ljava/lang/String; Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO'],
	'Lcom/android/camera/ui/TimeIntervalPopup$Listener;-onListPrefChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/event/EditEventFragment$Done;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/phone/sip/SipSettings;-retrieveSipLists-()V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/SIMRecords;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ThrottleService;-enforceAccessPermission-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-isSyncActive-()Z' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/launcher2/AppsCustomizePagedView;-handleTouchEvent-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Landroid/nfc/INfcTag$Stub;-getTechList-(I)[I' : ['android.permission.NFC'],
	'Lcom/android/calendar/DayView;-doDown-(Landroid/view/MotionEvent;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/browser/Controller;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/Settings$PrivacySettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/IWindowSession$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/providers/contacts/VoicemailPermissions;-callerHasFullAccess-()Z' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-sendMeetingResponse-(J I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/ui/PhotoView$MyGestureListener;-onScale-(F F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/Settings$InputMethodAndSubtypeEnablerActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/DictionaryCollection;-getSuggestionsWithSessionId-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; I)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/WebViewClassic;-onAttachedToWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/GLRootView$IdleRunner;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/calendar/CalendarReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmReceiver;-handleIntent-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5VideoInline;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/PeopleActivity$ContactBrowserActionListener;-onSelectionChange-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-notifyPageSwitching-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-init-(Landroid/content/Context; Landroid/view/IWindowManager; Landroid/view/WindowManagerPolicy$WindowManagerFuncs;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/accounts/SyncSettings;-onSyncStateUpdated-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-onWakeMotionWhenKeyguardShowingTq-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/vcard/NotificationImportExportListener;-onImportFinished-(Lcom/android/contacts/vcard/ImportRequest; I Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/ContactsActivity;-onSearchRequested-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-launchDevicePicker-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/LocalePicker;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WifiService;-updateWifiState-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub;-clearForcedDisplayDensity-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneBase;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoSource;-load-(Lcom/android/dreams/phototable/PhotoSource$ImageData; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/ContactsAsyncHelper$ImageTracker;-getPhotoUri-()Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-access$400-(Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterState$OffState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetooth$Stub;-getRemoteAlias-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/EasOutboxService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-setNetworkPreference-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/systemui/usb/UsbPermissionActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockScreen$2;-run-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/CropView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/util/Recycler$MmsRecycler;-deleteOldMessagesByThreadId-(Landroid/content/Context; J)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/test/mock/MockContentResolver;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/SlidingTab;-cancelGrab-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/phone/InCallScreen;-requestUpdateScreen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-bindView-(Landroid/view/View; Landroid/database/Cursor; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewClassic;-access$6800-(Landroid/webkit/WebViewClassic;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/PeopleActivity;-onAction-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/MessageListView$OnSizeChangedListener;-onSizeChanged-(I I I I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/browser/DataController$DataControllerHandler;-doQueryBookmarkStatus-(Ljava/lang/String; Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView;-access$700-(Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/IApplicationThread;-scheduleRegisteredReceiver-(Landroid/content/IIntentReceiver; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mms/ui/MmsThumbnailPresenter;-presentImageThumbnail-(Lcom/android/mms/ui/SlideViewInterface; Lcom/android/mms/model/ImageModel;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-ensureBootCompleted-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/Settings$DeviceAdminSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/InCallScreen;-onDisconnect-(Landroid/os/AsyncResult;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/AlbumPage;-onSelectionModeChange-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/net/wifi/WifiManager;-getScanResults-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/power/ShutdownThread$5;-run-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/FindActionModeCallback;-onActionItemClicked-(Landroid/view/ActionMode; Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame$7;-onBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/handover/HandoverTransfer;-updateStateAndNotification-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/AllInOneActivity;-access$700-(Lcom/android/calendar/AllInOneActivity; J)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/SettingsSafetyLegalActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/AlbumPage;-showDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Landroid/accounts/IAccountManager$Stub;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-getHealthDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/nfc/tech/NfcV;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/contacts/activities/TransactionSafeActivity;-onSearchRequested-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$StateTracker;-onActualStateChange-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/media/effect/effects/LomoishEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/SlideViewInterface;-startVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/AlertService;-dismissOldAlerts-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onSetDependencyMet-(Ljava/lang/String; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateCmdParamsReady;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/power/PowerManagerService;-handleScreenOnBlockerReleased-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/content/ContentResolver;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-selectFirstFoundContactAfterDelay-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/event/AttendeesView;-constructAttendeeView-(Lcom/android/calendar/event/EditEventHelper$AttendeeItem;)Landroid/view/View;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/ZoomManager;-access$1700-(Landroid/webkit/ZoomManager; F Z Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView$1;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/SlotView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/Settings$RunningServicesActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/transaction/NotificationTransaction;-notifyObservers-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/phone/PhoneUtils$3;-onQueryComplete-(I Ljava/lang/Object; Lcom/android/internal/telephony/CallerInfo;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Landroid/content/ContentResolver;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/bluetooth/BluetoothDevice;-setAlias-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/StateManager;-startState-(Ljava/lang/Class; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/Settings$WifiP2pSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/location/CountryDetectorBase;-setCountryListener-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/TelephonyRegistry;-broadcastDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/list/JoinContactListAdapter;-getContactUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/ConnectivityManager;-getTetherableUsbRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/CallCard;-updateRingingCall-(Lcom/android/internal/telephony/CallManager;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/Phone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/group/GroupEditorFragment;-onAccountChosen-(Lcom/android/contacts/model/account/AccountWithDataSet; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/camera/SecureCameraActivity;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/commands/bmgr/Bmgr;-run-([Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService;-setWiredDeviceConnectionState-(I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/PeopleActivity$ContactBrowserActionListener;-onInvalidSelection-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/handover/HandoverTransfer;-onScanCompleted-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub$Proxy;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/accounts/AccountManager$1;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/ExchangeService;-stopNonAccountMailboxSyncsForAccount-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/WifiService;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-sendDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-autoDiscover-(Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-handleKeyguardVisibilityChanged-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/recent/RecentTasksLoader;-loadFirstTask-()Lcom/android/systemui/recent/TaskDescription;' : ['android.permission.GET_TASKS'],
	'Lcom/android/gallery3d/app/PhotoPage$6;-onPhotoChanged-(I Lcom/android/gallery3d/data/Path;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/DataConnectionTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-setupData-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/AddBookmarkPage$1;-onLoadFinished-(Landroid/content/Loader; Lcom/android/browser/AddBookmarkPage$EditBookmarkInfo;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManagerProxy;-copyMessageToIccEf-(I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/camera/VideoModule;-onSharedPreferenceChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/media/MediaProvider;-access$1200-(Lcom/android/providers/media/MediaProvider; Lcom/android/providers/media/MediaProvider$ThumbData;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.READ_EXTERNAL_STORAGE', 'android.permission.WRITE_EXTERNAL_STORAGE'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-notifyAndAcknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback;-keyguardDone-(Z)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebView;-addJavascriptInterface-(Ljava/lang/Object; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub;-getLastTetherError-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/CalendarController;-sendEventRelatedEventWithExtra-(Ljava/lang/Object; J J J J I I J J)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-disableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/group/GroupEditorFragment;-isGroupMembershipEditable-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/SlideshowView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/DriverCall; Lcom/android/internal/telephony/cdma/CdmaCallTracker; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CallManager;-setAudioMode-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/providers/media/RingtonePickerActivity;-onStop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-prepareEmptyView-()V' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$9;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wm/Session;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/cdma/CdmaCallWaitingNotification; Lcom/android/internal/telephony/cdma/CdmaCallTracker; Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-setScrollState-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/IdentityDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/DownUpDetector$DownUpListener;-onDown-(Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-startDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/group/GroupDetailFragment$2;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/INfcTag$Stub;-setTimeout-(I I)I' : ['android.permission.NFC'],
	'Lcom/android/browser/SnapshotTab;-onNewPicture-(Landroid/webkit/WebView; Landroid/graphics/Picture;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/UiController;-closeTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/wifi/WifiApEnabler$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/TrimVideo;-setProgress-()I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/LegacyContactListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$StorageSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/service/wallpaper/WallpaperService$Engine$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/providers/contacts/VoicemailPermissions;-checkCallerHasFullAccess-()V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/server/ConnectivityService;-stopUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/wifi/WifiStateMachine;-updateBatteryWorkSource-(Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ViewRootImpl;-deliverPointerEvent-(Landroid/view/ViewRootImpl$QueuedInputEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/MessageItem$PduLoadedMessageItemCallback;-onItemLoaded-(Ljava/lang/Object; Ljava/lang/Throwable;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$VpnSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager$Stub;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SyncStateTracker;-setImageViewResources-(Landroid/content/Context; Landroid/widget/RemoteViews;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/soundrecorder/SoundRecorder;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/DirectoryListLoader;-loadInBackground-()Landroid/database/Cursor;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-setSearchMode-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/CaptivePortalTracker$NoActiveNetworkState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManagerProxy;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$SetIpForwardingEnabledErrorState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/nfc/INfcAdapter$Stub;-enableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$ScreenLockTimeout;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-restartRadio-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/phone/BluetoothPhoneService;-access$200-(Lcom/android/phone/BluetoothPhoneService; Lcom/android/internal/telephony/Connection;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/EasOutboxService;-messageMoveRequest-(Lcom/android/exchange/MessageMoveRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/DownloadThread;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/view/PieMenu;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/Settings$AccessibilitySettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/ContactsSyncAdapterService;-performSync-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle; Ljava/lang/String; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-bindEditorsForNewContact-(Lcom/android/contacts/model/account/AccountWithDataSet; Lcom/android/contacts/model/account/AccountType; Lcom/android/contacts/model/RawContactDelta; Lcom/android/contacts/model/account/AccountType;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/NfcService;-sendMessage-(I Ljava/lang/Object;)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-onPreferenceChange-(Landroid/preference/Preference; Ljava/lang/Object;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/AccessibilitySettings$SettingsPackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/AbstractSyncParser;-parse-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppIncomingFileConfirmActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Landroid/accounts/AccountManagerService;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/IMediaPlaybackService;-setQueuePosition-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler;-<init>-(Lcom/android/contacts/editor/ContactEditorFragment; Landroid/content/Context; Lcom/android/contacts/editor/BaseRawContactEditorView; I Lcom/android/contacts/model/RawContactDeltaList;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-dispatchHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/media/AudioService;-checkAudioSettingsPermission-(Ljava/lang/String;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/display/DisplayManagerService;-handleDisplayDeviceChanged-(Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothA2dp;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Utils;-getLocalProfileGivenName-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/calendar/alerts/AlertActivity$1;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Browser;-getAllBookmarks-(Landroid/content/ContentResolver;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/CommandsInterface;-hangupForegroundResumeBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame$1;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/RuimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Browser;-deleteFromHistory-(Landroid/content/ContentResolver; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-addPhoneStateListener-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiStateMachine$SoftApStoppingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/accounts/AccountPreferenceBase;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/power/PowerManagerService;-setMaximumScreenOffTimeoutFromDeviceAdminInternal-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/NotificationPlayer$CmdThread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-removeNotification-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onDataSetupComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-handleEnterEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-confirmSendMessageIfNeeded-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$1;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-prev-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/DisplayPowerController$Callbacks;-onStateChanged-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/GoogleAccountLogin$1;-onUnhandledKeyEvent-(Landroid/webkit/WebView; Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/WaveView;-setGrabbedState-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView;-onFinishInflate-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/ExpandableDictionary;-getSuggestionsWithSessionId-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; I)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/event/EditEventView;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/app/IBatteryStats;-getAwakeTimeBattery-()J' : ['android.permission.BATTERY_STATS'],
	'Landroid/content/Context;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager$Callbacks;-onRemoveViewAnimationCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-deleteAccountPIMData-(J)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/LocationManagerService;-access$1300-(Lcom/android/server/LocationManagerService; Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SlotView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/webkit/WebViewClassic;-documentHasImages-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccCardProxy;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppManager;-disableBluetooth-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-<init>-(Landroid/content/Context; I I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-revertToNitzTimeZone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Landroid/webkit/WebViewClassic;-scrollDraggedSelectionHandleIntoView-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/MessageUtils$4;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/editor/ContactEditorFragment$4;-onLoadFinished-(Landroid/content/Loader; Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-onActionItemClicked-(Landroid/view/ActionMode; Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/filterpacks/base/CallbackFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-processCdmaWapPdu-([B I Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-pollCallsWhenSafe-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub$Proxy;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/calendar/DayView$ContextMenuHandler;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-dispatchMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/RIL;-cancelPendingUssd-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewStateManager;-onPageSwitching-(Landroid/view/View; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewProvider;-clearHistory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-updateDoubleTapZoom-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/BitmapOverlayEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/IConnectivityManager;-getNetworkPreference-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/wm/InputMonitor;-notifyANR-(Lcom/android/server/input/InputApplicationHandle; Lcom/android/server/input/InputWindowHandle;)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-pickAlbum-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/Bookmarks;-removeFromBookmarks-(Landroid/content/Context; Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView$2;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/VignetteEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterfw/core/Filter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-resetTouchState-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ContactPickerFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/AllInOneActivity;-setTitleInActionBar-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/nfc/INfcAdapterExtras$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Landroid/filterpacks/videosrc/MediaSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/CallNotifier;-ignoreAllIncomingCalls-(Lcom/android/internal/telephony/Phone;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/GlobalActions$5;-onLongPress-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/power/Notifier;-sendNextBroadcast-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/sip/SipService;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/browser/TabBar;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/camera/ui/InLineSettingSwitch;-changeIndex-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/soundrecorder/SoundRecorder;-access$100-(Lcom/android/soundrecorder/SoundRecorder;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter;-getSyncKey-()Ljava/lang/String;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/BrowserContract$SyncState;-get-(Landroid/content/ContentProviderClient; Landroid/accounts/Account;)[B' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/providers/downloads/DownloadService$UpdateThread;-run-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebView;-goForward-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/event/EditEventView;-<init>-(Landroid/app/Activity; Landroid/view/View; Lcom/android/calendar/event/EditEventHelper$EditDoneRunnable;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/ExchangeService$2;-updateFolderList-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-disableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-ndefMakeReadOnly-(I)I' : ['android.permission.NFC'],
	'Lcom/android/camera/CameraPreference$OnPreferenceChangedListener;-onSharedPreferenceChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/ConnectivityService;-enforceTetherAccessPermission-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetooth;-startDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/am/ActivityManagerService;-inputDispatchingTimedOut-(I Z)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/app/IBatteryStats;-getAwakeTimePlugged-()J' : ['android.permission.BATTERY_STATS'],
	'Lcom/android/browser/Tab;-setWebView-(Landroid/webkit/WebView; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/HTML5VideoViewProxy$VideoPlayer;-onPrepared-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/SendTransaction;-notifyObservers-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/browser/Tab$5;-onNewHistoryItem-(Landroid/webkit/WebHistoryItem;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-access$1300-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/browser/WebViewTimersControl;-onPrerenderDone-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/Ringer$VibratorThread;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/settings/bluetooth/BluetoothDeviceFilter$PanuFilter;-matches-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/videosrc/CameraSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-handleMove-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-getMessageItem-(Ljava/lang/String; J Z)Lcom/android/mms/ui/MessageItem;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-sendCdmaSms-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/MessagingNotification;-notifySendFailed-(Landroid/content/Context;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/gallery3d/ui/ImportCompleteListener;-onProgressComplete-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/quicklaunch/QuickLaunchSettings;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-createLoaderUri-()Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/view/IApplicationToken$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-createIntent-(Landroid/content/Context; J)Landroid/content/Intent;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$4;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/BluetoothManagerService;-handleEnable-(Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-disable-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/transaction/ReadRecTransaction;-sendPdu-([B Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-notifyOffApnsOfAvailability-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/data/WorkingMessage;-readDraftSmsMessage-(Lcom/android/mms/data/Conversation;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/TetherSettings;-updateUsbState-([Ljava/lang/String; [Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumDataLoader;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/media/IAudioService$Stub$Proxy;-setBluetoothA2dpDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice; I)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ThrottleService$SettingsObserver;-register-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/vcard/ImportVCardActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityManagerService;-closeSystemDialogsLocked-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/providers/downloads/DownloadService$1;-scanCompleted-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendCDMAFeatureCode-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService$Stub;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/contacts/ContactPhotoManagerImpl$LoaderThread;-loadPhotosInBackground-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/calllog/CallLogFragment;-fetchCalls-()V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/accounts/AccountManager$6;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/sip/SipManager;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/browser/Controller;-closeEmptyTab-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-doSaveAction-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/model/dataitem/NoteDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ActivityController;-onDestroy-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ThrottleService$MyHandler;-onRebootRecovery-()V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/webkit/HTML5VideoFullScreen;-reprepareData-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/usb/UsbResolverActivity;-onCreate-(Landroid/os/Bundle; Landroid/content/Intent; Ljava/lang/CharSequence; [Landroid/content/Intent; Ljava/util/List; Z)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/providers/media/MtpService;-updateDisabledStateLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CsimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-processMotionEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/preferences/FontSizePreview;-onBindView-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/SizeClustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/EditText;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-validate-(Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/model/AccountTypeManager;-createAccountTypeManager-(Landroid/content/Context;)Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/launcher2/BubbleTextView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$3800-(Lcom/android/mms/ui/ComposeMessageActivity; J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/app/IBackupAgent$Stub;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-stopDtmf-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ITelephony$Stub;-disableDataConnectivity-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/calendar/DayView;-init-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/nfc/handover/HandoverManager;-tryHandoverRequest-(Landroid/nfc/NdefMessage;)Landroid/nfc/NdefMessage;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/SecureAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/wifi/WifiSettings$3;-onClick-(Landroid/view/View;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/nfc/tech/NfcA;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/contacts/model/RawContactDeltaList;-buildSplitContactDiff-(Ljava/util/ArrayList; I I [I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/InCallScreen$11;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/gallery3d/data/LocalAlbumSet$AlbumsLoader;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/transaction/SmsMessageSender;-getOutgoingServiceCenter-(J)Ljava/lang/String;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/phone/BluetoothPhoneService;-sendClccResponseGsm-(I Lcom/android/internal/telephony/Connection;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/launcher2/Launcher;-skipCustomClingIfNoAccounts-()Z' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/ActivityState;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/webkit/WebViewCore;-webkitDraw-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/voicedialer/RecognizerEngine;-onRecognitionSuccess-(Lcom/android/voicedialer/RecognizerClient;)V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/nfc/handover/HandoverService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/util/AccountsListAdapter;-<init>-(Landroid/content/Context; Lcom/android/contacts/util/AccountsListAdapter$AccountListFilter; Lcom/android/contacts/model/account/AccountWithDataSet;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/phone/QuickSettingsModel;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/providers/contacts/GlobalSearchSupport$SearchSuggestion;-addColumnValue-(Ljava/util/ArrayList; Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/LegacyPhoneNumberListAdapter;-bindQuickContact-(Lcom/android/contacts/list/ContactListItemView; I Landroid/database/Cursor; I I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DataUsageSummary$10;-onLoadFinished-(Landroid/content/Loader; Lcom/android/settings/net/ChartData;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/speech/SpeechRecognizer;-access$000-(Landroid/speech/SpeechRecognizer; Landroid/content/Intent;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-removeTrack-(J)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/launcher2/PagedView;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processAtCpbr-(Ljava/lang/String; I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ComposeMessageActivity$41;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/SIMRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-onStart-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/MtpDevice;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiManager;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/browser/Bookmarks;-queryCombinedForUrl-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/list/LegacyPhoneNumberListAdapter;-getPhoneUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/launcher2/Workspace;-startDrag-(Lcom/android/launcher2/CellLayout$CellInfo;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/camera/CameraActivity;-onCameraSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/provider/Contacts$People;-createPersonInMyContactsGroup-(Landroid/content/ContentResolver; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/ContentResolver;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverServer;-start-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ActivityBase$MyAppBridge;-switchWithCaptureAnimation-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccCardProxy;-changeIccLockPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/UiModeManagerService;-sendConfigurationLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/data/Contact;-getByPhoneUris-([Landroid/os/Parcelable;)Ljava/util/List;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-connectWithoutResettingTimer-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-startSample-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/PieControl$1;-onLayout-(I I Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/base/FrameStore;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/net/ConnectivityManagerCompat$GingerbreadConnectivityManagerCompatImpl;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-onDestroy-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-rejectCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/HeadsetProfile$HeadsetServiceListener;-onServiceConnected-(I Landroid/bluetooth/BluetoothProfile;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/provider/ContactsContract$Groups$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/DayFragment;-goTo-(Landroid/text/format/Time; Z Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/editor/SplitContactConfirmationDialogFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/backup/IBackupManager$Stub$Proxy;-opComplete-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/accounts/AccountManager$8;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebViewProvider$ScrollDelegate;-computeScroll-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WifiService;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/webkit/WebViewClassic;-copySelection-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/MtpDevice;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService;-getAllowedResolutionLevel-(I I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/BackupManagerService$PerformRestoreTask;-finalizeRestore-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl$FindInvitablesTask;-doInBackground-([Ljava/lang/Void;)Ljava/util/Map;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/fuelgauge/PowerUsageSummary;-load-()V' : ['android.permission.BATTERY_STATS', 'android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment$1;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/RespondViaSmsManager$RespondViaSmsCancelListener;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/tech/NfcV;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/server/WiredAccessoryManager;-access$700-(Lcom/android/server/WiredAccessoryManager; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$NfcAdapterExtrasService;-_open-(Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-getDictionary-()[B' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIccCardStatus-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/Editor;-onFieldChanged-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/TrimVideo;-access$100-(Lcom/android/gallery3d/app/TrimVideo;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/CameraActivity;-closeModule-(Lcom/android/camera/CameraModule;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$SocketAcceptThread;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/IntentHandler$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/webkit/WebViewClassic$1;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/FrameSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/StateMachine$SmHandler$QuittingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/PrivacySettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/RecognitionManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewClassic;-selectText-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-killAllBackgroundProcesses-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/server/wm/WindowManagerService;-reconfigureDisplayLocked-(Lcom/android/server/wm/DisplayContent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/media/Ringtone;-setStreamType-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/detail/ContactDetailFragment$3;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/SearchActivity;-search-(Ljava/lang/String; Landroid/text/format/Time;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/WebViewClassic;-onScrollChanged-(I I I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/wifi/AdvancedWifiSettings;-onResume-()V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine;-sendRssiChangeBroadcast-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/calllog/ContactInfoHelper;-lookupContactFromUri-(Landroid/net/Uri;)Lcom/android/contacts/calllog/ContactInfo;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/ManageSimMessages;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity$QueryEntitiesTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/UiController;-loadUrl-(Lcom/android/browser/Tab; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager;-setCountryCode-(Ljava/lang/String; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-getSuggestionsWithSessionId-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; I)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardMessageArea;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/ActivityManagerNative;-moveTaskBackwards-(I)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/webkit/WebViewClassic;-onHandleUiTouchEvent-(Landroid/view/MotionEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/DTMFTwelveKeyDialer;-onDialerOpen-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-detectAndSendKey-(Lcom/android/inputmethod/keyboard/Key; I I J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/google/common/io/InputSupplier;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/view/IWindowManager$Stub$Proxy;-clearForcedDisplayDensity-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasOutboxService;-validate-(Ljava/lang/Class; Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mms/data/WorkingMessage;-access$800-(Lcom/android/mms/data/WorkingMessage; Lcom/android/mms/data/Conversation;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/OutputStreamTarget;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/ShowOrCreateActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/ComboAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setPairingConfirmation-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-getInstance-(Landroid/content/Context;)Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/location/ILocationManager$Stub;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManagerService;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/cat/CatService;-handleCommand-(Lcom/android/internal/telephony/cat/CommandParams; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/providers/downloads/SystemFacade;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/common/widget/CompositeCursorAdapter;-bindView-(Landroid/view/View; I Landroid/database/Cursor; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-onTouch-(Landroid/view/View; Landroid/view/MotionEvent;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/btservice/BondStateMachine$PendingCommandState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-getLastCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/WallpaperHandler;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.SET_WALLPAPER', 'android.permission.INTERNET'],
	'Landroid/net/sip/ISipService$Stub;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Landroid/accounts/AccountManager$4;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-requestChildFocus-(Landroid/view/View; Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/quicksearchbox/util/SingleThreadNamedTaskExecutor$Worker;-loop-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/AccelerometerListener;-setOrientation-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-postEventFromNative-(Ljava/lang/Object; I I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/Connectivity;-access$1500-(Lcom/android/development/Connectivity;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/SecureSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-access$2300-(Lcom/android/contacts/editor/ContactEditorFragment; J Landroid/graphics/Bitmap; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/transaction/ReadRecTransaction;-sendPdu-(J [B Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/webkit/AccessibilityInjector$CallbackHandler;-access$500-(Landroid/webkit/AccessibilityInjector$CallbackHandler; Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-turnOffUpstreamMobileConnection-()Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/view/PieMenu;-layoutPieView-(Lcom/android/browser/view/PieMenu$PieView; I I F)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/BootReceiver;-access$100-(Lcom/android/server/BootReceiver; Landroid/content/Context;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/mms/ui/AudioAttachmentView$2;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/ProfileProvider;-updateInTransaction-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_PROFILE'],
	'Lcom/android/settings/ProxySelector$1;-onClick-(Landroid/view/View;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/calendar/agenda/AgendaListView;-refresh-(Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/net/sip/SipAudioCall;-stopCall-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarUtils$TimeZoneUtils;-formatDateRange-(Landroid/content/Context; J J I)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/AddBookmarkPage;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/transaction/NotificationTransaction;-sendPdu-([B Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/net/IConnectivityManager;-getTetherableUsbRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/WifiService;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/NfcService$TagService;-ndefWrite-(I Landroid/nfc/NdefMessage;)I' : ['android.permission.NFC'],
	'Lcom/android/exchange/ExchangeService;-canSyncEmail-(Landroid/accounts/Account;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/DataUsageSummary$1;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-getLastKnownLocation-()Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/CharStreams$2;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/server/WallpaperManagerService$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-startFreezingDisplayLocked-(Z I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$AutoRegistrationProcess;-onError-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-isContactWritable-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/content/ContentService;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/contacts/model/RawContactDelta;-buildDiff-(Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/view/WindowManagerImpl;-addView-(Landroid/view/View; Landroid/view/ViewGroup$LayoutParams;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/bluetooth/BluetoothAdapter;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$WifiSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/model/dataitem/PhoneDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/camera/SecureCameraActivity;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/Settings$DataUsageSummaryActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/CallDetailActivity$6;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/ui/CropView$FaceHighlightView;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-handleNotificationError-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification; Ljava/lang/String;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/AccountUnlockScreen;-onTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/net/sip/SipAudioCall;-releaseWifiHighPerfLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManager;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/exchange/adapter/CalendarSyncAdapter;-<init>-(Lcom/android/exchange/EasSyncService;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/music/MediaPlaybackActivity$7;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onDataStateChanged-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-userSwitched-(I)V' : ['android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/commands/bmgr/Bmgr;-doRestoreAll-(J Ljava/util/HashSet;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-trySetupData-(Lcom/android/internal/telephony/ApnContext;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/dreams/phototable/PhotoTable$PhotoLaunchTask;-doInBackground-([Ljava/lang/Void;)Landroid/view/View;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler;-<init>-(Lcom/android/contacts/activities/PhotoSelectionActivity; Landroid/content/Context; Landroid/view/View; I Lcom/android/contacts/model/RawContactDeltaList;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-onClose-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/vcard/ImportProcessor;-onEntryCreated-(Lcom/android/vcard/VCardEntry;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/vcard/VCardParser;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/event/EditEventFragment;-onPause-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/activities/GroupDetailActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/PhoneGlobals;-access$1100-(Lcom/android/phone/PhoneGlobals;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Ljava/net/Socket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Landroid/net/IConnectivityManager$Stub;-getActiveNetworkQuotaInfo-()Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/vcard/VCardParser_V40;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$1400-(Lcom/android/mms/ui/ComposeMessageActivity; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/LocalAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-access$200-(Lcom/android/internal/policy/impl/PhoneWindowManager;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter$QueryHandler$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-checkPermissionsAndCreateUriData-(Landroid/net/Uri; [Landroid/content/ContentValues;)Lcom/android/providers/contacts/VoicemailContentProvider$UriData;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/gallery3d/app/AlbumPage;-access$1700-(Lcom/android/gallery3d/app/AlbumPage; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-onQueryCfComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/RawContactDeltaList;-buildDiff-()Ljava/util/ArrayList;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-getLastDialledNumber-()Ljava/lang/String;' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaObject-(Ljava/lang/String;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/commands/svc/DataCommand;-run-([Ljava/lang/String;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/UndoBarView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/os/IVibratorService$Stub$Proxy;-vibrate-(J Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/commands/pm/Pm;-runTrimCaches-()V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-gotoIdleAndNotifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView$KeyTimerHandler;-startLongPressTimer-(Lcom/android/inputmethod/keyboard/PointerTracker;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/util/Recycler$MmsRecycler;-anyThreadOverLimit-(Landroid/content/Context;)Z' : ['android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/PhotoPage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/ui/SlideView;-setAudio-(Landroid/net/Uri; Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$GpsStateTracker$1;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/PreloadController;-onPageStarted-(Lcom/android/browser/Tab; Landroid/webkit/WebView; Landroid/graphics/Bitmap;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-checkIfBootAnimationFinished-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/users/UserSettings;-getProfileName-()Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/InputMethodManagerService;-showInputMethodMenuInternal-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WifiService;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;-onDeviceDisappeared-(Lcom/android/settings/bluetooth/CachedBluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-start-()V' : ['android.permission.VIBRATE', 'android.permission.DISABLE_KEYGUARD', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothHeadset;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/activities/PeopleActivity;-onBackPressed-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-setGsmBroadcastConfig-([Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5Audio;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidWordLevelSpellCheckerSession;-onGetSuggestions-(Landroid/view/textservice/TextInfo; I)Landroid/view/textservice/SuggestionsInfo;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/accounts/IAccountManager;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter;-access$500-(Lcom/android/gallery3d/app/SlideshowDataAdapter;)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewInputDispatcher;-enqueueWebKitCancelTouchEventIfNeededLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$UserDictionarySettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/NavigationBarPhone;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/app/WallpaperManager;-setBitmap-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCallWaiting-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-setDataSource-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/SipBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.USE_SIP'],
	'Landroid/content/ContentService;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mms/util/Recycler$SmsRecycler;-getAllThreads-(Landroid/content/Context;)Landroid/database/Cursor;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Landroid/widget/QuickContactBadge$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/widget/LockPatternView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaMmiCode;-processCode-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/MediaSource;-pauseVideo-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-access$100-(Lcom/android/gallery3d/ui/ActionModeHandler; Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/dreams/phototable/PhotoSource$ImageData;-getStream-(I)Ljava/io/InputStream;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-onScreenTurnedOn-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager$ShowListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeBarringPassword-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-resetCaretTimer-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/SlideViewInterface;-stopAudio-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebView;-stopLoading-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothHeadset;-stopScoUsingVirtualVoiceCall-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/voicedialer/VoiceDialerActivity$CommandRecognizerClient$2;-run-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-<init>-(Landroid/content/Context; Lcom/android/internal/widget/LockPatternUtils;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-computePanoramaSharingIntent-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Landroid/content/Intent;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebView;-reload-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/VibratorService;-access$300-(Lcom/android/server/VibratorService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-updateThisDevice-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/DataConnection$DcDisconnectionErrorCreatingConnection;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/PhoneFavoriteFragment;-onStart-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame$7;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-getContactUriForPhoneNumber-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/GroupDetailActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/MediaPlaybackService;-gotoNext-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-access$2300-(Lcom/android/server/power/PowerManagerService; Lcom/android/server/power/PowerManagerService$WakeLock;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/net/DataUsageMeteredSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleInCallMmiCommands-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$DevelopmentSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$1200-(Lcom/android/server/net/NetworkPolicyManagerService; Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/power/Notifier;-access$700-(Lcom/android/server/power/Notifier;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/Conversation;-get-(Landroid/content/Context; Lcom/android/mms/data/ContactList; Z)Lcom/android/mms/data/Conversation;' : ['android.permission.READ_SMS'],
	'Lcom/android/mms/transaction/NotificationTransaction;-run-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/data/ComboSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getBondedDevices-()[Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/calllog/ClearCallLogDialog$1$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/filterpacks/text/ToUpperCase;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mms/transaction/RetryScheduler;-isConnected-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/nfc/P2pLinkManager;-enableDisable-(Z Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$GpsStateTracker$1;-onPostExecute-(Ljava/lang/Boolean;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/launcher2/Launcher;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.VIBRATE'],
	'Landroid/webkit/HTML5Audio;-play-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-access$1200-(Lcom/android/bluetooth/pbap/BluetoothPbapService;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$1600-(Lcom/android/gallery3d/ui/PhotoView; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/detail/ContactLoaderFragment$1;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/Controller;-pauseWebViewTimers-(Lcom/android/browser/Tab;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/NfcService$TagService;-getTechList-(I)[I' : ['android.permission.NFC'],
	'Lcom/android/browser/Tab$SubWindowClient;-doUpdateVisitedHistory-(Landroid/webkit/WebView; Ljava/lang/String; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/nfc/NfcService$TagService;-connect-(I I)I' : ['android.permission.NFC'],
	'Landroid/location/LocationManager;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService;-onSetA2dpConnectionState-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-doRecovery-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-showBackupSecurityScreen-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebView;-loadDataWithBaseURL-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/SlideshowView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/speech/IRecognitionService;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/browser/NavigationBarPhone;-onAction-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/hid/HidService;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/SoundSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/AccountsChangedReceiver$DeleteRemovedAccounts;-delete-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/Settings$DateTimeSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/DataConnection$DcActiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/googlecode/mp4parser/authoring/adaptivestreaming/PackageWriter;-write-(Lcom/googlecode/mp4parser/authoring/Movie;)V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/TrimVideo;-playVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/Settings$AccountSyncSettingsInAddAccountActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/PhotoView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/nfc/tech/NfcF;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/server/am/BroadcastQueue;-performReceiveLocked-(Lcom/android/server/am/ProcessRecord; Landroid/content/IIntentReceiver; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/alerts/DismissAlarmsService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/WebViewCore$EventHub;-sendMessageAtFrontOfQueue-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$InactiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator$1;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/data/ComboSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/SyncManager;-onUserStarting-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/contacts/editor/LabeledEditorView$1;-onItemSelected-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/tech/NfcA;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/browser/PreloadController;-onPageFinished-(Lcom/android/browser/Tab;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/calendar/CalendarAlarmManager;-acquireScheduleNextAlarmWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/PostalAddressListAdapter;-getContactUri-(I Landroid/database/Cursor; I I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/activities/ContactEditorActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/server/pm/PackageManagerService;-scanDirLI-(Ljava/io/File; I I J)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onResetDone-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/RuimSmsInterfaceManager;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/settings/CryptKeeper$ValidationTask;-onPostExecute-(Ljava/lang/Boolean;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler$PhotoEditorListener;-onUseAsPrimaryChosen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/InputMethodManagerService;-resetAllInternalStateLocked-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/MessagingNotification;-getNewMessageNotificationInfo-(Landroid/content/Context; Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J J Landroid/graphics/Bitmap; Lcom/android/mms/data/Contact; I)Lcom/android/mms/transaction/MessagingNotification$NotificationInfo;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/app/ApplicationPackageManager;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onEnableApn-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/WebViewTimersControl;-maybePauseTimers-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/LomoishEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/DeliveryReportActivity;-getSmsReportItems-()Ljava/util/List;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/RIL;-setGsmBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-onStart-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/phone/InCallScreen;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/gallery3d/ui/EdgeView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/google/android/mms/util/PduCache;-put-(Ljava/lang/Object; Ljava/lang/Object;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/content/pm/IPackageManager;-getPackageGids-(Ljava/lang/String;)[I' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/providers/contacts/GlobalSearchSupport$SearchSuggestion;-buildUri-()Ljava/lang/String;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/WifiStateMachine$DriverFailedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/VibratorService;-startNextVibrationLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/filterpacks/base/InputStreamSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MoviePlayer;-pauseVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStack;-moveTaskToFrontLocked-(Lcom/android/server/am/TaskRecord; Lcom/android/server/am/ActivityRecord; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$DriverStartedState;-enter-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothDevice;-createInsecureRfcommSocketToServiceRecord-(Ljava/util/UUID;)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/activities/ActionBarAdapter;-onClose-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/CursorEntityIterator;-getEntityAndIncrementCursor-(Landroid/database/Cursor;)Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/development/Connectivity;-onRoutedRequest-(I)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/browser/preferences/AccessibilityPreferencesFragment;-onResume-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/phone/StatusBarWindowView;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/nfc/tech/NdefFormatable;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/quicksearchbox/SuggestionCursorProvider;-getSuggestions-(Ljava/lang/String; I Z)Lcom/android/quicksearchbox/SuggestionCursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/vcard/ImportVCardActivity$VCardCacheThread;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimPukUnlockScreen$1$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/vcard/SelectAccountActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/common/widget/GroupingListAdapter;-onContentChanged-()V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/settings/fuelgauge/PowerUsageSummary;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BATTERY_STATS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/BluetoothDeviceFilter$NapFilter;-matches-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/AccessibilitySettings$ToggleSwitch;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-isBluetoothAudioConnectedOrPending-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-snapToPageWithVelocity-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/bluetooth/hid/HidService;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerPolicy;-init-(Landroid/content/Context; Landroid/view/IWindowManager; Landroid/view/WindowManagerPolicy$WindowManagerFuncs;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/location/GpsLocationProvider;-handleUpdateNetworkState-(I Landroid/net/NetworkInfo;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipWakeLock;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/Filter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/AudioPlaybackHandler$MessageLoop;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-generateBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/hid/HidService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifyNdefMessageListeners-(Lcom/android/nfc/dhimpl/NativeNfcTag;)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/calendar/event/EditEventView;-updateHomeTime-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/model/dataitem/EventDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/CallNotifier;-onUnknownConnectionAppeared-(Landroid/os/AsyncResult;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/am/ActivityManagerService$6;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/Settings$StorageSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/am/CoreSettingsObserver;-<init>-(Lcom/android/server/am/ActivityManagerService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-goTo-(J Z Z Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/model/dataitem/ImDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/PhoneGlobals;-clearOtaState-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewCallback;-keyguardDoneDrawing-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/mms/transaction/ReadRecTransaction;-sendPdu-([B)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-bindPhotoHandler-(Lcom/android/contacts/editor/BaseRawContactEditorView; Lcom/android/contacts/model/account/AccountType; Lcom/android/contacts/model/RawContactDeltaList;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$3;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/ContactsContract$ProfileSyncState;-set-(Landroid/content/ContentProviderClient; Landroid/accounts/Account; [B)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/IWifiManager;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/net/wifi/WifiStateMachine$TetheringState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/SamplingProfilerService;-registerSettingObserver-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewProvider;-loadData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$AndroidBeamSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-disable-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$OnlineState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/AudioService;-startBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/media/AudioManager;-setParameters-(Ljava/lang/String;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl$KeepAliveProcess;-onError-(Landroid/net/sip/ISipSession; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/HeadsetProfile;-<init>-(Landroid/content/Context; Lcom/android/settings/bluetooth/LocalBluetoothAdapter; Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager; Lcom/android/settings/bluetooth/LocalBluetoothProfileManager;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/SearchPanelView;-startAssistActivity-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-notifyAndAcknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebView;-clearSslPreferences-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityStack;-startPausingLocked-(Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/wm/WindowManagerService;-resumeRotationLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/PhotoPage$Model;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/wifi/WifiStateMachine$ConnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/CallbackFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/tech/MifareClassic;-writeBlock-(I [B)V' : ['android.permission.NFC'],
	'Lcom/android/dreams/phototable/PhotoCarousel$PhotoLoadTask;-doInBackground-([Ljava/lang/Void;)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/voicedialer/VoiceDialerActivity;-updateBluetoothParameters-(Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/HistoryItem;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/PhoneFactory;-makeSipPhone-(Ljava/lang/String;)Lcom/android/internal/telephony/sip/SipPhone;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/CallCard;-onImageLoadComplete-(I Landroid/graphics/drawable/Drawable; Landroid/graphics/Bitmap; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/AlbumPage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/sip/SipSessionGroup;-processRequest-(Ljavax/sip/RequestEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/variablespeed/VariableSpeed;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/MultiStateButton;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewInputDispatcher;-enqueueWebKitEventUnbatchedLocked-(Landroid/webkit/WebViewInputDispatcher$DispatchEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/SlideshowPage$SequentialSource;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/FetchUrlMimeType;-run-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewStateManager;-showBouncer-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothInputDevice;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ManageSimMessages;-deleteFromSim-(Landroid/database/Cursor;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/net/nsd/INsdManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.INTERNET'],
	'Landroid/view/WindowManagerPolicy;-dispatchUnhandledKey-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.CALL_PHONE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppManager;-getDeviceName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserWebView;-clearCache-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/handover/HandoverManager;-createHandoverSelectMessage-(Z)Landroid/nfc/NdefMessage;' : ['android.permission.BLUETOOTH'],
	'Landroid/app/ActivityManagerNative;-inputDispatchingTimedOut-(I Z)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/alerts/AlertReceiver;-makeDigestNotification-(Landroid/content/Context; Ljava/util/ArrayList; Ljava/lang/String; Z)Lcom/android/calendar/alerts/AlertService$NotificationWrapper;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enforceReceiveAndSend-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-fillData-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/WifiService;-setDeviceIdleAndUpdateWifi-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pan/PanService;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/ZoomManager;-startZoomAnimation-(F Z)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/adapter/CalendarSyncAdapter$EasCalendarSyncParser;-commit-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/quicksearchbox/google/AbstractGoogleSource;-getSuggestions-(Ljava/lang/String; I Z)Lcom/android/quicksearchbox/SuggestionCursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewBase;-reset-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/NsdService;-access$700-(Lcom/android/server/NsdService; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/webkit/WebSettingsClassic;-setDoubleTapZoom-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/OpenDownloadReceiver;-onReceiveAsync-(Landroid/content/Context; J)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/content/SyncManager;-access$1400-(Landroid/content/SyncManager; Landroid/content/SyncManager$ActiveSyncContext; Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ConnectivityService;-getAllNetworkInfo-()[Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/IWindowManager$Stub;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-showSecurityScreen-(Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityModel$SecurityMode;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/MenuExecutor$ConfirmDialogListener;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/editor/AggregationSuggestionEngine;-loadAggregationSuggestions-(Landroid/net/Uri;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/inputmethod/keyboard/PointerTracker$TimerProxy;-startLongPressTimer-(Lcom/android/inputmethod/keyboard/PointerTracker;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-explicitCallTransfer-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-onPageFinished-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/editor/EventFieldEditorView;-onOptionalFieldVisibilityChange-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/PrivilegedSmsReceiver;-onReceiveWithPrivilege-(Landroid/content/Context; Landroid/content/Intent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewBase;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/PhoneNumberListAdapter;-bindView-(Landroid/view/View; I Landroid/database/Cursor; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/calendar/CalendarProvider2;-doProcessEventRawTimes-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetherableBluetoothRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/view/IInputMethodCallback$Stub;-finishedEvent-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-reportSmsMemoryStatus-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/SpecialCharSequenceMgr;-handleChars-(Landroid/content/Context; Ljava/lang/String;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/ui/CropView$FaceHighlightView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/Utils;-getMeProfileName-(Landroid/content/Context; Z)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/nfc/tech/IsoDep;-getTimeout-()I' : ['android.permission.NFC'],
	'Lcom/android/gallery3d/app/DialogPicker;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/TelephonyRegistry;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/model/dataitem/NoteDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebViewProvider;-stopLoading-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WifiService;-getScanResults-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityModel;-getBackupSecurityMode-(Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityModel$SecurityMode;)Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityModel$SecurityMode;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothHeadset;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/preferences/AccessibilityPreferencesFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.INTERNET'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-composeAndSendPhonebookVcards-(Ljavax/obex/Operation; I I Z Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/calendar/widget/CalendarAppWidgetModel;-populateEventInfo-(J Z J J I I Ljava/lang/String; Ljava/lang/String; I I)Lcom/android/calendar/widget/CalendarAppWidgetModel$EventInfo;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView$2;-onRefreshCarrierInfo-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebViewClassic;-clearMatches-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/BrowserBookmarksPage$OpenAllInTabsTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/camera/SecureCameraActivity;-reuseCameraScreenNail-(Z)Lcom/android/gallery3d/ui/ScreenNail;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-openCompleteTransfer-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getCallWaiting-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$WifiP2pSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCDMASubscription-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-asyncReloadDictionaryIfRequired-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/WebView;-pauseTimers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothInputDevice;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebViewProvider$ViewDelegate;-setLayerType-(I Landroid/graphics/Paint;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasAccountService;-canProvision-(Lcom/android/exchange/EasSyncService;)Lcom/android/exchange/adapter/ProvisionParser;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-<init>-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/widget/WaveView;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/alerts/AlertService;-processQuery-(Landroid/database/Cursor; Landroid/content/Context; J Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList;)I' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/GeneralPreferences;-getRingtoneTitleFromUri-(Landroid/content/Context; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-performSnooze-(Landroid/net/NetworkTemplate; I)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/inputmethodservice/KeyboardView;-showPreview-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-access$1200-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/systemui/recent/RecentsHorizontalScrollView$3;-onClick-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-updateActivityLockScreenState-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/BluetoothPhoneService;-updateServiceState-(Landroid/telephony/ServiceState;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.CALL_PHONE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothDevice;-createInsecureRfcommSocket-(I)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/net/IConnectivityManager$Stub;-getNetworkPreference-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-makeAppCrashingLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/activities/PeopleActivity$ContactDetailLoaderFragmentListener;-onDetailsLoaded-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getLastPdpFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-getScanResults-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onLocalDisconnect-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManagerProxy;-updateMessageOnIccEf-(I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-initSocket-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/pm/PackageManagerService;-updateExternalMediaStatusInner-(Z Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/grass/GrassView;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/Resources$1;-getInput-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readPbrFileAndWait-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/Observable;-notifyObservers-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/phone/InCallScreen;-openDialpadInternal-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Landroid/provider/Telephony$Sms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/providers/settings/SettingsBackupAgent;-onRestore-(Landroid/app/backup/BackupDataInput; I Landroid/os/ParcelFileDescriptor;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/detail/ContactDetailLayoutController;-initialize-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/contacts/ContactsActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/media/AudioManager;-setMicrophoneMute-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/mms/transaction/NotificationTransaction;-ensureRouteToHost-(Ljava/lang/String; Lcom/android/mms/transaction/TransactionSettings;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/exchange/EasOutboxService;-sendHttpClientPost-(Ljava/lang/String; [B)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/RelationDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/HTML5VideoInline;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onDataSetupComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/calllog/CallLogFragment;-startCallsQuery-()V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/telephony/RIL;-getCDMASubscription-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmReceiver$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-access$400-(Lcom/android/systemui/statusbar/phone/QuickSettings;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAbsKeyInputView;-onResume-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService$2;-startSync-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-killBackgroundProcesses-(Ljava/lang/String; I)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/gallery3d/app/PhotoPage;-access$2400-(Lcom/android/gallery3d/app/PhotoPage; Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onCdmaDataDetached-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Tab;-capture-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DevicePolicyManagerService;-wipeDataLocked-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-deletePackageX-(Ljava/lang/String; I I)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$1;-run-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback;-pokeWakelock-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-updateExtensions-(J Landroid/content/ContentValues;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/accounts/AccountManager$BaseFutureTask;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/test/SyncBaseInstrumentation;-syncProvider-(Landroid/net/Uri; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/settings/Settings$BluetoothSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/app/IBatteryStats$Stub$Proxy;-setBatteryState-(I I I I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-isValidWord-(Ljava/lang/CharSequence;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/calendar/agenda/AgendaFragment;-eventsChanged-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/settings/DevelopmentSettings;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/phone/CdmaVoicePrivacyCheckBoxPreference;-onClick-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/ManageConferenceUtils;-updateManageConferencePanel-(Ljava/util/List;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Landroid/widget/CheckBox;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$DeviceDisappearedHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-snapToPage-(I I I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/voicedialer/VoiceDialerActivity$ErrorRunnable;-run-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/os/BatteryStatsImpl;-setOnBatteryLocked-(Z I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/ContactsContract$Contacts;-getLookupUri-(J Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-handleKeyguardDone-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothHeadset;-getAudioState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/google/common/io/ByteStreams;-copy-(Lcom/google/common/io/InputSupplier; Ljava/io/OutputStream;)J' : ['android.permission.INTERNET'],
	'Landroid/app/IActivityManager;-startRunning-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/VoicemailStatusTable;-update-(Lcom/android/providers/contacts/VoicemailContentProvider$UriData; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/exchange/ExchangeService;-stopAccountSyncs-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$2700-(Lcom/android/mms/ui/ComposeMessageActivity; Landroid/view/ContextMenu; Lcom/android/mms/ui/ComposeMessageActivity$MsgListMenuClickListener; Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/widget/CalendarAppWidgetModel;-buildFromCursor-(Landroid/database/Cursor; Ljava/lang/String;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-handleShow-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/HidProfile;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothDevice;-getBondState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/WaveView;-tryTransitionToStartAttemptState-(Landroid/view/MotionEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/camera/ui/TimeIntervalPopup$2;-onClick-(Landroid/view/View;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onFilmModeChanged-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ims/IsimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/CameraSwitcher;-access$000-(Lcom/android/camera/ui/CameraSwitcher; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/ConnectivityService;-access$1400-(Lcom/android/server/ConnectivityService; Landroid/net/NetworkInfo;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/sip/SipManager;-register-(Landroid/net/sip/SipProfile; I Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/contacts/model/dataitem/PhotoDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5VideoInline;-stopPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DateTimeView;-onAttachedToWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onCancelEvent-(I I J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/bluetooth/BluetoothAdapter;-getBondedDevices-()Ljava/util/Set;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor;-handleSimStateChange-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor$SimArgs;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/mms/data/Conversation;-sameRecipient-(Landroid/net/Uri; Landroid/content/Context;)Z' : ['android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$EventsEntity$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/base/FrameSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DeleteEventHelper;-deleteRepeatingEvent-(I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/calendar/CalendarController;-sendEvent-(Ljava/lang/Object; J Landroid/text/format/Time; Landroid/text/format/Time; J I J Ljava/lang/String; Landroid/content/ComponentName;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/content/pm/IPackageManager$Stub;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/cdma/RuimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$900-(Lcom/android/server/net/NetworkPolicyManagerService;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/BackupManagerService$PerformFullBackupTask;-backupOnePackage-(Landroid/content/pm/PackageInfo; Ljava/io/OutputStream;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/DockObserver;-handleDockStateChange-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-scanPackageLI-(Ljava/io/File; I I J Landroid/os/UserHandle;)Landroid/content/pm/PackageParser$Package;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/btservice/AdapterService;-access$200-(Lcom/android/bluetooth/btservice/AdapterService; Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/camera/CameraModule;-onResumeAfterSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/MoviePlayer$8;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame;-access$1200-(Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/MediaPlaybackActivity;-doPauseResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-createBond-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-access$100-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-stopManualSync-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasOutboxService;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/PeopleActivity$ContactBrowserActionListener;-onViewContactAction-(Landroid/net/Uri;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/launcher2/Workspace;-beginDragShared-(Landroid/view/View; Lcom/android/launcher2/DragSource;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/data/WorkingMessage;-access$200-(Landroid/content/Context; Lcom/android/mms/data/Conversation; Ljava/lang/StringBuilder;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-updateGroups-(J Landroid/content/ContentValues;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/power/PowerManagerService;-access$2000-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothHealth;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/AddBookmarkPage;-addFolderToCurrent-(Ljava/lang/String;)J' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/BrowserWebView;-loadUrl-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$KeyboardLayoutPickerActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/media/NotificationPlayer;-access$700-(Lcom/android/systemui/media/NotificationPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/phone/InCallTouchUi$2;-onAnimationStart-(Landroid/animation/Animator;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/ZoomControlEmbedded$ZoomListener;-onZoom-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetPhoneState;-listenForPhoneState-(Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/StatusBarManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/policy/impl/GlobalActions$SinglePressAction;-onPress-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/widget/RotarySelector;-vibrate-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask$1;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onActionBarWanted-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-getBasebandVersion-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getPreferredVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothPan;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/sip/SipAudioCall;-access$1200-(Landroid/net/sip/SipAudioCall; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DevicePolicyManagerService;-wipeDeviceOrUserLocked-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onSetPolicyDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/bluetooth/BluetoothDiscoverableEnabler;-access$000-(Lcom/android/settings/bluetooth/BluetoothDiscoverableEnabler;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityStack;-completeResumeLocked-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-finish-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-startRecording-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/activities/AttachPhotoActivity$Listener;-onContactLoaded-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-updateContent-(I Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-addProximityAlert-(D D F J Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService;-reportLocation-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewInputDispatcher;-scheduleClickLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/Controller$5;-doInBackground-([Ljava/lang/Void;)[Ljava/lang/String;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/nfc/NfcAdapter;-setNdefPushMessageCallback-(Landroid/nfc/NfcAdapter$CreateNdefMessageCallback; Landroid/app/Activity; [Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-handlePollCalls-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SelectionManager;-deSelectAll-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/browser/BrowserWebView;-performLongClick-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$Listener;-onSingleTapUp-(F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/Tab;-<init>-(Lcom/android/browser/WebViewController; Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/BreadCrumbView;-clear-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/WifiService;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/provider/ContactsContract$Contacts;-getLookupUri-(Landroid/content/ContentResolver; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimUnlockScreen;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-setMonthDisplayed-(Landroid/text/format/Time; Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/WebViewClassic;-switchOutDrawHistory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneProxy;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/OtaUtils;-otaShowHome-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/media/AudioManager;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-updateNotificationWhenBtDisabled-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransfer;-start-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/mms/transaction/Transaction;-process-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/voicedialer/VoiceDialerActivity;-access$2200-(Lcom/android/voicedialer/VoiceDialerActivity; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$UnavailableState;-enter-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/WifiService;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/MediaPlayer;-stayAwake-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebView;-requestFocus-(I Landroid/graphics/Rect;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-updateData-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-createView-()Landroid/view/View;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onCommitDeleteImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/bluetooth/DockService;-handleUnexpectedDisconnect-(Landroid/bluetooth/BluetoothDevice; Lcom/android/settings/bluetooth/LocalBluetoothProfile; I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/Controller;-loadUrl-(Lcom/android/browser/Tab; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity$35;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-getViewAt-(I)Landroid/widget/RemoteViews;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-prepareEmptyView-()V' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/calllog/ContactInfoHelper;-lookupNumber-(Ljava/lang/String; Ljava/lang/String;)Lcom/android/contacts/calllog/ContactInfo;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/wm/WindowManagerService;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/emailcommon/service/IEmailService;-updateFolderList-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$AndroidBeamSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/WifiService;-enforceAccessPermission-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$AccountAnalyzer;-start-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/data/Contact$ContactsCache;-access$600-(Lcom/android/mms/data/Contact$ContactsCache; Lcom/android/mms/data/Contact;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/content/ContentResolver;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/sip/SipWakeLock;-acquire-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService;-access$1600-(Lcom/android/bluetooth/hdp/HealthService; Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/BluetoothDevice; I I Landroid/os/ParcelFileDescriptor; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/am/ActivityManagerService;-handleApplicationWtf-(Landroid/os/IBinder; Ljava/lang/String; Landroid/app/ApplicationErrorReport$CrashInfo;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity;-startPhotoQuery-(J Landroid/net/Uri;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/inputmethod/SpellCheckersSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/PhoneCallDetailsHelper;-setPhoneCallDetails-(Lcom/android/contacts/PhoneCallDetailsViews; Lcom/android/contacts/PhoneCallDetails; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/view/IInputMethodCallback;-finishedEvent-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/calendar/CalendarAlarmManager;-releaseScheduleNextAlarmWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag;-isNdef-(I)Z' : ['android.permission.NFC'],
	'Lcom/android/contacts/editor/LabeledEditorView$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/UiModeManagerService;-updateAfterBroadcastLocked-(Ljava/lang/String; I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/CallLogAsync$AddCallTask;-doInBackground-([Lcom/android/phone/CallLogAsync$AddCallArgs;)[Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/RequestPermissionActivity;-access$200-(Lcom/android/settings/bluetooth/RequestPermissionActivity;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-removeTrack-(J)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity$8;-afterTextChanged-(Landroid/text/Editable;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/sip/SipService;-updateWakeLocks-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/CallbackProxy;-onJsAlert-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/ViewRootImpl;-dispatchImeFinishedEvent-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-access$400-(Lcom/android/internal/telephony/RIL; Landroid/os/Parcel;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab$2;-doUpdateVisitedHistory-(Landroid/webkit/WebView; Ljava/lang/String; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler$PhotoListener;-onRemovePictureChosen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/ContactSaveService;-buildJoinContactDiff-(Ljava/util/ArrayList; J J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/snep/SnepServer$Callback;-doGet-(I Landroid/nfc/NdefMessage;)Lcom/android/nfc/snep/SnepMessage;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/recent/RecentsCallback;-handleSwipe-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Lcom/android/providers/calendar/CalendarProvider2;-deleteInTransaction-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String; Z)I' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/mms/transaction/SendTransaction;-sendPdu-(J [B)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/EventInfoFragment$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/media/effect/effects/RotateEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/google/common/io/Files;-copy-(Ljava/io/File; Ljava/io/OutputStream;)V' : ['android.permission.INTERNET'],
	'Landroid/net/wifi/IWifiManager;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/WaveView;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.VIBRATE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/list/JoinContactListFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/event/EditEventFragment;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/PhoneProxy;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContentService;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView$OnTriggerListener;-onTrigger-(Landroid/view/View; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/BluetoothHeadset;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/sip/SipSettings;-processActiveProfilesFromSipService-()V' : ['android.permission.USE_SIP'],
	'Lcom/android/gallery3d/data/MediaSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/Settings$ManageApplicationsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/test/mock/MockContentResolver;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/ContentResolver;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/location/ILocationManager$Stub;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mms/transaction/SendTransaction;-run-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/month/SimpleWeekView;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/view/inputmethod/InputMethodManager$1;-onBindMethod-(Lcom/android/internal/view/InputBindResult;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/usb/UsbResolverActivity;-onStop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccRecords;-onRecordLoaded-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/location/provider/LocationProviderBase$Service;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/RuimSmsInterfaceManager;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onMoveEvent-(I I J Landroid/view/MotionEvent;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/power/PowerManagerService$DockReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-setForegroundDispatch-(Landroid/app/PendingIntent; [Landroid/content/IntentFilter; Landroid/nfc/TechListParcel;)V' : ['android.permission.NFC'],
	'Lcom/android/calendar/DeleteEventHelper;-delete-(J J J I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/ZonePicker;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/util/State;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/DataUsageSummary;-access$500-(Lcom/android/settings/DataUsageSummary; Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-processCode-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/common/widget/CompositeCursorAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-setName-(Ljava/lang/String;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-dispatchBroadcastMessage-(Landroid/telephony/SmsCbMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/FolderSyncParser;-parse-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewBase;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/ExchangeService$2;-sendMeetingResponse-(J I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover;-nextStepInit-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/PopupList$OnPopupItemClickListener;-onPopupItemClick-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/ILocationManager$Stub;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-access$2500-(Lcom/android/gallery3d/app/PhotoDataAdapter; Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/ui/CropView$HighlightRectangle;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/BrowserHistoryPage$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/ThrottleManager;-getPeriodStartTime-(Ljava/lang/String;)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/os/BatteryStatsImpl;-writeToParcelLocked-(Landroid/os/Parcel; Z I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/ActionImage;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/CharStreams$4;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/server/ConnectivityService$FeatureUser;-<init>-(Lcom/android/server/ConnectivityService; I Ljava/lang/String; Landroid/os/IBinder;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/inputmethod/InputMethodManager$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;-onConfirmDialogDismissed-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/AudioAttachmentView;-startAudio-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/media/NotificationPlayer$CreationAndCompletionThread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-openFile-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothNameDialogFragment;-onCreateDialog-(Landroid/os/Bundle;)Landroid/app/Dialog;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-openPopupIfRequired-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/WifiService;-enforceChangePermission-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/calendar/DayView;-doFling-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-notifyOffApnsOfAvailability-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UrlInputView;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/test/mock/MockContentResolver;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/NetworkPolicyManager;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewClassic;-updateHwAccelerated-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getVoiceRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/GroupEditorActivity$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/BitmapSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/DatabaseModifier;-insert-(Landroid/content/ContentValues;)J' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/gallery3d/data/FilterSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/transaction/MessagingNotification;-blockingUpdateAllNotifications-(Landroid/content/Context; J)V' : ['android.permission.READ_SMS'],
	'Landroid/content/IContentService$Stub$Proxy;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/preferences/WebViewPreview;-setupWebView-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/DayFragment;-makeView-()Landroid/view/View;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/EasAccountService;-hasConnectivity-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$3;-pokeWakelock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$8;-run-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/browser/homepages/RequestHandler;-run-()V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/WapPushOverSms;-dispatchWapPdu-([B)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-startSettingsActivity-(Ljava/lang/String;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/exchange/ExchangeService;-runAwake-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/util/JavaNetHttpHelper;-get-(Lcom/android/quicksearchbox/util/HttpHelper$GetRequest;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/app/IActivityManager;-setProcessLimit-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mms/ui/ConversationListAdapter$OnContentChangedListener;-onContentChanged-(Lcom/android/mms/ui/ConversationListAdapter;)V' : ['android.permission.READ_SMS'],
	'Landroid/net/sip/ISipService$Stub;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Landroid/webkit/WebViewClassic;-sendOurVisibleRect-()Landroid/graphics/Rect;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/alerts/AlertReceiver;-createEmailIntent-(Landroid/content/Context; J Ljava/lang/String;)Landroid/content/Intent;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/Settings$ManageAccountsSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/LocationManagerService;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SyncStateTracker;-getTriState-(Landroid/content/Context;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/TrimVideo;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-drawEvents-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/webkit/WebViewClassic;-access$300-(Landroid/webkit/WebViewClassic;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/DockEventReceiver;-beginStartingService-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-queryFacilityLockForApp-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/DbModifierWithNotification;-update-(Ljava/lang/String; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/settings/quicklaunch/QuickLaunchSettings;-updateShortcut-(C Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-updateAfterRadioTechnologyChange-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/WifiManager;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/BackupManagerService$PerformFullRestoreTask;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onCleanUpAllConnections-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/BackDropperEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-toggleUserAgent-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcAdapterExtras$Stub;-open-(Ljava/lang/String; Landroid/os/IBinder;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Connected;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothDevice;-getAlias-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/ConnectivityService$SettingsObserver;-observe-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/deskclock/AlarmClock$AlarmItemAdapter;-bindExpandArea-(Lcom/android/deskclock/AlarmClock$AlarmItemAdapter$ItemHolder; Lcom/android/deskclock/Alarm;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-resetUserSetupObserver-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getNeighboringCids-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$ApplyRoutingTask;-doInBackground-([Ljava/lang/Integer;)Ljava/lang/Void;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/InCallScreen;-access$900-(Lcom/android/phone/InCallScreen; Lcom/android/internal/telephony/MmiCode;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-reconnect-(I)I' : ['android.permission.NFC'],
	'Lcom/android/phone/InCallScreen;-access$1300-(Lcom/android/phone/InCallScreen;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/contacts/vcard/VCardImportExportListener;-onImportParsed-(Lcom/android/contacts/vcard/ImportRequest; I Lcom/android/vcard/VCardEntry; I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-next-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame$7;-onKeyguardVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/Utils;-getDisplayedDatetime-(J J J Ljava/lang/String; Z Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/Settings$DeviceInfoSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/PhoneUi;-toggleNavScreen-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/view/IInputMethodClient$Stub;-onUnbindMethod-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SyncStateTracker$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Boolean;' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/webkit/WebViewCore$4;-invoke-(Ljava/lang/String; Z Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ITelephony$Stub$Proxy;-call-(Ljava/lang/String;)V' : ['android.permission.CALL_PHONE'],
	'Lcom/android/contacts/calllog/CallLogListItemHelper;-configureCallSecondaryAction-(Lcom/android/contacts/calllog/CallLogListItemViews; Lcom/android/contacts/PhoneCallDetails;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/BluetoothManagerService;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-setupQuickSettings-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/activities/AttachPhotoActivity$2;-onLoadComplete-(Landroid/content/Loader; Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/PluginFullScreenHolder$CustomFrameLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$4600-(Lcom/android/mms/ui/ComposeMessageActivity; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/AccessibilitySettings$SettingsContentObserver;-register-(Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-toggleRecentsActivity-()V' : ['android.permission.GET_TASKS'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/LocationListener; Landroid/os/Looper; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ManageSimMessages;-startQuery-()V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-sendKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/ContactSaveService;-createRawContact-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setMute-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/inputmethod/UserDictionaryList;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/util/ThumbnailManager;-getThumbnail-(Landroid/net/Uri; Z Lcom/android/mms/util/ItemLoadedCallback;)Lcom/android/mms/util/ItemLoadedFuture;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/DataUsageListener;-access$100-(Lcom/android/phone/DataUsageListener;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/AccessibilityInjector$CallbackHandler;-performAction-(Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/FaceUnlock;-handleServiceConnected-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/DownloadManager;-enqueue-(Landroid/app/DownloadManager$Request;)J' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/google/common/io/Files;-readFirstLine-(Ljava/io/File; Ljava/nio/charset/Charset;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/activities/ActionBarAdapter;-onQueryTextChange-(Ljava/lang/String;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$CryptKeeperSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/statusbar/IStatusBarService;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/net/IConnectivityManager;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/DockService;-handleBtStateChange-(Landroid/content/Intent; I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/systemui/media/RingtonePlayer$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/ToUpperCase;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$TetherSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/IConnectivityManager$Stub;-getLinkProperties-(I)Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/calendar/SearchActivity;-deleteEvent-(J J J)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/wifi/WifiAPITest$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/webkit/HTML5VideoViewProxy$VideoPlayer;-enterFullScreenVideo-(I Ljava/lang/String; Landroid/webkit/HTML5VideoViewProxy; Landroid/webkit/WebViewClassic;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/TrimVideoUtils;-startTrim-(Ljava/io/File; Ljava/io/File; I I)V' : ['android.permission.INTERNET'],
	'Landroid/view/ViewRootImpl;-deliverGenericMotionEventPostIme-(Landroid/view/ViewRootImpl$QueuedInputEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/SnapshotTab;-createSubWindow-()Z' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/activities/PeopleActivity$ContactBrowserActionListener;-onFinishAction-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewClassic;-updateWebkitSelection-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-deleteData-(J [Ljava/lang/String;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/accounts/AccountSyncSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/WorkingMessage;-access$000-(Lcom/android/mms/data/WorkingMessage; Lcom/android/mms/data/Conversation;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/wm/WindowManagerService;-shutdown-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/deskclock/DeskClock$TabsAdapter;-registerPageChangedListener-(Lcom/android/deskclock/DeskClockFragment;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-changeIccPin-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/os/BatteryStatsImpl;-writeToParcelWithoutUids-(Landroid/os/Parcel; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/NavTabScroller$2;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/CallController;-placeCall-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/accounts/AccountManagerService;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/calendar/alerts/AlertService;-updateAlertNotification-(Landroid/content/Context;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/phone/PhoneGlobals$MediaButtonBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/wifi/WpsDialog$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/wifi/WifiStateMachine$SoftApStoppingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/DataUsageSummary$UidDetailTask;-bindView-(Lcom/android/settings/net/UidDetailProvider; Lcom/android/settings/DataUsageSummary$AppItem; Landroid/view/View;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/ui/MessageListItem;-bindCommonMessage-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/ui/CropView$HighlightRectangle;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/app/IBackupAgent;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/accounts/AccountManagerService;-checkManageAccountsOrUseCredentialsPermissions-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/systemui/SwipeHelper$Callback;-onChildDismissed-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-onFadeEnd-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterfw/core/FilterGraph;-closeFilters-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-onBackup-(Landroid/os/ParcelFileDescriptor; Landroid/app/backup/BackupDataOutput; Landroid/os/ParcelFileDescriptor;)V' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/phone/PhoneInterfaceManager;-enforceReadPermission-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCallForwardingOption-(I I Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/TextFieldsEditorView$1;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/latin/LatinIME;-updateSuggestionStrip-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/nfc/handover/ConfirmConnectActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/vcard/VCardParserImpl_V21;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-mayStartBatchInput-(Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/inputmethod/latin/spellcheck/DictionaryPool$1;-getSuggestionsWithSessionId-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; I)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/dreams/phototable/StockSource;-fillQueue-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-queryContactInfo-(Ljava/lang/String; Ljava/lang/String; Lcom/android/contacts/calllog/ContactInfo;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/calendar/CalendarAlarmManager;-runScheduleNextAlarm-(Z Lcom/android/providers/calendar/CalendarProvider2;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/detail/ContactDetailLayoutController;-showContactWithUpdates-(Z)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/SnailAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-access$500-(Lcom/android/calendar/event/EditEventView;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/webkit/WebViewCore;-didFirstLayout-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumPage;-clearLoadingBit-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/preference/VolumePreference;-onActivityStop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/NfcB;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/CommandsInterface;-getLastDataCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-onPageStarted-(Lcom/android/browser/Tab; Landroid/webkit/WebView; Landroid/graphics/Bitmap;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/development/Connectivity;-access$1400-(Lcom/android/development/Connectivity;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/bluetooth/BluetoothInputDevice;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/IWifiManager;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/handover/HandoverTransfer;-cancel-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/telephony/TelephonyManager;-getNeighboringCellInfo-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService$2;-onServiceConnected-(I Landroid/bluetooth/BluetoothProfile;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/google/AbstractGoogleSource;-getSuggestionsExternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-access$500-(Lcom/android/calendar/month/MonthByWeekFragment;)Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/location/GpsLocationProvider;-setRequest-(Lcom/android/internal/location/ProviderRequest; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-onScrubbingStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasSyncService;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionRequest;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler;-getWritableEntityId-()J' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/CallNotifier;-onCustomRingtoneQueryTimeout-(Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/quicksearchbox/util/HttpHelper;-get-(Lcom/android/quicksearchbox/util/HttpHelper$GetRequest;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/browser/AddBookmarkPage;-onTop-(Lcom/android/browser/BreadCrumbView; I Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/dreams/phototable/PhotoSource;-<init>-(Landroid/content/Context; Landroid/content/SharedPreferences;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/ManageCachePage$4;-onSingleTapUp-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/DateTimeSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-disable-(Z)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$UserDictionarySettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/PbapServerProfile;-<init>-(Landroid/content/Context;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-setSelectedContactUri-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/InCallScreen;-setInCallScreenMode-(Lcom/android/phone/InCallUiState$InCallScreenMode;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/nfc/NfcService;-enforceNfceeAdminPerm-(Ljava/lang/String;)V' : ['android.permission.NFC'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Landroid/webkit/WebViewInputDispatcher;-setWebKitWantsTouchEvents-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/mms/data/ContactList;-blockingGetByUris-([Landroid/os/Parcelable;)Lcom/android/mms/data/ContactList;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DockObserver;-systemReady-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DataConnection;-onConnect-(Lcom/android/internal/telephony/DataConnection$ConnectionParams;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/CallNotifier;-onCustomRingQueryComplete-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Landroid/filterfw/core/InputPort;-transfer-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$SpellCheckersSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/list/JoinContactListFragment;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$Data;-getContactLookupUri-(Landroid/content/ContentResolver; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/sip/SipAudioCall$1;-onCallEstablished-(Landroid/net/sip/SipSession; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-setCallWaiting-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/IAudioService;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/CalendarSettingsActivity$1;-run-()V' : ['android.permission.GET_ACCOUNTS'],
	'Ljava/net/NetworkInterface;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/detail/PhotoSelectionHandler;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/AccessibilitySettings$ToggleGlobalGesturePreferenceFragment;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/DeviceStorageMonitorService;-clearCache-()V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/contacts/calllog/ContactInfoHelper;-queryContactInfoForSipAddress-(Ljava/lang/String;)Lcom/android/contacts/calllog/ContactInfo;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewBase;-show-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/media/RingtonePlayer$1;-play-(Landroid/os/IBinder; Landroid/net/Uri; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/AlertReceiver;-makeExpandingNotification-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J J J I Z I)Lcom/android/calendar/alerts/AlertService$NotificationWrapper;' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStack;-checkReadyForSleepLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/interactions/ContactDeletionInteraction;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/LocationManagerService;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-registerListener-(Ljava/lang/String; Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-registerAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/IBluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/content/SyncManager$ActiveSyncContext;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onCleanUpAllConnections-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowManager$Stub;-setForcedDisplaySize-(I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/DialogPicker;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/power/PowerManagerService;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/SyncRunner;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/sip/ISipService;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/webkit/AccessibilityInjector;-sendActionToAndroidVox-(I Landroid/os/Bundle;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/DriverCall; Lcom/android/internal/telephony/gsm/GsmCallTracker; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/NonPhoneActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/InCallScreen;-access$500-(Lcom/android/phone/InCallScreen; Landroid/os/AsyncResult;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/provider/BrowserProvider2;-updateInTransaction-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String; Z)I' : ['android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/launcher2/PagedViewWithDraggableItems;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.VIBRATE'],
	'Ljava/net/DatagramSocket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Landroid/net/sip/SipManager;-takeAudioCall-(Landroid/content/Intent; Landroid/net/sip/SipAudioCall$Listener;)Landroid/net/sip/SipAudioCall;' : ['android.permission.USE_SIP'],
	'Lcom/android/browser/Bookmarks;-addBookmark-(Landroid/content/Context; Z Ljava/lang/String; Ljava/lang/String; Landroid/graphics/Bitmap; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-syncClirSetting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-getHealthDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-rebindEditorsForNewContact-(Lcom/android/contacts/model/RawContactDelta; Lcom/android/contacts/model/account/AccountWithDataSet; Lcom/android/contacts/model/account/AccountWithDataSet;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/HeadsetProfile;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-bindEditorsForNewContact-(Lcom/android/contacts/model/account/AccountWithDataSet; Lcom/android/contacts/model/account/AccountType;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/HTML5VideoView;-pauseAndDispatch-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerPolicy;-screenTurnedOff-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$StorageUseActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/DataConnection$DcActivatingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-onGet-(Ljavax/obex/Operation;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/contacts/list/AccountFilterActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Landroid/net/sip/SipAudioCall$1;-onCallEnded-(Landroid/net/sip/SipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BaseUi;-onBackKey-()Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/DeviceStorageMonitorService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/launcher2/Launcher;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.SET_WALLPAPER_HINTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/CallNotifier;-showMissedCallNotification-(Lcom/android/internal/telephony/Connection; J)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/bluetooth/HidProfile;-<init>-(Landroid/content/Context; Lcom/android/settings/bluetooth/LocalBluetoothAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/NetworkStatsService;-access$600-(Lcom/android/server/net/NetworkStatsService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-getLastDataCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-showLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/nfc/INfcTag;-setTimeout-(I I)I' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/MediaItem;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-generateBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/activities/PeopleActivity;-updateViewConfiguration-(Z)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/ContactsUtils;-areGroupWritableAccountsAvailable-(Landroid/content/Context;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/LocationManagerService;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/browser/ActivityController;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/providers/media/RingtonePickerActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/bluetooth/btservice/ProfileService;-cleanup-()Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/ActivityManager;-moveTaskToFront-(I I)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/settings/Settings$LocationSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityManagerService;-killPackageProcessesLocked-(Ljava/lang/String; I I I Z Z Z Z Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/activities/ActionBarAdapter;-update-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$RunningServicesActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/LatinIME;-access$500-(Lcom/android/inputmethod/latin/LatinIME; Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/app/ActivityManagerNative;-setLockScreenShown-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MoviePlayer$AudioBecomingNoisyReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$SecuritySettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/IntentHandler;-onNewIntent-(Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/contacts/model/dataitem/StructuredNameDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter;-postSuccessfulPrepareActions-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/nfc/NfcService$OpenSecureElement;-binderDied-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipAudioCall;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewInputDispatcher;-scheduleUiDispatchLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneProxy;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/CallbackProxy;-openFileChooser-(Ljava/lang/String; Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/BookmarkUtils;-getBookmarksUri-(Landroid/content/Context;)Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCallWaiting-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller$9;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-destroyRenderer-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/BackupManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-open-([J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardStatusView$1;-onBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-hangupConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-updateNotificationsLocked-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/latin/LatinIME;-commitCurrentAutoCorrection-(Ljava/lang/String;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/bluetooth/PanProfile;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/ContactSaveService;-deleteGroup-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPin-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/LatinIME;-getSuggestedWords-(I)Lcom/android/inputmethod/latin/SuggestedWords;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifyLlcpLinkActivation-(Lcom/android/nfc/dhimpl/NativeP2pDevice;)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-deleteData-(Ljava/lang/String; [Ljava/lang/String; Z)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/text/StringSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoSource;-fillQueue-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-resetStateLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/vcard/VCardEntry$AndroidCustomData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-onSplitContactConfirmed-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/UiccCardApplication;-update-(Lcom/android/internal/telephony/IccCardApplicationStatus; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CompoundButton;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/TransactionService;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DayView;-access$4500-(Lcom/android/calendar/DayView; Landroid/view/MotionEvent;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$3;-keyguardDone-(Z)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/content/IContentService$Stub$Proxy;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/Invert;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/fuelgauge/PowerUsageSummary;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ComposeMessageActivity$31;-onResizeResult-(Lcom/google/android/mms/pdu/PduPart; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/webkit/WebViewClassic;-setUseMockGeolocation-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/wifi/WifiSettingsForSetupWizardXL;-onBackButtonPressed-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/ConnectivityService;-startLegacyVpn-(Lcom/android/internal/net/VpnProfile;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/EasOutboxService;-sendMessage-(Ljava/io/File; J)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/ByteStreams$5;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/providers/downloads/DownloadInfo;-notifyPauseDueToSize-(Z)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/filterpacks/base/InputStreamSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/RIL;-startDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-dial-(Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-loadDictionaryAsync-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/transaction/NotificationTransaction;-sendNotifyRespInd-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/ui/PhotoView;-resetToFirstPicture-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-getBondedDevices-()Ljava/util/Set;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/DockService;-processMessage-(Landroid/os/Message;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/users/ProfileUpdateReceiver$1;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/ui/ComposeMessageActivity$10;-onCreateContextMenu-(Landroid/view/ContextMenu; Landroid/view/View; Landroid/view/ContextMenu$ContextMenuInfo;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncManager;-access$100-(Landroid/content/SyncManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/location/ILocationManager;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/recent/RecentTasksLoader$2;-run-()V' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$18;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/GLTextureSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/MessageSender;-sendMessage-(J)Z' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/pm/PackageManagerService;-deletePackageLI-(Ljava/lang/String; Landroid/os/UserHandle; Z I Lcom/android/server/pm/PackageManagerService$PackageRemovedInfo; Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processDialCall-(Ljava/lang/String;)V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/music/MediaPlaybackService;-play-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/CallerInfoCache$CacheAsyncTask;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Pending;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/BluetoothManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/location/GpsLocationProvider;-sendMessage-(I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/calendar/CalendarReceiver$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/WebViewClientClassicExt;-shouldOverrideUrlLoading-(Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallback;-onResponse-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService$Stub;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccProvider;-addIccRecordToEf-(I Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/google/common/io/ByteStreams;-toByteArray-(Lcom/google/common/io/InputSupplier;)[B' : ['android.permission.INTERNET'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$UnavailableState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-addNetworkPolicyLocked-(Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-setProfilePriorty-(Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/vcard/VCardEntryHandler;-onEntryCreated-(Lcom/android/vcard/VCardEntry;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$GpsStateTracker;-setImageViewResources-(Landroid/content/Context; Landroid/widget/RemoteViews;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/IWifiManager$Stub;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/SyncManager$8;-onSyncRequest-(Landroid/accounts/Account; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/MusicPicker;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAccountView;-onClick-(Landroid/view/View;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/hdp/HealthService;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$3900-(Lcom/android/mms/ui/ComposeMessageActivity; J Z)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/cdma/RuimSmsInterfaceManager;-getAllMessagesFromIccEf-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/calendar/agenda/AgendaFragment;-search-(Ljava/lang/String; Landroid/text/format/Time;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/os/PowerManager$WakeLock;-acquireLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/homepages/HomeProvider;-openFile-(Landroid/net/Uri; Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/Tab;-saveViewState-(Landroid/content/ContentValues;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-access$1800-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/CdmaVoicePrivacyCheckBoxPreference;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothDevicePreference;-getConnectionSummary-()I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-pullVcardListing-([B Lcom/android/bluetooth/pbap/BluetoothPbapObexServer$AppParamValue; Ljavax/obex/HeaderSet; Ljavax/obex/Operation;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/gallery3d/app/DialogPicker;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/bluetooth/BluetoothDiscoverableEnabler;-handleModeChanged-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-removeAllDevices-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/NavScreen$TabAdapter$1;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/videosrc/MediaSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/widget/CalendarAppWidgetProvider;-performUpdate-(Landroid/content/Context; Landroid/appwidget/AppWidgetManager; [I [J)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/webkit/WebViewInputDispatcher;-enqueueUiEventUnbatchedLocked-(Landroid/webkit/WebViewInputDispatcher$DispatchEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-setPostDialState-(Lcom/android/internal/telephony/Connection$PostDialState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasAccountService;-validate-(Ljava/lang/Class; Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.READ_CALENDAR'],
	'Landroid/view/IWindowManager;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/settings/Settings$PowerUsageSummaryActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-isForwardable-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/BluetoothAdapter;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView;-access$1100-(Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/support/v4/net/ConnectivityManagerCompatJellyBean;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getOwnerPhoneNumberVcardFromProfile-(Z [B)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCLIR-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/ui/UndoBarView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/webkit/WebViewClassic;-requestFocusNodeHref-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/accounts/AccountManagerService;-checkManageAccountsPermission-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/settings/DevelopmentSettings;-updateAllOptions-()V' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/providers/media/MediaProvider;-openFile-(Landroid/net/Uri; Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.READ_EXTERNAL_STORAGE', 'android.permission.WRITE_EXTERNAL_STORAGE'],
	'Lcom/android/browser/AddBookmarkPage;-onItemSelected-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/fuelgauge/BatteryHistoryPreference;-onBindView-(Landroid/view/View;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/net/NetworkPolicyEditor;-write-([Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/net/NetworkStatsService;-removeUidsLocked-([I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/soundrecorder/Recorder;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-updateNotification-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/model/dataitem/RelationDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/Gallery;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/RIL;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/MessageStatusService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/Controller;-goLive-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton$1;-onClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText;-getCarrierTextForSimState-(Lcom/android/internal/telephony/IccCardConstants$State; Ljava/lang/CharSequence; Ljava/lang/CharSequence;)Ljava/lang/CharSequence;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-onClose-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onRoamingOff-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/calllog/CallLogFragment;-onDestroy-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator$2;-keyguardDone-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-dispatch-(Landroid/content/Intent; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onSetInternalDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService;-getTetherableIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/deskclock/DeskClock;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/NfcAdapter;-disableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl;-getInstance-(Landroid/content/Context;)Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/PhoneUtils;-turnOnNoiseSuppression-(Landroid/content/Context; Z Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/RuimRecords;-setLocaleFromCsim-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/telephony/TelephonyManager;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/ComboAlbum;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/MtpDevice;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/bluetooth/PanProfile;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$AccountSyncSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/RecognitionManagerService$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/UrlHandler;-shouldOverrideUrlLoading-(Lcom/android/browser/Tab; Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/BluetoothA2dp;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/CalendarContract$Attendees;-query-(Landroid/content/ContentResolver; J [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/am/BatteryStatsService;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/activities/ContactSelectionActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-syncReloadDictionaryInternal-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/BatteryService$8;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator$1;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/AlarmReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/accounts/ManageAccountsSettings;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/text/ToUpperCase;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothHealth;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/vcard/ImportVCardActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/phone/QuickSettingsModel;-addBluetoothTile-(Lcom/android/systemui/statusbar/phone/QuickSettingsTileView; Lcom/android/systemui/statusbar/phone/QuickSettingsModel$RefreshCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/users/UserSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/AccessibilitySettings$ToggleScreenMagnificationPreferenceFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/ContactEditorActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/DeviceStorageMonitorService;-cancelNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-screenTurnedOff-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/GLRoot$OnGLIdleListener;-onGLIdle-(Lcom/android/gallery3d/ui/GLCanvas; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/Tab;-deleteThumbnail-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-access$000-(Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/NfcService$EnableDisableTask;-disableInternal-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/providers/contacts/DatabaseModifier;-insert-(Ljava/lang/String; Ljava/lang/String; Landroid/content/ContentValues;)J' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$UuidChangedHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/telephony/SmsManager;-updateMessageOnIcc-(I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/settings/wifi/WifiSettings;-getAccessPointsCount-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-resetTouchState-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/text/StringLogger;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/widget/CalendarAppWidgetProvider;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/Controller$7;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/Connection;-proceedAfterWaitChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/AccountUnlockScreen;-findIntendedAccount-(Ljava/lang/String;)Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/provider/CalendarContract$EventsEntity$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/CommandsInterface;-switchWaitingOrHoldingAndActive-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/ConnectivityManager;-setRadio-(I Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/ViewRootImpl;-scheduleProcessInputEvents-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/IWindowSession$Stub$Proxy;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setRemoteAlias-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wm/WindowManagerService;-computeNewConfigurationLocked-()Landroid/content/res/Configuration;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/media/NotificationPlayer;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/util/AccountsListAdapter;-<init>-(Landroid/content/Context; Lcom/android/contacts/util/AccountsListAdapter$AccountListFilter;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/PhotoPage;-switchWithCaptureAnimation-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/pm/PackageManagerService;-access$4200-(Lcom/android/server/pm/PackageManagerService; Z Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/ConnectivityManager;-getNetworkInfo-(I)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-createDevicePreference-(Lcom/android/settings/bluetooth/CachedBluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/launcher2/AccessibleTabView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame$1;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/TimeBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-getUriFor-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton;-updateEmergencyCallButton-(Lcom/android/internal/telephony/IccCardConstants$State; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/ContactSaveService;-deliverCallbackOnUiThread-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/inputmethod/KeyboardLayoutPickerFragment;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/DataUsageSummary$3;-onTabChanged-(Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/effect/effects/PosterizeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/UiModeManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothServerSocket;-<init>-(I Z Z Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/location/LocationManager;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/data/DownloadCache$DownloadTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/io/File;' : ['android.permission.INTERNET'],
	'Lcom/android/phone/InCallScreen;-hangupRingingCall-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/transaction/NotificationPlayer;-access$700-(Lcom/android/mms/transaction/NotificationPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-access$000-(Lcom/android/inputmethod/keyboard/MainKeyboardView; Lcom/android/inputmethod/keyboard/Key; Lcom/android/inputmethod/keyboard/PointerTracker;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/os/IHardwareService;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/gallery3d/data/MediaSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/performance/ThroughputFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/SIMRecords;-getSpnFsm-(Z Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/ContactSaveService;-saveContact-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag$Stub;-ndefWrite-(I Landroid/nfc/NdefMessage;)I' : ['android.permission.NFC'],
	'Landroid/telephony/gsm/SmsManager;-sendDataMessage-(Ljava/lang/String; Ljava/lang/String; S [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/webkit/HTML5VideoFullScreen;-prepareForFullScreen-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$LocalePickerActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$5100-(Lcom/android/mms/ui/ComposeMessageActivity;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/RIL;-deleteSmsOnRuim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/SlideViewInterface;-setVideo-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Call;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-removeBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-onDeviceBondStateChanged-(Lcom/android/settings/bluetooth/CachedBluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/google/common/io/Resources;-copy-(Ljava/net/URL; Ljava/io/OutputStream;)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/CommandsInterface;-deleteSmsOnRuim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/inputmethod/KeyboardLayoutPickerFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-stopSync-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/wifi/WpsDialog;-handleEvent-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/exchange/CalendarSyncEnabler;-enableEasCalendarSyncInternalForTest-()Ljava/lang/String;' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-updateInTransaction-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/SystemUIService;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/providers/downloads/RealSystemFacade;-isNetworkRoaming-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/wallpaper/galaxy/GalaxyView;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/downloads/DownloadThread;-notifyDownloadCompleted-(I Z I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/browser/XLargeUi;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/model/dataitem/ImDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-setCurrentPreferredNetworkType-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-notifyInvalidSelection-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-acknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab;-destroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/calllog/DefaultVoicemailNotifier$DefaultNewCallsQuery;-query-()[Lcom/android/contacts/calllog/DefaultVoicemailNotifier$NewCall;' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-<init>-(Landroid/content/Context; Ljava/util/Locale;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/WifiStateMachine$VerifyingLinkState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/webkit/WebViewCore;-contentDraw-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/opp/TestTcpServer;-run-()V' : ['android.permission.INTERNET'],
	'Lcom/android/browser/Preloader;-handlePreloadRequest-(Ljava/lang/String; Ljava/lang/String; Ljava/util/Map; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/ConnectivityService;-getTetherableBluetoothRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-resetTouchState-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onCleanUpConnection-(Z I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-access$400-(Landroid/inputmethodservice/KeyboardView; Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView$2;-onTextInput-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/server/BluetoothA2dpService;-handleSinkStateChange-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/phone/EmergencyCallHelper;-onRetryTimeout-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-requestNetworkTransitionWakelock-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/Presenter;-present-(Lcom/android/mms/util/ItemLoadedCallback;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/bluetooth/A2dpProfile;-getSummaryResourceForDevice-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/PhotoView;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/SIMFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$3;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$800-(Lcom/android/mms/ui/ComposeMessageActivity; Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$1500-(Lcom/android/gallery3d/ui/PhotoView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/Settings$WifiDisplaySettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-addAcount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/emailcommon/service/EmailServiceProxy;-<init>-(Landroid/content/Context; Ljava/lang/String; Lcom/android/emailcommon/service/IEmailServiceCallback;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/Utils;-copyMeProfilePhoto-(Landroid/content/Context; Landroid/content/pm/UserInfo;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/coremedia/iso/PropertyBoxParserImpl;-<init>-([Ljava/lang/String;)V' : ['android.permission.INTERNET'],
	'Lcom/android/mms/data/Conversation$3;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/net/IConnectivityManager$Stub;-updateLockdownVpn-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/MountService;-onDaemonConnected-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/AlbumPage;-pickPhoto-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/app/PhotoPage$Model;-moveTo-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/InCallTouchUi;-access$300-(Lcom/android/phone/InCallTouchUi; Lcom/android/internal/telephony/CallManager;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-addNavigationBar-()V' : ['android.permission.VIBRATE'],
	'Landroid/view/IWindowManager$Stub;-isKeyguardSecure-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/PhoneInterfaceManager;-enforceCallPermission-()V' : ['android.permission.CALL_PHONE'],
	'Landroid/media/effect/effects/GrayscaleEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/inputmethod/InputMethodAndLanguageSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupCreatedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS'],
	'Lcom/android/browser/GoogleAccountLogin;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetooth$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-connectAutoConnectableProfiles-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/content/ContentService;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService$8$1;-run-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/media/RingtonePlayer$1;-stop-(Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-acquireWakeLockInternal-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource; I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-asyncReloadDictionaryIfRequired-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/CommandsInterface;-getNetworkSelectionMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-endReordering-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/CaptivePortalTracker$DelayedCaptiveCheckState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/vcard/VCardEntry$InsertOperationConstrutor;-onElement-(Lcom/android/vcard/VCardEntry$EntryElement;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/calendar/CalendarProvider2;-query-(Landroid/database/sqlite/SQLiteDatabase; Landroid/database/sqlite/SQLiteQueryBuilder; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/RuimSmsInterfaceManager;-updateMessageOnIccEf-(I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/contacts/GroupMetaDataLoader;-ensureIsGroupUri-(Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/tech/NfcA;-connect-()V' : ['android.permission.NFC'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/UiccCardApplication;-supplyPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-removeAllGeofences-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/TrimVideo;-onSeekEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CallTracker;-handlePollCalls-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5VideoViewProxy;-play-(Ljava/lang/String; I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewBase;-interceptMediaKey-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/NfcService;-_nfcEeClose-(I Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.DISABLE_KEYGUARD', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterfw/core/Filter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothPan;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-enqueueRequest-(Ljava/lang/String; Ljava/lang/String; Lcom/android/contacts/calllog/ContactInfo; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/PhotoPage$MyMenuVisibilityListener;-onMenuVisibilityChanged-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/activities/GroupBrowserActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/model/Contact;-isWritableContact-(Landroid/content/Context;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-fixTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/contacts/activities/NonPhoneActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/MessagingNotification;-updateDownloadFailedNotification-(Landroid/content/Context;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-revertToNitzTime-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;-onBtClassChanged-(Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/transaction/TransactionService$ServiceHandler;-processPendingTransaction-(Lcom/android/mms/transaction/Transaction; Lcom/android/mms/transaction/TransactionSettings;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-doResumeUpdates-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/sip/SipPhoneFactory;-makePhone-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)Lcom/android/internal/telephony/sip/SipPhone;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CallerInfo;-getCallerId-(Landroid/content/Context; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-applyNewState-(Lcom/android/internal/telephony/ApnContext; Z Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/Utils;-getShorterNameIfPossible-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/ISms;-enableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/voicedialer/VoiceDialerActivity$CommandRecognizerClient;-onMicrophoneStart-(Ljava/io/InputStream;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/UrlInputView;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/Settings$DateTimeSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/PeopleActivity;-configureContactListFragmentForRequest-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/socialwidget/SocialWidgetProvider;-setDisplayNameAndSnippet-(Landroid/content/Context; Landroid/widget/RemoteViews; Ljava/lang/CharSequence; Ljava/lang/CharSequence; Ljava/util/List; Landroid/app/PendingIntent;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-access$1700-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewManager$ShowListener;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/telephony/MmsProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/keyboard/internal/SuddenJumpingTouchEventHandler$ProcessMotionEvent;-processMotionEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-dispatchUnhandledMove-(Landroid/view/View; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-saveWebArchive-(Ljava/lang/String; Z Landroid/webkit/ValueCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/tv/TvStatusBar;-hideSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/TelephonyRegistry;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/AccountUnlockScreen;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/BluetoothNameDialogFragment$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/MediaPlaybackService;-removeTrack-(J)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarUtils$TimeZoneUtils;-setTimeZone-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/EasSyncService;-sendPing-([B I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/editor/PhotoActionPopup$1;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/InCallTouchUi;-handleAudioButtonClick-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-acknowledgeLastIncomingCdmaSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/LocationManager;-stopReceivingLocationUpdates-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/DTMFTwelveKeyDialer;-clearInCallScreenReference-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/Controller;-<init>-(Landroid/app/Activity;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/ContactListFilterControllerImpl;-filterAccountExists-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebView;-zoomOut-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onSingleTapUp-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wm/WindowManagerService;-updateRotationUncheckedLocked-(Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/wallpaper/grass/GrassRS$TimezoneTracker;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/model/dataitem/NoteDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/data/Conversation;-<init>-(Landroid/content/Context; J Z)V' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/SMSDispatcher;-dispatch-(Landroid/content/Intent; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag;-formatNdef-(I [B)I' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-restartRadio-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/VideoView;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$10$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/content/SyncManager;-access$1200-(Landroid/content/SyncManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewClassic;-access$4600-(Landroid/webkit/WebViewClassic; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/vcard/VCardEntry$WebsiteData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/TelephonyRegistry;-broadcastDataConnectionStateChanged-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/LinkCapabilities; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/fall/FallRS;-start-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/SyncManager$ActiveSyncContext;-binderDied-()V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/nfc/handover/HandoverService;-onBluetoothHeadsetHandoverComplete-(Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/InCallScreen$4;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5Audio;-resetMediaPlayer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/UiController;-findOnPage-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewCore$EventHub;-access$7100-(Landroid/webkit/WebViewCore$EventHub; Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneFactory;-makeDefaultPhone-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/widget/RadioButton;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/BootReceiver$1;-run-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetooth$Stub;-disable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/DeliveryReportActivity;-getMmsReportStatus-()Ljava/util/Map;' : ['android.permission.READ_SMS'],
	'Lcom/android/settings/Settings$WifiDisplaySettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/soundrecorder/Recorder;-startPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$KeyboardLayoutPickerActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/PhoneInterfaceManager;-access$000-(Lcom/android/phone/PhoneInterfaceManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/WiredAccessoryManager;-updateLocked-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/ContactPhotoManagerImpl$LoaderThread;-loadThumbnails-(Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerPolicy;-checkAddPermission-(Landroid/view/WindowManager$LayoutParams;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/view/inputmethod/InputMethodManager;-enqueuePendingEventLocked-(J I Ljava/lang/String; Landroid/view/inputmethod/InputMethodManager$FinishedEventCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/OtaUtils;-otaShowProgramFailureNotice-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewProvider;-showFindDialog-(Ljava/lang/String; Z)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasOutboxService;-sync-(Lcom/android/exchange/adapter/AbstractSyncAdapter;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/soundrecorder/SoundRecorder;-updateTimeRemaining-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/telephony/MmsSmsProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-animateCollapsePanels-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/vcard/VCardComposer;-init-(Ljava/lang/String; [Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/transaction/RetrieveTransaction;-ensureRouteToHost-(Ljava/lang/String; Lcom/android/mms/transaction/TransactionSettings;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/LocalSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-access$1100-(Lcom/android/bluetooth/pbap/BluetoothPbapService;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/ChooseLockGeneric$ChooseLockGenericFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/providers/contacts/VoicemailStatusTable;-delete-(Lcom/android/providers/contacts/VoicemailContentProvider$UriData; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/accounts/AccountManager$2;-doWork-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/editor/Editor$EditorListener;-onRequest-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/IccCard;-changeIccLockPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-getTetheredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-showAppropriateWidgetPage-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/editor/TextFieldsEditorView$2;-afterTextChanged-(Landroid/text/Editable;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$BluetoothStateTracker;-onActualStateChange-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/BluetoothHeadset;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$4400-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-connect-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-onClose-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-access$300-(Lcom/android/deskclock/widget/multiwaveview/GlowPadView; I F F)V' : ['android.permission.VIBRATE'],
	'Lcom/android/voicedialer/RecognizerEngine;-setupGrammar-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/bluetooth/btservice/AdapterService;-autoConnectHeadset-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Ljava/lang/String; J F Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame;-onAttachedToWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/editor/ContactEditorFragment$3;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/NetworkStateHandler;-onResume-()V' : ['android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton;-access$100-(Lcom/android/internal/policy/impl/keyguard/EmergencyButton; Lcom/android/internal/telephony/IccCardConstants$State; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$KeyboardLayoutPickerActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAccountView;-findIntendedAccount-(Ljava/lang/String;)Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/Controller;-doUpdateVisitedHistory-(Lcom/android/browser/Tab; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/bluetooth/RequestPermissionActivity;-parseIntent-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/PhoneInterfaceManager;-enableDataConnectivity-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/app/IWallpaperManager$Stub;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-watchForDeviceProvisioning-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAbsKeyInputView;-reset-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/WindowManagerPolicy;-enableKeyguard-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/event/AttendeesView$PresenceQueryHandler$1;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/camera/SecureCameraActivity;-createCameraScreenNail-(Z)Lcom/android/gallery3d/ui/ScreenNail;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/speech/IRecognitionService$Stub$Proxy;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-handleForwardedMessage-()Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/phone/InCallScreen$13;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-handleCallSetupRequestFromSim-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker$3;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/browser/provider/BrowserProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/SearchActivity$1;-onChange-(Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-loadUrlImpl-(Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-removeTracks-(I I)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/detail/ContactDetailUpdatesFragment;-getAccountTypeForStreamItemEntry-(Lcom/android/contacts/util/StreamItemEntry;)Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterfw/core/Filter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/SimContacts;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab$3;-onCreateWindow-(Landroid/webkit/WebView; Z Z Landroid/os/Message;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/RadioInfo;-refreshSmsc-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/AlarmManagerService$AlarmThread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/calendar/AllInOneActivity;-onResume-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mms/util/Recycler$SmsRecycler;-deleteMessagesForThread-(Landroid/content/Context; J I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/research/ResearchLogger;-latinIME_onUpdateSelection-(I I I I I I I I Z Z Lcom/android/inputmethod/latin/RichInputConnection;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment$3;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-createSubWindow-(Lcom/android/browser/Tab;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/mms/ui/AudioAttachmentView;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerPolicy;-setSafeMode-(Z)V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/WebViewProvider$ViewDelegate;-requestFocus-(I Landroid/graphics/Rect;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-enableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-handleMenuKey-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/location/GpsLocationProvider;-access$2300-(Lcom/android/server/location/GpsLocationProvider;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator$1;-onKeyguardVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-prepareAppTransition-(I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-getContactCount-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/list/ContactListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/CallbackProxy;-onJsConfirm-(Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/tech/NfcF;-transceive-([B)[B' : ['android.permission.NFC'],
	'Landroid/net/wifi/WifiStateMachine;-access$000-(Landroid/net/wifi/WifiStateMachine; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService;-openCurrentAndNext-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-onMenuClicked-(I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-makeStatusBarView-()Lcom/android/systemui/statusbar/phone/PhoneStatusBarView;' : ['android.permission.DISABLE_KEYGUARD', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-access$1000-(Lcom/android/server/LocationManagerService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$5000-(Lcom/android/mms/ui/ComposeMessageActivity;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/Settings$DisplaySettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager$Stub;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-getDrmMimeMenuStringRsrc-(J)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-restoreSelectedUri-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewClassic;-pageSwapCallback-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothPan;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBarView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAccountView$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/Controller$SaveSnapshotTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/ui/ConversationList;-startAsyncQuery-()V' : ['android.permission.READ_SMS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-requestRouteToHost-(I I)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/DockService;-msgTypeUndockedPermanent-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-deleteShare-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/phone/CallNotifier;-resetAudioStateAfterDisconnect-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/MovieActivity;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection;-dial-()V' : ['android.permission.USE_SIP'],
	'Landroid/widget/QuickContactBadge;-assignContactFromPhone-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/InCallScreen;-internalAnswerAndEnd-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/media/IMediaScannerListener$Stub$Proxy;-scanCompleted-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-toggleRecentsActivity-()V' : ['android.permission.GET_TASKS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$InitialState;-turnOnUpstreamMobileConnection-(I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifyDataConnection-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IntentSender$FinishedDispatcher;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/webkit/WebViewClassic$InvokeListBox$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/wifi/WifiSettingsForSetupWizardXL;-showDisconnectedState-(Ljava/lang/String;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/exchange/adapter/AbstractSyncAdapter;-parse-(Ljava/io/InputStream;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/view/IWindowManager$Stub;-startFreezingScreen-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/vcard/VCardParserImpl_V30;-parseItems-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onSetUserDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DayView$CalendarGestureListener;-onSingleTapUp-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$1;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/NsdService$NsdStateMachine;-registerForNsdSetting-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$100-(Lcom/android/gallery3d/ui/PhotoView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/IccCard;-supplyPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/TileImageView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/location/ILocationManager$Stub$Proxy;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl$KeepAliveProcess;-process-(Ljava/util/EventObject;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$WifiStateTracker;-getActualState-(Landroid/content/Context;)I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-updatePhoto-(J Landroid/content/ContentValues;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-setupDataOnReadyApns-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/data/WorkingMessage;-setAttachment-(I Landroid/net/Uri; Z)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$ManageAccountsSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/InCallScreen;-onPhoneStateChanged-(Landroid/os/AsyncResult;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/TintFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexSession;-start-(Landroid/os/Handler;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SlotView;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/InCallScreen;-handleOnscreenButtonClick-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/MediaPlaybackService;-access$400-(Lcom/android/music/MediaPlaybackService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/spellcheck/DictionaryPool;-pollWithDefaultTimeout-()Lcom/android/inputmethod/latin/spellcheck/DictAndProximity;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-access$1400-(Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ContactEntryListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame;-access$100-(Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView;-maybeEnableFallback-(Landroid/content/Context;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/FdnSetting;-toggleFDNEnable-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-doKeyguardLocked-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/CallbackProxy;-onJsTimeout-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/tech/MifareClassic;-authenticateSectorWithKeyB-(I [B)Z' : ['android.permission.NFC'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/mms/data/WorkingMessage;-sendSmsWorker-(Ljava/lang/String; Ljava/lang/String; J)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/vpndialogs/ConfirmDialog;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetooth$Stub;-sendConnectionStateChange-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-loadCallHistoryList-(I)Ljava/util/ArrayList;' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/media/MediaPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/ThrottleManager;-getHelpUri-()Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/media/AsyncPlayer;-enqueueLocked-(Landroid/media/AsyncPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/applications/RunningServiceDetails$ActiveDetail;-onClick-(Landroid/view/View;)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-play-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowSession$Stub;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/mms/ui/MessagingPreferenceActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/Controller;-createNewTab-(Z Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/WifiStateMachine$ConnectedState;-exit-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-requestChildRectangleOnScreen-(Landroid/view/View; Landroid/graphics/Rect; Z)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/SurfaceView;-updateWindow-(Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-editSmsMessageItem-(Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/content/ContentService;-getSyncManager-()Landroid/content/SyncManager;' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/IThrottleManager;-getCliffLevel-(Ljava/lang/String; I)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/SIMRecords;-handleSimRefresh-(Lcom/android/internal/telephony/IccRefreshResponse;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/CryptKeeper;-setupUi-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$700-(Lcom/android/mms/ui/ComposeMessageActivity; Lcom/android/mms/ui/MessageItem;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/MasterClear;-loadAccountList-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/PhoneGlobals;-requestWakeState-(Lcom/android/phone/PhoneGlobals$WakeState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/AlertActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/providers/contacts/ContactsProvider2;-updateContactsAccountCount-([Landroid/accounts/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/phone/CallNotifier;-onRingbackTone-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/HidProfile;-getPreferred-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebViewInputDispatcher;-scheduleWebKitDispatchLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-access$1100-(Lcom/android/gallery3d/app/MoviePlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IMediaScannerListener$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/CropImage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/exchange/ExchangeService$2;-deleteAccountPIMData-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onCallBusy-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-onAddView-(Landroid/view/View; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/downloads/DownloadInfo;-getAllDownloadsUri-()Landroid/net/Uri;' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewBase;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActiveServices$ServiceRestarter;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/list/DirectoryListLoader;-loadInBackground-()Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/ui/MmsThumbnailPresenter;-presentAudioThumbnail-(Lcom/android/mms/ui/SlideViewInterface; Lcom/android/mms/model/AudioModel;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/util/Recycler$SmsRecycler;-deleteOldMessagesByThreadId-(Landroid/content/Context; J)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/PreloadedTabControl;-destroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothHeadsetPhone$Stub$Proxy;-answerCall-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/contacts/vcard/ImportVCardActivity$ImportRequestConnection;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-createAndAddWindows-()V' : ['android.permission.DISABLE_KEYGUARD', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/MessagingPreferenceActivity;-restoreDefaultPreferences-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/transaction/TransactionService;-onNewIntent-(Landroid/content/Intent; I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onSeMifareAccess-([B)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/gallery3d/onetimeinitializer/GalleryWidgetMigrator;-migrateGalleryWidgetsInternal-(Landroid/content/Context;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/widget/IndexerListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/AllInOneActivity;-onTabSelected-(Landroid/app/ActionBar$Tab; Landroid/app/FragmentTransaction;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/ui/PhotoView$MyGestureListener;-deleteAfterAnimation-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-onStopTrackingTouch-(Landroid/widget/SeekBar;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewCore;-jsAlert-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/sip/SipService$ConnectivityReceiver$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/google/common/io/MultiInputStream;-<init>-(Ljava/util/Iterator;)V' : ['android.permission.INTERNET'],
	'Landroid/bluetooth/BluetoothHeadset;-roamChanged-(Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/CallbackProxy;-onUnhandledKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/UserDictionarySettings;-createCursor-(Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/contacts/detail/PhotoSelectionHandler$PhotoActionListener;-onUseAsPrimaryChosen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothHeadset;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/systemui/statusbar/DelegateViewHelper;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/app/AlertActivity;-onStop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-chooseUpstreamType-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/PhotoPage;-showDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/DeleteEventHelper;-access$1200-(Lcom/android/calendar/DeleteEventHelper; I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-vibrate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/gallery3d/ui/SlotView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/webkit/WebViewClassic;-access$4200-(Landroid/webkit/WebViewClassic; I I Landroid/webkit/WebViewCore$TextSelectionData;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-loadEfFilesFromUsim-()Ljava/util/ArrayList;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DataUsageSummary$AppDetailsFragment;-onStart-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/WebViewController;-openTab-(Ljava/lang/String; Z Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/pan/PanService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/model/RawContactDeltaList;-beginKeepTogether-()Landroid/content/ContentProviderOperation$Builder;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/IccFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/SearchPanelView;-hide-(Z)V' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/IBluetoothHeadsetPhone$Stub$Proxy;-processChld-(I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/gallery3d/app/StateManager;-finishState-(Lcom/android/gallery3d/app/ActivityState;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/Settings;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/content/Context;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/gadget/MediaSetSource;-getImage-(I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/mms/data/Conversation;-startQuery-(Landroid/content/AsyncQueryHandler; I Ljava/lang/String;)V' : ['android.permission.READ_SMS'],
	'Landroid/widget/VideoView;-stopPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/ContactPickerFragment;-isSyncActive-()Z' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/UiController;-handleNewIntent-(Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/VibratorService;-vibrate-(J Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/editor/StructuredNameEditorView;-onFieldChanged-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$TetherSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/providers/UIProvider;-incrementRecipientsTimesContacted-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/HTML5VideoFullScreen$1;-surfaceCreated-(Landroid/view/SurfaceHolder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchFocus-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/activities/ActionBarAdapter;-setCurrentTab-(I Z)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/HTML5VideoFullScreen;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchToNextImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/GLTextureTarget;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UiController;-setActiveTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/ui/PhotoView$MyGestureListener;-flingImages-(F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/calendar/CalendarProvider2;-getOwner-(J)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Landroid/webkit/CallbackProxy;-shouldOverrideUrlLoading-(Ljava/lang/String;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$3;-pokeWakelock-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub$Proxy;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/browser/Tab$3;-onProgressChanged-(Landroid/webkit/WebView; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/EventInfoFragment;-createExceptionResponse-(J I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/RIL;-acknowledgeLastIncomingGsmSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/media/NotificationPlayer;-startSound-(Lcom/android/systemui/media/NotificationPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-access$300-(Lcom/android/bluetooth/btservice/BondStateMachine; Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-editSmsMessageItem-(Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/bluetooth/BluetoothDevicePreference;-pair-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-getIMEI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/CharStreams;-copy-(Lcom/google/common/io/InputSupplier; Lcom/google/common/io/OutputSupplier;)J' : ['android.permission.INTERNET'],
	'Lcom/android/settings/MasterClear;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/IMediaPlaybackService;-play-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/SmsReceiverService;-handleBootCompleted-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-onDayTapped-(Landroid/text/format/Time;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/ui/SlideshowView;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/IRingtonePlayer$Stub$Proxy;-stop-(Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/GoogleAccountLogin$2;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/IConnectivityManager;-startLegacyVpn-(Lcom/android/internal/net/VpnProfile;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-doCluster-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/providers/calendar/CalendarAlarmManager;-scheduleNextAlarmLocked-(Landroid/database/sqlite/SQLiteDatabase; Lcom/android/providers/calendar/CalendarProvider2;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dialInternal-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/contacts/model/AccountTypeManager;-getUsableInvitableAccountTypes-()Ljava/util/Map;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/app/AlbumPage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/am/ActivityManagerService;-sendPendingBroadcastsLocked-(Lcom/android/server/am/ProcessRecord;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/list/JoinContactListFragment;-prepareEmptyView-()V' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/AudioService$AudioHandler$2;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/NfcA;-setTimeout-(I)V' : ['android.permission.NFC'],
	'Landroid/webkit/WebViewClassic$8;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/calendar/CalendarProvider2;-deleteFromEventRelatedTable-(Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/sip/SipAudioCall;-startAudio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onLlcpLinkDeactivated-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-sendDisableMsg-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-access$700-(Lcom/android/bluetooth/pbap/BluetoothPbapService;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/wm/WindowManagerService;-sendNewConfiguration-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-checkMtFindNewRinging-(Lcom/android/internal/telephony/DriverCall; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-onStart-()V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/IBluetoothPan;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/voicedialer/VoiceDialerActivity$VoiceDialerBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/bluetooth/IBluetooth$Stub;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-getConnectedHealthDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/transaction/SmsReceiver;-onReceiveWithPrivilege-(Landroid/content/Context; Landroid/content/Intent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ConversationList$1;-onContentChanged-(Lcom/android/mms/ui/ConversationListAdapter;)V' : ['android.permission.READ_SMS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getActiveNetworkQuotaInfo-()Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/DisplaySettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/ConnectivityManager;-isNetworkSupported-(I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-removeBatch-(Lcom/android/bluetooth/opp/BluetoothOppBatch;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-wakeWhenReady-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneBase;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/PrivilegedSmsReceiver;-beginStartingService-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/provider/BrowserProvider2;-deleteInTransaction-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String; Z)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/transaction/NotificationPlayer;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/search/OpenSearchSearchEngine;-getActiveNetworkInfo-(Landroid/content/Context;)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewClassic$InvokeListBox$4;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/widget/VideoView;-access$2100-(Landroid/widget/VideoView; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/SyncAdapterDriver;-startSyncSelected-(Landroid/view/View;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/AlarmClock$AlarmItemAdapter;-getRingToneTitle-(Landroid/net/Uri;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab$3;-createWindow-(Z Landroid/os/Message;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/ContactsActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/media/AudioService;-sendSimulatedMediaButtonEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterfw/core/SyncRunner;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-pollStateDone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/contacts/model/RawContactDeltaList;-indexOfFirstWritableRawContact-(Landroid/content/Context;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$7300-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Landroid/net/wifi/p2p/WifiP2pConfig;)V' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-cleanUpAllConnections-(Z Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/RotateFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-handleBootCompletedLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getRemoteAlias-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/StockSource;-load-(Lcom/android/dreams/phototable/PhotoSource$ImageData; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/systemui/statusbar/phone/SettingsPanelView;-setup-(Lcom/android/systemui/statusbar/policy/NetworkController; Lcom/android/systemui/statusbar/policy/BluetoothController; Lcom/android/systemui/statusbar/policy/BatteryController; Lcom/android/systemui/statusbar/policy/LocationController;)V' : ['android.permission.BLUETOOTH', 'android.permission.DISABLE_KEYGUARD', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/alerts/AlertActivity;-onStop-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CdmaCall;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/SuggestionsAdapter;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/pm/PackageManagerService;-replaceSystemPackageLI-(Landroid/content/pm/PackageParser$Package; Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-stopLocationBasedDetector-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/adapter/AccountSyncAdapter;-setSyncKey-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/WifiManager;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onTrySetupData-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-broadcastConnectionState-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-findNextAlarmTime-(Landroid/content/ContentResolver; J)J' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/WifiService;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/BrowserSettings;-clearHistory-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-showJoinAggregateActivity-(Landroid/net/Uri;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/group/GroupEditorFragment$2;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/google/common/io/MultiInputStream;-read-([B I I)I' : ['android.permission.INTERNET'],
	'Landroid/os/SystemVibrator;-cancel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/VibratorService$1;-onChange-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/model/dataitem/OrganizationDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView;-reset-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView$2;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewBase;-onScreenTurnedOff-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/Telephony$Sms$Outbox;-addMessage-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long; Z J)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/cdma/RuimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/AlertActivity;-dismissAlarm-(J)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetoothHealth;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/TetherSettings;-access$100-(Lcom/android/settings/TetherSettings; [Ljava/lang/String; [Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider;-onEnabled-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-setNewText-(I I Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getCurrentSync-()Landroid/content/SyncInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/net/wifi/SupplicantStateTracker$DormantState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/BluetoothPairingDialog;-createConfirmationDialog-(Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/musicfx/ControlPanelPicker;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-cleanupAppWidgetIds-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/tech/NfcBarcode;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/contacts/quickcontact/QuickContactActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/GLTextureSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-interceptKeyBeforeDispatching-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)J' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/AccountFilterActivity;-access$100-(Landroid/content/Context;)Ljava/util/List;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/LocationManagerService;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/SearchActivity;-onResume-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/ExchangeService;-stopServiceThreads-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/detail/ContactDetailFragment$ViewAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/Tab;-setWebView-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mms/transaction/NotificationTransaction;-getPdu-(Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/calculator2/ColorButton;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebSettingsClassic;-setJavaScriptEnabled-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-setVisible-(Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/btservice/AdapterService;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-unpairDevice-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-registerAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/IBluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/vcard/VCardParserImpl_V40;-handleNest-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/soundrecorder/Recorder;-stopRecording-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler;-getDeltaForAttachingPhotoToContact-()Lcom/android/contacts/model/RawContactDeltaList;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/nfc/NfcService$EnableDisableTask;-executeEeWipe-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-access$400-(Lcom/android/server/LocationManagerService; I I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/emailcommon/service/ServiceProxy$ProxyTask;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-sendKeyEventsToTarget-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManager$BaseFutureTask;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/CallNotifier;-onDisconnect-(Landroid/os/AsyncResult;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-onClick-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/settings/accounts/SyncSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/RIL;-setRadioPower-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/Use2GOnlyCheckBoxPreference;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-updateMediaRecorderParams-()V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/AccountManager;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/widget/WaveView$OnTriggerListener;-onGrabbedStateChange-(Landroid/view/View; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/mms/ui/ConversationList$ThreadListQueryHandler;-onDeleteComplete-(I Ljava/lang/Object; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/settings/DreamSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ConnectivityService;-setNetworkPreference-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-updateScreen-(Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$Mode; Z)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/IRingtonePlayer$Stub;-playAsync-(Landroid/net/Uri; Landroid/os/UserHandle; Z I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$WifiStateTracker$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/settings/accounts/ManageAccountsSettings;-onSyncStateUpdated-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-hideLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/phone/CallNotifier$EmergencyTonePlayerVibrator;-start-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/PhoneProxy;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/Conversation;-access$800-(Landroid/content/Context;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/BluetoothHeadset;-acceptIncomingConnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/telephony/gsm/SmsManager;-sendTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/net/IThrottleManager$Stub$Proxy;-getByteCount-(Ljava/lang/String; I I I)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewClassic;-findNext-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-hideBouncer-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/net/wifi/IWifiManager;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/gadget/WidgetSource;-getImage-(I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Landroid/media/IAudioService;-dispatchMediaKeyEventUnderWakelock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewInputDispatcher;-enqueueUiCancelTouchEventIfNeededLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-hasContentChanged-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/nfc/tech/NfcB;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/commands/am/Am;-runKill-()V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/wallpaper/RenderScriptScene;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/activities/ActionBarAdapter;-setSearchMode-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-getSignalStrength-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/SubSettings;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/nfc/tech/Ndef;-close-()V' : ['android.permission.NFC'],
	'Landroid/accounts/AccountManager$9;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-setWfdInfo-(Landroid/net/wifi/p2p/WifiP2pWfdInfo;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/bluetooth/BluetoothNameDialogFragment;-<init>-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$6100-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/IContentService;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupNegotiationState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS'],
	'Landroid/net/DhcpStateMachine$WaitBeforeStartState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebView;-<init>-(Landroid/content/Context;)V' : ['android.permission.INTERNET'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$1;-handleMessage-(Landroid/os/Message;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onSetDependencyMet-(Ljava/lang/String; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/inputmethod/UserDictionaryAddWordFragment;-onPause-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/server/display/WifiDisplayAdapter$1;-run-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/camera/CameraActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/sip/SipAudioCall;-startAudioInternal-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/AlbumSetPage$4;-onLongTap-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-onConnectedInOrOut-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-setModification-(I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/vcard/VCardParser_V21;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/util/Recycler$MmsRecycler;-deleteMessagesOlderThanDate-(Landroid/content/Context; J J)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/nfc/tech/MifareClassic;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-setLightsOn-(Z)V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/SnapshotTab;-persistThumbnail-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-update-(Lcom/android/internal/telephony/DriverCall;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-handleDown-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/RIL;-getIMSIForApp-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/INetworkStatsService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-access$1300-(Lcom/android/systemui/statusbar/phone/QuickSettings;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/DetailsHelper$DetailsViewContainer;-reloadDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/DictionaryCollection;-getSuggestions-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo;)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/ui/CropView;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserBookmarksPage$LookupBookmarkCount;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/webkit/WebViewProvider;-requestFocusNodeHref-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/phone/CallerInfoCacheUpdateReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/DefaultContactListAdapter;-bindView-(Landroid/view/View; I Landroid/database/Cursor; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/INfcTag$Stub;-ndefRead-(I)Landroid/nfc/NdefMessage;' : ['android.permission.NFC'],
	'Landroid/net/wifi/WifiManager;-init-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/vcard/VCardInterpreter;-onEntryEnded-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$DeviceAdminSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/hid/HidService;-onConnectStateChanged-([B I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/EventInfoFragment;-updateCalendar-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/model/dataitem/StructuredPostalDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebViewClassic;-performLongClick-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-stopFreezingDisplayLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/AlbumPage;-onSingleTapUp-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/data/WorkingMessage$3;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/PhotoPage$Model;-setCurrentPhoto-(Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothHeadsetPhone$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager;-onScreenTurnedOn-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager$ShowListener;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-processMotionEvent-(I I I J Lcom/android/inputmethod/keyboard/PointerTracker$KeyEventHandler;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/event/EditEventView;-updateView-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/media/effect/effects/FisheyeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/AccessibilitySettings$ToggleFeaturePreferenceFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl;-createAccountTypeManager-(Landroid/content/Context;)Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/EmergencyCallHelper;-startSequenceInternal-(Landroid/os/Message;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/ComboAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor;-registerCallback-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitorCallback;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothCallback$Stub$Proxy;-onBluetoothStateChange-(I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onTrySetupData-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallBusy-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$DeviceInfoSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/InCallTouchUi;-onTrigger-(Landroid/view/View; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountAdapter$RefreshCalendars;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/nfc/tech/MifareUltralight;-readPages-(I)[B' : ['android.permission.NFC'],
	'Lcom/android/settings/accounts/SyncSettings$1;-onPreferenceChange-(Landroid/preference/Preference; Ljava/lang/Object;)Z' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$DriverStoppedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$3700-(Lcom/android/gallery3d/ui/PhotoView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/InputMethodManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-selectText-(I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/accessibility/AccessibleKeyboardViewProxy;-dispatchHoverEvent-(Landroid/view/MotionEvent; Lcom/android/inputmethod/keyboard/PointerTracker;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/detail/ContactDetailLayoutController;-showContactWithoutUpdates-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/IThrottleManager;-getByteCount-(Ljava/lang/String; I I I)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/BaseCommands;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$4;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onTextInput-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/app/PhotoPage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/deviceinfo/Status;-onPause-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getAdapterConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/PointerTracker$TimerProxy;-startLongPressTimer-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$AccountSyncSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/detail/PhotoSelectionHandler;-getWritableEntityId-()J' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/os/BatteryStatsImpl;-doPlugLocked-(J J)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/model/dataitem/WebsiteDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$WatchdogEnabledState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator$1;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-openMoreKeysPanel-(Lcom/android/inputmethod/keyboard/Key; Lcom/android/inputmethod/keyboard/PointerTracker;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/EventInfoFragment;-doEdit-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/WebViewInputDispatcher;-access$400-(Landroid/webkit/WebViewInputDispatcher;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcAdapter$Stub;-setNdefPushCallback-(Landroid/nfc/INdefPushCallback;)V' : ['android.permission.NFC'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$5000-(Lcom/android/bluetooth/hfp/HeadsetStateMachine;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/providers/downloads/DownloadThread;-transferData-(Lcom/android/providers/downloads/DownloadThread$State; Lcom/android/providers/downloads/DownloadThread$InnerState; [B Ljava/io/InputStream;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$DevelopmentSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiStateMachine$TetheredState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$Listener;-onFling-(F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setAndBroadcastNetworkSetTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/launcher2/PagedViewWithDraggableItems;-determineDraggingStart-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-createAndAddWindows-()V' : ['android.permission.VIBRATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/IConnectivityManager;-getAllNetworkState-()[Landroid/net/NetworkState;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/model/dataitem/IdentityDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/content/ContextWrapper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/phone/CallNotifier;-restartRinger-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/Settings$LocationSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/ControllerOverlay$Listener;-onSeekEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService$DragInputEventReceiver;-onInputEvent-(Landroid/view/InputEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-startListener-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/LegacyContactListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/GlobalActions$GlobalActionsDialog;-onStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/RotationPolicy;-registerRotationPolicyListener-(Landroid/content/Context; Lcom/android/internal/view/RotationPolicy$RotationPolicyListener;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-handleServiceCategoryProgramData-(Lcom/android/internal/telephony/cdma/SmsMessage;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/vcard/VCardEntry$PhoneData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/PreloadRequestReceiver;-isPreloadEnabledOnCurrentNetwork-(Landroid/content/Context;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/IWindowManager;-clearForcedDisplaySize-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-handleOtaCallEnd-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/ZoomManager;-refreshZoomScale-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame$1;-onKeyguardVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/location/GpsLocationProvider;-reportAGpsStatus-(I I I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebView;-saveWebArchive-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onRadioAvailable-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$Listener;-onScale-(F F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/ConnectivityManager;-requestNetworkTransitionWakelock-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/DockService;-msgTypeDisableBluetooth-(I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/contacts/DbModifierWithNotification;-notifyVoicemailChangeOnInsert-(Landroid/net/Uri; Ljava/util/Set;)V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/accounts/AccountManager$Future2Task;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-queueQuery-(I I Landroid/text/format/Time; Ljava/lang/String; I J)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetoothInputDevice;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/month/SimpleWeekView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/SnapshotTab;-goBack-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-getPackageRestrictionClause-()Ljava/lang/String;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-setComposingRegion-(I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/detail/PhotoSelectionHandler;-handlePhotoActivityResult-(I I Landroid/content/Intent;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$ReloadTask;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/media/AudioManager;-isBluetoothA2dpOn-()Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/browser/NavTabScroller$OnLayoutListener;-onLayout-(I I I I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/latin/LatinIME;-handleSeparator-(I I I I)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/exchange/ContactsSyncAdapterService$SyncAdapterImpl;-onPerformSync-(Landroid/accounts/Account; Landroid/os/Bundle; Ljava/lang/String; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/HTML5VideoViewProxy$VideoPlayer;-setBaseLayer-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-<init>-(Landroid/preference/VolumePreference; Landroid/content/Context; Landroid/widget/SeekBar; I Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/ReadRecTransaction;-process-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-snapToPage-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothPairingRequest;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-copyMessageToIccEf-(I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/LocationManagerService;-getCallerAllowedResolutionLevel-()I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onRemoteFieldDeactivated-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/app/ContextImpl;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/location/GpsLocationProvider;-requestUtcTime-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-access$000-(Lcom/android/server/location/GeofenceManager;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-setGsmBroadcastConfig-([Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareClassic;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/gallery3d/data/DataManager;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/GLTextureSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/wifi/WifiSettings;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/base/FrameBranch;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-releaseWakeLock-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ConversationList$ModeCallback;-onActionItemClicked-(Landroid/view/ActionMode; Landroid/view/MenuItem;)Z' : ['android.permission.READ_SMS'],
	'Landroid/content/ContentService;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/calendar/DayView$CalendarGestureListener;-onFling-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/EasAccountService;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-setSearchMode-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/tv/TvStatusBar;-handleNotificationError-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification; Ljava/lang/String;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-reloadDataAndSetSelectedUri-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/model/dataitem/NoteDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/display/DisplayManagerService;-blankAllDisplaysFromPowerManager-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager;-setWifiEnabled-(Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManagerService;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/contacts/model/dataitem/PhotoDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/DeskClock;-initViews-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccCardProxy;-processLockedState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-onDevicePreferenceClick-(Lcom/android/settings/bluetooth/BluetoothDevicePreference;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/ContactsActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-changeIccPin2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-sendDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/MtpSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/SlotView$SimpleListener;-onLongTap-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/list/ContactListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipCommandInterface;-setCurrentPreferredNetworkType-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-muteVolume-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-addGpsStatusListener-(Landroid/location/IGpsStatusListener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/effect/effects/BitmapOverlayEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/UiModeManagerService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/coremedia/iso/IsoFile;-<init>-(Ljava/nio/channels/ReadableByteChannel; Lcom/coremedia/iso/BoxParser;)V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/ui/PopupList$2;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/SyncManager;-access$900-(Landroid/content/SyncManager; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewCore;-access$2200-(Landroid/webkit/WebViewCore; Landroid/view/KeyEvent; I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/BrowserWebView;-documentHasImages-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/NicknameDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$BluetoothStateTracker;-getTriState-(Landroid/content/Context;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/editor/ContactEditorFragment$2;-onRequest-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/text/StringSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/phone/IccNetworkDepersonalizationPanel$3;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/inputmethod/research/ResearchLogger;-richInputConnection_commitText-(Ljava/lang/CharSequence; I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/ui/CropView$HighlightRectangle;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/provider/ContactsContract$Contacts$AggregationSuggestions$Builder;-build-()Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity;-attachPhotoHandler-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Connected;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService$Receiver;-callProviderEnabledLocked-(Ljava/lang/String; Z)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/camera/CameraActivity;-createCameraScreenNail-(Z)Lcom/android/gallery3d/ui/ScreenNail;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$2300-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/GLTextureSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/SyncManager$7;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener$Adapter;-onTextInput-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/Settings$BluetoothSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/telephony/SmsManager;-copyMessageToIcc-([B [B I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/telephony/gsm/SmsManager;-deleteMessageFromSim-(I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/mms/data/Contact$ContactsCache;-getContactInfoForEmailAddress-(Ljava/lang/String;)Lcom/android/mms/data/Contact;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/CallCard;-updateAlreadyDisconnected-(Lcom/android/internal/telephony/CallManager;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/GoogleAccountLogin;-startLoginIfNeeded-(Landroid/app/Activity; Ljava/lang/Runnable;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebViewClassic;-setFrame-(I I I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/SyncManager;-onUserStopping-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-doKeyguardLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Ljava/lang/String; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/SnapshotTab;-postCapture-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/WiredAccessoryManager$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/statusbar/IStatusBarService;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/settings/Settings$ManageApplicationsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/activities/PeopleActivity;-createViewsAndFragments-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/phone/SipCallOptionHandler$1;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_SIP', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/contacts/util/AccountFilterUtil;-handleAccountFilterResult-(Lcom/android/contacts/list/ContactListFilterController; I Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/BluetoothNameDialogFragment;-access$000-(Lcom/android/settings/bluetooth/BluetoothNameDialogFragment; Ljava/lang/String;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/IWindowManager;-setForcedDisplayDensity-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$5900-(Lcom/android/mms/ui/ComposeMessageActivity; Ljava/lang/String; Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/sip/SipService;-access$1300-(Lcom/android/server/sip/SipService; Landroid/net/sip/SipProfile; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/DefaultContactListAdapter;-getContactUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE', 'android.permission.WRITE_SETTINGS', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/common/contacts/BaseEmailAddressAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/DataUsageSummary;-onPrepareOptionsMenu-(Landroid/view/Menu;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/UiccCardApplication;-setIccFdnEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/filterpacks/text/StringLogger;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/am/ActivityManagerService;-access$200-(Lcom/android/server/am/ActivityManagerService; Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Landroid/content/Intent; Ljava/lang/String; Landroid/content/IIntentReceiver; I Ljava/lang/String; Landroid/os/Bundle; Ljava/lang/String; Z Z I I I)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/LocalSource;-<init>-(Landroid/content/Context; Landroid/content/SharedPreferences;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/VibratorService;-onInputDeviceAdded-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/activities/PeopleActivity;-access$700-(Lcom/android/contacts/activities/PeopleActivity; Landroid/net/Uri;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-setSelectedContactUri-(Landroid/net/Uri; Z Z Z Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/PhoneProxy;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/inputmethod/UserDictionaryAddWordContents;-apply-(Landroid/content/Context; Landroid/os/Bundle;)I' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/contacts/list/JoinContactListFragment;-finish-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/development/AccountsTester;-onStart-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/wifi/WifiSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/util/ContactLoaderUtils;-ensureIsContactUri-(Landroid/content/ContentResolver; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/backup/IBackupManager$Stub;-clearBackupData-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-handleTouchEventCommon-(Landroid/view/MotionEvent; I I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setAndBroadcastNetworkSetTime-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/vcard/VCardParserImpl_V21;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$SecuritySettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/SIMRecords;-fetchSimRecords-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_MULTICAST_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub$Proxy;-isProviderEnabled-(Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection;-separate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-incrementPendingBroadcasts-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/accounts/AccountSyncSettings;-updateAccountCheckboxes-([Landroid/accounts/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/contacts/list/JoinContactListFragment;-isSyncActive-()Z' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/vcard/VCardParserImpl_V30;-parseItem-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/SlideViewInterface;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/A2dpProfile;-getConnectionStatus-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/IntRangeManager;-finishUpdate-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkPolicyManager$Stub;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/p2p/WifiP2pManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-handleUserSwitched-(I Landroid/os/IRemoteCallback;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/hardware/Camera;-open-()Landroid/hardware/Camera;' : ['android.permission.CAMERA'],
	'Lcom/android/server/LocationManagerService;-access$800-(Lcom/android/server/LocationManagerService; Lcom/android/server/LocationManagerService$Receiver;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPuk2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-fixTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-onRestore-(Landroid/app/backup/BackupDataInput; I Landroid/os/ParcelFileDescriptor;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$FrequencyConflictState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/SyncManager$SyncHandler;-scheduleReadyPeriodicSyncs-()J' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-adjustStatusBarLocked-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/Tab;-dismissSubWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/LegacyContactListAdapter;-getContactUri-(I Landroid/database/Cursor; I I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/AlbumSetDataLoader;-access$1900-(Lcom/android/gallery3d/app/AlbumSetDataLoader; Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/Invert;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/AccelerometerListener$1;-onSensorChanged-(Landroid/hardware/SensorEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/AccountUnlockScreen;-<init>-(Landroid/content/Context; Landroid/content/res/Configuration; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback; Lcom/android/internal/widget/LockPatternUtils;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/calendar/DayFragment;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mms/data/Conversation;-blockMarkAsRead-(Z)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/BatteryService$9;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/media/AudioService;-access$3000-(Landroid/media/AudioService; Landroid/os/Handler; I I I Ljava/lang/Object; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-connectInt-(Lcom/android/settings/bluetooth/LocalBluetoothProfile;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getContactNamesByNumber-(Ljava/lang/String;)Ljava/util/ArrayList;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransfer;-startConnectSession-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onSetUserDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/server/BluetoothA2dpService;-suspendSink-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/accounts/AccountSyncSettings;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/settings/Settings$StorageUseActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityManagerService;-killApplicationWithAppId-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/webkit/WebViewClassic;-clearView-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/DayView;-access$5000-(Lcom/android/calendar/DayView; Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setPairingConfirmation-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockScreen$GlowPadViewMethods;-onGrabbedStateChange-(Landroid/view/View; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/power/Notifier;-onWirelessChargingStarted-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-handleExitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-prepareNavigationBarView-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/model/ContactLoader;-deliverResult-(Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/launcher2/Folder;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/BluetoothHeadset;-clccResponse-(I I I I Z Ljava/lang/String; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cat/IconLoader;-readClut-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/DownloadService;-access$600-(Lcom/android/providers/downloads/DownloadService; Lcom/android/providers/downloads/DownloadInfo$Reader; J)Lcom/android/providers/downloads/DownloadInfo;' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/sip/SipManager;-unregister-(Landroid/net/sip/SipProfile; Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.USE_SIP'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/ITelephony$Stub;-getNeighboringCellInfo-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendRawPdu-([B [B Landroid/app/PendingIntent; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothHealth;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BookmarkUtils$1$1;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/soundrecorder/Recorder;-restoreState-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-<init>-(Landroid/content/Context; Lcom/android/systemui/statusbar/phone/QuickSettingsContainerView;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-updatePersistentConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView$1;-onCodeInput-(I I I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/view/inputmethod/InputMethodManager;-dispatchTrackballEvent-(Landroid/content/Context; I Landroid/view/MotionEvent; Landroid/view/inputmethod/InputMethodManager$FinishedEventCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/SpeechRecognizer;-putMessage-(Landroid/os/Message;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/accounts/IAccountManager;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/AddBookmarkPage;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-sendUssdResponse-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NetworkStateHandler;-sendNetworkType-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-setAddWidgetEnabled-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/DownloadHandler;-onDownloadStartNoStream-(Landroid/app/Activity; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Z)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter$ReloadTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Landroid/webkit/WebViewCore;-key-(Landroid/view/KeyEvent; I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaSet-(Ljava/lang/String;)Lcom/android/gallery3d/data/MediaSet;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/XLargeUi;-setUseQuickControls-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-prepareEmptyView-()V' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-showChallenge-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getSignalStrength-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IPowerManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/group/GroupDetailFragment$3;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView$2;-onEndBatchInput-(Lcom/android/inputmethod/latin/InputPointers;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/LocationManagerService$Receiver;-binderDied-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView$MyGestureListener;-onDown-(F F)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$StateTracker;-toggleState-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/PanProfile;-<init>-(Landroid/content/Context;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/ActivityController;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-<init>-(Landroid/preference/VolumePreference; Landroid/content/Context; Landroid/widget/SeekBar; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/display/DisplayManagerService;-access$1300-(Lcom/android/server/display/DisplayManagerService; Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/Invert;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/ContactListFilterController$ContactListFilterListener;-onContactListFilterChanged-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/widget/NumberPicker$CustomEditText;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/downloads/StorageManager;-access$100-(Lcom/android/providers/downloads/StorageManager;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-confirmSendMessageIfNeeded-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setPin-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/bluetooth/BluetoothHeadset;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/google/common/io/Resources;-toByteArray-(Ljava/net/URL;)[B' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/widget/DateTimeView;-registerReceivers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/SyncManager$SyncHandler;-runBoundToSyncAdapter-(Landroid/content/SyncManager$ActiveSyncContext; Landroid/content/ISyncAdapter;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/provider/BrowserContract$SyncState;-getWithUri-(Landroid/content/ContentProviderClient; Landroid/accounts/Account;)Landroid/util/Pair;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/BrowserWebView;-computeScroll-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/speech/SpeechRecognizer;-handleCancelMessage-()V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/server/power/PowerManagerService;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$DateTimeSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/EmergencyCallHelper;-placeEmergencyCall-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView$UnlockPatternListener;-onPatternDetected-(Ljava/util/List;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/text/StringSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/Settings$LocalePickerActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-snapToPage-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/Controller;-onPause-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.NFC'],
	'Landroid/view/WindowManagerPolicy;-performHapticFeedbackLw-(Landroid/view/WindowManagerPolicy$WindowState; I Z)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/TabControl;-recreateWebView-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/transaction/RetrieveTransaction;-notifyObservers-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/RuimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onMoveEventInternal-(I I J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/CallContactActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/display/DisplayManagerService$DisplayAdapterListener;-onTraversalRequested-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/BrowserWebView;-freeMemory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/ContextWrapper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/tech/NfcBarcode;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/calendar/agenda/AgendaListView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/model/dataitem/PhotoDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/MediaRecorder;-setAudioSource-(I)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/exchange/EasAccountService;-sendPing-([B I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-showSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/telephony/CommandsInterface;-deactivateDataCall-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/AbstractSyncAdapter;-isSyncable-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/StructuredPostalDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/ConnectivityService;-getLinkProperties-(I)Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/IccCardProxy;-supplyPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/ContactSelectionActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothInputDevice;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$RunningServicesActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/browser/BrowserWebView;-removeJavascriptInterface-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-getNetworkPreference-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/nfc/INfcAdapter$Stub;-setForegroundDispatch-(Landroid/app/PendingIntent; [Landroid/content/IntentFilter; Landroid/nfc/TechListParcel;)V' : ['android.permission.NFC'],
	'Lcom/android/gallery3d/app/PhotoPage;-onPictureCenter-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/sip/SipService$AutoRegistrationProcess;-notCurrentSession-(Landroid/net/sip/ISipSession;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/ConnectivityManager;-setUsbTethering-(Z)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/providers/downloads/DownloadThread;-getFinalStatusForHttpError-(Lcom/android/providers/downloads/DownloadThread$State;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-enable-(Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiStateMachine$WaitForP2pDisableState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/power/ShutdownThread;-shutdownInner-(Landroid/content/Context; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/RingerVolumePreference;-onActivityStop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/hid/HidService;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserSettings;-syncManagedSettings-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewCore;-updateViewport-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcTag;-reconnect-(I)I' : ['android.permission.NFC'],
	'Landroid/location/ILocationManager$Stub;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/settings/Settings$TetherSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/DataUsageSummary$UidDetailTask;-doInBackground-([Ljava/lang/Void;)Lcom/android/settings/net/UidDetail;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/PhotoView;-swipeImages-(F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-onRemoveViewAnimationCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/ThrottleService$MyHandler;-onPollAlarm-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/content/SyncManager$10;-onStatusChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/DataUsageSummary$12;-onWarningChanged-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/providers/contacts/CallLogProvider;-checkVoicemailPermissionAndAddRestriction-(Landroid/net/Uri; Lcom/android/providers/contacts/util/SelectionBuilder;)V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/providers/media/MediaProvider;-makeThumbInternal-(Lcom/android/providers/media/MediaProvider$ThumbData;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.READ_EXTERNAL_STORAGE', 'android.permission.WRITE_EXTERNAL_STORAGE'],
	'Lcom/android/contacts/calllog/CallLogFragment;-unregisterPhoneCallReceiver-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-trySwitchToFirstTouchState-(F F)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/server/LocationManagerService;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/DayView$3;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService$Stub;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/homepages/HomeProvider;-shouldInterceptRequest-(Landroid/content/Context; Ljava/lang/String;)Landroid/webkit/WebResourceResponse;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewCore;-sendMessage-(I Ljava/lang/Object;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-createDevicePreference-(Lcom/android/settings/bluetooth/CachedBluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/LocationManagerService;-isProviderEnabled-(Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/statusbar/policy/NetworkController;-updateWifiState-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-loadAttachment-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-getType-(Landroid/net/Uri;)Ljava/lang/String;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/util/ItemLoadedCallback;-onItemLoaded-(Ljava/lang/Object; Ljava/lang/Throwable;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/AlbumPicker;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/VibratorService;-access$400-(Lcom/android/server/VibratorService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/bluetooth/BluetoothHealth;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ConnectivityService;-setRadio-(I Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexServerSession;-onClose-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/KeyguardDisableHandler$KeyguardTokenWatcher;-acquired-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothPan;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/model/account/AccountTypeWithDataSet;-hasData-(Landroid/content/Context;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/WebViewClassic;-setBackgroundColor-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/Tab;-putInBackground-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/hardware/Camera;-native_setup-(Ljava/lang/Object;)V' : ['android.permission.CAMERA'],
	'Lcom/android/mms/ui/AudioAttachmentView;-stopAudio-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CsimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/CommandQueue$Callbacks;-removeNotification-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setAndBroadcastNetworkSetTime-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/ContentService;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/LatinIME;-loadKeyboard-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/WebViewController;-onReceivedTitle-(Lcom/android/browser/Tab; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/exchange/EasAccountService;-tryProvision-(Lcom/android/exchange/EasSyncService;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingEncryptedRfcommOn-(I)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$StateTracker;-requestStateChange-(Landroid/content/Context; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/PhotoPage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/providers/media/MediaProvider;-getAlbumArtOutputUri-(Lcom/android/providers/media/MediaProvider$DatabaseHelper; Landroid/database/sqlite/SQLiteDatabase; J Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.READ_EXTERNAL_STORAGE', 'android.permission.WRITE_EXTERNAL_STORAGE'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-onDeviceAdded-(Lcom/android/settings/bluetooth/CachedBluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/util/StateMachine$SmHandler$QuittingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/IccFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/camera/CameraModule;-onPauseBeforeSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/transaction/SmsReceiverService;-access$300-(Lcom/android/mms/transaction/SmsReceiverService; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/calllog/CallLogFragment;-registerPhoneCallReceiver-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/bluetooth/DockService;-handleUndocked-(Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/Gallery;-startViewAction-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/widget/PinnedHeaderListAdapter;-bindView-(Landroid/view/View; I Landroid/database/Cursor; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/google/common/io/ByteStreams;-equal-(Lcom/google/common/io/InputSupplier; Lcom/google/common/io/InputSupplier;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/data/DataManager;-getPanoramaSupport-(Lcom/android/gallery3d/data/Path; Lcom/android/gallery3d/data/MediaObject$PanoramaSupportCallback;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/InputStreamSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewInputDispatcher;-updateStateTrackersLocked-(Landroid/webkit/WebViewInputDispatcher$DispatchEvent; Landroid/view/MotionEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/WifiService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/LocalAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexClientSession$ClientThread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$UpdateContent;-call-()Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/SlideshowEditActivity$SlideListAdapter;-createViewFromResource-(I Landroid/view/View; I)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/EasOutboxService;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity;)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/SearchPanelView;-access$200-(Lcom/android/systemui/SearchPanelView;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/power/PowerManagerService;-systemReady-(Lcom/android/server/TwilightService; Lcom/android/server/dreams/DreamManagerService;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/model/dataitem/NoteDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/wifi/WifiStateMachine;-startTethering-(Ljava/util/ArrayList;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-switchHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SmsManager;-deleteMessageFromIcc-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/calendar/DayView;-doLongPress-(Landroid/view/MotionEvent;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/list/JoinContactListFragment;-setQueryString-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$StateTracker;-getTriState-(Landroid/content/Context;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$2;-doInBackground-([Ljava/lang/Void;)Landroid/util/Pair;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WifiService;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManagerService;-checkReadAccountsPermission-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-access$500-(Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-notifyApnIdDisconnected-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/UserDictionarySettings;-deleteWord-(Ljava/lang/String; Ljava/lang/String; Landroid/content/ContentResolver;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-queueNextSignalStrengthPoll-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/os/IPowerManager;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$BluetoothSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$WirelessSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/HTML5Audio;-onAudioFocusChange-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/contacts/DatabaseModifier;-update-(Ljava/lang/String; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/browser/view/PieStackView;-layout-(I I Z F I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/BrowserContract$Settings;-setSyncEnabled-(Landroid/content/Context; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/IBluetoothInputDevice;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/ContactEditorFragment$3$1;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothHeadsetPhone;-hangupCall-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/MediaPlaybackService;-open-([J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/widget/PinnedHeaderListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-refreshProfilePreference-(Landroid/preference/CheckBoxPreference; Lcom/android/settings/bluetooth/LocalBluetoothProfile;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/TwilightService$LocationHandler;-retrieveLocation-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/CameraActivity;-access$100-(Lcom/android/camera/CameraActivity; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/BrowserWebView;-clearSslPreferences-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/CallDetailActivity$1UpdateContactDetailsTask;-doInBackground-([Ljava/lang/Void;)[Lcom/android/contacts/PhoneCallDetails;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/AllInOneActivity$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-showMessageDetails-(Lcom/android/mms/ui/MessageItem;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/GlobalActions;-<init>-(Landroid/content/Context; Landroid/view/WindowManagerPolicy$WindowManagerFuncs;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/inputmethod/InputMethodAndLanguageSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/IWindowSession$Stub$Proxy;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/editor/ContactEditorFragment$1;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/media/effect/effects/ContrastEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-startMsgListQuery-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-hostChanged-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getCLIR-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/CallNotifier;-onQueryComplete-(I Ljava/lang/Object; Lcom/android/internal/telephony/CallerInfo;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-handleClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-switchInCallAudio-(Lcom/android/phone/InCallScreen$InCallAudioMode;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-onRemoveViewAnimationCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker$TimerProxy;-startKeyRepeatTimer-(Lcom/android/inputmethod/keyboard/PointerTracker;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/vcard/VCardParserImpl_V40;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/accounts/AccountManager;-getAuthTokenByFeatures-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/app/Activity; Landroid/os/Bundle; Landroid/os/Bundle; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/nfc/NfcService$TagService;-close-(I)I' : ['android.permission.NFC'],
	'Lcom/android/browser/PieControl$TabAdapter$1;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/ui/MessageListItem;-access$500-(Lcom/android/mms/ui/MessageListItem; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/exchange/adapter/AbstractSyncAdapter;-cleanup-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/calendar/CalendarProvider2;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCurrentPreferredNetworkType-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/exchange/EasAccountService;-sync-(Lcom/android/exchange/adapter/AbstractSyncAdapter;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/google/common/io/Resources$1;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/mms/ui/ConversationListItem$1;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/WakeLockHoldingProgressListener;-onProgressStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.CALL_PHONE', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/quicksearchbox/google/AbstractGoogleSource;-queryExternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/activities/PeopleActivity;-onSearchRequested-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/SupplicantStateTracker$DisconnectedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/AsyncPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DayFragment;-eventsChanged-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/EventInfoFragment;-access$4300-(Lcom/android/calendar/EventInfoFragment;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/media/effect/effects/BlackWhiteEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/wifi/WifiSettings;-access$000-(Lcom/android/settings/wifi/WifiSettings; Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/data/ComboAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-deleteSurroundingText-(I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebView;-requestFocusNodeHref-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-checkOtaspStateOnResume-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/phone/DataUsage;-onResume-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/BackupManagerService$PerformInitializeTask;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-notifyOffApnsOfAvailability-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phone/CallNotifier;-access$1000-(Lcom/android/phone/CallNotifier;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-access$1400-(Lcom/android/bluetooth/btservice/BondStateMachine; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreen;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/development/SyncAdapterDriver;-startSync-(Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService;-enforceChangePermission-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetooth;-getAdapterConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Tab;-restoreState-(Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Disconnected;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebViewClassic;-loadUrl-(Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/BreadCrumbView;-popView-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-setNdefPushCallback-(Landroid/nfc/INdefPushCallback;)V' : ['android.permission.NFC'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-unpair-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cat/IconLoader;-postIcon-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getMute-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/ScrollerView;-onLayout-(Z I I I I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/agenda/AgendaListView$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/nfc/NfcService;-onCardEmulationDeselected-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/net/ConnectivityManager;-getAllNetworkInfo-()[Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-broadcastIntentLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Landroid/content/Intent; Ljava/lang/String; Landroid/content/IIntentReceiver; I Ljava/lang/String; Landroid/os/Bundle; Ljava/lang/String; Z Z I I I)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/gsm/SIMFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/VignetteEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$LocalePickerActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/app/ActivityThread$ApplicationThread;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/SlideshowPage$SequentialSource;-findItemIndex-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Connected;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/AccountManager;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor;-reportSimUnlocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/settings/Settings$WifiP2pSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/util/StateMachine$SmHandler$HaltingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/CommandsInterface;-setFacilityLockForApp-(Ljava/lang/String; Z Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity;-handleSelectSyncedCalendarsClicked-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/DayFragment$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/AccessibilitySettings$ToggleAccessibilityServicePreferenceFragment$1;-register-(Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/data/Conversation;-startDelete-(Lcom/android/mms/data/Conversation$ConversationQueryHandler; I Z Ljava/util/Collection;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/net/sip/SipManager;-getSessionFor-(Landroid/content/Intent;)Landroid/net/sip/SipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/browser/AddBookmarkPage;-switchToDefaultView-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/AudioManager;-setMode-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/contacts/activities/TransactionSafeActivity;-onPrepareOptionsMenu-(Landroid/view/Menu;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/content/AbstractThreadedSyncAdapter$ISyncAdapterImpl;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/power/ShutdownThread;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/ConnectivityService;-stopUsingNetworkFeature-(Lcom/android/server/ConnectivityService$FeatureUser; Z)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/provider/BrowserProvider2$DatabaseHelper;-enableSync-(Landroid/database/sqlite/SQLiteDatabase;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter;-access$700-(Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/emailcommon/Device;-getDeviceIdInternal-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/FilterTypeSet$2;-consume-(I Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/UndoBarView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/preferences/FontSizePreview;-onCreateView-(Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager;-show-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-updateWakefulnessLocked-(I)Z' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothHealth;-getHealthDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/statusbar/RotationToggle;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewInputDispatcher;-postLongPress-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/DTMFTwelveKeyDialerView;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-enableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/vcard/VCardEntry;-iterateOneList-(Ljava/util/List; Lcom/android/vcard/VCardEntry$EntryElementIterator;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetooth$Stub;-setDiscoverableTimeout-(I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasAccountService;-sync-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/IBluetoothManager;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/PhotoView;-setFilmMode-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/CharStreams;-toStringBuilder-(Lcom/google/common/io/InputSupplier;)Ljava/lang/StringBuilder;' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/model/dataitem/NicknameDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothHeadset;-getBatteryUsageHint-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/TwilightService$LocationHandler;-enableLocationUpdates-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-showMessageDetails-(Lcom/android/mms/ui/MessageItem;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/WebViewController;-onPageFinished-(Lcom/android/browser/Tab;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/VideoController;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/os/BatteryStatsImpl;-setBatteryState-(I I I I I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/view/ViewRootImpl;-deliverKeyEvent-(Landroid/view/ViewRootImpl$QueuedInputEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor;-watchForDeviceProvisioning-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-getAllMessagesFromIccEf-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-access$2500-(Lcom/android/calendar/agenda/AgendaWindowAdapter; Lcom/android/calendar/agenda/AgendaWindowAdapter$QuerySpec;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/AbstractSyncService;-validate-(Ljava/lang/Class; Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.CALL_PHONE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/vcard/VCardParser_V21;-parse-(Ljava/io/InputStream; Lcom/android/vcard/VCardInterpreter;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetooth$Stub;-getRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-createProfilePreference-(Lcom/android/settings/bluetooth/LocalBluetoothProfile;)Landroid/preference/CheckBoxPreference;' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-toggleRecentApps-()V' : ['android.permission.GET_TASKS'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStartingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/IccCardProxy;-setVoiceRadioTech-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothPbap;-disconnect-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/NativeDaemonConnector;-listenToSocket-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/test/mock/MockContentResolver;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$TetheredState;-enter-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity$2;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/LatinIME;-hapticAndAudioFeedback-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfile;-getPreferred-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-onMaxPendingMessagesReached-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/net/ConnectivityManager;-getActiveNetworkQuotaInfo-()Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/ExchangeService;-registerCalendarObserver-(Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/alerts/AlertReceiver;-beginStartingService-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/OverlayDisplayAdapter$1;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/XLargeUi;-onBackKey-()Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/os/PowerManager$WakeLock$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$2;-onPostExecute-(Ljava/lang/Exception;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/mms/ui/ManageSimMessages;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$InitialState;-turnOffUpstreamMobileConnection-()Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/IActivityManager;-updateConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothA2dp;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-access$300-(Lcom/android/gallery3d/ui/ActionModeHandler; Lcom/android/gallery3d/util/ThreadPool$JobContext;)Landroid/content/Intent;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/SecuritySettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/UndoBarView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/view/IWindowManager$Stub$Proxy;-setForcedDisplayDensity-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-performAccessibilityAction-(I Landroid/os/Bundle;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/Controller;-reuseTab-(Lcom/android/browser/Tab; Lcom/android/browser/IntentHandler$UrlData;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/widget/ToggleButton;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManager$5;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/ActivityController;-handleNewIntent-(Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/bluetooth/BluetoothDeviceFilter$BondedDeviceFilter;-matches-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity$DisplayAdapter;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/InCallScreen;-internalResolveIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-startFreezingScreen-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/app/IActivityManager;-getTasks-(I I Landroid/app/IThumbnailReceiver;)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendCdmaSms-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/EdgeView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/exchange/EasSyncService;-sync-(Lcom/android/exchange/adapter/AbstractSyncAdapter;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover;-nextStepConnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/effect/SingleFilterEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaByDayAdapter$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapUtils;-getProfileName-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/DevelopmentSettings;-writeAppProcessLimitOptions-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getNetworkPreference-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPasswordView;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hdp/HealthService;-getConnectedHealthDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/IccCardProxy;-updateQuietMode-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcAdapter$Stub;-disableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/CallCard;-updateState-(Lcom/android/internal/telephony/CallManager;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$UserDictionarySettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-queryCLIP-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PicasaSource;-getStream-(Lcom/android/dreams/phototable/PhotoSource$ImageData; I)Ljava/io/InputStream;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-resetStateLocked-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/AccessibilitySettings$ToggleAccessibilityServicePreferenceFragment;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/GroupDetailActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/widget/VideoView;-setVideoPath-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/interactions/ImportExportDialogFragment$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/BandMode;-loadBandList-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5VideoFullScreen;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-startScan-(Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/calendar/CalendarController;-refreshCalendars-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/SlideshowPage;-access$100-(Lcom/android/gallery3d/app/SlideshowPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/DeviceInfoSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/effect/effects/DocumentaryEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService$Receiver;-callLocationChangedLocked-(Landroid/location/Location;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/DialogPicker;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-notifyApnIdDisconnected-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/TrimVideo$4;-run-()V' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/DeliveryReportActivity;-initListAdapter-()V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/preferences/WebViewPreview;-updatePreview-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/BackupManagerService$ActiveRestoreSession;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/DbModifierWithNotification;-delete-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-getConnectedHealthDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/EasAccountService;-acknowledgeRemoteWipe-(Lcom/android/exchange/EasSyncService; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-setRadio-(I Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneProxy;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/GlobalActions$Action;-onLongPress-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-updateCurrentIndex-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView;-displayDefaultSecurityMessage-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/contacts/model/account/AccountWithDataSet;-hasData-(Landroid/content/Context;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/calendar/EventLoader$LoaderThread;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/providers/downloads/StorageManager;-discardPurgeableFiles-(I J)J' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-syncBluetoothState-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/wm/WindowManagerService;-requestTraversal-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-verifyUnlockLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/media/AudioService;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/wifi/AdvancedWifiSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-transceive-(I [B Z)Landroid/nfc/TransceiveResult;' : ['android.permission.NFC'],
	'Lcom/android/providers/contacts/CallLogProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/quicksearchbox/google/GoogleSource;-queryExternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/RIL;-send-(Lcom/android/internal/telephony/RILRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/OutputStreamTarget;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/browser/BrowserBookmarksPage;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/view/IWindowManager$Stub;-prepareAppTransition-(I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/ContactDetailActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$BluetoothStateTracker$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/ContactDetailActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/exchange/ExchangeService;-stopAccountSyncs-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-goToSleepFromNative-(J I)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-onStart-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkStatsService;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;-onDeviceNameUpdated-(Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/model/dataitem/SipAddressDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/location/GpsLocationProvider;-requestRefLocation-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager$Stub;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/activities/GroupBrowserActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Landroid/media/AudioManager;-setBluetoothA2dpDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice; I)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/IsoDep;-setTimeout-(I)V' : ['android.permission.NFC'],
	'Landroid/webkit/WebViewClassic;-destroyJava-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/event/EditEventFragment$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/WebViewClassic;-onWindowFocusChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/detail/ContactLoaderFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/widget/IndexerListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/homepages/RequestHandler;-writeTemplatedIndex-()V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity;-addEditableRawContact-(Landroid/content/Context; Lcom/android/contacts/model/RawContactDeltaList;)Lcom/android/contacts/model/RawContactDelta;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/UndoBarView;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-conference-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/SubSettings;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/Dictionary;-getSuggestionsWithSessionId-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo; I)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/exchange/EasSyncService;-validate-(Ljava/lang/Class; Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/Phone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/server/ConnectivityService;-startUsingNetworkFeature-(I Ljava/lang/String; Landroid/os/IBinder;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/JoinContactListAdapter;-bindQuickContact-(Lcom/android/contacts/list/ContactListItemView; I Landroid/database/Cursor; I I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/BluetoothPhoneService$3;-hangupCall-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-syncReloadDictionaryInternal-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/activities/AttachPhotoActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifySeFieldActivated-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/nfc/tech/NdefFormatable;-format-(Landroid/nfc/NdefMessage; Z)V' : ['android.permission.NFC'],
	'Lcom/android/phone/EnableFdnScreen;-enableFdn-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/interactions/ImportExportDialogFragment;-access$000-(Lcom/android/contacts/interactions/ImportExportDialogFragment; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/sip/SipSettings;-handleSipReceiveCallsOption-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE', 'android.permission.WRITE_SETTINGS', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine$WallpaperObserver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter;-access$2200-(Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter; I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothDevice;-getAliasName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/content/SyncManager;-doDatabaseCleanup-()V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/nfc/NfcAdapter;-disableForegroundDispatchInternal-(Landroid/app/Activity; Z)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/CommandsInterface;-dial-(Ljava/lang/String; I Lcom/android/internal/telephony/UUSInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(J F Landroid/location/Criteria; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/AlarmKlaxon;-stop-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-cleanUpConnection-(Z Ljava/lang/String; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/ui/ComposeMessageActivity$1;-run-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/camera/SecureCameraActivity;-closeModule-(Lcom/android/camera/CameraModule;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/DataUsageSummary$8;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewProvider$ViewDelegate;-setBackgroundColor-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/BrowserContract$SyncState;-newSetOperation-(Landroid/accounts/Account; [B)Landroid/content/ContentProviderOperation;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/music/AudioPreview;-onUserLeaveHint-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-isTemplateRelevant-(Landroid/net/NetworkTemplate;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/location/ILocationManager;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/browser/WallpaperHandler;-run-()V' : ['android.permission.SET_WALLPAPER', 'android.permission.INTERNET'],
	'Lcom/android/server/BackupManagerService$PerformFullBackupTask$FullBackupRunner;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/webkit/WebViewProvider;-goBackOrForward-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ConversationList;-access$100-(Lcom/android/mms/ui/ConversationList;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/phone/BluetoothPhoneService;-access$100-(Lcom/android/phone/BluetoothPhoneService; Landroid/telephony/ServiceState;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/CropFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-onReplay-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/sip/SipEditor;-deleteAndUnregisterProfile-(Landroid/net/sip/SipProfile;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/calendar/DayView;-switchViews-(Z F F F)Landroid/view/View;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/data/LocalAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStartedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-computeNewConfiguration-()Landroid/content/res/Configuration;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/ZeroTopPaddingTextView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/IAudioService$Stub$Proxy;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/net/nsd/INsdManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-onJoinCompleted-(Landroid/net/Uri;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/IConnectivityManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/nfc/INfcAdapterExtras$Stub$Proxy;-getCardEmulationRoute-(Ljava/lang/String;)I' : ['android.permission.NFC'],
	'Lcom/android/musicfx/ActivityMusic$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/settings/widget/ChartSweepView$OnSweepListener;-onSweep-(Lcom/android/settings/widget/ChartSweepView; Z)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView$AccountAnalyzer;-start-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-findImages-(I)Ljava/util/Collection;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/ui/CameraSwitcher$CameraSwitchListener;-onCameraSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/BrowserSettings;-clearCache-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/development/EnterURL$UrlEditText;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/CallLogAsync$GetLastOutgoingCallTask;-doInBackground-([Lcom/android/phone/CallLogAsync$GetLastOutgoingCallArgs;)Ljava/lang/String;' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/gallery3d/app/CropImage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/wifi/WifiManager$WifiLock;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/AudioPreview;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity$34;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/telephony/SmsManager;-getAllMessagesFromIcc-()Ljava/util/ArrayList;' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-addDeviceCategory-(Landroid/preference/PreferenceGroup; I Lcom/android/settings/bluetooth/BluetoothDeviceFilter$Filter;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/ComboAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncManager$SyncAlarmIntentReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHealth;-registerAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/IBluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-pullPhonebook-([B Lcom/android/bluetooth/pbap/BluetoothPbapObexServer$AppParamValue; Ljavax/obex/HeaderSet; Ljavax/obex/Operation; Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/wallpaper/nexus/NexusRS;-start-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/INetworkPolicyManager;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/nfc/tech/NfcF;-setTimeout-(I)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/PhoneProxy;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewBase;-handleMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService$BootCompletedReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter;-parse-(Ljava/io/InputStream;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitorCallback;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-updateLockdownVpn-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$1;-onRefreshCarrierInfo-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-postUrl-(Ljava/lang/String; [B)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/UiccCardApplication;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/exchange/ExchangeService;-access$400-(Lcom/android/exchange/ExchangeService; J Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/WifiService;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/ExpandHelper;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/phone/PhoneUtils;-setMute-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/EventDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/LocalePicker;-updateLocale-(Ljava/util/Locale;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ManageSimMessages;-deleteAllFromSim-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/videoeditor/VideoEditorActivity$23;-onStart-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcAdapter;-enableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewClassic;-setJsFlags-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManagerService;-getAccounts-(I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiStateMachine;-handleNetworkDisconnect-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phone/OtaUtils;-onClickOtaTryAgainButton-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator$1;-onClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothAdapter;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/SnailAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/preferences/FontSizePreview;-updatePreview-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/LocationManagerService;-applyRequirementsLocked-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavScreen;-close-(I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/accounts/AccountManager;-editProperties-(Ljava/lang/String; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-onScrubbingStart-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/FisheyeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-access$1200-(Lcom/android/server/VibratorService; J)V' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/IBluetoothHeadset;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ims/IsimUiccRecords;-fetchIsimRecords-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-unloadMediaPackages-(Ljava/util/HashMap; [I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/webkit/HTML5VideoInline;-prepareDataCommon-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-executeAndWait-(Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/data/Conversation;-startQueryForAll-(Landroid/content/AsyncQueryHandler; I)V' : ['android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/AlbumSetPage$MyDetailsSource;-setIndex-()I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/interactions/ContactDeletionInteraction;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-pokeWakelock-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onStateResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/power/ShutdownThread$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/BluetoothSocket;-initSocketFromFdNative-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/uicc/UiccController;-onGetIccCardStatusDone-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/TetherSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/GoogleAccountLogin;-run-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onRoamingOn-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PasswordUnlockScreen$2;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/providers/media/MediaProvider;-openFileAndEnforcePathPermissionsHelper-(Landroid/net/Uri; Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.READ_EXTERNAL_STORAGE', 'android.permission.WRITE_EXTERNAL_STORAGE'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-doInBackground-([Landroid/filterfw/core/SyncRunner;)Landroid/filterfw/core/AsyncRunner$RunnerResult;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/wifi/WifiStatusTest;-handleScanResultsAvailable-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/nfc/tech/NfcF;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setDiscoverableTimeout-(I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/sip/SipService;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-init-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-onDestroy-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/calllog/CallLogAdapter$QueryThread;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/tech/NfcF;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-onCreate-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/WiredAccessoryManager$WiredAccessoryObserver;-updateStateLocked-(Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService;-setShuffleMode-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitorCallback;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/pm/PackageManagerService;-access$1900-(Lcom/android/server/pm/PackageManagerService; Lcom/android/server/pm/PackageManagerService$InstallArgs; Z Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/DateTimeSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-repositionNavigationBar-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-dispatchMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/phone/PhoneUtils;-mergeCalls-(Lcom/android/internal/telephony/CallManager;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/SlideshowView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupForegroundResumeBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-removeBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-isWifiEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/BluetoothManagerService$BluetoothServiceConnection;-onServiceDisconnected-(Landroid/content/ComponentName;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/CallNotifier$InCallTonePlayer;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/filterfw/core/FinalPort;-transfer-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-access$200-(Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;)Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/GalleryActionBar$OnAlbumModeSelectedListener;-onAlbumModeSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/BrowserWebView;-findNext-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/http/RequestQueue;-setProxyConfig-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setPreferredVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangup-(Lcom/android/internal/telephony/cdma/CdmaConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/inputmethod/InputMethodManager$1;-onUnbindMethod-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-connectSocket-(Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/CalendarSyncAdapterService;-performSync-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle; Ljava/lang/String; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/browser/PieControl;-access$000-(Lcom/android/browser/PieControl;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/Controller$SaveSnapshotTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Long;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/BluetoothPan;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiStateMachine;-access$17800-(Landroid/net/wifi/WifiStateMachine; Ljava/util/ArrayList;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetooth;-fetchRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/telephony/MmsSmsProvider;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/phone/InCallScreen;-handlePostOnDialChars-(Landroid/os/AsyncResult; C)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/SlotView$SimpleListener;-onSingleTapUp-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$IntentBroadcastHandler;-onConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/PhotoPage;-access$1800-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame$1;-onBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/ViewRootImpl$ConsumeBatchedInputRunnable;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewCore;-setupViewport-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/PhotoView;-captureAnimationDone-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/SlotView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/widget/SlidingTab$OnTriggerListener;-onGrabbedStateChange-(Landroid/view/View; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleMove-(Landroid/view/MotionEvent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$ManageApplicationsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/emailcommon/service/IEmailService;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.READ_CALENDAR'],
	'Landroid/webkit/WebView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I Z)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-onLowMemory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumSetPage$7;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/media/Ringtone;-setUri-(Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/util/NotifyingAsyncQueryHandler$AsyncQueryListener;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/Tab$4;-onDownloadStart-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS', 'android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/dreams/phototable/PicasaSource;-fillQueue-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/BackupManagerService$ActiveRestoreSession;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-update-(I J Landroid/content/ContentValues;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onResume-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-getStatistics-()[B' : ['android.permission.BATTERY_STATS', 'android.permission.BLUETOOTH'],
	'Lcom/android/browser/Controller;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/TransactionSafeActivity;-onCreateDialog-(I Landroid/os/Bundle;)Landroid/app/Dialog;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-onScreenTurnedOn-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-access$1500-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/camera/LocationManager;-recordLocation-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter$EasContactsSyncParser;-deleteParser-(Lcom/android/exchange/adapter/ContactsSyncAdapter$ContactOperations;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/Transaction;-sendPdu-(J [B)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/ContactsApplication;-getSystemService-(Ljava/lang/String;)Ljava/lang/Object;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/SyncManager$ActiveSyncContext;-onServiceDisconnected-(Landroid/content/ComponentName;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-showTimeoutDialog-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/ConfirmLockPassword$ConfirmLockPasswordFragment;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-reportStkServiceIsRunning-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService$SettingsObserver;-<init>-(Lcom/android/server/InputMethodManagerService; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManagerService;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-onItemClick-(I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/Settings$ManageAccountsSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/DayView;-setSelectionFromPosition-(I I Z)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/RIL;-changeIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-onPageBeginMoving-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/BackupManagerService$BackupRestoreTask;-execute-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/BrowserSettings;-syncSetting-(Landroid/webkit/WebSettingsClassic;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/GrainFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkStatsService;-access$1100-(Lcom/android/server/net/NetworkStatsService; I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityModel;-getAlternateFor-(Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityModel$SecurityMode;)Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityModel$SecurityMode;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/LocalAlbumSet$AlbumsLoader;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/ui/ComposeMessageActivity$42;-onContentChanged-(Lcom/android/mms/ui/MessageListAdapter;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetoothHealth;-getHealthDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-hasContentChanged-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-callListenerOnPressAndCheckKeyboardLayoutChange-(Lcom/android/inputmethod/keyboard/Key;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/ComboAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine;-access$900-(Lcom/android/bluetooth/a2dp/A2dpStateMachine; Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/widget/ChartSweepView;-dispatchOnSweep-(Z)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/ims/IsimUiccRecords;-setVoiceCallForwardingFlag-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-createBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-onPause-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-ensureActiveMobilePolicyLocked-()V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/LabeledEditorView;-onOptionalFieldVisibilityChange-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/LocationManagerService;-access$200-(Lcom/android/server/LocationManagerService;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/NfcHandler;-register-(Landroid/app/Activity; Lcom/android/browser/Controller;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/CommandsInterface;-setRadioPower-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-removeFence-(Landroid/location/Geofence; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/ex/variablespeed/VariableSpeed;-prepare-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothPairingDialog;-onPair-(Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/BatteryService;-processValuesLocked-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/VibratorService;-doCancelVibrateLocked-()V' : ['android.permission.VIBRATE'],
	'Landroid/app/ActivityThread$ApplicationThread;-scheduleRegisteredReceiver-(Landroid/content/IIntentReceiver; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/BluetoothManagerService$1;-onBluetoothStateChange-(I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$7300-(Lcom/android/mms/ui/ComposeMessageActivity; I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/alerts/AlertUtils;-formatTimeLocation-(Landroid/content/Context; J Z Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SyncStateTracker;-onActualStateChange-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-findAllBody-(Ljava/lang/String; Z)I' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/contacts/list/ContactPickerFragment;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/AccessibilityInjector;-updateJavaScriptEnabled-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/dialpad/DialpadImageButton$OnPressedListener;-onPressed-(Landroid/view/View; Z)V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/Controller$10;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsFragment;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-onProfileStateChanged-(Lcom/android/settings/bluetooth/LocalBluetoothProfile; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/EdgeView;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-initializeP2pSettings-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-insertShare-(Landroid/database/Cursor; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/BaseCommands;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/server/search/SearchManagerService$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/data/WorkingMessage;-updateDraftSmsMessage-(Lcom/android/mms/data/Conversation; Ljava/lang/String;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$4;-onClickHandler-(Landroid/view/View; Landroid/app/PendingIntent; Landroid/content/Intent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$3000-(Lcom/android/mms/ui/ComposeMessageActivity; J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/ui/CropView$HighlightRectangle;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/adapter/Search$SearchParser;-parseStore-()Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-onWakeKeyWhenKeyguardShowingTq-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneFactory;-getGsmPhone-()Lcom/android/internal/telephony/Phone;' : ['android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/WebViewTimersControl;-onBrowserActivityPause-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/model/dataitem/RelationDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/VibratorUtils;-vibrate-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-onBluetoothStateChanged-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebIconDatabaseClassic$EventHandler;-postMessage-(Landroid/os/Message;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/model/ContactLoader;-loadInBackground-()Ljava/lang/Object;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/media/AudioService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-update-(Lcom/android/internal/telephony/DriverCall;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothA2dp;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity$AccountDisplay;-buildDiff-(Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/ims/IsimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/TransactionSafeActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-addPreferencesForActivity-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calculator2/CalculatorEditText;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/AudioAttachmentView$1;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/ActionModeHandler$ActionModeListener;-onActionItemClicked-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/accounts/AccountSyncSettings;-onPrepareOptionsMenu-(Landroid/view/Menu;)V' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/systemui/statusbar/CommandQueue$Callbacks;-toggleRecentApps-()V' : ['android.permission.GET_TASKS'],
	'Lcom/android/inputmethod/research/ResearchLogger$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-getNfcAdapterExtrasInterface-(Ljava/lang/String;)Landroid/nfc/INfcAdapterExtras;' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/detail/ContactLoaderFragment$1;-onLoadFinished-(Landroid/content/Loader; Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/view/IWindowManager$Stub$Proxy;-prepareAppTransition-(I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/provider/Browser;-clearHistory-(Landroid/content/ContentResolver;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/nfc/INfcTag$Stub;-reconnect-(I)I' : ['android.permission.NFC'],
	'Landroid/net/sip/SipManager;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Landroid/bluetooth/IBluetooth;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/wifi/WifiSettings;-handleEvent-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/activities/PeopleActivity;-setupContactDetailFragment-(Landroid/net/Uri;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/AccountManager$10;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/DataConnectionTracker$DataRoamingSettingObserver;-register-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/BrowserWebView;-requestFocusNodeHref-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/WebViewController;-onPageStarted-(Lcom/android/browser/Tab; Landroid/webkit/WebView; Landroid/graphics/Bitmap;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/CdmaSystemSelectListPreference$MyHandler;-handleSetCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$InitialState;-enter-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/accounts/AccountManager$AmsTask;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/ConnectivityManager;-isTetheringSupported-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/content/ISyncAdapter$Stub$Proxy;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/dreams/phototable/PicasaSource;-load-(Lcom/android/dreams/phototable/PhotoSource$ImageData; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-setNetworkPreference-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory$3$1;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/model/dataitem/StructuredNameDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/IsoDep;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/server/wm/WindowManagerService;-updateOrientationFromAppTokensLocked-(Landroid/content/res/Configuration; Landroid/os/IBinder;)Landroid/content/res/Configuration;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/os/IVibratorService$Stub;-vibrate-(J Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingRfcommWithServiceRecord-(Ljava/lang/String; Ljava/util/UUID;)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$MyDownUpListener;-onDown-(Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/camera/ActivityBase;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/systemui/SearchPanelView;-show-(Z Z)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/InputMethodManagerService;-needsToShowImeSwitchOngoingNotification-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/TelephonyManager;-getSimSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/NfcHandler;-unregister-(Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Lcom/android/settings/wifi/WifiStatusTest;-access$200-(Lcom/android/settings/wifi/WifiStatusTest;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getRemoteClass-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/RuimSmsInterfaceManager;-enforceReceiveAndSend-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/calendar/widget/CalendarAppWidgetProvider;-onUpdate-(Landroid/content/Context; Landroid/appwidget/AppWidgetManager; [I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/vcard/VCardEntry;-isIgnorable-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/vcard/VCardEntry$EmailData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothPbap;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/MediaPlaybackService;-getNextPosition-(Z)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/Ringer;-ring-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/am/ActivityManagerService$20;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/effect/effects/FlipEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/SoundRecorder;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO'],
	'Landroid/media/AsyncPlayer;-play-(Landroid/content/Context; Landroid/net/Uri; Z I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/IWifiP2pManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/IActivityManager;-getRecentTasks-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TwilightService$TwilightListener;-onTwilightStateChanged-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/UiController;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/power/Notifier;-sendGoToSleepBroadcast-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/CaptivePortalTracker$NoActiveNetworkState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/net/NetworkStatsService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$6200-(Lcom/android/bluetooth/hfp/HeadsetStateMachine;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-proceedAfterWildChar-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/IccProvider;-deleteIccRecordFromEf-(I Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter$EasContactsSyncParser;-commit-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/wifi/AdvancedWifiSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/IAccountManager$Stub;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-runAfterDetection-(Landroid/location/Country; Landroid/location/Country; Z Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$3700-(Lcom/android/mms/ui/ComposeMessageActivity; J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/systemui/statusbar/tablet/SettingsView;-onClickNetwork-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/app/IActivityManager;-handleApplicationWtf-(Landroid/os/IBinder; Ljava/lang/String; Landroid/app/ApplicationErrorReport$CrashInfo;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mms/util/ThumbnailManager;-getVideoThumbnail-(Landroid/net/Uri; Lcom/android/mms/util/ItemLoadedCallback;)Lcom/android/mms/util/ItemLoadedFuture;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame;-onAttachedToWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator$1;-onDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/data/Conversation$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCdmaBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/Recorder;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/common/contacts/BaseEmailAddressAdapter$DefaultPartitionFilter;-performFiltering-(Ljava/lang/CharSequence;)Landroid/widget/Filter$FilterResults;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/CallerInfoAsyncQuery$OnQueryCompleteListener;-onQueryComplete-(I Ljava/lang/Object; Lcom/android/internal/telephony/CallerInfo;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/location/GpsLocationProvider;-access$2000-(Lcom/android/server/location/GpsLocationProvider; I Landroid/net/NetworkInfo;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-showLocked-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewBase;-onScreenTurnedOn-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/QuickContactDivot;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-setRepeatMode-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity$QueryEntitiesTask;-queryForContactId-(Landroid/content/ContentResolver; J)J' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Pending;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/IWifiManager$Stub;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/nsd/NsdManager;-<init>-(Landroid/content/Context; Landroid/net/nsd/INsdManager;)V' : ['android.permission.INTERNET'],
	'Lcom/android/server/wm/WindowManagerService;-unsetAppFreezingScreenLocked-(Lcom/android/server/wm/AppWindowToken; Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/NfcF;-getTimeout-()I' : ['android.permission.NFC'],
	'Lcom/android/settings/WirelessSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/SyncManager;-access$1000-(Landroid/content/SyncManager; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity$CustomFilterConfigurationLoader;-loadInBackground-()Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/MccTable;-setTimezoneFromMccIfNeeded-(Landroid/content/Context; I)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/browser/TabControl;-createNewTab-(Landroid/os/Bundle; Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$AudioOn;-processAudioEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/imageproc/TintFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$InputMethodAndLanguageSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/google/common/io/ByteStreams$4;-getInput-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Landroid/webkit/AutoCompletePopup;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCallWaiting-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-access$1500-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothDiscoverableEnabler;-updateCountdownSummary-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/CalendarController;-sendEvent-(Ljava/lang/Object; Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-conference-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/media/IAudioService$Stub$Proxy;-startBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getRemoteAlias-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/app/ResolverActivity;-onCreate-(Landroid/os/Bundle; Landroid/content/Intent; Ljava/lang/CharSequence; [Landroid/content/Intent; Ljava/util/List; Z)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/exchange/ExchangeService$2;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/settings/bluetooth/DockService;-hasOtherConnectedDevices-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewClassic;-findAll-(Ljava/lang/String;)I' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/DataConnection$DcActivatingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-prepareEmptyView-()V' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getPhonebookSize-(I)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupNegotiationState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/camera/ShutterButton;-performClick-()Z' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/btservice/AdapterService;-sendConnectionStateChange-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/CalendarContract$Instances;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; J J)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Landroid/app/backup/IBackupManager;-clearBackupData-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/UiModeManagerService;-sendConfigurationAndStartDreamOrDockAppLocked-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/ThrottleService$DataRecorder;-record-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/data/WorkingMessage$MessageStatusListener;-onMaxPendingMessagesReached-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/net/NetworkStatsService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-isDismissable-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/DayView;-reloadEvents-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/webkit/WebViewClassic;-onSizeChanged-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/PanoramaModule;-init-(Lcom/android/camera/CameraActivity; Landroid/view/View; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-setSearchMode-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/WakeLockHoldingProgressListener;-onProgressComplete-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/ITelephony;-enableDataConnectivity-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/media/effect/effects/CrossProcessEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-hideSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame;-performAppWidgetSizeCallbacksIfNecessary-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/SupplicantStateTracker;-sendSupplicantStateChangedBroadcast-(Landroid/net/wifi/SupplicantState; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/provider/BrowserProvider;-doSuggestQuery-(Ljava/lang/String; [Ljava/lang/String; Z)Landroid/database/Cursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/JoinContactListFragment;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/voicedialer/CommandRecognizerEngine;-recognize-(Lcom/android/voicedialer/RecognizerClient; Landroid/app/Activity; Ljava/io/File; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mms/ui/SlideEditorActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$AccountSyncSettingsInAddAccountActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetherableUsbRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/media/effect/effects/ColorTemperatureEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardEntry$OrganizationData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/preferences/InvertedContrastPreview;-onSharedPreferenceChanged-(Landroid/content/SharedPreferences; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-onBondingDockConnect-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-disable-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/CryptKeeper;-encryptionProgressInit-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/preferences/WebViewPreview;-onSharedPreferenceChanged-(Landroid/content/SharedPreferences; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/Controller;-showPreloadedTab-(Lcom/android/browser/IntentHandler$UrlData;)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/Ndef;-connect-()V' : ['android.permission.NFC'],
	'Landroid/webkit/WebViewInputDispatcher;-access$300-(Landroid/webkit/WebViewInputDispatcher; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/SyncManager$SyncHandler;-cancelActiveSyncLocked-(Landroid/accounts/Account; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/inputmethod/UserDictionaryAddWordContents;-hasWord-(Ljava/lang/String; Landroid/content/Context;)Z' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccCard;-changeIccFdnPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/LocalSource;-load-(Lcom/android/dreams/phototable/PhotoSource$ImageData; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-setGrabbedState-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/AccessibilitySettings$ToggleGlobalGesturePreferenceFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/DateTimeView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/media/RingtonePlayer$Client;-binderDied-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/inputmethod/InputMethodAndSubtypeEnabler;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetooth$Stub;-removeBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfile;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/nfc/NfcService;-onCardEmulationAidSelected-([B)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/server/net/NetworkStatsService;-performPollLocked-(I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-assignContactFromPhone-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-<init>-(Lcom/android/internal/telephony/gsm/GSMPhone;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/deviceinfo/Status;-setBtStatus-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Controller;-doStart-(Landroid/os/Bundle; Landroid/content/Intent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-setSystemUiVisibility-(I I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/INativeDaemonConnectorCallbacks;-onDaemonConnected-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/RIL$RILReceiver;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PicasaSource;-next-(Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/power/PowerManagerService$DreamReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-clearLoadingBit-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/accounts/IAccountManager;-addAcount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/mms/data/Conversation$ConversationQueryHandler;-onDeleteComplete-(I Ljava/lang/Object; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-syncReloadDictionaryIfRequired-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/wifi/WpsDialog;-access$200-(Lcom/android/settings/wifi/WpsDialog; Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-removeAllDevices-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$DeviceAdminSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/net/NetworkStatsService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/ui/ClassZeroActivity;-storeMessage-(Landroid/telephony/SmsMessage;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-setUpDialog-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Controller;-openTab-(Ljava/lang/String; Z Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-shutdown-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebIconDatabaseClassic$EventHandler;-access$900-(Landroid/webkit/WebIconDatabaseClassic$EventHandler; Landroid/os/Message;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/ClockFragment;-onPageChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-setSelection-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-disconnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/net/NetworkStatsService$6;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/DataUsageSummary;-access$1800-(Lcom/android/settings/DataUsageSummary;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCallWaitingIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasAccountService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-revertToNitzTimeZone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/contacts/model/ContactLoader;-deliverResult-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider;-updateButtons-(Landroid/widget/RemoteViews; Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/location/LocationManager;-removeUpdates-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewClientClassicExt;-onUnhandledKeyEvent-(Landroid/webkit/WebView; Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/IActivityManager;-setLockScreenShown-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-onReceivedTitle-(Lcom/android/browser/Tab; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/DataUsageSummary;-updatePolicy-(Z)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/BluetoothPairingDialog;-onCancel-()V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/deskclock/TimerRingService;-startAlarm-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebSettingsClassic;-verifyNetworkAccess-()V' : ['android.permission.INTERNET'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipCall;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/VoicemailContentTable;-insert-(Lcom/android/providers/contacts/VoicemailContentProvider$UriData; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/browser/BrowserBookmarksPage$LookupBookmarkCount;-doInBackground-([Ljava/lang/Long;)Ljava/lang/Integer;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/TileImageView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onVoiceCallStarted-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-sendScreenState-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/AccountFilterActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/group/GroupEditorFragment;-onDoneClicked-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/InputMethodManagerService$7;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-sendEmergencyCallbackModeChange-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/ResolverActivity;-onStop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardStatusView;-onAttachedToWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/AutofillHandler$LoadFromDb;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/view/IWindowSession;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/android/mms/util/PduCache;-normalizeKey-(Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-addCachedDevices-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiStateMachine$DefaultState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/contacts/model/dataitem/ImDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/os/IVibratorService$Stub;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$1200-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Landroid/os/Messenger; Ljava/lang/String; Z)Z' : ['android.permission.GET_TASKS'],
	'Lcom/android/settings/fuelgauge/BatteryHistoryChart;-setStats-(Landroid/os/BatteryStats;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/providers/contacts/VoicemailPermissions;-packageHasOwnVoicemailAccess-(Ljava/lang/String;)Z' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/test/IsolatedContext$MockAccountManager;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/accounts/AccountManagerService;-addAccount-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Connection;-proceedAfterWildChar-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/BasicTagTechnology;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/settings/DataUsageSummary$WarningEditorFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/HeadsetProfile;-getConnectionStatus-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView$2;-onRefreshCarrierInfo-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-execute-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/Tab;-goForward-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService;-stop-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMSIForApp-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/InputMonitor;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.CALL_PHONE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/app/ActivityManager;-killBackgroundProcesses-(Ljava/lang/String;)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$2800-(Lcom/android/mms/ui/ComposeMessageActivity; J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-handleVerifyUnlock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/nfc/INfcTag$Stub;-transceive-(I [B Z)Landroid/nfc/TransceiveResult;' : ['android.permission.NFC'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processHangupCall-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfile;-isPreferred-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-hideLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-access$300-(Lcom/android/deskclock/stopwatch/StopwatchFragment;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardFaceUnlockView;-onDetachedFromWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$SyncStateTracker$1;-onPostExecute-(Ljava/lang/Boolean;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/MediaSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/statusbar/tablet/NotificationPanel$1;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/BrowserWebView;-loadUrl-(Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/providers/media/MediaScannerService;-scan-([Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator$1;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/editor/EventFieldEditorView;-clearAllFields-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/SyncManager;-scheduleSyncOperation-(Landroid/content/SyncOperation;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/activities/PeopleActivity;-setQueryTextToFragment-(Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/AccessibilitySettings$ToggleFeaturePreferenceFragment;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-prepareEmptyView-()V' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener$Adapter;-onEndBatchInput-(Lcom/android/inputmethod/latin/InputPointers;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/transaction/MessagingNotification$3;-onPostExecute-(Ljava/lang/Integer;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/browser/Preloader;-discardPreload-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/SipBroadcastReceiver$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onActionIntentReconnectAlarm-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phone/InCallScreen$5;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebView;-zoomIn-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/SipAddressDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pEnablingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-access$100-(Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/contacts/list/ProfileAndContactsLoader;-loadInBackground-()Landroid/database/Cursor;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/CallNotifier;-startIncomingCallQuery-(Lcom/android/internal/telephony/Connection;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/ex/variablespeed/SingleThreadedMediaPlayerProxy;-seekTo-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText;-getStatusForIccState-(Lcom/android/internal/telephony/IccCardConstants$State;)Lcom/android/internal/policy/impl/keyguard/CarrierText$StatusMode;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/app/IBatteryStats;-setBatteryState-(I I I I I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothAdapter;-startDiscovery-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/nfc/tech/IsoDep;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/inputmethod/latin/LatinIME;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/CommandsInterface;-getMute-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/LegacyPostalAddressListAdapter;-getContactUri-(I Landroid/database/Cursor; I I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/camera/SecureCameraActivity;-gotoGallery-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewBase;-show-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/GestureRecognizer;-onTouchEvent-(Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-loadAttachment-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/ContactListEmptyView;-isSyncActive-()Z' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/WifiService;-setWifiEnabled-(Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/tech/MifareUltralight;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/settings/inputmethod/UserDictionaryLocalePicker;-updateLocale-(Ljava/util/Locale;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-init-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE', 'android.permission.WRITE_SETTINGS', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-toggleRecentApps-()V' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/IccCard;-setIccFdnEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getAdapterConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-dismissAndSwitch-()V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback;-goToLockScreen-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-toggleRecentApps-()V' : ['android.permission.GET_TASKS'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/DataConnectionTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-setTextAndKeepSelection-(Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$UserDictionarySettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/CaptivePortalTracker;-isCaptivePortal-(Ljava/net/InetAddress;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/browser/UrlInputView;-finishInput-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/base/CallbackFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl;-access$400-(Lcom/android/contacts/model/AccountTypeManagerImpl; Landroid/content/Context;)Ljava/util/Map;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/BluetoothManagerService;-access$1800-(Lcom/android/server/BluetoothManagerService; Z Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity$GroupDelta;-buildDiff-()Landroid/content/ContentProviderOperation;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/SystemUIService;-onCreate-()V' : ['android.permission.VIBRATE', 'android.permission.DISABLE_KEYGUARD', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/util/HttpHelper;-get-(Ljava/lang/String; Ljava/util/Map;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/net/IThrottleManager;-getHelpUri-()Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/MoviePlayer;-access$1100-(Lcom/android/gallery3d/app/MoviePlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/RIL;-queryFacilityLock-(Ljava/lang/String; Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$DevelopmentSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/media/IAudioService$Stub;-dispatchMediaKeyEventUnderWakelock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-getTechList-(I)[I' : ['android.permission.NFC'],
	'Lcom/android/contacts/GroupMetaDataLoader;-<init>-(Landroid/content/Context; Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-setCallForward-(I I I Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IActivityManager;-updatePersistentConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/PhoneUtils;-setAudioMode-(Lcom/android/internal/telephony/CallManager;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/power/DisplayPowerController$Callbacks;-onProximityNegative-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/SmsReceiver;-beginStartingService-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewInputDispatcher;-access$600-(Landroid/webkit/WebViewInputDispatcher;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$VpnSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/transaction/NotificationTransaction;-sendPdu-(J [B)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/util/Recycler$MmsRecycler;-checkForThreadsOverLimit-(Landroid/content/Context;)Z' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine;-okToConnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothA2dp;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewInputDispatcher;-access$500-(Landroid/webkit/WebViewInputDispatcher;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getDataRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/content/PackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth;-getBondedDevices-()[Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Ljava/net/URL;-openConnection-()Ljava/net/URLConnection;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/CallerInfo;-getCallerInfo-(Landroid/content/Context; Ljava/lang/String;)Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-<init>-(Landroid/content/Context; Ljava/util/HashMap;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardMessageArea$2;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/provider/ContactsContract$RawContacts;-getContactLookupUri-(Landroid/content/ContentResolver; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/SpecialCharSequenceMgr;-handleChars-(Landroid/content/Context; Ljava/lang/String; Z Landroid/widget/EditText;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/LocationManagerService;-getReceiver-(Landroid/app/PendingIntent; I I Ljava/lang/String;)Lcom/android/server/LocationManagerService$Receiver;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/RotateFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setCountryCode-(Ljava/lang/String; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/browser/provider/SnapshotProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$MyScaleListener;-onScale-(Landroid/view/ScaleGestureDetector;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityModel;-isBiometricUnlockSuppressed-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/DialogDetailsView;-reloadDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/SnapshotTab$LoadData;-doInBackground-([Ljava/lang/Void;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/list/ShortcutIntentBuilder$ContactLoadingAsyncTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$TetheringState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/providers/downloads/DownloadService;-insertDownloadLocked-(Lcom/android/providers/downloads/DownloadInfo$Reader; J)Lcom/android/providers/downloads/DownloadInfo;' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/TrimVideo;-onReplay-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/ShowOrCreateActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Landroid/webkit/WebViewClassic;-onDetachedFromWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/PhotoView;-updateActionBar-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/wallpaper/grass/GrassRS;-updateLocation-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-setFilter-(Lcom/android/contacts/list/ContactListFilter; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-handleVerifyUnlock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/research/UploaderService;-uploadFile-(Ljava/io/File;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/server/ConnectivityService;-isNetworkMeteredUnchecked-(I)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/providers/downloads/DownloadService;-access$700-(Lcom/android/providers/downloads/DownloadService; Lcom/android/providers/downloads/DownloadInfo; Z Z)Z' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-getAcceptableAccountChoices-(Landroid/accounts/AccountManager;)Ljava/util/ArrayList;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-disconnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/IThrottleManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/calendar/DayView;-initNextView-(I)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$Listener;-onUp-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/DevelopmentSettings;-access$000-(Landroid/content/Context; Ljava/lang/String; Z)V' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/phone/InCallScreen;-internalRespondViaSms-()V' : ['android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/WiredAccessoryManager$WiredAccessoryObserver;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-initializeMulticastFiltering-()Z' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/media/AudioService$ScoClient;-decCount-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/OtaUtils;-otaShowSpcErrorNotice-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/ObjectSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/webkit/WebViewProvider;-saveWebArchive-(Ljava/lang/String; Z Landroid/webkit/ValueCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/location/GeofenceManager;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-handleSetNetworkPreference-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-setScanMode-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-separateConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/NfcV;-close-()V' : ['android.permission.NFC'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/EasAccountService;-sleep-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothDevice;-fetchUuidsWithSdp-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/BrowserWebView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I Z)V' : ['android.permission.INTERNET'],
	'Landroid/media/effect/effects/BrightnessEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/browser/AutoFillSettingsFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ProviderStatusWatcher$ProviderStatusListener;-onProviderStatusChange-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/UsimDataDownloadHandler;-acknowledgeSmsWithError-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/DetailsAddressResolver$AddressLookupJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/latin/spellcheck/DictionaryPool;-poll-(J Ljava/util/concurrent/TimeUnit;)Ljava/lang/Object;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/exchange/ExchangeService$AccountObserver;-onAccountChanged-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/EventLoader$LoadRequest;-processRequest-(Lcom/android/calendar/EventLoader;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/development/Connectivity;-access$2600-(Lcom/android/development/Connectivity;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setRemoteAlias-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/WifiService;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$1500-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/PhotoPage;-onActionBarAllowed-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-setQueryString-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/BrowserContract$Settings;-isSyncEnabled-(Landroid/content/Context;)Z' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/RIL;-access$200-(Lcom/android/internal/telephony/RIL;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/providers/media/MtpService;-onCreate-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMEI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/ObjectSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumSetDataLoader;-executeAndWait-(Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/bluetooth/BluetoothAdapter;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onSetDependencyMet-(Ljava/lang/String; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/accounts/ManageAccountsSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/PhoneInterfaceManager;-endCall-()Z' : ['android.permission.CALL_PHONE'],
	'Lcom/android/gallery3d/data/MediaSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewClassic;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/LocationManager;-removeUpdates-(Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-formatNdef-(I [B)I' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setPropertiesByCarrier-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/Phone;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/wifi/WifiSettingsForSetupWizardXL;-onConnectButtonPressed-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/view/IInputMethodClient;-onBindMethod-(Lcom/android/internal/view/InputBindResult;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hid/HidService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onEnableNewApn-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/DetailsHelper$DetailsSource;-setIndex-()I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/FrameStore;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/AccessibilitySettings$ToggleAccessibilityServicePreferenceFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/SnapshotTab;-updateShouldCaptureThumbnails-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiStateMachine$DriverStartingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/DockService;-createDialog-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/DockService;-initBtSettings-(Landroid/bluetooth/BluetoothDevice; I Z)[Ljava/lang/CharSequence;' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/util/RateController;-isLimitSurpassed-()Z' : ['android.permission.READ_SMS'],
	'Lcom/android/settings/Settings$UserDictionarySettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/adapter/AbstractSyncAdapter;-getSyncKey-()Ljava/lang/String;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/event/EditEventFragment;-saveReminders-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-getSingleSelectedPath-()Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/MoviePlayer$7;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CallManager;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/server/search/SearchManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-access$1100-(Lcom/android/gallery3d/app/AlbumSetPage; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/ThrottleService$DataRecorder;-retrieve-()V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-getTetheringErroredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-setLockScreenShown-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AutologinBar;-onClick-(Landroid/view/View;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-access$1100-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$WifiStateTracker;-setCurrentState-(Landroid/content/Context; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/IWifiManager;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-registerCallback-(Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitorCallback;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfile;-getSummaryResourceForDevice-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/EasOutboxService;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/userdictionary/UserDictionaryProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/music/AudioPreview;-access$700-(Lcom/android/music/AudioPreview;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStack;-completePauseLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/BluetoothHealth;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/TrimVideo;-pauseVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifyServiceState-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/sip/SipService;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/wifi/IWifiManager;-getWifiStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/VibratorService$VibrateThread;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/server/net/NetworkPolicyManagerService$6;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/app/IBatteryStats$Stub$Proxy;-getAwakeTimeBattery-()J' : ['android.permission.BATTERY_STATS'],
	'Lcom/android/server/ConnectivityService;-handleDeprecatedGlobalHttpProxy-()V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/net/wifi/WifiStateMachine;-fetchRssiAndLinkSpeedNative-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub$Proxy;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-onKeyMove-(I I)Lcom/android/inputmethod/pinyin/SoftKey;' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-deleteAccountPIMData-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-getSuggestions-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo;)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PasswordUnlockScreen$3;-afterTextChanged-(Landroid/text/Editable;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/app/ActivityManagerNative;-shutdown-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/launcher2/Workspace;-setWallpaperDimension-()V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$WifiStateTracker;-getTriState-(Landroid/content/Context;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/BrowserWebView;-showFindDialog-(Ljava/lang/String; Z)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/MmsThumbnailPresenter$1;-onItemLoaded-(Ljava/lang/Object; Ljava/lang/Throwable;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/DayView;-setSelected-(Landroid/text/format/Time; Z Z)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/ThrottleService;-getResetTime-(Ljava/lang/String;)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-onScrubbingEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-onStop-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$DataListener;-onLoadingFinished-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/vcard/VCardEntry$SipData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/CallManager;-dial-(Lcom/android/internal/telephony/Phone; Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/Utils;-showConnectingError-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHealth;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/location/ComprehensiveCountryDetector$1;-onCountryDetected-(Landroid/location/Country;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/data/WorkingMessage;-access$700-(Lcom/android/mms/data/WorkingMessage; Lcom/android/mms/data/Conversation; Landroid/net/Uri; Lcom/google/android/mms/pdu/PduPersister; Lcom/android/mms/model/SlideshowModel; Lcom/google/android/mms/pdu/SendReq; Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-setMobileDataEnabled-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-snapToPageImmediately-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/vcard/VCardEntry;-constructInsertOperations-(Landroid/content/ContentResolver; Ljava/util/ArrayList;)Ljava/util/ArrayList;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$400-(Lcom/android/mms/ui/ComposeMessageActivity;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/base/FrameFetch;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaSubscriptionSource-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag;-getTechList-(I)[I' : ['android.permission.NFC'],
	'Landroid/webkit/WebViewClassic;-findAllAsync-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-dispatchUnhandledMove-(Landroid/view/View; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHeadsetPhone$Stub;-hangupCall-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/exchange/EasAccountService;-messageMoveRequest-(Lcom/android/exchange/MessageMoveRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-sendDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler;-access$1600-(Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler;)J' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/ActionModeHandler$1;-onShareTargetSelected-(Landroid/widget/ShareActionProvider; Landroid/content/Intent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover;-handleIntent-(Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/google/common/io/CharStreams;-readLines-(Lcom/google/common/io/InputSupplier;)Ljava/util/List;' : ['android.permission.INTERNET'],
	'Lcom/android/calendar/DayView;-doScroll-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/bluetooth/BluetoothDevicePreference;-compareTo-(Ljava/lang/Object;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-access$1000-(Lcom/android/bluetooth/opp/BluetoothOppService; Ljavax/obex/ObexTransport;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine;-access$6700-(Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/transaction/SmsReceiverService;-sendFirstQueuedMessage-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/camera/CameraActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/am/ActivityManagerService$20;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/BluetoothPan;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-notifyScreenOffLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/NotificationPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-goBackOrForward-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallHeld-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/vcard/SelectAccountActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter;-loadItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/SimContacts;-importOneSimContact-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/HTML5VideoViewProxy$VideoPlayer;-play-(Ljava/lang/String; I Landroid/webkit/HTML5VideoViewProxy; Landroid/webkit/WebChromeClient; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/wallpaper/galaxy/GalaxyRS;-start-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManagerService;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/AccountUnlockScreen;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/location/GeofenceManager;-onLocationChanged-(Landroid/location/Location;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/CallbackProxy;-sendMessageToUiThreadSync-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiManager;-disconnect-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/INfcAdapter;-setNdefPushCallback-(Landroid/nfc/INdefPushCallback;)V' : ['android.permission.NFC'],
	'Landroid/net/wifi/WifiStateMachine;-access$14000-(Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMEISV-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-cleanup-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/CropImage;-initializeData-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/event/EditEventFragment;-startQuery-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/settings/DataUsageSummary$5;-onClick-(Landroid/view/View;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebViewCore;-<init>-(Landroid/content/Context; Landroid/webkit/WebViewClassic; Landroid/webkit/CallbackProxy; Ljava/util/Map;)V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/gadget/WidgetService$PhotoRVFactory;-onCreate-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/ConnectivityService;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-xtraDownloadRequest-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/AudioPlaybackQueueItem;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaListView;-deleteSelectedEvent-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/accounts/ManageAccountsSettings;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS'],
	'Lcom/android/browser/BrowserSettings;-startManagingSettings-(Landroid/webkit/WebSettings;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-pauseVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/calllog/CallLogQueryHandler;-fetchCalls-(I)V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/inputmethod/research/MainLogBuffer;-isSafeToLog-()Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/quicksearchbox/util/NamedTask;-run-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ConnectivityService;-tether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/ActivityManagerNative;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.REORDER_TASKS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.WAKE_LOCK', 'android.permission.GET_TASKS', 'android.permission.WRITE_SETTINGS', 'android.permission.RESTART_PACKAGES'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$ConnectedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl$FindInvitablesTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/model/dataitem/EventDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-loadDraft-()Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$2500-(Lcom/android/bluetooth/hfp/HeadsetStateMachine;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothManager;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/ExchangeService;-startManualSync-(J I Lcom/android/exchange/Request;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-setKeyguardEnabled-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/media/IRingtonePlayer;-play-(Landroid/os/IBinder; Landroid/net/Uri; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-acknowledgeIncomingGsmSmsWithPdu-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/ContactsProvider2;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-setCountryListener-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-setFacilityLock-(Ljava/lang/String; Z Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getNetworkSelectionMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/OtaUtils;-onClickOtaSpeakerButton-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/GLView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/ThrottleService$MyHandler;-checkThrottleAndPostNotification-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/ConnectivityManager;-setNetworkPreference-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/calllog/CallLogQueryHandler;-markNewVoicemailsAsOld-()V' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/am/Am;-runDisplaySize-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/GrainFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/phone/SimContacts;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask$1$1;-onResult-(Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/pan/PanService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-checkAddPermission-(Landroid/view/WindowManager$LayoutParams;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-updateAll-(Landroid/net/Uri; I Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$200-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$2100-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-requestFocus-(I Landroid/graphics/Rect;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiManager;-addToBlacklist-(Ljava/lang/String;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/inputmethod/research/FeedbackFragment$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/util/AccountSelectionUtil;-getSelectAccountDialog-(Landroid/content/Context; I Landroid/content/DialogInterface$OnClickListener;)Landroid/app/Dialog;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-showBouncer-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/common/widget/GroupingListAdapter;-bindChildView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/location/LocationBasedCountryDetector$2;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/StateManager;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/wifi/WifiManager;-getWifiStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/model/dataitem/IdentityDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/activities/DialtactsActivity$7;-onQueryTextChange-(Ljava/lang/String;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/videosrc/MediaSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/CommandsInterface;-resetRadio-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncManager$9;-onServiceChanged-(Ljava/lang/Object; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/GeneralPreferences;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/settings/DataUsageSummary;-setPolicyWarningBytes-(J)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/DataUsageSummary$ConfirmAutoSyncChangeFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/ActivityManagerNative;-killBackgroundProcesses-(Ljava/lang/String; I)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/providers/calendar/CalendarProvider2;-access$300-(Lcom/android/providers/calendar/CalendarProvider2;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-untether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-notifyLidSwitchChanged-(J Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/sip/SipSessionGroup;-process-(Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/ContactLoader;-onStartLoading-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/editor/EventFieldEditorView$2;-onDateSet-(Lcom/android/contacts/datepicker/DatePicker; I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/ITelephony;-call-(Ljava/lang/String;)V' : ['android.permission.CALL_PHONE'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/dreams/phototable/StockSource;-next-(Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/EasSyncService;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/NfcService$EnableDisableTask;-enableInternal-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/contacts/calllog/CallLogNotificationsService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/input/InputManagerService;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.CALL_PHONE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/FilterGraphEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHealth;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiStateMachine$DriverUnloadingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-prev-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-setEvents-(I I Ljava/util/ArrayList;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/location/GeofenceManager;-sendIntentExit-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/WorkingMessage;-access$1700-(Lcom/android/mms/data/WorkingMessage; Lcom/android/mms/data/Conversation; Ljava/lang/String;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/ContactsUtils;-getThumbnailSize-(Landroid/content/Context;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity;-displayPhoto-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/EmergencyCallbackModeExitDialog$4;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/Source;-getSuggestions-(Ljava/lang/String; I Z)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/provider/Settings$Global;-getUriFor-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/data/WorkingMessage;-load-(Lcom/android/mms/ui/ComposeMessageActivity; Landroid/net/Uri;)Lcom/android/mms/data/WorkingMessage;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/RIL;-writeSmsToRuim-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IPowerManager$Stub;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/ServerThread$3;-run-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_FINE_LOCATION', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/base/CallbackFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/MediaSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/RespondViaSmsManager$RespondViaSmsItemClickListener;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/am/BroadcastQueue;-sendPendingBroadcastsLocked-(Lcom/android/server/am/ProcessRecord;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/VibratorService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-initDevicePreference-(Lcom/android/settings/bluetooth/BluetoothDevicePreference;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onCleanUpConnection-(Z I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothTetheringDataTracker;-startMonitoring-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/nfc/NfcActivityManager;-setNdefPushMessageCallback-(Landroid/app/Activity; Landroid/nfc/NfcAdapter$CreateNdefMessageCallback;)V' : ['android.permission.NFC'],
	'Lcom/android/contacts/model/dataitem/StructuredNameDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/util/DownloadManager;-access$400-(Lcom/android/mms/util/DownloadManager; Landroid/net/Uri;)Ljava/lang/String;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/PhoneProxy;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/ChooseLockGeneric$ChooseLockGenericFragment;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-setKeyboard-(Landroid/inputmethodservice/Keyboard;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/settings/net/DataUsageMeteredSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/commands/am/Am;-main-([Ljava/lang/String;)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/CryptKeeper;-onStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-onPause-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-loadDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/SnapshotTab;-deleteThumbnail-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/TelephonyRegistry;-checkListenerPermission-(I)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-resetTimeouts-()V' : ['android.permission.NFC'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/nfc/AndroidBeam;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/WebViewController;-createSubWindow-(Lcom/android/browser/Tab;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/ManageCachePage;-initializeData-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$ErrorState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-getCallingPackage-()Ljava/lang/String;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStoppingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/LocalBluetoothManager;-<init>-(Lcom/android/settings/bluetooth/LocalBluetoothAdapter; Landroid/content/Context;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-reportStkServiceIsRunning-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-<init>-(Lcom/android/gallery3d/app/MovieActivity; Landroid/view/View; Lcom/android/gallery3d/app/MovieActivity; Landroid/net/Uri; Landroid/os/Bundle; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/CryptKeeper;-passwordEntryInit-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub$Proxy;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothPan$Stub$Proxy;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/NsdService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/ProfileService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/NfcService$NfcAdapterExtrasService;-getCardEmulationRoute-(Ljava/lang/String;)I' : ['android.permission.NFC'],
	'Landroid/webkit/WebViewInputDispatcher;-enqueueDoubleTapLocked-(Landroid/view/MotionEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/ActivityManagerNative;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-adjustOtherSinkPriorities-(Lcom/android/bluetooth/a2dp/A2dpService; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/IApplicationToken$Stub;-keyDispatchingTimedOut-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/content/IContentService;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/model/dataitem/EmailDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/GLView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/Controller$Download;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/net/IConnectivityManager;-startUsingNetworkFeature-(I Ljava/lang/String; Landroid/os/IBinder;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionActivity;-createConnectionDisplayText-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/accounts/AccountPreferenceBase$1$1;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/RIL;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManagerProxy;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/webkit/WebViewClassic;-zoomOut-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/inputmethodservice/ExtractEditText;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/AccelerometerListener;-access$000-(Lcom/android/phone/AccelerometerListener; D D D)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Landroid/telephony/TelephonyManager;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/ZoomManager;-animateZoom-(Landroid/graphics/Canvas;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppManager;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView$1;-onEndBatchInput-(Lcom/android/inputmethod/latin/InputPointers;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/SettingsSafetyLegalActivity$1;-onUnhandledKeyEvent-(Landroid/webkit/WebView; Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-onOffsetsChanged-(F F F F I I)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/phone/OtaUtils;-cleanOtaScreen-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/SearchActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchWithCaptureAnimationLocked-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService$ScoClient;-binderDied-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CallManager;-hangupForegroundResumeBackground-(Lcom/android/internal/telephony/Call;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/webkit/WebView;-setLayerType-(I Landroid/graphics/Paint;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/BluetoothManagerService$BluetoothServiceConnection;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/BootReceiver;-removeOldUpdatePackages-(Landroid/content/Context;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/app/IActivityManager;-moveTaskBackwards-(I)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/net/IConnectivityManager;-requestNetworkTransitionWakelock-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DataUsageSummary$1;-onPostExecute-(Ljava/lang/Void;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityViewFlipper;-onResume-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/MediaSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ComposeMessageActivity$21;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onPlayPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-setGlobalProxy-(Landroid/net/ProxyProperties;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getRemoteName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-pollStateDone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-setGrabbedState-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/gallery3d/data/MediaObject;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/AdnRecordCache;-requestLoadAllAdnLike-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-onDayTapped-(Landroid/text/format/Time;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/interactions/ImportExportDialogFragment;-access$100-(Lcom/android/contacts/interactions/ImportExportDialogFragment;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-removePhoneStateListener-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-doKeyguardTimeout-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/vcard/VCardParserImpl_V21;-parseOneVCard-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/ServerThread;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.BLUETOOTH', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/ThrottleService;-getHelpUri-()Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-unbroadcastIntent-(Landroid/app/IApplicationThread; Landroid/content/Intent; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage;-onStateResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/editor/PhotoEditorView;-setPhotoBitmap-(Landroid/graphics/Bitmap;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/UiController;-openTab-(Ljava/lang/String; Z Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/power/DisplayBlanker;-blankAllDisplays-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/dreams/phototable/PhotoSource;-getStream-(Lcom/android/dreams/phototable/PhotoSource$ImageData; I)Ljava/io/InputStream;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHeadset;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCLIP-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/EventFieldEditorView;-onFieldChanged-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/ExchangeService$CalendarObserver$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/PickerActivity;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/BrowserWebView;-pauseTimers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-notifyPageSwitching-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaType-(Lcom/android/gallery3d/data/Path;)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-getNeighboringCids-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-getActiveLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-viewContact-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-access$500-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/support/v4/net/ConnectivityManagerCompat;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/WifiManager;-setWifiEnabled-(Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/contacts/list/ShortcutIntentBuilder$PhoneNumberLoadingAsyncTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/vpn2/VpnSettings;-connect-(Lcom/android/internal/net/VpnProfile;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/media/AudioService;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/AccountsTester$ButtonClickListener;-onClick-(Landroid/view/View;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/list/LegacyContactListAdapter;-bindQuickContact-(Lcom/android/contacts/list/ContactListItemView; I Landroid/database/Cursor; I I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/BrowserWebView;-goBackOrForward-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/data/Conversation;-getOrCreateThreadId-(Landroid/content/Context; Ljava/lang/String;)J' : ['android.permission.READ_SMS'],
	'Lcom/android/settings/WallpaperTypeSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/media/RingtonePickerActivity;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/AccountSyncAdapter;-<init>-(Lcom/android/exchange/EasSyncService;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/power/PowerManagerService;-setTemporaryScreenAutoBrightnessAdjustmentSettingOverrideInternal-(F)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText$1;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/TileImageView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/WifiService;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/Settings$DateTimeSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/transaction/SmsSingleRecipientSender;-<init>-(Landroid/content/Context; [Ljava/lang/String; Ljava/lang/String; J)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/AlbumPicker;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/power/PowerManagerService;-wakeUpNoUpdateLocked-(J)Z' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/ContactEditorActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/ExpandHelper;-vibrate-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/server/location/LocationProviderInterface;-enable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/CallbackProxy;-onJsBeforeUnload-(Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$OnlineWatchState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/BrowserWebView;-clearHistory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/InCallScreen;-dumpBluetoothState-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiManager;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$DataListener;-onPhotoChanged-(I Lcom/android/gallery3d/data/Path;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/GLTextureTarget;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onRadioAvailable-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/webkit/WebView;-setNetworkAvailable-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/WallpaperTypeSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/ViewRootImpl;-deliverTrackballEvent-(Landroid/view/ViewRootImpl$QueuedInputEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/SlidingTab;-vibrate-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/settings/inputmethod/UserDictionaryList;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockScreen$SlidingTabMethods;-onGrabbedStateChange-(Landroid/view/View; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/browser/GoogleAccountLogin;-getAccounts-(Landroid/content/Context;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothHealth;-registerSinkAppConfiguration-(Ljava/lang/String; I Landroid/bluetooth/BluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/tts/TtsEngineSettingsFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/DayView;-getEventAccessLevel-(Landroid/content/Context; Lcom/android/calendar/Event;)I' : ['android.permission.READ_CALENDAR'],
	'Landroid/view/inputmethod/BaseInputConnection;-sendCurrentText-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/NsdService$NsdStateMachine$DefaultState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/transaction/MessagingNotification;-getUndeliveredMessageCount-(Landroid/content/Context; [J)I' : ['android.permission.READ_SMS'],
	'Lcom/android/settings/Settings$InputMethodAndSubtypeEnablerActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/UiController;-closeOtherTabs-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/settings/Settings$BluetoothSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/PhoneUtils;-setAudioMode-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/SMSDispatcher;-acknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/AccessibilityInjector;-addAccessibilityApisIfNecessary-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/MmsThumbnailPresenter;-present-(Lcom/android/mms/util/ItemLoadedCallback;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetoothManager;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-getMessageItem-(Ljava/lang/String; J Z)Lcom/android/mms/ui/MessageItem;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/data/Conversation;-asyncDeleteObsoleteThreads-(Landroid/content/AsyncQueryHandler; I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/NsdService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-dispatchPdus-([[B)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-connectOtherProfile-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/WorkingMessage$5;-run-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/mms/transaction/MessagingNotification;-addSmsNotificationInfos-(Landroid/content/Context; Ljava/util/Set; Ljava/util/SortedSet;)V' : ['android.permission.READ_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/providers/downloads/DownloadThread;-sendRequest-(Lcom/android/providers/downloads/DownloadThread$State; Landroid/net/http/AndroidHttpClient; Lorg/apache/http/client/methods/HttpGet;)Lorg/apache/http/HttpResponse;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/UiccCardApplication;-queryFdn-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DataUsageSummary$DataUsageAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/Connectivity;-onCrash-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/BrowserWebViewFactory;-createSubWebView-(Z)Landroid/webkit/WebView;' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/contacts/model/dataitem/WebsiteDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/dialpad/DialpadImageButton;-setPressed-(Z)V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/ZoomManager;-handleDoubleTap-(F F)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setRemoteAlias-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-updateResources-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-doKeyguardLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/webkit/WebViewClassic;-onResume-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/ActivityController;-start-(Landroid/content/Intent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCLIR-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService$ScreenOnBlockerImpl;-release-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onActionIntentReconnectAlarm-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/Settings$AdvancedWifiSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-refreshBtClass-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifyDataConnection-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/VibratorService;-updateInputDeviceVibrators-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton$1;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/ShutdownThread;-reboot-(Landroid/content/Context; Ljava/lang/String; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/net/NetworkPolicyManagerService$7;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/contacts/list/PostalAddressListAdapter;-getDataUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$000-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/ProfileProvider;-openAssetFile-(Landroid/net/Uri; Ljava/lang/String;)Landroid/content/res/AssetFileDescriptor;' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_PROFILE'],
	'Landroid/net/IThrottleManager$Stub$Proxy;-getCliffThreshold-(Ljava/lang/String; I)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IRingtonePlayer$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView$AccountAnalyzer;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Pending;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/am/ActivityManagerService$16;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-systemReady-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-getSuggestions-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo;)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-handleSetHidden-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/list/AccountFilterActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Connected;-processAudioEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/ConnectivityService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/IUserSwitchObserver$Stub$Proxy;-onUserSwitching-(I Landroid/os/IRemoteCallback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl;-loadAccountsInBackground-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-addAudio-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/DataManager;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/ex/variablespeed/VariableSpeed;-seekTo-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-loadUrlImpl-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/BandMode$1;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/CallLog$Calls;-removeExpiredEntries-(Landroid/content/Context;)V' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/server/LocationManagerService;-checkListenerOrIntent-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; I I Ljava/lang/String;)Lcom/android/server/LocationManagerService$Receiver;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/accounts/AccountSyncSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton$1;-onRefreshCarrierInfo-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/HTML5VideoViewProxy;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DevelopmentSettings;-setPrefsEnabledState-(Z)V' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/calendar/CalendarUtils$TimeZoneUtils;-forceDBRequery-(Landroid/content/Context; Ljava/lang/Runnable;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/content/IContentService$Stub$Proxy;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-getPDPContextList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity$DiscardDraftListener;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/content/pm/IPackageManager;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Landroid/net/IConnectivityManager;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/PhotoModule;-initializeSecondTime-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$WifiStateTracker;-toggleState-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/google/common/io/Files;-equal-(Ljava/io/File; Ljava/io/File;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/vcard/ImportVCardActivity;-onCreateDialog-(I Landroid/os/Bundle;)Landroid/app/Dialog;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcAdapterExtras$Stub;-close-(Ljava/lang/String; Landroid/os/IBinder;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/bluetooth/hid/HidService;-access$600-(Lcom/android/bluetooth/hid/HidService; Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/InputMethodManagerService;-access$1000-(Lcom/android/server/InputMethodManagerService; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/detail/ContactDetailFragment;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/ClusterSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/editor/PhoneticNameEditorView;-onLabelRebuilt-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfile;-setPreferred-(Landroid/bluetooth/BluetoothDevice; Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-doResumeUpdates-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/am/UsageStatsService$2;-onSomePackagesChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/VoicemailPermissions;-checkCallerHasOwnVoicemailAccess-()V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setPasskey-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/SimContacts;-access$100-(Landroid/database/Cursor; Landroid/content/ContentResolver; Landroid/accounts/Account;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/IWifiManager;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/wallpaper/grass/GrassRS;-access$300-(Lcom/android/wallpaper/grass/GrassRS;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/AllInOneActivity;-createButtonsSpinner-(I Z)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/UiccCardApplication;-changeIccFdnPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/RawContactModifier;-parseExtras-(Landroid/content/Context; Lcom/android/contacts/model/account/AccountType; Lcom/android/contacts/model/RawContactDelta; Landroid/os/Bundle;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/list/AccountFilterActivity$FilterLoader;-loadInBackground-()Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/SearchActivity;-eventsChanged-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/WebViewCore;-sendNotifyProgressFinished-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/net/NetworkPolicyManagerService$10;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/VideoView$6;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage;-setupNfcBeamPush-()V' : ['android.permission.NFC'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-connectProfile-(Lcom/android/settings/bluetooth/LocalBluetoothProfile;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPINView;-onResume-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewCore;-viewSizeChanged-(Landroid/webkit/WebViewClassic$ViewSizeData;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/telephony/SmsManager;-sendTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Telephony$Sms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/google/common/io/Files;-copy-(Lcom/google/common/io/InputSupplier; Ljava/io/File;)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handleRadioNotAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService$2;-autoDiscover-(Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/ConnectivityService;-getTetheredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/Use2GOnlyCheckBoxPreference$MyHandler;-handleSetPreferredNetworkTypeResponse-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiManager$MulticastLock;-finalize-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/widget/VideoView;-suspend-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/EmailDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/list/ContactBrowseListFragment$ContactLookupTask;-doInBackground-([Ljava/lang/Void;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterfw/core/SyncRunner;-processFilterNode-(Landroid/filterfw/core/Filter;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-close-(I)I' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-separate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallbackProxy$2;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Controller;-start-(Landroid/content/Intent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/app/ChooserActivity;-onCreate-(Landroid/os/Bundle; Landroid/content/Intent; Ljava/lang/CharSequence; [Landroid/content/Intent; Ljava/util/List; Z)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/server/display/DisplayAdapter$Listener;-onDisplayDeviceEvent-(Lcom/android/server/display/DisplayDevice; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity;-doSaveAction-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-sendConnectionStateChange-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/LocalAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/google/common/io/MultiReader;-read-([C I I)I' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/group/GroupBrowseListFragment;-bindGroupList-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-fetchName-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCallDeflectionIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pan/PanService;-cleanup-()Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/content/PackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic$1;-onPostExecute-(Ljava/util/Set;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getPDPContextList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/util/PduLoaderManager$PduTask$1;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/CalendarEventModel;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/DataUsageSummary$CycleEditorFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingEncryptedRfcommWithServiceRecord-(Ljava/lang/String; Ljava/util/UUID;)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/P2pEventManager;-onP2pSendComplete-()V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/text/StringSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/CountryDetectorBase;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/IWindowManager;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/SnailAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/nfc/tech/NfcF;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Landroid/filterpacks/base/FrameBranch;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/MenuExecutor$MediaOperation;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Void;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/RuimRecords;-onRecordLoaded-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/IWifiManager$Stub;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener$Adapter;-onPressKey-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewClassic;-doTrackball-(J I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;-executeMessage-(Landroid/os/Message;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/webkit/WebViewInputDispatcher;-postClick-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/CropView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/power/PowerManagerService$2;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/phone/InCallScreen;-onDialerOpen-(Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewClassic;-computeScroll-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/detail/ContactDetailTabCarousel;-loadData-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/NavigationBarBase;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/app/TrimVideo;-onSeekStart-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncManager;-maybeRescheduleSync-(Landroid/content/SyncResult; Landroid/content/SyncOperation;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/MediaPlaybackService;-onCreate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/ConnectivityManager;-requestRouteToHost-(I I)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processVrEvent-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/inputmethod/latin/LatinIME;-initSuggest-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/ZoomControlExternal$2;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/quicksearchbox/AppsCorpus;-getSuggestions-(Ljava/lang/String; I Z)Lcom/android/quicksearchbox/SuggestionCursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/CropView$FaceHighlightView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/UiController;-openTabToHomePage-()Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-handleCancel-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/users/UserSettings$3;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/inputmethod/InputMethodAndLanguageSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/SpecialCharSequenceMgr;-handleChars-(Landroid/content/Context; Ljava/lang/String; Landroid/widget/EditText;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$3800-(Lcom/android/mms/ui/ComposeMessageActivity; J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/BrowserWebView;-resumeTimers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/FrameFetch;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/exchange/adapter/CalendarSyncAdapter$CalendarOperations;-updatedExtendedProperty-(Ljava/lang/String; Ljava/lang/String; J)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/WiredAccessoryManager;-setDevicesState-(I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-access$1300-(Landroid/media/AudioService; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/tts/AudioPlaybackQueueItem;-finish-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-editMessageItem-(Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/model/dataitem/NicknameDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/view/WindowManagerGlobal;-addView-(Landroid/view/View; Landroid/view/ViewGroup$LayoutParams; Landroid/view/Display; Landroid/view/Window;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockScreen$GlowPadViewMethods;-onTrigger-(Landroid/view/View; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/AutoCorrection;-isValidWord-(Ljava/util/concurrent/ConcurrentHashMap; Ljava/lang/CharSequence; Z)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/bluetooth/IBluetoothA2dp;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/calendar/CalendarDebug$FetchInfoTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Landroid/view/IWindowManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.DISABLE_KEYGUARD', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/snep/SnepServer;-handleRequest-(Lcom/android/nfc/snep/SnepMessenger; Lcom/android/nfc/snep/SnepServer$Callback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/A2dpProfile;-isPreferred-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-fillQueue-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/base/OutputStreamTarget;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BatteryService;-updateLocked-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-initSeekBar-(Landroid/widget/SeekBar; Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingInsecureRfcommOn-(I)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter$QueryHandler;-processNewCursor-(Lcom/android/calendar/agenda/AgendaWindowAdapter$QuerySpec; Landroid/database/Cursor;)I' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/RIL;-setPreferredVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/voicedialer/VoiceDialerActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-switchToState-(I F F)V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/group/GroupBrowseListFragment$3;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/BaseCommands;-setCurrentPreferredNetworkType-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover;-complete-(Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/model/RawContact;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$900-(Lcom/android/gallery3d/ui/PhotoView; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/NavScreen;-onCloseTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/ui/CropView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/settings/inputmethod/InputMethodAndLanguageSettings;-updateUserDictionaryPreference-(Landroid/preference/Preference;)V' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/google/common/io/ByteStreams;-readBytes-(Lcom/google/common/io/InputSupplier; Lcom/google/common/io/ByteProcessor;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/server/am/ActivityManagerService;-getTasks-(I I Landroid/app/IThumbnailReceiver;)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareUltralight;-close-()V' : ['android.permission.NFC'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-setData-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/preference/VolumePreference;-onBindDialogView-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/app/IAlarmManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/camera/SecureCameraActivity;-doChangeCamera-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/data/WorkingMessage;-createDraftMmsMessage-(Lcom/google/android/mms/pdu/PduPersister; Lcom/google/android/mms/pdu/SendReq; Lcom/android/mms/model/SlideshowModel; Landroid/net/Uri; Landroid/content/Context; Ljava/util/HashMap;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/alerts/AlertAdapter;-updateView-(Landroid/content/Context; Landroid/view/View; Ljava/lang/String; Ljava/lang/String; J J Z)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mms/ui/MessagingPreferenceActivity;-loadPrefs-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewInputDispatcher;-handleWebKitTimeout-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$WifiStateTracker$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/view/IApplicationToken$Stub;-windowsVisible-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mms/transaction/MessagingNotification$3;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/contacts/activities/JoinContactActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/MessageUtils;-getLocalNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/DayView;-sendAccessibilityEventAsNeeded-(Z)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityCallback;-dismiss-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$3600-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Landroid/net/wifi/p2p/WifiP2pWfdInfo;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-skipCurrentReceiverLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/systemui/statusbar/CommandQueue$Callbacks;-updateNotification-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)V' : ['android.permission.VIBRATE'],
	'Landroid/media/MediaPlayer;-postEventFromNative-(Ljava/lang/Object; I I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-getSubscriptionInfoAndStartPollingThreads-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/TwilightService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/ApplicationSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/IWindowSession$Stub$Proxy;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onError-(Landroid/net/sip/SipAudioCall; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$RunningServicesActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/Event;-loadEvents-(Landroid/content/Context; Ljava/util/ArrayList; I I I Ljava/util/concurrent/atomic/AtomicInteger;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/BluetoothHeadset;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/FallbackEventHandler;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/AccessibilitySettings$ToggleGlobalGesturePreferenceFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/webkit/WebViewProvider;-clearView-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBarView;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/server/power/PowerManagerService;-access$1400-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-fetchRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitorCallback;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/activities/DialtactsActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/view/PieMenu$PieView;-layout-(I I Z F I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/IWifiManager$Stub;-startScan-(Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/mms/ui/SlideView;-pauseVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-getRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$SoundSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ManageSimMessages$1;-onChange-(Z)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/P2pLinkManager$SendTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$StorageSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/ITelephony$Stub;-isSimPinEnabled-()Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/PrivacySettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/ActivityBase;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/UiModeManagerService;-systemReady-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-requestChildFocus-(Landroid/view/View; Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/HeadsetProfile;-getSummaryResourceForDevice-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewProvider$ViewDelegate;-setFrame-(I I I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BaseUi;-setUseQuickControls-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/Settings$PrivacySettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/pm/PackageManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/pm/Installer; Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/group/GroupDetailFragment;-access$400-(Lcom/android/contacts/group/GroupDetailFragment; Landroid/database/Cursor;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/IAccountManager;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/mms/ui/ManageSimMessages$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pDisabledState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/wm/WindowManagerService;-setCurrentUser-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/LatinIME;-onStartInputView-(Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/media/effect/effects/RedEyeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-setupDataCall-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothPbap;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/Controller;-onActivityResult-(I I Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/ISms;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/settings/BrightnessPreference;-showDialog-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/RuimRecords;-onRefresh-(Z [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-rebootSafeMode-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/CommandsInterface;-acknowledgeLastIncomingCdmaSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/AdnRecordLoader;-loadFromEF-(I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserWebView;-loadDataWithBaseURL-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setPropertiesByCarrier-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/pm/PackageManagerService;-deleteSystemPackageLI-(Lcom/android/server/pm/PackageSetting; I Lcom/android/server/pm/PackageManagerService$PackageRemovedInfo; Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/os/PowerManager$WakeLock;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthListView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/BreadCrumbView;-notifyController-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/activities/PeopleActivity;-configureFragments-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/MessageListItem;-bind-(Lcom/android/mms/ui/MessageItem; Z I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-callListenerOnCodeInput-(Lcom/android/inputmethod/keyboard/Key; I I I J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/CallNotifier;-<init>-(Lcom/android/phone/PhoneGlobals; Lcom/android/internal/telephony/Phone; Lcom/android/phone/Ringer; Lcom/android/phone/CallLogAsync;)V' : ['android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-onMeasure-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-access$000-(Lcom/android/internal/telephony/RIL; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/DownloadHandler$1;-run-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/nfc/tech/Ndef;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/gallery3d/app/TimeBar$Listener;-onScrubbingEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/NotificationPlayer;-play-(Landroid/content/Context; Landroid/net/Uri; Z I F)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/inputmethod/UserDictionaryAddWordFragment;-onResume-()V' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/server/wm/WindowManagerService;-clearForcedDisplaySize-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-performBackgroundTask-(I Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/camera/PhotoModule;-onPauseAfterSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/DayView;-appendEventAccessibilityString-(Ljava/lang/StringBuilder; Lcom/android/calendar/Event;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/Settings$InputMethodAndLanguageSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-access$200-(Lcom/android/bluetooth/hfp/HeadsetService; [I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/content/pm/IPackageManager$Stub;-getPackageGids-(Ljava/lang/String;)[I' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Lcom/android/server/net/LockdownVpnTracker;-onNetworkInfoChanged-(Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/ContactPickerFragment;-onClose-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewClassic;-loadDataImpl-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/BrowserActivity;-createController-()Lcom/android/browser/Controller;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/CropFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/android/mms/pdu/PduPersister;-persist-(Lcom/google/android/mms/pdu/GenericPdu; Landroid/net/Uri; Z Z Ljava/util/HashMap;)Landroid/net/Uri;' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/group/GroupEditorFragment$4;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockScreen$1;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/widget/TextView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothHealth;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/imageproc/Invert;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onEnableNewApn-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/SamplingProfilerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/BondStateMachine$StableState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wm/WindowManagerService;-setForcedDisplaySize-(I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onEnableApn-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AppBridge$Server;-switchWithCaptureAnimation-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/ActivityState;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/filterpacks/base/GLTextureTarget;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine;-<init>-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/speech/SpeechRecognizer;-setRecognitionListener-(Landroid/speech/RecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/mms/ui/SlideshowPresenter;-presentAudio-(Lcom/android/mms/ui/SlideViewInterface; Lcom/android/mms/model/AudioModel; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/ShowOrCreateActivity;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-snapToPage-(I I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/ContactSaveService;-addMembersToGroup-(Landroid/content/ContentResolver; [J J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dispose-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManager;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardStatusView$1;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-handleCdmaSubscriptionSource-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-notifyApnIdUpToCurrent-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-proceedAfterWaitChar-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/IMediaPlaybackService;-enqueue-([J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewManager;-wakeWhenReadyTq-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterpacks/base/CallbackFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-dispatchBroadcastMessage-(Landroid/telephony/SmsCbMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalMediaItem;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/VolumePanel;-onRemoteVolumeUpdateIfShown-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/net/NetworkStatsService;-getNetworkStatsTethering-()Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-loadDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-access$2200-(Lcom/android/bluetooth/opp/BluetoothOppService; Landroid/database/Cursor; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/phone/InCallScreen;-internalSwapCalls-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/contacts/VoicemailTable$Delegate;-update-(Lcom/android/providers/contacts/VoicemailContentProvider$UriData; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/provider/BrowserContract$SyncState;-set-(Landroid/content/ContentProviderClient; Landroid/accounts/Account; [B)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/server/BluetoothA2dpService;-onBluetoothDisable-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupCreatingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/common/widget/GroupingListAdapter;-bindGroupView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor; I Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/Settings$ApplicationSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$DreamSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiManager;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/settings/inputmethod/UserDictionaryList;-createUserDictSettings-(Landroid/preference/PreferenceGroup;)V' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-updateCurrentCarrierInProvider-(Ljava/lang/String;)Z' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/contacts/detail/ContactDetailFragment$MakeLocalCopyQuickFix;-execute-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/view/IWindowManager$Stub$Proxy;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/phone/CallLogAsync$GetLastOutgoingCallTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/commands/am/Am;-run-([Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/sip/ISipService;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/vcard/VCardEntry$NoteData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/ims/IsimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionCallReceiverImpl;-access$1500-(Lcom/android/server/sip/SipSessionGroup$SipSessionImpl; Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SmsManager;-disableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton$1;-onBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/activities/PeopleActivity;-updateFragmentsVisibility-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-separate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/snep/SnepServer;-start-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/SlideshowEditActivity;-initSlideList-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/LocationManagerService;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$1400-(Lcom/android/mms/ui/ComposeMessageActivity; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$9;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/downloads/DownloadThread;-checkPausedOrCanceled-(Lcom/android/providers/downloads/DownloadThread$State;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-handleCallSetupRequestFromSim-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/DialogDetailsView;-show-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/TimerRingService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-setPairingConfirmation-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/widget/LayoutSuppressingQuickContactBadge;-assignContactFromEmail-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/WifiService;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/providers/contacts/ContactsProvider2;-updateData-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String; Z)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-reportFailedUnlockAttempt-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/UsimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$PlaybackView;-setSpeakerPhoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/calendar/event/EditEventHelper;-saveReminders-(Ljava/util/ArrayList; J Ljava/util/ArrayList; Ljava/util/ArrayList; Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/editor/SelectAccountDialogFragment;-access$000-(Lcom/android/contacts/editor/SelectAccountDialogFragment; Lcom/android/contacts/model/account/AccountWithDataSet;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ConversationListItem;-updateAvatarView-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-access$300-(Lcom/android/contacts/calllog/CallLogAdapter; Ljava/lang/String; Ljava/lang/String; Lcom/android/contacts/calllog/ContactInfo;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/TransactionService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/soundrecorder/SoundRecorder$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActiveServices;-bringUpServiceLocked-(Lcom/android/server/am/ServiceRecord; I Z)Ljava/lang/String;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingRfcommOn-(I)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManager;-hasFeatures-(Landroid/accounts/Account; [Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-vibrate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout$5;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-configAudioParameters-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/browser/Controller;-closeOtherTabs-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/BluetoothDevice;-createRfcommSocket-(I)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$SoundSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/SecureSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserActivity;-onDestroy-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/SIMFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameStore;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-vibrate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/server/power/PowerManagerService;-releaseWakeLockInternal-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$SetDnsForwardersErrorState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/MessageListItem$2;-onPduLoaded-(Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/view/IWindowSession;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-determineScrollingStart-(Landroid/view/MotionEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$AccessibilitySettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/ZoomManager$ScaleDetectorListener;-onScaleEnd-(Landroid/view/ScaleGestureDetector;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/TelephonyRegistry;-broadcastServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/activities/PeopleActivity;-onContactListFilterChanged-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/exchange/EasSyncService;-tryProvision-(Lcom/android/exchange/EasSyncService;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-doJoinContactAction-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/power/PowerUI$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-connectSocket-(Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView$2;-onCustomRequest-(I)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/IccCardProxy;-setExternalState-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/ui/ComposeMessageActivity$MsgListMenuClickListener;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/mms/widget/MmsWidgetService$MmsFactory;-queryAllConversations-()Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Lcom/android/contacts/calllog/CallLogQueryHandler;-markMissedCallsAsRead-()V' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/telephony/PhoneProxy;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-teardown-(Landroid/net/NetworkStateTracker;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/BrowserActivity;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/ConnectivityManager;-getLastTetherError-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-appNotResponding-(Lcom/android/server/am/ProcessRecord; Lcom/android/server/am/ActivityRecord; Lcom/android/server/am/ActivityRecord; Z Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-pollCallsWhenSafe-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareClassic;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/launcher2/PagedViewWithDraggableItems;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/server/sip/SipService;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-fadeOutChallenge-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/usb/UsbService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-runAsleep-(J J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/AccessibilityInjector;-onPageStarted-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/StructuredPostalDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleUp-(Landroid/view/MotionEvent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-startDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/TetherSettings;-initWifiTethering-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothInputDevice;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/SlotView;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-onCancelled-(Landroid/filterfw/core/AsyncRunner$RunnerResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/ContactBrowseListFragment$ContactLookupTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-ndefWrite-(I Landroid/nfc/NdefMessage;)I' : ['android.permission.NFC'],
	'Lcom/android/mms/data/Conversation;-get-(Landroid/content/Context; Landroid/net/Uri; Z)Lcom/android/mms/data/Conversation;' : ['android.permission.READ_SMS'],
	'Landroid/bluetooth/BluetoothDevice;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Landroid/media/Ringtone;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-onWakeMotionWhenKeyguardShowingTq-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/exchange/EasSyncService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/RuimRecords;-handleRuimRefresh-(Lcom/android/internal/telephony/IccRefreshResponse;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothInputDevice;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/sip/ISipService$Stub;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupForegroundResumeBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserBookmarksPage$2;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/activities/AttachPhotoActivity$2;-onLoadComplete-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/EditableInputConnection;-finishComposingText-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionActivity;-createPhonebookDialogView-()Landroid/view/View;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/LinkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-interceptMotionBeforeQueueingWhenScreenOff-(I)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/server/BluetoothA2dpService;-resumeSink-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/WallpaperManagerService;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/soundrecorder/SoundRecorder;-updateUi-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/wifi/WifiStatusTest;-handleNetworkStateChanged-(Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasSyncService;-searchGal-(Landroid/content/Context; J Ljava/lang/String; I)Lcom/android/exchange/provider/GalResult;' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewCore;-jsPrompt-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/data/WorkingMessage;-updateDraftMmsMessage-(Landroid/net/Uri; Lcom/google/android/mms/pdu/PduPersister; Lcom/android/mms/model/SlideshowModel; Lcom/google/android/mms/pdu/SendReq; Ljava/util/HashMap;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/systemui/statusbar/tv/TvStatusBar;-showSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Landroid/media/effect/effects/FillLightEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$ApplicationSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/ImportCompleteListener;-onProgressStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ThrottleService$DataRecorder;-setNextPeriod-(Ljava/util/Calendar; Ljava/util/Calendar;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockScreen;-<init>-(Landroid/content/Context; Landroid/content/res/Configuration; Lcom/android/internal/widget/LockPatternUtils; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetheredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/preferences/InvertedContrastPreview;-onCreateView-(Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/wifi/WifiSettings;-submit-(Lcom/android/settings/wifi/WifiConfigController;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onRoamingOn-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/downloads/DownloadInfo;-startIfReady-(J Lcom/android/providers/downloads/StorageManager;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidWordLevelSpellCheckerSession;-onGetSuggestions-(Landroid/view/textservice/TextInfo; Ljava/lang/String; I)Landroid/view/textservice/SuggestionsInfo;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/media/IRingtonePlayer;-stop-(Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/filterpacks/imageproc/CropFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-launchMultiplePhonePicker-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/EventInfoFragment;-updateEvent-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-makeCFQueryResultMessage-(Lcom/android/internal/telephony/CallForwardInfo; I)Ljava/lang/CharSequence;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getRemoteName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchToImage-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/SipBroadcastReceiver;-takeCall-(Landroid/content/Intent;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/RIL;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/AccessibilitySettings;-onResume-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/IdentityDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/adapter/CalendarSyncAdapter$EasCalendarSyncParser;-changeParser-(Lcom/android/exchange/adapter/CalendarSyncAdapter$CalendarOperations;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/systemui/ImageWallpaper$DrawableEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/wifi/SupplicantStateTracker$InactiveState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/nfc/INfcAdapter;-setForegroundDispatch-(Landroid/app/PendingIntent; [Landroid/content/IntentFilter; Landroid/nfc/TechListParcel;)V' : ['android.permission.NFC'],
	'Lcom/android/contacts/detail/ContactDetailFragment$ViewAdapter;-getHeaderEntryView-(Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-conference-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/accounts/AccountSyncSettings;-onSyncStateUpdated-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS'],
	'Landroid/webkit/ZoomManager;-zoomIn-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$KeyboardLayoutPickerActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-access$300-(Landroid/media/MediaPlayer; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-isValidWord-(Ljava/lang/CharSequence;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/InCallScreen;-access$1200-(Lcom/android/phone/InCallScreen;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/media/effect/SingleFilterEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/TwilightService$LocationHandler;-requestLocationUpdate-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-<init>-(Landroid/content/Context; Ljava/lang/String; Z)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/list/CustomContactListFilterActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/AccessibilityInjector;-removeTtsApis-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/http/RequestQueue;-access$300-(Landroid/net/http/RequestQueue;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-onLongPressTimeout-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/systemui/media/NotificationPlayer;-enqueueLocked-(Lcom/android/systemui/media/NotificationPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-<init>-(Landroid/content/Context; Ljava/util/HashMap;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/systemui/statusbar/policy/NetworkController;-<init>-(Landroid/content/Context;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/DockObserver;-updateLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/RetargetFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/PieListView;-layout-(I I Z F I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/CalendarController;-launchDeleteEvent-(J J J)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/camera/SecureCameraActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/InCallTouchUi;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Ljava/net/HttpURLConnection;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/browser/Controller;-handleNewIntent-(Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/BluetoothPan;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/detail/ContactDetailFragment$DetailViewEntry;-fromValues-(Landroid/content/Context; Lcom/android/contacts/model/dataitem/DataItem; Z J)Lcom/android/contacts/detail/ContactDetailFragment$DetailViewEntry;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothPan;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/systemui/SystemUI;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/WebView;-restoreState-(Landroid/os/Bundle;)Landroid/webkit/WebBackForwardList;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/MessageItem;-<init>-(Landroid/content/Context; Ljava/lang/String; Landroid/database/Cursor; Lcom/android/mms/ui/MessageListAdapter$ColumnsMap; Ljava/util/regex/Pattern;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewCore$EventHub;-access$7200-(Landroid/webkit/WebViewCore$EventHub; Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/P2pEventListener;-onP2pHandoverNotSupported-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/MessagingPreferenceActivity;-setRingtoneSummary-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/SyncManager;-access$500-(Landroid/content/SyncManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/SlideshowView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-deleteSmsOnSim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-getBondState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/base/FrameBranch;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/RIL;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$7;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/BrowserWebViewFactory;-instantiateWebView-(Landroid/util/AttributeSet; I Z)Landroid/webkit/WebView;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/TrafficStats;-startDataProfiling-(Landroid/content/Context;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Connected;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothA2dp;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/TetherSettings;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/calendar/AllInOneActivity;-initFragments-(J I Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/SMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/SmsUsageMonitor;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/accounts/AccountManager$7;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$RunningServicesActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/DeliveryReportActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/MoviePlayer;-<init>-(Landroid/view/View; Lcom/android/gallery3d/app/MovieActivity; Landroid/net/Uri; Landroid/os/Bundle; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-load-(Lcom/android/dreams/phototable/PhotoSource$ImageData; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothInputDevice;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapUtils;-createProfileVCard-(Landroid/content/Context; I [B)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/google/common/io/CharStreams;-toString-(Lcom/google/common/io/InputSupplier;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/accounts/AccountManager;-addAccount-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/sip/SipSettings$7;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/PhoneBase;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipManager;-open-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/exchange/ExchangeService$AccountObserver$2;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/AccessibilitySettings$ToggleAccessibilityServicePreferenceFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-onScreenTurnedOn-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewManager$ShowListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-sendSMS-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/LatinIME$UIHandler;-postUpdateSuggestionStrip-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/BluetoothPhoneService;-listCurrentCallsCdma-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/google/SearchBaseUrlHelper$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.INTERNET'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-startSync-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Browser;-deleteHistoryWhere-(Landroid/content/ContentResolver; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onGestureMoveEvent-(I I J Z Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/app/AlbumPage;-onAlbumModeSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/wifi/WifiStateMachine$DriverStartedState;-exit-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView$1;-onPressKey-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/EdgeView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/webkit/PluginFullScreenHolder$CustomFrameLayout;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasOutboxService;-sendSettings-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/LockdownVpnTracker;-initLocked-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/IccFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-afterTextChanged-(Landroid/text/Editable;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ims/IsimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_MOCK_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/transaction/NotificationPlayer;-access$500-(Lcom/android/mms/transaction/NotificationPlayer; Lcom/android/mms/transaction/NotificationPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCall;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/AccessibilityInjector;-access$300-(Landroid/webkit/AccessibilityInjector;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/StraightenEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-snapToDestination-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/inputmethod/InputMethodAndSubtypeEnabler;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/videosrc/MediaSource;-setupMediaPlayer-(Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/snep/SnepServer$ServerThread;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/connectivity/Tethering;-sendTetherStateChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/Utils;-formatMonthYear-(Landroid/content/Context; Landroid/text/format/Time;)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/IBluetooth;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetooth$Stub;-startDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/ShortcutIntentBuilder$PhoneNumberLoadingAsyncTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/DataUsageSummary;-hasReadyMobileRadio-(Landroid/content/Context;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/widget/SlidingTab;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.VIBRATE', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService$BatteryReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/StructuredNameDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/PhoneUtils;-startGetCallerInfo-(Landroid/content/Context; Lcom/android/internal/telephony/Call; Lcom/android/internal/telephony/CallerInfoAsyncQuery$OnQueryCompleteListener; Ljava/lang/Object;)Lcom/android/phone/PhoneUtils$CallerInfoToken;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/transaction/PrivilegedSmsReceiver;-finishStartingService-(Landroid/app/Service; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-queryPhonebook-(Ljava/lang/String; Lcom/android/bluetooth/hfp/AtPhonebook$PhonebookResult;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/Phone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccCardApplication;-supplyPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAbsKeyInputView;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/GroupMembershipDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/WifiService;-checkAndStartWifi-()V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/calllog/CallLogFragment;-updateOnExit-()V' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/providers/contacts/ContactsProvider2;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; Landroid/os/CancellationSignal;)Landroid/database/Cursor;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/CommandQueue$Callbacks;-disable-(I)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/Invert;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/voicedialer/CommandRecognizerEngine;-setupGrammar-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/OtaUtils;-onClickHandler-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/media/RingtonePlayer$1;-stopAsync-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-installPackageLI-(Lcom/android/server/pm/PackageManagerService$InstallArgs; Z Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/DayFragment;-onResume-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/providers/downloads/RealSystemFacade;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ConnectivityService;-getTetherableUsbRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/p2p/IWifiP2pManager$Stub$Proxy;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/google/common/io/ByteStreams;-getDigest-(Lcom/google/common/io/InputSupplier; Ljava/security/MessageDigest;)[B' : ['android.permission.INTERNET'],
	'Lcom/android/server/ConnectivityService;-handleConnectionFailure-(Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/group/GroupEditorFragment$2;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/IRingtonePlayer$Stub$Proxy;-playAsync-(Landroid/net/Uri; Landroid/os/UserHandle; Z I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewProvider;-postUrl-(Ljava/lang/String; [B)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-asyncReloadDictionaryIfRequired-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/interactions/PhoneNumberInteraction$PhoneItemAdapter;-<init>-(Landroid/content/Context; Ljava/util/List; Lcom/android/contacts/interactions/PhoneNumberInteraction$InteractionType;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/nfc/tech/MifareUltralight;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/DataConnection$DcDisconnectingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-cancelBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AlbumPage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-getDrmMimeSavedStringRsrc-(J Z)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-doFinish-()V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/WebViewCore;-access$1300-(Landroid/webkit/WebViewCore;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/BackupManagerService$BackupRestoreTask;-operationComplete-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/util/NameConverter;-displayNameToStructuredName-(Landroid/content/Context; Ljava/lang/String; Landroid/content/ContentValues;)Landroid/content/ContentValues;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/filterpacks/imageproc/Invert;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/editor/PhoneticNameEditorView;-notifyEditorListener-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetooth;-getRemoteClass-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewClassic;-letPageHandleNavKey-(I J Z I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewProvider;-saveWebArchive-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/StructuredNameDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/calendar/CalendarProvider2;-getAccount-(J)Landroid/accounts/Account;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/preferences/AccessibilityPreferencesFragment;-onPause-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/util/PduLoaderManager$PduTask;-run-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/RIL;-getCdmaBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/WirelessSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/data/WorkingMessage;-readDraftMmsMessage-(Landroid/content/Context; Lcom/android/mms/data/Conversation; Ljava/lang/StringBuilder;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-showChallenge-(Z I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/IContentService$Stub$Proxy;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/contacts/list/PhoneNumberListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/SnailItem;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/ConnectivityService;-handleConnectivityChange-(I Z)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/CallController;-placeCallInternal-(Landroid/content/Intent;)Lcom/android/phone/Constants$CallStatusCode;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/UiController;-toggleUserAgent-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/deviceinfo/UsbSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$TetherSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/ISms$Stub;-disableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-createList-(I I Ljava/lang/String; Ljava/lang/StringBuilder; Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/data/Conversation;-blockingMarkAllSmsMessagesAsSeen-(Landroid/content/Context;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/PhoneBase;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-setWifiEnabled-(Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-isKeyguardSecure-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/adapter/Search$SearchParser;-parse-()Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/accounts/ManageAccountsSettings;-requestOrCancelSyncForAccounts-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/systemui/statusbar/policy/BluetoothController;-updateBondedBluetoothDevices-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/model/dataitem/EmailDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-startSettingsActivity-(Landroid/content/Intent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/gallery3d/ui/PhotoView$Model;-moveTo-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/Ringtone;-destroyLocalPlayer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService$2;-loadAttachment-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardMessageArea$2;-onRefreshCarrierInfo-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-showDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityView;-onPause-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-detectCountry-()Landroid/location/Country;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-getSuggestions-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo;)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/calllog/CallLogFragment;-onStop-()V' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-generateBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/list/ContactListFilterControllerImpl;-setContactListFilter-(Lcom/android/contacts/list/ContactListFilter; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/sip/SipSessionGroup;-processResponse-(Ljavax/sip/ResponseEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupCreatingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewManager$KeyguardViewHost;-dispatchDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/nfc/NfcService;-onSeListenActivated-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/SoundRecorder;-onStateChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/SystemVibrator;-vibrate-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-onPressed-(Landroid/view/View; Z)V' : ['android.permission.VIBRATE'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-showSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-setRadios-(Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/LocationManagerService;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManager;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/common/contacts/DataUsageStatUpdater;-updateWithPhoneNumber-(Ljava/util/Collection;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/google/common/io/CharStreams;-copy-(Lcom/google/common/io/InputSupplier; Ljava/lang/Appendable;)J' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView$1;-onStartBatchInput-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/transaction/MessagingNotification;-getSmsNewDeliveryInfo-(Landroid/content/Context;)Lcom/android/mms/transaction/MessagingNotification$MmsSmsDeliveryInfo;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-access$500-(Lcom/android/gallery3d/ui/MenuExecutor; Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-fadeChallenge-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-hapticAndAudioFeedback-(I)V' : ['android.permission.VIBRATE'],
	'Landroid/preference/SeekBarDialogPreference;-showDialog-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/DataConnection$DcDisconnectingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothPbap;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$SpellCheckersSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/alerts/AlertActivity;-dismissFiredAlarms-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/ui/PhotoView$MyGestureListener;-onUp-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-onVisibilityChanged-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAccountsByTypeAsUser-(Ljava/lang/String; Landroid/os/UserHandle;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/AccountManager$9;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$3;-goToLockScreen-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/Controller;-removeTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/webkit/WebViewClassic;-setNetworkAvailable-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/Phone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-clearProfileConnectionState-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/research/ResearchLogger;-commitCurrentLogUnit-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/inputmethod/latin/LatinIME;-onCodeInput-(I I I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/widget/SearchView$SearchAutoComplete;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/LocationManagerService;-handleLocationChanged-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/UsimDataDownloadHandler;-handleDataDownload-(Lcom/android/internal/telephony/gsm/SmsMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getNetworkInfo-(I)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothAdapter;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ThrottleService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/transaction/TransactionService;-isNetworkAvailable-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/ui/ConversationList;-confirmDeleteThreads-(Ljava/util/Collection; Landroid/content/AsyncQueryHandler;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/AirplaneModeEnabler;-resume-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$LocationSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/base/GLTextureSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService$Stub$Proxy;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMSI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/GroupMemberLoader;-<init>-(Landroid/content/Context; J [Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/IntentHandler$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/latin/LatinIME;-access$900-(Lcom/android/inputmethod/latin/LatinIME; I)Lcom/android/inputmethod/latin/SuggestedWords;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/ui/MessageListView;-onSizeChanged-(I I I I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/base/FrameStore;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-handlePinMmi-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/CalendarSyncEnabler;-enableEasCalendarSync-()V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/speech/IRecognitionService;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-scrollLeft-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/ContextImpl;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/editor/ContactEditorFragment$5;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/sip/SipEditor;-unregisterOldPrimaryAccount-()V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-next-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewInputDispatcher;-enqueueHitTestLocked-(Landroid/view/MotionEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hdp/HealthService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/systemui/statusbar/tablet/SettingsView;-onFinishInflate-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/location/ILocationManager$Stub;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/vcard/ImportProcessor;-readOneVCard-(Ljava/io/InputStream; I Ljava/lang/String; Lcom/android/vcard/VCardInterpreter; [I)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/UsimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SelectionManager;-leaveSelectionMode-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/camera/CameraModule;-onPauseAfterSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/NavScreen;-switchToTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-onRemoteDisconnect-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/utility/CalendarUtilities;-createMessageForEventId-(Landroid/content/Context; J I Ljava/lang/String; Lcom/android/emailcommon/provider/Account;)Lcom/android/emailcommon/provider/EmailContent$Message;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$3900-(Lcom/android/mms/ui/ComposeMessageActivity; J Z)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-next-(Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-refresh-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/webkit/AutoCompletePopup;-pushTextToInputConnection-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame;-onKeyguardVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/A2dpProfile;-getPreferred-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-addCachedDevices-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/location/GpsLocationProvider;-enable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/CallNotifier;-onResendMute-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/view/IApplicationToken$Stub$Proxy;-keyDispatchingTimedOut-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/RIL;-setFacilityLockForApp-(Ljava/lang/String; Z Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/TimeIntervalPopup;-access$000-(Lcom/android/camera/ui/TimeIntervalPopup;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/PowerManagerService;-startWatchingForBootAnimationFinished-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/RuimRecords;-onAllRecordsLoaded-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-handleApplicationCrash-(Landroid/os/IBinder; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/RIL;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-setMobileDataEnabled-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/google/common/io/MultiInputStream;-skip-(J)J' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phone/CallNotifier;-onNewRingingConnection-(Landroid/os/AsyncResult;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-sendCDMAFeatureCode-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-access$200-(Lcom/android/server/location/ComprehensiveCountryDetector;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-access$000-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-keyguardDone-(Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/list/ProviderStatusWatcher$2;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/policy/NetworkController;-huntForSsid-(Landroid/net/wifi/WifiInfo;)Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-dispatchBroadcastMessage-(Landroid/telephony/SmsCbMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/HTML5VideoView;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onDisconnect-(Lcom/android/internal/telephony/Connection$DisconnectCause;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-fetchRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/BluetoothVisibilityTimeoutFragment;-<init>-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewInputDispatcher;-access$800-(Landroid/webkit/WebViewInputDispatcher; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/location/ILocationManager$Stub;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-turnOnUpstreamMobileConnection-(I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/PeopleActivity$TabPagerListener;-onPageSelected-(I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/BluetoothManagerService;-waitForOnOff-(Z Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService$WiredAccessoryCallbacks;-notifyWiredAccessoryChanged-(J I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/pm/PackageManagerService$10;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/NetworkStateTracker;-startMonitoring-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Downloads;-removeAllDownloadsByPackage-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/quicksearchbox/util/SingleThreadNamedTaskExecutor$Worker;-run-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/DayView;-updateEventDetails-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/ex/variablespeed/SingleThreadedMediaPlayerProxy;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPasswordView;-verifyPasswordAndUnlock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/EditableInputConnection;-sendCurrentText-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CsimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_MOCK_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/ByteStreams$4;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-invokeAgentForBackup-(Ljava/lang/String; Landroid/app/IBackupAgent; Lcom/android/internal/backup/IBackupTransport;)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebViewClassic;-init-(Ljava/util/Map; Z)V' : ['android.permission.INTERNET'],
	'Landroid/media/AudioService;-resetBluetoothSco-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/telephony/TelephonyManager;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/ComboSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-fetchBtClass-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/OtaUtils;-otaShowSuccessFailure-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setGsmBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/ContactListEmptyView;-show-(Z Z Z Z Z Z Z)V' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-handleCpbrCommand-(Ljava/lang/String; I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/SyncManager;-scheduleSync-(Landroid/accounts/Account; I Ljava/lang/String; Landroid/os/Bundle; J Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/dreams/phototable/PicasaSource;-findImages-(I)Ljava/util/Collection;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/wallpaper/grass/GrassRS;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame;-create-(Landroid/content/Context; Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame$Callbacks; Lcom/android/internal/policy/impl/keyguard/KeyguardActivityLauncher;)Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/SoundSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebViewClassic$PrivateHandler;-dispatchUiEvent-(Landroid/view/MotionEvent; I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/OrganizationDataItem;-getKindTypeColumn-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/view/ViewRootImpl;-doProcessInputEvents-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-isValidWord-(Ljava/lang/CharSequence;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/dialpad/DialpadImageButton;-performClick-()Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hdp/HealthService;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/LatinIME;-onCurrentInputMethodSubtypeChanged-(Landroid/view/inputmethod/InputMethodSubtype;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DigitalClock;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/dialog/ClearFrequentsDialog$1$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/ContactSaveService;-joinContacts-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-startMsgListQuery-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/bluetooth/HeadsetProfile;-setPreferred-(Landroid/bluetooth/BluetoothDevice; Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/settings/Settings$SpellCheckersSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-verifyUnlock-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/SIMFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/InCallScreen;-onMMICancel-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ManageSimMessages;-copyToPhoneMemory-(Landroid/database/Cursor;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/PieControl;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-startUsingNetworkFeature-(I Ljava/lang/String; Landroid/os/IBinder;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-<init>-(Landroid/view/View; Lcom/android/gallery3d/app/MovieActivity; Landroid/net/Uri; Landroid/os/Bundle; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/provider/ExchangeDirectoryProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/voicedialer/VoiceDialerActivity;-onDestroy-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/contacts/ContactSaveService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/location/ILocationManager;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-updateProfiles-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/wifi/AdvancedWifiSettings;-onPreferenceChange-(Landroid/preference/Preference; Ljava/lang/Object;)Z' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/SnapshotTab;-setController-(Lcom/android/browser/WebViewController;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/providers/contacts/VoicemailTable$Delegate;-insert-(Lcom/android/providers/contacts/VoicemailContentProvider$UriData; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/phone/InCallScreen;-onOpenCloseDialpad-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/TetherSettings;-setUsbTethering-(Z)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/GLRootView;-onDrawFrame-(Ljavax/microedition/khronos/opengles/GL10;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/IConnectivityManager;-getAllNetworkInfo-()[Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/vcard/VCardEntry$EntryElementIterator;-onElement-(Lcom/android/vcard/VCardEntry$EntryElement;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/phone/ManageConferenceUtils;-updateManageConferenceRow-(I Lcom/android/internal/telephony/Connection; Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-animateCollapsePanels-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-sendMessage-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-exitKeyguardSecurely-(Landroid/view/WindowManagerPolicy$OnKeyguardExitResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handleRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/SupplicantStateTracker$DormantState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.INTERNET', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetooth;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/SlideshowPage$ShuffleSource;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cat/IconLoader;-readIconData-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-<init>-(J)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetooth;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/net/NetworkPolicyEditor$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothPbap$Stub$Proxy;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/HTML5VideoInline;-reprepareData-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/ViewRootImpl;-setView-(Landroid/view/View; Landroid/view/WindowManager$LayoutParams; Landroid/view/View;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/vcard/VCardEntryConstructor;-onEntryEnded-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/phone/OtaUtils;-onOtaCloseFailureNotice-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/systemui/usb/UsbResolverActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Landroid/filterpacks/base/NullFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/location/fused/FusionEngine;-deinit-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PasswordUnlockScreen;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl;-onStatusChanged-(I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/PhoneGlobals;-orientationChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockScreen$WaveViewMethods;-onGrabbedStateChange-(Landroid/view/View; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/group/GroupDetailDisplayUtils;-bindGroupSourceView-(Landroid/content/Context; Landroid/view/View; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/data/WorkingMessage;-syncWorkingRecipients-()V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/IMediaScannerListener;-scanCompleted-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothHeadset;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewProperties;-createKeyguardView-(Landroid/content/Context; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewCallback; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardWindowController;)Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewBase;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/AudioService;-access$2700-(Landroid/media/AudioService;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/ComboAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/effect/effects/SaturateEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/inputmethod/keyboard/internal/SuddenJumpingTouchEventHandler;-handleSuddenJumping-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/DataUsageSummary$ConfirmDataDisableFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/activities/GroupEditorActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$InputMethodAndLanguageSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/PhotoView;-snapback-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/transaction/MessagingNotification;-addMmsNotificationInfos-(Landroid/content/Context; Ljava/util/Set; Ljava/util/SortedSet;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/media/IAudioService$Stub;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine;-sendNetworkStateChangeBroadcast-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$3500-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/ViewRootImpl;-dispatchKeyFromIme-(Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-snapToPage-(I I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/IConnectivityManager;-requestRouteToHostAddress-(I [B)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone; Landroid/telephony/CellInfo;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ITelephony;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-loadDraft-()Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/WifiService;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/settings/Settings$InputMethodAndSubtypeEnablerActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/google/common/io/CharStreams;-readFirstLine-(Lcom/google/common/io/InputSupplier;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/ui/GLView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/filterpacks/imageproc/BlendFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/launcher2/AppsCustomizePagedView;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Ljava/net/ServerSocket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/base/FrameStore;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/quickcontact/QuickContactActivity;-access$700-(Lcom/android/contacts/quickcontact/QuickContactActivity; Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/BrowserActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS', 'com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.USE_CREDENTIALS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/IMediaPlaybackService;-next-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-assignContactFromEmail-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountAdapter;-doSaveAction-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/AdnRecordLoader;-updateEF-(Lcom/android/internal/telephony/AdnRecord; I I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$DateTimeSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$SpeakerphoneListener;-onClick-(Landroid/view/View;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onError-(Landroid/net/sip/SipAudioCall; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowSession$Stub;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/development/Connectivity$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage;-access$400-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager;-getLastTetherError-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/base/FrameStore;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/CameraSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/TetherSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-notifyPageSwitching-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MovieActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-refresh-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/soundrecorder/Recorder;-signalStateChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView$TitleAdapter;-runQueryOnBackgroundThread-(Ljava/lang/CharSequence;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityView;-reset-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/CommandQueue$Callbacks;-topAppWindowChanged-(Z)V' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setDiscoverableTimeout-(I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/ui/ComposeMessageActivity$17;-run-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/ExchangeService;-startServiceThread-(Lcom/android/exchange/AbstractSyncService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/webkit/WebViewClassic;-updateJavaScriptEnabled-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManager$13;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-getContactUriForEmail-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/FrameFetch;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverTransfer;-processFiles-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetooth$Stub;-getBondedDevices-()[Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$2400-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-onLongPress-(Lcom/android/inputmethod/keyboard/Key; Lcom/android/inputmethod/keyboard/PointerTracker;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/CalendarContract$Reminders;-query-(Landroid/content/ContentResolver; J [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/PieControl;-buildTabs-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-access$500-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/providers/downloads/DownloadThread;-writeDataToDestination-(Lcom/android/providers/downloads/DownloadThread$State; [B I)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/IAccountManager;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/soundrecorder/Recorder;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/TabControl;-stopAllLoading-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSelectorView;-updateTargets-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/Context;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mms/util/Recycler;-anyThreadOverLimit-(Landroid/content/Context;)Z' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/agenda/AgendaListView$2;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendTerminalResponse-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareClassic;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitorCallback;-onClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/event/AttendeesView$PresenceQueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/content/ISyncAdapter$Stub$Proxy;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-dialThreeWay-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/location/fused/FusionEngine;-updateRequirements-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/DisplaySettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-prepareEmptyView-()V' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/camera/ActivityBase;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PasswordUnlockScreen;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/contacts/activities/AttachPhotoActivity;-saveContact-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$TetherSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/VideoModule;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/webkit/GeolocationService;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-conference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/MessagingPreferenceActivity;-onPreferenceChange-(Landroid/preference/Preference; Ljava/lang/Object;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/CommandQueue$Callbacks;-addNotification-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)V' : ['android.permission.VIBRATE'],
	'Landroid/location/ILocationManager;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-isVoicemailAvailable-()Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-loadUrl-(Lcom/android/browser/Tab; Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WifiService;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewBase;-wakeWhenReadyTq-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/calendar/DeleteEventHelper;-deleteExceptionEvent-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mms/ui/ConversationList$DeleteThreadListener;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/PhotoPage;-wantBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/webkit/WebViewClassic;-stopTouch-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/vcard/VCardEntry$NicknameData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/UrlHandler$RLZTask;-onPostExecute-(Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-onFinishInflate-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$TextToSpeechSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/os/IVibratorService$Stub$Proxy;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/event/EditEventFragment;-setModelIfDone-(I)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterfw/core/Filter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/Settings$DeviceAdminSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/ThrottleService$MyHandler;-onResetAlarm-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebView;-goBackOrForward-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/MessagingNotification;-blockingUpdateNewMessageIndicator-(Landroid/content/Context; J Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/browser/DataController$DataControllerHandler;-doUpdateVisitedHistory-(Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/latin/spellcheck/DictionaryPool;-poll-(J Ljava/util/concurrent/TimeUnit;)Lcom/android/inputmethod/latin/spellcheck/DictAndProximity;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/sip/SipEditor;-onPreferenceChange-(Landroid/preference/Preference; Ljava/lang/Object;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/voicedialer/VoiceDialerActivity$2;-run-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/BrowserWebView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.INTERNET'],
	'Landroid/net/wifi/IWifiManager;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-onStart-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-bindEditors-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/SystemUI;-start-()V' : ['android.permission.VIBRATE', 'android.permission.DISABLE_KEYGUARD', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserWebView;-clearMatches-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-onScroll-(Landroid/widget/AbsListView; I I I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/support/v4/net/ConnectivityManagerCompat$JellyBeanConnectivityManagerCompatImpl;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/deviceinfo/Memory;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ConversationList;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_SMS'],
	'Landroid/nfc/tech/NfcA;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/phone/PhoneGlobals;-handleQueryTTYModeResponse-(Landroid/os/Message;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/browser/BrowserSettings;-updateConnectionType-()V' : ['android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/util/Recycler;-deleteOldMessagesByThreadId-(Landroid/content/Context; J)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-handleMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.CALL_PHONE', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$AccountAnalyzer;-<init>-(Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView; Landroid/accounts/AccountManager;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$3;-showBackupSecurity-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/INetworkPolicyManager;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-access$100-(Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityViewFlipper;-onPause-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WiredAccessoryManager$WiredAccessoryObserver;-init-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/DefaultContactListAdapter;-getFirstContactUri-()Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/RuimSmsInterfaceManager;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/webkit/CallbackProxy;-switchOutDrawHistory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-compareTo-(Ljava/lang/Object;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/phone/CdmaSystemSelectListPreference;-<init>-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/UserDictionary$Words;-addWord-(Landroid/content/Context; Ljava/lang/String; I I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/vcard/VCardParser;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/DataUsageSummary$4;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/NavTabScroller$OnRemoveListener;-onRemovePosition-(I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/sip/ISipService$Stub;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService;-callHealthChannelCallback-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/BluetoothDevice; I I Landroid/os/ParcelFileDescriptor; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/nfc/handover/HandoverService;-doHeadsetHandover-(Landroid/os/Message;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-processPostDialChar-(C)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/SlideshowEditActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/detail/ContactDetailDisplayUtils$StreamPhotoTag;-getStreamItemPhotoUri-()Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/PhoneInterfaceManager;-getNeighboringCellInfo-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/provider/BrowserProvider2;-isValidParent-(Ljava/lang/String; Ljava/lang/String; J)Z' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/RadioInfo;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$3500-(Lcom/android/gallery3d/ui/PhotoView; F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewClassic;-onPause-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/ContactSaveService;-createGroup-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/deviceinfo/Status;-setIpAddressStatus-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/VideoModule;-onShutterButtonClick-()V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/location/LocationManager;-getProvider-(Ljava/lang/String;)Landroid/location/LocationProvider;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/bluetooth/HeadsetProfile;-getPreferred-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-stayAwake-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$3;-takeEmergencyCallAction-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-invokeEnterMethods-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/event/EditEventView;-setAllDayViewsVisibility-(Z)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/Settings$RunningServicesActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/location/GeofenceManager;-sendIntent-(Landroid/app/PendingIntent; Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-startObexServerSession-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/calendar/CalendarProviderIntentService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setAndBroadcastNetworkSetTime-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/TabControl;-freeMemory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getAllNetworkState-()[Landroid/net/NetworkState;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/NsdService;-sendNsdStateChangeBroadcast-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/ActivityController;-onSaveInstanceState-(Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/editor/PhoneticNameEditorView;-onOptionalFieldVisibilityChange-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/PhoneUtils;-access$100-(Lcom/android/internal/telephony/Phone; Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothDiscoverableEnabler;-onPreferenceClick-(Landroid/preference/Preference;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManager$8;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupAllConnections-(Lcom/android/internal/telephony/gsm/GsmCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/StatusBarManagerService$7;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/nfc/handover/HandoverService;-enableBluetooth-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/IWifiManager;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-getConnectedHealthDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/TelephonyRegistry;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/LinkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-updatePowerStateLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/OutgoingCallBroadcaster;-processIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.PROCESS_OUTGOING_CALLS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/PreloadedTabControl;-loadUrlIfChanged-(Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub;-setPasskey-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/os/SystemVibrator;-vibrate-([J I)V' : ['android.permission.VIBRATE'],
	'Landroid/nfc/INfcTag;-getTimeout-(I)I' : ['android.permission.NFC'],
	'Landroid/widget/VideoView;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DayView$GotoBroadcaster;-onAnimationEnd-(Landroid/view/animation/Animation;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/exchange/adapter/AbstractSyncAdapter;-sendLocalChanges-(Lcom/android/exchange/adapter/Serializer;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/DevelopmentSettings;-onPreferenceChange-(Landroid/preference/Preference; Ljava/lang/Object;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/hid/HidService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebViewCore;-sendMessage-(I I Ljava/lang/Object;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/JoinContactActivity;-onQueryTextChange-(Ljava/lang/String;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/AggregationSuggestionEngine;-buildAggregationSuggestionUri-(Lcom/android/contacts/model/RawContactDelta$ValuesDelta;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/sip/SipSettings;-addProfile-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$22;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/pan/PanService;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/IWifiP2pManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/wifi/WifiApConfigStore$DefaultState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/FindActionModeCallback;-onTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl;-access$1500-(Lcom/android/server/sip/SipSessionGroup$SipSessionImpl; Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/inputmethod/InputMethodManager$2;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-openAssetFileLocal-(Landroid/net/Uri; Ljava/lang/String;)Landroid/content/res/AssetFileDescriptor;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/systemui/recent/RecentsActivity;-handleIntent-(Landroid/content/Intent; Z)V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Landroid/bluetooth/BluetoothPbap;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothPbap$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/content/SyncManager;-access$2100-(Landroid/content/SyncManager;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/pm/PackageManagerService$AppDirObserver;-onEvent-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/sip/SipPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-getTetherableIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchToFirstImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/users/UserSettings$3;-doInBackground-([Ljava/lang/Void;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onSetPolicyDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/CropImage$LoadDataTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/contacts/activities/ActionBarAdapter$Listener;-onAction-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/DeviceAccountLogin;-login-(I Lcom/android/browser/DeviceAccountLogin$AutoLoginCallback;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/list/OnContactBrowserActionListener;-onSelectionChange-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/provider/Telephony$Mms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Lcom/android/calendar/CalendarSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$DateTimeSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/provider/BrowserProvider2;-deleteBookmarks-(Ljava/lang/String; [Ljava/lang/String; Z)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/sip/ISipService;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/nfc/tech/MifareClassic;-increment-(I I)V' : ['android.permission.NFC'],
	'Lcom/android/mms/transaction/MessagingNotification$3;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_SMS'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-sendCurrentText-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/IWindowSession$Stub;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/webkit/HTML5Audio;-<init>-(Landroid/webkit/WebViewCore; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$1300-(Lcom/android/server/net/NetworkPolicyManagerService;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/OtaUtils;-startNonInteractiveOtasp-(Landroid/content/Context;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/Browser;-onCreate-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/VibratorService$3;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/contacts/list/PostalAddressPickerFragment;-onClose-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getGsmBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/Connectivity;-onRoutedSocketRequest-()V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/net/sip/ISipService$Stub;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupAllConnections-(Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/IInputMethodClient$Stub$Proxy;-onBindMethod-(Lcom/android/internal/view/InputBindResult;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManager$4;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/SlotView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-addImage-(Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$StartingState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$AccessibilitySettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/UI;-setUseQuickControls-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/accounts/IAccountManager$Stub;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/calendar/AllInOneActivity;-updateSecondaryTitleFields-(J)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/Settings$DataUsageSummaryActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/QuickContactDivot;-assignContactFromPhone-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/AddBookmarkPage$1;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/CropImage$LoadDataTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Landroid/graphics/BitmapRegionDecoder;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/gallery3d/app/MoviePlayer;-access$900-(Lcom/android/gallery3d/app/MoviePlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/WidgetUtils;-createWidgetBitmap-(Lcom/android/gallery3d/data/MediaItem;)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/dreams/phototable/PhotoTable;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-addNotification-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/base/ObjectSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/DayView;-<init>-(Landroid/content/Context; Lcom/android/calendar/CalendarController; Landroid/widget/ViewSwitcher; Lcom/android/calendar/EventLoader; I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-disableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/socialwidget/SocialWidgetProvider$1;-onLoadComplete-(Landroid/content/Loader; Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/wifi/AdvancedWifiSettings;-updateSleepPolicySummary-(Landroid/preference/Preference; Ljava/lang/String;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-viewContact-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/WifiManager;-updateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IccProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/HTML5VideoFullScreen;-stopPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumSetPage$5;-onActionItemClicked-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/IccCard;-supplyPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/IWindowManager;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/nfc/INfcAdapterExtras$Stub;-getCardEmulationRoute-(Ljava/lang/String;)I' : ['android.permission.NFC'],
	'Landroid/nfc/tech/IsoDep;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Display;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$CaptivePortalCheckState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityRecord;-windowsVisible-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$PairingCancelHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/wifi/WifiEnabler;-setSwitch-(Landroid/widget/Switch;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/effect/effects/TintEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-isSyncActive-()Z' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/activities/PeopleActivity;-onSelectedTabChanged-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/AudioManager;-setWiredDeviceConnectionState-(I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService$2;-validate-(Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimUnlockScreen$TouchInput;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/sip/SipSessionGroup;-processTimeout-(Ljavax/sip/TimeoutEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/DictionaryCollection;-isValidWord-(Ljava/lang/CharSequence;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/ZoomManager;-zoomToOverview-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/ServiceWatcher$1;-onSomePackagesChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/tech/NfcB;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$StateTracker;-getActualState-(Landroid/content/Context;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/IRingtonePlayer;-stopAsync-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-notifyDataChange-([I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityView;-onResume-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setCdmaBroadcastConfig-([I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-acknowledgeLastIncomingGsmSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserWebView;-goBack-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readAdnFileAndWait-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms;-getAllMessagesFromIccEf-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/FaceUnlock;-handleReportFailedAttempt-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-render-(Lcom/android/gallery3d/ui/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/ExchangeService$CalendarObserver;-<init>-(Lcom/android/exchange/ExchangeService; Landroid/os/Handler; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/camera/PhotoModule$2;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-separate-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-getSuggestions-(Lcom/android/inputmethod/latin/WordComposer; Ljava/lang/CharSequence; Lcom/android/inputmethod/keyboard/ProximityInfo;)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$AccountAnalyzer;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/interactions/PhoneNumberInteraction$PhoneDisambiguationDialogFragment;-onCreateDialog-(Landroid/os/Bundle;)Landroid/app/Dialog;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getLinkProperties-(I)Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$BondStateChangedHandler;-showUnbondMessage-(Landroid/content/Context; Ljava/lang/String; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AlbumPage$4;-onSingleTapUp-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-revertToNitzTime-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/GeneralPreferences;-onPreferenceChange-(Landroid/preference/Preference; Ljava/lang/Object;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/detail/PhotoSelectionHandler;-getPhotoPickSize-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPukForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/PrivilegedSmsReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CallTracker;-handleRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity$PersistTask;-doInBackground-([Lcom/android/contacts/model/RawContactDeltaList;)Ljava/lang/Integer;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryFacilityLock-(Ljava/lang/String; Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/os/BatteryStatsImpl;-writeToParcel-(Landroid/os/Parcel; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/os/IPowerManager$Stub$Proxy;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/SoundSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-switchHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaBroadcastConfig-([I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-setGlobalProxy-(Landroid/net/ProxyProperties;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/am/ActiveServices;-performServiceRestartLocked-(Lcom/android/server/am/ServiceRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/exchange/EasOutboxService;-postAutodiscover-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/IInputMethodManager;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getBondState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-access$700-(Lcom/android/systemui/statusbar/phone/QuickSettings; Ljava/lang/String;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/bluetooth/BluetoothDevicePreference$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/net/LockdownVpnTracker;-handleStateChangedLocked-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnection;-tearDownData-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-onGetContent-(Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/adapter/CalendarSyncAdapter;-parse-(Ljava/io/InputStream;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/BaseStatusBar$5$1;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-access$1000-(Lcom/android/bluetooth/btservice/BondStateMachine; Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothPbap$Stub$Proxy;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Telephony$Sms$Inbox;-addMessage-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long; Z)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Browser;-canClearHistory-(Landroid/content/ContentResolver;)Z' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/LocationManagerService;-removeProviderLocked-(Lcom/android/server/location/LocationProviderInterface;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService;-clearAllScoClients-(I Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/ActivityManagerNative;-goingToSleep-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/list/ProfileAndContactsLoader;-loadProfile-()Landroid/database/MatrixCursor;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/Settings$LocationSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/util/Recycler$SmsRecycler;-deleteOldMessages-(Landroid/content/Context;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/phone/SimContacts$ImportAllSimContactsThread;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/data/WorkingMessage;-asyncDeleteDraftMmsMessage-(Lcom/android/mms/data/Conversation;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onLongPress-(Landroid/inputmethodservice/Keyboard$Key;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/music/MediaPlaybackActivity;-toggleShuffle-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/interactions/ImportExportDialogFragment;-doShareVisibleContacts-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/provider/BrowserProvider2$DatabaseHelper;-onCreate-(Landroid/database/sqlite/SQLiteDatabase;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/LatinIME$BatchInputUpdater;-onEndBatchInput-(Lcom/android/inputmethod/latin/InputPointers; Lcom/android/inputmethod/latin/LatinIME;)Lcom/android/inputmethod/latin/SuggestedWords;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/transaction/NotificationTransaction;-<init>-(Landroid/content/Context; I Lcom/android/mms/transaction/TransactionSettings; Ljava/lang/String;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/nfc/NfcService$TagService;-resetTimeouts-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/sip/SipPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager;-updateMessageOnIccEf-(I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/ThrottleService;-getThrottle-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/model/dataitem/PhotoDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cat/CatService;-handleRilMsg-(Lcom/android/internal/telephony/cat/RilMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/ContactsActivity;-onPrepareOptionsMenu-(Landroid/view/Menu;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/settings/applications/RunningProcessesView$TimeTicker;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-invokeNextAgent-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/power/PowerManagerService;-updateIsPoweredLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/TetherSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/LocationManagerService;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/net/wifi/IWifiManager;-startScan-(Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getRemoteClass-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/PhoneNumberPickerFragment;-finish-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/camera/PhotoModule;-access$3100-(Lcom/android/camera/PhotoModule; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/ShutdownThread;-rebootOrShutdown-(Z Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/nfc/tech/Ndef;-writeNdefMessage-(Landroid/nfc/NdefMessage;)V' : ['android.permission.NFC'],
	'Lcom/android/music/AudioPreview;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClient;-onUnhandledKeyEvent-(Landroid/webkit/WebView; Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor;-handlePhoneStateChanged-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/phone/InCallScreen;-getUpdatedInCallControlState-()Lcom/android/phone/InCallControlState;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$AccountAnalyzer;-<init>-(Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView; Landroid/accounts/AccountManager; Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$1;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/performance/ThroughputFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothInputDevice;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/providers/downloads/DownloadService;-updateDownload-(Lcom/android/providers/downloads/DownloadInfo$Reader; Lcom/android/providers/downloads/DownloadInfo; J)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/interactions/ImportExportDialogFragment;-handleImportRequest-(I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/calendar/CalendarProvider2;-updateInTransaction-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String; Z)I' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/base/NullFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoSource;-next-(Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-access$200-(Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView; Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$Mode; Z)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiStateMachine$DisconnectingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/EasSyncService;-acknowledgeRemoteWipe-(Lcom/android/exchange/EasSyncService; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/RequestPermissionActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/calllog/PhoneNumberHelper;-isVoicemailNumber-(Ljava/lang/CharSequence;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-performLayoutAndPlaceSurfacesLockedLoop-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-autoConnectA2dp-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-processMsg-(Landroid/os/Message;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.INTERNET', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/text/StringLogger;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/VoicemailCleanupService;-handleIntentInternal-(Landroid/content/Intent; Landroid/content/ContentResolver;)V' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/webkit/WebViewProvider;-setNetworkAvailable-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/NegativeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BreadCrumbView$Controller;-onTop-(Lcom/android/browser/BreadCrumbView; I Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/DayView;-doSingleTapUp-(Landroid/view/MotionEvent;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/settings/DisplaySettings;-onResume-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManager;-getAuthTokenLabel-(Ljava/lang/String; Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$800-(Lcom/android/gallery3d/ui/PhotoView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPasswordView;-onWindowFocusChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/quicksearchbox/QueryTask;-run-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/event/EditEventView;-setCalendarsCursor-(Landroid/database/Cursor; Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar$23;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Landroid/net/wifi/WifiApConfigStore$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView$1;-onCustomRequest-(I)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/WifiManager$MulticastLock;-acquire-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/voicedialer/PhoneTypeChoiceRecognizerEngine;-recognize-(Lcom/android/voicedialer/RecognizerClient; Landroid/app/Activity; Ljava/io/File; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-dispatch-(Landroid/content/Intent; Ljava/lang/String; Landroid/content/BroadcastReceiver;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/SupplicantStateTracker$CompletedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phone/OtaUtils;-otaShowProgramFailure-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetooth$Stub;-getAdapterConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$3100-(Lcom/android/mms/ui/ComposeMessageActivity; J)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/google/common/io/Files;-readLines-(Ljava/io/File; Ljava/nio/charset/Charset;)Ljava/util/List;' : ['android.permission.INTERNET'],
	'Lcom/android/browser/PreloadRequestReceiver;-isOnWifi-(Landroid/content/Context;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/SettingsLicenseActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.INTERNET'],
	'Landroid/net/wifi/WifiManager;-addNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/ILocationManager;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/location/ILocationManager;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/webkit/WebViewClassic;-onPageStarted-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/support/v4/net/ConnectivityManagerCompat$HoneycombMR2ConnectivityManagerCompatImpl;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/CropView$FaceHighlightView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/MmsMessageSender;-sendMessage-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothA2dp;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/nfc/tech/MifareClassic;-readBlock-(I)[B' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-operationComplete-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-proceedAfterWaitChar-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/CameraSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/TetherSettings;-onPreferenceChange-(Landroid/preference/Preference; Ljava/lang/Object;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/location/GpsLocationProvider;-handleInjectNtpTime-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/JoinContactActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/model/dataitem/OrganizationDataItem;-buildDataString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-getVoiceRadioTechnology-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-getLastKnownLocation-(Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-<init>-(Lcom/android/server/usb/UsbDeviceManager; Landroid/os/Looper;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/os/IHardwareService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-save-(I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/phone/MobileNetworkSettings;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-setBtClass-(Landroid/bluetooth/BluetoothClass;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/CalendarViewAdapter;-buildWeekNum-()Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-cleanup-()Z' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/ConnectivityManager;-updateLockdownVpn-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/phone/PhoneInterfaceManager;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_FINE_LOCATION', 'android.permission.CALL_PHONE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$BluetoothStateTracker;-getActualState-(Landroid/content/Context;)I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/location/CountryDetectorBase;-detectCountry-()Landroid/location/Country;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/pan/PanService;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/content/ContentService;-systemReady-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/BrowserWebView;-setFrame-(I I I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Pending;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/CallContactActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/google/common/io/ByteStreams;-copy-(Lcom/google/common/io/InputSupplier; Lcom/google/common/io/OutputSupplier;)J' : ['android.permission.INTERNET'],
	'Landroid/net/wifi/WifiStateMachine$InitialState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/nfc/snep/SnepServer$ConnectionThread;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/ObscureSpeechDelegate;-sendAccessibilityEvent-(Landroid/view/View; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-updateNetworkEnabledLocked-()V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/quicksearchbox/util/JavaNetHttpHelper;-createConnection-(Ljava/lang/String; Ljava/util/Map;)Ljava/net/HttpURLConnection;' : ['android.permission.INTERNET'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-selectAccountAndCreateContact-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/Controller;-goBackOnePageOrQuit-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-handleEnterEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub$Proxy;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/test/mock/MockContentResolver;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mms/ui/ConversationListAdapter;-onContentChanged-()V' : ['android.permission.READ_SMS'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-queryInternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/MediaPlaybackService;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$LocationSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/soundrecorder/SoundRecorder;-updateTimerView-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IRestoreSession$Stub$Proxy;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-onStop-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/AbstractSyncService;-validateAccount-(Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Landroid/view/View;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/telephony/SmsProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/net/wifi/WifiStateMachine$VerifyingLinkState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-handleSimStateChange-(Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor$SimArgs;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onPhantomUpEvent-(J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/hid/HidService;-access$100-(Lcom/android/bluetooth/hid/HidService; Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/nfc/NfcService$NfcServiceHandler;-llcpActivated-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)Z' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/server/NsdService$NsdStateMachine$EnabledState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/gsm/SmsManager;-updateMessageOnSim-(I I [B)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/settings/bluetooth/BluetoothDevicePreference;-compareTo-(Landroid/preference/Preference;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/DeleteEventHelper$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum$FetchCache;-getItem-(I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/DisplayPowerController$5;-run-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/AudioManager;-isWiredHeadsetOn-()Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-onBluetoothStateChanged-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-getHealthDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PatternUnlockScreen;-onWindowFocusChanged-(Z)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-createUnlockScreenFor-(Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$UnlockMode;)Landroid/view/View;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/VideoView$2;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccCardApplication;-queryPin1State-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-bindChildView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/ActivityPicker;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/contacts/activities/GroupEditorActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/MediaPlaybackService;-enqueue-([J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onCustomRequest-(I)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/WifiStateMachine$ConnectModeState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/UiModeManagerService;-access$400-(Lcom/android/server/UiModeManagerService; I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/DataConnectionTracker$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/transaction/MessagingNotification$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/RIL;-writeSmsToSim-(I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/calendar/CalendarProviderBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/ImDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/commands/pm/Pm;-run-([Ljava/lang/String;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/fuelgauge/PowerUsageSummary;-addBluetoothUsage-(J)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/PatternUnlockScreen$2;-onClick-(Landroid/view/View;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/AccountManagerService;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/LocationManagerService;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-removeTrack-(J)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebView;-setFrame-(I I I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/ShowOrCreateActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Ljava/lang/String; J F Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService$1;-onChange-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/HTML5VideoView;-stopPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/JoinContactListAdapter;-getContactUri-(I Landroid/database/Cursor; I I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-isForegroundApp-(Ljava/lang/String;)Z' : ['android.permission.GET_TASKS'],
	'Lcom/android/contacts/vcard/ImportProcessor;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-getCdmaSubscriptionSource-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/IconLoader;-startLoadingIcon-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/LegacyContactListAdapter;-getPersonUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-selectFirstFoundContactAfterDelay-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/RecipientListActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/browser/WebViewController;-switchToTab-(Lcom/android/browser/Tab;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/base/FrameFetch;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/view/IInputMethodClient;-onUnbindMethod-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wm/InputMonitor;-dispatchUnhandledKey-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.CALL_PHONE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/AutoCorrection;-isInTheDictionary-(Ljava/util/concurrent/ConcurrentHashMap; Ljava/lang/CharSequence; Z)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/net/DataUsageMeteredSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/provider/BrowserProvider2$DatabaseHelper;-onUpgrade-(Landroid/database/sqlite/SQLiteDatabase; I I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/SubtypeSwitcher;-initialize-(Landroid/content/Context;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-isSyncActive-()Z' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onLongTap-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-startUsingContactsDictionaryLocked-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackFragment$PlaybackViewImpl;-setSpeakerPhoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/preference/VolumePreference;-onDialogClosed-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/development/SyncAdapterDriver;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-showNextSecurityScreenIfPresent-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$VpnSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/BluetoothEnabler;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothInputDevice;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/SnapshotTab;-removeFromTree-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler;-getWritableEntityId-()J' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-bindStandAloneView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/IccLockSettings;-tryChangeIccLockState-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/DownloadThread;-checkConnectivity-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/PhoneFavoriteMergedAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-onAddView-(Landroid/view/View; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/UndoBarView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-finish-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/editor/LabeledEditorView;-onFieldChanged-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/IWifiManager$Stub;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/ConnectivityManager;-getLinkProperties-(I)Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/inputmethod/UserDictionaryAddWordActivity;-onClickConfirm-(Landroid/view/View;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Landroid/webkit/WebViewClassic$PastePopupWindow;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/ChallengeLayout;-showBouncer-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/BackupManagerService$BackupRestoreTask;-handleTimeout-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/WebViewController;-onUserCanceledSsl-(Lcom/android/browser/Tab;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/googlecode/mp4parser/authoring/builder/DefaultMp4Builder;-build-(Lcom/googlecode/mp4parser/authoring/Movie;)Lcom/coremedia/iso/IsoFile;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/webkit/WebViewCore;-destroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/LinkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;-onBluetoothStateChanged-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/calllog/CallLogQueryHandler;-markNewCallsAsOld-()V' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-play-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-hideSearchPanel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onRemoteDisconnect-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/ProviderStatusWatcher$LoaderTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/BluetoothEnabler;-resume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/ConnectivityManager;-untether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/content/IContentService$Stub;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-isTetheringSupported-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-launchAssistLongPressAction-()V' : ['android.permission.VIBRATE'],
	'Landroid/webkit/WebViewClassic;-performAccessibilityAction-(I Landroid/os/Bundle;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-<init>-(Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/alerts/AlertService;-doTimeChanged-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/os/BatteryStatsImpl;-getCurrentBluetoothPingCount-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$12;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/SmsUsageMonitor;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/VideoController;-onListPrefChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothInputDevice;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-initialize-(Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/activities/TransactionSafeActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/sip/SipCommandInterface;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/QuickContactBadge;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/ManageCachePage$4;-onLongTap-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/camera/CameraActivity$1;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/power/PowerManagerService$1;-onStateChanged-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/WifiStateMachine$SoftApStartedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$ManageAccountsSettingsActivity;-setListAdapter-(Landroid/widget/ListAdapter;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/SlideViewInterface;-pauseVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/Conversation;-markAsRead-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/provider/BrowserProvider2;-setParentValues-(J Landroid/content/ContentValues;)Z' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityCallback;-reportFailedUnlockAttempt-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/ContextImpl;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-processMessagePart-([B Ljava/lang/String; I I I J I Z)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/nfc/tech/IsoDep;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPINView;-resetState-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/SlideshowPresenter$2;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onLlcpLinkActivated-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/settings/net/UidDetailProvider;-getUidDetail-(I Z)Lcom/android/settings/net/UidDetail;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-enqueue-([J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityCallback;-getFailedAttempts-()I' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-dumpRenderTree-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/AudioManager;-setParameter-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/IThrottleManager$Stub$Proxy;-getThrottle-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$PowerUsageSummaryActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$TetheredState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/ZoomManager$ScaleDetectorListener;-handleScale-(Landroid/view/ScaleGestureDetector;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/data/WorkingMessage;-access$900-(Lcom/android/mms/data/WorkingMessage; Lcom/android/mms/data/Conversation; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-query-(Landroid/database/sqlite/SQLiteDatabase; Landroid/database/sqlite/SQLiteQueryBuilder; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/CancellationSignal;)Landroid/database/Cursor;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/providers/contacts/DbModifierWithNotification;-insert-(Ljava/lang/String; Ljava/lang/String; Landroid/content/ContentValues;)J' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/bluetooth/btservice/BondStateMachine$PendingCommandState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/BluetoothPairingDialog;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/providers/media/RingtonePickerActivity;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/ContactEditorFragment$PhotoHandler;-<init>-(Landroid/content/Context; Landroid/view/View; I Z Lcom/android/contacts/model/RawContactDeltaList;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/pan/PanService;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/media/MediaScannerConnection$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/filterfw/core/FinalPort;-setFieldFrame-(Landroid/filterfw/core/Frame; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/ContactPhotoManagerImpl$LoaderThread;-preloadPhotosInBackground-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BATTERY_STATS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/PhoneBase;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewInputDispatcher;-dispatchWebKitEvent-(Landroid/view/MotionEvent; I I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getPreferredVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/DeskClockFragment;-onPageChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Disconnected;-enter-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-registerReceiver-(Landroid/app/IApplicationThread; Ljava/lang/String; Landroid/content/IIntentReceiver; Landroid/content/IntentFilter; Ljava/lang/String; I)Landroid/content/Intent;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/wifi/WifiSettings;-refreshAccessPoints-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/exchange/EasSyncService;-postAutodiscover-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-handlePinMmi-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/dialpad/DigitsEditText;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/PeopleActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/model/RawContactModifier;-parseStructuredNameExtra-(Landroid/content/Context; Lcom/android/contacts/model/account/AccountType; Lcom/android/contacts/model/RawContactDelta; Landroid/os/Bundle;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$1;-onBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-queryWithContactIdAndLookupKey-(Landroid/database/sqlite/SQLiteQueryBuilder; Landroid/database/sqlite/SQLiteDatabase; Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J Ljava/lang/String; Ljava/lang/String; Landroid/os/CancellationSignal;)Landroid/database/Cursor;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/PhoneBase;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/PhotoModule;-setLocationPreference-(Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/PhoneInterfaceManager;-access$100-(Lcom/android/phone/PhoneInterfaceManager;)V' : ['android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService$9;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/usb/UsbDeviceManager;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/app/IAlarmManager;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-disable-(Z)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/RecipientsEditor;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-sendEnvelope-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkPolicyManager$Stub;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/model/dataitem/EmailDataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnection;-tearDownData-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NetworkStateHandler$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-expectVoiceRecognition-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener$Adapter;-onCodeInput-(I I I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/WebViewCore$3;-onReceiveValue-([Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/MessageListAdapter$OnDataSetChangedListener;-onContentChanged-(Lcom/android/mms/ui/MessageListAdapter;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-tether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar$3;-onTouch-(Landroid/view/View; Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/webkit/HTML5VideoViewProxy$VideoPlayer;-pause-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/mms/ui/SlideEditorActivity;-access$400-(Lcom/android/mms/ui/SlideEditorActivity;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/AccessibilitySettings$ToggleScreenMagnificationPreferenceFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/app/NetInitiatedActivity;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/JoinContactListFragment;-onClose-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onRoamingOff-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHealth;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothA2dp;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onSelectionModeChange-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/webkit/WebViewClassic;-onKeyMultiple-(I I Landroid/view/KeyEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/CropImage;-setAsWallpaper-(Lcom/android/gallery3d/util/ThreadPool$JobContext; Landroid/graphics/Bitmap;)Z' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/settings/Settings$LocationSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover;-nextStepDisconnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/Preloader$PreloaderSession$1;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/app/AlertActivity;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler;-getWritableEntityIndex-()I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/nfc/NfcService$NfcAdapterExtrasService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings;-addSystemTiles-(Landroid/view/ViewGroup; Landroid/view/LayoutInflater;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothA2dp;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/transaction/NotificationTransaction;-sendPdu-([B)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/commands/bmgr/Bmgr;-doRestore-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator$1;-onBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/WebViewController;-shouldOverrideUrlLoading-(Lcom/android/browser/Tab; Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/WebViewController;-closeTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/preference/ContactsPreferences;-registerChangeListener-(Lcom/android/contacts/preference/ContactsPreferences$ChangeListener;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppUtility;-queryRecord-(Landroid/content/Context; Landroid/net/Uri;)Lcom/android/bluetooth/opp/BluetoothOppTransferInfo;' : ['android.permission.BLUETOOTH'],
	'Landroid/media/IAudioService$Stub;-setBluetoothA2dpDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice; I)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$InitialState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothAdapter;-readOutOfBandData-()Landroid/util/Pair;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/EventInfoFragment;-access$100-(Lcom/android/calendar/EventInfoFragment; Landroid/view/View;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiManager$WifiLock;-acquire-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/GoogleAccountLogin;-startLogin-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/LocalAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker$2;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/bluetooth/DevicePickerFragment;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendSMS-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewCore$EventHub;-access$7300-(Landroid/webkit/WebViewCore$EventHub; Landroid/os/Message; J)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/speech/IRecognitionService$Stub;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/google/common/io/MultiInputStream;-advance-()V' : ['android.permission.INTERNET'],
	'Lcom/android/browser/provider/BrowserProvider;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-onDeviceAdded-(Lcom/android/settings/bluetooth/CachedBluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/inputmethodservice/KeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/systemui/SearchPanelView$GlowPadTriggerListener;-onTrigger-(Landroid/view/View; I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/phone/PhoneGlobals;-updateWakeState-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;-onProgressComplete-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/camera/ui/ListPrefSettingPopup$Listener;-onListPrefChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-restoreSelectedUri-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-notifyScreenOnLocked-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewManager$ShowListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/tv/TvStatusBar;-addNotificationViews-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)Lcom/android/systemui/statusbar/StatusBarIconView;' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/ISms$Stub;-updateMessageOnIccEf-(I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-cleanupPhotoStore-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/location/LocationManager;-addGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/MoreSettingPopup$Listener;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebView;-documentHasImages-(Landroid/os/Message;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-onCreate-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/calendar/agenda/AgendaFragment$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mms/ui/SlideshowPresenter$4;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DeleteEventHelper;-delete-(J J J I Ljava/lang/Runnable;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-dispatchMediaKeyWithWakeLockToAudioService-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-setUserActivityTimeoutOverrideFromWindowManagerInternal-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-compareTo-(Lcom/android/settings/bluetooth/CachedBluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/commands/monkey/Monkey;-getSystemInterfaces-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/SubSettings;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/DataUsage;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/location/GeofenceManager;-addFence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; I Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManager;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/VolumePanel;-onMuteChanged-(I I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onError-(Lcom/android/internal/telephony/Connection$DisconnectCause;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/Switch;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$DisplaySettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/detail/ContactLoaderFragment;-loadUri-(Landroid/net/Uri;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/nfc/NfcService$EnableDisableTask;-doInBackground-([Ljava/lang/Integer;)Ljava/lang/Void;' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-updateLegacyPhotoData-(J J Landroid/content/ContentValues;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-removeTracks-(I I)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/SMSDispatcher;-notifyAndAcknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$3;-getFailedAttempts-()I' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/WebsiteDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/NotificationMgr;-onQueryComplete-(I Ljava/lang/Object; Lcom/android/internal/telephony/CallerInfo;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/contacts/model/dataitem/DataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar$29;-run-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/launcher2/SmoothPagedView;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/view/WindowManagerPolicy;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.CALL_PHONE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/providers/contacts/DatabaseModifier;-delete-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-updateCallLogContactInfoCache-(Ljava/lang/String; Ljava/lang/String; Lcom/android/contacts/calllog/ContactInfo; Lcom/android/contacts/calllog/ContactInfo;)V' : ['android.permission.WRITE_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/net/IConnectivityManager$Stub;-setUsbTethering-(Z)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/nfc/INfcTag$Stub;-getTimeout-(I)I' : ['android.permission.NFC'],
	'Lcom/android/settings/bluetooth/PanProfile;-getConnectionStatus-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-processProfileStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/deviceinfo/UsbSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/event/EditEventFragment$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/WebViewClassic;-setActive-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/AllInOneActivity$3;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/AddBookmarkPage;-setAccount-(Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/music/AudioPreview$2;-onAudioFocusChange-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/wifi/WifiSettings;-updateConnectionState-(Landroid/net/NetworkInfo$DetailedState;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPatternView$2;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcAdapterExtras;-open-(Ljava/lang/String; Landroid/os/IBinder;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Landroid/app/PendingIntent$FinishedDispatcher;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/systemui/recent/RecentsActivity;-dismissAndGoBack-()V' : ['android.permission.REORDER_TASKS', 'android.permission.GET_TASKS'],
	'Landroid/webkit/WebIconDatabaseClassic;-requestIconForPageUrl-(Ljava/lang/String; Landroid/webkit/WebIconDatabase$IconListener;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/IccCard;-supplyPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl;-process-(Ljava/util/EventObject;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardPINView$1;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/NsdService;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Landroid/webkit/WebViewProvider;-loadDataWithBaseURL-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary$AsyncReloadDictionaryTask;-run-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/vcard/VCardParserImpl_V30;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewClassic;-scrollEditWithCursor-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-onPostExecute-(Landroid/filterfw/core/AsyncRunner$RunnerResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-reportSmsMemoryStatus-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccCard;-setIccLockEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/MessageUtils;-isLocalNumber-(Ljava/lang/String;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-onBondingStateChanged-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/PhotoPage;-onFilmModeChanged-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/DreamSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onPressKey-(I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/Transaction;-sendPdu-([B Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-setBluetoothEnabled-(Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/PostalAddressListAdapter;-bindQuickContact-(Lcom/android/contacts/list/ContactListItemView; I Landroid/database/Cursor; I I I I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/pm/PackageManagerService;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/contacts/activities/JoinContactActivity;-onStart-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/DataController$DataControllerHandler;-handleMessage-(Lcom/android/browser/DataController$DCMessage;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/launcher2/ButtonDropTarget;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/PhoneUtils;-hangupRingingAndActive-(Lcom/android/internal/telephony/Phone;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mms/ui/MessagingPreferenceActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/SnapshotBar;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/ContactsApplication$DelayedInitializer;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-notifyDefaultData-(Lcom/android/internal/telephony/ApnContext;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothSocket;-connect-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/activities/ActionBarAdapter$MyNavigationListener;-onNavigationItemSelected-(I J)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/sip/SipService;-unregisterReceivers-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/BluetoothPhoneService;-handleListCurrentCalls-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-setRssi-(S)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/nfc/NfcService$NfcAdapterExtrasService;-open-(Ljava/lang/String; Landroid/os/IBinder;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/mms/data/Contact$ContactsCache;-getContactInfoForPhoneUris-([Landroid/os/Parcelable;)Ljava/util/List;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/base/FrameStore;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$2900-(Lcom/android/mms/ui/ComposeMessageActivity; J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/ConnectivityService;-access$2400-(Lcom/android/server/ConnectivityService;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/settings/Settings$LocalePickerActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout$9;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$ApplicationSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/activities/ContactEditorActivity;-onServiceCompleted-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView$1;-onStartBatchInput-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/model/dataitem/SipAddressDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/transaction/Transaction;-sendPdu-([B)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-loadMessageContent-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$5800-(Lcom/android/mms/ui/ComposeMessageActivity; Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/IWifiManager;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/PhoneUi;-onBackKey-()Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/CallTracker;-pollCallsWhenSafe-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmAlertWakeLock;-acquireCpuWakeLock-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-sendPendingNotificationsLocked-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService;-setQueuePosition-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/data/Conversation;-getUri-()Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/app/IBatteryStats$Stub$Proxy;-getAwakeTimePlugged-()J' : ['android.permission.BATTERY_STATS'],
	'Landroid/media/AudioService;-access$6600-(Landroid/media/AudioService; Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/base/RetargetFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$NotConnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.INTERNET', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/A2dpProfile;-isA2dpPlaying-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/NetworkStateHandler;-onNetworkToggle-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/location/GpsLocationProvider$5;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/voicedialer/CommandRecognizerEngine;-onRecognitionSuccess-(Lcom/android/voicedialer/RecognizerClient;)V' : ['android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/test/IsolatedContext$MockAccountManager;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/calendar/DayView;-onCreateContextMenu-(Landroid/view/ContextMenu; Landroid/view/View; Landroid/view/ContextMenu$ContextMenuInfo;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/webkit/WebViewClassic;-onHandleUiEvent-(Landroid/view/MotionEvent; I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$ProvisionDiscoveryState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS'],
	'Lcom/android/server/ConnectivityService;-getNetworkInfo-(I)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-initializeData-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterfw/core/FilterFunction;-execute-(Landroid/filterfw/core/KeyValueMap;)Landroid/filterfw/core/Frame;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/ContextWrapper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewCallback;-keyguardDone-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/providers/downloads/StorageManager$1;-run-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/settings/bluetooth/BluetoothSettings;-onDevicePreferenceClick-(Lcom/android/settings/bluetooth/BluetoothDevicePreference;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/nfc/INfcTag;-connect-(I I)I' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/Phone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IRestoreSession$Stub;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/calendar/CalendarProvider2;-handleInsertException-(J Landroid/content/ContentValues; Z)J' : ['android.permission.READ_CALENDAR'],
	'Landroid/view/IWindowManager$Stub$Proxy;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.DISABLE_KEYGUARD', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/StatusBarManagerService;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/server/am/ActivityManagerService;-forceStopPackageLocked-(Ljava/lang/String; I Z Z Z Z I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/emailcommon/utility/AttachmentUtilities;-saveAttachment-(Landroid/content/Context; Ljava/io/InputStream; Lcom/android/emailcommon/provider/EmailContent$Attachment;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/ApplicationPackageManager;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Landroid/net/IConnectivityManager$Stub;-isTetheringSupported-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/google/common/io/CharStreams;-readLines-(Lcom/google/common/io/InputSupplier; Lcom/google/common/io/LineProcessor;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/base/RetargetFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/phone/CallNotifier;-onPhoneStateChanged-(Landroid/os/AsyncResult;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/launcher2/StrokedTextView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/vcard/VCardEntryCommitter;-onEntryCreated-(Lcom/android/vcard/VCardEntry;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/PhoneFallbackEventHandler;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothA2dp;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/downloads/DownloadThread;-reportProgress-(Lcom/android/providers/downloads/DownloadThread$State; Lcom/android/providers/downloads/DownloadThread$InnerState;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/systemui/SearchPanelView;-vibrate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/mms/ui/ComposeMessageActivity$39;-onSizeChanged-(I I I I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/music/MediaPlaybackService;-doAutoShuffleUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/InCallTouchUi;-refreshAudioModePopup-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-onPageChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getBasebandVersion-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$DeviceAdminSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-interceptMediaKey-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/view/IWindowSession;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/UiController;-stopLoading-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/os/PowerManager$WakeLock;-acquire-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStack;-awakeFromSleepingLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/VideoView;-openVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/Files;-getChecksum-(Ljava/io/File; Ljava/util/zip/Checksum;)J' : ['android.permission.INTERNET'],
	'Lcom/android/server/power/PowerManagerService;-goToSleepNoUpdateLocked-(J I)Z' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/speech/SpeechRecognizer;-access$200-(Landroid/speech/SpeechRecognizer;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-access$100-(Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/media/effect/effects/ColorTemperatureEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/ExchangeService$2;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-setCurrentPage-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/EditableInputConnection;-commitText-(Ljava/lang/CharSequence; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/MessageListAdapter;-getCachedMessageItem-(Ljava/lang/String; J Landroid/database/Cursor;)Lcom/android/mms/ui/MessageItem;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/downloads/DownloadInfo$Reader;-newDownloadInfo-(Landroid/content/Context; Lcom/android/providers/downloads/SystemFacade;)Lcom/android/providers/downloads/DownloadInfo;' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/calendar/DayView$8;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/list/ContactListFilterControllerImpl;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/CallDetailActivity$1UpdateContactDetailsTask;-onPostExecute-([Lcom/android/contacts/PhoneCallDetails;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/providers/contacts/VoicemailContentProvider;-checkPermissionsAndCreateUriDataForReadOperation-(Landroid/net/Uri;)Lcom/android/providers/contacts/VoicemailContentProvider$UriData;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pEnabledState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BluetoothManagerService;-access$600-(Lcom/android/server/BluetoothManagerService;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/WebViewTimersControl;-resumeTimers-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/detail/StreamItemAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/ImportCompleteListener;-goToImportedAlbum-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/InCallScreen;-disconnectBluetoothAudio-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CallerInfo;-doSecondaryLookupIfNecessary-(Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/CallerInfo;)Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-updateMessageOnIccEf-(I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/settings/DataUsageSummary;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/nfc/tech/NfcV;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-access$100-(Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/exchange/EasAccountService;-sendSettings-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadsetPhone$Stub$Proxy;-hangupCall-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Landroid/webkit/WebViewClassic;-onOverScrolled-(I I Z Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-getBluetoothState-()I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/editor/PhotoActionPopup$Listener;-onUseAsPrimaryChosen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/LoadedApk$ReceiverDispatcher$InnerReceiver;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/UserDictionarySettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-loadMessageContent-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/FaceUnlock;-handleCancel-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/app/ChooserActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Landroid/net/IConnectivityManager;-setNetworkPreference-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/AccountsTester;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/MediaPlaybackService;-removeTracks-(I I)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-interceptKeyBeforeDispatching-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)J' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$5;-onClick-(Landroid/view/View;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/server/WallpaperManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_WALLPAPER_HINTS', 'android.permission.SET_WALLPAPER'],
	'Landroid/app/IBackupAgent$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-p2pConnectWithPinDisplay-(Landroid/net/wifi/p2p/WifiP2pConfig;)V' : ['android.permission.GET_TASKS'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-updateNotification-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)V' : ['android.permission.VIBRATE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pNotSupportedState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-reconnectAfterFail-(Lcom/android/internal/telephony/DataConnection$FailCause; Lcom/android/internal/telephony/ApnContext; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/socialwidget/SocialWidgetProvider;-bindRemoteViews-(Landroid/content/Context; I Landroid/appwidget/AppWidgetManager; Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/SipCallOptionHandler;-isNetworkConnected-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/os/PowerManager$WakeLock;-setWorkSource-(Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-access$900-(Lcom/android/gallery3d/app/AlbumPage; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/wifi/WifiStateMachine;-isWifiTethered-(Ljava/util/ArrayList;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/commands/svc/Svc;-main-([Ljava/lang/String;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/text/StringLogger;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/inputmethod/UserDictionaryAddWordFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/RIL;-stopDtmf-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ITelephony$Stub;-call-(Ljava/lang/String;)V' : ['android.permission.CALL_PHONE'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-refresh-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/TelephonyRegistry;-notifyServiceState-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/settings/bluetooth/BluetoothDeviceFilter$Filter;-matches-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-pollStateDone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/settings/AccessibilitySettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCallHoldIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/InLineSettingItem$Listener;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-setupDataCall-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasOutboxService;-sendMeetingResponse-(Lcom/android/exchange/MeetingResponseRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/group/GroupDetailFragment$2;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/CursorLoader;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/handover/HandoverServer$ServerThread;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/Utils;-setTimeZone-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/NavScreen;-access$100-(Lcom/android/browser/NavScreen; Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/BatteryService$7;-run-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/contacts/ContactsProvider2;-insertInTransaction-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/EmailAddressListAdapter;-getDataUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/ZoomManager;-updateDefaultZoomDensity-(F)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/MediaPlayer;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-rejectCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/OtaUtils;-onOtaProvisionStatusChanged-(Landroid/os/AsyncResult;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-onClose-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$GpsStateTracker;-getTriState-(Landroid/content/Context;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/effect/SizeChangeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment$2;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/BackupManagerService;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/ProxySelector;-saveToDb-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/provider/ContactsContract$Contacts;-markAsContacted-(Landroid/content/ContentResolver; J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter$ReloadTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Void;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pNotSupportedState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-saveDraft-(Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/VibratorService;-vibratePattern-([J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/contacts/ContactPhotoManagerImpl$LoaderThread;-queryPhotosForPreload-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/imageproc/BlendFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE', 'android.permission.WRITE_SETTINGS', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setProfileAutoConnectionPriority-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/util/IState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/LocationSettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/google/common/io/MultiReader;-<init>-(Ljava/util/Iterator;)V' : ['android.permission.INTERNET'],
	'Landroid/location/LocationManager;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-initializeHdmiState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/transaction/SmsReceiverService;-handleSendMessage-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/Controller;-openTab-(Ljava/lang/String; Z Z Z Lcom/android/browser/Tab;)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/providers/downloads/DownloadThread;-readFromResponse-(Lcom/android/providers/downloads/DownloadThread$State; Lcom/android/providers/downloads/DownloadThread$InnerState; [B Ljava/io/InputStream;)I' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/IConnectivityManager;-setGlobalProxy-(Landroid/net/ProxyProperties;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/provider/Telephony$Threads;-getOrCreateThreadId-(Landroid/content/Context; Ljava/lang/String;)J' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/os/BatteryStatsImpl;-setOnBattery-(Z I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-notifyApnIdUpToCurrent-(Ljava/lang/String; Lcom/android/internal/telephony/ApnContext; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardMessageArea$2;-onBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/PosterizeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onModifiedTouchEvent-(Landroid/view/MotionEvent; Z)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/ex/chips/BaseRecipientAdapter$DefaultFilter;-performFiltering-(Ljava/lang/CharSequence;)Landroid/widget/Filter$FilterResults;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/providers/telephony/MmsSmsProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Landroid/content/Context;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/NavScreen;-close-(I Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/SMSDispatcher;-dispatchNormalMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/calllog/PhoneNumberHelper;-canSendSmsTo-(Ljava/lang/CharSequence;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiStateMachine;-access$12400-(Landroid/net/wifi/WifiStateMachine; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/LockPatternView$OnPatternListener;-onPatternDetected-(Ljava/util/List;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$AccountSyncSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$WifiDisplaySettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-copyMessageToIccEf-(I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/contacts/calllog/VoicemailNotifier;-updateNotification-(Landroid/net/Uri;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/SnapshotTab;-<init>-(Lcom/android/browser/WebViewController; Landroid/webkit/WebView; Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/accounts/IAccountManagerResponse$Stub$Proxy;-onResult-(Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/WebViewController;-openTab-(Ljava/lang/String; Lcom/android/browser/Tab; Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/widget/LockPatternView;-notifyPatternDetected-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/calllog/PhoneNumberHelper;-getDisplayNumber-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)Ljava/lang/CharSequence;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/view/IWindowManager$Stub;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/DeskClock$TabsAdapter;-notifySelectedPage-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothDeviceFilter$AudioFilter;-matches-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/editor/TextFieldsEditorView;-onLabelRebuilt-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/power/PowerManagerService;-access$1300-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/ManageConferenceUtils;-separateConferenceConnection-(I Lcom/android/internal/telephony/Connection;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewManager;-verifyUnlock-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/server/ConnectivityService;-getMobileDataEnabled-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/DevelopmentSettings$ConfirmEnforceFragment$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Landroid/location/Criteria; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/transaction/SmsReceiverService;-handleSmsSent-(Landroid/content/Intent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/BluetoothDevice;-createBond-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/CursorEntityIterator;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-operationComplete-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/settings/bluetooth/LocalBluetoothProfile;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/TrimVideo;-onPlayPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-startMsgListQuery-(I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/ContactsActivity;-onAttachFragment-(Landroid/app/Fragment;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-access$600-(Lcom/android/internal/policy/impl/keyguard/KeyguardHostView; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onClick-(Landroid/view/View;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/systemui/statusbar/policy/Clock;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/AudioService;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ConversationList;-openThread-(J)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/StatusBarManagerService;-enforceExpandStatusBar-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/gsm/SimSmsInterfaceManager$CellBroadcastRangeManager;-finishUpdate-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareClassic;-restore-(I)V' : ['android.permission.NFC'],
	'Lcom/android/contacts/vcard/NfcImportVCardActivity$ImportTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-<init>-(Lcom/android/gallery3d/data/Path; Ljava/util/Comparator; [Lcom/android/gallery3d/data/MediaSet; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/ILocationManager$Stub$Proxy;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/DeviceStorageMonitorService;-sendFullNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/GroupMembershipDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/effect/SizeChangeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/webkit/WebViewClassic$InvokeListBox$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/IAudioService$Stub;-setWiredDeviceConnectionState-(I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/ReadRecTransaction;-notifyObservers-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-<init>-(Landroid/content/Context; Lcom/android/calendar/agenda/AgendaListView; Z)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/mms/transaction/TransactionService;-beginMmsConnectivity-()I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupWaitingOrBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/inputmethodservice/KeyboardView;-onModifiedTouchEvent-(Landroid/view/MotionEvent; Z)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/app/ResolverActivity$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/ex/variablespeed/VariableSpeed;-stopAndStartPlayingAgain-(Lcom/android/ex/variablespeed/MediaPlayerDataSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkStatsService;-updateIfaces-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-access$2700-(Lcom/android/mms/ui/ComposeMessageActivity; Landroid/view/ContextMenu; Lcom/android/mms/ui/ComposeMessageActivity$MsgListMenuClickListener; Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/quicksearchbox/util/JavaNetHttpHelper;-get-(Ljava/lang/String; Ljava/util/Map;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/server/am/ActivityManagerService;-removeProcessLocked-(Lcom/android/server/am/ProcessRecord; Z Z Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/quickcontact/QuickContactActivity$5;-onLoadFinished-(Landroid/content/Loader; Lcom/android/contacts/model/Contact;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phone/DTMFTwelveKeyDialer;-closeDialer-(Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/sip/SipPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-prev-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-performAccessibilityAction-(I Landroid/os/Bundle;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-removeBond-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/os/BatteryStatsImpl;-setBtHeadset-(Landroid/bluetooth/BluetoothHeadset;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-getAwakeTimeBattery-()J' : ['android.permission.BATTERY_STATS'],
	'Landroid/content/IContentService$Stub;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/contacts/group/GroupDetailFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/UsimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService$Stub$Proxy;-dispatchMediaKeyEventUnderWakelock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onStateResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/phone/PhoneInterfaceManager;-getCellLocation-()Landroid/os/Bundle;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/ViewRootImpl;-doConsumeBatchedInput-(J)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WifiService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-filterVoiceInputKeyEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar;-topAppWindowChanged-(Z)V' : ['android.permission.VIBRATE'],
	'Landroid/accounts/IAccountManager$Stub;-addAcount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/browser/provider/BrowserProvider2;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/location/ILocationManager$Stub;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText;-onDetachedFromWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/Settings$System;-getUriFor-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityModel;-getSecurityMode-()Lcom/android/internal/policy/impl/keyguard/KeyguardSecurityModel$SecurityMode;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/SlideView;-stopAudio-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onRadioOffOrNotAvailable-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/pan/PanService;-onConnectStateChanged-([B I I I I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/development/LogTextBox;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/view/IInputMethodClient$Stub;-onBindMethod-(Lcom/android/internal/view/InputBindResult;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-dispatchPdus-([[B)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/Contact$ContactsCache;-getContactInfoForPhoneNumber-(Ljava/lang/String;)Lcom/android/mms/data/Contact;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/inputmethod/latin/LatinIME;-revertCommit-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/Settings$SoundSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/adapter/CalendarSyncAdapter;-isSyncable-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-setFilter-(Lcom/android/contacts/list/ContactListFilter; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/OpenDownloadReceiver$1;-run-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/content/ContentResolver;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/RIL;-setCLIR-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-onStart-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/SnailAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/bluetooth/DockService;-access$700-(Lcom/android/settings/bluetooth/DockService; Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/widget/PinnedHeaderListDemoActivity$TestPinnedHeaderListAdapter;-getView-(I Landroid/database/Cursor; I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/media/AudioService;-setWiredDeviceConnectionState-(I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmInitReceiver$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub$Proxy;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimUnlockScreen;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/DeviceListPreferenceFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/display/DisplayManagerService;-access$1200-(Lcom/android/server/display/DisplayManagerService; Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-getCurrentCalls-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/app/KeyguardManager$KeyguardLock;-reenableKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-findFirstDataId-(J Ljava/lang/String;)J' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-confirmDeleteThread-(J)V' : ['android.permission.READ_SMS'],
	'Lcom/android/browser/DownloadTouchIcon;-storeIcon-(Landroid/graphics/Bitmap;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/music/MediaPlaybackService;-setRepeatMode-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover;-startBonding-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/CropImage$LoadBitmapDataTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Landroid/provider/ContactsContract$SyncState;-getWithUri-(Landroid/content/ContentProviderClient; Landroid/accounts/Account;)Landroid/util/Pair;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter;-<init>-(Lcom/android/exchange/EasSyncService;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-startMsgListQuery-(I)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/Controller;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-runAddAccountForAuthenticator-(Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/ConnectivityManager;-getTetherableWifiRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/vcard/VCardImportExportListener;-onImportFinished-(Lcom/android/contacts/vcard/ImportRequest; I Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardStatusViewManager;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/am/ActivityManagerService;-startAppProblemLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/list/ProviderStatusWatcher$LoaderTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/Controller;-createSaveState-()Landroid/os/Bundle;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/editor/AggregationSuggestionEngine;-onNameChange-(Lcom/android/contacts/model/RawContactDelta$ValuesDelta;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/IAccountManager;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/content/IContentService;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/nfc/INfcAdapterExtras$Stub$Proxy;-close-(Ljava/lang/String; Landroid/os/IBinder;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.NFC'],
	'Lcom/android/exchange/ExchangeService;-waitForConnectivity-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/model/dataitem/OrganizationDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/CallbackProxy;-onPageStarted-(Ljava/lang/String; Landroid/graphics/Bitmap;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/SnapshotTab;-<init>-(Lcom/android/browser/WebViewController; Landroid/webkit/WebView;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/content/SyncManager;-access$800-(Landroid/content/SyncManager; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/latin/AudioAndHapticFeedbackManager;-vibrate-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/Settings$StorageUseActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-removeTracks-(I I)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/PeopleActivity;-areContactWritableAccountsAvailable-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/AlbumSetPage$MyLoadingListener;-onLoadingFinished-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/data/Contact$ContactsCache$1;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/IConnectivityManager;-getTetheringErroredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/latin/Dictionary;-isValidWord-(Ljava/lang/CharSequence;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-ndefRead-(I)Landroid/nfc/NdefMessage;' : ['android.permission.NFC'],
	'Lcom/android/settings/bluetooth/HidProfile;-getConnectionStatus-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/editor/StructuredNameEditorView;-rebuildFullName-(Lcom/android/contacts/model/RawContactDelta$ValuesDelta;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/display/DisplayManagerService;-access$1100-(Lcom/android/server/display/DisplayManagerService; Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/DeviceInfoSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/IWifiManager;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/sip/SipService;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/policy/impl/keyguard/CameraWidgetFrame$7;-onRefreshCarrierInfo-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/ApplicationThreadNative;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/os/IPowerManager;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/CallContactActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/calendar/CalendarViewAdapter;-<init>-(Landroid/content/Context; I Z)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-handleWakeWhenReady-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/input/InputManagerService;-notifySwitch-(J I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wm/WindowAnimator;-testTokenMayBeDrawnLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/display/DisplayAdapter$Listener;-onTraversalRequested-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/FillLightEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/power/PowerManagerService$DisplayBlankerImpl;-unblankAllDisplays-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/musicfx/ActivityMusic;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-keyguardDoneDrawing-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$ReloadTask;-findIndexOfTarget-(Lcom/android/gallery3d/app/PhotoDataAdapter$UpdateInfo;)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/support/v4/net/ConnectivityManagerCompatGingerbread;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/PhotoPage;-onUpPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/policy/impl/GlobalActions$Action;-onPress-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/webkit/WebViewClassic;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/telephony/SmsProvider;-getAllMessagesFromIcc-()Landroid/database/Cursor;' : ['android.permission.RECEIVE_SMS'],
	'Landroid/webkit/WebView;-computeScroll-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/AccessibilitySettings$SettingsPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/RingerVolumePreference;-onDialogClosed-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/CallerInfoCache;-access$400-(Lcom/android/phone/CallerInfoCache;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/accounts/AccountSyncSettings;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/UiModeManagerService;-updateDockState-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/GrainEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/ConfirmAddDetailActivity;-access$300-(Lcom/android/contacts/activities/ConfirmAddDetailActivity; Lcom/android/contacts/model/RawContactDeltaList;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/AccountUnlockScreen$2;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame$1;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/CrashRecoveryHandler;-startRecovery-(Landroid/content/Intent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/SlideView;-startAudio-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnection;-onConnect-(Lcom/android/internal/telephony/DataConnection$ConnectionParams;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/ContactSaveService;-setSuperPrimary-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-addAttachment-(Ljava/lang/String; Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/latin/LatinIME$UIHandler;-onStartInputView-(Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/btservice/AdapterState$OnState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ConnectivityService;-handleDisconnect-(Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/providers/downloads/DownloadProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$100-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/SlideshowPage;-access$500-(Lcom/android/gallery3d/data/MediaSet; I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-access$500-(Lcom/android/deskclock/stopwatch/StopwatchFragment;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/LatinIME;-pickSuggestionManually-(I Ljava/lang/CharSequence;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/camera/CameraActivity;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-cleanUpApplicationRecordLocked-(Lcom/android/server/am/ProcessRecord; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView$4;-onUserSwitched-(I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/UI;-showWeb-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangup-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewProvider;-init-(Ljava/util/Map; Z)V' : ['android.permission.INTERNET'],
	'Landroid/net/wifi/WifiManager;-reassociate-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/settings/DataUsageSummary;-setPolicyLimitBytes-(J)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/CropView$HighlightRectangle;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneBase;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/RequestPermissionActivity$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/IAccountManager$Stub;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitorCallback;-onDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccRecords;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-onDataConnectionDetached-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserWebView;-clearView-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readEmailFileAndWait-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/AttachPhotoActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-addPreferencesForProfiles-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/PhoneUtils;-hangupRingingCall-(Lcom/android/internal/telephony/Call;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/phone/PhoneGlobals;-handleOtaspEvent-(Landroid/os/Message;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub;-startUsingNetworkFeature-(I Ljava/lang/String; Landroid/os/IBinder;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/os/BatteryStatsImpl;-getBluetoothPingCount-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter;-setSyncKey-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/BluetoothHealth;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/HTML5Audio;-teardown-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/DetailsHelper;-show-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/ActivityBase;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/mms/transaction/RetryScheduler;-scheduleRetry-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/editor/SelectAccountDialogFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/UriImage;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewClassic;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/SelectionManager;-selectAll-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$1;-onClockVisibilityChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/DayView;-access$4600-(Lcom/android/calendar/DayView; Landroid/view/MotionEvent;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mms/ui/AudioAttachmentView;-cleanupMediaPlayer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionActivity;-createPhonebookDisplayText-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/Phone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView$1;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/VibratorService;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/RingerVolumePreference;-onSampleStarting-(Landroid/preference/VolumePreference$SeekBarVolumizer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-setRadios-(Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/IWallpaperManager$Stub;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-snapToPage-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-updateConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback;-takeEmergencyCallAction-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/data/MtpDevice;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/SupplicantStateTracker$ScanState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-prepareDragSurface-(Landroid/view/IWindow; Landroid/view/SurfaceSession; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-pageBeginMoving-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$ApplicationSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebViewClassic;-replaceTextfieldText-(I I Ljava/lang/String; I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/ADNList;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/media/AudioService;-startVoiceBasedInteractions-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/speech/SpeechRecognizer;-handleStopMessage-()V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/contacts/ContactSaveService;-clearPrimary-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/power/PowerManagerService;-napInternal-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasAccountService;-acknowledgeProvision-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-handleDownloadXtraData-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/HttpUtils;-httpConnection-(Landroid/content/Context; J Ljava/lang/String; [B I Z Ljava/lang/String; I)[B' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/XLargeUi;-showWeb-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/vcard/VCardParserImpl_V40;-parseOneVCard-()Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$1500-(Lcom/android/server/net/NetworkPolicyManagerService;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-completeChallengeScroll-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$CaptivePortalCheckState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/TabBar$TabView;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/OpenDownloadReceiver;-access$000-(Lcom/android/browser/OpenDownloadReceiver; Landroid/content/Context; J)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetooth;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ISms$Stub;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/inputmethod/keyboard/KeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$ScreenNailJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Lcom/android/gallery3d/ui/ScreenNail;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.SET_WALLPAPER', 'android.permission.INTERNET', 'android.permission.NFC'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Lcom/android/internal/telephony/PhoneProxy;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator$1;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcAdapterExtras$Stub;-setCardEmulationRoute-(Ljava/lang/String; I)V' : ['android.permission.NFC'],
	'Landroid/net/wifi/WifiStateMachine;-access$18200-(Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$WifiSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/systemui/statusbar/policy/NetworkController;-updateConnectivity-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$WifiSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityManagerService;-startProcessLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/LocationManagerService;-checkMockPermissionsSafe-()V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/systemui/statusbar/phone/NavigationBarView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/net/wifi/IWifiManager$Stub;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebView;-resumeTimers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/SnapshotTab;-putInBackground-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/DateTimeSettings;-initUI-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/content/ContentService;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/systemui/recent/RecentsPanelView;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/inputmethod/latin/LatinIME;-onTextInput-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/preferences/AccessibilityPreferencesFragment;-onDestroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/handover/HandoverManager;-createHandoverRequestMessage-()Landroid/nfc/NdefMessage;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/connectivity/Tethering;-access$900-(Lcom/android/server/connectivity/Tethering;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-finish-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/data/Conversation;-getOrCreateThreadId-(Landroid/content/Context; Lcom/android/mms/data/ContactList;)J' : ['android.permission.READ_SMS'],
	'Lcom/android/settings/bluetooth/HidProfile;-getSummaryResourceForDevice-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hid/HidService;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/WallpaperManager;-setResource-(I)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/settings/deviceinfo/Status;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/Event;-instancesQuery-(Landroid/content/ContentResolver; [Ljava/lang/String; I I Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/launcher2/UserInitializeReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/os/IVibratorService;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/phone/PhoneUtils;-restoreSpeakerMode-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/media/AudioService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/RotateEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/DhcpStateMachine$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-access$400-(Lcom/android/gallery3d/ui/ActionModeHandler; Lcom/android/gallery3d/util/ThreadPool$JobContext;)Landroid/content/Intent;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/development/Connectivity;-access$1300-(Lcom/android/development/Connectivity;)V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/calendar/CalendarController;-launchDeleteEventAndFinish-(Landroid/app/Activity; J J J I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/ThrottleService$DataRecorder;-checkForSubscriberId-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/MediaPlaybackService;-setNextTrack-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/DeskClock$TabsAdapter;-onPageSelected-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/IInputMethodCallback$Stub$Proxy;-finishedEvent-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-autoConnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ThrottleService$MyHandler;-onPolicyChanged-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/alerts/AlertService;-processMessage-(Landroid/os/Message;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/alerts/AlertReceiver;-getAttendeesCursor-(Landroid/content/Context; J)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/deskclock/AlarmKlaxon;-onDestroy-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/Files;-readLines-(Ljava/io/File; Ljava/nio/charset/Charset; Lcom/google/common/io/LineProcessor;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/GalleryActionBar$ClusterRunner;-doCluster-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/app/AlbumPage$MyDetailsSource;-setIndex-()I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/wifi/WifiSettings$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/AsyncPlayer;-startSound-(Landroid/media/AsyncPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/SettingsPreferenceFragment;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/Settings$VpnSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ComposeMessageActivity$18;-run-()V' : ['android.permission.READ_SMS'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phone/PhoneUtils;-hangup-(Lcom/android/internal/telephony/Connection;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$DriverUnloadingState$1;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/settings/wifi/WifiSettings;-resumeWifiScan-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/phone/CallCard;-updateCallStateWidgets-(Lcom/android/internal/telephony/Call;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/ControllerOverlay$Listener;-onReplay-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService;-access$1700-(Lcom/android/server/sip/SipService; Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/Connection;-separate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/phone/InCallScreen;-access$400-(Lcom/android/phone/InCallScreen; Landroid/os/AsyncResult;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/ConversationList$8;-run-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/EasSyncService;-acknowledgeProvisionImpl-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String; Z)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardStatusView$1;-onUserSwitched-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-finish-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/provider/BrowserProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/RIL;-switchToRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/quicksearchbox/util/JavaNetHttpHelper;-post-(Ljava/lang/String; Ljava/util/Map; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/provider/Telephony$Sms$Draft;-addMessage-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/media/effect/effects/SepiaEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView$2;-onCodeInput-(I I I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/Tab$3;-onReceivedTitle-(Landroid/webkit/WebView; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-editMmsMessageItem-(Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiApConfigStore$ActiveState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothHeadset;-phoneStateChanged-(I I I Ljava/lang/String; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackPresenter$3;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBarPolicy;-<init>-(Landroid/content/Context;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebView;-postUrl-(Ljava/lang/String; [B)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/downloads/DownloadReceiver;-handleNotificationBroadcast-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Lcom/android/server/am/ActivityManagerService;-systemReady-(Ljava/lang/Runnable;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService;-closeExternalStorageFiles-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/svc/Svc$Command;-run-([Ljava/lang/String;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/LocationManagerService;-decrementPendingBroadcasts-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-onContactUriQueryFinished-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/research/ResearchLogger;-latinIME_onFinishInputInternal-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/bluetooth/BluetoothNameDialogFragment$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/ITelephony;-disableDataConnectivity-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator$1;-onRefreshCarrierInfo-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/PhotoController;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/LatinIME;-onStartBatchInput-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/InCallTouchUi;-updateInCallControls-(Lcom/android/internal/telephony/CallManager;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/NetworkStateHandler;-<init>-(Landroid/app/Activity; Lcom/android/browser/Controller;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager;-verifyUnlock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/MobileNetworkSettings$MyHandler;-handleSetPreferredNetworkTypeResponse-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/editor/EventFieldEditorView;-onLabelRebuilt-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/Tab;-onNewPicture-(Landroid/webkit/WebView; Landroid/graphics/Picture;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/systemui/media/NotificationPlayer;-access$500-(Lcom/android/systemui/media/NotificationPlayer; Lcom/android/systemui/media/NotificationPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Bookmarks$1;-updateImages-(Landroid/content/ContentResolver; Ljava/lang/String; Landroid/content/ContentValues;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitorCallback;-onKeyguardVisibilityChanged-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/inputmethod/InputMethodManager$2;-finishedEvent-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/CameraActivity;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/music/IMediaPlaybackService;-open-([J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-animateCollapsePanels-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar;-addNotificationViews-(Landroid/os/IBinder; Lcom/android/internal/statusbar/StatusBarNotification;)Lcom/android/systemui/statusbar/StatusBarIconView;' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/BroadcastQueue;-processNextBroadcast-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/alerts/AlertReceiver;-getEventCursor-(Landroid/content/Context; J)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/latin/LatinIME;-onStartInputViewInternal-(Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/providers/contacts/ContactsProvider2;-getDefaultAccount-()Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/DnsPinger;-getDnsList-()Ljava/util/List;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/WifiService;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/server/TwilightService$TwilightListenerRecord;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/RetryScheduler;-getRetrieveStatus-(J)I' : ['android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/SkbContainer;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/event/AttendeesView;-addOneAttendee-(Lcom/android/calendar/CalendarEventModel$Attendee;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/contacts/list/ShortcutIntentBuilder$ContactLoadingAsyncTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/WebViewClassic;-scrollEditText-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccCardProxy;-updateExternalState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/transaction/MessagingNotification;-playInConversationNotificationSound-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/calllog/CallLogAdapter;-bindGroupView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor; I Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-setCountryListener-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/ShutdownThread;-rebootSafeMode-(Landroid/content/Context; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-onSaveCompleted-(Z I Z Landroid/net/Uri;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine;-<init>-(Landroid/content/Context;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/contacts/model/dataitem/ImDataItem;-buildDataStringForDisplay-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/IccProvider;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processLocalVrEvent-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$5100-(Lcom/android/mms/ui/ComposeMessageActivity;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetFrame$1;-onRefreshCarrierInfo-(Ljava/lang/CharSequence; Ljava/lang/CharSequence;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/EasSyncService;-messageMoveRequest-(Lcom/android/exchange/MessageMoveRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/Resources;-readLines-(Ljava/net/URL; Ljava/nio/charset/Charset; Lcom/google/common/io/LineProcessor;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/server/am/ActiveServices;-serviceTimeout-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-onResume-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/DownUpDetector$DownUpListener;-onUp-(Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/AccessibilityInjector;-addCallbackApis-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-onLongTap-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-updateContactMethods-(J Landroid/content/ContentValues;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/InCallTouchUi;-showAudioModePopup-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/text/ToUpperCase;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/RIL;-explicitCallTransfer-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/activities/AttachPhotoActivity$1;-onContactLoaded-(Lcom/android/contacts/model/Contact;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/CameraSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardScreenCallback;-forgotPattern-(Z)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/CropImage;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/server/wm/InputMonitor;-interceptKeyBeforeDispatching-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)J' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcAdapterExtras;-setCardEmulationRoute-(Ljava/lang/String; I)V' : ['android.permission.NFC'],
	'Landroid/test/mock/MockContentResolver;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/providers/settings/SettingsBackupAgent;-enableWifi-(Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/telephony/SmsManager;-sendDataMessage-(Ljava/lang/String; Ljava/lang/String; S [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/settings/DataUsageSummary;-updateAppDetail-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService$BluetoothShareContentObserver;-onChange-(Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/IConnectivityManager;-getMobileDataEnabled-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/DetailsHelper;-reloadDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/Settings$KeyboardLayoutPickerActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/adapter/CalendarSyncAdapter$EasCalendarSyncParser;-addParser-(Lcom/android/exchange/adapter/CalendarSyncAdapter$CalendarOperations;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/webkit/AccessibilityInjector;-toggleAccessibilityFeedback-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/TransactionService$ServiceHandler;-processTransaction-(Lcom/android/mms/transaction/Transaction;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/webkit/WebViewInputDispatcher;-postPointerEvent-(Landroid/view/MotionEvent; I I F)Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/os/IPowerManager$Stub$Proxy;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasAccountService;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR'],
	'Landroid/net/wifi/IWifiManager;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifySeMifareAccess-([B)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/deskclock/TimerRingService;-onDestroy-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/PhoneUtils;-startNewCall-(Lcom/android/internal/telephony/CallManager;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/content/ContentResolver;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewBase;-handleMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/Tag;-rediscover-()Landroid/nfc/Tag;' : ['android.permission.NFC'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$13;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/hid/HidService;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifyServiceState-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-onMeasure-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothPbap;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/NetworkStatsService;-systemReady-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/nfc/NfcService$NfcAdapterExtrasService;-getDriverName-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.NFC'],
	'Lcom/android/development/Connectivity$3;-onClick-(Landroid/view/View;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterfw/core/FilterFunction;-tearDown-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebIconDatabaseClassic$EventHandler;-bulkRequestIcons-(Landroid/os/Message;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/accounts/IAccountManager$Stub;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/exchange/EasSyncService;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity;)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/TabControl;-saveState-(Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/editor/StructuredNameEditorView;-switchFromStructuredNameToFullName-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/AllInOneActivity;-configureActionBar-(I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/adapter/AbstractSyncAdapter;-setSyncKey-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/bluetooth/HeadsetProfile;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/power/ScreenOnBlocker;-release-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/dataitem/RelationDataItem;-hasKindTypeColumn-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-editMmsMessageItem-(Lcom/android/mms/ui/MessageItem;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$10;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/UiccCardApplication;-setIccLockEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-onMaxPendingMessagesReached-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/media/effect/effects/CropEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/voicemail/VoicemailPlaybackFragment;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/SmsReceiverService;-access$100-(Lcom/android/mms/transaction/SmsReceiverService; Landroid/content/Intent; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/sip/SipService;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/gallery3d/data/SecureAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-onKeyPress-(I I Lcom/android/inputmethod/pinyin/SkbContainer$LongPressTimer; Z)Lcom/android/inputmethod/pinyin/SoftKey;' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/event/EditEventView$4;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/musicfx/ControlPanelEffect;-setEnabledAll-(Landroid/content/Context; Ljava/lang/String; I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/DataItem;-getAccountType-()Lcom/android/contacts/model/account/AccountType;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothPan;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-reconnectAfterFail-(Lcom/android/internal/telephony/DataConnection$FailCause; Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/list/DefaultContactListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/AlarmManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCurrentCalls-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MusicPicker;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/systemui/statusbar/BaseStatusBar;-createAndAddWindows-()V' : ['android.permission.VIBRATE', 'android.permission.DISABLE_KEYGUARD', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH'],
	'Landroid/preference/VolumePreference;-onKey-(Landroid/view/View; I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Ljava/net/MulticastSocket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/MmiCode;-cancel-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/StructuredPostalDataItem;-getKindString-()Ljava/lang/String;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/calendar/CalendarReceiver;-access$000-(Lcom/android/providers/calendar/CalendarReceiver; Landroid/content/ContentResolver;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/PhotoPage;-updateCurrentPhoto-(Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/SyncManager;-sendSyncAlarmMessage-()V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/TimeClustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitorCallback;-onSimStateChanged-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/Notifier$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/IAccountManager;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-notifyApnIdUpToCurrent-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/CameraSwitcher$1;-onClick-(Landroid/view/View;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/location/ILocationManager;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/commands/bmgr/Bmgr;-main-([Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/power/PowerManagerService;-handleWakeLockDeath-(Lcom/android/server/power/PowerManagerService$WakeLock;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/wifi/WifiApConfigStore$InactiveState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ServiceWatcher$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-setDialogListenerApp-(Landroid/os/Messenger; Ljava/lang/String; Z)Z' : ['android.permission.GET_TASKS'],
	'Lcom/android/settings/tts/TextToSpeechSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/NullFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/CallCard;-displayMainCallStatus-(Lcom/android/internal/telephony/CallManager; Lcom/android/internal/telephony/Call;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mms/ui/SlideView;-setVideo-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage;-onDeleteImage-(Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/event/AttendeesView;-addAttendees-(Ljava/util/HashMap;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/mms/transaction/NotificationPlayer;-startSound-(Lcom/android/mms/transaction/NotificationPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/EventInfoFragment;-onDestroyView-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mms/transaction/Transaction;-sendPdu-(J [B Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/view/inputmethod/BaseInputConnection;-finishComposingText-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/editor/TextFieldsEditorView;-onTypeSelectionChange-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/contacts/list/DefaultContactBrowseListFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar$9;-run-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onVoiceCallEnded-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/contacts/ContactsProvider2;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard/CarrierText;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/ContactSelectionActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/voicedialer/VoiceDialerActivity;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-access$1300-(Lcom/android/server/ConnectivityService; Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$StorageUseActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/wfd/WifiDisplaySettings;-onResume-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/research/UploaderService;-hasWifiConnection-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/HTML5VideoInline;-pauseAndDispatch-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onSeEmvCardRemoval-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/browser/Tab;-goBack-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/event/EditEventFragment$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/webkit/WebView;-clearView-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue;-releaseAllPointersExcept-(Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue$Element; J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/settings/bluetooth/DeviceProfilesSettings;-onProfileClicked-(Lcom/android/settings/bluetooth/LocalBluetoothProfile; Landroid/preference/CheckBoxPreference;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/location/LocationProviderInterface;-setRequest-(Lcom/android/internal/location/ProviderRequest; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/inputmethod/InputMethodManager;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/transaction/SmsMessageSender;-<init>-(Landroid/content/Context; [Ljava/lang/String; Ljava/lang/String; J)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/settings/bluetooth/PanProfile;-getSummaryResourceForDevice-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/data/Conversation;-verifySingleRecipient-(Landroid/content/Context; J Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_SMS'],
	'Landroid/view/IApplicationToken;-keyDispatchingTimedOut-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/IMediaPlaybackService;-setRepeatMode-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameBranch;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/wifi/p2p/WifiP2pSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDevice;-<init>-(Landroid/content/Context; Lcom/android/settings/bluetooth/LocalBluetoothAdapter; Lcom/android/settings/bluetooth/LocalBluetoothProfileManager; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/provider/CalendarContract$Instances;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; J J Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/Settings$SpellCheckersSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/CommandsInterface;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/P2pLinkManager;-doSnepProtocol-(Lcom/android/nfc/handover/HandoverManager; Landroid/nfc/NdefMessage; [Landroid/net/Uri; I I)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/ex/chips/RecipientEditTextView;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/provider/BrowserProvider2;-getAccountNameAndType-(J)[Ljava/lang/String;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/systemui/statusbar/phone/CarrierLabel;-onEditorAction-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/NfcService;-onSeApduReceived-([B)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/filterpacks/imageproc/CropFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/data/WorkingMessage;-readStateFromBundle-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/EasOutboxService;-validateAccount-(Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/systemui/statusbar/phone/QuickSettings$2;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/webkit/GeolocationService;-setEnableGps-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/IActivityManager;-shutdown-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/providers/contacts/ContactsProvider2;-call-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Landroid/os/Bundle;' : ['android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/Settings$5;-onStopTrackingTouch-(Landroid/widget/SeekBar;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-stopDtmf-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/TetherSettings;-updateState-()V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/power/PowerManagerService;-access$1900-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/calendar/CalendarProvider2;-getOriginalId-(Ljava/lang/String; Ljava/lang/String;)J' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView;-show-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/selectcalendars/SelectCalendarsSyncFragment;-onResume-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/settings/fuelgauge/PowerUsageSummary;-onResume-()V' : ['android.permission.BATTERY_STATS', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/ui/CropView$HighlightRectangle;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/contacts/model/dataitem/NoteDataItem;-getDataKind-()Lcom/android/contacts/model/dataitem/DataKind;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneBase;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getBondState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/media/MediaScannerService;-access$300-(Lcom/android/providers/media/MediaScannerService; [Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/ContactsContract$Directory;-notifyDirectoryChange-(Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/settings/DevelopmentSettings;-setPermissionEnforced-(Landroid/content/Context; Ljava/lang/String; Z)V' : ['android.permission.READ_EXTERNAL_STORAGE'],
	'Landroid/bluetooth/BluetoothDevice;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter;-parse-(Ljava/io/InputStream;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/HTML5VideoFullScreen;-prepareDataCommon-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewManager;-dismiss-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/location/ILocationManager$Stub;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AlbumPage;-initializeData-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmDataConnectionTracker;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS'],
	'Landroid/content/IContentService$Stub$Proxy;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/phone/PhoneUtils$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$PrivacySettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-processNextPostDialChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$InputMethodAndLanguageSettingsActivity;-onResume-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/development/Connectivity;-onStartScreenCycle-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetooth;-createSocketChannel-(I Ljava/lang/String; Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/HTML5VideoFullScreen;-prepareDataAndDisplayMode-(Landroid/webkit/HTML5VideoViewProxy;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/launcher2/Launcher;-setupViews-()V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/gallery3d/app/TimeBar$Listener;-onScrubbingStart-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-addGpsStatusListener-(Landroid/location/GpsStatus$Listener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-dispose-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/CatService;-sendMenuSelection-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppRfcommListener$1;-run-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/SupplicantStateTracker$HandshakeState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetooth;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/content/pm/IPackageManager;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimPukUnlockScreen;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/exchange/adapter/ContactsSyncAdapter;-cleanup-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/CalendarViewAdapter;-refresh-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-<init>-(Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/cdma/CdmaCallTracker; Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/Utils;-getTimeZone-(Landroid/content/Context; Ljava/lang/Runnable;)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;-onScanningStateChanged-(Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/MccTable;-updateMccMncConfiguration-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionActivity;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiManager;-<init>-(Landroid/content/Context; Landroid/net/wifi/IWifiManager;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/nfc/handover/HandoverService;-doOutgoingTransfer-(Landroid/os/Message;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/SlidingChallengeLayout;-sendInitialListenerUpdates-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/sip/SipAudioCall;-close-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebView;-clearCache-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/commands/svc/WifiCommand;-run-([Ljava/lang/String;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter;-getEstimate-(Ljava/lang/String;)I' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$DreamSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/EventInfoFragment$5;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE', 'android.permission.WRITE_SETTINGS', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/settings/Settings$AndroidBeamSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/TelephonyRegistry;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/WebViewTimersControl;-onBrowserActivityResume-(Landroid/webkit/WebView;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/emailcommon/service/IEmailService;-startSync-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/detail/ContactDetailFragment$QuickFix;-isApplicable-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/UiModeManagerService;-updateTwilight-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-onPageBeginMoving-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewProvider;-clearCache-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardHostView$1;-onPhoneStateChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/view/ViewRootImpl;-deliverInputEvent-(Landroid/view/ViewRootImpl$QueuedInputEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/BatteryStatsService;-getAwakeTimeBattery-()J' : ['android.permission.BATTERY_STATS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-handleForwardedMessage-()Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/BluetoothHeadset;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/content/ContextWrapper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothInputDevice;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-renderChild-(Lcom/android/gallery3d/ui/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothAdapter;-disable-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/hdp/HealthService;-updateAndSendIntent-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneBase;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipManager;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/filterpacks/base/GLTextureTarget;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManagerService;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/phone/PhoneInterfaceManager;-disableDataConnectivity-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-getStream-(Lcom/android/dreams/phototable/PhotoSource$ImageData; I)Ljava/io/InputStream;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager$NameChangedHandler;-onReceive-(Landroid/content/Context; Landroid/content/Intent; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$600-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/view/IApplicationToken$Stub$Proxy;-windowsVisible-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onSetInternalDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-onScreenTurnedOff-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/MountService$3;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$DefaultState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/ContactPickerFragment;-prepareEmptyView-()V' : ['android.permission.READ_SYNC_STATS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-handleWakeWhenReady-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/settings/Settings$DeviceAdminSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/os/Vibrator;-cancel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/camera/PhotoModule;-access$1800-(Lcom/android/camera/PhotoModule;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewClassic;-onPinchToZoomAnimationEnd-(Landroid/view/ScaleGestureDetector;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/SelectionManager;-expandMediaSet-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/tts/TextToSpeechSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/MtpDeviceSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebViewProvider;-loadUrl-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-isSecure-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothPbap$Stub$Proxy;-disconnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-pokeWakelock-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/camera/CameraActivity;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/browser/DownloadTouchIcon;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/IBluetoothHeadsetPhone;-answerCall-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.BLUETOOTH', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/media/IAudioService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-switchTo-(Lcom/android/internal/policy/impl/RecentApplicationsDialog$RecentTag;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-handleDevicePolicyManagerStateChanged-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetPager;-snapToPage-(I I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/DockService$4;-onDismiss-(Landroid/content/DialogInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ManageSimMessages;-refreshMessageList-()V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/systemui/statusbar/tablet/SettingsView;-onClickSettings-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onSetInternalDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/data/WorkingMessage$2;-run-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/ui/PhotoView;-slideToPrevPicture-()Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/CdmaSystemSelectListPreference$MyHandler;-resetCdmaRoamingModeToDefault-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareClassic;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/providers/applications/ApplicationsProvider$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Utils;-getDefaultIpAddresses-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/SimUnlockScreen;-checkPin-()V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardUpdateMonitor;-handleBootCompleted-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/PagedView;-snapToPageWithVelocity-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$WifiDisplaySettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-getPhonebookResult-(Ljava/lang/String; Z)Lcom/android/bluetooth/hfp/AtPhonebook$PhonebookResult;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/settings/SettingsLicenseActivity;-showPageOfText-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$5000-(Lcom/android/mms/ui/ComposeMessageActivity;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/DayView;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/list/ShortcutIntentBuilder$LoadingAsyncTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/providers/telephony/SmsProvider;-getSingleMessageFromIcc-(Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-onLoadComplete-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-startVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onUpEventInternal-(J)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/media/MediaPlayer;-setWakeMode-(Landroid/content/Context; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/ObjectSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/media/MtpService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/WifiService;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-registerAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/IBluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/data/WorkingMessage;-updateSendStats-(Lcom/android/mms/data/Conversation;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/IWallpaperManager;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/internal/os/RuntimeInit;-wtf-(Ljava/lang/String; Ljava/lang/Throwable;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-stopObexServerSession-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/ContentResolver;-getCurrentSync-()Landroid/content/SyncInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/bluetooth/IBluetoothPbap;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$CryptKeeperSettingsActivity;-insertAccountsHeaders-(Ljava/util/List; I)I' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/media/AsyncPlayer;-access$200-(Landroid/media/AsyncPlayer; Landroid/media/AsyncPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/google/GoogleSource;-queryInternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/RIL;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/ui/ConversationList;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/media/AudioService$AudioServiceBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/vcard/VCardParser;-parse-(Ljava/io/InputStream; Lcom/android/vcard/VCardInterpreter;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/WifiService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/activities/ContactEditorActivity$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/BrowserWebView;-setNetworkAvailable-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/BackDropperEffect;-apply-(I I I I)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/settings/widget/SettingsAppWidgetProvider$GpsStateTracker;-setCurrentState-(Landroid/content/Context; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/settings/bluetooth/DockService;-onCreate-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/SnapshotTab$LoadData;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/net/NetworkStatsService$5;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/ui/ConversationList$DeleteThreadListener$1;-run-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/UiccCardApplication;-changeIccLockPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-snapToPage-(I I I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcAdapter;-getNfcAdapterExtrasInterface-(Ljava/lang/String;)Landroid/nfc/INfcAdapterExtras;' : ['android.permission.NFC'],
	'Lcom/android/contacts/util/NotifyingAsyncQueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/location/provider/LocationProviderBase$Service;-disable-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/AccessibilitySettings;-handleToggleLargeTextPreferenceClick-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/AddBookmarkPage;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/vcard/VCardParser_V40;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/SlideshowPresenter;-presentVideo-(Lcom/android/mms/ui/SlideViewInterface; Lcom/android/mms/model/VideoModel; Lcom/android/mms/model/RegionModel; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/IconLoader;-loadIcons-([I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onRoamingOn-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/vcard/VCardParserImpl_V40;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/phone/DataUsageListener;-resume-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/wifi/WifiSettings$2;-onClick-(Landroid/view/View;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/webkit/WebView;-setBackgroundColor-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewMediator;-onWakeKeyWhenKeyguardShowingTq-(I Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/cat/CatService;-handleCmdResponse-(Lcom/android/internal/telephony/cat/CatResponseMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/StringSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/sip/SipWakeLock;-release-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-onResume-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/Controller;-createNewSnapshotTab-(J Z)Lcom/android/browser/SnapshotTab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/ITelephony;-getNeighboringCellInfo-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth$Stub;-getRemoteName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/location/LocationProviderInterface;-disable-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/ConnectivityService;-getAllNetworkState-()[Landroid/net/NetworkState;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/SmsUsageMonitor;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-access$700-(Lcom/android/mms/ui/ComposeMessageActivity; Lcom/android/mms/ui/MessageItem;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/widget/SlidingTab;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.VIBRATE', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardViewProperties;-createKeyguardView-(Landroid/content/Context; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewCallback; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardUpdateMonitor; Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardWindowController;)Lcom/android/internal/policy/impl/keyguard_obsolete/KeyguardViewBase;' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/vcard/VCardEntry$ImData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/usb/UsbConfirmActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/IConnectivityManager;-getTetheredIfacePairs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getCallForwardingOption-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/Recorder;-stopPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DataConnectionTracker;-onTrySetupData-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-setNextDataSource-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-setPin-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/ExchangeService;-shutdown-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$AutoRegistrationProcess;-onRegistering-(Landroid/net/sip/ISipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewFragment;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/providers/contacts/GlobalSearchSupport;-handleSearchShortcutRefresh-(Landroid/database/sqlite/SQLiteDatabase; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/location/fused/FusedLocationService;-onBind-(Landroid/content/Intent;)Landroid/os/IBinder;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/ITelephony$Stub;-endCall-()Z' : ['android.permission.CALL_PHONE'],
	'Lcom/android/inputmethod/latin/LatinIME;-restartSuggestionsOnWordBeforeCursorIfAtEndOfWord-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/quicksearchbox/util/HttpHelper;-post-(Lcom/android/quicksearchbox/util/HttpHelper$PostRequest;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/mms/ui/ComposeMessageActivity;-addAttachment-(Ljava/lang/String; Landroid/net/Uri; Z)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/media/AudioService;-dispatchMediaKeyEventUnderWakelock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/EventInfoFragment;-onResume-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/RuimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage$8;-onConfirmDialogDismissed-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$WirelessSettingsActivity;-onAccountsUpdated-([Landroid/accounts/Account;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/UsimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-handleLocationChangedLocked-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-<init>-(Landroid/content/Context; Ljava/util/Locale;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/server/NsdService$NsdStateMachine$DisabledState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/contacts/activities/PeopleActivity;-access$200-(Lcom/android/contacts/activities/PeopleActivity; I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-adjustOtherHeadsetPriorities-(Lcom/android/bluetooth/hfp/HeadsetService; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/location/GpsLocationProvider;-setEngineCapabilities-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/calllog/CallLogFragment;-refreshData-()V' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/media/effect/effects/DocumentaryEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/Context;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/NavigationBarTablet;-onAction-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/IBluetoothHeadset;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/IContentService$Stub;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-onDisconnect-(Lcom/android/internal/telephony/Connection$DisconnectCause;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab$SubWindowClient;-onUnhandledKeyEvent-(Landroid/webkit/WebView; Landroid/view/KeyEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/socialwidget/SocialWidgetProvider$1;-onLoadComplete-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/list/ContactEntryListFragment;-onStart-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumPage$MyLoadingListener;-onLoadingFinished-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-access$000-(Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-getOperator-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserWebView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I Ljava/util/Map; Z)V' : ['android.permission.INTERNET'],
	'Lcom/android/nfc/P2pEventListener;-onP2pReceiveComplete-(Z)V' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processAtCpbs-(Ljava/lang/String; I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/contacts/dialpad/DialpadFragment;-configureScreenFromIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/webkit/WebViewClassic;-adjustDefaultZoomDensity-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/WebViewClassic;-destroy-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-findFirstDataRow-(J Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/ISms;-copyMessageToIccEf-(I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-setMonthDisplayed-(Landroid/text/format/Time; Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/filterpacks/base/CallbackFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-clearAllDownloads-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardAccountView;-asyncCheckPassword-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/Tab$3$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/activities/ShowOrCreateActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/contacts/activities/PhotoSelectionActivity$PhotoHandler;-<init>-(Landroid/content/Context; Landroid/view/View; I Z Lcom/android/contacts/model/RawContactDeltaList;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-setShuffleMode-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/model/dataitem/PhotoDataItem;-getAccountTypeManager-()Lcom/android/contacts/model/AccountTypeManager;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetooth;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/EasAccountService;-searchGal-(Landroid/content/Context; J Ljava/lang/String; I)Lcom/android/exchange/provider/GalResult;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/AlertActivity;-onResume-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/ITelephony$Stub$Proxy;-getNeighboringCellInfo-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/phone/BluetoothPhoneService;-listCurrentCallsGsm-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/content/ContextWrapper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/vcard/VCardComposer;-initWithRawContactsEntityUri-(Landroid/net/Uri;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$AudioOn;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/net/LockdownVpnTracker;-init-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/ISms$Stub;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/bluetooth/hid/HidService;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/IAudioService;-startBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/mms/data/Conversation;-blockingMarkAllMmsMessagesAsSeen-(Landroid/content/Context;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-connect-(I I)I' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/RIL;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$AmsTask$Response;-onResult-(Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-finishPostLayoutPolicyLw-()I' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/editor/ContactEditorFragment;-setIntentExtras-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/DataUsageSummary;-access$2400-(Lcom/android/settings/DataUsageSummary; J)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/base/FrameFetch;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaDataConnectionTracker;-onDisconnectDone-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/AudioAndHapticFeedbackManager;-hapticAndAudioFeedback-(I Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$4900-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/VibratorService$VibrateThread;-<init>-(Lcom/android/server/VibratorService; Lcom/android/server/VibratorService$Vibration;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityRecord$Token;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-dispatchTriggerEvent-(I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/common/contacts/DataUsageStatUpdater;-update-(Ljava/util/Collection; Ljava/util/Collection; Ljava/lang/String;)Z' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/handover/BluetoothHeadsetHandover;-nextStep-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/webkit/WebView;-freeMemory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/list/LegacyPhoneNumberListAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/RetargetFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/ITelephony$Stub$Proxy;-endCall-()Z' : ['android.permission.CALL_PHONE'],
	'Lcom/android/internal/widget/LockPatternView$OnPatternListener;-onPatternCellAdded-(Ljava/util/List;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Disconnected;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/list/ContactBrowseListFragment;-notifyInvalidSelection-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/am/BatteryStatsService$1;-onServiceConnected-(I Landroid/bluetooth/BluetoothProfile;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/AllInOneActivity;-setMainPane-(Landroid/app/FragmentTransaction; I I J Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/activities/ContactDetailActivity$3$1;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/ITelephony$Stub;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/AlarmManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/development/AccountsTester$GetAndInvalidateAuthTokenCallback;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;-addDevice-(Lcom/android/settings/bluetooth/LocalBluetoothAdapter; Lcom/android/settings/bluetooth/LocalBluetoothProfileManager; Landroid/bluetooth/BluetoothDevice;)Lcom/android/settings/bluetooth/CachedBluetoothDevice;' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/downloads/Helpers;-generateSaveFile-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I J Z Lcom/android/providers/downloads/StorageManager;)Ljava/lang/String;' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/media/RingtoneManager;-getRingtone-(I)Landroid/media/Ringtone;' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/DataConnection$DcDisconnectionErrorCreatingConnection;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/nsd/NsdManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Lcom/android/systemui/statusbar/phone/PhoneStatusBar$28;-run-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/phone/EnableIccPinScreen;-enableIccPin-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BreadCrumbView;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/UsimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setTimeFromNITZString-(Ljava/lang/String; J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/PhoneFactory;-getCdmaPhone-()Lcom/android/internal/telephony/Phone;' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE', 'android.permission.WRITE_SETTINGS', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/calendar/DayView;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/net/wifi/WifiStateMachine$WpsRunningState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallEnded-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/ContrastEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/NetworkIdentity;-buildNetworkIdentity-(Landroid/content/Context; Landroid/net/NetworkState;)Landroid/net/NetworkIdentity;' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/media/AudioService$AudioHandler;-playSoundEffect-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/os/BatteryStatsImpl;-doUnplugLocked-(J J)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Tab;-updateShouldCaptureThumbnails-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/camera/PhotoModule;-initializeFirstTime-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mms/data/Conversation;-access$900-(Landroid/content/Context;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/DeviceStorageMonitorService;-checkMemory-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.CLEAR_APP_CACHE'],
	'Landroid/webkit/WebViewClassic$WebViewInputConnection;-setComposingText-(Ljava/lang/CharSequence; I)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/BrowserWebView;-goForward-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$WifiSettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/EventInfoFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/contacts/activities/JoinContactActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/camera/CameraActivity;-doChangeCamera-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-onPlayPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-removeBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView$1;-onTextInput-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-handleLongPressOnHome-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/ui/SlideshowPresenter;-present-(Lcom/android/mms/util/ItemLoadedCallback;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/DeviceStorageMonitorService;-cancelFullNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mms/transaction/TransactionService;-access$200-(Lcom/android/mms/transaction/TransactionService;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/view/ViewRootImpl;-enqueueInputEvent-(Landroid/view/InputEvent; Landroid/view/InputEventReceiver; I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/model/AccountTypeManagerImpl;-findUsableInvitableAccountTypes-(Landroid/content/Context;)Ljava/util/Map;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/internal/telephony/ims/IsimUiccRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/SupplicantStateTracker$UninitializedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.INTERNET', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/SlideListItemView;-setVideo-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$DataUsageSummaryActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/AddBookmarkPage$EditBookmarkInfoLoader;-loadInBackground-()Lcom/android/browser/AddBookmarkPage$EditBookmarkInfo;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/BluetoothA2dp;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/settings/Settings$DevelopmentSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/InputMethodManagerService;-isScreenLocked-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransfer$SocketConnectThread;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/LatinIME;-handleCharacter-(I I I I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/nfc/INfcTag$Stub;-resetTimeouts-()V' : ['android.permission.NFC'],
	'Lcom/android/browser/Controller;-onBackKey-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/transaction/SendTransaction;-sendPdu-(J [B Ljava/lang/String;)[B' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManagerService;-checkAuthenticateAccountsPermission-(Landroid/accounts/Account;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/content/SyncManager$SyncHandler;-dispatchSyncOperation-(Landroid/content/SyncOperation;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phone/CallCard;-onQueryComplete-(I Ljava/lang/Object; Lcom/android/internal/telephony/CallerInfo;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ManageSimMessages;-access$600-(Lcom/android/mms/ui/ManageSimMessages;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/systemui/statusbar/tablet/TabletStatusBar$6;-onClick-(Landroid/view/View;)V' : ['android.permission.GET_TASKS'],
	'Lcom/android/phone/OtaUtils;-otaShowProperScreen-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/systemui/statusbar/CommandQueue$Callbacks;-animateCollapsePanels-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/SIMFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/DataUsageSummary;-access$600-(Lcom/android/settings/DataUsageSummary; Z)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$2000-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/systemui/recent/RecentTasksLoader$3;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.GET_TASKS'],
	'Landroid/media/MediaPlayer;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-openTab-(Lcom/android/browser/IntentHandler$UrlData;)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/settings/bluetooth/BluetoothDiscoverableEnabler;-resume-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/backup/IRestoreSession;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewCore;-sendMessage-(I I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/PhotoView;-hideUndoBar-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowManagerService;-access$400-(Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cat/CatService;-eventDownload-(I I I [B Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/EmailAddressPickerFragment;-onItemClick-(I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/view/ViewRootImpl$WindowInputEventReceiver;-onInputEvent-(Landroid/view/InputEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/webkit/PluginFullScreenHolder$1;-onCustomViewHidden-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;-access$1800-(Lcom/android/internal/policy/impl/keyguard/KeyguardViewMediator;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-separateConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/CameraActivity;-gotoGallery-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardMessageArea;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-initSstIcc-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/VoicemailTable$Delegate;-delete-(Lcom/android/providers/contacts/VoicemailContentProvider$UriData; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/bluetooth/hid/HidService;-okToConnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/GrainFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mms/transaction/TransactionService;-launchTransaction-(I Lcom/android/mms/transaction/TransactionBundle; Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getPhonebookNameList-(I)Ljava/util/ArrayList;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/phone/BluetoothPhoneService;-handleQueryPhoneState-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/google/common/io/Files;-toByteArray-(Ljava/io/File;)[B' : ['android.permission.INTERNET'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-rediscover-(I)Landroid/nfc/Tag;' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/test/ModelInterpreter;-<init>-(Ljava/net/InetSocketAddress;)V' : ['android.permission.INTERNET'],
	'Lcom/android/mms/util/Recycler$SmsRecycler;-checkForThreadsOverLimit-(Landroid/content/Context;)Z' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/SIMRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$4;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mms/data/Conversation;-getUri-(J)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/media/effect/effects/BrightnessEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/bluetooth/PanProfile;-isPreferred-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-createLockScreen-()Landroid/view/View;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-<init>-(Landroid/content/Context; Ljava/lang/String; Z)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Lcom/android/browser/SnapshotTab;-capture-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-isKeyguardSecure-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/users/UserSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.INTERNET', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/search/OpenSearchSearchEngine;-isNetworkConnected-(Landroid/content/Context;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/MtpDeviceSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/LocationManager;-startReceivingLocationUpdates-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-releaseWakeLockIfDone-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/StateManager;-pause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/gallery3d/data/MtpDeviceSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/VoicemailContract$Voicemails;-buildSourceUri-(Ljava/lang/String;)Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/keyguard_obsolete/LockPatternKeyguardView;-maybeEnableFallback-(Landroid/content/Context;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/location/fused/FusionEngine;-enable-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-unregisterListener-(Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/contacts/ContactsActivity;-onCreateDialog-(I Landroid/os/Bundle;)Landroid/app/Dialog;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothAdapter;-enable-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/phone/ContactsAsyncHelper$OnImageLoadCompleteListener;-onImageLoadComplete-(I Landroid/graphics/drawable/Drawable; Landroid/graphics/Bitmap; Ljava/lang/Object;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/NavigationBarTablet;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/WebViewFactory;-createSubWebView-(Z)Landroid/webkit/WebView;' : ['android.permission.WRITE_SETTINGS', 'android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/PhotoPage;-onSingleTapUp-(I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/RIL;-resetRadio-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccCardApplication;-supplyPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-determineScrollingStart-(Landroid/view/MotionEvent;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-hasFeatures-(Landroid/accounts/Account; [Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/hdp/HealthService;-getHealthDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/LinkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/settings/SettingsHelper;-setLocaleData-([B I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phone/CallNotifier$EmergencyTonePlayerVibrator;-access$200-(Lcom/android/phone/CallNotifier$EmergencyTonePlayerVibrator;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/settings/Settings$CryptKeeperSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/AccountManager;-confirmCredentials-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/deviceinfo/UsbSettings;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.BLUETOOTH', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/SlideEditorActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/ControllerOverlay$Listener;-onSeekStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-getLocalAlbum-(Lcom/android/gallery3d/data/DataManager; I Lcom/android/gallery3d/data/Path; I Ljava/lang/String;)Lcom/android/gallery3d/data/MediaSet;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/ActivityManager;-restartPackage-(Ljava/lang/String;)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-separate-(Lcom/android/internal/telephony/cdma/CdmaConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/JoinContactListFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Ljava/lang/String; J F Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexServerSession;-releaseWakeLocks-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/State;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.INTERNET', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifySeApduReceived-([B)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/nfc/INfcTag$Stub;-connect-(I I)I' : ['android.permission.NFC'],
	'Lcom/android/launcher2/PagedViewWithDraggableItems;-handleTouchEvent-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mms/transaction/SmsReceiverService;-handleServiceStateChanged-(Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Landroid/nfc/NfcActivityManager;-setNdefPushMessage-(Landroid/app/Activity; Landroid/nfc/NdefMessage;)V' : ['android.permission.NFC'],
	'Landroid/view/WindowManagerPolicy;-interceptKeyBeforeDispatching-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)J' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetherableWifiRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/BitmapSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/contacts/dialpad/DialpadImageButton;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManagerProxy;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS'],
	'Landroid/inputmethodservice/KeyboardView;-setKeyboard-(Landroid/inputmethodservice/Keyboard;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/ActivityManagerNative;-broadcastStickyIntent-(Landroid/content/Intent; Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/calendar/CalendarProvider2;-updateEventRelatedTable-(Landroid/net/Uri; Ljava/lang/String; Z Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String; Z)I' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/NotificationManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/StatusBarManagerService; Lcom/android/server/LightsService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$DriverStartedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/phone/MobileNetworkSettings$MyHandler;-handleGetPreferredNetworkTypeResponse-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PicasaSource;-<init>-(Landroid/content/Context; Landroid/content/SharedPreferences;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/nfc/NfcAdapter;-enableForegroundNdefPush-(Landroid/app/Activity; Landroid/nfc/NdefMessage;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/INetworkPolicyManager;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wm/Session;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/nfc/handover/HandoverTransfer$Callback;-onTransferComplete-(Lcom/android/nfc/handover/HandoverTransfer; Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/PhotoPage;-refreshHidingMessage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewClassic;-setNetworkType-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/wifi/WifiConfigInfo;-onResume-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/AudioRecord;-<init>-(I I I I I)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/server/am/ActivityManagerService;-moveTaskToBack-(I)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/net/wifi/IWifiManager$Stub;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl;-processCommand-(Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-changeBarringPassword-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/list/PhoneFavoriteFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/webkit/WebViewCore;-jsInterrupt-()Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/DayView;-updateTitle-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/activities/PeopleActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/webkit/WebViewClassic;-clearHistory-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-setCdmaBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/NullFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/telephony/SmsProvider;-deleteMessageFromIcc-(Ljava/lang/String;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/quicksearchbox/util/JavaNetHttpHelper;-post-(Lcom/android/quicksearchbox/util/HttpHelper$PostRequest;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/settings/bluetooth/LocalBluetoothAdapter;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mms/ui/ComposeMessageActivityNoLockScreen;-confirmDeleteThread-(J)V' : ['android.permission.READ_SMS'],
	'Lcom/android/dreams/phototable/PhotoCarousel;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mms/data/Conversation;-cacheAllThreads-(Landroid/content/Context;)V' : ['android.permission.READ_SMS'],
	'Landroid/provider/CallLog$Calls;-addCall-(Lcom/android/internal/telephony/CallerInfo; Landroid/content/Context; Ljava/lang/String; I I J I)Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'com.android.voicemail.permission.ADD_VOICEMAIL', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/app/IBackupAgent;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/BluetoothManagerService;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/test/mock/MockContentResolver;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifyLlcpLinkDeactivated-(Lcom/android/nfc/dhimpl/NativeP2pDevice;)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifySeEmvCardRemoval-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/systemui/recent/RecentsCallback;-handleOnClick-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/inputmethod/latin/LatinIME;-access$000-(Lcom/android/inputmethod/latin/LatinIME;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SOCIAL_STREAM'],
	'Landroid/filterpacks/base/GLTextureTarget;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView$OnTriggerListener;-onGrabbedStateChange-(Landroid/view/View; I)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.VIBRATE', 'android.permission.READ_CONTACTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/Tab$7;-onDownloadStart-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.INTERNET', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS', 'android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-pollState-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/media/AudioService;-access$2200-(Landroid/media/AudioService;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/DownUpDetector;-setState-(Z Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/os/IPowerManager;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-goTo-(J Z Z Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/nfc/NfcService;-onRemoteFieldDeactivated-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/nfc/NfcService;-onLlcpLinkDeactivated-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/P2pLinkManager;-onLlcpActivated-()V' : ['android.permission.VIBRATE', 'android.permission.GET_TASKS'],
	'Lcom/android/internal/app/LocalePicker$LocaleSelectionListener;-onLocaleSelected-(Ljava/util/Locale;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendUSSD-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameBranch;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/Settings$SecuritySettingsActivity;-updateHeaderList-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CsimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-skipPendingBroadcastLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/ThrottleService;-getPeriodStartTime-(Ljava/lang/String;)J' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/DataController;-<init>-(Landroid/content/Context;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/camera/PhotoController;-onListPrefChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/AccessibilityInjectorFallback;-traverseGivenAxis-(I I Z Ljava/lang/String;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/SnailAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MusicPicker;-stopMediaPlayer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardSimPukView;-reset-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/vpn2/VpnSettings$LockdownConfigFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/keyguard/KeyguardWidgetCarousel;-performAccessibilityAction-(I Landroid/os/Bundle;)Z' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine;-broadcastConnectionState-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mms/ui/ConversationList;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/activities/ContactDetailActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.NFC'],
	'Lcom/android/exchange/adapter/CalendarSyncAdapter;-sendLocalChanges-(Lcom/android/exchange/adapter/Serializer;)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/settings/bluetooth/BluetoothPermissionActivity;-onPositive-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/google/common/io/Resources;-readLines-(Ljava/net/URL; Ljava/nio/charset/Charset;)Ljava/util/List;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.NFC'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-onMenuClicked-(I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener; Z Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.VIBRATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/settings/SettingsPreferenceFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/webkit/WebSettingsClassic;-setBlockNetworkLoads-(Z)V' : ['android.permission.INTERNET'],
	'Lcom/android/calendar/alerts/AlertReceiver;-buildBasicNotification-(Landroid/app/Notification$Builder; Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; J J J I Z I Z)Landroid/app/Notification;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-unregisterForSuppServiceNotification-(Landroid/os/Handler;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCfuQueryResult-([Lcom/android/internal/telephony/CallForwardInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarViewAdapter$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/data/DownloadUtils;-download-(Lcom/android/gallery3d/util/ThreadPool$JobContext; Ljava/net/URL; Ljava/io/OutputStream;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/keyguard/EmergencyButton$1;-onPhoneStateChanged-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/bluetooth/BluetoothEventManager;-access$1400-(Lcom/android/settings/bluetooth/BluetoothEventManager; Lcom/android/settings/bluetooth/CachedBluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothHeadset;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/BluetoothPairingDialog;-createDisplayPasskeyOrPinDialog-(Lcom/android/settings/bluetooth/CachedBluetoothDeviceManager;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/group/GroupDetailFragment$2;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/power/PowerManagerService;-setScreenBrightnessOverrideFromWindowManagerInternal-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/settings/Settings$AccessibilitySettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService$15;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/EventInfoFragment;-saveReminders-()Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/quickcontact/QuickContactActivity$5;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/tech/BasicTagTechnology;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/server/power/PowerManagerService;-scheduleSandmanLocked-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/providers/contacts/ContactsProvider2;-updateAccountsInBackground-([Landroid/accounts/Account;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/view/inputmethod/InputMethodManager$FinishedEventCallback;-finishedEvent-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$6;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BackupManagerService$PerformRestoreTask;-initiateOneRestore-(Landroid/content/pm/PackageInfo; I Landroid/app/IBackupAgent; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/settings/accounts/SyncSettings;-onStart-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/settings/Settings$DevelopmentSettingsActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/SuggestionsAdapter$CombinedCursor;-runQuery-(Ljava/lang/CharSequence;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/INetworkPolicyManager;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/hdp/HealthService;-broadcastHealthDeviceStateChange-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/settings/bluetooth/A2dpProfile;-setPreferred-(Landroid/bluetooth/BluetoothDevice; Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-startLegacyVpn-(Lcom/android/internal/net/VpnProfile;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/settings/Settings$StorageUseActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothDevice;-cancelPairingUserInput-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/contacts/activities/DialtactsActivity;-onBackPressed-()V' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS', 'android.permission.WRITE_CONTACTS'],
}

AOSP_PERMISSIONS_BY_FIELDS = {
	'Landroid/provider/ContactsContract;-AUTHORITY_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$Directory;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/TempFileProvider;-SCRAP_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_MULTI_VCARD_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Settings$Global;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/ContactsContract$DataUsageFeedback;-FEEDBACK_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$StreamItems;-CONTENT_LIMIT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/emailcommon/provider/EmailContent;-CONTENT_NOTIFIER_URI-Landroid/net/Uri;' : ['com.android.email.permission.ACCESS_PROVIDER'],
	'Landroid/provider/Telephony$Mms$Inbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$Mms$Draft;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/BrowserContract$Bookmarks;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Contacts$ContactMethods;-CONTENT_EMAIL_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/launcher2/LauncherSettings$Favorites;-CONTENT_URI_NO_NOTIFICATION-Landroid/net/Uri;' : ['com.android.launcher.permission.READ_SETTINGS', 'com.android.launcher.permission.WRITE_SETTINGS'],
	'Landroid/provider/Downloads$Impl;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/provider/Telephony$MmsSms$PendingMessages;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$Attendees;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Settings$System;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/providers/telephony/SmsProvider;-NOTIFICATION_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$AggregationExceptions;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/transaction/MessageStatusService;-STATUS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$SyncState;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/CallLog$Calls;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/provider/Contacts$GroupMembership;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Contacts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Telephony$Sms$Inbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/common/contacts/DataUsageStatUpdater$DataUsageFeedback;-FEEDBACK_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$Profile;-CONTENT_RAW_CONTACTS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Contacts$Groups;-DELETED_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Contacts$Settings;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/provider/GalleryProvider;-BASE_URI-Landroid/net/Uri;' : ['com.android.gallery3d.permission.GALLERY_PROVIDER'],
	'Landroid/provider/ContactsContract$Data;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Telephony$Threads;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$Profile;-CONTENT_VCARD_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$Settings;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/BrowserContract$Images;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/provider/BrowserProvider2$OmniboxSuggestions;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/VoicemailContract$Status;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/provider/ContactsContract$DataUsageFeedback;-DELETE_USAGE_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Telephony$Mms;-REPORT_REQUEST_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/BrowserContract$SyncState;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mms/data/RecipientIdCache;-sAllCanonical-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$MmsSms;-SEARCH_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/mms/data/RecipientIdCache;-sSingleCanonicalAddressUri-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/mms/data/Contact$ContactsCache;-PHONES_WITH_PRESENCE_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Contacts$Presence;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/provider/SnapshotProvider$Snapshots;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/BrowserContract$Searches;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$Mms$Outbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Contacts$Phones;-CONTENT_FILTER_URL-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/emailcommon/provider/EmailContent;-MAILBOX_NOTIFICATION_URI-Landroid/net/Uri;' : ['com.android.email.permission.ACCESS_PROVIDER'],
	'Lcom/android/providers/calendar/CalendarAlarmManager;-SCHEDULE_ALARM_REMOVE_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/emailcommon/provider/EmailContent;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.email.permission.ACCESS_PROVIDER'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-LIVE_FOLDERS_CONTACTS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Telephony$Mms$Rate;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$RawContactsEntity;-PROFILE_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/emailcommon/provider/EmailContent;-MAILBOX_MOST_RECENT_MESSAGE_URI-Landroid/net/Uri;' : ['com.android.email.permission.ACCESS_PROVIDER'],
	'Landroid/provider/CalendarContract$Colors;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_VCARD_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/CalendarContract$CalendarEntity;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Telephony$Sms$Sent;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_BY_DAY_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Contacts$GroupMembership;-RAW_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/launcher2/LauncherSettings$Favorites;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.launcher.permission.READ_SETTINGS', 'com.android.launcher.permission.WRITE_SETTINGS'],
	'Landroid/provider/Downloads$Impl;-PUBLICLY_ACCESSIBLE_DOWNLOADS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/provider/Telephony$Sms$Draft;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/bluetooth/opp/BluetoothShare;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.ACCESS_BLUETOOTH_SHARE'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_GROUP_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/CalendarContract;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_STREQUENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$DisplayPhoto;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_FREQUENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mms/data/Contact$ContactsCache;-EMAIL_WITH_PRESENCE_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_DRAFT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$Events;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/CallLog$Calls;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/contacts/model/account/AccountWithDataSet;-RAW_CONTACTS_URI_LIMIT_1-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Settings$Secure;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/Contacts$People;-WITH_EMAIL_OR_IM_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$DisplayPhoto;-CONTENT_MAX_DIMENSIONS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$ProviderStatus;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/CalendarContract$Reminders;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$StreamItems;-CONTENT_PHOTO_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-CONTENT_URI_BY_INSTANCE-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/BrowserContract$Settings;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_LOCKED_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$Groups;-CONTENT_SUMMARY_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/P2pLinkManager;-PROFILE_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_UNDELIVERED_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/VoicemailContract$Voicemails;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Email;-CONTENT_LOOKUP_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/contacts/debug/DumpFileProvider;-AUTHORITY_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/BrowserContract;-AUTHORITY_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$Mms;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/contacts/model/account/AccountTypeWithDataSet;-RAW_CONTACTS_URI_LIMIT_1-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$PhoneLookup;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/telephony/SmsProvider;-ICC_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/Contacts$Organizations;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/BrowserContract$Accounts;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$Threads;-THREAD_ID_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-LIVE_FOLDERS_CONTACTS_FAVORITES_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/CallLog$Calls;-CONTENT_URI_WITH_VOICEMAIL-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/provider/ContactsContract$StatusUpdates;-PROFILE_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/mms/ui/ManageSimMessages;-ICC_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/BrowserContract$Bookmarks;-CONTENT_URI_DEFAULT_FOLDER-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/CalendarContract$Events;-CONTENT_EXCEPTION_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/contacts/list/DirectoryListLoader$DirectoryQuery;-URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$StreamItems;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_SEARCH_BY_DAY_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Contacts$People;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Phone;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Contacts$Photos;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_SEARCH_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/ex/chips/BaseRecipientAdapter$DirectoryListQuery;-URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$Groups;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Browser;-BOOKMARKS_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/CalendarContract$Calendars;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/common/contacts/BaseEmailAddressAdapter$DirectoryListQuery;-URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_FILTER_BYPHONE_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/providers/userdictionary/UserDictionaryProvider;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/mms/data/Conversation;-sAllThreadsUri-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$SyncState;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/BrowserContract$History;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/emailcommon/utility/AttachmentUtilities;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.email.permission.READ_ATTACHMENT'],
	'Landroid/provider/UserDictionary$Words;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Landroid/provider/Contacts$ContactMethods;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Browser;-SEARCHES_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/launcher2/LauncherProvider;-CONTENT_APPWIDGET_RESET_URI-Landroid/net/Uri;' : ['com.android.launcher.permission.READ_SETTINGS', 'com.android.launcher.permission.WRITE_SETTINGS'],
	'Landroid/provider/BrowserContract$ImageMappings;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/CalendarContract$EventsEntity;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_CONVERSATIONS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$ProfileSyncState;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$StructuredPostal;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Contacts$Extensions;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Contacts$Phones;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Telephony$Sms;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$Profile;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/BrowserContract$Combined;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_STREQUENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/media/MediaProvider;-ALBUMART_URI-Landroid/net/Uri;' : ['android.permission.READ_EXTERNAL_STORAGE', 'android.permission.WRITE_EXTERNAL_STORAGE'],
	'Landroid/provider/Downloads$Impl;-ALL_DOWNLOADS_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.ACCESS_ALL_DOWNLOADS', 'android.permission.WRITE_EXTERNAL_STORAGE', 'android.permission.INTERNET'],
	'Landroid/provider/UserDictionary;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/mms/transaction/MessagingNotification;-UNDELIVERED_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$Sms$Outbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Callable;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Telephony$Sms$Conversations;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/Settings$Bookmarks;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/SMSDispatcher;-mRawUri-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/CallLog;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG', 'com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/provider/Telephony$Mms;-REPORT_STATUS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Email;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$RawContacts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Contacts$Groups;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/providers/contacts/LegacyApiSupport;-LIVE_FOLDERS_CONTACTS_WITH_PHONES_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Telephony$CellBroadcasts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CELL_BROADCASTS'],
	'Lcom/android/gallery3d/filtershow/provider/SharedImageProvider;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.gallery3d.filtershow.permission.WRITE', 'com.android.gallery3d.filtershow.permission.READ'],
	'Landroid/provider/Telephony$Threads;-OBSOLETE_THREADS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/providers/calendar/CalendarAlarmManager;-SCHEDULE_ALARM_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/CalendarContract$EventDays;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Telephony$Mms$Sent;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Contacts$People;-DELETED_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$StatusUpdates;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/provider/BrowserProvider2$Thumbnails;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/contacts/GroupListLoader;-GROUP_LIST_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Callable;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$RawContactsEntity;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/CalendarContract$ExtendedProperties;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Email;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/MusicUtils;-sArtworkUri-Landroid/net/Uri;' : ['android.permission.READ_EXTERNAL_STORAGE', 'android.permission.WRITE_EXTERNAL_STORAGE'],
	'Landroid/provider/Contacts$People;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/provider/SnapshotProvider;-AUTHORITY_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/CalendarContract$CalendarCache;-URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_LOOKUP_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Phone;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SOCIAL_STREAM', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SOCIAL_STREAM', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_CONTACTS'],
}
#################################################
