#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is a part of Androguard.
#
# This file is generated automatically from the data
# provided by PScout tool [http://pscout.csl.toronto.edu/]
# using script: extract_api_permission_mappings.py
#
# Author: Yury Zhauniarovich
#
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS-IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License"

#################################################
### API version: 19 
#################################################


AOSP_PERMISSIONS_BY_METHODS = {
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handlePinMmi-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/date/DatePickerDialog;-updateDisplay-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardEntry$BirthdayData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/GLTextureTarget;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UrlInputView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageAttachmentBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSync;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/ui/GLRootView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt;-registerClient-(Landroid/os/ParcelUuid; Landroid/bluetooth/IBluetoothGattCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserWebView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/DialogActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onPressKey-(I I Z)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/email/provider/EmailProvider;-restartPushForMailbox-(Landroid/content/Context; Landroid/database/sqlite/SQLiteDatabase; Landroid/content/ContentValues; Ljava/lang/String;)Z' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasSettings;-getRequestUri-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/agenda/AgendaFragment;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/support/v4/widget/ContentLoadingProgressBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$AudioOn;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$10300-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/SearchView$SearchAutoComplete;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/VpnService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/RecentApplicationsBackground;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/soundrecorder/Recorder;-delete-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager$WifiLock;-acquire-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter$SlideshowSource;-findItemIndex-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager;-getMobileProvisioningUrl-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-addWindow-(Lcom/android/server/wm/Session; Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment$3;-run-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-clearServices-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendRawPdu-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/PolicyService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/AccountService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/AbstractSettingPopup;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-moveQueueItem-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lorg/apache/http/impl/client/DefaultHttpClient;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/browser/SuggestionsAdapter$SlowFilterTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-moveStackWindowsLocked-(Lcom/android/server/wm/DisplayContent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/googlecode/mp4parser/authoring/builder/Mp4Builder;-build-(Lcom/googlecode/mp4parser/authoring/Movie;)Lcom/coremedia/iso/IsoFile;' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/CropFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/email/provider/FolderPickerActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController$FullHighPerfLockHeldState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageDraw;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/magicsmoke/MagicSmokeSelector;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/MailLogService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/camera/ui/FaceView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu;-updateSelection-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/view/SurfaceView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarViewAdapter;-<init>-(Landroid/content/Context; I Z)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothMap$Stub$Proxy;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/IConnectivityManager;-getAllNetworkInfo-()[Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/service/ImapAuthenticatorService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/FrameSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/picasasource/PicasaSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/ui/RotatableLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/statusbar/IStatusBarService;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/server/BackupManagerService;-opComplete-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/speech/tts/ITextToSpeechService$Stub$Proxy;-speak-(Landroid/os/IBinder; Ljava/lang/String; I Landroid/os/Bundle;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/fall/Fall;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneProxy;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/FilmstripPage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/ConnectivityService;-access$1600-(Lcom/android/server/ConnectivityService; Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-query-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/support/v13/app/FragmentTabHost;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageInviteView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-saveLockPassword-(Ljava/lang/String; I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-addAppWidget-(I I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaRecorder;-setVideoSource-(I)V' : ['android.permission.CAMERA'],
	'Landroid/filterpacks/base/GLTextureSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/timezonepicker/TimeZonePickerView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/DownloadHandler;-onDownloadStart-(Landroid/app/Activity; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Z)V' : ['android.permission.INTERNET'],
	'Lcom/android/camera/ProxyLauncher;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/emailcommon/service/IPolicyService$Stub$Proxy;-setAccountPolicy-(J Lcom/android/emailcommon/provider/Policy; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/compose/AutoSendActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationsInOutboxTipView;-goToOutbox-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub;-getTetherableUsbRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/Development;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processVolumeEvent-(I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/TrackBrowserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/TelephonyManager;-getDeviceSoftwareVersion-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/print/IPrintManager;-stopPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-hasMultipleEnabledSubtypesInThisIme-(Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/Session;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/media/AudioService;-adjustStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/service/AccountService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt;-beginReliableWrite-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/grass/Grass;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/FlipperDream;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/app/AlertActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSearch;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/content/MutableContextWrapper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/colorpicker/ColorPickerPalette;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/util/ReverseGeocoder;-computeAddress-(Lcom/android/gallery3d/util/ReverseGeocoder$SetLatLong;)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/PhotoView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phasebeam/PhaseBeamView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/EasService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/PopupWindow$PopupViewContainer;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/GridView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/SupplicantStateTracker$DormantState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-doNotifyDataConnection-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-getCLIR-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiService;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/mail/ui/FolderSelectionActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/view/PagerTitleStrip;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/ColorButton;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/UiModeManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountAuthenticatorActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/IActivityManager;-unbroadcastIntent-(Landroid/app/IApplicationThread; Landroid/content/Intent; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ActionBarContainer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/OverlayTitleTemplatePicker;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-confirmAndDelete-(I Ljava/util/Collection; Z I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/effect/FilterGraphEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService$Stub$Proxy;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/Phone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms;-copyMessageToIccEf-(Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/gallery3d/app/TrimTimeBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/ListMenuItemView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/hardware/location/GeofenceHardware;-pauseGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-stopScan-(I Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/VoicemailContract$Status;-buildSourceUri-(Ljava/lang/String;)Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/browser/TabScrollView$TabLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/NativeActivity$NativeContentView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/ModuleSwitcher;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-agentErrorCleanup-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/noisefield/NoiseFieldWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calculator2/ColorButton;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/InitAlarmsService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/packageinstaller/GrantActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceResumeStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/nexus/NexusWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/view/RotationPolicy;-isRotationLockToggleVisible-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/OnePaneController;-updateAccounts-(Lcom/android/mail/content/ObjectCursor;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/contacts/ContactsListActivity;-getUriToQuery-()Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/bluetooth/BluetoothHealth;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/EasOutboxService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AccountController;-changeAccount-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterApp;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupStepIndicatorView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasOperation;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/ConnectivityService;-<init>-(Landroid/content/Context; Landroid/os/INetworkManagementService; Landroid/net/INetworkStatsService; Landroid/net/INetworkPolicyManager;)V' : ['android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/photos/views/HeaderGridView$FullWidthFixedViewLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/ProfileService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/MailWebView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/EmailProvider;-startSync-(Lcom/android/emailcommon/provider/Mailbox; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/email/service/LegacyImapAuthenticatorService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/ShutdownActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/TextureView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-startPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/ConversationListFragment;-onConversationListStatusUpdated-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/MinTimeProgressView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/UsimDataDownloadHandler;-sendSmsAckForEnvelopeResponse-(Lcom/android/internal/telephony/uicc/IccIoResult; I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasSyncService;-canProvision-(Lcom/android/exchange/EasSyncService;)Lcom/android/exchange/adapter/ProvisionParser;' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CompoundButton;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/PlatLogoActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/VideoEditorActivity$23;-onStop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Telephony$Sms$Sent;-addMessage-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long;)Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/power/PowerManagerService$1;-onProximityPositive-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendTouchExplorationGestureStartAndHoverEnterIfNeeded-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-supplyPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapUtils;-isProfileSet-(Landroid/content/Context;)Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/email/activity/EventViewer;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/AbstractActivityController;-showConversation-(Lcom/android/mail/providers/Conversation;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/providers/AccountObserver;-onChanged-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Landroid/content/IContentService;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/widget/ExpandableListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/AllInOneActivity;-configureActionBar-(I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/development/PackageSummary;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/development/RunningProcesses;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-registerReceiver-(Landroid/app/IApplicationThread; Ljava/lang/String; Landroid/content/IIntentReceiver; Landroid/content/IntentFilter; Ljava/lang/String; I)Landroid/content/Intent;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/exchange/eas/EasOptions;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/ComboAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enableGsmBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/view/VolumePanel;-onRemoteVolumeChanged-(I I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/widget/ZoomButtonsController$Container;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/dreams/Sandman;-startDreamWhenDockedIfAppropriate-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/ConnectivityService;-setLockdownTracker-(Lcom/android/server/net/LockdownVpnTracker;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/app/LocalePicker;-updateLocale-(Ljava/util/Locale;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationListFragment;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/widget/WidgetService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine;-access$1100-(Lcom/android/bluetooth/a2dp/A2dpStateMachine; Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/wm/WindowManagerService;-requestTraversalLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-setRadioPower-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-removeDeletion-(Lcom/android/gallery3d/data/Path;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/RadioButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/AccountsTester;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/bmgr/Bmgr;-doListRestoreSets-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/compose/ComposeActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/view/StopProgressView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/keychain/KeyChainActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/LegacyImapAuthenticatorService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/ConnectivityManager;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/LocationManagerService;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/support/v4/app/FragmentActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/SecureConversationViewController;-renderMessage-(Lcom/android/mail/browse/ConversationMessage;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/app/MoviePlayer;-access$800-(Lcom/android/gallery3d/app/MoviePlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UrlInputView;-onSelect-(Ljava/lang/String; I Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/VibratorService;-onInputDeviceRemoved-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/ui/FramedTextButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/Pop3Service$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/DataManager;-delete-(Lcom/android/gallery3d/data/Path;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AbstractActivityController;-destroyPending-(Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine;-access$2200-(Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ActivityManagerNative;-handleApplicationWtf-(Landroid/os/IBinder; Ljava/lang/String; Landroid/app/ApplicationErrorReport$CrashInfo;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-setUpAdapter-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mail/compose/FromAddressSpinner;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BatteryService$BatteryListener;-batteryPropertiesChanged-(Landroid/os/BatteryProperties;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/uicc/UiccCard;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/uicc/IccCardStatus;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-updatePhoneObject-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-getInstalledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/AbstractSyncAdapterService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-acceptCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/WaveView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/widget/EllipsizeLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/ButtonSwitcher;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/support/v7/widget/GridLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/LatinIME;-handleNonSpecialCharacter-(I I I I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/PackageManagerBackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/browser/PageDialogsHandler$5;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractEditText;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/pinyin/SettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/SearchActivity;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mail/ui/TwoPaneController;-access$300-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/providers/Conversation; Ljava/util/Set; [B)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/speech/tts/ITextToSpeechService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-handleOnRotationChanged-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/view/CertificateSelector;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/view/CertificateSelector;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/GoogleAccountLogin;-invalidateTokens-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/service/wallpaper/WallpaperService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/EmojiCategoryPageIndicatorView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-isNetworkSupported-(I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/service/PingTask;-onCancelled-(Ljava/lang/Void;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateStart;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/net/wifi/WifiStateMachine;-handlePostDhcpSetup-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailcommon/service/AccountServiceProxy$6;-run-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/ConversationViewFragment$MailJsBridge$1;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Landroid/net/INetworkPolicyManager$Stub;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/WakeLockHoldingProgressListener;-onProgressStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-hangupConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/NotificationManagerService;-rebindListenerServices-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/AudioPreview;-playPauseClicked-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ingest/ui/IngestGridView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setPasskey-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/util/Log$1;-onTerribleFailure-(Ljava/lang/String; Landroid/util/Log$TerribleFailure;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager;-createPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/StatusBarManager;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/browser/TabControl;-setCurrentTab-(Lcom/android/browser/Tab; Z)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/text/StringSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationPager;-pageRight-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/NativeActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/print/PrintManagerService;-access$200-(Lcom/android/server/print/PrintManagerService; I)Lcom/android/server/print/UserState;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AsyncPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/Wallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorValueView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/SelectPrinterActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/MediaSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-showNextConversation-(Ljava/util/Collection;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/photos/FullscreenViewer;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarEventModel;-<init>-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractActivityController$8;-run-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/DigitalClock;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-wakingUp-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/AlertActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/AllInOneActivity;-onBackPressed-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetoothHeadset;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/CaptivePortalTracker;-<init>-(Landroid/content/Context; Landroid/net/IConnectivityManager;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/development/CacheAbuser;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/backup/LocalTransportService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/DatePicker;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/SnoozeAlarmsService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/ConversationCursor$3;-run-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationViewFragment;-onConversationSeen-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ShortcutNameActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/SnailSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/ConnectivityService;-requestNetworkTransitionWakelock-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/WeekSelector;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onUpEventInternal-(I I J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/CalendarView;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RotateFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/documentsui/DocumentsApplication;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/PackageBrowser;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ViewMode;-enterSearchResultsConversationMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-setAlphabetKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/RingtoneManager;-getActualDefaultRingtoneUri-(Landroid/content/Context; I)Landroid/net/Uri;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/photo/PhotoViewActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/map/BluetoothMapObexServer;-onPut-(Ljavax/obex/Operation;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/input/InputManagerService;-start-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calculator2/Calculator;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/TwoPaneController;-onAnimationEnd-(Lcom/android/mail/ui/AnimatedAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/view/PieListView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/LockPatternUtils;-saveLockPattern-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/os/RuntimeInit$UncaughtHandler;-uncaughtException-(Ljava/lang/Thread; Ljava/lang/Throwable;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/TestActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/compose/QuotedTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CheckedTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/widget/SearchViewCompatIcs$MySearchView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pEnabledState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/Phone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/GalleryAppImpl;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-dispatchUnhandledKey-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPickerActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcDisconnectionErrorCreatingConnection;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-publishUpdateWordListCompleted-(Landroid/content/Context; Z J Landroid/database/sqlite/SQLiteDatabase; Landroid/content/ContentValues; Ljava/lang/String;)V' : ['android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetooth$Stub;-getAdapterConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView$Listener;-pickSuggestionManually-(I Lcom/android/inputmethod/latin/SuggestedWords$SuggestedWordInfo;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/support/v7/widget/GridLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/AppHwPref;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/DayView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/mail/photomanager/ContactPhotoManager$ContactPhotoLoaderThread;-loadPhotosInBackground-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/LocationManagerService;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/mail/compose/FromAddressSpinner;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/LogViewer;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/NetInitiatedActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextClock$1;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onKeyRepeat-(I I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/server/WallpaperManagerService$MyPackageMonitor;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-matchRecipientSms-(Landroid/database/Cursor; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo; Ljava/lang/String;)Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-doRecovery-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHeadset;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/packageinstaller/TabsAdapter;-setPrimaryItem-(Landroid/view/ViewGroup; I Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetooth;-setPasskey-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/videosrc/MediaSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/email/service/LegacyEasAuthenticatorService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/sip/SipService;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getActiveLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/accounts/IAccountManager$Stub;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-handleVolumeKey-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/BatchService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/ViewStub;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/View$AttachInfo$Callbacks;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavTabView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/IAccountManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/widget/NumberPicker$CustomEditText;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/AccountsChangedReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/hdp/HealthService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/VideoModule;-initializeRecorder-()V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ingest/IngestService;-setDevice-(Landroid/mtp/MtpDevice;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-access$200-(Lcom/android/server/am/ActivityManagerService; Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-markAsDeleted-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; I I)V' : ['android.permission.INTERNET'],
	'Lcom/android/development/Details;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/CompatModeDialog$1;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/dreams/phototable/PhotoTableDream;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/gallery3d/ui/TileImageView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/ConversationsInOutboxTipView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/AddBookmarkPage;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/service/PolicyService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-stopDtmf-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/NonDistinctMultitouchHelper;-injectMotionEvent-(I F F J J Lcom/android/inputmethod/keyboard/PointerTracker; Lcom/android/inputmethod/keyboard/PointerTracker$KeyEventHandler;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/documentsui/DirectoryFragment;-access$1700-(Landroid/content/Context; J)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/fall/FallWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/provider/EmailProvider;-uiSaveDraftMessage-(J Landroid/os/Bundle;)Landroid/net/Uri;' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/ConsumerIrService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/emailcommon/Device;-getDeviceId-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$FullImageJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/RecognitionService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/location/LocationBlacklist;-getStringArrayLocked-(Ljava/lang/String;)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-access$200-()Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/net/wifi/WifiStateMachine;-access$8000-(Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSettings;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-setInitialAttachApn-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AddBookmarkPage;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/commands/svc/PowerCommand;-run-([Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/Exchange;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/ex/photo/PhotoViewPager;-dataSetChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IBatteryPropertiesListener$Stub;-batteryPropertiesChanged-(Landroid/os/BatteryProperties;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/CalendarView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/VerticalTextSpinner;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-msgListingSize-(Ljava/lang/String; Lcom/android/bluetooth/map/BluetoothMapAppParams;)I' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/mail/browse/ScrollIndicatorsView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/FeedbackActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/RIL;-setLocationUpdates-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/CameraRootView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/deskclock/timer/TimerView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/photo/fragments/PhotoViewFragment;-onResume-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/recurrencepicker/LinearLayoutWithMaxWidth;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onReleaseShift-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/DigitalClock;-access$300-(Landroid/widget/DigitalClock;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/app/ChooserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/effect/FilterGraphEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/os/IVibratorService$Stub$Proxy;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/BluetoothDevice;-createInsecureRfcommSocketToServiceRecord-(Ljava/util/UUID;)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/dreams/phototable/FlipperDream;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/keyboard/internal/PreviewPlacerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/Screensaver;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/app/NoSaveStateFrameLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/time/RadialSelectorView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/ContactStatusLoader;-getContactInfo-(Landroid/content/Context; Ljava/lang/String;)Lcom/android/email/activity/ContactStatusLoader$Result;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/alerts/InitAlarmsService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService$6;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/TrimVideo;-access$100-(Lcom/android/gallery3d/app/TrimVideo;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/development/BadBehaviorActivity$BadService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/grass/GrassWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/EventInfoFragment;-access$1700-(Lcom/android/calendar/EventInfoFragment; Landroid/view/View;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/Browser;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasPing;-getRequestEntity-()Lorg/apache/http/HttpEntity;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/packageinstaller/UninstallerActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/ConversationListFragment;-showSyncStatusBar-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/Controller;-onProgressChanged-(Lcom/android/browser/Tab;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/tts/compat/CompatTtsService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/WidgetTypeChooser;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-sendConnectionStateChange-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email2/ui/CreateShortcutActivityEmail;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/timer/TimerView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ResolverActivity$1;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AnimatedAdapter;-access$400-(Lcom/android/mail/ui/AnimatedAdapter; Ljava/lang/Object; Ljava/util/HashSet;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService$5;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/AttachmentDownloadService;-canPrefetchForAccount-(Lcom/android/emailcommon/provider/Account; Ljava/io/File;)Z' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/Notifier$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/PickerActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/OnePaneController;-onRefreshReady-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasSettings;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothPbap;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/view/ScrollerView;-onLayout-(Z I I I I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/utils/NotificationUtils;-findContacts-(Landroid/content/Context; Ljava/util/Collection;)Ljava/util/ArrayList;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ListView;-commonKey-(I I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-onFinishSlidingInput-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-disableCabMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/deskclock/AnalogClock;-onTimeChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/service/ApiService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/FaceView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/ImapService;-loadUnsyncedMessages-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/mail/Folder; Ljava/util/ArrayList; Lcom/android/emailcommon/provider/Mailbox;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/app/IBatteryStats;-getStatistics-()[B' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPuk2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-queryPin1State-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/NetInitiatedActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/ExchangeService$1;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/view/View;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/EventInfoFragment;-showContactInfo-(Lcom/android/calendar/CalendarEventModel$Attendee; Landroid/graphics/Rect;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/ui/PhotoView$1;-onClick-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/event/AttendeesView;-addAttendees-(Ljava/lang/String;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/net/IConnectivityManager$Stub;-getNetworkInfo-(I)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/IConnectivityManager;-tether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/inputmethodservice/KeyboardView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventView;-setDate-(Landroid/widget/TextView; J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationViewFragment;-processNewOutgoingMessage-(Lcom/android/mail/browse/ConversationMessage;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/media/effect/effects/GrayscaleEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$IndicatorView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-handleDragFromStarred-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/Wallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/FolderSelectionActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/WifiStateMachine$CaptivePortalCheckState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofencePauseStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/documentsui/TestActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/LocalSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-onBackPressed-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/XLargeUi;-<init>-(Landroid/app/Activity; Lcom/android/browser/UiController;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/ui/AnimatedAdapter;-setFooterVisibility-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/ArtistAlbumBrowserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/TwoPaneController;-showConversation-(Lcom/android/mail/providers/Conversation; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-confirmCredentialsAsUser-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler; Landroid/os/UserHandle;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/mail/ui/TwoPaneController;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ConversationSelectionSet;-clear-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/hardware/location/GeofenceHardware;-addGeofence-(I I Landroid/hardware/location/GeofenceHardwareRequest; Landroid/hardware/location/GeofenceHardwareCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/PackageBrowser;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/soundrecorder/SoundRecorder;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO'],
	'Landroid/content/IContentService$Stub;-sync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-supplyPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setGsmBroadcastConfig-([Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/photos/FullscreenViewer;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-executeNextState-(Lcom/android/server/BackupManagerService$BackupState;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/camera/ui/RenderOverlay$RenderView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-doFinish-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/widget/BookmarkWidgetConfigure;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService$WakeLock;-binderDied-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/CreatePlaylist;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/telephony/PhoneNumberUtils;-isVoiceMailNumber-(Ljava/lang/String;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/Phone;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/RenamePlaylist;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine;-stopTethering-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/dreams/phototable/PhotoCarousel;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/widget/FaceUnlockView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-onCreate-(Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/ActionModeHandler$2;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/mail/ui/FolderItemView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/ActivityState;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/GeolocationPermissionsPrompt;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/DeskClock;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/HandleView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/UserManagerService$5;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onReleaseKey-(I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-addSecureAlbumItem-(Z I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccProvider;-loadFromEf-(I)Landroid/database/MatrixCursor;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/browser/view/SnapshotGridView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Presentation$1;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/notification/NotificationListenerService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/SizeAdaptiveLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/galaxy4/GalaxyView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$System;-getShowGTalkServiceStatusForUser-(Landroid/content/ContentResolver; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setInitialAttachApn-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameStore;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/location/LocationManager;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/filtershow/CenteredLinearLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/PageProgressView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-addOnAccountsUpdatedListener-(Landroid/accounts/OnAccountsUpdateListener; Landroid/os/Handler; Z)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/TitleBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$2;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/timer/TimerAlertFullScreen;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/AlbumSlidingWindow;-onContentChanged-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/IAudioService$Stub;-reloadAudioSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-configHciSnoopLog-(Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/utils/NotificationUtils;-configureLatestEventInfoFromConversation-(Landroid/content/Context; Lcom/android/mail/providers/Account; Lcom/android/mail/preferences/FolderPreferences; Landroid/support/v4/app/NotificationCompat$Builder; Landroid/database/Cursor; Landroid/app/PendingIntent; Landroid/content/Intent; I I Lcom/android/mail/providers/Folder; J)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/effect/effects/TintEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage$7;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/effect/effects/SepiaEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/CommandsInterface;-deleteSmsOnSim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/BookmarkContainer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/inputmethodservice/KeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/MailboxSettings;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/FolderSpanTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorViewPager;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/BluetoothHeadset;-getAudioState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/SecureConversationViewFragment;-renderMessage-(Lcom/android/mail/browse/MessageCursor;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/music/MediaPlaybackActivity;-access$600-(Lcom/android/music/MediaPlaybackActivity;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-setFacilityLockForApp-(Ljava/lang/String; Z Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasSyncService;-validateAccount-(Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/ProfileService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-onKey-(I [I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$Receiver;-clearPendingBroadcastsLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/TimelineRelativeLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Spinner;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech;-getDefaultEngine-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Service;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/TextTime;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub;-requestNetworkTransitionWakelock-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UI;-onBackKey-()Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/WifiManager;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/music/PlaylistBrowserActivity;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/PhotoView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/alarmclock/DigitalWidgetViewsFactory;-getViewAt-(I)Landroid/widget/RemoteViews;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/AttendeesView;-constructAttendeeView-(Lcom/android/calendar/event/EditEventHelper$AttendeeItem;)Landroid/view/View;' : ['android.permission.READ_CONTACTS'],
	'Landroid/net/wifi/WifiStateMachine;-access$23400-(Landroid/net/wifi/WifiStateMachine; Ljava/util/ArrayList;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/sip/SipAudioCall;-transferToNewSession-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothA2dp;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/NullFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/speechrecorder/SpeechRecorderActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/sip/ISipService;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/coremedia/iso/IsoFile;-<init>-(Ljava/io/File;)V' : ['android.permission.INTERNET'],
	'Lcom/android/bluetooth/hid/HidService;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/TouchInterceptor$RemoveListener;-remove-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/PlaylistBrowserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothMap$Stub;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/EasCertificateRequestor;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/FragmentBreadCrumbs;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/ReplayerService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager$Stub;-getPrintJobInfos-(I I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/UriSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-handleSmsMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-fetchRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/app/ResolverActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email2/ui/MailActivityEmail;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/ViewMode;-enterConversationListMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hid/HidService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/widget/LockPatternUtils;-usingBiometricWeak-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardEntry$PhotoData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImagePoint;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/PhoneProxy;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-getRemoteType-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/content/SyncManager;-scheduleLocalSync-(Landroid/accounts/Account; I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/ConnectivityManager;-setMobileDataEnabled-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/printservice/PrintService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/deskclock/timer/CountingTimerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu;-destroy-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-downloadFinished-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-writeCharacteristic-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/google/common/io/MultiReader;-advance-()V' : ['android.permission.INTERNET'],
	'Landroid/media/IAudioService;-adjustStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/view/TextureView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/NotificationController;-access$1000-(Landroid/content/Context; J)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/ConversationListView$HintText;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-switchToDefaultInboxOrChangeAccount-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-getLastTetherError-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/SyncAdapterDriver;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/VolumePanel;-setStreamVolume-(I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue;-releaseAllPointers-(J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/wifi/WifiController$StaEnabledState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/widget/AnalogClock;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$SoftApStartedState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-access$100-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/ConnectivityService;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothMap;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/videoeditor/widgets/EditorRelativeLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/WiredAccessoryManager;-notifyWiredAccessoryChanged-(J I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-sendUSSD-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/ExtractButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IBackupAgent$Stub;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/widget/WaveView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-setWakeMode-(Landroid/content/Context; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/LocationFudger$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/ProcessInfo;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-make-(Lcom/android/internal/telephony/cat/BerTlv;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/ConversationItemView;-toggleSelectedState-(Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setProfileAutoConnectionPriority-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/CheckedTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/time/RadialPickerLayout;-onTouch-(Landroid/view/View; Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/support/v4/content/AsyncTaskLoader;-loadInBackground-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment;-onMessageCursorLoadFinished-(Landroid/content/Loader; Lcom/android/mail/browse/MessageCursor; Lcom/android/mail/browse/MessageCursor;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/packageinstaller/UninstallAppProgress;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email2/ui/MailboxSelectionActivityEmail;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/AccountItemView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasSyncService;-postAutodiscover-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getIMSI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email2/ui/MailActivityEmail;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/widgets/HandleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/LatinIME;-addWordToUserDictionary-(Ljava/lang/String;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/browser/AddBookmarkPage;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/setup/MailboxSettings$LoadMailboxTask;-callDoInBackgroundForTest-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/latin/settings/SettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/tts/TtsEngines;-getV1Locale-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageGrad;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AlarmTimelineView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-cancelPostDial-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-open-([J I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothInputDevice;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/AppSecurityPermissions$PermissionItemView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/QsbApplicationWrapper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasPing;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/support/v4/view/ViewPager;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothGattCallback$Stub$Proxy;-onDescriptorRead-(Ljava/lang/String; I I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/providers/AccountObserver;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/accounts/AccountManagerService;-checkReadAccountsPermission-()V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/media/SubtitleTrack;-hide-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/content/Context;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt;-refreshDevice-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-changeIccPin2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationListView$HintText;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/provider/Settings$System;-getInt-(Landroid/content/ContentResolver; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/ProfileService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/photo/adapters/BaseCursorPagerAdapter;-setPrimaryItem-(Landroid/view/ViewGroup; I Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/os/IPowerManager;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-bluetoothStateChangeHandler-(I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/common/contacts/BaseEmailAddressAdapter$DirectoryPartitionFilter;-performFiltering-(Ljava/lang/CharSequence;)Landroid/widget/Filter$FilterResults;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/IPowerManager$Stub$Proxy;-acquireWakeLockWithUid-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ChooserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine$ConnectModeState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/email/activity/setup/AccountSetupIncoming;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-disable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/MediaPlaybackActivity;-startPlayback-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/IWifiManager$Stub;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/gallery3d/app/MovieActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/FilmstripPage;-showDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/vcard/VCardEntry;-iterateAllData-(Lcom/android/vcard/VCardEntry$EntryElementIterator;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onActionBarWanted-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/google/android/mms/util/AbstractCache;-put-(Ljava/lang/Object; Ljava/lang/Object;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/widget/RatingBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/InLineSettingMenu;-changeIndex-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/ui/CountdownTimerPopup;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoTable;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterApp;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/content/SyncActivityTooManyDeletes;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-loadEfFilesFromUsim-()Ljava/util/ArrayList;' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-getStringForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/FolderItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/MailboxSettings;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/category/CategoryTrack;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms;-getAllMessagesFromIccEf-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/BackupManagerService$PerformFullRestoreTask;-restoreOneFile-(Ljava/io/InputStream; [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/browse/ConversationPager;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/CompoundButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationSyncDisabledTipView;-getShouldDisplayInList-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageRedEye;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/soundrecorder/Recorder;-signalStateChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/media/WebVttRenderingWidget$SpanLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/WidgetClickHandler;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/TimerRingService;-play-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/MediaItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/AlertActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/SuperCollapsedBlock;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/WidgetTypeChooser;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/AppHwPref;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/PageDialogsHandler;-access$700-(Lcom/android/browser/PageDialogsHandler; Lcom/android/browser/Tab;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PickerActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/app/CameraApp;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/EasAutoDiscover;-sendHttpClientPost-(Ljava/lang/String; [B J)Lcom/android/exchange/EasResponse;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/InputMethodManagerService$InputMethodAndSubtypeListManager;-getSortedInputMethodAndSubtypeList-(Z Z Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/monkey/Monkey;-run-([Ljava/lang/String;)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/LocationManagerService;-handleLocationChanged-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/UninstallerActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calculator2/CalculatorViewPager;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/providers/telephony/SmsProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/email/service/AuthenticatorService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-getFolderChange-(Ljava/util/Collection; Ljava/util/Collection; Z Z Z Z Lcom/android/mail/providers/Folder;)Lcom/android/mail/ui/DestructiveAction;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/music/IMediaPlaybackService;-removeTrack-(J)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AbstractCalendarActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/nfc/cardemulation/HostApduService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-handleUp-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/AliasActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/ConfigurationViewer;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageCrop;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-stopPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ActionBarView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub;-untether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/ConversationsInOutboxTipView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/MediaSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Browser;-getVisitedHistory-(Landroid/content/ContentResolver;)[Ljava/lang/String;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/AllInOneActivity$2;-run-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/EasSyncService;-sendHttpClientPost-(Ljava/lang/String; [B)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/InLineSettingCheckBox;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/EmailProvider;-restartPush-(Landroid/content/Context; Landroid/database/sqlite/SQLiteDatabase; Ljava/lang/String;)Z' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/bluetooth/IBluetoothGatt;-startScan-(I Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/app/CameraApp;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$AudioOn;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/mail/browse/ConversationPager;-executeKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiManager$WifiLock;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmKlaxon;-startAlarm-(Landroid/content/Context; Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/MoreSettingPopup;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onVoiceCallEnded-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService;-dispatchUnhandledKey-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/AbstractActivityController;-onAnimationEnd-(Lcom/android/mail/ui/AnimatedAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/RadioGroup;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothPan;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/CalendarSettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/ViewFlipper;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/Uploader;-hasWifiConnection-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/DigitalClock;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/accessibility/AccessibilityEntityProvider;-getKeyDescription-(Lcom/android/inputmethod/keyboard/Key;)Ljava/lang/String;' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TabWidget;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/BadBehaviorActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onTextInput-(Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/pan/PanService;-enableTethering-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/NsdService$NsdStateMachine;-<init>-(Lcom/android/server/NsdService; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/NativeActivity$NativeContentView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/noisefield/NoiseFieldWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/pinyin/SkbContainer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/AddBookmarkPage$CustomListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasAccountService;-sendMeetingResponse-(Lcom/android/exchange/MeetingResponseRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/editors/SwapButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/ImapAuthenticatorService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/ConnectivityManager;-tether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/BadBehaviorActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calculator2/ColorButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-showNextConversation-(Ljava/util/Collection;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/InboundSmsHandler$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/PieController$1;-onClick-(Lcom/android/camera/ui/PieItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/DayFragment;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/datetimepicker/time/AmPmCirclesView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BackupManagerService$RunBackupReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/wm/Wm;-run-([Ljava/lang/String;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.RESTART_PACKAGES'],
	'Lcom/android/development/ShowActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/utils/NotificationUtils;-getContactIcon-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; Lcom/android/mail/providers/Folder;)Landroid/graphics/Bitmap;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/email/activity/setup/AccountSetupOutgoing;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/PlaylistBrowserActivity;-playPodcasts-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/ViewOverlay$OverlayViewGroup;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/MediaSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/GLView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexClientSession$ClientThread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage;-notifyScreenNailChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/tts/TextToSpeechService;-access$400-(Landroid/speech/tts/TextToSpeechService;)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-scanPackageLI-(Landroid/content/pm/PackageParser$Package; I I J Landroid/os/UserHandle;)Landroid/content/pm/PackageParser$Package;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getRemoteName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/utility/CalendarUtilities;-createCalendar-(Landroid/content/Context; Landroid/content/ContentResolver; Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Mailbox;)J' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/UiModeManagerService;-access$100-(Lcom/android/server/UiModeManagerService; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/vis5/Visualization5;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-handleAccessPermissionResult-(Landroid/content/Intent;)V' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/location/LocationBlacklist;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$Listener;-onFling-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/os/BatteryStatsImpl;-setBatteryState-(I I I I I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/SubtitleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangup-(Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/widget/SearchViewCompatIcs$MySearchView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ControllableActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/documentsui/SettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/SnailAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/ILockSettings;-getLong-(Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-handleCdmaSubscriptionSource-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/wifi/IWifiManager;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub;-getCurrentSpellCheckerSubtype-(Ljava/lang/String; Z)Landroid/view/textservice/SpellCheckerSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity$1;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/PieControl;-buildTabs-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/BluetoothA2dp;-isAvrcpAbsoluteVolumeSupported-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/TrackBrowserActivity$TrackListAdapter;-runQueryOnBackgroundThread-(Ljava/lang/CharSequence;)Landroid/database/Cursor;' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/timer/TimerListItem;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/bluetooth/IBluetooth;-createBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothA2dp;-adjustAvrcpAbsoluteVolume-(I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManager;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/accounts/ChooseAccountTypeActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/base/RetargetFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/browser/PhoneUi;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/search/SearchManagerService$MyPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/UninstallerActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/adapter/CalendarSyncParser;-responsesParser-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/eas/EasFolderSync;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/am/UsageStatsService$2;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-stopPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-access$200-(Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;)V' : ['android.permission.CAMERA'],
	'Lcom/android/mail/browse/EmlTempFileDeletionService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/inputmethodservice/InputMethodService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/backup/LocalTransportService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService;-clearAllScoClients-(I Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothHeadset;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onUpEvent-(I I J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/InboundSmsHandler$StartupState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-pollState-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-getStatistics-()[B' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/ManageCachePage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;-onCreateOptionsMenu-(Landroid/view/Menu; Landroid/view/MenuInflater;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/Controller;-openTabToHomePage-()Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/sip/SipWakeLock;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calculator2/CalculatorViewPager;-setCurrentItemInternal-(I Z Z I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accounts/ChooseAccountActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/EmailServiceUtils;-setRemoteServicesLogging-(Landroid/content/Context; I)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/BluetoothAdapter;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/wifi/WifiController$FullHighPerfLockHeldState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/view/IWindowManager$Stub;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/accounts/IAccountManager;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/development/ConfigurationViewer;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-sendMessageRequest-(Lcom/android/emailsync/Request;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/FilterGraph;-tearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/widget/TextClock;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TelephonyRegistry;-broadcastSignalStrengthChanged-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/musicvis/vis4/Visualization4;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/AlarmTimelineView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/SmsBroadcastUndelivered;-scanRawTable-()V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/ExchangeService;-reconcileAccounts-(Landroid/content/Context;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/email/service/AuthenticatorService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/PhotoView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/TabScrollView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/AccessibleDateAnimator;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-showAndStartWelcomeVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/WidgetTypeChooser;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/textservice/SpellCheckerService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-requestRouteToHost-(I I Ljava/lang/String;)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-reevaluateStatusBarVisibility-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasAccountService;-runPingLoop-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-onUserContextChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/crop/CropView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/ComboViewActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/quicksearchbox/SearchActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calculator2/PanelSwitcher;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintJobConfigActivity$CustomEditText;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/EventViewer;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pEnabledState;-exit-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiService;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/cardemulation/OffHostApduService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/emailcommon/service/IEmailServiceCallback$Stub;-loadAttachmentStatus-(J J I I)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/BluetoothHealth;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/DataManager$NotifyBroker;-onChange-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/Browser;-truncateHistory-(Landroid/content/ContentResolver;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-getEnabledInputMethodList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/UrlInputView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/DockObserver;-access$000-(Lcom/android/server/DockObserver;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/app/Service;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/MediaSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-onBackPressed-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/CheckedLinearLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/MenuExecutor$MediaOperation;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothHealth;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/service/EasService$1;-updateFolderList-(J)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/development/DataList;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/WallpaperManagerService;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/server/pm/PackageManagerService$7;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/email/service/EmailServiceStub;-loadAttachment-(Lcom/android/emailcommon/service/IEmailServiceCallback; J J Z)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/NsdService$NsdStateMachine$DisabledState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/calendar/ExpandableTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/PieStackView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/CaffeinatedScrollView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-sendMeetingResponse-(J I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/OnePaneController;-refreshConversationList-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/content/IContentService;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/widget/DialerFilter;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/CaptivePortalTracker;-sendNetworkConditionsBroadcast-(Z Z J J)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getNeighboringCids-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoTable$LoadNaturalSiblingTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/PlaylistBrowserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/EventViewer;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Landroid/view/IWindowSession;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/development/DataList;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/support/v7/widget/GridLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaListView;-initView-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/agenda/AgendaListView$3;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/google/common/io/Files;-copy-(Ljava/io/File; Ljava/nio/charset/Charset; Lcom/google/common/io/OutputSupplier;)V' : ['android.permission.INTERNET'],
	'Landroid/telephony/TelephonyManager;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/print/IPrintManager;-startPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/EffectContext;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$ErrorState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ServiceWatcher$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-forceStopUserLocked-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/content/ISyncAdapter$Stub;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/email/provider/RefreshStatusMonitor;-access$300-(Lcom/android/email/provider/RefreshStatusMonitor;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/DayView;-access$4600-(Lcom/android/calendar/DayView; Landroid/view/MotionEvent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onRecordsLoaded-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/FilmstripPage;-access$2400-(Lcom/android/gallery3d/app/PhotoPage; Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/ScreensaverActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/DatePickerController;-onYearSelected-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/OverlayTitleEditor;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DeleteEventHelper$5;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/email/activity/setup/AccountSetupNames$3;-doInBackground-([Ljava/lang/Void;)Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/svox/pico/Pico;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/WapPushOverSms;-dispatchWapPdu-([B Landroid/content/BroadcastReceiver; Lcom/android/internal/telephony/InboundSmsHandler;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CellBroadcastRangeManager;-updateRanges-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UI;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/content/ISyncAdapter$Stub$Proxy;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-getActiveNetworkInfoUnfiltered-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/uicc/AdnRecordCache;-updateAdnByIndex-(I Lcom/android/internal/telephony/uicc/AdnRecord; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/CantAddAccountActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPickerActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-getEnabledInputMethodSubtypeList-(Ljava/lang/String; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/view/WindowManagerPolicy$WindowManagerFuncs;-shutdown-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-setRecipientName-(Lcom/android/bluetooth/map/BluetoothMapMessageListingElement; Landroid/database/Cursor; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo; Lcom/android/bluetooth/map/BluetoothMapAppParams;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/compose/AutoSendActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/LatinIME;-onCustomRequest-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Controller;-onDownloadStart-(Lcom/android/browser/Tab; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/location/LocationBasedCountryDetector$1;-onLocationChanged-(Landroid/location/Location;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneFallbackEventHandler;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManager$3;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mail/ui/AbstractActivityController;-assignFolder-(Ljava/util/Collection; Ljava/util/Collection; Z Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-queueNextSignalStrengthPoll-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/gallery3d/app/GalleryActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phasebeam/PhaseBeam;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/compose/AttachmentComposeView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt;-endReliableWrite-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/InstrumentationList;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/CalendarUtils$TimeZoneUtils;-getTimeZone-(Landroid/content/Context; Ljava/lang/Runnable;)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mail/ui/TwoPaneController;-handleDropInStarred-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/Tab;-<init>-(Lcom/android/browser/WebViewController; Landroid/webkit/WebView; Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/ScrollerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/ObjectSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$LabelView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/state/StatePanelTrack;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IIntentReceiver$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompatKitKat$AccessibilityNodeInfoBridge;-createAccessibilityNodeInfo-(I)Ljava/lang/Object;' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/music/VideoBrowserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/CalendarContract$CalendarEntity$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/AlarmManager;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ActionBarContextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/UninstallerActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/alarmclock/DigitalAppWidgetService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/TrackBrowserActivity$NowPlayingCursor;-makeNowPlayingCursor-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/Presentation$1;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/development/BadBehaviorActivity$BadService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AsyncPlayer$Thread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readTouchExplorationEnabledSettingLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-access$600-(Lcom/android/bluetooth/map/BluetoothMapService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LockSettingsService$DatabaseHelper;-onUpgrade-(Landroid/database/sqlite/SQLiteDatabase; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/CalendarColorSquare;-assignContactFromEmail-(Ljava/lang/String; Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/browse/ConversationWebView;-onScrollChanged-(I I I I)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/InputMethodManagerService$3;-onUserSwitching-(I Landroid/os/IRemoteCallback;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processUnknownAt-(Ljava/lang/String;)V' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/opp/BluetoothOppLiveFolder;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/AddBookmarkPage$CustomListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/FillLightEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncStorageEngine;-<init>-(Landroid/content/Context; Ljava/io/File;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallbackProxy$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothAdapter$GattCallbackWrapper;-stopLeScan-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/StateManager;-finishState-(Lcom/android/gallery3d/app/ActivityState; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/IBluetoothMap$Stub;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/CropFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/SwipeableListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActiveServices;-bringDownServiceLocked-(Lcom/android/server/am/ServiceRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/vcard/VCardComposer;-init-()Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/music/ScanningProgress;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-matchOriginatorSms-(Landroid/database/Cursor; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo; Ljava/lang/String;)Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/EasSyncService;-acknowledgeProvision-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IntentSender$FinishedDispatcher;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/RIL;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-handleRuimRefresh-(Lcom/android/internal/telephony/uicc/IccRefreshResponse;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/ScreensaverActivity$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-handleServiceCategoryProgramData-(Lcom/android/internal/telephony/cdma/SmsMessage;)Z' : ['android.permission.RECEIVE_SMS'],
	'Landroid/bluetooth/BluetoothInputDevice;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/text/ToUpperCase;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/SpamWarningView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventHelper;-saveEvent-(Lcom/android/calendar/CalendarEventModel; Lcom/android/calendar/CalendarEventModel; I)Z' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/filtershow/pipeline/ProcessingService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasMoveItems;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-init-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/hdp/HealthService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/SeekBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/CaptivePortalTracker;-isActiveNetwork-(Landroid/net/NetworkInfo;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/RotateFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/AsyncService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/datetimepicker/date/TextViewWithCircularIndicator;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/EmailIntentService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/Screensaver;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/os/PowerManager$WakeLock$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/research/ReplayerService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/ViewSwitcher;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/AbstractSyncAdapterService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/development/AppHwConfigList;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/crop/CropActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-sendMmsMessage-(Ljava/lang/String; Ljava/lang/String; Lcom/android/bluetooth/map/BluetoothMapbMessageMmsEmail;)J' : ['android.permission.READ_SMS'],
	'Lcom/android/exchange/adapter/AbstractSyncParser;-commit-()V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothInputDevice;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/support/v7/widget/GridLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/StackView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/alarmclock/DigitalAppWidgetService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/ConversationPager;-setCurrentItemInternal-(I Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/view/IWindowSession$Stub$Proxy;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-changeIccFdnPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationContainer;-onGeometryChange-([Lcom/android/mail/browse/ConversationContainer$OverlayPosition;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/am/ActivityStackSupervisor;-pauseBackStacks-(Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/browser/WallpaperHandler;-openStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/mail/ui/TwoPaneLayout;-onTransitionComplete-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/Development;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasOptions;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/media/RingtoneManager;-getRingtone-(Landroid/content/Context; Landroid/net/Uri;)Landroid/media/Ringtone;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-performTransitions-(Lcom/android/internal/util/State; Landroid/os/Message;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/TrackBrowserActivity$NowPlayingCursor;-<init>-(Lcom/android/music/TrackBrowserActivity; Lcom/android/music/IMediaPlaybackService; [Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MusicUtils;-togglePartyShuffle-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Browser;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-attachApplicationLocked-(Landroid/app/IApplicationThread; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/IAudioService$Stub;-adjustStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Settings$System;-getConfiguration-(Landroid/content/ContentResolver; Landroid/content/res/Configuration;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/WidgetConfigure;-setChoosenAlbum-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/RecognitionManagerService;-access$000-(Lcom/android/server/RecognitionManagerService; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/LegacyEmailAuthenticatorService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/BluetoothManagerService;-handleDisable-()V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmCall;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/Pop3AuthenticatorService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/AbsSpinner;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowSession$Stub$Proxy;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/PlatLogoActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/ScrollView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageStraighten;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/fall/FallView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-getSubscriptionInfoAndStartPollingThreads-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$2900-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaSetsFromString-(Ljava/lang/String;)[Lcom/android/gallery3d/data/MediaSet;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/SettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/ComboAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/NumberPicker;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/AbsActionBarView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/musicvis/vis4/Visualization4;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/TwoPaneLayout;-dispatchConversationListVisibilityChange-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/camera/ui/RotateLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmTimelineView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/monkey/Monkey;-main-([Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavScreen;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/SharpenEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/musicvis/vis1/Visualization1;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/RemoteViewsService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodAndSubtypeHashCodeListLocked-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/widget/Button;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/LocalSource;-naturalPrevious-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/CalendarApplication;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/CompoundButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SmsApplication;-getApplication-(Landroid/content/Context; Z)Lcom/android/internal/telephony/SmsApplication$SmsApplicationData;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/RedEyeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofencePause-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TimePicker;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/GrantActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine$DriverStartedState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/ZoomControl;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/net/ConnectivityManagerCompat$ConnectivityManagerCompatImpl;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/videoeditor/widgets/HandleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/PackageManagerBackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationUpdater;-updateConversation-(Ljava/util/Collection; Landroid/content/ContentValues;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/Button;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarController$EventHandler;-eventsChanged-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mail/ui/FolderSelectionActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/WebVttRenderingWidget;-manageChangeListener-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/inputmethod/latin/settings/SeekBarDialogPreference$ValueProxy;-feedbackValue-(I)V' : ['android.permission.VIBRATE'],
	'Landroid/speech/tts/ITextToSpeechService$Stub;-getClientDefaultLanguage-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/DismissAlarmsService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/RingtoneManager;-getRingtone-(Landroid/content/Context; Landroid/net/Uri; I)Landroid/media/Ringtone;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageHeaderView;-refresh-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/ingest/ui/IngestGridView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/homepages/HomeProvider;-openFile-(Landroid/net/Uri; Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/widget/DateTimeView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/SystemServer;-main-([Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.VIBRATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/widget/RadioButton;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/ime/Ime;-runList-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterApp;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/RotateLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/PrintDialogFrame;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/ui/InLineSettingCheckBox;-changeIndex-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/PointerLocation;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/MoreSettingPopup;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/support/v13/app/FragmentTabHost;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/documentsui/DocumentsApplication;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/AbsActionBarView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSettings;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/ConversationLongPressTipView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/GeneralPreferences;-onTimeZoneSet-(Lcom/android/timezonepicker/TimeZoneInfo;)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/alarmclock/DigitalAppWidgetService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/PointerTracker$TimerProxy;-startUpdateBatchInputTimer-(Lcom/android/inputmethod/keyboard/PointerTracker;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/test/mock/MockContentResolver;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/calendar/DayView;-handleOnResume-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/widget/WidgetService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/backup/FullBackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/DayPickerView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onError-(Lcom/android/internal/telephony/Connection$DisconnectCause;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/musicvis/vis5/Visualization5;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/TrimVideo;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ContextThemeWrapper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/svox/pico/GetSampleText;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/ScrollingTabContainerView$TabView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-closeCurrentTab-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/os/PowerManager$WakeLock;-release-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab;-setController-(Lcom/android/browser/WebViewController;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/IAudioService$Stub;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/widget/DatePicker;-init-(I I I Landroid/widget/DatePicker$OnDateChangedListener;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/MediaFocusControl;-restoreMediaButtonReceiver-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/MultiAutoCompleteTextView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterState$OnState;-enter-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollKeyboardView;-onSingleTapUp-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Landroid/print/PrintJob;-restart-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ProxyLauncher;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/info/HistogramView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/FaceUnlockView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView$OnTriggerListener;-onTrigger-(Landroid/view/View; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/email2/ui/MailActivityEmail;-onRestoreInstanceState-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/musicvis/vis1/Visualization1;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/NoSearchActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/PassiveProvider;-updateLocation-(Landroid/location/Location;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/widget/BookmarkWidgetConfigure;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/EventViewer;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/ShowActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-showBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AbstractActivityController;-changeFolder-(Lcom/android/mail/providers/Folder; Ljava/lang/String; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/svox/pico/GetSampleText;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/PackageBrowser;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/AlertService;-rescheduleMissedAlarms-(Landroid/content/ContentResolver; Landroid/content/Context; Lcom/android/calendar/alerts/AlarmManagerInterface;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/music/AlbumBrowserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/TabScrollView$TabLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkStatsService$Stub$Proxy;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/TimePicker;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email2/ui/MailActivityEmail;-updateLoggingFlags-(Landroid/content/Context;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/view/IWindowManager$Stub;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/exchange/eas/EasOperation;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService$12;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PackagesMonitor$AsyncService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/FolderItemView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapObexServer;-sendMessageListingRsp-(Ljavax/obex/Operation; Lcom/android/bluetooth/map/BluetoothMapAppParams; Ljava/lang/String;)I' : ['android.permission.READ_CONTACTS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Editor$SelectionEndHandleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-exitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-addAccount-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/wallpaper/fall/FallWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/GalleryActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/AndroidClockTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/UserManagerService$5;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/widget/ViewSwitcher;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/FolderItemView;-onDragEvent-(Landroid/view/DragEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/net/NetworkPolicyManagerService$8;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/svox/pico/DownloadVoiceData;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/grass/GrassView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService$CheckMp;-doInBackground-([Lcom/android/server/ConnectivityService$CheckMp$Params;)Ljava/lang/Integer;' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/PhotoPage;-requestDeferredUpdate-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-processCpbrCommand-()I' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/ingest/ui/MtpFullscreenView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageInviteView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/CantAddAccountActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/onetimeinitializer/GalleryWidgetMigrator;-migrateGalleryWidgets-(Landroid/content/Context;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManager;-updateCredentials-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/gallery3d/app/FilmstripPage;-updateUIForCurrentPhoto-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/time/AmPmCirclesView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPickerActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/ExchangeService;-canAutoSync-(Lcom/android/emailcommon/provider/Account;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$AccessibilityContentObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/SyncAdapterDriver;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/printspooler/SelectPrinterActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/AddBookmarkPage;-completeOrCancelFolderNaming-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/email/service/LegacyEmailAuthenticatorService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/PhotoPage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ISms$Stub;-enableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/base/InputStreamSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IEmailServiceCallback$Stub$Proxy;-loadAttachmentStatus-(J J I I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onCallEnded-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaScanner$MyMediaScannerClient;-endFile-(Landroid/media/MediaScanner$FileEntry; Z Z Z Z Z)Landroid/net/Uri;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothInputDevice;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/InputMethodManagerService;-resetSelectedInputMethodAndSubtypeLocked-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService$BootCompletedReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/documentsui/DirectoryContainerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ScrollView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/EmlTempFileDeletionService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/SearchView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/TimeIntervalPopup$Listener;-onListPrefChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/alerts/SnoozeAlarmsService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-onTouch-(Landroid/view/View; Landroid/view/MotionEvent;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/ui/PhotoView;-snapToNeighborImage-()Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CallerInfoAsyncQuery;-startQuery-(I Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/CallerInfoAsyncQuery$OnQueryCompleteListener; Ljava/lang/Object;)Lcom/android/internal/telephony/CallerInfoAsyncQuery;' : ['android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/os/Vibrator;-vibrate-(I Ljava/lang/String; J)V' : ['android.permission.VIBRATE'],
	'Landroid/provider/ContactsContract$Groups$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/month/MonthByWeekFragment$3;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/filterfw/core/Filter;-process-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/WiredAccessoryManager$WiredAccessoryObserver;-updateStateLocked-(Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ViewRootImpl$EarlyPostImeInputStage;-processKeyEvent-(Landroid/view/ViewRootImpl$QueuedInputEvent;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/EmptyConversationListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SlideshowPage$SequentialSource;-getMediaItem-(I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/PhotoPage$9;-onConfirmDialogDismissed-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-shutdown-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/power/ShutdownThread$5;-run-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupWaitingOrBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity;-access$700-(Lcom/android/calendar/AllInOneActivity; J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-assignContactFromEmail-(Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-setCdmaBroadcastConfig-([Lcom/android/internal/telephony/cdma/CdmaSmsBroadcastConfigInfo;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings; Landroid/os/IBinder; Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothGattCallback$Stub$Proxy;-onClientRegistered-(I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-restartRadio-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/widget/DialerFilter;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$UntetheringState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/browser/DataController$DataControllerHandler;-doUpdateVisitedHistory-(Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperPreview;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/SmsApplication;-getDefaultSmsApplication-(Landroid/content/Context; Z)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cat/IconLoader;-readClut-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/DataManager;-getContentUri-(Lcom/android/gallery3d/data/Path;)Landroid/net/Uri;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub$Proxy;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/FolderSpanTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-addSubtypeToHistory-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/Wallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/documentsui/DirectoryContainerView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/CandidatesContainer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/PieMenu;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE', 'com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-hangupForegroundResumeBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-getRemoteName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Browser;-deleteFromHistory-(Landroid/content/ContentResolver; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/content/IContentService;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/Application;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleCancel-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processKeyPressed-()V' : ['android.permission.READ_CALL_LOG'],
	'Lcom/android/gallery3d/app/AlbumPage;-onUpPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/tts/TextToSpeech$2;-run-(Landroid/speech/tts/ITextToSpeechService;)Ljava/lang/Integer;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/PieMenu;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipAudioCall$1;-onCallBusy-(Landroid/net/sip/SipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/AlertController$RecycleListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/AllInOneActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/CalendarContract$EventsEntity$EntityIteratorImpl;-getEntityAndIncrementCursor-(Landroid/database/Cursor;)Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/wifi/WifiService;-stopBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/support/v4/view/PagerTitleStrip;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendP2pPersistentGroupsChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/service/AbstractSyncAdapterService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-checkIfSubtypeBelongsToImeAndEnabled-(Landroid/view/inputmethod/InputMethodInfo; Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/CameraPreference$OnPreferenceChangedListener;-onSharedPreferenceChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getMobileDataEnabled-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/CellBroadcastHandler;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AnimatedAdapter;-fadeOutSpecificLeaveBehindItem-(J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/vcard/VCardParserImpl_V21;-parseItem-()Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-albumShouldOpenInFilmstrip-(Lcom/android/gallery3d/data/MediaSet;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AsyncPlayer;-play-(Landroid/content/Context; Landroid/net/Uri; Z I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/settings/DebugSettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Tab$3;-onRequestFocus-(Landroid/webkit/WebView;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Settings$System;-getIntForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$5;-doWork-()V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-<init>-(Landroid/os/Handler; Landroid/content/Context;)V' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/development/CacheAbuser;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine;-access$7900-(Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/documentsui/TestActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/OnePaneController;-transitionToInbox-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-inputDispatchingTimedOut-(I Z Ljava/lang/String;)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/AbstractActivityController;-updateAccounts-(Lcom/android/mail/content/ObjectCursor;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/Chronometer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallEnded-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/TrimVideo$2;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/SpeechRecognizer;-access$000-(Landroid/speech/SpeechRecognizer; Landroid/content/Intent;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/mail/ui/ConversationsInOutboxTipView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/ContactsContract$SyncState;-newSetOperation-(Landroid/accounts/Account; [B)Landroid/content/ContentProviderOperation;' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/EventInfoFragment;-saveReminders-()Z' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/net/wifi/IWifiManager;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/app/ExpandableListActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/fall/FallWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationContainer;-onLayout-(Z I I I I)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/app/PhotoPage;-onActionBarWanted-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/util/StateMachine$SmHandler$QuittingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/IContentService;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothInputDevice;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/camera/ui/TwoStateImageView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IMagnificationCallbacks$Stub$Proxy;-onUserContextChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/nexus/NexusWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/preference/PreferenceActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/ActionBarView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LockSettingsService;-setLockPassword-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-registerServer-(Landroid/os/ParcelUuid; Landroid/bluetooth/IBluetoothGattServerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextClock;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationPager;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasAccountService;-sync-(Lcom/android/exchange/adapter/AbstractSyncAdapter;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/power/PowerManagerService;-handleBatteryStateChangedLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorEditText;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calculator2/CalculatorViewPager;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Ljava/lang/String; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/datetimepicker/time/TimePickerDialog$2;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mail/ui/OnePaneController;-access$2600-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/providers/Conversation;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollKeyboardView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/speechrecorder/SpeechRecorderActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/ActivityManagerNative;-getTasks-(I I Landroid/app/IThumbnailReceiver;)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-switchUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/RetargetFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/ObjectSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavScreen;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperChange;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/GlobalActions$SinglePressAction;-onLongPress-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TwilightService;-systemReady-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/musicvis/vis2/Visualization2;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameFetch;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ComboViewActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/NavigationBarPhone;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/ContextThemeWrapper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-onConversationVisibilityChanged-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/InputView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageAttachmentTile;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/StickyHeaderListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/IBackupManager$Stub;-opComplete-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/os/PowerManager$WakeLock;-acquire-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/ExtractEditLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/UninstallerActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-initiateInstall-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/HorizontalScrollView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/Device;-getConsistentDeviceId-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/backup/FullBackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/PointerLocation;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/SearchActivityViewSinglePane;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHeadset;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/location/GpsLocationProvider;-disable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/TransitionsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothMap;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/Browser;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowManager;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/gallery3d/app/GalleryActivity;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/AlertActivity$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mail/ui/OnePaneController;-assignFolder-(Ljava/util/Collection; Ljava/util/Collection; Z Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accounts/AccountManager;-getAccounts-()[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/support/v4/content/CursorLoader;-onLoadInBackground-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/content/IContentService$Stub;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/ConversationViewHeader;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/time/RadialPickerLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/SecureCameraActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/agenda/AgendaAdapter;-bindView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/accessibilityservice/AccessibilityService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/musicvis/vis1/Visualization1;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/view/ViewPager;-populate-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/support/v4/app/FragmentTabHost;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/browser/Preloader;-cancelSearchBoxPreload-(Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/preference/PreferenceActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-updatePersistentNetworks-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/DuotoneEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/provider/Contacts$People;-markAsContacted-(Landroid/content/ContentResolver; J)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/ex/photo/PhotoViewPager;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/dreams/phototable/PhotoSource;-naturalPrevious-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/eas/EasFolderSync;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Controller;-closeCurrentTab-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/sip/SipManager;-register-(Landroid/net/sip/SipProfile; I Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/mail/SenderInfoLoader;-loadInBackground-()Lcom/google/common/collect/ImmutableMap;' : ['android.permission.READ_CONTACTS'],
	'Landroid/widget/CheckedTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/MultiStateButton;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setNetworkSelectionModeManual-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/pm/IPackageManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CLEAR_APP_CACHE', 'android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MusicUtils;-addToCurrentPlaylist-(Landroid/content/Context; [J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorSaturationView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/BaseSuggestionView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationListFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-closeService-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeech;-runAction-(Landroid/speech/tts/TextToSpeech$Action; Ljava/lang/Object; Ljava/lang/String; Z Z)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/UrlInputView$UrlInputListener;-onAction-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserActivity;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email2/ui/MailboxSelectionActivityEmail;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/DontPressWithParentLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService$MyExecutor;-executeInternal-(Ljava/lang/Runnable;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-createBond-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/photos/GalleryActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-updatePhoneObject-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-applyNewState-(Lcom/android/internal/telephony/dataconnection/ApnContext; Z Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/SearchActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/DeleteItems;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifyServiceState-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/Ringtone;-play-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/FeedbackActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-addCircularFence-(I I D D D I I I I Landroid/hardware/location/IGeofenceHardwareCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/LeaveBehindItem;-dismiss-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-getShortcutInputMethodsAndSubtypes-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasProvision;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/alarmclock/DigitalWidgetViewsFactory$RemoteWorldClockAdapter;-updateView-(Landroid/view/View; Lcom/android/deskclock/worldclock/CityObj;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/AdnRecordLoader;-loadAllFromEF-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/EllipsizeLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/PrintJob;-isCancelled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/RestoreSession;-getAvailableRestoreSets-(Landroid/app/backup/RestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Contacts$People;-addToGroup-(Landroid/content/ContentResolver; J J)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/hardware/IConsumerIrService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.TRANSMIT_IR'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/development/BadBehaviorActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/VerticalTextSpinner;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/recurrencepicker/LinearLayoutWithMaxWidth;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothDevice;-cancelBondProcess-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/magicsmoke/MagicSmoke;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/StackView;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertAdapter;-bindView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/widget/NumberPicker;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-writeDescriptor-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/datetimepicker/time/RadialPickerLayout;-dispatchPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/Clustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/AppHwConfigList;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/ObjectSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/NumberPicker;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/selectcalendars/SelectCalendarsSyncAdapter;-<init>-(Landroid/content/Context; Landroid/database/Cursor; Landroid/app/FragmentManager;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-startLongPressTimer-(Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/mail/ui/TwoPaneController;-onConversationVisibilityChanged-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/power/ShutdownThread;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/am/ActivityManagerService;-backgroundServicesFinishedLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/event/EditEventActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/Pop3AuthenticatorService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/spellcheck/SpellCheckerSettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/PreloadRequestReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-composeAndSendPhonebookOneVcard-(Ljavax/obex/Operation; I Z Ljava/lang/String; I)I' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/videoeditor/VideoEditorActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/telephony/SmsProvider;-insertInner-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/pinyin/PinyinDecoderService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/SpellChecker;-resetSession-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/InitAlarmsService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/widget/TextTime;-setFormat12Hour-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/polarclock/PolarClockSettings;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/dreams/phototable/LocalSource;-next-(Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/SuggestionsAdapter$SuggestCursor;-runQuery-(Ljava/lang/CharSequence;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/EventLoader;-startBackgroundThread-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-access$1500-(Lcom/android/gallery3d/ingest/IngestActivity;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/DayView;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/inputmethod/latin/BackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/os/BatteryStatsImpl;-writeToParcelWithoutUids-(Landroid/os/Parcel; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/CheckedLinearLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/gsm/SmsManager;-copyMessageToSim-([B [B I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-cancelBondProcess-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/app/ResolverActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/updates/IntentFirewallInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/ColorButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityStackSupervisor;-scheduleSleepTimeout-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/opp/TestActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationViewFragment$MailJsBridge$1;-go-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/UiModeManagerService;-access$500-(Lcom/android/server/UiModeManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/ColorTemperatureEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CdmaBroadcastRangeManager;-updateRanges-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/TextureView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/gatt/GattService;-unregisterClient-(I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-access$400-(Lcom/android/gallery3d/data/LocalAlbumSet; Lcom/android/gallery3d/data/DataManager; I Lcom/android/gallery3d/data/Path; I Ljava/lang/String;)Lcom/android/gallery3d/data/MediaSet;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/GpsLocationProvider;-updateNetworkState-(I Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/EmlViewerActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/emailcommon/provider/Mailbox;-resyncMailbox-(Landroid/content/ContentResolver; Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/media/effect/effects/SepiaEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/setup/SetupActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-access$1800-(Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/AlbumBrowserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserSettings;-onSharedPreferenceChanged-(Landroid/content/SharedPreferences; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/TintFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService$5;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.REORDER_TASKS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.WRITE_SETTINGS', 'android.permission.RESTART_PACKAGES', 'android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/DuotoneEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/SoundRecorder;-access$100-(Lcom/android/soundrecorder/SoundRecorder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/EasAuthenticatorService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/GeolocationPermissionsPrompt;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/research/UploaderService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/IntentService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/search/SearchManagerService$MyPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationListFragment;-commitDestructiveActions-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiManager;-getWifiStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/widget/AutoScrollHelper;-cancelTargetTouch-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/Calculator;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/AbsActionBarView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calculator2/ColorButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/TintFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/contacts/ContactsListActivity;-getContactUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-setKeyboard-(Lcom/android/inputmethod/keyboard/Keyboard;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-removeService-(I I I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/CameraRootView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/CalendarContract$EventDays;-query-(Landroid/content/ContentResolver; I I [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/InputMethodManagerService;-resetStateIfCurrentLocaleChangedLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hdp/HealthService;-doStop-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/browser/DataController;-getInstance-(Landroid/content/Context;)Lcom/android/browser/DataController;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/providers/DrawerClosedObserver;-onDrawerClosed-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-onRefresh-(Z [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-render-(Lcom/android/gallery3d/glrenderer/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/DeleteEventHelper;-access$900-(Lcom/android/calendar/DeleteEventHelper;)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/text/format/DateUtils;-getRelativeDateTimeString-(Landroid/content/Context; J J J I)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-upgradeLegacyBackgroundData-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-requestUpdatingShiftState-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/alerts/DismissAlarmsService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/backup/BackupAgentHelper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onDataSetupCompleteError-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/filtershow/category/CategorySelected;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/EmailProvider;-restartPushForAccount-(Landroid/content/Context; Landroid/database/sqlite/SQLiteDatabase; Landroid/content/ContentValues; Ljava/lang/String;)Z' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-access$000-(Lcom/android/internal/telephony/RIL; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/content/SyncManager;-doDatabaseCleanup-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;-access$100-(Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/LatinIME;-onFinishInput-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-setPostDialState-(Lcom/android/internal/telephony/Connection$PostDialState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-loadDictionaryAsyncForUri-(Landroid/net/Uri;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/policy/impl/PhoneWindow$DecorView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/DownloadCache$DownloadTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/GrainFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupCreatedState;-exit-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationPager;-onMeasure-(I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/Invert;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationSyncDisabledTipView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/CertBlacklister$BlacklistObserver;-getValue-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/OverlayView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Application;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService;-getShortcutInputMethodsAndSubtypes-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/galaxy/GalaxyWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-markConversationsRead-(Ljava/util/Collection; Z Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-updateShare-(Landroid/database/Cursor; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Landroid/speech/RecognitionService;-access$700-(Landroid/speech/RecognitionService; Landroid/speech/IRecognitionListener;)Z' : ['android.permission.RECORD_AUDIO'],
	'Landroid/support/v4/app/NoSaveStateFrameLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ShortcutNameActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/views/HeaderGridView$FullWidthFixedViewLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-isAllowedByCurrentUserSettingsLocked-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationListView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/SubtypeSwitcher;-onSubtypeChanged-(Landroid/view/inputmethod/InputMethodSubtype;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/PinyinDecoderService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/quicksearchbox/google/GoogleSearch;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/view/CertificateSelector;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub;-connectSocket-(Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/SettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/QueryBrowserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService$1;-onProximityNegative-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/widget/CompoundButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BaseUi;-<init>-(Landroid/app/Activity; Lcom/android/browser/UiController;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/vcard/VCardParser_V30;-parse-(Ljava/io/InputStream; Lcom/android/vcard/VCardInterpreter;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/magicsmoke/MagicSmoke;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/EasAccountService;-acknowledgeProvisionImpl-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String; Z)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage;-showBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-stopObexServerSession-()V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/BlendFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Application;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine;-handleSupplicantConnectionLoss-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/gadget/WidgetConfigure;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-setNetworkPreference-(I)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getProvisioningOrActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiService;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/mail/browse/MailWebView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorViewPager;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/TagClustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$20;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/photos/GalleryActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/CheckBox;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/SelectPrinterFragment;-updateAddPrintersAdapter-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/service/PolicyServiceProxy;-setAccountHoldFlag-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/SearchActivity$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/photos/AlbumFragment;-deleteItemWithPath-(Ljava/lang/Object;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/TabControl;-removeTab-(Lcom/android/browser/Tab;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/widget/EllipsizeLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-dataSetChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompat;-performAction-(I I Landroid/os/Bundle;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/widget/VideoView;-setVideoURI-(Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/RecognitionService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/MediaSource;-process-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ActionableToastBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-getPreferredVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/NotificationActionIntentService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/SyncRunner;-performStep-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/widget/FaceUnlockView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Controller;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/accounts/IAccountManager$Stub;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/power/Notifier;-onGoToSleepFinished-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/commands/bmgr/Bmgr;-doWipe-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperChange;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/TestActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/accessibility/TouchExplorer;-handleMotionEventStateDragging-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Switch;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailcommon/service/PolicyServiceProxy;-<init>-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/uicc/AdnRecordLoader;-loadFromEF-(I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/email/mail/store/ServiceStore;-autoDiscover-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/videoeditor/widgets/HorizontalScrollView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/pacprocessor/PacService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/print/UserState;-readConfigurationLocked-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/CalendarColorSquare;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/FieldPort;-setFieldFrame-(Landroid/filterfw/core/Frame; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/widget/BookmarkWidgetConfigure;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/VideoMenu;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/IWindowManager$Stub$Proxy;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/filterpacks/base/CallbackFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler$DefaultState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/soundrecorder/Recorder;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IIntentReceiver;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/ConnectivityService$CheckMp;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/NavTabScroller$ContentLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/CreateShortcutActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationItemView;-beginDragMode-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/dreams/phototable/PicasaSource;-naturalPrevious-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/eas/EasPing;-requestSyncForSyncList-(Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/development/UnderdevelopedSettings;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/views/SquareImageView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/NullFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/CameraSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/IBluetooth$Stub;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/TwoPaneController;-markConversationsRead-(Ljava/util/Collection; Z Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService;-access$000-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR'],
	'Landroid/net/wifi/WifiManager;-isWifiApEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/calendar/CalendarViewAdapter;-buildWeekDate-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationBlacklist;-reloadBlacklistLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-installSystemProviders-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/ImapService;-processUploadMessage-(Landroid/content/Context; Lcom/android/email/mail/Store; Lcom/android/emailcommon/provider/Mailbox; J)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/IBluetooth$Stub;-createSocketChannel-(I Ljava/lang/String; Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/IConnectivityManager;-getLinkQualityInfo-(I)Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/PackageBrowser;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextSwitcher;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/PhotoPage;-setCurrentPhotoByIntent-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/IWindowManager;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/exchange/ExchangeService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/PhotoPage;-toggleBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ListMenuItemView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$3200-(Lcom/android/gallery3d/ui/PhotoView; I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/IConnectivityManager$Stub;-getNetworkPreference-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/content/SyncActivityTooManyDeletes;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/LegacyEmailAuthenticatorService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/SearchMailActionBarView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/Uploader;-uploadFile-(Ljava/io/File;)V' : ['android.permission.INTERNET'],
	'Lcom/android/exchange/eas/EasPing;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/alerts/AlarmScheduler;-queryNextReminderAndSchedule-(Landroid/database/Cursor; Landroid/content/Context; Landroid/content/ContentResolver; Lcom/android/calendar/alerts/AlarmManagerInterface; I J)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/provider/Telephony$Sms;-getDefaultSmsPackage-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/QuickContactBadge;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onResetDone-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/MediaItemView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmActivity$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/BalloonHint$BalloonView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollKeyboardView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavigationBarTablet;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventFragment;-onDestroy-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine$DriverStoppedState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/deskclock/timer/TimerFragment;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/time/RadialTextsView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/CaptivePortalTracker;-lookupHost-(Ljava/lang/String;)Ljava/net/InetAddress;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/LinearLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/TrackBrowserActivity$NowPlayingCursor;-removeItem-(I)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setTimeFromNITZString-(Ljava/lang/String; J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/mail/ui/ConversationViewFragment;-markUnread-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/location/GeofenceManager;-sendIntent-(Landroid/app/PendingIntent; Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/CalendarBackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onDownEventInternal-(I I J)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/ex/photo/PhotoViewPager;-endFakeDrag-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService$DragInputEventReceiver;-onInputEvent-(Landroid/view/InputEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/EasAuthenticatorServiceAlternate;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/SpeechRecognizer;-handleStartListening-(Landroid/content/Intent;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/net/INetworkStatsService$Stub;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/VideoModule;-onResumeAfterSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-initSocket-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/StateManager;-restoreFromState-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/LatinIME;-handleSeparator-(I I I I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-<init>-(Landroid/preference/VolumePreference; Landroid/content/Context; Landroid/widget/SeekBar; I Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/view/ViewRootImpl$ViewPostImeInputStage;-processKeyEvent-(Landroid/view/ViewRootImpl$QueuedInputEvent;)I' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Landroid/widget/AbsSpinner;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/ime/Ime;-run-([Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pan/PanService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/PointerLocationView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/UiModeManagerService;-updateLocked-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/documentsui/SettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/OverlayTitleEditor;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/AlertService;-generateAlerts-(Landroid/content/Context; Lcom/android/calendar/alerts/NotificationMgr; Lcom/android/calendar/alerts/AlarmManagerInterface; Landroid/content/SharedPreferences; Landroid/database/Cursor; J I)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-serverDisconnect-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/NavigationBarPhone;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/app/ActivityManager;-getRecentTasksForUser-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Landroid/os/IPowerManager;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/StateManager;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getAllNetworkState-()[Landroid/net/NetworkState;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-doStart-(Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/service/EasService$1;-pushModify-(J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/pinyin/ComposingView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/Notifier$1;-onScreenOn-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/widget/CheckBox;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/EasAutoDiscover;-makeUriString-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/documentsui/DirectoryView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountCheckSettingsFragment$AccountCheckTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/LockSettingsService;-checkPassword-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine;-registerForWatchdogToggle-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/ActivityManagerNative;-setProcessLimit-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/location/ILocationManager$Stub$Proxy;-isProviderEnabled-(Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/view/menu/ListMenuItemView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/tts/compat/CompatTtsService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/AlertService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/ex/photo/views/PhotoView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/LogViewer;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSecurity$SecurityNeededDialog;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/research/Uploader;-doUpload-()V' : ['android.permission.INTERNET'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendThisDeviceChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/DeskClock;-registerPageChangedListener-(Lcom/android/deskclock/DeskClockFragment;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioManager;-startBluetoothSco-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/performance/ThroughputFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/selectcalendars/SelectCalendarsSyncFragment;-onPause-()V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readPbrFileAndWait-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/GlobalActions$4;-onItemLongClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/view/IWindowSession$Stub$Proxy;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu;-onDestroyActionMode-(Landroid/view/ActionMode;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/http/RequestQueue$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/KeyguardManager;-exitKeyguardSecurely-(Landroid/app/KeyguardManager$OnKeyguardExitResult;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/RIL;-changeIccPin-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-readRemoteRssi-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/view/StopProgressView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ILockSettings;-getString-(Ljava/lang/String; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth;-disable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCellInfoListRate-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/wm/WindowManagerService;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/RemoteViewsAdapter$RemoteViewsFrameLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setPairingConfirmation-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/downloads/ui/DownloadItem;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/provision/DefaultActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/Wallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActiveServices;-realStartServiceLocked-(Lcom/android/server/am/ServiceRecord; Lcom/android/server/am/ProcessRecord; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/camera/ui/TwoStateImageView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityGroup;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaCall;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService$InputMethodAndSubtypeListManager;-getNextInputMethod-(Z Landroid/view/inputmethod/InputMethodInfo; Landroid/view/inputmethod/InputMethodSubtype;)Lcom/android/server/InputMethodManagerService$ImeSubtypeListItem;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/SwipeableListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-addCharacteristic-(I Landroid/os/ParcelUuid; I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-getRequestUri-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/content/IContentService$Stub;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$StartTetheringErrorState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/mail/browse/FolderSpanTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/EngineSettings;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/PhotoPage;-switchToGrid-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/ReceiverRestrictedContext;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/ContextWrapper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/view/StopProgressView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSecurity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/AuthenticatorService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$4800-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/browser/view/BookmarkContainer;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary$12;-run-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-matchRecipientMms-(Landroid/database/Cursor; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo; Ljava/lang/String;)Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-markMessagesAsRead-(Ljava/util/ArrayList;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calculator2/CalculatorViewPager;-dataSetChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/SlideshowPage$ShuffleSource;-getMediaItem-(I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/compose/ComposeActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/chips/RecipientEditTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowSession$Stub$Proxy;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/content/ContentService;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationUpdater;-starMessage-(Lcom/android/mail/browse/ConversationMessage; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/grass/GrassWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/FrameSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/BummerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$SyncAdapterImpl;-syncMailbox-(Landroid/content/Context; Landroid/content/ContentResolver; Landroid/accounts/Account; Lcom/android/emailcommon/provider/Account; J Landroid/os/Bundle; Landroid/content/SyncResult; Ljava/util/HashSet; Z)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/InputMethodService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/NavScreen;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/sip/SipService$MyExecutor;-execute-(Ljava/lang/Runnable;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/RenderScriptScene;-start-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calculator2/CalculatorDisplay;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-getNetworkInfo-(I)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/browse/ConversationViewHeader;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorBrightnessView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasAccountService;-sync-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-getPersistedDeviceName-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/quicksearchbox/ui/SearchActivityView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothPbap;-disconnect-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ShutdownActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/PreparePageFadeoutTexture;-onGLIdle-(Lcom/android/gallery3d/glrenderer/GLCanvas; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/alerts/AlertActivity;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/input/InputManagerService;-registerPointerSpeedSettingObserver-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/AdnRecordCache;-updateAdnBySearch-(I Lcom/android/internal/telephony/uicc/AdnRecord; Lcom/android/internal/telephony/uicc/AdnRecord; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EasMeetingResponder;-makeUriString-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/updates/ConfigUpdateInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-updateParams-(Ljava/util/HashMap;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mail/MailIntentService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/ConfigurationViewer;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/MonthListView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mail/ui/SwipeHelper$Callback;-onScroll-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/CacheAbuser;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumPage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/OnePaneController;-access$300-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/providers/Conversation; Ljava/util/Set; [B)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-startManualSync-(J I Lcom/android/emailsync/Request;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipManager;-makeAudioCall-(Ljava/lang/String; Ljava/lang/String; Landroid/net/sip/SipAudioCall$Listener; I)Landroid/net/sip/SipAudioCall;' : ['android.permission.USE_SIP'],
	'Landroid/widget/TextClock;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/SearchView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/HandleApiCalls;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/datetimepicker/time/RadialPickerLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/RecentApplicationsBackground;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DayView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mail/ui/AbstractActivityController$5;-onLoadFinished-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/UrlInputView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-addAccount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/VideoView;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/SimpleWeekView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/printspooler/PrintDialogFrame;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipManager;-createSipSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/SipSession$Listener;)Landroid/net/sip/SipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-reloadButtons-()V' : ['android.permission.GET_TASKS'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneFallbackEventHandler;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/LocationManagerService;-access$700-(Lcom/android/server/LocationManagerService; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/net/ConnectivityManagerCompatHoneycombMR2;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-toggleBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/videoeditor/widgets/PlayheadView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkPolicyManager$Stub;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/ui/CountDownView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/VpnService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/telephony/SmsManager;-getAllMessagesFromIcc-()Ljava/util/ArrayList;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/content/ISyncAdapter$Stub;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager;-disable-(Z)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/IContentService$Stub$Proxy;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu$1;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/documentsui/SettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/event/AttendeesView$PresenceQueryHandler$1;-run-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/StatusBarManagerService;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/SecureSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/accessibility/AccessibilityEntityProvider;-performActionForKey-(Lcom/android/inputmethod/keyboard/Key; I Landroid/os/Bundle;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/telephony/TelephonyManager;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/preference/YesNoPreference;-onDialogClosed-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/datetimepicker/time/RadialSelectorView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AbstractCalendarActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email2/ui/CreateShortcutActivityEmail;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-access$200-(Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;)Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/Exchange;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/display/WifiDisplayController;-<init>-(Landroid/content/Context; Landroid/os/Handler; Lcom/android/server/display/WifiDisplayController$Listener;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/email/activity/setup/AccountSetupOutgoingFragment;-setCallback-(Lcom/android/email/activity/setup/AccountServerBaseFragment$Callback;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/gadget/WidgetClickHandler;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/tts/TtsEngines;-updateLocalePrefForEngine-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue$Element;-onPhantomUpEvent-(J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/inputmethodservice/InputMethodService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService$Stub$Proxy;-sync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub$Proxy;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub;-getMobileProvisioningUrl-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub;-pollBatchedScan-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/SuggestionsAdapter;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/WebVttRenderingWidget$CueLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/quicksearchbox/ui/SearchActivityView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine;-startBatchedScan-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/AuthenticatorService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/printservice/PrintService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/ITextToSpeechService;-synthesizeToFileDescriptor-(Landroid/os/IBinder; Ljava/lang/String; Landroid/os/ParcelFileDescriptor; Landroid/os/Bundle;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController;-perhapsEnterWaitMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/googlecode/mp4parser/authoring/adaptivestreaming/FlatPackageWriterImpl;-write-(Lcom/googlecode/mp4parser/authoring/Movie;)V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/ui/DetailsHelper$DetailsViewContainer;-show-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-processMotionEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/exchange/adapter/CalendarSyncParser$CalendarOperations;-updatedExtendedProperty-(Ljava/lang/String; Ljava/lang/String; J)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/FilmstripPage;-toggleBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/OnePaneController;-handleBackPress-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/widgets/EditorRelativeLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Ljava/net/Socket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/emailsync/SyncManager;-stopServiceThreads-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/SupplicantStateTracker;-access$200-(Landroid/net/wifi/SupplicantStateTracker; Landroid/net/wifi/SupplicantState; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/PhotoView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/agenda/AgendaListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/galaxy/GalaxyWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/SpeechRecognizer;-cancel-()V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/calendar/alerts/GlobalDismissManager;-dismissGlobally-(Landroid/content/Context; Ljava/util/List;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/email/EmailIntentService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperChange;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/pinyin/CandidatesContainer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/AccessibleLinearLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ContentListener;-onContentDirty-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;-startRefresh-()V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/data/SecureAlbum;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/ConversationPhotoTeaserView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/TimelineRelativeLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EasTestAuthenticatorService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Pending;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/search/SearchManagerService$MyPackageMonitor;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasAccountService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/SearchActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-access$300-(Lcom/android/internal/widget/multiwaveview/GlowPadView; I F F)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AlarmTimelineView;-init-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/DebugFragment;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/provider/CalendarContract$Instances;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; J J)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Landroid/app/ListActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/TextureView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getDisabledSystemInputMethods-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RelativeLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/GrantActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService$SettingsObserver;-<init>-(Landroid/media/AudioService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mail/browse/ConversationViewAdapter$MessageHeaderItem;-bindView-(Landroid/view/View; Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/development/AppHwConfigList;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-validate-(Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/widget/Switch;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmKlaxon$1;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-beginLayoutLw-(Z I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/IconMenuView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/IConnectivityManager;-isTetheringSupported-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterfw/core/Filter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationViewFragment;-onSuperCollapsedClick-(Lcom/android/mail/browse/ConversationViewAdapter$SuperCollapsedBlockItem;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-handleTimeout-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/IntRangeManager;-enableRange-(I I Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothPan;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-access$2400-(Lcom/android/gallery3d/app/PhotoPage; Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/videoeditor/KenBurnsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/widget/Button;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/ImapService$1;-loadAttachment-(Lcom/android/emailcommon/service/IEmailServiceCallback; J J Z)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/printspooler/FusedPrintersProvider;-onStopLoading-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/SuggestionsAdapter$SuggestFilter;-performFiltering-(Ljava/lang/CharSequence;)Landroid/widget/Filter$FilterResults;' : ['android.permission.ACCESS_NETWORK_STATE', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/widget/LockPatternUtils;-saveLockPassword-(Ljava/lang/String; I Z I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/google/SearchBaseUrlHelper$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/email/activity/setup/AccountSetupBasics;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/MovieActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/PowerManager$WakeLock;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-onFolderSelected-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/Phone;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-switchToState-(I F F)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-setMobileDataEnabled-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-onBootCompleted-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_CALENDAR', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getMute-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/AnimatedListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.INTERNET'],
	'Landroid/net/CaptivePortalTracker;-isCaptivePortal-(Ljava/net/InetAddress;)Z' : ['android.permission.INTERNET', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-onProgressComplete-(I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/ConnectivityManager;-stopUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/google/common/io/Resources;-toString-(Ljava/net/URL; Ljava/nio/charset/Charset;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/AutologinBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/basic/Colors;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtErrorActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getDataRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/server/BluetoothA2dpService;-onBluetoothEnable-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/mail/ui/ViewMode;-enterConversationMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-<init>-(Lcom/android/music/MediaPlaybackService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/TimerSetupView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/SyncAdapterDriver;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/GLView$OnClickListener;-onClick-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setupDataOnConnectableApns-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-disconnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/service/EasService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/WebViewController;-onProgressChanged-(Lcom/android/browser/Tab;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-setViewStates-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/data/ChangeNotifier;-onChange-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserActivity;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/service/EasService$1;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/data/MediaSet;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-setRadios-(Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/mail/browse/BorderView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/ExchangeService;-runAccountReconcilerSync-(Landroid/content/Context;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/ConversationLongPressTipView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/grass/GrassWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage;-transitionFromAlbumPageIfNeeded-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/text/StringSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-update-(Lcom/android/internal/telephony/DriverCall;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/HistoryItem;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothDevice;-getAliasName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readIapFileAndWait-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/setup/LauncherIconVisibilityManager;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/agenda/AgendaByDayAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/server/content/SyncManager$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/location/LocationManager;-getLastLocation-()Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/ScreensaverActivity;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/BaseSuggestionView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/TrackBrowserActivity$5;-remove-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState$SwitchActions;-setAlphabetAutomaticShiftedKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-processResponse-(Landroid/os/Parcel;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-onConversationVisibilityChanged-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/service/EasAutoDiscover;-sendHttpClientPost-(Ljava/lang/String; [B)Lcom/android/exchange/EasResponse;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/GLRootView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Disconnected;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/AccountsTester$ButtonClickListener;-onClick-(Landroid/view/View;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/mail/ui/AbstractActivityController$7;-onLoadFinished-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/TwoLineListItem;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/timer/TimerAlertFullScreen;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-restartRadio-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/polarclock/PolarClockSettings;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/ui/MtpThumbnailTileView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BreadCrumbView;-pop-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/videoeditor/OverlayTitleTemplatePicker;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/GridLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/Connectivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-getCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$4200-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-queryExternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/AttachmentBitmapHolder;-thumbnailLoadFailed-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppIncomingFileConfirmActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/gadget/WidgetClickHandler;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityStackSupervisor;-scheduleIdleTimeoutLocked-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/am/ActivityStackSupervisor$ActivityStackSupervisorHandler;-activityIdleInternal-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/DialogTitle;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/MonthListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/SecureCameraActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/ex/photo/PhotoViewPager;-addNewItem-(I I)Landroid/support/v4/view/ViewPager$ItemInfo;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/WebVttRenderingWidget$CueLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMnsObexClient;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothDevice; Landroid/os/Handler;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/widget/CalendarAppWidgetModel;-buildFromCursor-(Landroid/database/Cursor; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/ChooseAccountTypeActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/Details;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/AdnRecordCache;-requestLoadAllAdnLike-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothHeadset;-isDisabled-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothDevice;-getBondState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hdp/HealthService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/Exchange;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/google/common/io/Files;-toString-(Ljava/io/File; Ljava/nio/charset/Charset;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/camera/ui/RotateImageView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/CantAddAccountActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/backup/IBackupManager;-clearBackupData-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-isNetworkSupported-(I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/widget/LockPatternUtils;-checkPasswordHistory-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/MediaSetSource$CheckedMediaSetSource;-onContentDirty-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory$2;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/LocationManagerService;-systemRunning-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/BackupManagerService$PerformRestoreTask;-restorePmMetadata-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/am/BroadcastQueue;-broadcastTimeoutLocked-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/VerticalTextSpinner;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RemoteViewsAdapter$RemoteViewsFrameLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/IRecognitionService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/camera/ui/TimeIntervalPopup$2;-onClick-(Landroid/view/View;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/google/AbstractGoogleSource;-queryInternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/deskclock/timer/TimerView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/ComposingView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/vis5/Visualization5;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity$3;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-conference-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onFilmModeChanged-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/ChooseAccountTypeActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/AlarmClockFragment$9;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-setAccount-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionaryAddWordFragment;-onResume-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/Controller;-start-(Landroid/content/Intent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/SizeChangeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/base/CallbackFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendMotionEvent-(Landroid/view/MotionEvent; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-killAppAtUsersRequest-(Lcom/android/server/am/ProcessRecord; Landroid/app/Dialog;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/provider/Settings$System;-getIntForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-queryCallForwardStatus-(I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-canUnbindBluetoothService-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/RemoteDevices;-pinRequestCallback-([B [B I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/MonthListView;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getBondedDevices-()[Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/INetworkPolicyManager;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/ui/InLineSettingCheckBox;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/GLRoot;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/DeskClock;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/MessageHeaderView;-showExpandedDetails-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AnimatedAdapter;-changeCursor-(Landroid/database/Cursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine;-handleScreenStateChanged-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/category/CategoryView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationViewFragment$4;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/media/MediaFocusControl;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/documentsui/DocumentsApplication;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/PlayheadView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasOperation;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiStateMachine;-startTethering-(Ljava/util/ArrayList;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/provider/BrowserProvider2;-updateBookmarksInTransaction-(Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String; Z)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/SearchActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkPolicyManagerService$12;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/event/EditEventActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/RepeatingImageButton;-access$000-(Lcom/android/music/RepeatingImageButton; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallback;-onError-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-makeInboundSmsHandler-(Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase;)Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment$LoadAccountTask;-onPostExecute-(Ljava/util/Map;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/widgets/MediaLinearLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageCrop;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSyncCalendar;-cleanup-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/BackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/VideoBrowserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-checkAccessPermission-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/IConnectivityManager$Stub;-getAllNetworkState-()[Landroid/net/NetworkState;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothGatt;-removeService-(I I I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/sip/SipService;-unregisterReceivers-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService;-broadcastConnectionState-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageWebView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/TimeIntervalPopup;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BookmarkSearch;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/util/StateMachine$SmHandler$HaltingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/CommandsInterface;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationOverlayItem;-bindView-(Landroid/view/View; Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/TwoPaneController;-access$1100-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/EasSyncService;-hasConnectivity-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothHeadset;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiManager;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/exchange/EasSyncService;-tryProvision-(Lcom/android/exchange/EasSyncService;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/agenda/AgendaFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/view/SurfaceView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService;-performSync-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/app/TrimVideo;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoSource;-naturalNext-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/inputmethodservice/ExtractButton;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/MediaSaveService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub;-startLegacyVpn-(Lcom/android/internal/net/VpnProfile;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/location/SettingInjectorService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onPressSymbol-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/DisplayPowerController$Callbacks;-onProximityPositive-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/EventViewer;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-queueNextSignalStrengthPoll-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/calendar/EventInfoFragment;-updateEvent-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IBackupManager$Stub$Proxy;-clearBackupData-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/DismissAlarmsService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/ArtistAlbumBrowserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackActivity;-access$1000-(Lcom/android/music/MediaPlaybackActivity; I J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calculator2/CalculatorViewPager;-pageLeft-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/SlideshowView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-makeCellBroadcastHandler-(Landroid/content/Context;)Lcom/android/internal/telephony/CellBroadcastHandler;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setOutgoingCallerIdDisplay-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/magicsmoke/MagicSmoke;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/grass/Grass;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-handleRadioNotAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/WidgetTypeChooser;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/ExchangeService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/EasAccountService;-parsePingResult-(Ljava/io/InputStream; Landroid/content/ContentResolver; Ljava/util/HashMap;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractActivityController$MailDrawerListener;-onDrawerSlide-(Landroid/view/View; F)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/base/GLTextureSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/Files;-copy-(Ljava/io/File; Lcom/google/common/io/OutputSupplier;)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/PhoneWindow$DecorView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EasServerConnection;-makeUserString-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/IAudioService;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/browser/preferences/WebsiteSettingsFragment$SiteAdapter$UpdateFromBookmarksDbTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/sip/SipService;-restartPortMappingLifetimeMeasurement-(Landroid/net/sip/SipProfile; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-updateClustersContents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/DayFragment;-onResume-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getCallHistorySize-(I)I' : ['android.permission.READ_CALL_LOG'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorCompareView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Service;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-access$000-(Landroid/accounts/Account;)Ljava/util/HashSet;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/TwoPaneController;-perhapsShowFirstSearchResult-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/AppHwConfigList;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-setHoldScreenLocked-(Lcom/android/server/wm/Session;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-buildAppWidgetModel-(Landroid/content/Context; Landroid/database/Cursor; Ljava/lang/String;)Lcom/android/calendar/widget/CalendarAppWidgetModel;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/inputmethodservice/KeyboardView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/CreateShortcutActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calculator2/Calculator$PageAdapter;-setPrimaryItem-(Landroid/view/ViewGroup; I Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/BluetoothAdapter;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/AbstractActivityController;-access$1600-(Lcom/android/mail/ui/AbstractActivityController;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setWifiEnabled-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/inputmethod/latin/LatinIME;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/videosrc/MediaSource;-pauseVideo-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/dreams/phototable/PhotoTableDream;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-setLocaleFromCsim-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/magicsmoke/MagicSmoke;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowSession$Stub;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStack;-moveTaskToBackLocked-(I Lcom/android/server/am/ActivityRecord;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Service;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/videoeditor/ProjectsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockApplication;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MusicPicker;-setSelected-(Landroid/database/Cursor;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager;-stopAccountSyncs-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-getWidgetsEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NotificationManagerService$SettingsObserver;-observe-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService$AudioHandler;-onPlaySoundEffect-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/InputMethodManagerService;-showInputMethodSubtypeMenu-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/util/SingleThreadNamedTaskExecutor$Worker;-run-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/TextClock;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-hideWarningView-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/datetimepicker/time/RadialTextsView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-starMessage-(Lcom/android/mail/browse/ConversationMessage; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/ProgressBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PicasaSource;-findImages-(I)Ljava/util/Collection;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/accounts/ChooseAccountActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL$RILReceiver;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-setAlarmState-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/widget/GridView;-commonKey-(I I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/StitchingChangeListener;-onStitchingResult-(Landroid/net/Uri;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/IWindowSession$Stub;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/widget/ActionBarContextView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompatJellyBean$1;-createAccessibilityNodeInfo-(I)Landroid/view/accessibility/AccessibilityNodeInfo;' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCallDeflectionIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/EasAuthenticatorServiceAlternate;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/development/Connectivity;-access$2300-(Lcom/android/development/Connectivity;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationsInOutboxTipView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ViewAnimator;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BluetoothManagerService;-disable-(Z)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/NavScreen;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/exchange/service/EasService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-onCreate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Landroid/widget/DateTimeView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IBackupAgent;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/deskclock/AlarmTimelineView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Ljava/net/URLConnection;-connect-()V' : ['android.permission.INTERNET'],
	'Lcom/android/email/service/EasAuthenticatorService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/AudioService;-adjustVolume-(I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/fall/FallWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/datetimepicker/date/DatePickerController;-onDayOfMonthSelected-(I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothMap;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/service/notification/NotificationListenerService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-onKeyDown-(I I Landroid/view/KeyEvent;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/CalendarView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService;-prev-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPicker;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/keychain/KeyChainActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/provider/BrowserProvider2$DatabaseHelper;-onCreate-(Landroid/database/sqlite/SQLiteDatabase;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/CircleTimerView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/alarmclock/DigitalAppWidgetService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/AlarmClockFragment$9;-doInBackground-([Ljava/lang/Void;)Lcom/android/deskclock/provider/AlarmInstance;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/AccessibilityDelegateCompatJellyBean$1;-performAccessibilityAction-(Landroid/view/View; I Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onReleaseSymbol-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$900-(Lcom/android/server/net/NetworkPolicyManagerService;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothMap;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/keychain/KeyChainActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorValueView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$AccessibilityContentObserver;-<init>-(Lcom/android/server/accessibility/AccessibilityManagerService; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiService;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/documentsui/DirectoryView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSettings;-sendDeviceInformation-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/TabScrollView$TabLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DockObserver;-handleDockStateChange-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/GeolocationPermissionsPrompt;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/FrameLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email2/ui/MailActivityEmail;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/SlotView$Listener;-onSingleTapUp-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/GalleryActionBar$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/pm/PackageManagerService;-scanPackageLI-(Ljava/io/File; I I J Landroid/os/UserHandle;)Landroid/content/pm/PackageParser$Package;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/packageinstaller/TabsAdapter;-restoreState-(Landroid/os/Parcelable; Ljava/lang/ClassLoader;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-getIMEISV-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-processMessagePart-(Lcom/android/internal/telephony/InboundSmsTracker;)Z' : ['android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/Controller;-onConfgurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-setPackageScreenCompatMode-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/ThumbnailLoadTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHeadset;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/renderscript/RSTextureView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/Connectivity;-onRoutedSocketRequest-()V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/provider/Telephony$Mms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Landroid/media/IAudioService$Stub$Proxy;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-vibrate-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/DataManager;-getSupportedOperations-(Lcom/android/gallery3d/data/Path;)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-broadcastStickyIntent-(Landroid/content/Intent; Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/GLTextureSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/AlbumBrowserActivity;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/AllInOneActivity;-updateSecondaryTitleFields-(J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/mail/ui/AccountItemView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameFetch;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/AccessibleTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/nsd/INsdManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Landroid/inputmethodservice/KeyboardView;-openPopupIfRequired-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-queryFacilityLockForApp-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtErrorActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-access$000-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-getMobileDataEnabled-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/LeaveBehindItem;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BatteryService;-access$600-(Lcom/android/server/BatteryService; Landroid/os/BatteryProperties;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/printspooler/FusedPrintersProvider;-access$1000-(Lcom/android/printspooler/FusedPrintersProvider;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationViewHeader$ConversationViewHeaderCallbacks;-onFoldersClicked-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/alerts/AlertService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$StartingState;-enter-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActiveServices;-performServiceRestartLocked-(Lcom/android/server/am/ServiceRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onPressShift-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/MovieActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-process-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/VideoBrowserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/UnderdevelopedSettings;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/accessibility/AccessibilityEntityProvider;-sendAccessibilityEventForKey-(Lcom/android/inputmethod/keyboard/Key; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/google/android/mms/pdu/PduPersister;-updateHeaders-(Landroid/net/Uri; Lcom/google/android/mms/pdu/SendReq;)V' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-shouldOfferSwitchingToNextInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageAttachmentBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hid/HidService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/SecureConversationViewFragment;-onFoldersClicked-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ButteryProgressBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-setUniverseTransformLocked-(Lcom/android/server/wm/WindowState; F F F F F F F)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-requestSetID-(I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/homepages/RequestHandler;-run-()V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/speech/SpeechRecognizer;-startListening-(Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'android.permission.RECORD_AUDIO'],
	'Lcom/android/bluetooth/pan/PanService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/SupplicantStateTracker$CompletedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/EasAuthenticatorService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-onMotionEvent-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/FusedPrintersProvider;-cancelInternal-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onPictureCenter-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/AllInOneActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/EasAuthenticatorServiceAlternate;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/musicvis/vis3/Visualization3;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-onBackPressed-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-clientConnect-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/utility/CalendarUtilities;-createMessageForEventId-(Landroid/content/Context; J I Ljava/lang/String; Lcom/android/emailcommon/provider/Account; Ljava/lang/String;)Lcom/android/emailcommon/provider/EmailContent$Message;' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterfw/core/Filter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub$Proxy;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/WifiController$DeviceActiveState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/videoeditor/OverlayTitleTemplatePicker;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailsync/SyncManager$AccountObserver$2;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/TintFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/commands/pm/Pm;-run-([Ljava/lang/String;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Landroid/net/sip/SipAudioCall;-releaseWifiHighPerfLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AddBookmarkPage;-getUriForFolder-(J)Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState$SwitchActions;-setSymbolsShiftedKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/EmailProvider;-call-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Landroid/os/Bundle;' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/am/CoreSettingsObserver;-beginObserveCoreSettings-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/WebVttRenderingWidget;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/StateManager;-notifyActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-setState-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/am/BroadcastQueue;-skipPendingBroadcastLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothInputDevice;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-scanInternalSubtitleTracks-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getCallWaiting-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/soundrecorder/Recorder;-setState-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/EmlTempFileDeletionService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onVoiceCallEnded-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManagerNative;-startRunning-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStack;-ensureActivitiesVisibleLocked-(Lcom/android/server/am/ActivityRecord; Lcom/android/server/am/ActivityRecord; Ljava/lang/String; I Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/browse/ConversationCursor;-notifyRefreshReady-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/Gallery;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/p2p/WifiP2pManager;-initialize-(Landroid/content/Context; Landroid/os/Looper; Landroid/net/wifi/p2p/WifiP2pManager$ChannelListener;)Landroid/net/wifi/p2p/WifiP2pManager$Channel;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/calendar/event/EventLocationAdapter$LocationFilter;-performFiltering-(Ljava/lang/CharSequence;)Landroid/widget/Filter$FilterResults;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/dreams/basic/Colors;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/CalendarView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/FilterFunction;-executeWithArgList-([Ljava/lang/Object;)Landroid/filterfw/core/Frame;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calculator2/CalculatorViewPager;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/musicvis/vis4/Visualization4;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/QsbApplicationWrapper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/CantAddAccountActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperChange;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IActivityManager;-registerReceiver-(Landroid/app/IApplicationThread; Ljava/lang/String; Landroid/content/IIntentReceiver; Landroid/content/IntentFilter; Ljava/lang/String; I)Landroid/content/Intent;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/os/IPowerManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/SmsApplication;-getDefaultRespondViaMessageApplication-(Landroid/content/Context; Z)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/EmptyService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/ExtractEditText;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getWidgetsEnabled-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleMultipartyIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/CoreSettingsObserver;-sendCoreSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-startCamera-()V' : ['android.permission.CAMERA'],
	'Lcom/android/music/MediaPlaybackActivity$10;-onRepeat-(Landroid/view/View; J I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService;-enforceChangePermission-()V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handlePollCalls-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventFragment$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/AllInOneActivity$4;-onChange-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/app/AlertActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/IPrintManager;-getPrintJobInfo-(Landroid/print/PrintJobId; I I)Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/TabScrollView$TabLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/FaceUnlockView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EmailServiceStub;-deleteAccountPIMData-(Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/effect/effects/SaturateEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/app/FragmentTabHost;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager;-untether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService$8;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsUsageMonitor; Lcom/android/internal/telephony/ImsSMSDispatcher; Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/dreams/phototable/PhotoTable;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceMonitorStatus-(I I Landroid/location/Location; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/ProjectsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/emailsync/SyncManager;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/webkit/WebView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifyDataConnectionFailed-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-access$2000-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavTabView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/ArtistAlbumBrowserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DialerFilter;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/AccountsChangedReceiver$DeleteRemovedAccounts;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/widget/LockPatternUtils;-getAppWidgets-()[I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavScreen;-access$000-(Lcom/android/browser/NavScreen; Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameBranch;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/CountdownTimerPopup;-access$000-(Lcom/android/camera/ui/CountdownTimerPopup;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/Invert;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/RIL;-sendTerminalResponse-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$DisconnectingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/browser/PageDialogsHandler;-createSslCertificateDialog-(Landroid/net/http/SslCertificate; Landroid/net/http/SslError;)Landroid/app/AlertDialog$Builder;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/ISipService;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/music/AudioPreview;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/util/State;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothGatt;-readRemoteRssi-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/gatt/GattService;-readRemoteRssi-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/emailcommon/service/IEmailService;-pushModify-(J)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/datetimepicker/time/RadialPickerLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-handleDreamFinishedLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-disable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-pollStateDone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageFooterView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/AsyncQueryService;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-conference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothHealth;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsImpl;-updateSubtypeEnabler-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-writeDescriptor-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/activity/ComposeActivityEmail;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/IPowerManager$Stub$Proxy;-updateWakeLockUids-(Landroid/os/IBinder; [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/dreams/DreamManagerService;-startDream-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/ViewPager;-setCurrentItemInternal-(I Z Z I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/AppHwPref;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/ExpandableListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AnimatedAdapter;-performAndSetNextAction-(Lcom/android/mail/ui/SwipeableListView$ListItemsRemovedListener;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/FragmentBreadCrumbs;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LinearLayoutWithDefaultTouchRecepient;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/QuickResponseActivity$QueryThread;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-showWarningView-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterfw/core/Filter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/LightsService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/calendar/alerts/AlarmScheduler;-scheduleNextAlarm-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/ExchangeService;-serviceRequest-(J I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Editor$SelectionEndHandleView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/OverlayTitleTemplatePicker;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getAllNetworkInfo-()[Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-showGlobalActionsDialog-()V' : ['android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccCard;-setIccLockEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/EmailProvider;-pickTrashFolder-(Landroid/net/Uri;)I' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/ConversationItemView;-bindAd-(Lcom/android/mail/browse/ConversationItemViewModel; Lcom/android/mail/ui/ControllableActivity; Lcom/android/mail/ui/AnimatedAdapter$ConversationListListener; Lcom/android/mail/browse/ConversationItemView$ConversationItemAreaClickListener; Lcom/android/mail/providers/Folder; I Lcom/android/mail/ui/AnimatedAdapter; I Landroid/graphics/Bitmap;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPickerActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/adapter/CalendarSyncParser;-commandsParser-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/CalendarColorPickerDialog$QueryService;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/gadget/MediaSetSource;-monitorRootPath-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/PhoneUi;-<init>-(Landroid/app/Activity; Lcom/android/browser/UiController;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/videoeditor/widgets/PlayheadView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/photo/util/ImageUtils$HttpInputStreamFactory;-downloadBytes-()[B' : ['android.permission.INTERNET'],
	'Lcom/svox/pico/PicoService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextSwitcher;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pan/PanService;-doStop-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/music/QueryBrowserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendEnvelopeWithStatus-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-adjustSuggestedStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityManagerService;-removeProcessLocked-(Lcom/android/server/am/ProcessRecord; Z Z Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/BluetoothAdapter;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine;-handleBatchedScanPollRequest-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/util/JobLimiter$JobWrapper;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothSocket;-bindListen-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/alerts/QuickResponseActivity;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/TelephonyRegistry;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ViewAnimator;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/datetimepicker/time/TimePickerDialog$4;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/IBluetoothMap;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Browser;-addSearchUrl-(Landroid/content/ContentResolver; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/soundrecorder/SoundRecorder;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/StatusBarManagerService;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/NonDistinctMultitouchHelper;-processMotionEvent-(Landroid/view/MotionEvent; Lcom/android/inputmethod/keyboard/PointerTracker$KeyEventHandler;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Landroid/filterpacks/base/GLTextureTarget;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ActionBarContextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/DatePickerDialog;-<init>-(Landroid/content/Context; I Landroid/app/DatePickerDialog$OnDateSetListener; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-access$2500-(Lcom/android/server/location/GpsLocationProvider; I Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-beginServiceDeclaration-(I I I I Landroid/os/ParcelUuid; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserPreferencesPage;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/SubtitleController;-getDefaultTrack-()Landroid/media/SubtitleTrack;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setPin-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/opp/BluetoothOppManager;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email2/ui/MailActivityEmail;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ILockSettings$Stub$Proxy;-checkPassword-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MusicPicker;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyLocationChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/StateManager;-startStateForResult-(Ljava/lang/Class; I Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothPbap$Stub$Proxy;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothManager;-getConnectionState-(Landroid/bluetooth/BluetoothDevice; I)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-removeBatch-(Lcom/android/bluetooth/opp/BluetoothOppBatch;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/AudioService;-access$1100-(Landroid/media/AudioService; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/dataconnection/DcController;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/category/CategorySelected;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/UiModeManagerService;-access$000-(Lcom/android/server/UiModeManagerService; Ljava/lang/String; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-postEventFromNative-(Ljava/lang/Object; I I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupOutgoing;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/FlipFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AnalogClock;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-serviceRequest-(J J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-onUpPressed-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/SupplicantStateTracker$ScanState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/pinyin/CandidatesContainer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-writeBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageVignette;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/ManageCachePage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowManager$Stub;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/bluetooth/IBluetoothA2dp;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorSVRectView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/TimerRingService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/browser/TabControl;-createNewTab-()Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/provision/DefaultActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/backup/BackupAgentHelper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/CheckedLinearLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/browse/MessageInviteView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CalendarView;-access$900-(Landroid/widget/CalendarView; Landroid/widget/AbsListView; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-setHideNotificationState-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/TabControl;-removeParentChildRelationShips-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/print/IPrintManager$Stub;-removePrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollKeyboardView$OnKeyClickListener;-onKeyClick-(Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/email/service/AuthenticatorService$Authenticator;-addAccount-(Landroid/accounts/AccountAuthenticatorResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/os/Bundle;)Landroid/os/Bundle;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/quicksearchbox/google/GoogleSearch;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/BrowserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/crop/CropActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/EasOutboxService;-tryAutodiscover-(Landroid/content/Context; Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/DontPressWithParentLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-focusChangedLw-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/WindowManagerPolicy$WindowState;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendEnvelope-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-access$100-(Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/NumberPicker;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/crop/CropActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/compose/ComposeActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/gsm/SmsManager;-getAllMessagesFromSim-()Ljava/util/ArrayList;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/gallery3d/app/FilmstripPage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/ConversationUpdater;-delete-(I Ljava/util/Collection; Lcom/android/mail/ui/DestructiveAction; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onPackageChanged-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityRecord$Token;-windowsVisible-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/camera/tinyplanet/TinyPlanetPreview;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractMailActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/IngestService;-onCreate-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/alerts/AlertActivity;-access$400-(Lcom/android/calendar/alerts/AlertActivity; J J J)V' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/filterpacks/base/NullFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ViewGroup;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationPager;-addNewItem-(I I)Landroid/support/v4/view/ViewPager$ItemInfo;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-gotoIdleAndNotifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityRecord;-windowsVisible-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/print/IPrintManager$Stub;-cancelPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-processNextPostDialChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-dialThreeWay-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorHueView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/FolderSelectionActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/SeekBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/vis3/Visualization3;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone; Landroid/telephony/CellInfo;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/google/common/io/ByteStreams;-getChecksum-(Lcom/google/common/io/InputSupplier; Ljava/util/zip/Checksum;)J' : ['android.permission.INTERNET'],
	'Lcom/android/emailsync/SyncManager;-startManualSync-(J I Lcom/android/emailsync/Request;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/LatinIME$UIHandler;-onStartInputView-(Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/EmptyConversationListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppLiveFolder;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/dataconnection/DcTrackerBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-handleLocationChangedLocked-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-getEnabledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AccountController;-switchToDefaultInboxOrChangeAccount-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/notification/NotificationScorer;-getScore-(Landroid/app/Notification; I)I' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/InboundSmsHandler$SmsBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/BrowserPreferencesPage;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/compose/AttachmentComposeView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/ISipService$Stub;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/calendar/event/AttendeesView;-addAttendees-(Ljava/util/ArrayList;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/TwoPaneController;-onFolderChanged-(Lcom/android/mail/providers/Folder; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/MediaPlayer;-setWakeMode-(Landroid/content/Context; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/SoundRecorder;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-cancelBondProcess-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$MyGestureListener;-onFling-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getPhonebookNameList-(I)Ljava/util/ArrayList;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-rescheduleMissedAlarms-(Landroid/content/ContentResolver; Landroid/content/Context; Landroid/app/AlarmManager;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/BluetoothInputDevice;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/ActionModeHandler$2;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Void;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hdp/HealthService;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHeadset;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/eas/EasSyncBase;-handleProvisionError-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/net/ConnectivityManager;-getMobileDataEnabled-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-handleSmsMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ViewFlipper;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/TabBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager$CalendarObserver;-<init>-(Lcom/android/emailsync/SyncManager; Landroid/os/Handler; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/print/IPrintManager$Stub;-startPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService$3;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/PrinterDiscoverySession;-finalize-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-deleteFromRawTable-(Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationContainer;-onOverlayModelUpdate-(Ljava/util/List;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/ConversationSyncDisabledTipView$4;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-startRecording-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-access$400-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView$DeleteKeyOnTouchListener;-onTouch-(Landroid/view/View; Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/gallery3d/ingest/ui/MtpThumbnailTileView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/gallery3d/data/SnailAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/timezonepicker/TimeZonePickerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkStatsService;-access$700-(Lcom/android/server/net/NetworkStatsService; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/ChooseAccountActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-updateWarningView-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-handleNewSms-(Landroid/os/AsyncResult;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/mail/browse/ScrollIndicatorsView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothMap$Stub;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/alerts/AlertReceiver;-getEventCursor-(Landroid/content/Context; J)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Landroid/nfc/cardemulation/OffHostApduService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/crop/CropActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub;-getTetherableIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/IConnectivityManager;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/eas/EasOperation;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-addTrackerToRawTableAndSendMessage-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$InitialState;-enter-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/category/CategoryTrack;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/PrintSpoolerService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/packageinstaller/UninstallAppProgress;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppService$UpdateThread;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/ConversationContainer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/EasSyncService;-sendSettings-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IRestoreSession;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-queueQuery-(Lcom/android/calendar/agenda/AgendaWindowAdapter$QuerySpec;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/AdapterView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/NumberPicker$CustomEditText;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-confirmAndDelete-(I Ljava/util/Collection; Z I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-switchWaitingOrHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-setFilterInfo-(Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-dial-(Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/EmptyAlbumImage;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/photos/AlbumSetFragment;-deleteItemWithPath-(Ljava/lang/Object;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/musicvis/vis1/Visualization1;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneBase;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/net/IConnectivityManager;-getTetherableIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AddBookmarkPage$CustomListView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/MultiStateButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/format/DateFormat;-getTimeFormat-(Landroid/content/Context;)Ljava/text/DateFormat;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardParserImpl_V30;-handleNest-()V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/event/EditEventView;-updateView-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/mail/browse/ConversationPager;-smoothScrollTo-(I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodSubtypeListLocked-(Landroid/view/inputmethod/InputMethodInfo;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/MediaSet$ItemConsumer;-consume-(I Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/crop/CropActivity$BitmapIOTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$1;-sendMeetingResponse-(J I)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/research/Replayer;-replay-(Lcom/android/inputmethod/research/MotionEventReader$ReplayData; Ljava/lang/Runnable;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/mail/ui/AbstractActivityController$4;-onPostExecute-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/AbstractSyncParser;-responsesParser-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calculator2/Calculator;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralView;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/Application;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/GeneralPreferences;-access$000-(Lcom/android/calendar/GeneralPreferences;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getSelectedInputMethod-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/WebVttRenderingWidget$CueLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UiController;-openIncognitoTab-()Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/Utils;-getDayOfWeekString-(I I J Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/support/v4/widget/DrawerLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$Receiver;-updateMonitoring-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/Connectivity$WifiScanReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiService;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/ExpandableListActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/text/method/TextKeyListener;-getPrefs-(Landroid/content/Context;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/CandidateView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActiveServices;-stopServiceLocked-(Lcom/android/server/am/ServiceRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/widget/CompoundButton;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/time/TimePickerDialog;-onResume-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/support/v13/app/FragmentTabHost;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-getConnectedHealthDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/timezonepicker/TimeZonePickerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$5400-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/app/ExpandableListActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/certinstaller/CertInstaller;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-dismiss-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-access$900-(Lcom/android/gallery3d/ui/MenuExecutor; I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/backup/BackupAgentHelper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/ClockFragment;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterState$PendingCommandState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getAllLinkQualityInfo-()[Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/EmailConnectivityManager;-isAutoSyncAllowed-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/development/LogViewer;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Switch;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-addPrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-removePeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-onCreate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsUsageMonitor; Lcom/android/internal/telephony/ImsSMSDispatcher;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onActionBarWanted-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/ViewRootImpl$EarlyPostImeInputStage;-onProcess-(Landroid/view/ViewRootImpl$QueuedInputEvent;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/SlideshowPage;-showPendingBitmap-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/Editor$SelectionStartHandleView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/WidgetService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/support/v4/widget/ContentLoadingProgressBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker$1;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage;-updateUIForCurrentPhoto-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothPan$Stub$Proxy;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/effect/effects/FisheyeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationPager;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/wallpaper/fall/FallView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/videoeditor/OverlayTitleEditor;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/ConnectivityService;-getAllLinkQualityInfo-()[Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/deskclock/DontPressWithParentLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ActionableToastBar$ActionClickedListener;-onActionClicked-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/LatinIME;-onFinishSlidingInput-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/dreams/phototable/CursorPhotoSource;-<init>-(Landroid/content/Context; Landroid/content/SharedPreferences;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/videoeditor/VideoEditorActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/PointerLocationView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-access$1800-(Lcom/android/bluetooth/opp/BluetoothOppService; Landroid/database/Cursor; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockContentResolver;-requestSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/MediaPlaybackService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/TextServicesManagerService$TextServicesSettings;-getSelectedSpellChecker-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/PinyinIME;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryDownloadProgressBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/TouchExplorer;-onMotionEvent-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth;-setDiscoverableTimeout-(I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/CalendarView$WeekView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/SupplicantStateTracker$CompletedState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/DownUpDetector;-setState-(Z Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accessibility/TouchExplorer$PerformLongPressDelayed;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationContainer;-positionSnapHeader-(I)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/magicsmoke/MagicSmokeSelector;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/IWindowManager$Stub;-prepareAppTransition-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EasAutoDiscover;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity; J)Lcom/android/exchange/EasResponse;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/EventLoader$LoadEventsRequest;-processRequest-(Lcom/android/calendar/EventLoader;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/speech/tts/TtsEngines;-getLocalePrefForEngine-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/text/format/DateFormat;-getDateFormatString-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-rightButtonAction-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationListFooterView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EmailServiceUtils;-getServiceForAccount-(Landroid/content/Context; J)Lcom/android/emailcommon/service/EmailServiceProxy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager;-isMailboxSyncable-(Lcom/android/emailcommon/provider/Account; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/AlarmManagerService;-setWakelockWorkSource-(Landroid/app/PendingIntent; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-startRfcommSocketListener-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/FlipFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/AdapterViewFlipper;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService;-openFile-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/widget/AutoScrollHelper;-cancelTargetTouch-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventView$TimeClickListener;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/WaveView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/email/service/PolicyService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/DeskClock$TabsAdapter;-onPageSelected-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ActivityController;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/PrintManager;-restartPrintJob-(Landroid/print/PrintJobId;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/coremedia/iso/IsoFile;-<init>-(Ljava/nio/channels/ReadableByteChannel;)V' : ['android.permission.INTERNET'],
	'Lcom/android/documentsui/SettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/shims/MediaSetLoader;-loadInBackground-()Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/research/FeedbackLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/soundrecorder/SoundRecorder$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationViewAdapter$ConversationHeaderItem;-rebindView-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/videoeditor/VideoEditorBaseActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/TestActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-dial-(Ljava/lang/String; I Lcom/android/internal/telephony/UUSInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmStateManager$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt;-sendResponse-(I Ljava/lang/String; I I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/DontPressWithParentLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/renderscript/RSSurfaceView;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserWebView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/DatePickerDialog;-onDateChanged-(Landroid/widget/DatePicker; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/FilterShowActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment;-onAccountChanged-(Lcom/android/mail/providers/Account; Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt;-sendNotification-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; Z [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/packageinstaller/UninstallAppProgress;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountAdapter;-getChildrenCursor-(Landroid/database/Cursor;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/AlbumSetPage$MyDetailsSource;-setIndex-()I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/SearchActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/music/TouchInterceptor;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationSelectionSet;-toggle-(Lcom/android/mail/providers/Conversation;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcActivatingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/TintFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavScreen$2;-onLayout-(I I I I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/location/ILocationManager$Stub;-isProviderEnabled-(Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/printspooler/SelectPrinterActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-unregisterServer-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ISms;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/emailcommon/mail/Folder;-fetch-([Lcom/android/emailcommon/mail/Message; Lcom/android/emailcommon/mail/FetchProfile; Lcom/android/emailcommon/mail/Folder$MessageRetrievalListener;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/camera/CameraBackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/AddBookmarkPage$UpdateBookmarkTask;-doInBackground-([Landroid/content/ContentValues;)Ljava/lang/Void;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$ProvisionDiscoveryState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-setCdmaBroadcastActivation-(Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/IState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/mail/browse/MessageAttachmentTile;-thumbnailLoadFailed-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/videoeditor/widgets/HandleView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/TrackBrowserActivity;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/views/GalleryThumbnailView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-processStartTimedOutLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/TrackBrowserActivity$NowPlayingCursor;-moveItem-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/fall/FallView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/PieItem;-performClick-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/AccountsTester$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/filterpacks/imageproc/CropFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/CantAddAccountActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/info/HistogramView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-processConnectOtherProfiles-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetooth;-connectSocket-(Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/ISyncAdapter;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/InputMethodManagerService;-access$200-(Lcom/android/server/InputMethodManagerService; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioManager;-preDispatchKeyEvent-(Landroid/view/KeyEvent; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wm/WindowManagerService$3;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumDataLoader;-access$1800-(Lcom/android/gallery3d/app/AlbumDataLoader; Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-access$1000-(Lcom/android/gallery3d/app/MoviePlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/GLTextureTarget;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/format/DateUtils;-formatDateTime-(Landroid/content/Context; J I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setLine1Number-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/GlobalActions$2;-onLongPress-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/latin/SubtypeSwitcher;-init-(Landroid/content/Context;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothGattServer;-clearServices-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/TabBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavTabView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-access$1700-(Lcom/android/gallery3d/app/AlbumSetPage; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollViewWithNotifier;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getBatchedScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/calendar/alerts/GlobalDismissManager;-syncReceiverDismissCache-(Landroid/content/Context;)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/TimeBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/crop/CropActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-stopUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onError-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/BorderView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/WaveView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-access$4600-(Lcom/android/server/pm/PackageManagerService; Z Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getRemoteType-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-requestNetworkTransitionWakelock-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Landroid/media/MediaPlayer;-access$900-(Landroid/media/MediaPlayer; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/SystemBackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-startLocationBasedDetector-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/print/PrintJob;-isStarted-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/RunningProcesses;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/IPrintManager$Stub;-getPrintJobInfo-(Landroid/print/PrintJobId; I I)Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-populate-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/camera/MediaSaveService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/RecognitionManagerService;-systemReady-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/LockdownVpnTracker;-onVpnStateChanged-(Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-acknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/FolderSpanTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/GLView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMALTEPhone;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/data/LocationClustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/TimerRingService;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/SelectPrinterActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-dispatchHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.VIBRATE'],
	'Landroid/bluetooth/IBluetooth$Stub;-setRemoteAlias-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-<init>-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/RIL;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-hideAndStopWelcomeVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/LogTextBox;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$Bookmarks;-add-(Landroid/content/ContentResolver; Landroid/content/Intent; Ljava/lang/String; Ljava/lang/String; C I)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageGrad;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ActionBarView$HomeView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-revertToNitzTimeZone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PhotoPage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/noisefield/NoiseFieldWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/DhcpStateMachine$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UiController;-closeCurrentTab-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationSetObserver;-onSetEmpty-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/VideoEditorBaseActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/musicvis/vis2/Visualization2;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/Controller;-openTab-(Ljava/lang/String; Lcom/android/browser/Tab; Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifyServiceState-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/DialogActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/GeneralPreferences;-onPreferenceChange-(Landroid/preference/Preference; Ljava/lang/Object;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/view/BookmarkContainer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/TimelineHorizontalScrollView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/RotationPolicy;-registerRotationPolicyListener-(Landroid/content/Context; Lcom/android/internal/view/RotationPolicy$RotationPolicyListener; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/MediaController;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/ExchangeService;-serviceRequest-(J J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-access$300-(Lcom/android/server/LocationManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/camera/crop/CropActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-acknowledgeIncomingGsmSmsWithPdu-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContentResolver;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/Chronometer;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/MultiStateButton;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/MediaSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/nsd/NsdManager;-init-()V' : ['android.permission.INTERNET'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/compose/QuotedTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/category/IconView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-getBoolean-(Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/SupplicantStateTracker$UninitializedState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/content/SyncManager;-access$1400-(Lcom/android/server/content/SyncManager; Lcom/android/server/content/SyncManager$ActiveSyncContext; Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/EventInfoActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarApplication;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/PhotoModule;-onResumeTasks-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/ILocationManager$Stub;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/videoeditor/widgets/PreviewSurfaceView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/RotarySelector;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter;-access$2600-(Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter; Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter$ItemHolder; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHeadset;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/downloads/ui/DownloadList;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasOperation;-handleProvisionError-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsFragment;-onResume-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/print/IPrintManager$Stub;-getInstalledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasProvision;-provision-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/text/StringLogger;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/PlaybackQueueItem;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phasebeam/PhaseBeam;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-sendEnvelopeWithStatus-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/QuickContactBadge;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GeofenceManager;-addFence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; I I Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-startAction-(I I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MediaPlaybackService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/am/Am;-main-([Ljava/lang/String;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.RESTART_PACKAGES'],
	'Lcom/android/server/SystemBackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetherableIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/EasSyncService;-sendMeetingResponse-(Lcom/android/exchange/MeetingResponseRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/MailActivity;-onRestoreInstanceState-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/Tab;-removeFromTree-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/widget/SizeAdaptiveLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/ui/WebSearchSuggestionView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/agenda/AgendaFragment$2;-run-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/documentsui/DirectoryFragment;-formatTime-(Landroid/content/Context; J)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/YearPickerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/ActionBatch$StartDownloadAction;-execute-(Landroid/content/Context;)V' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/keyboard/internal/PreviewPlacerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/ProjectsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/RotateLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/printspooler/FusedPrintersProvider;-onForceLoad-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageAttachmentTile;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/EventLoader$LoadEventDaysRequest;-processRequest-(Lcom/android/calendar/EventLoader;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/camera/ui/InLineSettingItem;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ActionMenuPresenter$OverflowMenuButton;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/IconLoader;-loadIcon-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationViewFragment;-access$400-(Lcom/android/mail/ui/ConversationViewFragment;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/media/effect/effects/CropEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/WallpaperManager;-setStream-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/QueryBrowserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/AutoCompleteTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/ComposeActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/compat/UserDictionaryCompatUtils;-addWord-(Landroid/content/Context; Ljava/lang/String; I Ljava/lang/String; Ljava/util/Locale;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyLocationChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasOutboxSync;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/ActivityGroup;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/AudioService$VolumeStreamState;-<init>-(Landroid/media/AudioService; Ljava/lang/String; I Landroid/media/AudioService$1;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub;-getMobileDataEnabled-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/hardware/location/GeofenceHardwareService;-access$200-(Landroid/hardware/location/GeofenceHardwareService; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/ui/ConversationViewFragment$3;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/app/TrimVideo;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ComboViewActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TableRow;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/photo/adapters/PhotoPagerAdapter;-setPrimaryItem-(Landroid/view/ViewGroup; I Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/widgets/AudioTrackLinearLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/CalendarContract$CalendarEntity$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/emailcommon/service/PolicyServiceProxy;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/GlobalActions$2;-onPress-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/camera/ShutterButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/TouchInterceptor;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DayView;-switchViews-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/app/IAlarmManager$Stub;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/calendar/agenda/AgendaListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RotateFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/deskclock/timer/TimerListItem;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager;-getActiveLinkQualityInfo-()Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/photos/views/SquareImageView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/BaseUi;-showWeb-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/content/ContentResolver;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityStack;-updateTransitLocked-(I Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/Session;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/deskclock/widget/ActionableToastBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/SpeechRecognizer;-stopListening-()V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/wallpaper/fall/FallView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/ui/MtpImageView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/ButtonSwitcher;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppIncomingFileConfirmActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/provider/BrowserProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calculator2/Calculator;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/NotificationManagerService$SettingsObserver;-update-(Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/CameraSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IHardwareService$Stub$Proxy;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Landroid/service/textservice/SpellCheckerService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/provider/EmailProvider;-uiDeleteAccountData-(Landroid/net/Uri;)I' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/event/EditEventView$DateListener;-onDateSet-(Lcom/android/datetimepicker/date/DatePickerDialog; I I I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/CommandsInterface;-iccIO-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/SnailAlbum;-notifyChange-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/pacprocessor/PacService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-updateFromProvider-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ISms;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/net/ConnectivityManager;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/BrowserBackupAgent;-addBookmark-(Lcom/android/browser/BrowserBackupAgent$Bookmark;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/Tab;-<init>-(Lcom/android/browser/WebViewController; Landroid/webkit/WebView;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/ui/PhotoView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AbstractActivityController;-onUpPressed-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/agenda/AgendaByDayAdapter;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mail/browse/ConversationPager;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/GridView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasProvision;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/exchange/eas/EasFolderSync;-doFolderSync-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/net/wifi/IWifiManager;-pollBatchedScan-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/IActivityManager;-wakingUp-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/AliasActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/FilterSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/map/BluetoothMnsObexClient;-handleRegistration-(I I)V' : ['android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Landroid/widget/GridLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-play-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-finishBooting-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/MovieActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/LatinIME;-onEndBatchInputAsyncInternal-(Lcom/android/inputmethod/latin/SuggestedWords;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-access$000-(Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/MovieActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IAccountService$Stub$Proxy;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub;-stopBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-updateUri-()Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/app/LauncherActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-notifyAndAcknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController;-registerForWifiIdleTimeChange-(Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/DeskClock$TabsAdapter;-<init>-(Lcom/android/deskclock/DeskClock; Landroid/app/Activity; Landroid/support/v4/view/ViewPager;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/AdapterView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/RecognitionManagerService$MyPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/WidgetConfigure;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/NotificationManagerService$SettingsObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-assignContactFromPhone-(Ljava/lang/String; Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/UiModeManagerService;-updateAfterBroadcastLocked-(Ljava/lang/String; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/DriverCall; Lcom/android/internal/telephony/gsm/GsmCallTracker; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ServiceWatcher$2;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/stopwatch/StopwatchService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/compose/AttachmentsView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hid/HidService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TabHost;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupIncomingFragment;-access$000-(Lcom/android/email/activity/setup/AccountSetupIncomingFragment;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-access$200-(Lcom/android/bluetooth/pbap/BluetoothPbapService;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-serverConnect-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-handleSendComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SlideshowPage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/LinearLayoutWithDefaultTouchRecepient;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-invokeNextAgent-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/preference/PreferenceActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackActivity;-scanForward-(I J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IEmailService;-autoDiscover-(Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/research/ReplayerService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/Tab$3;-onCloseWindow-(Landroid/webkit/WebView;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/LocationManagerService;-getReceiverLocked-(Landroid/app/PendingIntent; I I Ljava/lang/String; Landroid/os/WorkSource; Z)Lcom/android/server/LocationManagerService$Receiver;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/WeekSelector;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/deskclock/alarms/AlarmService;-startAlarm-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/tts/compat/CompatTtsService;-access$600-(Landroid/speech/tts/TextToSpeechService;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothA2dp;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiStateMachine$L2ConnectedState;-exit-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractMailActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationPager;-setCurrentItem-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/text/format/DateUtils;-formatDateRange-(Landroid/content/Context; J J I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-addAppLocked-(Landroid/content/pm/ApplicationInfo; Z)Lcom/android/server/am/ProcessRecord;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/browse/ConversationViewAdapter$MessageHeaderItem;-getTimestampShort-()Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt;-registerForNotification-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothTetheringDataTracker;-teardown-()Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/FilmstripPage;-switchWithCaptureAnimation-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/RunningProcesses;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/accessibility/CaptioningManager;-getUserStyle-()Landroid/view/accessibility/CaptioningManager$CaptionStyle;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/TrackBrowserActivity;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/smspush/WapPushManager;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/content/ModernAsyncTask$2;-call-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/TimerSetupView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardEntry$EntryElement;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-sendImsGsmSms-(Ljava/lang/String; Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/content/AbstractThreadedSyncAdapter$ISyncAdapterImpl;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub;-setPairingConfirmation-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/wallpaper/galaxy/Galaxy;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/compose/CcBccView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/AsyncService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment$2;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EventColorPickerDialog;-onColorSelected-(I)V' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/AliasActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getRemoteType-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/addbookmark/FolderSpinner;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageRedEye;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FlipFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processDisplayText-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-onPopupItemClick-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/updates/SELinuxPolicyInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/EmlViewerActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/wallpaper/galaxy/GalaxyRS;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/app/AlertActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-requestSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/tinyplanet/TinyPlanetPreview;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-markAsUsed-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; I I Z)V' : ['android.permission.INTERNET'],
	'Landroid/widget/TextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/EllipsizedMultilineTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/EllipsizedMultilineTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/tts/compat/CompatTtsService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/SlideshowPage;-findMediaItem-(Lcom/android/gallery3d/data/MediaSet; I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/ContentResolver;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/test/mock/MockContentResolver;-requestSync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/EdgeView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onLoadKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/NumberPicker$CustomEditText;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/MultiAutoCompleteTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lorg/apache/http/impl/client/DefaultHttpClient;-execute-(Lorg/apache/http/client/methods/HttpUriRequest;)Lorg/apache/http/HttpResponse;' : ['android.permission.INTERNET'],
	'Landroid/app/backup/BackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/VolumePanel;-onProgressChanged-(Landroid/widget/SeekBar; I Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/view/accessibility/CaptioningManager$2;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-setHdmiPlugged-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/PackageManagerService;-main-(Landroid/content/Context; Lcom/android/server/pm/Installer; Z Z)Landroid/content/pm/IPackageManager;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/app/IWallpaperManager$Stub$Proxy;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/development/CacheAbuser;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onActionBarAllowed-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/activity/setup/MailboxSettings$2;-callDoInBackgroundForTest-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingInsecureRfcommWithServiceRecord-(Ljava/lang/String; Ljava/util/UUID;)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$WatchdogEnabledState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/calendar/EventInfoActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/BackupManagerService$PerformRestoreTask;-execute-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/hdp/HealthService;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-onDataSetChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityStackSupervisor;-moveTaskToStack-(I I Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub$Proxy;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-setHighNotificationState-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-handleInjectNtpTime-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Controller;-switchToTab-(Lcom/android/browser/Tab;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$IndicatorView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ILockSettings$Stub$Proxy;-getLong-(Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-killAllBackgroundProcesses-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/calendar/alerts/AlertReceiver;-finishStartingService-(Landroid/app/Service; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/date/DatePickerDialog;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/ConnectivityManager;-setRadios-(Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-getHealthDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/alerts/AlertActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackActivity;-access$1100-(Lcom/android/music/MediaPlaybackActivity;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/svox/pico/PicoService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-onFinished-(Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/SettingsRedirector;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/LogTextBox;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/NativeActivity$NativeContentView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/WidgetService$PhotoRVFactory;-getViewAt-(I)Landroid/widget/RemoteViews;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/ListPrefSettingPopup;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipAudioCall;-setSpeakerMode-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/mail/ui/AbstractActivityController$DroppedInStarredAction;-performAction-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/BluetoothDevice;-createRfcommSocketToServiceRecord-(Ljava/util/UUID;)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/AnimatedAdapter;-startFadeOutLeaveBehindItemsAnimations-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calculator2/Calculator;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/LauncherActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/TrimVideo;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-handleLongPressOnHome-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/MultiAutoCompleteTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AdapterView;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/keychain/KeyChainActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/alarms/AlarmNotifications;-broadcastNextAlarm-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/widget/FrameLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/DocumentsApplication;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/WebSearchSuggestionView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onSingleTapUp-(I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Contacts$People;-createPersonInMyContactsGroup-(Landroid/content/ContentResolver; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/google/common/io/Files;-copy-(Ljava/io/File; Ljava/io/OutputStream;)V' : ['android.permission.INTERNET'],
	'Lcom/android/mail/ui/AbstractActivityController;-starMessage-(Lcom/android/mail/browse/ConversationMessage; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothHeadset;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/DatePicker;-updateDate-(I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/email/provider/EmailProvider$5;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/speechrecorder/SpeechRecorderActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/HorizontalScrollView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/PanoProgressBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-switchToNextInputSubtypeInThisIme-(Landroid/os/IBinder; Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-switchUser-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/SnoozeAlarmsService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/Session;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/LatinIME;-onCreate-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/printspooler/PrintJobConfigActivity$CustomEditText;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompatKitKat$1;-createAccessibilityNodeInfo-(I)Landroid/view/accessibility/AccessibilityNodeInfo;' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipAudioCall;-endCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/LegacyImapAuthenticatorService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/RatingBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneLayout;-dispatchConversationVisibilityChanged-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/EasOutboxService;-hasConnectivity-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/Bookmarks$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/widget/ListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/TintFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/development/DataList;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoCarousel;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$WaitingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothGattCallback$Stub$Proxy;-onAdvertiseStateChange-(I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-setAndPersistDeviceName-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothPan$Stub$Proxy;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/MediaPlaybackActivity;-access$900-(Lcom/android/music/MediaPlaybackActivity; I J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/LocationManager;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/browser/PieControl;-access$000-(Lcom/android/browser/PieControl;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/MoreSettingPopup;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-onConversationSelected-(Lcom/android/mail/providers/Conversation; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumPage;-access$1200-(Lcom/android/gallery3d/app/AlbumPage; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterApp;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt;-beginServiceDeclaration-(I I I I Landroid/os/ParcelUuid; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/eas/EasSync;-handleProvisionError-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/PhoneUi;-showNavScreen-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/SystemAllowGeolocationOrigins;-start-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/email/service/AuthenticatorService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompatKitKat$1;-performAction-(I I Landroid/os/Bundle;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/deskclock/worldclock/CitiesActivity$CityAdapter;-set24HoursMode-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/UninstallAppProgress;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/NavigationBarBase;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AbsoluteLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cat/CatService;-handleRilMsg-(Lcom/android/internal/telephony/cat/RilMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/SyncAdapterDriver;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/FragmentBreadCrumbs;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Disconnected;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/CreatePlaylist;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/CalendarView$WeekView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/TouchExplorer;-access$2800-(Lcom/android/server/accessibility/TouchExplorer; Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/filterfw/core/Filter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-addIncludedService-(I I I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/PieController;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/RotarySelector;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/Switch;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-getSuggestionsExternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-loadCallHistoryList-(I)Ljava/util/ArrayList;' : ['android.permission.READ_CALL_LOG'],
	'Lcom/android/mail/MailIntentService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccCard;-supplyPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-stopUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onRegistrationFailed-(Landroid/net/sip/ISipSession; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onPictureCenter-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hfp/HeadsetService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Landroid/view/LayoutInflater$BlinkLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller$Download;-saveDataUri-()V' : ['android.permission.INTERNET'],
	'Landroid/inputmethodservice/ExtractEditLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/OverlayView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-setSafeMode-(Z)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/PacManager;-access$200-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processSetupCall-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarController;-sendEvent-(Ljava/lang/Object; J Landroid/text/format/Time; Landroid/text/format/Time; Landroid/text/format/Time; J I J Ljava/lang/String; Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/Tab$2;-shouldOverrideUrlLoading-(Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/videoeditor/widgets/TimelineHorizontalScrollView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-getSyncManager-()Lcom/android/server/content/SyncManager;' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/CalendarView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readConfigurationForUserStateLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager;-startPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/SelectionManager;-getSelected-(Z)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/print/PrintJob;-isBlocked-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onVoiceCallStarted-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/CameraRootView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/MultiStateButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferAdapter;-bindView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/browser/SystemAllowGeolocationOrigins$1;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/AttachmentsView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-finishUserStop-(Lcom/android/server/am/UserStartedState;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/inputmethodservice/KeyboardView;-sendAccessibilityEventForUnicodeCharacter-(I I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgentHelper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/IPrintManager;-getPrintJobInfos-(I I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ConnectivityService;-<init>-(Landroid/content/Context; Landroid/os/INetworkManagementService; Landroid/net/INetworkStatsService; Landroid/net/INetworkPolicyManager; Lcom/android/server/ConnectivityService$NetworkFactory;)V' : ['android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasOutboxService;-tryProvision-(Lcom/android/exchange/EasSyncService;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/view/EventRedirectingFrameLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-access$1600-(Lcom/android/mail/ui/AbstractActivityController;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/service/EasAuthenticatorServiceAlternate;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt;-registerServer-(Landroid/os/ParcelUuid; Landroid/bluetooth/IBluetoothGattServerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/widget/WidgetService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockContentResolver;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/PhotoPage;-onAlbumModeSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-writeSmsToSim-(I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapService$SocketAcceptThread;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/ErrorConsoleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setAndBroadcastNetworkSetTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Landroid/content/Context;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController$NoLockHeldState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/bluetooth/gatt/GattService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/provider/Telephony$Threads;-getOrCreateThreadId-(Landroid/content/Context; Ljava/util/Set;)J' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/PhoneFactory;-makeDefaultPhones-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/photos/shims/MediaItemsLoader;-loadInBackground-()Landroid/database/Cursor;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/videoeditor/widgets/OverlayLinearLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationListView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/content/IContentService;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/event/EditEventActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/RepeatingImageButton$RepeatListener;-onRepeat-(Landroid/view/View; J I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/FrameFetch;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/ConfigurationViewer;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-processBatchedEvents-(J)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ZoomControls;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/ImmersiveModeConfirmation;-loadSetting-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/GLTextureSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IRestoreSession$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IntentService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/grass/GrassWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorOpacityView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/timezonepicker/TimeZoneInfo;-getGmtDisplayName-(Landroid/content/Context;)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/ScreensaverActivity;-layoutClockSaver-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/ProfileService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ActionBarView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/ComboViewActivity$TabsAdapter;-<init>-(Landroid/app/Activity; Landroid/support/v4/view/ViewPager;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/ExchangeService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/TouchExplorer;-access$2600-(Lcom/android/server/accessibility/TouchExplorer; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/PicoService;-getSettingsLocale-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/TwoStateImageView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/AccessibleTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/eas/EasOutboxSync;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService;-enforceAccessPermission-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/printspooler/PrintJobConfigActivity$CustomEditText;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-startProcessLocked-(Ljava/lang/String; Landroid/content/pm/ApplicationInfo; Z I Ljava/lang/String; Landroid/content/ComponentName; Z Z Z)Lcom/android/server/am/ProcessRecord;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/ConversationUpdater;-markConversationsRead-(Ljava/util/Collection; Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/wifi/WifiService;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CallerInfo;-markAsVoiceMail-()Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/calendar/event/EditEventFragment;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/widget/WeightedLinearLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/TelephonyManager;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/compose/EnterSubject;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-handleFileUpdate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/PagerTabStrip;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-onFolderChanged-(Lcom/android/mail/providers/Folder; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/SettingsRedirector;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/CalendarBackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothGattServer;-removeService-(Landroid/bluetooth/BluetoothGattService;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CallerInfoAsyncQuery$CallerInfoAsyncQueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/camera/ui/ListPrefSettingPopup;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/Pop3Service;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/backup/BackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/SlidingTab;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService;-registerContentObservers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothPan;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-setCellBroadcastActivation-(Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-access$2600-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/providers/Conversation;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/gadget/WidgetService$PhotoRVFactory;-onDataSetChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/ConversationListView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/VolumePanel;-onShowVolumeChanged-(I I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipCommandInterface;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService$7;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiWatchdogStateMachine;-makeWifiWatchdogStateMachine-(Landroid/content/Context;)Landroid/net/wifi/WifiWatchdogStateMachine;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/Notifier;-onWakeUpStarted-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/MovieActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-setKeyboardMode-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManagerNative;-killApplicationWithAppId-(Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/accessibilityservice/AccessibilityService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/google/common/io/Files;-copy-(Lcom/google/common/io/InputSupplier; Ljava/io/File; Ljava/nio/charset/Charset;)V' : ['android.permission.INTERNET'],
	'Lcom/android/server/content/SyncManager;-access$1000-(Lcom/android/server/content/SyncManager; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Disconnected;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/support/v4/view/ViewPager;-pageLeft-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/month/MonthByWeekAdapter$2;-run-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/location/ILocationManager$Stub$Proxy;-addGpsStatusListener-(Landroid/location/IGpsStatusListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/ShortcutActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu;-starConversations-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/noisefield/NoiseFieldView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/StateManager;-switchState-(Lcom/android/gallery3d/app/ActivityState; Ljava/lang/Class; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email2/ui/CreateShortcutActivityEmail;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/agenda/AgendaListView;-goTo-(Landroid/text/format/Time; J Ljava/lang/String; Z Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-doQuery-(Lcom/android/calendar/agenda/AgendaWindowAdapter$QuerySpec;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-copyMessageToIccEf-(Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/mail/ui/OnePaneController;-showConversation-(Lcom/android/mail/providers/Conversation;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/print/PrintManager;-removePrintJobStateChangeListener-(Landroid/print/PrintManager$PrintJobStateChangeListener;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-processMsg-(Landroid/os/Message;)Lcom/android/internal/util/State;' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/view/WindowManagerPolicy$WindowManagerFuncs;-rebootSafeMode-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/AudioManager;-adjustVolume-(I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/RecognitionManagerService$MyPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/RotarySelector;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-pauseGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/browse/EmlViewerActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/PolicyService$1;-setAccountPolicy-(J Lcom/android/emailcommon/provider/Policy; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/media/WebVttRenderingWidget;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/ComposeActivityEmail;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/tts/TextToSpeechService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getAdapterConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-updateCurrentCarrierInProvider-(Ljava/lang/String;)Z' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/camera/crop/CropActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActiveServices;-removeConnectionLocked-(Lcom/android/server/am/ConnectionRecord; Lcom/android/server/am/ProcessRecord; Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/IAudioService$Stub;-adjustSuggestedStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-onSurfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService$26;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/widget/TableLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/BaseSuggestionView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/LocalSource;-fillQueue-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-processCdmaWapPdu-([B I Ljava/lang/String; J)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-onWindowFocusChanged-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService$8;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiStateMachine$ObtainingIpState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-setSymbolsShiftedKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/music/TrackBrowserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/ExpandableTextView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$LabelView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/emailcommon/service/EmailServiceProxy;-<init>-(Landroid/content/Context; Ljava/lang/Class;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/development/MediaScannerActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/service/ApiService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/provider/EmailProvider;-notifyUIConversationMailbox-(J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/selectcalendars/CalendarColorSquare;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/DateTimeView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager;-getTetheredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/Switch;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/ConnectivityManager;-getTetheredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/google/common/io/Files;-copy-(Ljava/io/File; Ljava/nio/charset/Charset; Ljava/lang/Appendable;)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-loadInputMethodAndSubtypeHistoryLocked-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ViewOverlay$OverlayViewGroup;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/EnterSubject;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageMirror;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-onCreate-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/browse/SwipeableConversationItemView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ActionMenuView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackActivity;-scanBackward-(I J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/app/NativeActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/pinyin/PinyinIME;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/AutologinBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-beginReliableWrite-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/inputmethod/InputMethodManager;-getEnabledInputMethodList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoSource;-findImages-(I)Ljava/util/Collection;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/quicksearchbox/DialogActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-registerForNotification-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/MediaPlaybackService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/DownloadManager;-addCompletedDownload-(Ljava/lang/String; Ljava/lang/String; Z Ljava/lang/String; Ljava/lang/String; J Z)J' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/filtershow/category/CategoryTrack;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/gatt/GattService;-endReliableWrite-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorBrightnessView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/MediaLinearLayout$1;-onLongPress-(Landroid/view/View; Landroid/view/MotionEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-performHapticFeedbackLw-(Landroid/view/WindowManagerPolicy$WindowState; I Z)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/mail/ui/FolderListFragment$4;-onDrawerClosed-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothHeadset;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/SystemBackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-adjustOtherHeadsetPriorities-(Lcom/android/bluetooth/hfp/HeadsetService; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/gadget/WidgetClickHandler;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/SystemVibrator;-vibrate-(I Ljava/lang/String; J)V' : ['android.permission.VIBRATE'],
	'Lcom/android/development/ShowActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/LatinIME;-maybeDoubleSpacePeriod-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasPing;-requestFolderSync-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/calendar/EventInfoFragment;-updateAttendees-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/music/MediaPickerActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/app/CameraApp;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/documentsui/DirectoryView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofenceResumeStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AutologinBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/VideoView;-getAudioSessionId-()I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/Parser;-parse-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Lcom/android/development/Connectivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthDeathRecipient;-binderDied-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/accounts/AccountManagerService$TestFeaturesSession;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mail/ui/OnePaneController;-transitionBackToConversationListMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiManager;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/AudioService$2;-onServiceConnected-(I Landroid/bluetooth/BluetoothProfile;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/eas/EasOutboxSync;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/net/wifi/IWifiManager$Stub;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/net/MobileDataStateTracker;-startMonitoring-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/GeofenceManager;-getFreshLocationLocked-()Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter;-bindView-(Landroid/view/View; Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech;-isLanguageAvailable-(Ljava/util/Locale;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiService;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Contacts$Settings;-getSetting-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Landroid/view/IWindowSession;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onSetInternalDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/CameraRootView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumPicker;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/UnderdevelopedSettings;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-getSmsMessage-(J I)[B' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/eas/EasOptions;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/ConversationViewFragment;-onMessageCursorLoadFinished-(Landroid/content/Loader; Lcom/android/mail/browse/MessageCursor; Lcom/android/mail/browse/MessageCursor;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/datetimepicker/time/CircleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnablingActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/MediaScannerActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/SlidingTab;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager;-releaseMailbox-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-performAuditoryFeedbackForAccessibilityIfNeed-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-onKeyClick-(Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/Screensaver;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/FallbackEventHandler;-preDispatchKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/tts/TextToSpeechService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BreadCrumbView$Controller;-onTop-(Lcom/android/browser/BreadCrumbView; I Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/ui/ConversationListCopy;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CalendarView;-goTo-(Ljava/util/Calendar; Z Z Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ImageView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/media/AudioManager;-adjustLocalOrRemoteStreamVolume-(I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-sync-(Landroid/content/Context; J Landroid/os/Bundle; Landroid/content/SyncResult; Z I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-handleStatusReport-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SlideshowView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AlbumDataLoader;-executeAndWait-(Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/documentsui/DocumentsApplication;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/BrowserBackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/svox/pico/CheckVoiceData;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/ui/DownloadList;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager;-getAuthTokenLabel-(Ljava/lang/String; Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/ActionBarOverlayLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/CommandsInterface;-iccIOForApp-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/fall/Fall;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/ILocationManager$Stub$Proxy;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/mail/ui/ConversationSyncDisabledTipView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/LocationManager;-recordLocation-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/ui/SearchActivityView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService;-adjustLocalOrRemoteStreamVolume-(I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/ActivityManagerNative;-handleApplicationCrash-(Landroid/os/IBinder; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-setInitialAttachApn-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageStraighten;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$800-(Lcom/android/gallery3d/ui/PhotoView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/research/UploaderService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/SuggestionsView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageVignette;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ScrollingTabContainerView$TabView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readDisplayMagnificationEnabledSettingLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/DestructiveAction;-performAction-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasPing;-handleOneMailbox-(Lcom/android/exchange/adapter/Serializer; Lcom/android/emailcommon/provider/Mailbox;)Lcom/android/exchange/adapter/Serializer;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/PackagesMonitor$AsyncService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pan/PanService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/VibratorService;-onInputDeviceChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/LatinIME$UIHandler;-executePendingImsCallback-(Lcom/android/inputmethod/latin/LatinIME; Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/UrlInputView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/BalloonHint$BalloonView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/grass/GrassWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothAdapter;-getConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/app/PlatLogoActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/personalization/UserHistoryDictionary;-writeBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/SecureAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/CameraActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/UsageStatsService$2;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ClusterSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/app/ResolverActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/WebVttRenderingWidget$CueLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/SlotView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/Browser;-getVisitedLike-(Landroid/content/ContentResolver; Ljava/lang/String;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/base/RetargetFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/PageProgressView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/InputView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/GalleryAppImpl;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/recurrencepicker/LinearLayoutWithMaxWidth;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/InLineSettingCheckBox;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-requestLocationUpdatesLocked-(Landroid/location/LocationRequest; Lcom/android/server/LocationManagerService$Receiver; I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/IntentFirewallInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/TabBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/music/WeekSelector;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothMap$Stub;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/research/Uploader;-isConvenientToUpload-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/photos/shims/MediaItemsLoader;-deleteItemWithPath-(Ljava/lang/Object;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/TitleBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/WallpaperManagerService$MyPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/PrintJobConfigActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Connected;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/TabWidget;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockContentResolver;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-writeBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/IAudioService$Stub$Proxy;-setStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/ex/photo/views/PhotoView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt;-clearServices-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/OnePaneController;-perhapsEnterWaitMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/Pop3Service;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/ISms;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/mail/compose/EmptyService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/widget/DrawerLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/ReplayerService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/AlbumSetDataLoader$ReloadTask;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasOutboxSync;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/pm/UserManagerService$5;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipManager;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Landroid/widget/DateTimeView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/BondStateMachine$StableState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-setQueuePosition-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService;-connectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/os/Vibrator;-vibrate-(I Ljava/lang/String; [J I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/am/UsageStatsService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/LatinIME;-pickSuggestionManually-(I Lcom/android/inputmethod/latin/SuggestedWords$SuggestedWordInfo;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailcommon/utility/EmailAsyncTask$1;-callDoInBackgroundForTest-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/deskclock/AlarmTimelineView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/view/BookmarkContainer;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-addPrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/RefreshStatusMonitor$RemoveRefreshStatusRunnable;-run-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/deskclock/widget/TextTime;-access$000-(Lcom/android/deskclock/widget/TextTime;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService;-getEnabledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmNotifications;-showHighPriorityNotification-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/appwidget/AppWidgetHostView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothPbap;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/emailcommon/provider/Account;-isAutomaticSyncDisabledByRoaming-(Landroid/content/Context; J)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/emailsync/SyncManager;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/NavTabScroller$ContentLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/PrintSpoolerService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-updateProvidersLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-deleteSystemPackageLI-(Lcom/android/server/pm/PackageSetting; [I [Z I Lcom/android/server/pm/PackageManagerService$PackageRemovedInfo; Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsActivity;-onResume-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/polarclock/PolarClockSettings;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/updates/TZInfoInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/bluetooth/btservice/AdapterApp;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/svox/pico/PicoService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/PackageManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CLEAR_APP_CACHE', 'android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/email/service/PolicyService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ViewMode;-enterSearchResultsListMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/content/SyncActivityTooManyDeletes;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/RotarySelector;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/PinyinDecoderService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/DragState;-endDragLw-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Presentation$1;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/updates/ConfigUpdateInstallReceiver$1;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/AccountService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/text/StringLogger;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionCallReceiverImpl;-access$1600-(Lcom/android/server/sip/SipSessionGroup$SipSessionImpl; Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipAudioCall;-onError-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/MediaSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/AlertService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/alarmclock/DigitalAppWidgetService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendP2pDiscoveryChangedBroadcast-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/NavigationBarPhone;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/vcard/VCardParserImpl_V21;-handleNest-()V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ingest/IngestService;-deviceAdded-(Landroid/mtp/MtpDevice;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasProvision;-performInitialRequest-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/bluetooth/IBluetoothManager;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/MediaPlaybackService;-notifyChange-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-onConnect-(Lcom/android/internal/telephony/dataconnection/DataConnection$ConnectionParams;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaFocusControl;-filterMediaKeyEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService;-startBluetoothSco-(Landroid/os/IBinder; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/email/provider/EmailProvider;-uiDeleteMessage-(Landroid/net/Uri;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-deleteAllInstances-(Landroid/content/Context; J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/media/AudioService;-adjustLocalOrRemoteStreamVolume-(I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/contacts/ContactsListActivity;-getSelectedUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$People;-tryGetMyContactsGroupId-(Landroid/content/ContentResolver;)J' : ['android.permission.READ_CONTACTS'],
	'Lcom/svox/pico/CheckVoiceData;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneBase;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/RenamingDelegatingContext;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/AccessibleLinearLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/Pop3Service$1;-loadAttachment-(Lcom/android/emailcommon/service/IEmailServiceCallback; J J Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/backup/FullBackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/DhcpStateMachine$RunningState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/soundrecorder/SoundRecorder;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/IsolatedContext$MockAccountManager;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/location/SettingInjectorService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/LockSettingsService;-checkReadPermission-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getBondedDevices-()[Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/quicksearchbox/DialogActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/FrameLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/SupplicantStateTracker$InactiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/server/power/PowerManagerService;-updateDreamLocked-(I)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/widget/BookmarkThumbnailWidgetService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/EasCertificateRequestor;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/compose/EmptyService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/gatt/GattService;-sendResponse-(I Ljava/lang/String; I I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/selectcalendars/CalendarColorSquare;-assignContactFromPhone-(Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/deskclock/AlarmClockFragment$8;-doInBackground-([Ljava/lang/Void;)Lcom/android/deskclock/provider/AlarmInstance;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-clientDisconnect-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/MenuExecutor$ConfirmDialogListener;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasSync;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ComboViewActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/CompatModePackages;-setPackageScreenCompatModeLocked-(Landroid/content/pm/ApplicationInfo; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/SmsStorageMonitor$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-setDismissState-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/KeyboardView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/Service;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/documentsui/DocumentsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Gallery;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-killUid-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/BaseNetworkStateTracker;-startMonitoring-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/EmailServiceUtils;-getService-(Landroid/content/Context; Ljava/lang/String;)Lcom/android/emailcommon/service/EmailServiceProxy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/compose/AutoSendActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-addAccount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/mail/ui/OnePaneController;-stopDragMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calculator2/CalculatorDisplay;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$StopTetheringErrorState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppLiveFolder;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/FilterSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-setCurrentPhotoByIntent-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-writeSmsToRuim-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ToggleButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/InLineSettingMenu;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-enqueue-([J I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onEnableApn-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/widget/TextTime;-chooseFormat-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService;-getShowTouchesSetting-(I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TtsEngines;-getDefaultEngine-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService$WindowManagerFuncs;-requestTraversal-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-hasMultipleEnabledIMEsOrSubtypes-(Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/service/Pop3Service$1;-deleteAccountPIMData-(Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/AdapterViewAnimator;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl;-access$1600-(Lcom/android/server/sip/SipSessionGroup$SipSessionImpl; Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/Pop3AuthenticatorService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/VideoUtils;-writeMovieIntoFile-(Ljava/io/File; Lcom/googlecode/mp4parser/authoring/Movie;)V' : ['android.permission.INTERNET'],
	'Lcom/android/server/am/ActivityManagerService;-finishUserSwitch-(Lcom/android/server/am/UserStartedState;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/ConversationsInOutboxTipView;-access$000-(Lcom/android/mail/ui/ConversationsInOutboxTipView;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$UpdateContent;-call-()Ljava/lang/Void;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/ConversationListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageWebView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/WebSearchSuggestionView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/DigitalClock;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService;-registerShowTouchesSettingObserver-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-<init>-(Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/gsm/GsmCallTracker; Lcom/android/internal/telephony/gsm/GsmCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-loadDictionaryAsync-()V' : ['android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/ConnectivityManager;-getTetheringErroredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onRadioAvailable-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email2/ui/MailActivityEmail;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/category/CategorySelected;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/DeleteItems;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryProvider;-getDictionaryWordListsForLocale-(Ljava/lang/String; Ljava/lang/String; Z)Ljava/util/Collection;' : ['android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/AddBookmarkPage$SaveBookmarkRunnable;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/InputMethodManagerService;-access$1100-(Lcom/android/server/InputMethodManagerService;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationCursor;-notifyDataChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/IAudioService$Stub$Proxy;-adjustStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/noisefield/NoiseField;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/time/TimePickerDialog$OnTimeSetListener;-onTimeSet-(Lcom/android/datetimepicker/time/RadialPickerLayout; I I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/development/RunningProcesses;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiNotificationController;-<init>-(Landroid/content/Context; Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/adapter/AbstractSyncParser;-wipe-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/browser/Tab;-postCapture-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/accounts/AccountManager$6;-doWork-()V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/gallery3d/app/PhotoPage$7;-onSystemUiVisibilityChange-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothGatt;-registerApp-(Landroid/bluetooth/BluetoothGattCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/updates/TZInfoInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/service/IPolicyService$Stub;-setAccountPolicy-(J Lcom/android/emailcommon/provider/Policy; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onRegistering-(Landroid/net/sip/ISipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email2/ui/MailboxSelectionActivityEmail;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeBarringPassword-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothA2dp;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView;-onCodeInput-(I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/SlideshowPage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneProxy;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/SoundRecorder;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-addDescriptor-(I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/ColorChipView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readAdnFileAndWait-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-switchToState-(I F F)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/view/LayoutInflater$BlinkLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/certinstaller/CertInstallerMain;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$5;-run-()V' : ['android.permission.REORDER_TASKS'],
	'Landroid/media/AudioService;-readPersistedSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/FilmStripView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/UserState;-updateIfNeededLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/DismissAlarmsService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/vcard/VCardParser_V30;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/view/IWindowSession$Stub;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/quicksearchbox/ui/WebSearchSuggestionView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/DBHelper;-migrateLegacyAccounts-(Landroid/database/sqlite/SQLiteDatabase; Landroid/content/Context;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/content/ContentResolver;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexServerSession;-onPut-(Ljavax/obex/Operation;)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SmsManager;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/compose/QuotedTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcActiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/mail/ui/TwoPaneController;-onFolderSelected-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/widgets/MediaItemView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/PingSyncSynchronizer;-pingEnd-(J Landroid/accounts/Account;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/inputmethodservice/ExtractEditText;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/settings/SettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/ViewAnimator;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handleCallWaitingInfo-(Lcom/android/internal/telephony/cdma/CdmaCallWaitingNotification;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/AudioTrackView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email2/ui/MailActivityEmail;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothPan;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AlbumDataLoader$ReloadTask;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/DeviceStorageMonitorService;-access$000-(Lcom/android/server/DeviceStorageMonitorService; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.CLEAR_APP_CACHE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-stopScan-(I Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/TwoPaneController;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/StateManager;-itemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/commands/bmgr/Bmgr;-doList-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/dreams/DreamService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/Pop3AuthenticatorService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter$SlideshowSource;-getMediaItem-(I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/tts/TextToSpeech;-synthesizeToFile-(Ljava/lang/String; Ljava/util/HashMap; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSettings;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/printspooler/SelectPrinterActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/NoSearchActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/ErrorConsoleView$ErrorConsoleListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/mail/ui/AbstractActivityController;-stopDragMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Ljava/net/HttpURLConnection;-connect-()V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifyServiceState-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/alarmclock/DigitalAppWidgetProvider;-onUpdate-(Landroid/content/Context; Landroid/appwidget/AppWidgetManager; [I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BitmapSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/RunningProcesses;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/AsyncQueryServiceHelper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/coremedia/iso/IsoFile;-createBoxParser-()Lcom/coremedia/iso/BoxParser;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/deskclock/timer/TimerAlertFullScreen;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/text/ToUpperCase;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/app/DisableCarModeActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-stopBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/SubtitleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/NestedFolderTeaserView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/HandleApiCalls;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DatePicker;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-cancelBondProcess-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/pinyin/PinyinDecoderService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-onTouch-(Landroid/view/View; Landroid/view/MotionEvent;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/AllInOneActivity;-onUserLeaveHint-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/BluetoothManagerService;-sendDisableMsg-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/email/service/LegacyImapAuthenticatorService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-startDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/BadBehaviorActivity$BadService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/TransitionsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/crop/CropActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/renderscript/RSTextureView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserPreferencesPage;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Controller$10;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/ingest/ui/MtpThumbnailTileView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/inputmethod/InputMethodManager;-getEnabledInputMethodSubtypeList-(Landroid/view/inputmethod/InputMethodInfo; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/vis5/Visualization5;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/BatteryService;-processValuesLocked-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorRectView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/utils/NotificationUtils;-setNewEmailIndicator-(Landroid/content/Context; I I Lcom/android/mail/providers/Account; Lcom/android/mail/providers/Folder; Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-processSlidingKeyInput-(Lcom/android/inputmethod/keyboard/Key; I I J)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/NotificationController$AccountContentObserver;-onChange-(Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/documentsui/SettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/provider/EmailProvider;-uiSendDraftMessage-(J Landroid/os/Bundle;)Landroid/net/Uri;' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/print/PrintManagerService;-stopPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationViewFragment;-renderBorder-(Z Z Z Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/app/AlertActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothAdapter;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/wallpaper/grass/GrassWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/fall/Fall;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/crop/CropActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/GalleryActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/UrlHandler$RLZTask;-onPostExecute-(Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/Phone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-getEnabledProviders-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-startAction-(I I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener; Z Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/service/dreams/DreamService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onAlbumModeSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/ui/CorpusView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/PlaylistBrowserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-deleteFromRawTable-(Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/development/MediaScannerActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager$2;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/EasAccountService;-postAutodiscover-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/editors/SwapButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/PlatLogoActivity$3;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/date/TextViewWithCircularIndicator;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipSessionGroup;-processIOException-(Ljavax/sip/IOExceptionEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/EmailSyncParser;-parse-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Lcom/android/deskclock/ClockFragment;-onPageChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EasService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/MtpClient$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub$Proxy;-startFreezingScreen-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/ModuleSwitcher;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-updateSettingsLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub$Proxy;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/HandleApiCalls;-handleSetAlarm-(Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/CaptioningManager;-registerObserver-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-supplyPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RadioButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/ConnectivityManager;-updateLockdownVpn-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSyncBase;-getRequestEntity-()Lorg/apache/http/HttpEntity;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/AbstractActivityController;-access$2800-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/content/ObjectCursor;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/TwoLineListItem;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothA2dp;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-retrieveSettings-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-setState-(I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestionProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/Activity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/ActivityState;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/BadBehaviorActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/FolderSpanTextView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/ExpandedGridView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IAlarmManager;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-access$700-(Lcom/android/gallery3d/ui/ActionModeHandler; Lcom/android/gallery3d/util/ThreadPool$JobContext; I)Landroid/content/Intent;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-clearClearableFlagsLw-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/packageinstaller/UninstallerActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$ScreenNailJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/research/FeedbackActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/magicsmoke/MagicSmokeSelector;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/packageinstaller/UninstallAppProgress;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/pacprocessor/PacService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/UnderdevelopedSettings;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/text/StringLogger;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/base/InputStreamSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState$SwitchActions;-setAlphabetShiftLockedKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/ViewPager;-setCurrentItem-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/deskclock/widget/sgv/StaggeredGridView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-createSocketChannel-(I Ljava/lang/String; Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-sendVendorSpecificResultCode-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/FaceClustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-updateUsbState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/timezonepicker/TimeZonePickerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorHueView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-notifyScreenNailChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub$Proxy;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/packageinstaller/GrantActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/TelephonyRegistry;-broadcastDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/TrimVideo;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/ViewStub;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter$DayAdapterInfo;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterfw/core/FilterGraph;-closeFilters-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/picasasource/PicasaSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/alerts/GlobalDismissManager$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ReceiverRestrictedContext;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationCursor;-access$2000-(Lcom/android/mail/browse/ConversationCursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-getTetheringErroredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-setProcessLimit-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/email/activity/setup/AccountSecurity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-handleCancel-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/development/PointerLocation;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/GalleryActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-fixPeriodicSyncs-(Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/speech/tts/TextToSpeechService;-getSettingsLocale-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendP2pStateChangedBroadcast-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/GalleryActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calculator2/ColorButton;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AdapterViewAnimator;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-installNewPackageLI-(Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/wm/WindowManagerService;-performLayoutAndPlaceSurfacesLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService$13$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-getLastPdpFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/LocationFudger;-loadCoarseAccuracy-()F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtErrorActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/FlipEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-insertShare-(Landroid/database/Cursor; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/power/DisplayPowerController$4;-run-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/cardemulation/OffHostApduService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/RemoteViewsAdapter$RemoteViewsFrameLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/settings/GallerySettings;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/Pop3Service;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/Phone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-onSaveInstanceState-(Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/speech/tts/TextToSpeechService;-access$600-(Landroid/speech/tts/TextToSpeechService;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Activity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/CountryDetectorService$2;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BreadCrumbView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorEditText;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/FilterShowActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/quicksearchbox/util/HttpHelper;-post-(Ljava/lang/String; Ljava/util/Map; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/TrimVideo;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-invokeExitMethods-(Lcom/android/internal/util/StateMachine$SmHandler$StateInfo;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/provider/Contacts$People;-addToGroup-(Landroid/content/ContentResolver; J Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-doStop-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/CertBlacklister;-registerObservers-(Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractMailActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/provider/UserDictionary$Words;-addWord-(Landroid/content/Context; Ljava/lang/String; I Ljava/lang/String; Ljava/util/Locale;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/calendar/DayFragment;-goTo-(Landroid/text/format/Time; Z Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/AppHwConfigList;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/view/SnapshotGridView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStartingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumPicker;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager;-getTetherableWifiRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/telephony/TelephonyManager;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralTestActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/RepeatingImageButton;-doRepeat-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/KenBurnsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/AppHwPref;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/EasAuthenticatorServiceAlternate;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-setGrabbedState-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/BluetoothManagerService;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/HttpAuthenticationDialog$1;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/RenderScriptWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/ActivityController;-onConfgurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-getLastKnownLocation-()Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofenceAddStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/EmailServiceStub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothGatt;-discoverServices-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Tab$2;-onReceivedLoginRequest-(Landroid/webkit/WebView; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/media/effect/effects/BlackWhiteEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub$Proxy;-getCurrentSpellChecker-(Ljava/lang/String;)Landroid/view/textservice/SpellCheckerInfo;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-access$2600-(Landroid/media/AudioService;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifySignalStrength-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/SpamWarningView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/EdgeView;-render-(Lcom/android/gallery3d/glrenderer/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/ConversationViewFragment;-showUntransformedConversation-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/ConversationListCopy;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/net/wifi/WifiStateMachine$SoftApStartingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/deskclock/widget/sgv/StaggeredGridView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RelativeLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/LegacyEasAuthenticatorService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/ExpandableTextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHeadset;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/TouchInterceptor;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/CustomScreenLinearLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-setDataSource-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/state/StateView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/ExchangeService;-getDeviceId-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/deskclock/TimerRingService;-onCreate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationContainer;-positionOverlays-(I I)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/bluetooth/BluetoothPan;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/TwoPaneLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$OngoingGroupRemovalState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/mail/ui/MailActivity;-handleDrop-(Landroid/view/DragEvent; Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-addTrackerToRawTable-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/widget/CheckedTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AbsListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/CandidateView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Switch;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/ViewGroup;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/ime/Ime;-main-([Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumPage;-pickPhoto-(I Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/RotarySelector;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/PinyinDecoderService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/IPowerManager;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-setSnoozeState-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/FilterSurfaceView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofenceTransition-(I I D D D F F F J I J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/DataList;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/CursorEntityIterator;-getEntityAndIncrementCursor-(Landroid/database/Cursor;)Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$InitialState;-chooseUpstreamType-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/MediaPlaybackService;-pause-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PickerActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-composeAndSendCallLogVcards-(I Ljavax/obex/Operation; I I Z)I' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-onScrubbingEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractActivityController;-exitCabMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/NoSearchActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/location/GpsLocationProvider;-access$2800-(Lcom/android/server/location/GpsLocationProvider;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onSetUserDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/NullFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/eas/EasMoveItems;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-callListenerOnRelease-(Lcom/android/inputmethod/keyboard/Key; I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/provider/BrowserContract$Bookmarks;-buildFolderUri-(J)Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/widget/ScrollingTabContainerView$TabView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/BroadcastQueue;-deliverToRegisteredReceiverLocked-(Lcom/android/server/am/BroadcastRecord; Lcom/android/server/am/BroadcastFilter; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/gadget/WidgetService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/MoreSettingPopup;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ingest/IngestService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/RenderOverlay;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onUpdateShiftState-(I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/AudioPreview;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-isLocationProviderEnabledForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-sendRequest-(I Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/ActivityManager;-getRecentTasks-(I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Landroid/content/ISyncContext$Stub$Proxy;-onFinished-(Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/BrowserBookmarksPage$OpenAllInTabsTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/sip/ISipService$Stub;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-updateCarrierMccMncConfiguration-(Ljava/lang/String; Ljava/lang/String; Landroid/content/Context;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/wm/WindowManagerService;-handleAppTransitionReadyLocked-(Lcom/android/server/wm/WindowList;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-dispatchMessageRadioSpecific-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub;-setCurrentSpellCheckerSubtype-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/IRecognitionService$Stub;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/mail/ui/ConversationListFragment;-onCursorUpdated-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/Utils;-trySyncAndDisableUpgradeReceiver-(Landroid/content/Context;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/ConversationListFooterView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/PhotoPage;-access$1500-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/view/CustomScreenLinearLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/widget/CalendarView;-setDate-(J Z Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallback;-onResponse-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothInputDevice;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/service/ImapService$1;-requestSync-(J Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ConversationViewFragment$MailJsBridge$2;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Landroid/bluetooth/IBluetoothGatt;-serverDisconnect-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onActionBarAllowed-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/CalendarController;-sendEvent-(Ljava/lang/Object; J Landroid/text/format/Time; Landroid/text/format/Time; J I J Ljava/lang/String; Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/timezonepicker/TimeZonePickerDialog$OnTimeZoneSetListener;-onTimeZoneSet-(Lcom/android/timezonepicker/TimeZoneInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/photos/views/SquareImageView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarApplication;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/service/PingTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/calculator2/Calculator$PageAdapter;-instantiateItem-(Landroid/view/ViewGroup; I)Ljava/lang/Object;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/view/EventRedirectingFrameLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/googlecode/mp4parser/authoring/container/mp4/MovieCreator;-build-(Ljava/nio/channels/ReadableByteChannel;)Lcom/googlecode/mp4parser/authoring/Movie;' : ['android.permission.INTERNET'],
	'Landroid/view/ViewOverlay$OverlayViewGroup;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readTouchExplorationGrantedAccessibilityServicesLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorRectView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationCursor$ConversationProvider;-apply-(Ljava/util/Collection; Lcom/android/mail/browse/ConversationCursor;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine;-access$23000-(Landroid/net/wifi/WifiStateMachine; Ljava/util/ArrayList;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/deskclock/timer/TimerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActiveServices;-startServiceInnerLocked-(Lcom/android/server/am/ActiveServices$ServiceMap; Landroid/content/Intent; Lcom/android/server/am/ServiceRecord; Z Z)Landroid/content/ComponentName;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu$2;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/musicvis/vis5/Visualization5;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/view/BasePieView;-layout-(I I Z F I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/server/LocationManagerService;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/browser/BrowserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-update-(Lcom/android/internal/telephony/DriverCall;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IActivityManager;-goingToSleep-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/view/CustomScreenLinearLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-updateNowIfNotUpdatedInAVeryLongTime-(Landroid/content/Context;)V' : ['android.permission.INTERNET'],
	'Lcom/android/email/service/ImapService;-processPendingAppend-(Landroid/content/Context; Lcom/android/email/mail/Store; Lcom/android/emailcommon/provider/Mailbox; Lcom/android/emailcommon/provider/EmailContent$Message;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/email/activity/setup/AccountSecurity;-access$100-(Lcom/android/email/activity/setup/AccountSecurity; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/filterfw/core/FilterFunction;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoCarousel$PhotoLoadTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/DataList;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/LockPatternUtils;-getBoolean-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothA2dp;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-getLong-(Ljava/lang/String; J)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-getPersistedScale-()F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/TransitionsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/Utils;-formatMonthYear-(Landroid/content/Context; Landroid/text/format/Time;)Ljava/lang/String;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/widget/DatePicker;-onRestoreInstanceState-(Landroid/os/Parcelable;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IActivityManager;-killBackgroundProcesses-(Ljava/lang/String; I)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/browser/TabControl;-restoreState-(Landroid/os/Bundle; J Z Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/filtershow/pipeline/ProcessingService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/print/IPrintManager$Stub$Proxy;-restartPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextClock;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/PhotoView$Model;-moveTo-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/datetimepicker/time/CircleView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransfer;-access$400-(Lcom/android/bluetooth/opp/BluetoothOppTransfer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationViewFragment;-renderSuperCollapsedBlock-(I I)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryDownloadProgressBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/text/format/DateUtils;-formatDateRange-(Landroid/content/Context; Ljava/util/Formatter; J J I)Ljava/util/Formatter;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-buildQueryUri-(I I Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/app/ExpandableListActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Ljava/net/URLConnection;-getInputStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-render-(Lcom/android/gallery3d/glrenderer/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/CalendarColorPickerDialog;-onCreateDialog-(Landroid/os/Bundle;)Landroid/app/Dialog;' : ['android.permission.READ_CALENDAR'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/LocationManagerService$Receiver;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/TransitionsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/text/ToUpperCase;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/AlarmManagerService$ResultReceiver;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/ProfileService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/LocationListener; Landroid/os/Looper; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/OnePaneController;-changeAccount-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$InactiveState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/net/wifi/WifiStateMachine$TetheredState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-removeStack-(I)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/photos/views/HeaderGridView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/NotificationActionIntentService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/NavTabScroller$ContentLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/WidgetConfigure;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/google/common/io/MultiReader;-skip-(J)J' : ['android.permission.INTERNET'],
	'Landroid/telephony/TelephonyManager;-listen-(Landroid/telephony/PhoneStateListener; I)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/CalendarViewAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/eas/EasSearch;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/ScrollingTabContainerView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/EventInfoActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AsyncPlayer;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/development/PackageSummary;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/MediaSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/ActionBarView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TableLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSettings;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/IBackupAgent$Stub;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/gatt/GattService;-startScanWithUuids-(I Z [Ljava/util/UUID;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/IWindowManager$Stub$Proxy;-isRotationFrozen-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$1;-onDeleteGroup-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/LocationClustering;-generateName-(Ljava/util/ArrayList; Lcom/android/gallery3d/util/ReverseGeocoder;)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/music/TrackBrowserActivity$4;-drop-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/gesture/GestureOverlayView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/ui/WebSearchSuggestionView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumDataLoader;-get-(I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/print/PrintManagerService$3;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/ColorButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/widget/BookmarkThumbnailWidgetService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorRectView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothGatt;-disconnect-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/support/v4/view/ViewPager;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/ConnectivityManager;-getTetherableIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/AddNewBookmark;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/settings/DebugSettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/pm/PackageManagerService;-killApplication-(Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasCertificateRequestor;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email2/ui/MailboxSelectionActivityEmail;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/OverlayTitleTemplatePicker;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub;-fetchRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/ex/photo/PhotoViewPager;-pageLeft-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-access$000-(Lcom/android/datetimepicker/date/SimpleMonthView; I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/RIL;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/StitchingChangeListener;-onStitchingQueued-(Landroid/net/Uri;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/AdapterView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnablingActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-getImsRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MusicBrowserActivity$1;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/development/MediaScannerActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/support/v4/widget/SearchViewCompatIcs$MySearchView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/opengl/GLSurfaceView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/keychain/KeyChainActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/am/Am$IntentReceiver;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/widget/CheckBox;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-onDestroy-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/view/menu/ExpandedMenuView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/BatteryStatsService;-getStatistics-()[B' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/VideoBrowserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/IConnectivityManager;-getActiveLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothHeadset;-getBatteryUsageHint-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/PieMenuButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider$4;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/PhotoView$FullPicture;-draw-(Lcom/android/gallery3d/glrenderer/GLCanvas; Landroid/graphics/Rect;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/Contacts$People;-addToMyContactsGroup-(Landroid/content/ContentResolver; J)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/timezonepicker/TimeZonePickerDialog;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/DockObserver;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodListLocked-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-getActiveLinkQualityInfo-()Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/InputMethodManagerService;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/MovieActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/ViewPager;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/print/IPrintManager$Stub;-destroyPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-sendVendorSpecificResultCode-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-updateServiceLocation-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/mock/MockApplication;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCallForward-(I I I Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothPbap;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/ConversationPagerAdapter;-restoreState-(Landroid/os/Parcelable; Ljava/lang/ClassLoader;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/view/ViewGroup;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/SelectCalendarsSyncFragment;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/deskclock/Screensaver;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowSession$Stub;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/widget/ActivityChooserView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/PrintManager;-getEnabledPrintServices-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/PickerActivity;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserActivity;-onSaveInstanceState-(Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/datetimepicker/AccessibleTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoTableDream;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothMap;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wm/WindowManagerService;-setForcedDisplaySizeLocked-(Lcom/android/server/wm/DisplayContent; I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-sendRequest-(J Lcom/android/emailsync/Request;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/BadBehaviorActivity$BadService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiService;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/SizeAdaptiveLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AddBookmarkPage;-setupTopCrumb-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/print/PrintManagerService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthTokenLabel-(Ljava/lang/String; Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/TwoPaneLayout;-access$000-(Lcom/android/mail/ui/TwoPaneLayout;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/content/Context;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/month/SimpleWeekView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/time/RadialTextsView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BookmarkSearch;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/text/format/DateFormat;-getDateFormat-(Landroid/content/Context;)Ljava/text/DateFormat;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/Calculator;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/musicvis/vis2/Visualization2;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothMap$Stub$Proxy;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnablingActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/provider/EmailProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/VideoUtils;-startMuteUsingMp4Parser-(Ljava/lang/String; Lcom/android/gallery3d/util/SaveVideoFileInfo;)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/util/IState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/ex/photo/PhotoViewPager;-setCurrentItemInternal-(I Z Z I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-handleConnect-(Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter$EasEmailSyncParser;-parse-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/alarmclock/DigitalAppWidgetProvider;-onAppWidgetOptionsChanged-(Landroid/content/Context; Landroid/appwidget/AppWidgetManager; I Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EasTestAuthenticatorService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/deskclock/AlarmClockFragment;-access$4600-(Landroid/content/Context; Lcom/android/deskclock/provider/Alarm;)Lcom/android/deskclock/provider/AlarmInstance;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-stopBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/email/EmailApplication;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapService$MapBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-access$100-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/development/Connectivity;-onRoutedHttpRequest-()V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/widget/ScrollingTabContainerView$TabView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/polarclock/PolarClockSettings;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processPlayTone-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/ZoomControl;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/photo/util/ImageUtils;-decodeStream-(Lcom/android/ex/photo/util/ImageUtils$InputStreamFactory; Landroid/graphics/Rect; Landroid/graphics/BitmapFactory$Options;)Landroid/graphics/Bitmap;' : ['android.permission.INTERNET'],
	'Landroid/filterfw/core/FilterSurfaceView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$SoftApStartingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/printspooler/NotificationController$NotificationBroadcastReceiver;-handleRestartPrintJob-(Landroid/content/Context; Landroid/print/PrintJobId;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/GrainEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/IConnectivityManager;-setMobileDataEnabled-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/net/wifi/WifiWatchdogStateMachine;-registerForSettingsChanges-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-access$200-(Lcom/android/server/accessibility/ScreenMagnifier;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController;-delete-(I Ljava/util/Collection; Lcom/android/mail/ui/DestructiveAction; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/ConversationViewAdapter;-getView-(Lcom/android/mail/browse/ConversationOverlayItem; Landroid/view/View; Landroid/view/ViewGroup; Z)Landroid/view/View;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView$2;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/StackView$StackFrame;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BackupManagerService$ActiveRestoreSession;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/CertBlacklister;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/MediaPlayer;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/GridView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/galaxy/GalaxyWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/IConnectivityManager$Stub;-getTetherableWifiRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/media/effect/effects/VignetteEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-getWifiState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/wallpaper/galaxy/GalaxyView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationViewFragment;-measureOverlayHeight-(Lcom/android/mail/browse/ConversationOverlayItem;)I' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-fakeChange-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/VideoView;-stopPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/PhoneUi;-setUseQuickControls-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/vis1/Visualization1;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/widget/ContentLoadingProgressBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/FeedbackActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/content/ContentResolver;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/music/AudioPreview;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/phasebeam/PhaseBeam;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/WiredAccessoryManager;-bootCompleted-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService$AudioHandler$2;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity;-eventsChanged-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothGatt;-unregisterClient-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserPreferencesPage;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-explicitCallTransfer-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/alerts/SnoozeAlarmsService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/IConsumerIrService;-getCarrierFrequencies-()[I' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/internal/app/PlatLogoActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/smspush/WapPushManager;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Landroid/speech/tts/TextToSpeech;-access$1700-(Landroid/speech/tts/TextToSpeech;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/EasAccountService;-acknowledgeProvision-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/compose/ComposeAttachmentTile;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserSettings;-clearHistory-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/app/Presentation$1;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingRfcommWithServiceRecord-(Ljava/lang/String; Ljava/util/UUID;)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/CacheAbuser;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/SuperCollapsedBlock;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMSIForApp-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothMap;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-updateMonthHighlight-(Landroid/widget/AbsListView;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/emailcommon/service/IEmailService;-sendMeetingResponse-(J I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiStateMachine$DisconnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/WebViewController;-showSslCertificateOnError-(Landroid/webkit/WebView; Landroid/webkit/SslErrorHandler; Landroid/net/http/SslError;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService;-performSync-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR'],
	'Landroid/provider/Settings$Secure;-getLong-(Landroid/content/ContentResolver; Ljava/lang/String;)J' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStartingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/AbsSpinner;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/MultiStateButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getLinkQualityInfo-(I)Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/LogTextBox;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/Pico;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/view/ViewPager;-smoothScrollTo-(I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/speech/tts/TextToSpeech;-setLanguage-(Ljava/util/Locale;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasProvision;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calculator2/CalculatorEditText;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/google/common/io/CharStreams$2;-getInput-()Ljava/io/InputStreamReader;' : ['android.permission.INTERNET'],
	'Landroid/accounts/CantAddAccountActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IIntentReceiver$Stub$Proxy;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/PhoneBase;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/EasTestAuthenticatorService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getContactNamesByNumber-(Ljava/lang/String;)Ljava/util/ArrayList;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/deskclock/Screensaver;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/DayPickerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageAttachmentTile;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-dispatchMediaKeyWithWakeLock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/ui/FramedTextButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AndroidClockTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailcommon/service/IEmailServiceCallback$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/videoeditor/VideoEditorBaseActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasOptions;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/exchange/eas/EasSync;-getRequestUri-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/gadget/MediaSetSource;-<init>-(Lcom/android/gallery3d/data/DataManager; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-updateFolderList-(J)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/widget/LockPatternUtils;-isPatternEverChosen-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/EmojiCategoryPageIndicatorView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationsInOutboxTipView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/DismissAlarmsService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/TitleBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/DataManager;-onStitchingResult-(Landroid/net/Uri;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/AlarmInitReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/SettingsRedirector;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FlipFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/statusbar/IStatusBarService;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/filterpacks/base/OutputStreamTarget;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPickerActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/SyncActivityTooManyDeletes;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-updateProgressbar-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver$1;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/content/PackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/Calculator;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-setGsmBroadcastConfig-([Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothMap;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiStateMachine;-retrieveBatchedScanData-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/SearchActivity$2;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onProgressComplete-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarApplication;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/latin/BackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/SupplicantStateTracker$DisconnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/development/Connectivity;-access$3800-(Lcom/android/development/Connectivity;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-removeWindowInnerLocked-(Lcom/android/server/wm/Session; Lcom/android/server/wm/WindowState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-systemReady-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/eas/EasSearch;-handleProvisionError-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/media/AudioManager;-querySoundEffectsEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/RefreshStatusMonitor;-isConnected-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/Phone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/KenBurnsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/View;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/DatePickerDialog;-setCurrentView-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BluetoothManagerService;-waitForOnOff-(Z Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/printspooler/PrintSpoolerService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/accessibility/AccessibilityEntityProvider;-createAccessibilityNodeInfo-(I)Landroid/support/v4/view/accessibility/AccessibilityNodeInfoCompat;' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/widget/ToggleButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/noisefield/NoiseFieldWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/RadioButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/LocationManagerService;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/ui/MailActivity;-onBackPressed-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/EmailApplication;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/EventViewer;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/eas/EasMoveItems;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/ConversationContainer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/LegacyEasAuthenticatorService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onDeleteImage-(Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/videoeditor/widgets/PlayheadView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/gesture/GestureOverlayView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/EmailApplication;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Preloader;-handlePreloadRequest-(Ljava/lang/String; Ljava/lang/String; Ljava/util/Map; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_CALENDAR', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/setup/SetupActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;-onProgressStart-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/QuickContactBadge;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/VideoEditorBaseActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetooth;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/alerts/SnoozeAlarmsService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-getSignalStrength-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/accessibility/AccessibilityUtils;-onStartInputViewInternal-(Landroid/view/View; Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/QuickResponseActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/RecognitionService;-checkPermissions-(Landroid/speech/IRecognitionListener;)Z' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onRadioOffOrNotAvailable-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-changeAccount-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accounts/AccountManager;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/widget/ActionBarView$HomeView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/ContextWrapper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-trySwitchToFirstTouchState-(F F)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/monkey/MonkeyNetworkMonitor;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/OnePaneController;-commitDestructiveActions-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/CalendarSettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/AbstractSettingPopup;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/ViewPager;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/AddBookmarkPage;-onRootFolderFound-(J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Ljava/net/URL;-openStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Landroid/widget/RatingBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothPan;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/sip/ISipService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/bluetooth/IBluetoothPan;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-acknowledgeIncomingGsmSmsWithPdu-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setAndBroadcastNetworkSetTime-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/view/SnapshotGridView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/magicsmoke/MagicSmokeSelector;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/vcard/VCardEntry;-isIgnorable-()Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary$13;-run-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-render-(Lcom/android/gallery3d/glrenderer/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionaryAddWordContents;-delete-(Landroid/content/Context;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/vcard/VCardEntry$EmailData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/accounts/AccountManager;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/event/EditEventHelper;-saveRemindersWithBackRef-(Ljava/util/ArrayList; I Ljava/util/ArrayList; Ljava/util/ArrayList; Z)Z' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/AlbumPage;-doCluster-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventFragment;-access$800-(Lcom/android/calendar/event/EditEventFragment; I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/videoeditor/widgets/MediaItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/StraightenEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onSetInternalDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/packageinstaller/TabsAdapter;-<init>-(Landroid/app/Activity; Landroid/widget/TabHost; Landroid/support/v4/view/ViewPager;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/accessibility/TouchExplorer$SendHoverEnterAndMoveDelayed;-forceSendAndRemove-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/ShowActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/AddBookmarkPage;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/musicvis/vis2/Visualization2;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/TrackBrowserActivity;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStoppingState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/vcard/VCardParserImpl_V40;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/widget/AbsSpinner;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-onMenuClicked-(Landroid/view/MenuItem; Ljava/lang/String; Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-msgListing-(Ljava/lang/String; Lcom/android/bluetooth/map/BluetoothMapAppParams;)Lcom/android/bluetooth/map/BluetoothMapMessageListing;' : ['android.permission.READ_CONTACTS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/effect/effects/NegativeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/OtherPreferences;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-getPrintJobInfos-(I I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth;-getAdapterConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityManagerService;-makeAppNotRespondingLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/MediaFocusControl;-startVoiceBasedInteractions-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-addNmeaListener-(Landroid/location/GpsStatus$NmeaListener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/os/IHardwareService;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/calendar/CalendarApplication;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IWallpaperManager$Stub;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-onLoadComplete-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/mail/browse/SpamWarningView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/FilmstripPage;-access$400-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/compose/AutoSendActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/MultiAutoCompleteTextView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-sendRetrySms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/camera/ShutterButton;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/am/Am;-runKillAll-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/email/service/ImapService;-synchronizeMailboxSynchronous-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Mailbox; Z Z)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/FolderSelectionActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/BrightnessEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/RotationPolicy;-isRotationLocked-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/EmailProvider;-uiUpdateFolder-(Landroid/content/Context; Landroid/net/Uri; Landroid/content/ContentValues;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/exchange/service/EasServerConnection;-makeUriString-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wm/InputMonitor;-dispatchUnhandledKey-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hid/HidService;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/display/DisplayManagerService;-setWindowManager-(Lcom/android/server/display/DisplayManagerService$WindowManagerFuncs;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/QsbApplicationWrapper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/widget/CheckedTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupStepIndicatorView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/PhoneProxy;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothPbap$Stub$Proxy;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$WatchdogDisabledState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/email2/ui/MailActivityEmail;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-switchWaitingOrHoldingAndActive-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserBookmarksPage;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/ConnectivityManager;-startUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/camera/crop/CropActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-getIMEI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasOutboxSync;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityStackSupervisor;-ensureActivitiesVisibleLocked-(Lcom/android/server/am/ActivityRecord; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/gatt/GattService;-sendNotification-(I Ljava/lang/String; I I Ljava/util/UUID; I Ljava/util/UUID; Z [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/location/GeofenceManager;-updateFences-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/GLRootView;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/support/v7/widget/Space;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-dispatchNormalMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/content/SyncActivityTooManyDeletes;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/RotateImageView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/NetInitiatedActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-handleMsgListChangesMms-()V' : ['android.permission.READ_SMS'],
	'Lcom/android/inputmethod/latin/LatinIME;-onFinishInputView-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/service/ImapService;-synchronizeMailboxGeneric-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Mailbox; Z Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/WebVttRenderingWidget$SpanLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/CoreSettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/VerticalTextSpinner;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardEntry$PostalData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setPasskey-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/nexus/NexusRS;-start-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/SubtypeSwitcher;-initialize-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/quicksearchbox/SuggestionCursorProvider;-getSuggestions-(Ljava/lang/String; I)Lcom/android/quicksearchbox/SuggestionCursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onChanged-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/method/QwertyKeyListener;-onKeyDown-(Landroid/view/View; Landroid/text/Editable; I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/app/PickerActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/CameraActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/UiAutomationConnection;-connect-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-addGpsStatusListener-(Landroid/location/IGpsStatusListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MusicBrowserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/CaptivePortalTracker$NoActiveNetworkState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/content/MutableContextWrapper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/SyncRunner;-step-()I' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/deskclock/AlarmClockFragment$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertService;-postNotification-(Lcom/android/calendar/alerts/AlertService$NotificationInfo; Ljava/lang/String; Landroid/content/Context; Z Lcom/android/calendar/alerts/AlertService$NotificationPrefs; Lcom/android/calendar/alerts/NotificationMgr; I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/research/FeedbackLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/magicsmoke/MagicSmokeSelector;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/os/BatteryStatsImpl;-getBluetoothPingCount-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/support/v4/widget/SlidingPaneLayout$AccessibilityDelegate;-performAccessibilityAction-(Landroid/view/View; I Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/selectcalendars/CalendarColorSquare;-assignContactFromEmail-(Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/pinyin/ComposingView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-handleResponse-(Lcom/android/exchange/EasResponse;)I' : ['android.permission.INTERNET', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/widget/SubtitleView;-setStyle-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/state/StatePanelTrack;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupStepIndicatorView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/ScrollerView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-checkManageAccountsPermission-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onCleanUpAllConnections-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/event/EditEventView;-onRecurrenceSet-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub;-setRadio-(I Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/colorpicker/ColorPickerSwatch;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager;-getTetherableBluetoothRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/BackupManagerService;-prepareOperationTimeout-(I J Lcom/android/server/BackupManagerService$BackupRestoreTask;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/widget/SlidingDrawer;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/vcard/VCardParserImpl_V40;-parseItem-()Z' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/app/Presentation$1;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/ContextThemeWrapper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/ContentService;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/DigitalClock;-initClock-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/TrafficStats;-getDataLayerSnapshotForUid-(Landroid/content/Context;)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/ui/Switch;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService$SettingsObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/camera/WideAnglePanoramaModule$7;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/TabScrollView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/widget/RatingBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Tab$3;-access$1700-(Lcom/android/browser/Tab$3; Z Landroid/os/Message;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/dictionarypack/DownloadOverMeteredDialog;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/ex/photo/PhotoViewActivity;-onLoadFinished-(Landroid/support/v4/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/camera/PhotoModule;-onResumeAfterSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/AsyncQueryServiceHelper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSetupBasics;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-registerAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/IBluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/LogTextBox;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHealth;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorSaturationView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-sendEmergencyCallbackModeChange-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accessibilityservice/AccessibilityService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$800-(Lcom/android/server/net/NetworkPolicyManagerService;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/ex/photo/PhotoViewActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/TabActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/widget/sgv/StaggeredGridView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/FlipperDream;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView$DeleteKeyOnTouchListener$DummyRepeatKeyRepeatTimer;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImagePoint;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/LeaveBehindItem;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-systemReady-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/printspooler/NotificationController$NotificationBroadcastReceiver$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionaryAddWordFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/internal/widget/ActionBarContextView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavTabScroller;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/adapter/ItemOperationsParser;-parseFetch-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/CountdownTimerPopup;-updateInputState-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/filtershow/crop/CropView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-connectSocket-(Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/research/UploaderService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IActivityManager;-handleApplicationWtf-(Landroid/os/IBinder; Ljava/lang/String; Landroid/app/ApplicationErrorReport$CrashInfo;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/KeyboardView;-access$400-(Landroid/inputmethodservice/KeyboardView; Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-getMmsMessage-(J Lcom/android/bluetooth/map/BluetoothMapAppParams;)[B' : ['android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/mail/ui/SwipeableListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/EmailServiceUtils;-setupAccountManagerAccount-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Z Z Z Landroid/accounts/AccountManagerCallback;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/filterpacks/base/InputStreamSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/DownloadManager;-addCompletedDownload-(Ljava/lang/String; Ljava/lang/String; Z Ljava/lang/String; Ljava/lang/String; J Z Z)J' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/performance/ThroughputFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MusicBrowserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/ComboAlbum;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/ProfileService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/YearPickerView;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasFolderSync;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionaryAddWordFragment;-updateSpinner-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_PROFILE'],
	'Landroid/view/IApplicationToken$Stub;-windowsVisible-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCdmaBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-access$400-(Lcom/android/bluetooth/btservice/BondStateMachine; Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/quicksearchbox/ui/ClusteredSuggestionsView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/TabActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/widget/CalendarAppWidgetModel;-populateDayInfo-(I Landroid/text/format/Time;)Lcom/android/calendar/widget/CalendarAppWidgetModel$DayInfo;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-dumpMmsTable-()V' : ['android.permission.READ_SMS'],
	'Lcom/android/mail/ui/AbstractActivityController;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/adapter/ItemOperationsParser;-parseResponse-()V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/location/ILocationManager;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/research/FeedbackLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-pushModify-(J)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/view/IWindowSession$Stub$Proxy;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BackupManagerService$BackupRestoreTask;-handleTimeout-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/support/v4/app/NoSaveStateFrameLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onAlbumModeSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/WifiService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_MULTICAST_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/BrowserBackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/BitmapSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/preferences/BandwidthPreferencesFragment;-onResume-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onSingleTapUp-(I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/adapter/AbstractSyncParser;-commandsParser-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Landroid/print/IPrintManager$Stub$Proxy;-stopPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/MailboxSettings;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/TextServicesManagerService;-setCurrentSpellCheckerLocked-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-explicitCallTransfer-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TextServicesManagerService$TextServicesSettings;-getSelectedSpellCheckerSubtype-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothInputDevice;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityManagerService;-main-(I)Landroid/content/Context;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/RotateImageView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/crop/CropActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/FilmStripView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-attachApplicationLocked-(Lcom/android/server/am/ProcessRecord; Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-updateMonthHighlight-(Landroid/widget/AbsListView;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/eas/EasOperation;-getRequestEntity-()Lorg/apache/http/HttpEntity;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-dispatchBroadcast-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-handleSmsMessage-(Landroid/os/Message;)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/mail/ui/SwipeableListView;-dismissChild-(Lcom/android/mail/browse/ConversationItemView;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/speech/tts/TextToSpeechService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService$AudioHandler;-cleanupPlayer-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-handleUp-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/wallpaper/fall/FallWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumDataLoader$UpdateContent;-call-()Ljava/lang/Void;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/VibratorService;-doVibratorOff-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/noisefield/NoiseField;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/RenderOverlay$RenderView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AbsSpinner;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/EventRedirectingFrameLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendRetrySms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-handleExitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IActivityManager;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/internal/telephony/RIL;-explicitCallTransfer-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/widget/PopupWindow$PopupViewContainer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/InputStreamSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-doStop-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/camera/ui/CountdownTimerPopup;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/widget/TableLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/DeviceStorageMonitorService;-sendNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageCrop;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setDiscoverableTimeout-(I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-isProviderEnabled-(Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/CalendarController;-refreshCalendars-()V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calculator2/ColorButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onTrySetupData-(Lcom/android/internal/telephony/dataconnection/ApnContext;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager;-onUserStarting-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/TextServicesManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/magicsmoke/MagicSmoke;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/keychain/KeyChainActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/DatePicker;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/views/HeaderGridView$FullWidthFixedViewLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/Pop3AuthenticatorService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothAdapter;-startLeScan-(Landroid/bluetooth/BluetoothAdapter$LeScanCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/calendar/DayFragment;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/music/MediaPlaybackActivity$5;-onClick-(Landroid/view/View;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionaryAddWordContents;-getLocalesList-(Landroid/app/Activity;)Ljava/util/ArrayList;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseAccountActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexClientSession;-start-(Landroid/os/Handler; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/ui/DefaultSuggestionView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Editor$InsertionHandleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService$SyncAdapterImpl;-onPerformSync-(Landroid/accounts/Account; Landroid/os/Bundle; Ljava/lang/String; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR'],
	'Landroid/preference/PreferenceFrameLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/LocationManagerService$Receiver;-callStatusChangedLocked-(Ljava/lang/String; I Landroid/os/Bundle;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCallForwardStatus-(I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager;-canAutoSync-(Lcom/android/emailcommon/provider/Account;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-removeFence-(Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-tearDownData-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/views/HeaderGridView;-sequenceScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/PieController$2;-onClick-(Lcom/android/camera/ui/PieItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MusicPicker;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-onRestoreInstanceState-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/compose/CcBccView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/CreatePlaylist;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/CheckedLinearLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/backup/LocalTransportService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Contacts$ContactMethods;-addPostalLocation-(Landroid/content/Context; J D D)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/BackupAgentHelper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BaseUi$FullscreenHolder;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/TestActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/QuickContactBadge;-assignContactFromEmail-(Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/media/WebVttRenderer;-createTrack-(Landroid/media/MediaFormat;)Landroid/media/SubtitleTrack;' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/PagerTabStrip$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/photos/PhotoSetFragment;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/OutputStreamTarget;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IBackupAgent;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/app/Presentation$1;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/CallLog$Calls;-getLastOutgoingCall-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_CALL_LOG'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/EdgeView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/datetimepicker/date/DatePickerDialog;-onYearSelected-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NsdService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.INTERNET'],
	'Lcom/android/server/LockSettingsService;-getBoolean-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/wallpaper/WallpaperService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/IBatteryStats;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/view/BasePieView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-updateNetworkRulesLocked-()V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/TwoPaneController;-registerDestructiveAction-(Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStack;-activityPausedLocked-(Landroid/os/IBinder; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/CalendarController;-sendEventRelatedEvent-(Ljava/lang/Object; J J J J I I J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorOpacityView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/FlipperDream;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothMap$Stub$Proxy;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onCodeInput-(I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$DefaultState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/fall/FallWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/AddNewBookmark;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/widget/TextTime;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CheckBox;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processSetUpIdleModeText-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/CaptioningManager;-getLocale-()Ljava/util/Locale;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSyncBase;-getRequestUri-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/latin/SubtypeSwitcher;-isShortcutImeEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardComposer;-init-(Ljava/lang/String; [Ljava/lang/String;)Z' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/settings/DebugSettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/development/Development;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-pollState-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/widget/RadioButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IPowerManager$Stub$Proxy;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationWebView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintJobConfigActivity$CustomEditText;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-disableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/input/InputManagerService;-interceptKeyBeforeDispatching-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)J' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub$Proxy;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/LatinIME;-onStartInput-(Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/datetimepicker/date/TextViewWithCircularIndicator;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/GlobalActions$GlobalActionsDialog;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/BookmarkSearch;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/AccountManager;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryProvider;-deleteDataFile-(Landroid/net/Uri;)I' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageShow;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumPicker;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/alarmclock/DigitalWidgetViewsFactory$RemoteWorldClockAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/IBackupManager;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/MediaLinearLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/ColorChipView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/ViewPager;-setPageTransformer-(Z Landroid/support/v4/view/ViewPager$PageTransformer;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/MessageScrollView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/GridLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ActionableToastBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService$10;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/AnimatedAdapter$4;-run-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calculator2/CalculatorDisplay;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$AudioOn;-processAudioEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/BatteryService$BatteryListener;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/timer/TimerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine;-<init>-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/NavigationBarTablet;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/time/TimePickerDialog$1;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService$ConnectivityReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-access$800-(Lcom/android/gallery3d/app/MoviePlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RotateFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/SecureAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/TextClock;-access$000-(Landroid/widget/TextClock;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(J F Landroid/location/Criteria; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothA2dp;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/ImmersiveModeConfirmation$ClingWindowView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoTableDream;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarController$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/ex/photo/PhotoViewPager;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/content/SyncStorageEngine;-requestSync-(Landroid/accounts/Account; I I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/os/BatteryStatsImpl;-getCurrentBluetoothPingCount-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/CountdownTimerPopup$Listener;-onListPrefChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserBackupAgent;-onRestore-(Landroid/app/backup/BackupDataInput; I Landroid/os/ParcelFileDescriptor;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/notification/DemoContactNotificationScorer;-getScore-(Landroid/app/Notification; I)I' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/AttachmentDownloadService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-createServerSession-(Ljavax/obex/ObexTransport;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-setShuffleMode-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SlotView$Listener;-onLongTap-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/SecureCameraActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/photos/FullscreenViewer;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/Editor$SelectionEndHandleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/content/PackageMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-inputDispatchingTimedOut-(I Z Ljava/lang/String;)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/printspooler/FusedPrintersProvider;-setTrackedPrinter-(Landroid/print/PrinterId;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BackupManagerService$PerformFullBackupTask;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/FolderSelectionDialog;-onListItemClick-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/BackupManagerService$PerformFullRestoreTask$RestoreFileRunnable;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/connectivity/PacManager$1;-run-()V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/widget/DialogTitle;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TabHost;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/AllInOneActivity;-initFragments-(J I Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/opp/BluetoothOppLiveFolder;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/gadget/WidgetClickHandler;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-getCurrentDeviceName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/AppHwPref;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/OverlayTitleEditor;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/event/AttendeesView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavTabScroller;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/widget/TextTime;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothA2dp;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/gatt/GattService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ImageView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/InLineSettingItem;-changeIndex-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/service/EasService$RestartPingsTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/vcard/VCardEntry;-toString-()Ljava/lang/String;' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/media/AudioService;-restoreMasterVolume-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity;-onCreate-(Landroid/os/Bundle; Landroid/content/Intent; Ljava/lang/CharSequence; [Landroid/content/Intent; Ljava/util/List; Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-beginReliableWrite-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/MessageHeaderView;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetooth;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/alerts/AlertActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-configHciSnoopLog-(Z)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/base/FrameFetch;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier; Landroid/net/sip/SipProfile;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiService;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/email/service/LegacyEasAuthenticatorService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PickerActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-setAlphabetShiftLockShiftedKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationCursor$2;-run-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/MailWebView;-onScrollChanged-(I I I I)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/content/PackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-addIncludedService-(I I I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/SupplicantStateTracker$DefaultState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/am/ActiveServices;-killServicesLocked-(Lcom/android/server/am/ProcessRecord; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu;-onSetEmpty-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/EasCertificateRequestor;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DayView$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/BluetoothManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/DhcpStateMachine$WaitBeforeRenewalState;-exit-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/view/CertificateSelector;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-loadAttachment-(Lcom/android/emailcommon/service/IEmailServiceCallback; J J Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/view/ViewStub;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHeadset;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/NetworkStatsService;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/am/ActivityManagerService;-handleApplicationCrash-(Landroid/os/IBinder; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventView;-access$500-(Lcom/android/calendar/event/EditEventView;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-onReleaseKey-(I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dialInternal-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/browser/NavScreen$1;-onRemovePosition-(I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/ui/RecentFolderList$1;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setPasskey-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/pinyin/PinyinDecoderService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DayView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/AttachmentDownloadService;-watchdogAlarm-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/AbstractActivityController;-onViewModeChanged-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/activity/setup/AccountSetupBasics;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/adapter/ContactsSyncParser;-changeParser-(Lcom/android/exchange/adapter/ContactsSyncParser$ContactOperations;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/widget/Editor$SelectionStartHandleView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/ArtistAlbumBrowserActivity;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowAnimator$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MusicPicker;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-isTetheringSupported-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/ActivityController;-onAnimationEnd-(Lcom/android/mail/ui/AnimatedAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-notifyOffApnsOfAvailability-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager;-maybeRescheduleSync-(Landroid/content/SyncResult; Lcom/android/server/content/SyncOperation;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-onCancelled-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoTableDream;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/IAudioService$Stub$Proxy;-reloadAudioSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$IndicatorView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/SwipeableListView;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/accessibility/TouchExplorer;-clear-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/InLineSettingMenu;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hdp/HealthService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/Activity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/AccountService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/EasAutoDiscover;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothMap$Stub;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/tts/TextToSpeech$Connection$SetupConnectionAsyncTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/google/common/io/CharStreams;-readFirstLine-(Lcom/google/common/io/InputSupplier;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/server/location/GpsLocationProvider$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/ui/CameraControls;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaScannerClient;-scanFile-(Ljava/lang/String; J J Z Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/CreatePlaylist;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageRotate;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-sendNotification-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; Z [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/service/AccountService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/datetimepicker/date/DayPickerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/EmptyConversationListView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/info/HistogramView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/SystemAllowGeolocationOrigins;-getSystemSetting-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/DeleteItems;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/dictionarypack/WordListPreference;-disableDict-()V' : ['android.permission.INTERNET'],
	'Lcom/android/browser/TabBar$TabView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hdp/HealthService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationSyncDisabledTipView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumPicker;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/MediaSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/CalendarSettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/development/LogViewer;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/eas/EasPing;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/widget/RelativeLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/power/PowerManagerService;-applyWakeLockFlagsOnAcquireLocked-(Lcom/android/server/power/PowerManagerService$WakeLock;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onResetDone-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/AbstractActivityController;-requestFolderRefresh-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-onContentDirty-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/music/MediaPlaybackService;-removeTracksInternal-(I I)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/providers/downloads/ui/TrampolineActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/effect/effects/AutoFixEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/email/service/Pop3AuthenticatorService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperPreview;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/CreatePlaylist;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/ResolverActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/SwipeableConversationItemView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/WallpaperManagerService$MyPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-cancel-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/TimelineRelativeLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncStorageEngine;-clearAndReadState-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothAdapter;-startAdvertising-(Landroid/bluetooth/BluetoothAdapter$AdvertiseCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/timezonepicker/TimeZoneData;-<init>-(Landroid/content/Context; Ljava/lang/String; J)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub;-getAllNetworkInfo-()[Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardParserImpl_V21;-parseItems-()V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment$MessageLoaderCallbacks;-onLoadFinished-(Landroid/content/Loader; Lcom/android/mail/content/ObjectCursor;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/content/IContentService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/browser/DeviceAccountLogin;-handleLogin-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/mail/ui/OnePaneController;-access$2800-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/content/ObjectCursor;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-onProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/AbstractActivityController;-switchToDefaultInboxOrChangeAccount-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/inputmethodservice/ExtractEditLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/AttendeesView;-access$100-(Lcom/android/calendar/event/AttendeesView; Lcom/android/calendar/event/EditEventHelper$AttendeeItem;)Landroid/view/View;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/email/EmailApplication;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/WebVttRenderingWidget$CueLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-loadAccountInbox-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/svox/pico/DownloadVoiceData;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/SecureCameraActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker$2;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/gallery3d/data/ClusterAlbum$2;-consume-(I Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/view/menu/ActionMenuPresenter$OverflowMenuButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/BatteryStatsService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/AddBookmarkPage$EditBookmarkInfoLoader;-loadInBackground-()Ljava/lang/Object;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/BluetoothManagerService;-access$1800-(Lcom/android/server/BluetoothManagerService; Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/display/DisplayManagerService;-setInputManager-(Lcom/android/server/display/DisplayManagerService$InputManagerFuncs;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EventColorPickerDialog;-onCreateDialog-(Landroid/os/Bundle;)Landroid/app/Dialog;' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendActionDownAndUp-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-access$100-(Lcom/android/bluetooth/btservice/AdapterService; Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/SnailSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/TabBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ReceiverRestrictedContext;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/EasSyncService;-messageMoveRequest-(Lcom/android/exchange/MessageMoveRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$DriverStartingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/grass/GrassWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/pinyin/CandidateView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/MailLogService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/gatt/GattService;-readDescriptor-(I Ljava/lang/String; I I Ljava/util/UUID; I Ljava/util/UUID; I Ljava/util/UUID; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/settings/SettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController;-initializeAndRegisterForSettingsChange-(Landroid/os/Looper;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/NavScreen;-openNewTab-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/musicvis/vis3/Visualization3;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email2/ui/MailActivityEmail;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/photo/PhotoViewActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/UiModeManagerService;-sendConfigurationAndStartDreamOrDockAppLocked-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleEctIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-setSenderName-(Lcom/android/bluetooth/map/BluetoothMapMessageListingElement; Landroid/database/Cursor; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo; Lcom/android/bluetooth/map/BluetoothMapAppParams;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/net/wifi/WifiStateMachine$DisconnectingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-autoConnectA2dp-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/SlotView$MyGestureListener;-onSingleTapUp-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/settings/GallerySettings;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/time/RadialTextsView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/CreateShortcutActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/chips/RecipientEditTextView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-<init>-(J Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/camera/PanoProgressBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/DayPickerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/service/IEmailService;-deleteAccountPIMData-(Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-trySwitchToFirstTouchState-(F F)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-clearLock-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarController;-sendEventRelatedEventWithExtraWithTitleWithCalendarId-(Ljava/lang/Object; J J J J I I J J Ljava/lang/String; J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/location/LocationManager;-getLastKnownLocation-(Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/app/IWallpaperManager;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSetupType;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-startScan-(I Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/net/NetworkStatsService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/EmailConnectivityManager;-waitForConnectivity-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-tryUpdate-(Landroid/content/Context; Z)Z' : ['android.permission.INTERNET'],
	'Landroid/widget/RemoteViewsAdapter$RemoteViewsFrameLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NsdService$NsdStateMachine$EnabledState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/display/DisplayManagerService;-access$1400-(Lcom/android/server/display/DisplayManagerService; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/InLineSettingMenu;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/VideoView$2;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BookmarkSearch;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/compose/CcBccView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-setState-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/os/Vibrator;-vibrate-(J)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/sip/SipService$SipSessionGroupExt;-onConnectivityChanged-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/Recorder;-startPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-access$4100-(Lcom/android/server/pm/PackageManagerService; Ljava/lang/String; I I)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/gadget/WidgetSource;-reload-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppIncomingFileConfirmActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/google/android/mms/pdu/PduPersister;-loadRecipients-(I Ljava/util/HashSet; Ljava/util/HashMap; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/compose/EmptyService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/NullFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/AlbumSlidingWindow;-setActiveWindow-(I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/ProcessInfo;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchToHitPicture-(I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/EasOutboxService;-acknowledgeProvisionImpl-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String; Z)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-crashApplication-(Lcom/android/server/am/ProcessRecord; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity$1;-onClick-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryDownloadProgressBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/agenda/AgendaListView$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/ExchangeService;-startServiceThread-(Lcom/android/emailsync/AbstractSyncService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-handleShift-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$5200-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; Ljava/lang/String;)V' : ['android.permission.READ_CALL_LOG'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-msgListingHasUnread-(Ljava/lang/String; Lcom/android/bluetooth/map/BluetoothMapAppParams;)Z' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-hangupForegroundResumeBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationPhotoTeaserView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-iccIOForApp-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ListActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/SyncRunner;-processFilterNode-(Landroid/filterfw/core/Filter;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/browser/AddBookmarkPage;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/SipManager;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollKeyboardView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/InputView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-access$1300-(Lcom/android/server/LocationManagerService; Lcom/android/server/LocationManagerService$Receiver;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-finalizeBackup-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaFocusControl;-filterVoiceInputKeyEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ResolverActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/SubtitleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothTetheringDataTracker;-startMonitoring-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/ArtistAlbumBrowserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setTimeFromNITZString-(Ljava/lang/String; J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Landroid/filterpacks/performance/ThroughputFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage$4;-onLongTap-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/alerts/QuickResponseActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/recurrencepicker/RecurrencePickerDialog;-onItemSelected-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/LatinIME;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/ShutdownThread;-beginShutdownSequence-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/app/backup/BackupAgentHelper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneProxy;-deleteAndCreatePhone-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/emailcommon/service/PolicyServiceProxy;-isActive-(Landroid/content/Context; Lcom/android/emailcommon/provider/Policy;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/musicvis/vis3/Visualization3;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TableLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-getActivePasswordQuality-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onPressKey-(I Z I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/LockPatternUtils;-setBiometricWeakLivelinessEnabled-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/TimelineHorizontalScrollView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DateTimeView$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/SearchActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-reportSmsMemoryStatus-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/CaptioningManager;-notifyFontScaleChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/DialogPicker;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/text/StringLogger;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NsdService;-create-(Landroid/content/Context;)Lcom/android/server/NsdService;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/accessibility/TouchExplorer$DoubleTapDetector;-onMotionEvent-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/ComposeActivity$SendOrSaveTask;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/app/IActivityManager;-handleApplicationCrash-(Landroid/os/IBinder; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-getProviders-(Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/TextServicesManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AnalogClock;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/TouchExplorer;-access$3000-(Lcom/android/server/accessibility/TouchExplorer; Landroid/view/MotionEvent; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/email/activity/setup/AccountSetupIncomingFragment;-setCallback-(Lcom/android/email/activity/setup/AccountServerBaseFragment$Callback;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/tts/compat/CompatTtsService;-getSettingsLocale-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/dreams/phototable/CursorPhotoSource;-fillQueue-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MusicBrowserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-initSstIcc-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-sendDelayedMotionEvents-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker$CustomEditText;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-killApplicationWithAppId-(Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-updateAudioSourceFunction-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-isNetworkSupported-(I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorValueView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-replacePackageLI-(Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-startEmojiPalettes-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/AbstractActivityController$18;-onPostExecute-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/musicvis/vis2/Visualization2;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TabHost;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-showInputMethodMenu-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/DownloadManager;-getUriForDownloadedFile-(J)Landroid/net/Uri;' : ['android.permission.INTERNET'],
	'Lcom/android/server/am/BroadcastQueue;-setBroadcastTimeoutLocked-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/camera/PhotoModule;-onSharedPreferenceChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/BatteryStatsService;-setBatteryState-(I I I I I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/support/v4/print/PrintHelperKitkat;-printBitmap-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BackupManagerService;-bindToAgentSynchronous-(Landroid/content/pm/ApplicationInfo; I)Landroid/app/IBackupAgent;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/WifiManager;-disconnect-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/NativeCrashListener$NativeCrashReporter;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-initializeHdmiState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-resumeGeofence-(I I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-unregisterForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-copyMessageToIccEf-(Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/filtershow/crop/CropView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/service/ApiService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/pinyin/BalloonHint$BalloonView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CheckedTextView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorViewPager;-executeKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-loadProvidersLocked-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/PhotoView$MyGestureListener;-onSingleTapUp-(F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/sip/SipService$SipKeepAliveProcessCallback;-onResponse-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/EllipsizedMultilineTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/SearchActivity;-onQueryTextSubmit-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/galaxy4/Galaxy4;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/content/IIntentReceiver$Stub;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/location/ILocationManager$Stub;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/AlarmManagerService;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment;-onPause-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$VerifyingLinkState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/net/wifi/WifiStateMachine;-stopDhcp-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pan/PanService;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/LockdownVpnTracker;-reset-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/magicsmoke/MagicSmoke;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/widget/SpellChecker;-<init>-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/WebVttRenderingWidget$SpanLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-access$2100-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-goingToSleep-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/ui/PhotoFallbackEffect$PositionProvider;-getItemIndex-(Lcom/android/gallery3d/data/Path;)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperPreview;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/DataManager;-rotate-(Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ingest/IngestService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/accessibility/CaptioningManager;-notifyUserStyleChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-removeUpdatesLocked-(Lcom/android/server/LocationManagerService$Receiver;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPagerController;-show-(Lcom/android/mail/providers/Account; Lcom/android/mail/providers/Folder; Lcom/android/mail/providers/Conversation; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/TrimVideo;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneProxy;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/SearchActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/musicvis/vis1/Visualization1;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/inputmethodservice/InputMethodService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/PackageManagerService;-loadMediaPackages-(Ljava/util/HashMap; [I Ljava/util/HashSet;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/AbstractActivityController$ConversationAction;-performAction-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/galaxy/GalaxyView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-getString-(Ljava/lang/String; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationItemView;-bind-(Lcom/android/mail/browse/ConversationItemViewModel; Lcom/android/mail/ui/ControllableActivity; Lcom/android/mail/ui/AnimatedAdapter$ConversationListListener; Lcom/android/mail/browse/ConversationItemView$ConversationItemAreaClickListener; Lcom/android/mail/ui/ConversationSelectionSet; Lcom/android/mail/providers/Folder; I Z Z Z Z Z Lcom/android/mail/ui/AnimatedAdapter; I Landroid/graphics/Bitmap;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/ScreensaverActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/server/print/PrintManagerService;-stopPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/CalendarColorSquare;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/photo/PhotoViewPager;-setAdapter-(Landroid/support/v4/view/PagerAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/colorpicker/ColorPickerSwatch$OnColorSelectedListener;-onColorSelected-(I)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageRedEye;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NativeDaemonConnector;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/ui/SlideshowView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/NavTabScroller;-onLayout-(Z I I I I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtErrorActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-discoverServices-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/ConversationPager;-setPageTransformer-(Z Landroid/support/v4/view/ViewPager$PageTransformer;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendHoverExitAndTouchExplorationGestureEndIfNeeded-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/MailActivity;-stopDragMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/compose/AttachmentsView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/NullFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ButteryProgressBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothPbap;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/ContentService;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/gallery3d/filtershow/state/StateView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/videoeditor/widgets/TimelineRelativeLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/text/format/DateUtils;-getRelativeTimeSpanString-(J)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/ISipService$Stub;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/view/WindowManagerPolicy$WindowManagerFuncs;-reevaluateStatusBarVisibility-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ReceiverRestrictedContext;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/os/IPowerManager$Stub$Proxy;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/eas/EasSyncBase;-handleResponse-(Lcom/android/exchange/EasResponse;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Landroid/widget/TextClock;-chooseFormat-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/providers/downloads/ui/TrampolineActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISms$Stub;-getAllMessagesFromIccEf-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$DriverStoppedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager;-checkMailboxes-()J' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/AbstractMailActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/emailcommon/utility/CertificateRequestor;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-moveTo-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-updatePhoneObject-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/browser/PhoneUi;-hideNavScreen-(I Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/content/ContentService;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompat$AccessibilityNodeProviderKitKatImpl$1;-performAction-(I I Landroid/os/Bundle;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/BaseUi;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/browse/ConversationItemView;-toggleSelectedState-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/EventInfoActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHealth;-getConnectedHealthDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/Invert;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/MmiCode;-cancel-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-closeService-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/wallpaper/grass/GrassView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSettings;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/calendar/CalendarController;-sendEvent-(Ljava/lang/Object; J Landroid/text/format/Time; Landroid/text/format/Time; J I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/ActivityController;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/VideoView;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-doVibratorOn-(J I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/email/activity/setup/AccountSettings;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/WordListPreference$OnActionButtonClick;-onClick-(Landroid/view/View;)V' : ['android.permission.INTERNET'],
	'Lcom/android/email/provider/EmailProvider;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-access$1600-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiApConfigStore$InactiveState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-access$100-(Landroid/hardware/location/GeofenceHardwareImpl;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-setTime-(Landroid/widget/TextView; J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-access$000-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/printspooler/FusedPrintersProvider;-onReset-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-onVisibilityChanged-(Z)V' : ['android.permission.CAMERA'],
	'Lcom/android/email/service/AuthenticatorService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity$2;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/InputMethodManagerService;-switchToNextInputMethod-(Landroid/os/IBinder; Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/AudioTrackView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/InstrumentationList;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/videoeditor/service/ApiService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/PhotoUI$7;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/ActionBarContainer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/ui/SearchActivityViewSinglePane;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/svc/WifiCommand;-run-([Ljava/lang/String;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$LabelView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/BatchService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/dreams/DreamService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothHeadset;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothAdapter;-disable-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/PhoneBase;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Spinner;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/AttachmentDownloadService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$MyGestureListener;-onSingleTapUp-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/Settings$System;-getLongForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/CorpusView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockContentResolver;-getCurrentSync-()Landroid/content/SyncInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/gallery3d/gadget/WidgetTypeChooser;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/ExchangeBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NetworkTimeUpdateService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothGatt;-unregisterApp-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothPbap;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-supplyPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventHelper$EditDoneRunnable;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/content/ContentResolver;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-dispatchMediaKeyRepeatWithWakeLock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralTestActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/WifiStateMachine;-startNextBatchedScan-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService$21;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/app/ResolverActivity$1;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackActivity;-cycleRepeat-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-setGrabbedState-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/content/SyncStorageEngine;-init-(Landroid/content/Context;)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/printspooler/FusedPrintersProvider$PersistenceManager$ReadTask;-onPostExecute-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ContextThemeWrapper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-sendScreenState-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AbsoluteLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-createBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/gadget/WidgetClickHandler;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityStackSupervisor;-shutdownLocked-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/TouchInterceptor;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setRadioPower-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/photomanager/PhotoManager$PhotoLoaderThread;-loadPhotosInBackground-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService;-access$1300-(Lcom/android/server/sip/SipService; Landroid/net/sip/SipProfile; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v13/app/FragmentTabHost;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkPolicyManager$Stub;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/IBluetoothGatt;-addCharacteristic-(I Landroid/os/ParcelUuid; I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/FolderSpanTextView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/AllInOneActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-cleanUpApplicationRecordLocked-(Lcom/android/server/am/ProcessRecord; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-alarmExists-(Landroid/content/ContentResolver; J J J)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setTimeFromNITZString-(Ljava/lang/String; J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/appwidget/AppWidgetHostView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AbsListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-editProperties-(Ljava/lang/String; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/RIL;-sendEnvelope-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeech;-playEarcon-(Ljava/lang/String; I Ljava/util/HashMap;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/EventInfoFragment$11;-onTouch-(Landroid/view/View; Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-queryCallWaiting-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/location/LocationManager;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AnimatedAdapter$3;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/email/service/PopImapSyncAdapterService$SyncAdapterImpl;-onPerformSync-(Landroid/accounts/Account; Landroid/os/Bundle; Ljava/lang/String; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/ComboViewActivity$TabsAdapter;-onTabSelected-(Landroid/app/ActionBar$Tab; Landroid/app/FragmentTransaction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorSVRectView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/HistoryItem;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$4;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mail/ui/FolderListFragment$4;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/svox/pico/CheckVoiceData;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/WakeLockStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/AlbumSetFragment;-getSubItemUrisForItem-(Ljava/lang/Object;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email2/ui/MailActivityEmail;-stopDragMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-updateLayout-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IWallpaperManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_WALLPAPER_HINTS', 'android.permission.SET_WALLPAPER'],
	'Landroid/accounts/IAccountManager$Stub;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/datetimepicker/time/TimePickerDialog;-access$800-(Lcom/android/datetimepicker/time/TimePickerDialog; I)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/os/PowerManager$WakeLock;-setWorkSource-(Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$DataListener;-onPhotoChanged-(I Lcom/android/gallery3d/data/Path;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/AlarmManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStoppingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeech;-playSilence-(J I Ljava/util/HashMap;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkPolicyManager;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/email/NotificationController;-access$1200-(Landroid/content/Context; J)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/wifi/WifiController$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/TileImageView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-startScanWithUuids-(I Z [Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/app/DisableCarModeActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/ScreensaverActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/TrackBrowserActivity;-getTrackCursor-(Lcom/android/music/TrackBrowserActivity$TrackListAdapter$TrackQueryHandler; Ljava/lang/String; Z)Landroid/database/Cursor;' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/WindowManagerGlobal;-addView-(Landroid/view/View; Landroid/view/ViewGroup$LayoutParams; Landroid/view/Display; Landroid/view/Window;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/bluetooth/hdp/HealthService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/tts/TextToSpeechService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-goTo-(J Z Z Z)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/media/AudioManager;-adjustStreamVolume-(I I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothMap$Stub;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$SyncHandlerSynchronizer;-modifyPing-(Z Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/camera/CameraBackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/Screensaver;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/OnePaneController;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/sip/ISipService;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Landroid/media/SubtitleController;-doSelectTrack-(Landroid/media/SubtitleTrack;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/ArtistAlbumBrowserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/app/CameraApp;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/widget/TextTime;-setFormat-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/ShortcutActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-loadAttachment-(Lcom/android/emailcommon/service/IEmailServiceCallback; J J Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperPreview;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiController$NoLockHeldState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/content/ContextWrapper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager;-stopBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/updates/SmsShortCodesInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ActivityController;-onBackPressed-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-updateCarrierMccMncConfiguration-(Ljava/lang/String; Ljava/lang/String; Landroid/content/Context;)V' : ['android.permission.SET_TIME_ZONE'],
	'Landroid/view/inputmethod/InputMethodManager;-getCurrentInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/EmlViewerActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getRemoteClass-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/grass/GrassView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/UiModeManagerService;-access$400-(Lcom/android/server/UiModeManagerService; I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/BaseUi$FullscreenHolder;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/Notifier;-playWirelessChargingStartedSound-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager$Stub;-validatePrinters-(Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/EasService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/DayView$CalendarGestureListener;-onDown-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onCleanUpConnection-(Z I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/UriSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-updateFolderList-(J)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-processLockedState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/view/ViewPager$3;-run-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ingest/ui/DateTileView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageGrad;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/MccTable;-setSystemLocale-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/SecureCameraActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/Connection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DigitalClock;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/ComposeAttachmentTile;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-removeBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/EasSyncService;-tryAutodiscover-(Landroid/content/Context; Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSettings;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/tts/TextToSpeech$8;-run-(Landroid/speech/tts/ITextToSpeechService;)Ljava/util/Locale;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmKlaxon;-start-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/mail/browse/ConversationCursor;-access$1100-(Lcom/android/mail/browse/ConversationCursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/BrowserWebView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/Context;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-getWifiApState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/EditText;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AutoCompleteTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-updateLockdownVpn-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-createPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/ComposeActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothHealth;-registerAppConfiguration-(Ljava/lang/String; I I I Landroid/bluetooth/BluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/RenderScriptWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-init-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/emailcommon/utility/CertificateRequestor;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorValueView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-updateConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/util/JavaNetHttpHelper;-post-(Ljava/lang/String; Ljava/util/Map; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/server/power/PowerManagerService;-goToSleepInternal-(J I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/PieMenu;-layoutPieView-(Lcom/android/browser/view/PieMenu$PieView; I I F)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/support/v4/print/PrintHelper;-printBitmap-(Ljava/lang/String; Landroid/graphics/Bitmap;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/AudioTrackLinearLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/DownloadOverMeteredDialog;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/development/Connectivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/DisplayPowerController;-<init>-(Landroid/os/Looper; Landroid/content/Context; Lcom/android/server/power/Notifier; Lcom/android/server/LightsService; Lcom/android/server/TwilightService; Landroid/hardware/SensorManager; Lcom/android/server/display/DisplayManagerService; Lcom/android/server/power/SuspendBlocker; Lcom/android/server/power/DisplayBlanker; Lcom/android/server/power/DisplayPowerController$Callbacks; Landroid/os/Handler;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavigationBarBase;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/support/v4/view/ViewPager;-smoothScrollTo-(I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-hangupWaitingOrBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/timezonepicker/TimeZonePickerView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; Ljava/lang/String; J Lcom/android/timezonepicker/TimeZonePickerView$OnTimeZoneSetListener; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EasTestAuthenticatorService$EasAuthenticator;-addAccount-(Landroid/accounts/AccountAuthenticatorResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/os/Bundle;)Landroid/os/Bundle;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneBase;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-getLinkProperties-(I)Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/AudioService;-checkScoAudioState-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/EdgeView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/vcard/VCardParserImpl_V40;-parseItems-()V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/mail/browse/SwipeableConversationItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSearch;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateStart;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/widget/DialogTitle;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getOwnerPhoneNumberVcard-(Z [B)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/emailcommon/service/ServiceProxy$ProxyConnection$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/videoeditor/widgets/ImageViewTouchBase;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorViewPager;-addNewItem-(I I)Landroid/support/v4/view/ViewPager$ItemInfo;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/support/v4/widget/DrawerLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService$CheckMp;-isMobileOk-(Lcom/android/server/ConnectivityService$CheckMp$Params;)Ljava/lang/Integer;' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/recurrencepicker/LinearLayoutWithMaxWidth;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-setUsbTethering-(Z)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/pinyin/SettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/timer/TimerAlertFullScreen;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothPbap;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/settings/SeekBarDialogPreference;-onStopTrackingTouch-(Landroid/widget/SeekBar;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-reconcileAndStartServices-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_CALENDAR', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/AsyncPlayer;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnablingActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/svox/pico/Pico;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/Application;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowManager;-isRotationFrozen-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/PageDialogsHandler$3;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AttachmentTile;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/SubtitleController;-doShow-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phasebeam/PhaseBeam;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/BroadcastQueue$AppNotResponding;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoSource;-next-(Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/SingleFolderSelectionDialog;-onListItemClick-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendSMS-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtErrorActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hid/HidService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/StackView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService;-setStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/media/AudioService;-checkAudioSettingsPermission-(Ljava/lang/String;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/input/InputManagerService;-updatePointerSpeedFromSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterApp;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/calculator2/CalculatorViewPager;-scrollToItem-(I Z I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/recurrencepicker/WeekButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService;-access$1700-(Lcom/android/server/sip/SipService; Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserBackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/EasService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/Pop3AuthenticatorService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer$2;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$System;-getInt-(Landroid/content/ContentResolver; Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/BluetoothDevice;-removeBond-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/certinstaller/CertInstaller;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-handleSettingsChangedLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/provider/Browser;-deleteHistoryTimeFrame-(Landroid/content/ContentResolver; J J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-createBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/util/ReverseGeocoder;-lookupAddress-(D D Z)Landroid/location/Address;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/eas/EasFolderSync;-getRequestUri-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/camera/app/CameraApp;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/IRecognitionService;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/widget/AnalogClock;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-writeCharacteristic-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter$9;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DeleteEventHelper;-deleteExceptionEvent-()V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/Wallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-startFreezingDisplayLocked-(Z I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/SecureAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothMap;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/service/LegacyEasAuthenticatorService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-acceptCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-sendIntentEnter-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-setQueuePosition-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/SuperCollapsedBlock;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/LatinIME;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-clearWakeLock-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/InputStreamSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-loadKeyboard-(Landroid/view/inputmethod/EditorInfo; Lcom/android/inputmethod/latin/settings/SettingsValues;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/GraphRunner;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/StatusBarManager;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/mail/browse/ConversationContainer;-access$200-(Lcom/android/mail/browse/ConversationContainer;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/app/IntentService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-cancelPendingUssd-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageScrollView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$NotConnectedState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/filtershow/FilterShowActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-sendAccessibilityEventForUnicodeCharacter-(I I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-supplyPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/MediaSetSource;-reload-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/MultiAutoCompleteTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/ComprehensiveCountryDetector$2;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-getCellLocationId-()I' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$LinkMonitoringState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-delete-(I Ljava/util/Collection; Lcom/android/mail/ui/DestructiveAction; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/photos/AlbumActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hid/HidService;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-getDeviceIdentity-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/View;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/StockSource;-next-(Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/RenamePlaylist;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothDevice;-setPin-([B)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-onSurfaceCreated-(Landroid/view/SurfaceHolder;)V' : ['android.permission.CAMERA'],
	'Lcom/android/deskclock/Screensaver;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-removeService-(I I I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/power/Notifier;-updatePendingBroadcastLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/AsyncQueryServiceHelper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/GlobalDismissManager$1;-doInBackground-([Landroid/util/Pair;)Ljava/lang/Void;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/GeolocationPermissionsPrompt;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/MailActionBarView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/eas/EasSyncCalendar;-updateAttendeesAndSendMail-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Landroid/content/Entity; Landroid/content/ContentValues; Z J Ljava/lang/String;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/WebViewController;-closeTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/exchange/Exchange;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/telephony/SmsManager;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/deskclock/TimerRingService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-setRepeatMode-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/VideoMenu;-onListPrefChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/Controller;-openIncognitoTab-()Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/widget/VideoView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/noisefield/NoiseField;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-access$400-(Lcom/android/mail/ui/AbstractActivityController; Ljava/util/Collection; Z Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/service/ApiService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/service/PingTask;-onCancelled-(Ljava/lang/Object;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/gadget/PhotoAppWidgetProvider;-onUpdate-(Landroid/content/Context; Landroid/appwidget/AppWidgetManager; [I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/SyncActivityTooManyDeletes;-startSyncReallyDelete-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/EllipsizedMultilineTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/FusedPrintersProvider$PersistenceManager$ReadTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-updateCarrierMccMncConfiguration-(Ljava/lang/String; Ljava/lang/String; Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/browser/NavigationBarBase;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-handleRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager;-isProviderEnabled-(Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/Spinner;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/SlotView;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-updateServiceLocation-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pan/PanService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/PickerActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/MusicUtils;-playPlaylist-(Landroid/content/Context; J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/SettingsRedirector;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-execute-(Lcom/android/gallery3d/data/DataManager; Lcom/android/gallery3d/util/ThreadPool$JobContext; I Lcom/android/gallery3d/data/Path;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterfw/core/Filter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-setSymbolsShiftedKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/TwoPaneController;-switchToDefaultInboxOrChangeAccount-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/gatt/GattService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncStorageEngine;-setSyncAutomatically-(Landroid/accounts/Account; I Ljava/lang/String; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/nfc/cardemulation/HostApduService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/IWifiManager;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/datetimepicker/AccessibleTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Service;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-handleSimRefresh-(Lcom/android/internal/telephony/uicc/IccRefreshResponse;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/StackView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/EmailIntentService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/DisplayPowerController$3;-run-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-addCircularFence-(I I D D D I I I I Landroid/hardware/location/IGeofenceHardwareCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/IdleMaintenanceService;-sendIdleMaintenanceStartIntent-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calculator2/CalculatorDisplay;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/soundrecorder/SoundRecorder;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/ConversationListFooterView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Connection;-cancelPostDial-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/preference/PreferenceActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/agenda/AgendaListView;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/PhoneBase;-setPropertiesByCarrier-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/date/TextViewWithCircularIndicator;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/State;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-setVoiceCallForwardingFlag-(I Z Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ZoomButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/backup/BackupAgentHelper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationViewAdapter$MessageFooterItem;-onModelUpdated-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/speech/tts/TextToSpeechService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-bindBackupAgent-(Landroid/content/pm/ApplicationInfo; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-insert-(Landroid/content/ContentResolver; J J J J I)Landroid/net/Uri;' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/media/MediaRecorder;-setAudioSource-(I)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-access$900-(Lcom/android/internal/policy/impl/PhoneWindowManager;)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub$Proxy;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/view/IWindowManager;-startFreezingScreen-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/provision/DefaultActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-setFacilityLock-(Ljava/lang/String; Z Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SlotView$MyGestureListener;-onLongPress-(Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/VUMeter;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/CaptivePortalTracker$DefaultState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/widget/TextClock;-setFormat24Hour-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/AccountService;-onBind-(Landroid/content/Intent;)Landroid/os/IBinder;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/widget/ScrollingTabContainerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/common/contacts/DataUsageStatUpdater;-updateWithRfc822Address-(Ljava/util/Collection;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/widget/MediaController;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/PreloadRequestReceiver;-handlePreload-(Landroid/content/Context; Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/Screensaver$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/ActivityController;-handleDrop-(Landroid/view/DragEvent; Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/os/BatteryStatsImpl;-writeToParcelLocked-(Landroid/os/Parcel; Z I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/PhoneProxy;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BookmarkItem;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/DialogActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/MediaRouteControllerDialog$1;-onProgressChanged-(Landroid/widget/SeekBar; I Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-onContentDirty-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/compose/ComposeActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/base/RetargetFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserActivity;-onNewIntent-(Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/Controller$PruneThumbnails;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-drawMonthTitle-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/pm/IPackageManager$Stub;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-sendP2pConnectionChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/FilmstripBottomControls;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-access$900-(Landroid/media/MediaPlayer; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSyncCollectionTypeBase;-cleanup-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothSocket;-connect-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-makeInboundSmsHandler-(Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;)Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/email/service/AttachmentDownloadService$AccountManagerStub;-getNumberOfAccounts-()I' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/BackupManagerService$RunInitializeReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-loadDictionaryAsync-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Landroid/location/Criteria; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AbstractActivityController;-onCreate-(Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/AttachmentTileGrid;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/WaveView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/DialogPicker;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-slideToNextPicture-()Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$MyDownUpListener;-onUp-(Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/RingtoneManager;-getRingtone-(I)Landroid/media/Ringtone;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-getLastInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavTabScroller;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/AbstractActivityController;-onBackPressed-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/support/v4/net/ConnectivityManagerCompat;-getNetworkInfoFromBroadcast-(Landroid/net/ConnectivityManager; Landroid/content/Intent;)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/base/FrameBranch;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TabWidget;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/Switch;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/RetargetFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcDisconnectingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/BookmarkItem;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/UsageStatsService$2;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine;-setWifiApState-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ActionBarView$HomeView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ActionBarOverlayLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/Gallery;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-doStart-(Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager;-getScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-getContactCount-()I' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Pending;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService$26;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/month/SimpleWeekView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothGattServer;-sendResponse-(Landroid/bluetooth/BluetoothDevice; I I I [B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/ex/photo/PhotoViewPager;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SlideshowView;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/svox/pico/CheckVoiceData;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/IAccountManager$Stub;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/test/mock/MockContentResolver;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/DialogPicker;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/CropFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasProvision;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/browser/Controller;-setActiveTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendUpForInjectedDownPointers-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifyDataConnectionFailed-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/ListPrefSettingPopup;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/textservice/TextServicesManager;-newSpellCheckerSession-(Landroid/os/Bundle; Ljava/util/Locale; Landroid/view/textservice/SpellCheckerSession$SpellCheckerSessionListener; Z)Landroid/view/textservice/SpellCheckerSession;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothA2dp;-shouldSendVolumeKeys-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/WebViewController;-doUpdateVisitedHistory-(Lcom/android/browser/Tab; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothDevice;-setPairingConfirmation-(Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-onDisconnect-(Lcom/android/internal/telephony/Connection$DisconnectCause;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager;-sendCancelSyncsMessage-(Landroid/accounts/Account; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/content/ContextWrapper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/text/StringLogger;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavigationBarTablet;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/EmailProvider;-uiFolderRefresh-(Lcom/android/emailcommon/provider/Mailbox; I)Landroid/database/Cursor;' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothMap$Stub;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/ConversationListFooterView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-changeIccLockPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ActivityController;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onDisconnectDcRetrying-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/TabBar$TabView;-closeTab-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/Preloader$PreloaderSession;-<init>-(Lcom/android/browser/Preloader; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/content/pm/IPackageManager$Stub;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/browser/search/OpenSearchSearchEngine;-getSuggestions-(Landroid/content/Context; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-switchToNextInputMethod-(Landroid/os/IBinder; Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/SimpleMonthView$MonthViewTouchHelper;-clearFocusedVirtualView-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/exchange/SettingsRedirector;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/service/LegacyImapAuthenticatorService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/LegacyImapAuthenticatorService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v7/widget/GridLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/CantAddAccountActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/content/AsyncTaskLoader$LoadTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/widget/FrameLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallEstablished-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/calendar/agenda/AgendaFragment$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/media/Ringtone;-getTitle-(Landroid/content/Context; Landroid/net/Uri; Z)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/soundrecorder/VUMeter;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/SearchActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-getVoiceRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-getHealthDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/musicvis/RenderScriptWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/internal/PreviewPlacerView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/DetailsAddressResolver$AddressLookupJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Landroid/location/Address;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/certinstaller/CertInstaller;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/keychain/KeyChainActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationListFragment;-onDestroyView-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/TileImageView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/content/ContentResolver;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/activity/setup/AccountSecurity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accessibilityservice/AccessibilityService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/app/ResolverActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/location/provider/LocationProviderBase;-reportLocation-(Landroid/location/Location;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSync;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/commands/monkey/MonkeyNetworkMonitor;-register-(Landroid/app/IActivityManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/videoeditor/widgets/AudioTrackLinearLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/InputMethodService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/Session;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/widget/SlidingPaneLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/AccountReconciler;-reconcileAccountsInternal-(Landroid/content/Context; Ljava/util/List; Ljava/util/List; Z)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-hangupWaitingOrBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-getIntentBySingleSelectedPath-(Ljava/lang/String;)Landroid/content/Intent;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$SettingsObserver;-observe-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/IWindowSession;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/widget/SearchView$SearchAutoComplete;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/CameraActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calculator2/CalculatorViewPager;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/photo/PhotoViewPager;-setPageTransformer-(Z Landroid/support/v4/view/ViewPager$PageTransformer;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/content/SyncManager;-access$900-(Lcom/android/server/content/SyncManager; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/svox/pico/EngineSettings;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/photos/AlbumActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/setup/SetupActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/soundrecorder/VUMeter;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/LinearLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/AccountsTester;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/KeyguardManager$KeyguardLock;-disableKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/ExpandableListActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$6700-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ViewFlipper;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/accessibility/AccessibilityEntityProvider;-simulateKeyPress-(Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/dreams/phototable/StockSource;-<init>-(Landroid/content/Context; Landroid/content/SharedPreferences;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/EdgeView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-doRecovery-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/ProjectsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/utils/FragmentStatePagerAdapter2;-instantiateItem-(Landroid/view/ViewGroup; I)Ljava/lang/Object;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/deskclock/AlarmUtils;-getAlarmText-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothAdapter;-setScanMode-(I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wifi/WifiService;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/sip/SipAudioCall;-access$800-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/TrimVideo;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage;-initializeData-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPin2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Preloader;-getSession-(Ljava/lang/String;)Lcom/android/browser/Preloader$PreloaderSession;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollViewWithNotifier;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserActivity;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/QueryBrowserActivity;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/EmojiCategoryPageIndicatorView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$4;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer;-scanInternalSubtitleTracks-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/gatt/GattService;-unregisterServer-(I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/media/MediaScanner$MyMediaScannerClient;-scanFile-(Ljava/lang/String; J J Z Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService;-sync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/view/menu/ActionMenuView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Controller;-onPreloginFinished-(Landroid/os/Bundle; Landroid/content/Intent; J Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-access$1700-(Lcom/android/bluetooth/opp/BluetoothOppService; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-access$1000-(Lcom/android/bluetooth/map/BluetoothMapService;)V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/videoeditor/widgets/MediaItemView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-access$000-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/TwoPaneController;-onRestoreInstanceState-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasPing;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActiveServices;-cleanUpRemovedTaskLocked-(Lcom/android/server/am/TaskRecord; Landroid/content/ComponentName; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/UiController;-switchToTab-(Lcom/android/browser/Tab;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/text/StringLogger;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncStorageEngine;-setIsSyncable-(Landroid/accounts/Account; I Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/timer/TimerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/VideoEditorActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/NotificationManagerService$6;-run-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/event/EditEventView;-populateWhen-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/google/common/io/InputSupplier;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/PackageSummary$1;-onClick-(Landroid/view/View;)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/music/VideoBrowserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/setup/AccountSetupActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/SmsManager;-sendMultipartTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/calendar/EventInfoFragment;-initAttendeesCursor-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-tryVibrate-()V' : ['android.permission.VIBRATE'],
	'Landroid/net/ConnectivityManager;-getActiveLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-sendResponse-(I Ljava/lang/String; I I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/filtershow/state/StateView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmNotifications;-showSnoozeNotification-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/WebSearchSuggestionView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MusicPicker;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-endReliableWrite-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/research/FeedbackActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.READ_PHONE_STATE', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothGattServer;-notifyCharacteristicChanged-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothGattCharacteristic; Z)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/ToggleButton;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms;-disableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/net/LockdownVpnTracker$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker$ApnChangeObserver;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService$10;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/IAccountManager$Stub;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-dispatchMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/view/menu/ActionMenuView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-access$1600-(Lcom/android/server/LocationManagerService; Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub$Proxy;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/app/NativeActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/browser/DataController$DataControllerHandler;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/pm/PackageManagerService;-scanAvailableAsecs-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/dreams/phototable/CursorPhotoSource;-next-(Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter$SlideshowSource;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/WidgetClickHandler;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-setUpAdapter-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterfw/core/FilterSurfaceView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-makeWindowFreezingScreenIfNeededLocked-(Lcom/android/server/wm/WindowState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageHeaderView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub;-setCurrentSpellChecker-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/browse/ConversationPagerAdapter;-setItemVisible-(Landroid/app/Fragment; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/documentsui/DirectoryView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-untether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/MoviePlayer;-startVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/TouchExplorer;-handleMotionEventGestureDetecting-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameStore;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/provider/Settings$Secure;-getFloat-(Landroid/content/ContentResolver; Ljava/lang/String; F)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/grass/GrassRS;-start-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/sip/SipWakeLock;-acquire-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContextWrapper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-handleGroupCreationFailure-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/DeleteItems;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService;-checkAuthenticateAccountsPermission-(Landroid/accounts/Account;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothDevice;-createInsecureRfcommSocket-(I)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountAuthenticatorActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView$FullPicture;-drawTileView-(Lcom/android/gallery3d/glrenderer/GLCanvas; Landroid/graphics/Rect;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralTestActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/compose/QuotedTextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/widget/BookmarkThumbnailWidgetService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/media/Media;-main-([Ljava/lang/String;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.RESTART_PACKAGES'],
	'Lcom/android/internal/app/AlertController$RecycleListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-getMyEnabledInputMethodSubtypeList-(Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractEditText;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/AutoFixEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/EditText;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/app/NoSaveStateFrameLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl$SipKeepAlive;-onError-(Landroid/net/sip/ISipSession; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/browse/MessageAttachmentBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/MailActivity;-onAnimationEnd-(Lcom/android/mail/ui/AnimatedAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/Editor$SelectionStartHandleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService$3;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-access$6600-(Landroid/media/AudioService;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wm/WindowManagerService;-updateResizingWindows-(Lcom/android/server/wm/WindowState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v13/app/FragmentTabHost;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardParser_V40;-parse-(Ljava/io/InputStream; Lcom/android/vcard/VCardInterpreter;)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/media/IAudioService$Stub$Proxy;-startBluetoothSco-(Landroid/os/IBinder; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmClockFragment$8;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/widget/ImageButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LockSettingsService$DatabaseHelper;-maybeEnableWidgetSettingForUsers-(Landroid/database/sqlite/SQLiteDatabase;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService;-onConnectivityChanged-(Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/DragState;-unregister-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/AdnRecordLoader;-updateEF-(Lcom/android/internal/telephony/uicc/AdnRecord; I I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/TimerRingService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/RadioGroup;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-startPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ActivityController;-showConversationList-(Lcom/android/mail/ConversationListContext;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-operationComplete-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MusicBrowserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/NavTabView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/InputStreamSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/ConnectivityManager;-requestRouteToHostAddress-(I Ljava/net/InetAddress;)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/development/Development;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/galaxy4/GalaxyView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/TabActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserActivity;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/WifiManager;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/VideoView;-release-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-access$1100-(Lcom/android/server/location/GpsLocationProvider; I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/SubtitleTrack$RenderingWidget;-setVisible-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/ui/PieMenuButton;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/ViewPager;-pageRight-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/power/Notifier$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetooth;-cancelBondProcess-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/widget/LockPatternUtils;-saveLockPattern-(Ljava/util/List; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Controller;-access$100-(Lcom/android/browser/Controller; Landroid/os/Bundle; Landroid/content/Intent; J Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/Screensaver;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-reportLocation-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryDownloadProgressBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/views/TiledImageView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmClockFragment;-startCreatingAlarm-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$7;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/mail/ui/CreateShortcutActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/LoadingListener;-onLoadingFinished-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-processUnsolicited-(Landroid/os/Parcel;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/widget/BookmarkWidgetConfigure;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/Preferences;-shouldShowRequireManualSync-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/Utils;-setTimeZone-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/mail/NotificationActionIntentService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/BluetoothManagerService;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Controller;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getLastSubtypeForInputMethodLocked-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$IdleState;-exit-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/RenderScriptWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/inputmethod/InputMethodManager;-getLastInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/SearchView$SearchAutoComplete;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothInputDevice;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/DhcpStateMachine$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/SubtitleController$Renderer;-createTrack-(Landroid/media/MediaFormat;)Landroid/media/SubtitleTrack;' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneProxy;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/camera/ui/FaceView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/SearchActivity;-initFragments-(J Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/galaxy4/GalaxyView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/UiModeManagerService$4;-onTwilightStateChanged-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/editors/SwapButton;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/FragmentBreadCrumbs;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/google/android/mms/pdu/PduPersister;-load-(Landroid/net/Uri;)Lcom/google/android/mms/pdu/GenericPdu;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/widget/SearchView$SearchAutoComplete;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-access$1100-(Lcom/android/bluetooth/pbap/BluetoothPbapService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/LatinIME;-onTextInput-(Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/ApplicationPackageManager;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/emailcommon/utility/CertificateRequestor;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/am/ActivityManagerService;-setProcessLimit-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/InputMethodManagerService;-switchUserLocked-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/provision/DefaultActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/GLView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/datetimepicker/AccessibleTextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/internal/telephony/BaseCommands;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/galaxy4/Galaxy4;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/event/EditEventActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/google/common/io/MultiInputStream;-read-()I' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/ui/GLRootView;-onDrawFrameLocked-(Ljavax/microedition/khronos/opengles/GL10;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/QsbApplicationWrapper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-hangupConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine;-setWifiState-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcDisconnectionErrorCreatingConnection;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/WindowManagerPolicy;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/ConversationViewAdapter$MessageFooterItem;-rebindView-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/exchange/ExchangeService$CalendarObserver;-access$400-(Lcom/android/exchange/ExchangeService$CalendarObserver;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Landroid/speech/tts/TextToSpeechService$1;-speak-(Landroid/os/IBinder; Ljava/lang/String; I Landroid/os/Bundle;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/packageinstaller/GrantActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/noisefield/NoiseFieldWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$RestartPingsTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/email/activity/setup/MailboxSettings;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/AlbumSetDataLoader;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/hardware/location/GeofenceHardware;-unregisterForMonitorStateChangeCallback-(I Landroid/hardware/location/GeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/alerts/AlertActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DigitalClock;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/LocalSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/widget/ActionableToastBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BluetoothManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorBrightnessView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothManager;-getDevicesMatchingConnectionStates-(I [I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/videoeditor/widgets/TransitionView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/chips/RecipientEditTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-openFile-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountAuthenticatorActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/settings/DebugSettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$FullImageJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Landroid/graphics/BitmapRegionDecoder;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/stopwatch/StopwatchService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService$SipSessionGroupExt;-openToReceiveCalls-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/AlertActivity$1;-doInBackground-([Ljava/util/List;)Ljava/lang/Void;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$3400-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/NetInitiatedActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService$AudioServiceBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageRotate;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/TimeBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/SizeAdaptiveLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RelativeLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/BackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/gatt/GattService;-discoverServices-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/TrimVideo;-onReplay-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/LegacyImapAuthenticatorService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/NavigationBarBase;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/ConfigUpdateInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-stopManualSync-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/category/CategorySelected;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPin-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/MultiStateButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-setWhereFilterPhones-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/BluetoothManagerService;-recoverBluetoothServiceFromError-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAccountsByType-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readEnabledAccessibilityServicesLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService;-beginServiceDeclaration-(I I I I Ljava/util/UUID; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-changeIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-isLockPasswordEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/AlbumBrowserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-pushMessage-(Lcom/android/bluetooth/map/BluetoothMapbMessage; Ljava/lang/String; Lcom/android/bluetooth/map/BluetoothMapAppParams;)J' : ['android.permission.SEND_SMS'],
	'Lcom/android/exchange/adapter/CalendarSyncParser;-addResponsesParser-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/deskclock/TimerSetupView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorViewPager;-setAdapter-(Landroid/support/v4/view/PagerAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/app/DisableCarModeActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/AdapterViewAnimator;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavScreen;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothInputDevice;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity$1;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipManager;-makeAudioCall-(Landroid/net/sip/SipProfile; Landroid/net/sip/SipProfile; Landroid/net/sip/SipAudioCall$Listener; I)Landroid/net/sip/SipAudioCall;' : ['android.permission.USE_SIP'],
	'Lcom/android/bluetooth/btservice/JniCallbacks;-pinRequestCallback-([B [B I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/compose/QuotedTextView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-updateServiceLocation-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity$3;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/LocationManager;-addTestProvider-(Ljava/lang/String; Z Z Z Z Z Z Z I I)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-getDataRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/PointerLocation;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/ex/photo/PhotoViewPager;-setCurrentItemInternal-(I Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/pm/PackageManagerService;-scanDirLI-(Ljava/io/File; I I J)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-addWordToUserDictionary-(Ljava/lang/String;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView$KeyTimerHandler;-startKeyRepeatTimer-(Lcom/android/inputmethod/keyboard/PointerTracker; I I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/server/wifi/WifiService;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/mail/ui/LeaveBehindItem;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/PackageBrowser;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/UrlHandler;-startActivityForUrl-(Lcom/android/browser/Tab; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/AudioManager;-handleKeyDown-(Landroid/view/KeyEvent; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/print/IPrintManager$Stub$Proxy;-startPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/timezonepicker/TimeZonePickerView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/VideoBrowserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/NotificationActionIntentService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/QueryBrowserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService$VolumeStreamState;-readSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/AudioPreview;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/OverlayLinearLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/ImageViewTouchBase;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget$RegionLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/TestActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-updateCarrierMccMncConfiguration-(Ljava/lang/String; Ljava/lang/String; Landroid/content/Context;)V' : ['android.permission.SET_TIME_ZONE'],
	'Landroid/app/ActivityManager;-getRunningTasks-(I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Landroid/renderscript/RSSurfaceView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoCarousel;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupOptions$2$1;-run-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/camera/VideoModule;-onPauseBeforeSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/OnePaneController;-onDataSetChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/DeleteEventHelper$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/net/INetworkPolicyManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/ComboAlbum;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/common/contacts/DataUsageStatUpdater;-updateWithAddress-(Ljava/util/Collection;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/camera/ui/ModuleSwitcher;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/LocationManager;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/deskclock/ClockFragment;-onResume-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/SuggestionsAdapter$CompletionListener;-onSelect-(Ljava/lang/String; I Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/accounts/AccountManagerService$GetAccountsByTypeAndFeatureSession;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/ILockSettings;-checkPassword-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager;-addGpsStatusListener-(Landroid/location/IGpsStatusListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/datetimepicker/time/RadialTextsView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NsdService$NsdStateMachine$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/WeightedLinearLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService$Stub;-dispatchMediaKeyEventUnderWakelock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/EasTestAuthenticatorService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockContentResolver;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/calendar/alerts/QuickResponseActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/SearchView$SearchAutoComplete;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-unDeleteMessageSms-(J)Z' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/filterfw/core/Filter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter;-getAutomaticLookback-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/notification/DemoContactNotificationScorer;-hasStarredContact-(Landroid/os/Bundle;)Z' : ['android.permission.READ_CONTACTS'],
	'Landroid/widget/RadioButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-access$600-(Landroid/media/MediaPlayer;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/gatt/GattService;-stopScan-(I Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/google/android/mms/util/PduCache;-put-(Landroid/net/Uri; Lcom/google/android/mms/util/PduCacheEntry;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/smspush/WapPushManager;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/test/mock/MockContentResolver;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$ObtainingIpState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/bluetooth/gatt/GattService$ClientDeathRecipient;-binderDied-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-switchHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-cancelPostDial-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/view/BookmarkExpandableView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/MailboxSettings;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/service/textservice/SpellCheckerService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-registerClient-(Landroid/os/ParcelUuid; Landroid/bluetooth/IBluetoothGattCallback;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/media/WebVttRenderingWidget;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/CrossProcessEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-naturalPrevious-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getAppWidgets-(I)[I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ViewMode;-setModeInternal-(I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/provider/EmailProvider$5;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/development/PointerLocation;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/AppHwPref;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/VideoView;-resume-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothMap;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothA2dp;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/MoviePlayer$9;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onSingleTapUp-(I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ISyncStatusObserver$Stub$Proxy;-onStatusChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothGatt;-serverConnect-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/LocationManagerService$Receiver;-<init>-(Lcom/android/server/LocationManagerService; Landroid/location/ILocationListener; Landroid/app/PendingIntent; I I Ljava/lang/String; Landroid/os/WorkSource; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/CommandsInterface;-getLastCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageFooterView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-getNetworkQuotaInfoUnchecked-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/MediaPlaybackService;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/ui/DownloadList;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/Call;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/noisefield/NoiseField;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/performance/ThroughputFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService$Receiver;-access$1400-(Lcom/android/server/LocationManagerService$Receiver;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller$2;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/mail/browse/SuperCollapsedBlock;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-removeGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calculator2/EventListener;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/am/BroadcastQueue;-skipCurrentReceiverLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/ingest/ui/MtpFullscreenView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/NetInitiatedActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/TimelineRelativeLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/IBackupManager$Stub$Proxy;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/GeneralPreferences$1;-onPreferenceClick-(Landroid/preference/Preference;)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/eas/EasSyncContacts;-dirtyContactsWithinDirtyGroups-(Landroid/content/ContentResolver; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsUsageMonitor; Lcom/android/internal/telephony/ImsSMSDispatcher;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/updates/CertPinInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/gsm/SmsManager;-deleteMessageFromSim-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/inputmethod/pinyin/BalloonHint$BalloonView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/util/ThreadPool$Job;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/SnailSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/printspooler/PrintDialogFrame;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/personalization/PersonalizationDictionary;-writeBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/AnalogClock;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-sendVendorSpecificResultCode-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/accounts/AccountManagerService;-access$200-(Lcom/android/server/accounts/AccountManagerService; Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/power/ShutdownThread;-shutdown-(Landroid/content/Context; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/am/ActivityManagerService$13;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/widget/Space;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/latin/LatinIME;-access$1000-(Lcom/android/inputmethod/latin/LatinIME; Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/packageinstaller/CaffeinatedScrollView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-access$100-(Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/gallery3d/ui/GLRootView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-onContentDirty-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/ListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/TimeIntervalPopup;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/EventInfoFragment;-onResume-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/widget/ExpandableListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/widget/LinearLayoutWithDefaultTouchRecepient;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$AmsTask;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/app/ContextImpl;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-restartDataStallAlarm-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ShortcutNameActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-dial-(Ljava/lang/String; I Lcom/android/internal/telephony/UUSInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsFragment;-toggleVisibility-(I)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/DayView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/RenderScriptWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-connectSocket-(Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/AllInOneActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager$Stub;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-clientDisconnect-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hdp/HealthService;-registerAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/IBluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothAdapter;-createNewRfcommSocketAndRecord-(Ljava/lang/String; Ljava/util/UUID; Z Z)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub;-getTetherableBluetoothRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActiveServices;-startServiceLocked-(Landroid/app/IApplicationThread; Landroid/content/Intent; Ljava/lang/String; I I I)Landroid/content/ComponentName;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/os/IPowerManager;-acquireWakeLockWithUid-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/SoundRecorder;-access$000-(Lcom/android/soundrecorder/SoundRecorder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/TrackBrowserActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/internal/PreviewPlacerView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/DnsPinger;-getCurrentLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/soundrecorder/SoundRecorder;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/CaptivePortalTracker;-access$2500-(Landroid/net/CaptivePortalTracker; Ljava/lang/String;)Ljava/net/InetAddress;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtErrorActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarColorPickerDialog;-setCalendarId-(J)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/widget/ActivityChooserView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/text/format/DateFormat;-is24HourFormat-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-getAccountsByTypeAsUser-(Ljava/lang/String; Landroid/os/UserHandle;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/AccountManager$1;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/content/MutableContextWrapper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/ExchangeService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/MediaPlayer$2;-run-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-onProgressStart-(Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothInputDevice;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ingest/IngestService;-onIndexFinish-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$8;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/preference/PreferenceActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/PingTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/camera/ui/PieMenuButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/selectcalendars/CalendarColorSquare;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onDataSetupComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationCursor;-commitMostlyDead-(Lcom/android/mail/providers/Conversation;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/deskclock/alarms/AlarmKlaxon;-stop-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/FolderPickerActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-setWhenString-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/ObjectSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarSettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-access$100-(Lcom/android/email/activity/setup/AccountSetupOptions;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/LocationManagerService;-access$500-(Lcom/android/server/LocationManagerService; I I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService$UpdateRecord;-disposeLocked-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindow$DecorView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/app/FragmentActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/TabActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/UiModeManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/TwilightService;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/videoeditor/widgets/PreviewSurfaceView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager$AccountObserver;-access$100-(Lcom/android/emailsync/SyncManager$AccountObserver;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/ConversationSelectionSet;-validateAgainstCursor-(Lcom/android/mail/browse/ConversationCursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasOutboxSync;-getRequestUri-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/LauncherActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ViewMode;-enterWaitingForInitializationMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/photos/shims/MediaItemsLoader;-<init>-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/SubtitleController;-addTrack-(Landroid/media/MediaFormat;)Landroid/media/SubtitleTrack;' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/ActivityState;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/browse/SpamWarningView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/TransitionsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/CountdownTimerPopup$2;-onClick-(Landroid/view/View;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/SettingsActivity;-onPreferenceTreeClick-(Landroid/preference/PreferenceScreen; Landroid/preference/Preference;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-setCurrentItemInternal-(I Z Z I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-showPreview-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onDeleteImage-(Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/CameraActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/datetimepicker/time/CircleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-markConversationMessagesUnread-(Lcom/android/mail/providers/Conversation; Ljava/util/Set; [B)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/WebVttRenderingWidget$SpanLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/ViewPager;-onLayout-(Z I I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasPing;-doPing-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/view/IWindowSession;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/EditorRelativeLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-onAnimationEnd-(Lcom/android/mail/ui/AnimatedAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/Connectivity;-onStopDelayedCycle-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-getProfileProxy-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener; I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/MessageView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendRawPdu-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-performLayoutAndPlaceSurfacesLockedInner-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateCmdParamsReady;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-cleanUpAllConnections-(Z Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/CaptivePortalTracker$ActiveNetworkState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/print/PrinterDiscoverySession;-validatePrinters-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/dataconnection/DcTrackerBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BootReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/CommandsInterface;-rejectCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/HorizontalScrollView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/TrafficStats;-stopDataProfiling-(Landroid/content/Context;)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/LogViewer;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothGatt;-writeCharacteristic-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-getSuggestions-(Ljava/lang/String; I)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/smspush/WapPushManager;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-startDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/ExpandableTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsUsageMonitor; Lcom/android/internal/telephony/ImsSMSDispatcher;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/WaveView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$InitialState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setAndBroadcastNetworkSetTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onSetDependencyMet-(Ljava/lang/String; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-wakingUp-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/sip/SipSessionGroup;-processTimeout-(Ljavax/sip/TimeoutEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/Controller;-shouldOverrideUrlLoading-(Lcom/android/browser/Tab; Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/Bookmarks$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/LocationManagerService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractEditLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BreadCrumbView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/WebVttRenderingWidget$SpanLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DayView$CalendarGestureListener;-onLongPress-(Landroid/view/MotionEvent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onCallEnded-(Lcom/android/internal/telephony/Connection$DisconnectCause;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/gatt/GattService;-doStart-(Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserPreferencesPage;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/PackageBrowser;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-onActionTapAndHold-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/CcBccView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RemoteViewsService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumPage$MyDetailsSource;-getDetails-()Lcom/android/gallery3d/data/MediaDetails;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/GlobalActions;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-decrementWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-comeOutOfSleepIfNeededLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothSocket;-<init>-(I I Z Z Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/pm/UserManagerService$5;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/widget/AutoCompleteTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-updateConversation-(Ljava/util/Collection; Landroid/content/ContentValues;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/CameraControls;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/galaxy/GalaxyWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/MediaPlaybackService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/AttendeesView;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/alerts/AlertReceiver;-makeBasicNotification-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; J J J I Z I)Lcom/android/calendar/alerts/AlertService$NotificationWrapper;' : ['android.permission.READ_CALENDAR'],
	'Landroid/app/backup/IBackupManager;-opComplete-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-refreshHidingMessage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/EmptyConversationListView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumDataLoader$DataListener;-onContentChanged-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/FrameSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/grass/GrassView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onScroll-(Landroid/widget/AbsListView; I I I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-isNetworkSupported-(I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/GalleryActivity;-startGetContent-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/gadget/WidgetService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/net/ConnectivityManagerCompat$BaseConnectivityManagerCompatImpl;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/speechrecorder/SpeechRecorderActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasSyncCollectionTypeBase;-setSyncOptions-(Landroid/content/Context; Lcom/android/exchange/adapter/Serializer; D Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Mailbox; Z I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/am/ActivityManagerService;-updateConfigurationLocked-(Landroid/content/res/Configuration; Lcom/android/server/am/ActivityRecord; Z Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EasServerConnection;-requestSyncForMailbox-(Landroid/accounts/Account; Ljava/lang/String; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityRecord;-setTask-(Lcom/android/server/am/TaskRecord; Lcom/android/server/am/ThumbnailHolder; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/provider/BrowserProvider2;-updateHistoryInTransaction-(Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView$EmojiPalettesAdapter;-restoreState-(Landroid/os/Parcelable; Ljava/lang/ClassLoader;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/AgendaWindowAdapter;-buildQueryUri-(I I)Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/net/wifi/WifiManager;-setWifiEnabled-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/IActivityManager;-killApplicationWithAppId-(Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/google/common/io/ByteStreams;-toByteArray-(Lcom/google/common/io/InputSupplier;)[B' : ['android.permission.INTERNET'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/IAudioService;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/deskclock/DeskClock;-createTabs-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncActivityTooManyDeletes;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/compat/ConnectivityManagerCompatUtils;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/browse/SwipeableConversationItemView;-bind-(Lcom/android/mail/providers/Conversation; Lcom/android/mail/ui/ControllableActivity; Lcom/android/mail/ui/AnimatedAdapter$ConversationListListener; Lcom/android/mail/ui/ConversationSelectionSet; Lcom/android/mail/providers/Folder; I Z Z Z Z Z Lcom/android/mail/ui/AnimatedAdapter;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BitmapSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/PageProgressView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TimePicker;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IBackupAgent$Stub;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/view/PieMenu;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ContentResolver;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/vcard/VCardParserImpl_V30;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/NetworkTimeUpdateService$SettingsObserver;-observe-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/ViewOverlay$OverlayViewGroup;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-startDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/pinyin/SettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/pm/PackageManagerService;-replaceSystemPackageLI-(Landroid/content/pm/PackageParser$Package; Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; [I [Z Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/camera/ui/InLineSettingCheckBox;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/googlecode/mp4parser/authoring/builder/FragmentedMp4Builder;-build-(Lcom/googlecode/mp4parser/authoring/Movie;)Lcom/coremedia/iso/IsoFile;' : ['android.permission.INTERNET'],
	'Lcom/android/mail/ui/TwoPaneController;-access$400-(Lcom/android/mail/ui/AbstractActivityController; Ljava/util/Collection; Z Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/provider/Contacts$People;-queryGroups-(Landroid/content/ContentResolver; J)Landroid/database/Cursor;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/alerts/AlertActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationLongPressTipView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-access$600-(Lcom/android/gallery3d/ui/MenuExecutor; Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-matchAddresses-(Landroid/database/Cursor; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo; Lcom/android/bluetooth/map/BluetoothMapAppParams;)Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/WakeLockStateMachine;-handleSmsMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/bluetooth/hdp/HealthService;-doStart-(Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/settings/SettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-unregisterListener-(Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/Settings$Secure;-getLongForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/service/textservice/SpellCheckerService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService;-access$1200-(Lcom/android/music/MediaPlaybackService;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/MailboxSelectionActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/OnePaneController;-markConversationsRead-(Ljava/util/Collection; Z Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/BluetoothDevice;-getType-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/GLTextureTarget;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ImageView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDataStallAlarm-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/MultiStateButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/FromAddressSpinner;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-startRfcommSocketListener-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/service/notification/NotificationListenerService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-processMotionEvent-(Landroid/view/MotionEvent; Lcom/android/inputmethod/keyboard/PointerTracker$KeyEventHandler;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/internal/app/ChooserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager;-getAccountsByType-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/TwilightService;-registerListener-(Lcom/android/server/TwilightService$TwilightListener; Landroid/os/Handler;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/browse/ConversationItemView;-setContentDescription-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EventColorPickerDialog$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/RIL;-getPDPContextList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-makeGsmCellBroadcastHandler-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/HistoryItem;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/CircleButtonsLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService;-getPrintJobInfo-(Landroid/print/PrintJobId; I I)Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/print/PrintHelper$PrintHelperVersionImpl;-printBitmap-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager;-getEnabledInputMethodSubtypeList-(Ljava/lang/String; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoTable$PhotoLaunchTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/deskclock/timer/TimerAlertFullScreen;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/mail/ui/AbstractMailActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-handleDrop-(Landroid/view/DragEvent; Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/FrameLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ScrollNotifier$ScrollListener;-onNotifierScroll-(I I)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-extractMmsAddresses-(J Lcom/android/bluetooth/map/BluetoothMapbMessageMmsEmail;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-sendConnectionStateChange-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/NetworkPolicyManager;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/camera/tinyplanet/TinyPlanetPreview;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiService;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/mail/ui/FolderListLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-onPageSelected-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/photos/FullscreenViewer;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodsAndSubtypeListLocked-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertService;-dismissOldAlerts-(Landroid/content/Context;)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/email/service/AttachmentDownloadService$ServiceCallback;-loadAttachmentStatus-(J J I I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/documentsui/SettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/support/v4/view/ViewPager$PagerObserver;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/certinstaller/CertInstallerMain;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/FolderSpanTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/google/android/mms/pdu/PduPersister;-getPendingMessages-(J)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Lcom/android/mail/ui/AbstractActivityController;-showConversation-(Lcom/android/mail/providers/Conversation; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment;-onFoldersClicked-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-shouldOfferSwitchingToNextInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/ListPopupWindow$DropDownListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/ConfigurationViewer;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ToggleButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/server/power/PowerManagerService;-setTemporaryScreenBrightnessSettingOverrideInternal-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-conference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ChooserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-deallocateMemory-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/musicvis/vis4/Visualization4;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothPan;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BrowserPreferencesPage;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/state/StatePanelTrack;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/PagerTabStrip;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaFocusControl;-<init>-(Landroid/os/Looper; Landroid/content/Context; Landroid/media/VolumeController; Landroid/media/AudioService;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService;-getEnabledInputMethodSubtypeList-(Ljava/lang/String; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityManagerService;-handleAppCrashLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/TabScrollView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-disableCdmaBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/hardware/location/GeofenceHardwareService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendNewSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/SmsHeader; I Landroid/app/PendingIntent; Landroid/app/PendingIntent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiService;-enforceMulticastChangePermission-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/deskclock/AndroidClockTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/PrinterDiscoverySession;-stopPrinterStateTracking-(Landroid/print/PrinterId;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-sendDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/AlertActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/eas/EasFolderSync;-doValidate-()Landroid/os/Bundle;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/ui/AlbumSlotRenderer;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/BluetoothManagerService;-loadStoredNameAndAddress-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/SecureConversationViewFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/AnimatedAdapter;-delete-(Ljava/util/Collection; Lcom/android/mail/ui/SwipeableListView$ListItemsRemovedListener;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/EventInfoFragment;-saveResponse-()Z' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/deskclock/AlarmClockFragment;-onPageChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/dreams/phototable/PicasaSource;-naturalNext-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/content/PackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/hardware/location/IGeofenceHardware;-unregisterForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/colorpicker/ColorPickerSwatch;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/WallpaperManager;-suggestDesiredDimensions-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/photos/views/TiledImageView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/ShutdownThread;-access$000-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/BatchService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/support/v4/app/FragmentTabHost;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/PhotoController;-enableRecordingLocation-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/SuggestionsAdapter$SlowFilterTask;-doInBackground-([Ljava/lang/CharSequence;)Ljava/util/List;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Landroid/net/CaptivePortalTracker$ProvisioningObserver;-<init>-(Landroid/net/CaptivePortalTracker;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EmailServiceUtils;-moveContactsData-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/IWifiManager$Stub;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthTokenByFeatures-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/app/Activity; Landroid/os/Bundle; Landroid/os/Bundle; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/inputmethod/latin/LatinIME;-onPressKey-(I I Z)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/exchange/adapter/CalendarSyncParser;-deleteParser-(Lcom/android/exchange/adapter/CalendarSyncParser$CalendarOperations;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$UnavailableState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/deskclock/AlarmInitReceiver$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/BummerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/RepeatingImageButton;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/homepages/RequestHandler;-doHandleRequest-()V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/galaxy4/GalaxyView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/StatusBarManager;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/widget/Chronometer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity$1;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/musicvis/vis2/Visualization2;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/AlarmTimelineView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-access$1700-(Lcom/android/server/InputMethodManagerService;)Ljava/util/HashMap;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/MoviePlayer;-access$1000-(Lcom/android/gallery3d/app/MoviePlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/vcard/VCardEntry$NameData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-updateExternalState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-setPreferredVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/SlidingTab;-vibrate-(J)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController$18;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/speech/tts/TextToSpeech;-<init>-(Landroid/content/Context; Landroid/speech/tts/TextToSpeech$OnInitListener; Ljava/lang/String; Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/TabControl;-addPreloadedTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/MailIntentService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/MonthListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService$Stub;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/support/v4/view/PagerAdapter;-setPrimaryItem-(Landroid/view/ViewGroup; I Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/IWallpaperManager$Stub$Proxy;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/internal/app/PlatLogoActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/provision/DefaultActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothManager;-openGattServer-(Landroid/content/Context; Landroid/bluetooth/BluetoothGattServerCallback;)Landroid/bluetooth/BluetoothGattServer;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/CameraRootView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/ImapService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-setNetworkSelectionModeManual-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/RepeatingImageButton;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/LocationManager;-removeProximityAlert-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-autoDiscover-(Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/DisplayPowerController$Callbacks;-onProximityNegative-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/net/BaseNetworkStateTracker;-setRadio-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/Context;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-restoreSelectedConversations-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/accounts/AccountManagerService$9;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/speech/tts/TextToSpeechService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Tab;-persistThumbnail-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/ProcessInfo;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-setRepeatMode-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/deskclock/AlarmAlertWakeLock;-acquireScreenCpuWakeLock-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifySignalStrength-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/MailLogService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth;-setRemoteAlias-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/EmailIntentService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/EditText;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/SyncBaseInstrumentation;-cancelSyncsandDisableAutoSync-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasProvision;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-disable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-startScan-(Landroid/os/WorkSource;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/calendar/Utils;-getWeekNumberFromTime-(J Landroid/content/Context;)I' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-setMissedState-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/photos/views/SquareImageView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/GeolocationPermissionsPrompt;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BrowserBookmarksPage$OpenAllInTabsTask;-doInBackground-([Ljava/lang/Void;)Landroid/database/Cursor;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/svox/pico/DownloadVoiceData;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityRecord;-keyDispatchingTimedOut-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-access$000-(Lcom/android/bluetooth/map/BluetoothMapContentObserver;)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/bluetooth/btservice/AdapterState$OffState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/providers/EmlAttachmentProvider;-copyAttachment-(Landroid/net/Uri; Landroid/content/ContentValues;)I' : ['android.permission.INTERNET'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-runSyncFinishedOrCanceledLocked-(Landroid/content/SyncResult; Lcom/android/server/content/SyncManager$ActiveSyncContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/location/SettingInjectorService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/content/AsyncTaskLoader$LoadTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/HorizontalScrollView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/ImapAuthenticatorService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/datetimepicker/HapticFeedbackController;-start-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/RecentApplicationsBackground;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-getCurrentInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings; Landroid/os/IBinder; Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/email/activity/setup/AccountSettings;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/GalleryActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/FilmstripPage;-requestDeferredUpdate-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/dreams/phototable/CursorPhotoSource;-naturalNext-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/power/PowerManagerService;-access$1800-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setSuppServiceNotifications-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/DialogTitle;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ExpandableListActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager;-cancelActiveSync-(Landroid/accounts/Account; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/ex/chips/RecipientEditTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/pacprocessor/PacService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiService;-enforceChangePermission-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/mail/ui/MailActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/AppSecurityPermissions$PermissionItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/crop/CropView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/PinyinDecoderService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/MediaSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/gatt/GattService;-startScan-(I Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/AudioPreview;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/BorderView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-dispatchSyncOperation-(Lcom/android/server/content/SyncOperation;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/Utils;-formatDateRange-(Landroid/content/Context; J J I)Ljava/lang/String;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/widget/TimePicker;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserBookmarksPage;-handleContextItem-(I I I)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/MailIntentService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothGatt;-writeCharacteristic-(Landroid/bluetooth/BluetoothGattCharacteristic;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisSpeechItem;-<init>-(Landroid/speech/tts/TextToSpeechService; Ljava/lang/Object; I I Landroid/os/Bundle; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothPbap;-disconnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/AlertReceiver;-beginStartingService-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-trySetupData-(Lcom/android/internal/telephony/dataconnection/ApnContext;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/MailActionBarView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasOutboxService;-searchGal-(Landroid/content/Context; J Ljava/lang/String; I)Lcom/android/exchange/provider/GalResult;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/camera/tinyplanet/TinyPlanetPreview;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/MediaSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/sip/SipAudioCall$1;-onError-(Landroid/net/sip/SipSession; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-dispatchNormalMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/calculator2/ColorButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/NavScreen;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/RotateFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/TextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/CallbackFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onSetPolicyDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Editor;-sendOnTextChanged-(I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/PackagesMonitor$AsyncService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService;-getBluetoothHeadset-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/ControllerOverlay$Listener;-onPlayPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/OverlayLinearLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/location/LocationFudger;-<init>-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/content/PackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/settings/DebugSettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/LocationManager;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-getLastInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/OverlayTitleEditor;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/LocalSource;-processMapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasOptions;-handleProvisionError-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/provider/BrowserProvider2;-insertInTransaction-(Landroid/net/Uri; Landroid/content/ContentValues; Z)Landroid/net/Uri;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/alarms/AlarmActivity$GlowPadController;-onTrigger-(Landroid/view/View; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/IdleMaintenanceService;-updateIdleMaintenanceState-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/crop/CropActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/AlertActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$SocketAcceptThread;-run-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/wallpaper/RenderScriptWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/IngestService;-deviceRemoved-(Landroid/mtp/MtpDevice;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/TabHost;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/LatinIME;-onReleaseKey-(I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/SearchActivity;-search-(Ljava/lang/String; Landroid/text/format/Time;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-launchDevicePicker-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/datetimepicker/time/TimePickerDialog;-tryVibrate-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/Phone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityStack;-ensureActivitiesVisibleLocked-(Lcom/android/server/am/ActivityRecord; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/IMediaPlaybackService;-pause-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-moveTaskToBack-(I)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/commands/am/Am;-onRun-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.RESTART_PACKAGES'],
	'Landroid/provider/Telephony$Mms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Lcom/android/mail/NotificationActionIntentService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-performSync-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/SuggestionsAdapter$CursorSource;-runQuery-(Ljava/lang/CharSequence;)V' : ['android.permission.ACCESS_NETWORK_STATE', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/LocationManagerService;-updateProviderListenersLocked-(Ljava/lang/String; Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasSyncBase;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/ActivityState;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AnimatedListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/DatePickerDialog;-<init>-(Landroid/content/Context; Landroid/app/DatePickerDialog$OnDateSetListener; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-getContactNameFromPhone-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Landroid/media/AudioService$ScoClient;-clearCount-(Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-mayUpdateBatchInput-(J Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-clearProfilePriorty-(Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/PlaylistBrowserActivity$1;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasOutboxService;-canProvision-(Lcom/android/exchange/EasSyncService;)Lcom/android/exchange/adapter/ProvisionParser;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/EventInfoFragment$6;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-addTrackerToRawTableAndSendMessage-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/am/ActivityManagerService;-handleApplicationCrashInner-(Ljava/lang/String; Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityStack;-resetAffinityTaskIfNeededLocked-(Lcom/android/server/am/TaskRecord; Lcom/android/server/am/TaskRecord; Z Z I)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/os/Vibrator;-vibrate-([J I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/camera/MediaSaveService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-resizeStackBox-(I F)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CalendarView$WeekView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SmsUsageMonitor$SettingsObserverHandler;-<init>-(Landroid/content/Context; Ljava/util/concurrent/atomic/AtomicBoolean;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/CalendarContract$EventsEntity$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mail/browse/MessageView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-reloadFolderList-(Landroid/content/Context; J Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-blockingGetAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Z)Ljava/lang/String;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/server/wifi/WifiController$EcmState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/exchange/eas/EasProvision;-performAckRequest-(Z)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/preferences/GeneralPreferencesFragment$3;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-canSyncEmail-(Landroid/accounts/Account;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/svox/pico/EngineSettings;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/MailActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/SearchView$SearchAutoComplete;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorSaturationView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MailWebView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/GalleryActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/SenderInfoLoader;-loadContactPhotos-(Landroid/content/ContentResolver; Ljava/util/Set; Z)Lcom/google/common/collect/ImmutableMap;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/music/PlaylistBrowserActivity;-access$000-(Lcom/android/music/PlaylistBrowserActivity;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSetupType;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/DownloadHandler$1;-run-()V' : ['android.permission.INTERNET'],
	'Lcom/android/server/pm/PackageManagerService;-deletePackageLI-(Ljava/lang/String; Landroid/os/UserHandle; Z [I [Z I Lcom/android/server/pm/PackageManagerService$PackageRemovedInfo; Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-moveQueueItem-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/pipeline/ProcessingService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/emailsync/SyncManager;-startServiceThread-(Lcom/android/emailsync/AbstractSyncService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onUndoDeleteImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/compose/QuotedTextView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/server/ServerThread$2;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/ui/DateTileView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/GalleryAppImpl;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/SelectPrinterFragment$AddPrinterAlertDialogFragment;-onCreateDialog-(Landroid/os/Bundle;)Landroid/app/Dialog;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertReceiver;-createBroadcastMailIntent-(Landroid/content/Context; J Ljava/lang/String;)Landroid/app/PendingIntent;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/email/provider/EmailProvider;-handleMessageUpdateNotifications-(Landroid/net/Uri; Ljava/lang/String; Landroid/content/ContentValues;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ingest/IngestService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/ImageViewTouchBase;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/WindowManagerPolicy;-checkAddPermission-(Landroid/view/WindowManager$LayoutParams; [I)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/musicvis/RenderScriptWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/FilmstripPage;-refreshHidingMessage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/alarms/AlarmService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ApplicationPackageManager;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/wallpaper/fall/FallView;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/sip/SipSessionGroup;-processDialogTerminated-(Ljavax/sip/DialogTerminatedEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSettings;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/sip/SipAudioCall;-grabWifiHighPerfLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupCreatedState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub;-isTetheringSupported-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/content/IContentService;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/MailboxSelectionActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/deskclock/stopwatch/StopwatchService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/settings/SettingsFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/ShowActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/eas/EasFolderSync;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-flashAndSetGenericTrue-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage;-access$2400-(Lcom/android/gallery3d/app/PhotoPage; Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calculator2/CalculatorViewPager;-onRestoreInstanceState-(Landroid/os/Parcelable;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/view/BookmarkExpandableView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiService;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/filterfw/core/GraphRunner;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/RenamingDelegatingContext;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage;-onUndoDeleteImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothGatt;-unregisterServer-(I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/test/IsolatedContext;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/AbstractActivityController$ConversationListLoaderCallbacks;-onLoadFinished-(Landroid/content/Loader; Lcom/android/mail/browse/ConversationCursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment;-setRingtoneSummary-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManagerNative;-unbroadcastIntent-(Landroid/app/IApplicationThread; Landroid/content/Intent; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/RestrictedActivity;-onBackPressed-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/alarmclock/DigitalAppWidgetProvider;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onRegistrationTimeout-(Landroid/net/sip/ISipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/alarmclock/DigitalAppWidgetService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/CaptivePortalTracker;-access$2600-(Landroid/net/CaptivePortalTracker; Ljava/net/InetAddress;)Z' : ['android.permission.INTERNET', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/videosrc/CameraSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/accessibility/CaptioningManager;-getRawLocale-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/PagerTitleStrip;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-exitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/app/ActivityManagerNative;-killAllBackgroundProcesses-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-readCharacteristic-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/eas/EasSearch;-getRequestUri-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/development/ProcessInfo;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/SpamWarningView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageCrop;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-onResume-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/view/ContextThemeWrapper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/Phone;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/gallery3d/data/UnlockImage;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Ljava/net/ServerSocket;-bind-(Ljava/net/SocketAddress;)V' : ['android.permission.INTERNET'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnablingActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumPage$5;-onActionItemClicked-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-pullVcardEntry-([B Lcom/android/bluetooth/pbap/BluetoothPbapObexServer$AppParamValue; Ljavax/obex/Operation; Ljava/lang/String; Ljava/lang/String;)I' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/colorpicker/ColorPickerDialog;-onColorSelected-(I)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$800-(Lcom/android/server/accessibility/AccessibilityManagerService; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-next-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BookmarkItem;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/Settings$System;-getStringForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController$FolderLoads;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaSet-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaSet;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-matchOriginatorMms-(Landroid/database/Cursor; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo; Ljava/lang/String;)Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/browse/MessageHeaderView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PackagesMonitor$AsyncService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ActivityGroup;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/widget/SearchViewCompatIcs$MySearchView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/AttachmentDownloadService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailcommon/service/IPolicyService;-setAccountPolicy-(J Lcom/android/emailcommon/provider/Policy; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppIncomingFileConfirmActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/CameraShortcutImage;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getRemoteName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Connected;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/EasOutboxService;-acknowledgeProvision-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardParser_V30;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/mail/browse/ConversationItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService$Stub$Proxy;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/widget/CompoundButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/FolderListLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/AddBookmarkPage;-descendInto-(Ljava/lang/String; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/LogTextBox;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/PieItem$OnClickListener;-onClick-(Lcom/android/camera/ui/PieItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CallManager;-acceptCall-(Lcom/android/internal/telephony/Call;)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/speech/tts/TextToSpeechService;-getDefaultSpeechRate-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/AudioPreview;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/settings/GallerySettings;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/DefaultSuggestionView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-updateAlphabetShiftState-(I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/view/menu/ExpandedMenuView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/SearchActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/BlendFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/PieMenu$PieView$OnLayoutListener;-onLayout-(I I Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-handleMove-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-onBackup-(Landroid/os/ParcelFileDescriptor; Landroid/app/backup/BackupDataOutput; Landroid/os/ParcelFileDescriptor;)V' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/dreams/basic/Colors;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage;-initializeViews-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/TextClock;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-handleSandman-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/svox/pico/GetSampleText;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-notifyApnIdUpToCurrent-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/util/HttpHelper;-get-(Lcom/android/quicksearchbox/util/HttpHelper$GetRequest;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/util/StateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/ScanningProgress;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/TimerRingService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-parseIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/PickerActivity;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/event/EditEventView;-updateHomeTime-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/widget/RelativeLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Contacts$Settings;-setSetting-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker$TimerProxy;-startKeyRepeatTimer-(Lcom/android/inputmethod/keyboard/PointerTracker; I I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Landroid/widget/QuickContactBadge;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserSettings;-getPreloadEnabled-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/IGeofenceHardware;-resumeGeofence-(I I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/service/EasServerConnection;-sendHttpClientPost-(Ljava/lang/String; [B)Lcom/android/exchange/EasResponse;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/speech/IRecognitionService$Stub;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/mail/ui/OnePaneController;-onViewModeChanged-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/AlbumPage;-initializeData-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowManagerService;-setTokenVisibilityLocked-(Lcom/android/server/wm/AppWindowToken; Landroid/view/WindowManager$LayoutParams; Z I Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/grass/Grass;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/AnimatedAdapter;-delete-(Ljava/util/Collection; Lcom/android/mail/ui/SwipeableListView$ListItemsRemovedListener; Ljava/util/HashSet;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accounts/AccountManager;-removeAccount-(Landroid/accounts/Account; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/email/activity/ComposeActivityEmail;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationCursorLoader;-onStartLoading-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/IccCard;-supplyPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/PlatLogoActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/WifiStateMachine$L2ConnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AsyncPlayer;-access$500-(Landroid/media/AsyncPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-setForcedDisplayDensityLocked-(Lcom/android/server/wm/DisplayContent; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexServerSession;-preStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasOperation;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/fall/FallRS;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-open-([J I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/TintFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/os/BatteryStatsImpl;-doPlugLocked-(J J J)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-UiThreadNotifyIndexChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-render-(Lcom/android/gallery3d/glrenderer/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleDown-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler$DeliveringState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$LabelView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService;-play-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationViewHeader;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-sendMessage-(Lcom/android/bluetooth/map/BluetoothMapContentObserver$PushMsgInfo; Ljava/lang/String;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/browser/CrashRecoveryHandler$2;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/email/activity/setup/AccountSettings;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/mail/ui/ConversationsInOutboxTipView$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/camera/ui/FilmStripView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/ImapAuthenticatorService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/WaveView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-switchToFilmstrip-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActiveServices;-processStartTimedOutLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/svox/pico/PicoService;-access$400-(Landroid/speech/tts/TextToSpeechService;)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/NumberPicker$CustomEditText;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothPan$Stub$Proxy;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/photos/FullscreenViewer;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcRetryingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/net/sip/SipManager;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/server/sip/SipService$SipSessionGroupExt;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-getSelectedMediaObjects-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/LeaveBehindItem;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupConnectionByIndex-(Lcom/android/internal/telephony/gsm/GsmCall; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-vibrate-(I Ljava/lang/String; J Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/os/IPowerManager$Stub;-acquireWakeLockWithUid-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/stopwatch/StopwatchService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/WiredAccessoryManager;-access$000-(Lcom/android/server/WiredAccessoryManager;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/os/IPowerManager$Stub;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$10;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/data/MediaSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/Pop3Service;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/SimpleMonthView$MonthViewTouchHelper;-setFocusedVirtualView-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/widget/AdapterViewFlipper;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-setCurrentPreferredNetworkType-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-initializeMulticastFiltering-()Z' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/gallery3d/ui/UndoBarView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/ex/photo/util/ImageUtils$InputStreamFactory;-createInputStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Landroid/support/v7/widget/Space;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$DriverStoppingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/TelephonyManager;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/WifiController;-registerForStayAwakeModeChange-(Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/phasebeam/PhaseBeamView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasMoveItems;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-isNetworkConnected-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TimePicker;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AnimatedAdapter$1;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/widget/CalendarView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/PieMenu;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/TabBar$TabView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/content/PackageMonitor;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/FilterShowActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionarySettings;-createCursor-(Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-chooseUpstreamType-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/videoeditor/NoSearchActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/MockProvider;-setLocation-(Landroid/location/Location;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-handleModeChange-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/noisefield/NoiseFieldView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaFocusControl;-onReevaluateRemoteControlDisplays-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/soundrecorder/SoundRecorder;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/BroadcastQueue;-sendPendingBroadcastsLocked-(Lcom/android/server/am/ProcessRecord;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/map/BluetoothMapService$1;-handleMessage-(Landroid/os/Message;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiManager;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/music/VideoBrowserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/DeskClock;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/DatePickerDialog;-onDayOfMonthSelected-(I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/cardemulation/HostApduService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/HistoryItem;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub;-disable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/dataconnection/DcTrackerBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/usb/UsbDeviceManager$1;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/PackageManagerBackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/location/ILocationManager$Stub$Proxy;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/widget/LinearLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/SizeAdaptiveLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/AudioManager;-handleKeyUp-(Landroid/view/KeyEvent; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/FolderSelectionActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/speech/RecognitionService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupBasics;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-access$200-(Lcom/android/bluetooth/map/BluetoothMapService;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/ConversationSelectionSet;-remove-(Ljava/lang/Long;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-initMsgList-()V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageMirror;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-changeAccount-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager;-getAllLinkQualityInfo-()[Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/CaptivePortalTracker$ActiveNetworkState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-getFloatForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/FilmstripPage;-switchToGrid-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth$Stub;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/ex/photo/PhotoViewActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MovieActivity;-onResume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService;-destroyPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CalendarView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$LabelView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/ActivityState;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/ex/photo/PhotoViewPager;-onMeasure-(I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setLocationUpdates-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-onKeyUpPanel-(I Landroid/view/KeyEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserWebView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/BookmarkExpandableView;-handleItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/OffHostApduService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompat$AccessibilityNodeProviderJellyBeanImpl$1;-performAction-(I I Landroid/os/Bundle;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/LatinIME;-onFinishInputViewInternal-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/AppSecurityPermissions$PermissionItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/InitAlarmsService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/RenamePlaylist;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/IUiAutomationConnection;-connect-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/PrintSpoolerService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/AllInOneActivity;-onQueryTextSubmit-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/app/IBatteryStats$Stub$Proxy;-getStatistics-()[B' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processLaunchBrowser-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/InputMethodService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$IndicatorView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/SeekBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiService;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-computePanoramaSharingIntent-(Lcom/android/gallery3d/util/ThreadPool$JobContext; I)Landroid/content/Intent;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/MailIntentService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/dictionarypack/DownloadOverMeteredDialog;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/CellBroadcastMessage;-getDateString-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/SpellChecker;-setLocale-(Ljava/util/Locale;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/print/PrintManager;-getPrintJob-(Landroid/print/PrintJobId;)Landroid/print/PrintJob;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/EmlTempFileDeletionService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/AbstractCalendarActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/AddBookmarkPage$UpdateBookmarkTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/content/ContentResolver;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/content/IContentService$Stub;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ingest/IngestService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/MailActionBarView$1;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageTinyPlanet;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$Future2Task;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/ActivityManagerNative;-updatePersistentConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/colorpicker/ColorPickerPalette;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager;-scheduleSyncOperation-(Lcom/android/server/content/SyncOperation;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/coremedia/iso/IsoFile;-<init>-()V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/SearchView$SearchAutoComplete;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ActionMenuView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/AbstractThreadedSyncAdapter$ISyncAdapterImpl;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AnimatedAdapter;-updateAnimatingConversationItems-(Ljava/lang/Object; Ljava/util/HashSet;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-systemReady-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/view/menu/ActionMenuPresenter$OverflowMenuButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertActivity$2;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/CalendarViewAdapter;-buildMonthDayDate-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/FusedPrintersProvider;-onAbandon-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/ToUpperCase;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/FolderPickerActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/ZoomView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService$3;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/os/IVibratorService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/mail/compose/FromAddressSpinner;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onFilmModeChanged-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindow$DecorView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hdp/HealthService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/Screensaver;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$8;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/ShowActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/research/FeedbackLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/view/CertificateSelector;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/ui/ClusteredSuggestionsView;-handleItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/accessibility/AccessibilityEntityProvider;-performAction-(I I Landroid/os/Bundle;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofenceStatus-(I I D D D F F F J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/location/ILocationManager$Stub;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/power/PowerManagerService;-updateUserActivitySummaryLocked-(J I)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/DownUpDetector;-onTouchEvent-(Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-setRadio-(I Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager;-confirmCredentialsAsUser-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler; Landroid/os/UserHandle;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/phasebeam/PhaseBeamView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/FeedbackLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-loadDictionaryAsyncForUri-(Landroid/net/Uri;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/DayView$8;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/RIL;-getIccCardStatus-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/PackageManagerBackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/EasAuthenticatorService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-sendRawPdu-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/AppHwConfigList;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/View;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$2600-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/SecureAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/ui/InLineSettingItem;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationCursor;-checkNotifyUI-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/BackupManagerService$PerformClearTask;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CalendarView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager;-setUsbTethering-(Z)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/view/menu/IconMenuView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EmailServiceUtils;-isServiceAvailable-(Landroid/content/Context; Ljava/lang/String;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/GlobalActions$GlobalActionsDialog;-onStart-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarController$EventHandler;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/test/IsolatedContext;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ListActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/UnderdevelopedSettings;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/IVibratorService$Stub;-vibrate-(I Ljava/lang/String; J Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-revertToNitzTime-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionCallReceiverImpl;-processCommand-(Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-getRemoteType-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/SyncAdapterDriver;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cat/IconLoader;-readId-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-handleNewSms-(Landroid/os/AsyncResult;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/accessibility/EventStreamTransformation;-clear-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/ZoomControl;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/pinyin/PinyinIME;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-handleDown-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mail/compose/ComposeAttachmentTile;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService;-access$800-(Lcom/android/server/TextServicesManagerService; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$ScanModeState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/camera/ui/RotatableLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/VerticalTextSpinner;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/CaptioningManager;-notifyLocaleChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/EnterSubject;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/SecureConversationViewFragment;-showUntransformedConversation-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/datetimepicker/date/DayPickerView;-layoutChildren-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/ServiceWatcher$2;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ShutdownActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/magicsmoke/MagicSmokeSelector;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/ProjectsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompat$AccessibilityNodeProviderKitKatImpl$1;-createAccessibilityNodeInfo-(I)Ljava/lang/Object;' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/DhcpStateMachine$StoppedState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsImpl;-setSubtypeEnablerTitle-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/Editor;-access$2300-(Landroid/widget/Editor; I I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-trimApplications-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-handleExitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothMap$Stub$Proxy;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/net/sip/SipAudioCall;-access$1300-(Landroid/net/sip/SipAudioCall; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ResolverActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/CorpusView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/IWindowManager;-prepareAppTransition-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationListFooterView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/spellcheck/SpellCheckerSettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/soundrecorder/SoundRecorder;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManager;-getRunningTasks-(I I Landroid/app/IThumbnailReceiver;)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/datetimepicker/date/AccessibleDateAnimator;-dispatchPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiApConfigStore;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/crop/CropActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/agenda/AgendaListView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/accounts/AccountManager;-getAccountsByTypeAndFeatures-(Ljava/lang/String; [Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-setCdmaSubscriptionSource-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/CameraBackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/TrimVideo;-setProgress-()I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/grass/Grass;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-turnOffUpstreamMobileConnection-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/widget/MediaController;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/ILocationManager;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/calendar/event/EditEventView;-setCalendarsCursor-(Landroid/database/Cursor; Z J)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/GalleryActionBar;-onNavigationItemSelected-(I J)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-access$100-(Lcom/android/server/am/ActivityManagerService; Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Landroid/content/Intent; Ljava/lang/String; Landroid/content/IIntentReceiver; I Ljava/lang/String; Landroid/os/Bundle; Ljava/lang/String; I Z Z I I I)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/FolderItemView$DropHandler;-handleDrop-(Landroid/view/DragEvent; Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-getShortcutInputMethodsAndSubtypes-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothInputDevice;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/content/ContentService;-sync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindow$DecorView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/TimerRingService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$LabelView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/LocalAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-startObexServerSession-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/ActivityManagerNative;-shutdown-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/RingtoneManager;-stopPreviousRingtone-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/InitAlarmsService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-setBatteryState-(I I I I I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/backup/BackupAgentHelper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothInputDevice;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/MessageHeaderView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/SecurityPolicy;-setAccountPolicy-(J Lcom/android/emailcommon/provider/Policy; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/photos/views/HeaderGridView$FullWidthFixedViewLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/PicoService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/email/EmailIntentService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/HapticFeedbackController$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/TrimTimeBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-dispatchMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DayFragment;-eventsChanged-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/email/service/ImapService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/ComposeActivityEmail;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMnsObexClient;-connect-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/BadBehaviorActivity$BadService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/ChooseAccountActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService;-getTetherableWifiRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-sendVcardListingXml-(I Ljavax/obex/Operation; I I Ljava/lang/String; Ljava/lang/String;)I' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothA2dp;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothMap$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/net/sip/SipManager;-getSessionFor-(Landroid/content/Intent;)Landroid/net/sip/SipSession;' : ['android.permission.USE_SIP'],
	'Landroid/widget/NumberPicker;-dispatchHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-addSmsEntry-()V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$4500-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationViewAdapter$MessageHeaderItem;-getTimestampLong-()Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ProxyLauncher;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/compose/EnterSubject;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingScoOn-()Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler$StartupState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/test/mock/MockContentResolver;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/content/IContentService$Stub$Proxy;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/app/ChooserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/BackupManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/app/LauncherActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/DialogPicker;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/SmsManager;-enableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/browser/DataController$DataControllerHandler;-doUpdateHistoryTitle-(Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/inputmethod/pinyin/SkbContainer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-onStart-()V' : ['android.permission.GET_TASKS'],
	'Ljava/net/URL;-getContent-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/widget/TextClock$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService$1;-onUserSwitching-(I Landroid/os/IRemoteCallback;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/GLTextureTarget;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/base/FrameBranch;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/speechrecorder/SpeechRecorderActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/print/PrintManagerService$1;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$1000-(Lcom/android/server/net/NetworkPolicyManagerService; Landroid/net/NetworkTemplate; I)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/base/CallbackFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/MailboxSettings;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/MultiAdapterSpinner;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-syncOutbox-(Landroid/content/Context; Landroid/content/ContentResolver; Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Mailbox;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/widget/BookmarkWidgetConfigure;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/INetworkStatsService;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/ex/photo/PhotoViewPager;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ShutterButton;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/AccountsTester;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/quicksearchbox/SearchActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/BadBehaviorActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService;-enforceTetherChangePermission-()V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/DBHelper;-upgradeFromVersion21ToVersion22-(Landroid/database/sqlite/SQLiteDatabase; Landroid/content/Context;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/webkit/WebView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ScrollingTabContainerView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/EditText;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$Receiver;-decrementPendingBroadcastsLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-handleMetadata-(Landroid/content/Context; Ljava/io/InputStream; Ljava/lang/String;)V' : ['android.permission.INTERNET'],
	'Lcom/android/browser/view/BookmarkExpandableView;-onTop-(Lcom/android/browser/BreadCrumbView; I Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/SecureAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-slideFromOldKeyToNewKey-(Lcom/android/inputmethod/keyboard/Key; I I J Lcom/android/inputmethod/keyboard/Key; I I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-proceedAfterWildChar-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationListFragment;-requestDelete-(I Ljava/util/Collection; Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/GridLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-registerCode-(I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/deskclock/widget/TextTime;-setFormat24Hour-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-updateRunningAccounts-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calculator2/PanelSwitcher;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/SearchMailActionBarView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$HideNavInputEventReceiver;-onInputEvent-(Landroid/view/InputEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/View;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RadioGroup;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/nfc/cardemulation/HostApduService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/AccessibleDateAnimator;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/ComposingView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasProvision;-test-()Lcom/android/emailcommon/provider/Policy;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/InputMethodManagerService;-switchToLastInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/MovieActivity;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/GLTextureSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/TransitionsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/view/menu/IconMenuView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarController;-sendEvent-(Ljava/lang/Object; Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/CalendarColorPickerDialog;-startQuery-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-updateLayout-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/MediaLinearLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-stopDtmf-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/TelephonyManager;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/view/IInputMethodManager;-getEnabledInputMethodList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/ConnectivityManager;-getTetherableBluetoothRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/SlidingDrawer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Gallery;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/polarclock/PolarClockSettings;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/eas/EasOperation;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/widget/TextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavigationBarBase;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getNextAlarm-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/provider/Browser;-updateVisitedHistory-(Landroid/content/ContentResolver; Ljava/lang/String; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorRectView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSync;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/Gallery;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/AnalogClock;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/AsyncQueryServiceHelper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityManagerService$25;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryDownloadProgressBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-addGpsStatusListener-(Landroid/location/IGpsStatusListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/IAudioService;-adjustSuggestedStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/preference/PreferenceActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/ActivityController;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/inputmethodservice/ExtractButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RemoteViewsService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/PackageSummary;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;-refreshNetworkState-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/base/FrameBranch;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/UI;-showWeb-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/base/FrameSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/browser/Controller;-onDestroy-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hfp/HeadsetPhoneState;-cleanup-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/print/PrintManagerService;-access$100-(Lcom/android/server/print/PrintManagerService;)Lcom/android/server/print/UserState;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothGatt;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/service/EasService;-pingNeededForAccount-(Lcom/android/emailcommon/provider/Account;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/photos/views/GalleryThumbnailView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/photos/views/BlockingGLTextureView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/FilmStripView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/CalendarView;-access$2200-(Landroid/widget/CalendarView; Ljava/util/Calendar;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSecurity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/AudioPreview;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/SharpenEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/text/method/TextKeyListener;-access$200-(Landroid/text/method/TextKeyListener; Landroid/content/ContentResolver;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/UrlHandler;-handleMenuClick-(Lcom/android/browser/Tab; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager$Stub;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/calculator2/CalculatorEditText;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/text/method/PasswordTransformationMethod;-onTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-onRestart-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothGatt;-writeDescriptor-(Landroid/bluetooth/BluetoothGattDescriptor;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/activity/setup/AccountSetupIncoming;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/provider/EmailProvider;-restartPush-(Landroid/accounts/Account;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/TwoPaneController;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/soundrecorder/Recorder;-startRecording-(I Ljava/lang/String; Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO'],
	'Lcom/android/bluetooth/hdp/HealthService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/hardware/location/IGeofenceHardware;-addCircularFence-(I I D D D I I I I Landroid/hardware/location/IGeofenceHardwareCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/ServiceWatcher$2;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/ModuleSwitcher;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/phasebeam/PhaseBeam;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-onLongPress-(Lcom/android/inputmethod/keyboard/PointerTracker;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.VIBRATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/base/FrameStore;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/magicsmoke/MagicSmokeSelector;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/photo/PhotoViewPager;-scrollToItem-(I Z I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/ex/photo/views/PhotoView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseAccountTypeActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/IWindowSession$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/ConnectivityManager;-getNetworkInfo-(I)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/service/ImapService$1;-updateFolderList-(J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/ActionBarContainer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-sendMeetingResponse-(J I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/widget/VideoView;-access$2200-(Landroid/widget/VideoView; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/CenteredLinearLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/ui/PhotoView$MyGestureListener;-onScale-(F F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/timer/TimerListItem;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/GLRootView$IdleRunner;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/RepeatingImageButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-init-(Landroid/content/Context; Landroid/view/IWindowManager; Landroid/view/WindowManagerPolicy$WindowManagerFuncs;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/AllInOneActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/NativeActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/alarmclock/DigitalAppWidgetService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/MessageAttachmentBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-updateConversation-(Ljava/util/Collection; Landroid/content/ContentValues;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/MultiAutoCompleteTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-restartDataStallAlarm-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/StackView$StackFrame;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/CalendarBackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/inputmethodservice/InputMethodService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoSource;-load-(Lcom/android/dreams/phototable/PhotoSource$ImageData; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/recurrencepicker/WeekButton;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/SyncAdapterDriver;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/ErrorConsoleView$ErrorConsoleListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextView$ChangeWatcher;-beforeTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationListView$HintText;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/LatinIME;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub;-getRemoteAlias-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/EasOutboxService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-setNetworkPreference-(I)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/camera/ui/MoreSettingPopup;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothA2dp;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/ScreensaverSettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/galaxy/Galaxy;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/compose/EmptyService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/ImapAuthenticatorService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasFolderSync;-validate-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Chronometer;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EventLocationAdapter$LocationFilter$1;-doInBackground-([Ljava/lang/Void;)Ljava/util/List;' : ['android.permission.READ_CALENDAR'],
	'Landroid/test/mock/MockContentResolver;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/speech/IRecognitionService$Stub$Proxy;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/widget/CalendarView$WeekView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/KeyboardView;-onLongPress-(Landroid/inputmethodservice/Keyboard$Key;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/AccessibleTextView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GeofenceManager;-scheduleUpdateFencesLocked-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/print/PrintManagerService;-addPrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/app/FragmentTabHost;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/grass/GrassWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/CameraBackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationPager;-setCurrentItem-(I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/settings/SettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub;-getProvisioningOrActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/net/NetworkPolicyManagerService$6;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/LocationManagerService$Receiver;-incrementPendingBroadcastsLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/RenamingDelegatingContext;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/WifiManager;-getScanResults-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/calendar/DayView;-setSelected-(Landroid/text/format/Time; Z Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/net/VpnService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/ViewMode;-handleRestore-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/gatt/GattService;-beginReliableWrite-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/TextServicesManagerService$TextServicesSettings;-isSpellCheckerEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/view/menu/ActionMenuPresenter$OverflowMenuButton;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AlbumPage;-showDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/RecognitionManagerService$MyPackageMonitor;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TelephonyRegistry;-notifyServiceState-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/widget/TextTime;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/display/DisplayManagerService;-setDisplayInfoOverrideFromWindowManager-(I Landroid/view/DisplayInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/svox/pico/CheckVoiceData;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-getHealthDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$SyncHandlerSynchronizer;-syncComplete-(Z Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/email2/ui/MailboxSelectionActivityEmail;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-setFacilityLock-(Ljava/lang/String; Z Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothGatt;-connect-(Ljava/lang/Boolean; Landroid/bluetooth/BluetoothGattCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/media/effect/effects/LomoishEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getAdapterConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/BackupManagerService$PerformRestoreTask;-restoreNextAgent-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-onFinishInflate-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-access$1500-(Lcom/android/calendar/agenda/AgendaWindowAdapter; Lcom/android/calendar/agenda/AgendaWindowAdapter$AgendaItem; J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/content/SyncManager$5;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/ScreensaverActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/stopwatch/StopwatchService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController$ApEnabledState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/certinstaller/CertInstallerMain;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/QsbApplicationWrapper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/AttachmentTile;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/AutoSendActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/UrlInputView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/SlotView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/pm/UserManagerService$5;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/PrintJobConfigActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/ActivityManagerNative;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/deskclock/AnalogClock;-updateContentDescription-(Landroid/text/format/Time;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-onMenuClicked-(I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/ContentResolver;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/music/DeleteItems;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/StateManager;-startState-(Ljava/lang/Class; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/SettingInjectorService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/CountryDetectorBase;-setCountryListener-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/browse/ConversationOverlayItem;-rebindView-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/research/ReplayerService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/ExchangeService;-isMailboxSyncable-(Lcom/android/emailcommon/provider/Account; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Landroid/net/ConnectivityManager;-getTetherableUsbRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/deskclock/AlarmClockFragment$7;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/UiController;-openTab-(Ljava/lang/String; Z Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/RIL;-setupDataCall-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Pending;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/btservice/ProfileService;-stop-()Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/email/activity/setup/AccountSetupType;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/commands/bmgr/Bmgr;-run-([Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/VideoEditorBaseActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/ListPrefSettingPopup;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub$Proxy;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/gallery3d/app/BatchService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/Application;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/ExchangeService;-stopNonAccountMailboxSyncsForAccount-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/CallbackFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-sendDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ActivityController;-start-(Landroid/content/Intent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/gallery3d/app/PhotoPage$6;-onPhotoChanged-(I Lcom/android/gallery3d/data/Path;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/AbsSeekBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/ColorChipView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/AddBookmarkPage$1;-onLoadFinished-(Landroid/content/Loader; Lcom/android/browser/AddBookmarkPage$EditBookmarkInfo;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/camera/VideoModule;-onSharedPreferenceChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/FolderSelectionActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/CameraSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/filterpacks/base/RetargetFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/ByteStreams$5;-getInput-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/musicvis/vis4/Visualization4;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService;-access$1000-(Lcom/android/server/InputMethodManagerService; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/performance/ThroughputFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ExpandableListView;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController;-showNextConversation-(Ljava/util/Collection;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-getService-()Lcom/android/bluetooth/map/BluetoothMapService;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/SlideshowView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/Activity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/DriverCall; Lcom/android/internal/telephony/cdma/CdmaCallTracker; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaRouter$RouteGroup;-requestUpdateVolume-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CallManager;-setAudioMode-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-updateQuietMode-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSettings;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailsync/EmailSyncAlarmReceiver$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SmsBroadcastUndelivered;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/speech/tts/TextToSpeech$Connection;-runAction-(Landroid/speech/tts/TextToSpeech$Action; Ljava/lang/Object; Ljava/lang/String; Z Z)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/TextViewWithCircularIndicator;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/TextViewWithCircularIndicator;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountAuthenticatorActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/connectivity/PacManager;-get-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/app/NativeActivity$NativeContentView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-access$1500-(Lcom/android/bluetooth/btservice/BondStateMachine; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/cdma/CdmaCallWaitingNotification; Lcom/android/internal/telephony/cdma/CdmaCallTracker; Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/service/ApiService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ShutterButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/photo/PhotoViewPager;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/galaxy/GalaxyView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/ArtistAlbumBrowserActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/UiController;-closeTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/emailsync/SyncManager$2;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-setIccLockEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/PhotoSetFragment;-deleteItemWithPath-(Ljava/lang/Object;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/MediaPlaybackActivity$9;-onRepeat-(Landroid/view/View; J I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/wallpaper/WallpaperService$Engine$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/LocationManagerService$Receiver;-callProviderEnabledLocked-(Ljava/lang/String; Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/dreams/phototable/FlipperDream;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextSwitcher;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/widget/DrawerLayout$AccessibilityDelegate;-performAccessibilityAction-(Landroid/view/View; I Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BaseUi;-setUseQuickControls-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/IWifiManager$Stub;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/emailcommon/utility/EmailAsyncTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/service/dreams/DreamService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneProxy;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/AbstractSettingPopup;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarController;-launchDeleteEvent-(J J J)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/music/CreatePlaylist;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$SetIpForwardingEnabledErrorState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/calculator2/CalculatorViewPager;-populate-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/gadget/MediaSetSource;-resolveAlbumPath-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/VibratorService;-access$200-(Lcom/android/server/VibratorService;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/alerts/DismissAlarmsService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/TouchInterceptor$1;-onFling-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/EasOutboxService;-messageMoveRequest-(Lcom/android/exchange/MessageMoveRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/SearchMailActionBarView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-interceptFallback-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/app/MediaRouteControllerDialog;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter;-bindExpandArea-(Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter$ItemHolder; Lcom/android/deskclock/provider/Alarm;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/svox/pico/GetSampleText;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/MediaPlayer;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/packageinstaller/UninstallAppProgress;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService;-notifyProfileRemoved-(Landroid/net/sip/SipProfile;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralTestActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hid/HidService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/sip/SipService;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/mail/ui/CreateShortcutActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/IMediaPlaybackService;-setQueuePosition-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageShow;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService;-handleDisplayDeviceChanged-(Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-releaseWakeLock-(J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothA2dp;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/service/EasService$RestartPingsTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/PointerLocation;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/provider/Browser;-getAllBookmarks-(Landroid/content/ContentResolver;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/videosrc/MediaSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/CountdownTimerPopup;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-changeIccLockPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager;-waitForConnectivity-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-addPhoneStateListener-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$2;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/Call;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-populateRepeats-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-setMaximumScreenOffTimeoutFromDeviceAdminInternal-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/accessibility/AccessibleKeyboardViewProxy;-onTransitionKey-(Lcom/android/inputmethod/keyboard/Key; Lcom/android/inputmethod/keyboard/Key; Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/FragmentRunnable;-go-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/development/Connectivity;-onStartScanCycle-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothPan;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/providers/downloads/ui/TrampolineActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/gadget/MediaSetSource;-onContentDirty-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-prev-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/DisplayPowerController$Callbacks;-onStateChanged-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-chooseNewDefaultIMELocked-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioManager;-isWiredHeadsetOn-()Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/videoeditor/widgets/ZoomControl;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ButteryProgressBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/widget/SizeAdaptiveLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/ui/MtpThumbnailTileView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IEmailServiceCallback;-loadAttachmentStatus-(J J I I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/event/EditEventView;-onCancel-(Landroid/content/DialogInterface;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/widget/ScrollingTabContainerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/TextProgressBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/CantAddAccountActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/photo/PhotoViewPager;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/camera/ui/Switch;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/os/IVibratorService;-vibrate-(I Ljava/lang/String; J Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/content/Context;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/IngestService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/widget/TextTime;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/AttachmentDownloadService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/app/DisableCarModeActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hdp/HealthService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/AsyncQueryServiceHelper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/DigitalClock;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SlotView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/NoSearchActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppManager;-disableBluetooth-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/PhotoView$MyGestureListener;-onUp-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-revertToNitzTimeZone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-access$700-(Lcom/android/deskclock/stopwatch/StopwatchFragment;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-onActionItemClicked-(Landroid/view/ActionMode; Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/photos/shims/MediaItemsLoader;-<init>-(Landroid/content/Context;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/CallbackFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/MediaRouteButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/photos/views/TiledImageView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-sendCheckAlarmsMessage-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/AllInOneActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/eas/EasSyncBase;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/view/inputmethod/InputMethodManager;-switchToLastInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupIncoming;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email2/ui/MailActivityEmail;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/RIL;-cancelPendingUssd-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/BitmapOverlayEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/IConnectivityManager;-getNetworkPreference-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/InboundSmsHandler$WaitingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-pickAlbum-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/widget/BookmarkWidgetConfigure;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/gatt/GattService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/galaxy4/GalaxyView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Bookmarks;-removeFromBookmarks-(Landroid/content/Context; Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-handleCpbrCommand-(Ljava/lang/String; I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-makeCellBroadcastHandler-(Landroid/content/Context;)Lcom/android/internal/telephony/CellBroadcastHandler;' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/VignetteEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterfw/core/Filter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/InputMethodManagerService;-shouldOfferSwitchingToNextInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/ArtistAlbumBrowserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService;-access$400-(Lcom/android/server/LocationManagerService; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/MediaSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/CreateEventDialogFragment$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/policy/impl/GlobalActions$5;-onLongPress-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/browser/TabBar;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/ui/OnePaneController;-handleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/photos/GalleryActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothPbap$Stub$Proxy;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/IAccountManager;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/photos/views/HeaderGridView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/ui/IngestGridView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/AbstractSyncService;-hasConnectivity-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processGetInkey-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-initSocket-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/pm/PackageManagerService;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcDisconnectingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/emailcommon/service/AccountServiceProxy;-<init>-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/vcard/VCardParserImpl_V30;-parseOneVCard-()Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/BrowserPreferencesPage;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService;-enforceTetherAccessPermission-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/compose/EnterSubject;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-startDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/email/EmailConnectivityManager;-getActiveNetworkType-(Landroid/content/Context;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/MediaPlaybackService;-access$2300-(Lcom/android/music/MediaPlaybackService;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendTerminalResponse-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/print/PrintManagerService;-getInstalledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppIncomingFileConfirmActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralTestActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/MultiAutoCompleteTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/CameraSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-handleMove-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationListFragment;-onItemLongClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallHeld-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/datetimepicker/date/SimpleMonthAdapter;-onDayTapped-(Lcom/android/datetimepicker/date/SimpleMonthAdapter$CalendarDay;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasProvision;-performAckRequestForWipe-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-handleMotionEventStateDelegating-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine;-access$4400-(Landroid/net/wifi/WifiStateMachine; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onDisconnectDone-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-createLoaderUri-()Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/view/IApplicationToken$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/CaptivePortalTracker;-access$1200-(Landroid/net/CaptivePortalTracker; Landroid/net/NetworkInfo;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/photos/views/TiledImageView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$4;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/BluetoothManagerService;-handleEnable-(Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/MailActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-setCellInfoListRate-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-handleDragFromStarred-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/TextServicesManagerService;-setCurrentSpellCheckerSubtype-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/IconMenuView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/PackageSummary;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchToFirstImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailcommon/service/IEmailService;-loadAttachment-(Lcom/android/emailcommon/service/IEmailServiceCallback; J J Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/appwidget/AppWidgetHostView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperPreview;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumDataLoader;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/musicvis/vis2/Visualization2;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/printservice/PrintService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityManagerService;-closeSystemDialogsLocked-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/widget/ActionBarContextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-handleIntent-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/FaceView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/alerts/DismissAlarmsService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractMailActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/content/ContentResolver;-startSync-(Landroid/net/Uri; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/MailLogService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/PopupWindow$PopupViewContainer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$6;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-updateState-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Controller;-closeEmptyTab-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/LinearLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-acknowledgeLastIncomingGsmSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AdapterView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/MediaSaveService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/ActivityController;-onDestroy-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/opp/TestActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/chips/RecipientEditTextView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasOutboxService;-postAutodiscover-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/spellcheck/SpellCheckerSettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationSyncDisabledTipView;-calculateReasonSyncOff-(Lcom/android/mail/preferences/MailPrefs; Lcom/android/mail/providers/Account; Lcom/android/mail/preferences/AccountPreferences;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/event/EventLocationAdapter;-queryRecentLocations-(Landroid/content/ContentResolver; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_CALENDAR'],
	'Ljava/net/URL;-openConnection-()Ljava/net/URLConnection;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/ui/EdgeView;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/mtp/MtpDatabase;-endSendObject-(Ljava/lang/String; I I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/NetInitiatedActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/SizeClustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/AbsListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/AccountService$1;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-validate-(Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/ex/chips/RecipientEditTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumPicker;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email2/ui/MailActivityEmail;-onBackPressed-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/NumberPicker$AccessibilityNodeProviderImpl;-performAction-(I I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/SuggestionsView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email2/ui/MailboxSelectionActivityEmail;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/MinTimeProgressView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/ContactsContract$RawContacts$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/ISms$Stub;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/calendar/event/EventLocationAdapter;-queryContacts-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/util/HashSet;)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-onResume-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/widget/DateTimeView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationSyncDisabledTipView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DayView;-init-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/camera/CameraBackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/SecureAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/activity/setup/AccountSetupType;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/CheckBox;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/ExpandedGridView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/RecognitionService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DayView;-access$5100-(Lcom/android/calendar/DayView; Landroid/view/MotionEvent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/provision/DefaultActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/LocalAlbumSet$AlbumsLoader;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/backup/IBackupManager$Stub;-clearBackupData-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/ClockFragment$2;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/ObjectSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/FolderListLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/gallery3d/app/ActivityState;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mail/ui/OnePaneController;-getFolderChange-(Ljava/util/Collection; Ljava/util/Collection; Z Z Z Z Lcom/android/mail/providers/Folder;)Lcom/android/mail/ui/DestructiveAction;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/app/PlatLogoActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/FullscreenViewer;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/WebViewController;-switchToTab-(Lcom/android/browser/Tab;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/gesture/GestureOverlayView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pan/PanService;-doStart-(Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/service/EasMeetingResponder;-sendHttpClientPost-(Ljava/lang/String; [B)Lcom/android/exchange/EasResponse;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/documentsui/SettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/photos/GalleryActivity$TabsAdapter;-onTabSelected-(Landroid/app/ActionBar$Tab; Landroid/app/FragmentTransaction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ConversationViewFragment;-renderMessageBodies-(Lcom/android/mail/browse/MessageCursor; Z)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Landroid/inputmethodservice/KeyboardView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-removeTrack-(J)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/WeightedLinearLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processAtCpbr-(Ljava/lang/String; I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-<init>-(J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/provider/BrowserProvider2;-setParentValues-(J Landroid/content/ContentValues;)Z' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/app/ContextImpl;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/print/PrintManager;-cancelPrintJob-(Landroid/print/PrintJobId;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Bookmarks;-queryCombinedForUrl-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/videoeditor/widgets/EditorRelativeLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorHueView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/FinalPort;-setFieldFrame-(Landroid/filterfw/core/Frame; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CalendarView;-setMaxDate-(J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/ImapService;-searchMailboxImpl-(Landroid/content/Context; J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter;-expandAlarm-(Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter$ItemHolder; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/CountDownView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/UiModeManagerService;-sendConfigurationLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/http/SslCertificate;-formatCertificateDate-(Landroid/content/Context; Ljava/util/Date;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/PieControl$1;-onLayout-(I I Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/os/IBatteryPropertiesListener$Stub$Proxy;-batteryPropertiesChanged-(Landroid/os/BatteryProperties;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/LogViewer;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImagePoint;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameStore;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/net/ConnectivityManagerCompat$GingerbreadConnectivityManagerCompatImpl;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/wifi/WifiController;-<init>-(Landroid/content/Context; Lcom/android/server/wifi/WifiService; Landroid/os/Looper;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/google/common/io/MultiInputStream;-advance-()V' : ['android.permission.INTERNET'],
	'Landroid/net/sip/SipAudioCall;-access$1000-(Landroid/net/sip/SipAudioCall; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/ContactsContract$Groups$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/ingest/ui/MtpFullscreenView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-removeAccount-(Landroid/accounts/Account; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$4400-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/AttachmentDownloadService$DownloadSet;-tryStartDownload-(Lcom/android/email/service/AttachmentDownloadService$DownloadRequest;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-adjustOtherSinkPriorities-(Lcom/android/bluetooth/a2dp/A2dpService; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/ViewStub;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AutoCompleteTextView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-initializeConversationListFragment-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onSetPolicyDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService;-getAllowedResolutionLevel-(I I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-addTrackerToRawTableAndSendMessage-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/BackupManagerService$PerformRestoreTask;-finalizeRestore-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/CacheAbuser;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/DhcpStateMachine$RunningState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/editors/SwapButton;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/dreams/DreamService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/photos/views/HeaderGridView$FullWidthFixedViewLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/svox/pico/DownloadVoiceData;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub;-getRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Landroid/support/v13/app/FragmentTabHost;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/WiredAccessoryManager;-access$700-(Lcom/android/server/WiredAccessoryManager; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/location/LocationFudger;-access$000-(Lcom/android/server/location/LocationFudger;)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/PrintJobConfigActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-removeGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-deleteAccountPIMData-(Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIccCardStatus-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phasebeam/PhaseBeam;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/test/mock/MockApplication;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-loadDictionaryAsync-()V' : ['android.permission.READ_USER_DICTIONARY'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/IntentHandler$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-turnOnUpstreamMobileConnection-(I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/base/FrameSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintJobConfigActivity$CustomEditText;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasAccountService;-messageMoveRequest-(Lcom/android/exchange/MessageMoveRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ingest/IngestService;-setClientActivity-(Lcom/android/gallery3d/ingest/IngestActivity;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/KenBurnsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pEnabledState;-enter-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/RecognitionManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/wifi/WifiManager$WifiLock;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AddBookmarkPage;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-killAllBackgroundProcesses-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/server/wm/WindowManagerService;-reconfigureDisplayLocked-(Lcom/android/server/wm/DisplayContent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/Ringtone;-setStreamType-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/AlertActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/support/v4/view/ViewPager;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker$AccessibilityNodeProviderImpl;-sendAccessibilityEventForVirtualText-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine;-sendRssiChangeBroadcast-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/NavigationBarPhone;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AlarmTimelineView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasPing;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActiveServices;-bringUpServiceLocked-(Lcom/android/server/am/ServiceRecord; I Z Z)Ljava/lang/String;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/keyboard/KeyboardView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-moveTaskBackwards-(I)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/development/ProcessInfo;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/alerts/GlobalDismissManager;-processEventIds-(Landroid/content/Context; Ljava/util/Set;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getRemoteClass-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-detectAndSendKey-(Lcom/android/inputmethod/keyboard/Key; I I J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/EasOutboxService;-validate-(Ljava/lang/Class; Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/browse/FolderSpanTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/EmailProvider;-deleteAccountData-(Landroid/content/Context; J)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/dreams/basic/Colors;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-createStack-(I I I F)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-<init>-(Landroid/content/Context; Lcom/android/calendar/agenda/AgendaListView; Z)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/widget/AnalogClock;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setPairingConfirmation-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$1;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/PackageManagerBackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/SecureConversationViewFragment$SecureConversationWebViewClient;-onPageFinished-(Landroid/webkit/WebView; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/location/ILocationManager$Stub;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/CharStreams;-readLines-(Lcom/google/common/io/InputSupplier;)Ljava/util/List;' : ['android.permission.INTERNET'],
	'Lcom/android/videoeditor/widgets/MediaItemView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-access$3000-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker$CustomEditText;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ContentResolver;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/widget/RotarySelector;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/videoeditor/VideoEditorActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/GalleryAppImpl;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/btservice/BondStateMachine$PendingCommandState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-getLastCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/TextClock;-setFormat12Hour-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/RotateImageView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/WallpaperHandler;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.SET_WALLPAPER', 'android.permission.INTERNET'],
	'Lcom/android/server/am/BroadcastQueue;-scheduleBroadcastsLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/ImmersiveModeConfirmation$ClingWindowView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/MutableContextWrapper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService$CheckMp;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/OnePaneController;-requestFolderRefresh-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/util/SingleThreadNamedTaskExecutor$Worker;-loop-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageDraw;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-postEventFromNative-(Ljava/lang/Object; I I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintJobConfigActivity$CustomEditText;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DialerFilter;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/SecureSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/PolicyService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/deskclock/TimerSetupView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/Calculator;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/videoeditor/KenBurnsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-turnOffUpstreamMobileConnection-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/bluetooth/gatt/GattService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/ConversationViewFragment$1$1;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/UrlInputView;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/print/PrintJob;-isQueued-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiController;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/ShowActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/IsolatedContext;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-access$5200-(Lcom/android/internal/telephony/dataconnection/DataConnection; Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarUtils$TimeZoneUtils;-formatDateRange-(Landroid/content/Context; J J I)Ljava/lang/String;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/widget/WeightedLinearLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/AddBookmarkPage;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/IConnectivityManager;-getTetherableUsbRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/IApplicationToken;-windowsVisible-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/content/pm/IPackageManager;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-refresh-(Landroid/text/format/Time; J Ljava/lang/String; Z Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/wallpaper/fall/FallWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AttachmentTileGrid;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/TransitionsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-markAsUnused-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; I I)V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/ui/TileImageView;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/TrackBrowserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothGattServer;-connect-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/INativeDaemonConnectorCallbacks;-onDaemonConnected-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/MediaPlaybackService;-closeExternalStorageFiles-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/ex/photo/PhotoViewPager;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onFinishSlidingInput-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/printspooler/PrintJobConfigActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/RotateLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/MountService;-onDaemonConnected-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/browse/ConversationViewAdapter$MessageHeaderItem;-ensureTimestamps-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/Wallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/MediaPickerActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/videosrc/CameraSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/MailActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/IWifiManager$Stub;-setWifiEnabled-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/textservice/ITextServicesManager;-setCurrentSpellChecker-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothGatt;-abortReliableWrite-(Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/ActivityManager;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/deskclock/HandleApiCalls;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/documentsui/DirectoryContainerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/widget/sgv/StaggeredGridView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-fixAlarmInstances-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/PhotoPage$Model;-moveTo-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService$ScoClient;-decCount-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-access$400-(Landroid/inputmethodservice/KeyboardView; Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/SwipeableConversationItemView;-toggleSelectedState-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/camera/ProxyLauncher;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/quicksearchbox/DialogActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-hasContentChanged-()Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/wm/WindowManagerService;-setOverscanLocked-(Lcom/android/server/wm/DisplayContent; I I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/EmailApplication;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/WifiManager;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/dreams/phototable/PhotoTable$PhotoLaunchTask;-doInBackground-([Ljava/lang/Void;)Landroid/view/View;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-<init>-(Landroid/app/Activity; Landroid/view/View; Lcom/android/calendar/event/EditEventHelper$EditDoneRunnable; Z Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/eas/EasOptions;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/vcard/VCardParser;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/certinstaller/CertInstaller;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-processSildeOutFromOldKey-(Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupConnectionByIndex-(Lcom/android/internal/telephony/cdma/CdmaCall; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/EllipsizedMultilineTextView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/provider/BrowserProvider2;-getAccountNameAndType-(J)[Ljava/lang/String;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/ui/MailboxSelectionActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasMoveItems;-handleProvisionError-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/LocalAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-access$200-(Lcom/android/internal/policy/impl/PhoneWindowManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter$QueryHandler$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mail/browse/BorderView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-access$1700-(Lcom/android/gallery3d/app/AlbumPage; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/DatePicker;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractEditText;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/PermissionDetails;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-getLastDialledNumber-()Ljava/lang/String;' : ['android.permission.READ_CALL_LOG'],
	'Lcom/android/wallpaper/RenderScriptWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaObject-(Ljava/lang/String;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/commands/svc/DataCommand;-run-([Ljava/lang/String;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/backup/LocalTransportService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/WebVttRenderingWidget$CueLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasSyncService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I Z)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-readDescriptor-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/NavScreen;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManagerNative;-setPackageScreenCompatMode-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/BatchService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/UndoBarView;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/datetimepicker/time/TimePickerDialog;-processKeyUp-(I)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/pinyin/SettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/ProcessInfo;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/pm/Pm;-runTrimCaches-()V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/mail/ui/TwoPaneController;-hideWaitForInitialization-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/usb/UsbService;-systemReady-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/PowerManagerService;-checkIfBootAnimationFinished-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-showInputMethodMenuInternal-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/NotificationController$NotificationBroadcastReceiver$2;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/app/TrimVideo;-onPlayPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CalendarView$WeekView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getBatchedScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/ViewAnimator;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintJobConfigActivity$Editor$12;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/CameraBackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ShortcutNameActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/widget/BookmarkWidgetConfigure;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/printspooler/PrintDialogFrame;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-prev-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/AlertActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothDevice;-getBluetoothClass-()Landroid/bluetooth/BluetoothClass;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextView$3;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter;-access$500-(Lcom/android/gallery3d/app/SlideshowDataAdapter;)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-openPopupIfRequired-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavigationBarPhone;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/app/WallpaperManager;-setBitmap-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCallWaiting-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/NotificationController;-refreshNotificationsForAccountInternal-(Landroid/content/Context; J)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/ConversationUpdater;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/activity/setup/AccountSetupOutgoing;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService;-updateFromSettingsLocked-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/backup/LocalTransportService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/ui/MtpImageView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/QuickContactBadge$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/accessibility/TouchExplorer;-handleMotionEventStateTouchExploring-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaMmiCode;-processCode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AllInOneActivity;-setMainPane-(Landroid/app/FragmentTransaction; I I J Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/wallpaper/fall/Fall;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-access$100-(Lcom/android/gallery3d/ui/ActionModeHandler; Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoSource$ImageData;-getStream-(I)Ljava/io/InputStream;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/musicvis/vis3/Visualization3;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/provider/EmailProvider;-uiUpdateMessage-(Landroid/net/Uri; Landroid/content/ContentValues; Z)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/ConversationListCallbacks;-onConversationSelected-(Lcom/android/mail/providers/Conversation; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/provider/FolderPickerActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionaryList;-createUserDictSettings-(Landroid/preference/PreferenceGroup;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHeadset;-stopScoUsingVirtualVoiceCall-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/LockPatternUtils;-getString-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/FolderPickerActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/print/PrintManagerService;-getCurrentUserStateLocked-()Lcom/android/server/print/UserState;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertUtils;-formatTimeLocation-(Landroid/content/Context; J Z Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-updateThisDevice-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/RenamePlaylist;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtErrorActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService;-gotoNext-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/ProfileService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/dreams/phototable/LocalSource;-naturalNext-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/SelectionManager;-getSelected-(Z I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/recurrencepicker/RecurrencePickerDialog;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$1200-(Lcom/android/server/net/NetworkPolicyManagerService; Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/activity/EventViewer;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/power/Notifier;-access$700-(Lcom/android/server/power/Notifier;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/TwoPaneController;-updateAccounts-(Lcom/android/mail/content/ObjectCursor;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/MediaFocusControl$NotificationListenerObserver;-<init>-(Landroid/media/MediaFocusControl;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/email/AttachmentInfo;-<init>-(Landroid/content/Context; Lcom/android/emailcommon/provider/EmailContent$Attachment;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/certinstaller/CertInstaller;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarApplication;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/ComboSource;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/server/BluetoothA2dpService;-checkSinkSuspendState-(I)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/widget/TabWidget;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/LocationManager;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/text/ToUpperCase;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/datetimepicker/time/CircleView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/ButtonSwitcher;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/datetimepicker/time/RadialTextsView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/DownloadOverMeteredDialog;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService$RemoveAccountSession;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-access$1200-(Lcom/android/bluetooth/pbap/BluetoothPbapService;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-registerPhoneServiceStateListener-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/tts/TextToSpeech;-initTts-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumPicker;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/inputmethodservice/ExtractEditLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameStore;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothSocket;-<init>-(I I Z Z Ljava/lang/String; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/galaxy/Galaxy;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/LocationManager;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService;-onSetA2dpConnectionState-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email2/ui/MailboxSelectionActivityEmail;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/widget/ZoomControls;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothGatt;-setCharacteristicNotification-(Landroid/bluetooth/BluetoothGattCharacteristic; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/CoreSettingsObserver;-populateCoreSettings-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/SlideshowView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/NavigationBarPhone;-onAction-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/ui/AbstractActivityController;-handleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView$KeyTimerHandler;-startLongPressTimer-(Lcom/android/inputmethod/keyboard/PointerTracker; I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/browser/AccountsChangedReceiver$DeleteRemovedAccounts;-delete-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/gadget/WidgetService$PhotoRVFactory;-onCreate-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/print/IPrintManager;-validatePrinters-(Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/ImportTask;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/googlecode/mp4parser/authoring/adaptivestreaming/PackageWriter;-write-(Lcom/googlecode/mp4parser/authoring/Movie;)V' : ['android.permission.INTERNET'],
	'Lcom/android/camera/PhotoModule;-access$700-(Lcom/android/camera/PhotoModule;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/TrimVideo;-playVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/camera/ui/InLineSettingCheckBox$1;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/videoeditor/widgets/PreviewSurfaceView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/soundrecorder/SoundRecorder;-onStop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IEmailService;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/am/BroadcastQueue;-performReceiveLocked-(Lcom/android/server/am/ProcessRecord; Landroid/content/IIntentReceiver; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/deskclock/AndroidClockTextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/DismissAlarmsService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-dispatchUnhandledKey-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$InactiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationWebView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/ComboSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accessibility/EventStreamTransformation;-onMotionEvent-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/Switch;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/DirectoryContainerView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/support/v4/view/ViewPager$MyAccessibilityDelegate;-performAccessibilityAction-(Landroid/view/View; I Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/deskclock/AndroidClockTextView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavigationBarTablet;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/category/IconView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/sip/SipService$SipKeepAliveProcessCallback;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/basic/Colors;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/MessageAttachmentBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService;-doStart-(Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/musicvis/vis4/Visualization4;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/personalization/AccountUtils;-getDeviceAccountsWithDomain-(Landroid/content/Context; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/support/v4/view/ViewPager;-setCurrentItem-(I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/spellcheck/SpellCheckerSettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/eas/EasPing;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/LatinIME$UIHandler;-onStartInput-(Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/ApplicationPackageManager;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calculator2/CalculatorEditText;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/CreateShortcutActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/LomoishEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageHeaderView;-showDetailsPopup-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/CircleTimerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/speech/tts/TextToSpeech;-<init>-(Landroid/content/Context; Landroid/speech/tts/TextToSpeech$OnInitListener;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ViewStub;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/google/android/mms/util/PduCache;-put-(Ljava/lang/Object; Ljava/lang/Object;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-assignContactFromPhone-(Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/net/wifi/IWifiManager$Stub;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/commands/bmgr/Bmgr;-doRestoreAll-(J Ljava/util/HashSet;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-notCurrentSession-(Landroid/net/sip/ISipSession;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-startNextVibrationLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/filterpacks/base/InputStreamSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onRoamingOff-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/MoviePlayer;-pauseVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-showWaitForInitialization-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityStack;-moveTaskToFrontLocked-(Lcom/android/server/am/TaskRecord; Lcom/android/server/am/ActivityRecord; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$DriverStartedState;-enter-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub;-isRotationFrozen-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarApplication;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothMap;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/LockPatternUtils;-isSecure-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowSession;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/speech/tts/TextToSpeech$12;-run-(Landroid/speech/tts/ITextToSpeechService;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapObexServer;-setMessageStatus-(Ljava/lang/String; Lcom/android/bluetooth/map/BluetoothMapAppParams;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/google/AbstractGoogleSource;-getSuggestions-(Ljava/lang/String; I)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/research/FeedbackLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/Connectivity;-onRoutedRequest-(I)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/test/RenamingDelegatingContext;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/IRecognitionService$Stub$Proxy;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processBIPClient-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/documentsui/DocumentsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/CreatePlaylist;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/preferences/WebsiteSettingsFragment$SiteAdapter$UpdateFromBookmarksDbTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/OnePaneController;-handleDropInStarred-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/ConversationCursor;-resume-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/FragmentBreadCrumbs;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment$1;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/HandleApiCalls;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService;-checkManageAccountsOrUseCredentialsPermissions-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PhotoPage;-addSecureAlbumItem-(Z I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/support/v4/print/PrintHelperKitkat;-printBitmap-(Ljava/lang/String; Landroid/graphics/Bitmap;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/WindowManagerPolicy;-init-(Landroid/content/Context; Landroid/view/IWindowManager; Landroid/view/WindowManagerPolicy$WindowManagerFuncs;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/location/GeofenceHardwareService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-proceedAfterWaitChar-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/Filter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaListView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-setShuffleMode-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/GeofenceHardware;-registerForMonitorStateChangeCallback-(I Landroid/hardware/location/GeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/mail/Store;-autoDiscover-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/camera/ui/ListPrefSettingPopup;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-processPostDialChar-(C)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/mail/store/ImapStore;-autoDiscover-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/CoreSettingsObserver;-<init>-(Lcom/android/server/am/ActivityManagerService;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-unDeleteMessageMms-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/datetimepicker/AccessibleTextView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-goTo-(J Z Z Z)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/music/ScanningProgress;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IActivityManager;-killUid-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/compose/AttachmentsView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofenceAdd-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationMessage;-star-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/printspooler/SelectPrinterActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/IWifiManager;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/net/wifi/WifiStateMachine$TetheringState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/development/AccountsTester;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/SamplingProfilerService;-registerSettingObserver-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/datetimepicker/date/DatePickerController;-tryVibrate-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/search/SearchManagerService$MyPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-wakeUpFromNative-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/mail/Store;-checkSettings-()Landroid/os/Bundle;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$OnlineState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/app/IntentService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/ILocationManager$Stub;-reportLocation-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/media/AudioManager;-setParameters-(Ljava/lang/String;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/widget/LinearLayoutWithDefaultTouchRecepient;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getOutgoingCallerIdDisplay-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ImageSwitcher;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseAccountTypeActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/PrinterDiscoverySession;-stopPrinterDiscovery-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService$1;-sendMeetingResponse-(J I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityGroup;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityStack;-startPausingLocked-(Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-resumeRotationLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/provision/DefaultActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage$Model;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailsync/SyncManager$4;-run-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/photos/AlbumSetFragment;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiStateMachine$ConnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager;-onUserRemoved-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/VideoBrowserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoCarousel$PhotoLoadTask;-doInBackground-([Ljava/lang/Void;)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/support/v4/widget/SearchViewCompatIcs$MySearchView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractButton;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/HistoryItem;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/PhoneFactory;-makeSipPhone-(Ljava/lang/String;)Lcom/android/internal/telephony/sip/SipPhone;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/LatinIME;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperChange;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/EasAccountService;-tryAutodiscover-(Landroid/content/Context; Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab$2$6;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/settings/SettingsValues;-isLanguageSwitchKeyEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/TextViewWithCircularIndicator;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService;-moveQueueItem-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/LogTextBox;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$SetIpForwardingDisabledErrorState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/sip/SipSessionGroup;-processRequest-(Ljavax/sip/RequestEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-saveLockPassword-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/QsbApplicationWrapper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcActivatingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/time/RadialPickerLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/SlideshowPage$SequentialSource;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/FetchUrlMimeType;-run-()V' : ['android.permission.INTERNET'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-updateNextAlarm-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/noisefield/NoiseFieldWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-access$300-(Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothInputDevice;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/nsd/INsdManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.INTERNET'],
	'Landroid/view/WindowManagerPolicy;-dispatchUnhandledKey-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/BluetoothOppManager;-getDeviceName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/ConsumerIrManager;-transmit-(I [I)V' : ['android.permission.TRANSMIT_IR'],
	'Landroid/hardware/IConsumerIrService$Stub;-getCarrierFrequencies-()[I' : ['android.permission.TRANSMIT_IR'],
	'Landroid/bluetooth/IBluetoothManager;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/EmlViewerActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AttachmentTileGrid;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enforceReceiveAndSend-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/inputmethod/research/UploaderService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiService;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/PickerActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/RenderOverlay$RenderView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-setSymbolsKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pan/PanService;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$IndicatorView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NsdService;-access$700-(Lcom/android/server/NsdService; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/OpenDownloadReceiver;-onReceiveAsync-(Landroid/content/Context; J)V' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-updateBatchInput-(J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/LogTextBox;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Switch;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-getAllNetworkInfo-()[Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/print/IPrintManager$Stub;-createPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-render-(Lcom/android/gallery3d/glrenderer/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/selectcalendars/CalendarColorSquare;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSyncCalendar;-setSyncOptions-(Landroid/content/Context; Lcom/android/exchange/adapter/Serializer; D Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Mailbox; Z I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/ui/MenuExecutor$ConfirmDialogListener;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/CircleButtonsLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ZoomButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/VolumePanel;-onVolumeChanged-(I I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-explicitCallTransfer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController$ScanOnlyLockHeldState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/location/FlpHardwareProvider;-getInstance-(Landroid/content/Context;)Lcom/android/server/location/FlpHardwareProvider;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/pm/PackageManagerService;-access$2100-(Lcom/android/server/pm/PackageManagerService; Ljava/io/File; I I J Landroid/os/UserHandle;)Landroid/content/pm/PackageParser$Package;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/wifi/WifiNotificationController$NotificationEnabledSettingObserver;-register-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/BroadcastQueue;-backgroundServicesFinishedLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/OnePaneController;-delete-(I Ljava/util/Collection; Lcom/android/mail/ui/DestructiveAction; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/compose/ComposeAttachmentTile;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/FilmstripPage;-showBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LockSettingsService;-systemReady-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService;-serverDisconnect-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetherableBluetoothRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/RadioGroup;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountAuthenticatorActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-reportSmsMemoryStatus-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-getLongForUser-(Landroid/content/ContentResolver; Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/PinyinIME;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActiveServices$ServiceMap;-rescheduleDelayedStarts-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/ex/photo/PhotoViewPager;-onRestoreInstanceState-(Landroid/os/Parcelable;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/renderscript/RSSurfaceView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/AddBookmarkPage$CustomListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/crop/CropActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/DialogPicker;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActiveServices;-bringDownServiceIfNeededLocked-(Lcom/android/server/am/ServiceRecord; Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/RecognitionManagerService$MyPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/EasService;-getAuthoritiesToSync-(Landroid/accounts/Account; [Ljava/lang/String;)Ljava/util/Set;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-sendNotification-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; Z [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/TelephonyRegistry;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/OverlayLinearLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringLogger;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/ConnectivityManager;-setNetworkPreference-(I)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/camera/ui/ExpandedGridView;-sequenceScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHeadset;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/quicksearchbox/ui/SearchActivityViewSinglePane;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-composeAndSendPhonebookVcards-(Ljavax/obex/Operation; I I Z Ljava/lang/String;)I' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/ingest/adapter/MtpPagerAdapter;-setPrimaryItem-(Landroid/view/ViewGroup; I Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorCompareView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisToFileOutputStreamSpeechItem;-setRequestParams-(Landroid/speech/tts/SynthesisRequest;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/ImapService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-markAsDeleting-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; I I)V' : ['android.permission.INTERNET'],
	'Lcom/android/camera/ui/FilmstripBottomControls;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/EmailProvider;-uiSaveMessage-(Lcom/android/emailcommon/provider/EmailContent$Message; Lcom/android/emailcommon/provider/Mailbox; Landroid/os/Bundle;)Landroid/net/Uri;' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/music/TrackBrowserActivity;-removeItem-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/MtpDeviceIndex$ProgressListener;-onIndexFinish-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/Connectivity;-access$3300-(Lcom/android/development/Connectivity;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/browser/view/PieMenu;-onEnter-(Lcom/android/browser/view/PieItem;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getCallWaiting-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/photo/PhotoViewPager;-executeKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu;-destroy-(I Ljava/util/Collection; Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCDMASubscription-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSyncBase;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/app/DisableCarModeActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/RecognitionService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/InLineSettingMenu;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/os/PowerManager$WakeLock;-acquire-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasAccountService;-canProvision-(Lcom/android/exchange/EasSyncService;)Lcom/android/exchange/adapter/ProvisionParser;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService$4;-run-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/TabBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/WaveView;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertService;-processQuery-(Landroid/database/Cursor; Landroid/content/Context; J Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList;)I' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/documentsui/DocumentsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ActionableToastBar$2;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/Wallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/SwipeableListView;-onChildDismissed-(Lcom/android/mail/ui/SwipeableItemView;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-checkIfSubtypeBelongsToThisImeAndEnabled-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/crop/CropView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ScrollingTabContainerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/KenBurnsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/GeneralPreferences;-getRingtoneTitleFromUri-(Landroid/content/Context; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-performSnooze-(Landroid/net/NetworkTemplate; I)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/inputmethodservice/KeyboardView;-showPreview-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService$22;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/OnePaneController;-onConversationListVisibilityChanged-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/base/FrameStore;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EasMeetingResponder;-makeUriString-(Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/deskclock/AlarmClockFragment;-onTimeSet-(Lcom/android/datetimepicker/time/RadialPickerLayout; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/backup/LocalTransportService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/GridView;-sequenceScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-removeBond-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/CountDownView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/svox/pico/DownloadVoiceData;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-makeAppCrashingLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/CommandsInterface;-getLastPdpFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/FilmStripView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onLocalDisconnect-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/NotificationActionIntentService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/TwoPaneController;-disableCabMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/test/mock/MockApplication;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppLiveFolder;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calculator2/CalculatorViewPager;-onMeasure-(I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/grass/GrassView;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/Resources$1;-getInput-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/filtershow/editors/SwapButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/google/common/io/ByteStreams;-equal-(Lcom/google/common/io/InputSupplier; Lcom/google/common/io/InputSupplier;)Z' : ['android.permission.INTERNET'],
	'Landroid/provider/Telephony$Sms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/wifi/WifiService;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/svox/pico/EngineSettings;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothGatt;-abortReliableWrite-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/ListPrefSettingPopup;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calculator2/CalculatorViewPager;-setCurrentItemInternal-(I Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/FolderItemView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/EasOutboxService;-sendHttpClientPost-(Ljava/lang/String; [B)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/ShortcutActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-getCDMASubscription-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calculator2/ColorButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/LogTextBox;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/ScreensaverSettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/OnePaneController;-handleUpPress-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/AbstractActivityController;-showNextConversation-(Ljava/util/Collection; Ljava/lang/Runnable;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/magicsmoke/MagicSmoke;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-killBackgroundProcesses-(Ljava/lang/String; I)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/email/service/PolicyService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-unregisterClient-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/view/PieMenu$PieView;-layout-(I I Z F I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/Tab;-capture-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Editor$SelectionEndHandleView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/Invert;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/accounts/AccountManager$BaseFutureTask;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/DeskClock$TabsAdapter;-notifyPageChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/documentsui/DirectoryFragment$DocumentsAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/IBatteryStats$Stub$Proxy;-setBatteryState-(I I I I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readAccessibilityEnabledSettingLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageRotate;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/SearchActivityView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService;-scheduleTraversalLocked-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/PlaylistBrowserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/ex/photo/adapters/BaseFragmentPagerAdapter;-setPrimaryItem-(Landroid/view/ViewGroup; I Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/print/PrintManagerService;-createPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/provider/ContactsContract$Contacts;-getLookupUri-(J Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/ex/photo/PhotoViewPager;-pageRight-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/TextClock;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker$CustomEditText;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/ByteStreams;-copy-(Lcom/google/common/io/InputSupplier; Ljava/io/OutputStream;)J' : ['android.permission.INTERNET'],
	'Landroid/app/IActivityManager;-startRunning-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/exchange/ExchangeService;-stopAccountSyncs-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/telephony/MmsProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/net/IConnectivityManager;-updateLockdownVpn-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/ConversationPhotoTeaserView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCdmaSubscriptionSource-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/RotarySelector;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupIncoming;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/SnoozeAlarmsService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService$AppDeathRecipient;-binderDied-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/alerts/QuickResponseActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment$2;-onChanged-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/certinstaller/CertInstaller;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothHeadset;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/ProfileService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BluetoothManagerService;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/ex/photo/PhotoViewActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailsync/AbstractSyncService;-validateAccount-(Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/alarmclock/DigitalAppWidgetProvider;-refreshAlarm-(Landroid/content/Context; Landroid/widget/RemoteViews;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-access$1800-(Lcom/android/calendar/agenda/AgendaWindowAdapter; Lcom/android/calendar/agenda/AgendaWindowAdapter$QuerySpec;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/speech/tts/TextToSpeech;-getFeatures-(Ljava/util/Locale;)Ljava/util/Set;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/FormattedDateBuilder;-formatLongDayAndDate-(J)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/IPowerManager$Stub$Proxy;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DateTimeView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/SearchActivity;-onCreateOptionsMenu-(Landroid/view/Menu;)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/am/ActivityStackSupervisor;-comeOutOfSleepIfNeededLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ViewFlipper;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-acquireWakeLockWithUid-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DatePicker;-dispatchPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/ComboViewActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readComponentNamesFromSettingLocked-(Ljava/lang/String; I Ljava/util/Set;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CalendarView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/Pico;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/OnePaneController;-perhapsShowFirstSearchResult-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/Development;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-switchToNextInputMethodAndSubtype-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DeleteEventHelper;-deleteRepeatingEvent-(I)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/wallpaper/nexus/NexusWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/content/pm/IPackageManager$Stub;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperChange;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/search/SearchManagerService$GlobalSearchProviderObserver;-<init>-(Lcom/android/server/search/SearchManagerService; Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/BookmarkSearch;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BackupManagerService$PerformFullBackupTask;-backupOnePackage-(Landroid/content/pm/PackageInfo; Ljava/io/OutputStream;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/development/InstrumentationList;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/inputmethodservice/ExtractEditText;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/utils/UsabilityStudyLogUtils$2;-run-()V' : ['android.permission.WRITE_EXTERNAL_STORAGE'],
	'Lcom/android/email2/ui/MailActivityEmail;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/MoviePlayer$8;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/SubtitleView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AnimatedAdapter;-changeCursorAndColumns-(Landroid/database/Cursor; [Ljava/lang/String; [I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService$UserSwitchedReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/development/DataList;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/MediaPlaybackActivity;-doPauseResume-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/widget/DateTimeView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserBackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-access$100-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Service;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/ExchangeService;-stopManualSync-(J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Gallery;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasOutboxService;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/print/IPrintManager;-cancelPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MusicPicker;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothMap;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperPreview;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/BluetoothManagerService;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/IAccountManager;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/exchange/eas/EasMoveItems;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/grass/Grass;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/AddBookmarkPage;-addFolderToCurrent-(Ljava/lang/String;)J' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/IConnectivityManager;-getActiveNetworkQuotaInfo-()Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/app/NetInitiatedActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/Wallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Button;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-handleSendComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipCall;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/printspooler/PrintSpoolerService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetPhoneState;-listenForPhoneState-(Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/StatusBarManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/bluetooth/BluetoothInputDevice;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/filtershow/pipeline/ProcessingService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/AudioPreview;-onUserLeaveHint-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/RotarySelector;-vibrate-(J)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask$1;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onActionBarWanted-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowStateAnimator;-createSurfaceLocked-()Landroid/view/SurfaceControl;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiService;-getWifiStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getBasebandVersion-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getPreferredVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EasMeetingResponder;-sendMeetingResponseMail-(Lcom/android/emailcommon/mail/PackedString; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/bluetooth/BluetoothSocket;-initSocketFromFdNative-(I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothPan;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/Chronometer;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/VideoEditorBaseActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-broadcastIccStateChangedIntent-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/TwoPaneController;-changeFolder-(Lcom/android/mail/providers/Folder; Ljava/lang/String; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/svox/pico/Pico;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/setup/SetupStepIndicatorView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-processMessagePart-(Lcom/android/internal/telephony/InboundSmsTracker;)Z' : ['android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/am/ActivityStack;-completeResumeLocked-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/svox/pico/PicoService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/MediaFocusControl;-sendSimulatedMediaButtonEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/TouchExplorer;-handleMotionEventStateDelegating-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getLastDataCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CheckedTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/ScreensaverActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-registerReceiver-(Landroid/app/IApplicationThread; Ljava/lang/String; Landroid/content/IIntentReceiver; Landroid/content/IntentFilter; Ljava/lang/String; I)Landroid/content/Intent;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/location/LocationManager;-addProximityAlert-(D D F J Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService;-reportLocation-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/Controller$5;-doInBackground-([Ljava/lang/Void;)[Ljava/lang/String;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/filtershow/crop/CropActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-handlePollCalls-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/ui/DownloadList;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/FolderSelectionActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSetupBasics;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService;-enforceAccessPermission-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Landroid/location/Criteria; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-unregisterForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/BluetoothManagerService;-access$2900-(Lcom/android/server/BluetoothManagerService;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$Listener;-onSingleTapUp-(F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManager;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/Tab;-<init>-(Lcom/android/browser/WebViewController; Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/IBluetooth$Stub;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/MoreSettingPopup;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/BreadCrumbView;-clear-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/ContactsContract$Contacts;-getLookupUri-(Landroid/content/ContentResolver; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mail/browse/SwipeableConversationItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/MultiAutoCompleteTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/NestedFolderTeaserView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/QuickResponseActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioManager;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-updateNotificationWhenBtDisabled-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/DialogTitle;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DateTimeView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/EmlTempFileDeletionService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-setDataSourceImpl-(Landroid/media/MediaPlayer; Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/chips/RecipientEditTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$UnavailableState;-enter-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingRfcommOn-(I)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/app/ListActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/MediaPlayer;-stayAwake-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-onRefresh-(Z [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-updateData-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-createView-()Landroid/view/View;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/quicksearchbox/ui/CorpusView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/event/EventInterpreter;-onEvent-(Lcom/android/inputmethod/event/Event;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/ex/photo/PhotoViewPager;-arrowScroll-(I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/app/ActivityGroup;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-setTemporaryScreenAutoBrightnessAdjustmentSettingOverrideInternal-(F)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/TouchExplorer;-access$1900-(Lcom/android/server/accessibility/TouchExplorer; Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-destroyPending-(Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageMirror;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calculator2/CalculatorViewPager;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/service/IEmailService;-updateFolderList-(J)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/hardware/location/GeofenceHardwareService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/SlidingTab;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/RecognitionService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofencePauseStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-getPowerButtonInstantlyLocks-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/AddBookmarkPage;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailcommon/service/AccountServiceProxy;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/camera/ui/ModuleSwitcher;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/widget/TabHost;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/personalization/PersonalizationPredictionDictionary;-access$700-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/WebViewController;-onDownloadStart-(Lcom/android/browser/Tab; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/content/ContentResolver;-requestSync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/EasSyncService;-acknowledgeRemoteWipe-(Lcom/android/exchange/EasSyncService; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/EmlTempFileDeletionService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/photos/views/TiledImageView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-handleApplicationWtf-(Landroid/os/IBinder; Ljava/lang/String; Landroid/app/ApplicationErrorReport$CrashInfo;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DayView;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/widget/CheckBox;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/IBackupManager$Stub;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/RunningProcesses;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/MuteVideo$1;-run-()V' : ['android.permission.INTERNET'],
	'Lcom/android/mail/widget/WidgetService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/CircleTimerView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/TestActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-removeTrack-(J)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService;-updateWakeLocks-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/timer/TimerAlertFullScreen;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/RenderOverlay$RenderView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/LogViewer;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/AccountManager;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/mail/ui/SecureConversationViewFragment;-onAccountChanged-(Lcom/android/mail/providers/Account; Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/music/MediaPlaybackActivity$4;-onClick-(Landroid/view/View;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/LockPatternUtils;-getSalt-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/GlobalDismissManager;-lookupEventToCalendarMap-(Landroid/content/Context; Ljava/util/Set;)Ljava/util/Map;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/phasebeam/PhaseBeam;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/SecurityPolicy;-syncAccount-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ingest/ui/MtpImageView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$WaitingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/views/TiledImageView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/ExchangeService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/svox/pico/EngineSettings;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/EmlViewerActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/views/BlockingGLTextureView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/WaveView;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/PagerTabStrip;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/speech/tts/TextToSpeech;-shutdown-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-getDeviceIdentity-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager;-addPrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventFragment;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/PhoneProxy;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/svox/pico/GetSampleText;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/Details;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setupData-(Lcom/android/internal/telephony/dataconnection/ApnContext; I)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-disableCabMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/view/BookmarkContainer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/MediaSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/print/PrinterDiscoverySession;-destroy-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralTestActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BackupManagerService$PerformRestoreTask;-initiateOneRestore-(Landroid/content/pm/PackageInfo; I Landroid/app/IBackupAgent; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/content/ContentResolver;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/location/ILocationManager$Stub;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/deskclock/widget/TextTime;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-access$2000-(Lcom/android/bluetooth/pbap/BluetoothPbapService; I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/CalendarViewAdapter;-getDropDownView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-transitionToDelegatingStateAndClear-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService$WiredAccessoryCallbacks;-notifyWiredAccessoryChanged-(J I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-openFile-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService;-getOrCreateUserStateLocked-(I)Lcom/android/server/print/UserState;' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/IConsumerIrService$Stub$Proxy;-transmit-(Ljava/lang/String; I [I)V' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/music/MediaPlaybackActivity$8;-onClick-(Landroid/view/View;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupOutgoing;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/eas/EasSearch;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/SuperCollapsedBlock;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/exchange/adapter/ContactsSyncParser;-parse-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment$2;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageTinyPlanet;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-switchToNextInputMethod-(Landroid/os/IBinder; Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/EmptyConversationListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings; Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onMoveEvent-(I I J Landroid/view/MotionEvent;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-resendPendingMessages-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/power/PowerManagerService$DockReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DeleteEventHelper;-delete-(J J J I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/camera/ui/CheckedLinearLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-startDataStallAlarm-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextClock;-is24HourModeEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/State;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/Files;-readLines-(Ljava/io/File; Ljava/nio/charset/Charset; Lcom/google/common/io/LineProcessor;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/widget/LockPatternUtils;-deleteGallery-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-startScan-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/mail/ui/ConversationPhotoTeaserView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollViewWithNotifier;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowAnimator;-testTokenMayBeDrawnLocked-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AbstractCalendarActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/ActivityController;-onUpPressed-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/provider/Settings$System;-getFloat-(Landroid/content/ContentResolver; Ljava/lang/String;)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSync;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/PopupList$OnPopupItemClickListener;-onPopupItemClick-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-access$2500-(Lcom/android/gallery3d/app/PhotoDataAdapter; Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService$AudioHandler$1;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;-refreshInterface-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserHistoryPage$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/MediaScanner$MyMediaScannerClient;-setSettingIfNotSet-(Ljava/lang/String; Landroid/net/Uri; J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/PointerLocationView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ActionImage;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/StickyHeaderListView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothMap;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/DigitalClock;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/mail/store/ImapFolder;-fetchInternal-([Lcom/android/emailcommon/mail/Message; Lcom/android/emailcommon/mail/FetchProfile; Lcom/android/emailcommon/mail/Folder$MessageRetrievalListener;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/backup/FullBackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;-onConfirmDialogDismissed-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/SkbContainer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-setSymbolsKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-openFile-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-initLoader-(Ljava/lang/String;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationViewAdapter$MessageHeaderItem;-onModelUpdated-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/DayView;-doFling-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment$1;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager$AccountObserver$1;-run-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/MovieActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserPreferencesPage;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/Presentation$1;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-setEmojiKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/UrlInputView;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/test/mock/MockContentResolver;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/ex/photo/adapters/PhotoPagerAdapter;-instantiateItem-(Landroid/view/ViewGroup; I)Ljava/lang/Object;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/SmsApplication;-getDefaultSendToApplication-(Landroid/content/Context; Z)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/NetworkPolicyManager;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/AttachmentTileGrid;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/WaveView;-vibrate-(J)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setMute-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-getVoiceRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/smspush/WapPushManager;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt;-readDescriptor-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/recurrencepicker/WeekButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasOutboxService;-acknowledgeRemoteWipe-(Lcom/android/exchange/EasSyncService; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub$Proxy;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-onSurfaceDestroyed-(Landroid/view/SurfaceHolder;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/ui/SearchActivityViewSinglePane;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DayFragment;-makeView-()Landroid/view/View;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-scheduleReadyPeriodicSyncs-()J' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/wifi/WifiService;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/inputmethodservice/KeyboardView;-setKeyboard-(Landroid/inputmethodservice/Keyboard;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/FilterShowActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountAdapter$3;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/ShortcutActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/base/OutputStreamTarget;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/SettingsRedirector;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/OnePaneController;-confirmAndDelete-(I Ljava/util/Collection; Z I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ConversationViewProgressController$2;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/colorpicker/ColorPickerPalette;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-scrollToItem-(I Z I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/quicksearchbox/util/JavaNetHttpHelper;-get-(Lcom/android/quicksearchbox/util/HttpHelper$GetRequest;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/widget/TableRow;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupOutgoing;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/net/sip/ISipService;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/server/power/PowerManagerService;-userActivityFromNative-(J I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/media/WebVttTrack;-hide-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/ex/photo/views/PhotoView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/FilmstripPage;-access$3300-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/sip/ISipService$Stub;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-onContentDirty-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/CheckedTextView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$4600-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/DeskClock$TabsAdapter;-onTabSelected-(Landroid/app/ActionBar$Tab; Landroid/app/FragmentTransaction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/TrimVideo;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextClock;-chooseFormat-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/DataList;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-startWatchingForBootAnimationFinished-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/event/EventInterpreter;-onSoftwareEvent-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ButteryProgressBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BackupManagerService$PerformFullRestoreTask;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/am/ActivityManagerService;-startAppProblemLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/view/CustomScreenLinearLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/BackDropperEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/chips/RecipientEditTextView$RecipientReplacementTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/month/MonthListView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth;-setPairingConfirmation-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/recurrencepicker/RecurrencePickerDialog;-onCheckedChanged-(Landroid/widget/RadioGroup; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/AudioTrackLinearLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Connected;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/gesture/GestureOverlayView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothDevice;-getAlias-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/ConnectivityService$SettingsObserver;-observe-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-reinvokePersistentGroup-(Landroid/net/wifi/p2p/WifiP2pConfig;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/MediaPlayer$2;-addTrack-()I' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/calculator2/Calculator;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getNeighboringCids-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/UiAutomationConnection;-storeRotationStateLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub;-getActiveLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/PermissionDetails;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/email/activity/setup/AccountSetupOutgoing;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ToggleButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-next-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/ex/photo/PhotoViewPager;-onLayout-(Z I I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/ZoomButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/Utils;-getDisplayedDatetime-(J J J Ljava/lang/String; Z Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/app/MediaRouteButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/AudioTrackLinearLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-setAdapter-(Landroid/support/v4/view/PagerAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/Editor$SelectionStartHandleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/EditorRelativeLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/utils/VeiledAddressMatcher$1;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/PhoneUi;-toggleNavScreen-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/app/MediaRouteButton;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/AbstractCalendarActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hid/HidService;-doStop-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceRemoveStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LockSettingsService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/FilmstripPage;-notifyScreenNailChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/BluetoothManagerService;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/compose/EnterSubject;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextClock;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/providers/EmlAttachmentProvider;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.INTERNET'],
	'Lcom/android/server/print/PrintManagerService;-access$500-(Lcom/android/server/print/PrintManagerService; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService$Stub;-setStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/TabControl;-setCurrentTab-(Lcom/android/browser/Tab;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setMute-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/musicvis/vis2/Visualization2;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment;-onUserVisibleHintChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/content/MutableContextWrapper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractMailActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/LockPatternView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/printservice/PrintService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ProxyLauncher;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/activity/setup/AccountSecurity;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/DownloadManager;-enqueue-(Landroid/app/DownloadManager$Request;)J' : ['android.permission.INTERNET'],
	'Lcom/android/noisefield/NoiseFieldWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/EasCertificateRequestor;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/provider/EmailProvider;-uiUpdateMessage-(Landroid/net/Uri; Landroid/content/ContentValues;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/gadget/LocalPhotoSource;-getImage-(I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/printspooler/PrintSpoolerService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/tts/compat/CompatTtsService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-callListenerOnPressAndCheckKeyboardLayoutChange-(Lcom/android/inputmethod/keyboard/Key; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/photos/views/BlockingGLTextureView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowAnimator;-animateLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/musicvis/vis2/Visualization2;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/Details;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/event/AttendeesView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/recurrencepicker/RecurrencePickerDialog$OnRecurrenceSetListener;-onRecurrenceSet-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/ToUpperCase;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/colorpicker/ColorPickerSwatch;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub;-getLinkProperties-(I)Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/TouchExplorer;-clear-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/EasMeetingResponder;-requestSyncForMailbox-(Landroid/accounts/Account; Ljava/lang/String; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ActionableToastBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService;-startPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/SecureAlbum;-onStitchingQueued-(Landroid/net/Uri;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/WebVttRenderingWidget$RegionLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/photos/views/GalleryThumbnailView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/IActivityManager;-bindBackupAgent-(Landroid/content/pm/ApplicationInfo; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/IccProvider;-deleteIccRecordFromEf-(I Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/packageinstaller/InstallAppProgress;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/research/ResearchLogger;-sendFeedback-(Ljava/lang/String; Z Z Z)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/hid/HidService;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController$FullLockHeldState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/widget/ImageButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/providers/DrawerClosedObserver;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-shutdown-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/deskclock/DeskClock$TabsAdapter;-registerPageChangedListener-(Lcom/android/deskclock/DeskClockFragment;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-onPressKey-(I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/VolumePanel;-onMuteChanged-(I I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/browser/NavTabScroller$2;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/research/FeedbackActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-onContentDirty-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/crop/CropView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-setOffscreenPageLimit-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/IActivityManager;-moveTaskToBack-(I)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/internal/widget/WaveView;-waveUpdateFrame-(F F Z)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertService;-updateAlertNotification-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/CalendarContract$Reminders;-query-(Landroid/content/ContentResolver; J [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchToPrevImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/AuthenticatorService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/updates/SmsShortCodesInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/Invert;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManager;-setPackageScreenCompatMode-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/accounts/IAccountManager;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/text/method/TextKeyListener;-updatePrefs-(Landroid/content/ContentResolver;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Browser;-clearSearches-(Landroid/content/ContentResolver;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/galaxy/GalaxyWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/google/common/io/ByteStreams;-getDigest-(Lcom/google/common/io/InputSupplier; Ljava/security/MessageDigest;)[B' : ['android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-discoverServices-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/printservice/PrintService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/TabActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/gadget/WidgetClickHandler;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/browser/NavTabView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/SearchDialog$SearchBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSearch;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCallForwardingOption-(I I Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MoviePlayer;-playVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-setLowNotificationState-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/widget/SlidingPaneLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/vcard/VCardParserImpl_V21;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/ServiceWatcher$2;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/dreams/phototable/StockSource;-fillQueue-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/SnailAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AnimatedAdapter;-destroy-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumPage;-clearLoadingBit-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/photos/GalleryActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/Pop3Service;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/app/IBatteryStats$Stub$Proxy;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/view/ViewStub;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/DownloadVoiceData;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/ui/DateTileView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IntentService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/SyncActivityTooManyDeletes;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothInputDevice;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityStackSupervisor;-scheduleIdleLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsImpl;-init-(Landroid/content/Context; Landroid/preference/PreferenceScreen;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/NotificationActionIntentService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-unRegisterPhoneServiceStateListener-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getRemoteAlias-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/sip/SipAudioCall$1;-onCallEstablished-(Landroid/net/sip/SipSession; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/support/v4/app/FragmentActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/google/AbstractGoogleSource;-getSuggestionsExternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-setRequest-(Lcom/android/internal/location/ProviderRequest; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/timezonepicker/TimeZonePickerView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasSyncService;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ImageButton;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/AddBookmarkPage;-onTop-(Lcom/android/browser/BreadCrumbView; I Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/AccountManager$3;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/selectcalendars/CalendarColorSquare;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumPicker;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-disable-(Z)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/opp/TestActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-startObservingNativeCrashes-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-confirmCredentials-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/filterpacks/base/FrameBranch;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/view/ContextThemeWrapper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BreadCrumbView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ProgressBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/connectivity/Vpn;-setRadio-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/calendar/event/EditEventFragment;-access$000-(Lcom/android/calendar/event/EditEventFragment; I)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/alerts/QuickResponseActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DockObserver;-systemReady-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-unregisterClient-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/activity/setup/AccountSetupBasics;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/BaseSuggestionView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/InputPort;-transfer-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/OutputStreamTarget;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/ContactsContract$Data;-getContactLookupUri-(Landroid/content/ContentResolver; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-setCallWaiting-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager;-done-(Lcom/android/emailsync/AbstractSyncService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/VibratorService;-startVibrationLocked-(Lcom/android/server/VibratorService$Vibration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/media/IAudioService;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarSettingsActivity$1;-run-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageCurves;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Ljava/net/NetworkInterface;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-clearAccessibilityFocus-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-sendResponse-(I Ljava/lang/String; I I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/NotificationController;-refreshNotificationsForAccount-(Landroid/content/Context; J)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/browse/ConversationCursor$RefreshTask;-onPostExecute-(Lcom/android/mail/browse/ConversationCursor$UnderlyingCursorWrapper;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/QuickContactBadge;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/Development;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/AbstractSyncAdapterService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageGrad;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/DownloadVoiceData;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/AlertReceiver;-makeExpandingNotification-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J J J I Z I)Lcom/android/calendar/alerts/AlertService$NotificationWrapper;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/BrowserBackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationViewFragment;-measureOverlayHeight-(I)I' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-onLayout-(Z I I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/app/ChooserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/BrowserBackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager;-requestRouteToHost-(I I Ljava/lang/String;)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-acquireLaunchWakelock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-access$300-(Lcom/android/calendar/event/EditEventView; Landroid/widget/TextView; J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-registerListener-(Ljava/lang/String; Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/OnePaneController;-updateConversationListFragment-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/sip/SipService;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/exchange/service/AbstractSyncAdapterService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/provision/DefaultActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/DetailsHelper$DetailsSource;-getDetails-()Lcom/android/gallery3d/data/MediaDetails;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment$5;-run-()V' : ['android.permission.INTERNET'],
	'Lcom/android/server/input/InputManagerService;-getPointerSpeedSetting-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/LegacyEmailAuthenticatorService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/selectcalendars/SelectCalendarsSyncAdapter$2;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/datetimepicker/time/CircleView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/DialogPicker;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/PowerManagerService;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/SyncRunner;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/sip/ISipService;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-closeActiveSyncContext-(Lcom/android/server/content/SyncManager$ActiveSyncContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/accounts/IAccountManager$Stub;-addAccount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/packageinstaller/InstallFlowAnalytics;-isUserConsentToVerifyAppsGranted-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/support/v4/view/PagerTabStrip;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/TrackBrowserActivity;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/DisableCarModeActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub;-getRemoteName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetooth;-getRemoteAlias-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageHeaderView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationSyncDisabledTipView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/provider/BrowserProvider2;-updateInTransaction-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String; Z)I' : ['android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/exchange/service/EasService$1;-validate-(Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-resumeGeofence-(I I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Ljava/net/DatagramSocket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Landroid/net/sip/SipManager;-takeAudioCall-(Landroid/content/Intent; Landroid/net/sip/SipAudioCall$Listener;)Landroid/net/sip/SipAudioCall;' : ['android.permission.USE_SIP'],
	'Lcom/android/browser/Bookmarks;-addBookmark-(Landroid/content/Context; Z Ljava/lang/String; Ljava/lang/String; Landroid/graphics/Bitmap; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-syncClirSetting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-getHealthDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/vcard/VCardEntry$AnniversaryData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/deskclock/ScreensaverSettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/packageinstaller/GrantActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/dictionarypack/ActionBatch$Action;-execute-(Landroid/content/Context;)V' : ['android.permission.INTERNET'],
	'Lcom/android/exchange/ExchangeService$1;-autoDiscover-(Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-createSocketChannel-(I Ljava/lang/String; Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/DataController$DataControllerHandler;-doSaveThumbnail-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/alarms/AlarmService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStoppingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/datetimepicker/date/YearPickerView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService$8;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-markConversationsRead-(Ljava/util/Collection; Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-access$1200-(Lcom/android/server/ConnectivityService; Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/sip/SipAudioCall$1;-onCallEnded-(Landroid/net/sip/SipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/LatinIME$UIHandler;-onFinishInputView-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/CalendarView;-setMonthDisplayed-(Ljava/util/Calendar;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onRegistrationDone-(Landroid/net/sip/ISipSession; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DeviceStorageMonitorService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/development/Development;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-switchToGrid-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/MediaScannerActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/alerts/QuickResponseActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AnimatedListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/AsyncService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkStatsService;-access$600-(Lcom/android/server/net/NetworkStatsService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/RIL;-getLastDataCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IVibratorService$Stub$Proxy;-vibrate-(I Ljava/lang/String; J Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/activity/EventViewer;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/MediaItem;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/activity/setup/AccountSettings;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/printspooler/PrintJobConfigActivity$CustomEditText;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageDraw;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/NoSearchActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/SwipeableConversationItemView;-toggleSelectedStateOrBeginDrag-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorSVRectView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/colorpicker/ColorPickerPalette;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-completeConnection-(Lcom/android/internal/telephony/dataconnection/ApnContext;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/GalleryAppImpl;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/ProfileService;-cleanup-()Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/IConsumerIrService$Stub;-transmit-(Ljava/lang/String; I [I)V' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/photos/AlbumActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/TwoPaneController;-requestUpdate-(Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/ActivityManager;-moveTaskToFront-(I I)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/content/IContentService$Stub$Proxy;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/BackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/InputStreamSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccCard;-changeIccLockPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/DeskClock;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationContainer;-onNotifierScroll-(I I)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/development/LogTextBox;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-handleCpbsCommand-(Ljava/lang/String; I)V' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityStackSupervisor;-findTaskToMoveToFrontLocked-(I I Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiService;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings; Landroid/os/IBinder; Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothInputDevice;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/MoviePlayer$AudioBecomingNoisyReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-stopDragMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/TextProgressBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/IntentHandler;-onNewIntent-(Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/development/AccountsTester;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/LatinIME;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/EmailServiceStub;-updateFolderList-(J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-access$000-(Lcom/android/internal/telephony/cdma/CdmaConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService$3;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/LocationBlacklist;-reloadBlacklist-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$DriverStoppingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/printspooler/SelectPrinterActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/SipAudioCall;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnablingActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/pacprocessor/PacService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/AlbumSlidingWindow;-setContentWindow-(I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/AppWidgetService;-systemRunning-(Z)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/BookmarkUtils;-getBookmarksUri-(Landroid/content/Context;)Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/browse/MessageAttachmentBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCallWaiting-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller$9;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-destroyRenderer-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/BackupManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/ErrorConsoleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-open-([J I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DayView$3;-run-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/am/UsageStatsService$2;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-updateNotificationsLocked-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/email2/ui/MailActivityEmail;-handleDrop-(Landroid/view/DragEvent; Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/packageinstaller/UninstallAppProgress;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/Development;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothAdapter;-getBondedDevices-()Ljava/util/Set;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-startScan-(I Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/AlbumActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/utils/NotificationUtils;-cancelAndResendNotifications-(Landroid/content/Context;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/dreams/phototable/BummerView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/MonthListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintJobConfigActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/dreams/phototable/PhotoSource;-fillQueue-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/MailLogService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/SubtitleTrack;-show-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/vcard/VCardEntry$AndroidCustomData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/filtershow/category/CategoryTrack;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationLongPressTipView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/NotificationActionIntentService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/RenamingDelegatingContext;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/ISyncAdapter$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/music/QueryBrowserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/OnePaneController;-access$1600-(Lcom/android/mail/ui/AbstractActivityController;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasProvision;-generateRequestEntitySerializer-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I D)Lcom/android/exchange/adapter/Serializer;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/DayView;-access$4500-(Lcom/android/calendar/DayView; Landroid/view/MotionEvent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/datetimepicker/date/DatePickerDialog;-onResume-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/content/IContentService$Stub$Proxy;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/calendar/event/EditEventFragment$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/development/RunningProcesses;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/OnePaneController;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-startLegacyVpn-(Lcom/android/internal/net/VpnProfile;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/EasOutboxService;-sendMessage-(Ljava/io/File; J)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothGattServer;-registerCallback-(Landroid/bluetooth/BluetoothGattServerCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/google/common/io/ByteStreams$5;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/exchange/eas/EasSync;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/MailActionBarView;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/base/InputStreamSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/RIL;-startDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/KeyboardView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-setAppFullscreen-(Landroid/os/IBinder; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/recurrencepicker/RecurrencePickerDialog;-updateDialog-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/ComposeAttachmentTile;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/providers/SuggestionsProvider$ContactsCursor;-query-(Ljava/lang/String;)Lcom/android/mail/providers/SuggestionsProvider$ContactsCursor;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-dial-(Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipCall;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DateTimeView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-loadDictionaryAsync-()V' : ['android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/nsd/INsdManager$Stub$Proxy;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/ui/PhotoView;-resetToFirstPicture-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/AttachmentDownloadService;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/mail/browse/ConversationItemView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendRetrySms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/updates/SmsShortCodesInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/OffHostApduService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/ILocationManager;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onCodeInput-(I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub;-requestRouteToHostAddress-(I [B Ljava/lang/String;)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/inputmethod/pinyin/CandidateView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-setStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/base/GLTextureSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller$Download;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processDialCall-(Ljava/lang/String;)V' : ['android.permission.READ_CALL_LOG'],
	'Landroid/widget/ProgressBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/certinstaller/CertInstallerMain;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/ComposeActivityEmail;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-supplyPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncActivityTooManyDeletes;-startSyncUndoDeletes-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/music/MusicUtils;-playAll-(Landroid/content/Context; Landroid/database/Cursor; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/documentsui/SettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Pending;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/videoeditor/ProjectsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/category/IconView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService$4;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/videoeditor/ProjectsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/AdapterViewAnimator;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/CertBlacklister$BlacklistObserver;-writeBlacklist-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BluetoothManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/location/GpsLocationProvider;-sendMessage-(I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EasService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/WakeLockStateMachine;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/chips/RecipientEditTextView$RecipientReplacementTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.READ_CONTACTS'],
	'Landroid/media/IAudioService$Stub;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccProvider;-addIccRecordToEf-(I Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-setMessageStatusDeleted-(J Lcom/android/bluetooth/map/BluetoothMapUtils$TYPE; I)Z' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-addNetworkPolicyLocked-(Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyLocationChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardEntryHandler;-onEntryCreated-(Lcom/android/vcard/VCardEntry;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/ShortcutActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IBackupAgent;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/MusicPicker;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/BlendFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onPollPdp-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-doUnbindHelperService-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/wifi/WifiController$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/agenda/AgendaFragment;-search-(Ljava/lang/String; Landroid/text/format/Time;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/os/PowerManager$WakeLock;-acquireLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/research/ReplayerService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/svox/pico/EngineSettings;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/deskclock/ScreensaverSettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/pacprocessor/PacService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-pullVcardListing-([B Lcom/android/bluetooth/pbap/BluetoothPbapObexServer$AppParamValue; Ljavax/obex/HeaderSet; Ljavax/obex/Operation;)I' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/app/DialogPicker;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-setShiftLocked-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/inputmethodservice/ExtractButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TimePicker;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment;-access$800-(Lcom/android/email/activity/setup/AccountSettingsFragment;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/providers/downloads/ui/DownloadItem;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavScreen$TabAdapter$1;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/videosrc/MediaSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/widget/CalendarAppWidgetProvider;-performUpdate-(Landroid/content/Context; Landroid/appwidget/AppWidgetManager; [I [J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/server/am/ActivityStack;-clearLaunchTime-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-setPostDialState-(Lcom/android/internal/telephony/Connection$PostDialState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasAccountService;-validate-(Ljava/lang/Class; Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Landroid/view/IWindowManager;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/view/menu/ExpandedMenuView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/SlidingDrawer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/CheckVoiceData;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationListFragment;-viewConversation-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/BluetoothAdapter;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/alerts/AlertActivity$3;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/photos/GalleryActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/net/ConnectivityManagerCompatJellyBean;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getOwnerPhoneNumberVcardFromProfile-(Z [B)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCLIR-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView$EmojiPalettesAdapter;-setPrimaryItem-(Landroid/view/ViewGroup; I Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/ui/UndoBarView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/cardemulation/CardEmulation;-getSelectionModeForCategory-(Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/opp/BluetoothOppIncomingFileConfirmActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/IntRangeManager;-tryAddRanges-(I I Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AddBookmarkPage;-onItemSelected-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/emailcommon/utility/CertificateRequestor;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/soundrecorder/Recorder;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/ui/ClusteredSuggestionsView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/NetworkPolicyManager;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/AccountItemView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-onBackPressed-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine;-access$17900-(Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IUiAutomationConnection$Stub;-connect-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$IdleState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendNewSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/SmsHeader; I Landroid/app/PendingIntent; Landroid/app/PendingIntent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ListActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/videoeditor/VideoEditorBaseActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager;-<init>-(Landroid/content/Context; Z)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/updates/SELinuxPolicyInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/GalleryActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/setup/AccountSecurity;-tryAdvanceSecurity-(Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumPage$1;-getItemIndex-(Lcom/android/gallery3d/data/Path;)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-init-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/NotificationManagerService$5;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Ljava/net/HttpURLConnection;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/ui/AlbumSlidingWindow;-prepareSlotContent-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService;-access$2000-(Landroid/media/AudioService;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/ConnectivityService;-getTetherableIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/deskclock/DeskClock;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/UsageStatsService$2;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MailWebView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/ViewPager$PagerObserver;-onInvalidated-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/datetimepicker/date/TextViewWithCircularIndicator;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onDataStallAlarm-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/BrowserYesNoPreference;-onDialogClosed-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/ComboAlbum;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/OnePaneController;-changeFolder-(Lcom/android/mail/providers/Folder; Ljava/lang/String; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-showDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/widget/sgv/StaggeredGridView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/PrintManager;-print-(Ljava/lang/String; Landroid/print/PrintDocumentAdapter; Landroid/print/PrintAttributes;)Landroid/print/PrintJob;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/UrlHandler;-shouldOverrideUrlLoading-(Lcom/android/browser/Tab; Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/UrlInputView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/CalendarContract$Attendees;-query-(Landroid/content/ContentResolver; J [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/am/BatteryStatsService;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-setUsbTethering-(Z)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/music/IMediaPlaybackService;-removeTracks-(I I)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/AlertActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/DisableCarModeActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-switchToNextInputMethod-(Landroid/os/IBinder; Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AlarmUtils;-getFormattedTime-(Landroid/content/Context; Ljava/util/Calendar;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionarySettings;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_USER_DICTIONARY'],
	'Landroid/filterpacks/text/ToUpperCase;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothHealth;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/google/common/io/Files;-readBytes-(Ljava/io/File; Lcom/google/common/io/ByteProcessor;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/latin/settings/DebugSettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-wakeUpInternal-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/colorpicker/ColorPickerPalette;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-okToConnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/DeviceStorageMonitorService;-cancelNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/documentsui/DocumentsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothAdapter;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Tab;-deleteThumbnail-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/exchange/ExchangeService$2;-onAccountChanged-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-access$000-(Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-notifyApnIdUpToCurrent-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/AbstractSyncAdapterService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/SmsManager;-updateMessageOnIcc-(I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-access$3300-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/print/UserState;-<init>-(Landroid/content/Context; I Ljava/lang/Object;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/SettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/menu/ActionMenuView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-handleDisconnect-(Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/widget/CalendarAppWidgetProvider;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/mail/ui/AbstractActivityController$12;-onActionClicked-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-fetchSimRecords-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller$7;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/Connection;-proceedAfterWaitChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/photo/util/ImageUtils;-getImageBounds-(Lcom/android/ex/photo/util/ImageUtils$InputStreamFactory;)Landroid/graphics/Point;' : ['android.permission.INTERNET'],
	'Landroid/provider/CalendarContract$EventsEntity$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Landroid/media/AudioService;-access$5700-(Landroid/media/AudioService;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchWithCaptureAnimation-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/ConnectivityManager;-setRadio-(I Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/textservice/TextServicesManager;-getCurrentSpellCheckerSubtype-(Z)Landroid/view/textservice/SpellCheckerSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/SyncAdapterDriver;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calculator2/CalculatorViewPager;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setRemoteAlias-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/gadget/WidgetConfigure;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarSettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/LockPatternUtils;-isLockPatternEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage;-switchWithCaptureAnimation-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-addDeletion-(Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accounts/AccountManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/providers/downloads/ui/DownloadList;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/IPrintManager$Stub;-restartPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-enqueue-([J I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/TimeBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Settings$Secure;-getUriFor-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/ui/AlbumSlotRenderer;-onVisibleRangeChanged-(I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/service/EasService$1;-loadAttachment-(Lcom/android/emailcommon/service/IEmailServiceCallback; J J Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/media/SubtitleController;-access$300-(Landroid/media/SubtitleController;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavigationBarBase;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/TwoStateImageView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiController$ApEnabledState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/event/AttendeesView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSecurity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/effect/effects/PosterizeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/eas/EasSettings;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/musicvis/vis3/Visualization3;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/UiModeManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/fall/Fall;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/ChooseAccountActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/InLineSettingItem;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/LogViewer;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager;-setRadios-(Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/calendar/CalendarController;-sendEventRelatedEventWithExtra-(Ljava/lang/Object; J J J J I I J J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/test/IsolatedContext;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/support/v4/view/PagerTabStrip;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MultiAdapterSpinner;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/gsm/SmsManager;-sendDataMessage-(Ljava/lang/String; Ljava/lang/String; S [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-access$300-(Lcom/android/deskclock/widget/multiwaveview/GlowPadView; I F F)V' : ['android.permission.VIBRATE'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-setCurrentCategoryId-(I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/WebVttRenderingWidget;-onLayout-(Z I I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/MountService$4;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/search/SearchManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/latin/settings/DebugSettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-enableAllIMEsIfThereIsNoEnabledIME-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-deleteSmsOnRuim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSecurity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/Connectivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-removeBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/latin/AudioAndHapticFeedbackManager;-performHapticFeedback-(Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/ISms;-sendData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/google/common/io/Resources;-copy-(Ljava/net/URL; Ljava/io/OutputStream;)V' : ['android.permission.INTERNET'],
	'Landroid/widget/QuickContactBadge;-assignContactFromPhone-(Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-deleteSmsOnRuim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/setup/SetupActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/AccountService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/app/CameraApp;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onUndoDeleteImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AbstractActivityController;-updateConversationListFragment-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/Connectivity;-access$3200-(Lcom/android/development/Connectivity;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/exchange/CalendarSyncEnabler;-enableEasCalendarSyncInternalForTest-()Ljava/lang/String;' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/SearchView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-markConversationsRead-(Ljava/util/Collection; Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/emailcommon/utility/EmailAsyncTask;-callDoInBackgroundForTest-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/filtershow/category/CategoryView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/Connectivity;-access$2400-(Lcom/android/development/Connectivity;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/SearchActivity;-handleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/Controller;-showSslCertificateOnError-(Landroid/webkit/WebView; Landroid/webkit/SslErrorHandler; Landroid/net/http/SslError;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/XLargeUi;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onError-(Landroid/net/sip/SipAudioCall; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/VideoUtils;-trimUsingMp4Parser-(Ljava/io/File; Ljava/io/File; I I)V' : ['android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/eas/EasOutboxSync;-handleProvisionError-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/AnimatedAdapter;-<init>-(Landroid/content/Context; Lcom/android/mail/browse/ConversationCursor; Lcom/android/mail/ui/ConversationSelectionSet; Lcom/android/mail/ui/ControllableActivity; Lcom/android/mail/ui/AnimatedAdapter$ConversationListListener; Lcom/android/mail/ui/SwipeableListView; Ljava/util/List;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/soundrecorder/Recorder$OnStateChangedListener;-onStateChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-getMonthAndYearString-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/AbstractSettingPopup;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/StackView$StackFrame;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/SimpleWeekView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine$VerifyingLinkState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/AdapterView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ActionBarView$HomeView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ZoomButtonsController$Container;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hid/HidService;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/accessibility/CaptioningManager;-access$500-(Landroid/view/accessibility/CaptioningManager;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-getTetherableBluetoothRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/BlendFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractActivityController;-access$1100-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/ExpandableListView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterfw/core/FieldPort;-setFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-access$2800-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/content/ObjectCursor;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-onUserStart-(I)V' : ['android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/ConversationUpdater;-getBatchAction-(I)Lcom/android/mail/ui/DestructiveAction;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/InstrumentationList;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/MediaObject;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/IAudioService$Stub;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/camera/ui/RenderOverlay;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Activity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-setGsmBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/inputmethod/latin/personalization/PersonalizationPredictionDictionary;-writeBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/ingest/ui/MtpImageView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$1500-(Lcom/android/gallery3d/ui/PhotoView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onEnableApn-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/AttachmentInfo;-<init>-(Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/coremedia/iso/PropertyBoxParserImpl;-<init>-([Ljava/lang/String;)V' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/base/FrameFetch;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/IConnectivityManager$Stub;-updateLockdownVpn-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/view/menu/ListMenuItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-stopEmojiPalettes-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasMoveItems;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/ex/photo/util/ImageUtils;-createLocalBitmap-(Landroid/content/ContentResolver; Landroid/net/Uri; I)Lcom/android/ex/photo/loaders/PhotoBitmapLoaderInterface$BitmapResult;' : ['android.permission.INTERNET'],
	'Landroid/widget/ExpandableListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractActivityController;-doMarkConversationMessagesUnread-(Lcom/android/mail/providers/Conversation; Ljava/util/Set; [B)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ActivityController;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActiveServices$ServiceMap;-ensureNotStartingBackground-(Lcom/android/server/am/ServiceRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/AlbumPage;-pickPhoto-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MediaPlaybackActivity$7;-onClick-(Landroid/view/View;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/photos/views/BlockingGLTextureView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ProxyLauncher;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/SettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/time/RadialSelectorView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-finishLoadAttachment-(Lcom/android/emailcommon/provider/EmailContent$Attachment; Ljava/io/File;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/latin/settings/DebugSettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/InboundSmsHandler$WaitingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/MediaRouteButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/text/format/DateUtils;-getRelativeTimeSpanString-(J J J)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState$SwitchActions;-setEmojiKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/quicksearchbox/ui/ClusteredSuggestionsView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupCreatedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/GoogleAccountLogin;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/bluetooth/IBluetooth$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/pan/PanService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/setup/SetupActivity;-isThisImeCurrent-(Landroid/content/Context; Landroid/view/inputmethod/InputMethodManager;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/EventViewer;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BookmarkSearch;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/wallpaper/nexus/NexusWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/SystemVibrator;-vibrate-(I Ljava/lang/String; [J I)V' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/IBluetoothGatt;-writeDescriptor-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I I [B)V' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/AdapterViewFlipper;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ShortcutNameActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/text/format/DateUtils;-getRelativeTimeSpanString-(Landroid/content/Context; J)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/MediaSetSource$CheckedMediaSetSource;-getImage-(I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-getNetworkSelectionMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-onContentDirty-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/InstrumentationList;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/search/SearchManagerService$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/net/CaptivePortalTracker$DelayedCaptiveCheckState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/vcard/VCardEntry$InsertOperationConstrutor;-onElement-(Lcom/android/vcard/VCardEntry$EntryElement;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/content/SyncManager;-scheduleSync-(Landroid/accounts/Account; I I Ljava/lang/String; Landroid/os/Bundle; J J Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/content/ContentService;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/deskclock/TimerRingService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/provider/DBHelper;-createAccountManagerAccount-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/service/notification/NotificationListenerService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/adapter/ItemOperationsParser;-parseProperties-()V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/location/LocationManager;-removeAllGeofences-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/TestTcpServer;-run-()V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/TrimVideo;-onSeekEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment;-setExtraUserVisibleHint-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/EventInfoFragment;-saveEventColor-()Z' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/mail/compose/EnterSubject;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageShow;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/ViewPager;-onRestoreInstanceState-(Landroid/os/Parcelable;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.DISABLE_KEYGUARD', 'android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/Filter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothPan;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/AddBookmarkPage$CustomListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/exchange/ExchangeService$2;-access$100-(Lcom/android/emailsync/SyncManager$AccountObserver;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/PhotoPage$MyMenuVisibilityListener;-onMenuVisibilityChanged-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/TrimVideo;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-fixTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/view/IInputMethodManager;-getShortcutInputMethodsAndSubtypes-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-addDescriptor-(I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-revertToNitzTime-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-access$600-(Landroid/media/MediaPlayer;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/PhotoPage;-onStateResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/CheckBox;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/Development;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-doResumeUpdates-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/sip/SipPhoneFactory;-makePhone-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)Lcom/android/internal/telephony/sip/SipPhone;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mail/ui/SecureConversationViewFragment;-onUserVisibleHintChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CallerInfo;-getCallerId-(Landroid/content/Context; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment;-setRingtone-(Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSyncBase;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/AddBookmarkPage$EditBookmarkInfoLoader;-loadInBackground-()Lcom/android/browser/AddBookmarkPage$EditBookmarkInfo;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/IBluetoothMap;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/CameraActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CellBroadcastRangeManager;-finishUpdate-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/noisefield/NoiseFieldWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-access$500-(Lcom/android/bluetooth/hfp/HeadsetService; Landroid/bluetooth/BluetoothDevice; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/RepeatingImageButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/AbstractActivityController$17;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/TelephonyRegistry;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/CameraModule;-onResumeAfterSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/LatinIME;-access$600-(Lcom/android/inputmethod/latin/LatinIME; Z I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterfw/core/FieldPort;-pushFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/TouchExplorer$ExitGestureDetectionModeDelayed;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/certinstaller/CertInstallerMain;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Switch;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService;-removeTrack-(J)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/SystemBackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/inputmethod/InputMethodUtils;-setNonSelectedSystemImesDisabledUntilUsed-(Landroid/content/pm/PackageManager; Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/galaxy/GalaxyWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/SubtypeSwitcher;-updateShortcutIME-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/photos/shims/LoaderCompatShim;-urisForSubItems-(Ljava/lang/Object;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calculator2/CalculatorEditText;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BluetoothManagerService;-access$1900-(Lcom/android/server/BluetoothManagerService;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/service/dreams/Sandman;-isScreenSaverActivatedOnDock-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EasAuthenticatorService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-disableGsmBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/camera/LocationManager;-stopReceivingLocationUpdates-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/Gallery;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventView;-access$900-(Lcom/android/calendar/event/EditEventView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/RotateEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-isLocationProviderEnabled-(Landroid/content/ContentResolver; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ButteryProgressBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ActivityChooserView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-createSocketChannel-(I Ljava/lang/String; Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/LockSettingsService;-migrateOldData-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/photo/util/ImageUtils$BaseInputStreamFactory;-createInputStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/browser/Controller;-<init>-(Landroid/app/Activity;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/media/WebVttRenderingWidget;-setVisible-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-clearServices-(I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiStateMachine$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/music/PlaylistBrowserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onRadioOffOrNotAvailable-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-updateRotationUncheckedLocked-(Z)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v7/widget/Space;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService;-vibratePattern-(I Ljava/lang/String; [J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-getSpnFsm-(Z Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/SupplicantStateTracker$HandshakeState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService$Session;-run-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothGatt;-readCharacteristic-(Landroid/bluetooth/BluetoothGattCharacteristic;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/videoeditor/VideoEditorActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/datetimepicker/date/DatePickerDialog;-onClick-(Landroid/view/View;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/VideoView;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ShortcutActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/vcard/VCardEntry$WebsiteData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/TelephonyRegistry;-broadcastDataConnectionStateChanged-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/LinkCapabilities; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/fall/FallRS;-start-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-slideOutFromOldKey-(Lcom/android/inputmethod/keyboard/Key; I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-registerForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PhotoPage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothPan$Stub$Proxy;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/PlaylistBrowserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView$KeyTimerHandler;-startUpdateBatchInputTimer-(Lcom/android/inputmethod/keyboard/PointerTracker;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneFactory;-makeDefaultPhone-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/music/WeekSelector;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CdmaBroadcastRangeManager;-tryAddRanges-(I I Z)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/widget/ContentLoadingProgressBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/SearchActivityView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/PhotoView$Picture;-draw-(Lcom/android/gallery3d/glrenderer/GLCanvas; Landroid/graphics/Rect;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/TabBar$TabView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-access$100-(Lcom/android/internal/telephony/RIL;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onReleaseKey-(I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/photo/PhotoViewPager;-initialize-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-deleteAccountPIMData-(Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/Wallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView;-onUpEvent-(I I I J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-access$900-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/ui/IngestGridView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/galaxy/Galaxy;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/LatinIME;-handleBackspace-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/TransitionsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/GalleryActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/IBackupAgent$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/exchange/EasOutboxService;-sync-(Lcom/android/exchange/adapter/AbstractSyncAdapter;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/soundrecorder/SoundRecorder;-updateTimeRemaining-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager;-sendRequest-(J Lcom/android/emailsync/Request;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/telephony/MmsSmsProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/inputmethodservice/ExtractEditText;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/EventInfoFragment;-access$3000-(Lcom/android/calendar/EventInfoFragment; Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/accessibilityservice/AccessibilityService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-readRemoteRssi-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-doStatusCallback-(Lcom/android/emailcommon/service/IEmailServiceCallback; J J I I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/LocalSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/VibratorService;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/widget/ScrollView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BaseUi;-onBackKey-()Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/development/BadBehaviorActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothHeadset;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManager$2;-doWork-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/personalization/PersonalizationDictionary;-access$700-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/IConnectivityManager$Stub;-getTetheredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/AsyncQueryServiceHelper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/MediaSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/app/FragmentActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/LegacyEasAuthenticatorService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothDevice;-setAlias-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-requestSync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/hardware/location/IGeofenceHardware;-pauseGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/browse/MessageFooterView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Ljava/lang/String; J F Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/EmailApplication;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService;-getEnabledInputMethodList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/Invert;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/DownloadOverMeteredDialog;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/NetworkStateHandler;-onResume-()V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/CellBroadcastHandler;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-doUpdateVisitedHistory-(Lcom/android/browser/Tab; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/AudioService;-startBluetoothSco-(Landroid/os/IBinder; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/deskclock/AlarmClockFragment$2;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cat/CatService;-sendTerminalResponse-(Lcom/android/internal/telephony/cat/CommandDetails; Lcom/android/internal/telephony/cat/ResultCode; Z I Lcom/android/internal/telephony/cat/ResponseData;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DatePicker;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/SettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PickerActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/ImapAuthenticatorService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/MessageHeaderView;-ensureExpandedDetailsView-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NotificationManagerService;-systemReady-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onSetUserDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/EmlViewerActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/CalendarView$2;-onScroll-(Landroid/widget/AbsListView; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/grass/GrassView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/timer/TimerListItem;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/ComboViewActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/accessibility/ScreenMagnifier$ScreenStateObserver;-access$2900-(Lcom/android/server/accessibility/ScreenMagnifier$ScreenStateObserver; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/UploaderService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/util/AsyncService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/UnderdevelopedSettings;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-updateIccAvailability-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/LegacyEasAuthenticatorService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-handleCallSetupRequestFromSim-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker$3;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/wallpaper/galaxy/Galaxy;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/print/PrintManagerService;-getPrintJobInfos-(I I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiService;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/camera/ui/RotatableLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/PolicyService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Settings$Secure;-getFloatForUser-(Landroid/content/ContentResolver; Ljava/lang/String; F I)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-render-(Lcom/android/gallery3d/glrenderer/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/preference/PreferenceFrameLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaScanner;-scanSingleFile-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageHeaderView$1;-onChanged-()V' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterfw/core/Filter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService$1;-run-()V' : ['android.permission.INTERNET'],
	'Lcom/android/noisefield/NoiseFieldView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/gesture/GestureOverlayView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment;-loadSettings-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageRedEye;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RadioGroup;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/PrintJob;-isCompleted-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ViewMode$ModeChangeListener;-onViewModeChanged-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-setState-(I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/documentsui/DocumentsApplication;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/Tab$3;-onCreateWindow-(Landroid/webkit/WebView; Z Z Landroid/os/Message;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/wm/WindowManagerService;-prepareDragSurface-(Landroid/view/IWindow; Landroid/view/SurfaceSession; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeechService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/AlarmManagerService$AlarmThread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AnalogClock;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/widget/TextTime;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-onResume-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/AllInOneActivity;-onResume-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/view/IInputMethodManager;-getCurrentInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment$3;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ShutdownActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/AccountService$1$1;-run-()V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/AutoCompleteTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/ScreensaverActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISms;-enableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-enableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/speech/tts/TextToSpeech;-getDefaultLanguage-()Ljava/util/Locale;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/PieMenuButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/photos/views/HeaderGridView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/personalization/AccountUtils;-getDeviceAccountsEmailAddresses-(Landroid/content/Context;)Ljava/util/List;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/svox/pico/EngineSettings;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-prepareAppTransition-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-getContactCount-()I' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/ConversationListView;-showToastIfSyncIsOff-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/Application;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/support/v4/print/PrintHelper;-printBitmap-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/UrlInputView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/settings/SettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine$ScanModeState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/svox/pico/DownloadVoiceData;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/view/BookmarkExpandableView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EmailServiceUtils;-moveCalendarData-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/LocationManagerService;-isAllowedByUserSettingsLocked-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/spellcheck/SpellCheckerSettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorCompareView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService;-openCurrentAndNext-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-handleMsgListChanges-()V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/LocationManagerService;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImagePoint;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/Files;-readFirstLine-(Ljava/io/File; Ljava/nio/charset/Charset;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/media/effect/effects/CropEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/SettingInjectorService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/EmlMessageViewFragment$MessageLoadCallbacks;-onLoadFinished-(Landroid/content/Loader; Lcom/android/mail/browse/ConversationMessage;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-deleteShare-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-readDescriptor-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/ex/photo/loaders/PhotoBitmapLoader;-onLoadInBackground-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/calendar/DayView$CalendarGestureListener;-onSingleTapUp-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/widget/SlidingDrawer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/alarmclock/DigitalAppWidgetService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onResetKeyboardStateToAlphabet-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/ConsumerIrService;-transmit-(Ljava/lang/String; I [I)V' : ['android.permission.TRANSMIT_IR'],
	'Landroid/widget/QuickContactBadge;-assignContactFromPhone-(Ljava/lang/String; Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-iccIO-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/inputmethod/dictionarypack/ActionBatch;-execute-(Landroid/content/Context; Lcom/android/inputmethod/dictionarypack/ProblemReporter;)V' : ['android.permission.INTERNET'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$InitialState;-turnOnUpstreamMobileConnection-(I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/widget/RadioButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/GetSampleText;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifyDataConnection-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-setQueuePosition-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/MailboxSelectionActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/adapter/AbstractSyncAdapter;-parse-(Ljava/io/InputStream;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/event/EditEventActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/AbstractSyncAdapterService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/category/CategoryView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/BatteryStatsImpl;-writeToParcel-(Landroid/os/Parcel; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/vcard/VCardParserImpl_V30;-parseItems-()V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$1;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/NsdService$NsdStateMachine;-registerForNsdSetting-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/accounts/AccountManagerService;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/mail/browse/ConversationItemView;-onDragEvent-(Landroid/view/DragEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/provider/Settings$Secure;-getInt-(Landroid/content/ContentResolver; Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AnimatedAdapter$3;-onChanged-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/TileImageView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase$DataRoamingSettingObserver;-register-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/AddBookmarkPage;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Browser;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/AbsSpinner;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/BadBehaviorActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/YearPickerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/IWifiManager;-isScanAlwaysAvailable-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/input/InputManagerService;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Landroid/widget/RemoteViewsService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/DialogTitle;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService;-access$400-(Lcom/android/music/MediaPlaybackService;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasAccountService;-sendHttpClientPost-(Ljava/lang/String; [B)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStackSupervisor;-activityIdleInternalLocked-(Landroid/os/IBinder; Z Landroid/content/res/Configuration;)Lcom/android/server/am/ActivityRecord;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/policy/impl/GlobalActions$SinglePressAction;-onPress-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/ViewGroup;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker$AccessibilityNodeProviderImpl;-sendAccessibilityEventForVirtualView-(I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-stopLocationBasedDetector-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/MailboxSelectionActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/AbstractActivityController;-onDataSetChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/speech/IRecognitionService$Stub$Proxy;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/media/MediaPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHealth;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/AsyncPlayer;-enqueueLocked-(Landroid/media/AsyncPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-play-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowSession$Stub;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/emailsync/SyncManager;-runAccountReconcilerSync-(Landroid/content/Context;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/documentsui/TestActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine$ConnectedState;-exit-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/SurfaceView;-updateWindow-(Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/view/ViewOverlay$OverlayViewGroup;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/certinstaller/CertInstaller;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/ConversationContainer;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/galaxy/GalaxyView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-onScrubbingStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/widget/WidgetService$MailFactory;-getViewAt-(I)Landroid/widget/RemoteViews;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hid/HidService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/StickyHeaderListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/SnoozeAlarmsService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/PackageManagerService;-deletePackageX-(Ljava/lang/String; I I)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/opengl/GLSurfaceView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/SyncActivityTooManyDeletes;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/TimerRingService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumPicker;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/OnePaneController;-loadAccountInbox-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accessibilityservice/AccessibilityService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/a2dp/Avrcp;-notifyVolumeChanged-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-requestDeferredUpdate-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onCallBusy-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/BatchService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/ChooseAccountActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/fall/FallWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActiveServices$ServiceRestarter;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/MusicUtils;-playAll-(Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$RestartPingsTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/vcard/VCardEntry$PhoneData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/email/activity/setup/AccountCheckSettingsFragment$AccountCheckTask;-doInBackground-([Ljava/lang/Void;)Lcom/android/emailcommon/mail/MessagingException;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-startInstallConfirm-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/onetimeinitializer/GalleryWidgetMigrator;-migrateGalleryWidgetsInternal-(Landroid/content/Context;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/IWindowManager$Stub;-startFreezingScreen-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/StringLogger;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onLongPress-(Landroid/inputmethodservice/Keyboard$Key;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/AllInOneActivity;-onTabSelected-(Landroid/app/ActionBar$Tab; Landroid/app/FragmentTransaction;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/datetimepicker/date/TextViewWithCircularIndicator;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/DialogTitle;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/DataConnectionStats;-startMonitoring-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/Editor$InsertionHandleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$ConnectivityReceiver$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/google/common/io/MultiInputStream;-<init>-(Ljava/util/Iterator;)V' : ['android.permission.INTERNET'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-resumeGeofence-(I I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/RecognitionManagerService$MyPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/KeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.VIBRATE'],
	'Lcom/google/common/io/CharStreams;-copy-(Lcom/google/common/io/InputSupplier; Ljava/lang/Appendable;)J' : ['android.permission.INTERNET'],
	'Landroid/speech/RecognitionService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/AbsoluteLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasAccountService;-tryProvision-(Lcom/android/exchange/EasSyncService;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/NestedFolderTeaserView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/ImapAuthenticatorService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/SystemBackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/PhotoPage;-access$3300-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/personalization/DecayingExpandableBinaryDictionaryBase;-writeBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/email/provider/EmailProvider;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setAndBroadcastNetworkSetTime-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService$7;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/PhotoPage;-showDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/DeleteEventHelper;-access$1200-(Lcom/android/calendar/DeleteEventHelper; I)V' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$FrequencyConflictState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneProxy;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasOptions;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/music/AlbumBrowserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/IntentService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/browser/WebViewController;-openTab-(Ljava/lang/String; Z Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/development/PointerLocation;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/view/SnapshotGridView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pan/PanService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/compose/ComposeAttachmentTile;-thumbnailLoadFailed-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService$14;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsImpl;-setSubtypeEnablerIcon-(Landroid/graphics/drawable/Drawable;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/DontPressWithParentLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-sendUssdResponse-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccCard;-supplyPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/PreviewSurfaceView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/UninstallAppProgress;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/StateManager;-finishState-(Lcom/android/gallery3d/app/ActivityState;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/videoeditor/widgets/ImageViewTouchBase;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/Context;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/MediaSetSource;-getImage-(I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/Button;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/fall/FallView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/UiController;-handleNewIntent-(Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/photos/AlbumActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/ILocationManager$Stub$Proxy;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/mail/ui/ConversationSpecialItemView;-getShouldDisplayInList-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/printspooler/PrintJobConfigActivity;-onResume-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/SenderInfoLoader;-loadInBackground-()Ljava/lang/Object;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/TrackBrowserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/StackView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/BluetoothManagerService;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchFocus-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/ImapService;-access$000-(Landroid/content/Context; J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchToNextImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/GLTextureTarget;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UiController;-setActiveTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/accounts/AccountManager$10;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/music/MusicUtils;-playAll-(Landroid/content/Context; [J I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/UserState;-enableSystemPrintServicesOnFirstBootLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-stopPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHeadset;-sendVendorSpecificResultCode-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MediaPlaybackService;-open-([J I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab$3;-onProgressChanged-(Landroid/webkit/WebView; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/ExpandedMenuView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu$2;-onPostExecute-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-acknowledgeLastIncomingGsmSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-handleSendComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageTinyPlanet;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-handleVoicemailTeleservice-(Lcom/android/internal/telephony/cdma/SmsMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/SyncRunner;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-access$300-(Lcom/android/bluetooth/btservice/BondStateMachine; Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/LeaveBehindItem;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/GalleryActivity;-startViewAction-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/CharStreams;-copy-(Lcom/google/common/io/InputSupplier; Lcom/google/common/io/OutputSupplier;)J' : ['android.permission.INTERNET'],
	'Lcom/android/music/IMediaPlaybackService;-play-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/media/AudioManager;-setParameter-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/browser/provider/BrowserProvider2$DatabaseHelper;-enableSync-(Landroid/database/sqlite/SQLiteDatabase;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-onDayTapped-(Landroid/text/format/Time;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-updateWakefulnessLocked-(I)Z' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/packageinstaller/GrantActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-doCluster-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/tts/ITextToSpeechService$Stub$Proxy;-getClientDefaultLanguage-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasProvision;-getRequestEntity-()Lorg/apache/http/HttpEntity;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/settings/GallerySettings;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-getDictionary-()[B' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/inputmethod/dictionarypack/WordListPreference;-access$800-(Lcom/android/inputmethod/dictionarypack/WordListPreference;)V' : ['android.permission.INTERNET'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetooth;-sendConnectionStateChange-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/ChooseAccountTypeActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-sendPendingBroadcastsLocked-(Lcom/android/server/am/ProcessRecord;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/gatt/GattService;-readCharacteristic-(I Ljava/lang/String; I I Ljava/util/UUID; I Ljava/util/UUID; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/media/effect/SingleFilterEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/ProfileService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/month/MonthListView;-init-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/widget/ScrollingTabContainerView$TabView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/DocumentsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/TwoPaneController;-restoreSelectedConversations-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/sip/SipAudioCall;-startAudio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/deskclock/widget/TextTime;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/EmojiCategoryPageIndicatorView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/ui/FramedTextButton;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarSettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/BackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-sendNewConfiguration-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-checkMtFindNewRinging-(Lcom/android/internal/telephony/DriverCall; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-readRemoteRssi-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiService;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v13/app/FragmentTabHost;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-setVoiceCallForwardingFlag-(I Z Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/galaxy4/Galaxy4;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getActiveNetworkQuotaInfo-()Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/ex/photo/PhotoViewPager;-setCurrentItem-(I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/net/NetworkStatsService;-performPoll-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/packageinstaller/InstallAppProgress;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/IRecognitionService;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/filterpacks/text/StringSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/mail/store/ImapFolder;-fetch-([Lcom/android/emailcommon/mail/Message; Lcom/android/emailcommon/mail/FetchProfile; Lcom/android/emailcommon/mail/Folder$MessageRetrievalListener;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/FilmstripPage;-access$1500-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/browse/MessageAttachmentTile;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/PhoneBase;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaBroadcastConfig-([Lcom/android/internal/telephony/cdma/CdmaSmsBroadcastConfigInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/provider/BrowserProvider2;-deleteInTransaction-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String; Z)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/videoeditor/widgets/OverlayView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryFacilityLockForApp-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcController$DccDefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/search/OpenSearchSearchEngine;-getActiveNetworkInfo-(Landroid/content/Context;)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/wallpaper/galaxy/GalaxyWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/ConnectivityManager;-getMobileProvisioningUrl-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/accounts/AccountManager;-editProperties-(Ljava/lang/String; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-untether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-access$000-(Lcom/android/inputmethod/keyboard/MainKeyboardView; Lcom/android/inputmethod/keyboard/PointerTracker;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-handleGsmBroadcastSms-(Landroid/os/AsyncResult;)Landroid/telephony/SmsCbMessage;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/Switch;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-isScreenMagnificationAutoUpdateEnabled-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Tab$3;-createWindow-(Z Landroid/os/Message;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/printservice/PrintService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/speechrecorder/SpeechRecorderActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/EmptyConversationListView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/wallpaper/WallpaperService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextClock;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/DeleteItems$2;-onClick-(Landroid/view/View;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/FilmstripBottomControls;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/TrimVideo;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/mail/ui/EllipsizedMultilineTextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/WifiService;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/Chronometer;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-onScrubbingEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/cardemulation/HostApduService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-onResume-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-pollStateDone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-gotoIdleAndNotifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/gatt/GattService;-clearServices-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/CalendarViewAdapter;-buildDayOfWeek-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-reportStkServiceIsRunning-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/LeaveBehindItem;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-showConversationList-(Lcom/android/mail/ConversationListContext;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-handleBootCompletedLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/SecureAlbum;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/photos/views/BlockingGLTextureView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/StockSource;-load-(Lcom/android/dreams/phototable/PhotoSource$ImageData; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/videoeditor/OverlayTitleEditor;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/RenamingDelegatingContext;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/AdapterViewFlipper;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollViewWithNotifier;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/EmlViewerActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/LatinIME;-onUpdateBatchInput-(Lcom/android/inputmethod/latin/InputPointers;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calculator2/CalculatorViewPager;-smoothScrollTo-(I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterfw/core/ProgramPort;-setFieldFrame-(Landroid/filterfw/core/Frame; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MultiAdapterSpinner;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-access$800-(Lcom/android/server/content/SyncManager; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/camera/ui/MoreSettingPopup;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-isNetworkMeteredUnchecked-(I)Z' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-broadcastConnectionState-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-findNextAlarmTime-(Landroid/content/ContentResolver; J)J' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/email/AttachmentInfo;-<init>-(Landroid/content/Context; J J Ljava/lang/String; Ljava/lang/String; J I)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email2/ui/MailActivityEmail;-onAnimationEnd-(Lcom/android/mail/ui/AnimatedAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-cancelActiveSyncLocked-(Landroid/accounts/Account; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-proceedAfterWildChar-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/service/EasServerConnection;-makeUriString-(Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/MediaPlaybackService;-getNextPosition-(Z)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/MultiInputStream;-read-([B I I)I' : ['android.permission.INTERNET'],
	'Landroid/os/SystemVibrator;-cancel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/noisefield/NoiseField;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/VibratorService$1;-onChange-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/event/EditEventView;-setAllDayViewsVisibility-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/gadget/WidgetTypeChooser;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/ExchangeService;-releaseMailbox-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView$2;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-handleProvisionError-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Sms$Outbox;-addMessage-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long; Z J)Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/widget/ActionBarView$HomeView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/ScreensaverActivity;-onResume-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/app/ReceiverRestrictedContext;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/WifiStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AnimatedAdapter;-notifyDataSetChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/NumberPicker$CustomEditText;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/SubtitleView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager;-getLastTetherError-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getCurrentSync-()Landroid/content/SyncInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/net/wifi/SupplicantStateTracker$DormantState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/preference/PreferenceFrameLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-runBoundToSyncAdapter-(Lcom/android/server/content/SyncManager$ActiveSyncContext; Landroid/content/ISyncAdapter;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/MediaPlaybackService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/research/ReplayerService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/crop/CropActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/GLTextureSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperChange;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager;-sendSyncFinishedOrCanceledMessage-(Lcom/android/server/content/SyncManager$ActiveSyncContext; Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-interceptKeyBeforeDispatching-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)J' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/ViewPager;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/PagerAdapter;-instantiateItem-(Landroid/view/ViewGroup; I)Ljava/lang/Object;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-setCLIR-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/SearchActivity;-onResume-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/ExchangeService;-stopServiceThreads-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/TabActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/AbstractActivityController$FolderLoads;-onLoadFinished-(Landroid/content/Loader; Lcom/android/mail/content/ObjectCursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/documentsui/TestActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/print/IPrintManager;-stopPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationViewAdapter$ConversationHeaderItem;-onModelUpdated-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/exchange/eas/EasOperation;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsImpl;-getEnabledSubtypesLabel-(Landroid/content/Context; Landroid/view/inputmethod/InputMethodManager; Landroid/view/inputmethod/InputMethodInfo;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/ui/AttachmentTile;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/TimerSetupView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/DownloadTouchIcon;-doInBackground-([Ljava/lang/String;)Ljava/lang/Void;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-fetchIsimRecords-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnablingActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/ChooseAccountTypeActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/SettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/datetimepicker/AccessibleTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardParserImpl_V40;-handleNest-()V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-updateShiftStateForRecapitalize-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/soundrecorder/Recorder;-stopRecording-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IAccountService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiManager;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/inputmethod/latin/settings/SettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/textservice/ITextServicesManager;-getCurrentSpellCheckerSubtype-(Ljava/lang/String; Z)Landroid/view/textservice/SpellCheckerSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AnalogClock;-updateContentDescription-(Landroid/text/format/Time;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/PolicyService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/CheckableRelativeLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/AppHwPref;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onLoadFinished-(Landroid/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/emailcommon/service/ServiceProxy$ProxyTask;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/commands/media/Media;-sendMediaKey-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$BaseFutureTask;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-onClick-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/net/DhcpStateMachine$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/development/BadBehaviorActivity$BadService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-updateMediaRecorderParams-()V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/MusicPicker;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;-downloadedMetadata-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/NativeActivity$NativeContentView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/music/MusicUtils;-shuffleAll-(Landroid/content/Context; Landroid/database/Cursor;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService;-setNetworkPreference-(I)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/exchange/EasAccountService;-sleep-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavTabScroller$ContentLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/ConversationViewFragment;-onUserVisibleHintChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$1;-loadAttachment-(Lcom/android/emailcommon/service/IEmailServiceCallback; J J Z)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/noisefield/NoiseFieldView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/PickerActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneProxy;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/ShowActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothHeadset;-acceptIncomingConnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/WeekSelector;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/telephony/gsm/SmsManager;-sendTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/camera/ui/RotateLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/compose/ComposeActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/deskclock/SettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService$4;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/WidgetSource;-getImage-(I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/timer/TimerAlertFullScreen;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/ImapService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/IAudioService;-dispatchMediaKeyEventUnderWakelock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-getConnectedHealthDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$InitialState;-turnOffUpstreamMobileConnection-()Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-hasContentChanged-()Z' : ['android.permission.READ_CONTACTS'],
	'Landroid/speech/tts/TextToSpeech;-runActionNoReconnect-(Landroid/speech/tts/TextToSpeech$Action; Ljava/lang/Object; Ljava/lang/String; Z)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarController;-launchDeleteEventAndFinish-(Landroid/app/Activity; J J J I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/commands/am/Am;-runKill-()V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/printspooler/NotificationController$NotificationBroadcastReceiver$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/RenderScriptScene;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/ConversationUpdater;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-getSignalStrength-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-pauseVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IActivityManager;-updateConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-wantBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManager$9;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/deskclock/SettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-setWfdInfo-(Landroid/net/wifi/p2p/WifiP2pWfdInfo;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/email2/ui/MailboxSelectionActivityEmail;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/DhcpStateMachine$WaitBeforeStartState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-broadcastIntentLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Landroid/content/Intent; Ljava/lang/String; Landroid/content/IIntentReceiver; I Ljava/lang/String; Landroid/os/Bundle; Ljava/lang/String; I Z Z I I I)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$1;-handleMessage-(Landroid/os/Message;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/email/activity/setup/AccountSetupBasics;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-processCode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/WifiDisplayAdapter$1;-run-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/ErrorConsoleView$ErrorConsoleListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorViewPager;-smoothScrollTo-(I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/search/SearchManagerService$MyPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/speechrecorder/SpeechRecorderActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/LinearLayoutWithDefaultTouchRecepient;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipAudioCall;-startAudioInternal-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$6100-(Lcom/android/bluetooth/hfp/HeadsetStateMachine;)V' : ['android.permission.READ_CALL_LOG'],
	'Lcom/android/deskclock/AnalogClock;-access$100-(Lcom/android/deskclock/AnalogClock;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageDraw;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppIncomingFileConfirmActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/date/AccessibleDateAnimator;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/AccessibleTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/nexus/NexusWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/Invert;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContextWrapper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/vcard/VCardParser_V21;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexSession;-start-(Landroid/os/Handler; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/EventRedirectingFrameLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-access$2300-(Lcom/android/server/power/PowerManagerService; Lcom/android/server/power/PowerManagerService$WakeLock;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onDownEvent-(I I J Lcom/android/inputmethod/keyboard/PointerTracker$KeyEventHandler;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/IntRangeManager;-finishUpdate-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TextServicesManagerService;-setCurrentSpellCheckerSubtypeLocked-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-setGrabbedState-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockApplication;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/setup/AccountSetupActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/net/INetworkStatsService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/wallpaper/galaxy/Galaxy;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/ui/NestedFolderTeaserView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/DetailsHelper$DetailsViewContainer;-reloadDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserBookmarksPage$LookupBookmarkCount;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/month/MonthListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/TintFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncStorageEngine$OnSyncRequestListener;-onSyncRequest-(Landroid/accounts/Account; I I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/EditText;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageHeaderView;-updateContactInfo-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-endServiceDeclaration-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/AlarmClockFragment;-onResume-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-init-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/vcard/VCardInterpreter;-onEntryEnded-()V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/media/AudioService$AudioHandler;-access$5100-(Landroid/media/AudioService$AudioHandler; Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/TimerRingService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/EventInfoFragment;-updateCalendar-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/email/service/EasAuthenticatorService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/setup/AccountSetupActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/TwoPaneController;-onConversationListVisibilityChanged-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/datetimepicker/time/AmPmCirclesView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService;-updateInputDeviceVibrators-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-getMediaItemFromPath-(Ljava/util/ArrayList; I I Lcom/android/gallery3d/data/DataManager;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/SkbContainer;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/am/UsageStatsService$2;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-access$1200-(Lcom/android/server/content/SyncManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onRoamingOn-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-access$800-(Lcom/android/exchange/service/EmailSyncAdapterService; Landroid/content/Context; Landroid/content/ContentResolver; Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Mailbox;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/media/effect/effects/FisheyeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/deskclock/CircleButtonsLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ComboAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/ByteStreams;-readBytes-(Lcom/google/common/io/InputSupplier; Lcom/google/common/io/ByteProcessor;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/media/AudioService$ScoClient;-requestScoState-(I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallBusy-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ingest/ui/MtpFullscreenView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountAdapter$RefreshCalendars;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiStateMachine$DriverStoppedState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$3700-(Lcom/android/gallery3d/ui/PhotoView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/Screensaver;-layoutClockSaver-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/AlbumSlidingWindow;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/view/EventRedirectingFrameLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/backup/LocalTransportService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/accessibility/AccessibleKeyboardViewProxy;-dispatchHoverEvent-(Landroid/view/MotionEvent; Lcom/android/inputmethod/keyboard/PointerTracker;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.VIBRATE'],
	'Landroid/view/IWindowManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.DISABLE_KEYGUARD', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/RetargetFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/EmailProvider;-uiFolderLoadMore-(Lcom/android/emailcommon/provider/Mailbox;)Landroid/database/Cursor;' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/service/EasServerConnection;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/content/SyncManager$9;-onServiceChanged-(Ljava/lang/Object; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/svox/pico/EngineSettings;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PackagesMonitor$AsyncService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/state/StatePanelTrack;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ActionBarView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-getEnabledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService;-registerClient-(Ljava/util/UUID; Landroid/bluetooth/IBluetoothGattCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/TrackBrowserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/AccountsTester;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/MoreKeysPanel;-onUpEvent-(I I I J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/commands/media/Media;-runDispatch-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisSpeechItem;-getSpeechRate-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-<init>-(Lcom/android/server/content/SyncManager; Lcom/android/server/content/SyncOperation; J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/InLineSettingItem;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processEventNotify-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/WidgetConfigure;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/HandleApiCalls;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/sip/SipCallBase;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/timer/CountingTimerView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-setIccLockEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/cardemulation/OffHostApduService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothGattServer;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/PolicyServiceProxy;-remoteWipe-(Landroid/content/Context;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiStateMachine$TetheredState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/net/NetworkStatsService$5;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setAndBroadcastNetworkSetTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/mail/MinTimeProgressView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/providers/downloads/ui/TrampolineActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DatePicker;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/gatt/GattService;-registerServer-(Ljava/util/UUID; Landroid/bluetooth/IBluetoothGattServerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-stopScan-(I Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/LatinIME;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager;-getAllNetworkState-()[Landroid/net/NetworkState;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/SwipeableListView;-commitDestructiveActions-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/location/GpsLocationProvider;-reportLocation-(I D D D F F F J)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/OrientationManager;-resume-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/view/PagerTabStrip;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothAdapter;-setDiscoverableTimeout-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/provider/EmailProvider;-updateAccountSyncInterval-(J Landroid/content/ContentValues;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/FormattedDateBuilder;-formatLongTime-(J)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/ControllerOverlay$Listener;-onSeekEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/Notifier;-sendWakeUpBroadcast-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-startListener-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/datetimepicker/AccessibleLinearLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHeadset;-sendVendorSpecificResultCode-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/UrlInputView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackActivity;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/media/WebVttRenderingWidget;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState$SwitchActions;-setSymbolsKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-findLastResortApplicableShortcutInputMethodAndSubtypeLocked-(Ljava/lang/String;)Landroid/util/Pair;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/LayoutInflater$BlinkLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCellInfoList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-reportAGpsStatus-(I I I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/MusicPicker;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/LogTextBox;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/AccountService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$Listener;-onScale-(F F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/ExchangeService;-canSyncEmail-(Landroid/accounts/Account;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/ConnectivityManager;-requestNetworkTransitionWakelock-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/Pop3Service;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/WebVttRenderingWidget;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/AsyncService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager$Future2Task;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-queueQuery-(I I Landroid/text/format/Time; Ljava/lang/String; I J)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/ShutdownActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendRawPdu-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$System;-getFloat-(Landroid/content/ContentResolver; Ljava/lang/String; F)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationViewHeader;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityRecord$Token;-keyDispatchingTimedOut-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-onStopSample-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisToFileOutputStreamSpeechItem;-getSpeechRate-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$ReloadTask;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioManager;-isBluetoothA2dpOn-()Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/calendar/alerts/InitAlarmsService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/NavTabScroller$OnLayoutListener;-onLayout-(I I I I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/recurrencepicker/WeekButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/UndoBarView;-render-(Lcom/android/gallery3d/glrenderer/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/PackageManagerBackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/widget/BookmarkThumbnailWidgetService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-reconnect-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-getCallerAllowedResolutionLevel-()I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accounts/AccountManagerService$7;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/email/provider/FolderPickerActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/test/mock/MockApplication;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/GpsLocationProvider;-requestUtcTime-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-access$000-(Lcom/android/server/location/GeofenceManager;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/VideoView$7;-surfaceDestroyed-(Landroid/view/SurfaceHolder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/AsyncService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/AlbumActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/tts/compat/CompatTtsService;-getDefaultSpeechRate-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationWebView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/AccountsTester;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Browser;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarApplication;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ingest/IngestService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/GLTextureSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/photo/PhotoViewActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasSearch;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/base/FrameBranch;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-releaseWakeLock-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/ErrorConsoleView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/TitleBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DayView$CalendarGestureListener;-onFling-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/eas/EasPing;-handleProvisionError-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/exchange/EasAccountService;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService$1;-validate-(Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer$1;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getLastTetherError-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-handlePinMmi-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService;-blankAllDisplaysFromPowerManager-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-setWifiEnabled-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetooth;-configHciSnoopLog-(Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BaseUi$FullscreenHolder;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPukForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-showNextConversation-(Ljava/util/Collection; Ljava/lang/Runnable;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasProvision;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/dictionarypack/WordListPreference;-deleteDict-()V' : ['android.permission.INTERNET'],
	'Landroid/content/MutableContextWrapper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/DeskClock;-initViews-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/compose/EmptyService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/view/StopProgressView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/PermissionDetails;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/CompoundButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cat/IconLoader;-loadIcons-([I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/TestActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/videoeditor/widgets/MediaLinearLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActiveServices;-attachApplicationLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/ActionableToastBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/photos/shims/MediaSetLoader;-loadInBackground-()Landroid/database/Cursor;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-sendDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/KeyboardView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-access$700-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/development/AppHwConfigList;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/SearchDialog$SearchBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/SlotView$SimpleListener;-onLongTap-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-acknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-refreshDevice-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/filtershow/category/CategoryTrack;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/VpnService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/tts/TextToSpeech;-setEngineByPackageName-(Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/AttachmentDownloadService$Watchdog$1;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipCommandInterface;-setCurrentPreferredNetworkType-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiService;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/email/SecurityPolicy;-setAccountPolicy-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Policy; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ActionableToastBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-getEnabledInputMethodSubtypeList-(Landroid/view/inputmethod/InputMethodInfo; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Gallery;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/photo/fragments/PhotoViewFragment$InternetStateBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/media/effect/effects/BitmapOverlayEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/widget/BookmarkThumbnailWidgetService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/os/BatteryStatsImpl;-setOnBatteryLocked-(Z I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/support/v4/view/ViewPager;-onMeasure-(I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/UiModeManagerService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/ComboViewActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/coremedia/iso/IsoFile;-<init>-(Ljava/nio/channels/ReadableByteChannel; Lcom/coremedia/iso/BoxParser;)V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/ui/PopupList$2;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/TimePicker;-dispatchPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/service/EmailServiceProxy;-<init>-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/test/RenamingDelegatingContext;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/ScreensaverActivity;-onStart-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/DeskClock$TabsAdapter;-notifySelectedPage-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/StringSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/WifiStateMachine;-access$16100-(Landroid/net/wifi/WifiStateMachine; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/updates/IntentFirewallInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/TouchInterceptor$DropListener;-drop-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/IsolatedContext$MockAccountManager;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/mail/ui/SwipeHelper$Callback;-onChildDismissed-(Lcom/android/mail/ui/SwipeableItemView;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/WebVttRenderingWidget$RegionLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/ContactsContract$Contacts$AggregationSuggestions$Builder;-build-()Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Connected;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/MultiAutoCompleteTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/videoeditor/widgets/AudioTrackView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothGattServer;-unregisterCallback-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/mail/store/ServiceStore;-getService-()Lcom/android/emailcommon/service/IEmailService;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/ToggleButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/GLTextureSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/ui/ConversationListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothMap$Stub;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-getCurrentInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/MediaScannerActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/RecognitionManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SmsManager;-copyMessageToIcc-([B [B I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$10;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/timezonepicker/TimeZoneResultAdapter;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/GoogleAccountLogin;-startLoginIfNeeded-(Landroid/app/Activity; Ljava/lang/Runnable;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/wallpaper/fall/FallWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MusicPicker;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/setup/AccountSetupActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/Presentation$1;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/MessageWebView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/inputmethod/latin/settings/SettingsFragment;-overwriteUserDictionaryPreference-(Landroid/preference/Preference;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_PROFILE'],
	'Landroid/view/TextureView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/SlidingDrawer;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-getIntForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Ljava/lang/String; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/CalendarView$WeeksAdapter;-onTouch-(Landroid/view/View; Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AnalogClock$2;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-access$300-(Lcom/android/server/InputMethodManagerService;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/WiredAccessoryManager$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService$CalendarObserver;-<init>-(Lcom/android/exchange/ExchangeService; Landroid/os/Handler; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/CountdownTimerPopup;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/wifi/WifiStateMachine;-stopBatchedScan-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/compose/AttachmentsView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-notifyAndAcknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/CircleTimerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/ViewRootImpl$ViewPostImeInputStage;-onProcess-(Landroid/view/ViewRootImpl$QueuedInputEvent;)I' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ISms$Stub;-copyMessageToIccEf-(Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/camera/MediaSaveService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-removeGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/ArtistAlbumBrowserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-access$100-(Lcom/android/server/accessibility/AccessibilityInputFilter; J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendRetrySms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-appendAndPutEnabledInputMethodLocked-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/state/StateView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ShutterButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/photos/shims/MediaSetLoader;-<init>-(Landroid/content/Context;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/printspooler/PrintJobConfigActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiService;-checkAndStartWifi-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/LauncherActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/support/v4/app/FragmentTabHost;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/chips/RecipientAlternatesAdapter;-getMatchingRecipients-(Landroid/content/Context; Lcom/android/ex/chips/BaseRecipientAdapter; Ljava/util/ArrayList; I Landroid/accounts/Account; Lcom/android/ex/chips/RecipientAlternatesAdapter$RecipientMatchCallback;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/LocalSource;-<init>-(Landroid/content/Context; Landroid/content/SharedPreferences;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/wallpaper/grass/Grass;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService;-getLinkQualityInfo-(I)Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/VibratorService;-onInputDeviceAdded-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/print/PrinterDiscoverySession;-destroyNoCheck-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/calculator2/CalculatorEditText;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-isBiometricWeakLivelinessEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/os/IHardwareService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/soundrecorder/SoundRecorder;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextView;-setText-(Ljava/lang/CharSequence; Landroid/widget/TextView$BufferType; Z I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/SwipeHelper;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/SlidingTab;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/AccountsTester;-onStart-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-queryInternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/browse/MessageHeaderView;-render-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-refreshCache-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardParserImpl_V21;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/Tab$3;-onReceivedTitle-(Landroid/webkit/WebView; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/music/MusicBrowserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/Switch;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-removeTrack-(J)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue;-releaseAllPointersExcept-(Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue$Element; J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/ViewPager;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection;-separate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/mail/ui/ConversationViewFragment;-onAccountChanged-(Lcom/android/mail/providers/Account; Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceTransition-(I Landroid/location/Location; I J I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/widget/WidgetService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/certinstaller/CertInstaller;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothAdapter;-closeProfileProxy-(I Landroid/bluetooth/BluetoothProfile;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/vcard/VCardParserImpl_V30;-parseItem-()Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageShow;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/FilterContext;-tearDown-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/CaptioningManager;-access$000-(Landroid/view/accessibility/CaptioningManager;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorViewPager;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/INetworkPolicyManager$Stub;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/camera/app/CameraApp;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/p2p/WifiP2pManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/email/service/PolicyService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapObexServer;-sendGetMessageRsp-(Ljavax/obex/Operation; Ljava/lang/String; Lcom/android/bluetooth/map/BluetoothMapAppParams;)I' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPuk2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-fixTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-onRestore-(Landroid/app/backup/BackupDataInput; I Landroid/os/ParcelFileDescriptor;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/calculator2/CalculatorViewPager;-pageRight-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SlotView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-getOperator-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoTable;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasCertificateRequestor;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/cardemulation/HostApduService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/date/AccessibleDateAnimator;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-removeTask-(Lcom/android/server/am/TaskRecord;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-onCodeInput-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothHealth;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-access$1500-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AlbumSetDataLoader;-access$1900-(Lcom/android/gallery3d/app/AlbumSetDataLoader; Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/CalendarColorPickerDialog;-newInstance-(J Z)Lcom/android/calendar/CalendarColorPickerDialog;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getContactsSize-()I' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/deskclock/DontPressWithParentLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterState;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-unregisterClient-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/pan/PanService$BluetoothPanBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppManager;-enableBluetooth-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/server/BluetoothA2dpService;-suspendSink-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/widget/PointerLocationView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationListCopy;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/ActivityController;-onSaveInstanceState-(Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/widget/RadioButton;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityGroup;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/hardware/IConsumerIrService;-transmit-(Ljava/lang/String; I [I)V' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/server/pm/PackageManagerService;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/mail/ui/TwoPaneController;-onRefreshReady-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/ConversationItemViewModel;-getContentDescription-(Landroid/content/Context;)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DayView;-access$5000-(Lcom/android/calendar/DayView; Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setPairingConfirmation-(Landroid/bluetooth/BluetoothDevice; Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/FrameStore;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/wifi/WifiStateMachine;-access$2100-(Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/MediaRouteButton;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/FilterSurfaceView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-getMessage-(Ljava/lang/String; Lcom/android/bluetooth/map/BluetoothMapAppParams;)[B' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-resetKeyboardStateToAlphabet-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ingest/adapter/MtpPagerAdapter;-restoreState-(Landroid/os/Parcelable; Ljava/lang/ClassLoader;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/agenda/AgendaListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestions$MoreSuggestionsListener;-onSuggestionSelected-(I Lcom/android/inputmethod/latin/SuggestedWords$SuggestedWordInfo;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-<init>-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/IBatteryPropertiesListener;-batteryPropertiesChanged-(Landroid/os/BatteryProperties;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/packageinstaller/CaffeinatedScrollView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipManager;-unregister-(Landroid/net/sip/SipProfile; Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.USE_SIP'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/google/common/io/Files;-getChecksum-(Ljava/io/File; Ljava/util/zip/Checksum;)J' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/TrimTimeBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHealth;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/BookmarkUtils$1$1;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/ui/PhotoView$MyGestureListener;-onFling-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/soundrecorder/Recorder;-restoreState-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-updatePersistentConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment;-markUnread-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/CalendarViewAdapter;-buildFullDate-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/SearchActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/OverlayLinearLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/ImapService$1;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/GalleryActivity;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/textservice/ITextServicesManager;-getCurrentSpellChecker-(Ljava/lang/String;)Landroid/view/textservice/SpellCheckerInfo;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ViewSwitcher;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/SpeechRecognizer;-putMessage-(Landroid/os/Message;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/musicvis/vis5/Visualization5;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/IAccountManager;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/AddBookmarkPage;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/music/AlbumBrowserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/downloads/ui/DownloadItem;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/PrintJobConfigActivity$Editor;-refreshCurrentPrinter-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RadioGroup;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/grass/GrassView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/DownloadHandler;-onDownloadStartNoStream-(Landroid/app/Activity; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Z)V' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/pinyin/CandidateView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter$ReloadTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/providers/telephony/MmsSmsProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu;-markConversationsImportant-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/CaptivePortalTracker$ActiveNetworkState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaSet-(Ljava/lang/String;)Lcom/android/gallery3d/data/MediaSet;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MediaPlaybackService;-access$1000-(Lcom/android/music/MediaPlaybackService; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/XLargeUi;-setUseQuickControls-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-doResumeUpdates-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-writeCharacteristic-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I I [B)V' : ['android.permission.BLUETOOTH'],
	'Landroid/nfc/cardemulation/OffHostApduService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/PrinterDiscoverySession;-<init>-(Landroid/print/IPrintManager; Landroid/content/Context; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/BrowserContract$Settings;-isSyncEnabled-(Landroid/content/Context;)Z' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-access$200-(Lcom/android/bluetooth/btservice/AdapterService; Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/ActivityController;-loadAccountInbox-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/provider/ContactsContract$ProfileSyncState;-newSetOperation-(Landroid/accounts/Account; [B)Landroid/content/ContentProviderOperation;' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ActivityController;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-<init>-(Landroid/preference/VolumePreference; Landroid/content/Context; Landroid/widget/SeekBar; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/app/Gallery;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/AliasActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/EasTestAuthenticatorService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-endServiceDeclaration-(I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-runAddAccountForAuthenticator-(Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/preference/PreferenceFrameLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setPin-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothHeadset;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/service/EmailServiceUtils;-getServiceFromInfo-(Landroid/content/Context; Lcom/android/email/service/EmailServiceUtils$EmailServiceInfo;)Lcom/android/emailcommon/service/EmailServiceProxy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/google/common/io/Resources;-toByteArray-(Ljava/net/URL;)[B' : ['android.permission.INTERNET'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-pause-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/widget/DateTimeView;-registerReceivers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/eas/EasSync;-handleResponse-(Lcom/android/exchange/EasResponse;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Lcom/android/photos/views/BlockingGLTextureView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/dreams/DreamManagerService;-getDreamComponentsForUser-(I)[Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity$1;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-addWordToUserDictionary-(Ljava/lang/String;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/server/am/ActivityManagerService;-handleAppDiedLocked-(Lcom/android/server/am/ProcessRecord; Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/text/StringSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/appwidget/AppWidgetHostView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSearch;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/Controller;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerPolicy;-performHapticFeedbackLw-(Landroid/view/WindowManagerPolicy$WindowState; I Z)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/TabControl;-recreateWebView-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onMoveEventInternal-(I I J)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Landroid/widget/ImageSwitcher;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/MailIntentService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/RecentApplicationsBackground;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/SpeechRecognizer;-handleStopMessage-()V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/gallery3d/filtershow/CenteredLinearLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$LabelView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-refreshDevice-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/print/IPrintManager;-removePrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/UninstallerActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/Screensaver;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/galaxy4/Galaxy4;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorOpacityView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/EmailApplication;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/providers/downloads/ui/TrampolineActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/opengl/GLSurfaceView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPagerController;-hide-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/ConnectivityService;-getLinkProperties-(I)Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnablingActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasMoveItems;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/WifiService;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothInputDevice;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/btservice/ProfileService;-doStop-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Landroid/inputmethodservice/InputMethodService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/browser/Bookmarks$1;-updateImages-(Landroid/content/ContentResolver; Ljava/lang/String; Landroid/content/ContentValues;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/ConnectivityService;-getNetworkPreference-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/PhotoPage;-onPictureCenter-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/ConnectivityManager;-getLinkQualityInfo-(I)Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/Gallery;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/am/ActiveServices;-serviceTimeout-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-enable-(Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiStateMachine$WaitForP2pDisableState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/power/ShutdownThread;-shutdownInner-(Landroid/content/Context; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/widget/TextView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageRotate;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHeadset;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/ObjectSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/adapter/ItemOperationsParser;-parse-()Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/TrimTimeBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/server/BluetoothA2dpService;-handleSinkStateChange-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/mail/ui/FolderChangeListener;-onFolderChanged-(Lcom/android/mail/providers/Folder; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/location/ILocationManager$Stub;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarApplication;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/BroadcastQueue;-processNextBroadcast-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/widget/AbsSeekBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/providers/userdictionary/UserDictionaryProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/development/PackageSummary;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/PhotoView;-swipeImages-(F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasSyncCalendar;-handleEntity-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Landroid/content/Entity; Ljava/lang/String; Z D)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/SystemBackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-makeScpHandler-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/musicvis/vis5/Visualization5;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/nexus/NexusWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onCancelEvent-(I I J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dispose-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-trySwitchToFirstTouchState-(F F)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/BreadCrumbView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/editors/SwapButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ServerThread;-initAndLoop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.VIBRATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/UrlInputView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/homepages/HomeProvider;-shouldInterceptRequest-(Landroid/content/Context; Ljava/lang/String;)Landroid/webkit/WebResourceResponse;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/widget/Editor$SelectionStartHandleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/textservice/SpellCheckerService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/calendar/EventInfoFragment;-onStop-()V' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/widget/DatePicker;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-isProviderEnabled-(Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/selectcalendars/SelectCalendarsSimpleAdapter;-<init>-(Landroid/content/Context; I Landroid/database/Cursor; Landroid/app/FragmentManager;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-dispose-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPicker;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/Utils;-getTimeZone-(Landroid/content/Context; Ljava/lang/Runnable;)Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/development/Details;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/DownloadOverMeteredDialog;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/app/FragmentActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/AttachmentInfo;-<init>-(Landroid/content/Context; Lcom/android/email/AttachmentInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/print/IPrintManager;-print-(Ljava/lang/String; Landroid/print/IPrintDocumentAdapter; Landroid/print/PrintAttributes; Ljava/lang/String; I I)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/EmailSyncAlarmReceiver;-handleReceive-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/research/FeedbackActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextClock;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothMap;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/ConnectivityService;-setRadio-(I Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/exchange/service/EasService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/view/menu/ListMenuItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexServerSession;-onClose-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/documentsui/DocumentsApplication;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/SpeechRecognizer;-access$100-(Landroid/speech/SpeechRecognizer;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/emailcommon/mail/Folder$MessageRetrievalListener;-loadAttachmentProgress-(I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothPan;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/AsyncQueryServiceHelper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/TrackBrowserActivity;-access$700-(Lcom/android/music/TrackBrowserActivity; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Tab;-putInBackground-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/hardware/Camera;-native_setup-(Ljava/lang/Object;)V' : ['android.permission.CAMERA'],
	'Lcom/android/mail/ui/SwipeableListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AsyncPlayer;-access$200-(Landroid/media/AsyncPlayer; Landroid/media/AsyncPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ActionBarView$HomeView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-updateBatchInputByTimer-(J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/LatinIME;-loadKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioManager;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/browser/WebViewController;-onReceivedTitle-(Lcom/android/browser/Tab; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/content/SyncStorageEngine;-readAndDeleteLegacyAccountInfoLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/IntentService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/service/dreams/DreamService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/AddBookmarkPage;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingEncryptedRfcommOn-(I)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalAlbum;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/filtershow/pipeline/ProcessingService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/WallpaperManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_WALLPAPER_HINTS', 'android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/PhotoPage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/datetimepicker/date/SimpleMonthView$OnDayClickListener;-onDayClick-(Lcom/android/datetimepicker/date/SimpleMonthView; Lcom/android/datetimepicker/date/SimpleMonthAdapter$CalendarDay;)V' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/ReceiverRestrictedContext;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/browser/BrowserPreferencesPage;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserBackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/LatinIME;-hapticAndAudioFeedback-(I I)V' : ['android.permission.VIBRATE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$3900-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/util/StateMachine$SmHandler$QuittingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/ConnectivityService;-getProvisioningNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/camera/CameraModule;-onPauseBeforeSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hid/HidService;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiManager;-pollBatchedScan-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-addTask-(I I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/chips/RecipientAlternatesAdapter;-getMatchingRecipients-(Landroid/content/Context; Lcom/android/ex/chips/BaseRecipientAdapter; Ljava/util/ArrayList; Landroid/accounts/Account; Lcom/android/ex/chips/RecipientAlternatesAdapter$RecipientMatchCallback;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccounts-(I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/DataManager;-getPanoramaSupport-(Lcom/android/gallery3d/data/Path; Lcom/android/gallery3d/data/MediaObject$PanoramaSupportCallback;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/InputStreamSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/eas/EasFolderSync;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-setAlphabetAutomaticShiftedKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/tts/compat/CompatTtsService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/UiController;-closeOtherTabs-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/btservice/AdapterService;-startDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$UpdateContent;-call-()Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/printservice/PrintService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneProxy;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/FilterSurfaceView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DateTimeView;-setTime-(J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/PowerManagerService;-systemReady-(Lcom/android/server/TwilightService; Lcom/android/server/dreams/DreamManagerService;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/Switch;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/tts/compat/CompatTtsService;-access$400-(Landroid/speech/tts/TextToSpeechService;)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-setCountryListener-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-switchHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarBackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/gadget/WidgetService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pan/PanService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/DayView;-doLongPress-(Landroid/view/MotionEvent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/deskclock/ScreensaverSettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncStorageEngine;-setMasterSyncAutomatically-(Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/base/FrameSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/EmailApplication;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-startDataStallAlarm-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-queueNextSignalStrengthPoll-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/RIL;-getIMSIForApp-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-getAuthsToSync-(Landroid/accounts/Account;)Ljava/util/HashSet;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/InstrumentationList;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Settings$System;-getString-(Landroid/content/ContentResolver; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/SearchActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/emailsync/SyncManager;-reconcileAccounts-(Landroid/content/Context;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-beginBackup-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/IMediaPlaybackService;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-pollState-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/browser/view/PieStackView;-layout-(I I Z F I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/email/activity/setup/AccountSetupIncoming;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/view/RotationPolicy;-registerRotationPolicyListener-(Landroid/content/Context; Lcom/android/internal/view/RotationPolicy$RotationPolicyListener;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/BrowserContract$Settings;-setSyncEnabled-(Landroid/content/Context; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$8400-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothInputDevice;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiService;-isScanAlwaysAvailable-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/ViewPager;-endFakeDrag-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/BluetoothHeadset;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/browse/BorderView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/FullBackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/RenderScriptWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/TwilightService$LocationHandler;-retrieveLocation-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processLocalVrEvent-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/smspush/WapPushManager;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ZoomButtonsController$Container;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/Calculator;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/AllInOneActivity$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/policy/impl/GlobalActions;-<init>-(Landroid/content/Context; Landroid/view/WindowManagerPolicy$WindowManagerFuncs;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/ViewRootImpl;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-getBatchAction-(I)Lcom/android/mail/ui/DestructiveAction;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/SearchActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/EventInfoFragment;-createExceptionResponse-(J I)V' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/media/AudioService$VolumeStreamState;-<init>-(Landroid/media/AudioService; Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ScrollIndicatorsView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/text/format/DateFormat;-getTimeFormatString-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/Utils;-refreshAlarm-(Landroid/content/Context; Landroid/view/View;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/GLView;-render-(Lcom/android/gallery3d/glrenderer/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/effect/effects/ContrastEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Ljava/net/MulticastSocket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu;-onActionItemClicked-(Landroid/view/ActionMode; Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/RenamePlaylist;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/ui/MtpFullscreenView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothMap;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/backup/LocalTransportService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-switchWithCaptureAnimation-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/activity/setup/MailboxSettings;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/datetimepicker/time/RadialSelectorView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/time/CircleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/OverlayTitleEditor;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager;-getAuthTokenByFeatures-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/app/Activity; Landroid/os/Bundle; Landroid/os/Bundle; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService;-getExplicitlyOrImplicitlyEnabledInputMethodsAndSubtypeListLocked-()Ljava/util/HashMap;' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-pauseGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/PieControl$TabAdapter$1;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/view/View;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothGattServer;-cancelConnection-(Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/printspooler/PrintJobConfigActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/LegacyImapAuthenticatorService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/ScanningProgress;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/TwoPaneController;-starMessage-(Lcom/android/mail/browse/ConversationMessage; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCurrentPreferredNetworkType-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/music/QueryBrowserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/service/IAccountService;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/ui/GLRootView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-element-(Landroid/database/Cursor; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo; Lcom/android/bluetooth/map/BluetoothMapAppParams;)Lcom/android/bluetooth/map/BluetoothMapMessageListingElement;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-resetMagnificationIfNeeded-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-registerForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/google/AbstractGoogleSource;-queryExternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/SupplicantStateTracker$DisconnectedState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/BlackWhiteEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/email/service/ImapAuthenticatorService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/ComboAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ingest/ui/MtpFullscreenView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setRadioPower-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/TabBar$TabView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSync;-upsync-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/photos/GalleryActivity$TabsAdapter;-<init>-(Lcom/android/photos/GalleryActivity; Landroid/support/v4/view/ViewPager;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/quicksearchbox/ui/ClusteredSuggestionsView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-access$1400-(Lcom/android/bluetooth/btservice/BondStateMachine; Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/eas/EasSyncBase;-getCollectionTypeHandler-(I)Lcom/android/exchange/eas/EasSyncCollectionTypeBase;' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/development/SyncAdapterDriver;-startSync-(Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService;-enforceChangePermission-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/browser/Tab;-restoreState-(Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$400-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Disconnected;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralTestActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BreadCrumbView;-popView-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cat/IconLoader;-postIcon-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/MailboxSettings;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calculator2/CalculatorViewPager;-setCurrentItem-(I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/AutoCompleteTextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/VpnService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/BatchService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager$9;-onServiceChanged-(Landroid/content/SyncAdapterType; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityManagerService;-getRecentTasks-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/camera/ui/RotatableLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/EventRedirectingFrameLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationListFragment$ConversationCursorObserver;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/NoSearchActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/ConnectivityManager;-getAllNetworkInfo-()[Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/CacheAbuser;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/EventInfoActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/crop/CropView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaFocusControl;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/CameraControls;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccCard;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage$SequentialSource;-findItemIndex-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Connected;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/widget/ILockSettings$Stub$Proxy;-getBoolean-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupOptions$1;-run-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/net/sip/SipAudioCall;-isWifiOn-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/util/StateMachine$SmHandler$HaltingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-pushModify-(J)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/SearchMailActionBarView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/text/method/TextKeyListener;-initPrefs-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController;-markConversationsRead-(Ljava/util/Collection; Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/AllInOneActivity;-handleSelectSyncedCalendarsClicked-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/DayFragment$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/Browser;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/ConfigurationViewer;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/AddBookmarkPage;-switchToDefaultView-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/AudioManager;-setMode-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Landroid/content/AbstractThreadedSyncAdapter$ISyncAdapterImpl;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ingest/ui/MtpThumbnailTileView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Space;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ImageButton;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getLastInputMethodAndSubtypeLocked-()Landroid/util/Pair;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getRemoteName-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/emailcommon/Device;-getDeviceIdInternal-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/FilterTypeSet$2;-consume-(I Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/MinTimeProgressView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/UndoBarView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/backup/BackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisToFileOutputStreamSpeechItem;-<init>-(Landroid/speech/tts/TextToSpeechService; Ljava/lang/Object; I I Landroid/os/Bundle; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoCarousel;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/providers/SuggestionsProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/photos/views/GalleryThumbnailView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiController$StaDisabledWithScanState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsFragment;-eventsChanged-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/dreams/basic/Colors;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/InputView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/timer/TimerAlertFullScreen;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub;-setDiscoverableTimeout-(I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wifi/WifiController$DeviceInactiveState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onCleanUpConnection-(Z I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/EmailIntentService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationUpdater;-refreshConversationList-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/PhotoView;-setFilmMode-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/service/EasService;-doOperation-(Lcom/android/exchange/eas/EasOperation; Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/net/IConnectivityManager;-startLegacyVpn-(Lcom/android/internal/net/VpnProfile;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/providers/downloads/ui/TrampolineActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-isWifiEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/TwilightService$LocationHandler;-enableLocationUpdates-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/ui/TimeIntervalPopup;-updateInputState-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/PinyinIME;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/GeofenceHardwareService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnablingActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ShortcutNameActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothDevice;-connectGatt-(Landroid/content/Context; Z Landroid/bluetooth/BluetoothGattCallback;)Landroid/bluetooth/BluetoothGatt;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/AsyncQueryServiceHelper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/noisefield/NoiseFieldView;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/CheckedTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/WordListPreference;-access$600-(Lcom/android/inputmethod/dictionarypack/WordListPreference;)V' : ['android.permission.INTERNET'],
	'Landroid/support/v4/app/NoSaveStateFrameLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSync;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Landroid/support/v4/view/ViewPager;-executeKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-isScanAlwaysAvailable-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/support/v4/app/FragmentActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardParser_V21;-parse-(Ljava/io/InputStream; Lcom/android/vcard/VCardInterpreter;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/inputmethod/research/ReplayerService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorBrightnessView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiService;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/widget/TextProgressBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-fetchRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/view/CustomScreenLinearLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockContentResolver;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStartingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-updateAccountManagerAccountsOfType-(Ljava/lang/String; Ljava/util/Map;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_CALENDAR', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/NativeDaemonConnector;-listenToSocket-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/power/Notifier;-access$500-(Lcom/android/server/power/Notifier;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-onLoadFinished-(Landroid/support/v4/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$TetheredState;-enter-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/opengl/GLSurfaceView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/utils/NotificationUtils;-resendNotifications-(Landroid/content/Context; Z Landroid/net/Uri; Lcom/android/mail/utils/FolderUri;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/filtershow/ui/FramedTextButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/FolderSelectionActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/DeskClock;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/providers/downloads/ui/TrampolineActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/BluetoothManagerService;-access$1700-(Lcom/android/server/BluetoothManagerService; Z Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-deinit-()V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/server/SystemBackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/ConnectivityManager;-getActiveNetworkQuotaInfo-()Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/ExchangeService;-registerCalendarObserver-(Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/ActivityController;-showWaitForInitialization-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/display/OverlayDisplayAdapter$1;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/VibratorService$VibrateThread;-<init>-(Lcom/android/server/VibratorService; Lcom/android/server/VibratorService$Vibration;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/shims/MediaSetLoader;-deleteItemWithPath-(Ljava/lang/Object;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/AlbumSetSlidingWindow;-getLoadingTexture-()Lcom/android/gallery3d/glrenderer/BitmapTexture;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/CompatModePackages;-setFrontActivityScreenCompatModeLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/usb/UsbDeviceManager;-systemReady-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-<init>-(Lcom/android/internal/telephony/uicc/UiccCard; Lcom/android/internal/telephony/uicc/IccCardApplicationStatus; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager;-runAsleep-(J J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IPowerManager$Stub;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/ActivityState;-onStateResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/Pop3Service$1;-requestSync-(J Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/widget/Editor$HandleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AlarmClockFragment$7;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothA2dp;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Browser;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/SecureConversationViewFragment;-onConversationSeen-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/ZoomControls;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/UndoBarView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/ui/CameraRootView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/widget/TextTime;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollKeyboardView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Controller;-reuseTab-(Lcom/android/browser/Tab; Lcom/android/browser/IntentHandler$UrlData;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager$5;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/browser/ActivityController;-handleNewIntent-(Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/EventRecurrenceFormatter;-getRepeatString-(Landroid/content/Context; Landroid/content/res/Resources; Lcom/android/calendarcommon2/EventRecurrence; Z)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/service/EasAutoDiscover;-makeUriString-(Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/deskclock/timer/CountingTimerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavigationBarPhone;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/SimpleMonthView$MonthViewTouchHelper;-onPerformActionForVirtualView-(I I Landroid/os/Bundle;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/http/RequestQueue;-enablePlatformNotifications-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/gadget/WidgetService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-startFreezingScreen-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperPreview;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendCdmaSms-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/EdgeView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/EasSyncService;-sync-(Lcom/android/exchange/adapter/AbstractSyncAdapter;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$2;-run-()V' : ['android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/widget/ActionableToastBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/MenuExecutor$MediaOperation;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Void;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaByDayAdapter$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/deskclock/alarms/AlarmService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getNetworkPreference-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/development/Connectivity;-onStartDelayedCycle-()V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/bluetooth/hdp/HealthService;-getConnectedHealthDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/ListPopupWindow$DropDownListView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ProxyLauncher;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/event/EditEventView;-onTimeZoneSet-(Lcom/android/timezonepicker/TimeZoneInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/DayView$CalendarGestureListener;-onScroll-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/accounts/AccountManagerService;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onActionBarAllowed-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-queryCLIP-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-access$1700-(Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler; Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PicasaSource;-getStream-(Lcom/android/dreams/phototable/PhotoSource$ImageData; I)Ljava/io/InputStream;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-serverConnect-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetooth$Stub;-configHciSnoopLog-(Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hdp/HealthService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/VideoView;-setVideoPath-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractActivityController;-onFolderChanged-(Lcom/android/mail/providers/Folder; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/ProcessInfo;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/print/PrintManagerService;-switchUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-setFiredState-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasOperation;-getRequestUri-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/email/service/EmailServiceStub$MessageRetrievalListenerBridge;-loadAttachmentProgress-(I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/ErrorConsoleView$ErrorConsoleListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage;-access$100-(Lcom/android/gallery3d/app/SlideshowPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$6;-run-()V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/deskclock/alarms/AlarmNotifications;-showLowPriorityNotification-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AccountItemView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-readPolicyLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHealth;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-access$400-(Lcom/android/packageinstaller/PackageInstallerActivity;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/ex/photo/PhotoViewActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/AbstractActivityController;-setAccount-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/GrainFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/DialogPicker;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothMap$Stub$Proxy;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/MailLogService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/category/IconView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/AbstractSettingPopup;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/Details;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/EasAccountService;-acknowledgeRemoteWipe-(Lcom/android/exchange/EasSyncService; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-setRadio-(I Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/GlobalActions$Action;-onLongPress-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/AddNewBookmark;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/gadget/WidgetConfigure;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/AbstractActivityController;-requestUpdate-(Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-updateCurrentIndex-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterApp;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/EventLoader$LoaderThread;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/bluetooth/IBluetoothHeadset;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/alerts/AlarmScheduler;-queryUpcomingEvents-(Landroid/content/Context; Landroid/content/ContentResolver; J)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/filtershow/info/HistogramView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-handleDownloadedFile-(Landroid/content/Context; Lcom/android/inputmethod/dictionarypack/DownloadRecord; Landroid/app/DownloadManager; J)Z' : ['android.permission.INTERNET'],
	'Lcom/android/server/wm/WindowManagerService;-requestTraversal-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/StackView$StackFrame;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/textservice/SpellCheckerService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/pan/PanService;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/AppHwConfigList;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryDownloadProgressBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/SnoozeAlarmsService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/GalleryActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-serverDisconnect-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/quicksearchbox/google/GoogleSource;-queryExternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/printspooler/FusedPrintersProvider;-onStartLoading-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/photo/util/ImageUtils$DataInputStreamFactory;-createInputStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/RIL;-send-(Lcom/android/internal/telephony/RILRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/OutputStreamTarget;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-updateClusters-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/EasMeetingResponder;-makeUserString-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/CalendarView$WeeksAdapter;-onDateTapped-(Ljava/util/Calendar;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/RotatableLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-stopAccountSyncs-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/photos/views/HeaderGridView$FullWidthFixedViewLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-goToSleepFromNative-(J I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationPhotoTeaserView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-addCharacteristic-(I Landroid/os/ParcelUuid; I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/music/MusicBrowserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/google/common/io/Files;-getDigest-(Ljava/io/File; Ljava/security/MessageDigest;)[B' : ['android.permission.INTERNET'],
	'Landroid/widget/CheckBox;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pan/PanService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/TwoStateImageView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/wallpaper/WallpaperService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/IWifiManager$Stub;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/ErrorConsoleView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/StickyHeaderListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasOutboxSync;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/browser/Browser;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/quicksearchbox/google/GoogleSearch;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/SwipeableConversationItemView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcDefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventFragment$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/GLRoot$OnGLIdleListener;-onGLIdle-(Lcom/android/gallery3d/glrenderer/GLCanvas; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth$Stub;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calculator2/PanelSwitcher;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/ButtonSwitcher;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/PlatLogoActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/browser/homepages/RequestHandler;-writeTemplatedIndex-()V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-requestRouteToHostAddress-(I [B Ljava/lang/String;)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/wallpaper/nexus/NexusRS;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/tinyplanet/TinyPlanetPreview;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-conference-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageTinyPlanet;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/NestedFolderTeaserView$3;-onLoadFinished-(Landroid/content/Loader; Lcom/android/mail/content/ObjectCursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/AudioService;-updateRingerModeAffectedStreams-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertReceiver;-createEmailIntent-(Landroid/content/Context; J Ljava/lang/String;)Landroid/content/Intent;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/EasSyncService;-validate-(Ljava/lang/Class; Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/Phone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackActivity$6;-onClick-(Landroid/view/View;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/server/ConnectivityService;-startUsingNetworkFeature-(I Ljava/lang/String; Landroid/os/IBinder;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiStateMachine$InitialState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calculator2/CalculatorViewPager;-populate-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/google/common/io/Files;-equal-(Ljava/io/File; Ljava/io/File;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/pinyin/CandidateView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/DataController$DataControllerHandler;-handleMessage-(Lcom/android/browser/DataController$DCMessage;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-checkPassword-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController$4;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/packageinstaller/InstallAppProgress;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/AppSecurityPermissions$PermissionItemView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/provision/DefaultActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/InboundSmsHandler$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/MultiStateButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/StopProgressView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/SecureConversationViewFragment;-onMessageCursorLoadFinished-(Landroid/content/Loader; Lcom/android/mail/browse/MessageCursor; Lcom/android/mail/browse/MessageCursor;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/app/TrimVideo;-onSeekStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/mail/ui/TwoPaneController;-showWaitForInitialization-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageVignette;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/category/CategorySelected;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl;-process-(Ljava/util/EventObject;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Service;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/emailcommon/service/EmailServiceProxy$7;-run-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/LockPatternUtils;-getLockoutAttemptDeadline-()J' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ZoomControls;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/crop/CropView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Space;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(J F Landroid/location/Criteria; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/PointerLocationView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/CaptioningManager$1;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/NativeCrashListener;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/FolderListLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-requestTransientBars-(Landroid/view/WindowManagerPolicy$WindowState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/NoSearchActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/TabControl;-createNewTab-(Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorSaturationView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/PackageBrowser;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService;-onCreate-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/IdleMaintenanceService;-sendIdleMaintenanceEndIntent-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$16;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/input/InputManager;-getPointerSpeed-(Landroid/content/Context;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/BrowserContract$SyncState;-newSetOperation-(Landroid/accounts/Account; [B)Landroid/content/ContentProviderOperation;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/certinstaller/CertInstallerMain;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-isTemplateRelevant-(Landroid/net/NetworkTemplate;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/emailsync/SyncManager;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/ILocationManager;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/browser/WallpaperHandler;-run-()V' : ['android.permission.SET_WALLPAPER', 'android.permission.INTERNET'],
	'Lcom/android/server/BackupManagerService$PerformFullBackupTask$FullBackupRunner;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/email/EmailIntentService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-onBackPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/galaxy4/GalaxyView;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/ActionBarContainer;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/widget/SlidingPaneLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageHeaderView;-rebind-(Lcom/android/mail/browse/ConversationViewAdapter$MessageHeaderItem;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS'],
	'Landroid/net/ConnectivityManager;-isNetworkSupported-(I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ingest/IngestService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/Pop3Service$1;-updateFolderList-(J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/CropFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-dumpMessages-()V' : ['android.permission.SEND_SMS', 'android.permission.READ_CONTACTS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-onReplay-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/GalleryActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BreadCrumbView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DayView;-switchViews-(Z F F F)Landroid/view/View;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/data/LocalAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiStateMachine$SupplicantStartedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/notification/NotificationListenerService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-computeNewConfiguration-()Landroid/content/res/Configuration;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowAnimator;-access$000-(Lcom/android/server/wm/WindowAnimator;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AbsSeekBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationListView$HintText;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-access$1000-(Lcom/android/bluetooth/pbap/BluetoothPbapService;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/email/provider/DBHelper;-isAccountPresent-(Landroid/accounts/AccountManager; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppIncomingFileConfirmActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hid/HidService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/SyncAdapterDriver;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/IConnectivityManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/widget/PopupWindow$PopupViewContainer;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/IconMenuView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/browse/EmlTempFileDeletionService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityStackSupervisor;-activitySleptLocked-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/hid/HidService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/videosrc/MediaSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-findImages-(I)Ljava/util/Collection;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/documentsui/DocumentsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub$Proxy;-setCurrentSpellCheckerSubtype-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-disconnectBluetoothSco-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/LockPatternUtils;-isLockScreenDisabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/music/MusicPicker;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/AliasActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/PhotoUI$9;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/NumberPicker$CustomEditText;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/AttachmentDownloadService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/ComboAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/BummerView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AttachmentTileGrid;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/accessibility/AccessibleKeyboardViewProxy;-onHoverKey-(Lcom/android/inputmethod/keyboard/Key; Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-onViewModeChanged-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothHealth;-registerAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/IBluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisToFileOutputStreamSpeechItem;-<init>-(Landroid/speech/tts/TextToSpeechService; Ljava/lang/Object; I I Landroid/os/Bundle; Ljava/lang/String; Ljava/io/FileOutputStream;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSyncCalendar;-requestSyncForMailbox-(Ljava/lang/String; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-pullPhonebook-([B Lcom/android/bluetooth/pbap/BluetoothPbapObexServer$AppParamValue; Ljavax/obex/HeaderSet; Ljavax/obex/Operation; Ljava/lang/String;)I' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/INetworkPolicyManager;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/wm/Session;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calculator2/CalculatorEditText;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDataSetupComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/DefaultSuggestionView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/FormattedDateBuilder;-formatLongDateTime-(J)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TableRow;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/noisefield/NoiseFieldView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-updateLockdownVpn-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/widget/ScrollingTabContainerView$TabView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-refreshDevice-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calculator2/CalculatorDisplay;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService;-onConnectStateChanged-([B I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/media/AudioService;-access$6900-(Landroid/media/AudioService; Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/gadget/WidgetTypeChooser;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AnimatedAdapter;-swapCursor-(Landroid/database/Cursor;)Landroid/database/Cursor;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/view/IInputMethodManager;-switchToNextInputMethod-(Landroid/os/IBinder; Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/RestrictedActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-notifyNoData-(Lcom/android/internal/telephony/dataconnection/DcFailCause; Lcom/android/internal/telephony/dataconnection/ApnContext;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/UsageStatsService$2;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/ColorChipView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/BummerView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/ui/MtpThumbnailTileView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-sendImsCdmaSms-([B I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/PicoService;-onCreate-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiStateMachine;-handleNetworkDisconnect-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationContainer;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothAdapter;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/SnailAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/google/GoogleSearch;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/LocationManagerService;-applyRequirementsLocked-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavScreen;-close-(I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/development/SyncAdapterDriver;-startSyncSelected-(Landroid/view/View;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getImsRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/power/PowerManagerService;-acquireWakeLockInternal-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; Landroid/os/WorkSource; I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ComboAlbum;-onContentDirty-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-updateParentAlarm-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/widget/AppSecurityPermissions$PermissionItemView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/ExchangeService;-requestSync-(Lcom/android/emailcommon/provider/Mailbox; I Lcom/android/emailsync/Request;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaFragment;-goTo-(Lcom/android/calendar/CalendarController$EventInfo; Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mail/ui/ConversationUpdater;-markConversationMessagesUnread-(Lcom/android/mail/providers/Conversation; Ljava/util/Set; [B)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/pm/PackageManagerService;-unloadMediaPackages-(Ljava/util/HashMap; [I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/TouchInterceptor;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-executeAndWait-(Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/AnalogClock;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/Exchange;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mail/ui/ConversationViewFragment$2;-go-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/GLRootView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/AccessibleLinearLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onStateResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/ExpandableListActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/power/ShutdownThread$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/support/v4/view/ViewPager;-arrowScroll-(I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService;-onCreate-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationItemView;-onLayout-(Z I I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/IngestActivity$4;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/KeyboardView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/GoogleAccountLogin;-run-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/support/v4/widget/DrawerLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-doInBackground-([Landroid/filterfw/core/SyncRunner;)Landroid/filterfw/core/AsyncRunner$RunnerResult;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnablingActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/service/dreams/DreamService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-setSuppServiceNotifications-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/calendar/CalendarBackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-init-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mail/ui/FolderItemView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-onDestroy-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/browse/MessageHeaderView;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumPage;-prepareAnimationBackToFilmstrip-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-onIndexFinish-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/camera/SecureCameraActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/AndroidClockTextView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationCursor$ConversationProvider;-commitMostlyDead-(Lcom/android/mail/providers/Conversation; Lcom/android/mail/browse/ConversationCursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/SubtitleController;-access$200-(Landroid/media/SubtitleController; Landroid/media/SubtitleTrack;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/personalization/UserHistoryDictionary;-access$700-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-onCreate-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/event/EditEventFragment;-onActionBarItemSelected-(I)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/compose/AutoSendActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/EmptyConversationListView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService;-setShuffleMode-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityGroup;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/printspooler/PrintJobConfigActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractMailActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/development/Connectivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/CenteredLinearLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-performSync-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/google/common/io/CharStreams$4;-getInput-()Ljava/io/Reader;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/app/AlertController$RecycleListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/IGeofenceHardware;-registerForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageRotate;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SlideshowView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-toggleShiftInSymbols-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/ToggleButton;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-removeBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/SettingsRedirector;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterfw/core/FinalPort;-transfer-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BlendFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/eas/EasSyncBase;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/location/FlpHardwareProvider;-<init>-(Landroid/content/Context;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/GalleryActionBar$OnAlbumModeSelectedListener;-onAlbumModeSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AbstractActivityController$11;-run-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-setWakeMode-(Landroid/content/Context; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment;-access$1300-(Lcom/android/email/activity/setup/AccountSettingsFragment;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/http/RequestQueue;-setProxyConfig-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/musicvis/RenderScriptWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Settings$System;-getLong-(Landroid/content/ContentResolver; Ljava/lang/String; J)J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangup-(Lcom/android/internal/telephony/cdma/CdmaConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ChooserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/TimerRingService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-connectSocket-(Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/videoeditor/KenBurnsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/wallpaper/WallpaperService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupForegroundResumeBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterApp;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Editor;-updateSpellCheckSpans-(I I Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/browser/TitleBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/InputMethodService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/FlipFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-fetchRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/telephony/MmsSmsProvider;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/browser/BaseUi$FullscreenHolder;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SlotView$SimpleListener;-onSingleTapUp-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/packageinstaller/UninstallAppProgress;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothPan;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/AlbumBrowserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/BackupManagerService$FullBackupObbConnection;-backupObbs-(Landroid/content/pm/PackageInfo; Ljava/io/OutputStream;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/app/PhotoPage;-access$1800-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ScrollView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/recurrencepicker/WeekButton;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/PhotoView;-captureAnimationDone-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/RadioButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/SlotView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/pacprocessor/PacService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleMove-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/google/GoogleSuggestClient;-getSuggestions-(Ljava/lang/String; I)Lcom/android/quicksearchbox/SuggestionCursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/CalendarViewAdapter;-buildMonthYearDate-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-detectSafeMode-()Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Spinner;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/AlbumSetPage$7;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/Ringtone;-setUri-(Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/accessibility/AccessibleKeyboardViewProxy;-performAccessibilityAction-(Landroid/view/View; I Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter$6;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Tab$4;-onDownloadStart-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-checkIfSubtypeBelongsToThisImeAndImplicitlyEnabled-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/chips/RecipientEditTextView$IndividualReplacementTask;-doInBackground-([Ljava/util/ArrayList;)Ljava/lang/Void;' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PicasaSource;-fillQueue-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavTabScroller$ContentLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService;-restartPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/MediaSaveService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/ImmersiveModeConfirmation$ClingWindowView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/AutologinBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/EditText;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Controller;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-getActiveLinkQualityInfo-()Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/activity/setup/AccountSecurity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BootReceiver$1;-run-()V' : ['android.permission.INTERNET'],
	'Lcom/android/server/NotificationManagerService$SettingsObserver;-<init>-(Lcom/android/server/NotificationManagerService; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/backup/IBackupManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/camera/ui/ExpandedGridView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImagePoint;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasAccountService;-validateAccount-(Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-access$900-(Lcom/android/bluetooth/map/BluetoothMapContentObserver;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/videoeditor/widgets/HorizontalScrollView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsImpl;-setInputMethodSettingsCategoryTitle-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService$SettingsObserver;-<init>-(Lcom/android/server/InputMethodManagerService; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/PageProgressView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasMoveItems;-getRequestUri-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/power/PowerManagerService;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/DayView;-setSelectionFromPosition-(I I Z)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/RIL;-changeIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RotateFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BackupManagerService$BackupRestoreTask;-execute-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/imageproc/GrainFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-playVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/CalendarSyncAdapterService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/LocalAlbumSet$AlbumsLoader;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasOptions;-getProtocolVersionFromServer-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CellBroadcastRangeManager;-tryAddRanges-(I I Z)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-clientDisconnect-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/ex/photo/loaders/PhotoBitmapLoader;-loadInBackground-()Lcom/android/ex/photo/loaders/PhotoBitmapLoaderInterface$BitmapResult;' : ['android.permission.INTERNET'],
	'Lcom/android/phasebeam/PhaseBeamView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech$2;-run-(Landroid/speech/tts/ITextToSpeechService;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/TestTcpSessionNotifier;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetoothHealth;-getHealthDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/svox/pico/PicoService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/pinyin/SettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/ComboAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AbstractMailActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/videoeditor/widgets/AudioTrackLinearLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/colorpicker/ColorPickerSwatch;-onClick-(Landroid/view/View;)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/CertBlacklister$BlacklistObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-access$300-(Lcom/android/deskclock/alarms/AlarmActivity;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub;-createBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/browse/EmlTempFileDeletionService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeech;-getLanguage-()Ljava/util/Locale;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-onUpdatePhoneObject-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSettings;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/accounts/IAccountManager;-addAccount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-ensureActiveMobilePolicyLocked-()V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/ConversationUpdater;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/camera/PhotoModule$5;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MediaPickerActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/emailsync/AbstractSyncService;-validate-(Ljava/lang/Class; Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeechService;-getSecureSettingInt-(Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-access$200-(Lcom/android/server/LocationManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisSpeechItem;-setRequestParams-(Landroid/speech/tts/SynthesisRequest;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/NullFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/location/GeofenceManager;-removeFence-(Landroid/location/Geofence; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-doStop-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/VideoBrowserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneProxy;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifyCellLocation-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService;-doCancelVibrateLocked-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/RIL;-setPreferredVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangup-(Lcom/android/internal/telephony/gsm/GsmCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/textservice/TextServicesManager;-setCurrentSpellChecker-(Landroid/view/textservice/SpellCheckerInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BluetoothManagerService$1;-onBluetoothStateChange-(I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/InputMethodManagerService;-setSelectedInputMethodAndSubtypeLocked-(Landroid/view/inputmethod/InputMethodInfo; I Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ViewAnimator;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSyncCalendar;-markParentsOfDirtyEvents-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/deskclock/CircleTimerView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/AllInOneActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/pinyin/SkbContainer;-onTouch-(Landroid/view/View; Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Landroid/widget/SearchView$SearchAutoComplete;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-connectChannelToSink-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/print/PrintJob;-cancel-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/RotatableLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendSubmitPdu-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsFragment;-handleEvent-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/app/Presentation$1;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/CenteredLinearLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-access$000-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/browse/FolderSpanTextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-initializeP2pSettings-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasOperation;-handleResponse-(Lcom/android/exchange/EasResponse;)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.INTERNET', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Landroid/widget/DateTimeView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupBasics;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-<init>-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment;-onConversationSeen-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/ex/photo/PhotoViewPager;-populate-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/email/activity/setup/AccountSetupOutgoing;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/gatt/GattService;-removeService-(I I I Ljava/util/UUID;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneFactory;-getGsmPhone-()Lcom/android/internal/telephony/Phone;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/renderscript/RSTextureView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-access$1400-(Lcom/android/bluetooth/map/BluetoothMapService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/deskclock/SettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BookmarkItem;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService$3;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TurnAutoSyncOnDialog$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-onMotionEvent-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/SearchView$SearchAutoComplete;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setPin-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/eas/EasSync;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/IConnectivityManager;-getProvisioningOrActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/MediaSaveService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/ActionModeHandler$ActionModeListener;-onActionItemClicked-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService$Stub;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/dreams/phototable/BummerView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/widget/SlidingPaneLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStack;-checkReadyForSleepLocked-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/Phone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-startScan-(I Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/EasCertificateRequestor;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/IWindowManager$Stub$Proxy;-prepareAppTransition-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Browser;-clearHistory-(Landroid/content/ContentResolver;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/mail/ui/AbstractMailActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/DefaultSuggestionView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipManager;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Landroid/widget/Editor;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-handleSendComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/GalleryAppImpl;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/AnimatedAdapter;-setSwipeUndo-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/WebVttTrack;-show-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/widget/GridLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/ExpandableTextView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/state/StatePanelTrack;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$InitialState;-enter-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/accounts/AccountManager$AmsTask;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/net/ConnectivityManager;-isTetheringSupported-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/content/ISyncAdapter$Stub$Proxy;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/ActivityManagerNative;-getRecentTasks-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/server/SystemBackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothMap$Stub;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-getHealthDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/Editor$HandleView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter;-getRingToneTitle-(Landroid/net/Uri;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-isOwnerInfoEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Editor$InsertionHandleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService;-startPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-doMarkConversationMessagesUnread-(Lcom/android/mail/providers/Conversation; Ljava/util/Set; [B)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/power/Notifier;-onWirelessChargingStarted-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-updateOrientationFromAppTokensLocked-(Landroid/content/res/Configuration; Landroid/os/IBinder;)Landroid/content/res/Configuration;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EventLocationAdapter;-access$700-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/util/HashSet;)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/latin/LatinIME;-handleLanguageSwitchKey-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/view/ScrollerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPagerAdapter;-instantiateItem-(Landroid/view/ViewGroup; I)Ljava/lang/Object;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/phasebeam/PhaseBeamView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Space;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/RenderScriptWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService;-needsToShowImeSwitchOngoingNotification-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/TelephonyManager;-getSimSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/LocationManagerService;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getRemoteClass-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/ConversationOverlayItem;-onModelUpdated-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-registerServer-(Landroid/os/ParcelUuid; Landroid/bluetooth/IBluetoothGattServerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/datetimepicker/time/AmPmCirclesView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/widget/CalendarAppWidgetProvider;-onUpdate-(Landroid/content/Context; Landroid/appwidget/AppWidgetManager; [I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/music/MediaPlaybackService;-open-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AbstractCalendarActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/PhotoView;-render-(Lcom/android/gallery3d/glrenderer/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/accessibility/CaptioningManager;-access$100-(Landroid/view/accessibility/CaptioningManager;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/MailActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/RenamePlaylist;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/FragmentRunnable;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/dreams/phototable/BummerView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService;-setCurrentSpellChecker-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-access$600-(Lcom/android/bluetooth/pbap/BluetoothPbapService;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/CallbackFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/FlipEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-onKeyUp-(I I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-<init>-(Landroid/content/Context; Ljava/util/HashMap; Landroid/os/Handler;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/wifi/p2p/IWifiP2pManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/IActivityManager;-getRecentTasks-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/Phone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/CountdownTimerPopup;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TwilightService$TwilightListener;-onTwilightStateChanged-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/UiController;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/emailcommon/utility/CertificateRequestor;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/fall/Fall;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkStatsService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/AsyncService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub$Proxy;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/ex/photo/PhotoViewPager;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/AttendeesView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/CircleTimerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter$GattCallbackWrapper;-onClientRegistered-(I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/IAccountManager$Stub;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/videoeditor/widgets/AudioTrackView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-runAfterDetection-(Landroid/location/Country; Landroid/location/Country; Z Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/DateTimeView;-getDateFormat-()Ljava/text/DateFormat;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/exchange/eas/EasOptions;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumPicker;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/base/OutputStreamTarget;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/time/TimePickerDialog$3;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.VIBRATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityManagerService;-startRunning-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter$7;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-populate-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/soundrecorder/Recorder;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/SearchDialog$SearchBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/common/contacts/BaseEmailAddressAdapter$DefaultPartitionFilter;-performFiltering-(Ljava/lang/CharSequence;)Landroid/widget/Filter$FilterResults;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/videoeditor/widgets/ZoomControl;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/LinearLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/EllipsizedMultilineTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/PackageSummary;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-isBluetoothAllowed-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-handleUserActivityTimeout-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/widget/BookmarkThumbnailWidgetService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/ResolverActivity$1;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/category/CategoryView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-setRepeatMode-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/nfc/cardemulation/HostApduService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-onResume-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/wifi/IWifiManager$Stub;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/email/service/ImapService$1;-deleteAccountPIMData-(Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/wallpaper/RenderScriptWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/nsd/INsdManager$Stub;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Landroid/net/nsd/NsdManager;-<init>-(Landroid/content/Context; Landroid/net/nsd/INsdManager;)V' : ['android.permission.INTERNET'],
	'Lcom/android/server/ConnectivityService;-access$5400-(Lcom/android/server/ConnectivityService; I)V' : ['android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowManagerService;-unsetAppFreezingScreenLocked-(Lcom/android/server/wm/AppWindowToken; Z Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/AnalogClock;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/deskclock/CircleButtonsLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/MccTable;-setTimezoneFromMccIfNeeded-(Landroid/content/Context; I)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/browser/TabControl;-createNewTab-(Landroid/os/Bundle; Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/email/provider/DBHelper$DatabaseHelper;-onUpgrade-(Landroid/database/sqlite/SQLiteDatabase; I I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/backup/BackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/TintFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-drawEvent-(Landroid/graphics/Canvas; Lcom/android/calendar/Event; I I I Z Z Z)I' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/google/common/io/ByteStreams$4;-getInput-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Landroid/service/wallpaper/WallpaperService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioManager;-stopBluetoothSco-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCallWaiting-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService;-setShuffleMode-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-checkPassword-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-conference-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasProvision;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getRemoteAlias-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/VideoView;-setVideoURI-(Landroid/net/Uri; Ljava/util/Map;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/display/DisplayManagerService;-setDisplayHasContent-(I Z Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager$CalendarObserver$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/camera/ui/CountDownView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ViewSwitcher;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/ImageViewTouchBase;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-shutdown-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-getPhonebookSize-(I)I' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupNegotiationState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/gadget/WidgetClickHandler;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/colorpicker/ColorPickerPalette;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/app/FragmentActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/DownloadOverMeteredDialog;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-sendConnectionStateChange-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/emailsync/SyncManager;-registerCalendarObserver-(Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/widget/DateTimeView;-update-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/timer/TimerListItem;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationViewFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/speechrecorder/SpeechRecorderActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$2800-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/event/EditEventFragment$Done;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/printspooler/NotificationController$NotificationBroadcastReceiver;-handleCancelPrintJob-(Landroid/content/Context; Landroid/print/PrintJobId; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/GrantActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/research/UploaderService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/stopwatch/StopwatchService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService$5;-onComplete-(Ljava/lang/Integer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/email/service/Pop3AuthenticatorService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/NavigationBarTablet;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getOwnerInfo-(I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IApplicationToken$Stub;-keyDispatchingTimedOut-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/widget/AbsSeekBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-markConversationMessagesUnread-(Lcom/android/mail/providers/Conversation; Ljava/util/Set; [B)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/DayView;-reloadEvents-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/widget/ActionBarContextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-updateClientsWithMetadataUri-(Landroid/content/Context; Z Ljava/lang/String;)V' : ['android.permission.INTERNET'],
	'Lcom/android/server/content/SyncManager$8;-onSyncRequest-(Landroid/accounts/Account; I I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/AppSecurityPermissions$PermissionItemView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/WakeLockHoldingProgressListener;-onProgressComplete-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothGatt;-executeReliableWrite-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Settings$System;-getConfigurationForUser-(Landroid/content/ContentResolver; Landroid/content/res/Configuration; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/PagerTitleStrip;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/CrossProcessEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/ui/ActivityController;-stopDragMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/SearchActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/SupplicantStateTracker;-sendSupplicantStateChangedBroadcast-(Landroid/net/wifi/SupplicantState; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-deleteSmsOnSim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/provider/BrowserProvider;-doSuggestQuery-(Ljava/lang/String; [Ljava/lang/String; Z)Landroid/database/Cursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/EditText;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/MailActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetherableUsbRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/service/EasTestAuthenticatorService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/DayView$ContextMenuHandler;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/vcard/VCardEntry$OrganizationData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/view/IWindowSession$Stub$Proxy;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock$FormatChangeObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/AttendeesView;-addAttendees-(Ljava/util/HashMap;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/camera/ui/FaceView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$IndicatorView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/SwipeableListView;-onScroll-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/Controller;-showPreloadedTab-(Lcom/android/browser/IntentHandler$UrlData;)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-sync-(Landroid/content/Context; J Landroid/os/Bundle; Landroid/content/SyncResult; Z I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/PhoneProxy;-phoneObjectUpdater-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/email/activity/setup/AccountSetupActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getRemoteAlias-(Landroid/bluetooth/BluetoothDevice;)Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/IccCard;-changeIccFdnPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DayView;-doDown-(Landroid/view/MotionEvent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/mail/ui/AccountItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/dreams/phototable/LocalSource;-load-(Lcom/android/dreams/phototable/PhotoSource$ImageData; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-access$700-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/downloads/ui/DownloadList;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetooth$Stub;-removeBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/email/service/LegacyEasAuthenticatorService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/notification/NotificationListenerService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/gatt/GattService;-doStop-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/browser/view/SnapshotGridView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoTable;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupIncoming;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver$2;-onServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-<init>-(Lcom/android/internal/telephony/gsm/GSMPhone;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-configHciSnoopLog-(Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/ConversationViewFragment;-processInPlaceUpdates-(Lcom/android/mail/browse/MessageCursor; Lcom/android/mail/browse/MessageCursor;)Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/browser/Controller;-doStart-(Landroid/os/Bundle; Landroid/content/Intent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSecurity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/CameraActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PicasaSource;-next-(Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/videoeditor/widgets/MediaLinearLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService$DreamReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/music/AudioPreview;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-clearLoadingBit-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onReplay-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController$ApStaDisabledState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/display/DisplayManagerService;-handleDisplayDeviceAddedLocked-(Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/ContentService;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/widgets/TransitionView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/timezonepicker/TimeZonePickerDialog;-onTimeZoneSet-(Lcom/android/timezonepicker/TimeZoneInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/MccTable;-updateMccMncConfiguration-(Landroid/content/Context; Ljava/lang/String; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/calendar/recurrencepicker/WeekButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/timezonepicker/TimeZonePickerView$OnTimeZoneSetListener;-onTimeZoneSet-(Lcom/android/timezonepicker/TimeZoneInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorBrightnessView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MusicBrowserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/AppHwConfigList;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/ConversationContainer$AdapterObserver;-onChanged-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-setUpDialog-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Controller;-openTab-(Ljava/lang/String; Z Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-shutdown-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-access$2600-(Lcom/android/server/location/GpsLocationProvider;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/app/CameraApp;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationPager;-onRestoreInstanceState-(Landroid/os/Parcelable;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/MessageScrollView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/basic/Colors;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/ContactStatusLoader;-loadInBackground-()Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/musicvis/vis3/Visualization3;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-disconnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/DatePicker;-setMinDate-(J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$6;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/AccountsTester;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/EasAccountService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-revertToNitzTimeZone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onCleanUpAllConnections-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/FilmstripBottomControls;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-completeConnection-(Lcom/android/internal/telephony/dataconnection/ApnContext;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-removeUpdates-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothAdapter;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/MinTimeProgressView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController;-access$300-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/providers/Conversation; Ljava/util/Set; [B)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/Controller;-onReceivedTitle-(Lcom/android/browser/Tab; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ingest/MtpDeviceIndex$IndexRunnable;-run-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/quicksearchbox/ui/DefaultSuggestionView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipCall;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ActionBarView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapVcardManager;-composeAndSendVCards-(Ljavax/obex/Operation; Ljava/lang/String; Z Ljava/lang/String; Z)I' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/camera/ui/ExpandedGridView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserBookmarksPage$LookupBookmarkCount;-doInBackground-([Ljava/lang/Long;)Ljava/lang/Integer;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/TileImageView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofenceTransition-(I Landroid/location/Location; I J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/shims/LoaderCompatShim;-deleteItemWithPath-(Ljava/lang/Object;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager$Stub;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/PieRenderer$3;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/compose/AttachmentComposeView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/appwidget/AppWidgetHostView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService$7;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/GalleryAppImpl;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/ActivityChooserView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/PicoService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/power/PowerManagerService;-access$1400-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/exchange/service/EasService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/NativeActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/location/GeofenceHardwareService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/ProcessInfo;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/google/android/mms/util/PduCache;-normalizeKey-(Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/research/UploaderService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/musicvis/vis4/Visualization4;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/WifiStateMachine$DefaultState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/camera/ui/InLineSettingItem;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/util/JavaNetHttpHelper;-get-(Ljava/lang/String; Ljava/util/Map;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-access$000-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.INTERNET'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Connected;-processSlcConnected-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/os/IVibratorService$Stub;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/DayView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/alerts/AlertActivity;-dismissAlarm-(J J J)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/dreams/phototable/StockSource;-naturalNext-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/CompoundButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/DirectoryView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-commitDestructiveActions-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/ConversationItemView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-formatDateString-(I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PicasaSource;-load-(Lcom/android/dreams/phototable/PhotoSource$ImageData; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/ex/chips/RecipientEditTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractActivityController;-markConversationMessagesUnread-(Lcom/android/mail/providers/Conversation; Ljava/util/Set; [B)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/BadBehaviorActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-setIccFdnEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/EditText;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/vis1/Visualization1;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/GoogleAccountLogin;-getAccounts-(Landroid/content/Context;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-execute-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/music/MediaPlaybackService;-stop-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-onScrubbingStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/settings/GallerySettings;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiService;-pollBatchedScan-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/InputMonitor;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I Z)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/browser/BreadCrumbView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManager;-killBackgroundProcesses-(Ljava/lang/String;)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/mail/browse/MessageAttachmentTile;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/printspooler/FusedPrintersProvider;-loadInternal-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/AttachmentDownloadService$DownloadSet;-processQueue-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/Space;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmAlertWakeLock;-releaseCpuLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/ImapService;-processPendingActionsSynchronous-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/ShutdownActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/UserBinaryDictionary;-writeBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/WeightedLinearLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/MediaSet;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/Button;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/ex/photo/loaders/PhotoBitmapLoader;-loadInBackground-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-startScan-(Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-setCdmaBroadcastConfig-([Lcom/android/internal/telephony/cdma/CdmaSmsBroadcastConfigInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasOutboxSync;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/EasAuthenticatorService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/EllipsizedMultilineTextView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/musicvis/RenderScriptWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/editors/SwapButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/HostApduService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-clientConnect-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/TwoPaneController;-updateConversationListFragment-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/RadioButton;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$Secure;-getLocationModeForUser-(Landroid/content/ContentResolver; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-onSystemAccountChanged-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_CALENDAR', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/email/activity/setup/AccountSetupOutgoing;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothServerSocket;-<init>-(I Z Z I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-startDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/SecureCameraActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$5100-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/renderscript/RSTextureView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pEnablingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/app/ContextImpl;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/VideoView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/CcBccView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-access$100-(Lcom/android/server/content/SyncManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/HandleApiCalls;-setupInstance-(Lcom/android/deskclock/provider/AlarmInstance; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/ex/chips/RecipientEditTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/app/IBatteryStats;-setBatteryState-(I I I I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/service/AbstractSyncAdapterService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/NotificationActionIntentService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/telephony/gsm/SmsManager;-sendMultipartTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList;)V' : ['android.permission.SEND_SMS'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothAdapter;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/ActionBarOverlayLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-createBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/RemoteViewsService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/FrameBranch;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/GestureRecognizer;-onTouchEvent-(Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/LatinIME;-access$700-(Lcom/android/inputmethod/latin/LatinIME; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/CropFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-dumpSmsTable-()V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/BluetoothAdapter;-stopLeScan-(Landroid/bluetooth/BluetoothAdapter$LeScanCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-init-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/email/service/ImapService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-registerClient-(Landroid/os/ParcelUuid; Landroid/bluetooth/IBluetoothGattCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/setup/SetupStepIndicatorView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/FilmstripPage;-transitionFromAlbumPageIfNeeded-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccCard;-setIccFdnEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getAdapterConnectionState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/ui/DownloadList;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-dismissAndSwitch-()V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/server/print/PrintManagerService;-print-(Ljava/lang/String; Landroid/print/IPrintDocumentAdapter; Landroid/print/PrintAttributes; Ljava/lang/String; I I)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/GalleryActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/development/AppHwPref;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-getMute-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/UrlInputView;-finishInput-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/base/CallbackFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/speech/tts/TextToSpeechService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/MailboxSelectionActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/EmptyConversationListView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/polarclock/PolarClockSettings;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/util/HttpHelper;-get-(Ljava/lang/String; Ljava/util/Map;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/picasasource/PicasaSource$EmptyAlbumSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/gatt/GattService$ServerDeathRecipient;-binderDied-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/TextProgressBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/FlipperDream;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/IngestActivity$2;-onActionItemClicked-(Landroid/view/ActionMode; Landroid/view/MenuItem;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-queryFacilityLock-(Ljava/lang/String; Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/ImapService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ScrollView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/DatePickerDialog$OnDateSetListener;-onDateSet-(Lcom/android/datetimepicker/date/DatePickerDialog; I I I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/net/sip/SipAudioCall;-stopCall-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setCallForward-(I I I Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipCall;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/ExpandableTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/ZoomButtonsController$Container;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CalendarView$WeekView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/RenamePlaylist;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onError-(Lcom/android/internal/telephony/Connection$DisconnectCause;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/FlipperDream;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine;-okToConnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/OnePaneController;-requestUpdate-(Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-sendUssd-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/GalleryActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/print/PrintManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/UiModeManagerService;-systemReady-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/content/PackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/preference/PreferenceActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetooth;-getBondedDevices-()[Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtErrorActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CallerInfo;-getCallerInfo-(Landroid/content/Context; Ljava/lang/String;)Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/PackagesMonitor$AsyncService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-<init>-(Landroid/content/Context; Ljava/util/HashMap;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mail/ui/OnePaneController;-onCreate-(Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/provider/ContactsContract$RawContacts;-getContactLookupUri-(Landroid/content/ContentResolver; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/photos/AlbumFragment;-onCreateLoader-(I Landroid/os/Bundle;)Landroid/content/Loader;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/VibratorService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.VIBRATE'],
	'Landroid/printservice/PrintService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/WebVttRenderingWidget$SpanLayout;-setStyle-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-setAlphabetShiftLockedKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/PieMenu;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/provider/SnapshotProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/print/IPrintManager;-restartPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$MyScaleListener;-onScale-(Landroid/view/ScaleGestureDetector;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onUpPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/PackagesMonitor$AsyncService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/gadget/WidgetConfigure;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/DialogDetailsView;-reloadDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/LatinIME;-handleRecapitalize-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/SpamWarningView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$TetheringState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/hardware/location/IGeofenceHardware;-removeGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/IConnectivityManager$Stub;-getLinkQualityInfo-(I)Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/activity/setup/AccountSetupType;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/email/service/Pop3Service;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/provider/EmailProvider;-pickFolder-(Landroid/net/Uri; I I)I' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/PhotoView;-updateActionBar-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/FolderPickerActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/wallpaper/grass/GrassRS;-updateLocation-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/os/BaseCommand;-run-([Ljava/lang/String;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.RESTART_PACKAGES'],
	'Lcom/android/mail/ui/ConversationListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt;-stopScan-(I Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/PhotoView;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-getAcceptableAccountChoices-(Landroid/accounts/AccountManager;)Ljava/util/ArrayList;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String; I Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onTrySetupData-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DayView;-initNextView-(I)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/ui/GestureRecognizer$Listener;-onUp-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onFinishSlidingInput-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/TextServicesManagerService;-getCurrentSpellChecker-(Ljava/lang/String;)Landroid/view/textservice/SpellCheckerInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/timer/TimerListItem;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/MailIntentService;-onHandleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/WiredAccessoryManager$WiredAccessoryObserver;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CallTracker;-handlePollCalls-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-updateCredentials-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/filterpacks/imageproc/CropFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityGroup;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/ToggleableItem;-toggleSelectedState-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/datetimepicker/time/RadialPickerLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-assignFolder-(Ljava/util/Collection; Ljava/util/Collection; Z Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/ObjectSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/location/GeofenceManager;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/quicksearchbox/Source;-getSuggestions-(Ljava/lang/String; I)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/ui/FaceView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enableCdmaBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/sip/SipService;-stopPortMappingMeasurement-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/pinyin/CandidatesContainer;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/MailboxSelectionActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/util/ThreadPool$Worker;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/svox/pico/Pico;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/TwoPaneController;-perhapsEnterWaitMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDisconnectDcRetrying-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/BrightnessEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/gsm/UsimDataDownloadHandler;-acknowledgeSmsWithError-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Editor$InsertionHandleView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/PrintSpoolerService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-displayReady-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/DetailsAddressResolver$AddressLookupJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Object;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/ImageView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TwoLineListItem;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AbsSpinner;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStack;-ensureActivitiesVisibleLocked-(Lcom/android/server/am/ActivityRecord; I Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/EventLoader$LoadRequest;-processRequest-(Lcom/android/calendar/EventLoader;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/development/Connectivity;-access$2600-(Lcom/android/development/Connectivity;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/service/notification/NotificationListenerService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-sendResponse-(I Ljava/lang/String; I I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setRemoteAlias-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHeadset;-stopVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/RemoteViewsService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/AlarmClockFragment;-setupAlarmInstance-(Landroid/content/Context; Lcom/android/deskclock/provider/Alarm;)Lcom/android/deskclock/provider/AlarmInstance;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-shutdown-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/development/PermissionDetails;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage;-onActionBarAllowed-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/tts/ITextToSpeechService;-speak-(Landroid/os/IBinder; Ljava/lang/String; I Landroid/os/Bundle;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/noisefield/NoiseField;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMEI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/ObjectSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumSetDataLoader;-executeAndWait-(Ljava/util/concurrent/Callable;)Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/LegacyEmailAuthenticatorService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothAdapter;-setScanMode-(I I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/photos/shims/MediaSetLoader;-<init>-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/MediaController;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine;-handlePreDhcpSetup-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/grass/GrassWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/galaxy/Galaxy;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/DataController$DataControllerHandler;-doQueryBookmarkStatus-(Ljava/lang/String; Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/data/MediaSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-getAllowedResolutionLevel-(I I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/LocationManager;-removeUpdates-(Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/os/IVibratorService$Stub$Proxy;-vibratePattern-(I Ljava/lang/String; [J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/Phone;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/CacheAbuser;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/ClockFragment$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothManager;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/service/AttachmentDownloadService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-stopDtmf-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextSwitcher;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hid/HidService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/server/VibratorService$Vibration;-binderDied-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/gallery3d/ui/DetailsHelper$DetailsSource;-setIndex-()I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-setupDataCall-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/pinyin/PinyinIME;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/packageinstaller/CaffeinatedScrollView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothPbap;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/AccountAuthenticatorActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/WifiStateMachine$DriverStartingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/policy/impl/PhoneFallbackEventHandler;-handleMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/RemoteViewsService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/webkit/WebView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContentResolver;-requestSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/printspooler/NotificationController$NotificationBroadcastReceiver$2;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/ReplayerService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/event/EditEventFragment;-saveReminders-()V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-getSingleSelectedPath-()Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/commands/am/Am;-runScreenCompat-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/CallManager;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-access$1100-(Lcom/android/gallery3d/app/AlbumSetPage; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accounts/AccountManagerService;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/app/LauncherActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub;-getTetheringErroredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothGatt;-startScanWithUuids-(I Z [Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationUpdater;-assignFolder-(Ljava/util/Collection; Ljava/util/Collection; Z Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ConversationViewFragment$2;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/RIL;-deactivateDataCall-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/dreams/basic/Colors;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/ConversationViewAdapter$MessageHeaderItem;-rebindView-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/exchange/EasOutboxService;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationPager;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$4800-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/AudioPreview;-access$700-(Lcom/android/music/AudioPreview;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStack;-completePauseLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/BluetoothHealth;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-startDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/TrimVideo;-pauseVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifyServiceState-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleInCallMmiCommands-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/InLineSettingCheckBox;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/calendar/CalendarSettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/utils/NotificationUtils;-validateNotifications-(Landroid/content/Context; Lcom/android/mail/providers/Folder; Lcom/android/mail/providers/Account; Z Z Lcom/android/mail/utils/NotificationUtils$NotificationKey;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/net/wifi/IWifiManager;-getWifiStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/VibratorService$VibrateThread;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/camera/PanoProgressBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/colorpicker/ColorPickerDialog;-onCreateDialog-(Landroid/os/Bundle;)Landroid/app/Dialog;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/colorpicker/ColorPickerSwatch;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/TimelineHorizontalScrollView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine;-fetchRssiAndLinkSpeedNative-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController$AccountLoads;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/SmsUsageMonitor;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-onKeyMove-(I I)Lcom/android/inputmethod/pinyin/SoftKey;' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-registerForSuppServiceNotification-(Landroid/os/Handler; I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ShortcutNameActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothHeadset;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/content/SyncManager$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/a2dp/A2dpService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/widget/Editor;-performLongClick-(Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-setMessageStatusRead-(J Lcom/android/bluetooth/map/BluetoothMapUtils$TYPE; I)Z' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/gatt/GattService;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/AccountsTester$GetAndInvalidateAuthTokenCallback;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/util/AsyncService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BookmarkSearch;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/tts/ITextToSpeechService$Stub;-speak-(Landroid/os/IBinder; Ljava/lang/String; I Landroid/os/Bundle;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/alarmclock/DigitalWidgetViewsFactory$RemoteWorldClockAdapter;-getViewAt-(I)Landroid/widget/RemoteViews;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorViewPager;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-onScrubbingEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$DataListener;-onLoadingFinished-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/vcard/VCardEntry$SipData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/certinstaller/CertInstallerMain;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHealth;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/GalleryActivity;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/ComprehensiveCountryDetector$1;-onCountryDetected-(Landroid/location/Country;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothGatt;-clientConnect-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-setMobileDataEnabled-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/GeneralPreferences;-showTimezoneDialog-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardEntry;-constructInsertOperations-(Landroid/content/ContentResolver; Ljava/util/ArrayList;)Ljava/util/ArrayList;' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/PackageManagerBackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSecurity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterpacks/base/FrameFetch;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaSubscriptionSource-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/GeolocationPermissionsPrompt;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-registerAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/IBluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/media/AudioManager;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-onDestroy-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/ex/photo/PhotoViewPager;-smoothScrollTo-(I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-access$700-(Lcom/android/gallery3d/ui/MenuExecutor; Lcom/android/gallery3d/data/DataManager; Lcom/android/gallery3d/util/ThreadPool$JobContext; I Lcom/android/gallery3d/data/Path;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/AsyncQueryServiceHelper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/Exchange;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-addIncludedService-(I I I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/CheckBox;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calculator2/CalculatorViewPager;-arrowScroll-(I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-sendDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/pacprocessor/PacService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailcommon/service/IEmailService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/service/AbstractSyncAdapterService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ILockSettings;-setLockPassword-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHeadset;-startScoUsingVirtualVoiceCall-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/CalendarContract$Instances;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; J J Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$SyncAdapterImpl;-onPerformSync-(Landroid/accounts/Account; Landroid/os/Bundle; Ljava/lang/String; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/DayView;-doScroll-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/InputMethodManagerService;-resetDefaultImeLocked-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-access$1000-(Lcom/android/bluetooth/opp/BluetoothOppService; Ljavax/obex/ObexTransport;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/PermissionDetails;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/CircleButtonsLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/DisableCarModeActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/emailsync/SyncManager;-getDeviceId-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/Editor$HandleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub;-setPin-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothPan;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/BadBehaviorActivity$BadService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/AlarmTimelineView;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/FilterShowActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityStack;-resetTargetTaskIfNeededLocked-(Lcom/android/server/am/TaskRecord; Z)Landroid/app/ActivityOptions;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-onGet-(Ljavax/obex/Operation;)I' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter;-loadItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/renderscript/RSSurfaceView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/text/method/TextKeyListener$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/galaxy/GalaxyRS;-start-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/ConversationViewFragment;-onFoldersClicked-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/location/GeofenceManager;-onLocationChanged-(Landroid/location/Location;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/service/dreams/Sandman;-isScreenSaverEnabled-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/ColorButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/LauncherActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/PageProgressView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMEISV-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-cleanup-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/dreams/phototable/FlipperDream;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/documentsui/DocumentsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-setEmojiKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/support/v4/view/ViewPager;-dataSetChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/hid/HidService;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/email/service/AttachmentDownloadService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/ConnectivityService;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/support/v7/widget/Space;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothDevice;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/LatinIME;-onUpdateSelection-(I I I I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/PickerActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/GpsLocationProvider;-xtraDownloadRequest-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/RecognitionService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/AlbumPicker;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/print/PrintManagerService;-cancelPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/AudioPlaybackQueueItem;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaListView;-deleteSelectedEvent-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/MailboxSettings;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EasServerConnection;-sendHttpClientPost-(Ljava/lang/String; [B J)Lcom/android/exchange/EasResponse;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/deskclock/AndroidClockTextView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/GrantActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/research/FeedbackActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ZoomControls;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-registerForNotification-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/opp/TestActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/compose/AttachmentComposeView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/util/NamedTask;-run-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ConnectivityService;-tether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/ActivityManagerNative;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.REORDER_TASKS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.BROADCAST_STICKY', 'android.permission.GET_TASKS', 'android.permission.WRITE_SETTINGS', 'android.permission.RESTART_PACKAGES', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$ConnectedState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/app/ChooserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/GpsLocationProvider;-enable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-getString-(Landroid/content/ContentResolver; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/EngineSettings;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/EditText;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/EmailConnectivityManager;-hasConnectivity-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/alerts/AlertReceiver;-makeDigestNotification-(Landroid/content/Context; Ljava/util/ArrayList; Ljava/lang/String; Z)Lcom/android/calendar/alerts/AlertService$NotificationWrapper;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/browser/UrlInputView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageGrad;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-updateSetupStepView-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-setCountryListener-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AbstractActivityController;-perhapsShowFirstSearchResult-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-getNetworkSelectionMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/NativeActivity$NativeContentView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PackagesMonitor$AsyncService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/GLView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/MediaRouteButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendNewSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/SmsHeader; I Landroid/app/PendingIntent; Landroid/app/PendingIntent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/pinyin/PinyinDecoderService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/NativeActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/GrainFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/SlidingDrawer;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-setVoiceRadioTech-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask$1$1;-onResult-(Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/bluetooth/pan/PanService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/server/accessibility/TouchExplorer$SendHoverEnterAndMoveDelayed;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/TitleBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RemoteViewsAdapter$RemoteViewsFrameLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupNegotiationState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/adapter/AttachmentLoader;-finishLoadAttachment-(Ljava/io/File;)V' : ['android.permission.INTERNET'],
	'Lcom/android/mail/ui/CreateShortcutActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/SimpleWeekView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$Receiver;-binderDied-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/providers/downloads/ui/TrampolineActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readEnhancedWebAccessibilityEnabledChangedLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockApplication;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/location/LocationBasedCountryDetector$2;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/StateManager;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-getRecoveryAction-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/DialogPicker;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/TextProgressBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/latin/LatinIME$UIHandler;-onFinishInput-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onConnectedInOrOut-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupIncoming;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/TimerRingService;-startAlarm-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Activity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-resetRadio-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/EmptyConversationListView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/ScreensaverSettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-serverDisconnect-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/packageinstaller/InstallFlowAnalytics;-writeToEventLog-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-setInputMethodEnabledLocked-(Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavigationBarBase;-onAction-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/app/ActivityManagerNative;-killBackgroundProcesses-(Ljava/lang/String; I)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-getViewAt-(I)Landroid/widget/RemoteViews;' : ['android.permission.READ_CALENDAR'],
	'Landroid/net/IConnectivityManager;-requestRouteToHostAddress-(I [B Ljava/lang/String;)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/exchange/eas/EasSyncContacts;-setSyncOptions-(Landroid/content/Context; Lcom/android/exchange/adapter/Serializer; D Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Mailbox; Z I)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/sip/SipWakeLock;-acquire-(J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TableLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipSessionGroup;-process-(Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue;-releaseAllPointersOlderThan-(Lcom/android/inputmethod/keyboard/internal/PointerTrackerQueue$Element; J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/service/Pop3Service;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/LegacyEmailAuthenticatorService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/EventInfoFragment$1;-run-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/music/DeleteItems;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/EasSyncService;-run-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/app/ChooserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/SimpleWeekView;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateCmdParamsReady;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/ConnectivityManager;-getActiveLinkQualityInfo-()Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-setCellBroadcastConfig-([Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/search/SearchEngine;-getSuggestions-(Landroid/content/Context; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHealth;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/alerts/InitAlarmsService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/info/HistogramView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hdp/HealthService;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/power/Notifier;-sendGoToSleepBroadcast-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/LocationManagerService$4;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/InstallAppProgress;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserHistoryPage$ClearHistoryTask;-run-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/app/DatePickerDialog;-updateTitle-(I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/ImapAuthenticatorService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/recurrencepicker/RecurrencePickerDialog;-onDateSet-(Lcom/android/datetimepicker/date/DatePickerDialog; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$System;-getFloatForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/BummerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AnalogClock;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-discoverServices-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/galaxy/GalaxyView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Settings$Global;-getUriFor-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/email/service/ImapService;-downloadFlagAndEnvelope-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Mailbox; Lcom/android/emailcommon/mail/Folder; Ljava/util/ArrayList; Ljava/util/HashMap; Ljava/util/ArrayList;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/ConversationLongPressTipView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-writeSmsToRuim-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/TrackBrowserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/AdapterViewAnimator;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/os/IPowerManager$Stub;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/dictionarypack/ButtonSwitcher;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/DownloadCache$DownloadTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/io/File;' : ['android.permission.INTERNET'],
	'Landroid/bluetooth/BluetoothHealth;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/base/CallbackFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/MediaSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/MovieControllerOverlay;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/calendar/SearchActivity;-deleteEvent-(J J J)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/Invert;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasFolderSync;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/VibratorService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-getCurrentInputMethodSubtypeLocked-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/EmailProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub;-tether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-onConnectedInOrOut-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/GeofenceHardware;-removeGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-switchToLastInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-update-(Lcom/android/internal/telephony/uicc/IccCardApplicationStatus; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService$Stub$Proxy;-adjustLocalOrRemoteStreamVolume-(I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/AbsoluteLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageStraighten;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-setKeyboard-(Landroid/inputmethodservice/Keyboard;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/certinstaller/CertInstallerMain;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/CreateShortcutActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/TintFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager;-destroyPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoSource;-<init>-(Landroid/content/Context; Landroid/content/SharedPreferences;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calculator2/CalculatorEditText;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupType;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/power/PowerManagerService;-updateWakeLockUids-(Landroid/os/IBinder; [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/ActionableToastBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DeviceStorageMonitorService;-clearCache-()V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/bluetooth/btservice/ProfileService;-start-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/TelephonyRegistry;-checkListenerPermission-(I)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-stayAwake-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-getDiscoverableTimeout-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/crop/CropActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/ManageCachePage;-initializeData-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$ErrorState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/BatchService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/TrimVideo;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/spellcheck/SpellCheckerSettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-reportStkServiceIsRunning-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-setModification-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-<init>-(Lcom/android/gallery3d/app/MovieActivity; Landroid/view/View; Lcom/android/gallery3d/app/MovieActivity; Landroid/net/Uri; Landroid/os/Bundle; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/ImmersiveModeConfirmation$ClingWindowView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothPan$Stub$Proxy;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/NsdService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/ProfileService;-onDestroy-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-unregisterServer-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasSettings;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/email/service/LegacyEmailAuthenticatorService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/TrimVideo$3;-run-()V' : ['android.permission.INTERNET'],
	'Landroid/net/ConnectivityManager;-untether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/Utils;-startCalendarMetafeedSync-(Landroid/accounts/Account;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/BackupManagerService;-clearBackupData-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageScrollView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget$SpanLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertReceiver;-getURLSpans-(Landroid/content/Context; J)[Landroid/text/style/URLSpan;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/email/NotificationController;-access$800-(Landroid/content/Context;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothGatt;-addIncludedService-(I I I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/GLView;-removeAllComponents-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/event/EditEventFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/deskclock/widget/TextTime;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager;-startUsingNetworkFeature-(I Ljava/lang/String; Landroid/os/IBinder;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/inputmethodservice/KeyboardView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-setProfilePriorty-(Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/IBluetoothMap$Stub$Proxy;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/PhoneFallbackEventHandler;-preDispatchKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/HorizontalScrollView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-refreshConversationList-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/EasAccountService;-sendPing-([B I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/adapter/CalendarSyncParser;-getClientIdCursor-(Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/print/PrintManagerService;-validatePrinters-(Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-isVisiblePatternEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-userActivityInternal-(J I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/text/format/DateUtils;-formatDateRange-(Landroid/content/Context; Ljava/util/Formatter; J J I Ljava/lang/String;)Ljava/util/Formatter;' : ['android.permission.READ_PROFILE'],
	'Landroid/preference/PreferenceActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CdmaBroadcastRangeManager;-finishUpdate-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchWithCaptureAnimationLocked-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService$ScoClient;-binderDied-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/os/IPowerManager$Stub;-updateWakeLockUids-(Landroid/os/IBinder; [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CallManager;-hangupForegroundResumeBackground-(Lcom/android/internal/telephony/Call;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/camera/ui/TimeIntervalPopup;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BootReceiver;-removeOldUpdatePackages-(Landroid/content/Context;)V' : ['android.permission.INTERNET'],
	'Lcom/android/mail/ui/AttachmentTile;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-moveTaskBackwards-(I)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/server/RecognitionManagerService;-getCurRecognizer-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasOptions;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/net/IConnectivityManager;-requestNetworkTransitionWakelock-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/SoundRecorder;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/AsyncService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ImageView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/MediaSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppLiveFolder;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/MoviePlayer;-onPlayPause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/galaxy/GalaxyView;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/text/ToUpperCase;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AnalogClock;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-dispatchPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-pollStateDone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-handleDown-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/ex/photo/PhotoViewActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LockSettingsService;-getString-(Ljava/lang/String; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasFolderSync;-handleProvisionError-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mail/browse/MessageFooterView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;-onResume-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-onDayTapped-(Landroid/text/format/Time;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/browser/TabScrollView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-removeAppWidget-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-removePhoneStateListener-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/MailLogService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/vcard/VCardParserImpl_V21;-parseOneVCard-()Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-beginServiceDeclaration-(I I I I Landroid/os/ParcelUuid; Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/ToggleButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasMoveItems;-upsyncMovedMessages-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mail/utils/FragmentStatePagerAdapter2;-setItemVisible-(Landroid/app/Fragment; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-unbroadcastIntent-(Landroid/app/IApplicationThread; Landroid/content/Intent; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine;-access$12100-(Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/VideoEditorBaseActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/packageinstaller/InstallAppProgress;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/InitAlarmsService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/camera/ui/ListPrefSettingPopup;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/HandleApiCalls;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/dreams/phototable/PhotoSource;-getStream-(Lcom/android/dreams/phototable/PhotoSource$ImageData; I)Ljava/io/InputStream;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHeadset;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCLIP-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/gatt/GattService;-endServiceDeclaration-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/MultiAdapterSpinner;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/EmailProvider;-uiDeleteAccount-(Landroid/net/Uri;)I' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/utils/FragmentStatePagerAdapter2;-restoreState-(Landroid/os/Parcelable; Ljava/lang/ClassLoader;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/ExchangeService$CalendarObserver$1;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/PickerActivity;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/PageProgressView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/LegacyImapSyncAdapterService;-performSync-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/data/DataManager;-getMediaType-(Lcom/android/gallery3d/data/Path;)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-getNeighboringCids-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-getActiveLinkProperties-()Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService$BluetoothShareContentObserver;-onChange-(Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/preference/PreferenceFrameLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-doStart-(Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/support/v4/net/ConnectivityManagerCompat;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/support/v4/content/ModernAsyncTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/server/am/ActivityManagerService;-forceStopPackageLocked-(Ljava/lang/String; I Z Z Z Z Z I Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/speech/tts/AudioPlaybackHandler$MessageLoop;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IAccountService$Stub;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/exchange/ExchangeService;-sendMessageRequest-(Lcom/android/emailsync/Request;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub$Proxy;-removeTracks-(I I)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/ZoomView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/widget/BookmarkThumbnailWidgetService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-removeService-(I I I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/development/BadBehaviorActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/galaxy/GalaxyWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/AudioPreview;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/AccessibleDateAnimator;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/TileImageView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/AttachmentDownloadService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/inputmethod/dictionarypack/ButtonSwitcher;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiService;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-setNetworkTemplateEnabled-(Landroid/net/NetworkTemplate; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/power/PowerManagerService;-wakeUpNoUpdateLocked-(J)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/location/LocationProviderInterface;-enable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$OnlineWatchState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/RIL;-access$300-(Lcom/android/internal/telephony/RIL;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Controller;-createNewTab-(Z Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/email/service/Pop3AuthenticatorService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/crop/CropView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationListView;-triggerSync-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/service/EasAutoDiscover;-makeUserString-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/base/GLTextureTarget;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/app/MovieActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/EasCertificateRequestor;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter;-getEstimate-(Ljava/lang/String;)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-chooseUpstreamType-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/http/SslCertificate;-inflateCertificateView-(Landroid/content/Context;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/BaseSuggestionView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothGatt;-discoverServices-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothHealth;-registerSinkAppConfiguration-(Ljava/lang/String; I Landroid/bluetooth/BluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/vcard/VCardEntry;-iterateOneList-(Ljava/util/List; Lcom/android/vcard/VCardEntry$EntryElementIterator;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/DayView;-getEventAccessLevel-(Landroid/content/Context; Lcom/android/calendar/Event;)I' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-proceedAfterWaitChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NsdService$NsdStateMachine$DefaultState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/accounts/AccountManager$1;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/view/accessibility/CaptioningManager$CaptionStyle;-getCustomStyle-(Landroid/content/ContentResolver;)Landroid/view/accessibility/CaptioningManager$CaptionStyle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStack;-goToSleep-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/videoeditor/widgets/AudioTrackView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/phasebeam/PhaseBeamView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPickerActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/AutoCompleteTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/photos/views/SquareImageView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/ViewPager;-addNewItem-(I I)Landroid/support/v4/view/ViewPager$ItemInfo;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/LayoutInflater$BlinkLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager$AccountObserver;-onAccountChanged-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-setIccFdnEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-updateMessageOnIccEf-(Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/music/TrackBrowserActivity;-moveItem-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/IMediaPlaybackService;-enqueue-([J I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothManager;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/photos/shims/MediaItemsLoader;-loadInBackground-()Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-addTrackerToRawTable-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/camera/ui/CountDownView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageStraighten;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiController$FullLockHeldState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/mail/compose/EnterSubject;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceAddStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService;-connectOtherProfile-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationCursor;-setConversationColumn-(Landroid/net/Uri; Ljava/lang/String; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$ConnectModeState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/telephony/CellBroadcastMessage;-getSpokenDateString-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/SwipeableListView;-destroyItems-(Ljava/util/Collection; Lcom/android/mail/ui/SwipeableListView$ListItemsRemovedListener;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/accounts/AccountManagerService$3;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/LocationManagerService;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/provider/DBHelper;-access$800-(Landroid/database/sqlite/SQLiteDatabase; Landroid/content/Context;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppUtility;-queryRecord-(Landroid/content/Context; Landroid/net/Uri;)Lcom/android/bluetooth/opp/BluetoothOppTransferInfo;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/Connectivity;-onCrash-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/AbstractActivityController;-showConversationList-(Lcom/android/mail/ConversationListContext;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment$3;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/NavigationBarTablet;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/galaxy/Galaxy;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/TabBar$TabView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setRemoteAlias-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-clearServices-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email2/ui/CreateShortcutActivityEmail;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/deskclock/AndroidClockTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHeadset;-clccResponse-(I I I I Z Ljava/lang/String; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCLIR-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService$ScreenOnBlockerImpl;-release-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-reloadAudioSettings-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifyDataConnection-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/VideoEditorBaseActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/CropFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/service/IEmailService;-validate-(Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/MailboxSelectionActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/power/ShutdownThread;-reboot-(Landroid/content/Context; Ljava/lang/String; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/net/NetworkPolicyManagerService$7;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-beginReliableWrite-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/IntentService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-assignContactFromEmail-(Ljava/lang/String; Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/TwoPaneController;-handleUpPress-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/emailcommon/utility/CertificateRequestor;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothMap$Stub$Proxy;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/TwoPaneController;-getBatchAction-(I)Lcom/android/mail/ui/DestructiveAction;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/ExchangeService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/MusicBrowserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-onPause-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu;-markConversationsRead-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState$SwitchActions;-requestUpdatingShiftState-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Pending;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/mail/browse/ConversationViewHeader;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsImpl;-setSubtypeEnablerIcon-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ReceiverRestrictedContext;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationViewFragment;-renderCollapsedHeaders-(Lcom/android/mail/browse/MessageCursor; Lcom/android/mail/browse/ConversationViewAdapter$SuperCollapsedBlockItem;)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/development/SyncAdapterDriver;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarBackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Connected;-processAudioEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/widget/PopupWindow$PopupViewContainer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendImsCdmaSms-([B I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email2/ui/MailActivityEmail;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IUserSwitchObserver$Stub$Proxy;-onUserSwitching-(I Landroid/os/IRemoteCallback;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/compose/EnterSubject;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EasTestAuthenticatorService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/OverlayTitleTemplatePicker;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/DataManager;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/gatt/GattService;-addDescriptor-(I Ljava/util/UUID; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/activity/setup/AccountSetupIncomingFragment$1;-onItemSelected-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/email/activity/ComposeActivityEmail;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/GalleryActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/SpellChecker;-spellCheck-(I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/CallLog$Calls;-removeExpiredEntries-(Landroid/content/Context;)V' : ['android.permission.WRITE_CALL_LOG'],
	'Landroid/widget/ImageButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/PlaylistBrowserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onResponse-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/os/IVibratorService;-vibratePattern-(I Ljava/lang/String; [J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAccountsByTypeAsUser-(Ljava/lang/String; Landroid/os/UserHandle;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/CalendarUtils$TimeZoneUtils;-forceDBRequery-(Landroid/content/Context; Ljava/lang/Runnable;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/LocationManagerService;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/ingest/ui/DateTileView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calculator2/CalculatorViewPager;-setCurrentItem-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/camera/PhotoModule;-initializeSecondTime-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/support/v4/content/AsyncTaskLoader;-onLoadInBackground-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/crop/CropActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/IConnectivityManager$Stub;-getLastTetherError-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/UpgradeReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/MtpClient$Listener;-deviceAdded-(Landroid/mtp/MtpDevice;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accounts/ChooseAccountActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/widget/WidgetService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/category/CategoryView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextSwitcher;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService;-access$600-(Lcom/android/bluetooth/hid/HidService; Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/ConversationListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/app/NoSaveStateFrameLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasPing;-requestPing-(Landroid/accounts/Account;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/development/InstrumentationList;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/Connectivity;-access$2200-(Lcom/android/development/Connectivity;)V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/filtershow/state/StateView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/Settings$Secure;-getFloat-(Landroid/content/ContentResolver; Ljava/lang/String;)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/FieldPort;-transfer-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/IAudioService$Stub;-adjustVolume-(I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/LocationBlacklist;-switchUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/TimeIntervalPopup;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/wallpaper/grass/GrassRS;-access$300-(Lcom/android/wallpaper/grass/GrassRS;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/AllInOneActivity;-createButtonsSpinner-(I Z)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/datetimepicker/date/TextViewWithCircularIndicator;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventView;-setModel-(Lcom/android/calendar/CalendarEventModel;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/SearchActivity;-eventsChanged-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/NetworkPolicyManagerService$10;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralTestActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/TestActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-processPostDialChar-(C)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-updateCurrentCarrierInProvider-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/exchange/eas/EasSettings;-getRequestEntity-()Lorg/apache/http/HttpEntity;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/telephony/SmsManager;-sendTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/music/QueryBrowserActivity;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Telephony$Sms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/google/common/io/Files;-copy-(Lcom/google/common/io/InputSupplier; Ljava/io/File;)V' : ['android.permission.INTERNET'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handleRadioNotAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageRedEye;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-getTetheredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/NativeActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager$MulticastLock;-finalize-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/widget/VideoView;-suspend-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/compose/AutoSendActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-separate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/wm/Wm;-main-([Ljava/lang/String;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.RESTART_PACKAGES'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallbackProxy$2;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/compose/AttachmentComposeView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TimePicker;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasFolderSync;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/VideoView$7;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ChooserActivity;-onCreate-(Landroid/os/Bundle; Landroid/content/Intent; Ljava/lang/CharSequence; [Landroid/content/Intent; Ljava/util/List; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/LockPatternUtils;-getString-(Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine;-access$24200-(Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/google/common/io/Resources$1;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/data/LocalAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/TrackBrowserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtErrorActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/sip/ISipService;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/email/provider/EmailProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ShortcutNameActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSettings;-onAttachFragment-(Landroid/app/Fragment;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/personalization/DecayingExpandableBinaryDictionaryBase;-access$700-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/GridView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/EmlViewerActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/sip/SipManager;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/bluetooth/pan/PanService;-cleanup-()Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/ex/chips/RecipientEditTextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/content/PackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/date/YearPickerView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getPDPContextList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingEncryptedRfcommWithServiceRecord-(Ljava/lang/String; Ljava/util/UUID;)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothMap;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSetupType;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/SuggestionsView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/ConversationListFragment$5;-onListItemsRemoved-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransfer;-startConnectSession-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/location/CountryDetectorBase;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/IWindowManager;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Landroid/app/ActivityManagerNative;-inputDispatchingTimedOut-(I Z Ljava/lang/String;)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/data/SnailAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-addCharacteristic-(I Landroid/os/ParcelUuid; I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/inputmethodservice/ExtractEditText;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/PlayheadView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-checkListenerOrIntentLocked-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; I I Ljava/lang/String; Landroid/os/WorkSource; Z)Lcom/android/server/LocationManagerService$Receiver;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/EmailIntentService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/CalendarView;-setDate-(J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageHeaderView;-access$000-(Lcom/android/mail/browse/MessageHeaderView;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/browse/ConversationItemView;-bind-(Lcom/android/mail/providers/Conversation; Lcom/android/mail/ui/ControllableActivity; Lcom/android/mail/ui/AnimatedAdapter$ConversationListListener; Lcom/android/mail/ui/ConversationSelectionSet; Lcom/android/mail/providers/Folder; I Z Z Z Z Z Lcom/android/mail/ui/AnimatedAdapter;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$Receiver;-callLocationChangedLocked-(Landroid/location/Location;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageHeaderView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/SettingInjectorService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationViewHeader;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/print/PrintHelper$PrintHelperKitkatImpl;-printBitmap-(Ljava/lang/String; Landroid/graphics/Bitmap;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;-executeMessage-(Landroid/os/Message;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/power/PowerManagerService$2;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/view/SurfaceView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasFolderSync;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-dispatchMessageRadioSpecific-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/widget/SlidingTab;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavigationBarBase;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/InputMethodManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IApplicationToken$Stub$Proxy;-windowsVisible-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendCDMAFeatureCode-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasMoveItems;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/magicsmoke/MagicSmoke;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/musicvis/RenderScriptWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wm/Session;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/ConnectivityManager;-requestRouteToHost-(I I)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processVrEvent-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UiController;-openTabToHomePage-()Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-handleCancel-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/mail/browse/FolderSpanTextView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/smspush/WapPushManager;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionaryList;-onResume-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/EmailIntentService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintDialogFrame;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapObexServer;-pushMessage-(Ljavax/obex/Operation; Ljava/lang/String; Lcom/android/bluetooth/map/BluetoothMapAppParams;)I' : ['android.permission.SEND_SMS'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/view/ViewOverlay$OverlayViewGroup;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/GLTextureSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-getBoolean-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/AudioPlaybackQueueItem;-finish-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/FilmstripPage;-addSecureAlbumItem-(Z I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/ExchangeService;-checkMailboxes-()J' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Landroid/test/IsolatedContext;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/alerts/AlertReceiver;-getLocationCursor-(Landroid/content/Context; J)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/ShortcutActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothA2dp;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/PhotoFallbackEffect;-setPositionProvider-(Lcom/android/gallery3d/ui/PhotoFallbackEffect$PositionProvider;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MediaPlaybackService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasMoveItems;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodSubtypeListLocked-(Landroid/content/Context; Landroid/view/inputmethod/InputMethodInfo; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-acquireWakeLock-(J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/SubtitleController;-doSelectDefaultTrack-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorEditText;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-fillQueue-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/base/OutputStreamTarget;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-initSeekBar-(Landroid/widget/SeekBar; Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingInsecureRfcommOn-(I)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/tts/TextToSpeech$12;-run-(Landroid/speech/tts/ITextToSpeechService;)Ljava/lang/Integer;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/BummerView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-switchToState-(I F F)V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/Controller$10;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/BaseCommands;-setCurrentPreferredNetworkType-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStack;-findTaskToMoveToFrontLocked-(I I Landroid/os/Bundle;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/activity/setup/AccountSetupActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$900-(Lcom/android/gallery3d/ui/PhotoView; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/NavScreen;-onCloseTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/ingest/IngestService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/os/BatteryStatsImpl;-doUnplugLocked-(J J J)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityManagerService;-getTasks-(I I Landroid/app/IThumbnailReceiver;)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/packageinstaller/CaffeinatedScrollView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/test/RenamingDelegatingContext;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/SearchView$SearchAutoComplete;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/preference/VolumePreference;-onBindDialogView-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/app/IAlarmManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/photos/FullscreenViewer;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/AlertAdapter;-updateView-(Landroid/content/Context; Landroid/view/View; Ljava/lang/String; Ljava/lang/String; J J Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/dreams/phototable/PhotoTableDream;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-onRefresh-(Z [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/mail/ui/FolderSelectionDialog$1;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/content/IContentService;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/quicksearchbox/ui/BaseSuggestionView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/UndoBarView;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/ui/CountDownView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/SlotView;-render-(Lcom/android/gallery3d/glrenderer/GLCanvas;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/DayView;-sendAccessibilityEventAsNeeded-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/TextTime;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$3600-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Landroid/net/wifi/p2p/WifiP2pWfdInfo;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/Gallery;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService;-buildInputMethodListLocked-(Ljava/util/ArrayList; Ljava/util/HashMap; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DatePicker;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/MediaScannerActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/ConversationItemView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer;-postEventFromNative-(Ljava/lang/Object; I I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-getSubscriptionInfoAndStartPollingThreads-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/TwilightService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/gatt/GattService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/location/SettingInjectorService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/service/ApiService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/Cluster;-generateCaption-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController;-access$2600-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/providers/Conversation;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/search/Searchables;-findGlobalSearchActivity-(Ljava/util/List;)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment;-saveSettings-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/camera/ui/CameraControls;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Landroid/widget/ProgressBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-onDayClick-(I)V' : ['android.permission.VIBRATE'],
	'Landroid/inputmethodservice/ExtractButton;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHeadset;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/FallbackEventHandler;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCallWaitingIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService;-updateShowTouchesFromSettings-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AutologinBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/TimeIntervalPopup;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ProxyLauncher;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/alarms/AlarmNotifications;-showMissedNotification-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/requestsync/RequestSync;-run-([Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/email/service/ImapService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/PreviewPlacerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier$MagnifiedContentInteractonStateHandler;-onMotionEvent-(Landroid/view/MotionEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/CarrierProvisioningUrlsInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-stopUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetooth;-getRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/service/LegacyEmailAuthenticatorService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/Invert;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/test/SimulatedRadioControl;-shutdown-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/gatt/GattService;-clientConnect-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/ZoomView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController;-handleDragFromStarred-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccountsAsUser-(Ljava/lang/String; I Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-writeBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/event/CreateEventDialogFragment;-access$500-(Lcom/android/calendar/event/CreateEventDialogFragment;)V' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/app/IActivityManager;-updatePersistentConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/inputmethod/latin/SubtypeSwitcher;-updateParametersOnStartInputView-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMnsObexClient;-shutdown-()V' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/LocationManagerService$4;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/settings/SettingsFragment$2;-feedbackValue-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/view/menu/ActionMenuView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/EventInfoActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-handleDropInStarred-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/ConversationWebView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/AccessibilityDelegateCompat;-performAccessibilityAction-(Landroid/view/View; I Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/pm/Installer; Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/quicksearchbox/ui/CorpusView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollViewWithNotifier;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/ui/RenderOverlay;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pDisabledState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-setCurrentUser-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-handleDrop-(Landroid/view/DragEvent; Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/LatinIME;-onStartInputView-(Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/media/effect/effects/RedEyeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/WebVttRenderingWidget;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountAuthenticatorActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Controller;-onActivityResult-(I I Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppIncomingFileConfirmActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/videoeditor/OverlayTitleTemplatePicker;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-rebootSafeMode-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/content/SyncManager;-sendSyncAlarmMessage-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-acknowledgeLastIncomingCdmaSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/ConfigurationViewer;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/widget/EllipsizeLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorCompareView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/GalleryActivity;-startDefaultPage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/AudioAndHapticFeedbackManager;-vibrate-(J)V' : ['android.permission.VIBRATE'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-callListenerOnFinishSlidingInput-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/month/MonthListView;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/BreadCrumbView;-notifyController-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/view/WindowManagerPolicy;-getUserRotationMode-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment;-showUntransformedConversation-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-callListenerOnCodeInput-(Lcom/android/inputmethod/keyboard/Key; I I I J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-isAudioConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/accounts/AccountManagerService;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/calendar/alerts/AlertService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/TimeBar$Listener;-onScrubbingEnd-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-access$900-(Landroid/media/MediaPlayer; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/SuperCollapsedBlock;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/RecognitionManagerService$MyPackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/NetInitiatedActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/SubtypeSwitcher;-getCurrentSubtypeLocale-()Ljava/util/Locale;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindow$DecorView;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/camera/PhotoModule;-onPauseAfterSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/CaptivePortalTracker$NoActiveNetworkState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/DayView;-appendEventAccessibilityString-(Ljava/lang/StringBuilder; Lcom/android/calendar/Event;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/TintFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/MediaSaveService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-access$200-(Lcom/android/bluetooth/hfp/HeadsetService; [I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/LockdownVpnTracker;-onNetworkInfoChanged-(Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/OnePaneController;-access$1100-(Lcom/android/mail/ui/AbstractActivityController; Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/providers/downloads/ui/DownloadItem;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/PagerTabStrip$2;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController$2;-onInit-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-getReceiverLocked-(Landroid/location/ILocationListener; I I Ljava/lang/String; Landroid/os/WorkSource; Z)Lcom/android/server/LocationManagerService$Receiver;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub;-getCurrentSpellChecker-(Ljava/lang/String;)Landroid/view/textservice/SpellCheckerInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BrowserActivity;-createController-()Lcom/android/browser/Controller;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/CropFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/android/mms/pdu/PduPersister;-persist-(Lcom/google/android/mms/pdu/GenericPdu; Landroid/net/Uri; Z Z Ljava/util/HashMap;)Landroid/net/Uri;' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-setEvents-(I I Ljava/util/ArrayList;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/AllInOneActivity;-onSharedPreferenceChanged-(Landroid/content/SharedPreferences; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$1;-validate-(Lcom/android/emailcommon/provider/HostAuth;)Landroid/os/Bundle;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-deleteMessageSms-(J)Z' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/camera/ui/TwoStateImageView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/grass/Grass;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/SamplingProfilerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/BondStateMachine$StableState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompatJellyBean$1;-performAction-(I I Landroid/os/Bundle;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-cleanup-()Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/pacprocessor/PacService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/timer/TimerAlertFullScreen;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/CircleButtonsLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/WeightedLinearLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Controller;-closeOtherTabs-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/app/AppBridge$Server;-switchWithCaptureAnimation-(I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/TimeBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-addCircularFence-(I I D D D I I I I Landroid/hardware/location/IGeofenceHardwareCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/ActivityState;-setContentPane-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/GLTextureTarget;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setInitialAttachApn-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/EmlMessageViewFragment$MessageLoadCallbacks;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-updatePhoneObject-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/app/AliasActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityManagerService;-bindBackupAgent-(Landroid/content/pm/ApplicationInfo; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/IConnectivityManager$Stub;-getAllLinkQualityInfo-()[Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationViewFragment;-renderContent-(Lcom/android/mail/browse/MessageCursor;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/latin/settings/SettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationListFragment;-onListItemSwiped-(Ljava/util/Collection;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/tts/TextToSpeechService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/gallery3d/app/VideoUtils;-startMute-(Ljava/lang/String; Lcom/android/gallery3d/util/SaveVideoFileInfo;)V' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-handleCdmaSubscriptionSource-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/printspooler/SelectPrinterActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/ConnectivityService;-getMobileProvisioningUrl-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-vibrate-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/music/TrackBrowserActivity;-removePlaylistItem-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/CallbackFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/GLRootView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/LocalMediaItem;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/VolumePanel;-onRemoteVolumeUpdateIfShown-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneController;-popView-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/SubtypeSwitcher;-getCurrentSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/FolderListLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-access$2200-(Lcom/android/bluetooth/opp/BluetoothOppService; Landroid/database/Cursor; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountAuthenticatorActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/NotificationController$MessageContentObserver;-onChange-(Z)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/net/wifi/WifiManager;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupCreatingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/printspooler/PrintSpoolerService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothDevice;-createRfcommSocket-(I)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofenceResume-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-requestRouteToHost-(I I Ljava/lang/String;)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/wallpaper/fall/Fall;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/VpnService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/YearPickerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompat;-createAccessibilityNodeInfo-(I)Landroid/support/v4/view/accessibility/AccessibilityNodeInfoCompat;' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-removeTracks-(I I)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/browser/TabBar$TabView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/mail/ui/ConversationUpdater;-showNextConversation-(Ljava/util/Collection;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/commands/am/Am;-run-([Ljava/lang/String;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.RESTART_PACKAGES'],
	'Landroid/content/IContentService;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/ImageButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationContainer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SmsManager;-disableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/musicvis/vis4/Visualization4;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-separate-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager$MulticastLock;-release-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-connectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/setup/SetupStepIndicatorView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService$2;-onChange-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/fall/Fall;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageMirror;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/dreams/phototable/StockSource;-naturalPrevious-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/ViewSwitcher;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-handleCallSetupRequestFromSim-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/DialogDetailsView;-show-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/backup/BackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/text/method/MultiTapKeyListener;-onKeyDown-(Landroid/view/View; Landroid/text/Editable; I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ExpandableListActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ActionBarOverlayLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcRetryingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/TimeBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-access$300-(Lcom/android/server/accessibility/ScreenMagnifier; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-registerForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/os/PowerManager$WakeLock;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/EditEventHelper;-saveReminders-(Ljava/util/ArrayList; J Ljava/util/ArrayList; Ljava/util/ArrayList; Z)Z' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/galaxy4/GalaxyView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/PageDialogsHandler;-showSSLCertificateOnError-(Landroid/webkit/WebView; Landroid/webkit/SslErrorHandler; Landroid/net/http/SslError;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/videoeditor/widgets/OverlayLinearLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calculator2/Calculator;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/ScanningProgress;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/adapter/AbstractSyncParser;-parse-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Lcom/android/deskclock/widget/TextTime;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager;-hasFeatures-(Landroid/accounts/Account; [Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/SupplicantStateTracker$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-configAudioParameters-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/phasebeam/PhaseBeam;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/SecureSource;-createMediaObject-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/MediaObject;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/pm/PackageManagerService$AppDirObserver;-onEvent-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/BrowserActivity;-onDestroy-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/emailcommon/service/ServiceProxy;-<init>-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/email/service/EmailServiceUtils;-updateAccountManagerType-(Landroid/content/Context; Landroid/accounts/Account; Ljava/util/Map;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_CALENDAR', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/base/FrameStore;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/os/IVibratorService$Stub;-vibratePattern-(I Ljava/lang/String; [J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-vibrate-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-dispatchMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/server/power/PowerManagerService;-releaseWakeLockInternal-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$SetDnsForwardersErrorState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/ex/photo/PhotoViewActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/DatePicker;-updateCalendarView-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothInputDevice;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/compose/AutoSendActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/WidgetTypeChooser;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneProxy;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/TextureView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageHeaderView;-toggleExpanded-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub;-sendConnectionStateChange-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calculator2/CalculatorViewPager;-endFakeDrag-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/TelephonyRegistry;-broadcastServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/IPrintManager$Stub$Proxy;-validatePrinters-(Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getSelectedInputMethodSubtypeHashCode-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/accessibilityservice/AccessibilityService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/PingSyncSynchronizer$AccountSyncState;-pingEnd-(Landroid/accounts/Account;)Z' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ConversationListFragment$1;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/exchange/EasAccountService;-setupServiceForAccount-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account;)Lcom/android/exchange/EasSyncService;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/ScanningProgress;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/NetInitiatedActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/crop/CropActivity$BitmapIOTask;-doInBackground-([Landroid/graphics/Bitmap;)Ljava/lang/Boolean;' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/ListActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserActivity;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/btservice/ProfileService;-doStart-(Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/wallpaper/galaxy/Galaxy;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/HandleApiCalls;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-pollCallsWhenSafe-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/svox/pico/GetSampleText;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasSyncMail;-cleanup-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/providers/userdictionary/DictionaryBackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/AnimatedListView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/usb/UsbService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-registerDestructiveAction-(Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/MediaFocusControl$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/views/SquareImageView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleUp-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/UserManagerService$5;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageTinyPlanet;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-scanInternalSubtitleTracks-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiStateTracker;-setRadio-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/exchange/EasAccountService;-hasConnectivity-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothInputDevice;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/printspooler/FusedPrintersProvider;-onCancelLoad-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/ZoomView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/InLineSettingItem;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/DayPickerView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getActiveLinkQualityInfo-()Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/XLargeUi;-onBackKey-()Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/music/PlaylistBrowserActivity;-playRecentlyAdded-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothDevice;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Landroid/media/Ringtone;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasSyncService;-executePostWithTimeout-(Lorg/apache/http/client/HttpClient; Lorg/apache/http/client/methods/HttpPost; I)Lcom/android/exchange/EasResponse;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasCertificateRequestor;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-access$1800-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/support/v4/content/WakefulBroadcastReceiver;-completeWakefulIntent-(Landroid/content/Intent;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/svox/pico/PicoService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserBookmarksPage$2;-onMenuItemClick-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/polarclock/PolarClockSettings;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/view/PagerAdapter;-restoreState-(Landroid/os/Parcelable; Ljava/lang/ClassLoader;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/LinkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationListCallbacks;-commitDestructiveActions-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-refresh-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/server/BluetoothA2dpService;-resumeSink-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/svox/pico/PicoService;-getSecureSettingInt-(Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-endFakeDrag-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/emailcommon/service/ServiceProxy$ProxyConnection$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/EasSyncService;-searchGal-(Landroid/content/Context; J Ljava/lang/String; I)Lcom/android/exchange/provider/GalResult;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/addbookmark/FolderSpinner;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-pause-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetheredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/WifiManager;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/email/activity/setup/AccountSetupIncomingFragment;-onStart-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/filtershow/crop/CropActivity$BitmapIOTask;-doInBackground-([Landroid/graphics/Bitmap;)Ljava/lang/Boolean;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-stopObexServerSession-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/SeekBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-dispatchNormalMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/videoeditor/widgets/MediaLinearLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/CameraControls;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/galaxy4/Galaxy4;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-setExternalState-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/filterpacks/imageproc/CropFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupType;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-makeCFQueryResultMessage-(Lcom/android/internal/telephony/CallForwardInfo; I)Ljava/lang/CharSequence;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-acknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/musicvis/vis3/Visualization3;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/ContextThemeWrapper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/EasService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/datetimepicker/AccessibleLinearLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/CarrierProvisioningUrlsInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/ITextToSpeechService$Stub$Proxy;-synthesizeToFileDescriptor-(Landroid/os/IBinder; Ljava/lang/String; Landroid/os/ParcelFileDescriptor; Landroid/os/Bundle;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSync;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Landroid/support/v4/view/ViewPager;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsImpl;-setInputMethodSettingsCategoryTitle-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/PrintJobConfigActivity$CustomEditText;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/MailboxSelectionActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/View$PerformClick;-run-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/AccessibilityDelegateCompat$AccessibilityDelegateJellyBeanImpl$1;-performAccessibilityAction-(Landroid/view/View; I Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/DateTimeView;-getTimeFormat-()Ljava/text/DateFormat;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/FlipperDream;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/media/effect/SingleFilterEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService$SyncAdapterImpl;-onPerformSync-(Landroid/accounts/Account; Landroid/os/Bundle; Ljava/lang/String; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/TwilightService$LocationHandler;-requestLocationUpdate-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/compose/ComposeActivity$SendOrSaveTask;-incrementRecipientsTimesContacted-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/development/PermissionDetails;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasPing;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/documentsui/SettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/DayPickerView;-findAccessibilityFocus-()Lcom/android/datetimepicker/date/SimpleMonthAdapter$CalendarDay;' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/inputmethod/latin/LatinIME;-retryResetCaches-(Z I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DockObserver;-updateLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/DialogActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/base/RetargetFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/PieListView;-layout-(I I Z F I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/AudioService;-readAudioSettings-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/SearchActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/ConnectivityManager;-getLastTetherError-(Ljava/lang/String;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/MailActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/photos/shims/MediaSetLoader;-urisForSubItems-(Landroid/database/Cursor;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-appNotResponding-(Lcom/android/server/am/ProcessRecord; Lcom/android/server/am/ActivityRecord; Lcom/android/server/am/ActivityRecord; Z Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/browser/Controller;-handleNewIntent-(Landroid/content/Intent;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/latin/InputView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothPan;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/deskclock/AnalogClock;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper$ClockEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/dreams/phototable/PhotoTable;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipSessionGroup;-processTransactionTerminated-(Ljavax/sip/TransactionTerminatedEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService;-getLastInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/SnailAlbum;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/service/EasAutoDiscover;-requestSyncForMailbox-(Landroid/accounts/Account; Ljava/lang/String; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/calculator2/CalculatorDisplay;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onPictureCenter-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/WifiService;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/datetimepicker/time/AmPmCirclesView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/DocumentsApplication;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-handleEnterEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/editors/SwapButton;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/VideoView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/BatteryService;-update-(Landroid/os/BatteryProperties;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetooth$Stub;-getBondState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AlbumSetPage$5;-onActionItemClicked-(Landroid/view/MenuItem;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-determineSetupStepNumber-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/TrafficStats;-startDataProfiling-(Landroid/content/Context;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$Connected;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/content/MutableContextWrapper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/RecognitionManagerService;-initForUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockApplication;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IUiAutomationConnection$Stub$Proxy;-connect-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$7;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getBondState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/BluetoothManagerService;-access$2800-(Lcom/android/server/BluetoothManagerService; I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/MoviePlayer;-<init>-(Landroid/view/View; Lcom/android/gallery3d/app/MovieActivity; Landroid/net/Uri; Landroid/os/Bundle; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-load-(Lcom/android/dreams/phototable/PhotoSource$ImageData; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/BrowserSettings;-getLinkPrefetchEnabled-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothInputDevice;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/widget/LockPatternUtils;-getKeyguardStoredPasswordQuality-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapUtils;-createProfileVCard-(Landroid/content/Context; I [B)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Lcom/google/common/io/CharStreams;-toString-(Lcom/google/common/io/InputSupplier;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/music/MusicUtils;-playAll-(Landroid/content/Context; [J I Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-setMainKeyboardFrame-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/photos/views/GalleryThumbnailView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-addAccount-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/support/v4/app/FragmentTabHost;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/TouchExplorer$SendHoverExitDelayed;-forceSendAndRemove-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ImageView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ShortcutActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-access$300-(Lcom/android/server/am/ActivityManagerService; Ljava/lang/String; I Z Z Z Z Z I Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetoothInputDevice;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-sendSMS-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager$10;-onStatusChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/dreams/phototable/PhotoTableDream;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-updateMessageOnIccEf-(Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/google/SearchBaseUrlHelper$1;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.INTERNET'],
	'Lcom/android/exchange/eas/EasSyncCalendar;-setUpsyncCommands-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; D Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/provider/Browser;-deleteHistoryWhere-(Landroid/content/ContentResolver; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onGestureMoveEvent-(I I J Z Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumPage;-onAlbumModeSelected-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailsync/SyncManager;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/EdgeView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/browse/ConversationViewAdapter$SuperCollapsedBlockItem;-rebindView-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/widget/ActionBarContainer;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/LockdownVpnTracker;-initLocked-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/exchange/eas/EasSyncBase;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/GalleryActivity;-initializeByIntent-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/EmojiCategoryPageIndicatorView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/EventInfoActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/accessibility/AccessibilityUtils;-shouldObscureInput-(Landroid/view/inputmethod/EditorInfo;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_MOCK_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/recurrencepicker/WeekButton;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCall;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/StraightenEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothPbap$Stub;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/LocationManagerService;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/calendar/alerts/AlertActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/app/PlatLogoActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/MediaSource;-setupMediaPlayer-(Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/connectivity/Tethering;-sendTetherStateChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/MailActionBarView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/CameraControls;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/PointerTracker$TimerProxy;-startLongPressTimer-(Lcom/android/inputmethod/keyboard/PointerTracker; I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Landroid/app/backup/FullBackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub;-startDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/ConversationViewFragment;-renderConversation-(Lcom/android/mail/browse/MessageCursor;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/widget/SlidingTab;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/TabHost;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService$BatteryReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/PagerTitleStrip;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-sendVendorSpecificResultCode-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothGatt;-readRemoteRssi-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-sendViewEvent-(Lcom/android/calendar/agenda/AgendaWindowAdapter$AgendaItem; J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/AbstractCalendarActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/category/IconView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextClock;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-queryPhonebook-(Ljava/lang/String; Lcom/android/bluetooth/hfp/AtPhonebook$PhonebookResult;)Z' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/ui/LayoutListener;-onConversationListVisibilityChanged-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService$Session;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/widget/AbsoluteLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/InitAlarmsService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/quicksearchbox/QsbApplicationWrapper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/CameraBackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/compose/ComposeAttachmentTile;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/AlertActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onRadioAvailable-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter$4;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/Invert;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/commands/monkey/Monkey;-getSystemInterfaces-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/gatt/GattService;-writeCharacteristic-(I Ljava/lang/String; I I Ljava/util/UUID; I Ljava/util/UUID; I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/pm/PackageManagerService;-installPackageLI-(Lcom/android/server/pm/PackageManagerService$InstallArgs; Z Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/browse/MessageFooterView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$7;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/p2p/IWifiP2pManager$Stub$Proxy;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/view/IInputMethodManager;-switchToLastInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-handleConnectionFailure-(Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/accounts/AccountManagerService;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/bluetooth/gatt/GattService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/PieMenuButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$4;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/am/ActivityManagerService;-finishInstrumentationLocked-(Lcom/android/server/am/ProcessRecord; I Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/appwidget/AppWidgetHostView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-cancelBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AlbumPage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/CheckedTextView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-doFinish-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/MessageWebView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/app/GalleryActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BackupManagerService$BackupRestoreTask;-operationComplete-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/test/IsolatedContext$MockAccountManager;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/email/service/ImapService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/support/v4/view/ViewPager;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/InstrumentationList;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/ShowActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/ConversationCursor$RefreshTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/Invert;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/IBluetooth;-getRemoteClass-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/EmptyConversationListView;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-getCurrentInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Switch;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getCdmaBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherModeAliveState;-exit-()V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/mail/ui/EllipsizedMultilineTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub$Proxy;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/SnailItem;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/ConnectivityService;-handleConnectivityChange-(I Z)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/am/ActiveServices;-forceStopLocked-(Ljava/lang/String; I Z Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/TwoPaneController;-requestFolderRefresh-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/ActivityChooserView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-disableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapObexServer;-createList-(I I Ljava/lang/String; Ljava/lang/StringBuilder; Ljava/lang/String;)I' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/BatteryService$7;-run-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/tts/compat/CompatTtsService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/timer/CountingTimerView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/setup/SetupActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-updateSystemBarsLw-(Landroid/view/WindowManagerPolicy$WindowState; I I)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/InboundSmsHandler$IdleState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setCurrentPreferredNetworkType-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-onCancelled-(Landroid/filterfw/core/AsyncRunner$RunnerResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AsyncPlayer;-startSound-(Landroid/media/AsyncPlayer$Command;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-sendIntentExit-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothDevice;-createBond-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-operationComplete-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/IConnectivityManager$Stub;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/compose/AttachmentsView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ControllableActivity;-stopDragMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-showDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/updates/CertPinInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/NativeCrashListener;-consumeNativeCrashData-(Ljava/io/FileDescriptor;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-detectCountry-()Landroid/location/Country;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailcommon/service/IEmailService$Stub$Proxy;-autoDiscover-(Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/CalendarSettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/Switch;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-setMonthDisplayed-(Landroid/text/format/Time; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/sip/SipSessionGroup;-processResponse-(Ljavax/sip/ResponseEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$GroupCreatingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView$DeleteKeyOnTouchListener;-startRepeat-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-startScan-(Landroid/os/WorkSource;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/accessibility/CaptioningManager;-getRawUserStyle-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothMap$Stub$Proxy;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/ConversationItemView;-dismiss-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/print/IPrintManager$Stub$Proxy;-cancelPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-setRadios-(Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-loadDeviceAccountsEmailAddresses-()V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/app/ContextImpl;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/AccountManager;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/common/contacts/DataUsageStatUpdater;-updateWithPhoneNumber-(Ljava/util/Collection;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/EllipsizedMultilineTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine;-access$100-(Landroid/net/wifi/WifiStateMachine; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/ScreensaverSettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationCursor$ConversationListener;-onRefreshReady-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/os/IPowerManager$Stub$Proxy;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupBasics;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/GlobalActions$Action;-onPress-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/effect/effects/GrainEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EasServerConnection;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity; J)Lcom/android/exchange/EasResponse;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/BluetoothPbap;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/eas/EasOutboxSync;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/AlertActivity;-dismissFiredAlarms-()V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/RIL;-<init>-(Landroid/content/Context; I I)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-onVisibilityChanged-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/IAudioService$Stub;-startBluetoothSco-(Landroid/os/IBinder; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/deskclock/alarms/AlarmService;-stopCurrentAlarm-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView;-onCodeInput-(I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/AccountManager$9;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/browser/Controller;-removeTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/camera/ui/RenderOverlay;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/worldclock/CitiesActivity$CityAdapter;-<init>-(Lcom/android/deskclock/worldclock/CitiesActivity; Landroid/content/Context; Landroid/view/LayoutInflater;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-computeSharingIntent-(Lcom/android/gallery3d/util/ThreadPool$JobContext; I)Landroid/content/Intent;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/LatinIME;-onCodeInput-(I I I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService$7;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/UsimDataDownloadHandler;-handleDataDownload-(Lcom/android/internal/telephony/gsm/SmsMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-exitCabMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ConversationSelectionSet;-removeAll-(Ljava/util/Collection;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getNetworkInfo-(I)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub$Proxy;-setCurrentSpellChecker-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothAdapter;-setName-(Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/TwoLineListItem;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertService;-processMessage-(Landroid/os/Message;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/camera/ui/FilmstripBottomControls;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/InputStreamSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/support/v4/content/WakefulBroadcastReceiver;-startWakefulService-(Landroid/content/Context; Landroid/content/Intent;)Landroid/content/ComponentName;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractActivityController;-onFolderSelected-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/AbstractActivityController;-commitDestructiveActions-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/compose/AutoSendActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/grass/Grass;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/IAudioService$Stub$Proxy;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/Controller$5;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/email/service/EasTestAuthenticatorService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/NativeActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMSI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipWakeLock;-release-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/IntentHandler$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-access$1100-(Lcom/android/server/accessibility/ScreenMagnifier;)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasOptions;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/AddNewBookmark;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Browser;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/base/FrameStore;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-onResume-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/data/DataManager;-getDefaultSetOf-(Lcom/android/gallery3d/data/Path;)Lcom/android/gallery3d/data/Path;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/videoeditor/OverlayTitleTemplatePicker;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/IRecognitionService;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/deskclock/timer/CountingTimerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-access$900-(Lcom/android/bluetooth/map/BluetoothMapService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/FolderController;-requestFolderRefresh-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/widgets/TransitionView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneLayout$2;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/TrackBrowserActivity;-access$300-(Lcom/android/music/TrackBrowserActivity; Lcom/android/music/TrackBrowserActivity$TrackListAdapter$TrackQueryHandler; Ljava/lang/String; Z)Landroid/database/Cursor;' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-next-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-pollCallsWhenSafe-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hdp/HealthService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/location/ILocationManager$Stub;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/MailActionBarView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseAccountActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/mail/store/ServiceStore;-checkSettings-()Landroid/os/Bundle;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/camera/CameraModule;-onPauseAfterSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/videoeditor/KenBurnsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/photos/FullscreenViewer;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/ChooserActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/NavScreen;-switchToTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-onRemoteDisconnect-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/FaceUnlockView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/NullFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/utility/CalendarUtilities;-createMessageForEventId-(Landroid/content/Context; J I Ljava/lang/String; Lcom/android/emailcommon/provider/Account;)Lcom/android/emailcommon/provider/EmailContent$Message;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/mail/ui/SecureConversationViewFragment;-setExtraUserVisibleHint-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothMap$Stub$Proxy;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/HandleApiCalls;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-next-(Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationViewFragment;-onNewMessageBarClick-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/app/TrimVideo;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/AlbumBrowserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofenceRemove-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/MessageHeaderView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaRouter$RouteGroup;-requestSetVolume-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/camera/ui/RenderOverlay$RenderView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockContentResolver;-startSync-(Landroid/net/Uri; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/emailcommon/service/PolicyServiceProxy;-setAccountPolicy-(Landroid/content/Context; J Lcom/android/emailcommon/provider/Policy; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/RIL;-setFacilityLockForApp-(Ljava/lang/String; Z Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DigitalClock;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/TimeIntervalPopup;-access$000-(Lcom/android/camera/ui/TimeIntervalPopup;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/BadBehaviorActivity$BadService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getCallForwardingOption-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onUpdateBatchInput-(Lcom/android/inputmethod/latin/InputPointers;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/CalendarViewAdapter;-refresh-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/RIL;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/MultiInputStream;-skip-(J)J' : ['android.permission.INTERNET'],
	'Lcom/android/server/WallpaperManagerService;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/soundrecorder/SoundRecorder;-updateUi-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/AliasActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-sendCDMAFeatureCode-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/soundrecorder/SoundRecorder;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-access$200-(Lcom/android/server/location/ComprehensiveCountryDetector;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-access$000-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/adapter/ContactsSyncParser$ContactOperations;-delete-(J)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/service/dreams/DreamService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/photo/PhotoViewPager;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/PieMenu$PieView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onDisconnect-(Lcom/android/internal/telephony/Connection$DisconnectCause;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/OverlayView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/RuntimeInit;-wtf-(Ljava/lang/String; Ljava/lang/Throwable;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-switchToLastInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Space;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ImageSwitcher;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/RichInputMethodManager;-hasMultipleEnabledSubtypes-(Z Ljava/util/List;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/RecognitionManagerService$MyPackageMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/tinyplanet/TinyPlanetPreview;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-turnOnUpstreamMobileConnection-(I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/email/activity/setup/AccountSecurity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/research/Uploader;-hasUploadingPermission-()Z' : ['android.permission.INTERNET'],
	'Landroid/widget/TextClock;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/LogViewer;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/photo/adapters/BaseFragmentPagerAdapter;-instantiateItem-(Landroid/view/ViewGroup; I)Ljava/lang/Object;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/NetworkStateTracker;-startMonitoring-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/settings/SettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-determineSetupStepNumberFromLauncher-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DayView;-updateEventDetails-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/mail/browse/MultiAdapterSpinner;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService;-createStreamStates-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/AlertController$RecycleListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/CheckedTextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_MOCK_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-saveCurrentInputMethodAndSubtypeToHistory-(Ljava/lang/String; Landroid/view/inputmethod/InputMethodSubtype;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/google/common/io/ByteStreams$4;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-isActiveNetworkMetered-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/BackupManagerService$PerformBackupTask;-invokeAgentForBackup-(Ljava/lang/String; Landroid/app/IBackupAgent; Lcom/android/internal/backup/IBackupTransport;)I' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-getMainChannelFd-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Settings$Secure;-getInt-(Landroid/content/ContentResolver; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService$InputMethodAndSubtypeListManager;-getSortedInputMethodAndSubtypeList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/widget/BookmarkWidgetConfigure;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/gatt/GattService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment$4;-run-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/data/ComboSource;-mapMediaItems-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/GridView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/PrintManager;-addPrintJobStateChangeListener-(Landroid/print/PrintManager$PrintJobStateChangeListener;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$TetherMasterUtilState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDataSetupCompleteError-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-setGsmBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/MediaSaveService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/print/UserState;-readEnabledPrintServicesLocked-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/SystemBackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/support/v4/view/ViewPager;-setCurrentItemInternal-(I Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/ui/TwoPaneController;-enableOrDisableCab-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/datetimepicker/AccessibleLinearLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupBasics;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-access$200-(Lcom/android/deskclock/alarms/AlarmActivity;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/print/PrintHelper$PrintHelperVersionImpl;-printBitmap-(Ljava/lang/String; Landroid/graphics/Bitmap;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/grass/GrassRS;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/SuggestionsView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-setExternalState-(Lcom/android/internal/telephony/IccCardConstants$State; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/TimerRingService$2;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-dispatchMediaKeyWithWakeLockToAudioService-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/ImapService;-processPendingUploadsSynchronous-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; [Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/Application;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/event/EditEventView$TimeListener;-onTimeSet-(Lcom/android/datetimepicker/time/RadialPickerLayout; I I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getBondedDevices-()[Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiStateMachine$UntetheringState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/AbstractActivityController;-handleBackPress-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/FormattedDateBuilder;-formatShortDate-(J)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/BackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-forceStopPackageLocked-(Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onVoiceCallStarted-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/CenteredLinearLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/pipeline/ProcessingService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/LatinIME;-onCurrentInputMethodSubtypeChanged-(Landroid/view/inputmethod/InputMethodSubtype;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/TextProgressBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/ExchangeService;-runAwake-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/phasebeam/PhaseBeam;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-getBatchedScanResults-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/calendar/AllInOneActivity;-setTitleInActionBar-(Lcom/android/calendar/CalendarController$EventInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/email/provider/AccountReconciler;-getAllAmAccounts-(Landroid/content/Context;)Ljava/util/List;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/noisefield/NoiseFieldWallpaper$RenderScriptEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/IBluetoothMap;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/DownloadUtils;-requestDownload-(Lcom/android/gallery3d/util/ThreadPool$JobContext; Ljava/net/URL; Ljava/io/File;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/quicksearchbox/util/HttpHelper;-post-(Lcom/android/quicksearchbox/util/HttpHelper$PostRequest;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-writeDescriptor-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I I [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-resetKeyboardStateToAlphabet-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/PieControl;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-startUsingNetworkFeature-(I Ljava/lang/String; Landroid/os/IBinder;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-<init>-(Landroid/view/View; Lcom/android/gallery3d/app/MovieActivity; Landroid/net/Uri; Landroid/os/Bundle; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/provider/ExchangeDirectoryProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.GET_ACCOUNTS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MovieActivity;-onPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/format/DateFormat;-getDateFormatOrder-(Landroid/content/Context;)[C' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/power/PowerManagerService;-setScreenBrightnessOverrideFromWindowManagerInternal-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumPicker;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/widget/TextTime;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TabWidget;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/SwipeableListView;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/DownloadOverMeteredDialog;-onClickAllow-(Landroid/view/View;)V' : ['android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-removePrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-onScrubbingStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/LeaveBehindItem;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/keychain/KeyChainActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ui/GLRootView;-onDrawFrame-(Ljavax/microedition/khronos/opengles/GL10;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/ISyncAdapter;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/vcard/VCardEntry$EntryElementIterator;-onElement-(Lcom/android/vcard/VCardEntry$EntryElement;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processNoiceReductionEvent-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/service/textservice/SpellCheckerService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoTable;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupType;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handleRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/FrameLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiService;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/bluetooth/IBluetooth;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/SlideshowPage$ShuffleSource;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cat/IconLoader;-readIconData-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService$1;-updateFolderList-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-<init>-(J)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/dictionarypack/DownloadOverMeteredDialog;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/AbsActionBarView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSyncBase;-requestSyncForMailbox-(Landroid/accounts/Account; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-access$8400-(Landroid/media/AudioService; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/ViewRootImpl;-setView-(Landroid/view/View; Landroid/view/WindowManager$LayoutParams; Landroid/view/View;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/quicksearchbox/ui/SearchActivityViewSinglePane;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardEntryConstructor;-onEntryEnded-()V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/content/SyncManager;-readDataConnectionState-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/google/GoogleSearch;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/RenderOverlay;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DatePicker;-access$800-(Landroid/widget/DatePicker;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/NullFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService$1;-loadAttachment-(Lcom/android/emailcommon/service/IEmailServiceCallback; J J Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/media/Media;-onRun-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/gatt/GattService;-addCharacteristic-(I Ljava/util/UUID; I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/AnimatedAdapter;-commitLeaveBehindItems-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email2/ui/CreateShortcutActivityEmail;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/MailActionBarView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/PhotoView;-switchToImage-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/TabActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService$10;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/media/Ringtone;-getTitle-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-getAllMessagesFromIccEf-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/SMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsUsageMonitor; Lcom/android/internal/telephony/ImsSMSDispatcher;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/BluetoothHeadset;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/SpeechRecognizer;-handleCancelMessage-()V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/gallery3d/data/ComboAlbum;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/effect/effects/SaturateEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/DeleteItems;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-supplyPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/providers/downloads/ui/DownloadItem;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/WebVttRenderingWidget;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/phasebeam/PhaseBeamView;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/BalloonHint$BalloonView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/datetimepicker/date/SimpleMonthAdapter;-onDayClick-(Lcom/android/datetimepicker/date/SimpleMonthView; Lcom/android/datetimepicker/date/SimpleMonthAdapter$CalendarDay;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/gallery3d/ui/PhotoView;-snapback-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-onActionTap-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/Calculator;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/WifiStateMachine;-sendNetworkStateChangeBroadcast-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/AppHwPref;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/NavTabScroller;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/galaxy/GalaxyWallpaper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BluetoothManagerService;-storeNameAndAddress-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EventLocationAdapter$LocationFilter$1;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/latin/BackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/musicvis/vis3/Visualization3;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapObexServer;-onGet-(Ljavax/obex/Operation;)I' : ['android.permission.READ_CONTACTS', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone; Landroid/telephony/CellInfo;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/requestsync/RequestSync;-main-([Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-configHciSnoopLog-(Z)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/GLView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calculator2/CalculatorViewPager;-setPageTransformer-(Z Landroid/support/v4/view/ViewPager$PageTransformer;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-startRepeatKey-(Lcom/android/inputmethod/keyboard/Key;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Landroid/net/wifi/SupplicantStateTracker$InactiveState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-setReport-(Landroid/bluetooth/BluetoothDevice; B Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Ljava/net/ServerSocket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/gadget/WidgetConfigure;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS', 'com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.USE_CREDENTIALS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.WRITE_SETTINGS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/IMediaPlaybackService;-next-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/UserState;-readPrintServicesFromSettingLocked-(Ljava/lang/String; Ljava/util/Set;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountAdapter;-doSaveAction-()V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/power/PowerManagerService;-goToSleepNoUpdateLocked-(J I)Z' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/ex/editstyledtext/EditStyledText;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/svox/pico/DownloadVoiceData;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/FilmstripPage;-setCurrentPhotoByIntent-(Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtErrorActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-getUserRotationMode-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onError-(Landroid/net/sip/SipAudioCall; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowSession$Stub;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/Connectivity$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ListActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage;-access$400-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasSyncBase;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/videoeditor/widgets/OverlayView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService$Stub;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/browser/Controller;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/base/FrameStore;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/CameraSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/ListMenuItemView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothServerSocket;-<init>-(I Z Z Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/WordListPreference;-access$700-(Lcom/android/inputmethod/dictionarypack/WordListPreference;)V' : ['android.permission.INTERNET'],
	'Landroid/view/IApplicationToken;-keyDispatchingTimedOut-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/quicksearchbox/DialogActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/SecureConversationViewFragment;-markUnread-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/LockPatternView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/SecurityPolicy;-syncAccount-(Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcActivatingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/latin/ContactsBinaryDictionary;-access$700-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageDraw;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/service/EasMeetingResponder;-sendHttpClientPost-(Ljava/lang/String; Lorg/apache/http/HttpEntity; J)Lcom/android/exchange/EasResponse;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-setEngineCapabilities-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView$1;-onSuggestionSelected-(I Lcom/android/inputmethod/latin/SuggestedWords$SuggestedWordInfo;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/support/v4/widget/ContentLoadingProgressBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-setDiscoverableTimeout-(I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/ComboViewActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/VideoEditorActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-notifyIndexChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/email/activity/setup/AccountServerBaseFragment;-setCallback-(Lcom/android/email/activity/setup/AccountServerBaseFragment$Callback;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendImsGsmSms-(Ljava/lang/String; Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment$LoadAccountTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-updateSurface-(Z Z Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/deskclock/HandleApiCalls;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/AccountManager$13;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService;-prev-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameFetch;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/widget/CalendarAppWidgetModel;-populateEventInfo-(J Z J J I I Ljava/lang/String; Ljava/lang/String; I I)Lcom/android/calendar/widget/CalendarAppWidgetModel$EventInfo;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/music/TouchInterceptor;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/service/textservice/SpellCheckerService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/ResolverActivity$1;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHeadset;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-updateSystemUiVisibilityLw-()I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ShortcutNameActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextClock;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/VideoView$7;-surfaceCreated-(Landroid/view/SurfaceHolder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/EllipsizedMultilineTextView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/SmsApplication;-getDefaultMmsApplication-(Landroid/content/Context; Z)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/nexus/NexusWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-dispatchTouchEvent-(Landroid/view/MotionEvent; I I Lcom/android/gallery3d/ui/GLView; Z)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/exchange/adapter/CalendarSyncParser;-addEvent-(Lcom/android/exchange/adapter/CalendarSyncParser$CalendarOperations; Ljava/lang/String; Z)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/camera/CameraActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/BorderView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/TimelineRelativeLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaListView$2;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView$2;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService$Stub$Proxy;-adjustSuggestedStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/DayView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/info/HistogramView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/PhotoModule;-access$3800-(Lcom/android/camera/PhotoModule;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/PanoProgressBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/event/AttendeesView$PresenceQueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/gatt/GattService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/PointerLocation;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/MultiStateButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter$QueryHandler;-processNewCursor-(Lcom/android/calendar/agenda/AgendaWindowAdapter$QuerySpec; Landroid/database/Cursor;)I' : ['android.permission.READ_CALENDAR'],
	'Landroid/accessibilityservice/AccessibilityService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionaryAddWordContents;-hasWord-(Ljava/lang/String; Landroid/content/Context;)Z' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/server/wifi/WifiController;-registerForWifiSleepPolicyChange-(Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageCurves;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationItemView;-toggleSelectedStateOrBeginDrag-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/AccountsTester;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-getBatchedScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/ex/photo/PhotoViewActivity;-onLoadFinished-(Landroid/support/v4/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionarySettings;-deleteWord-(Ljava/lang/String; Ljava/lang/String; Landroid/content/ContentResolver;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/exchange/eas/EasSyncBase;-makeRequest-()Lorg/apache/http/client/methods/HttpUriRequest;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/provider/ContactsContract$RawContacts$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/PackageManagerBackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/SnoozeAlarmsService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/soundrecorder/VUMeter;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavigationBarPhone;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofenceRemoveStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-onResume-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/view/accessibility/CaptioningManager;-isEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/widget/ActionableToastBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/print/PrintHelper$PrintHelperKitkatImpl;-printBitmap-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/DatePicker;-setMaxDate-(J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/tts/compat/CompatTtsService;-onCreate-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/polarclock/PolarClockSettings;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AddNewBookmark;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/Connectivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/input/InputManagerService;-notifyANR-(Lcom/android/server/input/InputApplicationHandle; Lcom/android/server/input/InputWindowHandle; Ljava/lang/String;)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/MailActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/photo/PhotoViewPager;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/SwipeableConversationItemView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController;-refreshConversationList-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/PhotoPage;-wantBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/ExpandableListView;-handleItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper$RenderScriptEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/musicvis/vis1/Visualization1;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/http/RequestQueue;-access$300-(Landroid/net/http/RequestQueue;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ConnectivityService;-getActiveNetworkQuotaInfo-()Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/vcard/VCardEntry$NicknameData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/UrlHandler$RLZTask;-onPostExecute-(Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/AudioManager;-setStreamVolume-(I I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-getCurrentCalls-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Landroid/media/MediaFocusControl;-access$2200-(Landroid/media/MediaFocusControl;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EmailBroadcastProcessorService;-onAppUpgrade-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.USE_CREDENTIALS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_CALENDAR', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/calendar/event/EditEventFragment;-setModelIfDone-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/eas/EasSyncCalendar;-sendMessage-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/filterfw/core/Filter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/os/IPowerManager$Stub;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/VideoUtils;-startTrim-(Ljava/io/File; Ljava/io/File; I I)V' : ['android.permission.INTERNET'],
	'Lcom/android/email/mail/store/Pop3Store;-autoDiscover-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/ContentService;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/uicc/UiccController;-onGetIccCardStatusDone-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/VideoEditorActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-deleteMessageMms-(J)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/FilmstripPage;-wantBars-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/AbstractActivityController;-hideWaitForInitialization-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneProxy;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ingest/ui/MtpImageView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/WidgetConfigure;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/dreams/phototable/PhotoTableDream;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/DeskClock;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/MediaFocusControl;-dispatchMediaKeyEventUnderWakelock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-deleteFromRawTable-(Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/mail/ui/AccountItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController;-cancel-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/ScanningProgress;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/inputmethodservice/ExtractEditText;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/StackView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-onScroll-(Landroid/widget/AbsListView; I I I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/support/v4/net/ConnectivityManagerCompat$JellyBeanConnectivityManagerCompatImpl;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onTrySetupData-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationContainer;-onDataSetChanged-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorCompareView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BookmarkItem;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-updateShiftState-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/btservice/ProfileService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/BrowserSettings;-updateConnectionType-()V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/IntRangeManager;-disableRange-(I I Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController;-updateBatteryWorkSource-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkPolicyManager;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/ActionableToastBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/WiredAccessoryManager$WiredAccessoryObserver;-init-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-access$600-(Lcom/android/calendar/month/MonthByWeekFragment;)Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationPager;-arrowScroll-(I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-onResume-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/UserDictionary$Words;-addWord-(Landroid/content/Context; Ljava/lang/String; I I)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/vcard/VCardParser;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/calendar/alerts/SnoozeAlarmsService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/GalleryActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/pan/PanService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/NavTabScroller$OnRemoveListener;-onRemovePosition-(I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/sip/ISipService$Stub;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/calendar/CalendarBackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hdp/HealthService;-callHealthChannelCallback-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/BluetoothDevice; I I Landroid/os/ParcelFileDescriptor; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/renderscript/RSTextureView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/HapticFeedbackController;-checkGlobalSetting-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-getGsmBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintJobConfigActivity$CustomEditText;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/worldclock/WorldClockAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/provider/BrowserProvider2;-isValidParent-(Ljava/lang/String; Ljava/lang/String; J)Z' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/app/SearchDialog$SearchBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/ui/PhotoView;-access$3500-(Lcom/android/gallery3d/ui/PhotoView; F F)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageCurves;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController$AccountLoads;-onLoadFinished-(Landroid/content/Loader; Lcom/android/mail/content/ObjectCursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-installIfNeverRequested-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; Z)V' : ['android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/ConversationListCopy;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ConversationViewHeader;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/location/LocationManager;-getProvider-(Ljava/lang/String;)Landroid/location/LocationProvider;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/MediaPlaybackActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/accessibility/CaptioningManager;-access$200-(Landroid/view/accessibility/CaptioningManager;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine;-access$3900-(Landroid/net/wifi/WifiStateMachine; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/downloads/ui/TrampolineActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-stayAwake-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/ViewPager;-populate-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/support/v4/widget/ExploreByTouchHelper;-performAccessibilityAction-(Landroid/view/View; I Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/NestedFolderTeaserView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/Exchange;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-invokeEnterMethods-(I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/development/RunningProcesses;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity$SettingsPoolingHandler;-startPollingImeSettings-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/AttachmentDownloadService$ServiceCallback;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/wallpaper/fall/Fall;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-onContentDirty-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-startObexServerSession-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-naturalNext-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setAndBroadcastNetworkSetTime-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AnimatedAdapter$2;-onListItemsRemoved-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiManager;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/NsdService;-sendNsdStateChangeBroadcast-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDisconnectDone-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub;-updateMessageOnIccEf-(Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/gallery3d/ui/PhotoView$Listener;-onUndoDeleteImage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/WebVttRenderingWidget$RegionLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/ShortcutActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/time/TimePickerDialog$KeyboardListener;-onKey-(Landroid/view/View; I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/accounts/AccountManager$8;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupAllConnections-(Lcom/android/internal/telephony/gsm/GsmCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/StatusBarManagerService$7;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/camera/ui/ModuleSwitcher;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/crop/CropView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-getConnectedHealthDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/AbstractActivityController;-getBatchAction-(I)Lcom/android/mail/ui/DestructiveAction;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/view/View$AccessibilityDelegate;-performAccessibilityAction-(Landroid/view/View; I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TelephonyRegistry;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/LinkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-updatePowerStateLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth$Stub;-setPasskey-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/os/SystemVibrator;-vibrate-([J I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/agenda/AgendaFragment;-onResume-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/widget/CompoundButton;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/DayView$GotoBroadcaster;-onAnimationEnd-(Landroid/view/animation/Animation;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/emailcommon/service/IPolicyService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/AudioPreview;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/updates/ConfigUpdateInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hid/HidService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/server/content/ContentService;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/deskclock/TimerSetupView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationViewAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_CONTACTS'],
	'Landroid/widget/ListView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/UploaderService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ShutdownActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/backup/LocalTransportService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Chronometer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/TimelineHorizontalScrollView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pan/PanService;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/display/DisplayManagerService;-handleDisplayDeviceRemovedLocked-(Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/recurrencepicker/LinearLayoutWithMaxWidth;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmNotifications;-showAlarmNotification-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/spellcheck/SpellCheckerSettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/p2p/IWifiP2pManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/wifi/WifiApConfigStore$DefaultState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/widget/AdapterViewFlipper;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/EventViewer;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/camera/PhotoMenu;-onListPrefChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/tts/ITextToSpeechService;-getClientDefaultLanguage-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/state/StateView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothA2dp;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/noisefield/NoiseFieldView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-systemRunning-(Lcom/android/server/StatusBarManagerService;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothPbap;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothPbap$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/IntentService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/settings/DebugSettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/app/FragmentActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageShow;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-getEnabledInputMethodSubtypeList-(Ljava/lang/String; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumSetPage$1;-onTouch-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-onTabChanged-(Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/Activity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-failPendingMessages-()V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/SearchActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/LockPatternUtils;-getFallbackAppWidgetId-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/PreviewSurfaceView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/time/AmPmCirclesView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/keyboard/MainKeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/EasSyncService;-setupServiceForAccount-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account;)Lcom/android/exchange/EasSyncService;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/AnimatedAdapter;-swipeDelete-(Ljava/util/Collection; Lcom/android/mail/ui/SwipeableListView$ListItemsRemovedListener;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/AnalogClock;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/wallpaper/grass/Grass;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/FilmstripBottomControls;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/ConnectivityManager;-getNetworkPreference-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/wallpaper/galaxy/GalaxyView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-beginServiceDeclaration-(I I I I Landroid/os/ParcelUuid; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/AbstractCalendarActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarSettingsActivity;-onBuildHeaders-(Ljava/util/List;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/view/ContextThemeWrapper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService$5;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/provider/BrowserProvider2;-deleteBookmarks-(Ljava/lang/String; [Ljava/lang/String; Z)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/timezonepicker/TimeZoneResultAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AlarmClockFragment;-access$100-(Lcom/android/deskclock/AlarmClockFragment;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationListFragment;-showList-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/ui/FramedTextButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-access$2600-(Lcom/android/calendar/agenda/AgendaWindowAdapter; Lcom/android/calendar/agenda/AgendaWindowAdapter$QuerySpec;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/net/sip/ISipService;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/musicvis/RenderScriptWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/server/content/SyncStorageEngine;-newTestInstance-(Landroid/content/Context;)Lcom/android/server/content/SyncStorageEngine;' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/research/UploaderService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/AlertReceiver;-getAttendeesCursor-(Landroid/content/Context; J)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionaryAddWordFragment;-onPause-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/crop/CropActivity$BitmapIOTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/adapter/ContactsSyncParser;-commit-()V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$1300-(Lcom/android/server/net/NetworkPolicyManagerService;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/widget/ActionBarOverlayLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-handleMsgListChangesSms-()V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-notifyApnIdDisconnected-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/VibratorService$3;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getGsmBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothA2dp;-isEnabled-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/activity/setup/AccountSetupIncoming;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoTable$LoadNaturalSiblingTask;-doInBackground-([Landroid/view/View;)Landroid/view/View;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupAllConnections-(Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailsync/SyncManager;-requestSync-(Lcom/android/emailcommon/provider/Mailbox; I Lcom/android/emailsync/Request;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/AbstractActivityController$LoadFinishedCallback;-onLoadFinished-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accounts/AccountManager$4;-startTask-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorRectView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$StartingState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/updates/CertPinInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/UI;-setUseQuickControls-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/CalendarEventModel;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/gatt/GattService;-refreshDevice-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/view/IApplicationToken$Stub$Proxy;-keyDispatchingTimedOut-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/AddBookmarkPage$1;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/support/v4/view/AccessibilityDelegateCompatJellyBean$AccessibilityDelegateBridgeJellyBean;-performAccessibilityAction-(Landroid/view/View; I Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/calendar/month/SimpleWeekView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/noisefield/NoiseField;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/Ringtone;-playFallbackRingtone-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/WidgetUtils;-createWidgetBitmap-(Lcom/android/gallery3d/data/MediaItem;)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/wallpaper/grass/GrassRS$TimezoneTracker;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/mail/store/ImapFolder;-decodeBody-(Ljava/io/InputStream; Ljava/lang/String; I Lcom/android/emailcommon/mail/Folder$MessageRetrievalListener;)Lcom/android/emailcommon/mail/Body;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/power/PowerManagerService;-handleScreenOnBlockerReleased-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-enumerateTotalMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/ObjectSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/BluetoothManager;-getConnectedDevices-(I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/DayView;-<init>-(Landroid/content/Context; Lcom/android/calendar/CalendarController; Landroid/widget/ViewSwitcher; Lcom/android/calendar/EventLoader; I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-disableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/ConnectivityService$CheckMp;-printNetworkInfo-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/NestedFolderTeaserView$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/AudioService;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/widget/EditText;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hdp/HealthService;-access$1700-(Lcom/android/bluetooth/hdp/HealthService; Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/BluetoothDevice; I I Landroid/os/ParcelFileDescriptor; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiManager;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/WifiManager;-updateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/packageinstaller/InstallFlowAnalytics;-setFlowFinished-(B)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/mail/ui/SearchMailActionBarView;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/base/FrameBranch;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/opp/BluetoothOppLiveFolder;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IccCard;-supplyPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/view/IWindowManager;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/inputmethod/research/ResearchLogger$3;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/inputmethod/research/FeedbackFragment;-onClick-(Landroid/view/View;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/mail/compose/EnterSubject;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-updateCurrentPhoto-(Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/musicvis/vis5/Visualization5;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/MessageInviteView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/ex/photo/PhotoViewPager;-setCurrentItem-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$CaptivePortalCheckState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageMirror;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgentHelper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/BaseUi$FullscreenHolder;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/TintEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/IContentService$Stub;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver$1;-onChange-(Z)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionaryAddWordContents;-apply-(Landroid/content/Context; Landroid/os/Bundle;)I' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/exchange/SettingsRedirector;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/ConversationListFragment;-requestListRefresh-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub;-getActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/EmailConnectivityManager;-getActiveNetworkType-(Landroid/net/ConnectivityManager;)I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/service/Pop3Service;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TableRow;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/MultiAutoCompleteTextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-isSubtypeSelected-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-explicitCallTransfer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/research/FeedbackActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/renderscript/RSSurfaceView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-drawEvents-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/internal/widget/FaceUnlockView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/LockdownVpnTracker;-init-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/ConversationViewFragment;-onLayoutChange-(Landroid/view/View; I I I I I I I I)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/widget/LockPatternView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TwoLineListItem;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-setDiscoverableTimeout-(I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/support/v7/widget/Space;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/SettingInjectorService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/webkit/WebView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/TrackBrowserActivity$NowPlayingCursor;-requery-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/gatt/GattService;-registerForNotification-(I Ljava/lang/String; I I Ljava/util/UUID; I Ljava/util/UUID; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/ex/photo/loaders/PhotoPagerLoader;-onLoadInBackground-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/music/MusicUtils;-playAll-(Landroid/content/Context; Landroid/database/Cursor; I Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothGatt;-refresh-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hdp/HealthService;-getHealthDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-disconnectAudio-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-separate-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ErrorConsoleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/EmailProvider;-pickSentFolder-(Landroid/net/Uri;)I' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/widget/VideoView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/wifi/WifiController$ScanOnlyLockHeldState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/widget/SearchView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompatJellyBean$AccessibilityNodeInfoBridge;-performAction-(I I Landroid/os/Bundle;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayManagerService;-unblankAllDisplaysFromPowerManager-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/textservice/TextServicesManager;-setSpellCheckerSubtype-(Landroid/view/textservice/SpellCheckerSubtype;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-switchWaitingOrHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getLinkProperties-(I)Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/CalendarSettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AnalogClock;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage$4;-onSingleTapUp-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-revertToNitzTime-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DeleteEventHelper;-delete-(J J J I Ljava/lang/Runnable;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/camera/ui/TimeIntervalPopup;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/photo/PhotoViewPager;-smoothScrollTo-(I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPukForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/PrintManager;-getPrintJobs-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CallTracker;-handleRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;-access$000-(Lcom/android/inputmethod/dictionarypack/DictionarySettingsFragment;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/ex/photo/PhotoViewPager;-populate-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryFacilityLock-(Ljava/lang/String; Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/NestedFolderTeaserView$3;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageVignette;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameFetch;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-access$000-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle; Landroid/content/ContentProviderClient; Landroid/content/SyncResult;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/IBluetoothInputDevice;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity$BluetoothTransferContentObserver;-onChange-(Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-switchHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/dreams/phototable/BummerView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/EmojiCategoryPageIndicatorView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoTableDreamSettings;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/EmlTempFileDeletionService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/IInputMethodManager;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AnimatedAdapter;-setUndo-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/IAlarmManager$Stub$Proxy;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/documentsui/DirectoryView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/WordListPreference;-enableDict-()V' : ['android.permission.INTERNET'],
	'Lcom/android/server/BluetoothManagerService;-access$100-(Lcom/android/server/BluetoothManagerService; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/LockdownVpnTracker;-handleStateChangedLocked-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-sync-(Landroid/content/Context; J Landroid/os/Bundle; Landroid/content/SyncResult; Z I)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/SupplicantStateTracker;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-onGetContent-(Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/AndroidClockTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/AudioAndHapticFeedbackManager;-performHapticAndAudioFeedback-(I Landroid/view/View;)V' : ['android.permission.VIBRATE'],
	'Landroid/service/wallpaper/WallpaperService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasSyncContacts;-setUpsyncCommands-(Lcom/android/exchange/adapter/Serializer; Landroid/content/ContentResolver; Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Mailbox; D)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState$SwitchActions;-setAlphabetManualShiftedKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/BondStateMachine;-access$1000-(Lcom/android/bluetooth/btservice/BondStateMachine; Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/videoeditor/OverlayTitleEditor;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothPbap$Stub$Proxy;-isConnected-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Telephony$Sms$Inbox;-addMessage-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long; Z)Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Browser;-canClearHistory-(Landroid/content/ContentResolver;)Z' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/LocationManagerService;-removeProviderLocked-(Lcom/android/server/location/LocationProviderInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiService;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ActivityManagerNative;-goingToSleep-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/email/activity/ComposeActivityEmail;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/GrainFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-access$600-(Lcom/android/internal/telephony/RIL; I Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/CreateShortcutActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/nexus/NexusWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackActivity;-toggleShuffle-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/SearchMailActionBarView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter;-resume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/packageinstaller/InstallAppProgress;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserSettings;-getDefaultPreloadSetting-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/AllInOneActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/mail/ui/OnePaneController;-exitCabMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/videoeditor/widgets/PlayheadView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage$6;-onLoadingFinished-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/SwipeableListView$ListItemsRemovedListener;-onListItemsRemoved-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/view/BookmarkExpandableView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager;-getEnabledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventFragment;-startQuery-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/view/View;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/SwipeableListView;-onScrollStateChanged-(Landroid/widget/AbsListView; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothGattCallback$Stub$Proxy;-onDescriptorWrite-(Ljava/lang/String; I I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/power/PowerManagerService;-updateIsPoweredLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ViewMode;-enterAdMode-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/media/MediaScanner;-scanMtpFile-(Ljava/lang/String; Ljava/lang/String; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailcommon/utility/EmailAsyncTask$InnerTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/power/ShutdownThread;-rebootOrShutdown-(Z Ljava/lang/String;)V' : ['android.permission.VIBRATE'],
	'Landroid/filterfw/core/AsyncRunner;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/AudioPreview;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/AccountService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/email2/ui/CreateShortcutActivityEmail;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/MailActionBarView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-notifyANR-(Lcom/android/server/input/InputApplicationHandle; Lcom/android/server/input/InputWindowHandle; Ljava/lang/String;)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/BluetoothInputDevice;-getProtocolMode-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/Button;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/opengl/GLSurfaceView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-<init>-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-sendData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/internal/telephony/PhoneBase;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DateTimeView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/ISipService$Stub;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/NullFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RemoteViewsService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/app/NetInitiatedActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/VideoEditorActivity$23;-onStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/ContactStatusLoader;-loadInBackground-()Lcom/android/email/activity/ContactStatusLoader$Result;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/wifi/WifiService;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/vcard/VCardParser_V21;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/browser/widget/BookmarkThumbnailWidgetService$BookmarkFactory;-loadBookmarks-()V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-snooze-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-performLayoutAndPlaceSurfacesLockedLoop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipManager;-open-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/mail/ui/NestedFolderTeaserView$2;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/text/StringLogger;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/UnderdevelopedSettings;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/effect/effects/NegativeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/InLineSettingMenu;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/QueryBrowserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DayView;-doSingleTapUp-(Landroid/view/MotionEvent;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceOperationStatus-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$LabelView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/LocationBlacklist;-init-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/quicksearchbox/QueryTask;-run-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/browse/ConversationPagerAdapter;-setPrimaryItem-(Landroid/view/ViewGroup; I Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/service/notification/NotificationListenerService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/text/StringLogger;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/browse/MessageInviteView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email2/ui/CreateShortcutActivityEmail;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiApConfigStore$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/IBackupAgent$Stub;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/browser/view/PieMenu;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager$MulticastLock;-acquire-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-hasContentChanged-()Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/videoeditor/NoSearchActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/support/v4/view/ViewPager;-setOffscreenPageLimit-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/speechrecorder/SpeechRecorderActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/google/common/io/Files;-readLines-(Ljava/io/File; Ljava/nio/charset/Charset;)Ljava/util/List;' : ['android.permission.INTERNET'],
	'Lcom/android/browser/PreloadRequestReceiver;-isOnWifi-(Landroid/content/Context;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/quicksearchbox/QsbApplicationWrapper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/SingleItemAlbum;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiManager;-addNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-inputDispatchingTimedOut-(Lcom/android/server/am/ProcessRecord; Lcom/android/server/am/ActivityRecord; Lcom/android/server/am/ActivityRecord; Z Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/location/ILocationManager;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/location/ILocationManager;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/gadget/WidgetService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-stopDtmf-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/support/v4/net/ConnectivityManagerCompat$HoneycombMR2ConnectivityManagerCompatImpl;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-startVoiceRecognition-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/IInputMethodManager;-shouldOfferSwitchingToNextInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/PrinterDiscoverySession;-startPrinterDisovery-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/WallpaperManager;-clear-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ActivityChooserView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/SubtitleView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothHealth$Stub;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-operationComplete-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupIncomingFragment;-loadSettings-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/ui/GLRootView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/widget/WidgetService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/gatt/GattService;-writeDescriptor-(I Ljava/lang/String; I I Ljava/util/UUID; I Ljava/util/UUID; I Ljava/util/UUID; I I [B)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/videosrc/CameraSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/accessibility/CaptioningManager;-notifyEnabledChanged-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothMap;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/pm/PackageManagerService;-access$2300-(Lcom/android/server/pm/PackageManagerService; Lcom/android/server/pm/PackageManagerService$InstallArgs; Z Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/widget/AdapterView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/CreateEventDialogFragment;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/RIL;-getOperator-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothInputDevice;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/AnimatedListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/UnderdevelopedSettings;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager;-setNetworkPreference-(I)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/RIL;-getVoiceRadioTechnology-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ILockSettings$Stub$Proxy;-setLockPassword-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-<init>-(Lcom/android/server/usb/UsbDeviceManager; Landroid/os/Looper;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/location/ILocationManager;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/app/AlertController$RecycleListView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/DirectoryContainerView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarViewAdapter;-buildWeekNum-()Ljava/lang/String;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-cleanup-()Z' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/wallpaper/nexus/NexusWallpaper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISms;-updateMessageOnIccEf-(Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/location/CountryDetectorBase;-detectCountry-()Landroid/location/Country;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/pan/PanService;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter$1;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupActivity;-isThisImeEnabled-(Landroid/content/Context; Landroid/view/inputmethod/InputMethodManager;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/DialogPicker;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-registerObserver-(Lcom/android/bluetooth/map/BluetoothMnsObexClient; I)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/deskclock/HandleApiCalls;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/google/common/io/ByteStreams;-copy-(Lcom/google/common/io/InputSupplier; Lcom/google/common/io/OutputSupplier;)J' : ['android.permission.INTERNET'],
	'Landroid/speech/tts/TextToSpeech;-runAction-(Landroid/speech/tts/TextToSpeech$Action; Ljava/lang/Object; Ljava/lang/String;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$InitialState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/camera/WideAnglePanoramaModule;-onPauseBeforeSuper-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onUpPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/PlaylistBrowserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-updateNetworkEnabledLocked-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/pinyin/SettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/quicksearchbox/util/JavaNetHttpHelper;-createConnection-(Ljava/lang/String; Ljava/util/Map;)Ljava/net/HttpURLConnection;' : ['android.permission.INTERNET'],
	'Lcom/android/gallery3d/app/AbstractGalleryActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/MinTimeProgressView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Controller;-goBackOnePageOrQuit-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-handleEnterEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralTestActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockContentResolver;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendDownForAllNotInjectedPointers-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/GalleryAppImpl;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper$RenderScriptEngine;-onCreate-(Landroid/view/SurfaceHolder;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-setSilentState-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/soundrecorder/SoundRecorder;-updateTimerView-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IRestoreSession$Stub$Proxy;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/smspush/WapPushManager;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/hardware/location/GeofenceHardwareService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/widget/TextTime$1;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-loadDictionaryAsync-()V' : ['android.permission.READ_USER_DICTIONARY'],
	'Lcom/android/music/CreatePlaylist;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/providers/telephony/SmsProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-getRemoteClass-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiStateMachine$VerifyingLinkState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/filtershow/pipeline/ProcessingService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-updateConversation-(Ljava/util/Collection; Landroid/content/ContentValues;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-onPhantomUpEvent-(J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/hid/HidService;-access$100-(Lcom/android/bluetooth/hid/HidService; Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/FrameFetch;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService;-moveQueueItem-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ActivityController;-onCreate-(Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/telephony/gsm/SmsManager;-updateMessageOnSim-(I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/gesture/GestureOverlayView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-readDescriptor-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/printspooler/SelectPrinterActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DeleteEventHelper$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum$FetchCache;-getItem-(I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/power/DisplayPowerController$5;-run-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/videoeditor/OverlayTitleEditor;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CallManager;-dial-(Lcom/android/internal/telephony/Phone; Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-getHealthDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-clearRequestList-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$6000-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; Ljava/lang/String;)V' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/Spinner;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/CaptivePortalTracker$DelayedCaptiveCheckState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.INTERNET', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothAdapter$GattCallbackWrapper;-onAdvertiseStateChange-(I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/RepeatingImageButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-replaceNonSystemPackageLI-(Landroid/content/pm/PackageParser$Package; Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; [I [Z Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/ui/ThumbnailLoadTask;-onPostExecute-(Landroid/graphics/Bitmap;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/TabScrollView$TabLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager;-getInstalledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarBackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/TrimTimeBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/Files;-copy-(Ljava/io/File; Ljava/io/File;)V' : ['android.permission.INTERNET'],
	'Lcom/android/mail/ui/OnePaneController;-access$400-(Lcom/android/mail/ui/AbstractActivityController; Ljava/util/Collection; Z Z Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/ViewGroup;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/category/IconView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiStateMachine$ConnectModeState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/PhoneUi;-showWeb-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/GoogleCalendarUriIntentFilter;-updateSelfAttendeeStatus-(I Ljava/lang/String; I Landroid/content/Intent;)V' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/net/wifi/WifiStateMachine;-updateBatteryWorkSource-(Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/TrimTimeBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/CursorPhotoSource;-naturalPrevious-(Landroid/graphics/Bitmap; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/sip/SipService;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/RIL;-writeSmsToSim-(I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothGatt$1;-onCharacteristicWrite-(Ljava/lang/String; I I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-onUpdatePhoneObject-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-startDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/textservice/ITextServicesManager;-setCurrentSpellCheckerSubtype-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/ChangeNotifier;-fakeChange-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/WeekSelector;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/PackageSummary;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_MULTICAST_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/deskclock/alarms/AlarmService;-onDestroy-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/gadget/WidgetConfigure;-onActivityResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageShow;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Ljava/lang/String; J F Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/ProfileService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/TwoPaneController;-showNextConversation-(Ljava/util/Collection; Ljava/lang/Runnable;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService$2;-getState-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/RIL;-getCdmaSubscriptionSource-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/IconLoader;-startLoadingIcon-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ex/photo/util/ImageUtils$HttpInputStreamFactory;-createInputStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/music/MediaPlaybackService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/FrameFetch;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/location/SettingInjectorService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-getEnabledInputMethodList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AndroidClockTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/preference/PreferenceFrameLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/selectcalendars/SelectCalendarsSyncFragment;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/provider/BrowserProvider2$DatabaseHelper;-onUpgrade-(Landroid/database/sqlite/SQLiteDatabase; I I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/AlbumPicker;-onResume-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothGatt;-beginReliableWrite-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/PointerLocation;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/VideoEditorActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/BackupManagerService;-handleTimeout-(I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/TextServicesManagerService;-switchUserLocked-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/preference/VolumePreference;-onDialogClosed-(Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/content/SyncManager;-access$2000-(Lcom/android/server/content/SyncManager;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/development/SyncAdapterDriver;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/polarclock/PolarClockWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-goingToSleepLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/BluetoothInputDevice;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/widget/ILockSettings;-getBoolean-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/PackageBrowser;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-supplyPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView$1;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/TextSwitcher;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/utils/FragmentStatePagerAdapter2;-setPrimaryItem-(Landroid/view/ViewGroup; I Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$DisconnectingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/email/service/AccountService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowManager$Stub;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/telephony/PhoneProxy;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/agenda/AgendaFragment;-eventsChanged-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/bluetooth/IBluetoothPan$Stub;-setBluetoothTethering-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/compose/EmptyService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/photo/adapters/BaseCursorPagerAdapter;-instantiateItem-(Landroid/view/ViewGroup; I)Ljava/lang/Object;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView;-onUpEvent-(I I I J)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/ConnectivityManager;-getLinkProperties-(I)Landroid/net/LinkProperties;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/PhoneBase;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/internal/PreviewPlacerView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$1;-updateFolderList-(J)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/googlecode/mp4parser/authoring/builder/DefaultMp4Builder;-build-(Lcom/googlecode/mp4parser/authoring/Movie;)Lcom/coremedia/iso/IsoFile;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/LinkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/gadget/WidgetTypeChooser;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UiccCard;-update-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/uicc/IccCardStatus;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onRemoteDisconnect-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-startScanWithUuids-(I Z [Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/MailLogService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-getProvisioningOrActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/LocationManagerService$4;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/ConnectivityService;-isTetheringSupported-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/mail/ui/OnePaneController;-doMarkConversationMessagesUnread-(Lcom/android/mail/providers/Conversation; Ljava/util/Set; [B)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-launchAssistLongPressAction-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-access$700-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/email/service/AttachmentDownloadService$DownloadSet;-onWatchdogAlarm-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/ImageSwitcher;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt;-endServiceDeclaration-(I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/support/v7/widget/GridLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertService;-doTimeChanged-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/videoeditor/OverlayTitleTemplatePicker;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/MediaPlaybackActivity;-access$700-(Lcom/android/music/MediaPlaybackActivity;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/widget/AnalogClock;-access$100-(Landroid/widget/AnalogClock;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorValueView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ActionBarOverlayLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/gadget/MediaSetSource$CheckedMediaSetSource;-reload-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/TabWidget;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-access$000-(Lcom/android/bluetooth/opp/BluetoothOppService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Landroid/net/nsd/NsdManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/sip/SipCommandInterface;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintJobConfigActivity$Editor$2;-onItemSelected-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/LayoutInflater$BlinkLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/gallery3d/app/ManageCachePage$4;-onLongTap-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasSettings;-handleProvisionError-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/view/CustomScreenLinearLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/EventInfoActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/power/PowerManagerService$1;-onStateChanged-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine$SoftApStartedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/content/MutableContextWrapper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/SwipeHelper$1;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/FolderSelector;-onFolderSelected-(Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/print/IPrintManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager;-stopUsingNetworkFeature-(I Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/UnderdevelopedSettings;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/RenamePlaylist;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/Screensaver;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/setup/SetupActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu;-performDestructiveAction-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupNames$3;-callDoInBackgroundForTest-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/hardware/Camera;-open-()Landroid/hardware/Camera;' : ['android.permission.CAMERA'],
	'Lcom/android/wallpaper/RenderScriptWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/DayView;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-moveQueueItem-(I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/datetimepicker/date/AccessibleDateAnimator;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-enqueue-([J I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/CursorEntityIterator;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/browser/view/StopProgressView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/GeneralPreferences;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/pinyin/SettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/compose/EnterSubject;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$TetheredState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/music/TrackBrowserActivity$7;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-<init>-(Landroid/content/Context; Landroid/content/SharedPreferences;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/BrowserHistoryPage;-onContextItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/btservice/BondStateMachine$PendingCommandState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/pm/PackageManagerService$11;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/effect/effects/ContrastEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/MediaPlaybackService;-reloadQueue-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-getBondState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/updates/SELinuxPolicyInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/Details;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/OnePaneController;-restoreSelectedConversations-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onStateResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager$SyncAlarmIntentReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/hardware/ConsumerIrManager;-getCarrierFrequencies-()[Landroid/hardware/ConsumerIrManager$CarrierFrequencyRange;' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/music/WeekSelector;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/DeskClockFragment;-onPageChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-setProtocolMode-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Disconnected;-enter-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/HorizontalScrollView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/ConfigurationViewer;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-isScanAlwaysAvailable-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/printspooler/PrintSpoolerService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-handlePinMmi-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/view/WindowManagerPolicy;-setSafeMode-(Z)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/music/MusicPicker;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/MediaSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/tts/compat/CompatTtsService;-getSecureSettingInt-(Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/PhotoModule;-setLocationPreference-(Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/ButteryProgressBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-ensureBootCompleted-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/usb/UsbDeviceManager;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-disable-(Z)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/browse/ConversationPager;-pageLeft-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/INetworkPolicyManager$Stub;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/videoeditor/ProjectsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/ConversationLongPressTipView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/GLView;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/NetworkStateHandler$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-expectVoiceRecognition-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/Tab;-updateShouldCaptureThumbnails-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/filterpacks/imageproc/FlipFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintJobConfigActivity$CustomEditText;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationPager;-smoothScrollTo-(I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-tether-(Ljava/lang/String;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/camera/ShutterButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/KeyboardView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/EventInfoFragment;-updateResponse-(J J I)V' : ['android.permission.WRITE_CALENDAR'],
	'Landroid/net/wifi/IWifiManager;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/CompatModePackages;-setPackageScreenCompatModeLocked-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/LightsService$1;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/packageinstaller/PackageInstallerActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/service/ApiService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/SearchActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/exchange/EasOutboxService;-setupServiceForAccount-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account;)Lcom/android/exchange/EasSyncService;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/ConversationViewFragment;-renderMessage-(Lcom/android/mail/browse/ConversationMessage; Z Z Z Z Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/packageinstaller/InstallAppProgress;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/ex/chips/RecipientEditTextView$IndividualReplacementTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_CONTACTS'],
	'Landroid/bluetooth/BluetoothA2dp;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/IAudioService$Stub$Proxy;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/wifi/WifiService;-registerForScanModeChange-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mail/ui/AbstractActivityController;-showWaitForInitialization-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-notifyContentChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/TwoPaneController;-showConversation-(Lcom/android/mail/providers/Conversation;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/OnePaneController;-onRestoreInstanceState-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-sendCdmaSms-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/compose/EmptyService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothA2dp;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/commands/bmgr/Bmgr;-doRestore-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AppBridge$Server;-notifyScreenNailChanged-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/WebViewController;-shouldOverrideUrlLoading-(Lcom/android/browser/Tab; Landroid/webkit/WebView; Ljava/lang/String;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/development/PermissionDetails;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-openCompleteTransfer-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/SecureAlbum;-addMediaItem-(Z I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-startScanWithUuids-(I Z [Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$InitialState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothAdapter;-readOutOfBandData-()Landroid/util/Pair;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/EventInfoFragment;-access$100-(Lcom/android/calendar/EventInfoFragment; Landroid/view/View;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/exchange/adapter/ContactsSyncParser;-deleteParser-(Lcom/android/exchange/adapter/ContactsSyncParser$ContactOperations;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/emailsync/SyncManager;-acquireWakeLock-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/GoogleAccountLogin;-startLogin-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/gallery3d/data/LocalAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker$2;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/google/GoogleSearch;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/documentsui/TestActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/IRecognitionService$Stub;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/RIL;-rejectCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/provider/BrowserProvider;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/commands/pm/Pm;-main-([Ljava/lang/String;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/RIL;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangup-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;-onProgressComplete-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/keyboard/KeyboardActionListener;-onCustomRequest-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/ListPrefSettingPopup$Listener;-onListPrefChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppLiveFolder;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/CacheAbuser;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/HistoryItem;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/htmlviewer/HTMLViewerActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothMap;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/Phone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/PhotoModule;-enableRecordingLocation-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/LocationManager;-addGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/VpnService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/Gallery;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/widget/BookmarkThumbnailWidgetService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/MoreSettingPopup$Listener;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/CameraActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/service/EasMeetingResponder;-sendResponse-(Lcom/android/emailcommon/provider/EmailContent$Message; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/WifiController;-access$700-(Lcom/android/server/wifi/WifiController;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/preference/PreferenceActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/ConversationViewAdapter$BorderItem;-onModelUpdated-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/accounts/IAccountManager;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-setUserActivityTimeoutOverrideFromWindowManagerInternal-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/PackageManagerBackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/MessageInviteView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-serverConnect-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/alerts/QuickResponseActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/AccountManager;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/provider/EmailProvider;-notifyUIConversation-(Landroid/net/Uri;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ConversationSelectionSet;-dispatchOnEmpty-(Ljava/util/ArrayList;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$2300-(Lcom/android/bluetooth/hfp/HeadsetStateMachine;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/FolderListFragment$2;-onChanged-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/IBackupManager$Stub$Proxy;-opComplete-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/svox/pico/PicoService;-getDefaultSpeechRate-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/view/VolumePanel;-onVibrate-()V' : ['android.permission.VIBRATE'],
	'Landroid/support/v4/widget/ContentLoadingProgressBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$8;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/bluetooth/BluetoothDevice;-fetchUuidsWithSdp-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-removeTracks-(I I)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onCreate-(Landroid/os/Bundle; Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/LockPatternUtils;-passwordToHash-(Ljava/lang/String;)[B' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getSelectedInputMethodSubtypeId-(Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/datetimepicker/time/RadialSelectorView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt;-clientDisconnect-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-pauseGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/google/common/io/CharStreams;-toStringBuilder-(Lcom/google/common/io/InputSupplier;)Ljava/lang/StringBuilder;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/widget/SlidingTab;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiService;-startScan-(Landroid/os/WorkSource;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/ViewFlipper;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/alerts/AlertReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-init-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/NavTabView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/TimerRingService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-setUsbTethering-(Z)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/addbookmark/FolderSpinner;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService$Stub$Proxy;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-registerForNotification-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiWatchdogStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiService;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-processProfileStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/alerts/AlertService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/AddBookmarkPage;-setAccount-(Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper$WalkAroundEngine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Landroid/bluetooth/BluetoothA2dp;-setAvrcpAbsoluteVolume-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/AudioPreview$2;-onAudioFocusChange-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-getTetherableUsbRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/pinyin/PinyinDecoderService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/eas/EasFolderSync;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Landroid/speech/tts/TextToSpeech$Action;-run-(Landroid/speech/tts/ITextToSpeechService;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/PendingIntent$FinishedDispatcher;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/pinyin/SkbContainer;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-start-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/MediaFocusControl;-dispatchMediaKeyEventForCalls-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/MultiReader;-read-([C I I)I' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState$SwitchActions;-setAlphabetShiftLockShiftedKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$System;-getFloatForUser-(Landroid/content/ContentResolver; Ljava/lang/String; F I)F' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Application;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-access$500-(Lcom/android/internal/telephony/RIL; Landroid/os/Parcel;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/SelectionManager;-expandMediaSet-(Ljava/util/ArrayList; Lcom/android/gallery3d/data/MediaSet; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$UserAuthorizingInviteRequestState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowManager$Stub$Proxy;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaFocusControl;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/compose/FromAddressSpinner;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/eas/EasProvision;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/vcard/VCardParserImpl_V30;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/music/RepeatingImageButton$1;-run-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/DeskClock;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-onPostExecute-(Landroid/filterfw/core/AsyncRunner$RunnerResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/sip/ISipService$Stub;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/ImapService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/CheckedLinearLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PhotoPage;-onFilmModeChanged-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/AccountsTester$3;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/server/updates/CarrierProvisioningUrlsInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccProvider;-updateIccRecordInEf-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/view/menu/ExpandedMenuView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ListActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkStatsService;-updateIfacesLocked-()V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/browser/AddNewBookmark;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AppBridge$Server;-addSecureAlbumItem-(Z I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/DataList;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/TwoPaneController;-handleBackPress-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/svox/pico/GetSampleText;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageCrop;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarUtils$TimeZoneUtils;-setTimeZone-(Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/am/ActivityManagerService$25;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorOpacityView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/widgets/TransitionView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPickerActivity;-onListItemClick-(Landroid/widget/ListView; Landroid/view/View; I J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/TabActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/SuperCollapsedBlock$1;-run-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/datetimepicker/date/SimpleMonthView$MonthViewTouchHelper;-performAccessibilityAction-(Landroid/view/View; I Landroid/os/Bundle;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/LocationManagerService;-applyAllProviderRequirementsLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-showConversationList-(Lcom/android/mail/ConversationListContext;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/base/FrameStore;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/EasAuthenticatorServiceAlternate;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/ui/SearchActivityView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email2/ui/CreateShortcutActivityEmail;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/DialogActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/IBluetoothMap;-getConnectedDevices-()Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/eas/EasSyncCalendar;-<init>-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/Mailbox;)V' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/videoeditor/widgets/EditorRelativeLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/Avrcp;-access$2000-(Lcom/android/bluetooth/a2dp/Avrcp; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Settings$System;-getShowGTalkServiceStatus-(Landroid/content/ContentResolver;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppObexServerSession;-releaseWakeLocks-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextView;-handleTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/music/ScanningProgress;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/polarclock/PolarClockSettings;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/RIL;-getCellInfoList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ViewMode;-dispatchModeChange-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/data/FilterEmptyPromptSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/FilmstripPage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/Event;-loadEvents-(Landroid/content/Context; Ljava/util/ArrayList; I I I Ljava/util/concurrent/atomic/AtomicInteger;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/Controller;-closeTab-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/inputmethodservice/ExtractButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/alarmclock/DigitalAppWidgetService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/PhoneUi;-onBackKey-()Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageStraighten;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CallTracker;-pollCallsWhenSafe-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-cleanUpConnection-(Z Lcom/android/internal/telephony/dataconnection/ApnContext;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/DisableCarModeActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-sendPendingNotificationsLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-onStartSample-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech;-speak-(Ljava/lang/String; I Ljava/util/HashMap;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/StickyHeaderListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/widgets/HandleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/CertBlacklister$BlacklistObserver$1;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/time/RadialSelectorView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TableRow;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService;-getCurrentSpellCheckerSubtype-(Ljava/lang/String; Z)Landroid/view/textservice/SpellCheckerSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/adapter/EmailSyncParser;-wipe-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/TwoPaneController;-getFolderChange-(Ljava/util/Collection; Ljava/util/Collection; Z Z Z Z Lcom/android/mail/providers/Folder;)Lcom/android/mail/ui/DestructiveAction;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/AudioService$ScoClient;-incCount-(I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/base/RetargetFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/bluetooth/hdp/HealthService$BluetoothHealthBinder;-unregisterAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/development/DataList;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$NotConnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/AllInOneActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/GpsLocationProvider$5;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/MailLogService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DayView;-onCreateContextMenu-(Landroid/view/ContextMenu; Landroid/view/View; Landroid/view/ContextMenu$ContextMenuInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-setShifted-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$ProvisionDiscoveryState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/SearchView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasSearch;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/ViewFlipper;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService;-getNetworkInfo-(I)Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-initializeData-(Landroid/os/Bundle;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterfw/core/FilterFunction;-execute-(Landroid/filterfw/core/KeyValueMap;)Landroid/filterfw/core/Frame;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/ContextWrapper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallEnded-(Lcom/android/internal/telephony/Connection$DisconnectCause;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/StringLogger;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/print/PrintManagerService;-removePrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IRestoreSession$Stub;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/selectcalendars/SelectCalendarsSimpleAdapter$2;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/preference/VolumePreference;-cleanup-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/photos/views/HeaderGridView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/server/StatusBarManagerService;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/net/wifi/WifiManager;-clearBlacklist-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/emailcommon/utility/AttachmentUtilities;-saveAttachment-(Landroid/content/Context; Ljava/io/InputStream; Lcom/android/emailcommon/provider/EmailContent$Attachment;)V' : ['android.permission.INTERNET'],
	'Lcom/android/calendar/selectcalendars/CalendarColorSquare;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/quicksearchbox/ui/SearchActivityViewSinglePane;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-virtualUnplug-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/VideoView;-access$2100-(Landroid/widget/VideoView;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/DialogTitle;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/IWifiP2pManager$Stub;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/mail/ui/ConversationViewProgressController$1;-run-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/inputmethod/latin/LatinIME;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/MediaPlayer;-access$600-(Landroid/media/MediaPlayer;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/google/common/io/CharStreams;-readLines-(Lcom/google/common/io/InputSupplier; Lcom/google/common/io/LineProcessor;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/base/RetargetFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl$SipKeepAlive;-process-(Ljava/util/EventObject;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-transitionFromAlbumPageIfNeeded-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/music/CheckableRelativeLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/vcard/VCardEntryCommitter;-onEntryCreated-(Lcom/android/vcard/VCardEntry;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/dreams/DreamManagerService;-chooseDreamForUser-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ConversationItemView;-performClick-()Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneFallbackEventHandler;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/email/NotificationController;-refreshAllNotifications-(Landroid/content/Context;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/media/MediaRouter$RouteInfo;-requestSetVolume-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/research/ReplayerService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountAdapter;-<init>-(Landroid/content/Context; Landroid/database/Cursor; Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/emailcommon/service/PolicyServiceProxy$2;-run-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection;-dial-()V' : ['android.permission.USE_SIP'],
	'Lcom/android/music/MediaPlaybackService;-doAutoShuffleUpdate-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/LocalPhotoSource$1;-onChange-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/videoeditor/widgets/TransitionView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-onPageChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AbsoluteLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-getBasebandVersion-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ScrollIndicatorsView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TabHost;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/IWindowSession;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompatJellyBean$AccessibilityNodeInfoBridge;-createAccessibilityNodeInfo-(I)Ljava/lang/Object;' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$1;-pushModify-(J)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-setVoiceCallForwardingFlag-(I Z Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/AbsActionBarView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractActivityController;-onConversationSelected-(Lcom/android/mail/providers/Conversation; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getCallWaiting-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/VideoView;-openVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupIncomingFragment;-updatePortFromSecurityType-()V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/text/StringSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupIncoming;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/ProjectsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/adapter/CalendarSyncParser;-addParser-(Lcom/android/exchange/adapter/CalendarSyncParser$CalendarOperations;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/mail/ui/FolderListLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pan/PanService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/ComposeActivityEmail;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/SpeechRecognizer;-access$200-(Landroid/speech/SpeechRecognizer;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/view/accessibility/CaptioningManager;-addCaptioningChangeListener-(Landroid/view/accessibility/CaptioningManager$CaptioningChangeListener;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/mail/browse/ConversationPager;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperChange;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/effect/effects/ColorTemperatureEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/ImageSwitcher;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/DeviceAccountLogin;-login-(I Lcom/android/browser/DeviceAccountLogin$AutoLoginCallback;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-checkAddPermission-(Landroid/view/WindowManager$LayoutParams; [I)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/settings/GallerySettings;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/spellcheck/SpellCheckerSettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/spellcheck/AndroidSpellCheckerService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationPhotoTeaserView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/MovieActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/BluetoothManagerService;-access$600-(Lcom/android/server/BluetoothManagerService;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/soundrecorder/SoundRecorder;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Switch;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/RepeatingImageButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.INTERNET'],
	'Lcom/android/music/AudioPreview;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getMobileProvisioningUrl-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/calculator2/PanelSwitcher;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CallerInfo;-doSecondaryLookupIfNecessary-(Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/CallerInfo;)Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getRemoteType-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Landroid/content/IContentService$Stub$Proxy;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/nfc/cardemulation/HostApduService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsFragment;-onResume-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasAccountService;-sendSettings-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment$1$1;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/MultiFoldersSelectionDialog;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/camera/CameraActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/exchange/service/AbstractSyncAdapterService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/email/activity/setup/AccountSetupType;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/LoadedApk$ReceiverDispatcher$InnerReceiver;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/email/activity/ComposeActivityEmail;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/CameraBackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/CheckableRelativeLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ShutdownActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/FullscreenViewer;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/MediaPlaybackService;-removeTracks-(I I)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-interceptKeyBeforeDispatching-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)J' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-pause-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/ArtistAlbumBrowserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/MediaScanner$MyMediaScannerClient;-doScanFile-(Ljava/lang/String; Ljava/lang/String; J J Z Z Z)Landroid/net/Uri;' : ['android.permission.READ_PROFILE'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getBondState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/email/activity/setup/AccountSettings;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiStateMachine$ConnectedState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NotificationManagerService;-disableNonexistentListeners-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/SelectSyncedCalendarsMultiAccountActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pNotSupportedState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/service/ContactsSyncAdapterService;-access$000-(Landroid/content/Context; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/CellBroadcastHandler;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory$3$1;-run-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/widget/TableLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/Details;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processGetInput-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/ActionMenuPresenter$OverflowMenuButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/AlbumPage;-access$900-(Lcom/android/gallery3d/app/AlbumPage; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/VibratorService;-access$1300-(Lcom/android/server/VibratorService; J I)V' : ['android.permission.VIBRATE'],
	'Landroid/net/wifi/WifiStateMachine;-isWifiTethered-(Ljava/util/ArrayList;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/commands/svc/Svc;-main-([Ljava/lang/String;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/wallpaper/RenderScriptWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/RIL;-stopDtmf-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/research/UploaderService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ProgressBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/SimpleWeeksAdapter;-refresh-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/music/ScanningProgress;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifyCellLocation-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothA2dp;-isA2dpPlaying-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothDevice;-createScoSocket-()Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCallHoldIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/ui/InLineSettingItem$Listener;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/PinyinIME;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Settings$System;-getLong-(Landroid/content/ContentResolver; Ljava/lang/String;)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasOutboxService;-sendMeetingResponse-(Lcom/android/exchange/MeetingResponseRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/browser/BookmarkSearch;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ProgressBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/musicvis/vis1/Visualization1;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/print/PrintManager;-createPrinterDiscoverySession-()Landroid/print/PrinterDiscoverySession;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/MailIntentService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/TransitionsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/MediaPlayer;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/TwoPaneController;-destroyPending-(Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/camera/ui/RotateLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BluetoothManagerService;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/opp/BluetoothOppIncomingFileConfirmActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TableRow;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/RetargetFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmClockFragment$AlarmItemAdapter;-getView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Gallery;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/gadget/WidgetService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-startAppFreezingScreenLocked-(Lcom/android/server/wm/AppWindowToken; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/MonthByWeekFragment$2;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/gadget/WidgetService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BackupManagerService;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/ContactsContract$Contacts;-markAsContacted-(Landroid/content/ContentResolver; J)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/gallery3d/app/SlideshowDataAdapter$ReloadTask;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Ljava/lang/Void;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/browse/MultiAdapterSpinner;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiService;-setWifiEnabled-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine$P2pNotSupportedState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/ConversationListView;-showSyncStatusBar-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/FilterShowActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeech$8;-run-(Landroid/speech/tts/ITextToSpeechService;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothGatt;-readDescriptor-(Landroid/bluetooth/BluetoothGattDescriptor;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/BlendFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Landroid/widget/DialerFilter;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/IState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/MultiReader;-<init>-(Ljava/util/Iterator;)V' : ['android.permission.INTERNET'],
	'Landroid/location/LocationManager;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/Controller;-openTab-(Ljava/lang/String; Z Z Z Lcom/android/browser/Tab;)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Telephony$Threads;-getOrCreateThreadId-(Landroid/content/Context; Ljava/lang/String;)J' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/os/BatteryStatsImpl;-setOnBattery-(Z I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/dreams/phototable/CursorPhotoSource;-load-(Lcom/android/dreams/phototable/PhotoSource$ImageData; Landroid/graphics/BitmapFactory$Options; I I)Landroid/graphics/Bitmap;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/inputmethodservice/ExtractEditLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/PosterizeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onModifiedTouchEvent-(Landroid/view/MotionEvent; Z)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onPackageAppeared-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CalendarView;-setMinDate-(J)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock;-setFormat-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsActivity$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/ex/chips/BaseRecipientAdapter$DefaultFilter;-performFiltering-(Ljava/lang/CharSequence;)Landroid/widget/Filter$FilterResults;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/ConversationListView$HintText;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/Context;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/NavScreen;-close-(I Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-disconnectMap-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/printservice/PrintService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperPreview;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-unregisterServer-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/emailsync/SyncManager;-stopNonAccountMailboxSyncsForAccount-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IntRangeManager;-updateRanges-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/AlarmAlertWakeLock;-acquireCpuWakeLock-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/SecureCameraActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManagerResponse$Stub$Proxy;-onResult-(Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorSVRectView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/WebViewController;-openTab-(Ljava/lang/String; Lcom/android/browser/Tab; Z Z)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/tts/compat/CompatTtsService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/ContentService;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/os/IHardwareService$Stub;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/LinearLayoutWithDefaultTouchRecepient;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavigationBarPhone;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Tab$2;-doUpdateVisitedHistory-(Landroid/webkit/WebView; Ljava/lang/String; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/net/sip/SipManager;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.USE_SIP'],
	'Landroid/accessibilityservice/AccessibilityService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-access$000-(Lcom/android/emailcommon/service/IEmailServiceCallback; J J I I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/ConnectivityService;-getMobileDataEnabled-()Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$4900-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/eas/EasPing;-handleResponse-(Lcom/android/exchange/EasResponse;)I' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/Pico;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController$DeviceInactiveState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/CursorEntityIterator;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/email/provider/EmailProvider;-notifyWidgets-(J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/bluetooth/BluetoothAdapter;-getUuids-()[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityRecord$Token;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/BackupManagerService$ActiveRestoreSession;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/galaxy4/Galaxy4;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/MoreKeysKeyboardView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/userdictionary/UserDictionaryList;-getUserDictionaryLocalesSet-(Landroid/app/Activity;)Ljava/util/TreeSet;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_PROFILE'],
	'Landroid/widget/TextClock;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/datetimepicker/date/DayPickerView;-restoreAccessibilityFocus-(Lcom/android/datetimepicker/date/SimpleMonthAdapter$CalendarDay;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/camera/ui/ZoomView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/EmailApplication;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-separateConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onRoamingOn-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/ManageCachePage;-onClick-(Landroid/view/View;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/galaxy4/Galaxy4;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController;-<init>-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/EmptyConversationListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/opengl/GLSurfaceView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/TouchExplorer$DoubleTapDetector;-onDoubleTap-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/browser/ComboViewActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/Button;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/AccountsTester;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/StatusBarManagerService;-enforceExpandStatusBar-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/exchange/ExchangeService;-runAsleep-(J J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/ByteStreams;-length-(Lcom/google/common/io/InputSupplier;)J' : ['android.permission.INTERNET'],
	'Landroid/renderscript/RSSurfaceView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockContentResolver;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-<init>-(Lcom/android/gallery3d/data/Path; Ljava/util/Comparator; [Lcom/android/gallery3d/data/MediaSet; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/compose/AttachmentComposeView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/calendar/recurrencepicker/LinearLayoutWithMaxWidth;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/DeviceStorageMonitorService;-sendFullNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/noisefield/NoiseFieldWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/ExtractButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-updateUIForCurrentPhoto-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/Editor$SuggestionsPopupWindow;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/SizeChangeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/commands/media/Media;-run-([Ljava/lang/String;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.RESTART_PACKAGES'],
	'Lcom/android/music/WeekSelector;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/Screensaver$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/tts/compat/CompatTtsService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/setup/AccountSetupIncomingFragment;-onUseSslChanged-(Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/music/AlbumBrowserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/SelectedConversationsActionMenu;-clearSelection-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/speech/tts/TextToSpeech;-<init>-(Landroid/content/Context; Landroid/speech/tts/TextToSpeech$OnInitListener; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/SearchDialog$SearchBar;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/CreateEventDialogFragment;-createAllDayEvent-()V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/email/view/CertificateSelector;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-onServiceDisconnected-(Landroid/content/ComponentName;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/phasebeam/PhaseBeamWallpaper$RenderScriptEngine;-reportVisibility-()V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupWaitingOrBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ServiceWatcher$2;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-access$200-(Lcom/android/internal/telephony/IccSmsInterfaceManager; [Lcom/android/internal/telephony/cdma/CdmaSmsBroadcastConfigInfo;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/KeyboardView;-onModifiedTouchEvent-(Landroid/view/MotionEvent; Z)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransfer;-start-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/data/LocalAlbumSet$1;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/LocationManagerService$1;-onOpChanged-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/CheckVoiceData;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkStatsService;-updateIfaces-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/ReceiverRestrictedContext;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/InLineSettingMenu;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/LayoutInflater$BlinkLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/videoeditor/KenBurnsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/soundrecorder/SoundRecorder;-onStateChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BackupManagerService$PerformInitializeTask;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/ErrorConsoleView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-prev-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-getVoiceRadioTechnology-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/pan/PanService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/QuickContactBadge;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/os/BatteryStatsImpl;-setBtHeadset-(Landroid/bluetooth/BluetoothHeadset;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/latin/suggestions/MoreSuggestionsView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/SubtitleController;-access$000-(Landroid/media/SubtitleController;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-onBootCompleted-()V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/widget/Button;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/app/CameraApp;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/SyncBaseInstrumentation;-syncProvider-(Landroid/net/Uri; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_STATS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/Utils;-setTimeFormat-(Landroid/widget/TextClock; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService$Stub$Proxy;-dispatchMediaKeyEventUnderWakelock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/QuickResponseActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/InputMethodManagerService;-setInputMethodLocked-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/SlideshowPage;-onStateResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/TwoPaneController;-handleDrop-(Landroid/view/DragEvent; Lcom/android/mail/providers/Folder;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/VpnService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-start-(Lcom/android/server/sip/SipSessionGroup;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/ColorChipView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$System;-getLongForUser-(Landroid/content/ContentResolver; Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/performance/ThroughputFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/PackagesMonitor$AsyncService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/provider/BrowserProvider2;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/widget/VideoView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/PagerTitleStrip;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/mail/ui/AbstractActivityController;-handleUpPress-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/app/DialogPicker;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-queryFdn-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/CatService;-handleCommand-(Lcom/android/internal/telephony/cat/CommandParams; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$System;-getUriFor-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/deskclock/alarms/AlarmService;-startAlarm-(Lcom/android/deskclock/provider/AlarmInstance;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/ContextThemeWrapper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/pan/PanService;-onConnectStateChanged-([B I I I I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/widget/LockPatternUtils;-isPermanentlyLocked-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-updateHeaderFooter-(I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-setAlphabetKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseAccountTypeActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/text/ToUpperCase;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/photos/AlbumActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Chronometer;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/browse/ToggleableItem;-toggleSelectedStateOrBeginDrag-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/emailsync/SyncManager;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/LegacyEasAuthenticatorService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/EasMeetingResponder;-sendMeetingResponse-(Landroid/content/Context; J I)V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/widget/CalendarView;-onScroll-(Landroid/widget/AbsListView; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/RecognitionManagerService$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-autoConnectHeadset-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/OpenDownloadReceiver$1;-run-()V' : ['android.permission.INTERNET'],
	'Landroid/view/SurfaceView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/svox/pico/GetSampleText;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/provider/Settings$Secure;-getIntForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-processProximateBogusDownMoveUpEventHack-(Lcom/android/inputmethod/keyboard/Key; I I J Lcom/android/inputmethod/keyboard/Key; I I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/gallery3d/data/SnailAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;-access$700-(Lcom/android/inputmethod/latin/ExpandableBinaryDictionary;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/deskclock/ScreensaverActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/AccessibleTextView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/DialogPicker;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/AttachmentDownloadService$DownloadSet;-access$800-(Lcom/android/email/service/AttachmentDownloadService$DownloadSet;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/widget/ILockSettings$Stub$Proxy;-getString-(Ljava/lang/String; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/google/GoogleSearch;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/LayoutListener;-onConversationVisibilityChanged-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/view/IWindowManager$Stub$Proxy;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/mail/compose/ComposeActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/keyboard/KeyboardSwitcher;-setAlphabetManualShiftedKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/galaxy4/Galaxy4;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/PrintManager;-getPrintJobInfo-(Landroid/print/PrintJobId;)Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Landroid/os/SystemVibrator;-vibrate-(J)V' : ['android.permission.VIBRATE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/app/KeyguardManager$KeyguardLock;-reenableKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$Pending;-processConnectionEvent-(I Landroid/bluetooth/BluetoothDevice;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/preference/VolumePreference$SeekBarVolumizer;-revertVolume-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/DownloadTouchIcon;-storeIcon-(Landroid/graphics/Bitmap;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/printspooler/PrintJobConfigActivity$Editor$5;-onChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/keyboard/PointerTracker;-processPhantomSuddenMoveHack-(Lcom/android/inputmethod/keyboard/Key; I I J Lcom/android/inputmethod/keyboard/Key; I I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Lcom/android/music/MediaPlaybackService;-setRepeatMode-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/email/provider/EmailProvider;-startSync-(Landroid/accounts/Account; J I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/ExpandableListActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/soundrecorder/Recorder;-clear-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-onAttach-(Landroid/app/Activity;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/provider/Settings$Secure;-getLong-(Landroid/content/ContentResolver; Ljava/lang/String; J)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-getAllMessagesFromIccEf-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-addDescriptor-(I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Controller;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/ConnectivityManager;-getTetherableWifiRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/deskclock/ScreensaverActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageVignette;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Controller;-createSaveState-()Landroid/os/Bundle;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/music/MediaPlaybackService;-setQueuePosition-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/music/AudioPreview;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/emailcommon/utility/CertificateRequestor;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/content/IContentService;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-waitForConnectivity-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/textservice/TextServicesManager;-getCurrentSpellChecker-()Landroid/view/textservice/SpellCheckerInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsImpl;-setSubtypeEnablerTitle-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupForegroundResumeBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/IConsumerIrService$Stub$Proxy;-getCarrierFrequencies-()[I' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/gallery3d/filtershow/FilterShowActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/widget/DialogTitle;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Button;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiService;-getBatchedScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/app/AlbumSetPage$MyLoadingListener;-onLoadingFinished-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/ToggleButton;-sendOnTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-onUpPressed-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/eas/EasProvision;-getRequestUri-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/IConnectivityManager;-getTetheringErroredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/sip/ISipService$Stub;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-processMessagePart-(Lcom/android/internal/telephony/InboundSmsTracker;)Z' : ['android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-processNextPostDialChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/filtershow/category/CategorySelected;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Spinner;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/LatinIME;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView$DeleteKeyOnTouchListener;-pressDelete-(I)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.VIBRATE'],
	'Landroid/net/wifi/IWifiManager;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/ServiceWatcher$2;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/TwoPaneLayout$1;-run-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/AbstractActivityController;-registerDestructiveAction-(Lcom/android/mail/ui/DestructiveAction;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/Chronometer;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardware;-resumeGeofence-(I I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/os/IPowerManager;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/pm/IPackageManager;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Landroid/widget/TextClock;-init-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/LockPatternView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationListCopy;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConnectivityService$CheckMp$CallBack;-onComplete-(Ljava/lang/Integer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/input/InputManagerService;-notifySwitch-(J I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncActivityTooManyDeletes;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/FillLightEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Editor$InsertionHandleView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$ReloadTask;-findIndexOfTarget-(Lcom/android/gallery3d/app/PhotoDataAdapter$UpdateInfo;)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/ConnectivityService;-requestRouteToHost-(I I Ljava/lang/String;)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetooth$Stub;-isDiscovering-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/support/v4/net/ConnectivityManagerCompatGingerbread;-isActiveNetworkMetered-(Landroid/net/ConnectivityManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/PhotoPage;-onUpPressed-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/widget/EllipsizeLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/QueryBrowserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/SecurityPolicy;-clearAccountPolicy-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ConversationListCopy;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/google/GoogleSearch;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$LabelView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-registerClient-(Landroid/os/ParcelUuid; Landroid/bluetooth/IBluetoothGattCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-setVoiceCallForwardingFlag-(I Z Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/ScreensaverActivity$2;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hdp/HealthService;-disconnectChannel-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/bluetooth/map/BluetoothMapService;-stop-()Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Landroid/view/ViewGroup;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/UiModeManagerService;-updateDockState-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/development/Details;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-onRefreshReady-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/ConversationCursor$ConversationProvider;-undo-(Lcom/android/mail/browse/ConversationCursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupOutgoing;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/CrashRecoveryHandler;-startRecovery-(Landroid/content/Intent;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/widget/ZoomButtonsController$Container;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager$Stub;-print-(Ljava/lang/String; Landroid/print/IPrintDocumentAdapter; Landroid/print/PrintAttributes; Ljava/lang/String; I I)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/editors/SwapButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech$Connection$SetupConnectionAsyncTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Integer;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/EditText;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/eas/EasSettings;-addDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/btservice/AdapterState$OnState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/music/PlaylistBrowserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/ConnectivityService;-access$1500-(Lcom/android/server/ConnectivityService; Landroid/net/NetworkInfo;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/wifi/WifiStateMachine$SoftApStartedState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-restoreAccessibilityFocus-(Lcom/android/datetimepicker/date/SimpleMonthAdapter$CalendarDay;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppLauncherActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SlideshowPage;-access$500-(Lcom/android/gallery3d/data/MediaSet; I)Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/stopwatch/StopwatchFragment;-access$500-(Lcom/android/deskclock/stopwatch/StopwatchFragment;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/email/service/Pop3Service$1;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/BrowserSettings;-getDefaultLinkPrefetchSetting-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calculator2/CalculatorViewPager;-setOffscreenPageLimit-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/IWifiManager;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings; Landroid/os/IBinder; Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/development/InstrumentationList;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodsStr-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$TetheredState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/provider/VoicemailContract$Voicemails;-buildSourceUri-(Ljava/lang/String;)Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/development/PackageSummary;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/inputmethod/latin/LatinIME$UIHandler;-postUpdateShiftState-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/dreams/phototable/PhotoTableDream;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothAdapter;-disable-(Z)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/IAccountManager$Stub;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/calendar/MultiStateButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/CreateEventDialogFragment$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/RIL;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/GrayscaleEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/MutableContextWrapper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readEmailFileAndWait-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/pinyin/CandidatesContainer;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SmsStorageMonitor;-handleIccFull-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintDialogFrame;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/IConnectivityManager$Stub;-startUsingNetworkFeature-(I Ljava/lang/String; Landroid/os/IBinder;)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/videoeditor/widgets/ZoomControl;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarBackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/NavScreen;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/UrlInputView;-onEditorAction-(Landroid/widget/TextView; I Landroid/view/KeyEvent;)Z' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/bluetooth/BluetoothHealth;-getConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/ui/DetailsHelper;-show-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/IAudioService$Stub;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-getMediaItem-(I I)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/alarmclock/DigitalWidgetViewsFactory$RemoteWorldClockAdapter;-updateView-(Landroid/widget/RemoteViews; Lcom/android/deskclock/worldclock/CityObj; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-acknowledgeLastIncomingCdmaSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/DialogTitle;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/BrowserBackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapService$BluetoothMapBinder;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/StackView$StackFrame;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/UriImage;-delete-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/BootReceiver;-access$100-(Lcom/android/server/BootReceiver; Landroid/content/Context;)V' : ['android.permission.INTERNET'],
	'Lcom/android/inputmethod/pinyin/SettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/Phone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/InputMonitor;-notifyANR-(Lcom/android/server/input/InputApplicationHandle; Lcom/android/server/input/InputWindowHandle; Ljava/lang/String;)J' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/deskclock/SettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ingest/ui/DateTileView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SmsApplication;-setDefaultApplication-(Ljava/lang/String; Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView$1;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/email/service/LegacyEmailAuthenticatorService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/NumberPicker$CustomEditText;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/ConnectivityService;-setRadios-(Z)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/videoeditor/service/ApiService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/AttachmentTile;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IWallpaperManager$Stub;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/music/MediaPlaybackActivity$11;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/provider/BrowserProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/am/ActivityManagerService;-updateConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-startSync-(Landroid/net/Uri; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/wifi/SupplicantStateTracker$ScanState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/calendar/alerts/AlertActivity$3;-doInBackground-([Landroid/content/Context;)Ljava/lang/Void;' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/exchange/Exchange;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/dreams/basic/Colors;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/GeofenceHardwareService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/textservice/SpellCheckerService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler$1;-sendScpResults-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/CheckableRelativeLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/ui/CountdownTimerPopup;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/AuthenticatorService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-napInternal-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onRoamingOff-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorHueView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/DocumentsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/GpsLocationProvider;-handleDownloadXtraData-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Browser;-getAllVisitedUrls-(Landroid/content/ContentResolver;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/IAudioService$Stub$Proxy;-adjustVolume-(I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/XLargeUi;-showWeb-(Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/vcard/VCardParserImpl_V40;-parseOneVCard-()Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$1500-(Lcom/android/server/net/NetworkPolicyManagerService;)V' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/RelativeLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech;-isSpeaking-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ListPopupWindow$DropDownListView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDataConnectionAttached-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$CaptivePortalCheckState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/browser/TabBar$TabView;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/OpenDownloadReceiver;-access$000-(Lcom/android/browser/OpenDownloadReceiver; Landroid/content/Context; J)V' : ['android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetooth;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ISms$Stub;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/calendar/agenda/AgendaListView;-refresh-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/service/wallpaper/WallpaperService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calculator2/PanelSwitcher;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Service;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoDataAdapter$ScreenNailJob;-run-(Lcom/android/gallery3d/util/ThreadPool$JobContext;)Lcom/android/gallery3d/ui/ScreenNail;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-onQueryCfComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-updateSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/ControllableActivity;-onAnimationEnd-(Lcom/android/mail/ui/AnimatedAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/exchange/eas/EasPing;-getRequestUri-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/browse/ConversationCursor;-sync-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/MusicUtils;-clearQueue-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onDeleteImage-(Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/app/ResolverActivity$1;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget$RegionLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-updateExternalMediaStatusInner-(Z Z Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/am/ActivityManagerService;-startProcessLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/LocationManagerService;-checkMockPermissionsSafe-()V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/mail/ui/NestedFolderTeaserView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-setFullscreenPagerVisibility-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-matchRecipient-(Landroid/database/Cursor; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo; Lcom/android/bluetooth/map/BluetoothMapAppParams;)Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager;-access$500-(Lcom/android/server/content/SyncManager;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/calendar/alerts/DismissAlarmsService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/SubtitleController;-access$100-(Landroid/media/SubtitleController;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/os/IPowerManager;-updateWakeLockUids-(Landroid/os/IBinder; [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/SearchActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/multiwaveview/MultiWaveView;-access$300-(Lcom/android/internal/widget/multiwaveview/MultiWaveView; I F F)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/musicvis/RenderScriptWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/AudioTrackView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/SwipeableListView$ListItemSwipedListener;-onListItemSwiped-(Ljava/util/Collection;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/WiredAccessoryManager;-<init>-(Landroid/content/Context; Lcom/android/server/input/InputManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/connectivity/Tethering;-access$900-(Lcom/android/server/connectivity/Tethering;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-handleNewSms-(Landroid/os/AsyncResult;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/support/v4/view/ViewPager;-scrollToItem-(I Z I Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/speech/tts/TextToSpeechService$1;-getClientDefaultLanguage-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-setPackageScreenCompatMode-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/app/WallpaperManager;-setResource-(I)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaCall;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/Event;-instancesQuery-(Landroid/content/ContentResolver; [Ljava/lang/String; I I Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/musicvis/RenderScriptWallpaper$RenderScriptEngine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/camera/ui/RenderOverlay$RenderView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EventLocationAdapter;-access$600-(Landroid/content/ContentResolver; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperPreview;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService;-addToPlayList-([J I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/ChooseAccountTypeActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-notifyAndAcknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/RotateEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/service/notification/NotificationListenerService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/input/InputManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-handleOnUserContextChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/view/SnapshotGridView;-sequenceScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/PanoProgressBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/BatchService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/view/menu/IconMenuView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/MediaPlaybackService;-setNextTrack-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/utility/CertificateRequestor;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/ActivityGroup;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/btservice/AdapterService;-autoConnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/ui/ConversationViewFragment;-renderMessage-(Lcom/android/mail/browse/ConversationMessage; Z Z Z Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/widget/LockPatternUtils;-hasWidgetsEnabledInKeyguard-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/PermissionDetails;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/filtershow/pipeline/ProcessingService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumPage;-onSingleTapUp-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasOperation;-performOperation-()I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/app/GalleryActionBar$ClusterRunner;-doCluster-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/alerts/DismissAlarmsService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/AlbumPage$MyDetailsSource;-setIndex-()I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/CaptivePortalTracker;-makeCaptivePortalTracker-(Landroid/content/Context; Landroid/net/IConnectivityManager;)Landroid/net/CaptivePortalTracker;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/photo/MailPhotoViewActivity;-onLoadFinished-(Landroid/support/v4/content/Loader; Landroid/database/Cursor;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$WatchdogEnabledState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/mail/browse/ConversationContainer;-addOverlayView-(I)Landroid/view/View;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/musicvis/vis1/Visualization1$CubeEngine;-doVisibilityChanged-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-setWhereFilterRecipient-(Lcom/android/bluetooth/map/BluetoothMapAppParams; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo;)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/ui/AbstractActivityController$ConversationListLoaderCallbacks;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/LockPatternUtils;-isTactileFeedbackEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget$RegionLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/CalendarColorPickerDialog;-onColorSelected-(I)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/calendar/month/SimpleWeekView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Landroid/widget/ZoomButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiStateMachine;-access$11100-(Landroid/net/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/AudioPreview$PreviewPlayer;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/ControllerOverlay$Listener;-onReplay-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/LauncherActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/widget/SlidingPaneLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Connection;-separate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasLoadAttachment;-requestNoOpSync-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/widget/AnalogClock$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/NetworkStateTracker;-setRadio-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/Activity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/EasSyncService;-acknowledgeProvisionImpl-(Lcom/android/exchange/EasSyncService; Ljava/lang/String; Ljava/lang/String; Z)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/AttachmentDownloadService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hid/HidService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/google/common/io/CharStreams$2;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/ex/photo/PhotoViewPager;-setOffscreenPageLimit-(I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapUtils;-getProfileName-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ListActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/provider/BrowserProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/LinearLayout;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/RecentApplicationsBackground;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/WallpaperManagerService$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-notifyApnIdDisconnected-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-switchToRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/print/PrintManagerService$2;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/statusbar/IStatusBarService;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-getReport-(Landroid/bluetooth/BluetoothDevice; B B I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/certinstaller/CertInstallerMain;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/HorizontalScrollView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/gallery3d/settings/GallerySettings;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/widget/ActionableToastBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/FilterSurfaceView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiApConfigStore$ActiveState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothHeadset;-phoneStateChanged-(I I I Ljava/lang/String; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/print/PrintManagerService$3;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/filtershow/FilterShowActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/PackageBrowser;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Button;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-killUid-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$WaitingState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/location/ILocationManager$Stub$Proxy;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-systemReady-(Ljava/lang/Runnable;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/AllInOneActivity;-onNavigationItemSelected-(I J)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/commands/svc/Svc$Command;-run-([Ljava/lang/String;)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/browser/DataController$DataControllerHandler;-doLoadThumbnail-(Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/InboundSmsHandler$IdleState;-exit-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/svox/pico/Pico;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-addTrackerToRawTable-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/calendar/alerts/InitAlarmsService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/holospiral/HoloSpiralWallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/BrowserBackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/NetworkStateHandler;-<init>-(Landroid/app/Activity; Lcom/android/browser/Controller;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageCurves;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/Tab;-onNewPicture-(Landroid/webkit/WebView; Landroid/graphics/Picture;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processSelectItem-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/eas/EasPing;-requestSyncForMailboxes-(Landroid/accounts/Account; Ljava/util/ArrayList;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/Activity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email2/ui/CreateShortcutActivityEmail;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-deactivateDataCall-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerImpl;-addView-(Landroid/view/View; Landroid/view/ViewGroup$LayoutParams;)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/deskclock/timer/CountingTimerView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/IMediaPlaybackService;-open-([J I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-addToBlacklist-(Ljava/lang/String;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/backup/FullBackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/AllInOneActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/camera/PhotoMenu;-onSettingChanged-(Lcom/android/camera/ListPreference;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-unregisterForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/RecognitionService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/eas/EasLoadAttachment$ProgressCallback;-doCallback-(I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/latin/LatinIME;-onStartInputViewInternal-(Landroid/view/inputmethod/EditorInfo; Z)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/deskclock/ScreensaverActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/development/BadBehaviorActivity$BadService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/base/FrameBranch;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Chronometer;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/research/ResearchLogger;-getAccountName-()Ljava/lang/String;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/DnsPinger;-getDnsList-()Ljava/util/List;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityManagerService;-cleanUpRemovedTaskLocked-(Lcom/android/server/am/TaskRecord; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-readCharacteristic-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/UsimDataDownloadHandler;-handleUsimDataDownload-(Lcom/android/internal/telephony/uicc/UsimServiceTable; Lcom/android/internal/telephony/gsm/SmsMessage;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TwilightService$TwilightListenerRecord;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/widget/ViewSwitcher;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/colorpicker/ColorPickerSwatch;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/SkbContainer;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-setVCardFromPhoneNumber-(Lcom/android/bluetooth/map/BluetoothMapbMessage; Ljava/lang/String; Z)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/exchange/ExchangeService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/ConnectivityManager;-setUsbTethering-(Z)I' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/inputmethodservice/ExtractButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppLiveFolder;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/QueryBrowserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/AlarmScheduler;-scheduleNextAlarm-(Landroid/content/Context; Lcom/android/calendar/alerts/AlarmManagerInterface; I J)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/widget/DateTimeView$2;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/ShutdownThread;-rebootSafeMode-(Landroid/content/Context; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/WifiWatchdogStateMachine;-<init>-(Landroid/content/Context;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/IccProvider;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/GLTextureTarget;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onApnChanged-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/Resources;-readLines-(Ljava/net/URL; Ljava/nio/charset/Charset; Lcom/google/common/io/LineProcessor;)Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/server/LocationManagerService$4;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/TabBar;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/pan/PanService;-handlePanDeviceStateChange-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String; I I I)V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.BLUETOOTH', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/gallery3d/ui/DownUpDetector$DownUpListener;-onUp-(Landroid/view/MotionEvent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/SurfaceView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage;-onLongTap-(I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/emailsync/EmailSyncAlarmReceiver;-access$000-(Lcom/android/emailsync/EmailSyncAlarmReceiver; Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/ToUpperCase;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/ListPopupWindow$DropDownListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/CalendarSyncEnabler;-enableEasCalendarSync-()V' : ['android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/search/Searchables;-getGlobalSearchProviderSetting-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-access$500-(Lcom/android/gallery3d/ui/MenuExecutor; I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperChange;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/IdleMaintenanceService;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConsumerIrService;-getCarrierFrequencies-()[I' : ['android.permission.TRANSMIT_IR'],
	'Landroid/bluetooth/IBluetoothHealth;-getHealthDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/InputMonitor;-interceptKeyBeforeDispatching-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)J' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AttachmentTileGrid;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v7/widget/Space;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$200-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/suggestions/SuggestionStripView$Listener;-addWordToUserDictionary-(Ljava/lang/String;)V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub$Proxy;-getCurrentSpellCheckerSubtype-(Ljava/lang/String; Z)Landroid/view/textservice/SpellCheckerSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SmsManager;-sendDataMessage-(Ljava/lang/String; Ljava/lang/String; S [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/widget/AutoCompleteTextView;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/app/FragmentActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/MailIntentService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-init-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/DetailsHelper;-reloadDetails-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email2/ui/MailboxSelectionActivityEmail;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/development/LogTextBox;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/OnePaneController;-onConversationSelected-(Lcom/android/mail/providers/Conversation; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/RenamePlaylist;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/effect/effects/DocumentaryEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/TrimControllerOverlay;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationSyncDisabledTipView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/Connectivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/TwoLineListItem;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/ToUpperCase;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/NavTabScroller;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/EasAccountService;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/widget/AbsActionBarView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/TimerRingService;-onDestroy-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-appDiedLocked-(Lcom/android/server/am/ProcessRecord; I Landroid/app/IApplicationThread;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/email/service/EasAuthenticatorServiceAlternate;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-access$000-(Lcom/android/deskclock/alarms/AlarmStateManager; Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/content/ContentResolver;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/widget/QuickContactBadge;-assignContactFromEmail-(Ljava/lang/String; Z)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/widget/AnalogClock;-onTimeChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ex/photo/views/PhotoView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/TabScrollView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/event/EditEventFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/email/service/EmailServiceStub;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/hid/HidService;-sendData-(Landroid/bluetooth/BluetoothDevice; Ljava/lang/String;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifyServiceState-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothPbap;-getClient-()Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Landroid/app/SearchDialog$SearchBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/development/Connectivity$3;-onClick-(Landroid/view/View;)V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/filtershow/pipeline/ProcessingService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/SmsApplication;-shouldWriteMessageForPackage-(Ljava/lang/String; Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/TabControl;-saveState-(Landroid/os/Bundle;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/bluetooth/btservice/AdapterService$AdapterServiceBinder;-cancelBondProcess-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/svox/pico/CheckVoiceData;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-endReliableWrite-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/MessageScrollView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/selectcalendars/CalendarColorCache;-<init>-(Landroid/content/Context; Lcom/android/calendar/selectcalendars/CalendarColorCache$OnCalendarColorsLoadedListener;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/widget/ToggleButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/alerts/AlertService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/browse/MessageHeaderView;-setMessageDetailsExpanded-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/ScreenOnBlocker;-release-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/inputmethod/dictionarypack/DownloadOverMeteredDialog;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/EmailConnectivityManager;-getActiveNetworkType-()I' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/deskclock/worldclock/WorldClockAdapter;-updateView-(Landroid/view/View; Lcom/android/deskclock/worldclock/CityObj;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/widget/TextTime;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/PackageSummary;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/event/AttendeesView;-updateAttendeeView-(Lcom/android/calendar/event/EditEventHelper$AttendeeItem;)Landroid/view/View;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/addbookmark/FolderSpinner;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/ui/ClusteredSuggestionsView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/NotificationController;-refreshAllNotificationsInternal-(Landroid/content/Context;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onFilmModeChanged-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/data/SecureAlbum;-getCoverMediaItem-()Lcom/android/gallery3d/data/MediaItem;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/print/PrintJob;-getInfo-()Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/SoftKeyboardView;-onKeyPress-(I I Lcom/android/inputmethod/pinyin/SkbContainer$LongPressTimer; Z)Lcom/android/inputmethod/pinyin/SoftKey;' : ['android.permission.VIBRATE'],
	'Lcom/android/calendar/event/EditEventView$4;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedContactsBinaryDictionary;-loadDeviceAccountsEmailAddresses-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/documentsui/DirectoryContainerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioManager;-reloadAudioSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/dataconnection/DcTrackerBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/packageinstaller/InstallAppProgress;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiService;-enforceAccessPermission-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine$IntentBroadcastHandler;-onConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/camera/ui/RenderOverlay;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/documentsui/DocumentsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/ScreensaverSettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/AlarmManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCurrentCalls-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-stopFreezingDisplayLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ingest/IngestActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.INTERNET', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/ZoomButtonsController$Container;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/noisefield/NoiseField;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/MailActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/month/MonthWeekEventsView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/HorizontalScrollView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/widget/BookmarkThumbnailWidgetService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PhotoPage;-updateCurrentPhoto-(Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/datetimepicker/date/TextViewWithCircularIndicator;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/RotateImageView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/BaseCommands;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ScrollIndicatorsView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/TimeClustering;-run-(Lcom/android/gallery3d/data/MediaSet;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/quicksearchbox/ui/QueryTextView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService;-checkPermission-(I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/print/IPrintManager$Stub$Proxy;-print-(Ljava/lang/String; Landroid/print/IPrintDocumentAdapter; Landroid/print/PrintAttributes; Ljava/lang/String; I I)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/exchange/eas/EasOperation;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Landroid/support/v4/view/ViewPager;-setAdapter-(Landroid/support/v4/view/PagerAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onSetDependencyMet-(Ljava/lang/String; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/WallpaperManagerService$MyPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService;-clientDisconnect-(I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/location/ILocationManager;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/commands/bmgr/Bmgr;-main-([Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/HapticFeedbackController;-access$200-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-handleWakeLockDeath-(Lcom/android/server/power/PowerManagerService$WakeLock;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/gallery3d/filtershow/editors/SwapButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompatKitKat$AccessibilityNodeInfoBridge;-performAction-(I I Landroid/os/Bundle;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/AbstractCalendarActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub;-getBondedDevices-()[Landroid/bluetooth/BluetoothDevice;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiApConfigStore$InactiveState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/mail/ui/AbstractActivityController;-loadAccountInbox-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/NavTabScroller$ContentLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/PreloadRequestReceiver;-isPreloadEnabledOnCurrentNetwork-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/power/PowerManagerService;-access$1500-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-scheduleInstanceStateChange-(Landroid/content/Context; Ljava/util/Calendar; Lcom/android/deskclock/provider/AlarmInstance; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/SettingsRedirector;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/service/EasMeetingResponder;-sendMessage-(Lcom/android/emailcommon/provider/Account; Lcom/android/emailcommon/provider/EmailContent$Message;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/service/EmailSyncAdapterService$1;-searchMessages-(J Lcom/android/emailcommon/service/SearchParams; J)I' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.INTERNET', 'android.permission.READ_CONTACTS'],
	'Lcom/android/mail/widget/WidgetService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/app/SlideshowPage$1;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$LabelView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/PhotoPage;-onDeleteImage-(Lcom/android/gallery3d/data/Path; I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/cardemulation/OffHostApduService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/Chronometer;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/CheckableRelativeLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioRecord;-<init>-(I I I I I)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/music/MusicUtils;-deleteTracks-(Landroid/content/Context; [J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/CheckableRelativeLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/TelephonyManager;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView$1;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothMap$Stub;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/tts/ITextToSpeechService$Stub;-synthesizeToFileDescriptor-(Landroid/os/IBinder; Ljava/lang/String; Landroid/os/ParcelFileDescriptor; Landroid/os/Bundle;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/gatt/GattService;-notifyProfileConnectionStateChanged-(Landroid/bluetooth/BluetoothDevice; I I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/photomanager/ContactPhotoManager$ContactPhotoLoaderThread;-loadPhotos-(Ljava/util/Collection;)Ljava/util/Map;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/mail/browse/ConversationPager;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/musicvis/vis4/Visualization4;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/svox/pico/PicoService;-access$600-(Landroid/speech/tts/TextToSpeechService;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-onUpdatePhoneObject-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/Files;-move-(Ljava/io/File; Ljava/io/File;)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-sendEmergencyCallbackModeChange-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/AlbumActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/event/EditEventFragment;-onPause-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/media/IAudioService;-reloadAudioSettings-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/NativeActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/pbap/BluetoothPbapActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-getRecoveryAction-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/SynchronouslyLoadedUserBinaryDictionary;-writeBinaryDictionary-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/LockPatternUtils;-isBiometricWeakEverChosen-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$DefaultState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/FilmstripPage;-updateCurrentPhoto-(Lcom/android/gallery3d/data/MediaItem;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/LocationProviderInterface;-setRequest-(Lcom/android/internal/location/ProviderRequest; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/settings/GallerySettings;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/inputmethod/InputMethodManager;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothAdapter;-startDiscovery-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/music/IMediaPlaybackService;-setRepeatMode-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/DictionaryService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiController$DeviceInactiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/widget/TextTime$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSettingsFragment;-onStart-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/compose/EmptyService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/Connectivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AutoCompleteTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/providers/downloads/ui/DownloadList;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/Ringtone;-destroyLocalPlayer-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IConnectivityManager$Stub;-getActiveNetworkQuotaInfo-()Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/magicsmoke/MagicSmoke;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/vcard/VCardParser_V40;-parse-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/email/service/AuthenticatorService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/utils/AttachmentUtils;-canDownloadAttachment-(Landroid/content/Context; Lcom/android/mail/providers/Attachment;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/AlbumDataLoader$UpdateContent;-call-()Ljava/lang/Object;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/EasOutboxService;-validateAccount-(Lcom/android/emailcommon/provider/HostAuth; Landroid/content/Context;)Landroid/os/Bundle;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Landroid/widget/AdapterViewFlipper;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/PrintJob;-isFailed-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/VpnService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothGatt$1;-onDescriptorRead-(Ljava/lang/String; I I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/ConversationViewAdapter$SuperCollapsedBlockItem;-onModelUpdated-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/ConnectivityService$CheckMp;-onPostExecute-(Ljava/lang/Integer;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/NsdService;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Lcom/android/server/power/PowerManagerService;-access$1900-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-onUserStopping-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/magicsmoke/MagicSmokeSelector;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/quicksearchbox/QsbApplicationWrapper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService$4;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AutoCompleteTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/SelectCalendarsSyncFragment;-onResume-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-skipCurrentReceiverLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/development/MediaScannerActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/DeskClock;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/adapter/CalendarSyncParser;-changeParser-(Lcom/android/exchange/adapter/CalendarSyncParser$CalendarOperations;)V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/documentsui/TestActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DayView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/LatinIME$SubtypeState;-switchSubtype-(Landroid/os/IBinder; Lcom/android/inputmethod/latin/RichInputMethodManager;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-getBondState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/DayView;-updateTitle-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/ContactsContract$Directory;-notifyDirectoryChange-(Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/BluetoothAdapter;-isAdvertising-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothMap;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/exchange/adapter/EmailSyncAdapter;-parse-(Ljava/io/InputStream;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$4700-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/MailIntentService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/setup/SetupStartIndicatorView$LabelView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CompoundButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/view/BookmarkContainer;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/CameraBackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/event/AttendeesView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getLastSubtypeForInputMethodLockedInternal-(Ljava/lang/String;)Landroid/util/Pair;' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub$Proxy;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/inputmethod/latin/BackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethodcommon/InputMethodSettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/event/EventInterpreter;-onHardwareKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/development/Connectivity;-onStartScreenCycle-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetooth;-createSocketChannel-(I Ljava/lang/String; Landroid/os/ParcelUuid; I I)Landroid/os/ParcelFileDescriptor;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-toggleAlphabetAndSymbols-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/music/PlaylistBrowserActivity;-access$100-(Lcom/android/music/PlaylistBrowserActivity;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calculator2/CalculatorViewPager;-onLayout-(Z I I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/PageDialogsHandler;-showSSLCertificate-(Lcom/android/browser/Tab;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/TimeBar$Listener;-onScrubbingStart-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-addGpsStatusListener-(Landroid/location/GpsStatus$Listener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cat/CatService;-sendMenuSelection-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/recurrencepicker/WeekButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppRfcommListener$1;-run-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/SupplicantStateTracker$HandshakeState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/mail/browse/ConversationListFooterView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/widget/WidgetService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/setup/AccountSetupNames$3;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/content/pm/IPackageManager;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/exchange/eas/EasSearch;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-<init>-(Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/cdma/CdmaCallTracker; Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-access$400-(Lcom/android/server/VibratorService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/inputmethod/InputMethodManager;-getShortcutInputMethodsAndSubtypes-()Ljava/util/Map;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-handleMotionEvent-(Landroid/view/MotionEvent; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/SwipeableListView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/FragmentBreadCrumbs;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHealth;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiManager;-<init>-(Landroid/content/Context; Landroid/net/wifi/IWifiManager;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/datetimepicker/AccessibleTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$3;-doWork-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$TetheredState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/View;-performAccessibilityAction-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-destroyPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Switch;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/soundrecorder/VUMeter;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService$1;-synthesizeToFileDescriptor-(Landroid/os/IBinder; Ljava/lang/String; Landroid/os/ParcelFileDescriptor; Landroid/os/Bundle;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/emailsync/SyncManager;-serviceRequest-(J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/colorpicker/ColorPickerSwatch;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipAudioCall;-close-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/MtpClient$Listener;-deviceRemoved-(Landroid/mtp/MtpDevice;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-isRotationFrozen-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetheringErroredIfaces-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/os/BaseCommand;-onRun-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.RESTART_PACKAGES'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH'],
	'Lcom/android/music/AudioPreview;-stopPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/ui/DefaultSuggestionView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/camera/SecureCameraActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/ILocationManager;-reportLocation-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/support/v4/view/ViewPager;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TelephonyRegistry;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/deskclock/stopwatch/StopwatchService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/event/AttendeesView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioManager;-adjustSuggestedStreamVolume-(I I I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/Editor$HandleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/calendar/ColorChipView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/SearchActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/UiModeManagerService;-updateTwilight-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-fetchRuimRecords-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/emailcommon/utility/CertificateRequestor;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothCallback$Stub$Proxy;-onBluetoothStateChange-(I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-notifyOffApnsOfAvailability-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/renderscript/RSTextureView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/a2dp/A2dpService$BluetoothA2dpBinder;-getPriority-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorOpacityView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getSubtypeHistoryStr-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ContextWrapper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/hdp/HealthService;-updateAndSendIntent-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/packageinstaller/InstallAppProgress;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/ClusterAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/browse/FolderSpanTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CompoundButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/GLTextureTarget;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/PhotoCarousel;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/IActivityManager;-getTasks-(I I Landroid/app/IThumbnailReceiver;)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorSaturationView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/dreams/phototable/PhotoSourcePlexor;-getStream-(Lcom/android/dreams/phototable/PhotoSource$ImageData; I)Ljava/io/InputStream;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/support/v4/widget/SearchViewCompatIcs$MySearchView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/support/v4/view/accessibility/AccessibilityNodeProviderCompat$AccessibilityNodeProviderJellyBeanImpl$1;-createAccessibilityNodeInfo-(I)Ljava/lang/Object;' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/event/EditEventView;-access$400-(Lcom/android/calendar/event/EditEventView; Landroid/widget/TextView; J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/DeskClock;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/wallpaper/walkaround/WalkAroundWallpaper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/deskclock/AnalogClock$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiWatchdogStateMachine$DefaultState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothGatt$1;-onClientRegistered-(I I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/location/GpsLocationProvider$3;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/browser/AddBookmarkPage;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/view/IInputMethodManager;-getLastInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/ConfigurationViewer;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/PanoProgressBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IBackupAgent;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/mail/browse/ConversationPager;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment$1;-onChanged-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/wifi/WifiService;-getScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/svox/pico/PicoService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/AliasActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/AutologinBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/ArtistAlbumBrowserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/tts/compat/CompatTtsService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/AliasActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/ConversationsInOutboxTipView$4;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/browse/ConversationContainer;-positionOverlay-(I I I)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/bluetooth/IBluetoothPbap$Stub$Proxy;-disconnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/wallpaper/RenderScriptWallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/PageDialogsHandler$4;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/DownloadTouchIcon;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/quicksearchbox/ui/CorpusView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-switchTo-(Lcom/android/internal/policy/impl/RecentApplicationsDialog$RecentTag;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-updateParams-(Ljava/util/HashMap;)V' : ['android.permission.READ_CALENDAR'],
	'Landroid/media/SubtitleController;-doHide-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/DeleteItems;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/PhotoView;-slideToPrevPicture-()Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/SpeechRecognizer;-setRecognitionListener-(Landroid/speech/RecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/calendar/EventInfoFragment$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/gatt/GattService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ui/UndoBarView;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/server/BluetoothA2dpService;-onBluetoothDisable-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/bluetooth/hfp/AtPhonebook;-getPhonebookResult-(Ljava/lang/String; Z)Lcom/android/bluetooth/hfp/AtPhonebook$PhonebookResult;' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-getLong-(Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardEntry$NoteData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/mail/browse/ConversationCursor$ConversationListener;-onDataSetChanged-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/gadget/WidgetService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/service/EasAuthenticatorService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/DayView;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/providers/telephony/SmsProvider;-getSingleMessageFromIcc-(Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/music/MusicBrowserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/EmailIntentService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService$CalendarFactory;-onLoadComplete-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR'],
	'Lcom/android/email/service/EasAuthenticatorServiceAlternate;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/AbstractSettingPopup;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt;-readCharacteristic-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-startVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothGatt$1;-onDescriptorWrite-(Ljava/lang/String; I I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState$SwitchActions;-setAlphabetKeyboard-()V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/email/service/EasAuthenticatorServiceAlternate;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/PickerActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/ObjectSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-dispatchMessageRadioSpecific-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-connectChannelToSource-(Landroid/bluetooth/BluetoothDevice; Landroid/bluetooth/BluetoothHealthAppConfiguration;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/inputmethod/keyboard/internal/ScrollKeyboardView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-getIndexOfItem-(Lcom/android/gallery3d/data/Path; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/ui/ActionModeHandler;-access$600-(Lcom/android/gallery3d/ui/ActionModeHandler; Lcom/android/gallery3d/util/ThreadPool$JobContext; I)Landroid/content/Intent;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothHealth$Stub$Proxy;-registerAppConfiguration-(Landroid/bluetooth/BluetoothHealthAppConfiguration; Landroid/bluetooth/IBluetoothHealthCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/IWallpaperManager;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/internal/telephony/PhoneBase;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/videoeditor/VideoEditorActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ImageSwitcher;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/DialogPicker;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/PrintManager;-getInstalledPrintServices-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ContentResolver;-getCurrentSync-()Landroid/content/SyncInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/exchange/adapter/ContactsSyncParser;-commandsParser-()V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/ingest/ui/IngestGridView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-hideWaitForInitialization-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/accessibility/AccessibilityEntityProvider;-createAccessibilityEvent-(Lcom/android/inputmethod/keyboard/Key; I)Landroid/view/accessibility/AccessibilityEvent;' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/calendar/month/MonthByWeekAdapter;-<init>-(Landroid/content/Context; Ljava/util/HashMap;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/CalendarColorPickerDialog$OnCalendarColorSelectedListener;-onColorSelected-(I)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/quicksearchbox/google/GoogleSource;-queryInternal-(Ljava/lang/String;)Lcom/android/quicksearchbox/SourceResult;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/svox/pico/Pico;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/vcard/VCardParser;-parse-(Ljava/io/InputStream; Lcom/android/vcard/VCardInterpreter;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/quicksearchbox/ui/ContactBadge;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/BackDropperEffect;-apply-(I I I I)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/wallpaper/livepicker/LiveWallpaperActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/EasOutboxService;-sendSettings-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Vpn;-startMonitoring-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.BLUETOOTH', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/pinyin/BalloonHint$BalloonView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/CaptivePortalTracker;-sendFailedCaptivePortalCheckBroadcast-(J)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/app/AlertActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/addbookmark/FolderSpinner;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/recurrencepicker/WeekButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-startScan-(Landroid/os/WorkSource;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/printspooler/PrintJobConfigActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/MessageAttachmentTile;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothGatt$1;-onCharacteristicRead-(Ljava/lang/String; I I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/AlbumBrowserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/AddBookmarkPage;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/vcard/VCardParser_V40;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/packageinstaller/InstallAppProgress;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/vcard/VCardParserImpl_V40;-parseOne-(Ljava/io/InputStream;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/deskclock/stopwatch/StopwatchService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/certinstaller/CertInstaller;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothA2dp$Stub;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/WiredAccessoryManager;-updateLocked-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/StringSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/ClusterAlbumSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/calendar/recurrencepicker/WeekButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/stopwatch/StopwatchService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/LocationProviderInterface;-disable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService;-getAllNetworkState-()[Landroid/net/NetworkState;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/SmsUsageMonitor;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/SlidingTab;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hid/HidService$BluetoothInputDeviceBinder;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/camera/ProxyLauncher;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/vcard/VCardEntry$ImData;-constructInsertOperation-(Ljava/util/List; I)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/widget/RemoteViewsService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/gallery3d/data/FilterDeleteSet;-clearDeletion-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/soundrecorder/Recorder;-stopPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/NotificationController;-access$1600-(Landroid/content/Context;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsFragment;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/packageinstaller/InstallFlowAnalytics;-setFlowFinishedWithPackageManagerResult-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget;-onDetachedFromWindow-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/music/MediaPlaybackService$MultiPlayer;-setNextDataSource-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendUSSD-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountAuthenticatorActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/exchange/ExchangeService;-shutdown-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-resetBluetoothSco-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/development/RunningProcesses;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/deskclock/alarms/AlarmStateManager;-registerInstance-(Landroid/content/Context; Lcom/android/deskclock/provider/AlarmInstance; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/magicsmoke/MagicSmokeSelector;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioManager;-setMicrophoneMute-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/calendar/StickyHeaderListView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/gatt/GattService$BluetoothGattBinder;-sendNotification-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; Z [B)V' : ['android.permission.BLUETOOTH'],
	'Landroid/provider/Downloads;-removeAllDownloadsByPackage-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.INTERNET'],
	'Landroid/media/AudioService;-dispatchMediaKeyEventUnderWakelock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameBranch;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/OnePaneController;-showConversation-(Lcom/android/mail/providers/Conversation; Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/packageinstaller/UninstallerActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/data/ComboAlbumSet;-reload-()J' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/IBluetoothGatt;-addDescriptor-(I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/SeekBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/AlbumPage$2;-renderChild-(Lcom/android/gallery3d/glrenderer/GLCanvas; Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/camera/ui/InLineSettingCheckBox;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier$ScreenStateObserver;-handleOnScreenStateChange-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/SinglePhotoPage;-onStateResult-(I I Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/CheckBox;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BlendFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/DayView;-access$4900-(Lcom/android/calendar/DayView; Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/app/CommonControllerOverlay;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AnimatedAdapter;-updateSpecialViews-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/NsdService$NsdStateMachine$DisabledState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/PackageManagerService;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Landroid/media/AudioManager;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/browse/ScrollIndicatorsView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/quicksearchbox/google/AbstractGoogleSource;-getSuggestions-(Ljava/lang/String; I)Lcom/android/quicksearchbox/SuggestionCursor;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/gallery3d/ingest/IngestService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/activity/setup/AccountSetupIncoming;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/support/v4/app/FragmentTabHost;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AnimatedAdapter$Listener;-onAnimationEnd-(Lcom/android/mail/ui/AnimatedAdapter;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/IConnectivityManager$Stub;-setMobileDataEnabled-(Z)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/media/effect/effects/DocumentaryEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/Context;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/browser/NavigationBarTablet;-onAction-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/TextServicesManagerService;-access$100-(Lcom/android/server/TextServicesManagerService; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/noisefield/NoiseField;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/IContentService$Stub;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/exchange/ExchangeService;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/IBluetoothMap$Stub$Proxy;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/gallery3d/app/GalleryAppImpl;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorHueView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/ingest/MtpDeviceIndex$IndexRunnable;-indexDevice-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-clientConnect-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/AlbumPage$MyLoadingListener;-onLoadingFinished-(Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-access$000-(Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MusicPicker;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/GridLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/printspooler/SelectPrinterFragment;-onResume-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/PinyinIME;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/renderscript/RSSurfaceView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-processAtCpbs-(Ljava/lang/String; I)V' : ['android.permission.READ_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/filtershow/ui/FramedTextButton;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/SystemAllowGeolocationOrigins;-access$000-(Lcom/android/browser/SystemAllowGeolocationOrigins;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/TimeBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Editor$HandleView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/adapter/CalendarSyncParser;-parse-()Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/month/SimpleDayPickerFragment;-setMonthDisplayed-(Landroid/text/format/Time; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/svox/pico/CheckVoiceData;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/CallbackFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/view/ScrollerView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransferHistory;-clearAllDownloads-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/browser/Tab$3$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/music/MediaPlaybackService$ServiceStub;-setShuffleMode-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-cancelDiscovery-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/EasAccountService;-searchGal-(Landroid/content/Context; J Ljava/lang/String; I)Lcom/android/exchange/provider/GalResult;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/camera/ui/Switch;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/CantAddAccountActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/alerts/AlertActivity;-onResume-()V' : ['android.permission.READ_CALENDAR'],
	'Landroid/content/ContextWrapper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/vcard/VCardComposer;-initWithRawContactsEntityUri-(Landroid/net/Uri;)Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine$AudioOn;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.READ_CONTACTS', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/calendar/EventInfoActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment$2;-go-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine;-access$5300-(Landroid/net/wifi/p2p/WifiP2pService$P2pStateMachine; Landroid/net/wifi/p2p/WifiP2pConfig;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IUiAutomationConnection$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaRouter$RouteInfo;-requestUpdateVolume-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/LocationManagerService;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/Connection;-proceedAfterWildChar-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/dictionarypack/UpdateHandler;-registerDownloadRequest-(Landroid/app/DownloadManager; Landroid/app/DownloadManager$Request; Landroid/database/sqlite/SQLiteDatabase; Ljava/lang/String; I)J' : ['android.permission.INTERNET'],
	'Landroid/accounts/AccountManager$AmsTask$Response;-onResult-(Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-finishPostLayoutPolicyLw-()I' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth$Stub;-getProfileConnectionState-(I)I' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/mail/photomanager/PhotoManager$PhotoLoaderThread;-loadPhotos-(Ljava/util/Collection;)Ljava/util/Map;' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/base/FrameFetch;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/NumberPicker$CustomEditText;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/provider/AccountReconciler;-reconcileAccounts-(Landroid/content/Context;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/alarmclock/DigitalAppWidgetProvider;-updateClock-(Landroid/content/Context; Landroid/appwidget/AppWidgetManager; I F)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/SwipeableItemView;-dismiss-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/deskclock/widget/EllipsizeLayout;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NsdService$NsdStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/opp/BluetoothOppService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/ReceiverRestrictedContext;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/common/contacts/DataUsageStatUpdater;-update-(Ljava/util/Collection; Ljava/util/Collection; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/deskclock/widget/multiwaveview/GlowPadView;-doFinish-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/DirectoryFragment$DocumentsAdapter;-getDocumentView-(I Landroid/view/View; Landroid/view/ViewGroup;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/FolderItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/packageinstaller/PackageInstallerActivity$9;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/RemoteViewsAdapter$RemoteViewsFrameLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService$CompatMediaPlayer;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/format/DateUtils;-getRelativeTimeSpanString-(J J J I)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/hid/HidService;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/FrameSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/app/FilmstripPage;-access$1800-(Lcom/android/gallery3d/app/PhotoPage;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/telephony/TelephonyManager;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/view/BookmarkExpandableView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-endServiceDeclaration-(I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/latin/setup/SetupWizardActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/accessibility/CaptioningManager;-getFontScale-()F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/ui/ScrollBarView;-removeComponent-(Lcom/android/gallery3d/ui/GLView;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/inputmethod/keyboard/KeyboardView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/provider/FolderPickerActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/btservice/AdapterService;-fetchRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/test/RenamingDelegatingContext;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarViewAdapter;-buildMonthDate-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-access$1300-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/BatteryStatsService$1;-onServiceConnected-(I Landroid/bluetooth/BluetoothProfile;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/InputMethodManagerService;-resetAllInternalStateLocked-(Z Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackService;-enqueue-([J I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/widget/CalendarAppWidgetService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-registerServer-(Landroid/os/ParcelUuid; Landroid/bluetooth/IBluetoothGattServerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/ShutdownActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/AbsListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/TrackBrowserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-onConversationListVisibilityChanged-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/mail/ui/ConversationListView$HintText;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/view/ScrollerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub$Proxy;-setPriority-(Landroid/bluetooth/BluetoothDevice; I)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/exchange/service/EasMeetingResponder;-sendHttpClientPost-(Ljava/lang/String; [B J)Lcom/android/exchange/EasResponse;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/TwoPaneController;-setAccount-(Lcom/android/mail/providers/Account;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/camera/ui/PieMenuButton;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/music/IMediaPlaybackService$Stub;-play-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/gallery3d/filtershow/editors/SwapButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NetworkTimeUpdateService;-systemRunning-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/widget/QuickContactBadge;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/datetimepicker/HapticFeedbackController;-tryVibrate-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/browser/BreadCrumbView;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/service/dreams/DreamService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/mail/ui/ConversationViewFragment;-setExtraUserVisibleHint-(Z)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/datetimepicker/date/SimpleMonthView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneFactory;-getCdmaPhone-()Lcom/android/internal/telephony/Phone;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/music/WeekSelector;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/ConversationsInOutboxTipView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiStateMachine$WpsRunningState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/MessageFooterView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/google/GoogleSearch;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/NetworkIdentity;-buildNetworkIdentity-(Landroid/content/Context; Landroid/net/NetworkState;)Landroid/net/NetworkIdentity;' : ['android.permission.ACCESS_WIFI_STATE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/ingest/ui/DateTileView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/RecentApplicationsBackground;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/UserManagerService$5;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/music/MediaPlaybackActivity;-access$800-(Lcom/android/music/MediaPlaybackActivity;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/camera/PhotoModule;-initializeFirstTime-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/mail/ui/MailActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/DeviceStorageMonitorService;-checkMemory-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.CLEAR_APP_CACHE'],
	'Landroid/inputmethodservice/ExtractEditText;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/AttachmentDownloadService$DownloadSet;-endDownload-(J I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/browser/AutologinBar;-onClick-(Landroid/view/View;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/bluetooth/btservice/AdapterApp;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/map/BluetoothMapContentObserver;-pushMmsToFolder-(I Ljava/lang/String; Lcom/android/bluetooth/map/BluetoothMapbMessageMmsEmail;)J' : ['android.permission.READ_SMS'],
	'Lcom/android/calendar/EventInfoFragment;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/camera/LocationManager;-startReceivingLocationUpdates-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/development/Connectivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/email/activity/setup/AccountSetupOutgoing;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/TabScrollView$TabLayout;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGattCallback$Stub$Proxy;-onCharacteristicRead-(Ljava/lang/String; I I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; [B)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/MovieActivity$1;-onPlayPause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetooth;-removeBond-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/providers/downloads/ui/DownloadList;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/PermissionDetails;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/a2dp/A2dpStateMachine;-broadcastConnectionState-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/os/IVibratorService;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/email/service/PopImapSyncAdapterService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/AdapterViewAnimator;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/BookmarkSearch;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController;-updateConversation-(Ljava/util/Collection; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothInputDevice$Stub;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/DeviceStorageMonitorService;-cancelFullNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcController$DccDefaultState;-onDataStateChanged-(Ljava/util/ArrayList;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/videoeditor/widgets/ImageViewTouchBase;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/SupplicantStateTracker$UninitializedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.SEND_SMS', 'android.permission.INTERNET', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.READ_CALL_LOG', 'android.permission.BLUETOOTH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS', 'android.permission.BLUETOOTH_ADMIN', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/compose/CcBccView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasSyncContacts;-cleanup-(Landroid/content/Context; Lcom/android/emailcommon/provider/Account;)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/bluetooth/BluetoothA2dp;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-pollBatchedScan-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/deskclock/ScreensaverSettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransfer$SocketConnectThread;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/deskclock/alarms/AlarmActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/alarms/AlarmService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/inputmethod/latin/LatinIME;-handleCharacter-(I I I I)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/photos/AlbumActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/browser/Controller;-onBackKey-()V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/accessibility/TouchExplorer$SendHoverExitDelayed;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/photos/views/HeaderGridView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/printspooler/PrintJobConfigActivity$CustomEditText;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/documentsui/DocumentsApplication;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothGatt$Stub;-endReliableWrite-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/ActivityController;-onRestoreInstanceState-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/emailsync/SyncManager;-stopAccountSyncs-(J Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcInactiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RatingBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/btservice/AdapterService;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/ViewAnimator;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/ConnectivityManager;-getProvisioningOrActiveNetworkInfo-()Landroid/net/NetworkInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/inputmethod/latin/ExpandableBinaryDictionary$14;-run-()V' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/view/SurfaceView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/pinyin/ComposingView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/bluetooth/hfp/HeadsetStateMachine;-access$1000-(Lcom/android/bluetooth/hfp/HeadsetStateMachine; Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothHeadset;-connect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/mail/browse/MessageHeaderView;-onClick-(Landroid/view/View; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavScreen;-access$100-(Lcom/android/browser/NavScreen; Lcom/android/browser/Tab;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/inputmethod/research/Uploader;-isPossibleToUpload-()Z' : ['android.permission.INTERNET'],
	'Lcom/android/browser/Controller;-openTab-(Lcom/android/browser/IntentHandler$UrlData;)Lcom/android/browser/Tab;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/am/ActivityManagerService;-killPackageProcessesLocked-(Ljava/lang/String; I I I Z Z Z Z Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-access$100-(Lcom/android/internal/telephony/IccSmsInterfaceManager; [Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/mail/ui/MailActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothGattServer;-addService-(Landroid/bluetooth/BluetoothGattService;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/browse/ConversationItemView;-onTouchEventNoSwipe-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/deskclock/AndroidClockTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/adapter/CalendarSyncParser;-getServerIdCursor-(Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/datetimepicker/time/RadialPickerLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-access$400-(Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SmsManager;-deleteMessageFromIcc-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/calendar/SearchActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/backup/FullBackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-access$300-(Lcom/android/server/VibratorService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/widget/MediaController;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/service/PolicyService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/ingest/ui/MtpImageView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-initSstIcc-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/LockPatternView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/settings/GallerySettings;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/musicvis/vis5/Visualization5;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/alerts/AlertReceiver;-buildBasicNotification-(Landroid/app/Notification$Builder; Landroid/content/Context; Ljava/lang/String; Ljava/lang/String; J J J I Z I Z)Landroid/app/Notification;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/view/menu/ActionMenuPresenter$OverflowMenuButton;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AbstractConversationViewFragment$MessageLoaderCallbacks;-onLoadFinished-(Landroid/content/Loader; Ljava/lang/Object;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/nfc/cardemulation/OffHostApduService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hid/HidService;-okToConnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/GrainFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/ingest/ui/IngestGridView;-sequenceScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/email/activity/setup/AccountSetupOptions;-optionsComplete-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/hfp/HeadsetService$BluetoothHeadsetBinder;-isAudioOn-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/google/common/io/Files;-toByteArray-(Ljava/io/File;)[B' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/test/ModelInterpreter;-<init>-(Ljava/net/InetSocketAddress;)V' : ['android.permission.INTERNET'],
	'Landroid/bluetooth/IBluetoothGattCallback$Stub$Proxy;-onCharacteristicWrite-(Ljava/lang/String; I I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/mail/ui/SearchMailActionBarView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/PrinterDiscoverySession;-startPrinterStateTracking-(Landroid/print/PrinterId;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/camera/app/OrientationManager;-resume-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-setWhereFilterOriginator-(Lcom/android/bluetooth/map/BluetoothMapAppParams; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo;)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/widget/DialogTitle;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCfuQueryResult-([Lcom/android/internal/telephony/CallForwardInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-isScanAlwaysAvailable-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/inputmethod/keyboard/EmojiPalettesView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/InputView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/deskclock/alarms/AlarmService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/ui/AbstractActivityController$FolderDestruction;-performAction-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/datetimepicker/date/DatePickerDialog$1;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CALENDAR', 'android.permission.VIBRATE'],
	'Landroid/net/CaptivePortalTracker;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService$Stub;-adjustLocalOrRemoteStreamVolume-(I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/music/AlbumBrowserActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/inputmethod/keyboard/internal/KeyboardState;-onRestoreKeyboardState-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityStackSupervisor;-checkReadyForSleepLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/gallery3d/filtershow/imageshow/ImageCurves;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/ConversationSelectionSet;-delete-(Ljava/util/ArrayList;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/browser/search/OpenSearchSearchEngine;-isNetworkConnected-(Landroid/content/Context;)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/soundrecorder/SoundRecorder$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/ExchangeService;-done-(Lcom/android/emailsync/AbstractSyncService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/gallery3d/app/StateManager;-pause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiManager;-reassociate-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/gallery3d/app/AlbumSetPage;-onPause-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDataConnectionDetached-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeech;-stop-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/data/LocalMergeAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer; I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/calendar/recurrencepicker/RecurrencePickerDialog;-onCreateView-(Landroid/view/LayoutInflater; Landroid/view/ViewGroup; Landroid/os/Bundle;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/gallery3d/app/Gallery;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-moveTaskBackwards-(I)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/bluetooth/BluetoothAdapter;-enable-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/camera/ui/RotateImageView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/NavigationBarTablet;-onClick-(Landroid/view/View;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-binderDied-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/gallery3d/app/PhotoPage;-onSingleTapUp-(I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-resetRadio-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/IsolatedContext$MockAccountManager;-hasFeatures-(Landroid/accounts/Account; [Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/music/PlaylistBrowserActivity;-onOptionsItemSelected-(Landroid/view/MenuItem;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hdp/HealthService;-getHealthDeviceConnectionState-(Landroid/bluetooth/BluetoothDevice;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/ConnectivityService;-access$1800-(Lcom/android/server/ConnectivityService; I Z)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/LinkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/photos/shims/MediaSetLoader;-urisForSubItems-(Ljava/lang/Object;)Ljava/util/ArrayList;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/DialogPicker;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/camera/ui/ZoomView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-maybeStartNextSyncLocked-()J' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/AbsListView;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/os/Vibrator;-cancel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/keychain/KeyChainActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/compose/ComposeActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/IBatteryPropertiesListener$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManager;-confirmCredentials-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/calculator2/Calculator;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/gallery3d/app/ControllerOverlay$Listener;-onSeekStart-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/gallery3d/data/LocalAlbumSet;-getLocalAlbum-(Lcom/android/gallery3d/data/DataManager; I Lcom/android/gallery3d/data/Path; I Ljava/lang/String;)Lcom/android/gallery3d/data/MediaSet;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/ActivityManager;-restartPackage-(Ljava/lang/String;)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-separate-(Lcom/android/internal/telephony/cdma/CdmaConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService$Stub;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Ljava/lang/String; J F Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/email/service/EasTestAuthenticatorService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/adapter/AttachmentLoader;-loadAttachment-()V' : ['android.permission.WAKE_LOCK', 'android.permission.INTERNET', 'android.permission.READ_PHONE_STATE'],
	'Landroid/widget/Editor$SelectionEndHandleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/AccountService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityManagerService;-setWindowManager-(Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/event/AttendeesView;-addOneAttendee-(Lcom/android/calendar/CalendarEventModel$Attendee;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/bluetooth/BluetoothSocket;-initSocketNative-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/exchange/eas/EasProvision;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/dreams/phototable/PhotoCarousel;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/PageDialogsHandler;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/WindowManagerPolicy;-interceptKeyBeforeDispatching-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)J' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-getTetherableWifiRegexs-()[Ljava/lang/String;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/BitmapSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/exchange/ExchangeService$CalendarObserver;-onChangeInBackground-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CALENDAR'],
	'Lcom/android/quicksearchbox/ui/SuggestionsView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/smspush/WapPushManager;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ActivityManagerNative;-broadcastStickyIntent-(Landroid/content/Intent; Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ex/photo/PhotoViewActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/NotificationManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/StatusBarManagerService; Lcom/android/server/LightsService;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiStateMachine$DriverStartedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK'],
	'Landroid/app/TabActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/dreams/phototable/PicasaSource;-<init>-(Landroid/content/Context; Landroid/content/SharedPreferences;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/service/Pop3SyncAdapterService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/alerts/AlertService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/Session;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/mail/browse/SuperCollapsedBlock$OnClickListener;-onSuperCollapsedClick-(Lcom/android/mail/browse/ConversationViewAdapter$SuperCollapsedBlockItem;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/app/PhotoPage;-refreshHidingMessage-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/soundrecorder/VUMeter;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-access$3100-(Lcom/android/internal/telephony/dataconnection/DataConnection; Lcom/android/internal/telephony/dataconnection/DataConnection$ConnectionParams;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/dreams/phototable/FlipperDreamSettings;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/email/service/EmailServiceStub;-requestSync-(J Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneBase;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/browser/ErrorConsoleView$ErrorConsoleListView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/packageinstaller/GrantActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/gallery3d/filtershow/colorpicker/ColorSVRectView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/bluetooth/opp/BluetoothOppTransfer;-startObexSession-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-moveTaskToBack-(I)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/app/LauncherActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl;-processCommand-(Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-changeBarringPassword-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/ExtractEditText;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/packageinstaller/TabsAdapter;-onTabChanged-(Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/LockSettingsService;-getLong-(Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/galaxy4/Galaxy4;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-setCdmaBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/NullFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/calendar/month/MonthListView$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/calendar/alerts/GlobalDismissManager;-lookupCalendarToAccountMap-(Landroid/content/Context; Ljava/util/Set;)Ljava/util/Map;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/providers/telephony/SmsProvider;-deleteMessageFromIcc-(Ljava/lang/String;)I' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/accounts/AccountManager;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/widget/SearchView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/quicksearchbox/util/JavaNetHttpHelper;-post-(Lcom/android/quicksearchbox/util/HttpHelper$PostRequest;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/music/CreatePlaylist;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/mail/browse/MessageHeaderView;-bind-(Lcom/android/mail/browse/ConversationViewAdapter$MessageHeaderItem; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS'],
	'Landroid/net/ConnectivityManager;-getAllLinkQualityInfo-()[Landroid/net/LinkQualityInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/BluetoothHeadset;-getDevicesMatchingConnectionStates-([I)Ljava/util/List;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/exchange/adapter/ItemOperationsParser;-readChunked-(Ljava/io/InputStream; Ljava/io/OutputStream; J Lcom/android/exchange/eas/EasLoadAttachment$ProgressCallback;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/backup/FullBackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/test/mock/MockContentResolver;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/media/IAudioService;-adjustVolume-(I I Ljava/lang/String;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/bluetooth/gatt/GattService;-addIncludedService-(I I I Ljava/util/UUID;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/text/format/DateUtils;-getRelativeTimeSpanString-(Landroid/content/Context; J Z)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-changeIccFdnPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/GLTextureTarget;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/emailsync/SyncManager;-runAwake-(J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/browser/Tab$7;-onDownloadStart-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.INTERNET', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/wifi/WifiController$DefaultState;-exit-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-handleUpdateNetworkState-(I Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/gallery3d/app/PhotoPage$4;-removeOneComponent-(Lcom/android/gallery3d/ui/GLView;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/backup/BackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/latin/personalization/AccountUtils;-getAccounts-(Landroid/content/Context;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AbsSeekBar;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-separateConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager$Stub$Proxy;-getPrintJobInfo-(Landroid/print/PrintJobId; I I)Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/mail/ui/AttachmentTile;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/CatService;-handleCmdResponse-(Lcom/android/internal/telephony/cat/CatResponseMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/inputmethod/pinyin/ComposingView;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/inputmethod/latin/LatinIME;-swapSwapperAndSpace-()V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterfw/core/FilterFunction;-tearDown-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameBranch;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/Notifier;-sendNextBroadcast-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/bluetooth/IBluetooth;-setPin-(Landroid/bluetooth/BluetoothDevice; Z I [B)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/printspooler/NotificationController$NotificationBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/updates/TZInfoInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-disableFeatures-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/CalendarColorSquare;-assignContactFromPhone-(Ljava/lang/String; Z)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofenceMonitorStatus-(I I Landroid/location/Location;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/email/service/ImapService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.INTERNET', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/bluetooth/map/BluetoothMapContent;-matchOriginator-(Landroid/database/Cursor; Lcom/android/bluetooth/map/BluetoothMapContent$FilterInfo; Lcom/android/bluetooth/map/BluetoothMapAppParams;)Z' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/email/provider/FolderPickerActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/gatt/GattService;-serverConnect-(I Ljava/lang/String; Z)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/calendar/agenda/AgendaWindowAdapter;-access$2400-(Lcom/android/calendar/agenda/AgendaWindowAdapter; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/calendar/selectcalendars/SelectVisibleCalendarsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityManagerService;-skipPendingBroadcastLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/google/common/io/CharStreams$4;-getInput-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/mail/ui/AttachmentTile;-thumbnailLoadFailed-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/bluetooth/IBluetoothHeadset$Stub$Proxy;-disconnect-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/browser/DataController;-<init>-(Landroid/content/Context;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/calendar/month/MonthByWeekFragment;-onResume-()V' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/packageinstaller/UninstallerActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/inputmethod/pinyin/PinyinIME;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/StackView$StackFrame;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/development/ProcessInfo;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/SnailAlbum;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/mail/ui/TwoPaneController;-handleIntent-(Landroid/content/Intent;)V' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/content/SyncActivityTooManyDeletes;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/MusicPicker;-stopMediaPlayer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ListPopupWindow$DropDownListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothGatt$Stub$Proxy;-readCharacteristic-(I Ljava/lang/String; I I Landroid/os/ParcelUuid; I Landroid/os/ParcelUuid; I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/INetworkPolicyManager;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/inputmethod/latin/spellcheck/SpellCheckerSettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothMap;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/ConnectivityService;-requestRouteToHostAddress-(I [B Ljava/lang/String;)Z' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-switchWaitingOrHoldingAndActive-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/common/io/Resources;-readLines-(Ljava/net/URL; Ljava/nio/charset/Charset;)Ljava/util/List;' : ['android.permission.INTERNET'],
	'Lcom/android/email/service/LegacyImapAuthenticatorService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/gallery3d/data/FilterTypeSet;-enumerateMediaItems-(Lcom/android/gallery3d/data/MediaSet$ItemConsumer;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/gallery3d/app/ManageCachePage$1;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/magicsmoke/RenderScriptWallpaper$RenderScriptEngine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/gallery3d/ui/MenuExecutor;-onMenuClicked-(I Lcom/android/gallery3d/ui/MenuExecutor$ProgressListener; Z Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/exchange/eas/EasOutboxSync;-expandedAddDeviceInformationToSerializer-(Lcom/android/exchange/adapter/Serializer; Landroid/content/Context; Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/bluetooth/opp/BluetoothOppBtEnableActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/test/IsolatedContext;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/TrackBrowserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/SearchActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-unregisterForSuppServiceNotification-(Landroid/os/Handler;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/calendar/CalendarViewAdapter$1;-run-()V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/gallery3d/data/DownloadUtils;-download-(Lcom/android/gallery3d/util/ThreadPool$JobContext; Ljava/net/URL; Ljava/io/OutputStream;)Z' : ['android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/ImmersiveModeConfirmation$ClingWindowView;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/browser/ComboViewActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/camera/ui/ExpandedGridView;-arrowScroll-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/soundrecorder/SoundRecorder;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/calendar/MultiStateButton;-access$800-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetooth;-getScanMode-()I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/gallery3d/app/Gallery;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/exchange/eas/EasPing;-init-(Z)Z' : ['android.permission.READ_CALENDAR', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/exchange/Exchange;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/galaxy4/Galaxy4Wallpaper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothAdapter;-startLeScan-([Ljava/util/UUID; Landroid/bluetooth/BluetoothAdapter$LeScanCallback;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/net/CaptivePortalTracker$DefaultState;-enter-()V' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.READ_PHONE_STATE', 'android.permission.BLUETOOTH', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/accounts/AccountManagerService;-onUserStarted-(Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothGatt;-connect-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/widget/BookmarkWidgetConfigure;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/development/MediaScannerActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetooth$Stub$Proxy;-getRemoteUuids-(Landroid/bluetooth/BluetoothDevice;)[Landroid/os/ParcelUuid;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/power/PowerManagerService;-scheduleSandmanLocked-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.READ_PROFILE'],
	'Landroid/provider/CallLog$Calls;-addCall-(Lcom/android/internal/telephony/CallerInfo; Landroid/content/Context; Ljava/lang/String; I I J I)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.WRITE_CALL_LOG', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/filtershow/category/CategoryTrack;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/datetimepicker/date/DatePickerDialog;-tryVibrate-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/music/MediaPlaybackActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-setLockPassword-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockApplication;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/content/ContentService;-systemReady-()V' : ['android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.ACCESS_NETWORK_STATE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/browser/SuggestionsAdapter$CombinedCursor;-runQuery-(Ljava/lang/CharSequence;)V' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/deskclock/worldclock/CitiesActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/music/DeleteItems;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/bluetooth/hdp/HealthService;-broadcastHealthDeviceStateChange-(Landroid/bluetooth/BluetoothDevice; I)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/IConnectivityManager$Stub$Proxy;-startLegacyVpn-(Lcom/android/internal/net/VpnProfile;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/email/service/EasAuthenticatorService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothDevice;-cancelPairingUserInput-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/exchange/service/EmailSyncAdapterService;-onStartCommand-(Landroid/content/Intent; I I)I' : ['android.permission.WRITE_SYNC_SETTINGS'],
}

AOSP_PERMISSIONS_BY_FIELDS = {
	'Landroid/provider/ContactsContract;-AUTHORITY_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$Directory;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Callable;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Settings$Global;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/ContactsContract$DataUsageFeedback;-FEEDBACK_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$StreamItems;-CONTENT_LIMIT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$DeletedContacts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Mms$Inbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$Mms$Draft;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/BrowserContract$Bookmarks;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Contacts$ContactMethods;-CONTENT_EMAIL_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Downloads$Impl;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.INTERNET'],
	'Landroid/provider/Telephony$MmsSms$PendingMessages;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$Attendees;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Settings$System;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/providers/telephony/SmsProvider;-NOTIFICATION_URI-Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$AggregationExceptions;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_CONVERSATIONS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/CallLog$Calls;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG'],
	'Landroid/provider/Telephony$Sms$Draft;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/Contacts$GroupMembership;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Sms$Inbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Lcom/android/common/contacts/DataUsageStatUpdater$DataUsageFeedback;-FEEDBACK_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$Profile;-CONTENT_RAW_CONTACTS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Groups;-DELETED_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/email/activity/setup/AccountSetupNames;-PROFILE_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/gallery3d/provider/GalleryProvider;-BASE_URI-Landroid/net/Uri;' : ['com.android.gallery3d.permission.GALLERY_PROVIDER'],
	'Landroid/provider/ContactsContract$Data;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Threads;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$StreamItems;-CONTENT_PHOTO_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$Profile;-CONTENT_VCARD_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$Settings;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract$Images;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/browser/provider/BrowserProvider2$OmniboxSuggestions;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/VoicemailContract$Status;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/provider/ContactsContract$DataUsageFeedback;-DELETE_USAGE_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Mms;-REPORT_REQUEST_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/BrowserContract$SyncState;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$MmsSms;-SEARCH_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Contacts$Presence;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/provider/SnapshotProvider$Snapshots;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/BrowserContract$Searches;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/SmsBroadcastUndelivered;-sRawUri-Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Contactables;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Phones;-CONTENT_FILTER_URL-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-sRawUri-Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$Mms$Rate;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$RawContactsEntity;-PROFILE_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$Colors;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_VCARD_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$CalendarEntity;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Telephony$Sms$Sent;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_BY_DAY_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Contacts$GroupMembership;-RAW_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Contactables;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/photos/data/PhotoSetLoader;-GLOBAL_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_EXTERNAL_STORAGE', 'android.permission.WRITE_EXTERNAL_STORAGE'],
	'Lcom/android/bluetooth/opp/BluetoothShare;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.ACCESS_BLUETOOTH_SHARE'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_GROUP_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_STREQUENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$Events;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_FREQUENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$SyncState;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_DRAFT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CallLog$Calls;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG'],
	'Landroid/provider/Settings$Secure;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/Contacts$People;-WITH_EMAIL_OR_IM_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$DisplayPhoto;-CONTENT_MAX_DIMENSIONS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$ProviderStatus;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$Reminders;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_MULTI_VCARD_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-CONTENT_URI_BY_INSTANCE-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/notification/DemoContactNotificationScorer;-CONTACTS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract$Settings;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_LOCKED_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$Groups;-CONTENT_SUMMARY_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_UNDELIVERED_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/VoicemailContract$Voicemails;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Email;-CONTENT_LOOKUP_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Settings;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/calendar/alerts/InitAlarmsService;-SCHEDULE_ALARM_REMOVE_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/BrowserContract;-AUTHORITY_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$PhoneLookup;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/providers/telephony/SmsProvider;-ICC_URI-Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/Contacts$Organizations;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract$Accounts;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$Threads;-THREAD_ID_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$DisplayPhoto;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$StatusUpdates;-PROFILE_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract$Bookmarks;-CONTENT_URI_DEFAULT_FOLDER-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/CalendarContract$Events;-CONTENT_EXCEPTION_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Contacts$People;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_SEARCH_BY_DAY_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Telephony$Mms$Sent;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Phone;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Photos;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_SEARCH_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/ex/chips/BaseRecipientAdapter$DirectoryListQuery;-URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$Groups;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Browser;-BOOKMARKS_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$Mms$Outbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$Calendars;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/common/contacts/BaseEmailAddressAdapter$DirectoryListQuery;-URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_FILTER_BYPHONE_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/providers/userdictionary/UserDictionaryProvider;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Landroid/provider/ContactsContract$SyncState;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract$History;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/UserDictionary$Words;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Landroid/provider/Contacts$ContactMethods;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Browser;-SEARCHES_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/BrowserContract$ImageMappings;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/CalendarContract$EventsEntity;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Telephony$Mms;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$ProfileSyncState;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$StructuredPostal;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Extensions;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Phones;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$CellBroadcasts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CELL_BROADCASTS'],
	'Landroid/provider/Telephony$Sms;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$Profile;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract$Combined;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_STREQUENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Downloads$Impl;-ALL_DOWNLOADS_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.ACCESS_ALL_DOWNLOADS'],
	'Landroid/provider/UserDictionary;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Landroid/provider/Telephony$Sms$Outbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$Sms$Conversations;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_CONTACTS', 'android.permission.READ_SMS'],
	'Landroid/provider/Settings$Bookmarks;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/CallLog;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG'],
	'Landroid/provider/Telephony$Mms;-REPORT_STATUS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Email;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$RawContacts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Groups;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CallLog$Calls;-CONTENT_URI_WITH_VOICEMAIL-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG'],
	'Lcom/android/gallery3d/filtershow/provider/SharedImageProvider;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.gallery3d.filtershow.permission.WRITE', 'com.android.gallery3d.filtershow.permission.READ'],
	'Landroid/provider/Telephony$Threads;-OBSOLETE_THREADS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$EventDays;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$StreamItems;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$People;-DELETED_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$StatusUpdates;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/provider/BrowserProvider2$Thumbnails;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/ContactsContract$PinnedPositions;-UPDATE_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Callable;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$RawContactsEntity;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$ExtendedProperties;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Email;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/music/MusicUtils;-sArtworkUri-Landroid/net/Uri;' : ['android.permission.READ_EXTERNAL_STORAGE', 'android.permission.WRITE_EXTERNAL_STORAGE'],
	'Landroid/provider/Contacts$People;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/browser/provider/SnapshotProvider;-AUTHORITY_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'android.permission.GET_ACCOUNTS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/CalendarContract$CalendarCache;-URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_LOOKUP_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Phone;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
}
#################################################
