#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is a part of Androguard.
#
# This file is generated automatically from the data
# provided by PScout tool [http://pscout.csl.toronto.edu/]
# using script: extract_api_permission_mappings.py
#
# Author: Yury Zhauniarovich
#
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS-IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License"

#################################################
### API version: 21 
#################################################


AOSP_PERMISSIONS_BY_METHODS = {
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handlePinMmi-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/GLTextureTarget;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ims/internal/IImsEcbmListener;-exitedECBM-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/PackageManagerService;-access$3900-(Lcom/android/server/pm/PackageManagerService; Ljava/lang/String; I I)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getConnectionStatistics-()Landroid/net/wifi/WifiConnectionStatistics;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/SearchView$SearchAutoComplete;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/VpnService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-forceStopPackageLocked-(Ljava/lang/String; I Z Z Z Z Z I Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/connectivity/PacManager;-get-(Landroid/net/Uri;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/net/wifi/WifiManager$WifiLock;-acquire-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/VoicemailContract$Status;-buildSourceUri-(Ljava/lang/String;)Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Lcom/android/server/wm/WindowManagerService;-addWindow-(Lcom/android/server/wm/Session; Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getUidTagPersistBytes-(J)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getGlobalAlertBytes-(J)J' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-enableSystemApp-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$TetheredState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$SoftApStartedState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/view/IWindowManager;-inputMethodClientHasFocus-(Lcom/android/internal/view/IInputMethodClient;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/ims/internal/IImsCallSessionListener$Stub$Proxy;-callSessionHoldFailed-(Lcom/android/ims/internal/IImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-moveStackWindowsLocked-(Lcom/android/server/wm/DisplayContent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CropFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/display/AutomaticBrightnessController;-<init>-(Lcom/android/server/display/AutomaticBrightnessController$Callbacks; Landroid/os/Looper; Landroid/hardware/SensorManager; Landroid/util/Spline; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/WifiController$FullHighPerfLockHeldState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UiccCard;-iccCloseLogicalChannel-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl$2;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-openPanelsAfterRestore-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub;-switchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/SmsMessage;-calculateLength-(Ljava/lang/CharSequence; Z)Lcom/android/internal/telephony/GsmAlphabet$TextEncodingDetails;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/base/FrameSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/SupplicantStateTracker;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-enableScreenIfNeeded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/statusbar/IStatusBarService;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/accounts/IAccountManager$Stub;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ConsumerIrService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/internal/policy/impl/PolicyControl;-reloadFromSetting-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-saveLockPassword-(Ljava/lang/String; I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-setLockTaskModeLocked-(Lcom/android/server/am/TaskRecord; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-addAppWidget-(I I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/SimpleMonthView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/MediaRecorder;-setVideoSource-(I)V' : ['android.permission.CAMERA'],
	'Landroid/preference/SeekBarVolumizer;-onStartSample-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/GLTextureSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/hdmi/HdmiCecFeatureAction$ActionTimerHandler;-sendTimerMessage-(I J)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setScreenCaptureDisabled-(Landroid/content/ComponentName; I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processPlayTone-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/TelephonyManager;-getDeviceSoftwareVersion-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/print/IPrintManager;-stopPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/Session;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/RampAnimator;-animateTo-(I I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/TextClock;-access$000-(Landroid/widget/TextClock;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-deleteStoredConversation-(Ljava/lang/String; J)Z' : ['android.permission.WRITE_SMS'],
	'Landroid/content/MutableContextWrapper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager$Stub;-getPrintJobInfos-(I I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/gsm/SmsMessage;-getSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Z)Landroid/telephony/gsm/SmsMessage$SubmitPdu;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/LocationManagerService$4;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-doVisibilityChanged-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-doNotifyDataConnection-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/NfcAdapter;-removeNfcUnlockHandler-(Landroid/nfc/NfcAdapter$NfcUnlockHandler;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ExpandableListActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/RecentApplicationsBackground;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/UiModeManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountAuthenticatorActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/IActivityManager;-unbroadcastIntent-(Landroid/app/IApplicationThread; Landroid/content/Intent; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ActionBarContainer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getApplicationRestrictions-(Landroid/content/ComponentName; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms;-copyMessageToIccEf-(Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/print/UserState;-enableSystemPrintServicesLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardware;-pauseGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/videosrc/MediaSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/Vibrator;-vibrate-(J Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Landroid/widget/TextClock;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/PhoneNumberUtils;-isEmergencyNumber-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-proceedAfterWildChar-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$SupplicantStartedState;-enter-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceResumeStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/RotationPolicy;-isRotationLockToggleVisible-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/MifareClassic;-decrement-(I I)V' : ['android.permission.NFC'],
	'Lcom/android/contacts/ContactsListActivity;-getUriToQuery-()Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/widget/TextClock;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiNotificationController$NotificationEnabledSettingObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendRawPdu-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/accessibilityservice/IAccessibilityServiceConnection$Stub;-getWindows-()Ljava/util/List;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/ShutdownActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/RttService$RttServiceImpl$RttStateMachine$DefaultState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/print/IPrintManager$Stub;-startPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Chronometer;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/dreams/DreamManagerService;-chooseDreamForUser-(Z I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-importTextMessage-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; J Z Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/UiccPhoneBookController;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/ISub;-getDefaultSmsSubId-()J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/UsimDataDownloadHandler;-sendSmsAckForEnvelopeResponse-(Lcom/android/internal/telephony/uicc/IccIoResult; I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/PlatLogoActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Telephony$Sms$Sent;-addMessage-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long;)Landroid/net/Uri;' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendTouchExplorationGestureStartAndHoverEnterIfNeeded-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setInternalDataEnabled-(Z Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-registerReceiver-(Landroid/app/IApplicationThread; Ljava/lang/String; Landroid/content/IIntentReceiver; Landroid/content/IntentFilter; Ljava/lang/String; I)Landroid/content/Intent;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enableGsmBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/media/IAudioService;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/wifi/SupplicantStateTracker$DisconnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/backup/BackupManagerService$PerformBackupTask;-finalizeBackup-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ZoomButtonsController$Container;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lorg/apache/http/impl/client/DefaultHttpClient;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Landroid/service/dreams/Sandman;-startDreamWhenDockedIfAppropriate-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiManager;-isDeviceToApRttSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/app/LocalePicker;-updateLocale-(Ljava/util/Locale;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RadioButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-setOperatorBrandOverride-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/commands/bmgr/Bmgr;-doListRestoreSets-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-setColor-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/dreams/DreamManagerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/VibratorService;-onInputDeviceRemoved-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/RadioGroup;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine;-handleNetworkDisconnect-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneProxy;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-updatePhoneObject-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-getInstalledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-acceptCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation$Stub;-registerAidGroupForService-(I Landroid/content/ComponentName; Landroid/nfc/cardemulation/AidGroup;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/NetworkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager;-getAllowScansWithTraffic-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/nfc/INfcAdapter;-setReaderMode-(Landroid/os/IBinder; Landroid/nfc/IAppCallback; I Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DatePicker$DatePickerSpinnerDelegate;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-handleOnRotationChanged-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-setOperatorBrandOverride-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/service/wallpaper/WallpaperService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateStart;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-getVerboseLoggingLevel-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/INetworkPolicyManager$Stub;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-notifyAndAcknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-hangupConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SubscriptionManager;-getSubInfoUsingIccId-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/AudioService;-checkZen-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager;-createPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/StatusBarManager;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/nfc/NfcAdapter;-setBeamPushUrisCallback-(Landroid/nfc/NfcAdapter$CreateBeamUrisCallback; Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Lcom/android/server/search/SearchManagerService$MyPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/NativeActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/location/GpsLocationProvider;-updateLowPowerMode-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/ClosedCaptionWidget$CCLineBox;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AsyncPlayer;-stop-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/app/LocalePicker;-constructAdapter-(Landroid/content/Context; I I)Landroid/widget/ArrayAdapter;' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/NfcAdapter;-enableForegroundDispatch-(Landroid/app/Activity; Landroid/app/PendingIntent; [Landroid/content/IntentFilter; [[Ljava/lang/String;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/SMSDispatcher$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/MediaSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPasswordMinimumLength-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/NotificationManagerService$5;-setAutomaticZenModeConditions-([Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisToFileOutputStreamSpeechItemV1;-setRequestParams-(Landroid/speech/tts/SynthesisRequest;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayPowerController;-handleProximitySensorEvent-(J Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-wakingUp-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/AlertActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-updateDataConnectionTracker-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-enforceCanManageCaCerts-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/backup/LocalTransportService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/inputmethodservice/InputMethodService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/voiceinteraction/SoundTriggerHelper;-stopRecognition-(I Landroid/hardware/soundtrigger/IRecognitionStatusCallback;)I' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/DigitalClock;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-requestShutdown-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/RingtoneManager;-getActualDefaultRingtoneUri-(Landroid/content/Context; I)Landroid/net/Uri;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-disableCellBroadcastRangeForSubscriber-(J I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/bluetooth/BluetoothSocket;-<init>-(I I Z Z Landroid/bluetooth/BluetoothDevice; I Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/input/InputManagerService;-start-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/speech/tts/ITextToSpeechService$Stub;-speak-(Landroid/os/IBinder; Ljava/lang/CharSequence; I Landroid/os/Bundle; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService;-access$200-(Lcom/android/server/VibratorService;)V' : ['android.permission.VIBRATE'],
	'Landroid/widget/CheckedTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/Phone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-dispatchUnhandledKey-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/VpnService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcDisconnectionErrorCreatingConnection;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccPhoneBookController;-getAdnRecordsInEfForSubscriber-(J I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/app/NetInitiatedActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextClock$1;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothAdapter;-createNewRfcommSocketAndRecord-(Ljava/lang/String; Ljava/util/UUID; Z Z)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ISms$Stub;-getAllMessagesFromIccEfForSubscriber-(J Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/videosrc/MediaSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/uiautomator/testrunner/UiAutomatorTestRunner;-start-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/sip/SipService;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Landroid/accounts/IAccountManager$Stub;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover$Callback;-onBluetoothPeripheralHandoverComplete-(Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/uicc/UiccCard;-iccTransmitApduBasicChannel-(I I I I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/View$AttachInfo$Callbacks;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-shutdownRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/server/content/SyncStorageEngine;-setSyncableStateForEndPoint-(Lcom/android/server/content/SyncStorageEngine$EndPoint; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-createPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/lights/LightsService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/CompatModeDialog$1;-onCheckedChanged-(Landroid/widget/CompoundButton; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/INetworkPolicyManager;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/PersistentDataBlockService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/media/ClosedCaptionWidget;-onAttachedToWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BootReceiver;-removeOldUpdatePackages-(Landroid/content/Context;)V' : ['android.permission.INTERNET'],
	'Landroid/media/IAudioService;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/speech/RecognitionService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$MyWakeGestureListener;-onWakeUp-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationBlacklist;-getStringArrayLocked-(Ljava/lang/String;)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/TechListChooserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/Service;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-setInitialAttachApn-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/AccessibilityController$DisplayMagnifier$MagnifiedViewport;-<init>-(Lcom/android/server/wm/AccessibilityController$DisplayMagnifier;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/hdmi/HdmiControlService;-initializeLocalDevices-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/svc/PowerCommand;-run-([Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/WakeLockStateMachine;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DockObserver;-access$100-(Lcom/android/server/DockObserver;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ISub$Stub;-setColor-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPermittedAccessibilityServices-(Landroid/content/ComponentName; Ljava/util/List;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneFactory;-getGsmPhone-(I)Lcom/android/internal/telephony/Phone;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setLocationUpdates-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService$AppDeathRecipient;-binderDied-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISub$Stub;-getActiveSubInfoList-()Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getLine1NumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setMaximumFailedPasswordsForWipe-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-keyguardGoingAway-(Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-setUniverseTransformLocked-(Lcom/android/server/wm/WindowState; F F F F F F F)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/DigitalClock;-access$300-(Landroid/widget/DigitalClock;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothSocket;-<init>-(I I Z Z Ljava/lang/String; I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/app/ChooserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/effect/FilterGraphEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/BluetoothDevice;-createInsecureRfcommSocketToServiceRecord-(Ljava/util/UUID;)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/sip/SipPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-switchWaitingOrHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/dreams/DreamManagerService;-testDreamInternal-(Landroid/content/ComponentName; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag$Stub;-close-(I)I' : ['android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/LockPatternUtils;-getWidgetsEnabled-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/AlertActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService$6;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/nfc/NfcService;-isAirplaneModeOn-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/EditText;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/RotationPolicy;-registerRotationPolicyListener-(Landroid/content/Context; Lcom/android/internal/view/RotationPolicy$RotationPolicyListener; I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/SubscriptionController;-getActiveSubInfoList-()Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-setPermittedInputMethods-(Landroid/content/ComponentName; Ljava/util/List;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NdefFormatable;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/Tag;-rediscover-()Landroid/nfc/Tag;' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-getActiveSubInfoList-()Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/tv/TvInputInfo$TvInputSettings;-getHiddenTvInputIds-(Landroid/content/Context; I)Ljava/util/Set;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine;-setFrequencyBand-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-hangupWaitingOrBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getAggressiveHandover-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/AudioService$AudioServiceInternal;-setStreamVolumeForUid-(I I I Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService;-deviceIsProvisioned-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;-writeSentMessage-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/MmsServiceBroker;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/preference/PreferenceFrameLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/content/ContentService;-getCurrentSyncsAsUser-(I)Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPuk2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/UiAutomation;-getWindows-()Ljava/util/List;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-switchWaitingOrHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/format/DateFormat;-getTimeFormat-(Landroid/content/Context;)Ljava/text/DateFormat;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Connection;-cancelPostDial-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/IRecognitionService$Stub$Proxy;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/inputmethodservice/KeyboardView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/AppChooserActivity;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getTimeCacheMaxAge-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/EventStreamTransformation;-clear-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-getPasswordExpirationTimeout-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofencePauseStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/format/DateFormat;-getDateFormatString-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation$Stub$Proxy;-setPreferredService-(Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-getInstallLocation-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-confirmCredentialsAsUser-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler; Landroid/os/UserHandle;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/hardware/location/GeofenceHardware;-addGeofence-(I I Landroid/hardware/location/GeofenceHardwareRequest; Landroid/hardware/location/GeofenceHardwareCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/TextView;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayManagerService$DisplayAdapterListener;-onDisplayDeviceEvent-(Lcom/android/server/display/DisplayDevice; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService$Stub;-sync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-supplyPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setGsmBroadcastConfig-([Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SubscriptionController;-getSubInfoUsingIccId-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-doFinish-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService$WakeLock;-binderDied-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/PhoneNumberUtils;-isVoiceMailNumber-(Ljava/lang/String;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/Phone;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/WakeGestureListener$2;-run-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService$9;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISub$Stub;-getSubInfoForSubscriber-(J)Landroid/telephony/SubInfoRecord;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/inputmethodservice/ExtractEditText;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/TintFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendEnvelopeWithStatus-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IBackupManager$Stub$Proxy;-clearBackupData-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Global;-getStringForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getAccountTypesWithManagementDisabledAsUser-(I)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$4;-onCallProgressing-(Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Chronometer;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/notification/NotificationListenerService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/SizeAdaptiveLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Ljava/net/ServerSocket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-renameAccount-(Landroid/accounts/Account; Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-setInitialAttachApn-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/bluetooth/le/BluetoothLeAdvertiser;-startAdvertising-(Landroid/bluetooth/le/AdvertiseSettings; Landroid/bluetooth/le/AdvertiseData; Landroid/bluetooth/le/AdvertiseCallback;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManager;-addOnAccountsUpdatedListener-(Landroid/accounts/OnAccountsUpdateListener; Landroid/os/Handler; Z)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/backup/BackupManagerService$PerformUnifiedRestoreTask$EngineThread;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager$2;-startTask-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/IAudioService$Stub;-reloadAudioSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/effect/effects/TintEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/SepiaEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lorg/apache/http/impl/client/DefaultHttpClient;-execute-(Lorg/apache/http/client/methods/HttpUriRequest;)Lorg/apache/http/HttpResponse;' : ['android.permission.INTERNET'],
	'Lcom/android/server/accounts/AccountManagerService;-checkManageAccountsOrUseCredentialsPermissions-()V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/ims/internal/ImsCallSession$Listener;-callSessionStartFailed-(Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/SystemGesturesPointerEventListener$Callbacks;-onSwipeFromBottom-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/ims/ImsEcbm$ImsEcbmListenerProxy;-enteredECBM-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/service/voice/VoiceInteractionService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-sendMessage-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/internal/telephony/ISms$Stub;-disableCellBroadcastForSubscriber-(J I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/bluetooth/BluetoothDevice;-createRfcommSocketToServiceRecord-(Ljava/util/UUID;)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/app/AlertActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-onKey-(I [I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$Receiver;-clearPendingBroadcastsLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcAdapter;-addNfcUnlockHandler-(Landroid/nfc/INfcUnlockHandler; [I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-isSyncPendingAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/speech/tts/TextToSpeech;-getDefaultEngine-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/Filter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-getTasks-(I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Landroid/app/admin/IDevicePolicyManager;-clearCrossProfileIntentFilters-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onVoiceCallStarted-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/DragState;-getTouchedWinAtPointLw-(F F)Lcom/android/server/wm/WindowState;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/location/GeofenceHardwareService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setAndBroadcastNetworkSetTime-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/dreams/DreamManagerService;-access$2400-(Lcom/android/server/dreams/DreamManagerService;)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/NullFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/tv/TvInputManagerService$1;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/sip/ISipService;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/nfc/P2pEventManager;-onP2pReceiveComplete-(Z)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/tv/TvInputService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setPropertiesByCarrier-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-setOperatorBrandOverride-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPermittedInputMethodsForCurrentUser-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-handleSmsMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-injectSmsPdu-([B Ljava/lang/String; Landroid/app/PendingIntent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-usingBiometricWeak-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$ActingState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/content/SyncManager;-scheduleLocalSync-(Landroid/accounts/Account; I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/printservice/PrintService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/pm/PackageManagerService;-main-(Landroid/content/Context; Lcom/android/server/pm/Installer; Z Z)Lcom/android/server/pm/PackageManagerService;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Contacts$Settings;-getSetting-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_CONTACTS'],
	'Landroid/view/TextureView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$ConnectedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-iccIOForApp-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getIsimChallengeResponse-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub;-setDisplayNumber-(Ljava/lang/String; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getAllowScansWithTraffic-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/ContextImpl;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController$StaEnabledState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/widget/AnalogClock;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-requestLocationUpdatesLocked-(Landroid/location/LocationRequest; Lcom/android/server/LocationManagerService$Receiver; I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/WiredAccessoryManager;-notifyWiredAccessoryChanged-(J I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-sendUSSD-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/ExtractButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IBackupAgent$Stub;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/media/MediaSessionService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationFudger$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-make-(Lcom/android/internal/telephony/cat/BerTlv;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager;-scheduleSync-(Landroid/content/ComponentName; I I Landroid/os/Bundle; J J)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/sip/SipService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/media/TtmlRenderingWidget;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getGroupIdLevel1ForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/SearchView$SearchAutoComplete;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/DropBoxManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/NotificationManagerService$NotificationListeners;-<init>-(Landroid/content/Context; Landroid/os/Handler; Ljava/lang/Object; Lcom/android/server/notification/ManagedServices$UserProfiles;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/sip/SipService$SipSessionGroupExt;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TtsEngines;-getDefaultEngine-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setApplicationRestrictions-(Landroid/content/ComponentName; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-cancelPostDial-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/MediaSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-checkReadAccountsPermission-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/wm/WindowManagerService;-computeScreenConfigurationLocked-(Landroid/content/res/Configuration;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/Context;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowStateAnimator;-setSurfaceBoundariesLocked-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Settings$System;-getInt-(Landroid/content/ContentResolver; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/HandoverTransfer;-onScanCompleted-(Ljava/lang/String; Landroid/net/Uri;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/os/IPowerManager$Stub$Proxy;-acquireWakeLockWithUid-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-clearCache-()V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/server/notification/ZenModeHelper$SettingsObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/LocationFudger;-loadCoarseAccuracy-()F' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/google/android/mms/util/AbstractCache;-put-(Ljava/lang/Object; Ljava/lang/Object;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/wifi/WifiController$StaDisabledWithScanState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-disable-(Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/notification/ValidateNotificationPeople;-access$600-(Lcom/android/server/notification/ValidateNotificationPeople; Landroid/content/Context; Ljava/lang/String;)Lcom/android/server/notification/ValidateNotificationPeople$LookupResult;' : ['android.permission.READ_CONTACTS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setProfileEnabled-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$Secure;-getStringForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/usage/UsageStatsService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager$Stub;-setAllowScansWithTraffic-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISms;-getAllMessagesFromIccEf-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/connectivity/Tethering;-updateConfiguration-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-getInstallLocation-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CompoundButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcController;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService$BinderService;-acquireWakeLockWithUid-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordQuality-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget$SpanLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/tts/TextToSpeech;-setVoice-(Landroid/speech/tts/Voice;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothInputDevice;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/app/AlertActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/inputmethod/InputMethodSubtypeSwitchingController$InputMethodAndSubtypeList;-getSortedInputMethodAndSubtypeList-(Z Z Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/service/voice/VoiceInteractionService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-access$2600-(Lcom/android/server/power/PowerManagerService; Lcom/android/server/power/PowerManagerService$WakeLock;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeech;-playEarcon-(Ljava/lang/String; I Landroid/os/Bundle; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-injectSmsPdu-([B Ljava/lang/String; Landroid/app/PendingIntent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/monkey/Monkey;-run-([Ljava/lang/String;)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/cardemulation/RegisteredAidCache;-onPreferredPaymentServiceChanged-(Landroid/content/ComponentName;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/notification/ZenModeHelper;-isDefaultMessagingApp-(Lcom/android/server/notification/NotificationRecord;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-invokeBeam-()V' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/server/notification/ConditionProviders;-setAutomaticZenModeConditions-([Landroid/net/Uri; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/printservice/PrintService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/nfc/cardemulation/HostApduService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothPbap;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/IEthernetManager$Stub;-setConfiguration-(Landroid/net/IpConfiguration;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/app/AliasActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-stopBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/print/IPrintManager$Stub;-stopPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Browser;-getVisitedHistory-(Landroid/content/ContentResolver;)[Ljava/lang/String;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/tv/TvInputService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/admin/IDevicePolicyManager;-getGlobalProxyAdmin-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/input/InputManager$InputDeviceVibrator;-vibrate-([J I Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Landroid/net/wifi/WifiManager$WifiLock;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accessibilityservice/IAccessibilityServiceConnection$Stub;-getWindow-(I)Landroid/view/accessibility/AccessibilityWindowInfo;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onVoiceCallEnded-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService;-dispatchUnhandledKey-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-access$9800-(Landroid/media/AudioService; I I I Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/TtmlRenderingWidget;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-setCdmaBroadcastActivation-(Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-getAllSubInfoCount-()I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/NsdService$NsdStateMachine;-<init>-(Lcom/android/server/NsdService; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/util/IState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/BarController;-adjustSystemUiVisibilityLw-(I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/DevicePolicyManagerInternal;-getCrossProfileWidgetProviders-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/FullBackupJob;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/IPrintManager;-cancelPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPasswordMinimumNumeric-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-handleShift-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/app/ActivityThread$DropBoxReporter;-addText-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService$SettingsObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub$Stub;-getDefaultDataSubId-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcAdapter$Stub;-enable-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-wipeDataLocked-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ViewOverlay$OverlayViewGroup;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation;-isDefaultServiceForAid-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Landroid/filterpacks/imageproc/BitmapSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DateTimeView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/tv/TvInputManagerService$1;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/service/voice/VoiceInteractionService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/notification/ZenModeHelper$SettingsObserver;-update-(Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/UiModeManagerService;-access$100-(Lcom/android/server/UiModeManagerService; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-onSystemReady-()V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-access$3400-(Lcom/android/internal/telephony/dataconnection/DataConnection; Lcom/android/internal/telephony/dataconnection/DataConnection$ConnectionParams;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$ObtainingIpState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/SubtitleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-setDefaultSubId-(J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityManagerService;-killAppAtUsersRequest-(Lcom/android/server/am/ProcessRecord; Landroid/app/Dialog;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService$Stub;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifyDataConnectionForSubscriber-(J I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/NetworkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ILockSettings;-getLong-(Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/voiceinteraction/SoundTriggerHelper;-onRecognition-(Landroid/hardware/soundtrigger/SoundTrigger$RecognitionEvent;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-handleCdmaSubscriptionSource-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/wifi/IWifiManager;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-requestShutdown-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Landroid/accounts/IAccountManager;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/widget/LinearLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/accounts/ChooseAccountTypeActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/nfc/INfcCardEmulation$Stub;-setPreferredService-(Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/server/connectivity/Tethering;-checkDunRequired-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/search/SearchManagerService$MyPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/ActivityGroup;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/TouchExplorer;-clear-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-getApplicationRestrictions-(Landroid/content/ComponentName; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/VideoView$7;-surfaceCreated-(Landroid/view/SurfaceHolder;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RelativeLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/InputMethodService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/backup/LocalTransportService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler$StartupState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-pollState-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/view/menu/ListMenuItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISub$Stub;-getAllSubInfoCount-()I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/nfc/NfcActivityManager;-setNdefPushContentUri-(Landroid/app/Activity; [Landroid/net/Uri;)V' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getUidConfig-()Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings$Config;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$DefaultState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/nfc/handover/ConfirmConnectActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/pm/PackageManagerService;-unloadMediaPackages-(Landroid/util/ArrayMap; [I Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiController$FullHighPerfLockHeldState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Landroid/view/IWindowManager$Stub;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/server/wifi/WifiStateMachine$L2ConnectedState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/QuickContactBadge;-assignContactFromEmail-(Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-enableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setUserRestriction-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/filterfw/core/FilterGraph;-tearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-setOffsetNotificationsEnabled-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/DhcpStateMachine$RunningState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/SRTRenderer;-createTrack-(Landroid/media/MediaFormat;)Landroid/media/SubtitleTrack;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/VideoView;-setVideoURI-(Landroid/net/Uri; Ljava/util/Map;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/BackgroundDexOptService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setProfileOwner-(Landroid/content/ComponentName; Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/MifareClassic;-restore-(I)V' : ['android.permission.NFC'],
	'Lcom/android/server/DockObserver;-init-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/cardemulation/PreferredServices;-setDefaultForNextTap-(Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/hdmi/SystemAudioAutoInitiationAction;-processCommand-(Lcom/android/server/hdmi/HdmiCecMessage;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService$AudioServiceInternal;-adjustStreamVolumeForUid-(I I I Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/CallbackFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/SeekBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/OffHostApduService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/nfc/NfcBackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/tts/TextToSpeechService;-access$400-(Landroid/speech/tts/TextToSpeechService;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Browser;-truncateHistory-(Landroid/content/ContentResolver;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/CallManager;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SubscriptionController;-setDefaultDataSubId-(J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-getEnabledInputMethodList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/NetworkScoreManager;-disableScoring-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/CardEmulation;-categoryAllowsForegroundPreference-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setKeyguardDisabledFeatures-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/TechListChooserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/NsdService$NsdStateMachine$DisabledState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/app/IWallpaperManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_WALLPAPER_HINTS', 'android.permission.SET_WALLPAPER'],
	'Landroid/telephony/SmsMessage;-fragmentText-(Ljava/lang/String;)Ljava/util/ArrayList;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-startProfilesLocked-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getNeighboringCids-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CompoundButton;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ActivityThread$DropBoxReporter;-addData-(Ljava/lang/String; [B I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowSession;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-access$000-(Lcom/android/internal/telephony/imsphone/ImsPhone;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BatteryService;-logBatteryStatsLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/DialogTitle;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/trust/TrustManagerService;-access$1600-(Lcom/android/server/trust/TrustManagerService; Lcom/android/internal/widget/LockPatternUtils; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub;-getCurrentSyncsAsUser-(I)Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/telephony/TelephonyManager;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/print/IPrintManager;-startPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/BroadcastQueue$AppNotResponding;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/ServiceWatcher$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-forceStopUserLocked-(I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/ISyncAdapter$Stub;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/nfc/INfcTag;-rediscover-(I)Landroid/nfc/Tag;' : ['android.permission.NFC'],
	'Landroid/view/accessibility/CaptioningManager;-access$100-(Landroid/view/accessibility/CaptioningManager;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onRecordsLoaded-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/widget/SimpleMonthView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/search/SearchManagerService$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/net/wifi/WifiManager;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/WapPushOverSms;-dispatchWapPdu-([B Landroid/content/BroadcastReceiver; Lcom/android/internal/telephony/InboundSmsHandler;)I' : ['android.permission.RECEIVE_MMS', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CellBroadcastRangeManager;-updateRanges-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/AlarmManagerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/CantAddAccountActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/SystemServer;-startOtherServices-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-getEnabledInputMethodSubtypeList-(Ljava/lang/String; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyLocationChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DigitalClock;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Landroid/location/Criteria; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/LocationBasedCountryDetector$1;-onLocationChanged-(Landroid/location/Location;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManager$3;-startTask-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/sip/SipManager;-makeAudioCall-(Ljava/lang/String; Ljava/lang/String; Landroid/net/sip/SipAudioCall$Listener; I)Landroid/net/sip/SipAudioCall;' : ['android.permission.USE_SIP'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-addNfcUnlockHandler-(Landroid/nfc/INfcUnlockHandler; [I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$ConnectModeState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/ObjectSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/CalendarContract$CalendarEntity$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ActionBarContextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$4;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-createCondition-(Landroid/service/notification/ZenModeConfig$DowntimeInfo; I)Landroid/service/notification/Condition;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Presentation$1;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiSettingsStore;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcAdapter$Stub;-removeNfcUnlockHandler-(Landroid/nfc/INfcUnlockHandler;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService$BinderService;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/AsyncPlayer$Thread;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/InputStreamSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/NetInitiatedActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LockSettingsService$DatabaseHelper;-onUpgrade-(Landroid/database/sqlite/SQLiteDatabase; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/BackgroundDexOptService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-cancelNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/FillLightEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/ExtractEditText;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallbackProxy$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-disableCellBroadcastForSubscriber-(J I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncActivityTooManyDeletes;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/Phone;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/CropFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActiveServices;-bringDownServiceLocked-(Lcom/android/server/am/ServiceRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/AudioService;-startBluetoothScoInt-(Landroid/os/IBinder; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getChannelList-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-handleRuimRefresh-(Lcom/android/internal/telephony/uicc/IccRefreshResponse;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-handleServiceCategoryProgramData-(Lcom/android/internal/telephony/cdma/SmsMessage;)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/widget/RemoteViews$2;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/SipAudioCall;-releaseWifiHighPerfLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/GraphRunner;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/ActivityManagerNative;-setPackageScreenCompatMode-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/nsd/INsdManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.INTERNET'],
	'Landroid/widget/SeekBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-handleSupplicantConnectionLoss-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/RotateFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/AsyncService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/NetworkTimeUpdateService;-isAutomaticTimeRequested-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/WifiDisplayController$1;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcApplication;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/ISipService$Stub;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/widget/TextViewWithCircularIndicator;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/media/MediaBrowserService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPukForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setRecommendedGlobalProxy-(Landroid/content/ComponentName; Landroid/net/ProxyInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-addGpsNavigationMessageListener-(Landroid/location/GpsNavigationMessageEvent$Listener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/StackView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowSession$Stub$Proxy;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Landroid/widget/CompoundButton;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActiveServices;-startServiceLocked-(Landroid/app/IApplicationThread; Landroid/content/Intent; Ljava/lang/String; I I I)Landroid/content/ComponentName;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-performTransitions-(Lcom/android/internal/util/State; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/commands/settings/SettingsCmd;-deleteForUser-(Landroid/content/IContentProvider; I Ljava/lang/String; Ljava/lang/String;)I' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getIccSerialNumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-attachApplicationLocked-(Landroid/app/IApplicationThread; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/IAudioService$Stub;-adjustStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$System;-getConfiguration-(Landroid/content/ContentResolver; Landroid/content/res/Configuration;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/AccessibilityController$WindowsForAccessibilityObserver;-scheduleComputeChangedWindowsLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordMinimumNumeric-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcTag;-ndefMakeReadOnly-(I)I' : ['android.permission.NFC'],
	'Landroid/widget/AbsSpinner;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enforceReceiveAndSend-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/app/PlatLogoActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub$1;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/backup/IBackupManager;-opComplete-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-getSubscriptionInfoAndStartPollingThreads-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/appwidget/AppWidgetServiceImpl;-bindAppWidgetId-(Ljava/lang/String; I I Landroid/content/ComponentName; Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/monkey/Monkey;-main-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/job/JobSchedulerService$JobSchedulerStub;-schedule-(Landroid/app/job/JobInfo;)I' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/media/effect/effects/SharpenEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodAndSubtypeHashCodeListLocked-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-pollCallsWhenSafe-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getCrossProfileWidgetProviders-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-disableCdmaBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/widget/CompoundButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/RedEyeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/widget/SwipeDismissLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-getAnyDataEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getGlobalProxyAdmin-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService$TagService;-formatNdef-(I [B)I' : ['android.permission.NFC'],
	'Lcom/android/server/wm/WindowManagerService;-moveInputMethodWindowsIfNeededLocked-(Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getOutgoingCallerIdDisplay-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/WebVttRenderingWidget;-manageChangeListener-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/view/View;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService;-startVibrationLocked-(Lcom/android/server/VibratorService$Vibration;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/media/RingtoneManager;-getRingtone-(Landroid/content/Context; Landroid/net/Uri; I)Landroid/media/Ringtone;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/dreams/DreamManagerService;-startDozingInternal-(Landroid/os/IBinder; I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/DateTimeView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setOperatorBrandOverride-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/pm/LauncherAppsService$LauncherAppsImpl$MyPackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/SystemServer;-main-([Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$UserState$1;-binderDied-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/AlarmManagerService;-onStart-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPasswordMinimumLowerCase-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NfcA;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiStateMachine;-sendConnectedState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-calculateLength-(Ljava/lang/CharSequence; Z)Lcom/android/internal/telephony/GsmAlphabet$TextEncodingDetails;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActivityStack;-resetAffinityTaskIfNeededLocked-(Lcom/android/server/am/TaskRecord; Lcom/android/server/am/TaskRecord; Z Z I)I' : ['android.permission.SEND_SMS', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/telecom/InCallService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/ILocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/IWindowManager;-setAppVisibility-(Landroid/os/IBinder; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/AbsActionBarView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/mock/MockContentResolver;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService$HandlerParams;-handleStartCopy-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/FullBackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/FilterPort;-setFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ContextThemeWrapper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/ScrollingTabContainerView$TabView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/mock/MockContentResolver;-setSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/media/IAudioService$Stub;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-onRecordLoaded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneBase;-getDataConnectionState-()Lcom/android/internal/telephony/PhoneConstants$DataState;' : ['android.permission.READ_PROFILE'],
	'Landroid/print/PrintJob;-restart-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/TapAgainDialog;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-getOrientationFromAppTokensLocked-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/cardemulation/TapAgainDialog;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService$Stub;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-getSubInfoUsingIccId-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/DockObserver;-onBootPhase-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/InputMethodService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiStateMachine$VerifyingLinkState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TimePicker;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/net/wifi/IWifiManager;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/ZenModeHelper$Callback;-onZenModeChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendSmsByPstn-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getCurrentCalls-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Bookmarks;-add-(Landroid/content/ContentResolver; Landroid/content/Intent; Ljava/lang/String; Ljava/lang/String; C I)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-exitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/FinalPort;-pushFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService;-handleDisplayDeviceAdded-(Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/ViewSwitcher;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CellBroadcastRangeManager;-tryAddRanges-(I I Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService$8;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-setMicrophoneMute-(Z Ljava/lang/String;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/MountServiceIdler;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager;-updateCredentials-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/IccProvider;-updateIccRecordInEf-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; J)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/notification/ConditionProviders$ZenModeHelperCallback;-onZenModeChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$AccessibilityContentObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/GLTextureTarget;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterfw/core/InputPort;-setFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-handleSetRequest-(Lcom/android/internal/location/ProviderRequest; Landroid/os/WorkSource;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/WindowManagerInternal;-setWindowsForAccessibilityCallback-(Landroid/view/WindowManagerInternal$WindowsForAccessibilityCallback;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/performance/ThroughputFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/DhcpStateMachine$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/InputStreamSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/IInputMethodManager;-getEnabledInputMethodList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaScanner$MyMediaScannerClient;-endFile-(Landroid/media/MediaScanner$FileEntry; Z Z Z Z Z)Landroid/net/Uri;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-resetSelectedInputMethodAndSubtypeLocked-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-setDataAllowed-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList; Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/widget/NumberPicker$CustomEditText;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/ContextThemeWrapper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CallerInfoAsyncQuery;-startQuery-(I Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/CallerInfoAsyncQuery$OnQueryCompleteListener; Ljava/lang/Object;)Lcom/android/internal/telephony/CallerInfoAsyncQuery;' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-acceptCall-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/NotificationManagerService;-loadPolicyFile-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/MultiAutoCompleteTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/Filter;-process-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiServiceImpl;-enableAggressiveHandover-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/Chronometer;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GeofenceManager;-scheduleUpdateFencesLocked-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-shutdown-(I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/NfcActivityManager;-requestNfcServiceCallback-()V' : ['android.permission.NFC'],
	'Landroid/app/ActivityView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesSettings;-isSpellCheckerEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getPollInterval-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-setCdmaBroadcastConfig-([Lcom/android/internal/telephony/cdma/CdmaSmsBroadcastConfigInfo;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings; Landroid/os/IBinder; Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-restartRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/content/SyncActivityTooManyDeletes;-startSyncReallyDelete-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifyServiceState-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioManager;-startBluetoothScoVirtualCall-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/SystemBackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService$Stub$Proxy;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/connectivity/PacManager;-longSchedule-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-addSubtypeToHistory-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getAccountTypesWithManagementDisabledAsUser-(I)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-interceptFallback-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiConfigStore;-handleSSIDStateChange-(I Z Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getIsSyncableAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/tv/TvInputInfo$TvInputSettings;-getCustomLabels-(Landroid/content/Context; I)Ljava/util/Map;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-hangupForegroundResumeBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Browser;-deleteFromHistory-(Landroid/content/ContentResolver; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS'],
	'Landroid/content/IContentService;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/Application;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleCancel-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/commands/uiautomator/Launcher$Command;-run-([Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/tts/TextToSpeech$2;-run-(Landroid/speech/tts/ITextToSpeechService;)Ljava/lang/Integer;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipAudioCall$1;-onCallBusy-(Landroid/net/sip/SipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/CalendarContract$EventsEntity$EntityIteratorImpl;-getEntityAndIncrementCursor-(Landroid/database/Cursor;)Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Landroid/provider/Settings$Global;-getString-(Landroid/content/ContentResolver; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcAdapter$Stub;-disableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/CellBroadcastHandler;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/hdmi/HdmiControlService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$5;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/notification/NotificationManagerService$NotificationListeners;-<init>-(Lcom/android/server/notification/NotificationManagerService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-inputDispatchingTimedOut-(I Z Ljava/lang/String;)J' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/ims/ImsCall;-notifyConferenceSessionTerminated-(Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/SpeechRecognizer;-access$000-(Landroid/speech/SpeechRecognizer; Landroid/content/Intent;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/provider/ContactsContract$SyncState;-newSetOperation-(Landroid/accounts/Account; [B)Landroid/content/ContentProviderOperation;' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-isApplicationHidden-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/app/ExpandableListActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/content/IContentService;-setMasterSyncAutomaticallyAsUser-(Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-packageHasActiveAdmins-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TtsEngines;-updateLocalePrefForEngine-(Ljava/lang/String; Ljava/util/Locale;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/base/ObjectSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/preference/PreferenceActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/nfc/P2pEventListener;-onP2pNfcTapRequested-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/widget/ActionBarView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LockSettingsService;-setLockPassword-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextClock;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManagerNative;-showBootMessage-(Ljava/lang/CharSequence; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/power/PowerManagerService;-handleBatteryStateChangedLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$7600-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Ljava/lang/String; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-switchUser-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/ViewAnimator;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/base/ObjectSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-startNavigating-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/PreferredServices;-<init>-(Landroid/content/Context; Lcom/android/nfc/cardemulation/RegisteredServicesCache; Lcom/android/nfc/cardemulation/RegisteredAidCache; Lcom/android/nfc/cardemulation/PreferredServices$Callback;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiSettingsStore;-getPersistedWifiState-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-getDataEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-notifyLockTaskModeChanged-(Z Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameFetch;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-rebuildAppWindowListLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/GhostView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/backup/IBackupManager$Stub;-opComplete-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/os/PowerManager$WakeLock;-acquire-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SubscriptionManager;-setDisplayName-(Ljava/lang/String; J J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService$Lifecycle;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-clearUserRestrictions-(Landroid/os/UserHandle;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/connectivity/NetworkMonitor$DefaultState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-processPostDialChar-(C)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-disable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/ITextToSpeechService$Stub;-synthesizeToFileDescriptor-(Landroid/os/IBinder; Ljava/lang/CharSequence; Landroid/os/ParcelFileDescriptor; Landroid/os/Bundle; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ImageSwitcher;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/media/effect/EffectContext;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccProvider;-loadAllSimContacts-(I)Landroid/database/Cursor;' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS'],
	'Landroid/content/IContentService$Stub;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getXtPersistBytes-(J)J' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ISyncAdapter$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getDataEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActiveServices;-bringDownServiceIfNeededLocked-(Lcom/android/server/am/ServiceRecord; Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/preference/PreferenceActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/DuotoneEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/provider/Contacts$People;-markAsContacted-(Landroid/content/ContentResolver; J)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/media/IAudioService$Stub$Proxy;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/sip/ISipService;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Landroid/widget/CheckedTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setNetworkSelectionModeManual-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/pm/IPackageManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.GET_PACKAGE_SIZE'],
	'Landroid/nfc/tech/MifareUltralight;-getTimeout-()I' : ['android.permission.NFC'],
	'Lcom/android/server/BluetoothManagerService;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/app/ContextImpl;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/tts/TextToSpeech;-runAction-(Landroid/speech/tts/TextToSpeech$Action; Ljava/lang/Object; Ljava/lang/String; Z Z)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IApplicationToken;-windowsVisible-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/PersistentDataBlockService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-updatePhoneObject-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-applyNewState-(Lcom/android/internal/telephony/dataconnection/ApnContext; Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-getAllMessagesFromIccEfForSubscriber-(J Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/media/Ringtone;-play-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-getShortcutInputMethodsAndSubtypes-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ListActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/ISub;-setDisplayNumberFormat-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/backup/RestoreSession;-getAvailableRestoreSets-(Landroid/app/backup/RestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-initializeP2pSettings-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/provider/Contacts$People;-addToGroup-(Landroid/content/ContentResolver; J J)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/hardware/IConsumerIrService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.TRANSMIT_IR'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/IAccountManager;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/trust/TrustManagerService;-resetAgent-(Landroid/content/ComponentName; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcRootActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getPrivilegedConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/backup/BackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-onRecordsLoaded-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcTag$Stub;-ndefMakeReadOnly-(I)I' : ['android.permission.NFC'],
	'Landroid/widget/TextClock;-chooseFormat-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SmsApplication;-getApplication-(Landroid/content/Context; Z I)Lcom/android/internal/telephony/SmsApplication$SmsApplicationData;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-displayReady-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService;-scheduleTraversalLocked-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/GhostView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService$Stub;-setStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setCdmaBroadcastConfig-([Lcom/android/internal/telephony/cdma/CdmaSmsBroadcastConfigInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-invokeBeam-()V' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Landroid/test/mock/MockContentResolver;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getDevPersistBytes-(J)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-fetchDunApn-()Lcom/android/internal/telephony/dataconnection/ApnSetting;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/SupplicantStateTracker$UninitializedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Landroid/nfc/INfcTag$Stub;-reconnect-(I)I' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaRecorder;-setAudioSource-(I)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/filterfw/core/FilterSurfaceView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/ILocationManager;-addGpsNavigationMessageListener-(Landroid/location/IGpsNavigationMessageListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/LockToAppRequestDialog;-getLockString-(I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-scheduleSleepTimeout-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-setRequest-(Lcom/android/internal/location/ProviderRequest; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CdmaBroadcastRangeManager;-updateRanges-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-confirmCredentials-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/server/accounts/AccountManagerService;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getImei-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/provider/CallLog$Calls;-addCall-(Lcom/android/internal/telephony/CallerInfo; Landroid/content/Context; Ljava/lang/String; I I I Landroid/telecom/PhoneAccountHandle; J I Ljava/lang/Long;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/nfc/INfcAdapter;-setAppCallback-(Landroid/nfc/IAppCallback;)V' : ['android.permission.NFC'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-disableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/nfc/NfcAdapter;-setNdefPushMessageCallback-(Landroid/nfc/NfcAdapter$CreateNdefMessageCallback; Landroid/app/Activity; I)V' : ['android.permission.NFC'],
	'Landroid/content/IContentService$Stub;-getSyncStatusAsUser-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName; I)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/media/effect/effects/SepiaEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccPhoneBookController;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-access$1800-(Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService$8;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/TintFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.REORDER_TASKS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.GET_TASKS', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/DuotoneEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IPowerManager;-updateWakeLockUids-(Landroid/os/IBinder; [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-dispatchMediaKeyWithWakeLock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/BarController;-checkHiddenLw-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/IntentService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/NetworkScorerAppManager;-setActiveScorer-(Landroid/content/Context; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech;-synthesizeToFile-(Ljava/lang/CharSequence; Landroid/os/Bundle; Ljava/io/File; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NfcA;-getTimeout-()I' : ['android.permission.NFC'],
	'Lcom/android/contacts/ContactsListActivity;-getContactUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/accessibility/ScreenMagnifier$MagnificationController;-setMagnifiedRegionCenter-(F F Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/AudioService;-setStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-removeAccountAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/InputMethodManagerService;-resetStateIfCurrentLocaleChangedLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-fetchRssiLinkSpeedAndFrequencyNative-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/app/ActivityManagerNative;-getTaskThumbnail-(I)Landroid/app/ActivityManager$TaskThumbnail;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setOperatorBrandOverride-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/speech/RecognitionService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService;-access$1400-(Lcom/android/server/display/DisplayManagerService; Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/backup/BackupAgentHelper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onDataSetupCompleteError-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-doDatabaseCleanup-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-setPostDialState-(Lcom/android/internal/telephony/Connection$PostDialState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISms$Stub;-updateMessageOnIccEfForSubscriber-(J Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/GrainFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$P2pEnabledState;-exit-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-onStart-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.CLEAR_APP_CACHE', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/MifareClassic;-authenticateSectorWithKeyA-(I [B)Z' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifySignalStrength-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/CertBlacklister$BlacklistObserver;-getValue-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Application;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-getDefaultVoiceSubId-()J' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$2;-doWork-()V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/view/accessibility/IAccessibilityManager$Stub;-registerUiTestAutomationService-(Landroid/os/IBinder; Landroid/accessibilityservice/IAccessibilityServiceClient; Landroid/accessibilityservice/AccessibilityServiceInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/DefaultRemovedActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/SubscriptionManager;-getDefaultDataPhoneId-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-sendStoredText-(Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifyServiceStateForPhoneId-(I J Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService;-isAllowedByCurrentUserSettingsLocked-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-dialInternal-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/LockPatternUtils;-setOwnerInfoEnabled-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/trust/TrustAgentService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowStateAnimator;-finishExit-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/power/PowerManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NfcB;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/SmsMessage;-getSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Z)Lcom/android/internal/telephony/gsm/SmsMessage$SubmitPdu;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/commands/am/Am;-runStartUserInBackground-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/BlendFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Application;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/RttService$RttServiceImpl$RttStateMachine$EnabledState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-statusBarVisibilityChanged-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setActivePasswordState-(I I I I I I I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-access$600-(Lcom/android/server/LocationManagerService; I I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/BaseCommands;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-setAllowScansWithTraffic-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/telephony/gsm/SmsManager;-sendMultipartTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-writeInboxMessage-(Landroid/content/Intent;)Landroid/net/Uri;' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SMS'],
	'Lcom/android/internal/telephony/TelephonyDevController;-registerRIL-(Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/LauncherAppsService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-updateSmsSendStatusHelper-(I Ljava/util/List; Lcom/android/internal/telephony/SMSDispatcher; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/RecognitionService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/MediaSource;-process-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-createAndInitializeUser-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String; Landroid/content/ComponentName; Landroid/os/Bundle;)Landroid/os/UserHandle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-handleCallDeflectionIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-lockNow-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/SyncRunner;-performStep-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/commands/bmgr/Bmgr;-doWipe-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/restrictions/RestrictionsManagerService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/webkit/WebViewFactory;-onWebViewUpdateInstalled-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/WindowManagerPolicy;-adjustSystemUiVisibilityLw-(I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/effect/effects/CropEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/speech/RecognitionService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/WallpaperManager;-setStream-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/test/ModelInterpreter;-<init>-(Ljava/net/InetSocketAddress;)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/uicc/AdnRecordLoader;-loadFromEF-(I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService$InstallParams;-startCopy-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/UserState;-readConfigurationLocked-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-hasUserSetupCompleted-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/ITextToSpeechService$Stub$Proxy;-getClientDefaultLanguage-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/filterpacks/base/CallbackFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/content/IIntentReceiver;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-isApnTypeAvailable-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-startBluetoothScoVirtualCall-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/widget/Editor;-performLongClick-(Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/SystemService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/CameraSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/WifiManager;-isWifiApEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/nfc/cardemulation/RegisteredAidCache;-updateRoutingLocked-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/location/LocationBlacklist;-reloadBlacklistLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-installSystemProviders-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$9;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/WindowManagerPolicy;-applyPostLayoutPolicyLw-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/WindowManager$LayoutParams;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/backup/BackupManagerService;-handleTimeout-(I Ljava/lang/Object;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/SyncActivityTooManyDeletes;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/dreams/DreamController$3;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeech;-runActionNoReconnect-(Landroid/speech/tts/TextToSpeech$Action; Ljava/lang/Object; Ljava/lang/String; Z)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/DialogTitle;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/pm/IPackageManager$Stub;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onResetDone-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IBackupAgent$Stub;-doRestoreFinished-(I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-access$900-(Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker; Lcom/android/ims/ImsCall; Lcom/android/internal/telephony/Call$State; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setTimeFromNITZString-(Ljava/lang/String; J)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-setIccLockEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService$DragInputEventReceiver;-onInputEvent-(Landroid/view/InputEvent;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/SpeechRecognizer;-handleStartListening-(Landroid/content/Intent;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/net/INetworkStatsService$Stub;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/ActivityManagerNative;-killApplicationWithAppId-(Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/RotateFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/ime/Ime;-run-([Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/NfcAdapter;-invokeBeam-(Landroid/app/Activity;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/server/UiModeManagerService;-updateLocked-(I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$L2ConnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncStorageEngine$OnSyncRequestListener;-onSyncRequest-(Lcom/android/server/content/SyncStorageEngine$EndPoint; I Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendNewSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/SmsHeader; I Landroid/app/PendingIntent; Landroid/app/PendingIntent; Z Ljava/util/concurrent/atomic/AtomicInteger; Ljava/util/concurrent/atomic/AtomicBoolean; Landroid/net/Uri;)V' : ['android.permission.RECEIVE_SMS'],
	'Landroid/accounts/AccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/app/ActivityManager;-getRecentTasksForUser-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; Z I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-access$100-(Lcom/android/server/accessibility/AccessibilityInputFilter; J)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/MediaPlayer;-addTimedTextSource-(Ljava/io/FileDescriptor; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/LockSettingsService;-checkPassword-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/widget/LockPatternUtils;-isBiometricWeakLivelinessEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/SupplicantStateTracker$HandshakeState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/app/ActivityManagerNative;-setProcessLimit-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/ClosedCaptionWidget;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothAdapter;-closeProfileProxy-(I Landroid/bluetooth/BluetoothProfile;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-getIccSimChallengeResponse-(I Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioManager;-startBluetoothSco-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/performance/ThroughputFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStackSupervisor$VirtualActivityContainer;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readPbrFileAndWait-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowSession$Stub$Proxy;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/display/DisplayManagerService;-createVirtualDisplayInternal-(Landroid/hardware/display/IVirtualDisplayCallback; Landroid/media/projection/IMediaProjection; I Ljava/lang/String; Ljava/lang/String; I I I Landroid/view/Surface; I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/KeyguardManager;-exitKeyguardSecurely-(Landroid/app/KeyguardManager$OnKeyguardExitResult;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setActiveAdmin-(Landroid/content/ComponentName; Z I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/RIL;-changeIccPin-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-getCurrentFailedPasswordAttempts-(I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/server/am/ActivityManagerService;-handleAppDiedLocked-(Lcom/android/server/am/ProcessRecord; Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/RemoteViewsAdapter$RemoteViewsFrameLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-setNetworkTemplateEnabled-(Landroid/net/NetworkTemplate; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/ActivityGroup;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/notification/CountdownConditionProvider;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaCall;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/WaveView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$StartTetheringErrorState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-getSubInfoUsingSlotId-(I)Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ReceiverRestrictedContext;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/ContextWrapper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getKeyguardDisabledFeatures-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioManager;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-markMessagesAsRead-(Ljava/util/ArrayList;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ims/ImsCall$Listener;-onCallStarted-(Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/media/MediaSessionService$SessionManagerImpl$KeyEventWakeLockReceiver;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl$1;-onChange-(Z Landroid/net/Uri; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowSession$Stub$Proxy;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/content/ContentService;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/WindowManagerPolicy;-focusChangedLw-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/WindowManagerPolicy$WindowState;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/InputMethodService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/RIL;-getMute-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CalendarView$LegacyCalendarViewDelegate$WeekView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/sip/SipService$MyExecutor;-execute-(Ljava/lang/Runnable;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountAuthenticatorActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/WallpaperManager;-setResource-(I)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowStateAnimator;-destroySurfaceLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-showGlobalActionsInternal-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/telephony/SubscriptionManager;-allDefaultsSelected-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService;-registerPointerSpeedSettingObserver-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/updates/ConfigUpdateInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/notification/ConditionProviderService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPasswordExpirationTimeout-(Landroid/content/ComponentName; J I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextViewWithCircularIndicator;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActivityStack;-updateTransitLocked-(I Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-removeUser-(Landroid/content/ComponentName; Landroid/os/UserHandle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-addAccount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterfw/core/FinalPort;-setFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-isActivePasswordSufficient-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordExpirationLocked-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-connect-(I I)I' : ['android.permission.NFC'],
	'Landroid/nfc/tech/BasicTagTechnology;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/server/wifi/WifiStateMachine;-stopBatchedScan-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiServiceImpl;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/service/notification/NotificationListenerService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/INetworkPolicyManager$Stub;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/videosrc/MediaSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SmsManager;-getAllMessagesFromIcc-()Ljava/util/ArrayList;' : ['android.permission.RECEIVE_SMS'],
	'Landroid/content/ISyncAdapter$Stub;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/wm/Session;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/commands/monkey/MonkeySourceNetwork;-startServer-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/SimpleMonthView;-getMonthAndYearString-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$SoftApStartedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/RttService$RttServiceImpl$RttStateMachine$RequestPendingState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/telephony/TelephonyManager;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/BlendFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/display/WifiDisplayController;-<init>-(Landroid/content/Context; Landroid/os/Handler; Lcom/android/server/display/WifiDisplayController$Listener;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-setP2pModes-(I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-sendFullNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/Ringtone;-setUri-(Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/inputmethodservice/InputMethodService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService$Stub$Proxy;-sync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-pollBatchedScan-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-performLayoutLockedInner-(Lcom/android/server/wm/DisplayContent; Z Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/Filter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ToggleButton;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/printservice/PrintService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-clearPackagePersistentPreferredActivities-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-getAdnRecordsSizeForSubscriber-(J I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Ljava/net/Socket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/server/am/ActivityStack;-checkReadyForSleepLocked-()Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/ShutdownActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/notification/ConditionProviderService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/AccessibleDateAnimator;-dispatchPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/handover/HandoverManager;-createBluetoothOobDataRecord-()Landroid/nfc/NdefRecord;' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/base/RetargetFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/nsd/INsdManager$Stub;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-access$300-(Lcom/android/internal/widget/multiwaveview/GlowPadView; I F F)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/NsdService$NsdStateMachine;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/CalendarContract$Instances;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; J J)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Landroid/app/ListActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/TextureView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-getInitialDisplaySize-(I Landroid/graphics/Point;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getDisabledSystemInputMethods-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService$SettingsObserver;-<init>-(Landroid/media/AudioService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/widget/Switch;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-beginLayoutLw-(Z I I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/CalendarView$LegacyCalendarViewDelegate;-goTo-(Ljava/util/Calendar; Z Z Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/PhoneBase;-isDataConnectivityPossible-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IntRangeManager;-enableRange-(I I Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothPan;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/widget/LockPatternUtilsCache;-getLong-(Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/backup/PackageManagerBackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Button;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/LockPatternUtils;-saveLockPassword-(Ljava/lang/String; I Z I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getSubscriberIdForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/os/PowerManager$WakeLock;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getMute-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-flashAndSetGenericTrue-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/backup/PackageManagerBackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getDataRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/appwidget/AppWidgetService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-getPreferredSmsSubscription-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setupDataOnConnectableApns-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String; I Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-removeActiveAdmin-(Landroid/content/ComponentName; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-checkForCompatibleConnectedApnContext-(Lcom/android/internal/telephony/dataconnection/ApnContext;)Lcom/android/internal/telephony/dataconnection/DcAsyncChannel;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-addPersistentPreferredActivity-(Landroid/content/ComponentName; Landroid/content/IntentFilter; Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService$LocalService;-setMagnificationCallbacks-(Landroid/view/WindowManagerInternal$MagnificationCallbacks;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService$Vibration;-binderDied-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Landroid/widget/RemoteViewsService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-getInitialMaxRetry-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$8;-onServiceChanged-(Landroid/content/SyncAdapterType; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier$MagnificationController;-offsetMagnifiedRegionCenter-(F F)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine$ConnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/usb/UsbDeviceManager$AdbSettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList; Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/nfc/cardemulation/TapAgainDialog;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/RIL;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/input/InputManager$InputDeviceVibrator;-vibrate-(J)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/RIL;-processResponse-(Landroid/os/Parcel;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/appwidget/AppWidgetService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcBackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/dreams/DreamManagerService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/CountdownConditionProvider;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onConnectedInOrOut-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-restartRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/SystemGesturesPointerEventListener;-onPointerEvent-(Landroid/view/MotionEvent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/P2pLinkManager$3;-doGet-(I Landroid/nfc/NdefMessage;)Lcom/android/nfc/snep/SnepMessage;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityStackSupervisor$ActivityStackSupervisorHandler;-activityIdleInternal-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/SyncActivityTooManyDeletes;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/ClosedCaptionWidget$CCLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-clearLock-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ManagedServices$SettingsObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipCommandInterface;-setDataAllowed-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-access$3700-(Lcom/android/server/power/PowerManagerService; J I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/backup/IBackupManager;-clearBackupData-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/EditText;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-checkPasswordHistory-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ApplicationErrorReport;-getErrorReportReceiver-(Landroid/content/Context; Ljava/lang/String; I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-systemRunning-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/Button;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/BroadcastQueue;-broadcastTimeoutLocked-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/IRecognitionService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onResponse-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaPlayer;-selectOrDeselectTrack-(I Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-napInternal-(J I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/QuickContactBadge;-assignContactFromEmail-(Ljava/lang/String; Z)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/media/MediaHTTPConnection;-getSize-()J' : ['android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-onMenuModeChange-(Lcom/android/internal/view/menu/MenuBuilder;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-hasGrantedPolicy-(Landroid/content/ComponentName; I I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/ForegroundUtils;-handleUidToBackground-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getActiveAdmins-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/Settings$System;-getIntForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-queryCallForwardStatus-(I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IMms;-sendMessage-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getVoiceMailAlphaTagForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/speech/tts/TextToSpeech;-speak-(Ljava/lang/CharSequence; I Landroid/os/Bundle; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/TelephonyManager;-getSubscriberId-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/YearPickerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-access$1300-(Lcom/android/server/content/SyncManager; Lcom/android/server/content/SyncManager$ActiveSyncContext; Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/admin/IDevicePolicyManager;-addCrossProfileWidgetProvider-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringLogger;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getIccSimChallengeResponse-(J I Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ims/ImsEcbmStateListener;-onECBMExited-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setAccountManagementDisabled-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService;-setRingerMode-(I Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getVerboseLoggingLevel-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/util/AsyncService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallback;-onError-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameStore;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendStoredMultipartText-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-handleFileUpdate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/BeamShareActivity$2;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.VIBRATE'],
	'Landroid/net/wifi/WifiManager;-isWifiScannerSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/sip/SipManager;-register-(Landroid/net/sip/SipProfile; I Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SmsMessage;-getSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Z)Landroid/telephony/SmsMessage$SubmitPdu;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/effect/effects/NegativeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$7;-onSwipeFromRight-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/StateMachine$SmHandler$HaltingState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-addGpsMeasurementsListener-(Landroid/location/IGpsMeasurementsListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/IWindowSession;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/WifiManager;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/view/SurfaceView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$10000-(Lcom/android/server/wifi/WifiStateMachine; Landroid/net/NetworkInfo$DetailedState;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/BrowserContract$SyncState;-newSetOperation-(Landroid/accounts/Account; [B)Landroid/content/ContentProviderOperation;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS'],
	'Lcom/android/server/am/ActivityManagerService;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/internal/app/ChooserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/display/DisplayManagerService;-handleDisplayDeviceRemoved-(Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/NfcService;-onLlcpFirstPacketReceived-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/backup/BackupManagerService$PerformBackupTask;-beginBackup-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-queueNextSignalStrengthPoll-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/wifi/WifiManager$MulticastLock;-release-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/server/power/PowerManagerService$2;-onStateChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/view/ContextThemeWrapper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/widget/YearPickerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/usb/UsbService$Lifecycle;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordHistoryLength-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-handleRadioNotAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/TouchExplorer$SendHoverExitDelayed;-forceSendAndRemove-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/commands/media/Media;-run-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-handleSrvccStateChanged-([I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/GLTextureSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telecom/InCallService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/app/IntentForwarderActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/sip/SipService;-restartPortMappingLifetimeMeasurement-(Landroid/net/sip/SipProfile; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-finishInstrumentationLocked-(Lcom/android/server/am/ProcessRecord; I Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiController;-readWifiSleepPolicy-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService$TagService;-ndefRead-(I)Landroid/nfc/NdefMessage;' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/dataconnection/DcController;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setDataProfile-([Lcom/android/internal/telephony/dataconnection/DataProfile; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-setHoldScreenLocked-(Lcom/android/server/wm/Session;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/IAccessibilityManager$Stub$Proxy;-registerUiTestAutomationService-(Landroid/os/IBinder; Landroid/accessibilityservice/IAccessibilityServiceClient; Landroid/accessibilityservice/AccessibilityServiceInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcApplication;-onCreate-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/inputmethodservice/KeyboardView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/sip/ISipService;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/server/wifi/WifiController$2;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SmsStorageMonitor;-handleIccFull-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-updateSmsSendStatus-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-dialInternal-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/cardemulation/PreferredServices;-onUserSwitched-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/MediaSource;-pauseVideo-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/LauncherActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-setLocaleFromCsim-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/IWindowSession$Stub;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityStack;-moveTaskToBackLocked-(I Lcom/android/server/am/ActivityRecord;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getSubscriberIdForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/service/voice/VoiceInteractionSessionService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IUiAutomationConnection$Stub;-disconnect-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/TextClock;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-showInputMethodSubtypeMenu-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-handleSrvccStateChanged-([I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ProgressBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/ChooseAccountActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-setExternalState-(Lcom/android/internal/telephony/IccCardConstants$State; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL$RILReceiver;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ToggleButton;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/DockObserver$2;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/BroadcastQueue;-skipPendingBroadcastLocked-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService$10;-run-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DateTimeView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IBackupAgent;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Ljava/net/URLConnection;-connect-()V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/widget/ILockSettings$Stub$Proxy;-checkVoldPassword-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/notification/NotificationListenerService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub$Proxy;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/pm/IPackageManager$Stub;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/NetworkScoreManager;-requestScores-([Landroid/net/NetworkKey;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallEstablished-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/commands/dpm/Dpm;-runSetDeviceOwner-()V' : ['android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$Service;-getWindow-(I)Landroid/view/accessibility/AccessibilityWindowInfo;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-notifyApnIdDisconnected-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getSampleEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-initializeMulticastFiltering-()Z' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPasswordExpirationTimeout-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/widget/FrameLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/ClosedCaptionWidget$CCLineBox;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getApplicationRestrictions-(Landroid/content/ComponentName; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-resetRadio-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/ExtractEditText;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-setMasterSyncAutomaticallyAsUser-(Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/nfc/NfcBackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/nfc/handover/HandoverService;-enableBluetooth-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/IAccountManager;-renameAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/location/LocationManager;-addGpsMeasurementListener-(Landroid/location/GpsMeasurementsEvent$Listener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-processMessagePart-(Lcom/android/internal/telephony/InboundSmsTracker;)Z' : ['android.permission.RECEIVE_MMS', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Landroid/widget/AdapterViewFlipper;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-setPackageScreenCompatMode-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setLockTaskPackages-(Landroid/content/ComponentName; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-dialInternal-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/renderscript/RSTextureView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Telephony$Mms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Landroid/media/IAudioService$Stub$Proxy;-stopBluetoothSco-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getUserData-(I)Lcom/android/server/devicepolicy/DevicePolicyManagerService$DevicePolicyData;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent; Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/am/ActivityManagerService;-broadcastStickyIntent-(Landroid/content/Intent; Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/GLTextureSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/BasicTagTechnology;-connect-()V' : ['android.permission.NFC'],
	'Landroid/filterpacks/base/FrameFetch;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-getKeyguardStoredPasswordQuality-(I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/backup/BackupAgentHelper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/inputmethodservice/KeyboardView;-openPopupIfRequired-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IMms$Stub;-updateStoredMessageStatus-(Ljava/lang/String; Landroid/net/Uri; Landroid/content/ContentValues;)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-access$000-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/OverlayDisplayAdapter;-updateOverlayDisplayDevices-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$StartingState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPermittedAccessibilityServicesForUser-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActiveServices;-performServiceRestartLocked-(Lcom/android/server/am/ServiceRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityStackSupervisor$VirtualActivityContainer;-release-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getIccSerialNumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/InputMethodManagerService;-startInputInnerLocked-()Lcom/android/internal/view/InputBindResult;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-process-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/tech/MifareClassic;-setTimeout-(I)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-setOperatorBrandOverride-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/google/android/mms/pdu/PduPersister;-updateHeaders-(Landroid/net/Uri; Lcom/google/android/mms/pdu/SendReq;)V' : ['android.permission.READ_SMS'],
	'Landroid/app/admin/IDevicePolicyManager;-setPasswordHistoryLength-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifyDataConnectionForSubscriber-(J I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/NetworkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/server/BluetoothA2dpService;-handleSinkStateChange-(Landroid/bluetooth/BluetoothDevice; I I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/speech/SpeechRecognizer;-startListening-(Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-onMotionEvent-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerInternal;-startIsolatedProcess-(Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I Ljava/lang/Runnable;)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-access$100-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/P2pEventListener;-onP2pSendComplete-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/display/DisplayManagerService$LocalService$1;-requestDisplayState-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterfw/core/Filter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub$Proxy;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-processBatchedEvents-(J)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-addAccountAsUser-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler; Landroid/os/UserHandle;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/filterpacks/imageproc/TintFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/commands/pm/Pm;-run-([Ljava/lang/String;)V' : ['android.permission.CLEAR_APP_CACHE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getVoiceMailNumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/telephony/SmsManager;-importMultimediaMessage-(Landroid/net/Uri; Ljava/lang/String; J Z Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-getPrintJobInfo-(Landroid/print/PrintJobId; I I)Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiScanningServiceImpl$WifiScanningStateMachine;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/CoreSettingsObserver;-beginObserveCoreSettings-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/WebVttRenderingWidget;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setMute-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ResolverActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getCallWaiting-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onVoiceCallEnded-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setScreenCaptureDisabled-(Landroid/content/ComponentName; I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/lights/LightsService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pManager;-initialize-(Landroid/content/Context; Landroid/os/Looper; Landroid/net/wifi/p2p/WifiP2pManager$ChannelListener;)Landroid/net/wifi/p2p/WifiP2pManager$Channel;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$5500-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-setUiccSubscription-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/FilterFunction;-executeWithArgList-([Ljava/lang/Object;)Landroid/filterfw/core/Frame;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$P2pEnabledState;-enter-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/IAccessibilityManager;-unregisterUiTestAutomationService-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-registerReceiver-(Landroid/app/IApplicationThread; Ljava/lang/String; Landroid/content/IIntentReceiver; Landroid/content/IntentFilter; Ljava/lang/String; I)Landroid/content/Intent;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/os/IPowerManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-dumpAnimatorLocked-(Ljava/io/PrintWriter; [Ljava/lang/String; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleMultipartyIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/CoreSettingsObserver;-sendCoreSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IMms$Stub;-importTextMessage-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; J Z Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/nfc/handover/HandoverManager;-createBluetoothHandoverSelectMessage-(Z)Landroid/nfc/NdefMessage;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendTextForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPermittedAccessibilityServices-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handlePollCalls-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifySignalStrengthForSubscriber-(J Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/IPrintManager;-getPrintJobInfo-(Landroid/print/PrintJobId; I I)Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/SaturateEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsUsageMonitor; Lcom/android/internal/telephony/ImsSMSDispatcher; Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/app/IBatteryStats$Stub$Proxy;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/webkit/WebView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processSetUpIdleModeText-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/widget/LockPatternUtils;-getAppWidgets-()[I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getAccountTypesWithManagementDisabled-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameBranch;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-sendNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getCrossProfileCallerIdDisabledForUser-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/Invert;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/RIL;-sendTerminalResponse-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/RegisteredServicesCache$Callback;-onServicesUpdated-(I Ljava/util/List;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/State;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/widget/RadialTimePickerView;-dispatchPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent; Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/BlendFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-pollStateDone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-conference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerPolicy;-wakingUp-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/os/IPowerManager$Stub$Proxy;-updateWakeLockUids-(Landroid/os/IBinder; [I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IHardwareService$Stub;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Landroid/widget/ExpandableListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/FragmentBreadCrumbs;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TelephonyRegistry;-notifyDataConnectionForSubscriber-(J I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/NetworkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/Filter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/backup/BackupManagerService$PerformInitializeTask;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/pm/Installer;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/AccessibilityController;-drawMagnifiedRegionBorderIfNeededLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-isUninstallBlocked-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub;-setColor-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/telephony/SmsManager;-sendStoredMultipartTextMessage-(Landroid/net/Uri; Ljava/lang/String; Ljava/util/ArrayList; Ljava/util/ArrayList;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/UiccSmsController;-getAllMessagesFromIccEf-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/display/DisplayAdapter$2;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accessibilityservice/IAccessibilityServiceConnection;-getWindows-()Ljava/util/List;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/inputmethodservice/InputMethodService;-onCreate-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameFetch;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover;-complete-(Z)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/AlarmManagerService$2;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/nfc/cardemulation/AppChooserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/admin/IDevicePolicyManager;-getPasswordMinimumNonLetter-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipCallBase;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/GLTextureTarget;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/SubtitleController;-getDefaultTrack-()Landroid/media/SubtitleTrack;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getAccountTypesWithManagementDisabled-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ILockSettings$Stub$Proxy;-checkPassword-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcTag;-formatNdef-(I [B)I' : ['android.permission.NFC'],
	'Landroid/nfc/tech/Ndef;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Landroid/nfc/tech/Ndef;-makeReadOnly-()Z' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/dataconnection/DcController;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/WebVttRenderingWidget$RegionLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneFactory;-setDefaultSubscription-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FlipFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/PowerManager$WakeLock;-acquire-(J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/trust/TrustAgentService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-ensureWindowsAvailableTimed-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/hdmi/HdmiCecController$4;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/LocationManagerService$4;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$5100-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/server/display/AutomaticBrightnessController;-updateAutoBrightness-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/BackupAgentHelper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/VideoView;-release-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-inputDispatchingTimedOut-(I Z Ljava/lang/String;)J' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-removePrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService;-notifyProfileRemoved-(Landroid/net/sip/SipProfile;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService;-setDisplayPropertiesInternal-(I Z F Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-focusChangedLw-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/WindowManagerPolicy$WindowState;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendEnvelope-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ResolverDrawerLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/gsm/SmsManager;-getAllMessagesFromSim-()Ljava/util/ArrayList;' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/server/LocationManagerService;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityRecord$Token;-windowsVisible-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-updateAdnRecordsInEfByIndexForSubscriber-(J I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/net/sip/SipAudioCall;-access$800-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-gotoIdleAndNotifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-cancelPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-processNextPostDialChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager$CardEmulationInterface;-setDefaultForNextTap-(I Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone; Landroid/telephony/CellInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-makeCellBroadcastHandler-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)Lcom/android/internal/telephony/CellBroadcastHandler;' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler$SmsBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WRITE_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController$4;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setMaximumTimeToLock-(Landroid/content/ComponentName; J I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer;-setWakeMode-(Landroid/content/Context; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/backup/SystemBackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-rescheduleMissedAlarms-(Landroid/content/ContentResolver; Landroid/content/Context; Landroid/app/AlarmManager;)V' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/wifi/WifiScanningServiceImpl$WifiScanningStateMachine$StartedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendImsCdmaSms-([B I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/tv/TvInputManagerService$1;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/SystemServiceManager;-startBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/NetworkScorerAppManager;-isCallerActiveScorer-(Landroid/content/Context; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-startPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/pm/IPackageManager;-getInstallLocation-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/print/PrinterDiscoverySession;-finalize-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-deleteFromRawTable-(Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.WRITE_SMS'],
	'Lcom/android/ims/internal/IImsCallSessionListener$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-getNetworkCapabilities-(Ljava/lang/String;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier$MagnificationController;-setScaleAndMagnifiedRegionCenter-(F F F Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/ChooseAccountActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/display/AutomaticBrightnessController;-updateAmbientLux-(J)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/tts/TextToSpeech;-getVoices-()Ljava/util/Set;' : ['android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/CaptioningManager;-getFontScale-()F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-findTaskToMoveToFrontLocked-(Lcom/android/server/am/TaskRecord; I Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/SupplicantStateTracker;-access$300-(Lcom/android/server/wifi/SupplicantStateTracker; Landroid/net/wifi/SupplicantState; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telecom/ConnectionService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-addTrackerToRawTableAndSendMessage-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/SystemGesturesPointerEventListener$Callbacks;-onSwipeFromRight-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcTag$Stub;-resetTimeouts-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-updateSmsSendStatus-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IRestoreSession;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-queryPin1State-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/display/DisplayManagerService;-access$3700-(Lcom/android/server/display/DisplayManagerService; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/AdapterView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/NumberPicker$CustomEditText;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/NetInitiatedActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-moveTaskBackwards-(I)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getCrossProfileCallerIdDisabled-(Landroid/content/ComponentName;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Telephony$Sms$Sent;-addMessage-(J Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/accounts/AccountManagerService;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodSubtypeListLocked-(Landroid/view/inputmethod/InputMethodInfo;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ims/internal/IImsCallSessionListener$Stub$Proxy;-callSessionStartFailed-(Lcom/android/ims/internal/IImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/dataconnection/DcTrackerBase;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-explicitCallTransfer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$P2pDisabledState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextViewWithCircularIndicator;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getSelectedInputMethod-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/WebVttRenderingWidget$CueLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SmsManager;-deleteStoredMessage-(Landroid/net/Uri;)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/app/DisableCarModeActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/sip/ISipService;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/server/LocationManagerService$Receiver;-updateMonitoring-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ExpandableListActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onRemoteEndpointDiscovered-(Lcom/android/nfc/DeviceHost$TagEndpoint;)V' : ['android.permission.VIBRATE'],
	'Landroid/text/method/TextKeyListener;-getPrefs-(Landroid/content/Context;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActiveServices;-stopServiceLocked-(Lcom/android/server/am/ServiceRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-updateImsPhone-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/text/format/DateUtils;-getRelativeDateTimeString-(Landroid/content/Context; J J J I)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-setHdmiPlugged-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-getDisplayContentLocked-(I)Lcom/android/server/wm/DisplayContent;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-addGpsNavigationMessageListener-(Landroid/location/IGpsNavigationMessageListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/display/AutomaticBrightnessController$2;-onTwilightStateChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-wipeData-(I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgentHelper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/GlobalActions;-onAirplaneModeChanged-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wm/AccessibilityController$WindowsForAccessibilityObserver;-populateVisibleWindowsOnScreenLocked-(Landroid/util/SparseArray;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendStoredMultipartText-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/am/ActivityManagerService;-startProcessLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-supplyPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager$Stub;-addPrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$DriverStoppingState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/media/projection/MediaProjectionManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipManager;-makeAudioCall-(Landroid/net/sip/SipProfile; Landroid/net/sip/SipProfile; Landroid/net/sip/SipAudioCall$Listener; I)Landroid/net/sip/SipAudioCall;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$DeactingState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/voiceinteraction/SoundTriggerHelper;-stopAllRecognitions-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker$1;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-addCircularFence-(I Landroid/hardware/location/GeofenceHardwareRequestParcelable; Landroid/hardware/location/IGeofenceHardwareCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowManagerService$2;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/service/textservice/SpellCheckerService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/RIL;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setSuppServiceNotifications-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub;-renameAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/mock/MockContentResolver;-requestSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/ActivityManager;-getTaskThumbnail-(I)Landroid/app/ActivityManager$TaskThumbnail;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesSettings;-getSelectedSpellChecker-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Global;-getFloat-(Landroid/content/ContentResolver; Ljava/lang/String; F)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/TouchExplorer$PerformLongPressDelayed;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/NotificationManagerService$NotificationListeners;-onPackagesChanged-(Z [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowStateAnimator;-applyEnterAnimationLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-setAndPersistDeviceName-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/NfcService$ApplyRoutingTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/GrayscaleEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IEthernetManager;-setConfiguration-(Landroid/net/IpConfiguration;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FlipFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/WaveView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/print/PrintManager;-restartPrintJob-(Landroid/print/PrintJobId;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkScoreService$Stub$Proxy;-clearScores-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-dial-(Ljava/lang/String; I Lcom/android/internal/telephony/UUSInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowAnimator;-setAppLayoutChanges-(Lcom/android/server/wm/AppWindowAnimator; I Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/NfcService$TagService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/provider/Telephony$Sms$Outbox;-addMessage-(J Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long; Z J)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncActivityTooManyDeletes;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcActivatingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiStateMachine$ObtainingIpState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Settings$Global;-getInt-(Landroid/content/ContentResolver; Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-addMultimediaMessageDraft-(Ljava/lang/String; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/am/ActivityManagerService$15;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/NativeActivity$NativeContentView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getVoiceMailNumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/dreams/DreamManagerService;-startDreamLocked-(Landroid/content/ComponentName; Z Z I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-processStartTimedOutLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cat/CatService;-sendTerminalResponse-(Lcom/android/internal/telephony/cat/CommandDetails; Lcom/android/internal/telephony/cat/ResultCode; Z I Lcom/android/internal/telephony/cat/ResponseData;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IMms$Stub;-sendMessage-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCommandInterface;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub;-getCurrentSpellCheckerSubtype-(Ljava/lang/String; Z)Landroid/view/textservice/SpellCheckerSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CropFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/accounts/CantAddAccountActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-packageHasActiveAdmins-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ISyncAdapter;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/EthernetManager;-setConfiguration-(Landroid/net/IpConfiguration;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/wm/WindowManagerService$3;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/nfc/NfcService$TagService;-rediscover-(I)Landroid/nfc/Tag;' : ['android.permission.NFC'],
	'Lcom/android/server/wm/WindowManagerService;-reevaluateStatusBarVisibility-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/twilight/TwilightService$TwilightListenerRecord;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/GLTextureTarget;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/format/DateUtils;-formatDateTime-(Landroid/content/Context; J I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setLine1Number-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/uicc/AdnRecordCache;-updateAdnByIndex-(I Lcom/android/internal/telephony/uicc/AdnRecord; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DockObserver;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiController$ScanOnlyLockHeldState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISms;-sendTextForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager;-onPreferredForegroundServiceChanged-(Landroid/content/ComponentName;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onError-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/WaveView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/ISipService;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/server/wifi/WifiScanningServiceImpl$WifiChangeStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/os/IPowerManager$Stub$Proxy;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; Landroid/os/WorkSource; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/ExpandedMenuView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService$10;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/nfc/NfcApplication;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-getIMEI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$DriverStoppingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcDefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-acknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMALTEPhone;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiStateMachine$DriverStoppingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/ContentService;-getSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub$Proxy;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/widget/Editor$SelectionEndHandleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NfcB;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-revertToNitzTimeZone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockContentResolver;-isSyncPendingAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/notification/ManagedServices;-access$600-(Lcom/android/server/notification/ManagedServices;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/NativeActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/notification/NotificationManagerService$NotificationListeners;-onUserSwitched-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/SearchDialog$SearchBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayPowerController;-animateScreenStateChange-(I Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPasswordMinimumNumeric-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/printservice/PrintService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/MediaController;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-access$300-(Lcom/android/server/LocationManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/app/AliasActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-acknowledgeIncomingGsmSmsWithPdu-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/MediaSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/nsd/NsdManager;-init-()V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/widget/LockPatternUtils;-getBoolean-(Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/notification/ConditionProviderService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/backup/FullBackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/widget/RotarySelector;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/ContentResolver;-getSyncStatusAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/InboundSmsHandler$DefaultState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/print/IPrintManager$Stub;-getInstalledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-enableCellBroadcastForSubscriber-(J I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/app/AlertActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/PlaybackQueueItem;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/app/admin/IDevicePolicyManager;-getPermittedAccessibilityServices-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-sendEnvelopeWithStatus-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/QuickContactBadge;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GeofenceManager;-addFence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; I I Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Landroid/widget/SpellChecker;-resetSession-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothManager;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/media/effect/effects/ColorTemperatureEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/os/SystemVibrator;-vibrate-([J I Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/TintFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/DisplayContent;-switchUserStacks-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-updateThisDevice-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService$Lifecycle;-onBootPhase-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cat/IconLoader;-loadIcon-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager;-onServicesUpdated-(I Ljava/util/List;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl$2;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getAutoTimeRequired-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NfcF;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getTimeCacheMaxAge-()J' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityGroup;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/AudioService$VolumeStreamState;-<init>-(Landroid/media/AudioService; Ljava/lang/String; I Landroid/media/AudioService$1;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-systemReady-(Lcom/android/internal/app/IAppOpsService;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService;-access$200-(Landroid/hardware/location/GeofenceHardwareService; I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-access$3600-(Lcom/android/server/power/PowerManagerService; J I I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setAccountManagementDisabled-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TableRow;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService;-disableSafeMediaVolume-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/CalendarContract$CalendarEntity$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/power/PowerManagerService;-userActivityInternal-(J I I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-showBootMessage-(Ljava/lang/CharSequence; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/IAlarmManager$Stub;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/pm/PackageManagerService$11;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-clearSubInfo-()I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getDataRoamingEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-sendAdminCommandToSelfAndProfilesLocked-(Ljava/lang/String; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IMediaHTTPConnection$Stub$Proxy;-getMIMEType-()Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/server/connectivity/PacManager;-access$700-(Lcom/android/server/connectivity/PacManager;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/Session;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/power/PowerManagerService$BinderService;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/preference/PreferenceActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getKeyguardDisabledFeatures-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/SpeechRecognizer;-stopListening-()V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPermittedAccessibilityServices-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-moveInputMethodDialogsLocked-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/SubscriptionController;-clearSubInfo-()I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/os/IHardwareService$Stub$Proxy;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Landroid/service/textservice/SpellCheckerService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/Presentation$1;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-iccIO-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub;-keyguardGoingAway-(Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/GridView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-pollBatchedScan-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/AliasActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/TechListChooserActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/nfc/tech/IsoDep;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Landroid/provider/Settings$System;-getShowGTalkServiceStatusForUser-(Landroid/content/ContentResolver; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-onSubscriptionActivated-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/wifi/IWifiManager$Stub;-stopBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-clearPackagePersistentPreferredActivities-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/LauncherActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-isLockTaskPermitted-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService$Stub;-disableSafeMediaVolume-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/NfcAdapter;-setP2pModes-(I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech;-playSilentUtterance-(J I Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/input/InputManagerService; Z Z Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/CheckedTextView;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/ServiceWatcher$2;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-access$1100-(Lcom/android/server/wm/WindowManagerService;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/SystemServiceManager;-startService-(Ljava/lang/Class;)Lcom/android/server/SystemService;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-getSubInfoForSubscriber-(J)Landroid/telephony/SubInfoRecord;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/app/IntentForwarderActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-setDisplayNumber-(Ljava/lang/String; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/widget/LinearLayoutWithDefaultTouchRecepient;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/preference/PreferenceActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/SubscriptionManager;-setDefaultSmsSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/WifiServiceImpl$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-getReceiverLocked-(Landroid/app/PendingIntent; I I Ljava/lang/String; Landroid/os/WorkSource; Z)Lcom/android/server/LocationManagerService$Receiver;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ProxyController;-updateDataConnectionTracker-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/NetworkMonitor$OfflineState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService;-addAccountAsUser-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle; I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/text/format/DateUtils;-formatDateRange-(Landroid/content/Context; J J I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService$8;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/accessibility/CaptioningManager;-getUserStyle-()Landroid/view/accessibility/CaptioningManager$CaptionStyle;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceMonitorStatus-(I I Landroid/location/Location; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/app/UiAutomationConnection;-registerUiTestAutomationServiceLocked-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-sendImsGsmSms-(Ljava/lang/String; Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/AbstractThreadedSyncAdapter$ISyncAdapterImpl;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiManager;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/ActivityView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/util/AsyncService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-detachStack-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/BaseCommands;-setUiccSubscription-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-notifyANR-(Lcom/android/server/input/InputApplicationHandle; Lcom/android/server/input/InputWindowHandle; Ljava/lang/String;)J' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/AliasActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangup-(Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/MultiAutoCompleteTextView;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FlipFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processDisplayText-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/UiccSmsController;-updateMessageOnIccEf-(Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/updates/SELinuxPolicyInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getPasswordQuality-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/app/AlertActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-requestSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/LauncherAppsService$LauncherAppsImpl$MyPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/widget/TextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/ContentResolver;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/test/mock/MockContentResolver;-requestSync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-updatePhoneObject-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/am/ActivityStack;-completePauseLocked-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-removeProviderLocked-(Lcom/android/server/location/LocationProviderInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverService;-disableBluetoothIfNeeded-()V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/backup/BackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/accessibility/CaptioningManager$2;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-getTaskThumbnail-(I)Landroid/app/ActivityManager$TaskThumbnail;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/display/DisplayPowerController$2;-onAnimationEnd-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterfw/core/FilterFunction;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingInsecureRfcommWithServiceRecord-(Ljava/lang/String; Ljava/util/UUID;)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-deleteSmsOnSim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStackSupervisor;-moveTaskToStack-(I I Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/display/DisplayManagerService;-handleDisplayDeviceRemovedLocked-(Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothSocket;-initSocketNative-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/UiAutomationConnection;-disconnect-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent; Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/widget/ILockSettings$Stub$Proxy;-getLong-(Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-killAllBackgroundProcesses-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/server/wm/WindowManagerService;-requestTraversalLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager;-getBatchedScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-reportSuccessfulPasswordAttempt-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-onFinished-(Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Landroid/nfc/INfcAdapter$Stub;-setReaderMode-(Landroid/os/IBinder; Landroid/nfc/IAppCallback; I Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/NativeActivity$NativeContentView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/SipAudioCall;-setSpeakerMode-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/app/LauncherActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/TabActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-getIsimChallengeResponse-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AdapterView;-performItemClick-(Landroid/view/View; I J)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/widget/FrameLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-finishBooting-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/wifi/WifiSettingsStore;-getPersistedScanAlwaysAvailable-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AdapterView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getDevPersistBytes-(J)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayPowerController;-<init>-(Landroid/content/Context; Landroid/hardware/display/DisplayManagerInternal$DisplayPowerCallbacks; Landroid/os/Handler; Landroid/hardware/SensorManager; Lcom/android/server/display/DisplayBlanker;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/WaveView;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/PreferredServices;-unregisteredPreferredForegroundService-(I)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-getBoolean-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-isDataPossible-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/TintFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$WaitingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/LocationManager;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/LocationManagerService;-switchUser-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/PreferredServices;-loadDefaultsFromSettings-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/WapPushOverSms;-writeInboxMessage-(J [B)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/nfc/cardemulation/TapAgainDialog;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneProxy;-shutdownRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/GpsLocationProvider;-access$3600-(Lcom/android/server/location/GpsLocationProvider; I Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/widget/AbsoluteLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/util/Log$1;-onTerribleFailure-(Ljava/lang/String; Landroid/util/Log$TerribleFailure; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/IActivityContainer$Stub$Proxy;-release-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/power/PowerManagerService;-handleSandman-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/BackgroundDexOptService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-createUser-(Landroid/content/ComponentName; Ljava/lang/String;)Landroid/os/UserHandle;' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/CardEmulation;-getAidsForService-(Landroid/content/ComponentName; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Landroid/preference/SeekBarVolumizer;-onStopSample-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-access$000-(Lcom/android/server/accessibility/ScreenMagnifier; Landroid/graphics/Region;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/backup/BackupManagerService$Lifecycle;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setMaximumFailedPasswordsForWipe-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-updateWindowsForAccessibilityCallbackLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IccCard;-supplyPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onRegistrationFailed-(Landroid/net/sip/ISipSession; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-appDiedLocked-(Lcom/android/server/am/ProcessRecord; I Landroid/app/IApplicationThread;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/LayoutInflater$BlinkLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/inputmethodservice/ExtractEditLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-setSafeMode-(Z)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService;-vibratePattern-(I Ljava/lang/String; [J I I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processSetupCall-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/os/IDropBoxManagerService$Stub$Proxy;-add-(Landroid/os/DropBoxManager$Entry;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-getSyncManager-()Lcom/android/server/content/SyncManager;' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/CalendarView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readConfigurationForUserStateLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager;-startPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IUiAutomationConnection$Stub$Proxy;-shutdown-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/tech/MifareUltralight;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-setDeviceOwner-(Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-update-(Lcom/android/ims/ImsCall; Lcom/android/internal/telephony/Call$State;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController;-enableAccessibility-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-finishUserStop-(Lcom/android/server/am/UserStartedState;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/Ndef;-getNdefMessage-()Landroid/nfc/NdefMessage;' : ['android.permission.NFC'],
	'Landroid/inputmethodservice/KeyboardView;-sendAccessibilityEventForUnicodeCharacter-(I I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgentHelper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/wallpaper/WallpaperService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/location/GeofenceHardwareService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiServiceImpl;-checkAndStartWifi-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/app/admin/IDevicePolicyManager;-getCrossProfileCallerIdDisabled-(Landroid/content/ComponentName;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/SupplicantStateTracker$HandshakeState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/cardemulation/CardEmulation;-removeAidsForService-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-setP2pModes-(I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub$Stub;-setDisplayName-(Ljava/lang/String; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-writeSmsToSim-(I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/Ndef;-writeNdefMessage-(Landroid/nfc/NdefMessage;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setAndBroadcastNetworkSetTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiController$NoLockHeldState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-cleanUpAllConnections-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/provider/Telephony$Threads;-getOrCreateThreadId-(Landroid/content/Context; Ljava/util/Set;)J' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/PhoneFactory;-makeDefaultPhones-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-isGlobalAccessibilityGestureEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipAudioCall;-transferToNewSession-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/base/FrameFetch;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/GLTextureSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IRestoreSession$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IntentService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/service/dreams/DreamService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/CertBlacklister;-<init>-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/notification/ManagedServices;-disableNonexistentServices-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/HdmiControlService;-access$1200-(Lcom/android/server/hdmi/HdmiControlService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/AlarmManagerService$InteractiveStateReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setDataRoamingEnabled-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/voice/VoiceInteractionSessionService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager;-enableAggressiveHandover-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-bindBackupAgent-(Landroid/content/pm/ApplicationInfo; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-applyPostLayoutPolicyLw-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/WindowManager$LayoutParams;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-enableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CallerInfo;-markAsVoiceMail-()Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-updateMaximumTimeToLockLocked-(Lcom/android/server/devicepolicy/DevicePolicyManagerService$DevicePolicyData;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/tv/TvView$3;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/TelephonyManager;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/BluetoothServerSocket;-<init>-(I Z Z I)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-handleFileUpdate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-reportActivityInfo-()Landroid/net/wifi/WifiActivityEnergyInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/TelephonyRegistry;-broadcastDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String; J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CallerInfoAsyncQuery$CallerInfoAsyncQueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS'],
	'Landroid/app/backup/BackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/SlidingTab;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/NullFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-removeCrossProfileWidgetProvider-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/SubtitleTrack$RenderingWidget;-setVisible-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipCommandInterface;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-isLocationProviderEnabledForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-setKeyboardMode-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-deletePackageLI-(Ljava/lang/String; Landroid/os/UserHandle; Z [I [Z I Lcom/android/server/pm/PackageManagerService$PackageRemovedInfo; Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accessibilityservice/AccessibilityService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readTouchExplorationGrantedAccessibilityServicesLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/IccProvider;-query-(Landroid/net/Uri; [Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-addGpsStatusListener-(Landroid/location/IGpsStatusListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager;-isAdminActive-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-updateWallpaperVisibilityLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/sip/ISipService$Stub;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-copyMessageToIccEf-(Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/print/PrintManager;-removePrintJobStateChangeListener-(Landroid/print/PrintManager$PrintJobStateChangeListener;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-processMsg-(Landroid/os/Message;)Lcom/android/internal/util/State;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Landroid/view/WindowManagerPolicy$WindowManagerFuncs;-rebootSafeMode-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/RotarySelector;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-pauseGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/opengl/GLSurfaceView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SubscriptionManager;-setDefaultVoiceSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeechService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-updateCurrentCarrierInProvider-(Ljava/lang/String;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifySignalStrengthForSubscriber-(J Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActiveServices;-removeConnectionLocked-(Lcom/android/server/am/ConnectionRecord; Lcom/android/server/am/ProcessRecord; Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setDataAllowed-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TableLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer;-create-(Landroid/content/Context; Landroid/net/Uri; Landroid/view/SurfaceHolder;)Landroid/media/MediaPlayer;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/SubInfoRecordUpdater;-updateSimInfoByIccId-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/twilight/TwilightService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService$TagService;-isNdef-(I)Z' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-processCdmaWapPdu-([B I Ljava/lang/String; J)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/DockObserver;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-setTimeout-(I I)I' : ['android.permission.NFC'],
	'Lcom/android/server/accounts/AccountManagerService$8;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifySignalStrengthForSubscriber-(J Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiServiceImpl;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/NfcActivityManager;-setNdefPushContentUriCallback-(Landroid/app/Activity; Landroid/nfc/NfcAdapter$CreateBeamUrisCallback;)V' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/cardemulation/PreferredServices;-onUidToBackground-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/RIL;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiAutoJoinController;-newHalScanResults-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-setTokenVisibilityLocked-(Lcom/android/server/wm/AppWindowToken; Landroid/view/WindowManager$LayoutParams; Z I Z Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getPDPContextList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telecom/ConnectionService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-loadInputMethodAndSubtypeHistoryLocked-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ViewOverlay$OverlayViewGroup;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-sendStoredText-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/net/wifi/IWifiManager$Stub;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-<init>-(Landroid/content/Context; Landroid/os/Messenger;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPasswordQuality-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/NativeActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/AlarmManager;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/nfc/handover/HandoverService;-access$100-(Lcom/android/nfc/handover/HandoverService; Landroid/content/Intent; I)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/PhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendMultipartSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-onMotionEvent-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/content/SyncManager$7;-onSyncRequest-(Lcom/android/server/content/SyncStorageEngine$EndPoint; I Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/DownloadManager;-addCompletedDownload-(Ljava/lang/String; Ljava/lang/String; Z Ljava/lang/String; Ljava/lang/String; J Z)J' : ['android.permission.INTERNET'],
	'Landroid/hardware/display/DisplayManagerInternal;-initPowerManagement-(Landroid/hardware/display/DisplayManagerInternal$DisplayPowerCallbacks; Landroid/os/Handler; Landroid/hardware/SensorManager;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/cardemulation/CardEmulation;-setDefaultForNextTap-(Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doRestoreFinished-(I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifyDataConnectionForSubscriber-(J I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/NetworkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-performHapticFeedbackLw-(Landroid/view/WindowManagerPolicy$WindowState; I Z)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/widget/EditText;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/RotationPolicy;-getRotationLockOrientation-(Landroid/content/Context;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/InstrumentationCoreTestRunner;-finish-(I Landroid/os/Bundle;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/INfcTag$Stub;-isNdef-(I)Z' : ['android.permission.NFC'],
	'Landroid/media/ClosedCaptionWidget;-manageChangeListener-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager;-onNfcEnabled-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofenceResumeStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/VideoView;-getAudioSessionId-()I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setSecureSetting-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService$TestFeaturesSession;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/LocationManagerService$Receiver;-callStatusChangedLocked-(Ljava/lang/String; I Landroid/os/Bundle;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/server/DropBoxManagerService;-access$300-(Lcom/android/server/DropBoxManagerService;)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GeofenceManager;-getFreshLocationLocked-()Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/ims/ImsEcbmStateListener;-onECBMEntered-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeech;-isLanguageAvailable-(Ljava/util/Locale;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaSubscriptionSourceManager;-getDefault-(Landroid/content/Context;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowSession;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-setPasswordMinimumNonLetter-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/wifi/WifiStateMachine$TetheringState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/media/MediaBrowserService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/CheckBox;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/SlidingTab;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-performAuditoryFeedbackForAccessibilityIfNeed-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/speech/tts/TextToSpeechService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService$17;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendNewSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/SmsHeader; I Landroid/app/PendingIntent; Landroid/app/PendingIntent; Z Ljava/util/concurrent/atomic/AtomicInteger; Ljava/util/concurrent/atomic/AtomicBoolean; Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/view/inputmethod/InputMethodManager;-getEnabledInputMethodSubtypeList-(Landroid/view/inputmethod/InputMethodInfo; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ims/ImsCall$ImsCallSessionListenerProxy;-callSessionTerminated-(Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub;-disableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-handleStatusReport-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcAdapter;-invokeBeamInternal-(Landroid/nfc/BeamShareData;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/am/ActivityStackSupervisor$ActivityContainer;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCommandInterface;-setDataAllowed-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub$Proxy;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getRemoveWarning-(Landroid/content/ComponentName; Landroid/os/RemoteCallback; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/ITextToSpeechService;-speak-(Landroid/os/IBinder; Ljava/lang/CharSequence; I Landroid/os/Bundle; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/AutomaticBrightnessController;-configure-(Z F)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ActivityManagerNative;-handleApplicationCrash-(Landroid/os/IBinder; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-setInitialAttachApn-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/NdefFormatable;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/server/am/ActivityManagerService;-broadcastIntentLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Landroid/content/Intent; Ljava/lang/String; Landroid/content/IIntentReceiver; I Ljava/lang/String; Landroid/os/Bundle; Ljava/lang/String; I Z Z I I I)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readDisplayMagnificationEnabledSettingLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService$SipSessionGroupExt;-onConnectivityChanged-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaPlayer$4;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getCameraDisabled-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService;-onInputDeviceChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IMms$Stub;-setAutoPersisting-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SMS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getScreenCaptureDisabled-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/tv/TvView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-installKeyPair-(Landroid/content/ComponentName; [B [B Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-handleAnimatingStoppedAndTransitionLocked-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getNetworkCapabilities-(Ljava/lang/String;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPasswordExpirationTimeout-(Landroid/content/ComponentName; J I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService$13;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService$Stub;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/search/SearchManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/app/ResolverActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/provider/Browser;-getVisitedLike-(Landroid/content/ContentResolver; Ljava/lang/String;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/app/ActivityManagerNative;-bootAnimationComplete-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/MediaFocusControl;-access$1400-(Landroid/media/MediaFocusControl;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPasswordMinimumNumeric-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/IntentFirewallInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-access$100-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/app/PlatLogoActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/test/mock/MockContentResolver;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService$Stub$Proxy;-setStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/widget/DialogTitle;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/HostApduService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/nfc/handover/ConfirmConnectActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getBatchedScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/ActivityView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/AccessibilityInteractionClient;-getWindow-(I I)Landroid/view/accessibility/AccessibilityWindowInfo;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ProxyController;-enableDataConnectivity-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneProxy;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/sip/SipManager;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Landroid/app/IActivityManager;-getTasks-(I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Landroid/accounts/IAccountManager;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Landroid/speech/tts/TextToSpeech$8;-run-(Landroid/speech/tts/ITextToSpeechService;)Ljava/util/Locale;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-handleApplicationWtf-(Landroid/os/IBinder; Ljava/lang/String; Z Landroid/app/ApplicationErrorReport$CrashInfo;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/DropBoxManager;-addFile-(Ljava/lang/String; Ljava/io/File; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setDataProfile-([Lcom/android/internal/telephony/dataconnection/DataProfile; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setAutoTimeRequired-(Landroid/content/ComponentName; I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/appwidget/AppWidgetHostView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getDataEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/webkit/WebViewUpdateService;-access$000-(Lcom/android/server/webkit/WebViewUpdateService;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService;-updatePointerSpeedFromSettings-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/InstrumentationCoreTestRunner;-onStart-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/pm/PackageManagerService;-deleteSystemPackageLI-(Lcom/android/server/pm/PackageSetting; [I [Z I Lcom/android/server/pm/PackageManagerService$PackageRemovedInfo; Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/SubInfoRecordUpdater;-queryIccId-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine;-sendNetworkStateChangeBroadcast-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/ComprehensiveCountryDetector$2;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/updates/TZInfoInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier; Lcom/android/internal/telephony/Phone;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/HdmiControlService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/pm/PackageManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/internal/widget/RotarySelector;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/ActionMenuPresenter$OverflowMenuButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/DragState;-endDragLw-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/telephony/TelephonyManager;-getMsisdn-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/updates/ConfigUpdateInstallReceiver$1;-run-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringLogger;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionCallReceiverImpl;-access$1600-(Lcom/android/server/sip/SipSessionGroup$SipSessionImpl; Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/MountServiceIdler;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/sip/SipAudioCall;-onError-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/MediaSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/TabWidget;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService$PerformClearTask;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiScanningServiceImpl$WifiChangeStateMachine;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-onConnect-(Lcom/android/internal/telephony/dataconnection/DataConnection$ConnectionParams;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaFocusControl;-filterMediaKeyEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService;-startBluetoothSco-(Landroid/os/IBinder; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-getStorageEncryption-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/SRTTrack;-hide-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/contacts/ContactsListActivity;-getSelectedUri-(I)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$People;-tryGetMyContactsGroupId-(Landroid/content/ContentResolver;)J' : ['android.permission.READ_CONTACTS'],
	'Landroid/widget/CheckedTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/IsoDep;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/location/provider/LocationProviderBase;-reportLocation-(Landroid/location/Location;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/view/WindowManagerPolicy;-finishPostLayoutPolicyLw-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/backup/FullBackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/DhcpStateMachine$RunningState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupConnectionByIndex-(Lcom/android/internal/telephony/cdma/CdmaCall; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/SMSDispatcher$SettingsObserver;-<init>-(Landroid/os/Handler; Ljava/util/concurrent/atomic/AtomicInteger; Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LockSettingsService;-checkReadPermission-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/FrameLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/MifareClassic;-authenticate-(I [B Z)Z' : ['android.permission.NFC'],
	'Landroid/net/INetworkScoreService;-clearScores-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiServiceImpl;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings; Landroid/os/IBinder; Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-removeActiveAdmin-(Landroid/content/ComponentName; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-updateStoredMessageStatus-(Ljava/lang/String; Landroid/net/Uri; Landroid/content/ContentValues;)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncStorageEngine;-setIsTargetServiceActive-(Landroid/content/ComponentName; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/provider/ContactsContract$RawContacts$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/am/CompatModePackages;-setPackageScreenCompatModeLocked-(Landroid/content/pm/ApplicationInfo; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/SmsStorageMonitor$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/SmsMessage;-getSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Z [B I I I)Lcom/android/internal/telephony/gsm/SmsMessage$SubmitPdu;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPermittedInputMethodsForCurrentUser-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/TouchExplorer;-onMotionEvent-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCall;-merge-(Lcom/android/internal/telephony/imsphone/ImsPhoneCall; Lcom/android/internal/telephony/Call$State;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-updateSmsSendStatus-(I Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Service;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-setProcessLimit-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActiveServices;-killServicesLocked-(Lcom/android/server/am/ProcessRecord; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/hardware/location/IGeofenceHardware;-addCircularFence-(I Landroid/hardware/location/GeofenceHardwareRequestParcelable; Landroid/hardware/location/IGeofenceHardwareCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-getIMSI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-addAccount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/SystemServer$2;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-getNetworkCapabilities-(Ljava/lang/String;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$StopTetheringErrorState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/AlarmManagerService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-access$500-(Lcom/android/server/devicepolicy/DevicePolicyManagerService; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-clearProfileOwner-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/CertBlacklister$BlacklistObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-writeSmsToRuim-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ToggleButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onEnableApn-(I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/telecom/ConnectionService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/input/InputManagerService;-getShowTouchesSetting-(I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcBackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISms$Stub;-enableCellBroadcastRangeForSubscriber-(J I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/BaseCommands;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AdapterViewAnimator;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/AudioService$VolumeController;-loadSettings-(Landroid/content/ContentResolver;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-finishUserSwitch-(Lcom/android/server/am/UserStartedState;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-isFeatureSupported-(I)Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/DigitalClock;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkStatsService$1;-getSummaryForNetwork-(Landroid/net/NetworkTemplate; J J)Landroid/net/NetworkStats;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onRadioAvailable-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/dreams/DreamManagerService$LocalService;-startDream-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-deleteStoredConversation-(Ljava/lang/String; J)Z' : ['android.permission.WRITE_SMS'],
	'Landroid/media/IAudioService$Stub$Proxy;-adjustStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/GhostView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$6;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/net/TrafficStats;-stopDataProfiling-(Landroid/content/Context;)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/updates/TZInfoInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/AlarmManagerService;-interactiveStateChangedLocked-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telecom/InCallService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager$1;-startTask-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneProxy;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/CoreSettingsObserver;-populateSettings-(Landroid/os/Bundle; Ljava/util/Map;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/NotificationManagerService$5;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/AccessibilityController$DisplayMagnifier$MagnifiedViewport;-recomputeBoundsLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/IAudioService;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-switchToState-(I F F)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-replaceSystemPackageLI-(Landroid/content/pm/PackageParser$Package; Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; [I [Z Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-operationComplete-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-handlePackagesChanged-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/UserState;-updateIfNeededLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcCardEmulation$Stub;-getAidGroupForService-(I Landroid/content/ComponentName; Ljava/lang/String;)Landroid/nfc/cardemulation/AidGroup;' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/view/IWindowSession$Stub;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/ContentResolver;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/telephony/SmsManager;-enableCellBroadcastRange-(I I)Z' : ['android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcActiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/power/PowerManagerService;-updateUserActivitySummaryLocked-(J I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-removePrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractEditText;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ViewAnimator;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handleCallWaitingInfo-(Lcom/android/internal/telephony/cdma/CdmaCallWaitingNotification;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-access$200-(Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ConditionProviders$DowntimeCallback;-onDowntimeChanged-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/bmgr/Bmgr;-doList-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/PackageManagerService$14;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-saveSettingsLocked-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ims/internal/IImsCallSessionListener;-callSessionStartFailed-(Lcom/android/ims/internal/IImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Ljava/net/HttpURLConnection;-connect-()V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifyServiceState-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wallpaper/WallpaperManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_WALLPAPER_HINTS', 'android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/SmsUsageMonitor$SettingsObserverHandler;-<init>-(Landroid/content/Context; Ljava/util/concurrent/atomic/AtomicBoolean;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/text/ToUpperCase;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/app/DisableCarModeActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/display/DisplayPowerController;-access$500-(Lcom/android/server/display/DisplayPowerController;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-killAllBackgroundProcesses-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-startDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setGlobalSetting-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/SubtitleTrack;-hide-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/view/menu/ExpandedMenuView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/ContextWrapper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/display/DisplayManagerService;-access$1600-(Lcom/android/server/display/DisplayManagerService; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/ClosedCaptionWidget;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Telephony$Sms$Draft;-addMessage-(J Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/BluetoothAdapter;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardHelper;-handleModeChange-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-getEnabledProviders-()Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-setLocationUpdates-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/service/dreams/DreamService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/display/AutomaticBrightnessController;-handleLightSensorEvent-(J F)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService$Stub$Proxy;-setMasterSyncAutomaticallyAsUser-(Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/provider/CalendarContract$EventDays;-query-(Landroid/content/ContentResolver; I I [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-deleteFromRawTable-(Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.WRITE_SMS'],
	'Landroid/accounts/AccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/accounts/AccountManager$2;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/DropBoxManagerService;-trimToFit-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-sendRawPdu-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipSessionGroup;-processIOException-(Ljavax/sip/IOExceptionEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms;-enableCellBroadcastForSubscriber-(J I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/server/display/OverlayDisplayAdapter;-access$000-(Lcom/android/server/display/OverlayDisplayAdapter;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisToFileOutputStreamSpeechItemV1;-getSpeechRate-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-updateSettingsLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub$Proxy;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/accessibility/CaptioningManager;-registerObserver-(Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-supplyPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RadioButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setPropertiesByCarrier-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-temporaryEnableAccessibilityStateUntilKeyguardRemoved-(Landroid/content/ComponentName; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/Activity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/AlarmManagerService;-formatNextAlarm-(Landroid/content/Context; Landroid/app/AlarmManager$AlarmClockInfo;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-updateCurrentCarrierInProvider-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telecom/InCallService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/content/ContentResolver;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/IAlarmManager;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/BluetoothManagerService;-isBluetoothPersistedStateOn-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-iccTransmitApduLogicalChannel-(I I I I I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ChooserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothManager;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/DeviceHost$TagDisconnectedCallback;-onTagDisconnected-(J)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringLogger;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/base/InputStreamSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-handleOnRectangleOnScreenRequested-(I I I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setApplicationRestrictions-(Landroid/content/ComponentName; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-ensureBootCompleted-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/DhcpStateMachine;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub$Proxy;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/view/ViewStub;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Presentation$1;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ReceiverRestrictedContext;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowManager$Stub;-getBaseDisplayDensity-(I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/TechListChooserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-<init>-(Landroid/content/Context; Landroid/location/ILocationManager; Landroid/os/Looper;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/CalendarView$LegacyCalendarViewDelegate$WeekView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getLockTaskPackages-(Landroid/content/ComponentName;)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/WaveView;-waveUpdateFrame-(F F Z)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-injectSmsPdu-([B Ljava/lang/String; Landroid/app/PendingIntent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService$AudioServiceBroadcastReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Landroid/speech/tts/TextToSpeechService;-getSettingsLocale-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisSpeechItemV1;-getSpeechRate-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService;-access$700-(Lcom/android/server/net/NetworkStatsService; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/MmsServiceBroker;-access$900-()Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/widget/AdapterViewAnimator;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/Invert;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getLastPdpFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/FlipEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/cardemulation/OffHostApduService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/DateTimeView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Activity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/CountryDetectorService$2;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-invokeExitMethods-(Lcom/android/internal/util/StateMachine$SmHandler$StateInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Contacts$People;-addToGroup-(Landroid/content/ContentResolver; J Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/text/ToUpperCase;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/PreferredServices;-onServicesUpdated-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/CertBlacklister;-registerObservers-(Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/UserDictionary$Words;-addWord-(Landroid/content/Context; Ljava/lang/String; I Ljava/lang/String; Ljava/util/Locale;)V' : ['android.permission.WRITE_USER_DICTIONARY'],
	'Landroid/test/IsolatedContext$MockAccountManager;-removeAccount-(Landroid/accounts/Account; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/content/SyncManager$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/TelephonyManager;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/NumberPicker$AccessibilityNodeProviderImpl;-sendAccessibilityEventForVirtualText-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/CardEmulation;-registerAidsForService-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/util/List;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/server/power/ShutdownThread;-access$000-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-setGrabbedState-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendMultipartTextForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getSubscriberIdForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/Switch;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/AlarmManagerService;-deliverAlarmsLocked-(Ljava/util/ArrayList; J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-getLastKnownLocation-()Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/notification/ConditionProviders;-access$300-(Lcom/android/server/notification/ConditionProviders;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkStatsRecorder;-recoverFromWtf-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/BlackWhiteEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub$Proxy;-getCurrentSpellChecker-(Ljava/lang/String;)Landroid/view/textservice/SpellCheckerInfo;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/ExtractEditText;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/voice/VoiceInteractionService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/backup/LocalTransportService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiStateMachine$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-processNextPostDialChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-getInitialMaxRetry-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BluetoothManagerService;-access$400-(Lcom/android/server/BluetoothManagerService;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms;-sendStoredMultipartText-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/RemoteViewsService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/trust/TrustManagerService;-access$1100-(Lcom/android/server/trust/TrustManagerService; Z I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-addTrackerToRawTable-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/widget/CheckedTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Switch;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/dreams/DreamManagerInternal;-stopDream-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/RotarySelector;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService;-setSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-processPostDialChar-(C)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-print-(Ljava/lang/String; Landroid/print/IPrintDocumentAdapter; Landroid/print/PrintAttributes; Ljava/lang/String; I I)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager$CardEmulationInterface;-registerAidGroupForService-(I Landroid/content/ComponentName; Landroid/nfc/cardemulation/AidGroup;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/HdmiCecLocalDeviceTv;-<init>-(Lcom/android/server/hdmi/HdmiControlService;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAccountsByType-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-isOffChannelTdlsSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onSetUserDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccPhoneBookController;-getAdnRecordsSizeForSubscriber-(J I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-update-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/BrowserContract$Bookmarks;-buildFolderUri-(J)Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/widget/ScrollingTabContainerView$TabView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/BroadcastQueue;-deliverToRegisteredReceiverLocked-(Lcom/android/server/am/BroadcastRecord; Lcom/android/server/am/BroadcastFilter; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$ProvisionDiscoveryState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/GpsMeasurementListenerTransport;-registerWithServer-()Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/ZoomButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/dreams/DreamManagerService;-finishSelfInternal-(Landroid/os/IBinder; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManager;-getRecentTasks-(I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Landroid/content/ISyncContext$Stub$Proxy;-onFinished-(Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/sip/ISipService$Stub;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-updateCarrierMccMncConfiguration-(Ljava/lang/String; Ljava/lang/String; Landroid/content/Context;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/wm/WindowManagerService;-handleAppTransitionReadyLocked-(Lcom/android/server/wm/WindowList;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-dispatchMessageRadioSpecific-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-getAnyDataEnabled-(Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/IRecognitionService$Stub;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/server/wifi/WifiServiceImpl$3;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/IAudioService$Stub;-setMicrophoneMute-(Z Ljava/lang/String;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneProxy;-setInternalDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallback;-onResponse-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub$Stub;-setDefaultVoiceSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-enableAggressiveHandover-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService$11;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/backup/FullBackupJob;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/IWindowManager;-keyguardGoingAway-(Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-dialInternal-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/media/MediaPlayer;-create-(Landroid/content/Context; I)Landroid/media/MediaPlayer;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/RetargetFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-openPanel-(Lcom/android/internal/policy/impl/PhoneWindow$PanelFeatureState; Landroid/view/KeyEvent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-update-(Lcom/android/internal/telephony/DriverCall;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-getBoolean-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityRecord;-setTask-(Lcom/android/server/am/TaskRecord; Lcom/android/server/am/TaskRecord;)V' : ['android.permission.SEND_SMS', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/monkey/MonkeySourceNetwork;-getNextEvent-()Lcom/android/commands/monkey/MonkeyEvent;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/hardware/input/InputManager$InputDeviceVibrator;-vibrate-([J I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-access$2600-(Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService$2;-onProximityNegative-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-importMultimediaMessage-(Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; J Z Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/app/IActivityManager;-killBackgroundProcesses-(Ljava/lang/String; I)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Landroid/print/IPrintManager$Stub$Proxy;-restartPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-enableSystemApp-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-userActivityFromNative-(J I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/ExpandableListActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Ljava/net/URLConnection;-getInputStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/server/content/SyncManager;-increaseBackoffSetting-(Lcom/android/server/content/SyncOperation;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/text/method/QwertyKeyListener;-onKeyDown-(Landroid/view/View; Landroid/text/Editable; I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPasswordMinimumNonLetter-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub$Proxy;-getSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-onRequestConditions-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer$2;-addTrack-()I' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/LocationListener; Landroid/os/Looper; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/notification/NotificationManagerService;-access$4000-(Lcom/android/server/notification/NotificationManagerService; Landroid/os/Message;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS'],
	'Landroid/location/ILocationManager;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/notification/NotificationManagerService$SettingsObserver;-update-(Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/monkey/Monkey;-runMonkeyCycles-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/tts/TtsEngines;-isLocaleSetToDefaultForEngine-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ContextWrapper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/telephony/TelephonyManager;-listen-(Landroid/telephony/PhoneStateListener; I)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPasswordMinimumSymbols-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcAdapter$Stub;-setP2pModes-(I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AsyncPlayer;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/am/ActivityStackSupervisor;-scheduleIdleLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/handover/HandoverTransfer;-cancel-()V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/pm/LauncherAppsService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-uninstallCaCert-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IBackupAgent$Stub;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/AudioService$ScoClient;-incCount-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Landroid/view/IWindowManager$Stub$Proxy;-isRotationFrozen-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-conference-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-setDefaultDataSubId-(J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$5400-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setKeyguardDisabledFeatures-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/gesture/GestureOverlayView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-getDataOnRoamingEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/KeyguardManager$KeyguardLock;-disableKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/server/pm/PackageManagerService$12;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/hdmi/HdmiControlService;-setControlEnabled-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/NfcActivityManager;-setNdefPushMessage-(Landroid/app/Activity; Landroid/nfc/NdefMessage; I)V' : ['android.permission.NFC'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtilsCache;-checkPassword-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NfcV;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/server/hdmi/HdmiCecLocalDeviceTv$1;-onDeviceDiscoveryDone-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-handleDisplayAddedLocked-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getImsRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-getSubInfoForSubscriber-(J)Landroid/telephony/SubInfoRecord;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/PacManager;-getDownloadDelay-(I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/am/Am$IntentReceiver;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/CheckBox;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyLocationChanged-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPasswordMinimumNumeric-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$2;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/format/DateUtils;-getRelativeTimeSpanString-(J J J I)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Contacts$People;-addToMyContactsGroup-(Landroid/content/ContentResolver; J)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/net/sip/SipManager;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodListLocked-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/SELinuxPolicyInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/uiautomator/core/UiAutomationShellWrapper;-disconnect-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IMms;-updateStoredMessageStatus-(Ljava/lang/String; Landroid/net/Uri; Landroid/content/ContentValues;)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-updateImsPhone-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-updateServiceLocation-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/mock/MockApplication;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCallForward-(I I I Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$Service$2;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getSupportedFeatures-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/CalendarView$LegacyCalendarViewDelegate;-onScroll-(Landroid/widget/AbsListView; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-setDisplayNumberFormat-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/ethernet/EthernetNetworkFactory$1;-run-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/telephony/TelephonyManager;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPasswordExpirationTimeout-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowSession$Stub;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/tts/TextToSpeechService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/PrintManager;-getEnabledPrintServices-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-setForcedDisplaySizeLocked-(Lcom/android/server/wm/DisplayContent; I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/RankingReconsideration;-work-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/wifi/WifiStateMachine;-setNetworkDetailedState-(Landroid/net/NetworkInfo$DetailedState;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-access$000-(Lcom/android/internal/telephony/RIL;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setCameraDisabled-(Landroid/content/ComponentName; Z I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/MenuBuilder$Callback;-onMenuModeChange-(Lcom/android/internal/view/menu/MenuBuilder;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthTokenLabel-(Ljava/lang/String; Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getMsisdnForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getMaximumFailedPasswordsForWipe-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RadialTimePickerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/GpsNavigationMessageListenerTransport;-add-(Ljava/lang/Object;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/JobSchedulerImpl;-schedule-(Landroid/app/job/JobInfo;)I' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/wm/AccessibilityController;-onWindowFocusChangedLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/text/format/DateFormat;-getDateFormat-(Landroid/content/Context;)Ljava/text/DateFormat;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setUninstallBlocked-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-isMasterVolumeMuted-(Landroid/content/ComponentName;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/tv/TvInputService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-getSupportedFeatures-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/util/IState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/UiccSmsController;-getAllMessagesFromIccEfForSubscriber-(J Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/widget/EditText;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiManager;-isAdditionalStaSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-access$100-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/widget/ScrollingTabContainerView$TabView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getSampleEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayManagerService$LocalService;-setDisplayProperties-(I Z F Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SubscriptionManager;-getDefaultSmsSubId-()J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/FilterSurfaceView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManager;-getRunningTasks-(I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView$2;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$InactiveState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ViewOverlay$OverlayViewGroup;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/GridView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/VignetteEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService$BinderService;-updateWakeLockUids-(Landroid/os/IBinder; [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NsdService$NsdStateMachine;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/AlertActivity;-onStop-()V' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Landroid/content/pm/IPackageManager;-systemReady-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setCameraDisabled-(Landroid/content/ComponentName; Z I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setMute-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/VideoView;-stopPlayback-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IApplicationToken$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/ContentResolver;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService$AudioHandler$2;-onError-(Landroid/media/MediaPlayer; I I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-reportActivityInfo-()Landroid/net/wifi/WifiActivityEnergyInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wifi/SupplicantStateTracker$InactiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Landroid/nfc/NfcAdapter;-disableForegroundDispatch-(Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Landroid/location/ILocationManager$Stub;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/hardware/IConsumerIrService;-getCarrierFrequencies-()[I' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/internal/app/PlatLogoActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/MountServiceIdler;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/trust/TrustAgentService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeech;-access$1700-(Landroid/speech/tts/TextToSpeech;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-pollStateDone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/WiredAccessoryManager;-access$200-(Lcom/android/server/WiredAccessoryManager;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/ClosedCaptionWidget$CCLineBox;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextSwitcher;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-startProcessLocked-(Ljava/lang/String; Landroid/content/pm/ApplicationInfo; Z I Ljava/lang/String; Landroid/content/ComponentName; Z Z Z)Lcom/android/server/am/ProcessRecord;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/textservice/ITextServicesManager;-getCurrentSpellCheckerSubtype-(Ljava/lang/String; Z)Landroid/view/textservice/SpellCheckerSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DayPickerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-pollState-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl;-enforceAccessPermission-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onSomePackagesChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/tv/TvInputManagerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/NfcService$TagService;-getTimeout-(I)I' : ['android.permission.NFC'],
	'Landroid/speech/tts/TextToSpeech;-setLanguage-(Ljava/util/Locale;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/InputPort;-pushFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/DownloadManager;-enqueue-(Landroid/app/DownloadManager$Request;)J' : ['android.permission.INTERNET'],
	'Landroid/accounts/CantAddAccountActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IIntentReceiver$Stub$Proxy;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneBase;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/IAudioService$Stub$Proxy;-setRingerMode-(I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getDataConnectionState-()Lcom/android/internal/telephony/PhoneConstants$DataState;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setAndBroadcastNetworkSetTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Landroid/nfc/NfcActivityManager;-enableReaderMode-(Landroid/app/Activity; Landroid/nfc/NfcAdapter$ReaderCallback; I Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService;-unrouteAids-(Ljava/lang/String;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/ISub$Stub;-getDefaultVoiceSubId-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipWakeLock;-release-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-isPatternEverChosen-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList; Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/FlipFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/OutputStreamTarget;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncActivityTooManyDeletes;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/tv/TvInputService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/content/PackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$LinkMonitoringState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothMap;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/P2pLinkManager$ConnectTask;-onPostExecute-(Ljava/lang/Object;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$FrequencyConflictState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/P2pEventManager;-onP2pHandoverNotSupported-()V' : ['android.permission.VIBRATE'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$ActedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccSmsController;-isImsSmsSupported-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-removeWindowInnerLocked-(Lcom/android/server/wm/Session; Lcom/android/server/wm/WindowState;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/VibratorService;-systemReady-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub$SettingsObserver;-<init>-(Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$17700-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/View;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-goingToSleepLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setCameraDisabled-(Landroid/content/ComponentName; Z I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/text/format/DateUtils;-formatDateRange-(Landroid/content/Context; Ljava/util/Formatter; J J I)Ljava/util/Formatter;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-onUserStateChangedLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/LocationManagerService;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-createUser-(Landroid/content/ComponentName; Ljava/lang/String;)Landroid/os/UserHandle;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/BatteryService;-processValuesLocked-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/server/BluetoothA2dpService;-onBluetoothEnable-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/app/ContextImpl;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/WindowManagerPolicy;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/PackageManagerService;-isPackageDeviceAdmin-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/QuickContactBadge;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SmsManager;-addMultimediaMessageDraft-(Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/view/accessibility/AccessibilityWindowInfo;-getChild-(I)Landroid/view/accessibility/AccessibilityWindowInfo;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/print/PrinterDiscoverySession;-<init>-(Landroid/print/IPrintManager; Landroid/content/Context; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/RecognitionService;-checkPermissions-(Landroid/speech/IRecognitionListener;)Z' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onRadioOffOrNotAvailable-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onRemoteFieldActivated-()V' : ['android.permission.VIBRATE'],
	'Landroid/accounts/AccountManager;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/dreams/DreamManagerService;-access$2000-(Lcom/android/server/dreams/DreamManagerService; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/ContextWrapper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-restatDataDir-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-clearPackagePersistentPreferredActivities-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/monkey/MonkeyNetworkMonitor;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine$SoftApStartingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/widget/TextView;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/TelephonyManager;-getImei-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/trust/TrustManagerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/sip/ISipService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/provider/ContactsContract$Groups$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/ClosedCaptionWidget$CCLineBox;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCommandInterface;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$TagService;-setTimeout-(I I)I' : ['android.permission.NFC'],
	'Landroid/bluetooth/BluetoothAdapter;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/view/IWindowManager$Stub;-getBaseDisplaySize-(I Landroid/graphics/Point;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-addTask-(I I Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$7200-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-reportSuccessfulPasswordAttempt-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/backup/LocalTransportService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcCardEmulation$Stub$Proxy;-unsetPreferredService-()Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Landroid/media/effect/effects/StraightenEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onSetInternalDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiStateMachine;-startBatchedScan-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IProcessObserver$Stub$Proxy;-onProcessDied-(I I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/print/PrintManagerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/INfcCardEmulation$Stub$Proxy;-isDefaultServiceForAid-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/server/power/PowerManagerService$LocalService;-setUserActivityTimeoutOverrideFromWindowManager-(J)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-access$8900-(Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcAdapter$Stub;-invokeBeam-()V' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/server/notification/NotificationManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-getPrintJobInfos-(I I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/service/fingerprint/FingerprintUtils;-getFingerprintIdsForUser-(Landroid/content/ContentResolver; I)[I' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-startScan-(Landroid/net/wifi/ScanSettings; Landroid/os/WorkSource;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/MediaFocusControl;-startVoiceBasedInteractions-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/ims/internal/ImsCallSession$Listener;-callSessionMerged-(Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/ImsCallProfile;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SmsUsageMonitor$SettingsObserver;-<init>-(Landroid/os/Handler; Landroid/content/Context; Ljava/util/concurrent/atomic/AtomicBoolean;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-setReaderMode-(Landroid/os/IBinder; Landroid/nfc/IAppCallback; I Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getPasswordHistoryLength-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UiccCarrierPrivilegeRules;-<init>-(Lcom/android/internal/telephony/uicc/UiccCard; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/AccessibleDateAnimator;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-isDataAllowed-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneFactory;-calculatePreferredNetworkType-(Landroid/content/Context;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/am/Am;-runKillAll-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/internal/view/RotationPolicy;-isRotationLocked-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/media/MediaBrowserService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPasswordMinimumLowerCase-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/widget/CheckedTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-wipeData-(I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/LauncherAppsService$LauncherAppsImpl$MyPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/INfcTag$Stub;-formatNdef-(I [B)I' : ['android.permission.NFC'],
	'Landroid/net/wifi/WifiManager;-isPasspointSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wm/AccessibilityController;-onWindowLayersChangedLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/UiccSmsController;-updateMessageOnIccEfForSubscriber-(J Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl$2;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GeofenceManager;-updateFences-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/SubscriptionController;-getSubInfoUsingSlotIdWithCheck-(I Z)Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-dispatchNormalMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/wm/WindowManagerService;-setAppVisibility-(Landroid/os/IBinder; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/WebVttRenderingWidget$SpanLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/CoreSettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/job/JobServiceContext;-access$300-(Lcom/android/server/job/JobServiceContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onChanged-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/nfc/P2pLinkManager$ConnectTask;-onPostExecute-(Ljava/lang/Boolean;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-addGpsStatusListener-(Landroid/location/IGpsStatusListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/sip/SipManager;-getSessionFor-(Landroid/content/Intent;)Landroid/net/sip/SipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCommandInterface;-setUiccSubscription-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService$3;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/MutableContextWrapper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/SyncRunner;-step-()I' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/notification/ConditionProviders;-access$600-(Lcom/android/server/notification/ManagedServices;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/SimpleMonthView;-drawMonthTitle-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub;-setDefaultVoiceSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/SubtitleView;-setStyle-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-lockNow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/PlatLogoActivity$1;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onCleanUpAllConnections-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-isApplicationHidden-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/ContextThemeWrapper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/DigitalClock;-initClock-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/TrafficStats;-getDataLayerSnapshotForUid-(Landroid/content/Context;)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RatingBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/FieldPort;-transfer-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-switchWaitingOrHoldingAndActive-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-setCrossProfileCallerIdDisabled-(Landroid/content/ComponentName; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-onMotionEvent-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-createAllApnList-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/AccessibilityController$DisplayMagnifier$MagnifiedViewport;-populateWindowsOnScreenLocked-(Landroid/util/SparseArray;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/TabActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/telephony/PhoneNumberUtils;-isLocalEmergencyNumberInternal-(Ljava/lang/String; Landroid/content/Context; Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/FilterGraph;-closeFilters-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-systemReady-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getIccSimChallengeResponse-(J I Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/AlarmManagerService$ResultReceiver;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/KeyboardView;-access$400-(Landroid/inputmethodservice/KeyboardView; Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-screenshotApplications-(Landroid/os/IBinder; I I I Z)Landroid/graphics/Bitmap;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/NotificationIntrusivenessExtractor$1;-run-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/wifi/WifiStateMachine$ScanModeState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setDeviceOwner-(Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/InputStreamSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/DownloadManager;-addCompletedDownload-(Ljava/lang/String; Ljava/lang/String; Z Ljava/lang/String; Ljava/lang/String; J Z Z)J' : ['android.permission.INTERNET'],
	'Lcom/android/server/INativeDaemonConnectorCallbacks;-onDaemonConnected-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/IWindowManager$Stub;-statusBarVisibilityChanged-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/dhimpl/NativeNfcTag$PresenceCheckWatchdog;-run-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setProfileName-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/AppChooserActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-stopDtmf-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/TabActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Button;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService;-setMicrophoneMute-(Z Ljava/lang/String;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/location/ILocationManager;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/view/IWindowSession$Stub$Proxy;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-removeUser-(Landroid/content/ComponentName; Landroid/os/UserHandle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-stopPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService;-setCurrentSpellCheckerLocked-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcTag$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-explicitCallTransfer-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TextServicesManagerService$TextServicesSettings;-getSelectedSpellCheckerSubtype-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPasswordMinimumLowerCase-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$5800-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/AutoFixEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/pm/PackageManagerService$8;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-handleSmsMessage-(Landroid/os/Message;)Z' : ['android.permission.RECEIVE_SMS'],
	'Landroid/net/wifi/WifiManager;-startCustomizedScan-(Landroid/net/wifi/ScanSettings;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDisconnectDcRetrying-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/media/AudioService$AudioHandler;-cleanupPlayer-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-doVibratorOff-()V' : ['android.permission.VIBRATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setWifiEnabled-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneProxy;-updateCurrentCarrierInProvider-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/ClosedCaptionWidget;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendRetrySms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-handleExitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IActivityManager;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getTrustAgentFeaturesEnabled-(Landroid/content/ComponentName; Landroid/content/ComponentName; I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/widget/PopupWindow$PopupViewContainer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/InputStreamSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/widget/TableLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ILockSettings;-setLockPassword-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/method/TextKeyListener$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onTrySetupData-(Lcom/android/internal/telephony/dataconnection/ApnContext;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/SystemBackupAgent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/P2pEventListener;-onP2pInRange-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/connectivity/PacManager$1;-run-()V' : ['android.permission.READ_PROFILE', 'android.permission.INTERNET'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/restrictions/RestrictionsManagerService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DatePicker;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/display/DisplayPowerController;-requestPowerState-(Landroid/hardware/display/DisplayManagerInternal$DisplayPowerRequest; Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/Installer;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordMinimumSymbols-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$6;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/FilterGraphEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-getAllSubInfoList-()Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/accounts/ChooseAccountActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Editor$InsertionHandleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/net/NetworkStatsService;-advisePersistThreshold-(J)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkStatsService$Stub$Proxy;-getNetworkTotalBytes-(Landroid/net/NetworkTemplate; J J)J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-removeFence-(Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-tearDownData-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/HorizontalScrollView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-addAppLocked-(Landroid/content/pm/ApplicationInfo; Z Ljava/lang/String;)Lcom/android/server/am/ProcessRecord;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/MobileDataStateTracker;-startMonitoring-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/backup/LocalTransportService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/SupplicantStateTracker;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/BackupAgentHelper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/FilterSurfaceView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderer;-createTrack-(Landroid/media/MediaFormat;)Landroid/media/SubtitleTrack;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-fetchDunApn-()Lcom/android/internal/telephony/dataconnection/ApnSetting;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-trySwitchToFirstTouchState-(F F)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-shutdownRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-access$300-(Lcom/android/internal/telephony/RIL;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getUidConfig-()Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings$Config;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/OutputStreamTarget;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/Presentation$1;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiServiceImpl;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/test/mock/MockContentResolver;-getSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/BeamShareActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IBackupAgent;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LockSettingsService;-getBoolean-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-showBootMessage-(Ljava/lang/CharSequence; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/app/IBatteryStats;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-updateNetworkRulesLocked-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStack;-activityPausedLocked-(Landroid/os/IBinder; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/TabActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-getPDPContextList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/CheckBox;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-access$1400-(Lcom/android/server/LocationManagerService; Lcom/android/server/LocationManagerService$Receiver;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onSetInternalDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ims/ImsCall$ImsCallSessionListenerProxy;-callSessionHoldFailed-(Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/DeviceHost$TagEndpoint;-startPresenceChecking-(I Lcom/android/nfc/DeviceHost$TagDisconnectedCallback;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-getIccSimChallengeResponse-(I Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/MountServiceIdler;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/job/JobServiceContext$JobServiceHandler;-handleCancelH-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub;-systemRunning-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/RadioButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ContentResolver;-getIsSyncableAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService;-interceptKeyBeforeDispatching-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)J' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-queryCLIP-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$DriverStoppedState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/nfc/NfcAdapter;-setOnNdefPushCompleteCallback-(Landroid/nfc/NfcAdapter$OnNdefPushCompleteCallback; Landroid/app/Activity; [Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-launchAssistLongPressAction-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getGlobalLong-(Ljava/lang/String; J)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/GlobalActions$GlobalActionsDialog;-dispatchTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/IContentService;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/accounts/AccountManager;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/os/SystemVibrator;-vibrate-(I Ljava/lang/String; J Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/app/IntentForwarderActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/trust/TrustManagerService;-dispatchUnlockAttempt-(Z I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/IBackupManager;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-onRefresh-(Z [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/AppChooserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$ConnectivityReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RotateFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(J F Landroid/location/Criteria; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-addTextMessageDraft-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/bluetooth/BluetoothHeadsetClient;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wifi/WifiStateMachine$DisconnectingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/content/SyncStorageEngine;-requestSync-(Landroid/accounts/Account; I I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/TelephonyRegistry;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/NetworkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordMinimumLetters-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager$CardEmulationInterface;-getAidGroupForService-(I Landroid/content/ComponentName; Ljava/lang/String;)Landroid/nfc/cardemulation/AidGroup;' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getRemoveWarning-(Landroid/content/ComponentName; Landroid/os/RemoteCallback; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$3;-onCallMerged-(Lcom/android/ims/ImsCall; Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Editor$SelectionEndHandleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/content/PackageMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-checkPassword-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-inputDispatchingTimedOut-(I Z Ljava/lang/String;)J' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/DialogTitle;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IUiAutomationConnection$Stub$Proxy;-connect-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/cardemulation/DefaultRemovedActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/IWindowManager$Stub;-screenshotApplications-(Landroid/os/IBinder; I I I Z)Landroid/graphics/Bitmap;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$2400-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/BeamShareActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.VIBRATE'],
	'Landroid/media/MediaPlayer;-create-(Landroid/content/Context; Landroid/net/Uri;)Landroid/media/MediaPlayer;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/AudioService;-restoreMasterVolume-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameFetch;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier; Landroid/net/sip/SipProfile;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/TtmlRenderingWidget;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-setWfdInfo-(Landroid/net/wifi/p2p/WifiP2pWfdInfo;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/WapPushOverSms;-isDuplicateNotification-(Landroid/content/Context; Lcom/google/android/mms/pdu/NotificationInd;)Z' : ['android.permission.READ_SMS'],
	'Lcom/android/internal/content/PackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneBase;-setPropertiesByCarrier-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-handlePinMmi-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/DhcpStateMachine$WaitBeforeRenewalState;-exit-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/ChooseAccountActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/ViewStub;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-requestShutdown-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-handleApplicationCrash-(Landroid/os/IBinder; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/BatteryService;-updateBatteryWarningLevelLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayManagerService;-windowManagerAndInputReady-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/commands/settings/SettingsCmd;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ISub;-getSubInfoUsingSlotId-(I)Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-getAdnRecordsInEfForSubscriber-(J I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setAutoTimeRequired-(Landroid/content/ComponentName; I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-cancelActiveSync-(Lcom/android/server/content/SyncStorageEngine$EndPoint; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getIsimChallengeResponse-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-installNewPackageLI-(Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/pm/PackageManagerService$MeasureParams;-startCopy-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Activity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-enforceCanManageCaCerts-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech$Connection$SetupConnectionAsyncTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/ConfirmConnectActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/MediaScannerClient;-scanFile-(Ljava/lang/String; J J Z Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccProvider;-delete-(Landroid/net/Uri; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/app/admin/IDevicePolicyManager;-setActiveAdmin-(Landroid/content/ComponentName; Z I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/pm/IPackageManager;-performBootDexOpt-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Telephony$Sms;-addMessageToUri-(Landroid/content/ContentResolver; Landroid/net/Uri; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long; Z Z)Landroid/net/Uri;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/RuntimeInit$UncaughtHandler;-uncaughtException-(Ljava/lang/Thread; Ljava/lang/Throwable;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/DropBoxManager;-addData-(Ljava/lang/String; [B I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/PlatLogoActivity$1;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setRecommendedGlobalProxy-(Landroid/content/ComponentName; Landroid/net/ProxyInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/wifi/WifiScanningServiceImpl$WifiChangeStateMachine;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-makeNetworkCapabilities-()Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-cancel-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setUserRestriction-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncStorageEngine;-clearAndReadState-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/appwidget/AppWidgetServiceImpl$SecurityPolicy;-isProviderInCallerOrInProfileAndWhitelListed-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/display/DisplayManagerInternal;-setDisplayProperties-(I Z F Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/media/MediaSessionService$SessionManagerImpl;-startVoiceInput-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub;-getInitialDisplayDensity-(I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/NetworkScoreManager;-updateScores-([Landroid/net/ScoredNetwork;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioManager;-playSoundEffect-(I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/AccessibilityWindowInfo;-getParent-()Landroid/view/accessibility/AccessibilityWindowInfo;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/ims/ImsCall$Listener;-onCallResumed-(Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/trust/TrustManagerService$3;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget$CueLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl$2;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker$2;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/am/BatteryStatsService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendActionDownAndUp-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/ReceiverRestrictedContext;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ims/internal/ImsCallSession$IImsCallSessionListenerProxy;-callSessionStartFailed-(Lcom/android/ims/internal/IImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiNetworkScoreCache;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordMinimumLowerCase-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-shutdownRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController;-initializeAndRegisterForSettingsChange-(Landroid/os/Looper;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/NfcService$TagService;-resetTimeouts-()V' : ['android.permission.NFC'],
	'Lcom/android/nfc/handover/HandoverTransfer;-updateFileProgress-(F)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/telecom/InCallService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/dreams/DreamManagerService;-access$600-(Lcom/android/server/dreams/DreamManagerService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleEctIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SubscriptionController;-clearDefaultsForInactiveSubIds-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getLine1AlphaTagForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/ISms$Stub;-getPreferredSmsSubscription-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CdmaBroadcastRangeManager;-tryAddRanges-(I I Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/ManagedServices;-onPackagesChanged-(Z [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/notification/ConditionProviderService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/IAudioService$Stub$Proxy;-disableSafeMediaVolume-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ILockSettings$Stub$Proxy;-setLockPassword-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/MmsServiceBroker;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/IWallpaperManager;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/TimePickerSpinnerDelegate;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub;-cancelRequest-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/accounts/ChooseAccountActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/NsdService$NsdStateMachine$EnabledState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifyDataConnectionFailedForSubscriber-(J Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/VideoView$2;-onPrepared-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/WifiDisplayController;-updateSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/Vibrator;-vibrate-(J)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getIsimChallengeResponse-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SubscriptionManager;-getAllSubInfoCount-()I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-setInstallLocation-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/SettingInjectorService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/google/android/mms/pdu/PduPersister;-loadRecipients-(I Ljava/util/HashSet; Ljava/util/HashMap; Z)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/app/PlatLogoActivity$2$1;-onLongClick-(Landroid/view/View;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-updateMessageOnIccEf-(Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/wifi/WifiServiceImpl;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS', 'android.permission.SEND_SMS'],
	'Lcom/android/server/LockSettingsService;-systemReady-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-crashApplication-(Lcom/android/server/am/ProcessRecord; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getUidTagConfig-()Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings$Config;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity$1;-onClick-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/content/MutableContextWrapper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiServiceImpl;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPermittedInputMethodsForCurrentUser-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$WpsRunningState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-iccIOForApp-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms;-updateMessageOnIccEfForSubscriber-(J Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setProfileEnabled-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipManager;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Landroid/inputmethodservice/ExtractButton;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-performBootTimeout-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/AudioService$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setTimeFromNITZString-(Ljava/lang/String; J)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-setCellBroadcastActivation-(Z)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/ShutdownThread;-beginShutdownSequence-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/backup/BackupAgentHelper;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneProxy;-deleteAndCreatePhone-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getActivePasswordQuality-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/accessibilityservice/IAccessibilityServiceConnection$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-iccTransmitApduBasicChannel-(I I I I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag$Stub;-getTimeout-(I)I' : ['android.permission.NFC'],
	'Lcom/android/server/wm/WindowManagerService;-resizeStack-(I Landroid/graphics/Rect;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$GroupCreatingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/DateTimeView$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-reportSmsMemoryStatus-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/CaptioningManager;-notifyFontScaleChanged-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringLogger;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NsdService;-create-(Landroid/content/Context;)Lcom/android/server/NsdService;' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/TouchExplorer$DoubleTapDetector;-onMotionEvent-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager$3;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/ClosedCaptionWidget$CCLineBox;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-dispatchPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/tech/MifareClassic;-transfer-(I)V' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getCallWaiting-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wallpaper/WallpaperManagerService;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-checkAddPermission-(Landroid/view/WindowManager$LayoutParams; [I)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-initSstIcc-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-sendDelayedMotionEvents-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-killApplicationWithAppId-(Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-updateAudioSourceFunction-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager;-switchUser-(Landroid/content/ComponentName; Landroid/os/UserHandle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/BackgroundDexOptService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/FrameBranch;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifyCellLocation-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/PhoneNumberUtils;-isEmergencyNumberInternal-(Ljava/lang/String; Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$System;-getLong-(Landroid/content/ContentResolver; Ljava/lang/String;)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/BroadcastQueue;-setBroadcastTimeoutLocked-(J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-dispatchUnhandledKey-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/effect/SizeChangeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ContextThemeWrapper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ViewFlipper;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-resumeGeofence-(I I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-createDisplayContentLocked-(Landroid/view/Display;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-copyMessageToIccEf-(Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/accounts/IAccountManager;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/nfc/tech/IsoDep;-transceive-([B)[B' : ['android.permission.NFC'],
	'Landroid/app/IWallpaperManager$Stub$Proxy;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-loadProvidersLocked-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/sip/SipService$SipKeepAliveProcessCallback;-onResponse-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-doVibratorOn-(J I I)V' : ['android.permission.VIBRATE'],
	'Landroid/widget/RadioButton;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IIntentReceiver$Stub;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/ILocationManager$Stub;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-disableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/widget/SpellChecker;-<init>-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-displayReady-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/power/PowerManagerService;-access$2100-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-goingToSleep-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$19100-(Lcom/android/server/wifi/WifiStateMachine; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/view/accessibility/CaptioningManager;-notifyUserStyleChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-removeUpdatesLocked-(Lcom/android/server/LocationManagerService$Receiver;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/InputMethodService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-removeCrossProfileWidgetProvider-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setScreenCaptureDisabled-(Landroid/content/ComponentName; I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-getBaseDisplaySize-(I Landroid/graphics/Point;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wifi/WifiStateMachine$UntetheringState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPasswordMinimumSymbols-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPermittedInputMethods-(Landroid/content/ComponentName; Ljava/util/List;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/os/IHardwareService;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/nfc/NfcService;-onRemoteFieldActivated-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/app/ResolverActivity$1;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-handleSrvccStateChanged-([I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag;-close-(I)I' : ['android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IActivityManager;-bindBackupAgent-(Landroid/content/pm/ApplicationInfo; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/input/InputManagerService;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-getBaseDisplayDensity-(I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-onDisconnect-(I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtilsCache;-getBoolean-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextClock;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/WindowManagerPolicy$WindowManagerFuncs;-reevaluateStatusBarVisibility-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ReceiverRestrictedContext;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-unregisterForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-getEnabledInputMethodSubtypeList-(Ljava/lang/String; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer$2;-run-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ISms$Stub;-getAllMessagesFromIccEf-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/CallManager;-acceptCall-(Lcom/android/internal/telephony/Call;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/dreams/DreamManagerInternal;-startDream-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/telephony/gsm/SmsMessage;-calculateLength-(Ljava/lang/CharSequence; Z)[I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setCallWaiting-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayPowerController;-sendUpdatePowerStateLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/content/ContentService;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/job/JobServiceContext;-executeRunnableJob-(Lcom/android/server/job/controllers/JobStatus;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/MmiCode;-cancel-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/trust/TrustManagerService;-maybeEnableFactoryTrustAgents-(Lcom/android/internal/widget/LockPatternUtils; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-getAccountsAsUser-(I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-pollCallsWhenSafe-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/VideoView;-onKeyDown-(I Landroid/view/KeyEvent;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView$1;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$DisconnectedState;-enter-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-access$100-(Landroid/hardware/location/GeofenceHardwareImpl;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RemoteViewsService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPasswordMinimumLowerCase-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccSmsController;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService;-applyRouting-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setApplicationHidden-(Landroid/content/ComponentName; Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$TetheredState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ActionBarContainer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/commands/svc/WifiCommand;-run-([Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/Editor$HandleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/service/dreams/DreamService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setDataRoamingEnabled-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothHeadset;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/bluetooth/BluetoothAdapter;-disable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/PhoneBase;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$System;-getLongForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/MountService;-access$2000-(Lcom/android/server/MountService;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/mock/MockContentResolver;-getCurrentSync-()Landroid/content/SyncInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-onUserContextChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ImageSwitcher;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SubscriptionManager;-getAllSubInfoList-()Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-dispatchMediaKeyRepeatWithWakeLock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/connectivity/Tethering$StateReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncStorageEngine;-init-(Landroid/content/Context;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/text/StringSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Instrumentation;-finish-(I Landroid/os/Bundle;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/ContextThemeWrapper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/NfcService;-commitRouting-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-shutdownLocked-(I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setRadioPower-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getCurrentFailedPasswordAttempts-(I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setPropertiesByCarrier-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcCardEmulation$Stub;-isDefaultServiceForCategory-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/media/IAudioService$Stub;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/net/INetworkPolicyManager$Stub;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-updateProviderListenersLocked-(Ljava/lang/String; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/ShutdownActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setTimeFromNITZString-(Ljava/lang/String; J)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/policy/impl/ImmersiveModeConfirmation$ClingWindowView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-editProperties-(Ljava/lang/String; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/inputmethodservice/ExtractButton;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-queryCallWaiting-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-setTouchEventsEnabled-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/LocationManager;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl$2;-onSomePackagesChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ZenModeHelper;-updateZenMode-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setUiccSubscription-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$4;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/ActionMenuView$MenuBuilderCallback;-onMenuModeChange-(Lcom/android/internal/view/menu/MenuBuilder;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-addCrossProfileIntentFilter-(Landroid/content/ComponentName; Landroid/content/IntentFilter; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/wifi/WifiSettingsStore;-handleAirplaneModeToggled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ChooserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/INetworkPolicyManager;-isNetworkMetered-(Landroid/net/NetworkState;)Z' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/app/DisableCarModeActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/SearchView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$ActingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-isAirplaneModeOn-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager$9;-onStatusChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-setForegroundDispatch-(Landroid/app/PendingIntent; [Landroid/content/IntentFilter; Landroid/nfc/TechListParcel;)V' : ['android.permission.NFC'],
	'Landroid/widget/ToggleButton;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-<init>-(Lcom/android/internal/telephony/gsm/GSMPhone;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/content/ContentResolver;-setMasterSyncAutomaticallyAsUser-(Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/media/SubtitleController;-doSelectTrack-(Landroid/media/SubtitleTrack;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wallpaper/WallpaperManagerService$MyPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneMmiCode;-onQueryCfComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController$NoLockHeldState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Lcom/android/server/voiceinteraction/SoundTriggerHelper;-onRecognitionFailureLocked-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/NfcService$TagService;-reconnect-(I)I' : ['android.permission.NFC'],
	'Landroid/net/wifi/IWifiManager;-stopBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/updates/SmsShortCodesInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-updateCarrierMccMncConfiguration-(Ljava/lang/String; Ljava/lang/String; Landroid/content/Context;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/media/MediaPlayer;-deselectTrack-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-access$000-(Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextViewWithCircularIndicator;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-validatePrinters-(Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onCleanUpConnection-(Z I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ManagedServices;-rebindServices-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/IWifiP2pManager$Stub$Proxy;-getP2pStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-processLockedState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-isLockTaskPermitted-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/MccTable;-setSystemLocale-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-disableDataCallInEmergencyCall-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/Connection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DigitalClock;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-sendData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/base/NullFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/media/MediaBrowserService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/handover/ConfirmConnectActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-getAutoTime-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/Context;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisToFileOutputStreamSpeechItemV1;-<init>-(Landroid/speech/tts/TextToSpeechService; Ljava/lang/Object; I I Landroid/os/Bundle; Ljava/lang/String; Ljava/lang/CharSequence; Ljava/io/FileOutputStream;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-getWifiApState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/EditText;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AutoCompleteTextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-createPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStack;-resetTaskIfNeededLocked-(Lcom/android/server/am/ActivityRecord; Lcom/android/server/am/ActivityRecord;)Lcom/android/server/am/ActivityRecord;' : ['android.permission.SEND_SMS', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/ims/ImsCall$ImsCallSessionListenerProxy;-callSessionStartFailed-(Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-updateConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/tech/MifareUltralight;-setTimeout-(I)V' : ['android.permission.NFC'],
	'Landroid/view/IWindowManager$Stub;-inputMethodClientHasFocus-(Lcom/android/internal/view/IInputMethodClient;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine$InitialState;-enter-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/backup/BackupManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/widget/TextClock;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/sip/ISipService;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/notification/NotificationManagerService;-access$3300-(Lcom/android/server/notification/NotificationManagerService; Lcom/android/server/notification/NotificationRecord;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothManager;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/os/IVibratorService$Stub;-vibrate-(I Ljava/lang/String; J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateStart;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcRetryingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService$3;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/AccessibilityController;-onSomeWindowResizedOrMovedLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager;-getAggressiveHandover-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/nfc/INfcTag;-transceive-(I [B Z)Landroid/nfc/TransceiveResult;' : ['android.permission.NFC'],
	'Landroid/media/AsyncPlayer;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-updateProgramWithTarget-(I Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IBackupAgent;-doRestoreFinished-(I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/Application;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowManager;-isRotationFrozen-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$P2pNotSupportedState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/SubtitleController;-doShow-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$ErrorState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendSMS-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService$Stub$Proxy;-getCurrentSyncsAsUser-(I)Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$9800-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/BeamShareActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/NfcRootActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/dreams/DreamManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayPowerController;-access$1000-(Lcom/android/server/display/DisplayPowerController; J Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/ListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Landroid/provider/Settings$System;-getInt-(Landroid/content/ContentResolver; Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/SmsBroadcastUndelivered;-scanRawTable-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/power/PowerManagerService;-handleSettingsChangedLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/provider/Browser;-deleteHistoryTimeFrame-(Landroid/content/ContentResolver; J J)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-handleRadioNotAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService$Stub$Proxy;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/nfc/NfcAdapter$1;-onPaused-(Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/accounts/IAccountManager;-removeAccountAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/speech/IRecognitionService;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/ProgramPort;-pushFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccSmsController;-copyMessageToIccEfForSubscriber-(J Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/SubscriptionController;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-addMultimediaMessageDraft-(Ljava/lang/String; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/RIL;-acceptCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-sendIntentEnter-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordMinimumUpperCase-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cat/IconLoader;-readId-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-syncClirSetting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-clearWakeLock-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Global;-getInt-(Landroid/content/ContentResolver; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/app/StatusBarManager;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-shutdownRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-explicitCallTransfer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-sendAccessibilityEventForUnicodeCharacter-(I I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-supplyPin2-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/MultiAutoCompleteTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-handleInjectSms-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-getCellLocationId-()I' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/FieldPort;-setFieldFrame-(Landroid/filterfw/core/Frame; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IVibratorService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/server/InputMethodManagerService;-updateKeyboardFromSettingsLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-getDeviceIdentity-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub$Proxy;-getPreferredOptionsPanelGravity-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateCmdParamsReady;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCdmaSubscriptionSource-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/appops/AppOpsCommand;-main-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-<init>-(Landroid/content/Context; I I Ljava/lang/Integer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$OngoingGroupRemovalState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/backup/SystemBackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/VideoView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/PrinterDiscoverySession;-validatePrinters-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DialerFilter;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/INfcCardEmulation$Stub$Proxy;-setDefaultForNextTap-(I Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-handleRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ethernet/EthernetServiceImpl;-enforceAccessPermission-()V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/EditText;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-syncDeviceCapabilitiesLocked-(Lcom/android/server/devicepolicy/DevicePolicyManagerService$DevicePolicyData;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/Filter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/format/DateUtils;-getRelativeTimeSpanString-(J J J)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncStorageEngine;-setSyncAutomatically-(Landroid/accounts/Account; I Ljava/lang/String; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/nfc/cardemulation/HostApduService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/Presentation$1;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub$1;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-onLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Service;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-handleSimRefresh-(Lcom/android/internal/telephony/uicc/IccRefreshResponse;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/ForegroundUtils;-onForegroundActivitiesChanged-(I I Z)V' : ['android.permission.VIBRATE'],
	'Landroid/widget/StackView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/lights/LightsService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/server/WiredAccessoryManager;-onSystemReady-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/preference/PreferenceActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/AdnRecordLoader;-loadAllFromEF-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-setVoiceCallForwardingFlag-(I Z Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-sendRetrySms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/content/IContentService$Stub$Proxy;-syncAsUser-(Landroid/content/SyncRequest; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/speech/tts/TextToSpeechService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/net/NetworkStatsService;-getNetworkTotalBytes-(Landroid/net/NetworkTemplate; J J)J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-bindBackupAgent-(Landroid/content/pm/ApplicationInfo; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getCrossProfileWidgetProviders-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/gsm/SmsManager;-copyMessageToSim-([B [B I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-setFacilityLock-(Ljava/lang/String; Z Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-downloadMessage-(J Ljava/lang/String; Ljava/lang/String; Landroid/net/Uri; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.RECEIVE_MMS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-finishWindowsDrawn-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/Instrumentation;-getUiAutomation-()Landroid/app/UiAutomation;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/TextClock;-setFormat24Hour-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ScrollingTabContainerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/display/DisplayManagerService;-requestGlobalDisplayStateInternal-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/MediaController;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-getShortcutInputMethodsAndSubtypes-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/NotificationManagerService$SettingsObserver;-observe-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ValidateNotificationPeople;-resolvePhoneContact-(Landroid/content/Context; Ljava/lang/String;)Lcom/android/server/notification/ValidateNotificationPeople$LookupResult;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/net/NetworkStatsService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-removeActiveAdmin-(Landroid/content/ComponentName; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-access$3900-(Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/statusbar/StatusBarManagerService;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/app/ContextImpl;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-getNetworkCapabilities-(Ljava/lang/String;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/notification/NotificationListenerService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/RttService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/trust/TrustManagerService$3;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getDataConnectionState-(Ljava/lang/String;)Lcom/android/internal/telephony/PhoneConstants$DataState;' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcTag;-resetTimeouts-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ApplicationPackageManager;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-makeInboundSmsHandler-(Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;)Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/widget/SubtitleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActiveServices;-realStartServiceLocked-(Lcom/android/server/am/ServiceRecord; Lcom/android/server/am/ProcessRecord; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/AudioService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/WakeGestureListener;-onWakeUp-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/view/inputmethod/InputMethodManager;-getCurrentInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager;-getDefaultServiceForCategory-(I Ljava/lang/String; Z)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-getLastInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-dial-(Ljava/lang/String; I I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-getRemoveWarning-(Landroid/content/ComponentName; Landroid/os/RemoteCallback; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$Global;-getLong-(Landroid/content/ContentResolver; Ljava/lang/String;)J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/base/FrameBranch;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/wifi/WifiStateMachine$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/tech/MifareUltralight;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/server/accounts/AccountManagerService;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcDisconnectingState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/DayPickerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ActionBarView$HomeView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService$4;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-getIccSimChallengeResponse-(I Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-getScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wm/AccessibilityController;-setWindowsForAccessibilityCallback-(Landroid/view/WindowManagerInternal$WindowsForAccessibilityCallback;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/handover/ConfirmConnectActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-getAllMessagesFromIccEf-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/app/admin/IDevicePolicyManager;-setPasswordExpirationTimeout-(Landroid/content/ComponentName; J I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/ConfirmConnectActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/test/mock/MockContentResolver;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/CropFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DropBoxManagerService$1$1;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendUpForInjectedDownPointers-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifyDataConnectionFailed-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/textservice/TextServicesManager;-newSpellCheckerSession-(Landroid/os/Bundle; Ljava/util/Locale; Landroid/view/textservice/SpellCheckerSession$SpellCheckerSessionListener; Z)Landroid/view/textservice/SpellCheckerSession;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/content/IContentService$Stub$Proxy;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/nfc/tech/NdefFormatable;-formatReadOnly-(Landroid/nfc/NdefMessage;)V' : ['android.permission.NFC'],
	'Landroid/net/wifi/IWifiManager$Stub;-getAggressiveHandover-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/preference/SeekBarVolumizer;-<init>-(Landroid/content/Context; I Landroid/net/Uri; Landroid/preference/SeekBarVolumizer$Callback;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-finishKeyguardDrawn-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/dreams/DreamManagerService$6;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getIccSerialNumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendMotionEvent-(Landroid/view/MotionEvent; I I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/AbsSpinner;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/ContextWrapper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-access$800-(Lcom/android/internal/policy/impl/PhoneWindowManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IMms;-deleteStoredMessage-(Ljava/lang/String; Landroid/net/Uri;)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-screenTurningOn-(Landroid/view/WindowManagerPolicy$ScreenOnListener;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService;-getIsSyncableAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/service/voice/VoiceInteractionService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkStatsService;-internalGetSummaryForNetwork-(Landroid/net/NetworkTemplate; J J)Landroid/net/NetworkStats;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-changeIccLockPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/ims/ImsCall;-notifyConferenceStateUpdated-(Lcom/android/ims/ImsConferenceState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/pm/IPackageManager$Stub;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/server/backup/FullBackupJob;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayBlanker;-requestDisplayState-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/backup/BackupManagerService$PerformFullTransportBackupTask$SinglePackageBackupRunner;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService$28;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/CantAddAccountActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/LocationManager;-addGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/Ringtone;-getTitle-(Landroid/content/Context; Landroid/net/Uri; Z)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/view/WindowManagerPolicy;-screenTurningOn-(Landroid/view/WindowManagerPolicy$ScreenOnListener;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCall;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareUltralight;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/RIL;-getIMEISV-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPasswordMinimumLetters-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-getVoiceRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContentResolver;-requestSyncAsUser-(Landroid/accounts/Account; Ljava/lang/String; I Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/widget/DateTimeView;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/ContentResolver;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accessibilityservice/AccessibilityService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/RenamingDelegatingContext;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/AlarmManagerService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-access$5200-(Lcom/android/server/power/PowerManagerService; I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/InputMethodService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/Session;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-hangupWaitingOrBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$SettingsObserver;-observe-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-getNetworkQuotaInfo-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/app/NativeActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/SearchView$SearchAutoComplete;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-access$900-(Lcom/android/server/content/SyncManager; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/Button;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-isDataPossible-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager;-getInitialDisplaySize-(I Landroid/graphics/Point;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/preference/PreferenceActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-doRecovery-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer;-prepare-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/ViewGroup;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getCrossProfileWidgetProviders-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-loadMediaPackages-(Landroid/util/ArrayMap; [I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState;-access$1200-(Lcom/android/internal/telephony/dataconnection/DcSwitchState; Ljava/lang/String;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-onActionTapAndHold-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer;-scanInternalSubtitleTracks-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-updateProgramWithTarget-(I Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaScanner$MyMediaScannerClient;-scanFile-(Ljava/lang/String; J J Z Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RemoteViewsService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getUidTagPersistBytes-(J)J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-toStringSimple-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-access$000-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActiveServices;-cleanUpRemovedTaskLocked-(Lcom/android/server/am/TaskRecord; Landroid/content/ComponentName; Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/text/StringLogger;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncStorageEngine;-setIsSyncable-(Landroid/accounts/Account; I Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-onCreate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-enableScreenAfterBoot-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SmsManager;-sendMultipartTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/SmsApplication;-setDefaultApplicationInternal-(Ljava/lang/String; Landroid/content/Context; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-setAppCallback-(Landroid/nfc/IAppCallback;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/ISms;-getAllMessagesFromIccEfForSubscriber-(J Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/InputMethodManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub;-clearDefaultsForInactiveSubIds-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/hdmi/HdmiControlService;-onWakeUp-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms;-disableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker$ApnChangeObserver;-onChange-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/TelephonyRegistry;-listenForSubscriber-(J Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-dispatchMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/notification/ConditionProviders;-<init>-(Landroid/content/Context; Landroid/os/Handler; Lcom/android/server/notification/ManagedServices$UserProfiles; Lcom/android/server/notification/ZenModeHelper;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-getNetworkCapabilities-(Ljava/lang/String;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-disableCellBroadcastRangeForSubscriber-(J I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/base/InputStreamSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-scanAvailableAsecs-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IMediaHTTPConnection$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.INTERNET'],
	'Lcom/android/server/wm/WindowManagerService;-makeWindowFreezingScreenIfNeededLocked-(Lcom/android/server/wm/WindowState;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub;-setCurrentSpellChecker-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/accessibility/TouchExplorer;-handleMotionEventGestureDetecting-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/base/FrameStore;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/sip/SipWakeLock;-acquire-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-getPersistedDeviceName-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Telephony$Sms$Inbox;-addMessage-(J Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long; Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getAccountTypesWithManagementDisabledAsUser-(I)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ShutdownActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/accounts/AccountManagerService;-checkAuthenticateAccountsPermission-(Landroid/accounts/Account;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/accounts/AccountAuthenticatorActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-getPreferredOptionsPanelGravity-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-hangup-(Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/media/Media;-main-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/app/AlertController$RecycleListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/power/PowerManagerService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncStorageEngine;-<init>-(Landroid/content/Context; Ljava/io/File;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/effect/effects/AutoFixEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-updateImsPhone-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/am/CompatModePackages;-setFrontActivityScreenCompatModeLocked-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl$SipKeepAlive;-onError-(Landroid/net/sip/ISipSession; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiNotificationController;-<init>-(Landroid/content/Context; Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-updateResizingWindows-(Lcom/android/server/wm/WindowState;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL$1;-onDisplayChanged-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ImageButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/AppChooserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService;-onConnectivityChanged-(Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/DragState;-unregister-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-startPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-operationComplete-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/base/InputStreamSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowState;-hideLw-(Z Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/TabActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-loadSettingsLocked-(Lcom/android/server/devicepolicy/DevicePolicyManagerService$DevicePolicyData; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-importMultimediaMessage-(Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; J Z Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/widget/LockPatternUtils;-saveLockPattern-(Ljava/util/List; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/HandoverService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub$Proxy;-reportLocation-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-isApnTypeAvailable-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioManager;-setRingerMode-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$7;-doWork-()V' : ['android.permission.USE_CREDENTIALS'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/LocalePicker;-onActivityCreated-(Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setMaximumFailedPasswordsForWipe-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-processUnsolicited-(Landroid/os/Parcel;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager$CardEmulationInterface;-setPreferredService-(Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/server/wifi/WifiServiceImpl;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/BluetoothManagerService;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager$CardEmulationInterface;-setDefaultServiceForCategory-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getLastSubtypeForInputMethodLocked-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$IdleState;-exit-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/SearchView$SearchAutoComplete;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/DhcpStateMachine$DefaultState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/SubtitleController$Renderer;-createTrack-(Landroid/media/MediaFormat;)Landroid/media/SubtitleTrack;' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/content/ContentResolver;-setSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneProxy;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getXtPersistBytes-(J)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/UiModeManagerService$4;-onTwilightStateChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/BeamShareActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/FragmentBreadCrumbs;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/google/android/mms/pdu/PduPersister;-load-(Landroid/net/Uri;)Lcom/google/android/mms/pdu/GenericPdu;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/server/backup/BackupManagerService$Lifecycle;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-addAccountAsUser-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle; I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/telecom/ConnectionService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService;-switchUserLocked-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/view/WindowManagerPolicy$WindowManagerFuncs;-shutdown-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/MmsServiceBroker;-access$800-()Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/content/IContentService;-getMasterSyncAutomaticallyAsUser-(I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/MediaPlayer;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/ToUpperCase;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcDisconnectionErrorCreatingConnection;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/backup/PackageManagerBackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getImei-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/webkit/WebViewUpdateService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-getUnknownSourcesSettings-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayAdapter$1;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/hardware/location/GeofenceHardware;-unregisterForMonitorStateChangeCallback-(I Landroid/hardware/location/GeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/Phone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DigitalClock;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-setDisplayNumberFormat-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActivityManagerService$Lifecycle;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getIccSimChallengeResponse-(J I Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService$Stub;-syncAsUser-(Landroid/content/SyncRequest; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$4;-onCallMerged-(Lcom/android/ims/ImsCall; Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountAuthenticatorActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Switch;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/dreams/DreamManagerService;-access$1700-(Lcom/android/server/dreams/DreamManagerService; Landroid/os/IBinder; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$SipSessionGroupExt;-openToReceiveCalls-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcAdapter;-disableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/NetworkTimeUpdateService;-onPollNetworkTime-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/NetInitiatedActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/PackageManagerService;-access$1100-(Lcom/android/server/pm/PackageManagerService;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/SizeAdaptiveLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneFactory;-getSMSSubscription-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/tv/TvInputManagerService$1;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/ConfigUpdateInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPasswordExpiration-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-wakingUp-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofenceTransition-(I I D D D F F F J I J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-addCrossProfileWidgetProvider-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BatteryService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readEnabledAccessibilityServicesLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-changeIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$IdleState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/LockPatternUtils;-isLockPasswordEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub;-setCurrentSpellCheckerSubtype-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getDevConfig-()Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings$Config;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getCrossProfileCallerIdDisabledForUser-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/DisableCarModeActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-hangup-(Lcom/android/internal/telephony/imsphone/ImsPhoneCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.RECEIVE_SMS'],
	'Landroid/app/admin/IDevicePolicyManager;-setPasswordMinimumLength-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-updateServiceLocation-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-addTestProvider-(Ljava/lang/String; Z Z Z Z Z Z Z I I)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-getDataRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-isDeviceProvisioned-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-killApplication-(Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/display/DisplayPowerController$ScreenOnUnblocker;-onScreenOn-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/print/IPrintManager$Stub$Proxy;-startPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$3;-onCallStartFailed-(Lcom/android/ims/ImsCall; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService$VolumeStreamState;-readSettings-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/INotificationManager;-setZenModeConfig-(Landroid/service/notification/ZenModeConfig;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/MifareUltralight;-writePage-(I [B)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-updateCarrierMccMncConfiguration-(Ljava/lang/String; Ljava/lang/String; Landroid/content/Context;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-access$200-(Lcom/android/server/accessibility/ScreenMagnifier;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkPolicyManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/widget/PreferenceImageView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/IActivityManager;-killUid-(I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-sendP2pStateChangedBroadcast-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/cardemulation/AidRoutingManager;-clearNfcRoutingTableLocked-()V' : ['android.permission.VIBRATE'],
	'Landroid/location/LocationManager;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setActiveAdmin-(Landroid/content/ComponentName; Z I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-isApplicationHidden-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-handleOnMagnifiedBoundsChanged-(Landroid/graphics/Region;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accounts/AccountManagerService$GetAccountsByTypeAndFeatureSession;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/ILockSettings;-checkPassword-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager;-addGpsStatusListener-(Landroid/location/IGpsStatusListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/IMms;-downloadMessage-(J Ljava/lang/String; Ljava/lang/String; Landroid/net/Uri; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.RECEIVE_MMS'],
	'Lcom/android/nfc/TechListChooserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/NsdService$NsdStateMachine$DefaultState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/WeightedLinearLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/PendingIntent$FinishedDispatcher;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/mock/MockContentResolver;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/wm/AccessibilityController;-onRotationChangedLocked-(Lcom/android/server/wm/DisplayContent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-performLayoutAndPlaceSurfacesLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneFactory;-isPromptEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/Filter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$1;-onDeleteGroup-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-registerContentObservers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneBase;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RadioButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/google/android/mms/util/PduCache;-put-(Landroid/net/Uri; Lcom/google/android/mms/util/PduCacheEntry;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/test/mock/MockContentResolver;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/service/dreams/Sandman;-isScreenSaverActivatedOnDock-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getChannelList-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-switchHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-cancelPostDial-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$SupplicantStoppingState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/textservice/SpellCheckerService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleSrvccStateChanged-([I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/CrossProcessEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation$Stub;-setDefaultForNextTap-(I Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/view/WindowManagerInternal;-waitForAllWindowsDrawn-(Ljava/lang/Runnable; J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getAppWidgets-(I)[I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-updateMessageOnIccEfForSubscriber-(J Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/widget/VideoView;-resume-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl;-access$1600-(Lcom/android/server/sip/SipSessionGroup$SipSessionImpl; Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wallpaper/WallpaperManagerService;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/server/wm/WindowManagerService;-startAppFreezingScreenLocked-(Lcom/android/server/wm/AppWindowToken;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$P2pEnabledState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/ISyncStatusObserver$Stub$Proxy;-onStatusChanged-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/LocationManagerService$Receiver;-<init>-(Lcom/android/server/LocationManagerService; Landroid/location/ILocationListener; Landroid/app/PendingIntent; I I Ljava/lang/String; Landroid/os/WorkSource; Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/CommandsInterface;-getLastCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifyRfFieldDeactivated-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-getNetworkQuotaInfoUnchecked-(Landroid/net/NetworkState;)Landroid/net/NetworkQuotaInfo;' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/nfc/handover/HandoverService;-access$000-(Lcom/android/nfc/handover/HandoverService; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/Call;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/UserManager;-isUserSwitcherEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-removeGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/DialogTitle;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPasswordMinimumNonLetter-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/NetInitiatedActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-changeIccFdnPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IBackupManager$Stub$Proxy;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsUsageMonitor; Lcom/android/internal/telephony/ImsSMSDispatcher;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/updates/CertPinInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BluetoothManagerService;-access$1000-(Lcom/android/server/BluetoothManagerService;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getCrossProfileWidgetProviders-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-access$200-(Lcom/android/server/accounts/AccountManagerService; Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/power/ShutdownThread;-shutdown-(Landroid/content/Context; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowAnimator$1;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-handleRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ExpandableListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LinearLayoutWithDefaultTouchRecepient;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$AmsTask;-doWork-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/ISms;-disableCellBroadcastRange-(I I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-handleVoicemailTeleservice-(Lcom/android/internal/telephony/cdma/SmsMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$7;-onSwipeFromBottom-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-restartDataStallAlarm-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Chronometer;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$GroupNegotiationState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/monkey/MonkeyEventSource;-getNextEvent-()Lcom/android/commands/monkey/MonkeyEvent;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IccCard;-setIccLockEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-<init>-(Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/gsm/GsmCallTracker; Lcom/android/internal/telephony/gsm/GsmCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/RingtoneManager;-getRingtone-(Landroid/content/Context; Landroid/net/Uri;)Landroid/media/Ringtone;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/os/IPowerManager;-acquireWakeLockWithUid-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-queueNextSignalStrengthPoll-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/IccCard;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/job/JobService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/content/SyncManager$8;-onServiceChanged-(Ljava/lang/Object; I Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/wifi/RttService$RttServiceImpl$RttStateMachine$EnabledState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/text/format/DateFormat;-is24HourFormat-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-getAccountsByTypeAsUser-(Ljava/lang/String; Landroid/os/UserHandle;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/AccountManager$1;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getChannelList-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/content/MutableContextWrapper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/MultiAutoCompleteTextView;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IActivityManager;-handleApplicationWtf-(Landroid/os/IBinder; Ljava/lang/String; Z Landroid/app/ApplicationErrorReport$CrashInfo;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayPowerController;-setScreenState-(I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/preference/PreferenceActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-startPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onDataSetupComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-access$1100-(Lcom/android/server/InputMethodManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/IntentForwarderActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/tech/NfcBarcode;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/WakeLockStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/ObjectSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService$Stub$Proxy;-setMicrophoneMute-(Z Ljava/lang/String;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityStackSupervisor;-pauseBackStacks-(Z Z Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService$UpdateRecord;-disposeLocked-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NfcBarcode;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/nfc/P2pEventListener;-onP2pHandoverNotSupported-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/InputMethodManagerService;-updateInputMethodsFromSettingsLocked-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/LauncherActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/ContentService;-syncAsUser-(Landroid/content/SyncRequest; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/SubtitleController;-addTrack-(Landroid/media/MediaFormat;)Landroid/media/SubtitleTrack;' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/handover/HandoverService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/ILocationManager;-addGpsMeasurementsListener-(Landroid/location/IGpsMeasurementsListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-showPreview-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/telephony/SmsManager;-archiveStoredConversation-(J Z)Z' : ['android.permission.WRITE_SMS'],
	'Landroid/view/IWindowSession;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkStatsService$1;-getHistoryForNetwork-(Landroid/net/NetworkTemplate; I)Landroid/net/NetworkStatsHistory;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothAdapter;-getProfileProxy-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener; I)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/P2pEventListener;-onP2pResumeSend-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wm/WindowManagerService;-performLayoutAndPlaceSurfacesLockedInner-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-access$200-(Lcom/android/server/am/ActivityManagerService; Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Landroid/content/Intent; Ljava/lang/String; Landroid/content/IIntentReceiver; I Ljava/lang/String; Landroid/os/Bundle; Ljava/lang/String; I Z Z I I I)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-cleanUpAllConnections-(Z Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider$7;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/dataconnection/DcTrackerBase;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/BootReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-updateServiceLocation-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-startCustomizedScan-(Landroid/net/wifi/ScanSettings; Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-handleIsimRefresh-(Lcom/android/internal/telephony/uicc/IccRefreshResponse;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RadioButton;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsUsageMonitor; Lcom/android/internal/telephony/ImsSMSDispatcher;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/WaveView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$InitialState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Settings$Global;-getLong-(Landroid/content/ContentResolver; Ljava/lang/String; J)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onSetDependencyMet-(Ljava/lang/String; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IMms$Stub;-deleteStoredConversation-(Ljava/lang/String; J)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/widget/ActionBarOverlayLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcRootActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setStorageEncryption-(Landroid/content/ComponentName; Z I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/app/UiAutomation;-connect-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/MifareUltralight;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendNewSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/SmsHeader; I Landroid/app/PendingIntent; Landroid/app/PendingIntent; Z Ljava/util/concurrent/atomic/AtomicInteger; Ljava/util/concurrent/atomic/AtomicBoolean; Landroid/net/Uri;)V' : ['android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPin2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/AccessibleDateAnimator;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager;-getAccountsByType-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/content/IContentService;-sync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/policy/impl/GlobalActions;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/nfc/NfcRootActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityManagerService;-comeOutOfSleepIfNeededLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AutoCompleteTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/TextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$UntetheringState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowStateAnimator;-commitFinishDrawingLocked-(J)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendSMSExpectMore-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-getFloat-(Landroid/content/ContentResolver; Ljava/lang/String; F)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-updateConfigurationLocked-(Landroid/content/res/Configuration; Lcom/android/server/am/ActivityRecord; Z Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-handleStartNetStatPoll-(Lcom/android/internal/telephony/DctConstants$Activity;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getSupportedFeatures-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/power/PowerManagerService$BinderService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ISub$Stub;-getSubInfoUsingSlotId-(I)Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/display/AutomaticBrightnessController$Callbacks;-updateBrightness-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/calendar/AgendaWindowAdapter;-buildQueryUri-(I I)Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/commands/dpm/Dpm;-run-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/nfc/NfcService$TagService;-transceive-(I [B Z)Landroid/nfc/TransceiveResult;' : ['android.permission.NFC'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/SyncActivityTooManyDeletes;-onItemClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/media/MediaSessionService;-enforceMediaPermissions-(Landroid/content/ComponentName; I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BitmapSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DropBoxManagerService;-add-(Landroid/os/DropBoxManager$Entry;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IBackupAgent$Stub;-doBackup-(Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-backgroundServicesFinishedLocked-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-onRefresh-(Z [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/ManagedServices$SettingsObserver;-<init>-(Lcom/android/server/notification/ManagedServices; Landroid/os/Handler; Lcom/android/server/notification/ManagedServices$1;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-getAutoTime-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/NetworkTimeUpdateService$SettingsObserver;-observe-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/WebVttRenderingWidget;-onAttachedToWindow-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/trust/TrustAgentService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getLine1AlphaTagForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/telephony/SubscriptionManager;-clearDefaultsForInactiveSubIds-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/provider/Contacts$People;-queryGroups-(Landroid/content/ContentResolver; J)Landroid/database/Cursor;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/BluetoothManagerService;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/WakeLockStateMachine;-handleSmsMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-unregisterListener-(Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/provider/Settings$Secure;-getLongForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService;-access$1000-(Lcom/android/server/net/NetworkStatsService; [I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setProfileEnabled-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/textservice/SpellCheckerService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-acquireWakeLockInternal-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; Landroid/os/WorkSource; Ljava/lang/String; I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/GLTextureTarget;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/P2pEventManager;-onP2pNfcTapRequested-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDataStallAlarm-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ims/ImsCall$ImsCallSessionListenerProxy;-callSessionStarted-(Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/ImsCallProfile;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-startDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/notification/NotificationListenerService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/WebVttRenderingWidget$SpanLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/AdnRecordCache;-requestLoadAllAdnLike-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/hdmi/HdmiControlService;-onInitializeCecComplete-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/AudioService;-readPersistedSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/provider/Settings$System;-getShowGTalkServiceStatus-(Landroid/content/ContentResolver;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneFactory;-isSMSPromptEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService$28;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/CallbackFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/view/IInputMethodManager;-getEnabledInputMethodSubtypeList-(Ljava/lang/String; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/RegisteredServicesCache;-removeAidGroupForService-(I I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/net/NetworkPolicyManager;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/nfc/INfcCardEmulation;-unsetPreferredService-()Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodsAndSubtypeListLocked-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-dispose-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/input/InputManagerService; Z Z Z Lcom/android/server/wm/WindowManagerService$1;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService;-removeUserLocked-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/google/android/mms/pdu/PduPersister;-getPendingMessages-(J)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/content/IContentService;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/nfc/P2pLinkManager$SendTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/ListPopupWindow$DropDownListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TelephonyRegistry;-broadcastDataConnectionStateChanged-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/NetworkCapabilities; Z J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-access$2400-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/media/projection/MediaProjectionManagerService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-conference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-finishScreenTurningOn-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/app/ChooserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/webkit/WebViewFactory;-prepareWebViewInSystemServer-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPasswordQuality-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-startDataStallAlarm-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService;-onRemoteEndpointDiscovered-(Lcom/android/nfc/DeviceHost$TagEndpoint;)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/INetworkScoreService;-disableScoring-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase$3;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-getLastInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getIsimChallengeResponse-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-handleAppCrashLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/statusbar/StatusBarManagerService;-enforceExpandStatusBar-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/server/BluetoothManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/admin/IDevicePolicyManager;-setTrustAgentFeaturesEnabled-(Landroid/content/ComponentName; Landroid/content/ComponentName; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/PrinterDiscoverySession;-stopPrinterStateTracking-(Landroid/print/PrinterId;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-sendDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/BluetoothManagerService;-loadStoredNameAndAddress-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/content/PackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/hardware/location/IGeofenceHardware;-unregisterForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService$1$1;-run-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/WallpaperManager;-suggestDesiredDimensions-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-processNextPostDialChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/BeamShareActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/voice/VoiceInteractionService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-restartPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/AppChooserActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-isUninstallBlocked-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/WaveView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/NumberPicker$CustomEditText;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthTokenByFeatures-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/app/Activity; Landroid/os/Bundle; Landroid/os/Bundle; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-reportSuccessfulPasswordAttempt-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/UiccPhoneBookController;-updateAdnRecordsInEfBySearchForSubscriber-(J I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-getPasswordMinimumUpperCase-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/IWifiP2pManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/StatusBarManager;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/widget/Chronometer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity$1;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-updateExternalState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-setPreferredVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService$Receiver;-access$1500-(Lcom/android/server/LocationManagerService$Receiver;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeech;-<init>-(Landroid/content/Context; Landroid/speech/tts/TextToSpeech$OnInitListener; Ljava/lang/String; Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService$Stub;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/app/PlatLogoActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-setNetworkSelectionModeManual-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-removeProximityAlert-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-shutdownRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CallerInfo;-markAsVoiceMail-(J)Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub;-switchImplementationIfNeededLocked-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accessibility/ScreenMagnifier$MagnifiedContentInteractonStateHandler;-onScroll-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; F F)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accounts/AccountManagerService$9;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/speech/tts/TextToSpeechService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/app/LauncherActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/AudioService;-adjustSuggestedStreamVolume-(I I I Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/SpeechRecognizer;-destroy-()V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-requestShutdown-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/test/SyncBaseInstrumentation;-cancelSyncsandDisableAutoSync-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-setUiccSubscription-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager;-getInstalledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/WakeLockStateMachine;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/fingerprint/FingerprintService;-handleNotify-(I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityRecord;-keyDispatchingTimedOut-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SubscriptionManager;-setColor-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-runSyncFinishedOrCanceledLocked-(Landroid/content/SyncResult; Lcom/android/server/content/SyncManager$ActiveSyncContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/admin/IDevicePolicyManager;-getKeyguardDisabledFeatures-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$Receiver;-incrementPendingBroadcastsLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/VpnService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/InputMethodManagerService;-getCurrentInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IMms$Stub;-archiveStoredConversation-(Ljava/lang/String; J Z)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-getActiveSubInfoCount-()I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setSuppServiceNotifications-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/hdmi/HdmiControlService;-registerContentObserver-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/ActivityView;-release-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ExpandableListActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiController;-readStayAwakeConditions-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/StateMachine;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AppSecurityPermissions$PermissionItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-setVoiceCallForwardingFlag-(I Z Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-dispatchSyncOperation-(Lcom/android/server/content/SyncOperation;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/TimePicker;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CallerInfoAsyncQuery;-startQuery-(I Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/CallerInfoAsyncQuery$OnQueryCompleteListener; Ljava/lang/Object; J)Lcom/android/internal/telephony/CallerInfoAsyncQuery;' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$5600-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-trySetupData-(Lcom/android/internal/telephony/dataconnection/ApnContext;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/NetworkScoreService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipAudioCall$1;-onError-(Landroid/net/sip/SipSession; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-dispatchNormalMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-registerForScanModeChange-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/widget/MultiAutoCompleteTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/CallbackFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onSetPolicyDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-access$000-(Lcom/android/server/devicepolicy/DevicePolicyManagerService; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-getBluetoothHeadset-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/commands/ime/Ime;-runList-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/text/StringSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/telephony/SubscriptionManager;-clearSubInfo-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/location/LocationFudger;-<init>-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/content/PackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-sendSMSExpectMore-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/InputMethodManagerService;-getEnabledInputMethodSubtypeList-(Ljava/lang/String; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/WeightedLinearLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/content/PackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/job/JobService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/backup/IBackupManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/TimePickerClockDelegate;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofenceAddStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/SystemServer;-performPendingShutdown-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/Phone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/BroadcastQueue;-scheduleBroadcastsLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ActivityManagerNative;-moveTaskToBack-(I)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/commands/am/Am;-onRun-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES'],
	'Landroid/provider/Telephony$Mms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub$1;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/DigitalClock$FormatChangeObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-access$900-(Lcom/android/internal/telephony/InboundSmsHandler; Landroid/content/Intent;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS'],
	'Landroid/media/AudioService$ScoClient;-clearCount-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/PreferredServices;-onHostEmulationDeactivated-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-addTrackerToRawTableAndSendMessage-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/am/ActivityManagerService;-handleApplicationCrashInner-(Ljava/lang/String; Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/AppWindowAnimator;-showAllWindowsLocked-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/DriverCall; Lcom/android/internal/telephony/gsm/GsmCallTracker; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/test/mock/MockContentResolver;-getIsSyncableAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/media/TtmlTrack;-hide-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/base/FrameStore;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-blockingGetAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Z)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/server/backup/BackupManagerService$RunBackupReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-onDestroy-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService;-onLlcpLinkActivated-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/nfc/P2pLinkManager;-onManualBeamInvoke-(Landroid/nfc/BeamShareData;)V' : ['android.permission.VIBRATE'],
	'Landroid/view/IWindowManager;-startFreezingScreen-(I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/print/IPrintManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-stopBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothManager;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/am/ActivityStackSupervisor$ActivityContainer;-release-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/twilight/TwilightService$LocationHandler;-enableLocationUpdates-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/backup/FullBackupJob;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/ApplicationPackageManager;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-explicitCallTransfer-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAvrcpController;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/sip/SipSessionGroup;-processDialogTerminated-(Ljavax/sip/DialogTerminatedEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipAudioCall;-grabWifiHighPerfLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/StringLogger;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/NfcAdapter;-setBeamPushUris-([Landroid/net/Uri; Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-enableSystemAppWithIntent-(Landroid/content/ComponentName; Landroid/content/Intent;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setDataAllowed-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-dialInternal-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/DialogTitle;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-writeInboxMessage-(Landroid/content/Intent;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS'],
	'Landroid/filterfw/core/GraphRunner;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/RenamingDelegatingContext;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/IsolatedContext;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/location/GpsLocationProvider;-checkSmsSuplInit-(Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManagerNative;-unbroadcastIntent-(Landroid/app/IApplicationThread; Landroid/content/Intent; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onRegistrationTimeout-(Landroid/net/sip/ISipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IVibratorService$Stub$Proxy;-vibrate-(I Ljava/lang/String; J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getAutoTimeRequired-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/CameraSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/CaptioningManager;-getRawLocale-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-exitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/hardware/location/IGeofenceHardware;-removeGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl;-enforceChangePermission-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/text/StringLogger;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/job/JobSchedulerService$JobHandler;-maybeRunPendingJobsH-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-wipeData-(I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/Phone;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-access$100-(Lcom/android/internal/telephony/imsphone/ImsPhone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/DigitalClock;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStackSupervisor;-handleDisplayRemovedLocked-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/WindowManagerInternal;-requestTraversalFromDisplayManager-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/IRecognitionService;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/telephony/TelephonyManager;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/provider/Settings$System;-getStringForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-createAndInitializeUser-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String; Landroid/content/ComponentName; Landroid/os/Bundle;)Landroid/os/UserHandle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$IdleState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Landroid/net/wifi/IWifiManager;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/ClosedCaptionWidget$CCLineBox;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getCrossProfileCallerIdDisabled-(Landroid/content/ComponentName;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/WaveView;-vibrate-(J)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/os/IPowerManager;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-handleGroupCreationFailure-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService$SetupContentObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CompoundButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-access$9100-(Landroid/media/AudioService; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-handleInjectSms-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/wm/WindowAnimator;-animateLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/DragState;-broadcastDragStartedLw-(F F)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/os/BaseCommand;-run-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/IActivityManager;-moveTaskToBack-(I)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/filterpacks/imageproc/BlendFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DebugService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/TextClock;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/NfcAdapter;-enable-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-notifyApnIdUpToCurrent-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/FilterContext;-tearDown-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverServer$ConnectionThread;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-iccCloseLogicalChannel-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RelativeLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Contacts$Settings;-setSetting-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-addCrossProfileIntentFilter-(Landroid/content/ComponentName; Landroid/content/IntentFilter; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/IGeofenceHardware;-resumeGeofence-(I I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/IRecognitionService$Stub;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getIsimChallengeResponse-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-removeAccount-(Landroid/accounts/Account; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/location/GpsLocationProvider$6;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AsyncPlayer;-access$500-(Landroid/media/AsyncPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-setForcedDisplayDensityLocked-(Lcom/android/server/wm/DisplayContent; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISub;-getDefaultVoiceSubId-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/TintFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getStorageEncryption-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService;-sendMockNdefTag-(Landroid/nfc/NdefMessage;)V' : ['android.permission.VIBRATE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-validatePrinters-(Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleDown-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler$DeliveringState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/server/voiceinteraction/SoundTriggerHelper;-onServiceDiedLocked-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/GpsLocationProvider;-sendMessage-(I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/NetworkPolicyManager;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/pm/PackageManagerService;-scanPackageLI-(Landroid/content/pm/PackageParser$Package; I I J Landroid/os/UserHandle;)Landroid/content/pm/PackageParser$Package;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiController$ApEnabledState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/am/ActiveServices;-processStartTimedOutLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/lights/LightsService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/CountdownConditionProvider;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/NfcAdapter;-disableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcRetryingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/SipManager;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-notifyApnIdUpToCurrent-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupConnectionByIndex-(Lcom/android/internal/telephony/gsm/GsmCall; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/CountdownConditionProvider;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-enableFeatures-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/os/IPowerManager$Stub;-acquireWakeLockWithUid-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayPowerController;-debounceProximitySensor-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager$10;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPasswordMinimumUpperCase-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getRestrictionsProvider-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DctController;-enableApnType-(J Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AdapterViewFlipper;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService$PerformUnifiedRestoreTask;-restoreFinished-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/IAccountManager$Stub;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/DnsPinger;-<init>-(Landroid/content/Context; Ljava/lang/String; Landroid/os/Looper; Landroid/os/Handler; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameFetch;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/webkit/WebViewUpdateService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/CalendarView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextView;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkStatsService$Stub;-getNetworkTotalBytes-(Landroid/net/NetworkTemplate; J J)J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaFocusControl;-onReevaluateRemoteControlDisplays-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/TtmlRenderer;-createTrack-(Landroid/media/MediaFormat;)Landroid/media/SubtitleTrack;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Gallery;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-getSpnFsm-(Z Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/cardemulation/HostApduService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-access$100-(Lcom/android/internal/telephony/RIL;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/dataconnection/DcTrackerBase;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/usb/UsbDeviceManager$1;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-updateSurface-(Z Z Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/widget/DigitalClock;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/LinearLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Ljava/lang/String; J F Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accessibility/ScreenMagnifier$MagnifiedContentInteractonStateHandler;-onScale-(Landroid/view/ScaleGestureDetector;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/NotificationManagerService$NotificationListeners;-rebindServices-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/RecognitionService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-iccOpenLogicalChannel-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-getFloatForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)F' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService;-cancelRequest-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/content/PackageHelper;-resolveInstallLocation-(Landroid/content/Context; Ljava/lang/String; I J I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-iccIO-(I I Ljava/lang/String; I I I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag;-ndefRead-(I)Landroid/nfc/NdefMessage;' : ['android.permission.NFC'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-onKeyUpPanel-(I Landroid/view/KeyEvent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-activityIdleInternalLocked-(Landroid/os/IBinder; Z Landroid/content/res/Configuration;)Lcom/android/server/am/ActivityRecord;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/os/Vibrator;-vibrate-([J I Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$Service;-getWindows-()Ljava/util/List;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-processCallStateChange-(Lcom/android/ims/ImsCall; Lcom/android/internal/telephony/Call$State; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPasswordMinimumUpperCase-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AppSecurityPermissions$PermissionItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-installCaCert-(Landroid/content/ComponentName; [B)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/CaptioningManager;-addCaptioningChangeListener-(Landroid/view/accessibility/CaptioningManager$CaptioningChangeListener;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager;-disable-(Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onError-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processLaunchBrowser-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Editor$InsertionHandleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/NfcService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/telephony/CellBroadcastMessage;-getDateString-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkStatsService$Stub$Proxy;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/widget/SpellChecker;-setLocale-(Ljava/util/Locale;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-performEnableScreen-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/print/PrintManager;-getPrintJob-(Landroid/print/PrintJobId;)Landroid/print/PrintJob;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService$Session;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/content/ContentResolver;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/restrictions/RestrictionsManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$Future2Task;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/ActivityManagerNative;-updatePersistentConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/effect/effects/ColorTemperatureEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/backup/BackupManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/AbstractThreadedSyncAdapter$ISyncAdapterImpl;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/view/IWindowManager$Stub$Proxy;-inputMethodClientHasFocus-(Lcom/android/internal/view/IInputMethodClient;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/AudioService$ScoClient;-requestScoState-(I I)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/text/ToUpperCase;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiScanningServiceImpl$WifiChangeStateMachine$StationaryState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-updateImsPhone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-updateImsPhone-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BatteryService;-logOutlierLocked-(J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindow$DecorView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-archiveStoredConversation-(Ljava/lang/String; J Z)Z' : ['android.permission.WRITE_SMS'],
	'Landroid/content/IContentService$Stub;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readHighTextContrastEnabledSettingLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SubscriptionManager;-getDefaultVoicePhoneId-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofenceStatus-(I I D D D F F F J)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneProxy;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService;-<init>-(Landroid/app/Application;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/net/wifi/IWifiManager;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/ActionMenuPresenter$OverflowMenuButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/ILocationManager$Stub;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/media/ClosedCaptionRenderer;-createTrack-(Landroid/media/MediaFormat;)Landroid/media/SubtitleTrack;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-confirmCredentialsAsUser-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler; Landroid/os/UserHandle;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPasswordMinimumNumeric-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NdefFormatable;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Landroid/app/backup/FullBackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/INetworkScoreService$Stub$Proxy;-disableScoring-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/NotificationManagerService;-listenForCallState-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityStack;-setVisibile-(Lcom/android/server/am/ActivityRecord; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/sip/SipManager;-createSipSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/SipSession$Listener;)Landroid/net/sip/SipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/power/PowerManagerService;-access$1100-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-checkBootAnimationCompleteLocked-()Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/View;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/app/PlatLogoActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$2500-(Lcom/android/server/wifi/WifiStateMachine; Z Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccSmsController;-getPremiumSmsPermission-(Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/GlobalActions$GlobalActionsDialog;-onStart-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/AccountManager$6;-startTask-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/LockPatternUtils;-getEnabledTrustAgents-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ListActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-revertToNitzTime-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionCallReceiverImpl;-processCommand-(Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-handleNewSms-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-setDisplayName-(Ljava/lang/String; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/TextServicesManagerService;-access$800-(Lcom/android/server/TextServicesManagerService; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/CaptioningManager;-notifyLocaleChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/dreams/DreamManagerService;-access$1000-(Lcom/android/server/dreams/DreamManagerService; I)[Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ServiceWatcher$2;-onSomePackagesChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-isWatchdogEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/le/BluetoothLeAdvertiser;-totalBytes-(Landroid/bluetooth/le/AdvertiseData;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityManagerService$Lifecycle;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService$11;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-refreshCache-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-expandNotificationsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/DhcpStateMachine$StoppedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowManager$Stub$Proxy;-startFreezingScreen-(I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask;-doWork-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-handleExitEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/sip/SipAudioCall;-access$1300-(Landroid/net/sip/SipAudioCall; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/notification/ConditionProviderService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/twilight/TwilightService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager;-prepareAppTransition-(I Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$DeactingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/provider/ContactsContract$ProfileSyncState;-newSetOperation-(Landroid/accounts/Account; [B)Landroid/content/ContentProviderOperation;' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/display/DisplayManagerService;-access$1300-(Lcom/android/server/display/DisplayManagerService; Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-getAccountsByTypeAndFeatures-(Ljava/lang/String; [Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/nfc/TechListChooserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/MediaController;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/ILocationManager;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-getShortcutInputMethodsAndSubtypes-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-sync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/nfc/cardemulation/HostApduService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-pollBatchedScan-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover;-nextStepConnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-notifyLockTaskModeChanged-(Z Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/RingtoneManager;-stopPreviousRingtone-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/Context;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-access$10800-(Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-dispatchMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/app/admin/IDevicePolicyManager;-setAutoTimeRequired-(Landroid/content/ComponentName; I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendDataForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/nfc/handover/HandoverTransfer;-sendBluetoothCancelIntentAndUpdateState-()V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/NullFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-setPasswordMinimumUpperCase-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/HdmiCecLocalDeviceTv;-getSystemAudioModeSetting-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker;-dispatchHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/tv/TvView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/PhoneNumberUtils;-getDefaultVoiceSubId-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/InboundSmsHandler$StartupState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-getDefaultWindowListLocked-()Lcom/android/server/wm/WindowList;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/mock/MockContentResolver;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/content/IContentService$Stub$Proxy;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/widget/Button;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/ims/ImsCall;-setListener-(Lcom/android/ims/ImsCall$Listener; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/usb/UsbDeviceManager;-systemReady-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/SmsManager;-enableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$P2pNotSupportedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/filterpacks/base/RetargetFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-onStart-()V' : ['android.permission.GET_TASKS'],
	'Ljava/net/URL;-getContent-()Ljava/lang/Object;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-proceedAfterWaitChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifySignalStrength-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextClock$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService$1;-onUserSwitching-(I Landroid/os/IRemoteCallback;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPermittedAccessibilityServicesForUser-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameBranch;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/CallbackFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService;-startBluetoothScoVirtualCall-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/base/InputStreamSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setApplicationHidden-(Landroid/content/ComponentName; Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkStatsService;-getDataLayerSnapshotForUid-(I)Landroid/net/NetworkStats;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/ShutdownActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/webkit/WebView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/EditText;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$Receiver;-decrementPendingBroadcastsLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-proceedAfterWildChar-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/fingerprint/FingerprintService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/IContentService;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/view/accessibility/CaptioningManager;-isEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-disconnectBluetoothSco-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/content/SyncManager;-updateRunningAccounts-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/display/DisplayPowerController;-updatePowerState-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$HideNavInputEventReceiver;-onInputEvent-(Landroid/view/InputEvent;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/View;-performAccessibilityActionInternal-(I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RadioGroup;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/nfc/cardemulation/HostApduService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService;-switchToLastInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-enableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Landroid/speech/tts/TextToSpeechService;-access$500-(Landroid/speech/tts/TextToSpeechService;)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService$Stub$Proxy;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/SubscriptionController;-getDefaultVoiceSubId-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaPlayer;-create-(Landroid/content/Context; I Landroid/media/AudioAttributes; I)Landroid/media/MediaPlayer;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-stopDtmf-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/TelephonyManager;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/nfc/INfcCardEmulation$Stub;-removeAidGroupForService-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getAutoTimeRequired-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/SlidingDrawer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-detectSafeMode-()Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-iccTransmitApduLogicalChannel-(I I I I I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/ConditionProviders;-setZenModeCondition-(Landroid/service/notification/Condition; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Browser;-updateVisitedHistory-(Landroid/content/ContentResolver; Ljava/lang/String; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/UiccSmsController;-sendStoredMultipartText-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/ISub$Stub;-clearSubInfo-()I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/notification/NotificationManagerService;-applyZenModeLocked-(Lcom/android/server/notification/NotificationRecord;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/inputmethodservice/ExtractButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RemoteViewsService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/cardemulation/DefaultRemovedActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager;-removeCrossProfileWidgetProvider-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/BeamShareActivity;-parseShareIntentAndFinish-(Landroid/content/Intent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-cancelPostDial-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/GeofenceHardwareService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telecom/InCallService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/voice/VoiceInteractionSessionService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onCallEnded-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/tv/TvInputService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/NetworkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityStackSupervisor;-processStoppingActivitiesLocked-(Z)Ljava/util/ArrayList;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/effect/effects/SharpenEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/service/voice/VoiceInteractionService;-isActiveService-(Landroid/content/Context; Landroid/content/ComponentName;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$UserState;-onSwitchToAnotherUser-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/media/MediaSessionService;-updateActiveSessionListeners-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BatteryService;-onBootPhase-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/text/method/PasswordTransformationMethod;-onTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setOperatorBrandOverride-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/bluetooth/BluetoothAvrcpController;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/InboundSmsHandler$IdleState;-exit-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ethernet/EthernetService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/connectivity/PacManager;-access$800-(Lcom/android/server/connectivity/PacManager;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-isMasterVolumeMuted-(Landroid/content/ComponentName;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/base/FrameStore;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/IEthernetManager;-getConfiguration-()Landroid/net/IpConfiguration;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/view/IWindowSession$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/MmsServiceBroker;-access$1100-()Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/widget/VideoView;-access$2200-(Landroid/widget/VideoView; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService$Lifecycle;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getLong-(Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-handleInCallMmiCommands-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getVoiceMailAlphaTagForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-init-(Landroid/content/Context; Landroid/view/IWindowManager; Landroid/view/WindowManagerPolicy$WindowManagerFuncs;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/backup/BackupManagerService$ProvisionedObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/view/menu/SubMenuBuilder;-changeMenuMode-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/NativeActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/nfc/cardemulation/HostApduService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setUserRestriction-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-restartDataStallAlarm-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/StackView$StackFrame;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setMaximumTimeToLock-(Landroid/content/ComponentName; J I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextView$ChangeWatcher;-beforeTextChanged-(Ljava/lang/CharSequence; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/AutomaticBrightnessController;-access$300-(Lcom/android/server/display/AutomaticBrightnessController; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$21;-run-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/UiAutomationConnection;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/commands/svc/Svc;-main-([Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/job/JobServiceContext$JobServiceHandler;-handleOpTimeoutH-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-updateInputFilter-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/ClosedCaptionWidget$CCLineBox;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-clearProfileOwner-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IWallpaperManager$Stub;-setDisplayPadding-(Landroid/graphics/Rect;)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Landroid/test/mock/MockContentResolver;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wm/WindowState;-setAppOpVisibilityLw-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$1;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-getInitialDisplayDensity-(I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/SettingInjectorService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-getConnectionStatistics-()Landroid/net/wifi/WifiConnectionStatistics;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-updateUsbState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/IAudioService$Stub$Proxy;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/net/NetworkPolicyManagerService$6;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/test/RenamingDelegatingContext;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/net/wifi/WifiManager;-getScanResults-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/power/ShutdownThread$5;-run-()V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/connectivity/Tethering;-getUpstreamIfaceTypes-()[I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupWaitingOrBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverTransfer;-updateStateAndNotification-(I)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/InputMethodManagerService;-resetDefaultImeLocked-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/PopupWindow$PopupViewContainer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityStack;-resetTargetTaskIfNeededLocked-(Lcom/android/server/am/TaskRecord; Z)Landroid/app/ActivityOptions;' : ['android.permission.SEND_SMS', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/service/dreams/Sandman;-isScreenSaverEnabled-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-stopPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-writeOutboxMessage-(J Ljava/lang/String; Ljava/lang/String; Z Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS'],
	'Landroid/nfc/tech/NfcV;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/RIL;-setFacilityLock-(Ljava/lang/String; Z Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/DefaultRemovedActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-handleStartNetStatPoll-(Lcom/android/internal/telephony/DctConstants$Activity;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/LomoishEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/job/JobServiceContext$JobServiceHandler;-handleServiceBoundH-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IMms;-importMultimediaMessage-(Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; J Z Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder$StateCmdParamsReady;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/DatePickerDialog;-updateTitle-(I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ContentResolver;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-adjustStreamVolume-(I I I Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setExpirationAlarmCheckLocked-(Landroid/content/Context; Lcom/android/server/devicepolicy/DevicePolicyManagerService$DevicePolicyData;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-installPackageLI-(Lcom/android/server/pm/PackageManagerService$InstallArgs; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/LauncherActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryFacilityLockForApp-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityManagerNative;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-onDestroy-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/trust/TrustManagerService;-onBootPhase-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ContentResolver;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-registerUiTestAutomationService-(Landroid/os/IBinder; Landroid/accessibilityservice/IAccessibilityServiceClient; Landroid/accessibilityservice/AccessibilityServiceInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/AudioService$ScoClient;-binderDied-()V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/location/CountryDetectorBase;-setCountryListener-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-showEmulatorDisplayOverlayIfNeeded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/commands/bmgr/Bmgr;-run-([Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayPowerState$PhotonicModulator;-requestDisplayState-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/IAudioService$Stub;-setRingerMode-(I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub$Proxy;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/server/SystemServer;-startCoreServices-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.READ_PROFILE'],
	'Landroid/app/Application;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setRestrictionsProvider-(Landroid/content/ComponentName; Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/TtmlRenderingWidget;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/CallbackFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-sendDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-sendMultipartSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-setPropertiesByCarrier-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayPowerController;-initialize-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/videosrc/CameraSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-handleSrvccStateChanged-([I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService;-access$1000-(Lcom/android/server/InputMethodManagerService; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-installCaCert-(Landroid/content/ComponentName; [B)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub$Proxy;-getMasterSyncAutomaticallyAsUser-(I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-disableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/app/Activity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getDataRoamingEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/DriverCall; Lcom/android/internal/telephony/cdma/CdmaCallTracker; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/SmsBroadcastUndelivered;-run-()V' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/speech/tts/TextToSpeech$Connection;-runAction-(Landroid/speech/tts/TextToSpeech$Action; Ljava/lang/Object; Ljava/lang/String; Z Z)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$TetheredState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextSwitcher;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/ManagedServices$SettingsObserver;-observe-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/Session;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/cdma/CdmaCallWaitingNotification; Lcom/android/internal/telephony/cdma/CdmaCallTracker; Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocalListenerHelper;-add-(Ljava/lang/Object;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/backup/BackupManagerService$PerformBackupTask;-invokeAgentForBackup-(Ljava/lang/String; Landroid/app/IBackupAgent; Lcom/android/internal/backup/IBackupTransport;)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/INfcTag$Stub;-setTimeout-(I I)I' : ['android.permission.NFC'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-setIccLockEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayPowerController$8;-onSensorChanged-(Landroid/hardware/SensorEvent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/input/InputManagerService;-updateShowTouchesFromSettings-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getRestrictionsProvider-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$Receiver;-callProviderEnabledLocked-(Ljava/lang/String; Z)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/net/INetworkScoreService$Stub;-updateScores-([Landroid/net/ScoredNetwork;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcTag$Stub;-ndefWrite-(I Landroid/nfc/NdefMessage;)I' : ['android.permission.NFC'],
	'Lcom/android/server/backup/FullBackupJob;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/ims/ImsCall$Listener;-onCallStartFailed-(Lcom/android/ims/ImsCall; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPasswordExpirationTimeout-(Landroid/content/ComponentName; J I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/gsm/SmsManager;-sendDataMessage-(Ljava/lang/String; Ljava/lang/String; S [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/PhoneProxy;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcBackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$SetIpForwardingEnabledErrorState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcAdapter$Stub;-enableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/backup/BackupManagerService$PerformBackupTask;-agentErrorCleanup-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/AdnRecordCache;-updateAdnBySearch-(I Lcom/android/internal/telephony/uicc/AdnRecord; Lcom/android/internal/telephony/uicc/AdnRecord; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/NetworkIdentity;-buildNetworkIdentity-(Landroid/content/Context; Landroid/net/NetworkState;)Landroid/net/NetworkIdentity;' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/NfcService;-sendMessage-(I Ljava/lang/Object;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ViewSwitcher;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/BluetoothManagerService;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingRfcommOn-(I)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/sip/SipService;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/server/wm/WindowManagerService$LocalService;-waitForAllWindowsDrawn-(Ljava/lang/Runnable; J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService;-handleDisplayDeviceChanged-(Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-cleanUpAllConnections-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/provider/Browser;-getAllBookmarks-(Landroid/content/ContentResolver;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/PhoneBase;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordMinimumLength-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-addPhoneStateListener-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/LockPatternUtils;-passwordToHash-(Ljava/lang/String; I)[B' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Call;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-setMaximumScreenOffTimeoutFromDeviceAdminInternal-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-getAllowScansWithTraffic-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/pm/BackgroundDexOptService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiServiceImpl;-enforceMulticastChangePermission-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-handleEnterEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BatteryService;-update-(Landroid/os/BatteryProperties;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-access$400-(Lcom/android/server/am/ActivityManagerService; Ljava/lang/String; I Z Z Z Z Z I Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/dreams/DreamManagerService;-stopDreamLocked-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-hasUserSetupCompleted-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-getPrintJobInfos-(I I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-chooseNewDefaultIMELocked-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getDeviceIdForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-access$000-(Lcom/android/server/wifi/WifiWatchdogStateMachine;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/location/ILocationManager$Stub$Proxy;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-enableCellBroadcastRangeForSubscriber-(J I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/widget/ScrollingTabContainerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/TextProgressBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/CantAddAccountActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/Context;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/DisableCarModeActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-toString-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/BeamShareActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-wipeData-(I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-<init>-(Landroid/content/Context; I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-revertToNitzTimeZone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-access$4600-(Lcom/android/server/power/PowerManagerService; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/view/inputmethod/InputMethodManager;-switchToLastInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-cancelPendingUssd-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setActiveAdmin-(Landroid/content/ComponentName; Z I Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowStateAnimator;-cancelExitAnimationForNextAnimationLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/effect/effects/BitmapOverlayEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setGlobalProxy-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String; I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler$WaitingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/print/PrintManager;-addPrintJobStateChangeListener-(Landroid/print/PrintManager$PrintJobStateChangeListener;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/UserState;-readEnabledPrintServicesLocked-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent; Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterfw/core/Filter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/UiAutomation;-disconnect-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-access$400-(Lcom/android/server/LocationManagerService; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/MediaSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService$LocalService;-initPowerManagement-(Landroid/hardware/display/DisplayManagerInternal$DisplayPowerCallbacks; Landroid/os/Handler; Landroid/hardware/SensorManager;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Landroid/service/textservice/SpellCheckerService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/hdmi/SystemAudioAutoInitiationAction;-handleTimerEvent-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService$Stub;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/widget/TextViewWithCircularIndicator;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/Installer;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/WifiStateMachine$DriverStoppedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-getChannelList-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getLockTaskPackages-(Landroid/content/ComponentName;)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendTerminalResponse-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowState;-hideLw-(Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/RotateFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/videosrc/CameraSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/IContentService$Stub;-getMasterSyncAutomaticallyAsUser-(I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/usage/UsageStatsService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallHeld-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/trust/TrustManagerService$3;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-handleMotionEventStateDelegating-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setProfileName-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/CaptioningManager;-access$200-(Landroid/view/accessibility/CaptioningManager;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService$SettingsObserver;-<init>-(Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getAccountTypesWithManagementDisabled-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/PersistentDataBlockService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setCellInfoListRate-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TextServicesManagerService;-setCurrentSpellCheckerSubtype-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-getActiveSubInfoCount-()I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/nfc/cardemulation/RegisteredServicesCache;-getInstalledServices-(I)Ljava/util/ArrayList;' : ['android.permission.NFC'],
	'Lcom/android/server/wm/WindowManagerService;-handleDisplayRemovedLocked-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/IAccountManager;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/accounts/AccountManager$11;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/nfc/INfcCardEmulation;-setPreferredService-(Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager;-onHostCardEmulationDeactivated-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getMsisdnForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/widget/ActionBarContextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$OnlineState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/telecom/ConnectionService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/IAccountManager$Stub;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/content/ContentResolver;-startSync-(Landroid/net/Uri; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/renderscript/RSSurfaceView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/voiceinteraction/SoundTriggerHelper;-startRecognition-(I Landroid/hardware/soundtrigger/SoundTrigger$KeyphraseSoundModel; Landroid/hardware/soundtrigger/IRecognitionStatusCallback; Landroid/hardware/soundtrigger/SoundTrigger$RecognitionConfig;)I' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/app/NetInitiatedActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/LinearLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-acknowledgeLastIncomingGsmSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AdapterView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/trust/TrustManagerService;-access$1400-(Lcom/android/server/trust/TrustManagerService; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Ljava/net/URL;-openConnection-()Ljava/net/URLConnection;' : ['android.permission.INTERNET'],
	'Landroid/print/IPrintManager;-getEnabledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/mtp/MtpDatabase;-endSendObject-(Ljava/lang/String; I I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-onSubscribe-(Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AbsListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker$AccessibilityNodeProviderImpl;-performAction-(I I Landroid/os/Bundle;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IBackupAgent$Stub;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/pm/PackageManagerService$InstallParams$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-stopDtmf-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverManager;-tryHandoverRequest-(Landroid/nfc/NdefMessage;)Landroid/nfc/NdefMessage;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/ethernet/EthernetServiceImpl;-setConfiguration-(Landroid/net/IpConfiguration;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/nfc/tech/NfcA;-reconnect-()V' : ['android.permission.NFC'],
	'Landroid/speech/RecognitionService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/TelephonyRegistry;-broadcastServiceStateChanged-(Landroid/telephony/ServiceState; J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/voice/VoiceInteractionSessionService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/backup/IBackupManager$Stub;-clearBackupData-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkScoreService$Stub;-disableScoring-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-adjustWallpaperWindowsLocked-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager;-onPreferredPaymentServiceChanged-(Landroid/content/ComponentName;)V' : ['android.permission.VIBRATE'],
	'Landroid/view/WindowManagerPolicy$WindowState;-hideLw-(Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/TextViewWithCircularIndicator;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-copyMessageToIccEf-(Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/nfc/handover/HandoverService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/nfc/P2pLinkManager$SendTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Void;' : ['android.permission.BLUETOOTH'],
	'Landroid/gesture/GestureOverlayView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine$DefaultState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/am/ActivityStack;-releaseBackgroundResources-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-launchVoiceAssistWithWakeLock-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/content/Context;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IMms$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_MMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/wifi/WifiController;-readWifiReEnableDelay-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/KeyboardView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ims/ImsCall$ImsCallSessionListenerProxy;-callSessionHeld-(Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/ImsCallProfile;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendDataForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/net/EthernetManager;-getConfiguration-()Landroid/net/IpConfiguration;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/nfc/cardemulation/TapAgainDialog;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/print/PrintManager;-cancelPrintJob-(Landroid/print/PrintJobId;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-logStateAndMessage-(Landroid/os/Message; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ConditionProviders;-onUserSwitched-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/FinalPort;-setFieldFrame-(Landroid/filterfw/core/Frame; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Contacts$People;-createPersonInMyContactsGroup-(Landroid/content/ContentResolver; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/content/ContentResolver;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverServer;-start-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/UiModeManagerService;-sendConfigurationLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPasswordMinimumSymbols-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getLine1NumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/http/SslCertificate;-formatCertificateDate-(Landroid/content/Context; Ljava/util/Date;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameStore;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/MultiAutoCompleteTextView;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-rejectCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipAudioCall;-access$1000-(Landroid/net/sip/SipAudioCall; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/ContactsContract$Groups$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Landroid/media/MediaPlayer;-access$900-(Landroid/media/MediaPlayer; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IBackupManager$Stub$Proxy;-opComplete-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager$8;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-<init>-(Lcom/android/internal/telephony/uicc/UiccCard; Lcom/android/internal/telephony/uicc/IccCardApplicationStatus; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-shutdownRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onSetPolicyDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService;-getAllowedResolutionLevel-(I I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-addTrackerToRawTableAndSendMessage-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/am/ActivityStackSupervisor;-ensureActivitiesVisibleLocked-(Lcom/android/server/am/ActivityRecord; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/service/media/MediaBrowserService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/service/dreams/DreamService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/nfc/tech/NfcV;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$DefaultState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/WiredAccessoryManager;-access$700-(Lcom/android/server/WiredAccessoryManager; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-getCopyNetworkCapabilities-()Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationFudger;-access$000-(Lcom/android/server/location/LocationFudger;)F' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-removeGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setGlobalSetting-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIccCardStatus-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/RttService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setGlobalSetting-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ViewFlipper;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-addPersistentPreferredActivity-(Landroid/content/ComponentName; Landroid/content/IntentFilter; Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccPhoneBookController;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/NetworkScoreService;-disableScoring-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/base/FrameSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/util/NtpTrustedTime;-getInstance-(Landroid/content/Context;)Landroid/util/NtpTrustedTime;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getLine1NumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/util/StateMachine$SmHandler$QuittingState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-uninstallCaCert-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-createUser-(Landroid/content/ComponentName; Ljava/lang/String;)Landroid/os/UserHandle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-startObservingNativeCrashes-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-resetPassword-(Ljava/lang/String; I I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pManager;-initializeInternal-(Landroid/content/Context; Landroid/os/Looper; Landroid/net/wifi/p2p/WifiP2pManager$ChannelListener;)Landroid/net/wifi/p2p/WifiP2pManager$Channel;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-killAllBackgroundProcesses-()V' : ['android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/server/wm/WindowManagerService;-reconfigureDisplayLocked-(Lcom/android/server/wm/DisplayContent;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/Ringtone;-setStreamType-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/input/InputManagerService$WiredAccessoryCallbacks;-notifyWiredAccessoryChanged-(J I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getIsSyncable-(Landroid/accounts/Account; Ljava/lang/String;)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/RetargetFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine;-sendRssiChangeBroadcast-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ActivityManagerNative;-moveTaskBackwards-(I)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/server/am/ActivityStackSupervisor;-handleDisplayChangedLocked-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-enforceCanManageCaCerts-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ims/internal/IImsCallSessionListener$Stub;-callSessionStartFailed-(Lcom/android/ims/internal/IImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/OutputStreamTarget;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Editor;-updateSpellCheckSpans-(I I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-getInitialDisplaySize-(I Landroid/graphics/Point;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAccountsAsUser-(I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/location/ILocationManager$Stub;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-access$3000-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker$CustomEditText;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ContentResolver;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/widget/RotarySelector;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/RIL;-getLastCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextClock;-setFormat12Hour-(Ljava/lang/CharSequence;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubInfoRecordUpdater$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/sip/ISipService$Stub;-isOpened-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/policy/impl/ImmersiveModeConfirmation$ClingWindowView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/MutableContextWrapper;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/RttService$RttServiceImpl$RttStateMachine;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/display/RampAnimator$Listener;-onAnimationEnd-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/TelephonyManager;-getDeviceId-(I)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/content/IContentService;-getCurrentSyncsAsUser-(I)Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/Phone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BootReceiver;-access$100-(Lcom/android/server/BootReceiver; Landroid/content/Context;)V' : ['android.permission.INTERNET'],
	'Landroid/widget/NumberPicker$CustomEditText;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readDisplayColorAdjustmentSettingsLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SmsManager;-isSMSPromptEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/notification/ConditionProviders;-saveZenConfigLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/AppChooserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/NfcService$TagService;-ndefWrite-(I Landroid/nfc/NdefMessage;)I' : ['android.permission.NFC'],
	'Lcom/android/server/am/ActivityStack;-ensureActivitiesVisibleLocked-(Lcom/android/server/am/ActivityRecord; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService$Stub$Proxy;-isSyncPendingAsUser-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName; I)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/telephony/TelephonyManager;-getVoiceMailNumber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-startFreezingDisplayLocked-(Z I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-inputMethodClientHasFocus-(Lcom/android/internal/view/IInputMethodClient;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService$LocalService;-startIsolatedProcess-(Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I Ljava/lang/Runnable;)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/WindowManagerImpl;-addView-(Landroid/view/View; Landroid/view/ViewGroup$LayoutParams;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothAdapter;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/app/TabActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifyServiceStateForPhoneId-(I J Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/videosrc/CameraSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation;-setDefaultServiceForCategory-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/internal/textservice/ITextServicesManager;-setCurrentSpellChecker-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManager;-moveTaskToFront-(I I Landroid/os/Bundle;)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/nfc/INfcCardEmulation$Stub$Proxy;-registerAidGroupForService-(I Landroid/content/ComponentName; Landroid/nfc/cardemulation/AidGroup;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-access$400-(Landroid/inputmethodservice/KeyboardView; Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-setOverscanLocked-(Lcom/android/server/wm/DisplayContent; I I I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/TextViewWithCircularIndicator;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/BaseCommands;-setDataAllowed-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-addGpsMeasurementsListener-(Landroid/location/IGpsMeasurementsListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeech;-getDefaultVoice-()Landroid/speech/tts/Voice;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-startProvisioningApnAlarm-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/PreferredServices;-computePreferredForegroundService-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wifi/RttService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-supplyPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService;-registerForAirplaneMode-(Landroid/content/IntentFilter;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-stopBatchedScan-(Landroid/net/wifi/BatchedScanSettings;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService$Stub$Proxy;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/hdmi/HdmiCecFeatureAction;-processCommand-(Lcom/android/server/hdmi/HdmiCecMessage;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-onQueryCfComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService$1;-onLowPowerModeChanged-(Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-isRotationFrozen-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-runBoundToAdapter-(Lcom/android/server/content/SyncManager$ActiveSyncContext; Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$VerifyingLinkState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/cardemulation/AppChooserActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/pm/Pm;-runTrimCaches-()V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/server/usb/UsbService;-systemReady-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/PhoneNumberUtils;-isVoiceMailNumber-(J Ljava/lang/String;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/InputMethodManagerService;-showInputMethodMenuInternal-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getBatchedScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handleRadioNotAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-validatePrinters-(Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/RetargetFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-doRecovery-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/usb/UsbHostManager;-endUsbDeviceAdded-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/PackageManagerService;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Lcom/android/server/wm/WindowManagerService;-newDisplayContentLocked-(Landroid/view/Display;)Lcom/android/server/wm/DisplayContent;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AbsSeekBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$GroupCreatedState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextView$3;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BatteryService$8;-run-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/ClosedCaptionWidget$CCLineBox;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/WallpaperManager;-setBitmap-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCallWaiting-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IMms;-addTextMessageDraft-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/content/ContentResolver;-getSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/backup/LocalTransportService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/QuickContactBadge$QueryHandler;-onQueryComplete-(I Ljava/lang/Object; Landroid/database/Cursor;)V' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/accessibility/TouchExplorer;-handleMotionEventStateTouchExploring-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaMmiCode;-processCode-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/usb/UsbHostManager;-sendDeviceNotification-(I I Z Z Z Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/SmsManager;-sendStoredTextMessage-(Landroid/net/Uri; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/os/Vibrator;-vibrate-([J I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getString-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ViewAnimator;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-checkMemory-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.CLEAR_APP_CACHE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService;-access$300-(Lcom/android/server/VibratorService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/server/SystemServer;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/backup/BackupManagerService$PerformBackupTask;-execute-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleInCallMmiCommands-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-isMasterVolumeMuted-(Landroid/content/ComponentName;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaFocusControl$NotificationListenerObserver;-<init>-(Landroid/media/MediaFocusControl;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub;-addAccount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/service/voice/VoiceInteractionService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendMultipartSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/location/LocationManager;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/text/ToUpperCase;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/Phone;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$VerifyingLinkState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/pm/Pm;-runGetInstallLocation-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-uninstallCaCert-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/inputmethod/InputMethodManager;-getEnabledInputMethodList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech;-initTts-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractEditLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$UserAuthorizingInviteRequestState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/NfcService$TagService;-getTechList-(I)[I' : ['android.permission.NFC'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPasswordQuality-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService$TagService;-connect-(I I)I' : ['android.permission.NFC'],
	'Landroid/location/LocationManager;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/NsdService$NsdStateMachine$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/CountdownConditionProvider;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/nfc/NfcService$EnableDisableTask;-doInBackground-([Ljava/lang/Object;)Ljava/lang/Object;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Landroid/widget/ZoomControls;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/SupplicantStateTracker$UninitializedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-renameAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/widget/AccessibleDateAnimator;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubInfoRecordUpdater;-access$300-(Lcom/android/internal/telephony/SubInfoRecordUpdater; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag;-ndefWrite-(I Landroid/nfc/NdefMessage;)I' : ['android.permission.NFC'],
	'Landroid/print/IPrintManager;-validatePrinters-(Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-removeWakeLockLocked-(Lcom/android/server/power/PowerManagerService$WakeLock; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/speech/tts/ITextToSpeechService$Stub;-getClientDefaultLanguage-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-notifyLockTaskModeChanged-(Z Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/hardware/input/InputManager$InputDeviceVibrator;-vibrate-(J Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/IMms$Stub;-downloadMessage-(J Ljava/lang/String; Ljava/lang/String; Landroid/net/Uri; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.RECEIVE_MMS'],
	'Lcom/android/server/am/BroadcastQueue;-performReceiveLocked-(Lcom/android/server/am/ProcessRecord; Landroid/content/IIntentReceiver; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothDevice; Ljava/lang/String; I Lcom/android/nfc/handover/BluetoothPeripheralHandover$Callback;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TelephonyRegistry;-notifyServiceStateForPhoneId-(I J Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/cardemulation/CardEmulation;-unsetPreferredService-(Landroid/app/Activity;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/server/content/ContentService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/accessibility/EventStreamTransformation;-onMotionEvent-(Landroid/view/MotionEvent; Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-installKeyPair-(Landroid/content/ComponentName; [B [B Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/power/PowerManagerService$BinderService;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$WaitForP2pDisableState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-setRadioPower-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService;-scanDirLI-(Ljava/io/File; I I J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/backup/BackupManagerService$ActiveRestoreSession;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/InstrumentationTestRunner;-finish-(I Landroid/os/Bundle;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$2300-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accessibilityservice/IAccessibilityServiceConnection$Stub$Proxy;-getWindow-(I)Landroid/view/accessibility/AccessibilityWindowInfo;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/media/MediaSessionService;-access$2300-(Lcom/android/server/media/MediaSessionService; Landroid/content/ComponentName; I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/dhimpl/NativeNfcTag;-startPresenceChecking-(I Lcom/android/nfc/DeviceHost$TagDisconnectedCallback;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ApplicationPackageManager;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/sip/SipPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/LomoishEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setKeyguardDisabledFeatures-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setGsmBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ProcessList;-applyDisplaySize-(Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-sendEmergencyCallbackModeChange-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeech;-<init>-(Landroid/content/Context; Landroid/speech/tts/TextToSpeech$OnInitListener;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-buildWaitingApns-(Ljava/lang/String; I)Ljava/util/ArrayList;' : ['android.permission.READ_PROFILE'],
	'Landroid/view/ViewStub;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/google/android/mms/util/PduCache;-put-(Ljava/lang/Object; Ljava/lang/Object;)Z' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub;-getCurInteractor-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-notCurrentSession-(Landroid/net/sip/ISipSession;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-startNextVibrationLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onRoamingOff-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-updateFocusedWindowLocked-(I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStack;-moveTaskToFrontLocked-(Lcom/android/server/am/TaskRecord; Lcom/android/server/am/ActivityRecord; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiApConfigStore$ActiveState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IMms$Stub;-addTextMessageDraft-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/view/IWindowManager$Stub;-isRotationFrozen-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/media/MediaSessionService;-access$1300-(Lcom/android/server/media/MediaSessionService;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-isSecure-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getVoiceMailAlphaTagForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/content/CursorEntityIterator;-getEntityAndIncrementCursor-(Landroid/database/Cursor;)Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/app/ResolverActivity$1;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/test/RenamingDelegatingContext;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/nfc/tech/NdefFormatable;-reconnect-()V' : ['android.permission.NFC'],
	'Landroid/os/IPowerManager$Stub;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NetworkScoreService;-clearScores-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/trust/TrustManagerService;-refreshAgentList-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/FragmentBreadCrumbs;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/pm/PackageManagerService$InstallParams;-handleStartCopy-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerPolicy;-init-(Landroid/content/Context; Landroid/view/IWindowManager; Landroid/view/WindowManagerPolicy$WindowManagerFuncs;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/location/GpsLocationProvider;-handleUpdateNetworkState-(I Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/SystemAudioAutoInitiationAction;-handleSystemAudioModeStatusMessage-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/Filter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/GeofenceHardware;-registerForMonitorStateChangeCallback-(I Landroid/hardware/location/GeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifyNdefMessageListeners-(Lcom/android/nfc/dhimpl/NativeNfcTag;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/AccessibilityController$DisplayMagnifier$MagnifiedViewport;-drawWindowIfNeededLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/service/wallpaper/WallpaperService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/accessibility/IAccessibilityManager$Stub$Proxy;-temporaryEnableAccessibilityStateUntilKeyguardRemoved-(Landroid/content/ComponentName; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/service/voice/VoiceInteractionSessionService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/CoreSettingsObserver;-<init>-(Lcom/android/server/am/ActivityManagerService;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getCrossProfileCallerIdDisabled-(Landroid/content/ComponentName;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/squareup/okhttp/internal/http/JavaApiConverter$CacheHttpsURLConnection;-connect-()V' : ['android.permission.INTERNET'],
	'Landroid/content/IContentService$Stub$Proxy;-getIsSyncableAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/IWifiManager;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readIapFileAndWait-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/SamplingProfilerService;-registerSettingObserver-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/search/SearchManagerService$MyPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/hdmi/HdmiControlService$1;-onAllocated-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/IntentService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/ILocationManager$Stub;-reportLocation-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/media/AudioManager;-setParameters-(Ljava/lang/String;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/PrinterDiscoverySession;-stopPrinterDiscovery-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/NewDeviceAction;-handleTimerEvent-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerInternal;-setMagnificationSpec-(Landroid/view/MagnificationSpec;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-resumeRotationLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/CheckedTextView;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-onUserRemoved-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/IActivityManager;-startUserInBackground-(I)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/tech/MifareClassic;-writeBlock-(I [B)V' : ['android.permission.NFC'],
	'Lcom/android/server/hdmi/HdmiControlService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ChooserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneFactory;-makeSipPhone-(Ljava/lang/String;)Lcom/android/internal/telephony/sip/SipPhone;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/TextClock;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$SetIpForwardingDisabledErrorState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipSessionGroup;-processRequest-(Ljavax/sip/RequestEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-saveLockPassword-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-getAccounts-()[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcActivatingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IIntentReceiver$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-startUser-(I Z)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$18900-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/WindowManagerPolicy;-dispatchUnhandledKey-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-setDataRoaming-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/DragState;-notifyMoveLw-(F F)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/hardware/ConsumerIrManager;-transmit-(I [I)V' : ['android.permission.TRANSMIT_IR'],
	'Landroid/hardware/IConsumerIrService$Stub;-getCarrierFrequencies-()[I' : ['android.permission.TRANSMIT_IR'],
	'Landroid/view/IWindowSession$Stub$Proxy;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setOperatorBrandOverride-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SmsMessage;-calculateLength-(Ljava/lang/CharSequence; Z)[I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofencePause-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TelephonyRegistry;-broadcastSignalStrengthChanged-(Landroid/telephony/SignalStrength; J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-getDataEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/IProcessObserver$Stub$Proxy;-onForegroundActivitiesChanged-(I I Z)V' : ['android.permission.VIBRATE'],
	'Landroid/net/INetworkScoreService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-createPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/FaceUnlockView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/INfcAdapter;-enable-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/AccessibilityController;-setMagnificationCallbacksLocked-(Landroid/view/WindowManagerInternal$MagnificationCallbacks;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-addAccount-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/server/content/ContentService;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController$ScanOnlyLockHeldState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Landroid/accessibilityservice/AccessibilityService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/FlpHardwareProvider;-getInstance-(Landroid/content/Context;)Lcom/android/server/location/FlpHardwareProvider;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/AccessibilityController$DisplayMagnifier;-drawMagnifiedRegionBorderIfNeededLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiNotificationController$NotificationEnabledSettingObserver;-register-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/BroadcastQueue;-backgroundServicesFinishedLocked-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/tech/NfcV;-close-()V' : ['android.permission.NFC'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-attach-(Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Settings$System;-getIntForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService;-systemRunning-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountAuthenticatorActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-reportSmsMemoryStatus-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-getLongForUser-(Landroid/content/ContentResolver; Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings; Landroid/os/IBinder; Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActiveServices$ServiceMap;-rescheduleDelayedStarts-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/InstrumentationTestRunner;-onStart-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/renderscript/RSSurfaceView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processGetInkey-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/IsoDep;-getTimeout-()I' : ['android.permission.NFC'],
	'Landroid/nfc/NfcAdapter;-setNdefPushMessage-(Landroid/nfc/NdefMessage; Landroid/app/Activity; I)V' : ['android.permission.NFC'],
	'Lcom/android/server/TelephonyRegistry;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setStorageEncryption-(Landroid/content/ComponentName; Z I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setTrustAgentFeaturesEnabled-(Landroid/content/ComponentName; Landroid/content/ComponentName; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/BeamShareActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/pm/PackageManagerService$HandlerParams;-startCopy-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService$BackupRestoreTask;-handleTimeout-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/SubscriptionController;-broadcastDefaultVoiceSubIdChanged-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/tech/BasicTagTechnology;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getCallWaiting-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCDMASubscription-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-upgradeLegacyBackgroundData-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getSupportedFeatures-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/widget/WaveView;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getActiveAdmins-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ScrollingTabContainerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiController$EcmState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-performSnooze-(Landroid/net/NetworkTemplate; I)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/telecom/InCallService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/inputmethodservice/KeyboardView;-showPreview-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub;-addGpsNavigationMessageListener-(Landroid/location/IGpsNavigationMessageListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/FrameStore;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/backup/LocalTransportService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/BluetoothDevice;-createInsecureRfcommSocket-(I)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/am/ActivityManagerService;-makeAppCrashingLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getLastPdpFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onLocalDisconnect-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/NotificationManagerService;-onStart-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-isAdminActive-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockApplication;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/PhoneBase;-acquireOwnershipOfImsPhone-(Lcom/android/internal/telephony/imsphone/ImsPhone;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/SupplicantStateTracker$DisconnectedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManagerProxy;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Sms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Landroid/speech/tts/TextToSpeechService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/menu/IconMenuView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NsdService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/HandoverTransfer;-setBluetoothTransferId-(I)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/admin/IDevicePolicyManager;-isLockTaskPermitted-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/trust/TrustManagerService;-updateUserHasAuthenticated-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-getCDMASubscription-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-killBackgroundProcesses-(Ljava/lang/String; I)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/internal/telephony/Phone;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/ContextMenuBuilder;-changeMenuMode-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/Invert;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$BaseFutureTask;-doWork-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/inputmethodservice/InputMethodService;-onConfigurationChanged-(Landroid/content/res/Configuration;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/notification/RankingHelper;-<init>-(Landroid/content/Context; Landroid/os/Handler; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readAccessibilityEnabledSettingLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub$Stub;-getActiveSubInfoCount-()I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/provider/ContactsContract$Contacts;-getLookupUri-(J Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/widget/NumberPicker$CustomEditText;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/RemoteViewsAdapter$RemoteViewsFrameLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-reportActivityInfo-()Landroid/net/wifi/WifiActivityEnergyInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getAggressiveHandover-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/os/PowerManagerInternal;-setScreenBrightnessOverrideFromWindowManager-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService;-unregisterReceivers-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/SettingInjectorService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/DateTimeView;-onAttachedToWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/SmsUsageMonitor$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/NotificationManagerService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/CalendarContract$EventsEntity$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readComponentNamesFromSettingLocked-(Ljava/lang/String; I Ljava/util/Set;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/tv/TvInputService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/ActivityManager;-setPackageScreenCompatMode-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getWidgetsEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-clearPassword-(Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/test/mock/MockApplication;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowManagerService;-getInitialDisplayDensity-(I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothPan;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/VideoView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/search/SearchManagerService$GlobalSearchProviderObserver;-<init>-(Lcom/android/server/search/SearchManagerService; Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/DockObserver;-handleDockStateChange-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/pm/PackageManagerService;-scanPackageLI-(Ljava/io/File; I I J Landroid/os/UserHandle;)Landroid/content/pm/PackageParser$Package;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/GlobalActions;-showDialog-(Z Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/telecom/InCallService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/SupplicantStateTracker$CompletedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService;-access$1500-(Landroid/media/AudioService; I)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/power/PowerManagerService$UserSwitchedReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-fireDowntimeChanged-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms;-disableCellBroadcastRangeForSubscriber-(J I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/accessibility/TouchExplorer;-access$2400-(Lcom/android/server/accessibility/TouchExplorer; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcActiveState;-enter-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BluetoothManagerService;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/power/PowerManagerService;-access$2000-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SmsManager;-downloadMultimediaMessage-(Landroid/content/Context; Ljava/lang/String; Landroid/net/Uri; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.RECEIVE_MMS'],
	'Landroid/bluetooth/BluetoothDevice;-createScoSocket-()Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCellInfoListRate-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipCall;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/wm/AccessibilityController$WindowsForAccessibilityObserver;-<init>-(Lcom/android/server/wm/WindowManagerService; Landroid/view/WindowManagerInternal$WindowsForAccessibilityCallback;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/IWallpaperManager$Stub$Proxy;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/policy/impl/GlobalActions$SinglePressAction;-onPress-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/RotarySelector;-vibrate-(J)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/display/AutomaticBrightnessController;-access$000-(Lcom/android/server/display/AutomaticBrightnessController;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask$1;-run-(Landroid/accounts/AccountManagerFuture;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/wm/WindowStateAnimator;-createSurfaceLocked-()Landroid/view/SurfaceControl;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getBasebandVersion-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getPreferredVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothSocket;-initSocketFromFdNative-(I)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-broadcastIccStateChangedIntent-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager;-removeAccountAsUser-(Landroid/accounts/Account; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler; Landroid/os/UserHandle;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/print/IPrintManager$Stub;-getEnabledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/dreams/DreamManagerService$LocalService;-stopDream-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-processMessagePart-(Lcom/android/internal/telephony/InboundSmsTracker;)Z' : ['android.permission.RECEIVE_MMS', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/server/am/ActivityStack;-completeResumeLocked-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-access$300-(Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity; Landroid/net/wifi/WifiManager;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-startRecording-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/MediaFocusControl;-sendSimulatedMediaButtonEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/TouchExplorer;-handleMotionEventStateDelegating-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getLastDataCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CheckedTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-installCaCert-(Landroid/content/ComponentName; [B)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-addProximityAlert-(D D F J Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/LocationManagerService;-reportLocation-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService$3;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/NfcAdapter;-setNdefPushMessageCallback-(Landroid/nfc/NfcAdapter$CreateNdefMessageCallback; Landroid/app/Activity; [Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-handlePollCalls-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-getWindowListLocked-(Landroid/view/Display;)Lcom/android/server/wm/WindowList;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Landroid/location/Criteria; Landroid/location/LocationListener; Landroid/os/Looper;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-unregisterForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/UiModeManagerService;-onStart-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/CheckBox;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsUsageMonitor; Lcom/android/internal/telephony/ImsSMSDispatcher;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/pm/PackageManagerService;-isVerificationEnabled-(I I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-handleEnterEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/media/tv/TvView$3;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/ContactsContract$Contacts;-getLookupUri-(Landroid/content/ContentResolver; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/widget/MultiAutoCompleteTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/ConfirmConnectActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-access$3400-(Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine; Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/DialogTitle;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DateTimeView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-isAdminActive-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$UnavailableState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-onAllRecordsLoaded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer;-stayAwake-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-onRefresh-(Z [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/MountServiceIdler;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/ActivityGroup;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/SpeechRecognizer;-handleCancelMessage-()V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/ISub$Stub;-addSubInfoRecord-(Ljava/lang/String; I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/wm/WindowManagerService;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SmsMessage;-calculateLength-(Ljava/lang/String; Z)[I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/hardware/location/GeofenceHardwareService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/SlidingTab;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/RecognitionService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofencePauseStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayPowerController$1;-onAnimationEnd-(Landroid/animation/Animator;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getPowerButtonInstantlyLocks-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AutoCompleteTextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/NotificationManagerService$NotificationListeners;-disableNonexistentServices-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TabHost;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/ContentResolver;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/backup/IBackupManager$Stub;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag;-isNdef-(I)Z' : ['android.permission.NFC'],
	'Lcom/android/server/UiModeManagerService;-updateAfterBroadcastLocked-(Ljava/lang/String; I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-getTaskThumbnail-(I)Landroid/app/ActivityManager$TaskThumbnail;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$GroupCreatingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/AlarmManagerService;-sendNextAlarmClockChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService$LocalService;-requestTraversalFromDisplayManager-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/Service;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipService;-updateWakeLocks-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-dialInternal-(Lcom/android/internal/telephony/imsphone/ImsPhoneConnection; I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/MountServiceIdler;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/snep/SnepServer$Callback;-doGet-(I Landroid/nfc/NdefMessage;)Lcom/android/nfc/snep/SnepMessage;' : ['android.permission.BLUETOOTH'],
	'Landroid/content/IContentService$Stub$Proxy;-getSyncStatusAsUser-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName; I)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/UiccSmsController;-sendTextForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/Session;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech;-shutdown-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CommandsInterface;-getDeviceIdentity-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager;-addPrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$12;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setupData-(Lcom/android/internal/telephony/dataconnection/ApnContext; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/print/PrinterDiscoverySession;-destroy-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockContentResolver;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/ISub;-getSubInfoUsingIccId-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/content/ContentResolver;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/location/ILocationManager$Stub;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-transitionToDelegatingStateAndClear-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/DeviceDiscoveryAction;-wrapUpAndFinish-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/IConsumerIrService$Stub$Proxy;-transmit-(Ljava/lang/String; I [I)V' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/server/input/InputManagerService$7;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NetworkTimeUpdateService;-access$300-(Lcom/android/server/NetworkTimeUpdateService; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/cdma/sms/BearerData;-calcTextEncodingDetails-(Ljava/lang/CharSequence; Z)Lcom/android/internal/telephony/GsmAlphabet$TextEncodingDetails;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/WebVttRenderingWidget;-setVisible-(Z)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/NsdService;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Landroid/net/wifi/WifiManager;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings; Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/power/PowerManagerService$DockReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-setForegroundDispatch-(Landroid/app/PendingIntent; [Landroid/content/IntentFilter; Landroid/nfc/TechListParcel;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-makeCellBroadcastHandler-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)Lcom/android/internal/telephony/CellBroadcastHandler;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiServiceImpl;-enableVerboseLogging-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-startDataStallAlarm-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextClock;-is24HourModeEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/State;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/Phone;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-requestTransientBars-(Landroid/view/WindowManagerPolicy$WindowState;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-deleteGallery-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothPan;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/WifiManager;-startScan-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService;-getSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Lcom/android/server/backup/BackupManagerService$PerformBackupTask;-handleTimeout-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowAnimator;-testTokenMayBeDrawnLocked-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-isDataAllowed-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/WifiApConfigStore$DefaultState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/power/Notifier$2;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/MediaScanner$MyMediaScannerClient;-setSettingIfNotSet-(Ljava/lang/String; Landroid/net/Uri; J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-access$600-(Lcom/android/server/wifi/WifiWatchdogStateMachine;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/PointerLocationView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/FullBackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/IMediaHTTPConnection;-getSize-()J' : ['android.permission.INTERNET'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService$FullBackupEngine;-backupOnePackage-(Landroid/content/pm/PackageInfo;)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/ListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-openPopupIfRequired-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/app/ActivityGroup;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ims/ImsCall$Listener;-onCallHeld-(Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccPhoneBookInterfaceManager;-updateAdnRecordsInEfByIndex-(I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Landroid/app/Presentation$1;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPasswordMinimumNonLetter-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub$Proxy;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/ISms;-copyMessageToIccEfForSubscriber-(J Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine;-retrieveBatchedScanData-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockContentResolver;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeBarringPassword-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/notification/ConditionProviderService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getVoiceRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-onDisconnect-(I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/statusbar/StatusBarManagerService;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/test/InstrumentationCoreTestRunner;-getUiAutomation-()Landroid/app/UiAutomation;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/BitmapSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-access$700-(Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/content/IContentService$Stub;-setMasterSyncAutomaticallyAsUser-(Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-access$3800-(Lcom/android/server/power/PowerManagerService; J I I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-scheduleReadyPeriodicSyncs-()J' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Landroid/inputmethodservice/KeyboardView;-setKeyboard-(Landroid/inputmethodservice/Keyboard;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/app/job/JobService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/NullFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiSettingsStore;-getPersistedAirplaneModeOn-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/media/MediaSessionService$SettingsObserver;-<init>-(Lcom/android/server/media/MediaSessionService;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneFactory;-getDataSubscription-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordExpirationTimeout-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-setProcessLimit-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/WebVttTrack;-hide-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/sip/ISipService$Stub;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/sip/SipPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$InitialState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcApplication;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextClock;-chooseFormat-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-queryFacilityLockForApp-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/RttService$RttServiceImpl$RttStateMachine$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityStack;-startPausingLocked-(Z Z Z Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/effect/effects/BackDropperEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService$Receiver;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConnectivityService$SettingsObserver;-observe-(Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-writeSmsToRuim-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/media/MediaSessionService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getNeighboringCids-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/UiAutomationConnection;-storeRotationStateLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService$ApplyRoutingTask;-doInBackground-([Ljava/lang/Integer;)Ljava/lang/Void;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/widget/ToggleButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getVerboseLoggingLevel-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/ZoomButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ActionBarView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/ZenModeHelper;-shouldIntercept-(Lcom/android/server/notification/NotificationRecord;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SubscriptionManager;-getDefaultVoiceSubInfo-()Landroid/telephony/SubInfoRecord;' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-loadEfLiAndEfPl-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getProfileOwnerAdmin-(I)Lcom/android/server/devicepolicy/DevicePolicyManagerService$ActiveAdmin;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LockSettingsService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BluetoothManagerService;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/notification/CountdownConditionProvider;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/pm/IPackageManager;-setInstallLocation-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/display/DisplayPowerController;-access$400-(Lcom/android/server/display/DisplayPowerController;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-setAutoPersisting-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SMS'],
	'Landroid/content/MutableContextWrapper;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowStateAnimator;-stepAnimationLocked-(J)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/IntentService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/printservice/PrintService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/net/NetworkStatsService;-updatePersistThresholds-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/InputMonitor;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-isShowImeWithHardKeyboardEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/statusbar/IStatusBarService;-collapsePanels-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getAdminWithMinimumFailedPasswordsForWipeLocked-(I)Lcom/android/server/devicepolicy/DevicePolicyManagerService$ActiveAdmin;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/ToUpperCase;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/TouchExplorer;-clear-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/TouchExplorer$SendHoverEnterAndMoveDelayed;-forceSendAndRemove-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/tts/TextToSpeech;-getFeatures-(Ljava/util/Locale;)Ljava/util/Set;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget$RegionLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$IdleState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-setAccountManagementDisabled-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-onDisplayAdded-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService$14;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/mock/MockContentResolver;-getCurrentSyncsAsUser-(I)Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/wm/WindowManagerService;-shutdown-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/AccessibilityController$DisplayMagnifier;-onWindowLayersChangedLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IMms$Stub;-importMultimediaMessage-(Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; J Z Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getUidPersistBytes-(J)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/UiModeManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/PacManager;-access$200-(Landroid/net/Uri;)Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/RotateFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/BootReceiver$1;-run-()V' : ['android.permission.INTERNET'],
	'Landroid/widget/SearchView$SearchAutoComplete;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/CalendarContract$Reminders;-query-(Landroid/content/ContentResolver; J [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onError-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPasswordQuality-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/PacManager;-reschedule-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/os/IVibratorService$Stub;-vibratePattern-(I Ljava/lang/String; [J I I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/updates/SmsShortCodesInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setCameraDisabled-(Landroid/content/ComponentName; Z I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/text/method/TextKeyListener;-updatePrefs-(Landroid/content/ContentResolver;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Browser;-clearSearches-(Landroid/content/ContentResolver;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS'],
	'Lcom/android/server/UiModeManagerService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IBackupAgent;-doFullBackup-(Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-setTemporaryScreenBrightnessSettingOverrideInternal-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-updateSmsSendStatus-(I Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/backup/PackageManagerBackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-deleteStoredMessage-(Ljava/lang/String; Landroid/net/Uri;)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-getListOfProfiles-()[Landroid/net/sip/SipProfile;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCallForwardingOption-(I I Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-hasUserSetupCompleted-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-clearProfileOwner-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/usb/UsbService$Lifecycle;-onBootPhase-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$3;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-doRestoreFinished-(I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/IAudioService$Stub$Proxy;-reloadAudioSettings-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NfcB;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-update-(Lcom/android/internal/telephony/uicc/IccCardApplicationStatus; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/backup/BackupManagerService$PerformUnifiedRestoreTask;-finalizeRestore-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiApConfigStore;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IntentService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/SyncActivityTooManyDeletes;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/backup/BackupManagerService;-opComplete-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/trust/TrustManagerService;-access$1200-(Lcom/android/server/trust/TrustManagerService; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/Editor$SelectionEndHandleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onRegistering-(Landroid/net/sip/ISipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$SoftApStartedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-getChannelList-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wifi/SupplicantStateTracker$ScanState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Landroid/nfc/tech/NdefFormatable;-format-(Landroid/nfc/NdefMessage;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager;-destroyPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager;-onUserSwitched-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/internal/widget/LockPatternUtils;-setBiometricWeakLivelinessEnabled-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-startProvisioningApnAlarm-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-disable-(Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/widget/TabHost;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/NativeCrashListener$NativeCrashReporter;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-getPrintJobInfo-(Landroid/print/PrintJobId; I I)Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/server/wifi/WifiController;-access$100-(Lcom/android/server/wifi/WifiController;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/os/NullVibrator;-vibrate-([J I Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-startPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Spinner;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterfw/core/InputPort;-transfer-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/OutputStreamTarget;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/ContactsContract$Data;-getContactLookupUri-(Landroid/content/ContentResolver; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/widget/TextViewWithCircularIndicator;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/ResolverDrawerLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-removeNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-isApplicationHidden-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Ljava/net/NetworkInterface;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/location/GpsNetInitiatedHandler;-<init>-(Landroid/content/Context; Landroid/location/INetInitiatedListener; Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/DropBoxManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-isBiometricWeakEverChosen-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getGroupIdLevel1ForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityStackSupervisor;-acquireLaunchWakelock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-registerListener-(Ljava/lang/String; Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/telecom/ConnectionService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/input/InputManagerService;-getPointerSpeedSetting-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCall;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-dialInternal-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/SyncRunner;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/media/MediaSessionService$SettingsObserver;-<init>-(Lcom/android/server/media/MediaSessionService; Lcom/android/server/media/MediaSessionService$1;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-closeActiveSyncContext-(Lcom/android/server/content/SyncManager$ActiveSyncContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/IAudioService;-setBluetoothScoOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/UiccCard;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/uicc/IccCardStatus;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-skipCurrentReceiverLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-resumeGeofence-(I I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Ljava/net/DatagramSocket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Landroid/widget/Toolbar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/sip/SipManager;-takeAudioCall-(Landroid/content/Intent; Landroid/net/sip/SipAudioCall$Listener;)Landroid/net/sip/SipAudioCall;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-getDefaultSmsSubId-()J' : ['android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/KeyboardView;-onLongPress-(Landroid/inputmethodservice/Keyboard$Key;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/widget/ScrollView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-isRegistered-(Ljava/lang/String;)Z' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-updateCarrierMccMncConfiguration-(Ljava/lang/String; Ljava/lang/String; Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/sip/SipAudioCall$1;-onCallEnded-(Landroid/net/sip/SipSession;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-onRegistrationDone-(Landroid/net/sip/ISipSession; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaPlayer;-selectTrack-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/notification/NotificationManagerService;-access$3400-(Lcom/android/server/notification/NotificationManagerService; Lcom/android/server/notification/NotificationRecord;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/AsyncService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/net/NetworkStatsService;-access$600-(Lcom/android/server/net/NetworkStatsService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothA2dpSink;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/nfc/INfcTag;-setTimeout-(I I)I' : ['android.permission.NFC'],
	'Lcom/android/server/power/PowerManagerService;-access$4700-(Lcom/android/server/power/PowerManagerService; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContentResolver;-requestSync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-makeWifiWatchdogStateMachine-(Landroid/content/Context; Landroid/os/Messenger;)Lcom/android/server/wifi/WifiWatchdogStateMachine;' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/widget/RemoteViews$2;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-completeConnection-(Lcom/android/internal/telephony/dataconnection/ApnContext;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-setWifiApState-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/IConsumerIrService$Stub;-transmit-(Ljava/lang/String; I [I)V' : ['android.permission.TRANSMIT_IR'],
	'Landroid/app/ActivityManager;-moveTaskToFront-(I I)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/filterpacks/base/InputStreamSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccCard;-changeIccLockPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/NetworkScorerAppManager;-getActiveScorer-(Landroid/content/Context;)Landroid/net/NetworkScorerAppManager$NetworkScorerAppData;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setMasterVolumeMuted-(Landroid/content/ComponentName; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Application;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/hdmi/NewDeviceAction;-addDeviceInfo-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationBlacklist;-reloadBlacklist-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipAudioCall;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SubscriptionManager;-getDefaultVoiceSubId-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/twilight/TwilightService$1;-registerListener-(Lcom/android/server/twilight/TwilightListener; Landroid/os/Handler;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneProxy;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/SamplingProfilerService$SamplingProfilerSettingsObserver;-<init>-(Lcom/android/server/SamplingProfilerService; Landroid/content/ContentResolver;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$800-(Lcom/android/server/wifi/WifiStateMachine; Landroid/os/Message; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCallWaiting-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContentResolver;-cancelSync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-getDefaultDataSubId-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-hangupConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-updateNotificationsLocked-()V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/view/accessibility/CaptioningManager;-getRawUserStyle-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPin-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$ProvisionDiscoveryState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifyLlcpLinkActivation-(Lcom/android/nfc/dhimpl/NativeP2pDevice;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-sendP2pConnectionChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/text/StringSource;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/os/NullVibrator;-vibrate-(I Ljava/lang/String; J Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Landroid/app/IUiAutomationConnection$Stub$Proxy;-disconnect-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/SubtitleTrack;-show-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getStorageEncryption-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/test/RenamingDelegatingContext;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/dreams/DreamManagerService;-startDreamInternal-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcCardEmulation$Stub$Proxy;-isDefaultServiceForCategory-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/content/IContentService$Stub$Proxy;-setIsSyncable-(Landroid/accounts/Account; Ljava/lang/String; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-updateSettings-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SmsManager;-addTextMessageDraft-(Ljava/lang/String; Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/widget/DatePickerCalendarDelegate;-updateDisplay-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-getDataConnectionState-(Ljava/lang/String;)Lcom/android/internal/telephony/PhoneConstants$DataState;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/InputStreamSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/RIL;-startDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$DefaultState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-setAppFullscreen-(Landroid/os/IBinder; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/backup/BackupManagerService$PerformUnifiedRestoreTask;-execute-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-dial-(Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipCall;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DateTimeView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/nsd/INsdManager$Stub$Proxy;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-updateNetworkState-(I Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-access$2300-(Lcom/android/server/location/GpsLocationProvider; I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-decrementWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/updates/SmsShortCodesInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/MediaPlayer;-addTimedTextSource-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$18;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-enableSystemAppWithIntent-(Landroid/content/ComponentName; Landroid/content/Intent;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/display/DisplayManagerService;-access$3500-(Lcom/android/server/display/DisplayManagerService; Landroid/os/IBinder;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/SystemService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/content/SyncActivityTooManyDeletes;-startSyncUndoDeletes-()V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-installCaCert-(Landroid/content/ComponentName; [B)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/BeamShareActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/AdapterViewAnimator;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/CertBlacklister$BlacklistObserver;-writeBlacklist-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-resetPassword-(Ljava/lang/String; I I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/HdmiCecController$AllocateAddressCallback;-onAllocated-(I I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/Context;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$UnavailableState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-addNetworkPolicyLocked-(Landroid/net/NetworkPolicy;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/notification/NotificationManagerService$NotificationListeners;-disableNonexistentServices-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ConditionProviders;-onPackagesChanged-(Z [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController$DefaultState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/os/PowerManager$WakeLock;-acquireLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/dreams/DreamController$Listener;-onDreamStopped-(Landroid/os/Binder;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AutoCompleteTextView;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-handlePreDhcpSetup-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/inputmethodservice/ExtractButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TimePicker;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubInfoRecordUpdater;-setUpdatedData-(I I I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/TelephonyManager;-getIntAtIndex-(Landroid/content/ContentResolver; Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/MediaSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/am/ActivityStack;-clearLaunchTime-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$OnlineWatchState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-setPostDialState-(Lcom/android/internal/telephony/Connection$PostDialState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/SlidingDrawer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-access$5300-(Lcom/android/server/power/PowerManagerService; J)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothAdapter;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/speech/tts/TextToSpeech;-getVoice-()Landroid/speech/tts/Voice;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCLIR-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation;-setDefaultForNextTap-(I Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/cardemulation/CardEmulation;-getSelectionModeForCategory-(Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/GlobalActions;-handleShow-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/DisplayContent;-checkForDeferredActions-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IntRangeManager;-tryAddRanges-(I I Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController$EcmState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RemoteViews$2;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/net/NetworkStatsService;-removeUidsLocked-([I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager;-getBaseDisplayDensity-(I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/IEthernetManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/net/NetworkPolicyManager;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$800-(Lcom/android/server/accessibility/AccessibilityManagerService; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/IUiAutomationConnection$Stub;-connect-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-performBootDexOpt-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/ListActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/nfc/INfcTag$Stub;-transceive-(I [B Z)Landroid/nfc/TransceiveResult;' : ['android.permission.NFC'],
	'Landroid/test/LaunchPerformanceBase;-getUiAutomation-()Landroid/app/UiAutomation;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/content/SyncManager;-<init>-(Landroid/content/Context; Z)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.GET_ACCOUNTS'],
	'Landroid/media/MediaFocusControl;-filterVoiceInputKeyEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/SearchView$SearchAutoComplete;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getImeiForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Service;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/ISub;-getDefaultDataSubId-()J' : ['android.permission.READ_PROFILE'],
	'Ljava/net/HttpURLConnection;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onDataStallAlarm-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayManagerService;-releaseVirtualDisplayInternal-(Landroid/os/IBinder;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPasswordMinimumNonLetter-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/TelephonyManager;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-setRestrictionsProvider-(Landroid/content/ComponentName; Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/TapAgainDialog;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CdmaBroadcastRangeManager;-finishUpdate-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/PrintManager;-print-(Ljava/lang/String; Landroid/print/PrintDocumentAdapter; Landroid/print/PrintAttributes;)Landroid/print/PrintJob;' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/CalendarContract$Attendees;-query-(Landroid/content/ContentResolver; J [Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/server/am/BatteryStatsService;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/os/IDropBoxManagerService;-add-(Landroid/os/DropBoxManager$Entry;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/DisableCarModeActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-access$7100-(Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneFactory;-makePhone-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier; Lcom/android/internal/telephony/Phone;)Lcom/android/internal/telephony/imsphone/ImsPhone;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/ToUpperCase;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/SyncActivityTooManyDeletes;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IccProvider;-addIccRecordToEf-(I Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; J)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/IMms$Stub;-sendStoredMessage-(J Ljava/lang/String; Landroid/net/Uri; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-reevaluateDowntime-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Landroid/test/IsolatedContext;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingScoOn-()Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-lockNow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IntRangeManager;-updateRanges-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$EnableDisableTask;-disableInternal-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/view/WindowManagerPolicy;-checkAddPermission-(Landroid/view/WindowManager$LayoutParams; [I)I' : ['android.permission.SYSTEM_ALERT_WINDOW'],
	'Lcom/android/internal/telephony/CallManager;-hangupForegroundResumeBackground-(Lcom/android/internal/telephony/Call;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController;-canEnableAccessibilityViaGesture-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SmsManager;-updateMessageOnIcc-(I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/print/UserState;-<init>-(Landroid/content/Context; I Ljava/lang/Object;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiController$3;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-fetchSimRecords-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Connection;-proceedAfterWaitChar-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/CalendarContract$EventsEntity$EntityIteratorImpl;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/CommandsInterface;-switchWaitingOrHoldingAndActive-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-retrieveSettings-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$4;-onCallResumed-(Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/textservice/TextServicesManager;-getCurrentSpellCheckerSubtype-(Z)Landroid/view/textservice/SpellCheckerSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/GLTextureSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-addGpsStatusListener-(Landroid/location/IGpsStatusListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/net/NetworkStatsService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-computeNewConfigurationLocked-()Landroid/content/res/Configuration;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/LockPatternUtils;-isLockPatternEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService$InstallArgs;-checkFreeStorage-(Lcom/android/internal/app/IMediaContainerService;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-teardownForDun-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordExpiration-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-restartPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$Secure;-getUriFor-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$3;-onCallHoldFailed-(Lcom/android/ims/ImsCall; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/MultiAutoCompleteTextView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/effect/effects/PosterizeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/UiModeManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/ChooseAccountActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiStateMachine$SupplicantStartingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/telephony/SubscriptionManager;-setDataRoaming-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/performance/ThroughputFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$11200-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/job/JobService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-clearDefaultsForInactiveSubIds-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/service/dreams/DreamService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/dreams/DreamManagerService;-getDozeComponent-()Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-deleteSmsOnRuim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getPrivilegedConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-enableVerboseLogging-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-deleteSmsOnRuim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/usb/UsbHostManager;-usbDeviceRemoved-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/accounts/AccountManagerService;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/power/PowerManagerService$LocalService;-setDozeOverrideFromDreamManager-(I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-sendCdmaSms-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/service/voice/VoiceInteractionService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/appwidget/AppWidgetHostView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-closeSystemDialogsLocked-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/StackView$StackFrame;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-sendMessage-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/CaptioningManager;-access$500-(Landroid/view/accessibility/CaptioningManager;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/FieldPort;-setFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/ForegroundUtils$Callback;-onUidToBackground-(I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/content/SyncManager;-access$1100-(Lcom/android/server/content/SyncManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/QuickContactBadge;-assignContactFromPhone-(Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/app/Activity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/content/SyncStorageEngine;-newTestInstance-(Landroid/content/Context;)Lcom/android/server/content/SyncStorageEngine;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController;-registerForStayAwakeModeChange-(Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ExpandableListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/MountService;-onDaemonConnected-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService$SettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/InboundSmsHandler$WaitingState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/MediaRouteButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/ITextToSpeechService;-synthesizeToFileDescriptor-(Landroid/os/IBinder; Ljava/lang/CharSequence; Landroid/os/ParcelFileDescriptor; Landroid/os/Bundle; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-reassociate-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/Phone;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-removeActiveAdmin-(Landroid/content/ComponentName; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPasswordQuality-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getNetworkSelectionMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPasswordMinimumLength-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NfcA;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/server/content/SyncManager;-scheduleSync-(Landroid/accounts/Account; I I Ljava/lang/String; Landroid/os/Bundle; J J Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/content/ContentService;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/service/notification/NotificationListenerService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/LocationManager;-removeAllGeofences-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/CallTracker;-handlePollCalls-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/ResolverActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/TextClock;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UiccCard;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/uicc/IccCardStatus; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-sendP2pDiscoveryChangedBroadcast-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-fixTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/view/IInputMethodManager;-getShortcutInputMethodsAndSubtypes-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/FullBackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-revertToNitzTime-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-disableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/SubscriptionController;-getDefaultDataSubId-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneFactory;-makePhone-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)Lcom/android/internal/telephony/sip/SipPhone;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-clearCrossProfileIntentFilters-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setActivePasswordState-(I I I I I I I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms;-enableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/backup/SystemBackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPermittedAccessibilityServices-(Landroid/content/ComponentName; Ljava/util/List;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/BeamShareActivity$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager$CellBroadcastRangeManager;-finishUpdate-()Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-access$1300-(Lcom/android/internal/policy/impl/PhoneWindowManager;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TelephonyRegistry;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/FieldPort;-pushFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/PhoneNumberUtils;-isLocalEmergencyNumber-(Landroid/content/Context; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-performBootDexOpt-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils;-setNonSelectedSystemImesDisabledUntilUsed-(Landroid/content/pm/PackageManager; Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-setDataAllowed-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-setCellBroadcastConfig-([Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-access$900-(Lcom/android/internal/telephony/InboundSmsHandler; Landroid/content/Intent;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS'],
	'Lcom/squareup/okhttp/internal/http/DelegatingHttpsURLConnection;-connect-()V' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-disableGsmBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/pm/PackageManagerService;-access$1400-(Lcom/android/server/pm/PackageManagerService;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService$3;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/effect/effects/RotateEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LockSettingsService;-migrateOldData-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPasswordMinimumUpperCase-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ZenModeHelper;-<init>-(Landroid/content/Context; Landroid/os/Handler;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onRadioOffOrNotAvailable-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-updateRotationUncheckedLocked-(Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-packageHasActiveAdmins-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofenceMonitorStatus-(I I Landroid/location/Location;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService$Session;-run-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/ContentService;-getIsSyncableAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/dreams/DreamManagerService;-access$1800-(Lcom/android/server/dreams/DreamManagerService; Landroid/os/IBinder; I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/VideoView;-pause-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$10300-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$ActedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/ims/internal/IImsEcbmListener$Stub;-enteredECBM-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISub;-setDisplayName-(Ljava/lang/String; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-registerForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/notification/NotificationManagerService;-handleRankingReconsideration-(Landroid/os/Message;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/PhoneFactory;-makeDefaultPhone-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-getWindowListLocked-(I)Lcom/android/server/wm/WindowList;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService$LocalService;-wakingUp-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipAudioCall$1;-onCallEstablished-(Landroid/net/sip/SipSession; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/IBackupAgent$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/inputmethodservice/ExtractEditText;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getDataConnectionState-()Lcom/android/internal/telephony/PhoneConstants$DataState;' : ['android.permission.READ_PROFILE'],
	'Landroid/accessibilityservice/AccessibilityService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-acceptCall-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-updateAdnRecordsInEfBySearchForSubscriber-(J I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/app/IActivityManager;-killApplicationWithAppId-(Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/ClosedCaptionWidget$CCLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager;-setProfileEnabled-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SubscriptionManager;-getActiveSubInfoList-()Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-killPackageProcessesLocked-(Ljava/lang/String; I I I Z Z Z Z Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/cardemulation/CardEmulation;-isDefaultServiceForCategory-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/filterpacks/videosrc/MediaSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-requestSync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/hardware/location/IGeofenceHardware;-pauseGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/WifiController;-readWifiIdleTime-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-getEnabledInputMethodList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/Invert;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/CellBroadcastHandler;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-startBluetoothSco-(Landroid/os/IBinder; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/BluetoothManagerService;-isBluetoothPersistedStateOnBluetooth-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/MifareClassic;-getTimeout-()I' : ['android.permission.NFC'],
	'Lcom/android/server/wm/WindowAnimator;-dumpLocked-(Ljava/io/PrintWriter; Ljava/lang/String; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/DatePicker;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub;-setAppVisibility-(Landroid/os/IBinder; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiScanningServiceImpl$WifiScanningStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService;-isSyncPendingAsUser-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName; I)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-isUninstallBlocked-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onSetUserDataEnabled-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-setAutoPersisting-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SMS'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-reportVisibility-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-getDefaultDisplayInfoLocked-()Landroid/view/DisplayInfo;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/ScreenMagnifier$ScreenStateObserver;-access$2900-(Lcom/android/server/accessibility/ScreenMagnifier$ScreenStateObserver; Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/AsyncService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-iccCloseLogicalChannel-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/RegisteredServicesCache;-registerAidGroupForService-(I I Landroid/content/ComponentName; Landroid/nfc/cardemulation/AidGroup;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-handleCallSetupRequestFromSim-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker$3;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/provider/Settings$Secure;-getFloatForUser-(Landroid/content/ContentResolver; Ljava/lang/String; F I)F' : ['android.permission.READ_PROFILE'],
	'Landroid/preference/PreferenceFrameLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaScanner;-scanSingleFile-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/Filter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager;-clearPackagePersistentPreferredActivities-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onError-(Landroid/net/sip/SipAudioCall; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService$LocalService;-setMagnificationSpec-(Landroid/view/MagnificationSpec;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/tv/TvInputService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/tv/TvInputInfo$TvInputSettings;-getCustomLabel-(Landroid/content/Context; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/AnalogClock;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-hasUserSetupCompleted-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiApConfigStore;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager;-getCurrentInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/Filter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ShutdownActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/location/GeofenceHardwareService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/AutoCompleteTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/RegisteredAidCache;-onPreferredForegroundServiceChanged-(Landroid/content/ComponentName;)V' : ['android.permission.VIBRATE'],
	'Landroid/view/WindowManagerPolicy;-setSafeMode-(Z)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-access$1000-(Lcom/android/server/wm/WindowManagerService; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-addCrossProfileIntentFilter-(Landroid/content/ComponentName; Landroid/content/IntentFilter; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-enableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Landroid/speech/tts/TextToSpeech;-getDefaultLanguage-()Ljava/util/Locale;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/VideoView;-setVideoURI-(Landroid/net/Uri;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/SupplicantStateTracker$InactiveState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/media/MediaSessionService;-isEnabledNotificationListener-(Landroid/content/ComponentName; I I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-prepareAppTransition-(I Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/VibratorService;-access$1400-(Lcom/android/server/VibratorService; J I I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/display/OverlayDisplayAdapter$1$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/CropEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager;-statusBarVisibilityChanged-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/tech/NfcF;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/server/usb/UsbSettingsManager$MyPackageMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-setPostDialState-(Lcom/android/internal/telephony/Connection$PostDialState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-isAllowedByUserSettingsLocked-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getDataConnectionState-(Ljava/lang/String;)Lcom/android/internal/telephony/PhoneConstants$DataState;' : ['android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/AccessibilityNodeInfo;-getWindow-()Landroid/view/accessibility/AccessibilityWindowInfo;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/ScreenMagnifier$StateViewportDraggingHandler;-onMotionEvent-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/net/wifi/IWifiManager$Stub;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/backup/BackupManagerService$Lifecycle;-onBootPhase-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/SettingInjectorService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/effect/effects/GrainEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/SlidingDrawer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaHTTPConnection;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection;-dial-()V' : ['android.permission.USE_SIP'],
	'Lcom/android/server/ConsumerIrService;-transmit-(Ljava/lang/String; I [I)V' : ['android.permission.TRANSMIT_IR'],
	'Landroid/widget/QuickContactBadge;-assignContactFromPhone-(Ljava/lang/String; Z)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/RadioButton;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifyDataConnection-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/content/IntentSender$FinishedDispatcher;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/IWindowManager$Stub;-startFreezingScreen-(I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-addTextMessageDraft-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/NsdService$NsdStateMachine;-registerForNsdSetting-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/IccCard;-supplyPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$3;-onCallProgressing-(Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-getInt-(Landroid/content/ContentResolver; Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/telecom/ConnectionService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/backup/BackupManagerService$PerformBackupTask;-invokeNextAgent-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/IWallpaperManager;-setDisplayPadding-(Landroid/graphics/Rect;)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Landroid/widget/AbsSpinner;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-archiveStoredConversation-(Ljava/lang/String; J Z)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/server/backup/SystemBackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/RemoteViewsService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/DialogTitle;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onLlcpLinkDeactivated-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/usage/UsageStatsService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/NotificationManagerService;-buzzBeepBlinkLocked-(Lcom/android/server/notification/NotificationRecord;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/cardemulation/OffHostApduService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$7100-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/ViewGroup;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker$AccessibilityNodeProviderImpl;-sendAccessibilityEventForVirtualView-(I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/MifareClassic;-authenticateSectorWithKeyB-(I [B)Z' : ['android.permission.NFC'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-stopLocationBasedDetector-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/MediaPlayer;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AsyncPlayer;-enqueueLocked-(Landroid/media/AsyncPlayer$Command;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-getTimeout-(I)I' : ['android.permission.NFC'],
	'Landroid/view/IWindowSession$Stub;-addToDisplay-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/app/IntentForwarderActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/ims/internal/IImsEcbmListener$Stub$Proxy;-exitedECBM-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/SurfaceView;-updateWindow-(Z Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState;-requestDataIdle-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AppSecurityPermissions$PermissionItemView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-deletePackageX-(Ljava/lang/String; I I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/squareup/okhttp/HttpResponseCache;-put-(Ljava/net/URI; Ljava/net/URLConnection;)Ljava/net/CacheRequest;' : ['android.permission.INTERNET'],
	'Landroid/service/media/MediaBrowserService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-handleDisplayAdded-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onRoamingOn-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onCallBusy-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/RetargetFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/ChooseAccountActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/FinalPort;-transfer-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActiveServices$ServiceRestarter;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getLine1Number-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/pm/PackageManagerService;-getVerificationTimeout-()J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/GLTextureTarget;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/sip/SipPhone;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/StringLogger;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onLongPress-(Landroid/inputmethodservice/Keyboard$Key;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/DialogTitle;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/connectivity/DataConnectionStats;-startMonitoring-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ISub;-setDisplayNameUsingSrc-(Ljava/lang/String; J J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/sip/SipService$ConnectivityReceiver$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-resumeGeofence-(I I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/accounts/AccountManager$6;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/Gallery;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/NfcBackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getRemoveWarning-(Landroid/content/ComponentName; Landroid/os/RemoteCallback; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiSettingsStore;-isAirplaneSensitive-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub;-getInitialDisplaySize-(I Landroid/graphics/Point;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/CountdownConditionProvider;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ims/ImsCall$Listener;-onCallMerged-(Lcom/android/ims/ImsCall; Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-onAllRecordsLoaded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/WifiManager;-getWifiState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/WallpaperManager;-setDisplayPadding-(Landroid/graphics/Rect;)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Landroid/app/IntentService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPasswordHistoryLength-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-isActivePasswordSufficient-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getPasswordExpiration-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-installKeyPair-(Landroid/content/ComponentName; [B [B Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-sendUssdResponse-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/server/LocationManagerService$Receiver;-binderDied-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/content/Context;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/VideoView$7;-surfaceChanged-(Landroid/view/SurfaceHolder; I I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContentResolver;-getMasterSyncAutomaticallyAsUser-(I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/content/SyncManager;-sendSyncFinishedOrCanceledMessage-(Lcom/android/server/content/SyncManager$ActiveSyncContext; Landroid/content/SyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/Button;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase$DataRoamingSettingObserver;-register-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BootReceiver;-addAuditErrorsToDropBox-(Landroid/os/DropBoxManager; Landroid/content/SharedPreferences; Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-access$600-(Lcom/android/internal/telephony/RIL; Landroid/os/Parcel;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/StackView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/restrictions/RestrictionsManagerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/pm/PackageManagerService;-updateExternalMediaStatusInner-(Z Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/base/GLTextureTarget;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccounts-(I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendTextForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-enable-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-updateDisplayColorAdjustmentSettingsLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub;-stopPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService$LocalService;-goingToSleep-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/service/media/MediaBrowserService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService$Stub$Proxy;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-cancelActiveSyncLocked-(Lcom/android/server/content/SyncStorageEngine$EndPoint; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-acknowledgeLastIncomingGsmSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-isAdminActive-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-getDefaultVerificationResponse-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/NfcAdapter;-invokeBeam-(Landroid/nfc/BeamShareData;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wm/WindowAnimator;-updateAppWindowsLocked-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/power/Notifier$1;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/AudioManager;-setParameter-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/AlarmManagerService;-setWakelockWorkSource-(Landroid/app/PendingIntent; Landroid/os/WorkSource; I Ljava/lang/String; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService$29;-performReceive-(Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle; Z Z I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/TintFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-hasFeatures-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/nfc/handover/HandoverService;-handleTransferEvent-(Landroid/content/Intent; I)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/ChooseAccountTypeActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-sendPendingBroadcastsLocked-(Lcom/android/server/am/ProcessRecord;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/effect/SingleFilterEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-isPortableHotspotSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/sip/SipAudioCall;-startAudio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/DefaultRemovedActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover;-getProfileProxys-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wifi/SupplicantStateTracker$ScanState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-sendNewConfiguration-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-checkMtFindNewRinging-(Lcom/android/internal/telephony/DriverCall; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IMms;-addMultimediaMessageDraft-(Ljava/lang/String; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/app/backup/BackupAgentHelper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/hdmi/SystemAudioAutoInitiationAction;-handleSystemAudioModeStatusTimeout-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-access$3200-(Landroid/media/AudioService;)Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/DockObserver;-access$400-(Lcom/android/server/DockObserver; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/SystemServer;-startBootstrapServices-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/SwipeDismissLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService$BinderService;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; Landroid/os/WorkSource; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/voice/VoiceInteractionSessionService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/net/NetworkStatsService;-performPoll-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/speech/IRecognitionService;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/widget/LockPatternUtils;-getLong-(Ljava/lang/String; J)J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/PhoneBase;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaBroadcastConfig-([Lcom/android/internal/telephony/cdma/CdmaSmsBroadcastConfigInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService$Lifecycle;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DcController$DccDefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-editProperties-(Ljava/lang/String; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-handleGsmBroadcastSms-(Landroid/os/AsyncResult;)Landroid/telephony/SmsCbMessage;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-isScreenMagnificationAutoUpdateEnabled-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/printservice/PrintService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/wallpaper/WallpaperService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISub$Stub;-setDefaultDataSubId-(J)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/net/INetworkScoreService$Stub$Proxy;-updateScores-([Landroid/net/ScoredNetwork;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-cancelPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/SyncRunner;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$5900-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Chronometer;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/PreferenceImageView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-gotoIdleAndNotifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-reportStkServiceIsRunning-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeechService$SpeechItemV1;-getSpeechRate-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/RotateFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/IAccessibilityManager$Stub;-temporaryEnableAccessibilityStateUntilKeyguardRemoved-(Landroid/content/ComponentName; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/test/RenamingDelegatingContext;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/uiautomator/Launcher;-main-([Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/pm/PackageManagerService;-scanPackageDirtyLI-(Landroid/content/pm/PackageParser$Package; I I J Landroid/os/UserHandle;)Landroid/content/pm/PackageParser$Package;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterfw/core/ProgramPort;-setFieldFrame-(Landroid/filterfw/core/Frame; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$2100-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiScanningServiceImpl$WifiScanningStateMachine$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISub;-setDataRoaming-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendStoredText-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/lights/LightsService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/speech/tts/TextToSpeechService$1;-speak-(Landroid/os/IBinder; Ljava/lang/CharSequence; I Landroid/os/Bundle; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-pauseChildStacks-(Lcom/android/server/am/ActivityRecord; Z Z Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getAllowScansWithTraffic-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/content/ContentService;-isSyncPendingAsUser-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName; I)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-proceedAfterWildChar-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Presentation$1;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/backup/BackupManagerService$PerformAdbRestoreTask$RestoreFileRunnable;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/ethernet/EthernetService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/os/IVibratorService;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/accessibility/TouchExplorer;-handleMotionEventStateDragging-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-getPassword-(Landroid/accounts/Account;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/provider/Telephony$Sms$Outbox;-addMessage-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long; Z J)Landroid/net/Uri;' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/widget/GridView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ReceiverRestrictedContext;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/content/ContentService;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/widget/NumberPicker$CustomEditText;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getCameraDisabled-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getCurrentSync-()Landroid/content/SyncInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/RIL;-getLastDataCallFailCause-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/NfcBarcode;-close-()V' : ['android.permission.NFC'],
	'Landroid/location/LocalListenerHelper;-registerWithServer-()Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowStateAnimator;-destroyDeferredSurfaceLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-interceptKeyBeforeDispatching-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)J' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/IContentService;-getSyncStatusAsUser-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName; I)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/nfc/NfcService$NfcServiceHandler$1;-onTagDisconnected-(J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setCLIR-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneFactory;-makeImsPhone-(Lcom/android/internal/telephony/PhoneNotifier; Lcom/android/internal/telephony/Phone;)Lcom/android/internal/telephony/imsphone/ImsPhone;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/CompoundButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-setIccFdnEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setCrossProfileCallerIdDisabled-(Landroid/content/ComponentName; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NativeDaemonConnector;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/squareup/okhttp/internal/http/JavaApiConverter;-createOkResponse-(Ljava/net/URI; Ljava/net/URLConnection;)Lcom/squareup/okhttp/Response;' : ['android.permission.INTERNET'],
	'Landroid/view/IWindowManager;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/telephony/SubscriptionController;-setDefaultSmsSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/ims/ImsCall;-notifyConferenceStateUpdatedThroughGroupOwner-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-cancelPendingUssd-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AnalogClock;-updateContentDescription-(Landroid/text/format/Time;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-setPropertiesByCarrier-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$Secure;-getLong-(Landroid/content/ContentResolver; Ljava/lang/String;)J' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/NfcAdapter;-enableReaderMode-(Landroid/app/Activity; Landroid/nfc/NfcAdapter$ReaderCallback; I Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$BaseFutureTask;-startTask-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-onClick-(Landroid/view/View;)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/server/wifi/WifiController;-<init>-(Landroid/content/Context; Lcom/android/server/wifi/WifiServiceImpl; Landroid/os/Looper;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/app/admin/IDevicePolicyManager;-getMaximumFailedPasswordsForWipe-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/service/notification/NotificationListenerService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-setRadioPower-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-updateMediaRecorderParams-()V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wm/WindowManagerService;-detachStackLocked-(Lcom/android/server/wm/DisplayContent; Lcom/android/server/wm/TaskStack;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/provider/CallLog$Calls;-getLastOutgoingCall-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_CALL_LOG'],
	'Lcom/android/server/am/ActivityStackSupervisor;-onDisplayRemoved-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine;-handleIpConfigurationLost-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/WapPushOverSms;-getDeliveryOrReadReportThreadId-(Landroid/content/Context; Lcom/google/android/mms/pdu/GenericPdu;)J' : ['android.permission.READ_SMS'],
	'Landroid/speech/tts/TextToSpeechService$1;-synthesizeToFileDescriptor-(Landroid/os/IBinder; Ljava/lang/CharSequence; Landroid/os/ParcelFileDescriptor; Landroid/os/Bundle; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-setDefaultVoiceSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService;-wakeUpInternal-(J I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneProxy;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountAuthenticatorActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/telephony/gsm/SmsManager;-sendTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/accounts/IAccountManager;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPasswordMinimumLetters-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/NfcAdapter;-disableForegroundNdefPush-(Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Lcom/android/commands/am/Am;-runKill-()V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/server/am/ActivityManagerService;-goToSleepIfNeededLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-getSignalStrength-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/Ndef;-close-()V' : ['android.permission.NFC'],
	'Landroid/accounts/AccountManager$9;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/nfc/NfcApplication;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/provider/Telephony$Sms$Draft;-addMessage-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long;)Landroid/net/Uri;' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/net/DhcpStateMachine$WaitBeforeStartState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-processCode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/WifiDisplayAdapter$1;-run-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/search/SearchManagerService$MyPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$SupplicantStartingState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/widget/LinearLayoutWithDefaultTouchRecepient;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/sip/SipAudioCall;-startAudioInternal-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-endLockTaskModeIfTaskEnding-(Lcom/android/server/am/TaskRecord;)V' : ['android.permission.DISABLE_KEYGUARD', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-onConnectedInOrOut-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NetworkScoreService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/DowntimeConditionProvider$Callback;-onDowntimeChanged-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-startAlarmForReconnect-(I Lcom/android/internal/telephony/dataconnection/ApnContext;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/Invert;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifyRfFieldActivated-()V' : ['android.permission.VIBRATE'],
	'Landroid/nfc/tech/MifareClassic;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/server/TextServicesManagerService;-setCurrentSpellCheckerSubtypeLocked-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockApplication;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/RIL;-getIMSIForApp-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/net/INetworkStatsService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/notification/ManagedServices$SettingsObserver;-<init>-(Lcom/android/server/notification/ManagedServices; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ISms;-enableCellBroadcastRange-(I I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/TelephonyManager;-getImei-(I)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag$Stub;-ndefRead-(I)Landroid/nfc/NdefMessage;' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/CropFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IntentService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/RIL;-getIccCardStatus-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-stopFreezingDisplayLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController;-<init>-(Landroid/content/Context; Ljava/lang/Runnable;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiStateMachine$RoamingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/accessibilityservice/AccessibilityService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/FisheyeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/NumberPicker$CustomEditText;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallBusy-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareUltralight;-readPages-(I)[B' : ['android.permission.NFC'],
	'Lcom/android/ims/internal/IImsCallSessionListener$Stub;-callSessionHoldFailed-(Lcom/android/ims/internal/IImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/WifiP2pManager;-getP2pStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/nfc/NfcRootActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/backup/BackupManagerService$ActiveRestoreSession;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/CountdownConditionProvider;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IWindowManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/PackageManagerBackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getMaximumTimeToLock-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/twilight/TwilightManager;-registerListener-(Lcom/android/server/twilight/TwilightListener; Landroid/os/Handler;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/ethernet/EthernetServiceImpl;-getConfiguration-()Landroid/net/IpConfiguration;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ActionBarView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-getEnabledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-<init>-(Lcom/android/server/content/SyncManager; Lcom/android/server/content/SyncOperation; J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processEventNotify-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub;-enableCellBroadcastForSubscriber-(J I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/nfc/cardemulation/OffHostApduService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setAndBroadcastNetworkSetTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-setDisplayNameUsingSrc-(Ljava/lang/String; J J)I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DatePicker;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/TtmlRenderingWidget;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-removeTaskByIdLocked-(I I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager;-getPreferredOptionsPanelGravity-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-reportLocation-(I D D D F F F J)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/ethernet/EthernetServiceImpl;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_NETWORK_STATE', 'android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceRemoveStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-getPrivilegedConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/dreams/DreamManagerService;-getDefaultDreamComponentForUser-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-reportSuccessfulPasswordAttempt-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPasswordMinimumNonLetter-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/service/trust/TrustAgentService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/RotationPolicy;-registerRotationPolicyListener-(Landroid/content/Context; Lcom/android/internal/view/RotationPolicy$RotationPolicyListener;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/InputMethodManagerService;-findLastResortApplicableShortcutInputMethodAndSubtypeLocked-(Ljava/lang/String;)Landroid/util/Pair;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendMultipartSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/widget/Toolbar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCellInfoList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/DisplayAdjustmentUtils;-hasAdjustments-(Landroid/content/Context; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/ForegroundUtils;-onProcessDied-(I I)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/AsyncService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiStateMachine$DriverStartedState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$Future2Task;-startTask-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/ShutdownActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/SmsManager;-sendStoredMultimediaMessage-(Landroid/net/Uri; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/provider/Settings$System;-getFloat-(Landroid/content/ContentResolver; Ljava/lang/String; F)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/RttService$RttServiceImpl$RttStateMachine;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityRecord$Token;-keyDispatchingTimedOut-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-finishUserBoot-(Lcom/android/server/am/UserStartedState;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/service/media/MediaBrowserService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/text/StringSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioManager;-isBluetoothA2dpOn-()Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/media/AudioService;-access$9600-(Landroid/media/AudioService; I I I Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-reconnect-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-getCallerAllowedResolutionLevel-()I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/job/JobServiceContext;-scheduleOpTimeOut-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService$7;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/service/notification/ConditionProviderService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockApplication;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/backup/BackupManagerService;-clearBackupData-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$500-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GeofenceManager;-access$000-(Lcom/android/server/location/GeofenceManager;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/BaseCommands;-requestShutdown-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiSettingsStore;-testAndClearWifiSavedState-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-readDockAudioSettings-(Landroid/content/ContentResolver;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setGsmBroadcastConfig-([Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareClassic;-reconnect-()V' : ['android.permission.NFC'],
	'Landroid/preference/SeekBarVolumizer;-onInitSample-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/speech/tts/AudioPlaybackHandler$MessageLoop;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/GLTextureSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/ClosedCaptionTrack;-hide-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/telephony/SubscriptionManager;-getDefaultDataSubId-()J' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Service;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowAnimator;-updateWindowsLocked-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/ZenModeHelper$SettingsObserver;-observe-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-onUserStarting-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/base/FrameBranch;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/PopupWindow$PopupViewContainer;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-dialInternal-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/Installer;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/AlertController$RecycleListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/TelephonyManager;-getIccSimChallengeResponse-(J I Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$ScanModeState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$3;-onCallHeld-(Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setDeviceOwner-(Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager;-setWifiEnabled-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiSettingsStore;-isAirplaneToggleable-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setMasterVolumeMuted-(Landroid/content/ComponentName; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-handleRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiServiceImpl;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/os/IPowerManager$Stub;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; Landroid/os/WorkSource; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-changeIccPin2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/ConditionProviders;-loadZenConfig-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActiveServices;-attachApplicationLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-sendDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-acknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkScoreService$Stub;-clearScores-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/VpnService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/nfc/TechListChooserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CalendarView$LegacyCalendarViewDelegate;-setMonthDisplayed-(Ljava/util/Calendar;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/BitmapOverlayEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/SystemServiceManager;-switchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-createUser-(Landroid/content/ComponentName; Ljava/lang/String;)Landroid/os/UserHandle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/UiModeManagerService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub;-getAllSubInfoCount-()I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/hdmi/HdmiCecLocalDevice;-create-(Lcom/android/server/hdmi/HdmiControlService; I)Lcom/android/server/hdmi/HdmiCecLocalDevice;' : ['android.permission.READ_PROFILE'],
	'Landroid/os/storage/StorageManager;-getStorageFullBytes-(Ljava/io/File;)J' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ListActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/GlobalActions;-createDialog-()Lcom/android/internal/policy/impl/GlobalActions$GlobalActionsDialog;' : ['android.permission.READ_PROFILE'],
	'Landroid/test/RenamingDelegatingContext;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ImageButton;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/test/IsolatedContext$MockAccountManager;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/IccProvider;-loadFromEf-(I J)Landroid/database/MatrixCursor;' : ['android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$Contacts$AggregationSuggestions$Builder;-build-()Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/net/wifi/IWifiManager;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/MultiAutoCompleteTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPasswordMinimumNonLetter-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ToggleButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-unregisterUiTestAutomationService-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/GLTextureSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-getCurrentInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcRootActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/usb/UsbService$Lifecycle;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/LauncherAppsService$LauncherAppsImpl$MyPackageMonitor;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/FullBackupJob;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/telephony/SmsManager;-copyMessageToIcc-([B [B I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/telephony/gsm/SmsManager;-deleteMessageFromSim-(I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/app/Presentation$1;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccCard;-supplyPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/mock/MockContentResolver;-getSyncStatusAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/view/TextureView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneBase;-getDataConnectionState-(Ljava/lang/String;)Lcom/android/internal/telephony/PhoneConstants$DataState;' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$Secure;-getIntForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-requestSingleUpdate-(Ljava/lang/String; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/InputMethodManagerService;-access$300-(Lcom/android/server/InputMethodManagerService;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-access$3700-(Lcom/android/server/location/GpsLocationProvider;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/PrintManager;-getPrintJobs-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-notifyAndAcknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ViewRootImpl$ViewPostImeInputStage;-onProcess-(Landroid/view/ViewRootImpl$QueuedInputEvent;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-copyMessageToIccEf-(Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-removeGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/dpm/Dpm;-runSetProfileOwner-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-isLockTaskPermitted-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowStateAnimator;-performShowLocked-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/ims/ImsEcbm$ImsEcbmListenerProxy;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/NetInitiatedActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/app/LauncherActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$3;-onCallResumed-(Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-onInputDeviceAdded-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/print/PrinterDiscoverySession;-destroyNoCheck-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/location/GpsLocationProvider;-handleInjectNtpTime-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/os/IHardwareService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.FLASHLIGHT'],
	'Lcom/android/internal/telephony/PhoneProxy;-getDataConnectionState-()Lcom/android/internal/telephony/PhoneConstants$DataState;' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/HostApduService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/MediaPlayer;-addTimedTextSource-(Ljava/io/FileDescriptor; J J Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setAndBroadcastNetworkSetTime-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-supplyNetworkDepersonalization-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtilsCache;-checkVoldPassword-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IMms$Stub;-deleteStoredMessage-(Ljava/lang/String; Landroid/net/Uri;)Z' : ['android.permission.WRITE_SMS'],
	'Landroid/net/wifi/IWifiManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_MULTICAST_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection;-separate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceTransition-(I Landroid/location/Location; I J I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-makeGsmCellBroadcastHandler-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/CaptioningManager;-access$000-(Landroid/view/accessibility/CaptioningManager;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/AppChooserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IntRangeManager;-finishUpdate-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkPolicyManager$Stub;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/wifi/p2p/WifiP2pManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/telephony/SmsManager;-importTextMessage-(Ljava/lang/String; I Ljava/lang/String; J Z Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/hardware/Camera;-open-()Landroid/hardware/Camera;' : ['android.permission.CAMERA'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPuk2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-fixTimeZone-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/widget/AbsActionBarView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/HostApduService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextClock;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ActivityManagerNative;-startUserInBackground-(I)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-loadEfFilesFromUsim-()Ljava/util/ArrayList;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-update-(Lcom/android/internal/telephony/DriverCall;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/HorizontalScrollView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getScreenCaptureDisabled-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getLine1AlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wm/WindowState;-removeLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-updatePersistentNetworks-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneMmiCode;-makeCFQueryResultMessage-(Lcom/android/internal/telephony/CallForwardInfo; I)Ljava/lang/CharSequence;' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/server/BluetoothA2dpService;-suspendSink-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/ActivityGroup;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/hardware/IConsumerIrService;-transmit-(Ljava/lang/String; I [I)V' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/server/pm/PackageManagerService;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-enableSystemApp-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BitmapSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager;-access$700-(Lcom/android/server/content/SyncManager; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Landroid/widget/RemoteViews$2;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-clear-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl$SipKeepAlive;-process-(Ljava/util/EventObject;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SubscriptionController;-setColor-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/test/mock/MockContentResolver;-setMasterSyncAutomaticallyAsUser-(Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/sip/SipManager;-unregister-(Landroid/net/sip/SipProfile; Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.USE_SIP'],
	'Landroid/filterpacks/imageproc/ImageEncoder;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/BluetoothHealth;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityManagerService;-updatePersistentConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/nfc/handover/ConfirmConnectActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager;-setStorageEncryption-(Landroid/content/ComponentName; Z I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/textservice/ITextServicesManager;-getCurrentSpellChecker-(Ljava/lang/String;)Landroid/view/textservice/SpellCheckerInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-addTrackerToRawTable-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/widget/ViewSwitcher;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-notifyDataConnectionFailedForSubscriber-(J Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/SpeechRecognizer;-putMessage-(Landroid/os/Message;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/PhoneBase$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Settings$Secure;-getFloat-(Landroid/content/ContentResolver; Ljava/lang/String;)F' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RadioGroup;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BlendFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover;-start-()Z' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/dreams/DreamManagerService;-access$2100-(Lcom/android/server/dreams/DreamManagerService; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CalendarView$LegacyCalendarViewDelegate$WeekView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-lockNow-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiController$DeviceInactiveState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextSwitcher;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/wm/Wm;-onRun-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getSignalStrength-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService$SetupContentObserver;-<init>-(Lcom/android/server/devicepolicy/DevicePolicyManagerService; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-resetPassword-(Ljava/lang/String; I I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setMute-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/cardemulation/OffHostApduService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-<init>-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/provider/BrowserContract$Settings;-isSyncEnabled-(Landroid/content/Context;)Z' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/power/PowerManagerService;-access$3900-(Lcom/android/server/power/PowerManagerService; J I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-setScanResults-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ILockSettings;-getString-(Ljava/lang/String; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/AliasActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ILockSettings$Stub$Proxy;-getString-(Ljava/lang/String; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/AccessibilityController$WindowsForAccessibilityObserver;-computeChangedWindows-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-runAddAccountForAuthenticator-(Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/textservice/TextServicesManager;-getCurrentSpellChecker-()Landroid/view/textservice/SpellCheckerInfo;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/CantAddAccountActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/DateTimeView;-registerReceivers-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-enableNoAutoConnect-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/inputmethodservice/InputMethodService;-initViews-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendRetrySms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/dreams/DreamManagerService;-getDreamComponentsForUser-(I)[Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-bootAnimationComplete-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processSelectItem-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerPolicy;-performHapticFeedbackLw-(Landroid/view/WindowManagerPolicy$WindowState; I Z)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService$DisplayAdapterListener;-onTraversalRequested-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/tech/NfcBarcode;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/SupplicantStateTracker$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiStateMachine$VerifyingLinkState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager;-removePrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/AidRoutingManager;-configureRouting-(Ljava/util/HashMap;)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/server/job/JobSchedulerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-checkAudioSettingsPermission-(Ljava/lang/String;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/power/PowerManagerService;-access$5100-(Lcom/android/server/power/PowerManagerService; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/service/notification/NotificationListenerService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/opengl/GLSurfaceView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getMasterSyncAutomaticallyAsUser-(I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$InactiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getLine1NumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/inputmethodservice/InputMethodService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/BluetoothManagerService;-registerForAirplaneMode-(Landroid/content/IntentFilter;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcAdapter$Stub;-setForegroundDispatch-(Landroid/app/PendingIntent; [Landroid/content/IntentFilter; Landroid/nfc/TechListParcel;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-getAutoTimeZone-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/media/MediaSessionService$SessionManagerImpl$KeyEventWakeLockReceiver;-onTimeout-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActiveServices;-serviceTimeout-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/power/ShutdownThread;-shutdownInner-(Landroid/content/Context; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/base/ObjectSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-getNetworkCapabilities-(Ljava/lang/String;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcTag;-reconnect-(I)I' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/ISub$Stub;-getDefaultSmsSubId-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-clearTestProviderEnabled-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/net/wifi/WifiManager;-getControllerActivityEnergyInfo-(I)Landroid/net/wifi/WifiActivityEnergyInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/accessibility/TouchExplorer;-access$1700-(Lcom/android/server/accessibility/TouchExplorer; Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-getTasks-(I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-processPostDialChar-(C)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-makeScpHandler-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallEnded-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/fingerprint/FingerprintUtils;-addFingerprintIdForUser-(I Landroid/content/ContentResolver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-onUpdatePhoneObject-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Ljava/net/URL;-openStream-()Ljava/io/InputStream;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-onRecordLoaded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/Editor$SelectionStartHandleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/textservice/SpellCheckerService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/DatePickerCalendarDelegate;-setCurrentView-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-disableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DockObserver;-access$900-(Lcom/android/server/DockObserver; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/media/ClosedCaptionWidget;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService;-access$400-(Lcom/android/server/VibratorService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisToFileOutputStreamSpeechItemV1;-<init>-(Landroid/speech/tts/TextToSpeechService; Ljava/lang/Object; I I Landroid/os/Bundle; Ljava/lang/String; Ljava/lang/CharSequence;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/trust/TrustAgentService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/print/IPrintManager;-print-(Ljava/lang/String; Landroid/print/IPrintDocumentAdapter; Landroid/print/PrintAttributes; Ljava/lang/String; I I)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Telephony$Sms;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_SMS'],
	'Lcom/android/nfc/P2pEventManager;-onP2pResumeSend-()V' : ['android.permission.VIBRATE'],
	'Landroid/widget/TextClock;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/ConfirmConnectActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/view/menu/ListMenuItemView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setMaximumTimeToLock-(Landroid/content/ComponentName; J I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/SpeechRecognizer;-access$100-(Landroid/speech/SpeechRecognizer;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/server/MountServiceIdler;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-<init>-(Landroid/content/Context; I Lcom/android/server/accessibility/AccessibilityManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService$Stub;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/inputmethodservice/ExtractButton;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-acknowledgeIncomingGsmSmsWithPdu-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AsyncPlayer;-access$200-(Landroid/media/AsyncPlayer; Landroid/media/AsyncPlayer$Command;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/content/ContentService;-setMasterSyncAutomaticallyAsUser-(Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/ActionBarView$HomeView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$6;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/content/SyncStorageEngine;-readAndDeleteLegacyAccountInfoLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingEncryptedRfcommOn-(I)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ReceiverRestrictedContext;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/display/DisplayPowerController;-access$000-(Lcom/android/server/display/DisplayPowerController;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-access$1000-(Lcom/android/server/am/ActivityManagerService; I I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/util/StateMachine$SmHandler$QuittingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager;-initializeMulticastFiltering-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/NsdService$NsdStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/NativeActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dialInternal-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/widget/VideoView;-suspend-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/printservice/PrintService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/fingerprint/FingerprintService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/DateTimeView;-setTime-(J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-access$100-(Lcom/android/internal/telephony/IccSmsInterfaceManager; [Lcom/android/internal/telephony/gsm/SmsBroadcastConfigInfo;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-switchHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPasswordMinimumLength-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-access$2800-(Lcom/android/server/pm/PackageManagerService; I I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/tv/TvInputManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncStorageEngine;-setMasterSyncAutomatically-(Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/nfc/NfcActivityManager;-setReaderMode-(Landroid/os/Binder; I Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation$Stub$Proxy;-removeAidGroupForService-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/server/power/ShutdownThread;-rebootSafeMode-(Landroid/content/Context; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-queueNextSignalStrengthPoll-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/os/IPowerManager;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/PreferredServices;-unregisterForegroundService-(I)Z' : ['android.permission.VIBRATE'],
	'Lcom/android/server/media/MediaSessionService$SessionManagerImpl$KeyEventWakeLockReceiver;-releaseWakeLockLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/BackgroundDexOptService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Settings$System;-getString-(Landroid/content/ContentResolver; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/dpm/Dpm;-main-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-pollState-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/media/IAudioService$Stub$Proxy;-startBluetoothScoVirtualCall-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-reopenMenu-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/BrowserContract$Settings;-setSyncEnabled-(Landroid/content/Context; Z)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS'],
	'Landroid/content/IContentService;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/ListPopupWindow$DropDownListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$4;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/Button;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/dreams/DreamManagerService;-cleanupDreamLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/FullBackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/GlobalActions;-<init>-(Landroid/content/Context; Landroid/view/WindowManagerPolicy$WindowManagerFuncs;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/ViewRootImpl;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-sendMultipartTextForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/media/AudioService$VolumeStreamState;-<init>-(Landroid/media/AudioService; Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/format/DateFormat;-getTimeFormatString-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getIccSerialNumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/effect/effects/ContrastEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Ljava/net/MulticastSocket;-<init>-(I)V' : ['android.permission.INTERNET'],
	'Landroid/media/AudioService;-adjustStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-access$3400-(Lcom/android/server/content/SyncManager; Lcom/android/server/content/SyncManager$ActiveSyncContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-updateImsPhone-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/net/NetworkStatsService;-registerPollAlarmLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-getCLIR-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/backup/LocalTransportService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-getAddress-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManager;-getAuthTokenByFeatures-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/app/Activity; Landroid/os/Bundle; Landroid/os/Bundle; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/nfc/NfcService$TagService;-close-(I)I' : ['android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-pauseGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/View;-onKeyUp-(I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-handleDisplayChangedLocked-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/pm/IPackageManager$Stub;-performBootDexOpt-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/RuimPhoneBookInterfaceManager;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_CONTACTS'],
	'Landroid/speech/tts/ITextToSpeechService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-resetMagnificationIfNeeded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-registerForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/AlarmManagerService;-access$600-(Lcom/android/server/AlarmManagerService;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/BlackWhiteEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-writeOutboxMessage-(J Ljava/lang/String; Ljava/lang/String; Z Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/telephony/uicc/AdnRecordLoader;-updateEF-(Lcom/android/internal/telephony/uicc/AdnRecord; I I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/gsm/SmsMessage;-calculateLength-(Ljava/lang/String; Z)[I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/PowerManagerService$LocalService;-setScreenBrightnessOverrideFromWindowManager-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$400-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-getIccSimChallengeResponse-(I Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-isAirplaneModeOff-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseAccountActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-importTextMessage-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; J Z Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/cat/IconLoader;-postIcon-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/job/JobSchedulerService$JobSchedulerStub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/net/VpnService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/cardemulation/RegisteredAidCache;-onNfcEnabled-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover;-handleIntent-(Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/DownloadManager;-getMaxBytesOverMobile-(Landroid/content/Context;)Ljava/lang/Long;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService$PerformUnifiedRestoreTask;-initiateOneRestore-(Landroid/content/pm/PackageInfo; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-getRecentTasks-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/server/wifi/WifiApConfigStore;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getPasswordMinimumNumeric-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-togglePanel-(I Landroid/view/KeyEvent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/MediaFocusControl;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getMaximumTimeToLock-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgentHelper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-isTdlsSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/location/LocationManager;-addNmeaListener-(Landroid/location/GpsStatus$NmeaListener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/sip/SipAudioCall;-isWifiOn-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub$1;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/StateMachine$SmHandler$HaltingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-setFacilityLockForApp-(Ljava/lang/String; Z Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/method/TextKeyListener;-initPrefs-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$4;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/telephony/TelephonyManager;-getLine1AlphaTagForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioManager;-setMode-(I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/statusbar/StatusBarManagerService;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/content/AbstractThreadedSyncAdapter$ISyncAdapterImpl;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/power/ShutdownThread;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-stopPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getLastInputMethodAndSubtypeLocked-()Landroid/util/Pair;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-makeAppNotRespondingLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-calculateLength-(Ljava/lang/CharSequence; Z)Lcom/android/internal/telephony/GsmAlphabet$TextEncodingDetails;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/RatingBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/DateTimeView;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/appwidget/AppWidgetServiceImpl;-getInstalledProvidersForProfile-(I I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IMms$Stub;-addMultimediaMessageDraft-(Ljava/lang/String; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/accounts/IAccountManager$Stub;-addAccountAsUser-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle; I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiController$DeviceInactiveState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onCleanUpConnection-(Z I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService$LocalService;-getCrossProfileWidgetProviders-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-setDisplayNameUsingSrc-(Ljava/lang/String; J J)I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiManager;-isWifiEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/commands/wm/Wm;-run-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-enableScreenAfterBoot-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/hardware/location/GeofenceHardwareService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager;-setGlobalProxy-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String; I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/CheckedTextView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/widget/RadialTimePickerView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-stopPrinterStateTracking-(Landroid/print/PrinterId; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-isScanAlwaysAvailable-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-updateAdnRecordsInEfBySearchForSubscriber-(J I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getImeiForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager;-startScan-(Landroid/net/wifi/ScanSettings; Landroid/os/WorkSource;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/TextProgressBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/NativeDaemonConnector;-listenToSocket-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/effect/effects/DocumentaryEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$TetheredState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onDisconnect-(I)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/WifiDisplayController;-access$000-(Lcom/android/server/display/WifiDisplayController;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ShutdownActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readTouchExplorationEnabledSettingLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RemoteViews$2;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/NsdService;-access$100-(Lcom/android/server/NsdService;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/OverlayDisplayAdapter$1;-run-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/VibratorService$VibrateThread;-<init>-(Lcom/android/server/VibratorService; Lcom/android/server/VibratorService$Vibration;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager$Stub$Proxy;-addPrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/os/PowerManager$WakeLock$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getLine1AlphaTagForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/Editor$HandleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothA2dp;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$NotConnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/CommandsInterface;-getOperator-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/PhoneNumberUtils;-isEmergencyNumberInternal-(Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/NetworkMonitor$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-writeInboxMessage-(Landroid/content/Intent;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS'],
	'Landroid/view/IWindowManager$Stub$Proxy;-enableScreenIfNeeded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/sip/SipService$SipKeepAliveProcessCallback;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/PreferredServices$Callback;-onPreferredPaymentServiceChanged-(Landroid/content/ComponentName;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-startFreezingScreen-(I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/tech/NfcF;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/server/display/DisplayPowerState$PhotonicModulator;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendCdmaSms-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ZoomControls;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/tech/NfcA;-setTimeout-(I)V' : ['android.permission.NFC'],
	'Landroid/media/ClosedCaptionWidget;-setVisible-(Z)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/media/AudioService;-createStreamStates-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-dial-(Ljava/lang/String; I Lcom/android/internal/telephony/UUSInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/ISms;-sendMultipartTextForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-access$1700-(Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler; Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub;-enableScreenIfNeeded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager$CardEmulationInterface;-unsetPreferredService-()Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-<init>-(Landroid/content/Context; Lcom/android/ims/ImsCall; Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker; Lcom/android/internal/telephony/imsphone/ImsPhoneCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-sendP2pPersistentGroupsChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/VideoView;-setVideoPath-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiServiceImpl;-reconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipCommandInterface;-setUiccSubscription-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-getPermittedAccessibilityServicesForUser-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiScanningService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-getNetworkCapabilities-(Ljava/lang/String;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getStorageEncryption-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/hardware/input/InputManager$InputDeviceVibrator;-vibrate-(I Ljava/lang/String; J Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService$AudioHandler$1;-onCompletion-(Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/updates/IntentFirewallInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-interceptKeyBeforeQueueing-(Landroid/view/KeyEvent; I)I' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setUserRestriction-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-requestTraversal-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/SupplicantStateTracker$DormantState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/notification/ZenModeHelper;-readXml-(Lorg/xmlpull/v1/XmlPullParser;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-transceive-(I [B Z)Landroid/nfc/TransceiveResult;' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onSetInternalDataEnabled-(Z Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/OutputStreamTarget;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-invokeBeamInternal-(Landroid/nfc/BeamShareData;)V' : ['android.permission.VIBRATE'],
	'Landroid/view/IWindowManager$Stub;-prepareAppTransition-(I Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/power/PowerManagerService;-setDozeOverrideFromDreamManagerInternal-(I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub;-initForUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothInputDevice;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/notification/ManagedServices;-onUserSwitched-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/HdmiControlService;-initializeCec-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-<init>-(Lcom/android/server/print/PrintManagerService; Landroid/content/Context;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/ims/internal/ImsCallSession$IImsCallSessionListenerProxy;-callSessionHoldFailed-(Lcom/android/ims/internal/IImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CheckBox;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/PlatLogoActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cat/RilMessageDecoder;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-reloadButtons-()V' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-conference-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-updateRingerModeAffectedStreams-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$NotConnectedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onDisconnectDcRetrying-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/ethernet/EthernetService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/GLTextureSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/NdefFormatable;-format-(Landroid/nfc/NdefMessage; Z)V' : ['android.permission.NFC'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setGlobalProxy-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String; I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/HandoverService;-onBluetoothPeripheralHandoverComplete-(Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/app/backup/BackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneProxy;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/pm/IPackageManager$Stub;-getInstallLocation-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl;-process-(Ljava/util/EventObject;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/SettingInjectorService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/twilight/TwilightService$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/LockPatternUtils;-getLockoutAttemptDeadline-()J' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/NfcAdapter;-disableForegroundDispatchInternal-(Landroid/app/Activity; Z)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-setVoiceCallForwardingFlag-(I Z Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(J F Landroid/location/Criteria; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/NativeCrashListener;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getConnectionStatistics-()Landroid/net/wifi/WifiConnectionStatistics;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/content/ContentService;-cancelRequest-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/commands/wm/Wm;-runDisplaySize-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/MediaFocusControl;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/SearchDialog$SearchBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-setApplicationRestrictions-(Landroid/content/ComponentName; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-access$8400-(Landroid/media/AudioService; Landroid/content/ContentResolver;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/AsyncService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/hardware/input/InputManager;-getPointerSpeed-(Landroid/content/Context;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$GroupNegotiationState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$5300-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-isTemplateRelevant-(Landroid/net/NetworkTemplate;)Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/location/ILocationManager;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindow$DecorView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/WifiManager;-enableVerboseLogging-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/backup/BackupManagerService$FullRestoreEngine;-restoreOneFile-(Ljava/io/InputStream;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/CropFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/ManagedServices$SettingsObserver;-update-(Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowManagerService;-computeNewConfiguration-()Landroid/content/res/Configuration;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/os/SystemVibrator;-vibrate-(J Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wm/WindowAnimator;-access$000-(Lcom/android/server/wm/WindowAnimator;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/AbsSeekBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-getSalt-(I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getActiveAdminUncheckedLocked-(Landroid/content/ComponentName; I)Lcom/android/server/devicepolicy/DevicePolicyManagerService$ActiveAdmin;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/nsd/INsdManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/widget/LockPatternUtilsCache;-setLockPassword-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier$MagnificationController;-setMagnificationSpec-(Landroid/view/MagnificationSpec;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/webkit/WebViewUpdateService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/IconMenuView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/WifiManager;-isPreferredNetworkOffloadSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-activitySleptLocked-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub$Proxy;-setCurrentSpellCheckerSubtype-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-isLockScreenDisabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager$WifiLock;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/AliasActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/NumberPicker$CustomEditText;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-updatePhoneObject-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/app/job/JobService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/connectivity/PacManager;-setDownloadIn-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/ProgramPort;-setFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/media/MediaSessionService$SessionManagerImpl;-verifySessionsRequest-(Landroid/content/ComponentName; I I I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-finishRunningVoiceLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/INetworkPolicyManager;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/nfc/tech/NfcF;-setTimeout-(I)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDataSetupComplete-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/widget/TableRow;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-startRunningVoiceLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/backup/BackupManagerService$PerformBackupTask;-executeNextState-(Lcom/android/server/backup/BackupManagerService$BackupState;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub$Proxy;-addGpsMeasurementsListener-(Landroid/location/IGpsMeasurementsListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/widget/LockPatternUtils;-isCredentialRequiredToDecrypt-(Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/GlobalActions$10;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getAccountTypesWithManagementDisabled-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-notifyNoData-(Lcom/android/internal/telephony/dataconnection/DcFailCause; Lcom/android/internal/telephony/dataconnection/ApnContext;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-iccTransmitApduHelper-(I I I I I I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-sendImsCdmaSms-([B I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/test/InstrumentationTestRunner;-getUiAutomation-()Landroid/app/UiAutomation;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wifi/WifiServiceImpl;-enforceAccessPermission-()V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/display/DisplayManagerService$LocalService;-setDisplayInfoOverrideFromWindowManager-(I Landroid/view/DisplayInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/NotificationManagerService$SettingsObserver;-<init>-(Lcom/android/server/notification/NotificationManagerService; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyCellInfo-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-applyRequirementsLocked-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/DockObserver;-setDockStateLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getImsRegistrationState-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/FisheyeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService;-access$200-(Lcom/android/server/InputMethodManagerService; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ManagedServices$SettingsObserver;-access$100-(Lcom/android/server/notification/ManagedServices$SettingsObserver;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/view/accessibility/IAccessibilityManager$Stub;-unregisterUiTestAutomationService-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/power/ShutdownThread$1;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wallpaper/WallpaperManagerService;-setDisplayPadding-(Landroid/graphics/Rect;)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Landroid/speech/tts/TextToSpeechService;-onCreate-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/UiccCard;-update-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/uicc/IccCardStatus;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-isMulticastEnabled-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/provider/Settings$System;-getFloat-(Landroid/content/ContentResolver; Ljava/lang/String;)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayManagerService;-access$1500-(Lcom/android/server/display/DisplayManagerService; Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/content/SyncManager;-access$3500-(Lcom/android/server/content/SyncManager; Lcom/android/server/content/SyncOperation;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-doInBackground-([Landroid/filterfw/core/SyncRunner;)Landroid/filterfw/core/AsyncRunner$RunnerResult;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/service/dreams/DreamService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/P2pEventManager;-onP2pSendComplete-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-processPendingRadioPowerOffAfterDataOff-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-enableAggressiveHandover-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getScreenCaptureDisabled-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/SubtitleController;-access$200-(Landroid/media/SubtitleController; Landroid/media/SubtitleTrack;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-renameAccount-(Landroid/accounts/Account; Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/WiredAccessoryManager$WiredAccessoryObserver;-updateStateLocked-(Ljava/lang/String; Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/CameraSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ActivityGroup;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-isActivePasswordSufficient-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/AlertController$RecycleListView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/appwidget/AppWidgetServiceImpl$SecurityPolicy;-isProviderWhitelListed-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/IGeofenceHardware;-registerForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/ActivityManagerService;-access$300-(Lcom/android/server/am/ActivityManagerService; Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/FlpHardwareProvider;-<init>-(Landroid/content/Context;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/commands/monkey/MonkeySourceNetworkViews;-setup-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/SmsMessage;-getSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Z [B)Lcom/android/internal/telephony/gsm/SmsMessage$SubmitPdu;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/nfc/NfcAdapter;-enableForegroundNdefPush-(Landroid/app/Activity; Landroid/nfc/NdefMessage;)V' : ['android.permission.NFC'],
	'Landroid/test/IsolatedContext;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/provider/Settings$System;-getLong-(Landroid/content/ContentResolver; Ljava/lang/String; J)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangup-(Lcom/android/internal/telephony/cdma/CdmaConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/ChooserActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService;-checkManageAccountsPermission-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/service/wallpaper/WallpaperService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/nfc/NfcApplication;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-startLocationBasedDetector-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/hdmi/NewDeviceAction;-mayProcessCommandIfCached-(I I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/FlipFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/print/IPrintManager;-getPrintJobInfos-(I I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-access$900-(Lcom/android/server/wm/WindowManagerService; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ScrollView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-showEmulatorDisplayOverlay-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/RadioButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-updatePasswordExpirationsLocked-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-onDisplayAdded-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleMove-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor$VirtualActivityDisplay;-<init>-(Lcom/android/server/am/ActivityStackSupervisor; I I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/job/IJobScheduler$Stub$Proxy;-schedule-(Landroid/app/job/JobInfo;)I' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getIsimChallengeResponse-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPermittedInputMethods-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/os/storage/StorageManager;-getStorageLowBytes-(Ljava/io/File;)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-requestIccSimAuthentication-(I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setGlobalSetting-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setDataRoamingEnabled-(Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/webkit/WebViewFactory;-createRelroFile-(Z [Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/display/DisplayPowerController;-animateScreenBrightness-(I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/ims/internal/IImsEcbmListener$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService$Stub$Proxy;-setSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/nfc/INfcAdapter$Stub;-addNfcUnlockHandler-(Landroid/nfc/INfcUnlockHandler; [I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ListMenuItemView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-access$2300-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/telephony/SubscriptionManager;-setDisplayNumber-(Ljava/lang/String; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/twilight/TwilightListener;-onTwilightStateChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$WaitForP2pDisableState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService$SettingsObserver;-<init>-(Lcom/android/server/InputMethodManagerService; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiApConfigStore$InactiveState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TabWidget;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/IEthernetManager$Stub;-getConfiguration-()Landroid/net/IpConfiguration;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setProperties-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/media/MediaSessionService$SessionManagerImpl$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-changeIccPin2ForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/statusbar/StatusBarManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.EXPAND_STATUS_BAR'],
	'Landroid/filterpacks/imageproc/GrainFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-access$300-(Lcom/android/server/print/PrintManagerService$PrintManagerImpl; I)Lcom/android/server/print/UserState;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService$25;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-getPasswordMinimumLowerCase-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/RankingReconsideration;-run-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-setInternalDataEnabled-(Z)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getMaximumFailedPasswordsForWipe-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech;-getLanguage-()Ljava/util/Locale;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-onUpdatePhoneObject-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation$Stub$Proxy;-setDefaultServiceForCategory-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/accounts/IAccountManager;-addAccount-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-ensureActiveMobilePolicyLocked-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-sendStoredMessage-(J Ljava/lang/String; Landroid/net/Uri; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Landroid/app/admin/IDevicePolicyManager;-setUninstallBlocked-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/os/NullVibrator;-vibrate-(J)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendSms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/LocationManagerService;-access$200-(Lcom/android/server/LocationManagerService;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-assignLayersLocked-(Lcom/android/server/wm/WindowList;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/base/NullFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/dreams/DreamManagerService;-getDozeComponent-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-isActivePasswordSufficient-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GeofenceManager;-removeFence-(Landroid/location/Geofence; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService;-access$4100-(Lcom/android/server/display/DisplayManagerService; I Landroid/view/DisplayInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneProxy;-setVoiceMessageWaiting-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/VibratorService;-doCancelVibrateLocked-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangup-(Lcom/android/internal/telephony/gsm/GsmCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/textservice/TextServicesManager;-setCurrentSpellChecker-(Landroid/view/textservice/SpellCheckerInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcCardEmulation$Stub$Proxy;-getAidGroupForService-(I Landroid/content/ComponentName; Ljava/lang/String;)Landroid/nfc/cardemulation/AidGroup;' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/server/InputMethodManagerService;-setSelectedInputMethodAndSubtypeLocked-(Landroid/view/inputmethod/InputMethodInfo; I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-showCircularDisplayMaskIfNeeded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController$StaEnabledState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/SearchView$SearchAutoComplete;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/PrintJob;-cancel-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendSubmitPdu-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/accessibility/ScreenMagnifier$MagnificationController;-reset-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/GlobalActions;-access$1800-(Lcom/android/internal/policy/impl/GlobalActions;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.VIBRATE', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-setSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/widget/ActionMenuPresenter$OpenOverflowRunnable;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/Phone;-getDataConnectionState-()Lcom/android/internal/telephony/PhoneConstants$DataState;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-<init>-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/RIL;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneFactory;-getGsmPhone-()Lcom/android/internal/telephony/Phone;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/TapAgainDialog;-onStop-()V' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Landroid/hardware/location/GeofenceHardwareService;-checkPermission-(I I I)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/FrameSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/AudioService;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/SearchView$SearchAutoComplete;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/NfcActivityManager;-setNdefPushMessageCallback-(Landroid/app/Activity; Landroid/nfc/NfcAdapter$CreateNdefMessageCallback; I)V' : ['android.permission.NFC'],
	'Landroid/filterpacks/text/ToUpperCase;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager;-access$800-(Lcom/android/server/content/SyncManager; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/notification/NotificationManagerService;-importOldBlockDb-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-enableVerboseLogging-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/AutoCompleteTextView;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcAdapter$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH', 'android.permission.NFC'],
	'Landroid/view/IWindowManager$Stub$Proxy;-prepareAppTransition-(I Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Browser;-clearHistory-(Landroid/content/ContentResolver;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-sendEmergencyCallbackModeChange-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-requestShutdown-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/WebVttTrack;-show-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/speech/SpeechRecognizer;-cancel-()V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/widget/GridLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accessibilityservice/AccessibilityService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$InitialState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager$AmsTask;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/content/ISyncAdapter$Stub$Proxy;-initialize-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/ActivityManagerNative;-getRecentTasks-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/internal/telephony/cat/CatService;-updateIccAvailability-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setProfileName-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/dreams/DreamManagerService;-requestAwakenInternal-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-isOwnerInfoEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Editor$InsertionHandleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-reRegisterNetwork-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/Notifier;-onWirelessChargingStarted-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-updateOrientationFromAppTokensLocked-(Landroid/content/res/Configuration; Landroid/os/IBinder;)Landroid/content/res/Configuration;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/hdmi/HdmiCecController;-handleHotplug-(I Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingRfcommWithServiceRecord-(Ljava/lang/String; Ljava/util/UUID;)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/DateTimeView;-update-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getPollInterval-()J' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Space;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-addNfcUnlockHandler-(Landroid/nfc/INfcUnlockHandler; [I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-needsToShowImeSwitchOngoingNotification-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getGlobalBoolean-(Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/TelephonyManager;-getSimSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/LocationManagerService;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-access$2200-(Lcom/android/internal/policy/impl/PhoneWindow;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/SubscriptionController;-updateAllDataConnectionTrackers-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-disableCellBroadcastForSubscriber-(J I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-setOperatorBrandOverride-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/fingerprint/FingerprintService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-getPreferredSmsSubscription-()J' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/FlipEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/method/TextKeyListener;-access$200-(Landroid/text/method/TextKeyListener; Landroid/content/ContentResolver;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-onKeyUp-(I I Landroid/view/KeyEvent;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/AsyncPlayer;-play-(Landroid/content/Context; Landroid/net/Uri; Z I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/p2p/IWifiP2pManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/IActivityManager;-getRecentTasks-(I I I)Ljava/util/List;' : ['android.permission.GET_TASKS'],
	'Lcom/android/server/wifi/RttService$RttServiceImpl$RttStateMachine$RequestPendingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkStatsService$4;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/NotificationManagerService;-onBootPhase-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-systemReady-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/IAccountManager$Stub;-getAuthToken-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Z Landroid/os/Bundle;)V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-runAfterDetection-(Landroid/location/Country; Landroid/location/Country; Z Z)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/DateTimeView;-getDateFormat-()Ljava/text/DateFormat;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/filterpacks/base/OutputStreamTarget;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCdmaBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/SwipeDismissLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/performance/ThroughputFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-sendStoredMultipartText-(Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/power/PowerManagerService;-handleUserActivityTimeout-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity$1;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/os/SystemVibrator;-cancel-()V' : ['android.permission.VIBRATE'],
	'Landroid/media/MediaHTTPConnection;-seekTo-(J)V' : ['android.permission.INTERNET'],
	'Landroid/net/wifi/IWifiManager$Stub;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/nsd/NsdManager;-<init>-(Landroid/content/Context; Landroid/net/nsd/INsdManager;)V' : ['android.permission.INTERNET'],
	'Lcom/android/server/wm/WindowManagerService;-unsetAppFreezingScreenLocked-(Lcom/android/server/wm/AppWindowToken; Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setInternalDataEnabled-(Z Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/service/wallpaper/WallpaperSettingsActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/tech/NfcF;-getTimeout-()I' : ['android.permission.NFC'],
	'Lcom/android/internal/statusbar/IStatusBarService$Stub;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/telephony/MccTable;-setTimezoneFromMccIfNeeded-(Landroid/content/Context; I)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/nfc/NfcBackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-getDefaultDisplayContentLocked-()Lcom/android/server/wm/DisplayContent;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/backup/BackupAgent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/TintFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/ImmersiveModeConfirmation$ClingWindowView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioManager;-stopBluetoothSco-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCallWaiting-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-conference-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/WebVttRenderingWidget$RegionLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCall;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager;-getProviderProperties-(Ljava/lang/String;)Lcom/android/internal/location/ProviderProperties;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/ims/ImsCall$Listener;-onCallProgressing-(Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiServiceImpl;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.CHANGE_WIFI_MULTICAST_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/hdmi/HdmiControlService;-onStart-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/SystemServiceManager;-startService-(Ljava/lang/String;)Lcom/android/server/SystemService;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/BatteryService;-access$400-(Lcom/android/server/BatteryService;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/NfcAdapter;-setNdefPushMessage-(Landroid/nfc/NdefMessage; Landroid/app/Activity; [Landroid/app/Activity;)V' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/widget/ActionBarView$HomeView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService$4;-run-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/UiModeManagerService;-sendConfigurationAndStartDreamOrDockAppLocked-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-onDisplayChanged-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/net/NetworkStatsService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getOwnerInfo-(I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IApplicationToken$Stub;-keyDispatchingTimedOut-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/AbsSeekBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine$VerifyingLinkState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ActionBarContextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-acquireMulticastLock-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setUninstallBlocked-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService;-disableSafeMediaVolume-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$System;-getConfigurationForUser-(Landroid/content/ContentResolver; Landroid/content/res/Configuration; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/CrossProcessEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-isApnTypeActive-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-setInputMethodLocked-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/ServiceWatcher$2;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/NfcService$TagService;-ndefMakeReadOnly-(I)I' : ['android.permission.NFC'],
	'Lcom/android/server/wifi/WifiAutoJoinController;-newSupplicantResults-(Z)I' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SubscriptionManager;-getDefaultSmsPhoneId-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/HandoverService;-handleBluetoothStateChanged-(Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/widget/PreferenceImageView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IMms;-setAutoPersisting-(Ljava/lang/String; Z)V' : ['android.permission.WRITE_SMS'],
	'Lcom/android/server/display/OverlayDisplayAdapter;-updateOverlayDisplayDevicesLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-handleCfQueryResult-([Lcom/android/ims/ImsCallForwardInfo;)[Lcom/android/internal/telephony/CallForwardInfo;' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-init-()V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/accessibility/ScreenMagnifier$MagnificationController;-setScale-(F F F Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/Ndef;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/PhoneProxy;-phoneObjectUpdater-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-cleanUpAllConnections-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccCard;-changeIccFdnPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiServiceImpl;-addOrUpdateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService$6;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/ClosedCaptionTrack;-show-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/media/ClosedCaptionWidget;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-isUserSetupComplete-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getMsisdnForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getNetworkCapabilities-(Ljava/lang/String;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService;-performPollLocked-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-updateCryptoUserInfo-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setPasswordMinimumNumeric-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub$Stub;-setDisplayNumber-(Ljava/lang/String; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getDeviceIdForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/power/Notifier;-playWirelessChargingStartedSound-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-updateSpnDisplay-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiScanningServiceImpl$WifiChangeStateMachine$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/test/IsolatedContext$MockAccountManager;-removeAccountAsUser-(Landroid/accounts/Account; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler; Landroid/os/UserHandle;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/power/PowerManagerService$DreamReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/notification/CountdownConditionProvider;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController$ApStaDisabledState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/display/DisplayManagerService;-handleDisplayDeviceAddedLocked-(Lcom/android/server/display/DisplayDevice;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/content/ContentService;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/nfc/NfcActivityManager;-setOnNdefPushCompleteCallback-(Landroid/app/Activity; Landroid/nfc/NfcAdapter$OnNdefPushCompleteCallback;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/MccTable;-updateMccMncConfiguration-(Landroid/content/Context; Ljava/lang/String; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getAllowScansWithTraffic-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/net/NetworkStatsService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-shutdown-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActiveServices$ServiceMap;-ensureNotStartingBackground-(Lcom/android/server/am/ServiceRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/PersistentDataBlockService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/WebVttRenderingWidget;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ActionBarContainer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkStatsService$6;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-revertToNitzTimeZone-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onCleanUpAllConnections-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-completeConnection-(Lcom/android/internal/telephony/dataconnection/ApnContext;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/location/LocationManager;-removeUpdates-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover;-onServiceConnected-(I Landroid/bluetooth/BluetoothProfile;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-enableAggressiveHandover-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipCall;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub$Stub;-getAllSubInfoList-()Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/base/RetargetFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/admin/IDevicePolicyManager;-setSecureSetting-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcCardEmulation$Stub;-isDefaultServiceForAid-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/server/BluetoothA2dpService;-checkSinkSuspendState-(I)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-addCircularFence-(I Landroid/hardware/location/GeofenceHardwareRequestParcelable; Landroid/hardware/location/IGeofenceHardwareCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofenceTransition-(I Landroid/location/Location; I J I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-sendScreenState-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-screenshotApplications-(Landroid/os/IBinder; I I I Z)Landroid/graphics/Bitmap;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/InputMethodManagerService$7;-onClick-(Landroid/content/DialogInterface; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ActivityChooserView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-cancelFullNotification-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$3;-onCallStarted-(Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowSession;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/hardware/location/GeofenceHardwareService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/android/mms/util/PduCache;-normalizeKey-(Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IVibratorService$Stub;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/media/tv/TvInputInfo;-loadCustomLabel-(Landroid/content/Context;)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/content/IContentService$Stub;-isSyncPendingAsUser-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName; I)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/am/ActivityManagerService;-startProcessLocked-(Ljava/lang/String; Landroid/content/pm/ApplicationInfo; Z I Ljava/lang/String; Landroid/content/ComponentName; Z Z I Z Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/Runnable;)Lcom/android/server/am/ProcessRecord;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/tech/BasicTagTechnology;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-getNetworkCapabilities-(Ljava/lang/String;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/job/IJobScheduler;-schedule-(Landroid/app/job/JobInfo;)I' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager$CardEmulationInterface;-isDefaultServiceForAid-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-removeAccountAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMSIForApp-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/job/JobService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ActivityManager;-killBackgroundProcesses-(Ljava/lang/String;)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/internal/telephony/UiccPhoneBookController;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/server/am/ActivityManagerService;-scheduleStartProfilesLocked-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wm/WindowManagerService;-createWatermarkInTransaction-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/SearchView$SearchAutoComplete;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/CaptioningManager$1;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Space;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/ShutdownActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BluetoothManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/telecom/ConnectionService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-startScan-(Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/ValidateNotificationPeople;-access$500-(Lcom/android/server/notification/ValidateNotificationPeople; Landroid/content/Context; Ljava/lang/String;)Lcom/android/server/notification/ValidateNotificationPeople$LookupResult;' : ['android.permission.READ_CONTACTS'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccPhoneBookController;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-registerForSettingsChanges-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-onDisplayRemoved-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothAvrcpController;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getGlobalProxyAdmin-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseAccountTypeActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Settings$Secure;-getLocationModeForUser-(Landroid/content/ContentResolver; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-setDataRoaming-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-statusBarVisibilityChanged-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/ContextThemeWrapper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/GrainFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/renderscript/RSTextureView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/test/SimulatedRadioControl;-shutdown-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager;-access$100-(Lcom/android/server/content/SyncManager;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/hdmi/HdmiCecFeatureAction;-handleTimerEvent-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setLockTaskPackages-(Landroid/content/ComponentName; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub$Proxy;-cancelRequest-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/SubscriptionController;-addSubInfoRecord-(Ljava/lang/String; I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getSubscriberIdForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendRawPdu-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/widget/RemoteViewsService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager;-isServiceRegistered-(I Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/filterpacks/imageproc/CropFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/InputMethodService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiStateMachine$DriverStoppedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-init-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IccCard;-setIccFdnEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-showBootMessage-(Ljava/lang/CharSequence; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-dismissAndSwitch-()V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/server/wm/WindowManagerService;-access$500-(Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/pm/PackageManagerService;-access$2300-(Lcom/android/server/pm/PackageManagerService; Lcom/android/server/pm/PackageManagerService$InstallArgs; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accounts/AccountManagerService$11;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-requestSyncAsUser-(Landroid/accounts/Account; Ljava/lang/String; I Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-iccOpenLogicalChannel-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeechService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/tv/TvInputManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiSettingsStore;-isWifiToggleEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$16;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager$Stub;-requestBatchedScan-(Landroid/net/wifi/BatchedScanSettings; Landroid/os/IBinder; Landroid/os/WorkSource;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telecom/InCallService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-showCircularMask-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-queryFacilityLock-(Ljava/lang/String; Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-getSettingsGlobalBoolean-(Landroid/content/ContentResolver; Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/SipAudioCall;-stopCall-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/dreams/DreamManagerService;-access$1200-(Lcom/android/server/dreams/DreamManagerService; I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setCallForward-(I I I Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipCall;-hangupIfAlive-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiNotificationController$NotificationEnabledSettingObserver;-getValue-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub;-getSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/ProxyController;-disableDataConnectivity-(I Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/backup/SystemBackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/job/JobServiceContext$JobServiceHandler;-sendStopMessageH-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Landroid/telephony/SmsManager;-divideMessage-(Ljava/lang/String;)Ljava/util/ArrayList;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/IEthernetManager$Stub$Proxy;-getConfiguration-()Landroid/net/IpConfiguration;' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Lcom/android/server/notification/ValidateNotificationPeople$PeopleRankingReconsideration;-run-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-sendUssd-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/SearchView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BootReceiver;-logBootEvents-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/preference/PreferenceActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/CallerInfo;-getCallerInfo-(Landroid/content/Context; Ljava/lang/String;)Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/wifi/WifiScanningServiceImpl$WifiScanningStateMachine$PausedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/provider/ContactsContract$RawContacts;-getContactLookupUri-(Landroid/content/ContentResolver; Landroid/net/Uri;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/VibratorService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.VIBRATE'],
	'Landroid/printservice/PrintService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/WebVttRenderingWidget$SpanLayout;-setStyle-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-clearBlacklist-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/print/IPrintManager;-restartPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-setRecommendedGlobalProxy-(Landroid/content/ComponentName; Landroid/net/ProxyInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/ClosedCaptionWidget$CCLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeechService;-getDefaultSpeechRate-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getReportXtOverDev-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-getAcceptableAccountChoices-(Landroid/accounts/AccountManager;)Ljava/util/ArrayList;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-onUninstalled-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SubscriptionManager;-addSubInfoRecord-(Ljava/lang/String; I)Landroid/net/Uri;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onTrySetupData-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService;-getCurrentSpellChecker-(Ljava/lang/String;)Landroid/view/textservice/SpellCheckerInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getTrustAgentFeaturesEnabled-(Landroid/content/ComponentName; Landroid/content/ComponentName; I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/WiredAccessoryManager$WiredAccessoryObserver;-onUEvent-(Landroid/os/UEventObserver$UEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-updateCredentials-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/app/IUiAutomationConnection;-shutdown-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ActivityGroup;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/AudioService$ScoClient;-decCount-()V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/wifi/WifiApConfigStore$InactiveState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/ObjectSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-enableScreenIfNeededLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enableCdmaBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/sip/SipService;-stopPortMappingMeasurement-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-separateConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/dreams/DreamManagerService;-stopDreamInternal-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/IRecognitionService$Stub$Proxy;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/server/wifi/WifiStateMachine;-<init>-(Landroid/content/Context; Ljava/lang/String; Lcom/android/server/wifi/WifiTrafficPoller;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS', 'android.permission.SEND_SMS'],
	'Landroid/media/effect/effects/BrightnessEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/gsm/UsimDataDownloadHandler;-acknowledgeSmsWithError-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone$2;-onECBMExited-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ImageView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/LauncherAppsService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/notification/NotificationListenerService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getCurrentSyncsAsUser-(I)Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/app/admin/IDevicePolicyManager;-getCrossProfileCallerIdDisabledForUser-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$SupplicantStoppingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-setCdmaSubscriptionSource-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/GpsNavigationMessageListenerTransport;-registerWithServer-()Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/DownloadManager;-getRecommendedMaxBytesOverMobile-(Landroid/content/Context;)Ljava/lang/Long;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getPermittedInputMethods-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover;-startBonding-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/filterpacks/base/ObjectSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub;-getAllSubInfoList-()Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-startUserInForeground-(I Landroid/app/Dialog;)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-getAllowedResolutionLevel-(I I)I' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/LocationManager;-removeUpdates-(Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-formatNdef-(I [B)I' : ['android.permission.NFC'],
	'Landroid/widget/TextView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SubscriptionManager;-setDefaultDataSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-setupDataCall-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/ChooseAccountTypeActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/dreams/DreamManagerService;-access$800-(Lcom/android/server/dreams/DreamManagerService; Ljava/io/PrintWriter;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/UiModeManagerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/webkit/WebView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-access$100-(Lcom/android/server/accessibility/ScreenMagnifier; I I I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/app/PlatLogoActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/ContentResolver;-requestSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/content/SyncManager;-access$2000-(Lcom/android/server/content/SyncManager;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-onUpdatePhoneObject-(Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/am/Am;-runScreenCompat-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/content/SyncManager;-onUserStopping-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/inputmethod/InputMethodSubtypeSwitchingController;-getSortedInputMethodAndSubtypeListLocked-(Z Z Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CalendarView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/os/IPowerManager;-acquireWakeLock-(Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; Landroid/os/WorkSource; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaPlayer;-create-(Landroid/content/Context; Landroid/net/Uri; Landroid/view/SurfaceHolder; Landroid/media/AudioAttributes; I)Landroid/media/MediaPlayer;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/RIL;-deactivateDataCall-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/IsoDep;-close-()V' : ['android.permission.NFC'],
	'Landroid/nfc/cardemulation/CardEmulation;-setPreferredService-(Landroid/app/Activity; Landroid/content/ComponentName;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-setReaderMode-(Landroid/os/IBinder; Landroid/nfc/IAppCallback; I Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService$VibrateThread;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/server/TelephonyRegistry;-notifyDataConnectionFailedForSubscriber-(J Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager$5;-startTask-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-readPolicyLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SubscriptionManager;-setDisplayNumberFormat-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/SmsUsageMonitor;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-registerForSuppServiceNotification-(Landroid/os/Handler; I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setSecureSetting-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub;-copyMessageToIccEfForSubscriber-(J Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/wifi/WifiServiceImpl;-isScanAlwaysAvailable-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/util/AsyncService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/admin/IDevicePolicyManager;-getTrustAgentFeaturesEnabled-(Landroid/content/ComponentName; Landroid/content/ComponentName; I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-access$6400-(Landroid/media/AudioService;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendCDMAFeatureCode-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneFactory;-getCdmaPhone-(I)Lcom/android/internal/telephony/Phone;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/accounts/IAccountManager;-addAccountAsUser-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Z Landroid/os/Bundle; I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/location/ComprehensiveCountryDetector$1;-onCountryDetected-(Landroid/location/Country;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-handlePollStateResult-(I Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-handleSrvccStateChanged-([I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccountsByFeatures-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/test/LaunchPerformanceBase;-finish-(I Landroid/os/Bundle;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/IAccountManager;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/filterpacks/base/FrameFetch;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/CommandsInterface;-setCdmaSubscriptionSource-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag;-getTechList-(I)[I' : ['android.permission.NFC'],
	'Landroid/location/LocationManager;-getLastLocation-()Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioManager;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub;-getCurRecognizer-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-makeInboundSmsHandler-(Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase;)Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CheckBox;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-sendDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getScreenCaptureDisabled-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/trust/TrustAgentService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/RenamingDelegatingContext;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-deleteStoredMessage-(Ljava/lang/String; Landroid/net/Uri;)Z' : ['android.permission.WRITE_SMS'],
	'Landroid/net/wifi/IWifiManager$Stub;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/tech/NfcB;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/internal/app/DisableCarModeActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Editor$HandleView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifyDataConnectionFailedForSubscriber-(J Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/BeamShareActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneBase;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayManagerService;-setDisplayInfoOverrideFromWindowManagerInternal-(I Landroid/view/DisplayInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/hdmi/HdmiControlService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-reassociate-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/INfcCardEmulation$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-getNetworkCapabilities-(Ljava/lang/String;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setApplicationHidden-(Landroid/content/ComponentName; Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GeofenceManager;-onLocationChanged-(Landroid/location/Location;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/LauncherActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMEISV-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/UiAutomationConnection;-shutdown-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine$TetheringState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-removeAccountAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/widget/LockPatternUtils;-setOwnerInfo-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-xtraDownloadRequest-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getImeiForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getIccSerialNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-addToBlacklist-(Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-getPasswordMinimumLetters-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipWakeLock;-reset-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/SettingInjectorService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ActionMenuView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManagerNative;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.REORDER_TASKS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.GET_TASKS', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ChooserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Settings$Secure;-getString-(Landroid/content/ContentResolver; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/EditText;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/SyncActivityTooManyDeletes;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/bluetooth/IBluetoothManager;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/accounts/AccountManager$12;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-setCountryListener-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-getNetworkSelectionMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/NativeActivity$NativeContentView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/connectivity/Tethering;-<init>-(Landroid/content/Context; Landroid/os/INetworkManagementService; Landroid/net/INetworkStatsService; Landroid/os/Looper;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccPhoneBookController;-updateAdnRecordsInEfByIndexForSubscriber-(J I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.WAKE_LOCK'],
	'Landroid/app/NativeActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/GrainFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-setVoiceRadioTech-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/accounts/AccountManager$GetAuthTokenByTypeAndFeaturesTask$1$1;-onResult-(Landroid/os/Bundle;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/server/accessibility/TouchExplorer$SendHoverEnterAndMoveDelayed;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/app/IActivityContainer$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/ManagedServices;-onBootPhaseAppsCanStart-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setPasswordMinimumLetters-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-readEnhancedWebAccessibilityEnabledChangedLocked-(Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/display/DisplayManagerInternal;-setDisplayInfoOverrideFromWindowManager-(I Landroid/view/DisplayInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/LocationBasedCountryDetector$2;-run-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-getRecoveryAction-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/TextProgressBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/cardemulation/RegisteredServicesCache;-invalidateCache-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/app/Activity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/Switch;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/InputMethodManagerService;-setInputMethodEnabledLocked-(Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManagerNative;-killBackgroundProcesses-(Ljava/lang/String; I)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Landroid/telephony/SmsManager;-updateStoredMessageStatus-(Landroid/net/Uri; Landroid/content/ContentValues;)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone$1;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-addCrossProfileIntentFilter-(Landroid/content/ComponentName; Landroid/content/IntentFilter; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/SupplicantStateTracker$DormantState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Lcom/android/server/sip/SipWakeLock;-acquire-(J)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SubscriptionManager;-getSubInfoUsingSlotId-(I)Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/sip/SipSessionGroup;-process-(Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/WiredAccessoryManager;-systemReady-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$DriverStartingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager$CardEmulationInterface;-isDefaultServiceForCategory-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/nfc/NfcService$EnableDisableTask;-enableInternal-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ChooserActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcAdapter;-removeNfcUnlockHandler-(Landroid/nfc/INfcUnlockHandler;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiController$ApEnabledState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-updateAppOpsState-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Settings$System;-getFloatForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-getBaseDisplaySize-(I Landroid/graphics/Point;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/text/ToUpperCase;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/backup/BackupManagerService$PerformUnifiedRestoreTask;-startRestore-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-checkAndEnableDataCallAfterEmergencyCallDropped-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-enableNetwork-(I Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Settings$Global;-getUriFor-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-allowedToSetDeviceOwnerOnDevice-()Z' : ['android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/wm/WindowManagerService;-onDisplayChanged-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/os/IPowerManager$Stub;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/widget/RemoteViews$2;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/BluetoothManagerService;-access$300-(Lcom/android/server/BluetoothManagerService;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/CallbackFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/BroadcastQueue;-sendPendingBroadcastsLocked-(Lcom/android/server/am/ProcessRecord;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/BarController;-setBarShowingLw-(Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/InputMethodManagerService;-getCurrentInputMethodSubtypeLocked-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-access$1300-(Lcom/android/server/location/GpsLocationProvider;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/trust/TrustManagerService$3;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardware;-removeGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-switchToLastInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-systemBooted-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-setKeyboard-(Landroid/inputmethodservice/Keyboard;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/TintFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/PacManager;-getPacChangeDelay-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/am/Am;-main-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/IWifiManager;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TelephonyRegistry;-checkListenerPermission-(I)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-resetTimeouts-()V' : ['android.permission.NFC'],
	'Landroid/filterpacks/videosrc/CameraSource;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl$3;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$ErrorState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-handleApplicationWtf-(Landroid/os/IBinder; Ljava/lang/String; Z Landroid/app/ApplicationErrorReport$CrashInfo;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-reportStkServiceIsRunning-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setRecommendedGlobalProxy-(Landroid/content/ComponentName; Landroid/net/ProxyInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ContrastFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub$Proxy;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/NsdService;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/am/ActivityManagerService;-appDiedLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCallForwardStatus-(I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager;-setPasswordMinimumSymbols-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setMasterVolumeMuted-(Landroid/content/ComponentName; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/cardemulation/OffHostApduService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/accessibility/TouchExplorer;-access$2800-(Lcom/android/server/accessibility/TouchExplorer; Landroid/view/MotionEvent; I I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-onSubscriptionActivated-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/commands/pm/Pm;-runSetInstallLocation-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService$Lifecycle;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/ethernet/EthernetServiceImpl;-enforceChangePermission-()V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Lcom/android/server/fingerprint/FingerprintService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-enforceSubscriptionPermission-()V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/widget/LockPatternUtils;-isVisiblePatternEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/text/format/DateUtils;-formatDateRange-(Landroid/content/Context; Ljava/util/Formatter; J J I Ljava/lang/String;)Ljava/util/Formatter;' : ['android.permission.READ_PROFILE'],
	'Landroid/preference/PreferenceActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowManagerService;-updateStatusBarVisibilityLocked-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/SettingInjectorService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/IPowerManager$Stub;-updateWakeLockUids-(Landroid/os/IBinder; [I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/print/PrintManagerService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/backup/BackupManagerService$BackupRestoreTask;-operationComplete-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/IMediaHTTPConnection$Stub;-getMIMEType-()Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-openPanel-(I Landroid/view/KeyEvent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/IActivityManager;-moveTaskBackwards-(I)V' : ['android.permission.REORDER_TASKS'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub;-isSMSPromptEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/ToUpperCase;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-pollStateDone-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/ChooseAccountTypeActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-setWifiEnabled-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/UiccSmsController;-getImsSmsFormat-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-updatePhoneObject-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/widget/LockPatternUtils;-removeAppWidget-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-removePhoneStateListener-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/AdapterViewFlipper;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-resetPassword-(Ljava/lang/String; I I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ToggleButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-unbroadcastIntent-(Landroid/app/IApplicationThread; Landroid/content/Intent; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/backup/BackupManagerService$PerformAdbRestoreTask;-restoreOneFile-(Ljava/io/InputStream; [B)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getHardwareConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/TouchExplorer$ExitGestureDetectionModeDelayed;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Contacts$ContactMethods;-addPostalLocation-(Landroid/content/Context; J D D)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/provider/Settings$Secure;-isLocationProviderEnabled-(Landroid/content/ContentResolver; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryCLIP-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-access$700-(Lcom/android/internal/telephony/RIL; I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/GlobalActions$PowerAction;-onLongPress-()Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/AccessibilityController$DisplayMagnifier;-onRotationChangedLocked-(Lcom/android/server/wm/DisplayContent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wm/AccessibilityController;-onWindowTransitionLocked-(Lcom/android/server/wm/WindowState; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/UiccSmsController;-enableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Landroid/service/trust/TrustAgentService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/cardemulation/DefaultRemovedActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-updateEFLinearFixed-(I I [B Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getNeighboringCids-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/preference/PreferenceFrameLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothHeadsetClient;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/AlarmManagerService$AlarmThread;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-updateCredentials-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String; Z Landroid/os/Bundle;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-setTemporaryScreenAutoBrightnessAdjustmentSettingOverrideInternal-(F)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/CallLog$Calls;-addCall-(Lcom/android/internal/telephony/CallerInfo; Landroid/content/Context; Ljava/lang/String; I I I Landroid/telecom/PhoneAccountHandle; J I Ljava/lang/Long; Z)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/accounts/CantAddAccountActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/LocationProviderInterface;-enable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/ContentService;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/net/wifi/WifiManager;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-isMasterVolumeMuted-(Landroid/content/ComponentName;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-access$3100-(Lcom/android/server/power/PowerManagerService; Landroid/os/IBinder; I Ljava/lang/String; Ljava/lang/String; Landroid/os/WorkSource; Ljava/lang/String; I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/SlidingTab;-vibrate-(J)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/http/SslCertificate;-inflateCertificateView-(Landroid/content/Context;)Landroid/view/View;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/job/JobService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/NsdService$NsdStateMachine$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getUidPersistBytes-(J)J' : ['android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/CaptioningManager$CaptionStyle;-getCustomStyle-(Landroid/content/ContentResolver;)Landroid/view/accessibility/CaptioningManager$CaptionStyle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStack;-goToSleep-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneBase;-setCdmaSubscription-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/IPowerManager$Stub;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AutoCompleteTextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-addCrossProfileWidgetProvider-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/LayoutInflater$BlinkLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-setIccFdnEnabled-(Z Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-updateMessageOnIccEf-(Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getConfigFile-()Ljava/lang/String;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-addTrackerToRawTable-(Lcom/android/internal/telephony/InboundSmsTracker;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/wifi/WifiStateMachine;-setCountryCode-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiController$FullLockHeldState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/NsdService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.INTERNET'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceAddStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/backup/BackupManagerService$PerformAdbBackupTask;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-findNextAlarmTime-(Landroid/content/ContentResolver; J)J' : ['android.permission.READ_CALENDAR'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/trust/TrustManagerService;-requireCredentialEntry-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/CellBroadcastMessage;-getSpokenDateString-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService$3;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/BatteryService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-getProviders-(Landroid/location/Criteria; Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/telephony/gsm/SmsManager;-divideMessage-(Ljava/lang/String;)Ljava/util/ArrayList;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/ActivityManagerNative;-killUid-(I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-setProfileOwner-(Landroid/content/ComponentName; Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-access$7500-(Landroid/media/AudioService;)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$3300-(Lcom/android/server/accessibility/AccessibilityManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-reloadAudioSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifyDataConnection-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-access$200-(Lcom/android/server/devicepolicy/DevicePolicyManagerService; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/FilterPort;-pushFrame-(Landroid/filterfw/core/Frame;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation$Stub;-unsetPreferredService-()Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/server/VibratorService;-updateInputDeviceVibrators-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/power/ShutdownThread;-reboot-(Landroid/content/Context; Ljava/lang/String; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/net/NetworkPolicyManagerService$7;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/handover/HandoverService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiStateMachine$DriverStartedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/VibratorService$SettingsObserver;-onChange-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendRawPdu-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/am/ActivityManagerService$16;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ReceiverRestrictedContext;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupForegroundResumeBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-setPermittedAccessibilityServices-(Landroid/content/ComponentName; Ljava/util/List;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IUserSwitchObserver$Stub$Proxy;-onUserSwitching-(I Landroid/os/IRemoteCallback;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/TapAgainDialog;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeech;-setEngineByPackageName-(Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/SpellChecker;-spellCheck-(I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/webkit/WebViewFactory;-prepareWebViewInSystemServer-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/IntentFirewallInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ConditionProviders;-disableNonexistentServices-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub$Proxy;-getSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-setDefaultSmsSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/pm/IPackageManager;-freeStorage-(J Landroid/content/IntentSender;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-setAppCallback-(Landroid/nfc/IAppCallback;)V' : ['android.permission.NFC'],
	'Lcom/android/nfc/TechListChooserActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/ChooseAccountActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/nfc/INfcAdapter;-setP2pModes-(I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wallpaper/WallpaperManagerService$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-updateAdnRecordsInEfByIndexForSubscriber-(J I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/notification/NotificationManagerService$SettingsObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayPowerState;-<init>-(Lcom/android/server/display/DisplayBlanker; Lcom/android/server/lights/Light; Lcom/android/server/display/ColorFade;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/MultiAutoCompleteTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/Invert;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccSmsController;-enableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/display/AutomaticBrightnessController;-updateAmbientLux-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/CaptioningManager;-getLocale-()Ljava/util/Locale;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseAccountTypeActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/P2pLinkManager$SendTask;-doHandover-([Landroid/net/Uri;)I' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wifi/WifiStateMachine$DisconnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wallpaper/WallpaperManagerService$MyPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-updateCurrentCarrierInProvider-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/VibratorService;-vibrate-(I Ljava/lang/String; J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/telephony/SmsManager;-sendTextMessage-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/ContextWrapper;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-destroyPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/InputStreamSource;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager$MulticastLock;-finalize-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/SyncRunner;-processFilterNode-(Landroid/filterfw/core/Filter;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-close-(I)I' : ['android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/net/wifi/IWifiManager$Stub;-getVerboseLoggingLevel-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-writeOutboxMessage-(J Ljava/lang/String; Ljava/lang/String; Z Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-separate-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/wm/Wm;-main-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/sip/SipSessionGroup$KeepAliveProcessCallbackProxy$2;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayAdapter$Listener;-onDisplayDeviceEvent-(Lcom/android/server/display/DisplayDevice; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/net/NetworkStatsService$DropBoxNonMonotonicObserver;-foundNonMonotonic-(Landroid/net/NetworkStats; I Landroid/net/NetworkStats; I Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getString-(Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms;-isSMSPromptEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodSubtypeSwitchingController;-resetCircularListLocked-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/SystemService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/InputMethodManagerService;-startInputUncheckedLocked-(Lcom/android/server/InputMethodManagerService$ClientState; Lcom/android/internal/view/IInputContext; Landroid/view/inputmethod/EditorInfo; I)Lcom/android/internal/view/InputBindResult;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCallDeflectionIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipManager;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/internal/content/PackageMonitor;-onSomePackagesChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-onInstalled-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneBase;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-updateImsPhone-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/InputMethodManagerService;-startInputLocked-(Lcom/android/internal/view/IInputMethodClient; Lcom/android/internal/view/IInputContext; Landroid/view/inputmethod/EditorInfo; I)Lcom/android/internal/view/InputBindResult;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-switchUser-(Landroid/content/ComponentName; Landroid/os/UserHandle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-pollState-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingEncryptedRfcommWithServiceRecord-(Ljava/lang/String; Ljava/util/UUID;)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/sip/SipPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/voice/VoiceInteractionService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/WindowManagerInternal;-setMagnificationCallbacks-(Landroid/view/WindowManagerInternal$MagnificationCallbacks;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$WatchdogEnabledState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/app/IActivityManager;-updateConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/IWindowManager;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accounts/AccountManagerService;-renameAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameBranch;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeech$15;-run-(Landroid/speech/tts/ITextToSpeechService;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-checkListenerOrIntentLocked-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; I I Ljava/lang/String; Landroid/os/WorkSource; Z)Lcom/android/server/LocationManagerService$Receiver;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/DayPickerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-rebuildAppWindowListLocked-(Lcom/android/server/wm/DisplayContent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/media/MediaSessionService;-onStart-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doRestoreFile-(Landroid/os/ParcelFileDescriptor; J I Ljava/lang/String; Ljava/lang/String; J J I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService$Receiver;-callLocationChangedLocked-(Landroid/location/Location;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getNetworkCapabilities-(Ljava/lang/String;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Landroid/location/SettingInjectorService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendImsGsmSms-(Ljava/lang/String; Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SmsMessage;-shouldAppendPageNumberAsPrefix-()Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/service/wallpaper/WallpaperService$IWallpaperEngineWrapper;-executeMessage-(Landroid/os/Message;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneProxy;-updateDataConnectionTracker-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setInitialAttachApn-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AbsListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/commands/settings/SettingsCmd;-main-([Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-dispatchMessageRadioSpecific-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/widget/SlidingTab;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/PhoneNumberUtils;-isEmergencyNumber-(Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Service;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/BatteryService$1;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-updateDreamLocked-(I Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-switchUser-(Landroid/content/ComponentName; Landroid/os/UserHandle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-changeIccPinForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-disableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/appwidget/AppWidgetService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-getMaximumTimeToLock-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcAdapter;-enableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/backup/LocalTransportService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/base/GLTextureSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiScanningService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/DisableCarModeActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/AudioPlaybackQueueItem;-finish-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/SupplicantStateTracker$CompletedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/view/WindowManagerGlobal;-addView-(Landroid/view/View; Landroid/view/ViewGroup$LayoutParams; Landroid/view/Display; Landroid/view/Window;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/AudioService;-checkForRingerModeChange-(I I I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-requestIccSimAuthentication-(I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/LauncherAppsService$LauncherAppsImpl$MyPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/snep/SnepServer;-handleRequest-(Lcom/android/nfc/snep/SnepMessenger; Lcom/android/nfc/snep/SnepServer$Callback;)Z' : ['android.permission.BLUETOOTH'],
	'Landroid/os/Vibrator;-cancel-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/RIL;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/SubtitleController;-doSelectDefaultTrack-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-isUninstallBlocked-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-applyAnimationLocked-(Lcom/android/server/wm/AppWindowToken; Landroid/view/WindowManager$LayoutParams; I Z Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/base/OutputStreamTarget;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPasswordHistoryLength-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager;-getLastInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothAdapter;-listenUsingInsecureRfcommOn-(I)Landroid/bluetooth/BluetoothServerSocket;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/nfc/handover/HandoverService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-setPreferredVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getDataRoamingEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/view/IWindowManager$Stub$Proxy;-keyguardGoingAway-(Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcAsyncChannel;-getNetworkCapabilitiesSync-()Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcRootActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/IsimRecords;-getIsimChallengeResponse-(Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/dreams/DreamManagerService;-access$1900-(Lcom/android/server/dreams/DreamManagerService; Landroid/os/IBinder;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/SearchView$SearchAutoComplete;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/preference/VolumePreference;-onBindDialogView-(Landroid/view/View;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/app/IAlarmManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/content/ContentService;-getMasterSyncAutomaticallyAsUser-(I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/TextServicesManagerService;-setCurrentSpellChecker-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-access$1100-(Lcom/android/internal/policy/impl/PhoneWindowManager; Landroid/view/WindowManagerPolicy$WindowState;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-setDisplayName-(Ljava/lang/String; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/GlobalActions$2;-onItemLongClick-(Landroid/widget/AdapterView; Landroid/view/View; I J)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/SRTTrack;-show-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/view/IApplicationToken$Stub;-windowsVisible-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ActionMenuView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-buildInputMethodListLocked-(Ljava/util/ArrayList; Ljava/util/HashMap; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-access$100-(Lcom/android/server/net/NetworkPolicyManagerService;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/MediaPlayer;-postEventFromNative-(Ljava/lang/Object; I I I Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-getSubscriptionInfoAndStartPollingThreads-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/net/wifi/IWifiManager$Stub;-enableVerboseLogging-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/view/IWindowSession$Stub$Proxy;-addToDisplayWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I I Landroid/graphics/Rect;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-updateCdmaSubscription-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/search/Searchables;-findGlobalSearchActivity-(Ljava/util/List;)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaFocusControl;-dispatchMediaKeyEvent-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-cancelSync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCallWaitingIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/wallpaper/WallpaperService$Engine$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CallerInfo;-getCallerInfo-(Landroid/content/Context; Ljava/lang/String; J)Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_CONTACTS'],
	'Landroid/telephony/SmsManager;-sendMultimediaMessage-(Landroid/content/Context; Landroid/net/Uri; Ljava/lang/String; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/internal/telephony/uicc/UiccCard;-sendEnvelopeWithStatus-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-unregisterAdapter-(Landroid/bluetooth/IBluetoothManagerCallback;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/commands/requestsync/RequestSync;-run-([Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-hangupAndPowerOff-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier$MagnifiedContentInteractonStateHandler;-onMotionEvent-(Landroid/view/MotionEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/CarrierProvisioningUrlsInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/Invert;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IMms;-sendStoredMessage-(J Ljava/lang/String; Landroid/net/Uri; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/server/am/ActivityStackSupervisor$VirtualActivityContainer;-detachLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/display/AutomaticBrightnessController;-access$200-(Lcom/android/server/display/AutomaticBrightnessController; J F)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayManagerService;-access$3200-(Lcom/android/server/display/DisplayManagerService; Landroid/hardware/display/IVirtualDisplayCallback; Landroid/media/projection/IMediaProjection; I Ljava/lang/String; Ljava/lang/String; I I I Landroid/view/Surface; I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccountsAsUser-(Ljava/lang/String; I Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/app/IActivityManager;-updatePersistentConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/os/SystemVibrator;-vibrate-(I Ljava/lang/String; [J I Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wm/WindowManagerService;-updateShowImeWithHardKeyboard-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-createAndInitializeUser-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String; Landroid/content/ComponentName; Landroid/os/Bundle;)Landroid/os/UserHandle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-selectNetworkManually-(Lcom/android/internal/telephony/OperatorInfo; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-onRecordLoaded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/DisplayAdjustmentUtils;-applyAdjustments-(Landroid/content/Context; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/HandoverService;-onTransferComplete-(Lcom/android/nfc/handover/HandoverTransfer; Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/pm/PackageManagerService;-<init>-(Landroid/content/Context; Lcom/android/server/pm/Installer; Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/IAccountManager;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/wifi/WifiStateMachine$SupplicantStartingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMEI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/RedEyeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/WebVttRenderingWidget;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountAuthenticatorActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCLIR-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-rebootSafeMode-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/content/SyncManager;-sendSyncAlarmMessage-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-acknowledgeLastIncomingCdmaSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setPropertiesByCarrier-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub;-setSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/view/WindowManagerPolicy;-getUserRotationMode-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/provider/CalendarContract$Instances;-query-(Landroid/content/ContentResolver; [Ljava/lang/String; J J Ljava/lang/String;)Landroid/database/Cursor;' : ['android.permission.READ_CALENDAR'],
	'Landroid/nfc/tech/Ndef;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/server/backup/SystemBackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub$Proxy;-listenForSubscriber-(J Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/WifiStateMachine$DriverStartingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindow$DecorView;-dispatchKeyEvent-(Landroid/view/KeyEvent;)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/TintFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/NotificationManagerService$NotificationListeners;-access$600-(Lcom/android/server/notification/ManagedServices;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/WindowAnimator;-getPendingLayoutChanges-(I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/pm/LauncherAppsService$LauncherAppsImpl$MyPackageMonitor;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-getReceiverLocked-(Landroid/location/ILocationListener; I I Ljava/lang/String; Landroid/os/WorkSource; Z)Lcom/android/server/LocationManagerService$Receiver;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub;-getCurrentSpellChecker-(Ljava/lang/String;)Landroid/view/textservice/SpellCheckerInfo;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CropFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/google/android/mms/pdu/PduPersister;-persist-(Lcom/google/android/mms/pdu/GenericPdu; Landroid/net/Uri; Z Z Ljava/util/HashMap;)Landroid/net/Uri;' : ['android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Lcom/android/server/LocationManagerService$4;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothHealth;-close-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/Invert;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/SamplingProfilerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/net/DnsPinger;-getDefaultDns-()Ljava/net/InetAddress;' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$Global;-getFloat-(Landroid/content/ContentResolver; Ljava/lang/String;)F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/WeightedLinearLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/CallLog$Calls;-updateNormalizedNumber-(Landroid/content/Context; Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/GLTextureTarget;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-getAdnRecordsInEfForSubscriber-(J I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/LockSettingsService;-getString-(Ljava/lang/String; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/AliasActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/speech/SpeechRecognizer;-setRecognitionListener-(Landroid/speech/RecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/app/admin/IDevicePolicyManager;-setLockTaskPackages-(Landroid/content/ComponentName; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/HandoverService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPasswordMinimumUpperCase-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getPrivilegedConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService$1;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-hasFeatures-(Landroid/accounts/Account; [Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-handleCdmaSubscriptionSource-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-proceedAfterWaitChar-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-addPersistentPreferredActivity-(Landroid/content/ComponentName; Landroid/content/IntentFilter; Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/CallbackFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContentResolver;-isSyncPendingAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/os/Vibrator;-vibrate-(I Ljava/lang/String; J Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/imageproc/FillLightFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountAuthenticatorActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setPowerStateToDesired-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisSpeechItemV1;-<init>-(Landroid/speech/tts/TextToSpeechService; Ljava/lang/Object; I I Landroid/os/Bundle; Ljava/lang/String; Ljava/lang/CharSequence;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService$5;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofenceResume-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-getPrivilegedConfiguredNetworks-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/VpnService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/WifiManager;-setWifiApConfiguration-(Landroid/net/wifi/WifiConfiguration;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/MmsServiceBroker;-access$1000-()Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getCurrentFailedPasswordAttempts-(I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/commands/am/Am;-run-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/sip/ISipService;-close-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Landroid/widget/ImageButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SmsManager;-disableCellBroadcast-(I)Z' : ['android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-separate-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accessibilityservice/AccessibilityService;-getWindows-()Ljava/util/List;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/snep/SnepServer;-start-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/LocationManagerService;-requestGeofence-(Landroid/location/LocationRequest; Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/base/CallbackFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropRectFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-doConnect-(Landroid/net/wifi/WifiManager;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IMms;-archiveStoredConversation-(Ljava/lang/String; J Z)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/server/SystemServiceManager;-startUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/EventStreamTransformation;-onDestroy-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setCrossProfileCallerIdDisabled-(Landroid/content/ComponentName; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-handleCallSetupRequestFromSim-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/text/method/MultiTapKeyListener;-onKeyDown-(Landroid/view/View; Landroid/text/Editable; I Landroid/view/KeyEvent;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ExpandableListActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getAllCellInfo-()Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameStore;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/MediaHTTPConnection;-readAt-(J [B I)I' : ['android.permission.INTERNET'],
	'Landroid/widget/ProgressBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-setAllowScansWithTraffic-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-registerForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$3;-onCallTerminated-(Lcom/android/ims/ImsCall; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/PowerManager$WakeLock;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$TetheredState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-attachApplicationLocked-(Lcom/android/server/am/ProcessRecord;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/WifiManager;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/ILocationManager$Stub$Proxy;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/am/BroadcastQueue;-skipCurrentReceiverLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/app/HeavyWeightSwitcherActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-setDisplayNumber-(Ljava/lang/String; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/content/SyncManager;-sendCancelSyncsMessage-(Lcom/android/server/content/SyncStorageEngine$EndPoint; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/AccountManager;-getUserData-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService$4;-onSomePackagesChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-postSyncExpiryMessage-(Lcom/android/server/content/SyncManager$ActiveSyncContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/BluetoothDevice;-createRfcommSocket-(I)Landroid/bluetooth/BluetoothSocket;' : ['android.permission.BLUETOOTH'],
	'Landroid/nfc/INfcAdapter$Stub;-setAppCallback-(Landroid/nfc/IAppCallback;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getGroupIdLevel1ForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/media/MediaSessionService$SettingsObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/MountService;-copyLocaleFromMountService-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-dispatchMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/power/PowerManagerService;-releaseWakeLockInternal-(Landroid/os/IBinder; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM$SetDnsForwardersErrorState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager;-isScanAlwaysAvailable-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/content/ContextWrapper;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/LocationManagerService;-addGpsNavigationMessageListener-(Landroid/location/IGpsNavigationMessageListener; Ljava/lang/String;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getSelectedInputMethodSubtypeHashCode-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/accessibilityservice/AccessibilityService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/IWallpaperManager$Stub$Proxy;-setDisplayPadding-(Landroid/graphics/Rect;)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/server/wifi/WifiServiceImpl;-enforceChangePermission-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-acknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/NetInitiatedActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/ims/internal/IImsCallSessionListener;-callSessionHoldFailed-(Lcom/android/ims/internal/IImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ListActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getVoiceMailNumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/tv/TvInputInfo$TvInputSettings;-isHidden-(Landroid/content/Context; Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-pollCallsWhenSafe-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareClassic;-connect-()V' : ['android.permission.NFC'],
	'Lcom/android/server/notification/ZenModeHelper$SettingsObserver;-<init>-(Lcom/android/server/notification/ZenModeHelper; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiStateMachine;-handleBatchedScanPollRequest-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-isSyncPending-(Landroid/accounts/Account; Ljava/lang/String;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/usb/UsbService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/media/tv/TvInputInfo;-isHidden-(Landroid/content/Context;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaFocusControl$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/TtmlTrack;-show-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-handleUp-(Landroid/view/MotionEvent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-fetchIsimRecords-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-startDtmf-(C)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActiveServices;-forceStopLocked-(Ljava/lang/String; I Z Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-onCancelled-(Landroid/filterfw/core/AsyncRunner$RunnerResult;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-getBaseDisplayDensity-(I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-ndefWrite-(I Landroid/nfc/NdefMessage;)I' : ['android.permission.NFC'],
	'Lcom/android/server/dreams/DreamManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/Ringtone;-stop-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/display/AutomaticBrightnessController$1;-onSensorChanged-(Landroid/hardware/SensorEvent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub$Proxy;-setCurrentSpellChecker-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-saveLastInputMethodWindowForTransition-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/sip/ISipService$Stub;-open-(Landroid/net/sip/SipProfile;)V' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupForegroundResumeBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-access$1100-(Lcom/android/server/accessibility/ScreenMagnifier;)F' : ['android.permission.READ_PROFILE'],
	'Landroid/media/tv/TvInputInfo$TvInputSettings;-access$000-(Landroid/content/Context; Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/media/MediaSessionService$SessionManagerImpl$KeyEventWakeLockReceiver;-onSendFinished-(Landroid/app/PendingIntent; Landroid/content/Intent; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/ZenModeHelper;-setConfig-(Landroid/service/notification/ZenModeConfig;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/media/projection/MediaProjectionManagerService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPasswordExpiration-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/job/JobServiceContext$JobServiceHandler;-handleFinishedH-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/RegisteredAidCache;-onServicesUpdated-(I Ljava/util/List;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/app/IntentForwarderActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/am/ActivityManagerService;-forceStopPackageLocked-(Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/view/menu/MenuBuilder;-changeMenuMode-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-notifySrvccState-(Lcom/android/internal/telephony/Call$SrvccState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TabWidget;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-setExternalState-(Lcom/android/internal/telephony/IccCardConstants$State;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/location/ILocationManager$Stub;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/filterpacks/imageproc/CropFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmMmiCode;-makeCFQueryResultMessage-(Lcom/android/internal/telephony/CallForwardInfo; I)Ljava/lang/CharSequence;' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/ContextThemeWrapper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/WebVttRenderingWidget;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/CarrierProvisioningUrlsInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/BootReceiver;-addFileWithFootersToDropBox-(Landroid/os/DropBoxManager; Landroid/content/SharedPreferences; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcAdapter;-setForegroundDispatch-(Landroid/app/PendingIntent; [Landroid/content/IntentFilter; Landroid/nfc/TechListParcel;)V' : ['android.permission.NFC'],
	'Landroid/os/IVibratorService$Stub$Proxy;-vibratePattern-(I Ljava/lang/String; [J I I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFTransparent-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/View$PerformClick;-run-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DateTimeView;-getTimeFormat-()Ljava/text/DateFormat;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/tv/TvInputInfo$TvInputSettings;-access$100-(Landroid/content/Context; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/SingleFilterEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/INfcTag$Stub;-rediscover-(I)Landroid/nfc/Tag;' : ['android.permission.NFC'],
	'Landroid/media/tv/TvView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$L2ConnectedState;-exit-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/NfcBackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/DockObserver;-updateLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/base/RetargetFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/MutableContextWrapper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/AudioService;-readAudioSettings-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/dreams/DreamManagerService$5;-onDreamStopped-(Landroid/os/Binder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/UiccCard;-iccTransmitApduLogicalChannel-(I I I I I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-setRingerMode-(I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/HdmiCecLocalDeviceTv;-onNewAvrAdded-(Landroid/hardware/hdmi/HdmiDeviceInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-appNotResponding-(Lcom/android/server/am/ProcessRecord; Lcom/android/server/am/ActivityRecord; Lcom/android/server/am/ActivityRecord; Z Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/TabHost;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/UiAutomationConnection;-connect-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$SupplicantStoppingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipSessionGroup;-processTransactionTerminated-(Ljavax/sip/TransactionTerminatedEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/ITextToSpeechService;-getClientDefaultLanguage-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-access$5300-(Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine; Landroid/net/wifi/p2p/WifiP2pConfig;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TtsEngines;-getLocalePrefForEngine-(Ljava/lang/String;)Ljava/util/Locale;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/nfc/tech/MifareClassic;-increment-(I I)V' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-deleteSmsOnSim-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-dispose-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onHoverEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/ClosedCaptionWidget$CCLineBox;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SmsApplication$SmsPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/TrafficStats;-startDataProfiling-(Landroid/content/Context;)V' : ['android.permission.ACCESS_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/MutableContextWrapper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockApplication;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager$7;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/app/admin/IDevicePolicyManager;-hasUserSetupCompleted-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/SELinuxPolicyInstallReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getKeyguardStoredPasswordQuality-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IWallpaperManager$Stub;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-addAccount-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/ISub$Stub$Proxy;-addSubInfoRecord-(Ljava/lang/String; I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ImageView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipManager;-open-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-reconnect-(I)I' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/RIL;-sendSMS-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-getDataConnectionState-(Ljava/lang/String;)Lcom/android/internal/telephony/PhoneConstants$DataState;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountAuthenticatorActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-onSignalStrengthResult-(Landroid/os/AsyncResult; Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/service/trust/TrustAgentService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/provider/Browser;-deleteHistoryWhere-(Landroid/content/ContentResolver; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/MediaHTTPConnection;-getMIMEType-()Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/ILocationManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_MOCK_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmCall;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-createAndInitializeUser-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String; Landroid/content/ComponentName; Landroid/os/Bundle;)Landroid/os/UserHandle;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/StraightenEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$16500-(Lcom/android/server/wifi/WifiStateMachine; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService$2;-onChange-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/PlatLogoActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/pm/PackageManagerService;-setInstallLocation-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/snep/SnepServer$ServerThread;-run-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/INotificationManager$Stub;-setZenModeConfig-(Landroid/service/notification/ZenModeConfig;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/Tethering;-sendTetherStateChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/job/JobService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/dreams/DreamManagerService;-access$1600-(Lcom/android/server/dreams/DreamManagerService;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/app/AlertActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiServiceImpl;-reportActivityInfo-()Landroid/net/wifi/WifiActivityEnergyInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/widget/SlidingTab;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/media/MediaSessionService$SessionManagerImpl;-dispatchMediaKeyEventLocked-(Landroid/view/KeyEvent; Z Lcom/android/server/media/MediaSessionRecord;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService$BatteryReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothA2dpSink;-close-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/le/BluetoothLeAdvertiser;-startAdvertising-(Landroid/bluetooth/le/AdvertiseSettings; Landroid/bluetooth/le/AdvertiseData; Landroid/bluetooth/le/AdvertiseData; Landroid/bluetooth/le/AdvertiseCallback;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/view/accessibility/IAccessibilityManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/Phone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-handleInCallMmiCommands-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AbsoluteLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-broadcastDefaultDataSubIdChanged-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockContentResolver;-requestSyncAsUser-(Landroid/accounts/Account; Ljava/lang/String; I Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/app/AlertActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-addCrossProfileWidgetProvider-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onRadioAvailable-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager;-screenshotApplications-(Landroid/os/IBinder; I I I Z)Landroid/graphics/Bitmap;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/Invert;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/location/GpsLocationProvider$8;-run-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/monkey/Monkey;-getSystemInterfaces-()Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindow$RotationWatcher;-dispatchRotationChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/power/Notifier;-access$600-(Lcom/android/server/power/Notifier;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiController$FullLockHeldState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-access$300-(Lcom/android/internal/policy/impl/PhoneWindowManager;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/view/IInputMethodManager;-switchToLastInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-addAccountExplicitly-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/accounts/AccountManager$4;-doWork-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/nfc/tech/MifareUltralight;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/server/wm/WindowAnimator;-hideWallpapersLocked-(Lcom/android/server/wm/WindowState;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/Service;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/IsolatedContext$MockAccountManager;-invalidateAuthToken-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.MANAGE_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/net/sip/ISipService$Stub$Proxy;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/filterpacks/imageproc/Invert;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/view/IInputMethodManager$Stub;-getCurrentInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-getInstalledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getCdmaBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/dpm/Dpm;-onRun-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onLlcpLinkActivated-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)V' : ['android.permission.VIBRATE'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/IGeofenceHardware$Stub;-addCircularFence-(I Landroid/hardware/location/GeofenceHardwareRequestParcelable; Landroid/hardware/location/IGeofenceHardwareCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/ActivityChooserView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-startUserInBackground-(I)Z' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-removeNfcUnlockHandler-(Landroid/nfc/INfcUnlockHandler;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$10;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/PhoneBase;-dispose-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cat/IconLoader;-loadIcons-([I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-packageHasActiveAdmins-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-setWifiEnabled-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/RadioButton;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-updateSystemBarsLw-(Landroid/view/WindowManagerPolicy$WindowState; I I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ZenModeHelper;-isMessage-(Lcom/android/server/notification/NotificationRecord;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler$IdleState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPasswordHistoryLength-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-switchUser-(Landroid/content/ComponentName; Landroid/os/UserHandle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ims/ImsEcbm$ImsEcbmListenerProxy;-exitedECBM-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GeofenceManager;-sendIntentExit-(Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/Ringtone;-destroyLocalPlayer-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityRecord$Token;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/tts/TextToSpeech;-getAvailableLanguages-()Ljava/util/Set;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/CertPinInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/NativeCrashListener;-consumeNativeCrashData-(Ljava/io/FileDescriptor;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-detectCountry-()Landroid/location/Country;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/webkit/WebViewUpdateService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/ServiceWatcher$2;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onRemoteFieldDeactivated-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipSessionGroup;-processResponse-(Ljavax/sip/ResponseEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getIccSimChallengeResponse-(J I Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ims/ImsCall$ImsCallSessionListenerProxy;-callSessionResumed-(Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/ImsCallProfile;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController;-access$000-(Lcom/android/server/wifi/WifiController;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodSubtypeSwitchingController;-<init>-(Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings; Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/ims/internal/IImsEcbmListener$Stub;-exitedECBM-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/SystemVibrator;-vibrate-(J)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/notification/ManagedServices;-disableNonexistentServices-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-handleSrvccStateChanged-([I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager;-registerStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/ContextImpl;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/nfc/INfcCardEmulation$Stub;-setDefaultServiceForCategory-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/telephony/SubscriptionManager;-getSubInfoForSubscriber-(J)Landroid/telephony/SubInfoRecord;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-changeIccLockPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/CsimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-isEnhancedPowerReportingSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/os/IPowerManager$Stub$Proxy;-releaseWakeLock-(Landroid/os/IBinder; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/GlobalActions$Action;-onPress-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/accessibility/IAccessibilityManager;-temporaryEnableAccessibilityStateUntilKeyguardRemoved-(Landroid/content/ComponentName; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/GrainEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-requestUtcTime-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-cleanUpApplicationRecordLocked-(Lcom/android/server/am/ProcessRecord; Z Z I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/gsm/SmsMessage;-getSubmitPdu-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Z [B)Landroid/telephony/gsm/SmsMessage$SubmitPdu;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/IAudioService$Stub;-startBluetoothSco-(Landroid/os/IBinder; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAccountsByTypeAsUser-(Ljava/lang/String; Landroid/os/UserHandle;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-onOptionsPanelRotationChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/AccountManager$9;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/Phone;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/performance/ThroughputFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/AsyncService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService;-handleLocationChanged-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/UsimDataDownloadHandler;-handleDataDownload-(Lcom/android/internal/telephony/gsm/SmsMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPermittedAccessibilityServicesForUser-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setScreenCaptureDisabled-(Landroid/content/ComponentName; I Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TwoLineListItem;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-hasWidgetsEnabledInKeyguard-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/appops/AppOpsCommand;-run-([Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Landroid/telephony/TelephonyManager;-getSimSerialNumber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/RotateFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setApplicationRestrictions-(Landroid/content/ComponentName; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/CallbackFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH', 'android.permission.NFC'],
	'Landroid/app/NativeActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-getIMSI-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPermittedAccessibilityServices-(Landroid/content/ComponentName; Ljava/util/List;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setProfileOwner-(Landroid/content/ComponentName; Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setAccountManagementDisabled-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameStore;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverManager;-tryBluetoothHandoverRequest-(Lcom/android/nfc/handover/HandoverManager$BluetoothHandoverData;)Landroid/nfc/NdefMessage;' : ['android.permission.BLUETOOTH'],
	'Landroid/os/NullVibrator;-vibrate-([J I)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-setVoiceMailNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager;-enableScreenIfNeeded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiSettingsStore;-handleWifiScanAlwaysAvailableToggled-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager;-sendCheckAlarmsMessage-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/hdmi/HdmiCecLocalDevice;-dispatchMessageToAction-(Lcom/android/server/hdmi/HdmiCecMessage;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub;-removeUpdates-(Landroid/location/ILocationListener; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/ISub;-setDefaultSmsSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/twilight/TwilightService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-onRemoteDisconnect-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/FaceUnlockView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/SizeAdaptiveLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/base/NullFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$ActingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$GroupCreatedState;-exit-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockApplication;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController$DeviceInactiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/NotificationManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcRootActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofenceRemove-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-shutdownRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/test/mock/MockContentResolver;-startSync-(Landroid/net/Uri; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/notification/NotificationSignalExtractor;-initialize-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setFacilityLockForApp-(Ljava/lang/String; Z Ljava/lang/String; I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wm/AccessibilityController$DisplayMagnifier;-<init>-(Lcom/android/server/wm/WindowManagerService; Landroid/view/WindowManagerInternal$MagnificationCallbacks;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPasswordMinimumLength-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getWifiApEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-getPreferredNetworkType-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeechService$SynthesisSpeechItemV1;-setRequestParams-(Landroid/speech/tts/SynthesisRequest;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/ImmersiveModeConfirmation;-loadSetting-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/AliasActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/MountServiceIdler;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-sendCDMAFeatureCode-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/location/ComprehensiveCountryDetector;-access$200-(Lcom/android/server/location/ComprehensiveCountryDetector;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-access$000-(Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/service/dreams/DreamService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/job/JobSchedulerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/SimpleMonthView;-onDraw-(Landroid/graphics/Canvas;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms;-enableCellBroadcastRangeForSubscriber-(J I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/media/AudioService;-access$9700-(Landroid/media/AudioService; I I I Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-handleScreenStateChanged-(Z Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-switchToLastInputMethod-(Landroid/os/IBinder;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Space;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$GroupCreatedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dial-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/telephony/CommandsInterface;-getPreferredVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier$ScreenStateObserver;-handleOnScreenStateChange-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-handleLaunchTaskBehindCompleteLocked-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/trust/TrustManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/CardEmulation;-setDefaultServiceForCategory-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/server/LocationManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_MOCK_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-saveCurrentInputMethodAndSubtypeToHistory-(Ljava/lang/String; Landroid/view/inputmethod/InputMethodSubtype;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifyDataConnection-(I Z Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/net/LinkProperties; Landroid/net/NetworkCapabilities; I Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Settings$Secure;-getInt-(Landroid/content/ContentResolver; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager$13;-doWork-()V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/server/pm/LauncherAppsService$LauncherAppsImpl$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDataSetupCompleteError-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-setGsmBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IMms;-deleteStoredConversation-(Ljava/lang/String; J)Z' : ['android.permission.WRITE_SMS'],
	'Landroid/filterpacks/imageproc/ToPackedGrayFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$Service;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/VignetteEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setRestrictionsProvider-(Landroid/content/ComponentName; Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Application;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CallManager;-dial-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeech;-stop-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-handleInjectSms-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onVoiceCallStarted-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/telephony/TelephonyManager;-getGroupIdLevel1-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager;-getBatchedScanResults-()Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-addPrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-notifyOffApnsOfAvailability-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/display/DisplayManagerService;-access$4300-(Lcom/android/server/display/DisplayManagerService; I Z F Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/NotificationManagerService$5;-setZenModeConfig-(Landroid/service/notification/ZenModeConfig;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService;-routeAids-(Ljava/lang/String; I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/RegisteredServicesCache$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover;-nextStepDisconnect-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/text/format/DateFormat;-getDateFormatOrder-(Landroid/content/Context;)[C' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager;-setTestProviderLocation-(Ljava/lang/String; Landroid/location/Location;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/power/PowerManagerService;-setScreenBrightnessOverrideFromWindowManagerInternal-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/ZoomControls;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-dispatchNormalMessage-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/ui/SurfaceRenderFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager$Stub$Proxy;-removePrintJobStateChangeListener-(Landroid/print/IPrintJobStateChangeListener; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ConditionProviders;-rebindServices-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IActivityManager;-bootAnimationComplete-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/ISyncAdapter;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISub;-setDefaultDataSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/lights/LightsService$1;-setFlashlightEnabled-(Z)V' : ['android.permission.FLASHLIGHT'],
	'Landroid/media/tv/TvInputService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/backup/FullBackupJob;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayPowerController;-sendUpdatePowerState-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-dialInternal-(Ljava/lang/String; Lcom/android/internal/telephony/UUSInfo; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-uninstallCaCert-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/ViewRootImpl;-setView-(Landroid/view/View; Landroid/view/WindowManager$LayoutParams; Landroid/view/View;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/base/NullFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/TouchExplorer$SendHoverExitDelayed;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Landroid/location/LocationRequest; Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/AudioService$AudioHandler;-onPlaySoundEffect-(I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/app/TabActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$TetheredState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/media/Ringtone;-getTitle-(Landroid/content/Context;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/os/IPowerManager$Stub$Proxy;-updateWakeLockWorkSource-(Landroid/os/IBinder; Landroid/os/WorkSource; Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-addAccountAsUser-(Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler; Landroid/os/UserHandle;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/bluetooth/BluetoothHeadset;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/IUiAutomationConnection$Stub;-shutdown-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/effect/effects/SaturateEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-supplyPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/CheckBox;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/ISub;-getActiveSubInfoCount-()I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/NetworkScoreManager;-getActiveScorerPackage-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier$DetectingStateHandler;-onActionTap-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-getPrintJobInfo-(Landroid/print/PrintJobId; I I)Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkPolicyManager$Stub$Proxy;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/AlarmManagerService$2;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/BluetoothManagerService;-storeNameAndAddress-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService$5;-onOpChanged-(I Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-<init>-(Lcom/android/internal/telephony/cdma/CDMAPhone; Landroid/telephony/CellInfo;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/UiccCard;-iccExchangeSimIO-(I I I I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/requestsync/RequestSync;-main-([Ljava/lang/String;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayPowerController$3;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcDisconnectingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/FrameStore;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-ndefMakeReadOnly-(I)I' : ['android.permission.NFC'],
	'Lcom/android/server/print/UserState;-readPrintServicesFromSettingLocked-(Ljava/lang/String; Ljava/util/Set;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPasswordMinimumUpperCase-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/ITextToSpeechService$Stub$Proxy;-synthesizeToFileDescriptor-(Landroid/os/IBinder; Ljava/lang/CharSequence; Landroid/os/ParcelFileDescriptor; Landroid/os/Bundle; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/PackageManagerBackupAgent;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/NfcPermissions;-enforceUserPermissions-(Landroid/content/Context;)V' : ['android.permission.NFC'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-getUserRotationMode-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService$20;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onError-(Landroid/net/sip/SipAudioCall; I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowSession$Stub;-prepareDrag-(Landroid/view/IWindow; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ListActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/backup/BackupManagerService$FullRestoreEngine$RestoreFileRunnable;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getMaximumFailedPasswordsForWipe-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/uiautomator/core/UiAutomationShellWrapper;-connect-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService$Stub;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState;-teardownConnection-(Ljava/lang/String;)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameStore;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SubscriptionManager;-setDisplayName-(Ljava/lang/String; J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/SimPhoneBookInterfaceManager;-getAdnRecordsSize-(I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcActivatingState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wm/TaskStack;-detachDisplay-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-getEnabledPrintServices-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService;-systemRunning-(Lcom/android/server/statusbar/StatusBarManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/UsimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-dial-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameFetch;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverTransfer;-processFiles-()V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$5200-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/service/textservice/SpellCheckerService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/app/ResolverActivity$1;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService$PerformAdbRestoreTask;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/notification/ConditionProviderService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TextClock;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ValidateNotificationPeople;-resolveEmailContact-(Landroid/content/Context; Ljava/lang/String;)Lcom/android/server/notification/ValidateNotificationPeople$LookupResult;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/IMms;-importTextMessage-(Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String; J Z Z)Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/internal/telephony/gsm/GsmCellBroadcastHandler;-handleSmsMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IActivityContainer;-release-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/IAccountManager;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Landroid/accessibilityservice/IAccessibilityServiceConnection$Stub$Proxy;-getWindows-()Ljava/util/List;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-removeAccount-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account;)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-hasGrantedPolicy-(Landroid/content/ComponentName; I I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/Context;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/RemoteViews$2;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService;-setStreamVolume-(I I I Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CheckBox;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareClassic;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-updateEFTransparent-(I [B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ISyncAdapter$Stub$Proxy;-startSync-(Landroid/content/ISyncContext; Ljava/lang/String; Landroid/accounts/Account; Landroid/os/Bundle;)V' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-dialThreeWay-(Ljava/lang/String;)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wallpaper/WallpaperManagerService$MyPackageMonitor;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/accessibilityservice/AccessibilityService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController;-registerForWifiSleepPolicyChange-(Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiController;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getBatchedScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-handleSrvccStateChanged-([I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/MmsServiceBroker;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService$3;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/ContactsContract$RawContacts$EntityIteratorImpl;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Landroid/accounts/AccountManager$11;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/widget/DigitalClock;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-reportGeofenceRemoveStatus-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-onResume-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-updateImsPhone-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/ILocationManager;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/input/InputManagerService;-notifyANR-(Lcom/android/server/input/InputApplicationHandle; Lcom/android/server/input/InputWindowHandle; Ljava/lang/String;)J' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccounts-(Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getDevConfig-()Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings$Config;' : ['android.permission.READ_PROFILE'],
	'Landroid/os/PowerManager$WakeLock;-release-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/PassiveProvider;-updateLocation-(Landroid/location/Location;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setProfileOwner-(Landroid/content/ComponentName; Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-updateAdnRecordsInEfBySearch-(I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/nfc/cardemulation/PreferredServices$SettingsObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/os/IVibratorService$Stub$Proxy;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/pm/BackgroundDexOptService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterfw/core/Filter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/location/LocationManager;-getProviders-(Z)Ljava/util/List;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/PhoneProxy;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/webkit/WebViewUpdateService;-onWebViewUpdateInstalled-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onResetDone-(Landroid/os/AsyncResult;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/INotificationManager$Stub$Proxy;-setZenModeConfig-(Landroid/service/notification/ZenModeConfig;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaFocusControl;-dispatchMediaKeyEventUnderWakelock-(Landroid/view/KeyEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-deleteFromRawTable-(Ljava/lang/String; [Ljava/lang/String;)V' : ['android.permission.WRITE_SMS'],
	'Lcom/android/server/am/ActivityStack;-screenshotActivities-(Lcom/android/server/am/ActivityRecord;)Landroid/graphics/Bitmap;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$AccessibilityContentObserver;-<init>-(Lcom/android/server/accessibility/AccessibilityManagerService; Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setInternalDataEnabled-(Z Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-onCancelled-(Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onTrySetupData-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/provider/CallLog$Calls;-updateDataUsageStatForData-(Landroid/content/ContentResolver; Ljava/lang/String;)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/tech/NfcA;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState$ActedState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/UiccSmsController;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/wifi/WifiStateMachine;-updateBatteryWorkSource-(Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IntRangeManager;-disableRange-(I I Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-dispose-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController;-updateBatteryWorkSource-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/INetworkPolicyManager;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$1;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/WiredAccessoryManager$WiredAccessoryObserver;-init-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-acquireWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/uiautomator/testrunner/UiAutomatorTestRunner;-run-(Ljava/util/List; Landroid/os/Bundle; Z Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeech$15;-run-(Landroid/speech/tts/ITextToSpeechService;)Landroid/speech/tts/Voice;' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/UserDictionary$Words;-addWord-(Landroid/content/Context; Ljava/lang/String; I I)V' : ['android.permission.WRITE_USER_DICTIONARY'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onDisconnectDone-(I Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/AccessibilityController$DisplayMagnifier$MagnifiedViewport;-onRotationChangedLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/handover/HandoverService;-handleCancelTransfer-(Landroid/content/Intent; I)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getDeviceSvn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/ims/internal/ImsCallSession$IImsCallSessionListenerProxy;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub;-createSession-(Landroid/net/sip/SipProfile; Landroid/net/sip/ISipSessionListener;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-processSolicited-(Landroid/os/Parcel;)Lcom/android/internal/telephony/RILRequest;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/twilight/TwilightService$LocationHandler;-retrieveLocation-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/RIL;-getGsmBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/accessibility/IAccessibilityManager;-registerUiTestAutomationService-(Landroid/os/IBinder; Landroid/accessibilityservice/IAccessibilityServiceClient; Landroid/accessibilityservice/AccessibilityServiceInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/SearchDialog$SearchBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-dial-(Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcApplication;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-hasGrantedPolicy-(Landroid/content/ComponentName; I I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IMediaHTTPConnection;-getMIMEType-()Ljava/lang/String;' : ['android.permission.INTERNET'],
	'Landroid/location/LocationManager;-getProvider-(Ljava/lang/String;)Landroid/location/LocationProvider;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setInternalDataEnabled-(Z)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ExternalMediaFormatActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/wifi/WifiStateMachine$InitialState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/util/StateMachine$SmHandler;-invokeEnterMethods-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GeofenceManager;-sendIntent-(Landroid/app/PendingIntent; Landroid/content/Intent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/ViewRootImpl$ViewPostImeInputStage;-processKeyEvent-(Landroid/view/ViewRootImpl$QueuedInputEvent;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setAndBroadcastNetworkSetTime-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcCardEmulation;-getAidGroupForService-(I Landroid/content/ComponentName; Ljava/lang/String;)Landroid/nfc/cardemulation/AidGroup;' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Landroid/net/wifi/WifiManager;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/NsdService;-sendNsdStateChangeBroadcast-(Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDisconnectDone-(I Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DocumentaryFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-onRefresh-(Z [I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$8;-doWork-()V' : ['android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangupAllConnections-(Lcom/android/internal/telephony/gsm/GsmCall;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/BatteryService;-access$900-(Lcom/android/server/BatteryService; Landroid/os/BatteryProperties;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-disableCellBroadcastRangeForSubscriber-(J I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/power/PowerManagerService;-updatePowerStateLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/os/SystemVibrator;-vibrate-([J I)V' : ['android.permission.VIBRATE'],
	'Landroid/nfc/INfcTag;-getTimeout-(I)I' : ['android.permission.NFC'],
	'Landroid/widget/VideoView;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/updates/ConfigUpdateInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService$TextServicesMonitor;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/dreams/DreamManagerService;-access$300-(Lcom/android/server/dreams/DreamManagerService; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-clearCrossProfileIntentFilters-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BrightnessFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService$MyExecutor;-executeInternal-(Ljava/lang/Runnable;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Chronometer;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-setSyncAutomaticallyAsUser-(Landroid/accounts/Account; Ljava/lang/String; Z I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-requestIsimAuthentication-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setRestrictionsProvider-(Landroid/content/ComponentName; Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/service/textservice/SpellCheckerService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/os/NullVibrator;-vibrate-(J Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/policy/impl/GlobalActions;-addUsersToMenu-(Ljava/util/ArrayList;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/GlobalActions$LongPressAction;-onLongPress-()Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/IAudioService$Stub;-startBluetoothScoVirtualCall-(Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/telephony/TelephonyManager;-getLine1NumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipAudioCallAdapter;-onCallEnded-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothPbap;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothPbap$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/IntentService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityStackSupervisor;-scheduleIdleTimeoutLocked-(Lcom/android/server/am/ActivityRecord;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/IContentService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_SYNC_SETTINGS', 'android.permission.READ_SYNC_STATS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/Activity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ActivityManagerInternal;-goingToSleep-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/media/MediaSessionService$SessionManagerImpl;-handleVoiceKeyEventLocked-(Landroid/view/KeyEvent; Z Lcom/android/server/media/MediaSessionRecord;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ims/internal/IImsEcbmListener;-enteredECBM-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-access$2500-(Lcom/android/internal/telephony/dataconnection/DataConnection;)Landroid/net/NetworkCapabilities;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-sendBurstDtmf-(Ljava/lang/String; I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/ExtractEditText;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$WatchdogDisabledState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/LauncherAppsService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/trust/TrustManagerService$Receiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setOutgoingCallerIdDisplay-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/UiccSmsController;-enableCellBroadcastRangeForSubscriber-(J I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/wifi/WifiController$StaDisabledWithScanState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/view/IWindowSession$Stub;-add-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect; Landroid/view/InputChannel;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowState;-reportResized-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/backup/FullBackupJob;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-iccTransmitApduBasicChannel-(I I I I I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ActionBarOverlayLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine$ConnectModeState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/VibratorService$3;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getGsmBroadcastConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/DialerFilter;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupAllConnections-(Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$4;-startTask-()V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/inputmethodservice/KeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM$StartingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/updates/CertPinInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/telephony/SubscriptionManager;-getActiveSubInfoCount-()I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-startScan-(Landroid/net/wifi/ScanSettings; Landroid/os/WorkSource;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/IApplicationToken$Stub$Proxy;-keyDispatchingTimedOut-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ValidateNotificationPeople$PeopleRankingReconsideration;-work-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/Ringtone;-playFallbackRingtone-()Z' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-shutdownRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService$29;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/UiccController;-onGetIccCardStatusDone-(Landroid/os/AsyncResult; Ljava/lang/Integer;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/ObjectSource;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/IMms$Stub$Proxy;-updateStoredMessageStatus-(Ljava/lang/String; Landroid/net/Uri; Landroid/content/ContentValues;)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setOperatorBrandOverride-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/widget/EditText;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getDeviceIdForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiManager;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/WifiManager;-updateNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IccProvider;-insert-(Landroid/net/Uri; Landroid/content/ContentValues;)Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/FrameBranch;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/view/IWindowManager;-dismissKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setTrustAgentFeaturesEnabled-(Landroid/content/ComponentName; Landroid/content/ComponentName; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getKeyguardDisabledFeatures-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IntentService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/voice/VoiceInteractionSessionService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/am/ActivityRecord;-windowsVisible-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/app/IntentForwarderActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/effect/effects/TintEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager$CardEmulationInterface;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/content/IContentService$Stub;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/VibratorService$1;-onLowPowerModeChanged-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ManagedServices;-<init>-(Landroid/content/Context; Landroid/os/Handler; Ljava/lang/Object; Lcom/android/server/notification/ManagedServices$UserProfiles;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/sip/SipSessionGroup;-processTimeout-(Ljavax/sip/TimeoutEvent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcApplication;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-setDataOnRoamingEnabled-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-requestShutdown-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-isSubtypeSelected-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setPasswordQuality-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/HandoverService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wm/WindowAnimator;-setPendingLayoutChanges-(I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-notifyServiceStateForPhoneId-(I J Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/backup/BackupAgent$BackupServiceBinder;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-updateState-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/FaceUnlockView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NfcB;-close-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneFactory;-setDataSubscription-(J)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getSupportedFeatures-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getUidTagConfig-()Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings$Config;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-access$4400-(Lcom/android/server/pm/PackageManagerService; Z Z Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/TextViewWithCircularIndicator;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readAdnFileAndWait-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-separate-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/VideoView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setKeyguardDisabledFeatures-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/textservice/TextServicesManager;-setSpellCheckerSubtype-(Landroid/view/textservice/SpellCheckerSubtype;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager;-getConnectionStatistics-()Landroid/net/wifi/WifiConnectionStatistics;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPasswordMinimumLetters-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ILockSettings$Stub$Proxy;-getBoolean-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-revertToNitzTime-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub$Proxy;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/internal/telephony/CommandsInterface;-supplyIccPukForApp-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CallTracker;-handleRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendSmsByPstn-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryFacilityLock-(Ljava/lang/String; Ljava/lang/String; I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onHostCardEmulationDeactivated-()V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/base/FrameFetch;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyLocationChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-switchHoldingAndActive-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms;-sendDataForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-setAllowScansWithTraffic-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/notification/ConditionProviders;-<init>-(Landroid/content/Context; Landroid/os/Handler; Ljava/lang/Object; Lcom/android/server/notification/ManagedServices$UserProfiles;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/view/IInputMethodManager;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/IAlarmManager$Stub$Proxy;-setTimeZone-(Ljava/lang/String;)V' : ['android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/BluetoothManagerService;-access$100-(Lcom/android/server/BluetoothManagerService; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-getOrientationFromWindowsLocked-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/service/wallpaper/WallpaperService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/SubInfoRecordUpdater;-setDisplayNameForNewSub-(Ljava/lang/String; I I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/job/JobServiceContext;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/RttService$RttServiceImpl$RttStateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/P2pLinkManager;-onLlcpServicesConnected-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/accessibility/ScreenMagnifier$StateViewportDraggingHandler;-access$600-(Lcom/android/server/accessibility/ScreenMagnifier$StateViewportDraggingHandler; Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/IUiAutomationConnection;-disconnect-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Telephony$Sms$Inbox;-addMessage-(Landroid/content/ContentResolver; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long; Z)Landroid/net/Uri;' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Browser;-canClearHistory-(Landroid/content/ContentResolver;)Z' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/am/ActivityManagerService$3;-onSomePackagesChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-clearAllScoClients-(I Z)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/dataconnection/DcSwitchState;-access$1100-(Lcom/android/internal/telephony/dataconnection/DcSwitchState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/GrainFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-removeCrossProfileWidgetProvider-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier; Z I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/app/backup/BackupAgent;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/telephony/sip/SipPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$P2pEnablingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cat/CatService;-handleRilMsg-(Lcom/android/internal/telephony/cat/RilMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/View;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-acceptCall-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStack;-finishAllActivitiesLocked-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/power/PowerManagerService;-updateIsPoweredLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/LocationManagerService;-addTestProvider-(Ljava/lang/String; Lcom/android/internal/location/ProviderProperties;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/media/MediaScanner;-scanMtpFile-(Ljava/lang/String; Ljava/lang/String; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/ShutdownThread;-rebootOrShutdown-(Z Ljava/lang/String;)V' : ['android.permission.VIBRATE'],
	'Landroid/filterfw/core/AsyncRunner;-close-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/commands/bmgr/Bmgr;-doRestoreAll-(J Ljava/util/HashSet;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/performance/ThroughputFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/Button;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-readForcedDisplaySizeAndDensityLocked-(Lcom/android/server/wm/DisplayContent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/UiccSmsController;-setPremiumSmsPermission-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-sendData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/widget/DateTimeView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/sip/ISipService$Stub;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/base/NullFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/RemoteViewsService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/cardemulation/DefaultRemovedActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-invokeOemRilRequestStrings-([Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-performLayoutAndPlaceSurfacesLockedLoop-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringLogger;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/NegativeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-getPermittedInputMethodsForCurrentUser-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-requestSetID-(I)V' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-access$800-(Lcom/android/server/location/GpsLocationProvider; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/AccountManager;-getAuthTokenLabel-(Ljava/lang/String; Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/hardware/location/GeofenceHardwareImpl;-reportGeofenceOperationStatus-(I I I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/WindowManagerPolicy;-systemBooted-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/LocationBlacklist;-init-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/app/PlatLogoActivity$1;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringLogger;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/WebVttRenderingWidget$CueLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseAccountTypeActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-dial-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/dreams/DreamManagerService;-access$1500-(Lcom/android/server/dreams/DreamManagerService; Landroid/content/ComponentName; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/WifiManager$MulticastLock;-acquire-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-sendStoredMessage-(J Ljava/lang/String; Landroid/net/Uri; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPasswordMinimumSymbols-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/HdmiControlService;-readBooleanSetting-(Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-listenForSubscriber-(J Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/net/wifi/WifiManager;-addNetwork-(Landroid/net/wifi/WifiConfiguration;)I' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-inputDispatchingTimedOut-(Lcom/android/server/am/ProcessRecord; Lcom/android/server/am/ActivityRecord; Lcom/android/server/am/ActivityRecord; Z Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/squareup/okhttp/internal/http/HttpsURLConnectionImpl;-connect-()V' : ['android.permission.INTERNET'],
	'Landroid/location/ILocationManager;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/location/ILocationManager;-clearTestProviderLocation-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/location/GpsLocationProvider;-access$3900-(Lcom/android/server/location/GpsLocationProvider;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/WindowManagerService;-addInputMethodWindowToListLocked-(Lcom/android/server/wm/WindowState;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ImageStitcher;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BlendFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getHardwareConfig-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/WallpaperManager;-clear-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-updateMessageOnIccEf-(Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-access$2900-(Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/print/PrinterDiscoverySession;-startPrinterDiscovery-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/INetworkStatsService$Stub$Proxy;-advisePersistThreshold-(J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-getSmscAddress-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/MifareClassic;-readBlock-(I)[B' : ['android.permission.NFC'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$ConnectedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-proceedAfterWaitChar-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/GpsLocationProvider;-updateRequirements-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/uiautomator/EventsCommand;-run-([Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/videosrc/CameraSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-setEnabledFeatures-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/accessibility/CaptioningManager;-notifyEnabledChanged-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider$5;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getApplicationRestrictions-(Landroid/content/ComponentName; Ljava/lang/String;)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/ClosedCaptionWidget;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-getVoiceRadioTechnology-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-getLastKnownLocation-(Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/usb/UsbDeviceManager$UsbHandler;-<init>-(Lcom/android/server/usb/UsbDeviceManager; Landroid/os/Looper;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/location/ILocationManager;-sendExtraCommand-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Bundle;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_LOCATION_EXTRA_COMMANDS', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getLockTaskPackages-(Landroid/content/ComponentName;)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/VideoView$7;-surfaceDestroyed-(Landroid/view/SurfaceHolder;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms;-updateMessageOnIccEf-(Ljava/lang/String; I I [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-releaseMulticastLock-()V' : ['android.permission.CHANGE_WIFI_MULTICAST_STATE'],
	'Lcom/android/server/location/CountryDetectorBase;-detectCountry-()Landroid/location/Country;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-clearClearableFlagsLw-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/tts/TextToSpeech;-runAction-(Landroid/speech/tts/TextToSpeech$Action; Ljava/lang/Object; Ljava/lang/String;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/IconMenuView$1;-onClick-(Landroid/view/View;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/snep/SnepServer$ConnectionThread;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-updateNetworkEnabledLocked-()V' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setMaximumTimeToLock-(Landroid/content/ComponentName; J I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-handleEnterEmergencyCallbackMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getImeiForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$9900-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendDownForAllNotInjectedPointers-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms;-disableCellBroadcastForSubscriber-(J I)Z' : ['android.permission.RECEIVE_SMS'],
	'Landroid/inputmethodservice/ExtractEditText;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/IRestoreSession$Stub$Proxy;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Landroid/hardware/location/GeofenceHardwareService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/backup/SystemBackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneBase;-getEnhancedVoicePrivacy-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/notification/NotificationManagerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-setMsisdnNumber-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-getPersistedScale-()F' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setLockTaskPackages-(Landroid/content/ComponentName; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameFetch;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService$NfcServiceHandler;-llcpActivated-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)Z' : ['android.permission.VIBRATE'],
	'Landroid/telephony/gsm/SmsManager;-updateMessageOnSim-(I I [B)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/gesture/GestureOverlayView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/NfcAdapter;-enableNdefPush-()Z' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/PhoneProxy;-setInternalDataEnabled-(Z Landroid/os/Message;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-isNdef-(I)Z' : ['android.permission.NFC'],
	'Landroid/filterpacks/imageproc/BitmapSource;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/AudioManager;-isWiredHeadsetOn-()Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/service/voice/VoiceInteractionSessionService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/nfc/cardemulation/PreferredServices;-registerPreferredForegroundService-(Landroid/content/ComponentName; I)Z' : ['android.permission.VIBRATE'],
	'Landroid/print/IPrintManager$Stub;-destroyPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager$Stub;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/pm/PackageManagerService;-replaceNonSystemPackageLI-(Landroid/content/pm/PackageParser$Package; Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; [I [Z Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService$LocalService;-setWindowsForAccessibilityCallback-(Landroid/view/WindowManagerInternal$WindowsForAccessibilityCallback;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiController$DeviceActiveState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/inputmethodservice/AbstractInputMethodService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/ViewGroup;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-requestShutdown-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/INfcAdapter$Stub$Proxy;-removeNfcUnlockHandler-(Landroid/nfc/INfcUnlockHandler;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/Ringtone;-setAudioAttributes-(Landroid/media/AudioAttributes;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$Service;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-access$3600-(Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine; Landroid/net/wifi/p2p/WifiP2pWfdInfo;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-writeSmsToSim-(I Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-notifySrvccState-(Lcom/android/internal/telephony/Call$SrvccState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getVoiceMailNumber-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-getPasswordMinimumLength-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/CommandsInterface;-startDtmf-(C Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/textservice/ITextServicesManager;-setCurrentSpellCheckerSubtype-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService;-access$400-(Lcom/android/server/net/NetworkStatsService; Landroid/net/NetworkTemplate; J J)Landroid/net/NetworkStats;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/job/JobService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/LocationManagerService;-setTestProviderEnabled-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Landroid/widget/ToggleButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Ljava/lang/String; J F Landroid/location/LocationListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/uicc/RuimFileHandler;-loadEFLinearFixed-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FixedRotationFilter;-fieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-getCdmaSubscriptionSource-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/IconLoader;-startLoadingIcon-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-getAutoTimeRequired-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockContentResolver;-getMasterSyncAutomaticallyAsUser-(I)Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/connectivity/Tethering$TetherMasterSM;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-downloadMessage-(J Ljava/lang/String; Ljava/lang/String; Landroid/net/Uri; Landroid/os/Bundle; Landroid/app/PendingIntent;)V' : ['android.permission.RECEIVE_MMS'],
	'Landroid/filterpacks/base/FrameFetch;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wm/InputMonitor;-dispatchUnhandledKey-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)Landroid/view/KeyEvent;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/IInputMethodManager$Stub$Proxy;-getEnabledInputMethodList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/MediaSource;-setupMediaPlayer-(Z)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/SystemGesturesPointerEventListener$Callbacks;-onSwipeFromTop-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-dialInternal-(Ljava/lang/String; I)Lcom/android/internal/telephony/Connection;' : ['android.permission.USE_SIP'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-hasGrantedPolicy-(Landroid/content/ComponentName; I I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/DefaultRemovedActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone$2;-onECBMEntered-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/TextServicesManagerService;-switchUserLocked-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TelephonyRegistry;-notifySignalStrengthForSubscriber-(J Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/DhcpStateMachine;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ILockSettings;-getBoolean-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService;-unbindCurrentMethodLocked-(Z Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-supplyPuk-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/tech/IsoDep;-setTimeout-(I)V' : ['android.permission.NFC'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/backup/BackupManagerService;-bindToAgentSynchronous-(Landroid/content/pm/ApplicationInfo; I)Landroid/app/IBackupAgent;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/view/IWindowManager$Stub;-reenableKeyguard-(Landroid/os/IBinder;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/internal/telephony/RIL;-getBasebandVersion-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-queryTTYMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaSMSDispatcher;-sendSmsByPstn-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneBase;-notifySignalStrength-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/media/MediaSessionService$SessionManagerImpl$KeyEventWakeLockReceiver;-aquireWakeLockLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/test/SimulatedCommands;-onRadioAvailable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager;-getBaseDisplaySize-(I Landroid/graphics/Point;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-sendThisDeviceChangedBroadcast-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-onRemoteDisconnect-(I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService$4;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub;-getMasterSyncAutomatically-()Z' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setDeviceOwner-(Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/view/IWindowManager$Stub$Proxy;-setAppVisibility-(Landroid/os/IBinder; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPasswordMinimumLength-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/GridLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothA2dpSink;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/content/SyncStorageEngine;-requestSync-(Lcom/android/server/content/SyncStorageEngine$AuthorityInfo; I Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/NetworkScoreService;-updateScores-([Landroid/net/ScoredNetwork;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telecom/ConnectionService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub$1;-onSomePackagesChanged-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ActionBarOverlayLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub;-getIsSyncableAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)I' : ['android.permission.READ_SYNC_SETTINGS'],
	'Landroid/widget/ImageView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/NetworkTimeUpdateService;-systemRunning-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-<init>-(Landroid/content/Context; Landroid/app/IActivityManager; Landroid/os/IPowerManager; Landroid/net/INetworkStatsService; Landroid/os/INetworkManagementService;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-access$4800-(Lcom/android/server/power/PowerManagerService; F)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipCommandInterface;-setRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation;-registerAidGroupForService-(I Landroid/content/ComponentName; Landroid/nfc/cardemulation/AidGroup;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPermittedInputMethods-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService$FullBackupEngine$FullBackupRunner;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/os/PowerManagerInternal;-setUserActivityTimeoutOverrideFromWindowManager-(J)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/ims/internal/IImsEcbmListener$Stub$Proxy;-enteredECBM-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/BeamShareActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/tech/IsoDep;-reconnect-()V' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cat/IconLoader;-readIconData-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FisheyeFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-hangupForegroundResumeBackground-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TableLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TextServicesManagerService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-invokeBeamInternal-(Landroid/nfc/BeamShareData;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-replacePackageLI-(Landroid/content/pm/PackageParser$Package; I I Landroid/os/UserHandle; Ljava/lang/String; Lcom/android/server/pm/PackageManagerService$PackageInstalledInfo;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ListActivity;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/INotificationManager$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/connectivity/NetworkMonitor$ValidatedState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/display/RampAnimator$1;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/hdmi/HdmiCecLocalDevicePlayback;-create-(Lcom/android/server/hdmi/HdmiControlService; I)Lcom/android/server/hdmi/HdmiCecLocalDevice;' : ['android.permission.READ_PROFILE'],
	'Landroid/accessibilityservice/IAccessibilityServiceConnection;-getWindow-(I)Landroid/view/accessibility/AccessibilityWindowInfo;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-access$400-(Lcom/android/internal/policy/impl/PhoneWindowManager;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-loadEFImgLinearFixed-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager$SyncAlarmIntentReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Landroid/hardware/ConsumerIrManager;-getCarrierFrequencies-()[Landroid/hardware/ConsumerIrManager$CarrierFrequencyRange;' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/server/am/ActivityManagerService;-registerReceiver-(Landroid/app/IApplicationThread; Ljava/lang/String; Landroid/content/IIntentReceiver; Landroid/content/IntentFilter; Ljava/lang/String; I)Landroid/content/Intent;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/DeviceHost$DeviceHostListener;-onLlcpFirstPacketReceived-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)V' : ['android.permission.VIBRATE'],
	'Landroid/net/wifi/IWifiManager$Stub;-isScanAlwaysAvailable-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFTransparent-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-handlePinMmi-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverService;-doIncomingTransfer-(Landroid/os/Message;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/LocationManagerService;-access$1800-(Lcom/android/server/LocationManagerService; Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-sendRetrySms-(Lcom/android/internal/telephony/SMSDispatcher$SmsTracker;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/videosrc/MediaSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/IContentService;-getPeriodicSyncs-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_SYNC_SETTINGS'],
	'Lcom/android/server/usb/UsbDeviceManager;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/accounts/ChooseTypeAndAccountActivity;-onCreate-(Landroid/os/Bundle;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/bluetooth/IBluetoothManager$Stub$Proxy;-disable-(Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/RIL;-sendEnvelope-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkPolicyManager$Stub;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/SepiaFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiController$DefaultState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/FlipFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TextViewWithCircularIndicator;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ActionMenuView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/KeyboardView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/SaturateFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/IWifiManager;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/CompatModePackages;-setPackageScreenCompatModeLocked-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/UiccSmsController;-sendDataForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/pm/PackageManagerService$FileInstallArgs;-checkFreeStorage-(Lcom/android/internal/app/IMediaContainerService;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-setRestrictBackground-(Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/connectivity/Tethering$TetherInterfaceSM;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setActiveAdmin-(Landroid/content/ComponentName; Z I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/SubtitleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RemoteViews$2;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-enableEnhancedVoicePrivacy-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/CantAddAccountActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/location/ILocationManager$Stub;-getBestProvider-(Landroid/location/Criteria; Z)Ljava/lang/String;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/bluetooth/BluetoothA2dp;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPasswordHistoryLength-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setPermittedInputMethods-(Landroid/content/ComponentName; Ljava/util/List;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/bmgr/Bmgr;-doRestore-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-updateProvidersLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/job/JobServiceContext$JobServiceHandler;-handleStartedH-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/SipAudioCall;-endCall-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-insert-(Landroid/content/ContentResolver; J J J J I)Landroid/net/Uri;' : ['android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker$2;-onChange-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/DockObserver;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-updateDisplayPowerStateLocked-(I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-notifyLockTaskModeChanged-(Z Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/IRecognitionService$Stub;-startListening-(Landroid/content/Intent; Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/app/ActivityManagerInternal;-wakingUp-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/job/IJobScheduler$Stub;-schedule-(Landroid/app/job/JobInfo;)I' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/commands/pm/Pm;-main-([Ljava/lang/String;)V' : ['android.permission.CLEAR_APP_CACHE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-setNetworkSelectionModeAutomatic-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-hangup-(Lcom/android/internal/telephony/gsm/GsmConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/VignetteFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiController;-registerForWifiIdleTimeChange-(Landroid/os/Handler;)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-pollBatchedScan-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothMap;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/Phone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/trust/TrustManagerService$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setProfileName-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/VpnService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/IContentService$Stub$Proxy;-isSyncActive-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName;)Z' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/app/admin/IDevicePolicyManager;-setMaximumFailedPasswordsForWipe-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/monkey/MonkeyNetworkMonitor;-register-(Landroid/app/IActivityManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiController;-access$700-(Lcom/android/server/wifi/WifiController;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ExpandableListActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-getAccountsForPackage-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/widget/LockPatternUtils;-saveLockPattern-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-setUserActivityTimeoutOverrideFromWindowManagerInternal-(J)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/AccountManager;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-setNetworkPolicies-([Landroid/net/NetworkPolicy;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/service/media/MediaBrowserService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/notification/ConditionProviders;-onBootPhaseAppsCanStart-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/AccessibilityController$DisplayMagnifier;-setMagnificationSpecLocked-(Landroid/view/MagnificationSpec;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/Editor$SelectionStartHandleView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/notification/NotificationManagerService$7;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LockSettingsService$DatabaseHelper;-maybeEnableWidgetSettingForUsers-(Landroid/database/sqlite/SQLiteDatabase;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/text/format/DateUtils;-getRelativeTimeSpanString-(Landroid/content/Context; J)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getCurrentFailedPasswordAttempts-(I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getSelectedInputMethodSubtypeId-(Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accounts/AccountManagerService;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/PhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/hardware/location/IGeofenceHardware$Stub$Proxy;-pauseGeofence-(I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/inputmethod/InputMethodManager;-getLastInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getGlobalAlertBytes-(J)J' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ViewFlipper;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-broadcastDefaultSmsSubIdChanged-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/pm/IPackageManager$Stub$Proxy;-systemReady-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-init-(Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getMsisdn-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getVerboseLoggingLevel-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$4;-onCallHoldFailed-(Lcom/android/ims/ImsCall; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SubscriptionManager;-getDefaultDataSubInfo-()Landroid/telephony/SubInfoRecord;' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/WifiApConfigStore$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/nfc/NfcActivityManager;-disableReaderMode-(Landroid/app/Activity;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ResolverDrawerLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/p2p/IWifiP2pManager$Stub$Proxy;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/telephony/TelephonyManager;-putIntAtIndex-(Landroid/content/ContentResolver; Ljava/lang/String; I I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-updateCurrentCarrierInProvider-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-supplyIccPuk2-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-checkVoldPassword-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaFocusControl;-dispatchMediaKeyEventForCalls-(Landroid/view/KeyEvent; Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccCard;-supplyPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActiveServices;-bringUpServiceLocked-(Lcom/android/server/am/ServiceRecord; I Z Z)Ljava/lang/String;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Settings$System;-getFloatForUser-(Landroid/content/ContentResolver; Ljava/lang/String; F I)F' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/YearPickerView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub$Stub;-setDataRoaming-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub;-startScan-(Landroid/net/wifi/ScanSettings; Landroid/os/WorkSource;)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/trust/TrustManagerService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterfw/core/AsyncRunner$AsyncRunnerTask;-onPostExecute-(Landroid/filterfw/core/AsyncRunner$RunnerResult;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/sip/ISipService$Stub;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Lcom/android/internal/app/LocalePicker;-constructAdapter-(Landroid/content/Context;)Landroid/widget/ArrayAdapter;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/updates/CarrierProvisioningUrlsInstallReceiver;-getCert-(Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Presentation$1;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/usb/UsbService$Lifecycle;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ExpandedMenuView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ListActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkStatsService;-updateIfacesLocked-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wm/AccessibilityController;-setMagnificationSpecLocked-(Landroid/view/MagnificationSpec;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-getCameraDisabled-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getLine1AlphaTagForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/accounts/AccountManager$5;-start-()Landroid/accounts/AccountManager$Future2Task;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiManager;-is5GHzBandSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/TabActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService;-applyAllProviderRequirementsLocked-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameStore;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-enableSystemAppWithIntent-(Landroid/content/ComponentName; Landroid/content/Intent;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/IEthernetManager$Stub$Proxy;-setConfiguration-(Landroid/net/IpConfiguration;)V' : ['android.permission.CHANGE_NETWORK_STATE'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/RIL;-getCellInfoList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/IActivityManager;-handleApplicationCrash-(Landroid/os/IBinder; Landroid/app/ApplicationErrorReport$CrashInfo;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractButton;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-getDhcpInfo-()Landroid/net/DhcpInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/IIccPhoneBook;-getAdnRecordsInEfForSubscriber-(J I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getVoiceMailAlphaTagForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/CallTracker;-pollCallsWhenSafe-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-cleanUpConnection-(Z Lcom/android/internal/telephony/dataconnection/ApnContext;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/DisableCarModeActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/CertBlacklister$BlacklistObserver$1;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/TextServicesManagerService;-getCurrentSpellCheckerSubtype-(Ljava/lang/String; Z)Landroid/view/textservice/SpellCheckerSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/RetargetFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/am/UserSwitchingDialog;-show-()V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/test/mock/MockApplication;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-getDataOnRoamingEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/DefaultPhoneNotifier;-notifyDataConnectionFailed-(Lcom/android/internal/telephony/Phone; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-pingSupplicant-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiStateMachine$DriverStartedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/widget/SearchView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/mock/MockApplication;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-access$000-(Lcom/android/internal/telephony/cdma/CdmaConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterfw/core/FilterFunction;-execute-(Landroid/filterfw/core/KeyValueMap;)Landroid/filterfw/core/Frame;' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/ContextWrapper;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Toolbar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/text/StringLogger;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub$Proxy;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/view/LayoutInflater$BlinkLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcTag;-connect-(I I)I' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/Phone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/backup/IRestoreSession$Stub;-getAvailableRestoreSets-(Landroid/app/backup/IRestoreObserver;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub$Proxy;-exitKeyguardSecurely-(Landroid/view/IOnKeyguardExitResult;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/net/wifi/WifiManager;-clearBlacklist-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/base/GLTextureSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/VideoView;-access$2100-(Landroid/widget/VideoView;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/p2p/IWifiP2pManager$Stub;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/MediaPlayer;-access$600-(Landroid/media/MediaPlayer;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/RetargetFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/cat/IconLoader;-readClut-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/SharpenFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService;-registerShowTouchesSettingObserver-()V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/view/accessibility/AccessibilityInteractionClient;-getWindows-(I)Ljava/util/List;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/util/StateMachine;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/RecognitionService;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/telephony/PhoneNumberUtils;-isPotentialLocalEmergencyNumber-(Landroid/content/Context; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowSession;-performDrag-(Landroid/view/IWindow; Landroid/os/IBinder; F F F F Landroid/content/ClipData;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService$VoiceInteractionManagerServiceStub$1;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/PackageManagerBackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioManager;-setRingerMode-(I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor$ActivityContainer;-detachLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-setActivePasswordState-(I I I I I I I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-releaseWakeLock-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/VideoView;-openVideo-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/text/StringSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/MockProvider;-setLocation-(Landroid/location/Location;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Landroid/widget/GridLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/twilight/TwilightService$LocationHandler;-requestLocationUpdate-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/wifi/WifiServiceImpl;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/SpeechRecognizer;-access$200-(Landroid/speech/SpeechRecognizer;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/IUiAutomationConnection;-connect-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setActivePasswordState-(I I I I I I I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/PhoneWindow;-getOptionsPanelGravity-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/content/SyncManager;-scheduleSyncOperation-(Lcom/android/server/content/SyncOperation;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ims/ImsCall$Listener;-onCallTerminated-(Lcom/android/ims/ImsCall; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation;-isDefaultServiceForCategory-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Lcom/android/server/InputMethodManagerService$3;-onUserSwitching-(I Landroid/os/IRemoteCallback;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-nvReadItem-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/LockPatternUtils;-checkVoldPassword-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService$3;-onPackageRemoved-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/Switch;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-trimApplications-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CallerInfo;-doSecondaryLookupIfNecessary-(Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/CallerInfo;)Lcom/android/internal/telephony/CallerInfo;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-listenForSubscriber-(J Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/media/tv/TvView$3;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/tech/NfcV;-transceive-([B Z)[B' : ['android.permission.NFC'],
	'Lcom/android/internal/policy/impl/PhoneWindow$DecorView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActiveServices;-startServiceInnerLocked-(Lcom/android/server/am/ActiveServices$ServiceMap; Landroid/content/Intent; Lcom/android/server/am/ServiceRecord; Z Z)Landroid/content/ComponentName;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub$Proxy;-updateAdnRecordsInEfByIndexForSubscriber-(J I Ljava/lang/String; Ljava/lang/String; I Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/uicc/IsimFileHandler;-loadEFImgTransparent-(I I I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/LoadedApk$ReceiverDispatcher$InnerReceiver;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/backup/BackupManagerService$RunInitializeReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStackSupervisor$ActivityContainer;-onTaskListEmptyLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getActiveAdmins-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService$WindowManagerCallbacks;-interceptKeyBeforeDispatching-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)J' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-onConnectedInOrOut-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager$CardEmulationInterface;-removeAidGroupForService-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/media/MediaScanner$MyMediaScannerClient;-doScanFile-(Ljava/lang/String; Ljava/lang/String; J J Z Z Z)Landroid/net/Uri;' : ['android.permission.READ_PROFILE'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/webkit/WebViewUpdateService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/TaskRecord;-getTaskThumbnailLocked-()Landroid/app/ActivityManager$TaskThumbnail;' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/AlphaBlendFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/SmsStorageMonitor; Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/CellBroadcastHandler;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcTag$Stub;-getTechList-(I)[I' : ['android.permission.NFC'],
	'Landroid/accounts/AccountManager$13;-start-()Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processGetInput-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/PowerManager$WakeLock;-setWorkSource-(Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-onDisconnect-()Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getRestrictionsProvider-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-stopDtmf-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/NfcAdapter;-addNfcUnlockHandler-(Landroid/nfc/NfcAdapter$NfcUnlockHandler; [Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/ProgressBar;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/display/DisplayManagerService$LocalService;-requestPowerState-(Landroid/hardware/display/DisplayManagerInternal$DisplayPowerRequest; Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/InputMethodManagerService;-showInputMethodMenu-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/P2pEventManager;-onP2pInRange-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCallHoldIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setupDataCall-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Telephony$Sms$Intents;-getMessagesFromIntent-(Landroid/content/Intent;)[Landroid/telephony/SmsMessage;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/NetInitiatedActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/LockPatternUtils;-getEnabledTrustAgents-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/HandoverServer$ServerThread;-run-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/pm/PackageManagerService;-access$3200-(Lcom/android/server/pm/PackageManagerService;)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-updateQuietMode-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/print/PrintManager;-createPrinterDiscoverySession-()Landroid/print/PrinterDiscoverySession;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setApplicationHidden-(Landroid/content/ComponentName; Ljava/lang/String; Z)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosink/MediaEncoderFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapSource;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/LomoishFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/nfc/cardemulation/CardEmulation;-isDefaultServiceForAid-(Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Lcom/android/server/tv/TvInputManagerService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/MediaPlayer;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/CommandsInterface;-rejectCall-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-handleCallWaitingIncallSupplementaryService-(Ljava/lang/String;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/SupplicantStateTracker;-sendSupplicantStateChangedBroadcast-(Landroid/net/wifi/SupplicantState; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/util/StateMachine;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-disconnect-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager;-addPersistentPreferredActivity-(Landroid/content/ComponentName; Landroid/content/IntentFilter; Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/os/IVibratorService;-vibrate-(I Ljava/lang/String; J I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/provider/ContactsContract$Contacts;-markAsContacted-(Landroid/content/ContentResolver; J)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/app/MediaRouteButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureSource;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/mock/MockContentResolver;-cancelSync-(Landroid/content/SyncRequest;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/speech/tts/TextToSpeech$8;-run-(Landroid/speech/tts/ITextToSpeechService;)Ljava/lang/Object;' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IMediaHTTPConnection$Stub;-getSize-()J' : ['android.permission.INTERNET'],
	'Lcom/android/server/wifi/WifiConfigStore;-getMaxDhcpRetries-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/DialerFilter;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/util/IState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/media/MediaSessionService$SessionManagerImpl$KeyEventWakeLockReceiver;-onReceiveResult-(I Landroid/os/Bundle;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/location/LocationManager;-clearTestProviderStatus-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-initializeHdmiState-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-removeActiveAdminLocked-(Landroid/content/ComponentName; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-disconnect-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getFrequencyBand-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Telephony$Threads;-getOrCreateThreadId-(Landroid/content/Context; Ljava/lang/String;)J' : ['android.permission.READ_SMS'],
	'Landroid/inputmethodservice/ExtractEditLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/PosterizeEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onModifiedTouchEvent-(Landroid/view/MotionEvent; Z)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/widget/DigitalClock;-setFormat-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/ConditionProviders;-setAutomaticZenModeConditions-([Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getPasswordMinimumSymbols-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-enableCellBroadcast-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Landroid/printservice/PrintService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getXtConfig-()Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings$Config;' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardwareService$1;-unregisterForMonitorStateChangeCallback-(I Landroid/hardware/location/IGeofenceHardwareMonitorCallback;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/notification/ValidateNotificationPeople;-initialize-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManagerResponse$Stub$Proxy;-onResult-(Landroid/os/Bundle;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/view/IWindowManager$Stub;-setNewConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/AnalogClock;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/pm/BackgroundDexOptService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/NsdService;-access$900-(Lcom/android/server/NsdService; Z)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/service/fingerprint/FingerprintUtils;-removeFingerprintIdForUser-(I Landroid/content/ContentResolver; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-onSubscriptionDeactivated-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getVoiceMailNumberForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/sip/SipManager;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/SipRegistrationListener;)V' : ['android.permission.USE_SIP'],
	'Landroid/accessibilityservice/AccessibilityService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accessibility/TouchExplorer;-sendHoverExitAndTouchExplorationGestureEndIfNeeded-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/OutputStreamTarget;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/os/PowerManagerInternal;-setDozeOverrideFromDreamManager-(I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onHandleForceStop-(Landroid/content/Intent; [Ljava/lang/String; I Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/inputmethodservice/ExtractButton;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/uiautomator/RunTestCommand;-run-([Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/backup/FullBackupJob;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/service/wallpaper/WallpaperService$Engine;-doDisplayPaddingChanged-(Landroid/graphics/Rect;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/content/CursorEntityIterator;-next-()Landroid/content/Entity;' : ['android.permission.READ_CALENDAR'],
	'Landroid/telephony/PhoneNumberUtils;-isPotentialEmergencyNumber-(Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor$VirtualActivityContainer;-onTaskListEmptyLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaSubscriptionSourceManager;-getInstance-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Landroid/os/Handler; I Ljava/lang/Object;)Lcom/android/internal/telephony/cdma/CdmaSubscriptionSourceManager;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/TextClock;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub$Stub;-setDisplayNameUsingSrc-(Ljava/lang/String; J J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover;-nextStep-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onRoamingOn-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/opengl/GLSurfaceView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/accessibility/TouchExplorer$DoubleTapDetector;-onDoubleTap-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/nfc/cardemulation/RegisteredServicesCache;-initialize-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/widget/Button;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Ljava/net/ServerSocket;-bind-(Ljava/net/SocketAddress;)V' : ['android.permission.INTERNET'],
	'Lcom/android/server/notification/NotificationManagerService$NotificationListeners;-onBootPhaseAppsCanStart-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getExplicitlyOrImplicitlyEnabledInputMethodsAndSubtypeListLocked-(Landroid/content/Context;)Ljava/util/HashMap;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getDeviceOwnerAdmin-()Lcom/android/server/devicepolicy/DevicePolicyManagerService$ActiveAdmin;' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockContentResolver;-getSyncStatus-(Landroid/accounts/Account; Ljava/lang/String;)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-handlePasswordExpirationNotification-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/CommandParamsFactory;-processBIPClient-(Lcom/android/internal/telephony/cat/CommandDetails; Ljava/util/List;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/SizeChangeEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiStateMachine$SoftApStartingState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/speech/tts/TextToSpeech;-<init>-(Landroid/content/Context; Landroid/speech/tts/TextToSpeech$OnInitListener; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/TapAgainDialog;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/admin/IDevicePolicyManager;-getAccountTypesWithManagementDisabledAsUser-(I)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-onServiceDisconnected-(Landroid/content/ComponentName;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-hangupWaitingOrBackground-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/INetworkStatsService$Stub;-advisePersistThreshold-(J)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccSmsInterfaceManager;-access$200-(Lcom/android/internal/telephony/IccSmsInterfaceManager; [Lcom/android/internal/telephony/cdma/CdmaSmsBroadcastConfigInfo;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/inputmethodservice/KeyboardView;-onModifiedTouchEvent-(Landroid/view/MotionEvent; Z)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/OffHostApduService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/LocationManagerService$1;-onOpChanged-(I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService;-updateIfaces-()V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/ReceiverRestrictedContext;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-writeOutboxMessage-(J Ljava/lang/String; Ljava/lang/String; Z Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS'],
	'Lcom/android/server/am/ActivityManagerService;-removeProcessLocked-(Lcom/android/server/am/ProcessRecord; Z Z Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getVoiceRadioTechnology-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/QuickContactBadge;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-removeUser-(Landroid/content/ComponentName; Landroid/os/UserHandle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-onBootCompleted-()V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/test/SyncBaseInstrumentation;-syncProvider-(Landroid/net/Uri; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_SYNC_STATS', 'android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/net/VpnService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/pm/PackageManagerService;-systemReady-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService$SipAutoReg;-start-(Lcom/android/server/sip/SipSessionGroup;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/backup/PackageManagerBackupAgent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/provider/Settings$System;-getLongForUser-(Landroid/content/ContentResolver; Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/performance/ThroughputFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService$24;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings;-getReportXtOverDev-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager$Stub;-removeTestProvider-(Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/uicc/UiccCardApplication;-queryFdn-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cat/CatService;-handleCommand-(Lcom/android/internal/telephony/cat/CommandParams; Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$System;-getUriFor-(Ljava/lang/String;)Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-access$300-(Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/view/ContextThemeWrapper;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/RedEyeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/widget/LockPatternUtils;-isPermanentlyLocked-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$DefaultNetworkStatsSettings;-getXtConfig-()Lcom/android/server/net/NetworkStatsService$NetworkStatsSettings$Config;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/ChooseAccountTypeActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService$Lifecycle;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService;-prepareOperationTimeout-(I J Lcom/android/server/backup/BackupManagerService$BackupRestoreTask;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getLockTaskPackages-(Landroid/content/ComponentName;)[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/view/accessibility/IAccessibilityManager$Stub$Proxy;-unregisterUiTestAutomationService-(Landroid/accessibilityservice/IAccessibilityServiceClient;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/provider/Settings$Secure;-getIntForUser-(Landroid/content/ContentResolver; Ljava/lang/String; I I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/view/IWindowManager$Stub$Proxy;-disableKeyguard-(Landroid/os/IBinder; Ljava/lang/String;)V' : ['android.permission.DISABLE_KEYGUARD'],
	'Lcom/android/server/am/ActivityStackSupervisor;-comeOutOfSleepIfNeededLocked-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/PrintManager;-getPrintJobInfo-(Landroid/print/PrintJobId;)Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/TelephonyManager;-getVoiceMailAlphaTag-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/KeyguardManager$KeyguardLock;-reenableKeyguard-()V' : ['android.permission.DISABLE_KEYGUARD'],
	'Landroid/widget/DateTimeView;-access$1200-(Landroid/widget/TextView;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/DefaultRemovedActivity;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ExpandableListActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-setDataAllowed-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-supplyPin-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Settings$Secure;-getLong-(Landroid/content/ContentResolver; Ljava/lang/String; J)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-getAllMessagesFromIccEf-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setUninstallBlocked-(Landroid/content/ComponentName; Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-startAppProblemLocked-(Lcom/android/server/am/ProcessRecord;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-isNanSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/nfc/tech/NfcF;-close-()V' : ['android.permission.NFC'],
	'Landroid/widget/Editor$SelectionStartHandleView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-setUserData-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Landroid/content/IContentService;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-wipeDeviceOrUserLocked-(I I Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-confirmCredentialsAsUser-(Landroid/accounts/IAccountManagerResponse; Landroid/accounts/Account; Landroid/os/Bundle; Z I)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-rejectCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-enableSystemApp-(Landroid/content/ComponentName; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/dreams/DreamManagerService;-dumpInternal-(Ljava/io/PrintWriter;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/Invert;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/UiccSmsController;-isSMSPromptEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/policy/impl/EnableAccessibilityController;-access$200-(Lcom/android/internal/policy/impl/EnableAccessibilityController;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-processMessagePart-(Lcom/android/internal/telephony/InboundSmsTracker;)Z' : ['android.permission.RECEIVE_MMS', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_SMS'],
	'Landroid/nfc/tech/NfcF;-reconnect-()V' : ['android.permission.NFC'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-ndefRead-(I)Landroid/nfc/NdefMessage;' : ['android.permission.NFC'],
	'Lcom/android/nfc/NfcRootActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/pm/IPackageManager$Stub;-setInstallLocation-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/PreferredServices$Callback;-onPreferredForegroundServiceChanged-(Landroid/content/ComponentName;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/ISub$Stub;-setDisplayNumberFormat-(I J)I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/ServiceWatcher$2;-onPackageModified-(Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/sip/SipService;-setRegistrationListener-(Ljava/lang/String; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.USE_SIP'],
	'Landroid/widget/Chronometer;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/location/GeofenceHardware;-resumeGeofence-(I I I)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/os/IPowerManager;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/content/pm/IPackageManager;-getPackageSizeInfo-(Ljava/lang/String; I Landroid/content/pm/IPackageStatsObserver;)V' : ['android.permission.GET_PACKAGE_SIZE'],
	'Landroid/widget/TextClock;-init-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/job/IJobScheduler$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/server/location/GpsLocationProvider;-access$3500-(Lcom/android/server/location/GpsLocationProvider; Lcom/android/internal/location/ProviderRequest; Landroid/os/WorkSource;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-getActiveAdmins-(I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/HandoverService;-access$200-(Lcom/android/nfc/handover/HandoverService; Landroid/content/Intent; I)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-onInputEvent-(Landroid/view/InputEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/ims/internal/ImsCallSession$Listener;-callSessionHoldFailed-(Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService;-notifySwitch-(J I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/voiceinteraction/SoundTriggerHelper;-onServiceDied-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/display/DisplayAdapter$Listener;-onTraversalRequested-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/AudioManager;-querySoundEffectsEnabled-(I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-getAllSubInfoList-()Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/effect/effects/FillLightEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/CellBroadcastHandler;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneBase;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmCallTracker;-acceptCall-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-updateScreenState-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/dreams/DreamManagerService;-stopDozingInternal-(Landroid/os/IBinder;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/UiccSmsController;-updateSmsSendStatus-(I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/voice/VoiceInteractionSessionService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/BlendFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/net/NetworkStatsService;-<init>-(Landroid/content/Context; Landroid/os/INetworkManagementService; Landroid/app/IAlarmManager;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-getSubInfoUsingSlotId-(I)Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/UiModeManagerService;-updateDockState-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TelephonyRegistry;-notifySignalStrength-(Landroid/telephony/SignalStrength;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ZoomButtonsController$Container;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/print/IPrintManager$Stub;-print-(Ljava/lang/String; Landroid/print/IPrintDocumentAdapter; Landroid/print/PrintAttributes; Ljava/lang/String; I I)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ImsSMSDispatcher;-sendData-(Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.RECEIVE_SMS'],
	'Landroid/speech/tts/TextToSpeech$Connection$SetupConnectionAsyncTask;-doInBackground-([Ljava/lang/Void;)Ljava/lang/Integer;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/EditText;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-access$300-(Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-dispose-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/media/SubtitleController;-access$300-(Landroid/media/SubtitleController;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getRestrictionsProvider-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-getWifiEnabledState-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/WebVttRenderingWidget;-onLayout-(Z I I I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-enableAllIMEsIfThereIsNoEnabledIME-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISms;-sendData-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setStorageEncryption-(Landroid/content/ComponentName; Z I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodsStr-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/HdmiControlService$SettingsObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodSubtypeSwitchingController$InputMethodAndSubtypeList;-getSortedInputMethodAndSubtypeList-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LocationManagerService;-access$800-(Lcom/android/server/LocationManagerService; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/bluetooth/BluetoothAdapter;-disable-(Z)Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/accounts/IAccountManager$Stub;-editProperties-(Landroid/accounts/IAccountManagerResponse; Ljava/lang/String; Z)V' : ['android.permission.MANAGE_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-queryAvailableBandMode-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/GrayscaleEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$17;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/MutableContextWrapper;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/UsimPhoneBookManager;-readEmailFileAndWait-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getPermittedInputMethods-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/tech/NfcA;-connect-()V' : ['android.permission.NFC'],
	'Landroid/widget/ActivityChooserModel$DataModelPackageMonitor;-onPackageDisappeared-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ServiceStateTracker;-notifySignalStrength-()Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/PasswordEntryKeyboardView;-onTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService$MyPackageMonitor;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-getString-(Ljava/lang/String; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Browser;-addSearchUrl-(Landroid/content/ContentResolver; Ljava/lang/String;)V' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-handleExitEmergencyCallbackMode-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService;-onBootPhase-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/net/NetworkStatsService;-internalGetHistoryForNetwork-(Landroid/net/NetworkTemplate; I)Landroid/net/NetworkStatsHistory;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-acknowledgeLastIncomingCdmaSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/ActionMenuPresenter$OverflowMenuButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiScanningServiceImpl$WifiScanningStateMachine;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/ClosedCaptionWidget;-onDetachedFromWindow-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-getAutoTime-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/uicc/SIMFileHandler;-loadEFLinearFixedAll-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Phone;-invokeOemRilRequestRaw-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager;-enforceCanManageCaCerts-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/VibratorService;-cancelVibrate-(Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$WaitingState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$WatchdogDisabledState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/app/IWallpaperManager$Stub;-setWallpaper-(Ljava/lang/String;)Landroid/os/ParcelFileDescriptor;' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-updateIccAvailability-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/am/ActivityManagerService;-updateConfiguration-(Landroid/content/res/Configuration;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiController;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-startSync-(Landroid/net/Uri; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-prepareDragSurface-(Landroid/view/IWindow; Landroid/view/SurfaceSession; I I I Landroid/view/Surface;)Landroid/os/IBinder;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/BlackWhiteFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/ObjectSource;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/textservice/SpellCheckerService;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceCategoryProgramHandler$1;-sendScpResults-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/SpeechRecognizer;-handleStopMessage-()V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onRoamingOff-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/location/GpsLocationProvider;-handleDownloadXtraData-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/provider/Browser;-getAllVisitedUrls-(Landroid/content/ContentResolver;)Landroid/database/Cursor;' : ['com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine$WatchdogEnabledState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/SubscriptionController;-getAllSubInfoCount-()I' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/speech/IRecognitionService$Stub$Proxy;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/widget/RelativeLayout;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$ActiveSyncContext;-binderDied-()V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/app/ResolverActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/TwoLineListItem;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-handlePostDhcpSetup-()V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/MediaPlayer;-addTimedTextSource-(Landroid/content/Context; Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ISms$Stub;-disableCellBroadcastRange-(I I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager;-<init>-(Landroid/content/Context;)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/app/Service;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/os/storage/StorageManager;-getStorageBytesUntilLow-(Ljava/io/File;)J' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SmsManager;-deleteStoredConversation-(J)Z' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-updateSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/input/InputManagerService$WiredAccessoryCallbacks;-systemReady-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioService;-setMode-(I Landroid/os/IBinder;)V' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityManagerService$18;-run-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-startProcessLocked-(Lcom/android/server/am/ProcessRecord; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/LocationManagerService;-checkMockPermissionsSafe-()V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/net/wifi/IWifiManager$Stub;-acquireWifiLock-(Landroid/os/IBinder; I Ljava/lang/String; Landroid/os/WorkSource;)Z' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/SubtitleController;-access$100-(Landroid/media/SubtitleController;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-setLocaleFromUsim-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-nvResetConfig-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IccProvider;-deleteIccRecordFromEf-(I Ljava/lang/String; Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; J)Z' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/nfc/INfcAdapter$Stub;-invokeBeamInternal-(Landroid/nfc/BeamShareData;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/nfc/handover/HandoverManager;-createHandoverRequestMessage-()Landroid/nfc/NdefMessage;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/connectivity/Tethering;-access$900-(Lcom/android/server/connectivity/Tethering;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-handleNewSms-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/speech/tts/TextToSpeechService$1;-getClientDefaultLanguage-()[Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/PackageManagerBackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/app/IActivityManager;-setPackageScreenCompatMode-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CdmaCall;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IBackupAgent$Stub$Proxy;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setDataOnRoamingEnabled-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-vibrate-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-notifyAndAcknowledgeLastIncomingSms-(Z I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/RotateEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/DhcpStateMachine$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/input/InputManagerService$1;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-handleOnUserContextChanged-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/display/DisplayPowerController;-updateBrightness-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/view/menu/IconMenuView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/ActivityGroup;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/net/NetworkScoreManager;-clearScores-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-setPasswordMinimumLowerCase-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/Telephony$Sms;-addMessageToUri-(Landroid/content/ContentResolver; Landroid/net/Uri; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/Long; Z Z J)Landroid/net/Uri;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-checkWaitingForWindowsLocked-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl;-getOrCreateUserStateLocked-(I)Lcom/android/server/print/UserState;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/NfcApplication;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/SIMRecords;-onAllRecordsLoaded-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/hdmi/HdmiControlService;-onHotplug-(I Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/pm/IPackageManager$Stub;-systemReady-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService$AudioHandler;-access$5800-(Landroid/media/AudioService$AudioHandler; Landroid/media/MediaPlayer;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-updateAdnRecordsInEfBySearchForSubscriber-(J I Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-isTactileFeedbackEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/handover/HandoverService;-doPeripheralHandover-(Landroid/os/Message;)V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/bluetooth/BluetoothPbap;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/storage/DeviceStorageMonitorService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/UiModeManagerService;-access$000-(Lcom/android/server/UiModeManagerService; Ljava/lang/String; I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/ZoomButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/view/menu/IconMenuItemView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISub$Stub;-setDefaultSmsSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/ContentService;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/sip/SipService;-access$1700-(Lcom/android/server/sip/SipService; Landroid/net/NetworkInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/Connection;-separate-()V' : ['android.permission.WAKE_LOCK', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Activity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-reinvokePersistentGroup-(Landroid/net/wifi/p2p/WifiP2pConfig;)Z' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/notification/DowntimeConditionProvider;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/policy/impl/RecentApplicationsBackground;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-notifyApnIdDisconnected-(Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/RIL;-switchToRadioState-(Lcom/android/internal/telephony/CommandsInterface$RadioState;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/statusbar/IStatusBarService;-expandSettingsPanel-()V' : ['android.permission.EXPAND_STATUS_BAR'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-appendAndPutEnabledInputMethodLocked-(Ljava/lang/String; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Switch;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/print/PrintManagerService$PrintManagerImpl$2;-onPackageChanged-(Ljava/lang/String; I [Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker$4;-onCallHeld-(Lcom/android/ims/ImsCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-cancel-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/widget/Button;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-killUid-(I Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$WaitingState;-enter-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/widget/PointerLocationView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-systemReady-(Ljava/lang/Runnable;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/commands/svc/Svc$Command;-run-([Ljava/lang/String;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/filterpacks/imageproc/StraightenFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/LockToAppRequestDialog;-showLockTaskPrompt-(Lcom/android/server/am/TaskRecord;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/commands/uiautomator/DumpCommand;-run-([Ljava/lang/String;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/speech/tts/ITextToSpeechService$Stub$Proxy;-speak-(Landroid/os/IBinder; Ljava/lang/CharSequence; I Landroid/os/Bundle; Ljava/lang/String;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-dump-(Ljava/io/FileDescriptor; Ljava/io/PrintWriter; [Ljava/lang/String;)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/WifiController;-access$200-(Lcom/android/server/wifi/WifiController;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/appwidget/AppWidgetHostView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/ImageSwitcher;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Activity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-getAdnRecordsInEf-(I)Ljava/util/List;' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/CommandsInterface;-deactivateDataCall-(I I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/location/FlpHardwareProvider;-onGeofenceAdd-(I I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getGroupIdLevel1ForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/power/PowerManagerService;-goToSleepInternal-(J I I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-addToBlacklist-(Ljava/lang/String;)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-getDataConnectionState-()Lcom/android/internal/telephony/PhoneConstants$DataState;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/BroadcastQueue;-processNextBroadcast-(Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/os/IPowerManager$Stub$Proxy;-setStayOnSetting-(I)V' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/speech/RecognitionService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/UiccCard;-iccOpenLogicalChannel-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameBranch;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/IBluetoothManager$Stub;-getName-()Ljava/lang/String;' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/am/ActivityManagerService;-cleanUpRemovedTaskLocked-(Lcom/android/server/am/TaskRecord; I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/UsimDataDownloadHandler;-handleUsimDataDownload-(Lcom/android/internal/telephony/uicc/UsimServiceTable; Lcom/android/internal/telephony/gsm/SmsMessage;)I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/ims/ImsCall$ImsCallSessionListenerProxy;-callSessionProgressing-(Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/ImsStreamMediaProfile;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/ImageCombineFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-access$3200-(Lcom/android/server/power/PowerManagerService; Landroid/os/IBinder; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/inputmethodservice/ExtractButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/LocationBasedCountryDetector;-setCountryListener-(Landroid/location/CountryListener;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-adjustSystemUiVisibilityLw-(I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/DateTimeView$2;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/os/IDropBoxManagerService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IccProvider;-update-(Landroid/net/Uri; Landroid/content/ContentValues; Ljava/lang/String; [Ljava/lang/String;)I' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/filterpacks/base/GLTextureTarget;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager$Stub$Proxy;-cancelPrintJob-(Landroid/print/PrintJobId; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onApnChanged-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService;-onHostCardEmulationDeactivated-()V' : ['android.permission.VIBRATE'],
	'Landroid/accounts/GrantCredentialsPermissionActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/p2p/IWifiP2pManager;-getP2pStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/view/SurfaceView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-setRadioPower-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/UiModeManagerService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/MmsServiceBroker;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/widget/ListPopupWindow$DropDownListView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/RIL;-explicitCallTransfer-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/search/Searchables;-getGlobalSearchProviderSetting-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/ConsumerIrService;-getCarrierFrequencies-()[I' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/server/BatteryService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/filterpacks/imageproc/NegativeFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wm/InputMonitor;-interceptKeyBeforeDispatching-(Lcom/android/server/input/InputWindowHandle; Landroid/view/KeyEvent; I)J' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/CheckBox;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/WifiScanningService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/LockSettingsService;-checkVoldPassword-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/test/mock/MockContentResolver;-setMasterSyncAutomatically-(Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/textservice/ITextServicesManager$Stub$Proxy;-getCurrentSpellCheckerSubtype-(Ljava/lang/String; Z)Landroid/view/textservice/SpellCheckerSubtype;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/InputMethodManagerService$SettingsObserver;-onChange-(Z Landroid/net/Uri;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-disableNetwork-(I)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SmsManager;-sendDataMessage-(Ljava/lang/String; Ljava/lang/String; S [B Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover;-nextStepInit-()V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Landroid/content/ContentResolver;-getCurrentSyncsAsUser-(I)Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Landroid/media/effect/effects/DocumentaryEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityStackSupervisor$VirtualActivityContainer;-setSurfaceLocked-(Landroid/view/Surface; I I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/GrainFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TwoLineListItem;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/text/ToUpperCase;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/UiAutomationConnection;-unregisterUiTestAutomationServiceLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-send-(Lcom/android/internal/telephony/RILRequest;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/AbsActionBarView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/TechListChooserActivity;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/content/ContentResolver;-setSyncAutomatically-(Landroid/accounts/Account; Ljava/lang/String; Z)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-checkPassword-(Ljava/lang/String; I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/location/LocationBlacklist;-switchUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-updateSmsSendStatus-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/widget/PointerLocationView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISms;-sendStoredText-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/net/NetworkStatsService;-systemReady-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/ResolverActivity;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/nfc/cardemulation/AppChooserActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/accounts/IAccountManager$Stub;-setAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/server/wifi/WifiService;-onBootPhase-(I)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/ISub;-getActiveSubInfoList-()Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-setWifiEnabled-(Z)Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ToGrayFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/ToggleButton;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/BackgroundDexOptService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/app/RestrictionsPinActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ImageSlicer;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/sip/SipService;-open3-(Landroid/net/sip/SipProfile; Landroid/app/PendingIntent; Landroid/net/sip/ISipSessionListener;)V' : ['android.permission.WAKE_LOCK', 'android.permission.USE_SIP'],
	'Landroid/print/PrintJob;-getInfo-()Landroid/print/PrintJobInfo;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService;-restoreAtInstall-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/AudioManager;-reloadAudioSettings-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmConnection;-finalize-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-powerOffRadioSafely-(Lcom/android/internal/telephony/dataconnection/DcTrackerBase;)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-getActiveAdminForCallerLocked-(Landroid/content/ComponentName; I)Lcom/android/server/devicepolicy/DevicePolicyManagerService$ActiveAdmin;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneFactory;-getVoiceSubscription-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Spinner;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService$PerformUnifiedRestoreTask;-restoreKeyValue-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/CommandsInterface;-getCurrentCalls-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/MountServiceIdler;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-notifyDataConnection-(Ljava/lang/String; Ljava/lang/String; Lcom/android/internal/telephony/PhoneConstants$DataState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/ZoomButtonsController$Container;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService;-getSecureSettingInt-(Ljava/lang/String; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/HorizontalScrollView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneNotifier;-notifyCellLocation-(Lcom/android/internal/telephony/Phone;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/backup/BackupAgent;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/wifi/WifiServiceImpl;-setFrequencyBand-(I Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/commands/wm/Wm;-runDisplayDensity-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setAutoTimeRequired-(Landroid/content/ComponentName; I Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter$1;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-print-(Ljava/lang/String; Landroid/print/IPrintDocumentAdapter; Landroid/print/PrintAttributes; Ljava/lang/String; I I)Landroid/os/Bundle;' : ['android.permission.READ_PROFILE'],
	'Landroid/accounts/IAccountManager;-getAccountsAsUser-(Ljava/lang/String; I)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/internal/telephony/RIL;-clearRequestList-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-onSetDependencyMet-(Ljava/lang/String; Z)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/location/ILocationManager;-getLastLocation-(Landroid/location/LocationRequest; Ljava/lang/String;)Landroid/location/Location;' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/commands/bmgr/Bmgr;-main-([Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmSMSDispatcher;-sendMultipartText-(Ljava/lang/String; Ljava/lang/String; Ljava/util/ArrayList; Ljava/util/ArrayList; Ljava/util/ArrayList; Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/server/power/PowerManagerService;-handleWakeLockDeath-(Lcom/android/server/power/PowerManagerService$WakeLock;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine$SupplicantStartedState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager;-saveConfiguration-()Z' : ['android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getDeviceIdForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-setPreferredNetworkType-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/handover/HandoverTransfer$Callback;-onTransferComplete-(Lcom/android/nfc/handover/HandoverTransfer; Z)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Landroid/net/wifi/WifiManager;-getConnectionInfo-()Landroid/net/wifi/WifiInfo;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/media/TtmlRenderingWidget;-<init>-(Landroid/content/Context; Landroid/util/AttributeSet; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getDeviceId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/widget/RemoteViews$2;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/widget/Gallery;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-getAggressiveHandover-()I' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPasswordHistoryLength-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/ui/SurfaceTargetFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/backup/BackupManagerService$BackupRestoreTask;-execute-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SmsManager;-disableCellBroadcastRange-(I I)Z' : ['android.permission.RECEIVE_SMS'],
	'Lcom/android/server/am/ActivityManagerService;-isGetTasksAllowed-(Ljava/lang/String; I I)Z' : ['android.permission.GET_TASKS'],
	'Landroid/nfc/cardemulation/OffHostApduService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/widget/Chronometer;-updateTextServicesLocaleLocked-()V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiServiceImpl;-pollBatchedScan-()V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/AudioRecord;-<init>-(I I I I I)V' : ['android.permission.RECORD_AUDIO'],
	'Landroid/view/IWindowManager;-getInitialDisplayDensity-(I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-copyMessageToIccEfForSubscriber-(J Ljava/lang/String; I [B [B)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/speech/RecognitionService;-access$700-(Landroid/speech/RecognitionService; Landroid/speech/IRecognitionListener;)Z' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/telephony/PhoneSubInfo;-getIccSimChallengeResponse-(J I Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/am/ActivityManagerService;-startIsolatedProcess-(Ljava/lang/String; [Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; I Ljava/lang/Runnable;)I' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/INetworkScoreService;-updateScores-([Landroid/net/ScoredNetwork;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl;-getP2pStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getTrustAgentFeaturesEnabled-(Landroid/content/ComponentName; Landroid/content/ComponentName; I)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-sendEmergencyCallbackModeChange-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/handover/BluetoothPeripheralHandover$2;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.BLUETOOTH', 'android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/wm/AppWindowAnimator;-stepAnimationLocked-(J)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setImsRegistrationState-(Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/media/IAudioService;-reloadAudioSettings-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/NativeActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/handover/HandoverTransfer;-finishTransfer-(Z Landroid/net/Uri; Ljava/lang/String;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-getRecoveryAction-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/WakeLockStateMachine$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/DockObserver;-setActualDockStateLocked-(I)V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/location/LocationProviderInterface;-setRequest-(Lcom/android/internal/location/ProviderRequest; Landroid/os/WorkSource;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/inputmethod/InputMethodManager;-setCurrentInputMethodSubtype-(Landroid/view/inputmethod/InputMethodSubtype;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/FrameBranch;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/service/notification/ConditionProviderService;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-nvWriteItem-(I Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService$NfcAdapterService;-enable-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/SystemService;-onBootPhase-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/AutoCompleteTextView;-access$200-(Landroid/widget/TextView;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiAutoJoinController;-addToScanCache-(Ljava/util/List;)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/job/JobServiceContext$JobServiceHandler;-closeAndCleanupJobH-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CropFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/service/fingerprint/FingerprintManager;-enrolledAndEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-setApplicationHiddenSettingAsUser-(Ljava/lang/String; Z I)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/voiceinteraction/VoiceInteractionManagerService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/IActivityManager;-shutdown-(I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/net/VpnService;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setPropertiesByCarrier-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-stopDtmf-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/power/PowerManagerService;-access$1900-(Lcom/android/server/power/PowerManagerService;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/NullFilter;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/GlobalActions;-access$1500-(Lcom/android/internal/policy/impl/GlobalActions;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AutoCompleteTextView;-onPopulateAccessibilityEvent-(Landroid/view/accessibility/AccessibilityEvent;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/SyncManager$3;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/app/ActivityManagerNative;-shutdown-(I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-enable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/telephony/SmsManager;-setAutoPersisting-(Z)V' : ['android.permission.WRITE_SMS'],
	'Lcom/android/server/backup/BackupManagerService$FullBackupObbConnection;-backupObbs-(Landroid/content/pm/PackageInfo; Ljava/io/OutputStream;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-getTechList-(I)[I' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-privatizeCellInfoList-(Ljava/util/List;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/provider/ContactsContract$PinnedPositions;-pin-(Landroid/content/ContentResolver; J I)V' : ['android.permission.WRITE_CONTACTS'],
	'Landroid/app/ActivityView$ActivityContainerWrapper;-finalize-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getMaximumTimeToLock-(Landroid/content/ComponentName; I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/provider/ContactsContract$Directory;-notifyDirectoryChange-(Landroid/content/ContentResolver;)V' : ['android.permission.WRITE_CONTACTS'],
	'Lcom/android/internal/telephony/WakeLockStateMachine;-getLogRecString-(Landroid/os/Message;)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/AudioPlaybackQueueItem;-run-()V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/wifi/WifiManager;-isP2pSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/inputmethodservice/ExtractEditText;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/widget/CompoundButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/os/DropBoxManager;-addText-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getLastSubtypeForInputMethodLockedInternal-(Ljava/lang/String;)Landroid/util/Pair;' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService$Stub$Proxy;-getCurrentSyncs-()Ljava/util/List;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker;-getImsService-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/NullFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/location/LocationManager;-addGpsStatusListener-(Landroid/location/GpsStatus$Listener;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/notification/ValidateNotificationPeople$2;-run-()V' : ['android.permission.READ_CONTACTS'],
	'Lcom/android/server/wm/WindowStateAnimator;-prepareSurfaceLocked-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/cat/CatService;-sendMenuSelection-(I Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneProxy;-exitEmergencyCallbackMode-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/hdmi/NewDeviceAction;-processCommand-(Lcom/android/server/hdmi/HdmiCecMessage;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/pm/IPackageManager;-freeStorageAndNotify-(J Landroid/content/pm/IPackageDataObserver;)V' : ['android.permission.CLEAR_APP_CACHE'],
	'Lcom/android/internal/telephony/cdma/CdmaConnection;-<init>-(Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/cdma/CdmaCallTracker; Lcom/android/internal/telephony/cdma/CdmaCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/TableRow;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/ToRGBAFilter;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/inputmethod/InputMethodManager;-getShortcutInputMethodsAndSubtypes-()Ljava/util/Map;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-handleMotionEvent-(Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub$Proxy;-getMsisdnForSubscriber-(J)Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/net/wifi/WifiManager;-<init>-(Landroid/content/Context; Landroid/net/wifi/IWifiManager;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/accounts/AccountManager$3;-doWork-()V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/nfc/handover/HandoverService;-doOutgoingTransfer-(Landroid/os/Message;)V' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-setSmscAddress-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/view/menu/ActionMenuItemView;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/print/IPrintManager$Stub$Proxy;-destroyPrinterDiscoverySession-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/Switch;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getDataEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/usb/UsbService$Lifecycle;-onStart-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.WRITE_SETTINGS'],
	'Landroid/net/sip/SipAudioCall;-close-(Z)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection;-access$5500-(Lcom/android/internal/telephony/dataconnection/DataConnection; Ljava/lang/Object;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/widget/AutoCompleteTextView;-performLongClick-()Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/os/BaseCommand;-onRun-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.KILL_BACKGROUND_PROCESSES', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.RESTART_PACKAGES', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiManager;-setAllowScansWithTraffic-(I)V' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/server/location/CountryDetectorBase;-stop-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/view/WindowManagerPolicy;-beginLayoutLw-(Z I I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-access$800-(Lcom/android/server/wm/WindowManagerService; I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/location/ILocationManager;-reportLocation-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-notifyServiceStateChangedP-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/TelephonyRegistry;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setTrustAgentFeaturesEnabled-(Landroid/content/ComponentName; Landroid/content/ComponentName; Ljava/util/List; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SubscriptionController;-getDefaultSmsSubId-()J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-updateCurrentCarrierInProvider-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/WifiManager;-getWifiServiceMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/filterpacks/numeric/SinWaveFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/content/SyncManager;-maybeRescheduleSync-(Landroid/content/SyncResult; Lcom/android/server/content/SyncOperation;)V' : ['android.permission.WAKE_LOCK', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/UiModeManagerService;-updateTwilight-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/pm/PackageManagerService;-performBootDexOpt-(Landroid/content/pm/PackageParser$Package; I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/uicc/RuimRecords;-fetchRuimRecords-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub;-getConnectionStatistics-()Landroid/net/wifi/WifiConnectionStatistics;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-notifyOffApnsOfAvailability-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/renderscript/RSTextureView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/job/JobSchedulerService$JobSchedulerStub;-canPersistJobs-(I I)Z' : ['android.permission.RECEIVE_BOOT_COMPLETED'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection;-hangup-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindow$RotationWatcher$1;-run-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getSubtypeHistoryStr-()Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-findDesiredInputMethodWindowIndexLocked-(Z)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/IPhoneSubInfo;-getVoiceMailAlphaTag-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getCameraDisabled-(Landroid/content/ComponentName; I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioService;-onSetStreamVolume-(I I I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CompoundButton;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/GLTextureTarget;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/os/Vibrator;-vibrate-(I Ljava/lang/String; [J I Landroid/media/AudioAttributes;)V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$17400-(Lcom/android/server/wifi/WifiStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/view/IApplicationToken$Stub$Proxy;-windowsVisible-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/MountService$3;-run-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-peekAuthToken-(Landroid/accounts/Account; Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-isSMSPromptEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/inputmethod/InputMethodUtils$InputMethodSettings;-getEnabledInputMethodSubtypeListLocked-(Landroid/content/Context; Landroid/view/inputmethod/InputMethodInfo; Z)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SubscriptionManager;-getDefaultSmsSubInfo-()Landroid/telephony/SubInfoRecord;' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/app/backup/FullBackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneBase;-setTTYMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/AliasActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/net/NetworkPolicyManagerService;-updateRulesForGlobalChangeLocked-(Z)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService$Service;-binderDied-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-updateUserSetupComplete-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/IAudioService$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/policy/impl/RecentApplicationsDialog;-switchTo-(Lcom/android/internal/policy/impl/RecentApplicationsDialog$RecentTag;)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/media/SubtitleController;-doHide-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/uicc/IccFileHandler;-getEFLinearRecordSize-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISms$Stub$Proxy;-sendMultipartTextForSubscriber-(J Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/util/List; Ljava/util/List; Ljava/util/List;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/nfc/tech/MifareClassic;-transceive-([B)[B' : ['android.permission.NFC'],
	'Lcom/android/internal/telephony/uicc/IccCardProxy;-changeIccFdnPassword-(Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/pm/PackageManagerService$AsecInstallArgs;-checkFreeStorage-(Lcom/android/internal/app/IMediaContainerService;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/server/BluetoothA2dpService;-onBluetoothDisable-()V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/service/dreams/DreamService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-getLong-(Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Landroid/content/IContentService;-syncAsUser-(Landroid/content/SyncRequest; I)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindow$DecorView;-onAttachedToWindow-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/backup/BackupAgent;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/SupplicantStateTracker;-onQuitting-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accessibility/ScreenMagnifier;-access$300-(Lcom/android/server/accessibility/ScreenMagnifier; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneBase;-notifyServiceStateChanged-(Landroid/telephony/ServiceState;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/ObjectSource;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GsmInboundSmsHandler;-dispatchMessageRadioSpecific-(Lcom/android/internal/telephony/SmsMessageBase;)I' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/hardware/Camera;-native_setup-(Ljava/lang/Object;)V' : ['android.permission.CAMERA'],
	'Landroid/location/GpsMeasurementListenerTransport;-add-(Ljava/lang/Object;)Z' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPasswordMinimumLowerCase-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/DownloadManager;-getUriForDownloadedFile-(J)Landroid/net/Uri;' : ['android.permission.INTERNET'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/IWallpaperManager;-setDimensionHints-(I I)V' : ['android.permission.SET_WALLPAPER_HINTS'],
	'Lcom/android/server/SamplingProfilerService;-handleSnapshotFile-(Ljava/io/File; Landroid/os/DropBoxManager;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/content/ContentResolver;-getCurrentSync-()Landroid/content/SyncInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/server/wifi/WifiSettingsStore;-handleWifiToggled-(Z)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-calculateLength-(Ljava/lang/CharSequence; Z)Lcom/android/internal/telephony/GsmAlphabet$TextEncodingDetails;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/SMSDispatcher$SMSDispatcherReceiver;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/InputMethodManagerService;-getLastInputMethodSubtype-()Landroid/view/inputmethod/InputMethodSubtype;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CompoundButton;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/SmsMessage;-hasEmsSupport-()Z' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/media/effect/effects/BackDropperEffect;-apply-(I I I I)V' : ['android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setMasterVolumeMuted-(Landroid/content/ComponentName; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/p2p/IWifiP2pManager$Stub;-getP2pStateMachineMessenger-()Landroid/os/Messenger;' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-nvWriteCdmaPrl-([B Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/SamplingProfilerService$SamplingProfilerSettingsObserver;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/net/NetworkStatsService$5;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/ContextImpl$ApplicationContentResolver;-getSyncStatusAsUser-(Landroid/accounts/Account; Ljava/lang/String; I)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/nfc/NfcService;-onRemoteFieldDeactivated-()V' : ['android.permission.VIBRATE'],
	'Landroid/widget/Switch;-shouldSpeakPasswordsForAccessibility-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-getDataCallList-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/display/DisplayManagerService;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/Application;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/media/AsyncPlayer;-startSound-(Landroid/media/AsyncPlayer$Command;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/WiredAccessoryManager;-updateLocked-(Ljava/lang/String; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/NfcService;-onLlcpLinkDeactivated-(Lcom/android/nfc/DeviceHost$NfcDepEndpoint;)V' : ['android.permission.VIBRATE'],
	'Landroid/filterpacks/text/StringSource;-performTearDown-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiServiceImpl;-releaseWifiLock-(Landroid/os/IBinder;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/os/storage/ExternalStorageFormatter;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/SimpleImageFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/IActivityContainer$Stub;-release-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/LocationProviderInterface;-disable-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/IAccountManager$Stub;-setPassword-(Landroid/accounts/Account; Ljava/lang/String;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS'],
	'Lcom/android/internal/telephony/SmsUsageMonitor;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/SlidingTab;-onInterceptTouchEvent-(Landroid/view/MotionEvent;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService$2;-onProximityPositive-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/RemoteViewsService;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ResizeFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-getCallForwardingOption-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/WebVttRenderingWidget;-onDetachedFromWindow-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$4400-(Lcom/android/server/wifi/WifiStateMachine; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/AudioService;-resetBluetoothSco-()V' : ['android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-updateDataConnectionTracker-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/AudioManager;-setMicrophoneMute-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/filterpacks/videoproc/BackDropperFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendText-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/filterpacks/imageproc/DuotoneFilter;-transferInputPortFrame-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/view/IWindowManager$Stub;-getPreferredOptionsPanelGravity-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/Downloads;-removeAllDownloadsByPackage-(Landroid/content/Context; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.INTERNET'],
	'Lcom/android/server/wifi/SupplicantStateTracker$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/base/FrameBranch;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiServiceImpl;-setWifiApEnabled-(Landroid/net/wifi/WifiConfiguration; Z)V' : ['android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getGlobalProxyAdmin-(I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/SeekBar;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/SMSDispatcher;-<init>-(Lcom/android/internal/telephony/PhoneBase; Lcom/android/internal/telephony/SmsUsageMonitor; Lcom/android/internal/telephony/ImsSMSDispatcher;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Landroid/widget/ScrollView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowAnimator;-updateWallpaperLocked-(I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-disableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BlendFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-handleLocationChangedLocked-(Landroid/location/Location; Z)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/NsdService$NsdStateMachine$DisabledState;-enter-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/os/RuntimeInit;-wtf-(Ljava/lang/String; Ljava/lang/Throwable; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/AlertActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Landroid/view/IApplicationToken;-keyDispatchingTimedOut-(Ljava/lang/String;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/location/GpsLocationProvider;-setEngineCapabilities-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/SmsMessage;-calculateLength-(Ljava/lang/CharSequence; Z)Lcom/android/internal/telephony/GsmAlphabet$TextEncodingDetails;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-updateSystemUiVisibilityLw-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/Context;-clearWallpaper-()V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/TextServicesManagerService;-access$100-(Lcom/android/server/TextServicesManagerService; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityManagerService;-access$5700-(Lcom/android/server/accessibility/AccessibilityManagerService; Lcom/android/server/accessibility/AccessibilityManagerService$UserState;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CdmaSubscriptionSourceManager;-<init>-(Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/CropFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/content/IContentService$Stub;-addPeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle; J)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Landroid/speech/RecognitionService$RecognitionServiceBinder;-stopListening-(Landroid/speech/IRecognitionListener;)V' : ['android.permission.RECORD_AUDIO'],
	'Lcom/android/internal/widget/ILockSettings;-checkVoldPassword-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getCrossProfileCallerIdDisabledForUser-(I)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/AbsoluteLayout;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/app/IntentForwarderActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/PosterizeFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/nfc/INfcCardEmulation;-removeAidGroupForService-(I Landroid/content/ComponentName; Ljava/lang/String;)Z' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.NFC'],
	'Landroid/app/ActivityView$ActivityContainerWrapper;-release-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/SystemService;-onStart-()V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CLEAR_APP_CACHE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-getOperator-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/nfc/P2pEventListener;-onP2pReceiveComplete-(Z)V' : ['android.permission.VIBRATE'],
	'Landroid/renderscript/RSSurfaceView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/voiceinteraction/SoundTriggerHelper;-internalClearStateLocked-()V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/backup/BackupManagerService$Lifecycle;-onStartUser-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-setPasswordMinimumLetters-(Landroid/content/ComponentName; I I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/LockPatternUtils;-getFallbackAppWidgetId-()I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneConnection;-<init>-(Landroid/content/Context; Ljava/lang/String; Lcom/android/internal/telephony/imsphone/ImsPhoneCallTracker; Lcom/android/internal/telephony/imsphone/ImsPhoneCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/CallbackFilter;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/media/AudioManager;-setSpeakerphoneOn-(Z)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/ISub$Stub;-getSubInfoUsingIccId-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/ISub;-getSubInfoForSubscriber-(J)Landroid/telephony/SubInfoRecord;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/accounts/CantAddAccountActivity;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-enableLocationUpdates-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/content/ContextWrapper;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performClose-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/IUiAutomationConnection$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/sip/SipService;-access$1300-(Lcom/android/server/sip/SipService; Landroid/net/sip/SipProfile; I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/LocationManagerService;-setTestProviderStatus-(Ljava/lang/String; I Landroid/os/Bundle; J)V' : ['android.permission.READ_PROFILE', 'android.permission.ACCESS_MOCK_LOCATION'],
	'Lcom/android/internal/telephony/Connection;-proceedAfterWildChar-(Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setCdmaRoamingPreference-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager$AmsTask$Response;-onResult-(Landroid/os/Bundle;)V' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-finishPostLayoutPolicyLw-()I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.BROADCAST_STICKY', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/gsm/GsmServiceStateTracker;-getAutoTimeZone-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcAdapter;-invokeBeam-()V' : ['android.permission.VIBRATE', 'android.permission.NFC'],
	'Landroid/media/AudioService;-adjustSuggestedStreamVolume-(I I I Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-setDataProfilesAsNeeded-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/TelephonyRegistry;-listen-(Ljava/lang/String; Lcom/android/internal/telephony/IPhoneStateListener; I Z J)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/location/SettingInjectorService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/widget/multiwaveview/GlowPadView;-dispatchTriggerEvent-(I)V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/app/ReceiverRestrictedContext;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/server/net/NetworkPolicyManagerService$13;-onReceive-(Landroid/content/Context; Landroid/content/Intent;)V' : ['android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/InputMethodManagerService;-updateFromSettingsLocked-(Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/nfc/ConfirmConnectToWifiNetworkActivity;-onClick-(Landroid/view/View;)V' : ['android.permission.CHANGE_WIFI_STATE', 'android.permission.ACCESS_WIFI_STATE'],
	'Landroid/widget/RemoteViewsAdapter$RemoteViewsFrameLayout;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/ISub;-addSubInfoRecord-(Ljava/lang/String; I)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.SET_TIME_ZONE'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-handleNewSms-(Landroid/os/AsyncResult;)V' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/widget/ActivityChooserView;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;-access$2800-(Lcom/android/server/wifi/p2p/WifiP2pServiceImpl$P2pStateMachine;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/PhoneSubInfoProxy;-getSubscriberId-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/base/RetargetFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext$MockAccountManager;-getAuthToken-(Landroid/accounts/Account; Ljava/lang/String; Z Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/location/ILocationManager$Stub$Proxy;-removeGeofence-(Landroid/location/Geofence; Landroid/app/PendingIntent; Ljava/lang/String;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/nfc/NfcBackupAgent;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/RenamingDelegatingContext;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/ExpandableListActivity;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/IPhoneSubInfo$Stub;-getGroupIdLevel1-()Ljava/lang/String;' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-getAdnRecordsSizeForSubscriber-(J I)[I' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/AlarmManagerService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-alarmExists-(Landroid/content/ContentResolver; J J J)Z' : ['android.permission.READ_CALENDAR'],
	'Landroid/widget/AbsListView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/cardemulation/OffHostApduService;-sendStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/media/RingtoneManager;-getRingtone-(I)Landroid/media/Ringtone;' : ['android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/telephony/imsphone/ImsPhoneCall;-dispose-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub$Stub;-clearDefaultsForInactiveSubIds-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE'],
	'Landroid/net/nsd/NsdManager;-getMessenger-()Landroid/os/Messenger;' : ['android.permission.INTERNET'],
	'Landroid/widget/QuickContactBadge;-onClick-(Landroid/view/View;)V' : ['android.permission.READ_CONTACTS'],
	'Landroid/service/dreams/DreamService;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/PhoneFactory;-getCdmaPhone-()Lcom/android/internal/telephony/Phone;' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE', 'android.permission.SET_TIME_ZONE'],
	'Landroid/widget/CalendarView$LegacyCalendarViewDelegate;-access$2100-(Landroid/widget/CalendarView$LegacyCalendarViewDelegate; Ljava/util/Calendar;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/RadialTimePickerView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/content/CursorEntityIterator;-next-()Ljava/lang/Object;' : ['android.permission.READ_CALENDAR'],
	'Lcom/android/internal/telephony/sip/SipPhone$SipConnection$1;-onCallEnded-(Landroid/net/sip/SipAudioCall;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/effect/effects/ContrastEffect;-apply-(I I I I)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/SubscriptionController;-setDefaultVoiceSubId-(J)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onEnableApn-(I I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/net/sip/ISipService;-getPendingSession-(Ljava/lang/String;)Landroid/net/sip/ISipSession;' : ['android.permission.USE_SIP'],
	'Lcom/android/internal/policy/impl/RecentApplicationsBackground;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/backup/BackupManagerService$PerformBackupTask;-operationComplete-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/hdmi/DeviceDiscoveryAction$DeviceDiscoveryCallback;-onDeviceDiscoveryDone-(Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/ColorTemperatureFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/Chronometer;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/nfc/NfcAdapter;-disableReaderMode-(Landroid/app/Activity;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-clearCrossProfileIntentFilters-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/RegisteredAidCache;-generateAidCacheLocked-()V' : ['android.permission.VIBRATE'],
	'Lcom/android/server/wifi/WifiController$ApStaDisabledState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/TtmlRenderingWidget;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/os/IDropBoxManagerService$Stub;-add-(Landroid/os/DropBoxManager$Entry;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-queryCdmaRoamingPreference-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager;-handleLongPressOnHome-()V' : ['android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DctController;-setDataSubId-(J)V' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-sendAdminCommandLocked-(Ljava/lang/String; I I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/widget/CheckedTextView;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/CardEmulationManager;-verifyDefaults-(I Ljava/util/List;)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager;-removeUser-(Landroid/content/ComponentName; Landroid/os/UserHandle;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcController$DccDefaultState;-onDataStateChanged-(Ljava/util/ArrayList;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/IIccPhoneBook$Stub;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/bluetooth/BluetoothA2dp;-finalize-()V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/MediaRouteButton;-startDrag-(Landroid/content/ClipData; Landroid/view/View$DragShadowBuilder; Ljava/lang/Object; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/IAudioService$Stub$Proxy;-startBluetoothSco-(Landroid/os/IBinder; I)V' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Lcom/android/server/wm/WindowManagerService;-showStrictModeViolation-(I I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/app/IBatteryStats$Stub;-noteBluetoothOn-()V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/server/wifi/WifiStateMachine;-access$4100-(Lcom/android/server/wifi/WifiStateMachine; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/dataconnection/DataConnection$DcInactiveState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/hdmi/HdmiCecLocalDeviceTv;-create-(Lcom/android/server/hdmi/HdmiControlService; I)Lcom/android/server/hdmi/HdmiCecLocalDevice;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/widget/NumberPicker$CustomEditText;-onInitializeAccessibilityNodeInfo-(Landroid/view/accessibility/AccessibilityNodeInfo;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/cardemulation/TapAgainDialog;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/ims/ImsCall$Listener;-onCallHoldFailed-(Lcom/android/ims/ImsCall; Lcom/android/ims/ImsReasonInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/CrossProcessFilter;-notifyFieldPortValueUpdated-(Ljava/lang/String; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaPlayer;-start-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiStateMachine$ObtainingIpState;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/display/DisplayManagerInternal;-requestPowerState-(Landroid/hardware/display/DisplayManagerInternal$DisplayPowerRequest; Z)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/wm/WindowManagerService;-access$400-(Lcom/android/server/wm/WindowManagerService;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/telephony/SmsManager;-deleteMessageFromIcc-(I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.SEND_SMS', 'android.permission.RECEIVE_SMS'],
	'Landroid/app/backup/FullBackupAgent;-setWallpaper-(Ljava/io/InputStream;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/uicc/IccRecords;-onReady-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-separateConnection-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/test/IsolatedContext;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CDMAPhone;-initSstIcc-()V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/widget/LockPatternView;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/devicepolicy/DevicePolicyManagerService;-enableSystemAppWithIntent-(Landroid/content/ComponentName; Landroid/content/Intent;)I' : ['android.permission.READ_PROFILE'],
	'Landroid/view/SurfaceView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/GrainFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-isDataAllowed-(Lcom/android/internal/telephony/dataconnection/ApnContext;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/media/SubtitleController;-access$000-(Landroid/media/SubtitleController;)V' : ['android.permission.READ_PROFILE', 'android.permission.WRITE_SETTINGS'],
	'Lcom/android/internal/policy/impl/PhoneWindowManager$7;-onSwipeFromTop-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/nfc/INfcTag$Stub$Proxy;-rediscover-(I)Landroid/nfc/Tag;' : ['android.permission.NFC'],
	'Lcom/android/nfc/NfcService$EnableDisableTask;-doInBackground-([Ljava/lang/Integer;)Ljava/lang/Void;' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE', 'android.permission.BLUETOOTH'],
	'Landroid/print/PrinterDiscoverySession;-startPrinterStateTracking-(Landroid/print/PrinterId;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/DialogTitle;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/media/effect/effects/BrightnessEffect;-release-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/net/wifi/IWifiManager$Stub$Proxy;-isScanAlwaysAvailable-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/server/net/NetworkStatsService;-access$500-(Lcom/android/server/net/NetworkStatsService; Landroid/net/NetworkTemplate; I)Landroid/net/NetworkStatsHistory;' : ['android.permission.READ_PROFILE'],
	'Landroid/hardware/IConsumerIrService$Stub$Proxy;-getCarrierFrequencies-()[I' : ['android.permission.TRANSMIT_IR'],
	'Lcom/android/internal/widget/LockPatternUtilsCache;-getString-(Ljava/lang/String; Ljava/lang/String; I)Ljava/lang/String;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStackSupervisor;-checkReadyForSleepLocked-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-restoreSavedNetworkSelection-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/uicc/IsimUiccRecords;-setVoiceCallForwardingFlag-(I Z Ljava/lang/String;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/PhoneSubInfoController;-getIccSimChallengeResponse-(J I Ljava/lang/String;)Ljava/lang/String;' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-clearProfileOwner-(Landroid/content/ComponentName;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/TouchExplorer;-access$2600-(Lcom/android/server/accessibility/TouchExplorer; Landroid/view/MotionEvent; I)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/provider/VoicemailContract$Voicemails;-buildSourceUri-(Ljava/lang/String;)Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/filterpacks/imageproc/DrawRectFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDataConnectionDetached-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/InputMonitor;-notifyANR-(Lcom/android/server/input/InputApplicationHandle; Lcom/android/server/input/InputWindowHandle; Ljava/lang/String;)J' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/am/ActivityManagerService;-deliverPreBootCompleted-(Ljava/lang/Runnable; Ljava/util/ArrayList; I)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothAdapter;-enable-()Z' : ['android.permission.BLUETOOTH_ADMIN'],
	'Lcom/android/server/NetworkScoreService;-systemReady-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeech;-isSpeaking-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-onDataConnectionAttached-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-resetRadio-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CDMALTEPhone;-notifyDataConnection-(Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.READ_PROFILE'],
	'Landroid/test/IsolatedContext$MockAccountManager;-hasFeatures-(Landroid/accounts/Account; [Ljava/lang/String; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS'],
	'Landroid/accounts/ChooseAccountTypeActivity;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/SyncManager$SyncHandler;-maybeStartNextSyncLocked-()J' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE', 'android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/wifi/WifiStateMachine;-startNextBatchedScan-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/ims/ImsCall$ImsCallSessionListenerProxy;-callSessionMerged-(Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/internal/ImsCallSession; Lcom/android/ims/ImsCallProfile;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/accounts/AccountManager;-confirmCredentials-(Landroid/accounts/Account; Landroid/os/Bundle; Landroid/app/Activity; Landroid/accounts/AccountManagerCallback; Landroid/os/Handler;)Landroid/accounts/AccountManagerFuture;' : ['android.permission.AUTHENTICATE_ACCOUNTS', 'android.permission.MANAGE_ACCOUNTS', 'android.permission.GET_ACCOUNTS', 'android.permission.USE_CREDENTIALS'],
	'Landroid/server/BluetoothA2dpService;-resumeSink-(Landroid/bluetooth/BluetoothDevice;)Z' : ['android.permission.MODIFY_AUDIO_SETTINGS'],
	'Landroid/app/ActivityManager;-restartPackage-(Ljava/lang/String;)V' : ['android.permission.RESTART_PACKAGES', 'android.permission.KILL_BACKGROUND_PROCESSES'],
	'Lcom/android/internal/telephony/cdma/CdmaCallTracker;-separate-(Lcom/android/internal/telephony/cdma/CdmaConnection;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/bluetooth/BluetoothServerSocket;-<init>-(I Z Z Landroid/os/ParcelUuid;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/location/LocationManager;-requestLocationUpdates-(Ljava/lang/String; J F Landroid/app/PendingIntent;)V' : ['android.permission.ACCESS_FINE_LOCATION', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/internal/util/State;-processMessage-(Landroid/os/Message;)Z' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_WAP_PUSH', 'android.permission.WRITE_SMS', 'android.permission.ACCESS_WIFI_STATE', 'android.permission.RECEIVE_MMS', 'android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.READ_PROFILE', 'android.permission.READ_SMS'],
	'Landroid/nfc/INfcTag$Stub;-connect-(I I)I' : ['android.permission.NFC'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-installKeyPair-(Landroid/content/ComponentName; [B [B Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/telephony/PhoneNumberUtils;-isPotentialEmergencyNumber-(Ljava/lang/String; Ljava/lang/String;)Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTracker;-getAnyDataEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setPermittedInputMethods-(Landroid/content/ComponentName; Ljava/util/List;)Z' : ['android.permission.READ_PROFILE'],
	'Landroid/view/WindowManagerPolicy;-interceptKeyBeforeDispatching-(Landroid/view/WindowManagerPolicy$WindowState; Landroid/view/KeyEvent; I)J' : ['android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/BitmapSource;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/net/wifi/IWifiManager;-getWifiApConfiguration-()Landroid/net/wifi/WifiConfiguration;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-setCrossProfileCallerIdDisabled-(Landroid/content/ComponentName; Z)V' : ['android.permission.READ_PROFILE'],
	'Landroid/app/ActivityManagerNative;-broadcastStickyIntent-(Landroid/content/Intent; Ljava/lang/String; I)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/app/TabActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/content/ContentService;-getSyncStatusAsUser-(Landroid/accounts/Account; Ljava/lang/String; Landroid/content/ComponentName; I)Landroid/content/SyncStatusInfo;' : ['android.permission.READ_SYNC_STATS'],
	'Lcom/android/internal/telephony/sip/SipPhone;-<init>-(Ljava/lang/String; Landroid/content/Context; Lcom/android/internal/telephony/PhoneNotifier;)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/WindowManagerService;-setCurrentUser-(I [I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ITelephonyRegistry;-notifyDataConnectionFailed-(Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wm/Session;-addWithoutInputChannel-(Landroid/view/IWindow; I Landroid/view/WindowManager$LayoutParams; I Landroid/graphics/Rect;)I' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SYSTEM_ALERT_WINDOW', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/UiccSmsController;-enableCellBroadcastForSubscriber-(J I)Z' : ['android.permission.WAKE_LOCK', 'android.permission.RECEIVE_SMS'],
	'Lcom/android/nfc/handover/HandoverService;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cdma/CdmaLteServiceStateTracker;-enableSingleLocationUpdate-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/tv/TvInputService;-removeStickyBroadcast-(Landroid/content/Intent;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/location/LocationBlacklist;-onChange-(Z)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/PhoneBase;-getCellLocation-()Landroid/telephony/CellLocation;' : ['android.permission.WAKE_LOCK', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityManagerService;-moveTaskToBack-(I)V' : ['android.permission.REORDER_TASKS'],
	'Landroid/app/LauncherActivity;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/sip/SipSessionGroup$SipSessionImpl;-processCommand-(Ljava/util/EventObject;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-changeBarringPassword-(Ljava/lang/String; Ljava/lang/String; Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/accounts/AccountManagerService;-getAccountsByTypeForPackage-(Ljava/lang/String; Ljava/lang/String;)[Landroid/accounts/Account;' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/NsdService;-isNsdEnabled-()Z' : ['android.permission.READ_PROFILE'],
	'Landroid/filterpacks/imageproc/AutoFixFilter;-initProgramInputs-(Landroid/filterfw/core/Program; Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/videosrc/SurfaceTextureTarget;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/LockSettingsService;-getLong-(Ljava/lang/String; J I)J' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/dataconnection/DcTrackerBase;-setDataProfilesAsNeeded-()V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/RIL;-setCdmaBroadcastActivation-(Z Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setSecureSetting-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/sip/SipPhoneBase;-setBandMode-(I Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/imsphone/ImsPhone;-<init>-(Ljava/lang/String; Lcom/android/internal/telephony/PhoneNotifier; Landroid/content/Context; Lcom/android/internal/telephony/CommandsInterface; Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Lcom/android/server/wm/DragState;-notifyDropLw-(F F)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPasswordMinimumSymbols-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub;-getPermittedAccessibilityServices-(Landroid/content/ComponentName;)Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Landroid/os/IVibratorService;-vibratePattern-(I Ljava/lang/String; [J I I Landroid/os/IBinder;)V' : ['android.permission.VIBRATE'],
	'Landroid/widget/RatingBar;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/nfc/P2pLinkManager;-enableDisable-(Z Z)V' : ['android.permission.BLUETOOTH'],
	'Landroid/app/IBackupAgent;-doRestore-(Landroid/os/ParcelFileDescriptor; I Landroid/os/ParcelFileDescriptor; I Landroid/app/backup/IBackupManager;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/PhoneBase;-setCellInfoListRate-(I)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/wifi/WifiWatchdogStateMachine;-registerForWatchdogToggle-()V' : ['android.permission.WRITE_SETTINGS'],
	'Lcom/android/server/BluetoothManagerService;-unregisterStateChangeCallback-(Landroid/bluetooth/IBluetoothStateChangeCallback;)V' : ['android.permission.BLUETOOTH'],
	'Landroid/test/mock/MockContentResolver;-removePeriodicSync-(Landroid/accounts/Account; Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.WRITE_SYNC_SETTINGS'],
	'Lcom/android/nfc/dhimpl/NativeNfcManager;-notifyLlcpLinkDeactivated-(Lcom/android/nfc/dhimpl/NativeP2pDevice;)V' : ['android.permission.VIBRATE'],
	'Landroid/text/format/DateUtils;-getRelativeTimeSpanString-(Landroid/content/Context; J Z)Ljava/lang/CharSequence;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/am/ActivityStack;-removeTask-(Lcom/android/server/am/TaskRecord;)V' : ['android.permission.SEND_SMS', 'android.permission.DISABLE_KEYGUARD', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/filterpacks/base/GLTextureTarget;-performOpen-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Lcom/android/server/wifi/WifiController$DefaultState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/notification/ConditionProviders;-disableNonexistentServices-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/print/PrintManager;-getInstalledPrintServices-()Ljava/util/List;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/usage/UsageStatsService;-<init>-(Landroid/content/Context;)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/RIL;-getAvailableNetworks-(Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/cdma/CdmaServiceStateTracker;-getAutoTimeZone-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/NetworkTimeUpdateService;-<init>-(Landroid/content/Context;)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-setWifiState-(I)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/internal/telephony/cat/CatService;-handleCmdResponse-(Lcom/android/internal/telephony/cat/CatResponseMessage;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/ISub;-clearSubInfo-()I' : ['android.permission.READ_PHONE_STATE'],
	'Lcom/android/nfc/P2pLinkManager;-onLlcpActivated-()V' : ['android.permission.VIBRATE'],
	'Landroid/filterfw/core/FilterFunction;-tearDown-()V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/base/FrameBranch;-transferInputFrames-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/print/IPrintManager;-stopPrinterDiscovery-(Landroid/print/IPrinterDiscoveryObserver; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/telephony/CommandsInterface;-sendUSSD-(Ljava/lang/String; Landroid/os/Message;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/media/MediaFocusControl;-<init>-(Landroid/os/Looper; Landroid/content/Context; Landroid/media/AudioService$VolumeController; Landroid/media/AudioService;)V' : ['android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Lcom/android/server/updates/TZInfoInstallReceiver;-access$000-(Lcom/android/server/updates/ConfigUpdateInstallReceiver; Landroid/content/ContentResolver;)Ljava/security/cert/X509Certificate;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/accessibility/AccessibilityInputFilter;-disableFeatures-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/filterpacks/imageproc/DrawOverlayFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
	'Landroid/filterpacks/imageproc/BitmapOverlayFilter;-open-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/app/Application;-sendStickyOrderedBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService$RemoveAccountSession;-onServiceConnected-(Landroid/content/ComponentName; Landroid/os/IBinder;)V' : ['android.permission.GET_ACCOUNTS'],
	'Lcom/android/server/am/ActivityManagerService;-skipPendingBroadcastLocked-(I)V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Landroid/media/IMediaHTTPConnection$Stub$Proxy;-getSize-()J' : ['android.permission.INTERNET'],
	'Landroid/filterpacks/imageproc/FlipFilter;-performProcess-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK', 'android.permission.RECORD_AUDIO', 'android.permission.CAMERA'],
	'Landroid/widget/StackView$StackFrame;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/wifi/WifiStateMachine;-stopDhcp-()V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/InputMethodManagerService;-resetAllInternalStateLocked-(Z Z)V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE', 'android.permission.READ_PROFILE'],
	'Landroid/content/SyncActivityTooManyDeletes;-sendStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Landroid/test/ActivityUnitTestCase$MockParent;-sendStickyOrderedBroadcast-(Landroid/content/Intent; Landroid/content/BroadcastReceiver; Landroid/os/Handler; I Ljava/lang/String; Landroid/os/Bundle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/wifi/WifiStateMachine$ConnectModeState;-exit-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/NfcService$ReaderModeDeathRecipient;-binderDied-()V' : ['android.permission.READ_PROFILE'],
	'Landroid/net/wifi/IWifiManager$Stub;-getScanResults-(Ljava/lang/String;)Ljava/util/List;' : ['android.permission.ACCESS_WIFI_STATE'],
	'Landroid/bluetooth/BluetoothMap;-<init>-(Landroid/content/Context; Landroid/bluetooth/BluetoothProfile$ServiceListener;)V' : ['android.permission.BLUETOOTH'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-shutdownRadio-()V' : ['android.permission.WAKE_LOCK', 'android.permission.BROADCAST_STICKY'],
	'Lcom/android/nfc/NfcApplication;-setWallpaper-(Landroid/graphics/Bitmap;)V' : ['android.permission.SET_WALLPAPER'],
	'Lcom/android/internal/telephony/cdma/CdmaInboundSmsHandler;-access$900-(Lcom/android/internal/telephony/InboundSmsHandler; Landroid/content/Intent;)Landroid/net/Uri;' : ['android.permission.WRITE_SMS'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-unregisterForSuppServiceNotification-(Landroid/os/Handler;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/internal/telephony/gsm/GSMPhone;-handleCfuQueryResult-([Lcom/android/internal/telephony/CallForwardInfo;)V' : ['android.permission.WAKE_LOCK'],
	'Lcom/android/server/MmsServiceBroker$BinderService;-onTransact-(I Landroid/os/Parcel; Landroid/os/Parcel; I)Z' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_MMS', 'android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/media/ClosedCaptionWidget$CCLineBox;-playSoundEffect-(I)V' : ['android.permission.READ_PROFILE'],
	'Landroid/speech/tts/TextToSpeechService$AudioSpeechItemV1;-getSpeechRate-()I' : ['android.permission.READ_PROFILE'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-getPasswordMinimumLetters-(Landroid/content/ComponentName; I)I' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/app/IntentForwarderActivity;-removeStickyBroadcastAsUser-(Landroid/content/Intent; Landroid/os/UserHandle;)V' : ['android.permission.BROADCAST_STICKY'],
	'Lcom/android/server/accounts/AccountManagerService;-onUserStarted-(Landroid/content/Intent;)V' : ['android.permission.GET_ACCOUNTS'],
	'Landroid/net/wifi/WifiManager;-isDeviceToDeviceRttSupported-()Z' : ['android.permission.ACCESS_WIFI_STATE'],
	'Lcom/android/internal/policy/impl/GlobalActions$PowerAction;-onPress-()V' : ['android.permission.BROADCAST_STICKY', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.SEND_SMS', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.WAKE_LOCK', 'android.permission.VIBRATE', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/server/twilight/TwilightService;-onSwitchUser-(I)V' : ['android.permission.READ_PROFILE', 'android.permission.READ_PHONE_STATE', 'android.permission.ACCESS_COARSE_LOCATION'],
	'Landroid/app/admin/IDevicePolicyManager$Stub$Proxy;-setGlobalProxy-(Landroid/content/ComponentName; Ljava/lang/String; Ljava/lang/String; I)Landroid/content/ComponentName;' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/power/PowerManagerService;-scheduleSandmanLocked-()V' : ['android.permission.SEND_SMS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.ACCESS_COARSE_LOCATION', 'android.permission.BROADCAST_STICKY', 'android.permission.WRITE_SETTINGS', 'android.permission.READ_PHONE_STATE', 'android.permission.CHANGE_WIFI_STATE'],
	'Lcom/android/internal/telephony/ISms$Stub;-sendStoredText-(J Ljava/lang/String; Landroid/net/Uri; Ljava/lang/String; Landroid/app/PendingIntent; Landroid/app/PendingIntent;)V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SMS', 'android.permission.READ_PHONE_STATE', 'android.permission.RECEIVE_SMS'],
	'Landroid/widget/SimpleMonthView;-performClick-()Z' : ['android.permission.READ_PROFILE'],
	'Lcom/android/internal/widget/ILockSettings$Stub;-setLockPassword-(Ljava/lang/String; I)V' : ['android.permission.READ_PROFILE'],
	'Lcom/android/server/content/ContentService;-systemReady-()V' : ['android.permission.SEND_SMS', 'android.permission.WRITE_SYNC_SETTINGS', 'android.permission.RECEIVE_BOOT_COMPLETED', 'android.permission.GET_ACCOUNTS'],
	'Landroid/net/INetworkPolicyManager;-getNetworkPolicies-()[Landroid/net/NetworkPolicy;' : ['android.permission.READ_PHONE_STATE'],
	'Landroid/filterpacks/imageproc/ToRGBFilter;-close-(Landroid/filterfw/core/FilterContext;)V' : ['android.permission.WAKE_LOCK'],
}

AOSP_PERMISSIONS_BY_FIELDS = {
	'Landroid/provider/ContactsContract;-AUTHORITY_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$Directory;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Callable;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Settings$Global;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/ContactsContract$Data;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$StreamItems;-CONTENT_LIMIT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$DeletedContacts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Mms$Inbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$Mms$Draft;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/BrowserContract$Bookmarks;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Contacts$ContactMethods;-CONTENT_EMAIL_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/server/MmsServiceBroker;-FAKE_MMS_SENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$MmsSms$PendingMessages;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$Attendees;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Settings$System;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Telephony$Mms$Sent;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$Contacts;-CORP_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$AggregationExceptions;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_CONVERSATIONS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_UNDELIVERED_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$Sms$Draft;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Contacts$GroupMembership;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$CellBroadcasts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CELL_BROADCASTS'],
	'Landroid/provider/ContactsContract$Profile;-CONTENT_RAW_CONTACTS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Groups;-DELETED_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/HbpcdLookup$MccSidConflicts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.MODIFY_PHONE_STATE'],
	'Landroid/provider/ContactsContract$DataUsageFeedback;-FEEDBACK_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Threads;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$StreamItems;-CONTENT_PHOTO_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/HbpcdLookup$MccLookup;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.MODIFY_PHONE_STATE'],
	'Landroid/provider/ContactsContract$Profile;-CONTENT_VCARD_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$Settings;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract$Images;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/VoicemailContract$Status;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/provider/ContactsContract$DataUsageFeedback;-DELETE_USAGE_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Mms;-REPORT_REQUEST_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/BrowserContract$SyncState;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$Sms$Inbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$MmsSms;-SEARCH_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/MmsServiceBroker;-FAKE_SMS_DRAFT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Contacts$Presence;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/BrowserContract$Searches;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Lcom/android/internal/telephony/SmsBroadcastUndelivered;-sRawUri-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Contactables;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Phones;-CONTENT_FILTER_URL-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/InboundSmsHandler;-sRawUri-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Lcom/android/server/MmsServiceBroker;-FAKE_MMS_DRAFT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$Mms$Rate;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$Colors;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_VCARD_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$CalendarEntity;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Telephony$Sms$Sent;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_BY_DAY_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Contacts$GroupMembership;-RAW_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Contactables;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_GROUP_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_STREQUENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$Events;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_FREQUENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$SyncState;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/HbpcdLookup$MccIdd;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.MODIFY_PHONE_STATE'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_DRAFT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CallLog$Calls;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG'],
	'Landroid/media/tv/TvContract$Programs;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.providers.tv.permission.WRITE_EPG_DATA', 'com.android.providers.tv.permission.READ_EPG_DATA'],
	'Landroid/provider/Settings$Secure;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/Contacts$People;-WITH_EMAIL_OR_IM_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$DisplayPhoto;-CONTENT_MAX_DIMENSIONS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$ProviderStatus;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$Reminders;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_MULTI_VCARD_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$CalendarAlerts;-CONTENT_URI_BY_INSTANCE-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Contacts$People;-DELETED_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_LOCKED_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$Groups;-CONTENT_SUMMARY_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CallLog$Calls;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG'],
	'Landroid/provider/VoicemailContract$Voicemails;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.voicemail.permission.ADD_VOICEMAIL'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Email;-CONTENT_LOOKUP_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Settings;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract;-AUTHORITY_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/tv/TvContract$Channels;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.providers.tv.permission.WRITE_EPG_DATA', 'com.android.providers.tv.permission.READ_EPG_DATA'],
	'Landroid/provider/ContactsContract$PhoneLookup;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Organizations;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract$Accounts;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/Telephony$Threads;-THREAD_ID_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$DisplayPhoto;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$PhoneLookup;-ENTERPRISE_CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$StatusUpdates;-PROFILE_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract$Bookmarks;-CONTENT_URI_DEFAULT_FOLDER-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/CalendarContract$Events;-CONTENT_EXCEPTION_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Contacts$People;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_SEARCH_BY_DAY_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/internal/telephony/HbpcdLookup$MccSidRange;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.MODIFY_PHONE_STATE'],
	'Landroid/provider/Downloads$Impl;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.INTERNET'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Phone;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Photos;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$Instances;-CONTENT_SEARCH_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Groups;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/HbpcdLookup;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.MODIFY_PHONE_STATE'],
	'Landroid/provider/ContactsContract$StreamItems;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Mms$Outbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$Calendars;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/Telephony$MmsSms;-CONTENT_FILTER_BYPHONE_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$SyncState;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract$History;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/media/tv/TvContract$WatchedPrograms;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.providers.tv.permission.WRITE_EPG_DATA', 'com.android.providers.tv.permission.READ_EPG_DATA'],
	'Landroid/provider/UserDictionary$Words;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Landroid/provider/Contacts$ContactMethods;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Browser;-SEARCHES_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/BrowserContract$ImageMappings;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/CalendarContract$EventsEntity;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Lcom/android/server/MmsServiceBroker;-FAKE_SMS_SENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$ProfileSyncState;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$StructuredPostal;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Extensions;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Phones;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Sms;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$Profile;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/BrowserContract$Combined;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_STREQUENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Downloads$Impl;-ALL_DOWNLOADS_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.ACCESS_ALL_DOWNLOADS'],
	'Landroid/provider/UserDictionary;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_USER_DICTIONARY', 'android.permission.WRITE_USER_DICTIONARY'],
	'Landroid/provider/Telephony$Sms$Outbox;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Telephony$Sms$Conversations;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/Settings$Bookmarks;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SETTINGS'],
	'Landroid/provider/CallLog;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG'],
	'Landroid/provider/Telephony$Mms;-REPORT_STATUS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Email;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$RawContacts;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$Groups;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CallLog$Calls;-CONTENT_URI_WITH_VOICEMAIL-Landroid/net/Uri;' : ['android.permission.WRITE_CALL_LOG', 'android.permission.READ_CALL_LOG'],
	'Landroid/provider/Telephony$Threads;-OBSOLETE_THREADS_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/CalendarContract$EventDays;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$RawContactsEntity;-PROFILE_CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/HbpcdLookup$ArbitraryMccSidMatch;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.MODIFY_PHONE_STATE'],
	'Landroid/provider/Browser;-BOOKMARKS_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/BrowserContract$Settings;-CONTENT_URI-Landroid/net/Uri;' : ['com.android.browser.permission.WRITE_HISTORY_BOOKMARKS', 'com.android.browser.permission.READ_HISTORY_BOOKMARKS'],
	'Landroid/provider/ContactsContract$StatusUpdates;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Telephony$Mms;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_SMS', 'android.permission.READ_SMS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Callable;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$RawContactsEntity;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/CalendarContract$ExtendedProperties;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Email;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/Contacts$People;-CONTENT_FILTER_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Lcom/android/internal/telephony/HbpcdLookup$NanpAreaCode;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.MODIFY_PHONE_STATE'],
	'Landroid/provider/CalendarContract$CalendarCache;-URI-Landroid/net/Uri;' : ['android.permission.READ_CALENDAR', 'android.permission.WRITE_CALENDAR'],
	'Landroid/provider/ContactsContract$Contacts;-CONTENT_LOOKUP_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
	'Landroid/provider/ContactsContract$CommonDataKinds$Phone;-CONTENT_URI-Landroid/net/Uri;' : ['android.permission.WRITE_CONTACTS', 'android.permission.READ_CONTACTS'],
}
#################################################
